package csbase.algorithms.flows.graph;

import java.awt.Image;

import tecgraf.javautils.LNG;
import csbase.algorithms.flows.ImageVO;
import csbase.client.applications.ApplicationImages;

/**
 * Representa uma decorao (um cone + tooltip) de um n do grafo.
 */
public class GraphNodeDecoration {

  /**
   * Tipos pr-definidos de decorao que podem ser usados pelo n.
   */
  public enum DecorationType {
        /**
     * Decorao que representa um algoritmo em execuo
     */
    EXECUTION,
        /** Decorao que representa um erro de execuo do algortimo **/
    EXECUTION_ERROR,
        /** Decorao que representa o sucesso de execuo do algortimo **/
    EXECUTION_SUCCESS,
        /**
     * Decorao que representa a incerteza sobre como terminou a execuo do
     * algorimto
     **/
    EXECUTION_UNKNOWN,
        /** Representa a opo de no ter nenhuma decorao **/
    BLANK;
  }

  /**
   * O tipo de decorao.
   */
  private DecorationType type;

  /**
   * cone que representa a decorao
   */
  private ImageVO image;

  /**
   * O texto que descreve a decorao. Deve ser usado como dica.
   */
  private String description;

  /**
   * Construtor
   */
  public GraphNodeDecoration() {
    this.description = "";
    this.image = new ImageVO(ApplicationImages.ICON_BLANK_16.getImage());
    this.type = DecorationType.BLANK;
  }

  /**
   * Construtor
   * 
   * @param type Tipo da decorao
   * @param image cone da decorao
   * @param description Descrio da decorao
   */
  public GraphNodeDecoration(DecorationType type, ImageVO image,
    String description) {
    this.type = type;
    this.image = image;
    this.description = description;
  }

  /**
   * Retorna a descrio da decorao
   * 
   * @return description A descrio
   */
  public String getDescription() {
    return description;
  }

  /**
   * Atribui a descrio
   * 
   * @param description A descrio
   */
  public void setDescription(String description) {
    this.description = description;
  }

  /**
   * Retorna a imagem que representa a decorao
   * 
   * @return image A imagem
   */
  public ImageVO getImage() {
    return image;
  }

  /**
   * 
   * Atribui a imagem que representa a decorao
   * 
   * @param image A imagem
   */
  public void setImage(ImageVO image) {
    this.image = image;
  }

  /**
   * Retorna o tipo da decorao
   * 
   * @return type O tipo
   */
  public DecorationType getType() {
    return type;
  }

  /**
   * Atribui o tipo da decorao
   * 
   * @param type O tipo
   */
  public void setType(DecorationType type) {
    this.type = type;
  }

  /**
   * Cria a decorao de um determinado tipo pr-definido
   * 
   * @param type O tipo pr-definido
   * @return Uma decorao do tipo especificado
   */
  public static GraphNodeDecoration createDecoration(DecorationType type) {
    switch (type) {
      case EXECUTION:
        return createExecutingDecoration();
      case EXECUTION_ERROR:
        return createExecutionErrorDecoration();
      case EXECUTION_SUCCESS:
        return createExecutionSuccessDecoration();
      case EXECUTION_UNKNOWN:
        return createExecutionUnknownDecoration();
      default:
        return createNullDecoration();
    }
  }

  /**
   * Cria uma decorao do tipo {@link DecorationType#EXECUTION}
   * 
   * @return Uma decorao do tipo {@link DecorationType#EXECUTION}
   */
  private static GraphNodeDecoration createExecutingDecoration() {
    return createDecoration(DecorationType.EXECUTION,
      ApplicationImages.ICON_EXECUTE_CMD_16.getImage(),
      "algorithms.tooltip.executing");
  }

  /**
   * Cria uma decorao do tipo {@link DecorationType#EXECUTION_SUCCESS}
   * 
   * @return Uma decorao do tipo {@link DecorationType#EXECUTION_SUCCESS}
   */
  private static GraphNodeDecoration createExecutionSuccessDecoration() {
    return createDecoration(DecorationType.EXECUTION_SUCCESS,
      ApplicationImages.ICON_OK_16.getImage(),
      "algorithms.tooltip.execution_end.success");
  }

  /**
   * Cria uma decorao do tipo {@link DecorationType#EXECUTION_ERROR}
   * 
   * @return Uma decorao do tipo {@link DecorationType#EXECUTION_ERROR}
   */
  private static GraphNodeDecoration createExecutionErrorDecoration() {
    return createDecoration(DecorationType.EXECUTION_ERROR,
      ApplicationImages.ICON_ERROR_16.getImage(),
      "algorithms.tooltip.execution_end.error");
  }

  /**
   * Cria uma decorao do tipo {@link DecorationType#EXECUTION_UNKNOWN}
   * 
   * @return Uma decorao do tipo {@link DecorationType#EXECUTION_UNKNOWN}
   */
  private static GraphNodeDecoration createExecutionUnknownDecoration() {
    return createDecoration(DecorationType.EXECUTION_UNKNOWN,
      ApplicationImages.ICON_HELP_16.getImage(),
      "algorithms.tooltip.execution_end.no_code");
  }

  /**
   * Cria uma decorao nula (sem imagem ou descrio)
   * 
   * @return Uma decorao nula
   */
  private static GraphNodeDecoration createNullDecoration() {
    return new GraphNodeDecoration();
  }

  /**
   * Cria uma decorao a partir tipo, da imagem a ser usada e da chave para
   * localizao da string de descrio.
   * 
   * @param type O tipo da decorao
   * @param image A imagem que representa a decorao
   * @param key A chave para localizao da string de descrio da decorao.
   * @return Uma decorao
   */
  private static GraphNodeDecoration createDecoration(DecorationType type,
    Image image, String key) {
    return new GraphNodeDecoration(type, new ImageVO(image), LNG.get(key));
  }

}
