package csbase.algorithms.flows.graph;

import tecgraf.javautils.vix.TypeVS;
import tecgraf.javautils.vix.VO;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.font.TextLayout;
import java.awt.geom.Rectangle2D;

public final class GraphString extends VO {
  private static final int DEFAULT_BOTTOM_PADDING = 5;
  private static final Color DEFAULT_COLOR = Color.BLACK;
  private static final Font DEFAULT_FONT = new Font("helvica", Font.BOLD, 12);
  private static final int DEFAULT_LEFT_PADDING = 10;
  private static final int DEFAULT_RIGHT_PADDING = 10;
  private static final int DEFAULT_TOP_PADDING = 5;
  private static final String ETC = "...";
  private static final double NIL = Double.NEGATIVE_INFINITY;
  private double bottomPadding;
  private double centerX;
  private double centerY;
  private Color color;
  private Font font;
  private double height;
  private double leftPadding;
  private double maxWidth;
  private double rightPadding;
  private String text;
  private double topPadding;
  private String visualText;
  private double width;
  private double x0;
  private double y0;

  public GraphString(TypeVS vs, String text) {
    this(vs, text, DEFAULT_FONT);
  }

  public GraphString(TypeVS vs, String text, Font font) {
    changeVS(null, vs);
    this.visualText = this.text = text;
    this.bottomPadding = DEFAULT_BOTTOM_PADDING;
    this.color = DEFAULT_COLOR;
    this.font = (font != null ? font : DEFAULT_FONT);
    this.leftPadding = DEFAULT_LEFT_PADDING;
    this.maxWidth = Double.POSITIVE_INFINITY;
    this.rightPadding = DEFAULT_RIGHT_PADDING;
    this.topPadding = DEFAULT_TOP_PADDING;
    computeMeasures();
  }

  @Override
  public void callbackRepaint(Graphics2D g) {
    if (this.x0 == NIL || this.y0 == NIL || this.width == NIL
      || this.height == NIL) {
      computeMeasures();
    }
    g.setColor(this.color);
    setGraphicsFont(g);
    g.drawString(toString(), (float) this.x0, (float) this.y0);
  }

  @Override
  public boolean equals(Object object) {
    if (object == null) {
      return false;
    }
    if (!(object instanceof GraphString)) {
      return false;
    }
    GraphString string = (GraphString) object;
    return this.text.equals(string.text);
  }

  public double getBottomPadding() {
    return this.bottomPadding;
  }

  public Rectangle2D getBounds2D() {
    Rectangle2D bounds = new Rectangle();
    bounds.setRect(this.x0 - this.leftPadding, this.y0 - this.topPadding,
      this.width + this.rightPadding + this.leftPadding, this.height
        + this.bottomPadding + this.topPadding);
    return bounds;
  }

  public double getLeftPadding() {
    return this.leftPadding;
  }

  public double getRightPadding() {
    return this.rightPadding;
  }

  public String getText() {
    return this.text;
  }

  public double getTopPadding() {
    return this.topPadding;
  }

  @Override
  public int hashCode() {
    return this.text.hashCode();
  }

  public void setBottomPadding(double bottomPadding) {
    this.bottomPadding = bottomPadding;
  }

  public void setColor(Color color) {
    this.color = (color != null ? color : DEFAULT_COLOR);
  }

  public void setLeftPadding(double leftPadding) {
    this.leftPadding = leftPadding;
  }

  public void setLocation(double centerX, double centerY) {
    this.centerX = centerX;
    this.centerY = centerY;
    computeMeasures();
  }

  public void setMaxWidth(double maxWidth) {
    if (maxWidth > 0) {
      this.maxWidth = maxWidth - this.leftPadding - this.rightPadding;
    }
    else {
      this.maxWidth = Double.POSITIVE_INFINITY;
    }
    computeMeasures();
  }

  public void setRightPadding(double rightPadding) {
    this.rightPadding = rightPadding;
  }

  public void setText(String text) {
    this.text = text;
    setLocation(this.x0 + this.width / 2, this.y0 - this.height / 2);
  }

  public void setTopPadding(double topPadding) {
    this.topPadding = topPadding;
  }

  @Override
  public String toString() {
    return this.visualText;
  }

  private boolean computeMeasures() {
    this.visualText = this.text;
    Graphics2D g2D = this.vs.getGraphics2D();
    if (g2D != null) {
      setGraphicsFont(g2D);
      TextLayout textLayout = new TextLayout(this.visualText, this.font, g2D
        .getFontRenderContext());
      Rectangle2D bounds = textLayout.getBounds();
      this.width = bounds.getWidth();
      double maxTextWidth = this.maxWidth;
      if (this.width > maxTextWidth) {
        TextLayout etcLayout = new TextLayout(ETC, this.font, g2D
          .getFontRenderContext());
        double etcWidth = etcLayout.getBounds().getWidth();
        maxTextWidth = maxTextWidth - etcWidth;
        while (this.width > maxTextWidth) {
          this.visualText = this.visualText.substring(0, this.visualText
            .length() - 1);
          textLayout = new TextLayout(this.visualText, this.font, g2D
            .getFontRenderContext());
          bounds = textLayout.getBounds();
          this.width = bounds.getWidth();
        }
        this.visualText += ETC;
        this.width += etcWidth;
      }
      this.height = bounds.getHeight();
      this.x0 = this.centerX - this.width / 2;
      this.y0 = this.centerY + this.height / 2;
      return true;
    }
    this.x0 = this.y0 = this.width = this.height = NIL;
    return false;
  }

  private void setGraphicsFont(Graphics2D g) {
    g.setFont(this.font);
    g.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
      RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
  }
}
