package csbase.algorithms.flows.graph.actions;

import java.awt.event.ActionEvent;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import javax.swing.ImageIcon;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.algorithms.flows.Workspace;
import csbase.algorithms.flows.filters.AddGraphPopupActionFilter;
import csbase.algorithms.flows.filters.WorkspaceFilter;
import csbase.algorithms.flows.graph.Graph;
import csbase.algorithms.flows.graph.GraphElement;
import csbase.algorithms.flows.graph.GraphNode;
import csbase.client.applications.ApplicationImages;
import csbase.logic.algorithms.flows.configurator.FlowAlgorithmConfigurator;

/**
 * Ao de menu popup que marca o n escolhido para que seja desviado durante a
 * execuo do fluxo. O desvio pode no ser possvel @see {@link GraphNode},
 * {@link FlowAlgorithmConfigurator}.
 * 
 * @author isabella
 */
public class BypassAction extends GraphAction {

  /**
   * Cria a ao.
   * 
   * @param graph Grafo cujo(s) n(s) deve(m) ser desviado(s)/removido(s) do
   *        desvio.
   */
  protected BypassAction(Graph graph) {
    super(graph, getName(graph), getIcon(graph));
    setEnabled(graph.hasSelectedElements());
  }

  /**
   * Determina o texto que aparecer no menu popup, dependendo do estado atual
   * do n. Se o n j estiver desviado, mostra a opo de desfazer o desvio
   * caso contrrio, mostra a opo de fazer o desvio.
   * 
   * @param graph Grafo cujo(s) n(s) deve(m) ser desviado(s)/removido(s) do
   *        desvio.
   * @return o texto a ser mostrado no menu popup.
   */
  private static String getName(Graph graph) {
    String suffix;
    if (areBeingBypassed(graph.getSelectedElements())) {
      suffix = ".undo_bypass";
    }
    else {
      suffix = ".bypass";
    }
    return BypassAction.class.getName() + suffix;
  }

  /**
   * Determina o cone que aparecer no menu popup, dependendo do estado atual
   * do n. Se o n j estiver desviado, mostra o cone de desfazer o desvio
   * caso contrrio, mostra o cone de fazer o desvio.
   * 
   * @param graph Grafo cujo(s) n(s) deve(m) ser desviado(s)/removido(s) do
   *        desvio.
   * @return o cone a ser mostrado no menu popup.
   */
  private static ImageIcon getIcon(Graph graph) {
    if (areBeingBypassed(graph.getSelectedElements())) {
      return ApplicationImages.ICON_TURNON_16;
    }
    else {
      return ApplicationImages.ICON_TURNOFF_16;
    }
  }

  /**
   * Determina se os ns de uma coleo esto sendo desviados ou no. S
   * ser retornado verdadeiro se todos os ns estiverem sendo desviados.
   * 
   * @param elements Elementos que devem ser testados.
   * @return verdadeiro se todos os ns estivem sendo desviados ou falso
   *         caso contrrio.
   */
  private static boolean areBeingBypassed(Collection<GraphElement> elements) {
    for (GraphElement element : elements) {
      if (element instanceof GraphNode) {
        GraphNode node = (GraphNode) element;
        if (!node.canBeBypassed() || !node.isBypassed()) {
          return false;
        }
      }
    }
    return true;
  }

  /**
   * Determina se os ns de uma coleo podem ser desviados ou no. S
   * ser retornado verdadeiro se todos os ns puderem ser desviados.
   * 
   * @param elements Elementos que devem ser testados
   * @return verdadeiro se todos os ns puderem ser desviados ou falso
   *         caso contrrio
   */
  private static boolean canBeBypassed(Collection<GraphElement> elements) {
    for (GraphElement element : elements) {
      if (element instanceof GraphNode) {
        GraphNode node = (GraphNode) element;
        if (!node.canBeBypassed()) {
          return false;
        }
      }
    }
    return true;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void actionPerformed(ActionEvent event) {
    Graph graph = getGraph();
    Set<GraphElement> selectedElements = graph.getSelectedElements();
    boolean canBeBypassed = canBeBypassed(selectedElements);
    boolean areBypassed = areBeingBypassed(selectedElements);
    if (canBeBypassed || areBypassed) {
      boolean bypassActionStatus = true;
      Map<GraphNode, Boolean> memento = new HashMap<GraphNode, Boolean>();
      for (GraphElement element : selectedElements) {
        if (element instanceof GraphNode) {
          GraphNode node = (GraphNode) element;
          memento.put(node, node.isBypassed());
          bypassActionStatus = node.setBypassed(!areBypassed);
          if (!bypassActionStatus) {
            undoBypass(memento);
            break;
          }
        }
      }
    }
    else {
      StandardDialogs
        .showErrorDialog(
          graph.getParentWindow(),
          LNG
            .get("csbase.algorithms.flows.graph.actions.BypassAction.error_title"),
          LNG
            .get("csbase.algorithms.flows.graph.actions.BypassAction.error_msg"));
    }
  }

  /**
   * Desfaz o desvio, utilizando
   * 
   * @param memento Map with previous bystate
   */
  private void undoBypass(Map<GraphNode, Boolean> memento) {
    for (GraphNode node : memento.keySet()) {
      node.setBypassed(memento.get(node));
    }
  }

  /**
   * Cria o filtro que cadastra a ao no menu popup.
   * 
   * @param workspace A rea de trabalho (No aceita {@code null}).
   * 
   * @return O filtro.
   */
  public static WorkspaceFilter createFilter(Workspace workspace) {
    return new AddGraphPopupActionFilter(workspace) {

      /**
       * {@inheritDoc}
       */
      @Override
      protected GraphAction createAction(Graph graph) {
        return new BypassAction(graph);
      }
    };
  }

}
