package csbase.algorithms.flows.graph.utils;

import java.awt.Component;
import java.awt.Window;
import java.awt.geom.Point2D;
import java.text.MessageFormat;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.StandardDialogs;
import tecgraf.javautils.vix.TypeVS;
import csbase.algorithms.flows.graph.Graph;
import csbase.algorithms.flows.graph.GraphFileDescriptor;
import csbase.algorithms.flows.graph.GraphLink;
import csbase.algorithms.flows.graph.GraphNode;
import csbase.algorithms.flows.graph.ParametersTransferable;
import csbase.algorithms.flows.graph.actions.PasteParameterValuesAction;
import csbase.algorithms.flows.messages.ClipboardIsEmptyMessage;
import csbase.client.util.StandardErrorDialogs;
import csbase.exception.BugException;
import csbase.exception.ParseException;
import csbase.exception.algorithms.ParameterNotFoundException;
import csbase.logic.algorithms.AlgorithmConfigurator;

/**
 * Classe com mtodos utilitrios para auxiliar as aes de copiar/colar
 * parmetros e ligaes entre os ns dos grafos.
 * 
 * @author isabella
 */
public class CopyAndPasteUtil {

  /**
   * Testa se tem algo armazenado no clipboard (algo que tenha sido copiado
   * previamente) de um determinado VS
   * 
   * @param vs VS a ser testado
   * @return Verdadeiro se o clipboard est vazio ou falso caso contrrio
   */
  public static boolean isClipboardEmpty(TypeVS vs) {
    ClipboardIsEmptyMessage clipboardIsEmptyMessage = new ClipboardIsEmptyMessage();
    clipboardIsEmptyMessage.sendVS(vs);
    Boolean clipboardIsEmpty = clipboardIsEmptyMessage.clipboardIsEmpty();
    if (clipboardIsEmpty == null) {
      clipboardIsEmpty = false;
    }
    return clipboardIsEmpty;
  }

  /**
   * "Cola" o valor da lista de parmetros em um determinado n. Caso todos os
   * parmetros existam com as mesmas propriedades no n, a operao pode ser
   * feita automaticamente. Caso alguns parmetros no existam, ou tenham
   * propriedades diferentes no n de destino, o usurio ter que confirmar a
   * operao.
   * 
   * @param parameters Lista de parmetros a serem "colados"
   * @param node N no qual os parmetros sero "colados"
   * @return Verdadeiro se os parmetros podem ser "colados" automaticamente ou
   *         o usurio confirmar a operao, ou Falso caso o usurio cancele a
   *         operao.
   */
  public static PasteStatus pasteParameters(ParametersTransferable parameters,
    GraphNode node) {

    if (!node.isWellFormed()) {
      throw new IllegalStateException(LNG.get(PasteParameterValuesAction.class
        .getName()
        + ".warning_bad_node"));
    }
    Map<Object, Object> memento =
      node.getAlgorithmConfiguratorView().getConfigurator().exportValues();
    List<String> warnings = new LinkedList<String>();
    List<String> errors = new LinkedList<String>();

    Map<String, String> parameterValuesByName =
      compareParameters(parameters, node, warnings, errors);

    // Tenta copiar os parmetros para o destino
    try {
      node.setParameterValuesByName(Collections
        .unmodifiableMap(parameterValuesByName));
    }
    catch (ParseException e) {
      throw new BugException(e);
    }

    boolean confirmPaste = true;

    // Mostra os erros e warnings, se houverem
    if (!warnings.isEmpty() || !errors.isEmpty()) {
      Window window = node.getGraph().getParentWindow();

      if (window == null) {
        throw new IllegalStateException("O parmetro window est nulo.");
      }
      if (showOptionPane(window, warnings, errors) == StandardDialogs.NO_OPTION) {
        rollbackPaste(node, memento, window);
        confirmPaste = false;
      }
    }
    return new PasteStatus(warnings, errors, confirmPaste);
  }

  /**
   * Compara uma determinada lista de parmetros com os parmetros do n
   * especificado para determinar se so equivalentes (mesma quantidade de
   * parmetros, mesmas propriedades, etc.). Essa comparao retorna num Mapa,
   * que contm o nome e o valor dos parmetros que so equivalentes. No caso de
   * parmetros que apresentam diferenas entre a lista e o n, so adicionados
   * um registro na lista de avisos ou na lista de erros (dependendo do tipo da
   * diferena identificada), que so passadas ao mtodo. Caso os parmetros
   * sejam equivalentes, essas listas no sofrem modificaes.
   * 
   * @param parameters Lista de parmetros que sero comparados ao n
   * @param node N cujos parmetros sero comparados  lista
   * @param warnings Lista de avisos que descrevem uma diferena entre um
   *        parmetro da lista e do n
   * @param errors Lista de erros que descrevem uma diferena crtica entre um
   *        parmetro da lista e do n
   * @return Mapa com os parmetros considerados equivalentes entre a lista de
   *         parmetros passada e a do n
   */
  private static Map<String, String> compareParameters(ParametersTransferable parameters, GraphNode node, List<String> warnings, List<String> errors) {
    Set<String> parameterNames = parameters.getParameterNames();

    Map<String, String> parameterValuesByName = new HashMap<String, String>();

    for (String parameterName : parameterNames) {
      // Confere se parmetro existe no destino
      if (parameterExists(parameterName, node, warnings)) {
        // Confere se o tipo do parmetro  o mesmo no destino
        String parameterType = parameters.getParameterType(parameterName);
        if (hasSameParameterType(parameterName, node, parameterType, errors)) {
          // Confere se o label do parmetro  o mesmo no destino
          // S um warning  necessrio, o campo pode ser considerado o mesmo
          String parameterLabel = parameters.getParameterLabel(parameterName);
          hasSameParameterLabel(parameterName, node, parameterLabel, warnings);

          String parameterValue = parameters.getParameterValue(parameterName);
          node.addParameter(parameterName, parameterLabel, parameterType);
          parameterValuesByName.put(parameterName, parameterValue);
        }
      }
    }
    return parameterValuesByName;
  }

  /**
   * Compara o label de um parmetro especfico do n, com um determinado label.
   * Caso sejam diferentes,  adicionado um registro na lista de avisos.
   * 
   * @param parameterName Nome do parmetro que ser comparado
   * @param node N cujo parmetro ter seu label comparado
   * @param parameterLabel Label que ser comparado ao do parmetro do n
   * @param warnings Lista de avisos que ser modificada caso os labels sejam
   *        diferentes
   * @return Verdadeiro se os labels so iguais ou Falso caso contrrio
   */
  private static boolean hasSameParameterLabel(String parameterName, GraphNode node, String parameterLabel, List<String> warnings) {
    String nodeParameterLabel;
    try {
      nodeParameterLabel = node.getParameterLabel(parameterName);
    }
    catch (ParameterNotFoundException exception) {
      throw new BugException(exception);
    }
    if (!nodeParameterLabel.equals(parameterLabel)) {
      String warningMessage = MessageFormat.format(LNG.get(PasteParameterValuesAction.class.getName() + ".wrong_labels"), parameterName, parameterLabel, nodeParameterLabel);
      warnings.add(warningMessage);
      return false;
    }
    return true;
  }

  /**
   * Compara o tipo de um parmetro especfico do n, com um determinado tipo.
   * Caso sejam diferentes,  adicionado um registro na lista de erros.
   * 
   * @param parameterName Nome do parmetro que ser comparado
   * @param node N cujo parmetro ter seu tipo comparado
   * @param parameterType Tipo que ser comparado ao do parmetro do n
   * @param errors Lista de erros que ser modificada caso os tipos sejam
   *        diferentes
   * @return Verdadeiro se os tipos so iguais ou Falso caso contrrio
   */
  private static boolean hasSameParameterType(String parameterName, GraphNode node, String parameterType, List<String> errors) {
    String nodeParameterType;
    try {
      nodeParameterType = node.getParameterType(parameterName);
    }
    catch (ParameterNotFoundException exception) {
      throw new BugException(exception);
    }
    if (!nodeParameterType.equals(parameterType)) {
      String errorMessage = MessageFormat.format(LNG.get(PasteParameterValuesAction.class.getName() + ".wrong_types"), parameterName);
      errors.add(errorMessage);
      return false;
    }
    return true;
  }

  /**
   * Determina se um determinado parmetro existe no n. Caso negativo, 
   * adicionado um registro na lista de avisos.
   * 
   * @param parameterName Nome do parmetro que ser testado
   * @param node N que ser testado
   * @param warnings Lista de avisos que ser modificada caso o parmetro no
   *        exista no n
   * @return Verdadeiro se o parmetro existir no n ou Falso caso contrrio
   */
  private static boolean parameterExists(String parameterName, GraphNode node, List<String> warnings) {
    if (!node.getParameterNames().contains(parameterName)) {
      String warningMessage = MessageFormat.format(LNG.get(PasteParameterValuesAction.class.getName() + ".warning_no_parameter"), parameterName);
      warnings.add(warningMessage);
      return false;
    }
    return true;
  }

  /**
   * Mostra uma janela com avisos e erros que so apresentados ao usurio e pede
   * uma confirmao para continuar a operao que estava sendo feita.
   * 
   * @param component O componente visual que solicitou a janela
   * @param warnings Lista de avisos que sero mostrados
   * @param errors Lista de erros que sero mostrados
   * @return Cdigo do boto da janela que foi pressionado pelo usurio
   *         (tecgraf.javautils.gui.StandardDialogs.YES_OPTION ou
   *         tecgraf.javautils.gui.StandardDialogs.NO_OPTION
   */
  private static int showOptionPane(Component component, List<String> warnings, List<String> errors) {
    String title = (LNG.get(CopyAndPasteUtil.class.getName() + ".title"));
    StringBuilder content = new StringBuilder();
    content.append("<html>");
    content.append("<b><i>");
    content.append(LNG.get(CopyAndPasteUtil.class.getName() + ".header"));
    content.append("</i><b>");
    if (!errors.isEmpty()) {
      content.append("<ul>");
      content.append(LNG.get(CopyAndPasteUtil.class.getName() + ".errors"));
      for (String error : errors) {
        content.append("<li>" + error + "</li>");
      }
      content.append("</ul>");
    }
    if (!warnings.isEmpty()) {
      content.append("<ul>");
      content.append(LNG.get(CopyAndPasteUtil.class.getName() + ".warnings"));
      for (String warning : warnings) {
        content.append("<li>" + warning + "</li>");
      }
      content.append("</ul>");
    }
    content.append("</html>");
    return StandardDialogs.showYesNoDialog(component, title, content);
  }

  /**
   * Restaura o estado dos parmetros de um n, aps o cancelamento de uma
   * operao de "colar".
   * 
   * @param node N que deve ser restaurado
   * @param memento Estado prvio do n, que deve ser restaurado
   * @param window Janela utilizada para apresentar erro na restaurao, caso
   *        ocorra
   */
  private static void rollbackPaste(GraphNode node, Map<Object, Object> memento, Window window) {
    AlgorithmConfigurator configurator = node.getAlgorithmConfiguratorView().getConfigurator();
    try {
      configurator.importValues(memento);
    }
    catch (ParseException e) {
      StandardErrorDialogs.showErrorDialog(window, e);
    }
  }

  /**
   * Copia todas as ligaes de um n e "cola" no outro. Os links originais so
   * removidos do grafo se a cpia for bem sucedida.
   * 
   * @param node N que ter suas ligaes copiadas
   * @param newNode N que receber as ligaces copiadas
   */
  public static void copyLinks(GraphNode node, GraphNode newNode) {
    copyFromLinks(node, newNode);
    copyToLinks(node, newNode);
  }

  /**
   * Copia todas as ligaes que <b>saem</b> de um n e "cola" no outro. Os
   * links originais so removidos do grafo se a cpia for bem sucedida.
   * 
   * @param node N que ter suas ligaes copiadas
   * @param newNode N que receber as ligaces copiadas
   */
  public static void copyFromLinks(GraphNode node, GraphNode newNode) {
    Collection<GraphLink> fromLinks = node.getLinkFromCollection();
    for (GraphLink fromLink : fromLinks) {
      GraphNode endNode = fromLink.getEndNode();
      copyLink(fromLink, newNode, endNode);
    }
  }

  /**
   * Copia todas as ligaes que <b>chegam</b> em um n e "cola" no outro. Os
   * links originais so removidos do grafo se a cpia for bem sucedida.
   * 
   * @param node N que ter suas ligaes copiadas
   * @param newNode N que receber as ligaces copiadas
   */
  public static void copyToLinks(GraphNode node, GraphNode newNode) {
    Collection<GraphLink> toLinks = node.getLinkToCollection();
    for (GraphLink toLink : toLinks) {
      GraphNode startNode = toLink.getStartNode();
      copyLink(toLink, startNode, newNode);
    }
  }

  /**
   * Copia o link especificado e o liga nos ns. O link original  removido do
   * grafo se a cpia for bem sucedida.
   * 
   * @param link Ligao a ser copiada
   * @param startNode N que representa a sada da ligao
   * @param endNode N que representa a entrada da ligao
   */
  private static void copyLink(GraphLink link, GraphNode startNode, GraphNode endNode) {
    String outputParamName = link.getOutputFileDescriptor().getParameterName();
    String inputParamName = link.getInputFileDescriptor().getParameterName();
    GraphFileDescriptor inputFileDescriptor = endNode.getInputFileDescriptor(inputParamName);
    GraphFileDescriptor outputFileDescriptor = startNode.getOutputFileDescriptor(outputParamName);
    if (inputFileDescriptor != null && outputFileDescriptor != null) {
      List<Point2D> pointList = link.getIntermediatePointList();
      link.deattach();
      Graph graph = startNode.getGraph();
      graph.createGraphLink(inputFileDescriptor, outputFileDescriptor, pointList);
    }
    else {
      throw new BugException();
    }
  }

}
