package csbase.algorithms.flows.graph.utils;

import java.awt.Dimension;
import java.awt.Window;
import java.awt.geom.Dimension2D;
import java.awt.geom.Point2D;

import csbase.algorithms.flows.graph.Graph;
import csbase.algorithms.flows.graph.GraphNode;
import csbase.algorithms.flows.graph.ParametersTransferable;
import csbase.client.algorithms.AlgorithmConfiguratorFactory;
import csbase.client.algorithms.AlgorithmConfiguratorView;
import csbase.exception.BugException;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.logic.algorithms.AlgorithmVersionInfo;

/**
 * Classe com mtodos utilitrios para auxiliar as aes de atualizao de
 * verses de algoritmos dos ns dos grafos.
 * 
 * @author isabella
 */
public class VersionUtils {

  /**
   * Determina se existe uma verso mais nova para o algoritmo que um
   * determinado n do grafo representa.
   * 
   * @param node N que dever ser testado
   * @return Verdadeiro se o n representa um algoritmo desatualizado
   */
  public static boolean hasNewerNodeVersion(GraphNode node) {
    AlgorithmVersionId nodeVersionId = node.getAlgorithmVersionId();
    AlgorithmInfo algorithm = node.getAlgorithm();
    if (algorithm != null) {
      AlgorithmVersionId latestVersionId = algorithm.getLastVersion().getId();
      if (!nodeVersionId.equals(latestVersionId)) {
        return true;
      }
    }
    return false;
  }

  /**
   * Determina se existe uma verso mais nova para o algoritmo de algum n do
   * grafo.
   * 
   * @param graph Grafo cujos ns devero ser testados
   * @return Verdadeiro se algum n representa um algoritmo dasatualizado, ou
   *         Falso, caso contrrio
   */
  public static boolean hasNewerVersion(Graph graph) {
    if (graph != null) {
      for (GraphNode node : graph.getNodeCollection()) {
        if (hasNewerNodeVersion(node)) {
          return true;
        }
      }
    }
    return false;
  }

  /**
   * Atualiza ns que representam algoritmos desatualizados (que tem pelo menos
   * uma verso mais nova disponvel).
   * 
   * @param graph Grafo que contm os ns a serem atualizados
   */
  public static void updateOutdatedNodes(Graph graph) {
    if (graph != null) {
      for (GraphNode node : graph.getNodeCollection()) {
        if (hasNewerNodeVersion(node)) {
          updateNode(node, false);
        }
      }
    }
  }

  /**
   * Atualiza um determinado n que representa um algoritmo desatualizado (que
   * tem pelo menos uma verso mais nova disponvel). A janela de configurao
   * de parmetros do n  mostrada aps a atualizao.
   * 
   * @param node N que deve ser atualizado
   */
  public static void updateNode(GraphNode node) {
    updateNode(node, true);
  }

  /**
   * Atualiza um determinado n que representa um algoritmo desatualizado (que
   * tem pelo menos uma verso mais nova disponvel). A janela de configurao
   * de parmetros do n  mostrada aps a atualizao no caso de avisos ou
   * erros se askForParameters for verdadeiro.
   * 
   * @param node N que deve ser atualizado
   * @param askForParameters Se verdadeiro, a janela de configurao de
   *        parmetros ser aberta aps a atualizao no caso de avisos ou erros
   */
  public static void updateNode(GraphNode node, boolean askForParameters) {
    if (node != null && node.isWellFormed()) {
      AlgorithmInfo algorithmInfo =
        node.getAlgorithmConfiguratorView().getConfigurator()
          .getAlgorithmVersion().getInfo();
      AlgorithmVersionInfo algorithmVersionInfo =
        algorithmInfo.getLastVersion();
      Graph graph = node.getGraph();
      Window window = graph.getParentWindow();
      AlgorithmConfiguratorView algorithmConfiguratorView =
        AlgorithmConfiguratorFactory.getInstance().createConfigurationView(
          window, algorithmVersionInfo);
      if (algorithmConfiguratorView == null) {
        return;
      }
      Point2D nodeLocation =
        new Point2D.Double(Math.max(0, node.getX()), Math.max(0, node.getY()));
      Dimension2D dimension = new Dimension();
      dimension.setSize(node.getWidth(), node.getHeight());
      GraphNode newNode =
        graph.createGraphNode(algorithmConfiguratorView, nodeLocation,
          dimension, node.isBypassed());
      PasteStatus pasteStatus =
        CopyAndPasteUtil.pasteParameters(new ParametersTransferable(node),
          newNode);
      if (pasteStatus.confirmPaste()) {
        //Se tudo correr bem, copia as ligaes do n e remove o n antigo
        CopyAndPasteUtil.copyLinks(node, newNode);
        node.deattach();
        if (askForParameters
          && !(pasteStatus.getWarnings().isEmpty() && pasteStatus.getErrors()
            .isEmpty())) {
          newNode.askForParameterValues();
        }
      }
      else {
        //rollback
        newNode.deattach();
      }
    }
    else {
      throw new BugException("Node is null or malformed!");
    }
  }
}
