#!/bin/ksh
#
# $Id: functions.ksh 90424 2009-04-13 20:11:38Z costa $

# Exibe uma mensagem de erro para o usurio e aborta a execuo.
#
# $1 = mensagem
function abort
{
    print "[ERROR] $1"
    exit 1
}

# Exibe uma mensagem para o usurio.
#
# $1 = mensagem
function info
{
    print "[INFO ] $1"
}

# Exibe uma mensagem de aviso para o usurio.
#
# $1 = mensagem
function warn
{
    print "[WARN ] $1"
}

# Faz uma pergunta ao usurio, e atribui o resultado  varivel
# especificada. Opcionalmente, usa um valor default.
# Se a varivel de ambiente _ASSUME_DEFAULTS foi definida (i.e. possui
# qualquer valor) e foi fornecido um valor default, usa o mesmo e no
# consulta o usurio. Se no foi fornecido default, sempre consulta o
# usurio.
#
# $1 = mensagem
# $2 = nome da var que vai conter o resultado (opcional)
# $3 = valor default (opcional)
function ask
{
    unset l_opt
    print "$1 \c"
    test -n "$3" && print "[$3] \c"
    # s precisamos do dado se _ASSUME_DEFAULTS no foi definida ou se
    # foi definida mas no existe default
    if [ -z "$_ASSUME_DEFAULTS" -o -z "$3" ]; then
        read l_opt
    else
        # j que no houve interao, simulamos o ENTER
        print
    fi
    if [ -z "$l_opt" ]; then
        l_opt=$3
    fi
    test -n "$2" && eval "$2='$l_opt'"
}

# Tenta mudar a permisso do arquivo para u+w. Se no conseguir, aborta
# a execuo
#
# $1 = arquivo
function makeWritable
{
    chmod u+w $1 2> /dev/null || {
        abort "voce nao tem permissao de escrita no arquivo $1"
    }
}

# Substitui todas as ocorrncias de um determinado padro por um texto
#
# $1 = path para o arquivo
# $2 = texto a ser trocado
# $3 = novo texto
function replaceTxt
{   
    makeWritable $1
    ex $1 > /dev/null << EOF
%s/$2/$3/g
wq!
EOF
}

# Substitui todas as ocorrncias de um determinado caminho por outro
#
# $1 = path para o arquivo
# $2 = caminho a ser trocado
# $3 = novo caminho
function replacePath
{
  makeWritable $1
  ex $1 > /dev/null << EOF
%s*$2*$3*g
wq!
EOF
}

# Substitui um padro apenas em uma linha (a 1a encontrada).
#
# $1 = path para o arquivo
# $2 = padro para localizar a linha desejada
# $3 = texto a ser trocado
# $4 = novo texto
function replaceSingleLine
{
    makeWritable $1
    ex $1 > /dev/null << EOF
/$2/s/$3/$4/g
wq!
EOF
}

# Substitui um padro apenas em uma linha encontrada a partir de um
# padro + deslocamento
#
# $1 = path para o arquivo
# $2 = padro para localizar a linha desejada
# $3 = deslocamento (num. de linhas. p.ex. +1 ou -2)
# $4 = texto a ser trocado
# $5 = novo texto
function replaceSingleLineWithOffset
{
    makeWritable $1
    ex $1 > /dev/null << EOF
/$2/${3}s/$4/$5/g
wq!
EOF
}

# Encontra todas as linhas com um determinado padro e substitui em cada
# uma delas um 2o padro por um texto. Esta funo difere de replaceTxt
# porque aqui os padres de busca das linhas e de substituio do texto
# so distintos.
#
# $1 = path para o arquivo
# $2 = padro para localizar as linhas desejadas
# $3 = texto a ser trocado
# $4 = novo texto
function replaceMultipleLines
{
    makeWritable $1
    ex $1 > /dev/null << EOF
g/$2/s/$3/$4/g
wq!
EOF
}

# Encontra todas as linhas com um determinado padro, aplica um
# deslocamento e substitui em cada uma delas um 2o padro por um texto.
# Esta funo difere de replaceTxt porque aqui os padres de busca das
# linhas e de substituio do texto so distintos.
#
# $1 = path para o arquivo
# $2 = padro para localizar as linhas desejadas
# $3 = deslocamento (num. de linhas. p.ex. +1 ou -2)
# $4 = texto a ser trocado
# $5 = novo texto
function replaceMultipleLinesWithOffset
{
    makeWritable $1
    ex $1 > /dev/null << EOF
g/$2/${3}s/$3/$4/g
wq!
EOF
}

# Remove espaos no incio e no fim de uma determinada string
#
# $1 = string a ser processada
function trim {
    print "$1" | sed 's#^ *\(.*\) *$#\1#'
}

# Converte um path relativo para absoluto.
#
function makeAbsolutePath {
    if [ "/" = `echo $1 | cut -c1` ]; then
        # j  um path absoluto
        print "$1"
    elif [ -d $1 ]; then
        #  um dir com path relativo
        (cd $1; pwd)
    else
        #  um arq com path relativo
        (cd $(dirname $1); print "$PWD/$(basename $1)")
    fi
}

# Copia JARs de um dir para outro, filtrando por um conjunto de nomes.
# As referncias para os JARs podem estar separadas por ":" (da forma
# como  feito no script [CSBASE]/conf/csbase.sh).
# 
# $1 = dir de origem (apenas um)
# $2 = dir de destino
# $3-n = JARs
function copyLibs {
    orig=$1
    dest=$2
    shift 2
    scratch=$(mktemp /tmp/copylib.XXXXXX)
    libs=$(print "$@" | tr : \ )
    for lib in $libs; do
        print "$(basename $lib)" >> $scratch
    done
    cp $(find $orig -type f -name \*.jar | grep -F -f $scratch) $dest
    rm -f $scratch
}

