/**
 * $Id$
 */

package csbase.client.algorithms;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Window;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ScrollPaneConstants;

import tecgraf.javautils.FileUtils;
import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.GUIUtils;
import csbase.client.remote.srvproxies.AlgorithmManagementProxy;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.logic.algorithms.HistoryProvider;

/**
 * Painel de seleo de algoritmo e sua respectiva verso.
 */
public final class AlgorithmChooserPanel extends JPanel {
  private JComboBox algorithmComboBox;
  /* Set<AlgorithmInfo> */
  private SortedSet<AlgorithmInfo> algorithmInfoSet;
  /* Componentes de GUI */
  private JLabel algorithmLabel;
  private JLabel descriptionLabel;
  private JTextArea descriptionTextArea;
  private HelpAlgorithmVersionAction helpVersionAction;
  private JButton helpVersionButton;
  /* List<AlgorithmChooserListener> */
  private List<AlgorithmChooserListener> listenerList;
  private ShowHistoryAction showHistoryAction;
  private JButton showHistoryButton;

  private JComboBox versionComboBox;

  private JLabel versionLabel;

  /**
   * Cria uma painel de seleo de algoritmos. O painel exibe uma lista de
   * algoritmos e suas verses. Tambm exibe a descrio e, opcionalmente, o
   * histrico.
   * 
   * @param window a janela.
   * @param showHistory flag que indica se deve ou no exibir o boto que
   *        permite que o usurio exiba o histrico.
   */
  public AlgorithmChooserPanel(Window window, boolean showHistory) {
    this.listenerList = new LinkedList<AlgorithmChooserListener>();
    this.algorithmInfoSet = new TreeSet<AlgorithmInfo>();
    this.algorithmLabel = new JLabel(getString("algorithmLabel"));
    this.algorithmComboBox = makeAlgorithmComboBox();
    this.versionLabel = new JLabel(getString("versionLabel"));
    this.versionComboBox = makeAlgorithmVersionComboBox();
    this.descriptionLabel = new JLabel(getString("descriptionLabel"));
    this.descriptionTextArea = makeDescriptionTextArea();
    this.helpVersionAction = new HelpAlgorithmVersionAction(window);
    this.helpVersionAction.hideName();
    this.helpVersionButton = new JButton(this.helpVersionAction);
    GUIUtils.trimImageButton(this.helpVersionButton);
    if (showHistory) {
      this.showHistoryAction = new ShowHistoryAction(window);
      this.showHistoryButton = new JButton(this.showHistoryAction);
      this.showHistoryButton.setText("");
      this.showHistoryButton.setToolTipText(getString("showHistoryButton"));
      GUIUtils.trimImageButton(this.showHistoryButton);
    }
    populatePanel();
  }

  /**
   * Adiciona um observador a lista de observadores deste painel. O observador 
   * notificado sempre que um novo algoritmo ou uma nova verso forem
   * selecionados. A ordem de insero dos observadores ser respeitada, logo, o
   * primeiro observador inserido ser o primeiro a receber eventos.
   * 
   * @param listener o observador (no pode ser nulo).
   * 
   * @throws IllegalArgumentException se o listener for nulo.
   */
  public void addAlgorithmChooserListener(AlgorithmChooserListener listener) {
    if (listener == null) {
      throw new IllegalArgumentException("O parmetro listener est nulo.");
    }
    this.listenerList.add(listener);
  }

  /**
   * Obtm o algoritmo de est selecionado.
   * 
   * @return o algoritmo selecionado ou null caso no haja nenhum.
   */
  public AlgorithmInfo getSelectedAlgorithmInfo() {
    return (AlgorithmInfo) this.algorithmComboBox.getSelectedItem();
  }

  /**
   * Obtm a verso do algoritmo que est selecionada.
   * 
   * @return a verso selecionada ou null caso no haja nenhum.
   */
  public AlgorithmVersionInfo getSelectedVersionInfo() {
    return (AlgorithmVersionInfo) this.versionComboBox.getSelectedItem();
  }

  /**
   * Verifica se existem algoritmos disponveis.
   * 
   * @return <code>true</code> se existe ou <code>false</code> caso contrrio.
   */
  public boolean hasAlgorithms() {
    return this.algorithmInfoSet.size() != 0;
  }

  /**
   * Verifica se existem algoritmos disponveis.
   * 
   * @return <code>true</code> se existe ou <code>false</code> caso contrrio.
   */
  public boolean hasVersions() {
    return this.versionComboBox.getModel().getSize() != 0;
  }

  /**
   * Seleciona um algoritmo.
   * 
   * @param algorithmName o nome do algoritmo a ser selecionado
   * 
   * @return {@code true} em caso de sucesso ou {@code false} se no houver o
   *         algoritmo.
   */
  public boolean selectAlgorithm(String algorithmName) {
    for (Iterator<AlgorithmInfo> iterator = this.algorithmInfoSet.iterator(); iterator
      .hasNext();) {
      AlgorithmInfo algorithmInfo = iterator.next();
      if (algorithmInfo.getName().equals(algorithmName)) {
        this.algorithmComboBox.setSelectedItem(algorithmInfo);
        return true;
      }
    }
    return false;
  }

  /**
   * Seleciona a verso do algoritmo.
   * 
   * @param versionId o identificador da verso.
   * 
   * @return {@code true} em caso de sucesso ou {@code false} se no houver o
   *         algoritmo ou a verso.
   */
  public boolean selectVersion(AlgorithmVersionId versionId) {
    AlgorithmInfo algorithmInfo = getSelectedAlgorithmInfo();
    if (algorithmInfo == null) {
      return false;
    }
    AlgorithmVersionInfo versionInfo = algorithmInfo.getVersionInfo(versionId);
    if (versionInfo == null) {
      return false;
    }
    this.versionComboBox.setSelectedItem(versionInfo);
    return true;
  }

  /**
   * Estabelece um conjunto de algoritmos que sero exibidos na lista de
   * seleo.
   * 
   * @param algorithmInfoSet um conjunto de <code>AlgorithmInfo</code>
   */
  public void setAlgorithmSet(Set<AlgorithmInfo> algorithmInfoSet) {
    this.algorithmInfoSet = new TreeSet<AlgorithmInfo>(algorithmInfoSet);
    loadAlgorithms();
  }

  /**
   * Estabelece um conjunto de verses que devem ser exibidas na lista de
   * seleo de verses de um algoritmo.
   * 
   * @param algorithmVersionSet um conjunto de <code>AlgorithmVersionInfo</code>
   */
  public void setAlgorithmVersion(Set<AlgorithmVersionInfo> algorithmVersionSet) {
    loadVersions(new TreeSet<AlgorithmVersionInfo>(algorithmVersionSet));
  }

  /**
   * Atribui uma descrio para ser exibida na rea de texto correspondente.
   * 
   * @param description o texto da descrio
   */
  public void setDescription(String description) {
    this.descriptionTextArea.setText(description);
    this.descriptionTextArea.setCaretPosition(0);
  }

  /**
   * Notifica aos listeners que um novo algoritmo foi selecionado.
   */
  private void fireChangedAlgorithm() {
    Iterator<AlgorithmChooserListener> listenerIterator =
      this.listenerList.iterator();
    while (listenerIterator.hasNext()) {
      AlgorithmChooserListener listener = listenerIterator.next();
      listener.wasChangedAlgorithm(new AlgorithmChooserEvent(this));
    }
  }

  /**
   * Notifica aos listeners que uma nova verso foi selecionada.
   */
  private void fireChangedVersion() {
    Iterator<AlgorithmChooserListener> listenerIterator =
      this.listenerList.iterator();
    while (listenerIterator.hasNext()) {
      AlgorithmChooserListener listener = listenerIterator.next();
      listener.wasChangedVersion(new AlgorithmChooserEvent(this));
    }
  }

  private String getString(String key) {
    return LNG.get(AlgorithmChooserPanel.class.getName() + "." + key);
  }

  /**
   * Carrega o combo box dos algoritmos.
   */
  private void loadAlgorithms() {
    this.algorithmComboBox.removeAllItems();
    for (AlgorithmInfo algorithm : this.algorithmInfoSet) {
      this.algorithmComboBox.addItem(algorithm);
    }
  }

  /**
   * Carrega o combo box das verses, de acordo com o algoritmo selecionado.
   */
  private void loadVersions() {
    AlgorithmInfo info = this.getSelectedAlgorithmInfo();
    SortedSet<AlgorithmVersionInfo> versions =
      new TreeSet<AlgorithmVersionInfo>(info.getVersions());
    this.versionComboBox.removeAllItems();
    for (AlgorithmVersionInfo version : versions) {
      this.versionComboBox.addItem(version);
    }
  }

  /**
   * Carrega um determinado conjunto de verses que devem ser exibidas na lista
   * de seleo de verses de um algoritmo.
   * 
   * @param versions conjunto de verses a serem exibidas para o algoritmo
   */
  private void loadVersions(Set<AlgorithmVersionInfo> versions) {
    this.versionComboBox.removeAllItems();
    for (AlgorithmVersionInfo version : versions) {
      this.versionComboBox.addItem(version);
    }
  }

  /**
   * Cria um componente para exibir a lista dos algoritmos disponveis para
   * seleo.
   * 
   * @return o combo box para a lista de algoritmos
   */
  private JComboBox makeAlgorithmComboBox() {
    JComboBox combo = new JComboBox();
    combo.addItemListener(new AlgorithmComboBoxItemListener());
    return combo;
  }

  /**
   * Cria um componente para exibir a lista das verses do algoritmo
   * selecionado.
   * 
   * @return o combo box para a lista das verses do algoritmo
   */
  private JComboBox makeAlgorithmVersionComboBox() {
    JComboBox combo = new JComboBox();
    combo.addItemListener(new VersionComboBoxItemListener());
    return combo;
  }

  /**
   * Cria o componente para exibir a descrio da verso do algoritmo.
   * 
   * @return o componente de descrio da verso do algoritmo.
   */
  private JTextArea makeDescriptionTextArea() {
    // Os valores para linhas e colunas visam garantir uma uniformidade das
    // dimenses do componente em todos os pontos do sistema em que este 
    // usado.
    JTextArea descriptionTxtArea = new JTextArea(3, 35);
    descriptionTxtArea.setLineWrap(true);
    descriptionTxtArea.setWrapStyleWord(true);
    descriptionTxtArea.setBackground(getBackground());
    descriptionTxtArea.setEditable(false);
    return descriptionTxtArea;
  }

  /**
   * Arruma os componentes no painel.
   */
  private void populatePanel() {
    setLayout(new GridBagLayout());
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.anchor = GridBagConstraints.NORTHWEST;
    constraints.fill = GridBagConstraints.NONE;
    constraints.gridheight = 1;
    constraints.gridwidth = 1;
    constraints.gridx = 1;
    constraints.gridy = 1;
    constraints.insets = new Insets(2, 2, 2, 2);
    constraints.weightx = 0.0;
    constraints.weighty = 0.0;
    add(this.algorithmLabel, constraints);
    constraints.fill = GridBagConstraints.HORIZONTAL;
    constraints.weightx = 1.0;
    constraints.gridx = 2;
    constraints.gridy = 1;
    constraints.gridwidth = 3;
    add(this.algorithmComboBox, constraints);
    constraints.fill = GridBagConstraints.NONE;
    constraints.weightx = 0.0;
    constraints.gridx = 1;
    constraints.gridy = 2;
    constraints.gridwidth = 1;
    add(this.versionLabel, constraints);
    constraints.fill = GridBagConstraints.HORIZONTAL;
    constraints.weightx = 1.0;
    constraints.gridx = 2;
    constraints.gridy = 2;
    add(this.versionComboBox, constraints);
    if (this.showHistoryButton != null) {
      constraints.fill = GridBagConstraints.NONE;
      constraints.weightx = 0.0;
      constraints.gridx++;
      add(this.showHistoryButton, constraints);
    }
    constraints.fill = GridBagConstraints.NONE;
    constraints.weightx = 0.0;
    constraints.gridx++;
    add(this.helpVersionButton, constraints);
    constraints.fill = GridBagConstraints.NONE;
    constraints.weightx = 0.0;
    constraints.gridx = 1;
    constraints.gridy = 3;
    constraints.gridwidth = 1;
    add(this.descriptionLabel, constraints);
    constraints.fill = GridBagConstraints.BOTH;
    constraints.weightx = 1.0;
    constraints.weighty = 2.0;
    constraints.gridx = 2;
    constraints.gridy = 3;
    constraints.gridwidth = 3;
    JScrollPane descPane = new JScrollPane(this.descriptionTextArea);
    descPane
      .setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
    add(descPane, constraints);
  }

  /**
   * Atualiza a ao ajuda com a verso selecionada.
   */
  private void updateHelpAction() {
    this.helpVersionAction.setAlgorithmVersionInfo(this
      .getSelectedVersionInfo());
  }

  /**
   * Atualiza a ao de exibio de histrico com a verso selecionada.
   */
  private void updateShowHistoryAction() {
    if (this.showHistoryAction != null) {
      HistoryProvider provider =
        new AlgorithmVersionHistoryProvider(this.getSelectedVersionInfo());
      this.showHistoryAction.setHistoryProvider(provider);
    }
  }

  /**
   * Trata eventos de seleo de um algoritmo.  o listener usado no componente
   * de exibio da lista de algoritmos.
   */
  private final class AlgorithmComboBoxItemListener implements ItemListener {
    /**
     * 
     * @see ItemListener#itemStateChanged(ItemEvent)
     */
    public void itemStateChanged(ItemEvent e) {
      if (e.getStateChange() == ItemEvent.SELECTED) {
        fireChangedAlgorithm();
        loadVersions();
      }
    }
  }

  /**
   * Objeto provedor de histrico para verses de algoritmos.
   */
  private final class AlgorithmVersionHistoryProvider implements
    HistoryProvider {
    /** Verso da qual pretende-se obter o histrico */
    private AlgorithmVersionInfo versionInfo;

    /**
     * Cria o objeto provedor de histrico.
     * 
     * @param versionInfo verso da qual pretende-se obter o histrico.
     */
    public AlgorithmVersionHistoryProvider(AlgorithmVersionInfo versionInfo) {
      this.versionInfo = versionInfo;
    }

    public List getHistory() {
      String[] path = getVersionPath();
      return AlgorithmManagementProxy.retrieveHistory(path);
    }

    public String getName() {
      return FileUtils.joinPath('/', getVersionPath());
    }

    /**
     * Obtm o caminho da verso.
     * 
     * @return caminho da verso.
     */
    private String[] getVersionPath() {
      String[] path =
        { AlgorithmInfo.REPOSITORY_NAME, this.versionInfo.getInfo().getName(),
            this.versionInfo.getId().toString() };
      return path;
    }
  }

  /**
   * Trata eventos de seleo de uma verso de algoritmo.  o listener usado no
   * componente de exibio da lista de verses.
   */
  private final class VersionComboBoxItemListener implements ItemListener {
    /**
     * 
     * @see ItemListener#itemStateChanged(ItemEvent)
     */
    public void itemStateChanged(ItemEvent e) {
      if (e.getStateChange() == ItemEvent.SELECTED) {
        updateHelpAction();
        updateShowHistoryAction();
        fireChangedVersion();
      }
    }
  }
}
