package csbase.client.algorithms;

import java.awt.Window;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.text.MessageFormat;

import tecgraf.javautils.LNG;
import csbase.algorithms.flows.configurator.FlowAlgorithmConfiguratorView;
import csbase.client.algorithms.parameters.ParameterView;
import csbase.client.algorithms.parameters.SimpleAlgorithmConfiguratorView;
import csbase.client.algorithms.parameters.ParameterView.Mode;
import csbase.client.desktop.RemoteTask;
import csbase.client.kernel.ClientException;
import csbase.exception.BugException;
import csbase.logic.algorithms.AlgorithmConfigurator;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.logic.algorithms.flows.configurator.FlowAlgorithmConfigurator;
import csbase.logic.algorithms.parameters.SimpleAlgorithmConfigurator;
import csbase.remote.ClientRemoteLocator;

/**
 * Fbrica de vises de configuradores de algoritmo.
 */
public final class AlgorithmConfiguratorFactory {
  /**
   * Constante que informa qual  o nome da classe que implementa a viso de um
   * configurador escrito em java.
   */
  private static final String CONFIGURATOR_VIEW_CLASS_NAME = "View";

  /**
   * A nica instncia desta classe.
   */
  private static AlgorithmConfiguratorFactory instance;

  /**
   * Cria a fbrica.
   */
  private AlgorithmConfiguratorFactory() {
  }

  /**
   * Obtm a instncia desta classe.
   * 
   * @return .
   */
  public static AlgorithmConfiguratorFactory getInstance() {
    if (instance == null) {
      instance = new AlgorithmConfiguratorFactory();
    }
    return instance;
  }

  /**
   * Cria a viso do configurador no modo de configurao
   * {@link ParameterView.Mode#REPORT}.
   * 
   * @param owner A janela que ser dona da viso (No aceita {@code null}).
   * @param configurator O configurador do algoritmo (No aceita {@code null}).
   * 
   * @return A viso.
   */
  public AlgorithmConfiguratorView createReportView(final Window owner,
    AlgorithmConfigurator configurator) {
    if (owner == null) {
      throw new IllegalArgumentException("O parmetro owner est nulo.");
    }
    if (configurator == null) {
      throw new IllegalArgumentException("O parmetro configurator est nulo.");
    }
    return createView(owner, configurator, Mode.REPORT);
  }

  /**
   * Cria a viso do configurador no modo de configurao
   * {@link ParameterView.Mode#CONFIGURATION}.
   * 
   * @param owner A janela que ser dona da viso (No aceita {@code null}).
   * @param version A verso do algoritmo.
   * 
   * @return A viso.
   */
  public AlgorithmConfiguratorView createConfigurationView(final Window owner,
    final AlgorithmVersionInfo version) {
    if (owner == null) {
      throw new IllegalArgumentException("O parmetro owner est nulo.");
    }
    if (version == null) {
      throw new IllegalArgumentException("O parmetro version est nulo.");
    }
    RemoteTask<AlgorithmConfigurator> task =
      new RemoteTask<AlgorithmConfigurator>() {

        @Override
        protected void performTask() throws Exception {
          String algorithmName = version.getInfo().getName();
          AlgorithmVersionId algorithmVersionId = version.getId();
          AlgorithmConfigurator algorithmConfigurator =
            ClientRemoteLocator.algorithmService.createAlgorithmConfigurator(
              algorithmName, algorithmVersionId);
          setResult(algorithmConfigurator);
        }
      };
    if (task.execute(owner, getString("reading_configurator_title"),
      getString("reading_configurator"))) {
      AlgorithmConfigurator algorithmConfigurator = task.getResult();
      return createView(owner, algorithmConfigurator, Mode.CONFIGURATION);
    }
    else {
      return null;
    }
  }

  /**
   * Cria a viso do configurador no modo escolhido.
   * 
   * @param owner A janela que ser dona da viso (No aceita {@code null}).
   * @param algorithmConfigurator O configurador do algoritmo (No aceita
   *        {@code null}).
   * @param mode Modo de visualizao do configurador. No aceita {@code null},
   *        os possveis valores so: {@link ParameterView.Mode#CONFIGURATION}
   *        ou {@link ParameterView.Mode#REPORT}
   * 
   * @return A viso.
   */
  private AlgorithmConfiguratorView createView(final Window owner,
    final AlgorithmConfigurator algorithmConfigurator, Mode mode) {
    if (mode == null) {
      throw new IllegalArgumentException("O parmetro mode est nulo.");
    }
    if (algorithmConfigurator == null) {
      throw new IllegalArgumentException(
        "O parmetro algorithmConfigurator est nulo.");
    }

    AlgorithmConfiguratorView configuratorView = null;
    try {
      if (algorithmConfigurator instanceof SimpleAlgorithmConfigurator) {
        SimpleAlgorithmConfigurator simpleAlgorithmConfigurator =
          (SimpleAlgorithmConfigurator) algorithmConfigurator;
        configuratorView =
          new SimpleAlgorithmConfiguratorView(owner,
            simpleAlgorithmConfigurator, mode);
      }
      else if (algorithmConfigurator instanceof FlowAlgorithmConfigurator) {
        FlowAlgorithmConfigurator flowAlgorithmConfigurator =
          (FlowAlgorithmConfigurator) algorithmConfigurator;
        configuratorView =
          new FlowAlgorithmConfiguratorView(owner, flowAlgorithmConfigurator,
            mode);
      }
      else {
        configuratorView =
          createCustomAlgorithmConfiguratorView(owner, algorithmConfigurator
            .getAlgorithmVersion(), algorithmConfigurator, mode);
      }
    }
    catch (ClientException e) {
      throw new BugException(getString("error_creating_configurator_view",
        algorithmConfigurator.getAlgorithmName()), e);
    }
    return configuratorView;
  }

  /**
   * Cria uma viso de configurador personalizada, ou seja, de configuradores
   * escritos em java.
   * 
   * @param owner A janela que ser dona da viso (No aceita {@code null}).
   * @param version A verso do algoritmo.
   * @param configurator O configurador.
   * @param mode
   * 
   * @return A viso ou {@code null} se no existir uma viso personalizada para
   *         esta verso.
   */
  private AlgorithmConfiguratorView createCustomAlgorithmConfiguratorView(
    Window owner, AlgorithmVersionInfo version,
    AlgorithmConfigurator configurator, Mode mode) {
    String configuratorViewClassName =
      version.getPackageName() + "." + CONFIGURATOR_VIEW_CLASS_NAME;
    Class<?> configuratorViewClass;
    try {
      configuratorViewClass = Class.forName(configuratorViewClassName);
    }
    catch (ClassNotFoundException e) {
      return null;
    }
    if (!AlgorithmConfiguratorView.class
      .isAssignableFrom(configuratorViewClass)) {
      throw new BugException(getString("error_wrong_component_type",
        configuratorViewClassName, AlgorithmConfiguratorView.class.getName()));
    }
    Constructor<?> configuratorViewConstructor;
    try {
      configuratorViewConstructor =
        configuratorViewClass.getConstructor(new Class[] { Window.class,
            configurator.getClass(), mode.getClass() });
    }
    catch (NoSuchMethodException e) {
      throw new BugException(getString("error_wrong_component_no_constructor",
        configuratorViewClassName, CONFIGURATOR_VIEW_CLASS_NAME, Window.class
          .getName(), configurator.getClass().getName()));
    }
    AlgorithmConfiguratorView configuratorView;
    try {
      configuratorView =
        (AlgorithmConfiguratorView) configuratorViewConstructor
          .newInstance(new Object[] { owner, configurator, mode });
    }
    catch (InstantiationException e) {
      throw new BugException(getString("error"), e);
    }
    catch (IllegalAccessException e) {
      throw new BugException(getString("error_wrong_component_no_constructor",
        configuratorViewClassName), e);
    }
    catch (InvocationTargetException e) {
      throw new BugException(getString("error"), e.getTargetException());
    }
    return configuratorView;
  }

  /**
   * <p>
   * Obtm uma mensagem no mecanismo de internacionalizao.
   * </p>
   * 
   * <p>
   *  um informado um sufixo ({@code suffixKey}) que  anexado ao nome da
   * classe.
   * </p>
   * 
   * @param suffixKey O sufixo da chave (No aceita {@code null}).
   * @param arguments
   * 
   * @return A mensagem.
   */
  private String getString(String suffixKey, Object... arguments) {
    return MessageFormat.format(LNG.get(AlgorithmConfiguratorFactory.class
      .getName()
      + "." + suffixKey), arguments);
  }
}
