/*
 * $Author:$ $Date:$ $Release:$
 */
package csbase.client.algorithms;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.text.MessageFormat;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;

import tecgraf.javautils.LNG;
import tecgraf.javautils.configurationmanager.Configuration;
import tecgraf.javautils.configurationmanager.ConfigurationManager;
import tecgraf.javautils.gui.GBC;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.algorithms.parameters.ParameterView;
import csbase.client.algorithms.parameters.ParameterView.Mode;
import csbase.client.algorithms.parameters.validators.ValidationMode;
import csbase.client.algorithms.parameters.validators.ValidationResult;
import csbase.client.desktop.DesktopComponentDialog;
import csbase.client.desktop.DesktopFrame;
import csbase.client.project.ProjectTreePath;
import csbase.client.util.ClientUtilities;
import csbase.exception.ParseException;
import csbase.logic.CommonClientProject;
import csbase.logic.algorithms.AlgorithmConfigurator;
import csbase.logic.algorithms.AlgorithmConfiguratorListener;
import csbase.logic.algorithms.AlgorithmVersionInfo;

/**
 * Viso do Configurador de Algoritmos.
 * 
 * @author lmoreira
 */
public abstract class AlgorithmConfiguratorView {
  /**
   * Largura mnima.
   */
  private static final int MINIMUM_WIDTH = 600;
  /**
   * Altura mnima.
   */
  private static final int MINIMUM_HEIGHT = 100;

  /**
   * O Boto Cancelar.
   */
  private JButton cancelButton;
  /**
   * O Boto Fechar.
   */
  private JButton closeButton;
  /**
   * O Configurador Lgico.
   */
  private AlgorithmConfigurator configurator;
  /**
   * O Boto Confirmar.
   */
  private JButton confirmButton;
  /**
   * O caminho do arquivo de parmetro corrente.
   */
  private ProjectTreePath currentPath;
  /**
   * O dilogo.
   */
  private DesktopComponentDialog dialog;
  /**
   * O Boto Ajuda.
   */
  private JButton helpButton;
  /**
   * Os observadores.
   */
  private final List<AlgorithmConfiguratorViewListener> listeners;

  /**
   * A janela que  dona do dilogo.
   */
  private Window owner;
  /**
   * O <i>memento</i> dos valores dos parmetros.
   */
  private Map<Object, Object> previousValues;
  /**
   * O Boto Restaurar.
   */
  private JButton restoreButton;

  /**
   * O Modo de Validao.
   */
  private ValidationMode validationMode;

  /**
   * Modo de visualizao do configurador. No aceita {@code null}, os possveis
   * valores so: {@link ParameterView.Mode#CONFIGURATION} ou
   * {@link ParameterView.Mode#REPORT}
   */
  private Mode mode;

  /**
   * Cria a viso.
   * 
   * @param owner A janela que  dona deste dilogo (No aceita {@code null}).
   * @param configurator O configurador lgico (No aceita {@code null}).
   * @param mode Modo de visualizao do configurador. No aceita {@code null},
   *        os possveis valores so: {@link ParameterView.Mode#CONFIGURATION}
   *        ou {@link ParameterView.Mode#REPORT}
   */
  protected AlgorithmConfiguratorView(Window owner,
    AlgorithmConfigurator configurator, ParameterView.Mode mode) {
    if (mode == null) {
      throw new IllegalArgumentException("O parmetro mode est nulo.");
    }
    this.mode = mode;
    this.listeners = new LinkedList<AlgorithmConfiguratorViewListener>();
    setConfigurator(configurator);
    setOwner(owner);
    createDialog();
    createButtons();
  }

  /**
   * Adiciona um observador.
   * 
   * @param listener O observador (No aceita {@code null}).
   */
  public final void addAlgorithmConfiguratorViewListener(
    AlgorithmConfiguratorViewListener listener) {
    if (listener == null) {
      throw new IllegalArgumentException("O parmetro listener est nulo.");
    }
    this.listeners.add(listener);
  }

  /**
   * <p>
   * Obtm os observadores.
   * </p>
   * 
   * <p>
   * A lista retornada  imutvel (veja
   * {@link Collections#unmodifiableList(List)}).
   * </p>
   * 
   * @return Os observadores (Se no houver observadores, retorna uma lista
   *         vazia).
   */
  public final List<AlgorithmConfiguratorViewListener> getAlgorithmConfiguratorViewListeners() {
    return Collections.unmodifiableList(this.listeners);
  }

  /**
   * Obtm o configurador lgico.
   * 
   * @return .
   */
  public AlgorithmConfigurator getConfigurator() {
    return this.configurator;
  }

  /**
   * Obtm o caminho do arquivo de parmetro corrente.
   * 
   * @return .
   */
  public final ProjectTreePath getCurrentPath() {
    return this.currentPath;
  }

  /**
   * @return Obtm o projeto corrente.
   */
  public final CommonClientProject getProject() {
    CommonClientProject currentProject =
      DesktopFrame.getInstance().getProject();
    return currentProject;
  }

  /**
   * Obtm o modo de validao atual.
   * 
   * @return O modo de validao atual.
   */
  public final ValidationMode getValidationMode() {
    if (this.validationMode == null) {
      setValidationMode(ValidationMode.DEFAULT);
    }
    return this.validationMode;
  }

  /**
   * Exibe o dalodo.
   */
  public final void launch() {
    this.previousValues = this.configurator.exportValues();
    populateDialog();
    this.dialog.center(this.owner);
    this.dialog.setVisible(true);
  }

  /**
   * Modifica o modo de validao atual.
   * 
   * @param validationMode O modo de validao atual (No aceita {@code null}).
   */
  public void setValidationMode(ValidationMode validationMode) {
    if (validationMode == null) {
      throw new IllegalArgumentException(
        "O parmetro validationMode est nulo.");
    }
    this.validationMode = validationMode;
  }

  /**
   * Atribui o caminho do arquivo de parmetro corrente.
   * 
   * @param currentPath O caminho do arquivo (Aceita {@code null}).
   */
  public void setCurrentPath(ProjectTreePath currentPath) {
    this.currentPath = currentPath;
  }

  /**
   * Habilita/Desabilita o configurador de algoritmos.
   * 
   * @param isEnabled {@code true}, habilita; {@code false}, desabilita.
   * 
   * @return {@code true}, sucesso; {@code false}, caso no haja mudana de
   *         estado.
   */
  public final boolean setEnabled(boolean isEnabled) {
    return configurator.setEnabled(isEnabled);
  }

  /**
   * @see java.lang.Object#toString()
   */
  @Override
  public String toString() {
    return getConfigurator().toString();
  }

  /**
   * Validador os valores atuais dos parmetros.
   * 
   * @param complain Indica se deve exibir dilogos quando houver um erro.
   * 
   * @return o resultado da validao.
   */
  public abstract ValidationResult validate(boolean complain);

  /**
   * Obtm o dilogo.
   * 
   * @return O dilogo.
   */
  public final DesktopComponentDialog getDialog() {
    return this.dialog;
  }

  /**
   * Obtm o componente principal que  exibido no dilogo do configurador de
   * algoritmos.
   * 
   * @return .
   */
  public abstract JComponent getMainComponent();

  /**
   * Obtm a janela que  dona do dilogo deste configurador.
   * 
   * @return .
   */
  protected final Window getOwner() {
    return this.owner;
  }

  /**
   * Reconecta os componentes ao dilogo.
   */
  protected final void repopulateDialog() {
    populateDialog();
    updateLocation();
  }

  /**
   * Cancela a edio dos parmetros.
   */
  private void cancel() {
    if (this.previousValues != null) {
      try {
        this.configurator.importValues(this.previousValues);
      }
      catch (ParseException e) {
        StandardDialogs.showErrorDialog(getDialog(),
          getString("title_wrong_parameters"), e.getLocalizedMessage());
      }
    }
    close();
    for (AlgorithmConfiguratorViewListener listener : this.listeners) {
      listener.wasCancelled(this);
    }
  }

  /**
   * Fecha o dilogo.
   */
  private void close() {
    this.previousValues = null;
    this.dialog.close();
  }

  /**
   * Confirmar a edio dos valores dos parmetros.
   */
  private void confirm() {
    ValidationResult validation = validate(true);
    if (validation.isWellSucceded()) {
      close();
      for (AlgorithmConfiguratorViewListener listener : this.listeners) {
        listener.wasConfirmed(this);
      }
    }
  }

  /**
   * Cria os botes.
   */
  private void createButtons() {
    this.cancelButton = new JButton(getString("cancelButton"));
    this.cancelButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        cancel();
      }
    });
    this.closeButton = new JButton(getString("closeButton"));
    this.closeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        close();
      }
    });
    this.confirmButton = new JButton(getString("confirmButton"));
    this.confirmButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        confirm();
      }
    });
    AlgorithmVersionInfo algorithmVersionInfo =
      getConfigurator().getAlgorithmVersion();
    HelpAlgorithmVersionAction helpAlgorithmVersionAction;
    if (algorithmVersionInfo == null) {
      helpAlgorithmVersionAction = new HelpAlgorithmVersionAction(this.dialog);
    }
    else {
      helpAlgorithmVersionAction =
        new HelpAlgorithmVersionAction(this.dialog, algorithmVersionInfo);
    }
    helpAlgorithmVersionAction.hideIcon();
    this.helpButton = new JButton(helpAlgorithmVersionAction);
    this.restoreButton = new JButton(getString("resetDefaultValues"));
    this.restoreButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        getConfigurator().resetValues();
      }
    });
  }

  /**
   * Cria o dilogo.
   */
  private void createDialog() {
    this.dialog = DesktopComponentDialog.createDialog(this.owner);
    this.dialog.addWindowListener(new WindowAdapter() {
      @Override
      public void windowClosing(WindowEvent e) {
        if (configurator.isEnabled()) {
          cancel();
        }
        else {
          close();
        }
      }
    });
    this.dialog.setModal(false);
  }

  /**
   * Cria o painel externo de botes.
   * 
   * @return O painel externo.
   */
  private JPanel createOuterButtonPanel() {
    FlowLayout flowLayout = new FlowLayout();
    JPanel outerButtonPanel = new JPanel(flowLayout);
    if (configurator.isEnabled()) {
      flowLayout.setAlignment(FlowLayout.CENTER);
      ClientUtilities.adjustEqualSizes(this.cancelButton, this.confirmButton,
        this.helpButton);
      outerButtonPanel.add(this.confirmButton);
      outerButtonPanel.add(this.cancelButton);
      outerButtonPanel.add(this.helpButton);
    }
    else {
      flowLayout.setAlignment(FlowLayout.CENTER);
      outerButtonPanel.add(this.closeButton);
    }
    return outerButtonPanel;
  }

  /**
   * Obtm um texto no mecanismo de internacionalizao.
   * 
   * @param key A chave para o texto (Ser prfixado o nome deste classe seguido
   *        de '.'; no aceita {@code null}).
   * 
   * @return O texto.
   */
  private String getString(String key) {
    return LNG.get(AlgorithmConfiguratorView.class.getName() + "." + key);
  }

  /**
   * Obtm um texto no mecanismo de internacionalizao.
   * 
   * @param key A chave para o texto (Ser prfixado o nome deste classe seguido
   *        de '.'; no aceita {@code null}).
   * @param keyArgs Os argumentos para montar a mensagem.
   * 
   * @return O texto.
   */
  private String getString(String key, Object... keyArgs) {
    return MessageFormat.format(getString(key), keyArgs);
  }

  /**
   * Liga os componentes ao painel principal.
   */
  private void populateDialog() {
    JPanel contentPanel = new JPanel(new BorderLayout());

    JComponent mainComponent = getMainComponent();
    JPanel mainPanel = new JPanel(new GridBagLayout());
    GBC gbc = new GBC(0, 0).both().insets(new Insets(12, 12, 11, 11));
    mainPanel.add(mainComponent, gbc);
    contentPanel.add(mainPanel, BorderLayout.CENTER);

    JPanel outerButtonPanel = createOuterButtonPanel();
    contentPanel.add(outerButtonPanel, BorderLayout.SOUTH);
    this.dialog.setContentPane(contentPanel);
    this.dialog.pack();
    updateSize();
    updateTitle();
  }

  /**
   * Atribui o configurador lgico a esta viso.
   * 
   * @param configurator O configurador (No aceita {@code null}).
   */
  private void setConfigurator(AlgorithmConfigurator configurator) {
    if (configurator == null) {
      throw new IllegalArgumentException("O parmetro configurator est nulo.");
    }
    this.configurator = configurator;
    this.configurator
      .addAlgorithmConfiguratorListener(new AlgorithmConfiguratorListener() {
      public void parameterLabelWasChanged(
        AlgorithmConfigurator configurator, String parameterName,
        String parameterLabel) {
      }

      public <V> void parameterValueWasChanged(
        AlgorithmConfigurator configurator, String parameterName,
        V parameterValue) {
      }

      public void parameterWasSetEnabled(AlgorithmConfigurator configurator,
        String parameterName, boolean parameterIsEnabled) {
      }

      public void parameterWasSetVisible(AlgorithmConfigurator configurator,
        String parameterName, boolean parameterIsVisible) {
        resizeOnVisibilityChanged();
      }

      public void wasSetEnabled(AlgorithmConfigurator configurator) {
        repopulateDialog();
      }
    });
  }

  /**
   * Atribui a janela que  dona deste dilogo.
   * 
   * @param owner A janela (No aceita {@code null}).
   */
  private void setOwner(Window owner) {
    if (owner == null) {
      throw new IllegalArgumentException("O parmetro owner est nulo.");
    }
    this.owner = owner;
  }

  /**
   * Ajusta a localizao do dilogo para que ele no passe dos limites do
   * monitor.
   */
  private void updateLocation() {
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    Dimension currentSize = this.dialog.getSize();
    double usuableWidth = screenSize.getWidth();
    double currentWidth = currentSize.getWidth();
    double usuableHeight = screenSize.getHeight();
    double currentHeight = currentSize.getHeight();
    Point currentLocation = this.dialog.getLocation();
    double currentLeftmostX = currentLocation.getX();
    double availableWidth = usuableWidth - currentLeftmostX;
    double currentUpperY = currentLocation.getY();
    double availableHeight = usuableHeight - currentUpperY;
    double newLeftmostX;
    if (availableWidth < currentWidth) {
      newLeftmostX = currentLeftmostX - (currentWidth - availableWidth);
      newLeftmostX = Math.max(newLeftmostX, 0);
    }
    else {
      newLeftmostX = currentLeftmostX;
    }
    double newUpperY;
    if (availableHeight < currentHeight) {
      newUpperY = currentUpperY - (currentHeight - availableHeight);
      newUpperY = Math.max(newUpperY, 0);
    }
    else {
      newUpperY = currentUpperY;
    }
    Point newLocation = new Point();
    newLocation.setLocation(newLeftmostX, newUpperY);
    this.dialog.setLocation(newLocation);
  }

  /**
   * Faz o resize do dilogo do configurador caso este tenha algum widget que
   * sofreu mudana de visibilidade (e seu configurador de classes <b>esteja</b>
   * configurado para tal.
   * 
   * @see ConfigurationManager
   */
  final public void resizeOnVisibilityChanged() {
    final ConfigurationManager cnfManager = ConfigurationManager.getInstance();
    if (cnfManager == null)
      return;
    try {
      final Class<AlgorithmConfiguratorView> propClass =
        AlgorithmConfiguratorView.class;
      final Configuration cnf = cnfManager.getConfiguration(propClass);
      final String propName = "resize.on.visibility.changed";
      if (cnf.getOptionalBooleanProperty(propName, false)) {
        updateSize();
      }
    }
    catch (Exception e) {
      // No faz nada mesmo, porque a opo default  no efetuar
      // o resize do dilogo.
    }
  }

  /**
   * Ajusta o tamanho do dilogo para que ele no passe dos limites do monitor.
   */
  private void updateSize() {
    /*
     * Existe o percentual mximo do tamanho de uma screen utilizvel pelo
     * dilogo de configurao. Isto para no permitir que toda a tela seja
     * usada. Tambm deixamos um offset para o tamanho final para alguma folga e
     * que evita o aparecimento da scrollbar horizontal.
     */
    final double factor = 0.85;
    final int offset = 35;

    final Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    // dialog.pack();
    // final Dimension currentSize = this.dialog.getSize();
    final Dimension currentSize = dialog.getContentPane().getPreferredSize();
    double usuableWidth = screenSize.getWidth() * factor;
    double currentWidth = currentSize.getWidth();
    double newWidth = Math.min(currentWidth, usuableWidth);
    newWidth = Math.max(MINIMUM_WIDTH, newWidth);
    double usuableHeight = screenSize.getHeight() * factor;
    double currentHeight = currentSize.getHeight();
    double newHeight = Math.min(currentHeight, usuableHeight);
    newHeight = Math.max(MINIMUM_HEIGHT, newHeight);
    final Dimension newSize = new Dimension();
    newSize.setSize(newWidth + offset, newHeight + offset);
    this.dialog.setSize(newSize);
  }

  /**
   * Atualiza o ttulo do dilogo.
   */
  private void updateTitle() {
    String title = getString("title", this);
    this.dialog.setTitle(title);
  }

  /**
   * Retorna o modo de visualizao do configurador.
   * 
   * @return o modo de visualizao do configurador.
   */
  public Mode getMode() {
    return mode;
  }
}
