package csbase.client.algorithms;

import java.awt.Window;
import java.awt.event.ActionEvent;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.MessageFormat;

import javax.swing.AbstractAction;
import javax.swing.Action;

import tecgraf.javautils.LNG;
import csbase.client.applications.ApplicationImages;
import csbase.client.externalresources.ExternalResources;
import csbase.client.remote.srvproxies.AlgorithmManagementProxy;
import csbase.client.util.StandardErrorDialogs;
import csbase.exception.CSBaseException;
import csbase.logic.Utilities;
import csbase.logic.algorithms.AlgorithmVersionInfo;

/**
 * Ao utilizada para exibir a documentao de uma verso de algoritmo.
 */
public final class HelpAlgorithmVersionAction extends AbstractAction {
  /** A verso do algoritmo. */
  private AlgorithmVersionInfo versionInfo;

  /** A janela. */
  private final Window window;

/**
   * Cria uma ao.
   *
   * @param window A janela que est requisitando a ao.
   *
   * @throws IllegalArgumentException Se window for nulo.
   */
  public HelpAlgorithmVersionAction(Window window) {
    if (window == null) {
      throw new IllegalArgumentException("O parmetro window est nulo.");
    }
    this.window = window;
    showName();
    showIcon();
    putValue(Action.SHORT_DESCRIPTION, LNG.get("algorithms.full_description"));
    setEnabled(false);
  }

/**
   * Cria uma ao.
   *
   * @param window A janela que est requisitando a ao.
   * @param versionInfo A verso do algoritmo.
   *
   * @throws IllegalArgumentException Se window ou versionInfo forem nulos.
   */
  public HelpAlgorithmVersionAction(
    Window window, AlgorithmVersionInfo versionInfo) {
    this(window);
    if (versionInfo == null) {
      throw new IllegalArgumentException("O parmetro versionInfo est nulo.");
    }
    setAlgorithmVersionInfo(versionInfo);
  }

  /**
   *
   * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
   */
  public void actionPerformed(ActionEvent ev) {
    if (this.versionInfo == null) {
      throw new IllegalStateException(
        "No existe verso corrente, ou seja, "+
        "o atributo versionInfo est nulo.\n"+
        "Use setAlgorithmVersionInfo ou o construtor que recebe "+
        "algorithmVersionInfo.");
    }
    String htmlDirPath = this.versionInfo.getDocDirPath();
    String filePath;
    if (this.versionInfo.getDocumentation().size() == 1) {
      filePath =
        htmlDirPath + "/"
          + this.versionInfo.getDocumentation().get(0).getPath();
    }
    else {
      filePath = htmlDirPath + "/" + AlgorithmVersionInfo.HTML_FILE;
    }
    String[] filePathArray = Utilities.splitProjectPath(filePath);
    String urlStr =
      AlgorithmManagementProxy.retrieveDownloadURL(
        this.versionInfo.getInfo().getId(), filePathArray, this.window);
    if (urlStr == null) {
      String msg = LNG.get("algorithms.error.no_help_for_version");
      Object[] args = { this.versionInfo.getInfo(), this.versionInfo };
      msg = MessageFormat.format(msg, args);
      showError(msg);
      return;
    }
    try {
      URL url = new URL(urlStr);
      if (ExternalResources.getInstance().isEnabled()) {
        ExternalResources.getInstance().showDocument(url);
        return;
      }
    }
    catch (CSBaseException e) {
      showError(e);
    }
    catch (MalformedURLException e) {
      showError(e);
    }
  }

  /**
   * Oculta o cone.
   */
  public void hideIcon() {
    putValue(Action.SMALL_ICON, null);
  }

  /**
   * Oculta o nome.
   */
  public void hideName() {
    putValue(Action.NAME, null);
  }

  /**
   * Associa uma verso de algoritmo a esta ao.
   *
   * @param versionInfo A verso do algoritmo.
   */
  public void setAlgorithmVersionInfo(AlgorithmVersionInfo versionInfo) {
    this.versionInfo = versionInfo;
    if (this.versionInfo != null) {
      if (this.versionInfo.getDocumentation() != null) {
        if (!this.versionInfo.getDocumentation().isEmpty()) {
          if (this.versionInfo.getDocumentation().size() == 1) {
            setEnabled(ExternalResources.getInstance().isEnabled());
            return;
          }
          else if (
            this.versionInfo.getDocumentation()
            .contains(AlgorithmVersionInfo.HTML_FILE)) {
            setEnabled(ExternalResources.getInstance().isEnabled());
            return;
          }
        }
      }
    }
    setEnabled(false);
  }

  /**
   * Mostra o cone.
   */
  public void showIcon() {
    putValue(Action.SMALL_ICON, ApplicationImages.ICON_HELP_24);
  }

  /**
   * Mostra o nome.
   */
  public void showName() {
    putValue(Action.NAME, LNG.get("algorithms.help"));
  }

  /**
   * Exibe um erro.
   *
   * @param throwable O erro.
   */
  private void showError(Throwable throwable) {
    StandardErrorDialogs.showErrorDialog(this.window, throwable);
  }

  /**
   * Exibe um erro.
   *
   * @param message A mensagem de erro.
   */
  private void showError(String message) {
    StandardErrorDialogs.showErrorDialog(this.window, message);
  }
}
