package csbase.client.algorithms.commands.view;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Set;

import javax.swing.SwingUtilities;

import tecgraf.javautils.LNG;
import tecgraf.javautils.configurationmanager.Configuration;
import tecgraf.javautils.configurationmanager.ConfigurationManager;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.logic.ClientProjectFile;
import csbase.logic.ClientProjectFileInputStream;
import csbase.logic.CommandInfo;

/**
 * Aba de logs adicionais de um comando, que consolida arquivos de log que
 * complementam os de sada-padro. Os arquivos de log apresentados nessa aba
 * so configurveis por sistema.
 */
public class AdditionalLogsTab extends AbstractConsolidatedLogTab {

  /**
   * Construtor
   * 
   * @param commandInfo Informaes sobre o comando cujo log deve ser
   *        apresentado.
   */
  public AdditionalLogsTab(CommandInfo commandInfo) {
    super(commandInfo);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected String getTitle() {
    ConfigurationManager cnfManager = ConfigurationManager.getInstance();
    final String defaultValue = LNG.get("AdditionalLogsTab.title");
    if (cnfManager == null)
      return defaultValue;
    try {
      Class<AdditionalLogsTab> propClass = AdditionalLogsTab.class;
      Configuration cnf = cnfManager.getConfiguration(propClass);
      String propName = "title." + LNG.getLocale();
      String title = cnf.getOptionalProperty(propName, defaultValue);
      return title;
    }
    catch (Exception e) {
      return defaultValue;
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected void loadLog() {
    textArea.setText("");
    Set<ClientProjectFile> additionalLogs = getAdditionalLogs();
    try {
      String logsContent = readFiles(additionalLogs);
      textArea.setText(logsContent);
    }
    catch (IOException e) {
      StandardDialogs.showErrorDialog(this.getParent(), "Erro",
        "Erro ao ler logs adicionais do comando " + getCommand().getId());
    }
  }

  /**
   * Retorna a lista de arquivos de logs adicionais a serem mostrados nessa aba.
   * 
   * @return A lista de logs adicionais.
   */
  private Set<ClientProjectFile> getAdditionalLogs() {
    FindAdditionalLogsTask task = new FindAdditionalLogsTask(getCommand());
    if (task.execute(SwingUtilities.getWindowAncestor(this), getTitle(), LNG
      .get("AdditionalLogsTab.loading.log"))) {
      Set<ClientProjectFile> additionalLogFiles = task.getResult();
      return additionalLogFiles;
    }
    else {
      StandardDialogs.showErrorDialog(this.getParent(), "Erro",
        "Erro ao obter arquivos de log adicionais do comando "
          + getCommand().getId());
      return null;
    }
  }

  /**
   * L e concatena o contedo de mltiplos arquivos para uma String.
   * 
   * @param files Os arquivos a serem lidos.
   * @return O contedo dos arquivos.
   * @throws IOException
   */
  public String readFiles(Set<ClientProjectFile> files) throws IOException {
    StringWriter writer = new StringWriter();
    if (writer == null) {
      throw new IllegalArgumentException("writer == null");
    }
    PrintWriter printWriter = new PrintWriter(writer);
    if (files != null) {
      for (ClientProjectFile file : files) {
        InputStream inputStream = null;
        BufferedReader bufferedReader = null;
        try {
          inputStream = new ClientProjectFileInputStream(file, 0);
          bufferedReader =
            new BufferedReader(new InputStreamReader(inputStream));
          printWriter.printf("\n---- %s ----\n", file.getName());
          String line = bufferedReader.readLine();
          while (line != null) {
            printWriter.println(line);
            line = bufferedReader.readLine();
          }
        }
        finally {
          if (bufferedReader != null) {
            bufferedReader.close();
            bufferedReader = null;
          }
          else if (inputStream != null) {
            inputStream.close();
            inputStream = null;
          }
        }
      }
    }
    return writer.getBuffer().toString();
  }

}
