/*
 * $Author:$ $Date:$ $Release:$
 */
package csbase.client.algorithms.parameters;

import java.awt.CardLayout;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Window;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import javax.swing.border.TitledBorder;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.GBC;
import csbase.client.Client;
import csbase.client.algorithms.parameters.validators.ValidationResult;
import csbase.client.algorithms.view.simple.SimpleAlgorithmConfiguratorPanel;
import csbase.client.applications.ApplicationImages;
import csbase.logic.algorithms.AlgorithmConfigurator;
import csbase.logic.algorithms.parameters.Parameter;
import csbase.logic.algorithms.parameters.ParameterGroup;
import csbase.logic.algorithms.parameters.ParameterLoader;

/**
 * A Viso para {@link ParameterGroup grupo de parmetros}.
 */
public final class ParameterGroupView extends ParameterView<ParameterGroup> {

  /**
   * Estado da viso: se est mostrando os parmetros do grupo ou se est
   * carregando um dos parmetros.
   */
  public enum Status {

    /** Mostrando os parmetros do grupo */
    PARAMETER,

    /** Carregando o valor de um parmetro (via {@link ParameterLoader}) */
    LOADING;
  }

  /**
   * Lista de observadores de {@link ParameterGroupView}
   */
  private final List<ParameterGroupViewListener> listeners;

  /**
   * Painel que armazena os componentes da viso dos parmetros do grupo.
   */
  private JPanel componentsPanel;

  /**
   * Painel utilizado para mostrar mensagem de espera quando um parmetro do
   * grupo est sendo carregado via {@link ParameterLoader}.
   */
  private JPanel loadingPanel;

  /**
   * Painel de base da viso.
   */
  private JPanel basePanel;

  /**
   * Lista das vises dos parmetros pertencentes ao grupo.
   */
  private List<ParameterView<?>> parameterViews;

  /**
   * Viso do carregador da parmetros do grupo (se houver).
   */
  private ParameterLoaderView parameterLoaderView;

  /**
   * Painel com a viso do configurador do algoritmo que contm este grupo.
   */
  private final SimpleAlgorithmConfiguratorPanel configurationPanel;

  /**
   * @param configurationPanel
   * @param group
   * 
   * @deprecated para manter compatibilidade com o WebSintesi
   */
  @Deprecated
  public ParameterGroupView(
    SimpleAlgorithmConfiguratorPanel configurationPanel, ParameterGroup group) {
    this(configurationPanel, group, Mode.CONFIGURATION);
  }

  /**
   * @param configurationPanel
   * @param group
   * @param mode Modo de visualizao. No aceita {@code null}, os possveis
   *        valores so: {@link ParameterView.Mode#CONFIGURATION} ou
   *        {@link ParameterView.Mode#REPORT}.
   */
  public ParameterGroupView(
    SimpleAlgorithmConfiguratorPanel configurationPanel, ParameterGroup group,
    Mode mode) {
    super(group, mode);
    this.configurationPanel = configurationPanel;
    this.listeners = new LinkedList<ParameterGroupViewListener>();
    createParameterViews();
    createParameterLoaderView();
    createComponentsPanel();
    createLoadingPanel();
    createBasePanel();

    createParameterListeners();
    setStatus(Status.PARAMETER);

  }

  /**
   * Retorna a janela que contm essa viso.
   * 
   * @return A janela.
   */
  public Window getWindow() {
    return configurationPanel.getWindow();
  }

  /**
   * Adiciona um observador de {@link ParameterGroupView}.
   * 
   * @param listener O observador.
   */
  public void addParameterGroupViewListener(ParameterGroupViewListener listener) {
    if (listener == null) {
      throw new IllegalArgumentException("O parmetro listener est nulo.");
    }
    this.listeners.add(listener);
  }

  /**
   * Retorna o parmetro do tipo grupo representado por esta viso.
   * 
   * @return O grupo.
   */
  public ParameterGroup getGroup() {
    return getParameter();
  }

  /**
   * Retorna o painel onde essa viso foi adicionada.
   * 
   * @return O painel.
   */
  public JPanel getPanel() {
    for (ParameterView<?> parameterView : getParameterViews()) {
      if (parameterView.isVisible()) {
        return basePanel;

      }
    }
    basePanel.setVisible(false);
    return basePanel;
  }

  /**
   * Retorna o configurador do algoritmo que contm esse grupo de parmetros.
   * 
   * @return O configurador do algoritmo.
   */
  public AlgorithmConfigurator getConfigurator() {
    return configurationPanel.getConfigurator();
  }

  /**
   * Retorna a viso do parmetro com o nome especificado, caso esteja contida
   * nesse grupo ou nulo, caso contrrio.
   * 
   * @param parameterName O nome de parmetro.
   * @return A viso do parmetro ou nulo, caso o parmetro no pertena a esse
   *         grupo.
   */
  public SimpleParameterView<?> getParameterView(String parameterName) {
    if (parameterName == null) {
      throw new IllegalArgumentException("O parmetro parameterName est nulo.");
    }
    for (SimpleParameterView<?> parameterView : getParameterViews()) {
      if (parameterView.getParameter().getName().equals(parameterName)) {
        return parameterView;
      }
    }
    return null;
  }

  /**
   * Retorna a lista das vises dos parmetros que pertencem a esse grupo.
   * 
   * @return A lista de vises dos parmetros.
   */
  public List<SimpleParameterView<?>> getParameterViews() {
    List<SimpleParameterView<?>> parameterViews =
      new LinkedList<SimpleParameterView<?>>();
    SimpleParameterView<?> simpleParameterView = null;
    for (ParameterView<?> parameterView : this.parameterViews) {
      if (parameterView instanceof ParameterGroupView) {
        ParameterGroupView groupView = (ParameterGroupView) parameterView;
        parameterViews.addAll(groupView.getParameterViews());
      }
      else if (parameterView instanceof SimpleParameterView<?>) {
        simpleParameterView = (SimpleParameterView<?>) parameterView;
        parameterViews.add(simpleParameterView);
      }
    }
    return Collections.unmodifiableList(parameterViews);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ValidationResult validate(boolean complain) {
    Iterator<ParameterView<?>> parameterViewIterator =
      this.parameterViews.iterator();
    while (parameterViewIterator.hasNext()) {
      ParameterView<?> view = parameterViewIterator.next();
      ValidationResult validation = view.validate(complain);
      if (!validation.isWellSucceded()) {
        return validation;
      }
    }
    if (parameterLoaderView != null) {
      return parameterLoaderView.validate(complain);
    }
    return new ValidationResult();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected boolean isVisible() {
    if (this.basePanel.isVisible()) {
      return true;
    }
    return false;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected void setVisible(boolean isVisible) {
    this.basePanel.setVisible(isVisible);
    this.basePanel.revalidate();

  }

  /**
   * Monta um grid com as vises dos parmetros listadas no painel especificado.
   * 
   * @param panel O painel.
   * @param views A vises dos parmetros.
   */
  private final void mountBasicGridPanel(final JPanel panel,
    final List<SimpleParameterView<?>> views) {
    panel.setLayout(new GridBagLayout());
    if (views == null) {
      return;
    }

    final int T = 12;
    final int TI = 6;
    final int L = 11;
    final int B = 12;
    final int R = 11;

    final int numRows = views.size();
    for (int rowNumber = 0; rowNumber < numRows; rowNumber++) {
      final SimpleParameterView<?> row = views.get(rowNumber);
      if (row == null) {
        continue;
      }
      JComponent[] components = row.getComponents();
      final int numCols = components.length;
      for (int colNumber = 0; colNumber < numCols; colNumber++) {
        final JComponent cell = components[colNumber];
        if (cell == null) {
          continue;
        }

        GBC gbc = new GBC(colNumber, rowNumber);
        gbc = gbc.gridheight(1).gridwidth(1);
        gbc = gbc.west().weights(1.0, 0.0);

        if (numRows == 1) {
          gbc = gbc.insets(T, L, B, R);
        }
        else {
          if (rowNumber == 0) {
            gbc = gbc.insets(T, L, 0, R);
          }
          else if (rowNumber == numRows - 1) {
            gbc = gbc.insets(TI, L, B, R);
          }
          else {
            gbc = gbc.insets(TI, L, 0, R);
          }
        }

        if (colNumber == 1) {
          if (row.fillVerticalSpace()) {
            gbc = gbc.both();
          }
          else {
            gbc = gbc.horizontal();
          }
          gbc = gbc.weightx(100.0);
        }
        else {
          gbc = gbc.none().west();
        }

        panel.add(cell, gbc);
      }
    }
  }

  /**
   * Cria o painel que armazena os componentes da viso dos parmetros do grupo.
   */
  private void createComponentsPanel() {
    this.componentsPanel = new JPanel(new GridBagLayout());
    String title = getParameter().toString();
    if (!title.trim().isEmpty()) {
      TitledBorder titledBorder = BorderFactory.createTitledBorder(title);
      titledBorder.setTitleJustification(TitledBorder.LEFT);
      this.componentsPanel.setBorder(titledBorder);
    }
    populatePanel();
  }

  /**
   * Cria o painel com a mensagem de espera para quando um parmetro do
   * grupo est sendo carregado via {@link ParameterLoader}.
   */
  private void createLoadingPanel() {
    this.loadingPanel = new JPanel(new GridBagLayout());
    String title = getParameter().toString();
    if (!title.trim().isEmpty()) {
      TitledBorder titledBorder = BorderFactory.createTitledBorder(title);
      titledBorder.setTitleJustification(TitledBorder.LEFT);
      this.loadingPanel.setBorder(titledBorder);
    }
    final JLabel imgLabel = new JLabel();
    imgLabel.setIcon(ApplicationImages.ICON_ANIMATEDENGINES_32);
    imgLabel.setHorizontalAlignment(SwingConstants.CENTER);
    imgLabel.setVerticalAlignment(SwingConstants.CENTER);
    final JLabel txtLabel = new JLabel(LNG
      .get("ParameterGroupView.loading.msg"));
    this.loadingPanel.add(imgLabel, new GBC(0, 0).none().center());
    this.loadingPanel.add(txtLabel, new GBC(1, 0).horizontal().west());
  }

  /**
   * Cria o painel de base da viso do grupo.
   */
  private void createBasePanel() {
    this.basePanel = new JPanel(new CardLayout());
    this.basePanel.add(this.componentsPanel, Status.PARAMETER.toString());
    this.basePanel.add(this.loadingPanel, Status.LOADING.toString());
  }

  /**
   * Atribui o estado da viso: se est mostrando os parmetros do grupo ou
   * carregando um parmetro (via {@link ParameterLoader}).
   * 
   * @param status {@link Status#PARAMETER} ou {@link Status#LOADING}.
   */
  final public void setStatus(final Status status) {
    CardLayout cl = (CardLayout) (basePanel.getLayout());
    cl.show(basePanel, status.toString());
  }

  /**
   * Cria uma lista de observadores dos parmetros pertencentes a este grupo.
   * Esta viso precisa ser notificada das mudanas de visibilidade dos
   * parmetros, para poder reconstruir seu, quando necessrio.
   */
  private void createParameterListeners() {
    for (ParameterView<?> view : getParameterViews()) {
      view.addParameterViewListener(new ParameterViewListener() {
        public void visibilityWasChanged(ParameterView<?> view) {
          updateVisibilyView();
          fireChildParameterVisibilityWasChanged();
        }
      });
    }
  }

  /**
   * Cria a viso dos parmetros pertencentes a este grupo.
   */
  private void createParameterViews() {
    this.parameterViews = new LinkedList<ParameterView<?>>();
    for (Parameter<?> parameter : getGroup().getParameters()) {
      ParameterViewFactory parameterViewFactory =
        Client.getInstance().getParameterViewFactory();
      ParameterView<?> parameterView =
        parameterViewFactory.create(configurationPanel, parameter, getMode());
      if (parameterView != null) {
        this.parameterViews.add(parameterView);
      }
    }
  }

  /**
   * Cria a viso do {@link ParameterLoader} do grupo, caso exista.
   */
  private void createParameterLoaderView() {
    ParameterLoader parameterLoader = getGroup().getParameterLoader();
    if (parameterLoader != null) {
      this.parameterLoaderView = new ParameterLoaderView(this, parameterLoader);
    }
  }

  /**
   * Avisa aos observadores do grupo sobre mudanas na visibilidade dos seus
   * parmetros.
   */
  private void fireChildParameterVisibilityWasChanged() {
    for (ParameterGroupViewListener listener : this.listeners) {
      listener.childParameterVisibilityChanged(this);
    }
  }

  /**
   * Monta o painel com as vises dos parmetros.
   */
  public void populatePanel() {
    this.componentsPanel.removeAll();
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.anchor = GridBagConstraints.NORTHWEST;
    constraints.gridheight = 1;
    constraints.gridwidth = 1;
    constraints.gridx = 1;
    constraints.gridy = 1;
    constraints.insets = new Insets(0, 0, 0, 0);
    constraints.weightx = 1.0;

    List<SimpleParameterView<?>> views =
      new LinkedList<SimpleParameterView<?>>();

    Iterator<ParameterView<?>> parameterViewIterator =
      this.parameterViews.iterator();
    while (parameterViewIterator.hasNext()) {
      ParameterView<?> parameterView = parameterViewIterator.next();
      if (parameterView instanceof SimpleParameterView<?>) {
        SimpleParameterView<?> simpleParameterView =
          (SimpleParameterView<?>) parameterView;
        views.add(simpleParameterView);
      }
      else if (parameterView instanceof ParameterGroupView) {
        if (!views.isEmpty()) {
          JPanel componentPanel = new JPanel();
          mountBasicGridPanel(componentPanel, views);
          if (resizesVertically(views)) {
            constraints.fill = GridBagConstraints.BOTH;
            constraints.weighty = 1.0;
          }
          else {
            constraints.fill = GridBagConstraints.HORIZONTAL;
            constraints.weighty = 0;
          }
          views.clear();
          this.componentsPanel.add(componentPanel, constraints);
          constraints.gridy++;
        }
        ParameterGroupView parameterGroupView =
          (ParameterGroupView) parameterView;
        parameterGroupView.populatePanel();
        JPanel groupPanel = parameterGroupView.getPanel();
        if (parameterGroupView.fillVerticalSpace()) {
          constraints.fill = GridBagConstraints.BOTH;
          constraints.weighty = 1.0;
        }
        else {
          constraints.fill = GridBagConstraints.HORIZONTAL;
          constraints.weighty = 0;
        }
        this.componentsPanel.add(groupPanel, constraints);
        constraints.gridy++;
      }
    }
    if (!views.isEmpty()) {
      JPanel componentPanel = new JPanel();
      mountBasicGridPanel(componentPanel, views);
      if (resizesVertically(views)) {
        constraints.fill = GridBagConstraints.BOTH;
        constraints.weighty = 1.0;
      }
      else {
        constraints.fill = GridBagConstraints.HORIZONTAL;
        constraints.weighty = 0;
      }
      this.componentsPanel.add(componentPanel, constraints);
      constraints.gridy++;
    }
    if (parameterLoaderView != null) {
      /* Adiciona o boto no final do grupo */
      JPanel loaderPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
      loaderPanel.add(parameterLoaderView.getComponent());
      constraints.fill = GridBagConstraints.HORIZONTAL;
      constraints.weighty = 0;
      this.componentsPanel.add(loaderPanel, constraints);
    }
    this.componentsPanel.revalidate();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean fillVerticalSpace() {
    return resizesVertically(parameterViews);
  }

  /**
   * Indica se a viso do grupo deve ser redimensionada verticalmente, de acordo
   * com a viso dos parmetros nele contidos.
   * 
   * @param parameterViews A lista de vises dos parmetros.
   * @return Verdadeiro se esse grupo deve ser redimensionado verticalmente ou
   *         falso, caso contrrio.
   */
  private boolean resizesVertically(
    List<? extends ParameterView<?>> parameterViews) {
    for (ParameterView<?> paramView : parameterViews) {
      if (paramView.fillVerticalSpace() && paramView.isVisible()) {
        return true;
      }
    }
    return false;
  }
}
