/*
 * $Author:$
 * $Date:$
 * $Release:$
 */
package csbase.client.algorithms.parameters;

import java.awt.Color;
import java.awt.Toolkit;
import java.awt.Window;
import java.text.MessageFormat;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;

import csbase.client.algorithms.parameters.validators.SimpleParameterValidator;
import csbase.client.algorithms.parameters.validators.ValidationResult;
import csbase.logic.algorithms.parameters.SimpleParameter;
import csbase.logic.algorithms.parameters.SimpleParameterListener;

/**
 * Viso para {@link SimpleParameter Parmetro Simples}.
 * 
 * @param <V> O tipo do parmetro.
 * 
 * @author lmoreira
 */
public abstract class SimpleParameterView<V> extends
  ParameterView<SimpleParameter<V>> {

  /**
   * O rtulo.
   */
  private JLabel label;

  /**
   * O validador do parmetro.
   */
  private SimpleParameterValidator<V> parameterValidator;

  /**
   * O componente principal da viso.
   */
  private JComponent component;

  /**
   * Cria a viso.
   * 
   * @param simpleParameter O parmetro (No aceita {@code null}).
   * @param mode Modo de visualizao. No aceita {@code null}, os possveis
   *        valores so: {@link ParameterView.Mode#CONFIGURATION} ou
   *        {@link ParameterView.Mode#REPORT}
   * @param componentArgs argumentos que sero passados aos mtodos de
   *        construo do componente representando a viso, tanto em modo
   *        {@link ParameterView.Mode#CONFIGURATION} quanto em modo
   *        {@link ParameterView.Mode#REPORT}.
   */
  protected SimpleParameterView(SimpleParameter<V> simpleParameter, Mode mode,
    Object... componentArgs) {
    super(simpleParameter, mode);

    this.component = createComponent(componentArgs);

    getParameter().addSimpleParameterListener(
      new SimpleParameterListener<V>() {
      public void capabilityWasChanged(SimpleParameter<V> parameter) {
        updateCapabilityView();
      }

      public void defaultValueWasChanged(SimpleParameter<V> parameter) {
      }

      public void valueWasChanged(SimpleParameter<V> parameter) {
        updateViewContents();
      }

      public void visibilityWasChanged(SimpleParameter<V> parameter) {
        if (getMode() == Mode.CONFIGURATION) {
          updateVisibilyView();
        }
      }

      public void labelWasChanged(SimpleParameter<V> parameter) {
        label.setText(parameter.getLabel());
      }
          });
    this.label = new JLabel(getParameter().getLabel());
    String toolTipText = MessageFormat.format("{0} ({1})",
      new Object[] { getParameter().getLabel(), getParameter().getName() });
    this.label.setToolTipText(toolTipText);
  }

  /**
   * Obtm os componentes da viso.
   * 
   * @return Os componentes.
   */
  public final JComponent[] getComponents() {
    return new JComponent[] { this.label, getComponent() };
  }

  /**
   * Obtm o rtulo da viso.
   * 
   * @return .
   */
  public final JLabel getLabel() {
    return this.label;
  }

  /**
   * Obtm o validador da viso.
   * 
   * @return O validador.
   */
  public SimpleParameterValidator<V> getParameterValidator() {
    if (this.parameterValidator == null) {
      this.parameterValidator = createParameterValidator();
    }
    return this.parameterValidator;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ValidationResult validate(boolean complain) {
    ValidationResult result = getParameterValidator().validateValue(
      getParameter(), getParameter().getValue());
    if (!result.isWellSucceded() && complain) {
      showValidationErrorMessage(result.getErrorMessage());
    }
    return result;
  }

  /**
   * Limpa a indicao de erro.
   */
  public final void clearError() {
    this.label.setForeground(Color.BLACK);
  }

  /**
   * @return a janela que ter o componente que representa esta viso.
   */
  public Window getWindow() {
    JComponent component = getComponent();
    return null == component ? null : SwingUtilities
      .windowForComponent(component);
  }

  /**
   * Obtm o componente principal.
   * 
   * @return .
   */
  public JComponent getComponent() {
    return component;
  }

  /**
   * Obtm o componente principal.
   * 
   * @param componentArgs Os parmetros para criao do componente.
   * @return O componente principal.
   */
  protected abstract JComponent createConfigurationComponent(
    Object... componentArgs);

  /**
   * Obtm o componente principal.
   * 
   * @param componentArgs Os parmetros para criao do componente.
   * @return O componente principal.
   */
  protected abstract JComponent createReportComponent(Object... componentArgs);

  /**
   * Cria o validador.
   * 
   * @return .
   */
  protected abstract SimpleParameterValidator<V> createParameterValidator();

  /**
   * Marca o erro de validao.
   */
  public final void highlightError() {
    this.label.setForeground(Color.RED);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected final boolean isVisible() {
    return this.label.isVisible();
  }

  /**
   * Habilita/Desabilita a viso.
   * 
   * @param isEnabled .
   */
  protected void setEnabled(boolean isEnabled) {
    if (null != component) {
      component.setEnabled(isEnabled);
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected final void setVisible(boolean isVisible) {
    this.label.setVisible(isVisible);
    if (null != getComponent()) {
      getComponent().setVisible(isVisible);
    }
  }

  /**
   * Exibe um erro de validao.
   * 
   * @param errorMessage A mensagem de erro (No aceita {@code null}).
   */
  protected final void showValidationErrorMessage(String errorMessage) {
    highlightError();
    Toolkit.getDefaultToolkit().beep();
    JOptionPane.showMessageDialog(getWindow(), errorMessage,
      "Erro no configurador de parmetros", JOptionPane.ERROR_MESSAGE);
    getComponent().requestFocus();
    clearError();
  }

  /**
   * Atualiza a viso em relao ao estado Habilitado/Desabilitado.
   */
  protected final void updateCapabilityView() {
    this.label.setEnabled(getParameter().isEnabled());
    setEnabled(getParameter().isEnabled());
  }

  /**
   * Atualiza o contedo exibido pela viso.
   */
  protected abstract void updateViewContents();

  /**
   * Cria o componente principal.
   * 
   * @param componentArgs parmetros para criao dos componentes.
   * 
   * @return O componente principal.
   */
  private JComponent createComponent(Object... componentArgs) {
    switch (getMode()) {
      case REPORT: {
        return createReportComponent(componentArgs);
      }
      default: {
        return createConfigurationComponent(componentArgs);
      }
    }
  }
}
