package csbase.client.algorithms.parameters;

import java.awt.Dimension;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Window;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;

import tecgraf.javautils.gui.GBC;
import csbase.client.algorithms.parameters.validators.TableParameterValidator;
import csbase.client.algorithms.parameters.validators.ValidationResult;
import csbase.client.util.table.AbstractCellModel;
import csbase.client.util.table.AbstractRowModel;
import csbase.client.util.table.CellModel;
import csbase.client.util.table.CellModelFactory;
import csbase.client.util.table.CellViewFactory;
import csbase.client.util.table.CheckBoxCellView;
import csbase.client.util.table.ComboBoxCellViewFactory;
import csbase.client.util.table.ComboBoxCellViewItem;
import csbase.client.util.table.DoubleConverter;
import csbase.client.util.table.IntegerConverter;
import csbase.client.util.table.ReflectionCellViewFactory;
import csbase.client.util.table.RowValueFactory;
import csbase.client.util.table.Table;
import csbase.client.util.table.TableColumn;
import csbase.client.util.table.TextFieldCellViewFactory;
import csbase.client.util.table.TextFieldCellView.HorizontalAlignment;
import csbase.logic.algorithms.parameters.BooleanColumn;
import csbase.logic.algorithms.parameters.Column;
import csbase.logic.algorithms.parameters.DoubleColumn;
import csbase.logic.algorithms.parameters.EnumerationColumn;
import csbase.logic.algorithms.parameters.EnumerationItem;
import csbase.logic.algorithms.parameters.IntegerColumn;
import csbase.logic.algorithms.parameters.RowValue;
import csbase.logic.algorithms.parameters.RowValueListener;
import csbase.logic.algorithms.parameters.TableParameter;
import csbase.logic.algorithms.parameters.TableParameterListener;
import csbase.logic.algorithms.parameters.TextColumn;

/**
 * Viso para o {@link TableParameter Parmetro do Tipo Tabela}.
 * 
 * @author lmoreira
 */
public final class TableParameterView extends
  SimpleParameterView<List<RowValue>> {

  /**
   * Cria uma viso em modo {@link ParameterView.Mode#CONFIGURATION}.
   * 
   * @param window NO EST SENDO UTILIZADO. Existe somente para manter
   *        compatibilidade com o WebSintesi.
   * @param parameter O parmetro (No aceita {@code null}).
   * 
   * @deprecated para manter compatibilidade com o WebSintesi
   */
  @Deprecated
  public TableParameterView(Window window, TableParameter parameter) {
    this(parameter, Mode.CONFIGURATION);
  }

  /**
   * Cria a viso.
   * 
   * @param parameter O parmetro (No aceita {@code null}).
   * @param mode Modo de visualizao. No aceita {@code null}, os possveis
   *        valores so: {@link ParameterView.Mode#CONFIGURATION} ou
   *        {@link ParameterView.Mode#REPORT}.
   */
  public TableParameterView(TableParameter parameter, Mode mode) {
    super(parameter, mode);

    updateCapabilityView();
    updateVisibilyView();
  }

  /**
   * @see csbase.client.algorithms.parameters.ParameterView#getParameter()
   */
  @Override
  public TableParameter getParameter() {
    return (TableParameter) super.getParameter();
  }

  /**
   * Verifica se os valores armazenados no parmetro so vlidos.
   * 
   * @param complain Indica se deve exibir mensagens de erro ao usurio.
   * 
   * @return O resultado da validao.
   */
  @Override
  public ValidationResult validate(boolean complain) {
    TableParameterValidator parameterValidator =
      (TableParameterValidator) getParameterValidator();
    ValidationResult result =
      parameterValidator.validateValue(getParameter(), getParameter()
      .getValue());
    if (!result.isWellSucceded() && complain) {
      ITableParameterComponent tableComponent =
        (ITableParameterComponent) getComponent();
      if (tableComponent.getRowCount() != 0) {
        tableComponent.selectCell(parameterValidator.getRowIndex(),
          parameterValidator.getColumnIndex());
      }
      showValidationErrorMessage(result.getErrorMessage());
    }
    return result;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected TableParameterValidator createParameterValidator() {
    return new TableParameterValidator(getParameter().isOptional(),
      getParameter().getColumns());
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected JComponent createConfigurationComponent(Object... componentArgs) {
    return new TableConfigurationParameter();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected JComponent createReportComponent(Object... componentArgs) {
    return new TableReportParameter();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected void updateViewContents() {
  }

  /**
   * Interface que o componente que representa a viso da tabela deve
   * implementar.
   */
  private interface ITableParameterComponent {
    /**
     * Atualiza o contedo exibido pela viso.
     */
    void updateViewContents();

    /**
     * @return A quantidade de linhas da tabela.
     */
    int getRowCount();

    /**
     * Seleciona uma clula especfica, removendo as seleo das outras clulas.
     * 
     * @param rowIndex O ndice da linha (O ndice da linha tem que ser maior ou
     *        igual a 0 e menor do que a quantidade de linhas).
     * @param columnIndex O ndice da coluna (O ndice da coluna tem que ser
     *        maior ou igual a 0 e menor do que a quantidade de colunas).
     */
    void selectCell(int rowIndex, int columnIndex);
  }

  /**
   * Viso da tabela em modo relatrio.
   */
  private final class TableReportParameter extends JPanel implements
    ITableParameterComponent {

    /**
     * A tabela.
     */
    private JTable table;

    /**
     * Construtor.
     */
    TableReportParameter() {

      setLayout(new GridLayout());

      this.table = new JTable(new AbstractTableModel() {
        public int getColumnCount() {
          return getParameter().getColumnCount();
        }

        @Override
        public String getColumnName(int column) {
          return getParameter().getColumns().get(column).getLabel();
        }

        public int getRowCount() {
          return getParameter().getRowCount();
        }

        @Override
        public Class<?> getColumnClass(int c) {
          return getValueAt(0, c).getClass();
        }

        public Object getValueAt(int rowIndex, int columnIndex) {
          return getParameter().getItemValue(rowIndex, columnIndex);
        }
      });
      ComponentProperties.setProperties(table, Mode.REPORT, false);

      JScrollPane scroll = new JScrollPane(table);
      scroll
        .setBorder(ComponentProperties.getInstance(Mode.REPORT).getBorder());
      scroll.setPreferredSize(new Dimension(400, 100));
      add(scroll, new GBC(0, 0).both());
      updateViewContents();
    }

    /**
     * @see csbase.client.algorithms.parameters.TableParameterView.ITableParameterComponent#updateViewContents()
     */
    public void updateViewContents() {
      ((AbstractTableModel) table.getModel()).fireTableDataChanged();
    }

    /**
     * @see csbase.client.algorithms.parameters.TableParameterView.ITableParameterComponent#selectCell(int,
     *      int)
     */
    public void selectCell(int rowIndex, int columnIndex) {
      table.changeSelection(rowIndex, columnIndex, false, false);
    }

    /**
     * @see csbase.client.algorithms.parameters.TableParameterView.ITableParameterComponent#getRowCount()
     */
    public int getRowCount() {
      return table.getRowCount();
    }

    /**
     * Este mtodo no faz nada, pois este componente no pode ser habilitado.
     * 
     * @see javax.swing.AbstractButton#setEnabled(boolean)
     */
    @Override
    public void setEnabled(boolean enabled) {
    }
  }

  /**
   * Viso da tabela em modo de configurao.
   */
  private final class TableConfigurationParameter extends JPanel implements
    ITableParameterComponent {

    /**
     * Tabela.
     */
    private Table<RowValue> table;

    /**
     * Construtor.
     */
    TableConfigurationParameter() {
      setLayout(new GridBagLayout());

      List<TableColumn<RowValue>> tableColumns = createTableColumns();
      AbstractRowModel<RowValue> rowModel = createRowModel();
      this.table = createTable(rowModel, tableColumns);
      add(table.getView(), new GBC(0, 0).both());
    }

    /**
     * @see csbase.client.algorithms.parameters.TableParameterView.ITableParameterComponent#updateViewContents()
     */
    public void updateViewContents() {

    }

    /**
     * @see csbase.client.algorithms.parameters.TableParameterView.ITableParameterComponent#getRowCount()
     */
    public int getRowCount() {
      return table.getRowCount();
    }

    /**
     * @see csbase.client.algorithms.parameters.TableParameterView.ITableParameterComponent#selectCell(int,
     *      int)
     */
    public void selectCell(int rowIndex, int columnIndex) {
      table.selectCell(rowIndex, columnIndex);
    }

    /**
     * @see javax.swing.JComponent#setEnabled(boolean)
     */
    @Override
    public void setEnabled(boolean isEnabled) {
      super.setEnabled(isEnabled);
      table.setEnabled(isEnabled);
    }

    /**
     * Cria colunas da tabela.
     * 
     * @return A lista com as colunas criadas.
     */
    private List<TableColumn<RowValue>> createTableColumns() {
      List<TableColumn<RowValue>> tableColumns =
        new ArrayList<TableColumn<RowValue>>(getParameter().getColumnCount());
      List<Column<?>> columns = getParameter().getColumns();
      for (int i = 0; i < getParameter().getColumnCount(); i++) {
        Column<?> column = columns.get(i);
        CellViewFactory cellViewFactory = createCellViewFactory(column);
        final int columnIndex = i;
        CellModelFactory<RowValue> cellModelFactory =
          new CellModelFactory<RowValue>() {
            public CellModel create(final RowValue rowValue) {
              final AbstractCellModel cellModel = new AbstractCellModel() {

                @Override
                protected void keepValue(Object value) {
                  rowValue.setCellValue(value, columnIndex);
                }

                public Object getValue() {
                  return rowValue.getCellValue(columnIndex);
                }
              };
              RowValueListener listener = new RowValueListener() {

                public void valueWasChanged(Object newValue, int aColumnIndex) {
                  if (aColumnIndex == columnIndex) {
                    cellModel.setValue(newValue);
                  }
                }

              };
              rowValue.addRowValueListener(listener);
              return cellModel;
            }
          };
        TableColumn<RowValue> tableColumn =
          new TableColumn<RowValue>(column.getLabel(), column.isEditable(),
            cellModelFactory, cellViewFactory);
        tableColumns.add(tableColumn);
      }

      return tableColumns;
    }

    /**
     * Cria o modelo das linhas da tabela.
     * 
     * @return O modelo.
     */
    private AbstractRowModel<RowValue> createRowModel() {
      final AbstractRowModel<RowValue> rowModel =
        new AbstractRowModel<RowValue>() {

          public void addRow(RowValue rowValue) {
            getParameter().addRow(rowValue);
            fireRowWasAdded(rowValue, getParameter().getRowCount() - 1);
          }

          public RowValue getRow(int rowIndex) {
            return getParameter().getRowValues().get(rowIndex);
          }

          public int getRowCount() {
            return getParameter().getRowCount();
          }

          public List<RowValue> getRows() {
            return getParameter().getRowValues();
          }

          public boolean removeRow(int rowIndex) {
            if (rowIndex < 0 || rowIndex >= getRowCount()) {
              return false;
            }
            getParameter().removeRow(rowIndex);
            return true;
          }
        };

      getParameter().addTableParameterListener(new TableParameterListener() {
        public void rowWasCreated(TableParameter parameter,
          RowValue newRowValue, int newRowIndex) {
          rowModel.fireRowWasAdded(newRowValue, newRowIndex);
        }

        public void rowWasRemoved(TableParameter parameter,
          RowValue oldRowValue, int oldRowIndex) {
          rowModel.fireRowWasRemoved(oldRowValue, oldRowIndex);
        }
      });

      return rowModel;
    }

    /**
     * Cria a tabela.
     * 
     * @param rowModel O modelo das linhas.
     * @param tableColumns As colunas da tabela.
     * @return A tabela.
     */
    private Table<RowValue> createTable(AbstractRowModel<RowValue> rowModel,
      List<TableColumn<RowValue>> tableColumns) {

      Table<RowValue> table;
      if (getParameter().hasFixedRowCount()) {
        table = new Table<RowValue>(rowModel, tableColumns);
      }
      else {
        table = new Table<RowValue>(new RowValueFactory<RowValue>() {
          public RowValue create(int rowIndex) {
            Object[] values = new Object[getParameter().getColumnCount()];
            for (int i = 0; i < getParameter().getColumnCount(); i++) {
              Column<?> column = getParameter().getColumns().get(i);
              values[i] = column.getDefaultValue(rowIndex);
            }
            return new RowValue(values);
          }
        }, rowModel, tableColumns, getParameter().getVisibleRowCount());
      }
      table.setToolTipText(getParameter().getDescription());

      return table;
    }

    /**
     * Cria a fbrica de vises de clulas para a coluna informada.
     * 
     * @param column A coluna.
     * 
     * @return .
     */
    private CellViewFactory createCellViewFactory(Column<?> column) {
      CellViewFactory cellViewFactory;
      if (column instanceof BooleanColumn) {
        cellViewFactory = new ReflectionCellViewFactory(CheckBoxCellView.class);
      }
      else if (column instanceof IntegerColumn) {
        IntegerConverter integerConverter = new IntegerConverter();
        cellViewFactory =
          new TextFieldCellViewFactory(HorizontalAlignment.RIGHT,
            integerConverter);
      }
      else if (column instanceof DoubleColumn) {
        DoubleConverter doubleConverter = new DoubleConverter();
        cellViewFactory =
          new TextFieldCellViewFactory(HorizontalAlignment.RIGHT,
            doubleConverter);
      }
      else if (column instanceof TextColumn) {
        cellViewFactory = new TextFieldCellViewFactory();
      }
      else if (column instanceof EnumerationColumn) {
        EnumerationColumn enumerationColumn = (EnumerationColumn) column;
        List<EnumerationItem> items = enumerationColumn.getItems();
        List<ComboBoxCellViewItem> cellViewItems =
          new ArrayList<ComboBoxCellViewItem>(items.size());
        for (EnumerationItem item : items) {
          ComboBoxCellViewItem cellViewItem =
            new ComboBoxCellViewItem(item.getLabel(), item.getDescription(),
              item);
          cellViewItems.add(cellViewItem);
        }
        cellViewFactory = new ComboBoxCellViewFactory(cellViewItems);
      }
      else {
        throw new IllegalStateException();
      }
      return cellViewFactory;
    }

  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean fillVerticalSpace() {
    return true;
  }
}
