/*
 * $Author:$ $Date:$ $Release:$
 */
package csbase.client.algorithms.parameters.validators;

import java.text.MessageFormat;

import csbase.logic.algorithms.parameters.SimpleParameter;

/**
 * Validador do 
 * {@link csbase.logic.algorithms.parameters.DoubleParameter 
 * Parmetro do Tipo Real}
 *
 * @author lmoreira
 */
public final class DoubleParameterValidator extends SimpleParameterValidator<Double> {
  /**
   * Indica se o valor mximo est entre os valores vlidos.
   */
  private boolean isMaximumIncluded;
  /**
   * Indica se o valor mnimo est entre os valores vlidos.
   */
  private boolean isMinimumIncluded;
  /**
   * O valor mximo.
   */
  private Double maximum;
  /**
   * O valor mnimo.
   */
  private Double minimum;

  /**
   * Cria um Validador.
   * 
   * @param isOptional Indica se o valor  opcional/obrigatrio.
   * @param maximum O mximo (Aceita {@code null}).
   * @param isMaximumIncluded Indica se o valor mximo est entre os valores 
   *                          vlidos.
   * @param minimum O mnimo (Aceita {@code null}).
   * @param isMinimumIncluded Indica se o valor mnimo est entre os valores 
   *                          vlidos.
   */
  public DoubleParameterValidator(
    boolean isOptional, Double maximum, boolean isMaximumIncluded,
    Double minimum, boolean isMinimumIncluded) {
    super(isOptional);
    setMaximum(maximum, isMaximumIncluded);
    setMinimum(minimum, isMinimumIncluded);
  }

  /**
   * Obtm o mximo.
   * 
   * @return O mximo ou {@code null} se ele no existir.
   */
  public Double getMaximum() {
    return this.maximum;
  }

  /**
   * Obtm o mnimo.
   * 
   * @return O mnimo ou {@code null} se ele no existir.
   */
  public Double getMinimum() {
    return this.minimum;
  }

  /**
   * Indica se o valor mximo est entre os valores vlidos.
   * 
   * @return .
   */
  public boolean isMaximumIncluded() {
    return this.isMaximumIncluded;
  }

  /**
   * Indica se o valor mnimo est entre os valores vlidos.
   * 
   * @return .
   */
  public boolean isMinimumIncluded() {
    return this.isMinimumIncluded;
  }

  /**
   * Modifica o mximo.
   * 
   * @param maximum O mximo (Aceita {@code null}).
   * 
   * @param isMaximumIncluded Indica se o valor mximo est entre os valores 
   *        vlidos.
   */
  public void setMaximum(Double maximum, boolean isMaximumIncluded) {
    if (this.minimum != null) {
      if (maximum != null && maximum.compareTo(this.minimum) < 0) {
        throw new IllegalArgumentException(
          "O valor mximo no pode ser menor do que o valor mnimo.");
      }
    }
    if (maximum == null) {
      this.isMaximumIncluded = false;
    }
    else {
      this.isMaximumIncluded = isMaximumIncluded;
    }
    this.maximum = maximum;
  }

  /**
   * Modifica o mnimo.
   * 
   * @param minimum O mnimo (Aceita {@code null}).
   * 
   * @param isMinimumIncluded Indica se o valor mnimo est entre os valores 
   *        vlidos.
   */
  public void setMinimum(Double minimum, boolean isMinimumIncluded) {
    if (this.maximum != null) {
      if (minimum != null && minimum.compareTo(this.maximum) > 0) {
        throw new IllegalArgumentException(
          "O valor mnimo no pode ser maior do que o valor mximo.");
      }
    }
    if (minimum == null) {
      this.isMinimumIncluded = false;
    }
    else {
      this.isMinimumIncluded = isMinimumIncluded;
    }
    this.minimum = minimum;
  }

  @Override
  public ValidationResult validateValue(
    SimpleParameter parameter, Double value) {
    ValidationResult validationResult = super.validateValue(parameter, value);
    if (!validationResult.isWellSucceded()) {
      return validationResult;
    }
    if (parameter.isEnabled() && parameter.isVisible()) {
      if (value != null) {
        if (this.minimum != null) {
          if (this.isMinimumIncluded) {
            if (value.compareTo(this.minimum) < 0) {
              return new ValidationResult(MessageFormat.format(
                getMessage(getClass(), "less_than_minimum"), 
                  parameter.getLabel(), value, this.minimum));
            }
          }
          else {
            if (value.compareTo(this.minimum) <= 0) {
              return new ValidationResult(MessageFormat.format(
                getMessage(getClass(), "less_or_equals_to_minimum"), 
                parameter.getLabel(), value, this.minimum));
            }
          }
        }
        if (this.maximum != null) {
          if (this.isMaximumIncluded) {
            if (value.compareTo(this.maximum) > 0) {
              return new ValidationResult(MessageFormat.format(
                getMessage(getClass(), "larger_than_maximum"), 
                parameter.getLabel(), value, this.maximum));
            }
          }
          else {
            if (value.compareTo(this.maximum) >= 0) {
              return new ValidationResult(MessageFormat.format(
                getMessage(getClass(), "larger_or_equals_to_maximum"), 
                parameter.getLabel(), value, this.maximum));
            }
          }
        }
      }
    }
    return new ValidationResult();
  }
}
