package csbase.client.algorithms.parameters.validators;

import java.text.MessageFormat;

import tecgraf.javautils.LNG;
import csbase.logic.algorithms.parameters.SimpleParameter;

/**
 * Validador de parmetro.
 * 
 * @param <V> O tipo do valor armazenado no parmetro.
 *
 * @author lmoreira
 */
public abstract class SimpleParameterValidator<V> {

  
  /**
   * Indica se o valor do parmetro  opcional/obrigatrio.
   */
  private boolean isOptional;

  /**
   * Modo de validao.
   */
  private ValidationMode mode;
  
  /**
   * Cria o validador.
   * 
   * @param isOptional Indica se o valor do parmetro  opcional/obrigatrio.
   */
  protected SimpleParameterValidator(boolean isOptional) {
    this.isOptional = isOptional;
    this.mode = ValidationMode.DEFAULT;
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == null) {
      return false;
    }
    if (!getClass().equals(obj.getClass())) {
      return false;
    }
    SimpleParameterValidator<?> validator = (SimpleParameterValidator)obj;
    return isOptional() == validator.isOptional();
  }

  @Override
  public int hashCode() {
    return new Boolean(isOptional()).hashCode();
  }

  /**
   * Valida.
   * 
   * @param parameter O parmetro envolvido (No aceita {@code null}).
   * @param value O valor (Aceita {@code null}).
   * 
   * @return O resultado da validao.
   */
  public ValidationResult validateValue(SimpleParameter parameter, V value) {
    if (parameter == null) {
      throw new IllegalArgumentException("O parmetro parameter est nulo.");
    }
    if (parameter.isEnabled() && parameter.isVisible()) {
      if (value == null) {
        if (isOptional() 
        || getMode().equals(ValidationMode.ALLOW_EMPY_VALUES)) {
          return new ValidationResult();
        }
        return new ValidationResult(MessageFormat.format(
          getMessage(SimpleParameterValidator.class, "empty_value"), 
          parameter.getLabel()));
      }
    }
    return new ValidationResult();
  }

  /**
   * Obtm o modo de validao atual.
   * 
   * @return .
   */
  public final ValidationMode getMode() {
    return mode;
  }

  /**
   * Modifica o modo de validao atual.
   * 
   * @param mode O novo modo (No aceita {@code null}).
   */
  public void setMode(ValidationMode mode) {
    if (mode == null) {
      throw new IllegalArgumentException("O parmetro this.mode est nulo.");
    }
  
    this.mode = mode;
  }

  /**
   * Indica se o valor do parmetro  opcional/obrigatrio.
   * 
   * @return .
   */
  public final boolean isOptional() {
    return this.isOptional;
  }

  /**
   * <p>Obtm uma mensagem especfica no mecanismo de internacionalizao.</p>
   * 
   * <p>A chave da mensagem  formada pela concatenao do nome da classe,
   * o caracter '.' e um sufixo.</p>
   * 
   * @param clazz A classe que servir de referncia para encontrar a mensagem 
   * (No aceita {@code null}).
   * @param keySuffix O sufixo da chave (No aceita {@code null}).
   * 
   * @return A mensagem.
   */
  protected final String getMessage(
    Class<? extends SimpleParameterValidator> clazz, String keySuffix) {
    return LNG.get(clazz.getName() + "." + keySuffix);
  }    
}
