package csbase.client.algorithms.parameters.validators;

import java.text.MessageFormat;

import csbase.logic.algorithms.parameters.SimpleParameter;

/**
 * Validador do 
 * {@link csbase.logic.algorithms.parameters.TextParameter 
 * Parmetro Tipo Texto}.
 * 
 * @author lmoreira
 */
public final class TextParameterValidator extends SimpleParameterValidator<String> {
  /**
   * Quantidade mxima de caracteres para que o texto seja considerado vlido.
   */
  private Integer maxCharacters;

  /**
   * Cria um validador.
   * 
   * @param isOptional Indica se o valor  opcional/obrigatrio.
   * @param maxCharacters Quantidade mxima de caracteres para que o texto seja 
   *        considerado vlido (Se no houver limite {@code null} - 
   *        no pode ser menor ou igual a 0).
   */
  public TextParameterValidator(boolean isOptional, Integer maxCharacters) {
    super(isOptional);
    setMaxCharacters(maxCharacters);
  }

  /**
   * Obtm a quantidade mxima de caracteres para que o texto seja considerado 
   * vlido.
   * 
   * @return Quantidade mxima de caracteres para que o texto seja considerado 
   *         vlido ou {@code null} se no houver limite.
   */
  public Integer getMaxCharacters() {
    return this.maxCharacters;
  }

  @Override
  public ValidationResult validateValue(
    SimpleParameter parameter, String value) {
    ValidationResult validationResult = super.validateValue(parameter, value);
    if (!validationResult.isWellSucceded()) {
      return validationResult;
    }
    if (parameter.isEnabled() && parameter.isVisible()) {
      if (value != null) {
        if (
          getMaxCharacters() != null &&
          value.length() > getMaxCharacters().intValue()) {
          return new ValidationResult(MessageFormat.format(
            getMessage(getClass(), "above_max_characteres"), 
            parameter.getLabel(), value, value.length(), 
            getMaxCharacters()));
        }
        if (!value.matches("[:/#\\w\\.-]+")) {
          return new ValidationResult(MessageFormat.format(
            getMessage(getClass(), "illegal_characteres"), parameter.getLabel(),
              value));
        }
      }
    }
    return new ValidationResult();
  }

  /**
   * Atribui a quantidade mxima de caracteres para que o texto seja considerado
   * vlido.
   * 
   * @param maxCharacters Quantidade mxima de caracteres para que o texto seja 
   *        considerado vlido (Se no houver limite {@code null} - 
   *        no pode ser menor ou igual a 0).
   */
  private void setMaxCharacters(Integer maxCharacters) {
    if (maxCharacters != null && maxCharacters.intValue() <= 0) {
      throw new IllegalArgumentException(
        MessageFormat.format(
          "O parmetro maxCharacters deveria ser positivo.\n"+
          "Valor encontrado: {0}.", new Object[] { maxCharacters }));
    }
    this.maxCharacters = maxCharacters;
  }
}
