package csbase.client.algorithms.view.flow;

import java.awt.BorderLayout;
import java.awt.GridBagLayout;
import java.awt.Window;
import java.util.Collection;

import javax.swing.JScrollPane;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.GBC;
import csbase.algorithms.flows.Workspace;
import csbase.algorithms.flows.filters.AskForParameterValuesActionFilter;
import csbase.algorithms.flows.filters.HighlightElementFilter;
import csbase.algorithms.flows.filters.HintElementFilter;
import csbase.algorithms.flows.graph.Graph;
import csbase.algorithms.flows.graph.GraphNode;
import csbase.algorithms.flows.zoom.ZoomControl;
import csbase.client.algorithms.AlgorithmConfiguratorView;
import csbase.client.algorithms.parameters.ParameterView;
import csbase.client.algorithms.parameters.ParameterView.Mode;
import csbase.client.algorithms.parameters.validators.ValidationMode;
import csbase.client.algorithms.parameters.validators.ValidationResult;
import csbase.client.algorithms.view.AbstractAlgorithmConfiguratorPanel;
import csbase.client.desktop.RemoteTask;
import csbase.client.kernel.ClientException;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.logic.algorithms.flows.Flow;
import csbase.logic.algorithms.flows.configurator.FlowAlgorithmConfigurator;

/**
 * Painel representando a viso dos parmetros de um configurador de fluxo.
 * 
 * @author Tecgraf / PUC-Rio
 */
public class FlowAlgorithmConfiguratorPanel extends
  AbstractAlgorithmConfiguratorPanel<FlowAlgorithmConfigurator> {

  /**
   * Viso do fluxo.
   */
  private Graph graph;

  /**
   * A rea de trabalho onde o fluxo  desenhado.
   */
  private Workspace workspace;

  /**
   * Construtor
   * 
   * @param owner A janela que criou esta (No aceita {@code null}).
   * @param configurator o configurador de algortimo que ser representado por
   *        este painel.
   * @param mode Modo de visualizao. Pode ser
   *        {@link ParameterView.Mode#CONFIGURATION} ou
   *        {@link ParameterView.Mode#REPORT}.
   * 
   * @throws ClientException
   */
  public FlowAlgorithmConfiguratorPanel(final Window owner,
    final FlowAlgorithmConfigurator configurator, final Mode mode)
    throws ClientException {

    super(owner, configurator, mode);
  }

  /**
   * Obtm o grafo.
   * 
   * @return O grafo.
   */
  public Graph getGraph() {
    return graph;
  }

  /**
   * <p>
   * Obtm a rea de trabalho.
   * </p>
   * 
   * <p>
   * A rea de trabalho  a regio onde so desenhados os elementos do grafo que
   * representa o fluxo de algoritmos. Ela  responsvel pelo controle de zoom e
   * o controle das barras de rolagem.
   * </p>
   * 
   * @return a rea de trabalho.
   */
  public Workspace getWorkspace() {
    return workspace;
  }

  /**
   * <p>
   * Obtm o configurador de algortimo representado por este painel.
   * </p>
   * <p>
   * O Graph representa a viso de um fluxo. Um Graph  construido a partir de
   * um modelo (Flow) que  obtido atravs de um configurador
   * (FlowAlgorithmConfigurator). Uma vez construido, o Graph se livra do modelo
   * e a partir dai passa a agir de forma independente a ele e como consequncia
   * ao configurador que criou o modelo. Sendo assim, se existe um Graph, o
   * configurador deve ser criado atravs deste, pois o configurador inicial
   * poderia estar desatualizado. Este mtodo j trata isso.
   * </p>
   * 
   * @return o configurador de algortimo representado por este painel.
   */
  @Override
  public FlowAlgorithmConfigurator getConfigurator() {
    FlowAlgorithmConfigurator algorithmConfigurator = super.getConfigurator();
    if (getGraph() == null) {
      return algorithmConfigurator;
    }
    else {
      AlgorithmVersionInfo version =
        algorithmConfigurator.getAlgorithmVersion();
      Flow flow = getGraph().toFlow();
      if (version == null) {
        String filePath = algorithmConfigurator.getFilePath();
        if (filePath == null) {
          return new FlowAlgorithmConfigurator(flow);
        }
        return new FlowAlgorithmConfigurator(filePath, flow);
      }
      return new FlowAlgorithmConfigurator(version, flow);
    }
  }

  /**
   * @see csbase.client.algorithms.view.AbstractAlgorithmConfiguratorPanel#resetValues()
   */
  @Override
  public void resetValues() {
    super.resetValues();
    Collection<GraphNode> nodes = graph.getNodeCollection();
    for (GraphNode node : nodes) {
      node.getAlgorithmConfiguratorView().getConfigurator().resetValues();
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected AbstractParametersPanel createParametersReportPanel()
    throws ClientException {
    return new FlowParametersReportPanel();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected AbstractParametersPanel createParametersConfigurationPanel()
    throws ClientException {
    return new FlowParametersConfigurationPanel();
  }

  /**
   * Painel interno que detm a rea de trabalho onde o fluxo  desenhado.
   */
  private abstract class AbstractFlowParametersPanel extends
    AbstractParametersPanel {

    /**
     * Construtor
     * 
     * @throws ClientException
     */
    protected AbstractFlowParametersPanel() throws ClientException {

      RemoteTask<Graph> graphCreatorTask = new RemoteTask<Graph>() {
        @Override
        protected void performTask() throws Exception {
          FlowAlgorithmConfigurator configurator = getConfigurator();
          Flow flow = configurator.getFlow();
          Graph graph =
            new Graph(FlowAlgorithmConfiguratorPanel.this.getOwner(), flow);
          setResult(graph);
        }
      };

      boolean successful =
        graphCreatorTask
          .execute(
            FlowAlgorithmConfiguratorPanel.this.getOwner(),
            LNG
              .get("csbase.client.algorithms.view.flow.FlowAlgorithmConfiguratorPanel.reading_configurator_title"),
            LNG
              .get("csbase.client.algorithms.view.flow.FlowAlgorithmConfiguratorPanel.reading_configurator"));

      if (successful) {
        graph = graphCreatorTask.getResult();
        graph.setFullValidation(true);
        workspace = new Workspace(graph, false);
        workspace.pack();

        createFilters(workspace);
        initialize(workspace);
      }
      else {
        throw new ClientException(graphCreatorTask.getError());
      }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void setValidationMode(ValidationMode validationMode) {
      for (GraphNode node : getGraph().getNodeCollection()) {
        AlgorithmConfiguratorView configuratorView =
          node.getAlgorithmConfiguratorView();
        if (configuratorView != null) {
          configuratorView.setValidationMode(validationMode);
        }
      }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public ValidationResult validate(boolean complain) {
      return getGraph().validate(complain);
    }

    /**
     * Inicializa os componentes da viso.
     * 
     * @param workspace A rea de trabalho onde o fluxo  desenhado.
     */
    protected abstract void initialize(final Workspace workspace);

    /**
     * Cria os filtros.
     * 
     * @param workspace A rea de trabalho onde o fluxo  desenhado.
     */
    private void createFilters(final Workspace workspace) {
      // Faz highlight no n que estiver com o mouse em cima.
      new HighlightElementFilter(workspace).attach();
      // Mostra a dica do n que estiver com o mouse em cima.
      new HintElementFilter(workspace).attach();
    }
  }

  /**
   * Painel de configurao do fluxo.
   */
  private final class FlowParametersConfigurationPanel extends
    AbstractFlowParametersPanel {

    /**
     * Construtor
     * 
     * @throws ClientException
     */
    protected FlowParametersConfigurationPanel() throws ClientException {
      super();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void initialize(final Workspace workspace) {
      setLayout(new GridBagLayout());
      add(new JScrollPane(workspace), new GBC(0, 0).both());

      new AskForParameterValuesActionFilter(workspace).attach();
    }
  }

  /**
   * Painel de visualizao do fluxo.
   */
  private final class FlowParametersReportPanel extends
    AbstractFlowParametersPanel {

    /**
     * Construtor
     * 
     * @throws ClientException
     */
    protected FlowParametersReportPanel() throws ClientException {
      super();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void initialize(final Workspace workspace) {
      setLayout(new BorderLayout());
      add(new JScrollPane(workspace), BorderLayout.CENTER);

      final ZoomControl zoomControl = new ZoomControl(workspace.getZoomModel());
      add(zoomControl, BorderLayout.NORTH);
    }
  }
}
