package csbase.client.algorithms.view.xapp;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.Window;
import java.net.InetAddress;
import java.net.UnknownHostException;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;

import tecgraf.javautils.gui.GBC;
import tecgraf.javautils.gui.GUIUtils;
import csbase.client.Client;
import csbase.client.algorithms.parameters.OutputFileParameterView;
import csbase.client.algorithms.parameters.ParameterView;
import csbase.client.algorithms.parameters.TextParameterView;
import csbase.client.algorithms.parameters.ParameterView.Mode;
import csbase.client.algorithms.parameters.validators.ValidationMode;
import csbase.client.algorithms.parameters.validators.ValidationResult;
import csbase.client.algorithms.view.AbstractAlgorithmConfiguratorPanel;
import csbase.client.kernel.ClientException;
import csbase.logic.algorithms.xapps.XAppAlgorithmConfigurator;

/**
 * Painel representando a viso dos parmetros de um configurador xapp.
 * 
 * @author Tecgraf / PUC-Rio
 */
public class XAppAlgorithmConfiguratorPanel extends
  AbstractAlgorithmConfiguratorPanel<XAppAlgorithmConfigurator> {

  /**
   * @param owner
   * @param configurator
   * @param mode Modo de visualizao. Pode ser
   *        {@link ParameterView.Mode#CONFIGURATION} ou
   *        {@link ParameterView.Mode#REPORT}.
   * 
   * @throws ClientException
   */
  public XAppAlgorithmConfiguratorPanel(Window owner,
    XAppAlgorithmConfigurator configurator, ParameterView.Mode mode)
    throws ClientException {
    super(owner, configurator, mode);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected AbstractParametersPanel createParametersConfigurationPanel() {
    return new XappParametersConfigurationPanel();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected AbstractParametersPanel createParametersReportPanel() {
    return new XappParametersReportPanel();
  }

  /**
   * Painel interno que detm a viso dos parmetros.
   */
  private abstract class AbstractXappParametersPanel extends
    AbstractParametersPanel {

    /** O parmetro display para redirecionamento. */
    private TextParameterView displayParameter;

    /** O parmetro log. */
    private OutputFileParameterView logFileParameter;

    /**
     * Construtor.
     * 
     * @param mode Modo de visualizao. Pode ser
     *        {@link ParameterView.Mode#CONFIGURATION} ou
     *        {@link ParameterView.Mode#REPORT}.
     */
    protected AbstractXappParametersPanel(final Mode mode) {
      super();
      executeBeforeInitialization();
      initialize(mode);
      executeAfterInitialization();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void setValidationMode(ValidationMode validationMode) {
      displayParameter.getParameterValidator().setMode(validationMode);
      logFileParameter.getParameterValidator().setMode(validationMode);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public final ValidationResult validate(boolean complain) {
      ValidationResult validation = this.displayParameter.validate(complain);
      if (!validation.isWellSucceded()) {
        return validation;
      }
      // Testar ainda se segue o formato pro DISPLAY
      String displayValue = this.displayParameter.getParameter().getValue();
      if (displayValue == null
        || !displayValue.matches("[^:]*:[0-9]+[.][0-9]+")) {
        String errorMessage = "Display de sada do programa invlido.";
        if (complain) {
          this.displayParameter.getLabel().setForeground(Color.RED);
          Toolkit.getDefaultToolkit().beep();
          JOptionPane.showMessageDialog(getWindow(),
            errorMessage,
            "Erro no configurador de parmetros", JOptionPane.ERROR_MESSAGE);
          this.displayParameter.getLabel().setForeground(Color.BLACK);
        }
        return new ValidationResult(errorMessage);
      }
      ValidationResult logFileValidation =
        this.logFileParameter.validate(complain);
      if (!logFileValidation.isWellSucceded()) {
        return logFileValidation;
      }
      return new ValidationResult();
    }

    /**
     * Mtodo a ser executado logo antes da inicializao do painel.
     */
    protected void executeBeforeInitialization() {
    }

    /**
     * Mtodo a ser executado logo aps da inicializao do painel.
     */
    protected void executeAfterInitialization() {
    }

    /**
     * Inicializa o painel.
     * 
     * @param mode Modo de visualizao. Pode ser
     *        {@link ParameterView.Mode#CONFIGURATION} ou
     *        {@link ParameterView.Mode#REPORT}.
     */
    private void initialize(final Mode mode) {
      setLayout(new GridBagLayout());
      createParameterViews(mode);
      populateMainPanel();
    }

    /**
     * Cria as vises de parmetros.
     * 
     * @param mode Modo de visualizao. Pode ser
     *        {@link ParameterView.Mode#CONFIGURATION} ou
     *        {@link ParameterView.Mode#REPORT}.
     */
    private void createParameterViews(final Mode mode) {
      getConfigurator().setDefaultDisplay(getDefaultDisplay());
      this.displayParameter =
        new TextParameterView(getConfigurator().getDisplayParameter(), mode);
      this.logFileParameter =
        new OutputFileParameterView(getConfigurator().getLogFileParameter(),
          mode);
    }

    /**
     * Monta o display default do cliente.
     * 
     * @return uma string com esse valor
     */
    private String getDefaultDisplay() {
      try {
        String addr = Client.getInstance().getParameter("client_address");
        if (addr == null) {
          addr = InetAddress.getLocalHost().getHostName();
        }
        return addr + ":0.0";
      }
      catch (UnknownHostException e) {
        return "";
      }
    }

    /**
     * Preenche o painel principal.
     */
    private void populateMainPanel() {
      JPanel parametersPanel = new JPanel(new GridBagLayout());

      GridBagConstraints constraints = new GridBagConstraints();
      constraints.anchor = GridBagConstraints.NORTHWEST;
      constraints.fill = GridBagConstraints.HORIZONTAL;
      constraints.gridwidth = 1;
      constraints.gridheight = 1;
      constraints.gridx = 1;
      constraints.gridy = 1;
      constraints.insets = new Insets(2, 2, 2, 2);
      constraints.weightx = 1.0;
      constraints.weighty = 0.0;
      parametersPanel.add(createInputParameterPanel(), constraints);
      constraints.gridy++;
      parametersPanel.add(createLogParameterPanel(), constraints);

      add(new JScrollPane(parametersPanel), new GBC(0, 0).both());
    }

    /**
     * Cria o painel de entrada.
     * 
     * @return O paneil de entrada.
     */
    private JPanel createInputParameterPanel() {
      JComponent[] displayComponents = this.displayParameter.getComponents();
      return createGroupPanel("Parmetros de Entrada",
        new JComponent[][] { displayComponents });
    }

    /**
     * Cria o painel de log.
     * 
     * @return O painel de log.
     */
    private JPanel createLogParameterPanel() {
      JComponent[] logFileComponents = this.logFileParameter.getComponents();
      return createGroupPanel("Parmetros de Log",
        new JComponent[][] { logFileComponents });
    }

    /**
     * Cria um painel que agrupa os componentes.
     * 
     * @param title O ttulo do grupo.
     * @param components Os componentes.
     * 
     * @return O painel.
     */
    private JPanel createGroupPanel(String title, JComponent[][] components) {
      JPanel panel = GUIUtils.createBasicGridPanel(components);
      createBorder(title, panel, Color.BLACK);
      return panel;
    }

    /**
     * Cria uma borda nomeada no painel.
     * 
     * @param title O ttulo da borda.
     * @param panel O painel.
     * @param color A cor da borda.
     */
    private void createBorder(String title, JPanel panel, Color color) {
      Border lineBorder = BorderFactory.createLineBorder(color);
      TitledBorder titledBorder =
        BorderFactory.createTitledBorder(lineBorder, title);
      titledBorder.setTitleJustification(TitledBorder.LEFT);
      titledBorder.setTitleColor(color);
      panel.setBorder(titledBorder);
    }
  }

  /**
   * Painel em modo {@link ParameterView.Mode#CONFIGURATION}.
   */
  private final class XappParametersConfigurationPanel extends
    AbstractXappParametersPanel {

    /**
     * Construtor.
     */
    protected XappParametersConfigurationPanel() {
      super(Mode.CONFIGURATION);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void executeBeforeInitialization() {
      getConfigurator().resetValues();
    }
  }

  /**
   * Painel em modo {@link ParameterView.Mode#REPORT}.
   */
  private final class XappParametersReportPanel extends
    AbstractXappParametersPanel {

    /**
     * Construtor.
     */
    protected XappParametersReportPanel() {
      super(Mode.REPORT);
    }
  }
}
