/*
 * $Id: ApplicationManager.java 95867 2009-09-04 19:53:23Z clinio $
 */
package csbase.client.applicationmanager;

import java.awt.event.ActionEvent;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.net.URL;
import java.rmi.RemoteException;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.Locale;
import java.util.Set;
import java.util.Vector;
import java.util.Map.Entry;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

import tecgraf.javautils.LNG;
import csbase.client.applications.ApplicationProject;
import csbase.client.desktop.DesktopComponentFrame;
import csbase.client.desktop.DesktopFrame;
import csbase.client.remote.srvproxies.EventLogProxy;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.ProjectFile;
import csbase.logic.applicationservice.ApplicationCategory;
import csbase.logic.applicationservice.ApplicationRegistry;
import csbase.remote.ApplicationServiceInterface;
import csbase.remote.ClientRemoteLocator;
import csbase.util.restart.RestartListener;
import csbase.util.restart.RestartManager;

/**
 * Estrura interna
 * 
 * @author Tecgraf/PUC-Rio
 */
class InternalStructure {
  /**
   * Link
   */
  final private ApplicationLink link;

  /**
   * Ao
   */
  final private AbstractAction action;

  /**
   * Classe
   */
  private Class<? extends ApplicationType> clazz;

  /**
   * @return o campo link
   */
  final ApplicationLink getLink() {
    return link;
  }

  /**
   * @return o campo action
   */
  final AbstractAction getAction() {
    return action;
  }

  /**
   * @return o campo clazz
   */
  final Class<? extends ApplicationType> getApplicationClass() {
    return clazz;
  }

  /**
   * @param clazz ajusta o campo clazz
   */
  final void setApplicationClass(Class<? extends ApplicationType> clazz) {
    this.clazz = clazz;
  }

  /**
   * Construtor
   * 
   * @param action ao
   * @param link link
   */
  public InternalStructure(AbstractAction action, ApplicationLink link) {
    this.action = action;
    this.link = link;
  }
}

/**
 * Classe que implementa o gerente de aplicaes do cliente, que ser
 * representado por um singleton.
 * 
 * @author Tecgraf/PUC-Rio
 */
/**
 * @author Tecgraf/PUC-Rio
 */
public final class ApplicationManager {

  /**
   * Nome da fila de eventos cliente bsica de aplicativos
   */
  private final String appQueueName = "Applications";

  /**
   * Fila de eventos cliente bsica de aplicativos
   */
  private final String[] appQueue = { appQueueName };

  /**
   * Instncia do singleton para o gerente de servios.
   */
  private static ApplicationManager instance = null;

  /**
   * Lista de tipos de aplicaes registradas.
   */
  private final Hashtable<String, ApplicationRegistry> registries;

  /**
   * Lista de instncias de aplicaes em execuo.
   */
  final private Hashtable<String, Vector<ApplicationType>> runningApplications =
    new Hashtable<String, Vector<ApplicationType>>();

  /**
   * Seleo de idioma do usurio.
   */
  private Locale locale = null;

  /**
   * Associa os tipos de arquivos com as aplicaes
   */
  private Hashtable<String, Vector<String>> fileTypeAssociations = null;

  /**
   * Listeners
   */
  private final ArrayList<ApplicationManagerListener> listeners =
    new ArrayList<ApplicationManagerListener>();

  /**
   * Armazenamento de estrutura por identificador de aplicao.
   */
  final private Hashtable<String, InternalStructure> structures =
    new Hashtable<String, InternalStructure>();

  /**
   * Categorias de aplicaes.
   */
  final private Hashtable<String, ApplicationCategory> categories;

  /**
   * Inicializador esttico.
   */
  static {
    RestartManager.getInstance().addListener(new RestartListener() {
      public void restart() {
        ApplicationManager.instance = null;
      }
          });
  }

  /**
   * Mtodo que retorna a instncia do gerente.
   * 
   * @return a instncia.
   */
  public static ApplicationManager getInstance() {
    return ApplicationManager.instance;
  }

  /**
   * Ajuste da instncia
   * 
   * @param locale o Locale desejado.
   * @throws RemoteException em caso de falha.
   */
  public static void setInstance(final Locale locale) throws RemoteException {
    if (ApplicationManager.instance == null) {
      ApplicationManager.instance = new ApplicationManager(locale);
    }
  }

  /**
   * Adio de listener
   * 
   * @param listener o listener
   */
  final public void addListener(final ApplicationManagerListener listener) {
    listeners.add(listener);
  }

  /**
   * Faz a carga da classe da aplicao, buscando o atributo <i>className </i>
   * em suas propriedades.
   * 
   * @param reg regitry
   */
  private void loadClass(final ApplicationRegistry reg) {
    final String className = reg.getClassName();
    final String id = reg.getId();
    if (className == null) {
      final String err = "Classe indefinida para aplicao: " + id;
      throw new IllegalArgumentException(err);
    }
    final InternalStructure structure = getInternalStructure(id);
    if (structure.getApplicationClass() != null) {
      final String err = "Classe dupla carga para: " + id;
      throw new IllegalStateException(err);
    }
    try {
      final Class<?> clazz = Class.forName(className);
      @SuppressWarnings("unchecked")
      final Class<? extends ApplicationType> appClass =
        (Class<? extends ApplicationType>) clazz;
      if (appClass == null) {
        final String err = "Classe no foi carregada: " + className + ".";
        throw new IllegalStateException(err);
      }
      structure.setApplicationClass(appClass);
    }
    catch (final Exception e) {
      final String err = "Falha de carga para a aplicao: " + className;
      throw new IllegalStateException(err + " - " + e.getMessage(), e);
    }
  }

  /**
   * Consulta estrutura interna
   * 
   * @param id identificador da aplicao
   * @return estrutura
   */
  private InternalStructure getInternalStructure(final String id) {
    return structures.get(id);
  }

  /**
   * Constroi o mapeamento dos tipos de arquivos com as aplicaes.
   */
  private void buildFileTypesAssociations() {
    if (this.registries == null) {
      return;
    }
    this.fileTypeAssociations = new Hashtable<String, Vector<String>>();
    for (final Enumeration<ApplicationRegistry> e = this.registries.elements(); e
      .hasMoreElements();) {
      final ApplicationRegistry reg = e.nextElement();
      final List<String> types = reg.getFileTypes();
      if (types == null) {
        continue;
      }
      for (final String type : types) {
        // Verifica se j tem o tipo com alguma aplicao
        final Vector<String> apps = this.fileTypeAssociations.get(type);
        if (apps == null) {
          final Vector<String> newApps = new Vector<String>();
          newApps.add(reg.getId());
          this.fileTypeAssociations.put(type, newApps);
        }
        else {
          apps.add(reg.getId());
        }
      }
    }
  }

  /**
   * Consulta a possibilidade de habilitar a aplicao.
   * 
   * @param hasProject se existe projeto aberto.
   * @param reg registro da aplicao.
   * 
   * @return um flag indicativo.
   */
  private boolean canEnableApplication(final boolean hasProject,
    final ApplicationRegistry reg) {
    if (!reg.isEnabled()) {
      return false;
    }
    if (!reg.isRunnable()) {
      return false;
    }
    if (reg.requireProject()) {
      return hasProject;
    }
    return true;
  }

  /**
   * Construo de uam ao que dispare a aplicao
   * 
   * @param aid o identificador da aplicao.
   * 
   * @return a ao.
   */
  public AbstractAction createApplicationAction(final String aid) {
    final ApplicationRegistry reg = getApplicationRegistry(aid);
    if (reg == null) {
      return null;
    }
    final InternalStructure structure = getInternalStructure(aid);
    final AbstractAction action = structure.getAction();
    return action;
  }

  /**
   * Construo de umitem de menu que dispara uma aplicao.
   * 
   * @param aid o identificador da aplicao.
   * @param text .
   * 
   * @return um item de menu.
   */
  public JMenuItem createApplicationMenuItem(final String aid, final String text) {
    final ApplicationRegistry reg = getApplicationRegistry(aid);
    if (reg == null) {
      return null;
    }
    final AbstractAction action = createApplicationAction(aid);
    final JMenuItem mi = new JMenuItem(action);
    if (text != null) {
      mi.setText(text);
    }
    return mi;
  }

  /**
   * Cria a aplicao.
   * 
   * @param id O identificador.
   * 
   * @return A aplicao.
   * 
   * @throws ApplicationException Em caso de erro.
   */
  public ApplicationType createApplicationType(final String id)
    throws ApplicationException {
    try {
      final ApplicationRegistry reg = getApplicationRegistry(id);
      if (reg == null) {
        final String message =
          MessageFormat.format(LNG.get("application.manager.no.registry"),
          new Object[] { id });
        throw new ApplicationException(message);
      }
      if (reg.isSingleton()) {
        final Vector<ApplicationType> running = runningApplications.get(id);
        if (running != null && running.size() > 0) {
          return running.get(0);
        }
      }
      final String className = reg.getClassName();
      final Class<?> appClass = Class.forName(className);
      final Class<?>[] paramClass = new Class[] { String.class };
      final Constructor<?> appBuilder = appClass.getConstructor(paramClass);
      final Object[] pObjs = new Object[] { id };
      return (ApplicationType) appBuilder.newInstance(pObjs);
    }
    catch (final OutOfMemoryError e) {
      final String message =
        MessageFormat.format(LNG.get("application.manager.out.of.memory"),
        new Object[] { id });
      throw new ApplicationException(message, e);
    }
    catch (final InvocationTargetException e) {
      final Throwable appException = e.getTargetException();
      final String message =
        MessageFormat.format(LNG.get("application.manager.no.creation"),
        new Object[] { id, appException.getLocalizedMessage() });
      throw new ApplicationException(message, appException);
    }
    catch (final NoSuchMethodException e) {
      final String message =
        MessageFormat.format(LNG.get("application.manager.bad.configuration"),
        new Object[] { id });
      throw new ApplicationException(message, e);
    }
    catch (final InstantiationException e) {
      final String message =
        MessageFormat.format(LNG.get("application.manager.bad.instanciation"),
        new Object[] { id });
      throw new ApplicationException(message, e);
    }
    catch (final IllegalAccessException e) {
      final String message =
        MessageFormat.format(LNG.get("application.manager.bad.implementation"),
        new Object[] { id });
      throw new ApplicationException(message, e);
    }
    catch (final ClassNotFoundException e) {
      final String message =
        MessageFormat.format(LNG.get("application.manager.no.class"),
        new Object[] { id });
      throw new ApplicationException(message, e);
    }
  }

  /**
   * Remoo de listener
   * 
   * @param listener o listener
   */
  final public void delListener(final ApplicationManagerListener listener) {
    if (listeners.contains(listener)) {
      listeners.remove(listener);
    }
  }

  /**
   * Mtodo que termina todas as aplicaes registradas no ApplicationManager.
   */
  public void finishAllApplications() {
    for (final Enumeration<String> appKeys = this.runningApplications.keys(); appKeys
      .hasMoreElements();) {
      final String key = appKeys.nextElement();
      if (key != null) {
        final Vector<ApplicationType> running =
          this.runningApplications.get(key);
        final int i = 0;
        while (running.size() > 0) {
          final ApplicationType app = running.get(i);
          killApplication(app);
        }
      }
    }
  }

  /**
   * @return a coleo.
   */
  final public ArrayList<ApplicationRegistry> getAllApplicationRegistries() {
    final ArrayList<ApplicationRegistry> list =
      new ArrayList<ApplicationRegistry>();
    for (final Enumeration<String> e = this.registries.keys(); e
      .hasMoreElements();) {
      final String aid = e.nextElement();
      final ApplicationRegistry reg = getApplicationRegistry(aid);
      if (reg != null) {
        list.add(reg);
      }
    }
    return list;
  }

  /**
   * Busca de um array para um event-log
   * 
   * @param id id
   * @return a nova lista de strings
   */
  final public String[] getApplicationEventQueue(final String id) {
    return new String[] { appQueueName, id };

  }

  /**
   * Obtem a aplicao correspondente ao tipo de arquivo.
   * 
   * @param fileType o tipo de arquivo.
   * 
   * @return o identificador da aplicao.
   */
  public String getApplicationFromType(final String fileType) {
    if (fileType == null) {
      return null;
    }
    final Vector<String> apps = this.fileTypeAssociations.get(fileType);
    if (apps == null) {
      return null;
    }

    // Retorna a primeira aplicao associada ao arquivo
    return apps.get(0);
  }

  /**
   * Busca do cone configurada para a aplicao.
   * 
   * @param id identificador da aplicao.
   * 
   * @return .
   */
  public ImageIcon getApplicationIcon(final String id) {
    final ApplicationRegistry reg = getApplicationRegistry(id);
    if (reg == null) {
      return null;
    }

    final byte[] icDef = reg.getIconDefinition();
    if (icDef == null) {
      return null;
    }
    return new ImageIcon(icDef);
  }

  /**
   * Mtodo para retornar o identificador de uma aplicao.
   * 
   * @param app A aplicao cujo identificador procura-se determinar
   * 
   * @return o identificador
   */
  public String getApplicationId(final ApplicationType app) {
    final Enumeration<String> appKeys = this.runningApplications.keys();
    String key = null;
    while (appKeys.hasMoreElements()) {
      key = appKeys.nextElement();
      final Vector<ApplicationType> vector = this.runningApplications.get(key);
      if ((vector != null) && vector.contains(app)) {
        return key;
      }
    }
    return null;
  }

  /**
   * Busca de um array para um event-log (do manager)
   * 
   * @return a nova lista de strings
   */
  final public String[] getApplicationManagerEventQueue() {
    return appQueue;
  }

  /**
   * Mtodo para retorno do registry de uma aplicao.
   * 
   * @param id o identificador de aplicao.
   * 
   * @return o registro.
   */
  public ApplicationRegistry getApplicationRegistry(final String id) {
    if (registries == null) {
      return null;
    }
    return this.registries.get(id);
  }

  /**
   * Mtodo para retorno categoria de aplicaes
   * 
   * @param id o identificador
   * 
   * @return a categoria.
   */
  public ApplicationCategory getApplicationCategory(final String id) {
    if (categories == null) {
      return null;
    }
    return this.categories.get(id);
  }

  /**
   * Retorna a hash table de categorias.
   * 
   * @return a tabela.
   */
  public Hashtable<String, ApplicationCategory> getAllApplicationCategories() {
    return categories;
  }

  /**
   * Busca de um array para um event-log
   * 
   * @param id id
   * @param tag tag
   * @return a nova lista de strings
   */
  final public String[] getDetailedApplicationEventQueue(final String id,
    final String tag) {
    return new String[] { appQueueName, id, tag };

  }

  /**
   * Consulta o identificador de instncia
   * 
   * @param app aplicao.
   * @return id o identificador se a aplicao estiver rodando, ou um valor
   *         negativo caso contrrio.
   */
  final public int getInstanceId(final ApplicationType app) {
    final String id = app.getId();
    final Vector<ApplicationType> running = this.runningApplications.get(id);
    if (running == null) {
      return -1;
    }
    return running.indexOf(app) + 1;
  }

  /**
   * Obtem o locale das aplicaes.
   * 
   * @return um locale.
   */
  public Locale getLocale() {
    return this.locale;
  }

  /**
   * Mtodo de acesso para a lista de instncias de aplicaes em execuo.
   * 
   * @return Lista de instncias de aplicaes em execuo.
   */
  final public Hashtable<String, Vector<ApplicationType>> getRunningApplications() {
    return this.runningApplications;
  }

  /**
   * Mtodo de acesso para a lista de instncias de aplicaes em execuo.
   * 
   * @param id id
   * @return Lista de instncias de aplicaes em execuo.
   */
  public Vector<ApplicationType> getRunningApplications(final String id) {
    return this.runningApplications.get(id);
  }

  /**
   * Indica se a aplicao est rodando.
   * 
   * @param app aplicao.
   * @return true se a aplicao est rodando, false caso contrrio.
   */
  public boolean isApplicationRunning(final ApplicationType app) {
    final Vector<ApplicationType> running =
      this.runningApplications.get(app.getId());
    if (running == null) {
      return false;
    }
    return running.contains(app);
  }

  /**
   * Mtodo para finalizar uma instncia de aplicao.
   * 
   * @param app a instncia a ser finalizada
   * 
   * @return flag de sucesso da operao.
   */
  public boolean killApplication(final ApplicationType app) {
    try {
      setAsDead(app);
      app.killApplication();
      app.finishApplication();
      return true;
    }
    catch (final Exception e) {
      return false;
    }
  }

  /**
   * Mtodo para notificao de fim de aplicao (via usurio) ao gerente de
   * aplicaes.
   * 
   * @param app a aplicao
   */
  public void notifyDeath(final ApplicationType app) {
    setAsDead(app);
  }

  /**
   * Executa uma aplicao.
   * 
   * @param application A aplicao.
   * 
   * @throws ApplicationException Em caso de erro ao executar a aplicao.
   */
  public void runApplication(final ApplicationType application)
    throws ApplicationException {
    application.launchApplication();
    setAsRunning(application);
  }

  /**
   * Cria e execuo uma aplicao que saiba lidar com o tipo do arquivo
   * informado.
   * 
   * @param file O arquivo (No aceita {@code null}).
   * 
   * @throws ApplicationException Se houver um erro durante a execuo da
   *         aplicao ou se no existe uma aplicao que lide com o tipo deste
   *         arquivo.
   */
  public void runApplication(final ProjectFile file)
    throws ApplicationException {
    if (file == null) {
      throw new IllegalArgumentException("O parmetro file est nulo.");
    }
    final String appId =
      ApplicationManager.getInstance().getApplicationFromType(file.getType());
    if (appId == null) {
      throw new ApplicationException(String.format(LNG
        .get("application.manager.no_application_found_for_file_type"), file
        .getType()));
    }
    ApplicationType appType;
    try {
      appType = createApplicationType(appId);
      runApplication(appType);
      appType.sendMessage(ApplicationProject.PROJECT_FILE_MESSAGE, file);
      appType.postInitialization();
    }
    catch (final Exception e) {
      final String msg =
        String.format(LNG.get("application.manager.failed.program"), appId);
      EventLogProxy.addClientInformation(appQueue, new String[] { msg });
      throw new ApplicationException(msg, e);
    }
  }

  /**
   * Mtodo para criao de uma instncia de aplicao. Extraordinariamente,
   * este mtodo inibe <i>warning</i> de unchecked type, pois precisa fazer um
   * <i>cast</i> explcito para o tipo (classe) que efetivamente define a
   * aplicao.
   * 
   * @param id o identificador da aplicao.
   * @param <A> a classe da aplicao chamada.
   * @return a aplicao instanciada.
   * @throws ApplicationException se houver falha no lanamento da aplicao.
   */
  @SuppressWarnings("unchecked")
  public <A extends ApplicationType> A runApplication(final String id)
    throws ApplicationException {
    if (id == null) {
      throw new ApplicationException("application null");
    }
    try {
      final ApplicationType app = createApplicationType(id);
      runApplication(app);
      app.postInitialization();
      return (A) app;
    }
    catch (final Exception e) {
      final String msg =
        String.format(LNG.get("application.manager.failed.program"), id);
      EventLogProxy.addClientInformation(appQueue, new String[] { msg });
      throw new ApplicationException(msg, e);
    }
  }

  /**
   * Mtodo para criao de uma instncia de aplicao com base em sua classe.
   * 
   * @param <A> classe da aplicao
   * @param clazz classe
   * @return a instncia executada.
   * @throws ApplicationException se houver falha no lanamento.
   */
  public <A extends ApplicationType> A runApplication(final Class<A> clazz)
    throws ApplicationException {
    final String id = getApplicationIdFromClass(clazz);
    // O trecho "this.<A>"  um workaround sugerido para um bug do javac 1.6: http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6302954
    return this.<A> runApplication(id);
  }

  /**
   * Indica a aplicao como terminada.
   * 
   * @param app o tipo da aplicao.
   */
  private void setAsDead(final ApplicationType app) {
    final String id = app.getId();
    final Vector<ApplicationType> running = this.runningApplications.get(id);
    if (running.contains(app)) {
      running.remove(app);
    }

    for (final ApplicationManagerListener lst : listeners) {
      lst.applicationEnded(id);
    }
  }

  /**
   * Indica a aplicao como em execuo.
   * 
   * @param app o tipo da aplicao.
   */
  private void setAsRunning(final ApplicationType app) {
    final String id = app.getId();
    Vector<ApplicationType> running = this.runningApplications.get(id);
    if (running == null) {
      this.runningApplications.put(id, new Vector<ApplicationType>());
      running = this.runningApplications.get(id);
    }
    final ApplicationRegistry registry = app.getApplicationRegistry();
    if (!running.contains(app) || !registry.isSingleton()) {
      running.add(app);
    }

    for (final ApplicationManagerListener lst : listeners) {
      lst.applicationStarted(id);
    }
  }

  /**
   * Ajuste de flag de necessidade de projeto.
   * 
   * @param hasProject flag indicativo.
   */
  public void setProjectFlag(final boolean hasProject) {
    if (registries == null) {
      return;
    }
    for (final ApplicationRegistry reg : registries.values()) {
      if (reg == null) {
        continue;
      }
      final String aid = reg.getId();
      final InternalStructure structure = getInternalStructure(aid);
      final ApplicationLink lnk = structure.getLink();
      final AbstractAction ac = structure.getAction();
      final boolean enable = canEnableApplication(hasProject, reg);
      if (lnk != null) {
        lnk.setEnabled(enable);
      }
      if (ac != null) {
        ac.setEnabled(enable);
      }
    }
  }

  /**
   * Obtm o link da aplicao.
   * 
   * @param reg registry
   * @return o link.
   */
  final public ApplicationLink getApplicationLink(ApplicationRegistry reg) {
    return structures.get(reg.getId()).getLink();
  }

  /**
   * Busca uma imagem para a aplicao.
   * 
   * @param reg registry
   * @param path caminho denro do diretrio da classe da aplicao.
   * @return a imagem ou null
   */
  final public ImageIcon getImageIcon(final ApplicationRegistry reg,
    final String path) {
    final String id = reg.getId();
    final InternalStructure structure = getInternalStructure(id);
    Class<? extends ApplicationType> clazz = structure.getApplicationClass();

    // TODO : Retirar essa carga pois anula o esquema de carga
    // de aplicaes sob demanda.
    if (clazz == null) {
      loadClass(reg);
      clazz = structure.getApplicationClass();
    }
    final String sep = "/";
    final String resPath = "resources" + sep + "images" + sep + path;
    try {
      final URL res = clazz.getResource(resPath);
      if (res == null) {
        // System.out.println("Path errado (resource): " + clazz.getName() + "/"+ resPath);
        return null;
      }
      return new ImageIcon(res);
    }
    catch (final Exception e) {
      // Comentado para que no saia mensagens de erro
      // quando a aplicao no usar o mecanismo default
      // de imagens. O cdigo abaixo deve ser usado par depurao.
      //            System.out.println("Path errado: " + path);
      //            System.out.println("Path errado (resource): " + resPath);
      //            e.printStackTrace();
      return null;
    }
  }

  /**
   * Obtm o link da aplicao.
   * 
   * @param reg registry
   * @return a ao.
   */
  final public AbstractAction getApplicationAction(ApplicationRegistry reg) {
    if (reg == null) {
      return null;
    }
    final String id = reg.getId();
    final InternalStructure structure = getInternalStructure(id);
    return structure.getAction();
  }

  /**
   * Construtor interno.
   * 
   * @param locale o Locale desejado.
   * @throws RemoteException em caso de falha.
   */
  private ApplicationManager(final Locale locale) throws RemoteException {
    this.locale = locale;
    this.registries = loadApplicationRegistries();
    this.categories = loadApplicationCategories();
    buildFileTypesAssociations();
    setProjectFlag(false);
  }

  /**
   * Faz a carga dos regostries de aplicao.
   * 
   * @return a hash.
   * @throws RemoteException em caso de falha.
   */
  private Hashtable<String, ApplicationRegistry> loadApplicationRegistries()
    throws RemoteException {
    final ApplicationServiceInterface appService =
      ClientRemoteLocator.applicationService;
    if (appService == null) {
      return null;
    }

    final Hashtable<String, ApplicationRegistry> regs =
      appService.getApplicationRegistries();
    for (ApplicationRegistry reg : regs.values()) {
      reg.loadBundles(locale);
      final String id = reg.getId();
      final AbstractAction action = buildAction(reg);
      final ApplicationLink link = buildLink(reg);
      final InternalStructure structure = new InternalStructure(action, link);
      structures.put(id, structure);
    }
    return regs;
  }

  /**
   * Faz a carga dos regostries de aplicao.
   * 
   * @return a hash.
   * @throws RemoteException em caso de falha.
   */
  private Hashtable<String, ApplicationCategory> loadApplicationCategories()
    throws RemoteException {
    final ApplicationServiceInterface appService =
      ClientRemoteLocator.applicationService;
    if (appService == null) {
      throw new IllegalStateException("ApplicationService not found or loaded");
    }

    final Hashtable<String, ApplicationCategory> cats =
      appService.getApplicationCategories();
    return cats;
  }

  /**
   * Cria o link
   * 
   * @param reg registry
   * @return link
   */
  private ApplicationLink buildLink(final ApplicationRegistry reg) {
    final String name = reg.getApplicationName(locale);

    final byte[] imageDefinition = reg.getImageDefinition();
    ImageIcon image = null;
    if (imageDefinition != null) {
      image = new ImageIcon(imageDefinition);
    }

    final ApplicationLink link = new ApplicationLink(name, image) {
      @Override
      public void executeAction() {
        runApplicationFromRegistry(reg);
      }
    };

    return link;
  }

  /**
   * Executa uma aplicao com bas em um registry
   * 
   * @param reg registry
   */
  private void runApplicationFromRegistry(final ApplicationRegistry reg) {
    final ApplicationManager manager = ApplicationManager.getInstance();
    try {
      final String id = reg.getId();
      final InternalStructure structure = getInternalStructure(id);
      final Class<? extends ApplicationType> clazz =
        structure.getApplicationClass();
      if (clazz == null) {
        loadClass(reg);
      }
      manager.runApplication(id);
    }
    catch (final ApplicationException e) {
      final DesktopFrame desktop = DesktopFrame.getInstance();
      final DesktopComponentFrame frame = desktop.getDesktopFrame();
      StandardErrorDialogs.showErrorDialog(frame, e);
    }
  }

  /**
   * Criao de ao interna.
   * 
   * @param reg registry
   * @return ao.
   */
  private AbstractAction buildAction(final ApplicationRegistry reg) {
    final AbstractAction action = new AbstractAction() {
      public void actionPerformed(final ActionEvent ae) {
        runApplicationFromRegistry(reg);
      }
    };

    action.putValue(Action.NAME, reg.getApplicationName(locale));

    final byte[] iconDefinition = reg.getIconDefinition();
    if (iconDefinition != null) {
      action.putValue(Action.SMALL_ICON, new ImageIcon(iconDefinition));
    }

    return action;
  }

  /**
   * Consulta o nome de uma aplicao.
   * 
   * @param applicationRegistry registry
   * @return nome
   */
  public String getApplicationName(ApplicationRegistry applicationRegistry) {
    return applicationRegistry.getApplicationName(locale);
  }

  /**
   * Consulta o nome de uma categoria.
   * 
   * @param category category
   * @return nome
   */
  public String getCategoryName(ApplicationCategory category) {
    return category.getCategoryName(locale);
  }

  /**
   * Consulta a descrio de uma aplicao.
   * 
   * @param applicationRegistry registry
   * @return descrio
   */
  public String getApplicationDescription(
    ApplicationRegistry applicationRegistry) {
    return applicationRegistry.getApplicationDescription(locale);
  }

  /**
   * Consulta o id de uma aplicao com base na classe que a implementa.
   * 
   * @param clazz classe da aplicao
   * @return o id da aplicao (se registrada) ou <code>null</code>.
   */
  private String getApplicationIdFromClass(
    final Class<? extends ApplicationType> clazz) {
    final String className = clazz.getName();
    final Set<Entry<String, ApplicationRegistry>> entries =
      registries.entrySet();
    for (Entry<String, ApplicationRegistry> entry : entries) {
      final ApplicationRegistry reg = entry.getValue();
      if (className.equals(reg.getClassName())) {
        return reg.getId();
      }
    }
    return null;
  }

  /**
   * Constroi um menu com applicaes de uma categoria.
   * 
   * @param cat categoria
   * @return menu
   */
  final public JMenu buildCategoryMenu(ApplicationCategory cat) {
    final JMenu menu = new JMenu();
    final ArrayList<String> ids = cat.getApplicationIds();
    for (String id : ids) {
      final ApplicationRegistry reg = getApplicationRegistry(id);
      if (reg.isShownAtApplicationMenu()) {
        final AbstractAction action = getApplicationAction(reg);
        menu.add(action);
      }
    }
    menu.setText(cat.getCategoryName(locale));
    final byte[] ic = cat.getIcon();
    if (ic != null) {
      menu.setIcon(new ImageIcon(ic));
    }
    return menu;
  }

}
