/*
 * $Id: Application.java 94191 2009-07-27 19:36:37Z clinio $
 */
package csbase.client.applications;

import java.awt.Component;
import java.awt.Frame;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.IOException;
import java.io.InputStream;
import java.util.Locale;
import java.util.Properties;
import java.util.ResourceBundle;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.SwingConstants;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.applicationmanager.ApplicationException;
import csbase.client.applicationmanager.ApplicationManager;
import csbase.client.applicationmanager.ApplicationType;
import csbase.client.desktop.DesktopComponentFrame;
import csbase.client.desktop.DesktopFrame;
import csbase.client.remote.srvproxies.EventLogProxy;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.applicationservice.ApplicationRegistry;

/**
 * Classe bsica de aplicao com implementao para um suporte mnimo:
 * lanamento e encerramento de suas intncias, janela principal etc.
 * 
 * @author Tecgraf/PUC-Rio
 */
public abstract class Application extends ApplicationType {

  /**
   * Janela principal
   */
  private final ApplicationFrame mainFrame;

  /**
   * Barra de Status de uma aplicao
   */
  private JLabel statusBar;

  /**
   * Vetor de janelas
   */
  private final Vector<Window> dependentWindows;

  /**
   * estado inicial da janela da aplicao
   */
  private int initialFrameState;

  /**
   * {@inheritDoc}
   */
  @Override
  public final boolean closeApplication() {
    if (userCanKillApplication()) {
      killApplication();
      finishApplication();
      return true;
    }
    return false;
  }

  /**
   * Criacao da barra de status que pode ser adicionada em uma aplicao.
   */
  private void buildStatusBar() {
    this.statusBar = new JLabel();
    this.statusBar.setHorizontalAlignment(SwingConstants.LEFT);
  }

  /**
   * Obtem a barra de status para ser usada em uma aplicao.
   * 
   * @return .
   */
  public JLabel getStatusBar() {
    return this.statusBar;
  }

  /**
   * Retorna o locale
   * 
   * @return o locale
   */
  public Locale getLocale() {
    return ApplicationManager.getInstance().getLocale();
  }

  /**
   * Consulta as propriedades pr-configuradas no manager.
   * 
   * @return as propriedades
   */
  public final String getApplicationCommand() {
    return this.getApplicationRegistry().getApplicationCommand();
  }

  /**
   * Consulta ao diretrio onde est o comando da aplicao.
   * 
   * @return o diretrio onde est o comando da aplicao
   */
  public final String getApplicationCommandDir() {
    return this.getApplicationRegistry().getApplicationCommandDir();
  }

  /**
   * Obtm a representao de um caminho relativo  raiz do sistema a partir de
   * uma representao relativa  raiz da aplicao.
   * 
   * @param relativePath o caminho relativo  raiz da aplicao (ex:
   *        resources/Notepad_pt_BR.properties).
   * 
   * @return o caminho relativo  raiz do sistema (ex:
   *         /csbase/client/applications/notepad/resources/
   *         Notepad_pt_BR.properties).
   * 
   * @throws ApplicationException se houver falha.
   */
  private String getInternalPath(final String relativePath)
    throws ApplicationException {
    String className = null;
    try {
      className = this.getApplicationRegistry().getClassName();
      final int idx = className.lastIndexOf(".");
      final String path = className.substring(0, idx);
      final String resPath = "/" + path.replace('.', '/') + "/" + relativePath;
      return resPath;
    }
    catch (final Exception e) {
      final String err = "[" + className + "] - Falha de path de resource";
      throw new ApplicationException(err + " : " + e.getMessage());
    }
  }

  /**
   * Indica se a aplicao est rodando em ambiente de execuo client-servidor;
   * ou seja, conectada a um servidor CSBASE.
   * 
   * @return indicativo
   */
  final public boolean isConnectedToServer() {
    return true;
  }

  /**
   * Faz a carga de um arquivo da aplicao.
   * 
   * @param relativePath path relativo  aplicao.
   * 
   * @return um objeto do tipo <code>Properties</code>.
   * 
   * @throws ApplicationException se houver falha.
   */
  public final Properties loadProperty(final String relativePath)
    throws ApplicationException {
    final String resPath = getInternalPath("resources/" + relativePath);
    final InputStream in = getClass().getResourceAsStream(resPath);
    if (in == null) {
      final String err = "[" + resPath + "] falhou na carga!";
      throw new ApplicationException(err);
    }
    try {
      final Properties property = new Properties();
      property.load(in);
      return property;
    }
    catch (final Exception e) {
      final String err = "[" + relativePath + "] falhou na carga de bundle!";
      throw new ApplicationException(err + " : " + e.getMessage());
    }
    finally {
      try {
        in.close();
      }
      catch (final IOException e) {
        e.printStackTrace();
      }
    }
  }

  /**
   * Faz a carga de um arquivo como uma stream.
   * 
   * @param relativePath path relativo  aplicao.
   * 
   * @return um objeto do tipo <code>InputStream</code>.
   * 
   * @throws ApplicationException se houver falha.
   */
  public final InputStream loadResourceAsStream(final String relativePath)
    throws ApplicationException {
    final String resPath = getInternalPath("resources/" + relativePath);
    final InputStream in = getClass().getResourceAsStream(resPath);
    if (in == null) {
      final String err = "[" + resPath + "] falhou na carga!";
      throw new ApplicationException(err);
    }
    return in;
  }

  /**
   * Monta uma imagem do resource da aplicao.
   * 
   * @param tag nome do arquivo de imagem.
   * 
   * @return uma imagem (cone)
   */
  public final ImageIcon buildApplicationImage(final String tag) {
    final ApplicationManager manager = ApplicationManager.getInstance();
    final ApplicationRegistry reg = getApplicationRegistry();
    final ImageIcon image = manager.getImageIcon(reg, tag + ".gif");
    if (image == null) {
      return ApplicationImages.ICON_NONE;
    }
    return image;
  }

  /**
   * Criao de um dilogo modal da aplicao.
   * 
   * @param father um dilogo pai.
   * 
   * @return um dilogo da aplicao.
   */
  public final ApplicationComponentDialog buildApplicationDialog(
    final ApplicationComponentFrame father) {
    return new ApplicationComponentDialog(father, this.getName(), this);
  }

  /**
   * Criao de um frame da aplicao.
   * 
   * @return um frame da aplicao.
   */
  public final ApplicationComponentFrame buildApplicationFrame() {
    return new ApplicationComponentFrame(this.getName(), this);
  }

  /**
   * Retorna o dilogo principal da aplicao.
   * 
   * @return a janela.
   */
  public final ApplicationFrame getApplicationFrame() {
    return this.mainFrame;
  }

  /**
   * Consulta ao bundle da aplicao.
   * 
   * @return o bundle
   */
  public final ResourceBundle getResourceBundle() {
    return this.getApplicationRegistry().getResourceBundle();
  }

  /**
   * Consulta a existncia de uma string de locale
   * 
   * @param key a chave de consulta
   * 
   * @return um indicativo
   */
  public final boolean hasString(final String key) {
    return this.getApplicationRegistry().hasString(key);
  }

  /**
   * Consulta a uma string de locale
   * 
   * @param key a chave de consulta
   * 
   * @return a string baseada no locale.
   */
  public final String getString(final String key) {
    return this.getApplicationRegistry().getString(key);
  }

  /**
   * Obtm a verso localizada de uma mensagem que contenha parmetros.
   * 
   * @param key chave para a mensagem que contm parmetros.
   * @param args argumentos para a mensagem.
   * 
   * @return mensagem preenchida pelos argumentos e localizada.
   * 
   * @see java.text.MessageFormat
   */
  public final String getString(final String key, final Object[] args) {
    return this.getApplicationRegistry().getString(key, args);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void startApplication() throws ApplicationException {
    final DesktopFrame desktop = DesktopFrame.getInstance();
    final DesktopComponentFrame frame = desktop.getDesktopFrame();

    this.mainFrame.setTitle(getName());
    final ApplicationManager manager = ApplicationManager.getInstance();
    final ApplicationRegistry reg = manager.getApplicationRegistry(getId());
    final byte[] iconDefinition = reg.getIconDefinition();
    if (iconDefinition != null) {
      final ImageIcon imageIcon = new ImageIcon(iconDefinition);
      this.mainFrame.setIconImage(imageIcon.getImage());
    }

    if (this.initialFrameState != Frame.NORMAL) {
      this.mainFrame.pack();
      this.mainFrame.setExtendedState(this.initialFrameState);
      // usamos validate() para garantir que mudanas no tamanho (p.ex.
      // maximize) sejam consideradas antes do frame ser exibido
      this.mainFrame.validate();
    }
    // centralizamos a nova janela com relao ao seu pai ou ao fundo da tela
    if (frame == null) {
      final Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
      this.mainFrame.center(defaultToolkit.getScreenSize(), 0, 0);
    }
    else {
      this.mainFrame.center(frame);
    }
    this.mainFrame.setVisible(reg.mainFrameIsVisible());
    this.mainFrame.toFront();

    final String[] aQueue = manager.getApplicationManagerEventQueue();
    final String[] initMsg =
      { LNG.get("application.manager.started.program"), getName(),
          "HASHCODE:" + this.hashCode(), getId() };
    EventLogProxy.addClientInformation(aQueue, initMsg);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public final void finishApplication() {
    shutdownDependents();
    this.mainFrame.setVisible(false);
    this.mainFrame.delApplicationManagerListener();
    final ApplicationManager manager = ApplicationManager.getInstance();
    manager.notifyDeath(this);
    this.mainFrame.close();
    final String[] aQueue = manager.getApplicationManagerEventQueue();
    final String[] initMsg =
      { LNG.get("application.manager.finished.program"), getName(),
          "HASHCODE:" + this.hashCode(), getId() };
    EventLogProxy.addClientInformation(aQueue, initMsg);
  }

  /**
   * Mtodo para log de informaes especficas do cliente (applicao).
   * 
   * @param info mensagem a ser exibida.
   */
  public final void logApplicationEvent(final String info) {
    logApplicationEvent(new String[] { info });
  }

  /**
   * Mtodo para log de informaes especficas do cliente (applicao).
   * 
   * @param info mensagem a ser exibida.
   */
  public final void logApplicationEvent(final String[] info) {
    final String id = getId();
    final ApplicationManager mgr = ApplicationManager.getInstance();
    final String[] aQueue = mgr.getApplicationEventQueue(id);
    EventLogProxy.addClientInformation(aQueue, info);
  }

  /**
   * Mtodo para log de informaes especficas do cliente (applicao).
   * 
   * @param tag ??
   * @param info mensagem a ser exibida.
   */
  public final void logDetailedApplicationEvent(final String tag,
    final String info) {
    logDetailedApplicationEvent(tag, new String[] { info });
  }

  /**
   * Mtodo para log de informaes especficas do cliente (applicao).
   * 
   * @param tag ??
   * @param info mensagem a ser exibida.
   */
  public final void logDetailedApplicationEvent(final String tag,
    final String[] info) {
    final String id = getId();
    final ApplicationManager mgr = ApplicationManager.getInstance();
    final String[] aQueue = mgr.getDetailedApplicationEventQueue(id, tag);
    EventLogProxy.addClientInformation(aQueue, info);
  }

  /**
   * Mtodo para exibio de uma mensagem informativa.
   * 
   * @param msg mensagem a ser exibida.
   */
  public final void showMessage(final Object msg) {
    final String title = getApplicationFrame().getTitle();
    showMessage(this.mainFrame, title, msg, JOptionPane.INFORMATION_MESSAGE);
  }

  /**
   * Mtodo para exibio de uma mensagem do tipo especificado.
   * 
   * @param parent elemento (janela) pai.
   * @param title ttulo.
   * @param msg mensagem a ser exibida.
   * @param type tipo da mensagem.
   */
  final public void showMessage(final Component parent, final String title,
    final Object msg, final int type) {
    final Component father = (parent == null ? this.mainFrame : parent);
    JOptionPane.showMessageDialog(father, msg, title, type);
  }

  /**
   * Consulta a opes do programa.
   * 
   * @param parent componente-pai do dilogo a ser exibido.
   * @param msg mensagem a ser exibida.
   * @param options array de opes.
   * @return a opo selecionada.
   */
  final public int showOptionDialog(final Component parent, final String msg,
    final String[] options) {
    final String title = getName();
    return StandardDialogs.showOptionDialog(parent, title, msg, options);
  }

  /**
   * Consulta a opes do programa.
   * 
   * @param msg mensagem a ser exibida.
   * @param options array de opes.
   * @return a opo selecionada.
   */
  final public int showOptionDialog(final String msg, final String[] options) {
    return showOptionDialog(this.mainFrame, msg, options);
  }

  /**
   * Mtodo para exibio de uma pilha de erro (exceo).
   * 
   * @param t a exceo.
   */
  final public void showExceptionStack(final Throwable t) {
    StandardErrorDialogs.showExceptionDialog(this.mainFrame, getName(), t);
  }

  /**
   * Mtodo para exibio de uma mensagem de erro, juntamente com a exceo
   * associada.
   * 
   * @param msg uma mensagem.
   * @param t a exceo.
   */
  final public void showException(final String msg, final Throwable t) {
    StandardErrorDialogs.showErrorDialog(this.mainFrame, getName(), msg, t);
  }

  /**
   * Mtodo para exibio de uma mensagem de erro.
   * 
   * @param msg o texto mensagem.
   */
  final public void showError(final Object msg) {
    showError(getApplicationFrame(), msg);
  }

  /**
   * Mtodo para exibio de uma mensagem de erro.
   * 
   * @param parent elemento (janela) pai.
   * @param msg o texto da mensagem.
   */
  final public void showError(final Component parent, final Object msg) {
    showMessage(parent, getName(), msg, JOptionPane.ERROR_MESSAGE);
  }

  /**
   * Mtodo para exibio de uma mensagem informativa.
   * 
   * @param parent elemento (janela) pai.
   * @param msg o texto da mensagem.
   */
  final public void showInformation(final Component parent, final Object msg) {
    showMessage(parent, getName(), msg, JOptionPane.INFORMATION_MESSAGE);
  }

  /**
   * Mtodo para exibio de uma mensagem informativa.
   * 
   * @param msg o texto da mensagem.
   */
  final public void showInformation(final Object msg) {
    showInformation(null, msg);
  }

  /**
   * Mtodo para exibio de uma mensagem de aviso.
   * 
   * @param parent elemento (janela) pai.
   * @param msg o texto da mensagem.
   */
  final public void showWarning(final Component parent, final Object msg) {
    showMessage(parent, getName(), msg, JOptionPane.WARNING_MESSAGE);
  }

  /**
   * Mtodo para exibio de uma mensagem de aviso.
   * 
   * @param msg o texto da mensagem.
   */
  final public void showWarning(final Object msg) {
    showWarning(null, msg);
  }

  /**
   * Registra uma janela cliente na janela principal de modo que, ao fechar a
   * janela pricipal, todas as janelas registradas tambm so fechadas.
   * 
   * @param win uma nova janela registrada
   */
  public void addWindow(final Window win) {
    this.dependentWindows.add(win);
  }

  /**
   * Descadastra uma janela da cliente janela principal.
   * 
   * @param win a janela que deve ser removida da janela principal.
   */
  public void removeWindow(final Window win) {
    this.dependentWindows.remove(win);
  }

  /**
   * Fecha todas as janelas dependentes.
   */
  public void shutdownDependents() {
    final int base = 0;
    final int numDependents = this.dependentWindows.size();
    for (int i = 0; i < numDependents; i++) {
      final Window window = this.dependentWindows.get(base);

      // O dispose  redefinido na classe ApplicationComponentFrame,
      // retirando a janela da lista dependentWindows
      // ** dispose no parece estar removendo a janela !!! **
      removeWindow(window);
      window.dispose();
    }
  }

  /**
   * Fecha uma janela dependente especfica.
   * 
   * @param win a janela.
   */
  public void shutdownDependent(final Window win) {
    for (int i = 0; i < this.dependentWindows.size(); i++) {
      final Window window = this.dependentWindows.get(i);
      if (window == win) {
        window.dispose();
        break;
      }
    }
  }

  /**
   * Constri uma aplicao desktop
   * 
   * @param id identificador da aplicao.
   */
  protected Application(final String id) {
    super(id);
    setInitialFrameState(Frame.NORMAL);
    this.mainFrame = new ApplicationFrame(this, id);
    this.mainFrame.addApplicationManagerListener();
    this.mainFrame.addWindowListener(new WindowAdapter() {
      @Override
      public void windowClosing(final WindowEvent e) {
        closeApplication();
      }
    });

    // Criando a barra de status
    buildStatusBar();

    // Criando Vetor para armazenar janelas abertas
    this.dependentWindows = new Vector<Window>();
  }

  /**
   * Busca de propriedades especficas.
   * 
   * @return o objeto <code>Properties</code>
   * @deprecated usar mtodos para os tipos especficos 
   * {@link #getIntSpecificProperty(String, int)}, {@link #getBooleanSpecificProperty(String, boolean)}
   * {@link #getStringSpecificProperty(String)} etc.
   */
  @Deprecated
  public Properties getSpecificProperties() {
    return this.getApplicationRegistry().getSpecificProperties();
  }

  /**
   * Mtodo padro de tratamento de excees da aplicao.
   * 
   * @param ex a exceo.
   * @return <code>false</code> por default.
   */
  protected boolean handleError(final Exception ex) {
    return false;
  }

  /**
   * Indica que a janela da aplicao deve ser iniciada no estado indicado, que
   * pode ser {@link Frame#NORMAL}, {@link Frame#MAXIMIZED_BOTH}, etc..
   * 
   * @param initState - o estado inicial.
   */
  protected final void setInitialFrameState(final int initState) {
    this.initialFrameState = initState;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public abstract void killApplication();

  /**
   * Mtodo de kill pelo usurio  especfico do desktop applications
   * 
   * @return um flag indicativo de possibilidade de fechamento da aplicao.
   */
  protected abstract boolean userCanKillApplication();
}
