package csbase.client.applications.algorithmsmanager;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.SortedSet;
import java.util.TreeSet;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;

import csbase.client.applications.ApplicationFrame;
import csbase.client.applications.ApplicationImages;
import csbase.client.applications.ApplicationProject;
import csbase.client.applications.algorithmsmanager.actions.AlgorithmsManagerAction;
import csbase.client.applications.algorithmsmanager.actions.CategoriesManageAction;
import csbase.client.applications.algorithmsmanager.actions.ReloadAlgorithmsAction;
import csbase.client.applications.algorithmsmanager.models.AlgorithmData;
import csbase.client.applications.algorithmsmanager.models.CategoryData;
import csbase.client.remote.srvproxies.AlgorithmManagementProxy;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.Category;

/**
 * Classe que representa a aplicao para Gerncia de Algoritmos. Essa aplicao
 * visa inicialmente prover funcionalidades administrativas relacionadas a
 * instalao de algoritmos no sistema. Por exemplo, essa aplicao deve prover
 * uma funcionalidade que permite associar categorias a algoritmos e vice-versa.
 * 
 */
public class AlgorithmsManager extends ApplicationProject {
  /** Valor default para o tamanho da janela principal da aplicao */
  private static final Dimension DEFAULT_MAIN_FRAME_SIZE = new Dimension(1000,
    700);

  /** Categorias de algoritmos carregadas do servidor */
  public static SortedSet<CategoryData> ALL_CATEGORIES;

  /** Algoritmos carregados do servidor */
  public static SortedSet<AlgorithmData> ALL_ALGORITHMS;

  /** Mapeamento dos algoritmos e as categorias associadas */
  private static HashMap<AlgorithmData, SortedSet<CategoryData>> ALGORITHM_CATEGORIES_MAP;

  /** Painel principal */
  private JPanel mainPanel;

  /** Ao de gerncia de algoritmos */
  private CategoriesManageAction categoriesManageAction;

  /** Ao de recarregar os algoritmos a partir do servidor */
  private ReloadAlgorithmsAction reloadAlgorithmsAction;

  /**
   * Constri a aplicao para gerncia de algoritmos.
   * 
   * @param id Identificador da aplicao
   */
  public AlgorithmsManager(String id) {
    super(id);
    loadCategoriesAndAlgorithmsData();
    buildFrame();
  }

  /**
   * Constri a janela principal da aplicao.
   * 
   */
  private void buildFrame() {
    ApplicationFrame mainFrame = getApplicationFrame();
    mainFrame.setJMenuBar(buildMenuBar());
    mainFrame.getContentPane().add(buildToolBar(), BorderLayout.NORTH);
    mainFrame.getContentPane().add(getMainPanel());
    mainFrame.getContentPane().add(getStatusBar(), BorderLayout.SOUTH);
    mainFrame.setSize(DEFAULT_MAIN_FRAME_SIZE);
    mainFrame.setLocationRelativeTo(null);
  }

  /**
   * Obtm o painel principal da aplicao. Se ainda no existir, cria pela
   * primeira vez.
   * 
   * @return o painel principal
   */
  private JPanel getMainPanel() {
    if (mainPanel == null) {
      mainPanel = new JPanel();
      mainPanel.setLayout(new BorderLayout());
    }
    return mainPanel;
  }

  /**
   * Constri a barra de menu.
   * 
   * @return a barra de menu
   */
  private JMenuBar buildMenuBar() {
    JMenuBar menuBar = new JMenuBar();
    menuBar.add(buildAdminMenu());
    return menuBar;
  }

  /**
   * Constri o menu de administrao.
   * 
   * @return o menu criado
   */
  private JMenu buildAdminMenu() {
    JMenu adminMenu = new JMenu(getString("AlgorithmsManager.menu.admin"));
    adminMenu.add(new JMenuItem(getCategoriesManageAction()));
    // adminMenu.add(new JMenuItem("Gerenciar Algoritmos..."));
    // adminMenu.add(new JMenuItem("Gerenciar Propriedades de Algoritmos..."));
    adminMenu.addSeparator();
    adminMenu.add(new JMenuItem(getReloadAlgorithmsAction()));

    return adminMenu;
  }

  /**
   * Constri a barra de ferramentas.
   * 
   * @return o componente criado
   */
  private JComponent buildToolBar() {
    JToolBar toolBar = new JToolBar();
    toolBar.add(getToggleButton(getCategoriesManageAction()));
    return toolBar;
  }

  /**
   * Obtm a ao de fechar um arquivo. Se ela ainda no existir, ento  criada
   * nesse momento.
   * 
   * @return a ao
   */
  public CategoriesManageAction getCategoriesManageAction() {
    if (categoriesManageAction == null) {
      categoriesManageAction = new CategoriesManageAction(this,
        ApplicationImages.ICON_FOLDER_16);
      categoriesManageAction.setEnabled(true);
    }
    return categoriesManageAction;
  }

  /**
   * Obtm a ao de recarregar os algoritmos instalados no servidor.
   * 
   * @return a ao
   */
  public ReloadAlgorithmsAction getReloadAlgorithmsAction() {
    if (reloadAlgorithmsAction == null) {
      reloadAlgorithmsAction = new ReloadAlgorithmsAction(this);
      reloadAlgorithmsAction.setEnabled(true);
    }
    return reloadAlgorithmsAction;
  }

  /**
   * Obtm o boto associado a uma determinada ao.
   * 
   * @param action Ao associada ao boto.
   * @return o boto
   */
  private JToggleButton getToggleButton(AlgorithmsManagerAction action) {
    JToggleButton button = new JToggleButton(action);
    button.setText(null);
    button.setSelected(false);
    return button;
  }

  /**
   * Modifica o painel principal da janela.
   * 
   * @param panel painel a ser exibido na janela principal da aplicao
   */
  public void changeMainPanel(JPanel panel) {
    mainPanel.removeAll();
    if (panel != null) {
      mainPanel.add(panel);
    }
    mainPanel.revalidate();
    getApplicationFrame().repaint();
  }

  /**
   * (non-Javadoc)
   * 
   * @see csbase.client.applications.Application#killApplication()
   */
  @Override
  public void killApplication() {
    final JFrame mainFrame = getApplicationFrame();
    mainFrame.dispose();
  }

  /**
   * (non-Javadoc)
   * 
   * @see csbase.client.applications.Application#userCanKillApplication()
   */
  @Override
  protected boolean userCanKillApplication() {
    return true;
  }

  /**
   * Recarrega os algoritmos instalados no servidor.
   */
  public void reloadAlgorithms() {
    AlgorithmManagementProxy.reloadAlgorithms(null);
    loadCategoriesAndAlgorithmsData();
    getCategoriesManageAction().updatePanel();
  }

  /**
   * Carrega o conjunto de dados de categorias da rvore, a partir de um
   * conjunto de categorias.
   * 
   */
  private void reloadCategories() {
    SortedSet<Category> allCategories = AlgorithmManagementProxy
      .retrieveAllCategories(getApplicationFrame());
    ALL_CATEGORIES = new TreeSet<CategoryData>();
    for (Category category : allCategories) {
      addCategoryData(category);
    }
  }

  /**
   * Carrega o conjunto de dados de algoritmos da rvore, a partir de uma lista
   * de algoritmos.
   * 
   */
  private void loadAlgorithmsSet() {
    List<AlgorithmInfo> allAlgorithms = Arrays.asList(AlgorithmManagementProxy
      .retrieveAllAlgorithmInfos(getApplicationFrame()));

    ALL_ALGORITHMS = new TreeSet<AlgorithmData>();
    for (AlgorithmInfo algorithmInfo : allAlgorithms) {
      addAlgorithmData(algorithmInfo);
    }
  }

  /**
   * Adiciona os dados de um algoritmo na lista de algoritmos instalados no
   * sistema.
   * 
   * @param algorithmInfo algoritmo a ser adicionado
   */
  private void addAlgorithmData(AlgorithmInfo algorithmInfo) {
    ALL_ALGORITHMS.add(new AlgorithmData(algorithmInfo));
  }

  /**
   * Adiciona os dados de uma categoria na lista de categorias instaladas no
   * sistema.
   * 
   * @param category categoria a ser adicionada
   */
  private void addCategoryData(Category category) {
    ALL_CATEGORIES.add(new CategoryData(category, ""));
  }

  /**
   * Carrega os dados de categorias a partir dos dados disponveis no servio de
   * algoritmos, e refaz o mapeamento de algoritmos e categorias.
   */
  public void loadCategoryData() {
    reloadCategories();
    mapAlgorithmsToCategories();
  }

  /**
   * Mapeia os algoritmos s categorias associadas.
   */
  private void mapAlgorithmsToCategories() {
    ALGORITHM_CATEGORIES_MAP = new HashMap<AlgorithmData, SortedSet<CategoryData>>();

    for (CategoryData category : ALL_CATEGORIES) {
      Iterator<AlgorithmData> algorithmIterator = ALL_ALGORITHMS.iterator();
      while (algorithmIterator.hasNext()) {
        AlgorithmData algo = algorithmIterator.next();
        if (category.contains(algo)) {
          if (!ALGORITHM_CATEGORIES_MAP.containsKey(algo)) {
            ALGORITHM_CATEGORIES_MAP.put(algo, new TreeSet<CategoryData>());
          }
          ALGORITHM_CATEGORIES_MAP.get(algo).add(category);
        }
      }
    }
  }

  /**
   * Obtm o conjunto de algoritmos carregados do servidor.
   * 
   * @return o conjunto de algoritmos
   */
  public SortedSet<AlgorithmData> getAlgorithms() {
    return ALL_ALGORITHMS;
  }

  /**
   * Obtm o conjunto de categorias carregadas do servidor.
   * 
   * @return o conjunto de categorias de algoritmos
   */
  public SortedSet<CategoryData> getCategories() {
    return ALL_CATEGORIES;
  }

  /**
   * Obtm as categorias associadas a um determinado algoritmo.
   * 
   * @param algorithmData dados do algoritmo procurado
   * 
   * @return as categorias associadas ao algoritmo
   */
  public static SortedSet<CategoryData> getAlgorithmCategories(
    AlgorithmData algorithmData) {
    return ALGORITHM_CATEGORIES_MAP.get(algorithmData);
  }

  /**
   * Carrega os dados de categorias e de algoritmos.
   * 
   */
  public void loadCategoriesAndAlgorithmsData() {
    loadAlgorithmsSet();
    loadCategoryData();
  }

}
