package csbase.client.applications.algorithmsmanager.actions;

import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.Vector;

import javax.swing.ImageIcon;

import csbase.client.applications.algorithmsmanager.AlgorithmsManager;
import csbase.client.applications.algorithmsmanager.dialogs.AlgorithmsAndCategoriesBindDialog;
import csbase.client.applications.algorithmsmanager.dialogs.CategoriesManagePanel;
import csbase.client.applications.algorithmsmanager.dialogs.CategoriesManagePanel.TreeMode;
import csbase.client.applications.algorithmsmanager.models.AlgorithmData;
import csbase.client.applications.algorithmsmanager.models.CategoryData;
import csbase.client.applications.algorithmsmanager.models.DataNodeInterface;
import csbase.client.remote.srvproxies.AlgorithmManagementProxy;

/**
 * Representa a ao de associar algoritmos  categorias e vice-versa.
 * 
 */
public class AlgorithmsAndCategoriesBindAction extends AlgorithmsManagerAction {
  /** Painel de gerncia de categorias que criou essa ao */
  private CategoriesManagePanel managePanel;

  /**
   * Dilogo de associao de algoritmos a categorias
   */
  private AlgorithmsAndCategoriesBindDialog bindDialog;

  /**
   * Dados selecionados na rvore e que sero associados a outros dados
   */
  private List<DataNodeInterface<?, ?>> parentDataList;

  /** Modo corrente de visualizao da rvore de dados */
  private TreeMode currentMode;

  /**
   * Constri a ao de associao de algoritmos e categorias.
   * 
   * @param parentPanel painel pai que criou a ao
   * @param icon imagem da ao
   */
  public AlgorithmsAndCategoriesBindAction(CategoriesManagePanel parentPanel,
    ImageIcon icon) {
    super(parentPanel.getApplication(), icon);
    this.managePanel = parentPanel;
  }

  /**
   * Associa efetivamente os algoritmos s categorias.
   * 
   * @param algorithmIds identificadores dos algoritmos
   * @param categoryIds identificadores das categorias
   * 
   * @return retorna true se os algoritmos foram associados s categorias com
   *         sucesso, caso contrrio retorna false
   */
  public boolean bindAlgorithmsAndCategories(final List<Object> algorithmIds,
    final List<String> categoryIds) {
    return AlgorithmManagementProxy.bindAlgorithmsAndCategories(algorithmIds,
      categoryIds, getApplication().getApplicationFrame());
  }

  /**
   * Obtm os identificadores dos dados selecionados na rvore de dados do
   * painel pai dessa ao.
   * 
   * @return retorna o identificador dos dados selecionados no painel pai
   */
  protected List<Object> getParentDataIds() {
    List<Object> dataIdList = new ArrayList<Object>();
    for (DataNodeInterface<?, ?> parentData : parentDataList) {
      dataIdList.add(parentData.getId());
    }
    return dataIdList;
  }

  /**
   * Obtm o nome da categoria pai.
   * 
   * @return retorna o nome da categoria pai, ou null, caso as novas categorias
   *         devam ser criadas na raiz (primeiro nvel)
   */
  public Vector<String> getParentDataFullNames() {
    Vector<String> dataNameList = new Vector<String>();
    for (DataNodeInterface<?, ?> parentData : parentDataList) {
      dataNameList.add(parentData.getFullName());
    }
    return dataNameList;
  }

  /**
   * Estabelece o modo corrente de visualizao da rvore de dados.
   * 
   * @param mode modo corrente de visualizao da rvore de dados
   */
  public void setCurrentMode(TreeMode mode) {
    this.currentMode = mode;
  }

  /**
   * Obtm o modo corrente de visualizao da rvore de dados.
   * 
   * @return o modo corrente de visualizao da rvore de dados
   */
  private TreeMode getCurrentMode() {
    return currentMode;
  }

  /**
   * (non-Javadoc)
   * 
   * @see csbase.client.applications.algorithmsmanager.actions.AlgorithmsManagerAction#handleActionPerformed(java.awt.event.ActionEvent)
   */
  @Override
  protected void handleActionPerformed(ActionEvent ae) throws Exception {
    parentDataList = managePanel.getSelectedDataList();
    bindDialog = new AlgorithmsAndCategoriesBindDialog(this,
      getParentSelectedNodes(), getAvailableChildren(), getSelectedChildren(),
      getParentDataLabel(), getDataBindingDescription());
    bindDialog.initDialog();
  }

  /**
   * Obtm o texto para identificar o tipo do dado selecionado para associao
   * no painel pai.
   * 
   * @return o texto para identificar o tipo do dado selecionado
   */
  private String getParentDataLabel() {
    String label;
    if (getCurrentMode().equals(TreeMode.BY_CATEGORY)) {
      label = getString("label.parent.category");
    }
    else {
      label = getString("label.parent.algorithm");
    }
    return label;
  }

  /**
   * Obtm o texto para descrever o tipo de associo a ser feita com os dados.
   * 
   * @return o texto que descreve o tipo de associao dos dados
   */
  private String getDataBindingDescription() {
    String label;
    if (getCurrentMode().equals(TreeMode.BY_CATEGORY)) {
      label = getString("label.binding.algorithm");
    }
    else {
      label = getString("label.binding.category");
    }
    return label;
  }

  /**
   * Obtm os ns (pai na rvore) selecionados para realizar a associao com os
   * filhos.
   * 
   * @return os ns selecionados na rvore de dados
   */
  private SortedSet<DataNodeInterface<?, ?>> getParentSelectedNodes() {
    SortedSet<DataNodeInterface<?, ?>> data = new TreeSet<DataNodeInterface<?, ?>>();
    for (DataNodeInterface<?, ?> parentData : parentDataList) {
      data.add(parentData);
    }
    return data;
  }

  /**
   * Obtm os filhos disponveis para associao com os ns selecionados. Deve
   * incluir todos os filhos existentes que ainda no estejam associados.
   * 
   * @return os filhos disponveis para a associao
   */
  private SortedSet<DataNodeInterface<?, ?>> getAvailableChildren() {
    SortedSet<DataNodeInterface<?, ?>> data = new TreeSet<DataNodeInterface<?, ?>>();
    if (getCurrentMode().equals(TreeMode.BY_CATEGORY)) {
      data = getAllAgorithms();
    }
    else {
      data = getAllCategories();
    }
    return data;
  }

  /**
   * Obter todas as categorias disponveis no servidor, inclusive as
   * subcategorias.
   * 
   * @return as categorias disponveis no servidor, inclusive as subcategorias
   */
  private SortedSet<DataNodeInterface<?, ?>> getAllCategories() {
    SortedSet<CategoryData> mainCategories = AlgorithmsManager.ALL_CATEGORIES;
    SortedSet<DataNodeInterface<?, ?>> allCategories = new TreeSet<DataNodeInterface<?, ?>>();
    for (CategoryData categoryData : mainCategories) {
      allCategories.add(categoryData);
      SortedSet<? extends CategoryData> subNodes = categoryData.getSubNodes();
      if (!subNodes.isEmpty()) {
        allCategories.addAll(subNodes);
      }
    }
    return allCategories;
  }

  /**
   * Obter todas os algoritmos disponveis no servidor.
   * 
   * @return as algoritmos disponveis no servidor
   */
  private SortedSet<DataNodeInterface<?, ?>> getAllAgorithms() {
    SortedSet<AlgorithmData> allAlgorithms = AlgorithmsManager.ALL_ALGORITHMS;
    SortedSet<DataNodeInterface<?, ?>> algorithmsSet = new TreeSet<DataNodeInterface<?, ?>>();
    Collections.addAll(algorithmsSet, allAlgorithms
      .toArray(new DataNodeInterface<?, ?>[0]));
    return algorithmsSet;
  }

  /**
   * Obtm os filhos previamente associados com os ns selecionados. Deve
   * incluir todos os filhos que j estejam associados.
   * 
   * @return os filhos previamente associados com os ns
   */
  private SortedSet<DataNodeInterface<?, ?>> getSelectedChildren() {
    SortedSet<DataNodeInterface<?, ?>> dataSet = new TreeSet<DataNodeInterface<?, ?>>();

    // Por enquanto, tratar o caso de um nico n pai selecionado
    DataNodeInterface<?, ?> selectedNode = getParentSelectedNodes().first();
    SortedSet<?> dataChildren = selectedNode.getDataChildren();
    for (Object child : dataChildren) {
      dataSet.add((DataNodeInterface<?, ?>) child);
    }
    return dataSet;
  }
}
