package csbase.client.applications.algorithmsmanager.dialogs;

import java.awt.GridBagLayout;
import java.util.List;
import java.util.SortedSet;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;

import tecgraf.javautils.gui.GBC;
import tecgraf.javautils.gui.selector.ContainerSelection;
import tecgraf.javautils.gui.table.ObjectTableProvider;
import csbase.client.applications.AbstractSimpleApplicationPanel;
import csbase.client.applications.algorithmsmanager.AlgorithmsManager;
import csbase.client.applications.algorithmsmanager.models.DataNodeInterface;

/**
 * Painel para associar os dados de acordo com o modo de seleo da rvore de
 * dados (categorias ou algoritmos).
 * 
 * Este componente vai permitir associar ou desassociar algoritmos de categorias
 * e vice-versa.
 * 
 */
public class DataBindingPanel extends
  AbstractSimpleApplicationPanel<AlgorithmsManager> {

  /**
   * O componente que contm os elementos a serem selecionados.
   */
  private ContainerSelection<DataNodeInterface<?, ?>> containerSelection;

  /** Dados disponveis para serem associados */
  private SortedSet<DataNodeInterface<?, ?>> availableData;

  /** Dados previamente selecionados como associados */
  private SortedSet<DataNodeInterface<?, ?>> previousSelectedData;

  /** Texto que descreve o tipo de dados a ser associado */
  private String description;

  /**
   * Constri o painel.
   * 
   * @param application aplicao de gerncia de algoritmos
   * @param availableData dados disponveis para serem associados
   * @param selectedData dados selecionados para serem associados
   * @param description Texto que descreve o tipo de dados a ser associado
   */
  DataBindingPanel(AlgorithmsManager application,
    SortedSet<DataNodeInterface<?, ?>> availableData,
    SortedSet<DataNodeInterface<?, ?>> selectedData, String description) {
    super(application);
    this.availableData = availableData;
    this.previousSelectedData = selectedData;
    this.description = description;
    buildPanel();
  }

  @Override
  protected void buildPanel() {
    setLayout(new GridBagLayout());
    setBorder(BorderFactory.createEtchedBorder());
    JPanel mainPanel = getMainPanel();
    add(mainPanel, new GBC(0, 0).both().west().insets(5, 5, 5, 5));
  }

  /**
   * Obtm o painel para seleo dos dados a serem associados.
   * 
   * @return o painel criado
   */
  private JPanel getMainPanel() {
    JPanel panel = new JPanel(new GridBagLayout());
    JLabel descriptionLabel = new JLabel(description);

    panel.add(descriptionLabel, new GBC(0, 0).none().west().insets(5, 5, 5, 5));
    panel.add(getDataSelectionPanel(getAvailableItems(), previousSelectedData),
      new GBC(0, 1).none().west().insets(5, 5, 5, 5));
    return panel;
  }

  /**
   * Cria o painel para seleo dos dados.
   * 
   * @param availableItems dados disponveis para adio
   * @param selectedItems dados selecionados no grupo
   * 
   * @return o painel criado
   */
  private JPanel getDataSelectionPanel(
    SortedSet<DataNodeInterface<?, ?>> availableItems,
    SortedSet<DataNodeInterface<?, ?>> selectedItems) {
    ObjectTableProvider selectableProvider = new MyObjectTableProvider(
      getString("DataBindingPanel.available_items"));
    ObjectTableProvider selectedProvider = new MyObjectTableProvider(
      getString("DataBindingPanel.selected_items"));

    containerSelection = new ContainerSelection<DataNodeInterface<?, ?>>(
      selectableProvider, selectedProvider, true, false);
    containerSelection.loadItems(availableItems, selectedItems);
    containerSelection.adjustTableColumns();
    return containerSelection.getPanel();
  }

  /**
   * Obtm os itens de dados disponveis para seleo.
   * 
   * @return a lista contendo os itens disponveis para seleo
   */
  private SortedSet<DataNodeInterface<?, ?>> getAvailableItems() {
    return availableData;
  }

  /**
   * Obtm os itens de dados selecionados para seleo.
   * 
   * @return a lista contendo os itens selecionados para seleo
   */
  public List<DataNodeInterface<?, ?>> getSelectedItems() {
    return containerSelection.getSelectedItems();
  }

  /**
   * Utilizada para fornecer dados sobre o tem de enumerao para as tabelas do
   * container de seleo.
   */
  private final class MyObjectTableProvider implements ObjectTableProvider {
    /**
     * O nome da coluna.
     */
    private String columnName;

    /**
     * @param columnName O nome da coluna (No aceita {@code null}).
     */
    MyObjectTableProvider(String columnName) {
      this.columnName = columnName;
    }

    /**
     * @see tecgraf.javautils.gui.table.ObjectTableProvider#getColumnNames()
     */
    public String[] getColumnNames() {
      return new String[] { this.columnName };
    }

    /**
     * @see tecgraf.javautils.gui.table.ObjectTableProvider#getColumnClasses()
     */
    public Class<?>[] getColumnClasses() {
      return new Class[] { String.class };
    }

    /**
     * @see tecgraf.javautils.gui.table.ObjectTableProvider#getCellValues(java.lang.Object)
     */
    public Object[] getCellValues(Object row) {
      DataNodeInterface<?, ?> item = (DataNodeInterface<?, ?>) row;
      return new Object[] { item.getFullName() };
    }
  }
}
