package csbase.client.applications.algorithmsmanager.dialogs;

import java.util.ArrayList;
import java.util.List;
import java.util.SortedSet;

import javax.swing.JTree;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import tecgraf.javautils.gui.tree.FilterableTreePanel;
import csbase.client.applications.algorithmsmanager.AlgorithmsManager;
import csbase.client.applications.algorithmsmanager.models.DataNodeInterface;
import csbase.client.applications.algorithmsmanager.models.DataTreeNode;
import csbase.client.applications.algorithmsmanager.models.DataTreeRootNode;

/**
 * Representa uma viso da rvore de dados e seus respectivos filhos. Para
 * inserir essa viso grfica voc deve obter o painel correspondente.
 * 
 * @param <N> classe dos ns da rvore de dados
 * @param <C> classe dos filhos dos ns da rvore de dados
 * 
 */
public class DataTreeView<N extends DataNodeInterface<N, C>, C extends DataNodeInterface<C, N>> {

  /** rvore de dados, que pode exibir ou no os filhos dos ns principais */
  private FilterableTreePanel dataTreePanel;

  /** Modo selecionado para visualizao da rvore de dados */
  private ViewChildrenMode viewMode;

  /** Ns da rvore de dados */
  private SortedSet<N> nodes;

  /** Texto a ser exibido no n raiz da rvore de dados */
  private String rootLabel;

  /**
   * Modo de visualizao dos filhos dos ns da rvore de dados
   */
  public enum ViewChildrenMode {
    /** Exibe os filhos na rvore */
    SHOW_CHILDREN,
    /** Oculta os filhos na rvore */
    HIDE_CHILDREN;
  }

  /**
   * Constri o painel da rvore de dados.
   * 
   * @param application aplicao de gerncia de algoritmos
   * @param nodes ns da rvore de dados
   * @param rootLabel texto a ser exibido no n raiz da rvore
   * @param viewMode modo de visualizao da rvore de dados
   */
  public DataTreeView(AlgorithmsManager application, SortedSet<N> nodes,
    String rootLabel, ViewChildrenMode viewMode) {
    setNodes(nodes);
    setViewMode(viewMode);
    this.rootLabel = rootLabel;
  }

  /**
   * Estabelece o conjunto de ns da rvore de dados.
   * 
   * @param nodes ns da rvore de dados
   */
  private void setNodes(SortedSet<N> nodes) {
    this.nodes = nodes;
  }

  /**
   * Obtm o painel correspondente a viso da rvore de dados
   * 
   * @return o painel criado
   */
  public FilterableTreePanel getPanel() {
    if (dataTreePanel == null) {
      dataTreePanel = createDataTreePanel();
    }
    return dataTreePanel;
  }

  /**
   * Constri o painel com a rvore de dados.
   * 
   * @return o painel com a rvore de dados
   */
  private FilterableTreePanel createDataTreePanel() {
    FilterableTreePanel algorithmVersionTreePanel = new FilterableTreePanel(
      new DataTreeRootNode<N, C>(rootLabel, nodes, getViewMode()));
    JTree algorithmVersionTree = algorithmVersionTreePanel.getTree();
    algorithmVersionTree.getSelectionModel().setSelectionMode(
      TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);

    return algorithmVersionTreePanel;
  }

  /**
   * Obtm o painel com a rvore de categorias de algoritmos.
   * 
   * @return o painel com a rvore de categorias de algoritmos
   */
  public FilterableTreePanel getAlgorithmVersionTreePanel() {
    return dataTreePanel;
  }

  /**
   * Estabelece o modo de visualizao da rvore de categorias de algoritmos.
   * 
   * @param viewMode modo de visualizao que exibe ou oculta os algoritmos da
   *        rvore de categorias.
   */
  public void setViewMode(ViewChildrenMode viewMode) {
    this.viewMode = viewMode;
  }

  /**
   * Obtm o modo de visualizao da rvore de categorias de algoritmos.
   * 
   * @return o modo de visualizao da rvore, que exibe ou oculta os algoritmos
   *         da rvore de categorias
   */
  public ViewChildrenMode getViewMode() {
    return viewMode;
  }

  /**
   * Obtm o primeiro n selecionado na rvore de dados.
   * 
   * @return o primeiro n selecionado na rvore de dados
   */
  public DataNodeInterface<?, ?> getFirstSelectedNode() {
    DataNodeInterface<?, ?> node = null;
    JTree dataTree = dataTreePanel.getTree();
    int selectedCount = dataTree.getSelectionCount();
    if (selectedCount == 1) {
      Object selectionPath = dataTree.getSelectionPath().getLastPathComponent();
      if (selectionPath instanceof DataTreeNode<?, ?>) {
        DataTreeNode<?, ?> dataTreeNode = (DataTreeNode<?, ?>) selectionPath;
        node = dataTreeNode.getNode();
      }
      if (selectionPath instanceof DataTreeRootNode<?, ?>) {
        DataTreeRootNode<?, ?> dataTreeRootNode = (DataTreeRootNode<?, ?>) selectionPath;
        node = dataTreeRootNode;
      }
    }
    return node;
  }

  /**
   * Obtm a lista dos dados selecionados na rvore de dados.
   * 
   * @return retorna a lista de dados selecionados na rvore de dados. Se a
   *         seleo
   */
  public List<DataNodeInterface<?, ?>> getSelectedDataList() {
    List<DataNodeInterface<?, ?>> selectedDataList = new ArrayList<DataNodeInterface<?, ?>>();
    JTree dataTree = dataTreePanel.getTree();
    TreePath[] selectionPaths = dataTree.getSelectionPaths();
    for (TreePath treePath : selectionPaths) {
      Object treeNode = treePath.getLastPathComponent();
      if (treeNode instanceof DataTreeRootNode<?, ?>) {
        continue;
      }
      DataTreeNode<?, ?> dataTreeNode = (DataTreeNode<?, ?>) treeNode;
      selectedDataList.add(dataTreeNode.getNode());
    }
    return selectedDataList;
  }

  /**
   * Obtm os ns referentes ao caminho completo do n selecionado.
   * 
   * @return uma lista com os ns referentes ao caminho completo do n
   *         selecionado
   */
  public List<DataNodeInterface<?, ?>> getFullPathSelectedNode() {
    List<DataNodeInterface<?, ?>> fullPath = new ArrayList<DataNodeInterface<?, ?>>();
    JTree dataTree = dataTreePanel.getTree();
    int selectedCount = dataTree.getSelectionCount();
    if (selectedCount == 1) {
      Object[] paths = dataTree.getSelectionPath().getPath();
      DataTreeRootNode<?, ?> rootNode = (DataTreeRootNode<?, ?>) paths[0];
      fullPath.add(rootNode);
      for (Object path : paths) {
        if (path instanceof DataTreeNode<?, ?>) {
          DataTreeNode<?, ?> node = (DataTreeNode<?, ?>) path;
          fullPath.add(node.getNode());
        }
      }
    }
    return fullPath;
  }

  /**
   * Obtm o n raiz da rvore de dados.
   * 
   * @return retorna o n raiz da rvore de dados, caso contrrio, retorna null
   * 
   */
  public DataTreeRootNode<?, ?> getRootNode() {
    JTree dataTree = dataTreePanel.getTree();
    TreePath rootPath = dataTree.getPathForRow(0);
    Object treeNode = rootPath.getLastPathComponent();
    if (treeNode instanceof DataTreeRootNode<?, ?>) {
      return (DataTreeRootNode<?, ?>) treeNode;
    }
    return null;
  }

  /**
   * Seleciona o n raiz da rvore de dados.
   * 
   */
  public void selectRootNode() {
    JTree dataTree = dataTreePanel.getTree();
    dataTree.setSelectionRow(0);
  }

  /**
   * Seleciona o n especificado na rvore de dados.
   * 
   * @param nodePath n a ser selecionado
   * 
   */
  public void selectNode(TreePath nodePath) {
    JTree dataTree = dataTreePanel.getTree();
    dataTree.setSelectionPath(nodePath);
  }

  /**
   * Verifica se o n raiz da rvore de dados est selecionado.
   * 
   * @return retorna true, se o n raiz da rvore de dados estiver selecionado,
   *         caso contrrio, retorna false
   * 
   */
  public boolean isRootNodeSelected() {
    JTree dataTree = dataTreePanel.getTree();
    return dataTree.isRowSelected(0);
  }

  /**
   * Obtm o nmero de itens selecionados na rvore de dados.
   * 
   * @return o nmero de itens selecionados na rvore de dados
   */
  public int getSelectionCount() {
    return dataTreePanel.getTree().getSelectionCount();
  }

}
