/*
 * $Id$
 */

package csbase.client.applications.commandsmonitor;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.HashMap;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JToolBar;
import javax.swing.SwingConstants;

import csbase.client.applicationmanager.ApplicationException;
import csbase.client.applications.Application;
import csbase.client.applications.ApplicationExitAction;
import csbase.client.applications.commandsmonitor.actions.ExecuteCommandAction;
import csbase.client.applications.commandsmonitor.actions.KillCommandsAction;
import csbase.client.applications.commandsmonitor.actions.RemoveCommandsAction;
import csbase.client.applications.commandsmonitor.actions.SchedulerSwitchAction;
import csbase.client.applications.commandsmonitor.actions.ShowCommandDetailsAction;
import csbase.client.applications.commandsmonitor.actions.ShowCommandLogAction;
import csbase.client.applications.commandsmonitor.actions.ShowCommandParametersAction;
import csbase.client.applications.commandsmonitor.actions.SchedulerSwitchAction.StateChangedEvent;
import csbase.client.applications.commandsmonitor.events.SelectionChangedEvent;
import csbase.client.applications.commandsmonitor.table.CommandsTableFactory;
import csbase.client.applications.commandsmonitor.table.dal.ICommandsTableDAO;
import csbase.client.applications.commandsmonitor.table.dal.xml.XmlCommandsTableDAO;
import csbase.client.desktop.DesktopFrame;
import csbase.client.util.event.EventListener;
import csbase.client.util.event.EventManager;
import csbase.logic.CommonClientProject;

/**
 * Aplicao de monitorao de comandos.
 * 
 * @author Tecgraf / PUC-Rio
 */
public class CommandsMonitor extends Application {

  /** Gerencia o envio de eventos. */
  private EventManager eventManager;

  /** Fbrica de tabelas. */
  private CommandsTableFactory tablesFactory;

  /** Painel principal composto por abas. */
  private JTabbedPane tabbedPane;

  /** Aes de comando presentes na interface. */
  /** Mostra os parmetros do comando selecionado. */
  private ShowCommandParametersAction showParametersAction;
  /** Mostra o log do comando selecionado. */
  private ShowCommandLogAction showLogAction;
  /** Mostra os detalhes do comando selecionado. */
  private ShowCommandDetailsAction showDetailsAction;
  /** Cria uma cpia do comando selecionado, em memria, para execuo. */
  private ExecuteCommandAction executeCommandAction;
  /** Mata o comando selecionado em execuo. */
  private KillCommandsAction killCommandsAction;
  /** Destroi a persistncia do comando selecionado, terminado. */
  private RemoveCommandsAction removeCommandsAction;
  /** Hash que mapeia um titulo para uma tab */
  private HashMap<String, TablesTab> titleTabMap =
    new HashMap<String, TablesTab>();
  /** Indica se a tabela do escalonador deve ser exibida. */
  private boolean showSchedulerTable;

  /**
   * Construtor.
   * 
   * @param id identificador nico da aplicao.
   * 
   * @throws ApplicationException caso no seja possvel criar dinamicamente a
   *         interface.
   */
  public CommandsMonitor(String id) throws ApplicationException {
    super(id);

    this.eventManager = new EventManager();

    try {
      // Cria a fbrica de tabelas.
      String path = getStringSpecificProperty("tables.factory.xml.path");
      showSchedulerTable = getBooleanSpecificProperty("show.scheduler", true);
      Reader reader =
        new InputStreamReader(getClass().getResourceAsStream(path));
      ICommandsTableDAO tableDS = new XmlCommandsTableDAO(reader);
      this.tablesFactory =
        new CommandsTableFactory(getApplicationRegistry(), tableDS);

      /*
       * Cria as aes de comandos. Deve ser criado antes pois  utilizado pelo
       * painel principal e pela barra de menu.
       */
      createActions();
      // Cria o painel da aplicao.
      getApplicationFrame().setContentPane(createContentPane());
      // Cria a barra de menu da aplicao.
      getApplicationFrame().setJMenuBar(createMenuBar());
      getApplicationFrame().pack();
    }
    catch (ApplicationException e) {
      throw e;
    }
    catch (Exception e) {
      throw new ApplicationException(e);
    }
  }

  /**
   * Adiciona um ouvinte interessado nas alteraes na seleo dos comandos.
   * 
   * @param listener ouvinte interessado.
   */
  public void addSelectionChangedListener(
    EventListener<SelectionChangedEvent> listener) {
    eventManager.addEventListener(listener, SelectionChangedEvent.class);
  }

  /**
   * Inicializa o monitor de comando de cada tabela presente.
   * 
   * {@inheritDoc}
   */
  @Override
  public void startApplication() throws ApplicationException {
    Component tab = tabbedPane.getSelectedComponent();
    if (null != tab && TablesTab.class.isAssignableFrom(tab.getClass())) {
      TablesTab.class.cast(tab).start();
    }
    super.startApplication();
  }

  /**
   * Interrompe a monitorao de comandos.
   * 
   * {@inheritDoc}
   */
  @Override
  public void killApplication() {
    for (int inx = 0; inx < tabbedPane.getComponentCount(); inx++) {

      Component component = tabbedPane.getComponent(inx);

      if (null != component
        && TablesTab.class.isAssignableFrom(component.getClass())) {

        TablesTab.class.cast(component).stop();
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected boolean userCanKillApplication() {
    return true;
  }

  /**
   * Cria as aes de comandos.
   */
  private void createActions() {
    showParametersAction = new ShowCommandParametersAction(this);
    showLogAction = new ShowCommandLogAction(this);
    showDetailsAction = new ShowCommandDetailsAction(this);
    executeCommandAction = new ExecuteCommandAction(this);
    killCommandsAction = new KillCommandsAction(this);
    removeCommandsAction = new RemoveCommandsAction(this);
  }

  /**
   * Cria a barra de menu.
   * 
   * @return A barra de menu.
   */
  private JMenuBar createMenuBar() {
    JMenuBar menuBar = new JMenuBar();
    menuBar.add(createCommandsMenu());
    //menuBar.add(createConfigurationMenu()); //TODO: falta implementar o servio de configurao.  
    if (showSchedulerTable) {
      menuBar.add(createSchedulerMenu());
    }
    return menuBar;
  }

  /**
   * Cria o menu de configuraes da aplicao.
   * 
   * @return O menu de configuraes da aplicao.
   */
  private JMenu createConfigurationMenu() {
    JMenu configMenu =
      new JMenu(getString("CommandsMonitor.menu.configuration"));
    return configMenu;
  }

  /**
   * Cria o menu de administrao do agendador de comandos.
   * 
   * @return O menu de administrao do agendador de comandos.
   */
  private JMenu createSchedulerMenu() {
    JMenu schedulerMenu =
      new JMenu(getString("CommandsMonitor.menu.scheduler"));
    /** Item de menu de auto-reload. */
    SchedulerSwitchAction switchAction = new SchedulerSwitchAction(this);
    final JCheckBoxMenuItem switchItem = new JCheckBoxMenuItem(switchAction);
    switchItem.setSelected(switchAction.isSchedulerBlocked());
    switchAction
      .addStateChangedListener(new EventListener<StateChangedEvent>() {
        @Override
        public void eventFired(StateChangedEvent event) {
          switchItem.setSelected(event.isSchedulerBlocked());
        }
      });
    schedulerMenu.add(switchItem);
    return schedulerMenu;
  }

  /**
   * Cria o menu Comandos.
   * 
   * @return O menu.
   */
  private JMenu createCommandsMenu() {
    JMenu commandsMenu = new JMenu(getString("CommandsMonitor.menu.commands"));
    commandsMenu.add(new JMenuItem(showParametersAction));
    commandsMenu.add(new JMenuItem(showLogAction));
    commandsMenu.add(new JMenuItem(showDetailsAction));
    commandsMenu.add(new JMenuItem(executeCommandAction));
    commandsMenu.add(new JMenuItem(killCommandsAction));
    commandsMenu.add(new JMenuItem(removeCommandsAction));

    commandsMenu.addSeparator();
    commandsMenu.add(new ApplicationExitAction(this));

    return commandsMenu;
  }

  /**
   * Cria o painel da aplicao.
   * 
   * @return O painel da aplicao.
   * 
   * @throws ApplicationException
   */
  private JComponent createContentPane() throws ApplicationException {
    JPanel panel = new JPanel(new BorderLayout());
    panel.add(createToolBar(), BorderLayout.NORTH);
    panel.add(createTabbedPane(), BorderLayout.CENTER);

    return panel;
  }

  /**
   * Cria o painel que ir conter as abas que detm as tabelas.
   * 
   * @return O painel que ir conter as abas que detm as tabelas.
   * 
   * @throws ApplicationException
   */
  private JTabbedPane createTabbedPane() throws ApplicationException {
    //TODO Fazer com que as tabelas venham do servio de configurao 
    TablesTab tab_1 = new TablesTab(eventManager);
    tab_1.addTable(getString("table.executing.title"), tablesFactory
      .createTable("executing"));
    if (showSchedulerTable) {
      tab_1.addTable(getString("table.scheduled.title"), tablesFactory
        .createTable("scheduled"));
    }
    TablesTab tab_2 = new TablesTab(eventManager);
    CommonClientProject project = DesktopFrame.getInstance().getProject();
    String projectId = null;
    if (project != null) {
      projectId = project.getName();
    }
    tab_2.addTable(getString("table.view.title", new String[] { projectId }),
      tablesFactory.createTable("view"));

    this.tabbedPane = new JTabbedPane();
    tabbedPane.addChangeListener(new JTabbedPaneSelectionListener());
    addTab("monitor", tab_1);
    addTab("view", tab_2);
    TablesTab[] tabs = new TablesTab[] { tab_1, tab_2 };
    int w = 0;
    for (TablesTab aTab : tabs) {
      Dimension d = aTab.getPreferredSize();
      w = Math.max(w, aTab.getPreferredSize().width);
    }
    this.tabbedPane.setPreferredSize(new Dimension(w + 100, tabbedPane
      .getPreferredSize().height));

    // L a propriedade "tabDefault" para saber qual tab ser exibida.
    String tabName = this.getStringSpecificProperty("tabDefault");
    if (tabName != null) {
      TablesTab tab = this.titleTabMap.get(tabName);
      if (tab != null) {
        tabbedPane.setSelectedComponent(tab);
      }
    }

    return tabbedPane;
  }

  /**
   * Adiciona uma tab ao painel de abas.
   * 
   * @param tabId Ttulo da tab.
   * @param tab Tab a ser adicionada.
   */
  private void addTab(final String tabId, final TablesTab tab) {
    titleTabMap.put(tabId, tab);
    tabbedPane.addTab(getString("tab." + tabId + ".title"), tab);
  }

  /**
   * Cria a barra de ferramentas contendo algumas aes de comando.
   * 
   * @return A barra de ferramentas contendo algumas aes de comando.
   */
  private JToolBar createToolBar() {
    JToolBar toolBar = new JToolBar();
    toolBar.setOrientation(SwingConstants.HORIZONTAL);
    toolBar.add(showParametersAction);
    toolBar.add(showLogAction);
    toolBar.add(showDetailsAction);
    toolBar.add(executeCommandAction);
    toolBar.add(killCommandsAction);
    toolBar.add(removeCommandsAction);
    return toolBar;
  }
}
