/*
 * $Id$
 */

package csbase.client.applications.commandsmonitor;

import static java.awt.GridBagConstraints.BOTH;
import static java.awt.GridBagConstraints.RELATIVE;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingUtilities;

import csbase.client.facilities.commandtable.AbstractCommandMonitoringTable;
import csbase.client.util.event.EventManager;

/**
 * Aba de tabelas de monitorao de comandos.
 * 
 * @author Tecgraf / PUC-Rio
 */
public class TablesTab extends JScrollPane implements ISelectionListenerTab {

  /**
   * Indica se a monitorao dos comandos est ativa.
   */
  private boolean running = false;

  /**
   * Gerenciador de seleo das tabelas desta aba.
   */
  private UniqueTableSelectionManager selectionManager;
  /**
   * Painel que detm as tabelas.
   */
  private JPanel view;
  /**
   * Lista das tabelas contidas nesta aba.
   */
  private List<AbstractCommandMonitoringTable> tables;

  /**
   * Construtor.
   * 
   * @param eventManager intermedia os eventos lanados por esta.
   */
  public TablesTab(EventManager eventManager) {
    super();

    this.setPreferredSize(new Dimension(800, 500));

    this.selectionManager = new UniqueTableSelectionManager(eventManager);
    this.tables = new ArrayList<AbstractCommandMonitoringTable>();

    view = new JPanel(new GridBagLayout());
    this.setViewportView(view);
  }

  /**
   * {@inheritDoc}
   * 
   * @param selected Indica se essa aba foi selecionada - ganhou foco - ou no.
   *        Se ganhou foco, inicia a monitorao de comandos. Se perdeu o foco,
   *        limpa as clulas selecionadas.
   */
  @Override
  public void onSelectionChanged(boolean selected) {
    if (selected) {
      start();
    }
    else {
      selectionManager.clearSelections();
    }
  }

  /**
   * Inicia a monitorao das tabelas de comandos nesta aba.
   */
  public void start() {
    if (!running)
      for (AbstractCommandMonitoringTable aTable : tables) {
        aTable.start();
      }
    running = true;
  }

  /**
   * Para a monitorao das tabelas de comandos nesta aba.
   */
  public void stop() {
    for (AbstractCommandMonitoringTable aTable : tables) {
      aTable.stop();
    }
    running = false;
  }

  /**
   * Adiciona uma tabela abaixo da ltima que foi adicionada antes.<br>
   * Caso este mtodo no esteja rodando na EDT, ele entrar na fila da mesma.
   * 
   * @param title Ttulo da tabela.
   * @param table Tabela a ser adicionada.
   */
  public void addTable(final String title,
    final AbstractCommandMonitoringTable table) {
    selectionManager.startManaging(table);
    if (SwingUtilities.isEventDispatchThread()) {
      _addTable(title, table);
    }
    else {
      SwingUtilities.invokeLater(new Runnable() {
        @Override
        public void run() {
          _addTable(title, table);
        }
      });
    }
  }

  /**
   * Adiciona uma tabela abaixo da ltima que foi adicionada antes.
   * 
   * @param title Ttulo da tabela.
   * @param table Tabela a ser adicionada.
   */
  private void _addTable(String title, AbstractCommandMonitoringTable table) {
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.anchor = GridBagConstraints.NORTHWEST;
    constraints.insets = new Insets(2, 2, 0, 2);
    constraints.weightx = 1.0;
    constraints.weighty = 1.0;
    constraints.fill = BOTH;
    constraints.gridx = 0;
    constraints.gridy = RELATIVE;
    constraints.gridwidth = 1;
    constraints.gridheight = 1;

    view.add(createTablePanel(title, table), constraints);
    tables.add(table);
  }

  /**
   * Encapsula a tabela e seu ttulo em um componente.
   * 
   * @param title Ttulo da tabela.
   * @param table Tabela.
   * 
   * @return Componente contendo a tabela e seu ttulo.
   */
  private static JComponent createTablePanel(String title, JTable table) {
    table.setPreferredScrollableViewportSize(new Dimension(800, 100));

    JPanel panel = new JPanel(new GridBagLayout());

    JLabel label = new JLabel(title);
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets = new Insets(5, 5, 0, 5);
    constraints.gridx = 0;
    constraints.gridy = 0;
    constraints.gridwidth = 2;
    constraints.fill = GridBagConstraints.BOTH;
    constraints.anchor = GridBagConstraints.WEST;
    panel.add(label, constraints);

    constraints = new GridBagConstraints();
    constraints.insets = new Insets(5, 5, 5, 5);
    constraints.gridx = 0;
    constraints.gridy = 1;
    constraints.gridwidth = 2;
    constraints.weightx = 1.0;
    constraints.weighty = 1.0;
    constraints.fill = GridBagConstraints.BOTH;
    constraints.anchor = GridBagConstraints.WEST;

    panel.add(new JScrollPane(table), constraints);

    return panel;
  }
}
