/**
 * $Id: ExecutorFrame.java 98246 2009-10-29 18:12:34Z juliana $
 */
package csbase.client.applications.executor;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.MessageFormat;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import tecgraf.javautils.FileUtils;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.algorithms.AlgorithmChooserEvent;
import csbase.client.algorithms.AlgorithmChooserListener;
import csbase.client.algorithms.AlgorithmChooserPanel;
import csbase.client.algorithms.AlgorithmConfiguratorFactory;
import csbase.client.algorithms.AlgorithmConfiguratorView;
import csbase.client.applicationmanager.ApplicationException;
import csbase.client.applicationmanager.ApplicationManager;
import csbase.client.applications.ApplicationFrame;
import csbase.client.applications.ApplicationProject;
import csbase.client.desktop.DesktopFrame;
import csbase.client.desktop.RemoteTask;
import csbase.client.project.ProjectTreePath;
import csbase.client.remote.srvproxies.AlgorithmManagementProxy;
import csbase.client.util.ClientUtilities;
import csbase.client.util.sga.CommandExecutionDialog;
import csbase.client.util.sga.CommandRequestedListener;
import csbase.exception.ParseException;
import csbase.logic.ClientProjectFile;
import csbase.logic.ClientProjectFileInputStream;
import csbase.logic.CommandInfo;
import csbase.logic.algorithms.AlgorithmConfigurator;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.logic.algorithms.ExecutionType;
import csbase.logic.algorithms.FileParameterValue;
import csbase.logic.algorithms.serializer.AlgorithmConfigurationSerializerFactory;
import csbase.logic.algorithms.serializer.IAlgorithmConfigurationSerializer;
import csbase.logic.algorithms.serializer.exception.AlgorithmConfigurationSerializerParameterNotFoundException;

/**
 * A classe <code>ExecutorFrame</code> implementa o dilogo de execuo de
 * algoritmos.
 */
public final class ExecutorFrame extends ApplicationProject {
  /** Painel seletor de verses de algoritmos */
  private AlgorithmChooserPanel algorithmChooserPanel;

  /** Viso do configurador de algoritmos */
  private AlgorithmConfiguratorView algorithmConfiguratorView;

  /** Configurador de algoritmos */
  private AlgorithmConfigurator algorithmConfigurator;

  /** Algoritmos carregados na inicializao do executor */
  private SortedSet<AlgorithmInfo> algorithms = new TreeSet<AlgorithmInfo>();

  /** Indica se nessa execuo deve-se realizar a carga inicial de algoritmos */
  private boolean mustLoadAlgorithms;

  /** Painel de botes */
  private JPanel buttonPanel;

  /** Boto Executar */
  private JButton executeButton;

  /** Boto Informar Parmetros */
  private JButton paramButton;

  /** Boto Fechar */
  private JButton closeButton;

  /**
   * Listeners interessados nos comandos solicitados com o dilogo de execuo
   * de comandos
   */
  private final List<CommandRequestedListener> listeners;

  /**
   * Indica se deve informar ao usurio que o comando foi submetido ao servidor,
   * e se deseja monitor-lo
   */
  private boolean mustShowCommandRequested;

  /**
   * Cria a janela executora de algoritmos.
   * 
   * @param id O identificador da aplicao.
   */
  public ExecutorFrame(String id) {
    super(id);
    buildFrame();
    listeners = new LinkedList<CommandRequestedListener>();
    mustShowCommandRequested = true;
    mustLoadAlgorithms = true;
  }

  /**
   * (non-Javadoc)
   * 
   * @see csbase.client.applications.Application#killApplication()
   */
  @Override
  public void killApplication() {
  }

  /**
   * (non-Javadoc)
   * 
   * @see csbase.client.applicationmanager.ApplicationType#sendMessage(java.lang.String,
   *      java.lang.Object)
   */
  @Override
  public void sendMessage(String name, Object value) {
    if (name == null) {
      throw new IllegalArgumentException("O parmetro name no pode ser nulo.");
    }
    if (value == null) {
      throw new IllegalArgumentException("O parmetro value no pode ser nulo.");
    }
    if (name.equals(PROJECT_FILE_MESSAGE)) {
      ClientProjectFile clientProjectFile = (ClientProjectFile) value;
      if (clientProjectFile.getType().equals("PARAMETERS")) {

        AlgorithmConfigurator configurator =
          readParameterFile(clientProjectFile);
        if (configurator == null) {
          return;
        }
        String algorithmName = configurator.getAlgorithmName();
        AlgorithmVersionId versionId = configurator.getAlgorithmVersionId();
        if ((algorithmName != null) && (versionId != null)) {
          if (!createConfigurator(algorithmName, versionId)) {
            closeApplication();
            return;
          }
        }
        else {
          String message =
            MessageFormat.format(getString("error.missing_information"),
              FileUtils.joinPath('/', clientProjectFile.getPath()));
          showError(message);
          closeApplication();
          return;
        }
        this.algorithmConfiguratorView.setCurrentPath(new ProjectTreePath(
          clientProjectFile));
        this.algorithmConfiguratorView.launch();
        return;
      }
      String path = FileUtils.joinPath('/', clientProjectFile.getSimplePath());
      String type = clientProjectFile.getType();
      FileParameterValue inputFile = new FileParameterValue(path, type);
      this.algorithmConfigurator.setDefaultInputFile(inputFile);
      return;
    }
  }

  /**
   * Exibe um configurador para um algoritmo na sua verso mais recente.
   * 
   * @param algorithmName O nome do algoritmo.
   * @param algorithmVersionId A verso do algoritmo.
   * @param algorithmParameters Os parmetros do algoritmo.
   * 
   * @throws ApplicationException Caso ocorra erro durante o procedimento.
   */
  public void showConfigurator(String algorithmName,
    AlgorithmVersionId algorithmVersionId,
    Map<String, String> algorithmParameters) throws ApplicationException {
    try {
      if (this.algorithmChooserPanel.selectAlgorithm(algorithmName)) {
        if (this.algorithmChooserPanel.selectVersion(algorithmVersionId)) {
          if (algorithmParameters != null) {
            this.algorithmConfigurator
              .setParameterValuesByName(algorithmParameters);
          }
          if (this.algorithmConfigurator != null) {
            this.algorithmConfiguratorView.launch();
          }
        }
      }
    }
    catch (ParseException e) {
      throw new ApplicationException(e);
    }
  }

  /**
   * Exibe um configurador para um algoritmo na sua verso mais recente.
   * 
   * @param algorithmName O nome do algoritmo.
   * @param algorithmParameters Os parmetros do algoritmo.
   * 
   * @throws ApplicationException Caso ocorra erro durante o procedimento.
   */
  public void showConfigurator(String algorithmName,
    Map<String, String> algorithmParameters) throws ApplicationException {
    if (this.algorithmChooserPanel.selectAlgorithm(algorithmName)) {
      AlgorithmInfo info =
        this.algorithmChooserPanel.getSelectedAlgorithmInfo();
      AlgorithmVersionId algorithmVersionId = info.getLastVersion().getId();

      showConfigurator(algorithmName, algorithmVersionId, algorithmParameters);
    }
  }

  /**
   * (non-Javadoc)
   * 
   * @see csbase.client.applications.Application#startApplication()
   */
  @Override
  public void startApplication() throws ApplicationException {
    if (this.algorithmChooserPanel != null) {
      this.algorithmChooserPanel.setAlgorithmSet(this.getAlgorithmInfoSet());
      if (!this.algorithmChooserPanel.hasAlgorithms() && mustLoadAlgorithms) {
        Window window = DesktopFrame.getInstance().getDesktopFrame();
        StandardDialogs.showWarningDialog(window, 
          getString("application.title"), getString("error.no_algorithms"));
        return;
      }
    }
    super.startApplication();
  }

  /**
   * (non-Javadoc)
   * 
   * @see csbase.client.applications.Application#userCanKillApplication()
   */
  @Override
  public boolean userCanKillApplication() {
    return true;
  }

  /**
   * Cria a janela de execuo de algoritmos
   */
  private void buildFrame() {
    createAlgorithmChooserPanel();
    createButtonsPanel();
    populateFrame();
  }

  /**
   * Preenche a janela da aplicao com os widgets.
   */
  private void populateFrame() {
    Container cp = getApplicationFrame().getContentPane();
    cp.removeAll();
    cp.add(this.algorithmChooserPanel, BorderLayout.CENTER);
    cp.add(this.buttonPanel, BorderLayout.SOUTH);
    cp.validate();
    getApplicationFrame().pack();
  }

  /**
   * Remove o configurador de algoritmos atual e atualiza os widgets
   * relacionados.
   */
  private void clearConfigurator() {
    this.algorithmConfigurator = null;
    this.algorithmConfiguratorView = null;
  }

  /**
   * Cria o painel de seleo de algoritmos e verses.
   */
  private void createAlgorithmChooserPanel() {
    this.algorithmChooserPanel =
      new AlgorithmChooserPanel(getApplicationFrame(), mustShowHistory());
    this.algorithmChooserPanel
      .addAlgorithmChooserListener(new AlgorithmChooserListener() {
        public void wasChangedVersion(AlgorithmChooserEvent event) {
          updateVersionInfo();
          updateButtons();
        }

        public void wasChangedAlgorithm(AlgorithmChooserEvent event) {
          clearConfigurator();
          updateButtons();
        }
      });
  }

  /**
   * Cria o painel de botes.
   */
  private void createButtonsPanel() {
    this.closeButton = new JButton(getString("button.close"));
    this.closeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ev) {
        closeApplication();
      }
    });
    this.executeButton = new JButton(getString("button.execute"));
    this.executeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ev) {
        showExecuteDialog();
      }
    });
    this.paramButton = new JButton(getString("button.parameter"));
    this.paramButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ev) {
        showParamsDialog();
      }
    });
    ClientUtilities.adjustEqualSizes(this.paramButton, this.closeButton,
      this.executeButton);
    this.buttonPanel = new JPanel(new FlowLayout());
    this.buttonPanel.add(this.paramButton);
    this.buttonPanel.add(this.executeButton);
    this.buttonPanel.add(this.closeButton);
    updateButtons();
  }

  /**
   * Cria o configurador de uma determinada verso de algoritmo.
   * 
   * @param algorithmName O nome do algoritmo.
   * @param versionId O identificador da verso.
   * 
   * @return O configurador.
   */
  private boolean createConfigurator(String algorithmName,
    AlgorithmVersionId versionId) {
    if (!this.algorithmChooserPanel.selectAlgorithm(algorithmName)) {
      showError(MessageFormat.format(getString("error.algorithm_not_found"),
        algorithmName));
      return false;
    }
    if (!this.algorithmChooserPanel.selectVersion(versionId)) {
      showError(MessageFormat.format(getString("error.version_not_found"),
        algorithmName, versionId));
      return false;
    }
    return true;
  }

  /**
   * Verifica se  necessrio permitir que o histrico seja exibido.
   * 
   * @return <code>true</code> se o histrico precisa ser exibido ou
   *         <code>false</code> caso contrrio.
   */
  private boolean mustShowHistory() {
    return getBooleanSpecificProperty("showHistory", true);
  }

  /**
   * L o arquivo de parmetros do arquivo de projeto informado.
   * 
   * @param clientProjectFile O arquivo de projeto.
   * 
   * @return O arquivo de parmetros.
   */
  private AlgorithmConfigurator readParameterFile(
    final ClientProjectFile clientProjectFile) {
    RemoteTask<AlgorithmConfigurator> task =
      new RemoteTask<AlgorithmConfigurator>() {

        @Override
        protected void performTask() throws Exception {
          ClientProjectFileInputStream inputStream = null;
          try {
            inputStream =
              new ClientProjectFileInputStream(clientProjectFile, 0);

            IAlgorithmConfigurationSerializer serializer =
              AlgorithmConfigurationSerializerFactory
                .getSerializer(AlgorithmConfigurator.class);

            AlgorithmConfigurator configurator = serializer.read(inputStream);

            setResult(configurator);
          }
          finally {
            if (inputStream != null) {
              inputStream.close();
            }
          }
        }

        @Override
        protected void handleError(Exception error) {
          if (error instanceof ParseException) {
            showError(error.getLocalizedMessage());
          }
          else if (error instanceof AlgorithmConfigurationSerializerParameterNotFoundException) {
            showError(error.getLocalizedMessage());
          }
          else {
            super.handleError(error);
          }
        }
      };
    if (task.execute(getApplicationFrame(), getName(),
      getString("msg.reading.file"))) {
      return task.getResult();
    }
    return null;
  }

  /**
   * Atribui a viso do configurador a esta aplicao.
   * 
   * @param configuratorView A viso do configurador.
   */
  private void setConfiguratorView(AlgorithmConfiguratorView configuratorView) {
    this.algorithmConfiguratorView = configuratorView;
    this.algorithmConfigurator = configuratorView.getConfigurator();
  }

  /**
   * Exibe o dilogo de obteno de parmetros do algoritmo
   */
  private void showParamsDialog() {
    this.algorithmConfiguratorView.launch();
  }

  /**
   * Exibe o dilogo de seleo de servidores.
   */
  private void showExecuteDialog() {
    CommandExecutionDialog commandExecutionDialog =
      new CommandExecutionDialog(getApplicationFrame(),
        this.algorithmConfiguratorView, getApplicationProject());
    commandExecutionDialog
      .addCommandRequestedListener(new CommandRequestedListener() {
        public void commandsWereRequested(Set<CommandInfo> submittedCommands) {
          if (!mustShowCommandRequested) {
            return;
          }
          int numberOfSubmittedCommands = submittedCommands.size();
          if (numberOfSubmittedCommands == 1) {
            CommandInfo command = submittedCommands.iterator().next();
            if (command.isAutomatic()) {
              showCommandRequest(MessageFormat.format(
                getString("msg.command_auto_requested"), command.getTip()));
            }
            else {
              showCommandRequest(MessageFormat.format(
                getString("msg.command_requested_on_server"), command.getTip(),
                command.getSGAName()));
            }
          }
          else {
            // Quando mais de um comando foi submetido, basta testar se o 
            // primeiro  automtico
            CommandInfo firstCommand = submittedCommands.iterator().next();
            if (firstCommand.isAutomatic()) {
              showCommandRequest(MessageFormat.format(
                getString("msg.command_auto_requested_on_servers"),
                firstCommand.getTip(), numberOfSubmittedCommands));
            }
            else {
              String requestedServers = "";
              Set<String> sgaNames = new HashSet<String>();
              for (CommandInfo command : submittedCommands) {
                if (sgaNames.add(command.getSGAName())) {
                  requestedServers =
                    requestedServers + "\n - " + command.getSGAName();
                }
              }
              int nExecutions;
              if (firstCommand.getExecutionType() == ExecutionType.DISTRIBUTED) {
                nExecutions =
                  firstCommand.getExecutionCountPerSGAForDistributedExecution();
              }
              else {
                nExecutions =
                  firstCommand.getExecutionCountPerSGAForMultipleExecution();
              }
              showCommandRequest(MessageFormat.format(
                getString("msg.command_requested_on_servers"), firstCommand
                  .getTip(), requestedServers, nExecutions));
            }
          }
        }
      });
    addExternalListeners(commandExecutionDialog);
    commandExecutionDialog.setVisible(true);
  }

  /**
   * Exibe uma mensagem de comando submetido.
   * 
   * @param message A mensagem (No aceita {@code null}).
   */
  private void showCommandRequest(String message) {
    if (message == null) {
      throw new IllegalArgumentException("O parmetro message est nulo.");
    }
    String[] buttons =
      new String[] { getString("button.monitor"), getString("button.close") };
    ApplicationFrame applicationFrame = getApplicationFrame();
    String applicationTitle = applicationFrame.getTitle();
    int option =
      JOptionPane.showOptionDialog(applicationFrame, message, applicationTitle,
        JOptionPane.DEFAULT_OPTION, JOptionPane.INFORMATION_MESSAGE, null,
        buttons, buttons[1]);
    if (option == 0) {
      try {
        ApplicationManager.getInstance().runApplication("commandsmonitor");
      }
      catch (ApplicationException ex) {
        showException(ex.getMessage(), ex);
      }
    }
  }

  /**
   * Adiciona listeners externos para o comando solicitado pela janela de
   * execuo de comandos. Esses listeners so cadastrados por classes que
   * executem essa aplicao.
   * 
   * @param commandExecutionDialog janela de execuo de comandos
   */
  private void addExternalListeners(
    CommandExecutionDialog commandExecutionDialog) {
    for (CommandRequestedListener listener : listeners) {
      commandExecutionDialog.addCommandRequestedListener(listener);
    }
  }

  /**
   * Adiciona um listener para um comando solicitado com o dilogo de execuo
   * de comandos.
   * 
   * @param listener listener de um comando solicitado com o dilogo de execuo
   *        de comandos
   */
  public void addCommandRequestedListener(CommandRequestedListener listener) {
    if (listener == null) {
      throw new IllegalArgumentException("O parmetro listener est nulo.");
    }
    listeners.add(listener);
  }

  /**
   * Atualiza o botes. Caso o algoritmo no possa ser executado, exibe uma
   * mensagem na barra de status explicando o porqu.
   */
  private void updateButtons() {
    boolean enableButton = true;
    if (this.algorithmConfigurator == null) {
      enableButton = false;
      getApplicationFrame().getStatusBar().setError(
        getString("status.error.null_configurator"));
    }
    else if (algorithmConfigurator.hasParameterThatRequiresPipe()) {
      enableButton = false;
      getApplicationFrame().getStatusBar().setError(
        getString("status.error.flow_only_algorithm"));
    }
    else {
      getApplicationFrame().getStatusBar().setStatus(null);
    }
    this.executeButton.setEnabled(enableButton);
    this.paramButton.setEnabled(enableButton);
  }

  /**
   * Atualiza o painel de seleo de verses de algoritmos.
   */
  private void updateChooserPanel() {
    this.algorithmChooserPanel.setDescription(this.algorithmConfigurator
      .getDescription());
  }

  /**
   * Obtm o conjunto de todos os algoritmos. Esse conjunto  obtido usando-se o
   * proxy do servio (<code>AlgorithManagementProxy</code>), que retorna um
   * array de <code>AlgorithmInfo</code>. O array  convertido para um
   * <code>Set</code>. o prprio proxy poderia j retornar o <code>Set</code>.
   * 
   * @return o conjunto de todos os algoritmos obtidos do servio
   */
  private SortedSet<AlgorithmInfo> getAlgorithmInfoSet() {
    algorithms = new TreeSet<AlgorithmInfo>();
    List<String> algoIds = getStringListSpecificProperty("algo.id");

    //Verifica se deve ser feita a carga inicial dos algoritmos do servidor
    if (mustLoadAlgorithms) {
      final ApplicationFrame frame = this.getApplicationFrame();
      if (algoIds.size() > 0) {
        // Algoritmos especficos da aplicao
        for (String algoId : algoIds) {
          final AlgorithmInfo info =
            AlgorithmManagementProxy.getInfoById(algoId, frame);
          if (info != null) {
            algorithms.add(info);
          }
        }
      }
      else {
        // Todos os algoritmos
        final AlgorithmInfo[] infos =
          AlgorithmManagementProxy.retrieveAllAlgorithmInfos(frame);
        if (infos != null) {
          for (AlgorithmInfo info : infos) {
            final String hideProp = AlgorithmInfo.HIDE_ALGORITHM_PROPERTY;
            if (info.getPropertyValue(hideProp) == null) {
              algorithms.add(info);
            }
          }
        }
      }
    }
    return algorithms;
  }

  /**
   * Atualiza as informaes da verso selecionada.
   */
  private void updateVersionInfo() {
    AlgorithmVersionInfo selectedVersionInfo =
      this.algorithmChooserPanel.getSelectedVersionInfo();
    clearConfigurator();
    if (selectedVersionInfo != null) {
      AlgorithmConfiguratorView configuratorView =
        AlgorithmConfiguratorFactory.getInstance().createConfigurationView(
          getApplicationFrame(), selectedVersionInfo);
      if (configuratorView == null) {
        return;
      }
      setConfiguratorView(configuratorView);
      updateChooserPanel();
    }
    getApplicationFrame().pack();
  }

  /**
   * Obtm remotamente as informaes de um determinado algoritmo, dado o seu
   * nome.
   * 
   * @param algorithmName nome do algoritmo procurado
   * @return uma estrutura com as informaes do algoritmo procurado, ou null,
   *         caso ele no tenha sido encontrado no servidor
   * @throws ApplicationException no caso do algoritmo no ser encontrado.
   */
  private AlgorithmInfo getAlgorithmInfo(String algorithmName)
    throws ApplicationException {
    AlgorithmInfo info =
      AlgorithmManagementProxy.getInfoByName(algorithmName,
        getApplicationFrame());
    if (info == null) {
      throw new ApplicationException(getString("error.inexistent_algorithm"));
    }
    return info;
  }

  /**
   * Obtm um conjunto ordenado com as verses dos algoritmos, cujos
   * identificadores da verso especificados.
   * 
   * @param algorithmInfo informaes do algoritmo
   * @param versionIdList array contendo os identificadores das verses
   *        procuradas do algoritmo
   * @return retorna um conjunto ordenado com as verses dos algoritmos
   */
  private SortedSet<AlgorithmVersionInfo> getAlgorithmVersionSet(
    AlgorithmInfo algorithmInfo, AlgorithmVersionId[] versionIdList) {
    SortedSet<AlgorithmVersionInfo> sortedVersionSet =
      new TreeSet<AlgorithmVersionInfo>();
    for (AlgorithmVersionId versionId : versionIdList) {
      AlgorithmVersionInfo versionInfo =
        algorithmInfo.getVersionInfo(versionId);
      if (versionInfo != null) {
        sortedVersionSet.add(versionInfo);
      }
    }
    return sortedVersionSet;
  }

  /**
   * Estabelece um nico algoritmo, com seu nome e verses especificados, para
   * ser exibido na lista de algoritmos que o usurio pode editar e executar.
   * 
   * @param algorithmName nome do algoritmo
   * @param versionIdList array contendo os identificadores das verses
   *        procuradas do algoritmo
   * @throws ApplicationException exceo lanada quando o algoritmo requerido
   *         no est cadastrado no servidor
   */
  public void setSingleAlgorithm(String algorithmName,
    AlgorithmVersionId[] versionIdList) throws ApplicationException {
    //Algoritmo a ser exibido na lista
    AlgorithmInfo algorithmInfo = getAlgorithmInfo(algorithmName);
    algorithms.add(algorithmInfo);
    this.algorithmChooserPanel.setAlgorithmSet(algorithms);

    if (versionIdList != null) {
      //Verses do algoritmo a serem exibidas na lista
      SortedSet<AlgorithmVersionInfo> algorithmVersionSet =
        getAlgorithmVersionSet(algorithmInfo, versionIdList);
      this.algorithmChooserPanel.setAlgorithmVersion(algorithmVersionSet);
    }
  }

  /**
   * Estabelece um nico algoritmo para ser exibido na lista de algoritmos que o
   * usurio pode editar e executar, de acordo com o nome especificado. Todas as
   * verses do algoritmo so exibidas na lista de seleo de verses.
   * 
   * @param algorithmName nome do algoritmo
   * @throws ApplicationException exceo lanada quando o algoritmo requerido
   *         no est cadastrado no servidor
   */
  public void setSingleAlgorithm(String algorithmName)
    throws ApplicationException {
    setSingleAlgorithm(algorithmName, null);
  }

  /**
   * Verifica se a informao de que o comando foi solicitado deve ser exibida.
   * 
   * @return retorna true, se deve ser exibida a informao de que o comando foi
   *         solicitado, caso contrrio, retorna false
   */
  public boolean isMustShowCommandRequested() {
    return mustShowCommandRequested;
  }

  /**
   * Determina se deve ou no exibir uma janela com a informao de que o
   * comando foi solicitado, e se deseja monitor-lo.
   * 
   * @param state se true, exibe a informao quando o comando for solicitado,
   *        caso contrrio, a janela no  exibida
   */
  public void showCommandRequestedInfo(boolean state) {
    mustShowCommandRequested = state;
  }

  /**
   * Determina se deve ou no carregar os algoritmos do servidor inicialmente,
   * ou se isso ser feito sob demanda.
   * 
   * @param state se true, carrega os algoritmos inicialmente, caso contrrio,
   *        cada algoritmo ser carregado sob demanda
   */
  public void enableLoadAlgorithms(boolean state) {
    mustLoadAlgorithms = state;
  }

  /**
   * Estabelece a descrio default do comando a ser gerado pelo algoritmo.
   * 
   * @param commandDescription descrio do comando
   */
  public void setCommandDescription(String commandDescription) {
    algorithmConfigurator.setCommandDescription(commandDescription);
  }
}
