package csbase.client.applications.projectsmanager;

import java.awt.Dimension;
import java.awt.GridBagLayout;
import java.util.ArrayList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.RowSorter;
import javax.swing.SwingConstants;
import javax.swing.table.TableModel;

import tecgraf.javautils.gui.GBC;
import tecgraf.javautils.gui.MenuButton;
import tecgraf.javautils.gui.StandardDialogs;
import tecgraf.javautils.gui.MenuButton.PopupPosition;
import csbase.client.applications.ApplicationExitAction;
import csbase.client.applications.ApplicationFrame;
import csbase.client.applications.ApplicationProject;
import csbase.client.applications.projectsmanager.actions.AllocateAreaAction;
import csbase.client.applications.projectsmanager.actions.AppendUsersAction;
import csbase.client.applications.projectsmanager.actions.ChangeStatusAllocateAction;
import csbase.client.applications.projectsmanager.actions.ChangeStatusNoAllocationAction;
import csbase.client.applications.projectsmanager.actions.DeallocateAreaAction;
import csbase.client.applications.projectsmanager.actions.DisplayFilterAction;
import csbase.client.applications.projectsmanager.actions.FillProjectsTableAction;
import csbase.client.applications.projectsmanager.actions.OpenMailDialogAction;
import csbase.client.applications.projectsmanager.actions.OpenProjectAreaOccupationDialogAction;
import csbase.client.applications.projectsmanager.actions.OpenProjectCreationDialogAction;
import csbase.client.applications.projectsmanager.actions.OpenProjectHistoryDialogAction;
import csbase.client.applications.projectsmanager.actions.OpenProjectOccupationDialogAction;
import csbase.client.applications.projectsmanager.actions.ProjectRemovalAction;
import csbase.client.applications.projectsmanager.actions.RefreshProjectsAction;
import csbase.client.applications.projectsmanager.actions.RemoveUsersAction;
import csbase.client.applications.projectsmanager.actions.ReplicateSharingDataAction;
import csbase.client.applications.projectsmanager.actions.SearchFilesDialogAction;
import csbase.client.applications.projectsmanager.actions.SetProjectPrivateAction;
import csbase.client.applications.projectsmanager.actions.SetProjectPublicAction;
import csbase.client.applications.projectsmanager.actions.SetProjectSharedAction;
import csbase.client.applications.projectsmanager.actions.UserRemovalFromProjectAction;
import csbase.client.applications.projectsmanager.models.ProjectSpaceAllocation;
import csbase.client.applications.projectsmanager.models.ProjectsManagerData;
import csbase.client.applications.projectsmanager.models.ProjectsManagerScope;
import csbase.client.applications.projectsmanager.panels.FilterPanel;
import csbase.client.applications.projectsmanager.panels.InfoPanel;
import csbase.client.applications.projectsmanager.panels.TablePanel;
import csbase.client.applications.projectsmanager.proxy.RetrieveUsersTask;
import csbase.client.desktop.RemoteTask;
import csbase.client.project.tasks.CheckAreaReservedTask;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.CommonClientProject;
import csbase.logic.CommonProjectInfo;
import csbase.logic.User;
import csbase.logic.UserOutline;

/**
 * Classe que representa a aplicao ProjectsManager.
 * 
 * @author jnlopes
 */
public class ProjectsManager extends ApplicationProject implements Observer {

  /**
   * Painel que apresenta a tabela onde so listados os projetos.
   */
  private TablePanel tablePanel;

  /**
   * Painel que apresenta informaes do(s) projeto(s) selecionado(s).
   */
  private InfoPanel infoPanel;

  /**
   * Painel de filtragem dos projetos listados na tabela.
   */
  private FilterPanel filterPanel;

  /**
   * Ao de preencher inicialmente a tabela com todos os projetos do sistema
   * sem carregar o espao ocupado por eles.
   */
  private FillProjectsTableAction updateProjectsAction;

  /**
   * Ao da toolbar, de preencher a tabela com todos os projetos do sistema,
   * carregando o espao ocupado por eles.
   */
  private FillProjectsTableAction updateProjectsWithDiskUsageAction;

  /**
   * Ao de visualizar/esconder o painel de filtros.
   */
  private DisplayFilterAction displayFilterAction;

  /**
   * Toolbar
   */
  final private JToolBar toolbar = new JToolBar(SwingConstants.HORIZONTAL);

  /**
   * Lista de projetos visveis.
   * <p>
   */
  private List<ProjectsManagerData> projectsList;

  /**
   * Boto que exibe/oculta os filtros.
   */
  private JToggleButton filterButton;

  /**
   * Boto da toolbar que calcula espao ocupado por projetos.
   */
  private JButton projectsSpaceButton;

  /**
   * Boto para definir projeto como tendo sua rea alocada.
   */
  private JButton allocateButton;

  /**
   * Boto para definir projeto como tendo sua rea desalocada.
   */
  private JButton deallocateButton;

  /**
   * MenuButton para definio de projeto pblico RO ou RW.
   */
  private MenuButton setPublicMenuButton;

  /**
   * Boto para definir projeto como privado.
   */
  private JButton setPrivateButton;

  /**
   * Boto para definir projeto como compartilhado.
   */
  private JButton overwriteUsersButton;

  /**
   * Boto para cpia de esquema de compartilhamento entre projetos.
   */
  private JButton replicateSharedButton;

  /**
   * Boto para remoo de projeto.
   */
  private JButton projectRemovalButton;

  /**
   * Boto onde usurio se remove do projeto.
   */
  private JButton userRemovalButton;

  /**
   * Boto para adicionar comentrios no histrico de projetos.
   */
  private JButton writeHistoryButton;

  /**
   * Boto para enviar mails referentes a projetos para usurios.
   */
  private JButton mailButton;

  /**
   * Ao que calcula espao ocupado por projetos.
   */
  private RefreshProjectsAction multipleProjectSpaceAction;

  /**
   * Ao para definir projeto como tendo sua rea alocada.
   */
  private AllocateAreaAction allocateAreaAction;

  /**
   * Ao para definir projeto como tendo sua rea desalocada.
   */
  private DeallocateAreaAction deallocateAreaAction;

  /**
   * Ao para definir projeto como pblico RO.
   */
  private SetProjectPublicAction setPublicROProjectSharingStatsAction;

  /**
   * Ao para definir projeto como pblico RW.
   */
  private SetProjectPublicAction setPublicRWProjectSharingStatsAction;

  /**
   * Ao para definir projeto como privado.
   */
  private SetProjectPrivateAction setPrivateProjectSharingStatsAction;

  /**
   * Ao para abrir o dilogo de seleo de usurios compartilhados nos
   * projetos selecionados.
   */
  private SetProjectSharedAction setProjectSharedAction;

  /**
   * Ao para cpia de esquema de compartilhamento entre projetos.
   */
  private ReplicateSharingDataAction replicateSharingDataAction;

  /**
   * Ao para abrir o dilogo de adio de comentrios a histrico de projetos.
   */
  private OpenProjectHistoryDialogAction openProjectHistoryDialogAction;

  /**
   * Ao que abre dilogo de envio de email referente a projetos.
   */
  private OpenMailDialogAction openMailDialogAction;

  /**
   * Ao que abre dilogo para busca de arquivos referente a projetos.
   */
  private SearchFilesDialogAction searchFilesDialogAction;

  /**
   * Ao para abrir o dilogo de criao de projetos.
   */
  private OpenProjectCreationDialogAction openProjectCreationDialogAction;

  /**
   * Ao para remoo de projeto.
   */
  private ProjectRemovalAction projectRemovalAction;

  /**
   * Ao para usurio se remover de projetos.
   */
  private UserRemovalFromProjectAction userRemovalFromProjectAction;

  /**
   * Ao para abrir dilogo de informaes gerais sobre ocupao do espao de
   * projetos.
   */
  private OpenProjectOccupationDialogAction openProjectOccupationDialogAction;

  /**
   * Ao para abrir dilogo de informaes gerais sobre ocupao do espao de
   * projetos.
   */
  private OpenProjectAreaOccupationDialogAction openProjectAreaOccupationDialogAction;

  /**
   * Ao para adio de usurios a projetos com permisso de leitura.
   */
  private AppendUsersAction appendUsersROAction;

  /**
   * Ao para adio de usurios a projetos com permisso de leitura/escrita.
   */
  private AppendUsersAction appendUsersRWAction;

  /**
   * Ao para remoo de usurios compartilhados de projetos.
   */
  private RemoveUsersAction removeUsersAction;

  /**
   * Ao que abre dilogo para definio de projetos selecionados como contendo
   * rea alocada.
   */
  private ChangeStatusAllocateAction changeStatusAllocateAction;

  /**
   * Ao que abre dilogo para definio de projetos selecionados como no
   * contendo rea alocada.
   */
  private ChangeStatusNoAllocationAction changeStatusNoAllocationAction;

  /**
   * Item que calcula espao ocupado por projetos.
   */
  private JMenuItem multipleProjectSpaceItem;

  /**
   * Item para criar projetos.
   */
  private JMenuItem createProjectItem;

  /**
   * Item para definir projeto como tendo sua rea alocada.
   */
  private JMenuItem allocateAreaItem;

  /**
   * Item para definir projeto como tendo sua rea desalocada.
   */
  private JMenuItem deallocateAreaItem;

  /**
   * Item que define projetos selecionados como contendo rea alocada.
   */
  private JMenuItem setAllocatedStatusItem;

  /**
   * Item que define projetos selecionados como no contendo rea alocada.
   */
  private JMenuItem setNoAllocationStatusItem;

  /**
   * Item para definir projeto como pblico RO.
   */
  private JMenuItem setPublicROProjectSharingStatsItem;

  /**
   * Item para definir projeto como pblico RW.
   */
  private JMenuItem setPublicRWProjectSharingStatsItem;

  /**
   * Item para definir projeto como privado.
   */
  private JMenuItem setPrivateProjectSharingStatsItem;

  /**
   * Item para definir projeto como compartilhado.
   */
  private JMenuItem overwriteUsersItem;

  /**
   * Inclui usurios a projetos com acesso RO.
   */
  private JMenuItem appendUsersROItem;

  /**
   * Inclui usurios a projetos com acesso RW.
   */
  private JMenuItem appendUsersRWItem;

  /**
   * Remove usurios de projetos compartilhados.
   */
  private JMenuItem removeUsersItem;

  /**
   * Item de menu para cpia de esquema de compartilhamento entre projetos.
   */
  private JMenuItem replicateSharedItem;

  /**
   * Item para remoo de projeto.
   */
  private JMenuItem projectRemovalItem;

  /**
   * Item para usurio se remover de projetos.
   */
  private JMenuItem userRemovalFromProjectItem;

  /**
   * Item para adicionar comentrios no histrico de projetos.
   */
  private JMenuItem writeHistoryItem;

  /**
   * Item para enviar mails referentes a projetos para usurios.
   */
  private JMenuItem mailItem;

  /**
   * Item para pesquisar arquivos referentes a projetos para usurios.
   */
  private JMenuItem searchItem;

  /**
   * Objeto que observa o servio de administrao de projetos para criao e
   * remoo de projetos..
   */
  private ProjectsCreationRemovalObserver projectsCreationRemovalObserver;

  /**
   * Objeto que observa o servio de administrao de projetos para alteraes
   * no compartilhamento de projetos.
   */
  private AbstractProjectsManagerObserver projectSharingStatusObserver;

  /**
   * Boolean que indica se as informaes de reserva de rea devem ser exibidas
   * ou no.
   */
  private boolean areaReserved;

  /**
   * Construtor padro.
   * 
   * @param id o identificador da aplicao.
   * @throws Exception Erro na obteno do boolean de ProjectService que indica
   *         se o sistema utiliza reserva de rea.
   */
  public ProjectsManager(final String id) throws Exception {
    super(id);

    //  areaReserved = getBooleanSpecificProperty("reserveArea", false);
    defineReservedAreaUsage();
    initComponents();
    initActions();
    initToolbar();
    initMenu();
    enableItems(false);
    ensembleFrame();

    refreshProjectsTable(true, false);
    selectCurrentProject();

    final ApplicationFrame frame = getApplicationFrame();
    frame.setPreferredSize(new Dimension(940, 680));
    frame.pack();
    frame.update(null);

    projectsCreationRemovalObserver = new ProjectsCreationRemovalObserver(this);
    projectSharingStatusObserver = new ProjectSharingStatusObserver(this);
  }

  /**
   * Obtm o boolean que indica se o sistema usa ou no reserva de rea.
   * 
   * TODO - Preencher texto internacionalizado das tags do mtodo abaixo.
   * 
   * @throws Exception Erro na obteno do boolean de ProjectService que indica
   *         se o sistema utiliza reserva de rea.
   */
  private void defineReservedAreaUsage() throws Exception {

    final CheckAreaReservedTask task = new CheckAreaReservedTask();
    task.execute(getApplicationFrame(), ProjectsManagerUI.getString(getClass(),
      "area.reserved.title"), ProjectsManagerUI.getString(getClass(),
      "area.reserved.msg"));

    if (task.wasCancelled()) {
      areaReserved = false;
      final String title =
        ProjectsManagerUI
          .getString(getClass(), "area.reserved.cancelled.title");
      final String msg =
        ProjectsManagerUI.getString(getClass(), "area.reserved.cancelled.msg");
      StandardDialogs.showErrorDialog(getApplicationFrame(), title, msg);
      return;
    }

    if (task.getStatus() != true) {
      areaReserved = false;
      final Exception exception = task.getError();
      throw exception;
    }
    areaReserved = task.getResult();
  }

  /**
   * Seleciona imediatamente o projeto corrente (caso haja algum aberto) quando
   * esta aplicao for disparada. O projeto corrente (quando houver) ser
   * movido para o topo da tabela, de modo a garantir que alm de selecionado,
   * ele estar visvel ao usurio.
   */
  private void selectCurrentProject() {

    final CommonClientProject ccp;
    try {
      ccp = getApplicationProject();
    }
    catch (RuntimeException re) {
      /*
       * Esta exceo  lanada quando a aplicao for disparada sem que haja um
       * projeto aberto.
       */
      return;
    }

    if (ccp == null) {
      return;
    }
    final String projectName = ccp.getName();

    int counter = 0;
    final List<ProjectsManagerData> filteredProjects =
      tablePanel.getVisibleProjects();
    for (ProjectsManagerData pmd : filteredProjects) {
      if (pmd.getProjectName().equals(projectName)) {
        final ProjectsManagerData aux = filteredProjects.get(0);
        filteredProjects.set(0, pmd);
        filteredProjects.set(counter, aux);
        tablePanel.getProjectsTable().getSelectionModel().setSelectionInterval(
          0, 0);
        return;
      }
      counter++;
    }
  }

  /**
   * Inicializa as aes da toolbar e do menu que variam em funo da seleo na
   * tabela de projetos.
   */
  private void initActions() {

    multipleProjectSpaceAction = new RefreshProjectsAction(this, projectsList);
    openProjectCreationDialogAction = new OpenProjectCreationDialogAction(this);
    allocateAreaAction = new AllocateAreaAction(this, projectsList);
    deallocateAreaAction = new DeallocateAreaAction(this, projectsList);
    setPublicROProjectSharingStatsAction =
      new SetProjectPublicAction(this, projectsList, true);
    setPublicRWProjectSharingStatsAction =
      new SetProjectPublicAction(this, projectsList, false);
    setPrivateProjectSharingStatsAction =
      new SetProjectPrivateAction(this, projectsList);
    setProjectSharedAction = new SetProjectSharedAction(this, projectsList);

    replicateSharingDataAction =
      new ReplicateSharingDataAction(this, projectsList);

    appendUsersROAction = new AppendUsersAction(this, projectsList, true);
    appendUsersRWAction = new AppendUsersAction(this, projectsList, false);
    removeUsersAction = new RemoveUsersAction(this, projectsList);
    openProjectHistoryDialogAction =
      new OpenProjectHistoryDialogAction(this, projectsList);
    openMailDialogAction = new OpenMailDialogAction(this, projectsList);
    searchFilesDialogAction = new SearchFilesDialogAction(this);
    projectRemovalAction = new ProjectRemovalAction(this, projectsList);
    userRemovalFromProjectAction =
      new UserRemovalFromProjectAction(this, projectsList);
    openProjectOccupationDialogAction =
      new OpenProjectOccupationDialogAction(this);
    openProjectAreaOccupationDialogAction =
      new OpenProjectAreaOccupationDialogAction(this);
    this.changeStatusAllocateAction =
      new ChangeStatusAllocateAction(this, this.projectsList);

    this.changeStatusNoAllocationAction =
      new ChangeStatusNoAllocationAction(this, this.projectsList);

  }

  /**
   * Inicia componentes.
   */
  private void initComponents() {

    final ApplicationFrame frame = getApplicationFrame();

    displayFilterAction = new DisplayFilterAction(this);
    updateProjectsAction =
      new FillProjectsTableAction(this, isCurrentUserAdmin(), false);
    updateProjectsWithDiskUsageAction =
      new FillProjectsTableAction(this, isCurrentUserAdmin(), true);

    frame.setLayout(new GridBagLayout());
    filterPanel = new FilterPanel(this);
    tablePanel = new TablePanel(this);
    infoPanel = new InfoPanel(this);

    setPublicMenuButton =
      new MenuButton(ProjectsManagerScope.PUBLIC.getIcon(),
        PopupPosition.BOTTOM);
  }

  /**
   * Monta o frame principal da aplicao.
   */
  private void ensembleFrame() {
    final ApplicationFrame frame = getApplicationFrame();
    frame.add(toolbar, new GBC(0, 0).horizontal());
    frame.add(this.tablePanel, new GBC(0, 2).both());
    frame.add(infoPanel, new GBC(0, 3).horizontal());
  }

  /**
   * Exibe ou esconde os filtros da tabela.
   * 
   * @param showFilters Indica se os filtros devem ser exibidos ou escondidos.
   */
  final public void displayFilters(final boolean showFilters) {
    final ApplicationFrame ap = this.getApplicationFrame();

    if (showFilters) {
      ap.add(filterPanel, new GBC(0, 1).horizontal());
      filterButton.setSelected(true);
    }
    else {
      filterPanel.emptyFiltersData();
      filterPanel.runFilters(true);
      ap.remove(filterPanel);
      filterButton.setSelected(false);
    }

    ap.validate();
    ap.repaint();
  }

  /**
   * Altera a lista de projetos das aes da toolbar e do menu que variam em
   * funo da seleo na tabela de projetos.
   */
  private void setActions() {
    multipleProjectSpaceAction.setProjectsManagerData(projectsList);
    allocateAreaAction.setProjectsManagerData(projectsList);
    deallocateAreaAction.setProjectsManagerData(projectsList);
    setPublicROProjectSharingStatsAction.setProjectsManagerData(projectsList);
    setPublicRWProjectSharingStatsAction.setProjectsManagerData(projectsList);
    setPrivateProjectSharingStatsAction.setProjectsManagerData(projectsList);
    setProjectSharedAction.setProjectsManagerData(projectsList);

    replicateSharingDataAction.setProjectsManagerData(projectsList);

    appendUsersROAction.setProjectsManagerData(projectsList);
    appendUsersRWAction.setProjectsManagerData(projectsList);
    removeUsersAction.setProjectsManagerData(projectsList);
    openProjectHistoryDialogAction.setProjectsManagerData(projectsList);
    openMailDialogAction.setProjectsManagerData(projectsList);
    projectRemovalAction.setProjectsManagerData(projectsList);
    userRemovalFromProjectAction.setProjectsManagerData(projectsList);

    changeStatusAllocateAction.setProjectsManagerData(projectsList);
    changeStatusNoAllocationAction.setProjectsManagerData(projectsList);
  }

  /**
   * Inicia componentes da toolbar.
   */
  private void initToolbar() {

    setPublicMenuButton.add(setPublicROProjectSharingStatsAction);
    setPublicMenuButton.add(setPublicRWProjectSharingStatsAction);

    toolbar.setFloatable(false);

    toolbar.add(openProjectCreationDialogAction);

    toolbar.add(updateProjectsWithDiskUsageAction);
    projectsSpaceButton = toolbar.add(multipleProjectSpaceAction);

    filterButton = new JToggleButton(displayFilterAction);
    filterButton.setSelected(false);
    toolbar.add(filterButton);

    toolbar.addSeparator();

    if (isCurrentUserAdmin() && hasAreaReserved()) {
      toolbar.addSeparator();
      allocateButton = toolbar.add(allocateAreaAction);
      deallocateButton = toolbar.add(deallocateAreaAction);
    }

    toolbar.addSeparator();
    setPrivateButton = toolbar.add(setPrivateProjectSharingStatsAction);
    overwriteUsersButton = toolbar.add(setProjectSharedAction);
    toolbar.add(setPublicMenuButton);
    toolbar.addSeparator();

    replicateSharedButton = toolbar.add(replicateSharingDataAction);

    toolbar.addSeparator();

    /*
     * Inicialmente a toolbar contava com os botes de "Adicionar" e "Remover"
     * usurios de projetos compartilhados (como existem no menu). Retiramos
     * estes botes sem remov-los ou alterar sua lgica, pois caso eles voltem
     * a fazer parte da toolbar, bastar eliminar estas duas operaes abaixo.
     */

    projectRemovalButton = toolbar.add(projectRemovalAction);
    toolbar.addSeparator();

    if (!isCurrentUserAdmin()) {
      userRemovalButton = toolbar.add(userRemovalFromProjectAction);
      toolbar.addSeparator();
    }

    writeHistoryButton = toolbar.add(openProjectHistoryDialogAction);
    mailButton = toolbar.add(openMailDialogAction);
    toolbar.addSeparator();
    toolbar.add(searchFilesDialogAction);
  }

  /**
   * Inicializa os menus.
   */
  private void initMenu() {
    final ApplicationFrame frame = getApplicationFrame();

    final JMenu optionsMenu = initOptionsMenu();
    final JMenu projectsMenu = initProjectsMenu();

    final JMenuBar menu = new JMenuBar();
    menu.add(optionsMenu);
    menu.add(projectsMenu);

    frame.setJMenuBar(menu);
  }

  /**
   * Constri o menu de opes
   * 
   * @return o menu
   */
  private JMenu initOptionsMenu() {
    final String generalText = getMenuString("general");
    final JMenu generalMenu = new JMenu(generalText);

    createProjectItem = new JMenuItem(openProjectCreationDialogAction);
    createProjectItem.setText(getString("create.project.text"));
    generalMenu.add(createProjectItem);
    generalMenu.addSeparator();

    final JMenu reportMenu = initReportMenu();
    generalMenu.add(reportMenu);

    generalMenu.add(updateProjectsWithDiskUsageAction);

    multipleProjectSpaceItem = new JMenuItem(multipleProjectSpaceAction);
    generalMenu.add(multipleProjectSpaceItem);

    final JMenuItem filterItem = new JMenuItem(displayFilterAction);
    filterItem.setText(getString("filter.text"));

    generalMenu.add(filterItem);
    generalMenu.addSeparator();
    generalMenu.add(new ApplicationExitAction(this));
    return generalMenu;
  }

  /**
   * Inicia menu com opes de ocupao.
   * 
   * @return Menu com opes de ocupao.
   */
  private JMenu initReportMenu() {
    final String otherText = getMenuString("report");
    final JMenu occupationMenu = new JMenu(otherText);
    occupationMenu.add(openProjectOccupationDialogAction);
    occupationMenu.add(openProjectAreaOccupationDialogAction);
    return occupationMenu;
  }

  /**
   * Cria o menu referente a seleo de projetos.
   * 
   * @return O menu referente a seleo de projetos.
   */
  private JMenu initProjectsMenu() {
    final String projectsText = getMenuString("projects");
    final JMenu projectsMenu = new JMenu(projectsText);

    if (isCurrentUserAdmin()) {
      allocateAreaItem = new JMenuItem(allocateAreaAction);
      deallocateAreaItem = new JMenuItem(deallocateAreaAction);

      setAllocatedStatusItem = new JMenuItem(changeStatusAllocateAction);
      setNoAllocationStatusItem = new JMenuItem(changeStatusNoAllocationAction);
    }

    setPublicROProjectSharingStatsItem =
      new JMenuItem(setPublicROProjectSharingStatsAction);
    setPublicRWProjectSharingStatsItem =
      new JMenuItem(setPublicRWProjectSharingStatsAction);

    setPrivateProjectSharingStatsItem =
      new JMenuItem(setPrivateProjectSharingStatsAction);

    overwriteUsersItem = new JMenuItem(setProjectSharedAction);

    appendUsersROItem = new JMenuItem(appendUsersROAction);
    appendUsersRWItem = new JMenuItem(appendUsersRWAction);

    removeUsersItem = new JMenuItem(removeUsersAction);

    replicateSharedItem = new JMenuItem(replicateSharingDataAction);

    projectRemovalItem = new JMenuItem(projectRemovalAction);

    if (!isCurrentUserAdmin()) {
      userRemovalFromProjectItem = new JMenuItem(userRemovalFromProjectAction);
    }

    writeHistoryItem = new JMenuItem(openProjectHistoryDialogAction);
    mailItem = new JMenuItem(openMailDialogAction);
    searchItem = new JMenuItem(searchFilesDialogAction);

    projectsMenu.addSeparator();

    if (isCurrentUserAdmin() && hasAreaReserved()) {
      projectsMenu.add(allocateAreaItem);
      projectsMenu.add(deallocateAreaItem);
      projectsMenu.addSeparator();
      final JMenu advancedMenu = new JMenu();
      advancedMenu.add(setAllocatedStatusItem);
      advancedMenu.add(setNoAllocationStatusItem);
      advancedMenu.setText(getMenuString("advanced"));
      projectsMenu.add(advancedMenu);
      projectsMenu.addSeparator();
    }

    final JMenu publicMenu = new JMenu();
    publicMenu.setText(ProjectsManagerUI.getString(getClass(),
      "set.public.item"));
    publicMenu.add(setPublicROProjectSharingStatsItem);
    publicMenu.add(setPublicRWProjectSharingStatsItem);

    projectsMenu.add(setPrivateProjectSharingStatsItem);
    projectsMenu.add(overwriteUsersItem);
    projectsMenu.add(publicMenu);
    projectsMenu.addSeparator();

    final JMenu appendMenu = new JMenu();
    appendMenu.setText(ProjectsManagerUI.getString(this.getClass(),
      "append.users.item"));
    appendMenu.add(appendUsersROItem);
    appendMenu.add(appendUsersRWItem);
    projectsMenu.add(appendMenu);

    projectsMenu.add(removeUsersItem);
    projectsMenu.addSeparator();

    projectsMenu.add(replicateSharedItem);

    projectsMenu.addSeparator();
    projectsMenu.add(projectRemovalItem);

    if (!isCurrentUserAdmin()) {
      projectsMenu.addSeparator();
      projectsMenu.add(userRemovalFromProjectItem);
    }

    projectsMenu.addSeparator();
    projectsMenu.add(writeHistoryItem);
    projectsMenu.add(mailItem);
    projectsMenu.add(searchItem);

    return projectsMenu;
  }

  /**
   * Atualiza a lista de projetos selecionados, e avalia quais itens do menu
   * lateral devem estar habilitados de acordo com a seleo.
   * 
   * @param projectsList A lista de projetos selecionados.
   */
  public void setProjectsList(final List<ProjectsManagerData> projectsList) {
    this.projectsList = projectsList;
    if (projectsList == null || projectsList.size() == 0) {
      enableItems(false);
    }
    else {
      enableItems(true);
      setActions();
    }
    infoPanel.setSelectedProjects(projectsList);
  }

  /**
   * Retorna o texto de internacionalizao de menu.
   * 
   * @param tag tag do menu
   * @return o texto
   */
  final private String getMenuString(final String tag) {
    final String menuTag = tag + ".menu";
    return ProjectsManagerUI.getString(this.getClass(), menuTag);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  final public void killApplication() {
    final JFrame mainFrame = getApplicationFrame();
    projectsCreationRemovalObserver.removeItselfAsObserver();
    projectSharingStatusObserver.removeItselfAsObserver();
    mainFrame.dispose();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  final protected boolean userCanKillApplication() {
    return true;
  }

  /**
   * Retorna um vetor com todos os projetos acessveis.
   * 
   * @return vetor com todos os projetos acessveis
   */
  final public List<ProjectsManagerData> getAllProjects() {
    return tablePanel.getAllProjects();
  }

  /**
   * Retorna uma lista os projetos visveis aps aplicao do filtro.
   * 
   * @return lista os projetos disponveis aps aplicao do filtro. ATENO:
   *         retorna a prpria lista, e no uma cpia
   */
  final public List<ProjectsManagerData> getVisibleProjects() {
    return tablePanel.getVisibleProjects();
  }

  /**
   * Retorna usurio corrente.
   * 
   * @return O usurio corrente.
   */
  public User getUser() {
    return User.getLoggedUser();
  }

  /**
   * Informa se o usurio corrente  administrador do sistema.
   * 
   * @return boolean indicando se o usurio corrente  administrador do sistema.
   */
  public boolean isCurrentUserAdmin() {
    final User loggedUser = User.getLoggedUser();
    if (loggedUser.isAdmin())
      return true;

    return false;
  }

  /**
   * Verifica se um determinado usurio  o usurio corrente.
   * 
   * @param userID - identificador do usurio
   * @return <code>true</code> se o usurio  o usurio corrente
   */
  public boolean isCurrentUser(final Object userID) {
    final User user = getUser();
    return user.getId().equals(userID);
  }

  /**
   * Retorna uma lista de usurios do sistema
   * 
   * @return A lista de usurios do sistema.
   * @throws Exception se a tarefa de busca dos usurios o fizer
   */
  public List<UserOutline> getUsers() throws Exception {
    final RetrieveUsersTask raut = new RetrieveUsersTask(this);
    final ApplicationFrame frame = getApplicationFrame();
    raut.execute(frame, "", "");

    if (raut.wasCancelled()) {
      final String err = getString("cancelled.message");
      raut.showError(err);
      return null;
    }

    if (raut.getStatus() != true) {
      final Exception exception = raut.getError();
      throw exception;
    }

    final List<UserOutline> users = raut.getResult();
    if (users == null) {
      showMessage(getString("error.obtaining.users.message"));
    }
    return users;
  }

  /**
   * Desabilita todos os itens ta toolbar e do menu.
   * 
   * @param enabled Define se os itens devem ser habilitados ou desabilitados.
   */
  public void enableItems(final boolean enabled) {
    projectsSpaceButton.setEnabled(enabled);
    multipleProjectSpaceItem.setEnabled(enabled);

    if (allocateButton != null) {
      allocateButton.setEnabled(enabled);
    }

    if (allocateAreaItem != null) {
      allocateAreaItem.setEnabled(enabled);
    }

    if (deallocateButton != null) {
      deallocateButton.setEnabled(enabled);
    }

    if (deallocateAreaItem != null) {
      deallocateAreaItem.setEnabled(enabled);
    }

    if (setAllocatedStatusItem != null) {
      setAllocatedStatusItem.setEnabled(enabled);
    }

    if (setNoAllocationStatusItem != null) {
      setNoAllocationStatusItem.setEnabled(enabled);
    }

    setPublicMenuButton.setEnabled(enabled);
    setPublicROProjectSharingStatsItem.setEnabled(enabled);

    setPublicRWProjectSharingStatsItem.setEnabled(enabled);

    setPrivateButton.setEnabled(enabled);
    setPrivateProjectSharingStatsItem.setEnabled(enabled);

    overwriteUsersButton.setEnabled(enabled);
    overwriteUsersItem.setEnabled(enabled);

    appendUsersROItem.setEnabled(enabled);
    appendUsersRWItem.setEnabled(enabled);

    removeUsersItem.setEnabled(enabled);

    replicateSharedButton.setEnabled(enabled);
    replicateSharedItem.setEnabled(enabled);

    projectRemovalButton.setEnabled(enabled);
    projectRemovalItem.setEnabled(enabled);

    if (!isCurrentUserAdmin()) {
      userRemovalButton.setEnabled(enabled);
      userRemovalFromProjectItem.setEnabled(enabled);
    }

    writeHistoryButton.setEnabled(enabled);
    writeHistoryItem.setEnabled(enabled);

    mailButton.setEnabled(enabled);
    mailItem.setEnabled(enabled);
  }

  /**
   * Obtm um projeto com o mesmo nome e dono que outro (especificado via
   * {@link CommonProjectInfo}).
   * 
   * @param cpi informaes do projeto. Apenas o nome e o dono so usados na
   *        busca.
   * @return o projeto encontrado, ou null.
   */
  public ProjectsManagerData getProject(final CommonProjectInfo cpi) {
    return tablePanel.getProject(cpi.name, cpi.userId);
  }

  /**
   * Obtm um projeto com o mesmo nome e dono que outro (especificado via
   * {@link CommonClientProject}).
   * 
   * @param ccp informaes do projeto. Apenas o nome e o dono so usados na
   *        busca.
   * @return o projeto encontrado, ou null.
   */
  public ProjectsManagerData getProject(final CommonClientProject ccp) {
    return tablePanel.getProject(ccp.getName(), ccp.getUserId());
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void update(Observable o, Object arg) {

    if (o == null || !(o instanceof CommonClientProject)) {
      return;
    }

    final CommonClientProject ccp = (CommonClientProject) o;
    final String projectName = ccp.getName();

    ProjectsManagerData pmd = getProject(ccp);
    if (pmd == null) {
      pmd = new ProjectsManagerData(projectName, (String) ccp.getUserId());
    }

    updateSpecificProject(pmd, ccp);
  }

  /**
   * Atualiza um projeto da classe ProjectsManagerData a partir de um projeto da
   * classe CommonClientProject.
   * 
   * @param newProjectData Projeto da classe ProjectsManagerData.
   * @param newProject Projeto da classe CommonClientProject.
   */
  private void updateSpecificProject(final ProjectsManagerData newProjectData,
    final CommonClientProject newProject) {

    if (newProjectData == null || newProject == null) {
      return;
    }

    final GetUserNameTask gunt = new GetUserNameTask(newProject.getUserId());

    if (gunt.wasCancelled()) {
      final String err = getString("cancelled.message");
      StandardErrorDialogs.showErrorDialog(getApplicationFrame(), err);
      return;
    }

    if (gunt.getStatus() != true) {
      final Exception exception = gunt.getError();
      StandardErrorDialogs.showErrorDialog(getApplicationFrame(), exception);
      return;
    }

    final String username = gunt.getResult();
    if (username == null) {
      final String err = getString("user.not.found.message");
      StandardErrorDialogs.showErrorDialog(getApplicationFrame(), err);
      return;
    }

    newProjectData.setOwnerName(username);
    newProjectData
      .setProjectSpaceAllocation(ProjectSpaceAllocation.NO_ALLOCATION);

    newProjectData.setProjectDescription(newProject.getDescription());
    newProjectData.setCreationDate(newProject.getCreationDate());
    newProjectData.setModificationDate(newProject.getLastModificationDate());

    newProjectData.setSharingType(newProject.getSharingType());
    newProjectData.setUsers(newProject.getUsersRO(), newProject.getUsersRW());
    newProjectData.setOwner(true);

    addProject(newProjectData, true);
  }

  /**
   * Informa se o painel de filtragem est sendo exibido ou no.
   * 
   * @return boolean indicando se o painel de filtragem est sendo exibido ou
   *         no.
   */
  public boolean isFilterOn() {
    return filterButton.isSelected();
  }

  /**
   * Atualiza a lista de projetos <b>localmente</b>.
   * 
   * @see #refreshProjectsTable(boolean, boolean)
   */
  public void refreshProjectsTable() {
    refreshProjectsTable(false, false);
  }

  /**
   * Atualiza a lista de projetos, opcionalmente obtendo dados do servidor.
   * 
   * TODO verificar a relao com o filtro
   * 
   * @param updateFromServer - obtm dados atuais do servidor
   * @param updateOccupiedSpace - atualiza a ocupao em disco dos projetos.
   *        Esta opo tem precedncia sobre <code>updateFromServer</code>
   */
  public void refreshProjectsTable(final boolean updateFromServer,
    final boolean updateOccupiedSpace) {
    if (updateOccupiedSpace || updateFromServer) {
      try {
        if (updateOccupiedSpace) {
          updateProjectsWithDiskUsageAction.actionDone();
        }
        else {
          updateProjectsAction.actionDone();
        }
      }
      catch (Exception e) {
        StandardErrorDialogs.showErrorDialog(getApplicationFrame(), e);
      }
    }
    tablePanel.refreshTable();
  }

  /**
   * Desmarca linhas selecionadas da tabela de projetos.
   */
  public void clearProjectSelection() {
    tablePanel.clearSelection();
  }

  /**
   * Remove um projeto da estrutura projectsList a partir do seu nome.
   * 
   * @param cpi Nome do projeto.
   */
  public void removeProject(final CommonProjectInfo cpi) {
    tablePanel.removeProject(cpi.name, cpi.userId);
  }

  /**
   * Remove projetos da lista de projetos.
   * 
   * @param projectsToRemove projetos a serem removidos
   */
  public void removeProjects(final List<ProjectsManagerData> projectsToRemove) {
    final List<ProjectsManagerData> newList =
      new ArrayList<ProjectsManagerData>();
    for (ProjectsManagerData prj : projectsList) {
      final ProjectSpaceAllocation spaceAllocation =
        prj.getProjectSpaceAllocation();
      if (!projectsToRemove.contains(prj)) {
        newList.add(prj);
      }
      else if (spaceAllocation == ProjectSpaceAllocation.ALLOCATED) {
        updateProjectToWaitingDeallocation(prj);
        newList.add(prj);
      }
      else {
        tablePanel.removeProject(prj.getProjectName(), prj.getOwnerId());
      }
    }
    setProjectsList(newList);
  }

  /**
   * Atualiza as propriedades de um projeto do tipo
   * <code>ProjectsManagerData</code> que estava configurado como "alocado",
   * para "esperando desalocao".
   * 
   * @param pmd O projeto a ser definido como esperando desalocao.
   */
  private void updateProjectToWaitingDeallocation(ProjectsManagerData pmd) {
    pmd.setProjectSpaceAllocation(ProjectSpaceAllocation.WAITING_DEALLOCATION);
    pmd.setAllocatedSpace(0);
    pmd.setOccupiedSpace(-1);
    pmd.setScope(ProjectsManagerScope.NOT_APPLICABLE);
    pmd.setUsers(null, null);
  }

  /**
   * Atualiza o painel com as informaes sobre os projetos selecionados.
   */
  public void refreshInfoPanel() {
    infoPanel.update();
  }

  /**
   * Limpa campos do painel de informaes detalhadas dos projetos selecionados
   */
  public void clearInfoFields() {
    infoPanel.clearFields();
  }

  /**
   * Define a lista dos projetos visveis, i.e. que passaram pelo filtro.
   * 
   * @param filteredProjects projetos que passaram pelo filtro
   */
  public void setFilteredProjects(
    final List<ProjectsManagerData> filteredProjects) {
    tablePanel.setVisibleProjects(filteredProjects);
  }

  /**
   * Limpa a tabela.
   */
  public void clearTable() {
    tablePanel.emptyTableData();
  }

  /**
   * Limpa todos os filtros.
   */
  public void clearFilters() {
    filterPanel.emptyFiltersData();
  }

  /**
   * Adiciona um novo projeto  lista. Caso haja filtros ativos, o projeto ser
   * adicionado apenas se passar por todos os filtros.
   * 
   * @param prj projeto
   * @param refresh flag que indica se a tabela deve ser atualizada aps a
   *        incluso
   */
  public void addProject(final ProjectsManagerData prj, final boolean refresh) {
    if (prj == null) {
      return;
    }
    final boolean isVisible =
      !isFilterOn() || filterPanel.projectMatchesFilters(prj);
    tablePanel.addProject(prj, isVisible, refresh);
  }

  /**
   * Obtm uma lista com os projetos selecionados pelo usurio.
   * 
   * @return lista com os projetos selecionados pelo usurio
   */
  public List<ProjectsManagerData> getSelectedProjects() {
    return tablePanel.getSelectedProjects();
  }

  /**
   * Atualiza o valor de ocupao de um projeto em todas as listas onde este
   * projeto se encontra.
   * 
   * @param pmd O projeto cuja ocupao foi alterada.
   */
  public void updateProjectOccupiedSpace(final ProjectsManagerData pmd) {
    tablePanel.updateProjectOccupiedSpace(pmd);
  }

  /**
   * Aplica todos os filtros e atualiza a tabela.
   * 
   * @param clearSelection boolean que indica se a seleo de linhas corrente
   *        deve ser desfeita.
   */
  public void runFilters(final boolean clearSelection) {
    filterPanel.runFilters(clearSelection);
  }

  /**
   * Quando projetos esto aguardando desalocao de rea, eles s constam na
   * estrutura que mantm projetos em algum estgio da alocao. Caso os
   * projetos sejam removidos desta estrutura com sucesso, este mtodo 
   * utilizado para remover as referncias na lista de projetos local (do
   * projectsmanager).
   * 
   * @param projectsManagerData Lista de projetos a serem removidos localmente.
   */
  public void removeLocalProjects(
    final List<ProjectsManagerData> projectsManagerData) {
    tablePanel.removeProjects(projectsManagerData);
  }

  /**
   * Retorna boolean que indica se as informaes de reserva de rea devem ser
   * exibidas ou no.
   * 
   * @return Boolean que indica se as informaes de reserva de rea devem ser
   *         exibidas ou no.
   */
  public boolean hasAreaReserved() {
    return areaReserved;
  }

  /**
   * Retorna o objeto responsvel pela ordenao da tabela de projetos.
   * 
   * @return O objeto responsvel pela ordenao da tabela de projetos.
   */
  public RowSorter<? extends TableModel> getTableRowSorter() {
    return tablePanel.getTableRowSorter();
  }
}

/**
 * Remote Task para obteno de nome de usurio.
 * 
 * @author Tecgraf
 */
class GetUserNameTask extends RemoteTask<String> {

  /**
   * Id do usurio.
   */
  final Object userId;

  /**
   * Construtor.
   * 
   * @param userId Id do usurio.
   */
  GetUserNameTask(final Object userId) {
    this.userId = userId;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected void performTask() throws Exception {

    final String username = User.getName(userId);
    setResult(username);
  }

}
