/*
 * $Id$
 */

package csbase.client.applications.projectsmanager.actions;

import java.util.ArrayList;
import java.util.List;

import javax.swing.ImageIcon;

import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.applications.ApplicationFrame;
import csbase.client.applications.projectsmanager.ProjectsManager;
import csbase.client.applications.projectsmanager.ProjectsManagerUI;
import csbase.client.applications.projectsmanager.actions.core.ProjectsManagerAction;
import csbase.client.applications.projectsmanager.dialogs.StatusDialog;
import csbase.client.applications.projectsmanager.models.ProjectSpaceAllocation;
import csbase.client.applications.projectsmanager.models.ProjectsManagerData;
import csbase.client.applications.projectsmanager.models.ProjectsManagerScope;
import csbase.client.applications.projectsmanager.proxy.AllocateAreaTask;

/**
 * Ao que trata alterao na alocao de projetos. Projetos com espao alocado
 * que sero desalocados e projeto esperando alocao que sero alocados.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class AllocateAreaAction extends ProjectsManagerAction {

  /**
   * Projetos que tero sua alocao alterada.
   */
  private List<ProjectsManagerData> projectsManagerData;

  /**
   * Construtor
   * 
   * @param projectsManager A aplicao
   * @param projectsManagerData Projetos que tero sua alocao alterada.
   */
  public AllocateAreaAction(final ProjectsManager projectsManager,
      final List<ProjectsManagerData> projectsManagerData) {
    super(projectsManager);
    this.projectsManagerData = projectsManagerData;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void actionDone() throws Exception {
    if (projectsManagerData == null || projectsManagerData.size() <= 0) {
      return;
    }

    final ProjectsManager projectsManager = getProjectsManager();
    final ApplicationFrame appFrame = projectsManager.getApplicationFrame();
    final List<ProjectsManagerData> projects = validateProjects();
    if (projects == null) {
      return;
    }

    final boolean ret = runTask(projects);
    if (ret) {
      updateStatusOfLocalProjects();
      projectsManager.refreshProjectsTable();
    } else {
      final String errMsg = getString("error.allocation.message");
      final String errTitle = getString("error.allocation.title");
      StandardDialogs.showErrorDialog(appFrame, errTitle, errMsg);
    }
  }

  /**
   * Define o status de alocao dos projetos como
   * <code>ProjectSpaceAllocation.ALLOCATED</code> e o espao de alocao.
   */
  private void updateStatusOfLocalProjects() {
    for (ProjectsManagerData pmd : projectsManagerData) {
      pmd.setProjectSpaceAllocation(ProjectSpaceAllocation.ALLOCATED);
      final long requestedSpace = pmd.getRequestedSpace();
      pmd.setAllocatedSpace(requestedSpace);
      pmd.setScope(ProjectsManagerScope.PRIVATE);
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ImageIcon getStandardImageIcon() {
    return ProjectsManagerUI.ALLOCATE;
  }

  /**
   * Atualiza lista de projetos sobre os quais a ao ser realizada.
   * 
   * @param projectsManagerData Lista de projetos
   */
  public void setProjectsManagerData(
      final List<ProjectsManagerData> projectsManagerData) {
    this.projectsManagerData = projectsManagerData;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected List<ProjectsManagerData> validateProjects() {

    final List<ProjectsManagerData> prjList = new ArrayList<ProjectsManagerData>();
    prjList.addAll(projectsManagerData);

    final List<ProjectsManagerData> failedProjects = new ArrayList<ProjectsManagerData>();

    int i = 0;
    while (i < prjList.size()) {
      final ProjectsManagerData pmd = prjList.get(i);
      if (pmd.getProjectSpaceAllocation() != ProjectSpaceAllocation.WAITING_ALLOCATION) {
        failedProjects.add(pmd);
        prjList.remove(i);
      } else {
        i++;
      }
    }

    final ProjectsManager projectsManager = getProjectsManager();
    if (prjList.size() == 0) {
      final String deniedMessage = getString("project.selection.denied.message")
          + getString("project.requirements.message");
      StatusDialog.showNoneOkDialog(projectsManager, deniedMessage,
          failedProjects, getString("project.selection.failure.message"));
      return null;
    } else if (failedProjects.size() > 0) {
      final String someDeniedMessage = getString("project.selection.some.denied.message")
          + getString("project.requirements.message");
      final int res = StatusDialog.showSomeOkDialog(projectsManager,
          someDeniedMessage, prjList,
          getString("project.selection.some.failure.message"));
      if (res == 0) {
        return null;
      }
    }

    return prjList;
  }

  /**
   * Roda e trata a execuo da task.
   * 
   * @param projects Lista de projetos vlidos para a operao.
   * @return Boolean indicando se a operao foi bem sucedida.
   * @throws Exception
   */
  public boolean runTask(final List<ProjectsManagerData> projects)
      throws Exception {

    final String[] projectNames = new String[projects.size()];
    final String[] ownerIds = new String[projects.size()];
    int i = 0;
    for (ProjectsManagerData pmd : projects) {
      projectNames[i] = pmd.getProjectName();
      ownerIds[i] = pmd.getOwnerId();
      i++;
    }

    final ProjectsManager projectsManager = getProjectsManager();
    final AllocateAreaTask aat = new AllocateAreaTask(projectsManager,
        projectNames, ownerIds, true);

    final ApplicationFrame appFrame = projectsManager.getApplicationFrame();
    aat.execute(appFrame, getString("title"), getString("message"));

    if (aat.wasCancelled()) {
      final String err = getString("cancelled.message");
      aat.showError(err);
      return false;
    }

    if (aat.getStatus() != true) {
      final Exception exception = aat.getError();
      throw exception;
    }

    return aat.getResult();

  }
}
