package csbase.client.applications.projectsmanager.actions;

import java.util.ArrayList;
import java.util.List;

import javax.swing.ImageIcon;

import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.applications.ApplicationFrame;
import csbase.client.applications.projectsmanager.ProjectsManager;
import csbase.client.applications.projectsmanager.actions.core.ProjectsManagerAction;
import csbase.client.applications.projectsmanager.dialogs.ProjectStatusModificationDialog;
import csbase.client.applications.projectsmanager.dialogs.StatusDialog;
import csbase.client.applications.projectsmanager.models.ProjectSpaceAllocation;
import csbase.client.applications.projectsmanager.models.ProjectsManagerData;
import csbase.client.applications.projectsmanager.proxy.StatusModificationTask;

/**
 * Ao para alterao do status de alocao de "Sem alocao" para "Alocado".
 * 
 * @author jnlopes
 * 
 */
public class ChangeStatusAllocateAction extends ProjectsManagerAction {

  /**
   * Projetos que sero modificados.
   */
  private List<ProjectsManagerData> projectsManagerData;

  /**
   * Construtor
   * 
   * @param projectsManager A aplicao
   * @param projectsManagerData Lista de projetos selecionados.
   */
  public ChangeStatusAllocateAction(final ProjectsManager projectsManager,
      final List<ProjectsManagerData> projectsManagerData) {
    super(projectsManager);
    this.projectsManagerData = projectsManagerData;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void actionDone() throws Exception {

    if (projectsManagerData == null || projectsManagerData.size() <= 0) {
      return;
    }

    final List<ProjectsManagerData> projects = validateProjects();
    if (projects == null) {
      return;
    }

    final ProjectsManager projectsManager = getProjectsManager();
    final int allocatedSpace = ProjectStatusModificationDialog.openDialog(
        projectsManager, projects);
    if (allocatedSpace <= 0) {
      return;
    }

    final boolean ret = runTask(projects, allocatedSpace);
    if (ret) {
      updateStatusOfLocalProjects(projects, allocatedSpace);
      projectsManager.refreshProjectsTable();
    } else {
      final String errMsg = getString("error.allocation.message");
      final String errTitle = getString("error.allocation.title");
      StandardDialogs.showErrorDialog(projectsManager.getApplicationFrame(),
          errTitle, errMsg);
    }
  }

  /**
   * Define o status de alocao dos projetos validados como
   * <code>ProjectSpaceAllocation.ALLOCATED</code> e o espao de alocao.
   * 
   * @param projects Projetos validados.
   * @param allocatedSpace Espao a ser definido como alocado.
   */
  private void updateStatusOfLocalProjects(
      final List<ProjectsManagerData> projects, final int allocatedSpace) {
    for (ProjectsManagerData pmd : projects) {
      pmd.setProjectSpaceAllocation(ProjectSpaceAllocation.ALLOCATED);
      pmd.setRequestedSpace(allocatedSpace);
      pmd.setAllocatedSpace(allocatedSpace);
    }
  }

  /**
   * Roda e trata a execuo da task.
   * 
   * @param projects Lista de projetos vlidos para a operao.
   * @param allocatedSpace Espao a ser definido como alocado.
   * @return Boolean indicando se a operao foi bem sucedida.
   * @throws Exception
   */
  private boolean runTask(final List<ProjectsManagerData> projects,
      final int allocatedSpace) {

    final String[] projectNames = new String[projects.size()];
    final String[] ownerIds = new String[projects.size()];
    int i = 0;
    for (ProjectsManagerData pmd : projects) {
      projectNames[i] = pmd.getProjectName();
      ownerIds[i] = pmd.getOwnerId();
      i++;
    }

    final ProjectsManager projectsManager = getProjectsManager();
    final ApplicationFrame appFrame = projectsManager.getApplicationFrame();
    final StatusModificationTask smt = new StatusModificationTask(
        projectsManager, projectNames, ownerIds, allocatedSpace);

    final String taskTitle = getString("running.title");
    final String taskMsg = getString("running.message");
    smt.execute(appFrame, taskTitle, taskMsg);

    if (smt.wasCancelled()) {
      final String err = getString("cancelled.message");
      smt.showError(err);
      return false;
    }

    if (smt.getStatus() != true) {
      final Exception exception = smt.getError();
      final String err = getString("allocation.status.error.message");
      final String msg = err + "\n" + exception.getMessage();
      StandardDialogs.showErrorDialog(appFrame, msg, "");
      return false;
    }

    return true;
  }

  /**
   * Atualiza lista de projetos sobre os quais a ao ser realizada.
   * 
   * @param projectsManagerData lista de projetos.
   */
  public void setProjectsManagerData(
      final List<ProjectsManagerData> projectsManagerData) {
    this.projectsManagerData = projectsManagerData;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected List<ProjectsManagerData> validateProjects() {

    final List<ProjectsManagerData> prjList = new ArrayList<ProjectsManagerData>();

    prjList.addAll(projectsManagerData);
    final List<ProjectsManagerData> failedProjects = new ArrayList<ProjectsManagerData>();

    int i = 0;
    while (i < prjList.size()) {
      final ProjectsManagerData pmd = prjList.get(i);
      final ProjectSpaceAllocation spaceAllocation = pmd
          .getProjectSpaceAllocation();
      if (spaceAllocation != ProjectSpaceAllocation.NO_ALLOCATION) {
        failedProjects.add(pmd);
        prjList.remove(i);
      } else {
        i++;
      }
    }

    final ProjectsManager projectsManager = getProjectsManager();
    if (prjList.size() == 0) {
      final String deniedMessage = getString("project.selection.denied.message")
          + getString("project.requirements.message");
      StatusDialog.showNoneOkDialog(projectsManager, deniedMessage,
          failedProjects, getString("project.selection.failure.message"));
      return null;
    } else if (failedProjects.size() > 0) {
      final String someDeniedMessage = getString("project.selection.some.denied.message")
          + getString("project.requirements.message");
      final int res = StatusDialog.showSomeOkDialog(projectsManager,
          someDeniedMessage, prjList,
          getString("project.selection.some.failure.message"));
      if (res == 0) {
        return null;
      }
    }

    return prjList;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ImageIcon getStandardImageIcon() {
    return null;
  }
}
