package csbase.client.applications.projectsmanager.actions;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import javax.swing.ImageIcon;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.applications.ApplicationFrame;
import csbase.client.applications.ApplicationImages;
import csbase.client.applications.projectsmanager.ProjectsManager;
import csbase.client.applications.projectsmanager.actions.core.ProjectsManagerAction;
import csbase.client.applications.projectsmanager.dialogs.ReplicateSharedDialog;
import csbase.client.applications.projectsmanager.dialogs.StatusDialog;
import csbase.client.applications.projectsmanager.models.ProjectSpaceAllocation;
import csbase.client.applications.projectsmanager.models.ProjectsManagerData;
import csbase.client.applications.projectsmanager.proxy.core.ProjectsManagerTask;
import csbase.logic.CommonClientProject;
import csbase.logic.User;
import csbase.logic.ProjectPermissions.SharingType;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.ProjectServiceInterface;

/**
 * Ao para cpia de esquema de compartilhamento entre projetos.
 * 
 * @author Tecgraf
 */
public class ReplicateSharingDataAction extends ProjectsManagerAction {

  /**
   * Lista de projetos selecionados.
   */
  private List<ProjectsManagerData> projectsManagerData;

  /**
   * Construtor
   * 
   * @param projectsManager a aplicao
   * @param projectsManagerData a lista de projetos.
   */
  public ReplicateSharingDataAction(final ProjectsManager projectsManager,
    final List<ProjectsManagerData> projectsManagerData) {
    super(projectsManager);
    this.projectsManagerData = projectsManagerData;
  }

  /**
   * {@inheritDoc}
   * 
   * @throws Exception
   */
  @Override
  public void actionDone() throws Exception {
    if (projectsManagerData == null || projectsManagerData.size() == 0) {
      return;
    }

    final ProjectsManager projectsManager = getProjectsManager();
    final ApplicationFrame owner = projectsManager.getApplicationFrame();
    /*
     * Se o usurio selecionou mltiplos projetos, avisamos antes que as
     * alteraes sobrescrevero as configuraes de a todos os projetos
     * selecionados
     * 
     * configurao de permisses de acesso s pode ser feita para um projeto
     * por vez
     */
    if (projectsManagerData.size() > 1) {
      StandardDialogs.showInfoDialog(owner, LNG
        .get("OpenSharingDialogAction.info.dialog.title"), LNG
        .get("OpenSharingDialogAction.info.dialog.message"));
      return;
    }

    final List<ProjectsManagerData> projects = validateProjects();
    if (projects == null) {
      return;
    }

    /*
     *  garantido que s existe um projeto selecionado
     */
    final ProjectsManagerData sourceProject = projects.get(0);
    final ReplicateSharedDialog rsd =
      new ReplicateSharedDialog(projectsManager, sourceProject);

    final Set<ProjectsManagerData> targetProjects = rsd.getSelectedProjects();
    if (targetProjects == null || targetProjects.size() == 0) {
      return;
    }

    final ReplicateSharingDataTask rst =
      new ReplicateSharingDataTask(projectsManager, sourceProject,
        targetProjects);

    final String title = getString("title");
    final String msg = getString("message");
    rst.execute(projectsManager.getApplicationFrame(), title, msg);

    if (rst.wasCancelled()) {
      final String err = getString("cancelled.message");
      rst.showError(err);
      return;
    }

    if (rst.getStatus() != true) {
      final Exception exception = rst.getError();
      throw exception;
    }

    final SharingType sharingType = sourceProject.getSharingType();
    final Set<Object> uRO = sourceProject.getUsersRO();
    final Set<Object> uRW = sourceProject.getUsersRW();
    for (ProjectsManagerData pmd : targetProjects) {
      pmd.setSharingType(sharingType);
      pmd.setUsers(uRO, uRW);
    }
    projectsManager.refreshProjectsTable();
    projectsManager.refreshInfoPanel();
  }

  /**
   * Atualiza lista de projetos sobre os quais a ao ser realizada.
   * 
   * @param projectsManagerData lista de projetos.
   */
  public void setProjectsManagerData(
    final List<ProjectsManagerData> projectsManagerData) {
    this.projectsManagerData = projectsManagerData;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ImageIcon getStandardImageIcon() {
    /*
     * FIXME - Substituir cone!
     */
    //  return ProjectsManagerUI.DEFINE_USERS;
    return ApplicationImages.ICON_COPY_16;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected List<ProjectsManagerData> validateProjects() {

    final User loggedUser = User.getLoggedUser();
    final Object loggedUserId = loggedUser.getId();

    final ProjectsManagerData pmd = projectsManagerData.get(0);

    final ProjectSpaceAllocation spaceAllocation =
      pmd.getProjectSpaceAllocation();

    final boolean isAdmin = loggedUser.isAdmin();
    final boolean isOwner = pmd.getOwnerId().equals(loggedUserId);

    /*
     * Para que um usurio possa alterar o compartilhamento de um projeto 
     * preciso que: - O projeto possa ser aberto (no esteja aguardando alocao
     * / desalocao). - O usurio seja o administrador ou o dono do projeto.
     */
    if (spaceAllocation.isOpenable() && (isAdmin || isOwner)) {
      final List<ProjectsManagerData> l = new ArrayList<ProjectsManagerData>();
      l.add(pmd);
      return l;
    }

    final String deniedMessage =
      getString("project.selection.denied.message")
        + getString("project.requirements.message");

    StatusDialog.showNoneOkDialog(getProjectsManager(), deniedMessage, null,
      getString("project.selection.failure.message"));

    return null;
  }
}

/**
 * Task que copia configuraes de compartilhamento de um projeto a outros.
 * 
 * @author Tecgraf
 */
class ReplicateSharingDataTask extends ProjectsManagerTask<Void> {

  /**
   * Projeto cuja configurao de compartilhamento ser copiada a outros
   * projetos.
   */
  final ProjectsManagerData sourceProject;

  /**
   * Projetos que tero suas configuraes de compartilhamento sobreescritas.
   */
  final Set<ProjectsManagerData> targetProjects;

  /**
   * Construtor
   * 
   * @param projectsManager A aplicao.
   * @param sourceProject Projeto origem.
   * @param targetProjects Projetos destino.
   */
  public ReplicateSharingDataTask(final ProjectsManager projectsManager,
    final ProjectsManagerData sourceProject,
    final Set<ProjectsManagerData> targetProjects) {
    super(projectsManager);
    this.sourceProject = sourceProject;
    this.targetProjects = targetProjects;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected void performTask() throws Exception {

    final ProjectServiceInterface psi = ClientRemoteLocator.projectService;

    final CommonClientProject sourceCCP =
      psi.openProject(sourceProject.getOwnerId(), sourceProject
        .getProjectName(), false);

    final Set<Object> usersRO = sourceCCP.getUsersRO();
    final Set<Object> usersRW = sourceCCP.getUsersRW();
    final SharingType sharingType = sourceCCP.getSharingType();
    sourceCCP.close(false);

    for (ProjectsManagerData pmd : targetProjects) {
      final CommonClientProject targetCCP =
        psi.openProject(pmd.getOwnerId(), pmd.getProjectName(), true);
      targetCCP.updateUsers(sharingType, usersRO, usersRW);
      targetCCP.close(true);
    }
  }
}
