package csbase.client.applications.projectsmanager.actions;

import java.util.ArrayList;
import java.util.List;

import javax.swing.Action;
import javax.swing.ImageIcon;

import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.applications.ApplicationFrame;
import csbase.client.applications.projectsmanager.ProjectsManager;
import csbase.client.applications.projectsmanager.actions.core.ProjectsManagerAction;
import csbase.client.applications.projectsmanager.dialogs.StatusDialog;
import csbase.client.applications.projectsmanager.models.ProjectSpaceAllocation;
import csbase.client.applications.projectsmanager.models.ProjectsManagerData;
import csbase.client.applications.projectsmanager.models.ProjectsManagerScope;
import csbase.client.applications.projectsmanager.proxy.SetProjectSharingStatsTask;
import csbase.logic.User;
import csbase.logic.ProjectPermissions.SharingType;

/**
 * Ao que altera o status de compartilhamento de projetos para modo "pblico".
 * 
 * @author jnlopes
 * 
 */
public class SetProjectPublicAction extends ProjectsManagerAction {

  /**
   * Projetos que sero tornados pblicos.
   */
  private List<ProjectsManagerData> projectsManagerData;

  /**
   * Modo de acesso (ALL_RO ou ALL_RW).
   */
  private SharingType sharingType;

  /**
   * Construtor
   * 
   * @param projectsManager A aplicao
   * @param projectsManagerData Projetos a serem modificados
   * @param readOnly Boolean que indica se os projetos sero RO ou RW.
   */
  public SetProjectPublicAction(final ProjectsManager projectsManager,
      final List<ProjectsManagerData> projectsManagerData,
      final boolean readOnly) {
    super(projectsManager);
    this.projectsManagerData = projectsManagerData;

    if (readOnly) {
      sharingType = SharingType.ALL_RO;
    } else {
      sharingType = SharingType.ALL_RW;
    }
    setRightName();
  }

  /**
   * Define o nome da ao em funo do modo de acesso.
   */
  private void setRightName() {
    if (sharingType == SharingType.ALL_RO) {
      putValue(Action.NAME, getString("readonly.name"));
    } else {
      putValue(Action.NAME, getString("readwrite.name"));
    }
  }

  /**
   * {@inheritDoc} Mtodo que invoca a remote task que efetivamente altera o
   * status de compartilhamento de projetos para modo "pblico".
   */
  @Override
  public void actionDone() throws Exception {

    if (projectsManagerData == null || projectsManagerData.size() <= 0) {
      return;
    }

    final ProjectsManager projectsManager = getProjectsManager();
    final int confirm = StandardDialogs.showYesNoDialog(projectsManager
        .getApplicationFrame(), getString("warning.title"),
        getString("warning.msg"));

    if (confirm != StandardDialogs.YES_OPTION) {
      return;
    }

    final List<ProjectsManagerData> projects = validateProjects();
    if (projects == null) {
      return;
    }

    final boolean result = runTask(projects);
    if (result) {
      for (ProjectsManagerData pmd : projects) {
        pmd.setSharingType(sharingType);
      }
      projectsManager.refreshProjectsTable();
      projectsManager.refreshInfoPanel();
    } else {
      final String errMsg = getString("set.public.error.message");
      final String errTitle = getString("set.public.error.title");
      StandardDialogs.showErrorDialog(projectsManager.getApplicationFrame(),
          errTitle, errMsg);
    }
  }

  /**
   * Roda e trata a execuo da task.
   * 
   * @param projects Lista de projetos vlidos para a operao.
   * @return Boolean indicando se a operao foi bem sucedida.
   * @throws Exception
   */
  private boolean runTask(List<ProjectsManagerData> projects) throws Exception {

    final String[] projectNames = new String[projects.size()];
    final String[] ownerIds = new String[projects.size()];

    int i = 0;
    for (ProjectsManagerData pmd : projects) {
      projectNames[i] = pmd.getProjectName();
      ownerIds[i] = pmd.getOwnerId();
      i++;
    }

    final ProjectsManager projectsManager = getProjectsManager();
    final SetProjectSharingStatsTask fptt = new SetProjectSharingStatsTask(
        projectsManager, projectNames, ownerIds, ProjectsManagerScope.PUBLIC,
        sharingType);
    final ApplicationFrame frame = projectsManager.getApplicationFrame();
    fptt.execute(frame, getString("title"), getString("message"));

    if (fptt.wasCancelled()) {
      final String err = getString("cancelled.message");
      fptt.showError(err);
      return false;
    }

    if (fptt.getStatus() != true) {
      final Exception exception = fptt.getError();
      throw exception;
    }
    return fptt.getResult();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ImageIcon getStandardImageIcon() {
    return ProjectsManagerScope.PUBLIC.getIcon();
  }

  /**
   * Atualiza lista de projetos sobre os quais a ao ser realizada.
   * 
   * @param projectsManagerData Lista de projetos
   */
  public void setProjectsManagerData(
      final List<ProjectsManagerData> projectsManagerData) {
    this.projectsManagerData = projectsManagerData;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected List<ProjectsManagerData> validateProjects() {

    final List<ProjectsManagerData> prjList = new ArrayList<ProjectsManagerData>();
    prjList.addAll(projectsManagerData);

    final List<ProjectsManagerData> failedProjects = new ArrayList<ProjectsManagerData>();

    final User loggedUser = User.getLoggedUser();
    final Object loggedUserId = loggedUser.getId();

    int i = 0;
    while (i < prjList.size()) {
      final ProjectsManagerData pmd = prjList.get(i);
      final ProjectSpaceAllocation spaceAllocation = pmd
          .getProjectSpaceAllocation();

      final boolean isAdmin = loggedUser.isAdmin();
      final boolean isOwner = pmd.getOwnerId().equals(loggedUserId);

      /*
       * Para que um usurio possa tornar um projeto pblico  preciso que: - O
       * projeto possa ser aberto (no esteja aguardando alocao /
       * desalocao). - O usurio seja o administrador ou o dono do projeto.
       */
      if (spaceAllocation.isOpenable() && (isAdmin || isOwner)) {
        i++;
      } else {
        failedProjects.add(pmd);
        prjList.remove(i);
      }
    }

    final ProjectsManager projectsManager = getProjectsManager();
    if (prjList.size() == 0) {
      final String deniedMessage = getString("project.selection.denied.message")
          + getString("project.requirements.message");
      StatusDialog.showNoneOkDialog(projectsManager, deniedMessage,
          failedProjects, getString("project.selection.failure.message"));
      return null;
    } else if (failedProjects.size() > 0) {
      final String someDeniedMessage = getString("project.selection.some.denied.message")
          + getString("project.requirements.message");
      final int res = StatusDialog.showSomeOkDialog(projectsManager,
          someDeniedMessage, prjList,
          getString("project.selection.some.failure.message"));
      if (res == 0) {
        return null;
      }
    }

    return prjList;
  }

}
