package csbase.client.applications.projectsmanager.dialogs;

import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import tecgraf.javautils.gui.BorderUtil;
import tecgraf.javautils.gui.GBC;
import tecgraf.javautils.gui.GUIUtils;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.applications.ApplicationFrame;
import csbase.client.applications.projectsmanager.ProjectsManager;
import csbase.client.applications.projectsmanager.dialogs.core.ProjectsManagerDialog;
import csbase.client.applications.projectsmanager.models.ProjectsManagerData;
import csbase.client.applications.projectsmanager.panels.ProjectSharingInfoPanel;
import csbase.client.applications.projectsmanager.proxy.AdminSendEmailTask;

/**
 * Dilogo de envio de emails referentes a projetos.
 * 
 * @author jnlopes
 */
public class AdminMailDialog extends ProjectsManagerDialog {
  /**
   * Lista de projetos selecionados.
   */
  private List<ProjectsManagerData> projectsManagerData;

  /**
   * Campo de texto onde o "Assunto" deve ser preenchido.
   */
  private final JTextField subjectField = new JTextField();

  /**
   * rea de texto onde o mail deve ser redigido.
   */
  private final JTextArea mailArea = new JTextArea();

  /**
   * Construtor
   * 
   * @param projectsManager A aplicao.
   * @param projectsManagerData Lista de projetos selecionados.
   */
  public AdminMailDialog(final ProjectsManager projectsManager,
      final List<ProjectsManagerData> projectsManagerData) {
    super(projectsManager);
    this.projectsManagerData = projectsManagerData;

    buildPanel();

    setTitle(getString("title"));
    pack();
    center();
  }

  /**
   * Monta o painel com os botes.
   * 
   * @return o painel.
   */
  private JPanel buildButtonsPanel() {
    final JButton sendButton = new JButton();
    final JButton cancelButton = new JButton();

    final AdminMailDialog amd = this;
    sendButton.setAction(new AbstractAction() {
      /**
       * {@inheritDoc}
       */
      @Override
      public void actionPerformed(ActionEvent e) {
        final ProjectsManager projectsManager = getProjectsManager();

        final Map<String, Set<String>> userProjectMap = generateUserProjectMap();

        final AdminSendEmailTask aset = new AdminSendEmailTask(projectsManager,
            userProjectMap, subjectField.getText(), mailArea.getText());

        final ApplicationFrame appFrame = projectsManager.getApplicationFrame();
        final String taskTitle = getString("running.title");
        final String taskMsg = getString("running.message");
        aset.execute(appFrame, taskTitle, taskMsg);

        if (aset.wasCancelled()) {
          final String err = getString("cancelled.message");
          aset.showError(err);
          return;
        }

        if (aset.getStatus() != true) {
          final Exception exception = aset.getError();
          final String err = getString("send.mail.error.message");
          final StringBuilder strBuilder = new StringBuilder(err);
          strBuilder.append("\n");
          strBuilder.append(exception.getMessage());
          StandardDialogs.showErrorDialog(appFrame, "", strBuilder.toString());
        }

        final String[] ret = aset.getResult();
        if (ret != null) {

          if (ret.length == 0) {
            final String errMsg = getString("user.mail.error");
            StandardDialogs.showErrorDialog(appFrame, "", errMsg);
            return;
          }

          final StringBuilder users = new StringBuilder();
          for (String str : ret) {
            users.append(" - " + str + "\n");
          }
          final String err = getString("failure.sending.mail.message");
          final String errMsg = getString("failure.sending.mail.complement.message");
          final String errTitle = getString("failure.sending.mail.title");
          final StringBuilder msg = new StringBuilder(err);
          msg.append("\n" + users.toString() + "\n" + errMsg);
          StandardDialogs.showErrorDialog(appFrame, errTitle, msg.toString());
          return;
        }
        amd.dispose();
      }

    });
    sendButton.setText(getString("send.text"));

    cancelButton.setAction(new AbstractAction() {
      public void actionPerformed(ActionEvent e) {
        amd.dispose();
      }
    });
    cancelButton.setText(getString("cancel.text"));

    final JComponent[] buttons = { sendButton, cancelButton };
    GUIUtils.matchPreferredSizes(buttons);

    final JPanel buttonsPanel = new JPanel();
    buttonsPanel.add(sendButton);
    buttonsPanel.add(cancelButton);
    return buttonsPanel;
  }

  /**
   * Gera o HashMap contendo a relao (1-N) Usurio - Projetos.
   * 
   * @return HashMap contendo a relao (1-N) Usurio - Projetos.
   */
  private Map<String, Set<String>> generateUserProjectMap() {

    final Map<String, Set<String>> userProjectMap = new HashMap<String, Set<String>>();

    for (ProjectsManagerData pmd : projectsManagerData) {
      Set<String> aux = userProjectMap.get(pmd.getOwnerId());

      if (aux == null) {
        aux = new HashSet<String>();
      }
      aux.add(pmd.getProjectName());
      userProjectMap.put(pmd.getOwnerId(), aux);
    }

    return userProjectMap;
  }

  /**
   * Monta o componente de insero do texto.
   * 
   * @return o componente
   */
  private JScrollPane buildTextPane() {
    mailArea.setRows(5);
    mailArea.setColumns(40);
    mailArea.setLineWrap(true);

    final JScrollPane pane = new JScrollPane(mailArea);
    BorderUtil.setTitledBorder(pane, getString("mail.area.title"));
    return pane;
  }

  /**
   * Monta o componente dos projetos.
   * 
   * @return o componente
   */
  private ProjectSharingInfoPanel buildProjectPanel() {
    final ProjectsManager projectsManager = getProjectsManager();
    final ProjectSharingInfoPanel projectsPanel = new ProjectSharingInfoPanel(
        projectsManager, projectsManagerData, false, true);
    BorderUtil.setTitledBorder(projectsPanel, getString("projects.title"));
    return projectsPanel;
  }

  /**
   * Monta o painel.
   */
  private void buildPanel() {
    final JPanel buttonsPanel = buildButtonsPanel();
    final JPanel subjectPanel = buildSubjectPanel();
    final JScrollPane pane = buildTextPane();
    final ProjectSharingInfoPanel projectsPanel = buildProjectPanel();

    setLayout(new GridBagLayout());
    add(projectsPanel, new GBC(0, 0).horizontal());
    add(subjectPanel, new GBC(0, 1).horizontal());
    add(pane, new GBC(0, 2).both());
    add(buttonsPanel, new GBC(0, 3).horizontal());
  }

  /**
   * Monta painel referente ao campo de "Assunto".
   * 
   * @return O painel referenter ao campo de "Assunto".
   */
  private JPanel buildSubjectPanel() {

    final JPanel panel = new JPanel(new GridBagLayout());
    panel.add(subjectField, new GBC(0, 0).horizontal());
    BorderUtil.setTitledBorder(panel, getString("subject.title"));
    return panel;
  }
}
