package csbase.client.applications.projectsmanager.dialogs;

import java.awt.FlowLayout;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.text.MessageFormat;
import java.util.Set;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.Document;

import tecgraf.javautils.gui.BorderUtil;
import tecgraf.javautils.gui.GBC;
import tecgraf.javautils.gui.GUIUtils;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.applications.ApplicationFrame;
import csbase.client.applications.ApplicationImages;
import csbase.client.applications.projectsmanager.ProjectsManager;
import csbase.client.applications.projectsmanager.dialogs.core.ProjectsManagerDialog;
import csbase.client.applications.projectsmanager.proxy.CreateProjectRemoteTask;
import csbase.client.applications.projectsmanager.usersearch.UserDialog;
import csbase.client.util.ClientUtilities;
import csbase.client.util.NumericTextField;
import csbase.logic.CommonClientProject;
import csbase.logic.CommonProjectInfo;
import csbase.logic.User;
import csbase.logic.UserProjectInfo;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.ProjectServiceInterface;

/**
 * Dilogo para criao de projetos atravs da aplicao ProjectsManager.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class ProjectCreationDialog extends ProjectsManagerDialog {

  /** O campo que indica se o projeto quer ou no uma reserva de rea. */
  final private JCheckBox lockingOption =
    new JCheckBox(getString("locking.checkbox"));

  /** O campo com o tamanho da rea reservada. */
  final private NumericTextField sizeField = new NumericTextField(20);

  /**
   * Campo com nome do dono do projeto.
   */
  final private JTextField projectOwnerField =
    new JTextField(User.getLoggedUser().getLogin());

  /**
   * Campo de nome do projeto.
   */
  final private JTextField nameField = new JTextField();

  /**
   * Campo de descrio do projeto.
   */
  final private JTextField descrField = new JTextField();

  /**
   * Boto de confirmao para criao de projeto.
   */
  final private JButton defaultButton =
    new JButton(getString("creation.button"));

  /**
   * Projeto a ser criado.
   */
  private CommonClientProject project;

  /**
   * Instancia a viso de informaes de projeto usada para criao de projeto.
   * O dilogo  criado e apresentado.
   * 
   * @param projectsManager A aplicao
   * 
   */
  private ProjectCreationDialog(final ProjectsManager projectsManager) {
    super(projectsManager);
    setModalityType(ModalityType.APPLICATION_MODAL);
    clearDialog();
    buildDialog();
    this.pack();
    this.center();
    this.setVisible(true);
  }

  /**
   * Mostra um dilogo para que o usurio crie um projeto. O usurio informa os
   * dados do projeto. Caso a operao seja confirmada, o servidor  requisitado
   * para executar o servio de criao do projeto.
   * 
   * @param projectsManager A aplicao
   * 
   * @return O projeto criado.
   */
  public static CommonClientProject createProject(
    final ProjectsManager projectsManager) {
    final CommonClientProject ccp =
      new ProjectCreationDialog(projectsManager).getProject();
    return ccp;
  }

  /**
   * Cria um painel com opes adicionais ou no, em funo do usurio e do
   * sistema sendo utilizado.
   * 
   * @return O painel recm-criado.
   */
  private JPanel makeAdditionalPanel() {

    final ProjectsManager projectsManager = getProjectsManager();
    final boolean hasLockPanel = projectsManager.hasAreaReserved();

    final JPanel lockingAreaPanel =
      hasLockPanel ? makeLockingAreaPanel() : null;

    final JPanel panel;
    final User user = User.getLoggedUser();
    if (user.isAdmin()) {
      panel = new JPanel(new GridBagLayout());
      panel.add(makeProjectOwnerPanel(), new GBC(0, 0).horizontal().insets(11,
        11, 11, 11));
      if (lockingAreaPanel != null) {
        panel.add(lockingAreaPanel, new GBC(0, 1).horizontal().insets(11, 11,
          11, 11));
      }
    }
    else {
      if (lockingAreaPanel != null) {
        JComponent[][] components = { { lockingAreaPanel }, };
        panel = GUIUtils.createBasicGridPanel(components);
      }
      else {
        panel = null;
      }
    }
    return panel;
  }

  /**
   * Cria um painel com as opes de reserva de rea para o projeto.
   * 
   * @return O painel recm-criado.
   */
  private JPanel makeLockingAreaPanel() {
    // Check-box com a opo de reservar ou no uma rea de projeto
    lockingOption.setSelected(false);
    // Tamanho da rea a ser reservada
    final JLabel sizeLabel = new JLabel(getString("locking.label"));
    sizeLabel.setEnabled(false);
    sizeField.setEnabled(false);
    final JLabel gbLabel = new JLabel(getString("gb.label"));
    gbLabel.setEnabled(false);
    final JLabel note = new JLabel(getString("locking.message.label"));
    note.setEnabled(false);
    // O tamnho somente  habilitado se a opo de reserva for selecionada
    lockingOption.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        final boolean condition = e.getStateChange() == ItemEvent.SELECTED;
        sizeLabel.setEnabled(condition);
        sizeField.setEnabled(condition);
        gbLabel.setEnabled(condition);
        note.setEnabled(condition);
      }
    });

    final JComponent[][] components =
      { { lockingOption }, { sizeLabel, sizeField, gbLabel }, { null, note } };
    final JPanel panel = GUIUtils.createBasicGridPanel(components);
    panel.setBorder(BorderFactory
      .createTitledBorder(getString("locking.title")));
    return panel;
  }

  /**
   * Cria um painel com as opes de seleo do usurio dono do projeto
   * (disponvel apenas ao administrador).
   * 
   * @return O painel recm-criado.
   */
  private JPanel makeProjectOwnerPanel() {

    projectOwnerField.setEditable(false);

    final JButton searchButton = new JButton(new AbstractAction() {

      public void actionPerformed(ActionEvent e) {

        final ProjectsManager projectsManager = getProjectsManager();
        final ApplicationFrame appFrame = projectsManager.getApplicationFrame();
        final String title = getString("owner.selection.title");
        try {
          final Set<Object> ids =
            UserDialog.showUserSelectionDialog(appFrame, projectsManager
              .getUsers(), false, title);

          if (ids != null) {
            final Object userId = ids.toArray()[0];
            projectOwnerField.setText((String) userId);
          }
        }
        catch (Exception e1) {
          e1.printStackTrace();
          StandardDialogs.showErrorDialog(appFrame, "Error",
            getString("obtaining.user.error.text"));
          return;
        }

      }
    });

    searchButton.setIcon(ApplicationImages.ICON_FIND_USER_16);
    searchButton.setToolTipText(getString("search.owner.tooltip"));

    GUIUtils.trimImageButton(searchButton);

    final JLabel label = new JLabel(getString("user.login.label"));
    final JComponent[][] components =
      { { label, projectOwnerField, searchButton } };
    final JPanel panel = GUIUtils.createBasicGridPanel(components);
    panel.setBorder(BorderFactory
      .createTitledBorder(getString("project.owner.text")));
    return panel;
  }

  /**
   * Limpa os campos do dilogo e atribui os listeners a esses campos.
   */
  private void clearDialog() {
    // Inicializa os campos
    nameField.setText("");
    descrField.setText("");
    /*
     * Adiciona um listener ao campo de nome para que, quando um nome for
     * digitado, o boto principal seja habilitado.
     */
    final Document document = nameField.getDocument();
    document.addDocumentListener(new DocumentListener() {
      @Override
      public void changedUpdate(DocumentEvent e) {
        // vazio
      }

      @Override
      public void insertUpdate(DocumentEvent e) {
        setConfirmButtonState();
      }

      @Override
      public void removeUpdate(DocumentEvent e) {
        setConfirmButtonState();
      }

      private void setConfirmButtonState() {
        String name = nameField.getText().trim();
        if (name.isEmpty()) {
          defaultButton.setEnabled(false);
        }
        else {
          defaultButton.setEnabled(true);
        }
      }
    });
  }

  /**
   * Monta o dilogo de criao de projetos.
   */
  private void buildDialog() {
    nameField.setColumns(30);
    descrField.setColumns(30);

    final JPanel firstPanel = makeCommonPanel();
    final JPanel secondPanel = makeAdditionalPanel();
    final JPanel thirdPanel = makeButtonPanel();
    this.setLayout(new GridBagLayout());

    this.add(firstPanel, new GBC(0, 0).horizontal().insets(11, 11, 0, 11));

    int i = 1;
    int gapToNextPanel = 11;
    if (secondPanel != null) {
      this.add(secondPanel, new GBC(0, i++).horizontal());
      gapToNextPanel = 0;
    }
    this.add(thirdPanel, new GBC(0, i++).horizontal().insets(gapToNextPanel,
      11, 11, 11));
  }

  /**
   * Cria um painel com os botes que vo ser usados no dilogo de criao de
   * projeto. Na criao de um projeto, o usurio pode acionar o boto de
   * criao, que confirma a operao, ou pode acionar o boto de cancelamento,
   * que desiste da operao. O boto de criao somente  habilitado quando o
   * usurio tiver informado um nome do projeto.
   * 
   * @return O painel recm-criado.
   */
  private JPanel makeButtonPanel() {

    final ProjectCreationDialog dialog = this;
    final JPanel panel = new JPanel(new FlowLayout(FlowLayout.CENTER, 11, 0));

    // Boto de criao de projeto
    panel.add(defaultButton);
    final JButton cancelButton =
      (JButton) panel.add(new JButton(getString("cancel.button")));
    defaultButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        handleAction();
      }
    });
    defaultButton.setEnabled(false);
    // Boto de cancelamento da operao de criao.
    cancelButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        dialog.dispose();
      }
    });
    ClientUtilities.adjustEqualSizes(defaultButton, cancelButton);
    return panel;
  }

  /**
   * Monta o painel para entrada de informaes necessrias a todos os usurios
   * e a todos os sistemas (nome e descrio do projeto).
   * 
   * @return O painel montado.
   */
  private JPanel makeCommonPanel() {

    final JComponent[][] components =
      { { new JLabel(getString("project.name.label")), nameField },
          { new JLabel(getString("project.descr.label")), descrField }, };

    final JPanel panel = GUIUtils.createBasicGridPanel(components);
    BorderUtil.setTitledBorder(panel, getString("common.title"));
    return panel;
  }

  /**
   * Informa o ttulo que vai ser utilizado na janela de criao de projeto.
   * 
   * @return O ttulo da janela.
   */
  @Override
  public String getTitle() {
    return getString("creation.title");
  }

  /**
   * Executa a ao de criao de um projeto. O servio  chamado no servidor.
   * Caso algum problema ocorra, o usurio  informado com uma mensagem em uma
   * outra janela de dilogo.
   */
  private void handleAction() {
    if (lockingOption != null && lockingOption.isSelected()) {
      createLockedProject();
    }
    else {
      createNonLockedProject();
    }
  }

  /**
   * Cria um projeto e o torna pronto para ser usado no desktop.
   */
  private void createNonLockedProject() {

    final Object userId;

    final User loggedUser = User.getLoggedUser();
    /*
     * Caso o usurio seja o administrador, ento o dono do projeto sendo criado
     * pode ser outro usurio (usurios comuns no podem criar projetos no nome
     * de outros).
     */
    if (loggedUser.isAdmin()) {
      userId = projectOwnerField.getText();
    }
    else {
      userId = loggedUser.getId();
    }

    final ProjectCreationDialog dialog = this;

    final String name = nameField.getText();
    if (!ClientUtilities.isValidFileName(name)) {
      StandardDialogs.showErrorDialog(dialog, getTitle(),
        getString("character.error.message"));
      return;
    }
    final String description = descrField.getText();
    final CreateProjectRemoteTask<CommonClientProject> task =
      new CreateProjectRemoteTask<CommonClientProject>(getProjectsManager()) {
        @Override
        public void performTask() throws Exception {

          final ProjectServiceInterface psi =
            ClientRemoteLocator.projectService;

          final CommonProjectInfo cpi = new CommonProjectInfo();
          cpi.name = name;
          cpi.userId = userId;
          cpi.description = description;

          final CommonClientProject prj = psi.createProject(cpi);
          setResult(prj);
          ClientRemoteLocator.server.setServerSessionProperty(
            ProjectServiceInterface.CURRENT_PROJECT_SESSION_PROPERTY,
            new UserProjectInfo(userId, name));
        }
      };
    if (task.execute(dialog, getTitle(), getString("waiting.create.title"))) {
      project = task.getResult();
      dialog.dispose();
    }
  }

  /**
   * Cria um projeto bloqueado.
   */
  private void createLockedProject() {

    final Object userId;
    final User user = User.getLoggedUser();
    /*
     * Caso o usurio seja o administrador, ento o dono do projeto sendo criado
     * pode ser outro usurio (usurios comuns no podem criar projetos no nome
     * de outros).
     */
    if (user.isAdmin()) {
      userId = projectOwnerField.getText();
    }
    else {
      userId = User.getLoggedUser().getId();
    }

    final ProjectCreationDialog dialog = this;
    final String name = nameField.getText();
    if (!ClientUtilities.isValidFileName(name)) {
      final String err = getString("character.error.message");
      StandardDialogs.showErrorDialog(dialog, getTitle(), err);
      return;
    }
    final String description = descrField.getText();
    final int size;
    try {
      if (lockingOption.isSelected()) {
        final String txt = sizeField.getText();
        size = Integer.parseInt(txt);
      }
      else {
        size = 0;
      }
    }
    catch (Exception e) {
      final String err = getString("locksize.error.message");
      StandardDialogs.showErrorDialog(dialog, getTitle(), err);
      return;
    }

    final CreateProjectRemoteTask<Void> task =
      new CreateProjectRemoteTask<Void>(getProjectsManager()) {
        @Override
        public void performTask() throws Exception {

          final ProjectServiceInterface psi =
            ClientRemoteLocator.projectService;

          final CommonProjectInfo cpi = new CommonProjectInfo();
          cpi.name = name;
          cpi.userId = userId;
          cpi.description = description;

          psi.createProjectWithAllocation(cpi, size);
        }
      };
    if (task.execute(dialog, getTitle(), getString("waiting.create.message"))) {
      /* Informa que o projeto foi criado, mas est bloqueado */
      final ProjectsManager projectsManager = getProjectsManager();
      final ApplicationFrame appFrame = projectsManager.getApplicationFrame();
      final String message = getString("project.creation.withlock.message");
      StandardDialogs.showInfoDialog(appFrame, getTitle(), MessageFormat
        .format(message, new Object[] { name }));
      dialog.dispose();
    }
  }

  /**
   * Retorna o projeto criado.
   * 
   * @return O projeto criado.
   */
  public CommonClientProject getProject() {
    return project;
  }
}
