package csbase.client.applications.projectsmanager.panels;

import java.awt.GridBagLayout;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.RowSorter;
import javax.swing.RowSorter.SortKey;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;

import tecgraf.javautils.gui.GBC;
import tecgraf.javautils.gui.table.SortableTable;
import csbase.client.applications.projectsmanager.ProjectsManager;
import csbase.client.applications.projectsmanager.models.ProjectsManagerData;
import csbase.client.applications.projectsmanager.panels.renderers.ServerLocationRenderer;
import csbase.client.applications.projectsmanager.panels.renderers.TableAllocationRenderer;
import csbase.client.applications.projectsmanager.panels.renderers.TableDateRenderer;
import csbase.client.applications.projectsmanager.panels.renderers.TableOccupationRenderer;
import csbase.client.applications.projectsmanager.panels.renderers.TableScopeRenderer;
import csbase.client.applications.projectsmanager.panels.renderers.TableSpaceAllocationRenderer;

/**
 * Painel que contem a tabela que apresenta os projetos existentes.
 * 
 * @author jnlopes
 * 
 */
public class TablePanel extends ProjectsManagerPanel {

  /**
   * Tabela que apresenta os projetos existentes.
   */
  final private SortableTable projectsTable = new SortableTable();

  /**
   * ScrollPane da tabela que apresenta os projetos existentes.
   */
  final private JScrollPane projectsTablePane = new JScrollPane(
      this.projectsTable);

  /**
   * Lista dos projetos existentes.
   */
  final private List<ProjectsManagerData> allProjects = new ArrayList<ProjectsManagerData>();

  /**
   * Lista dos projetos filtrados.
   */
  private List<ProjectsManagerData> visibleProjects;

  /**
   * Construtor
   * 
   * @param projectsManager A aplicao
   */
  public TablePanel(final ProjectsManager projectsManager) {
    super(projectsManager);

    initComponents();
    buildPanel();

    addSelectionListeners();
    defineColumns();
    // Clinio: Comentado para futura remoo? [CSBASE-1056]
    // addMouseListeners();

    this.setVisible(true);
    this.validate();
    this.repaint();
  }

  /**
   * Adiciona listener de seleo de campos da tabela.
   */
  private void addSelectionListeners() {
    projectsTable
        .setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);

    projectsTable.getSelectionModel().addListSelectionListener(
        new ListSelectionListener() {
          /**
           * {@inheritDoc}
           */
          public void valueChanged(ListSelectionEvent e) {
            if (e.getValueIsAdjusting()) {
              /*
               * este evento  intermedirio e deve ser descartado
               */
              return;
            }
            final ProjectsManager projectsManager = getProjectsManager();
            projectsManager.setProjectsList(TablePanel.this
                .getSelectedProjects());
          }
        });
  }

  // /**
  // * Adiciona MouseListeners.
  // */
  // private void addMouseListeners() {
  //
  // this.projectsTable
  // .addMouseListener(new ProjectsTableOccupationMouseAdapter(
  // getProjectsManager(), projectsTable));
  // this.projectsTable
  // .addMouseListener(new ProjectsTableAllocationMouseAdapter(
  // getProjectsManager(), projectsTable));
  // this.projectsTable.addMouseListener(new ProjectsTableSharingMouseAdapter(
  // getProjectsManager(), projectsTable));
  // }

  /**
   * Define os renderers e propriedades gerais das colunas da tabela de
   * projetos.
   */
  private void defineColumns() {
    projectsTable.setAutoResizeMode(JTable.AUTO_RESIZE_LAST_COLUMN);

    final TableColumnModel columnModel = projectsTable.getColumnModel();
    final TableColumn column0 = columnModel.getColumn(0);
    column0.setHeaderValue(getString("project.access.tablecolumn"));
    column0.setCellRenderer(new TableScopeRenderer());
    column0.setPreferredWidth(50);

    final ProjectsManager projectsManager = getProjectsManager();
    final boolean areaReserved = projectsManager.hasAreaReserved();

    int idx = 1;
    if (areaReserved) {
      final TableColumn column1 = columnModel.getColumn(idx);
      column1.setHeaderValue(getString("project.allocation.tablecolumn"));
      column1.setCellRenderer(new TableSpaceAllocationRenderer());
      column1.setPreferredWidth(50);
      idx++;

      final TableColumn column2 = columnModel.getColumn(idx);
      column2.setHeaderValue(getString("allocated.space.tablecolumn"));
      column2.setCellRenderer(new TableAllocationRenderer());
      column2.setPreferredWidth(90);
      idx++;
    }

    final TableColumn column3 = columnModel.getColumn(idx);
    column3.setHeaderValue(getString("project.name.tablecolumn"));
    column3.setPreferredWidth(170);
    idx++;

    final TableColumn column4 = columnModel.getColumn(idx);
    column4.setHeaderValue(getString("owner.name.tablecolumn"));
    column4.setPreferredWidth(60);
    idx++;

    final TableColumn column5 = columnModel.getColumn(idx);
    column5.setHeaderValue(getString("occupied.space.tablecolumn"));
    column5.setCellRenderer(new TableOccupationRenderer());
    column5.setPreferredWidth(90);
    idx++;

    final TableColumn column6 = columnModel.getColumn(idx);
    column6.setHeaderValue(getString("creation.date.tablecolumn"));
    column6.setCellRenderer(new TableDateRenderer());
    column6.setPreferredWidth(160);
    idx++;

    final TableColumn column7 = columnModel.getColumn(idx);
    column7.setHeaderValue(getString("last.used.date.tablecolumn"));
    column7.setCellRenderer(new TableDateRenderer());
    column7.setPreferredWidth(160);
    idx++;

    if (areaReserved) {
      final TableColumn column8 = columnModel.getColumn(idx);
      column8.setHeaderValue(getString("server.location.tablecolumn"));
      column8.setCellRenderer(new ServerLocationRenderer());
      column8.setPreferredWidth(250);
    }
  }

  /**
   * Pega todos os projetos selecionados.
   * 
   * @return array dos projetos selecionados.
   */
  public List<ProjectsManagerData> getSelectedProjects() {
    final int[] rows = projectsTable.getSelectedRows();

    final List<ProjectsManagerData> projectsManagerData = new ArrayList<ProjectsManagerData>();
    for (int i = 0; i < rows.length; i++) {
      projectsManagerData.add(visibleProjects.get(projectsTable
          .convertRowIndexToModel(rows[i])));
    }

    return projectsManagerData;
  }

  /**
   * Inicia os componentes.
   */
  private void initComponents() {
    visibleProjects = new ArrayList<ProjectsManagerData>();

    projectsTable.setNoSortStateEnabled(true);
    final SimpleTableModel stm = new SimpleTableModel(this);
    projectsTable.setModel(stm);
    projectsTable.setUpdateSelectionOnSort(true);

    final ProjectsManager projectsManager = getProjectsManager();

    if (!projectsManager.hasAreaReserved()) {
      final TableColumnModel model = projectsTable.getColumnModel();
      final TableColumn column1 = model.getColumn(1);
      final TableColumn column2 = model.getColumn(2);
      final TableColumn column8 = model.getColumn(8);
      model.removeColumn(column1);
      model.removeColumn(column2);
      model.removeColumn(column8);
    }
  }

  /**
   * Monta o painel.
   */
  private void buildPanel() {

    setLayout(new GridBagLayout());
    add(projectsTablePane, new GBC(0, 0).both());
  }

  /**
   * Adiciona um novo projeto  tabela.
   * 
   * @param prj projeto
   * @param isVisible flag que indica se o projeto deve ser adicionado tambm 
   *          lista de projetos visveis
   * @param refreshTable flag que indica se a tabela deve ser atualizada
   */
  public void addProject(final ProjectsManagerData prj,
      final boolean isVisible, final boolean refreshTable) {
    if (prj == null) {
      return;
    }

    final RowSorter<? extends TableModel> rs = projectsTable.getRowSorter();
    final List<? extends SortKey> l = rs.getSortKeys();
    rs.setSortKeys(null);

    allProjects.add(prj);
    if (isVisible) {
      visibleProjects.add(prj);
    }

    rs.setSortKeys(l);

    if (refreshTable) {
      refreshTable();
    }
  }

  /**
   * Substitui projeto.
   * 
   * @param project Dados de um projeto que substituiro os dados atuais.
   */
  public void replaceProjectData(final ProjectsManagerData project) {

    if (project == null) {
      return;
    }

    ProjectsManagerData pmd = getProject(project);
    if (pmd == null) {
      return;
    }

    int idx = visibleProjects.indexOf(pmd);
    if (idx != -1) {
      visibleProjects.set(idx, project);
    }

    idx = allProjects.indexOf(pmd);
    if (idx != -1) {
      allProjects.set(idx, project);
    }

    refreshTable();
  }

  /**
   * Obtm o projeto cujo nome e dono correspondem s informaes passadas. O
   * projeto  buscado na lista de todos os projetos disponveis, e no apenas
   * dos projetos visveis.
   * 
   * @param prjName nome do projeto
   * @param ownerID dono do projeto
   * @return O objeto ProjectsManagerData encontrado, ou null.
   */
  public ProjectsManagerData getProject(final String prjName,
      final Object ownerID) {

    if (allProjects == null) {
      return null;
    }

    for (ProjectsManagerData pmd : allProjects) {
      final String projectName = pmd.getProjectName();
      final String ownerId = pmd.getOwnerId();
      if (projectName.equals(prjName) && ownerId.equals(ownerID)) {
        return pmd;
      }
    }

    return null;
  }

  /**
   * Procura na lista de todos os projetos por um projeto com mesmo nome e dono
   * que outro. O projeto  procurado na lista que contm todos os projetos, no
   * apenas os projetos visveis.
   * 
   * @param prj projeto a ser procurado
   * @return projeto da lista cujo nome e dono so os mesmos que os do projeto
   *         especificado, ou null caso tal projeto no exista na lista
   */
  public ProjectsManagerData getProject(final ProjectsManagerData prj) {
    return getProject(prj.getProjectName(), prj.getOwnerId());
  }

  /**
   * Remove um projeto das listas (todos e apenas visveis) a partir do seu nome
   * e do seu dono.
   * 
   * @param name nome do projeto.
   * @param ownerID ID do dono do projeto
   */
  public void removeProject(final String name, final Object ownerID) {
    final ProjectsManagerData pmd = getProject(name, ownerID);

    if (pmd == null) {
      return;
    }

    final RowSorter<? extends TableModel> rs = projectsTable.getRowSorter();
    final List<? extends SortKey> l = rs.getSortKeys();
    rs.setSortKeys(null);

    allProjects.remove(pmd);
    visibleProjects.remove(pmd);
    rs.setSortKeys(l);
  }

  /**
   * Esvazia conteudo da tabela.
   */
  final public void emptyTableData() {
    projectsTable.clearSelection();
    final ProjectsManager projectsManager = getProjectsManager();
    projectsManager.clearInfoFields();
    allProjects.clear();
    visibleProjects.clear();
    refreshTable();
  }

  /**
   * Realiza refresh na tabela.
   */
  public void refreshTable() {
    projectsTable.revalidate();
    projectsTable.repaint();
  }

  /**
   * Ajusta a preciso de um double para que tenha apenas 3 casas decimais.
   * 
   * @param num O nmero a ser ajustado.
   * @return public double adjustPrecision(double num) { num =
   *         (double)Math.round(num * 1000) / 1000; return num; }
   */

  /**
   * @return lista de todos os projetos
   */
  final public List<ProjectsManagerData> getAllProjects() {
    return allProjects;
  }

  /**
   * @param projects lista de projetos visveis
   */
  final public void setVisibleProjects(final List<ProjectsManagerData> projects) {
    this.visibleProjects = projects;
  }

  /**
   * Retorna a lista de projetos filtrados.
   * 
   * @return lista de projetos filtrados. ATENO: retorna a prpria lista, e
   *         no uma cpia.
   */
  final public List<ProjectsManagerData> getVisibleProjects() {
    return visibleProjects;
  }

  /**
   * Atualiza o valor de ocupao de um projeto em todas as listas onde este
   * projeto se encontra.
   * 
   * @param projectsManagerData O projeto cuja ocupao foi alterada.
   */
  public void updateProjectOccupiedSpace(
      final ProjectsManagerData projectsManagerData) {
    final String projectName = projectsManagerData.getProjectName();
    final String projectOwner = projectsManagerData.getOwnerId();

    for (ProjectsManagerData pmd : allProjects) {
      final String pName = pmd.getProjectName();
      final String oId = pmd.getOwnerId();
      if (pName.equals(projectName) && oId.equals(projectOwner)) {
        pmd.setOccupiedSpace(projectsManagerData.getOccupiedSpace());
      }
    }
  }

  /**
   * Consulta a tabela
   * 
   * @return a tabela
   */
  final public SortableTable getProjectsTable() {
    return projectsTable;
  }

  /**
   * Desmarca as linhas selecionadas da tabela.
   */
  public void clearSelection() {
    projectsTable.clearSelection();
  }

  /**
   * Remove os projetos localmente, pois eles j foram excludos do sistema.
   * 
   * @param projectsManagerData Lista de projetos a serem removidos.
   */
  public void removeProjects(final List<ProjectsManagerData> projectsManagerData) {
    for (ProjectsManagerData pmd : projectsManagerData) {
      removeProject(pmd.getProjectName(), pmd.getOwnerId());
    }
  }

  /**
   * Retorna o objeto responsvel pela ordenao da tabela de projetos.
   * 
   * @return O objeto responsvel pela ordenao da tabela de projetos.
   */
  public RowSorter<? extends TableModel> getTableRowSorter() {
    final RowSorter<? extends TableModel> rs = projectsTable.getRowSorter();
    return rs;
  }
}
