package csbase.client.applications.projectsmanager.proxy;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import csbase.client.applications.projectsmanager.ProjectsManager;
import csbase.client.applications.projectsmanager.models.ProjectSpaceAllocation;
import csbase.client.applications.projectsmanager.models.ProjectsInfoData;
import csbase.client.applications.projectsmanager.models.ProjectsManagerData;
import csbase.client.applications.projectsmanager.models.ProjectsManagerScope;
import csbase.client.applications.projectsmanager.proxy.core.ProjectsManagerTask;
import csbase.exception.BugException;
import csbase.exception.ServiceFailureException;
import csbase.logic.CommonClientProject;
import csbase.logic.ProjectAdminInfo;
import csbase.logic.User;
import csbase.logic.UserProjectInfo;
import csbase.logic.ProjectPermissions.SharingType;
import csbase.remote.AdministrationServiceInterface;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.DiskUsageServiceInterface;
import csbase.remote.ProjectServiceInterface;

/**
 * Tarefa que preenche a tabela com os dados dos projetos existentes no
 * servidor.
 * 
 * @author jnlopes
 */
public class FillProjectsTableAdminTask extends
  ProjectsManagerTask<ProjectsInfoData> {

  /**
   * Indica se espao ocupado pelos projetos deve ser carregado ou no.
   */
  final private boolean loadOccupiedSpace;

  /**
   * Projetos que no puderam ser abertos pois o arquivo .csbase_project_info
   * no existe ou est corrompido.
   */
  final private Set<String> failedProjectNames = new HashSet<String>();

  /**
   * Projetos que esto aguardando alocao / desalocao de rea.
   */
  final private HashSet<String> waitingAreaAction = new HashSet<String>();

  /**
   * Construtor
   * 
   * @param projectsManager A aplicao
   * @param loadOccupiedSpace boolean que indica se espao ocupado pelos
   *        projetos deve ser carregado ou no.
   */
  public FillProjectsTableAdminTask(final ProjectsManager projectsManager,
    final boolean loadOccupiedSpace) {
    super(projectsManager);
    this.loadOccupiedSpace = loadOccupiedSpace;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void performTask() throws Exception {
    final List<ProjectsManagerData> projectsData = getProjectsData();

    final ProjectsInfoData pid =
      new ProjectsInfoData(projectsData, failedProjectNames);
    setResult(pid);
  }

  /**
   * Mtodo que efetivamente obtm os dados dos projetos.
   * 
   * @return vetor de dados de projeto.
   * @throws Exception
   */
  private List<ProjectsManagerData> getProjectsData() throws Exception {
    final List<ProjectsManagerData> projectList =
      new ArrayList<ProjectsManagerData>();

    getProjectsWithAllocation(projectList);
    getProjectsWithoutAllocation(projectList);

    return projectList;
  }

  /**
   * Pega todos os projetos que no tem alocao.
   * 
   * @param projectList Lista de projetos a ser atualizada.
   * @throws RemoteException
   * @throws Exception
   */
  private void getProjectsWithoutAllocation(
    final List<ProjectsManagerData> projectList) throws RemoteException,
    Exception {

    final ProjectServiceInterface psi = ClientRemoteLocator.projectService;
    final AdministrationServiceInterface asi =
      ClientRemoteLocator.administrationService;
    final DiskUsageServiceInterface dus = ClientRemoteLocator.diskUsageService;

    /*
     * Agora pegamos os projetos que no tem nenhuma alocao.
     */
    final List<User> users = asi.getAllUsers();

    Set<UserProjectInfo> userProjectsInfo;
    UserProjectInfo upi;
    Iterator<UserProjectInfo> iter;

    for (User tmp : users) {
      userProjectsInfo = psi.getAllProjectsInfo(tmp.getId());
      iter = userProjectsInfo.iterator();
      while (iter.hasNext()) {
        upi = iter.next();

        if (this.waitingAreaAction.contains(upi.getProjectName())) {
          continue;
        }

        /*
         * Se usuario corrente nao for dono do projeto corrente, ignora.
         */
        if (!upi.getUserId().equals(tmp.getId())) {
          continue;
        }

        final CommonClientProject ccp;
        try {
          ccp = psi.openProject(tmp.getId(), upi.getProjectName(), false);
        }
        catch (ServiceFailureException sfe) {
          /*
           * Execuo cai nesta exceo quando tentamos abrir um projeto que
           * est aguardando reserva / desalocao de rea.
           */
          continue;
        }
        catch (BugException be) {
          /*
           * Esta exceo  lanada quando o arquivo .csbase_project_info do
           * projeto que estamos tentando abrir no existe ou est corrompido. A
           * execuo no deve parar por causa disto, apenas uma mensagem de
           * erro deve ser exibida.
           */
          failedProjectNames.add(upi.getProjectName());
          continue;
        }

        final long allocatedSpace = ccp.getLockingAreaSize();

        /*
         * Se o projeto nao tem alocao, ento ele no foi lido pelo loop
         * anterior.
         */
        if (ProjectSpaceAllocation.getItemFromSpace(allocatedSpace) == ProjectSpaceAllocation.NO_ALLOCATION) {

          final ProjectsManagerData pmd =
            new ProjectsManagerData(ccp.getName(), (String) ccp.getUserId());

          pmd.setOwnerName(User.getName(ccp.getUserId()));
          pmd.setProjectSpaceAllocation(ProjectSpaceAllocation.NO_ALLOCATION);
          pmd.setProjectDescription(ccp.getDescription());
          pmd.setCreationDate(ccp.getCreationDate());
          pmd.setLocationInServer(ccp.getLocationInServer());
          pmd.setModificationDate(ccp.getLastModificationDate());

          if (loadOccupiedSpace) {
            pmd.setOccupiedSpace(dus.getUsedSpaceForProjectMb(ccp.getUserId(),
              ccp.getName()));
          }
          else {
            pmd.setOccupiedSpace(-1);
          }
          pmd.setSharingType(ccp.getSharingType());
          pmd.setUsers(ccp.getUsersRO(), ccp.getUsersRW());
          pmd.setOwner(true);
          ccp.close(false);

          projectList.add(pmd);
        }
        else if (ProjectSpaceAllocation.getItemFromSpace(allocatedSpace) != ProjectSpaceAllocation.ALLOCATED) {
          /*
           * Esta condio  atingida quando um projeto possui espao alocado no
           * entanto seu estado indica que ele no possui alocao. Esta
           * situao pode indicar que o arquivo locked_projects.csbase no est
           * sendo atualizado corretamente.
           */
          failedProjectNames.add(upi.getProjectName());
        }
      }
    }

  }

  /**
   * Pega todos os projetos que esto passando por algum estado de alocao
   * (esperando alocao, alocado, esperando desalocao).
   * 
   * @param projectList Lista de projetos a ser atualizada.
   * @throws RemoteException
   * @throws Exception
   */
  private void getProjectsWithAllocation(
    final List<ProjectsManagerData> projectList) throws RemoteException,
    Exception {

    final ProjectServiceInterface psi = ClientRemoteLocator.projectService;
    final DiskUsageServiceInterface dus = ClientRemoteLocator.diskUsageService;

    final ProjectAdminInfo[] allInfo = ProjectAdminInfo.getAll();

    if (allInfo == null) {
      return;
    }

    for (int i = 0; i < allInfo.length; i++) {

      final ProjectsManagerData pmd =
        new ProjectsManagerData(allInfo[i].getProjectName(),
          (String) allInfo[i].getUserId());

      /*
       * inicializando com valores vazios.
       */
      pmd.setProjectDescription(null);
      pmd.setCreationDate(0);
      pmd.setLocationInServer(allInfo[i].getAbsolutePath());
      pmd.setModificationDate(0);
      pmd.setOccupiedSpace(0);

      pmd.setOwnerName(User.getName(allInfo[i].getUserId()));

      pmd.setOwner(true);

      if (allInfo[i].getState() == ProjectAdminInfo.UNLOCKED_WITH_AREA_ALLOCATED) {

        pmd.setProjectSpaceAllocation(ProjectSpaceAllocation.ALLOCATED);
        final long allocatedSpace = allInfo[i].getAreaLockedSize();
        pmd.setAllocatedSpace(allocatedSpace);

        final CommonClientProject ccp;
        try {
          ccp =
            psi.openProject(allInfo[i].getUserId(),
              allInfo[i].getProjectName(), false);

          if (ccp != null) {

            pmd.setProjectDescription(ccp.getDescription());
            pmd.setCreationDate(ccp.getCreationDate());
            pmd.setModificationDate(ccp.getLastModificationDate());

            pmd.setLocationInServer(ccp.getLocationInServer());

            if (loadOccupiedSpace) {
              pmd.setOccupiedSpace(dus.getUsedSpaceForProjectMb(
                ccp.getUserId(), ccp.getName()));
            }
            else {
              pmd.setOccupiedSpace(-1);
            }

            /*
             * FIXME - try adicionado para que excees lanadas por tentativa
             * de abrir projetos no formato de permisses antigo no impeam a
             * abertura do gerenciador de projetos.
             */
            try {
              pmd.setSharingType(ccp.getSharingType());
            }
            catch (IllegalStateException ise) {
              ccp.close(false);
              continue;
            }
            pmd.setUsers(ccp.getUsersRO(), ccp.getUsersRW());
            ccp.close(false);
          }
        }
        catch (ServiceFailureException sfe) {
          continue;
        }
        catch (BugException be) {
          /*
           * Esta exceo  lanada quando o arquivo .csbase_project_info do
           * projeto que estamos tentando abrir no existe ou est corrompido. A
           * execuo no deve parar por causa disto, apenas uma mensagem de
           * erro deve ser exibida.
           */
          failedProjectNames.add(allInfo[i].getProjectName());
          continue;
        }
      }
      else if (allInfo[i].isLocked()) {
        pmd.setRequestedSpace(allInfo[i].getAreaRequestSize());
        pmd
          .setProjectSpaceAllocation(ProjectSpaceAllocation.WAITING_ALLOCATION);
        pmd.setScope(ProjectsManagerScope.NOT_APPLICABLE);
        /*
         * assumimos o escopo default "privado"
         */
        pmd.setSharingType(SharingType.PRIVATE);
        pmd.setUsers(new HashSet<Object>(), new HashSet<Object>());
        waitingAreaAction.add(pmd.getProjectName());

        pmd.setCreationDate(allInfo[i].getAreaRequestDate().getTime());
      }
      else if (allInfo[i].isWaitingAreaFree()) {
        pmd
          .setProjectSpaceAllocation(ProjectSpaceAllocation.WAITING_DEALLOCATION);
        pmd.setScope(ProjectsManagerScope.NOT_APPLICABLE);
        /*
         * assumimos o escopo default "privado"
         */
        pmd.setSharingType(SharingType.PRIVATE);
        waitingAreaAction.add(pmd.getProjectName());
      }

      /*
       * Projetos sem alocao (ou solicitao de alocao) no so mantidos na
       * lista de ProjectAdminInfos.
       */
      projectList.add(pmd);
    }
  }

}
