/**
 * $Id$
 */

package csbase.client.applications.projectsmanager.proxy;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import csbase.client.applications.projectsmanager.ProjectsManager;
import csbase.client.applications.projectsmanager.models.ProjectSpaceAllocation;
import csbase.client.applications.projectsmanager.models.ProjectsInfoData;
import csbase.client.applications.projectsmanager.models.ProjectsManagerData;
import csbase.client.applications.projectsmanager.proxy.core.ProjectsManagerTask;
import csbase.exception.BugException;
import csbase.exception.ServiceFailureException;
import csbase.logic.CommonClientProject;
import csbase.logic.User;
import csbase.logic.UserProjectInfo;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.DiskUsageServiceInterface;
import csbase.remote.ProjectServiceInterface;

/**
 * Tarefa que preenche a tabela com os dados dos projetos existentes no servidor
 * e relevantes ao usurio corrente.
 * 
 * @author jnlopes
 */
public class FillProjectsTableUserTask extends
  ProjectsManagerTask<ProjectsInfoData> {

  /**
   * Indica se espao ocupado pelos projetos deve ser carregado ou no.
   */
  final private boolean loadOccupiedSpace;

  /**
   * Projetos que no puderam ser abertos pois o arquivo .csbase_project_info
   * no existe ou est corrompido.
   */
  final private Set<String> failedProjectNames = new HashSet<String>();

  /**
   * Construtor
   * 
   * @param projectsManager A aplicao
   * @param loadOccupiedSpace boolean que indica se espao ocupado pelos
   *        projetos deve ser carregado ou no.
   */
  public FillProjectsTableUserTask(final ProjectsManager projectsManager,
    final boolean loadOccupiedSpace) {
    super(projectsManager);
    this.loadOccupiedSpace = loadOccupiedSpace;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void performTask() throws Exception {
    final List<ProjectsManagerData> projectsData = getProjectsData();

    final ProjectsInfoData pid =
      new ProjectsInfoData(projectsData, failedProjectNames);
    setResult(pid);
  }

  /**
   * Mtodo que efetivamente obtm os dados dos projetos.
   * 
   * @return vetor de dados de projeto.
   * @throws Exception
   */
  private List<ProjectsManagerData> getProjectsData() throws Exception {
    final List<ProjectsManagerData> projectList =
      new ArrayList<ProjectsManagerData>();

    getProjectsWhereUserIsOwner(projectList);
    getProjectsWhereUserIsNotOwner(projectList);

    return projectList;
  }

  /**
   * Pega os projetos onde o usurio  o criador do projeto.
   * 
   * @param projectList Lista de projetos a ser atualizada.
   * @throws RemoteException
   * @throws Exception
   */
  private void getProjectsWhereUserIsOwner(
    final List<ProjectsManagerData> projectList) throws RemoteException,
    Exception {

    final ProjectServiceInterface psi = ClientRemoteLocator.projectService;
    final DiskUsageServiceInterface dus = ClientRemoteLocator.diskUsageService;

    /*
     * Pegamos os projetos cujo dono  o usurio.
     */

    final ProjectsManager projectsManager = getProjectsManager();
    final User user = projectsManager.getUser();
    final Object ownerId = user.getId();
    final String[] projectNames = psi.getAllProjectNames(ownerId);

    if (projectNames == null) {
      /*
       * usurio no  dono de nenhum projeto
       */
      return;
    }

    for (String prjName : projectNames) {
      try {
        final CommonClientProject ccp =
          psi.openProject(ownerId, prjName, false);
        final long allocatedSpace = ccp.getLockingAreaSize();

        /*
         * Se o projeto nao tem alocao, ento ele no foi lido pelo loop
         * anterior.
         */
        final ProjectSpaceAllocation psa =
          ProjectSpaceAllocation.getItemFromSpace(allocatedSpace);
        if (psa == ProjectSpaceAllocation.NO_ALLOCATION
          || psa == ProjectSpaceAllocation.ALLOCATED) {

          final String userId = (String) ccp.getUserId();
          final String name = ccp.getName();
          final ProjectsManagerData pmd = new ProjectsManagerData(name, userId);
          pmd.setOwnerName(User.getName(userId));
          pmd.setProjectSpaceAllocation(ProjectSpaceAllocation
            .getItemFromSpace(allocatedSpace));
          pmd.setProjectDescription(ccp.getDescription());
          pmd.setCreationDate(ccp.getCreationDate());
          pmd.setLocationInServer(ccp.getLocationInServer());
          pmd.setModificationDate(ccp.getLastModificationDate());

          if (loadOccupiedSpace) {
            pmd.setOccupiedSpace(dus.getUsedSpaceForProjectMb(userId, name));
          }
          else {
            pmd.setOccupiedSpace(-1);
          }

          /*
           * FIXME - try adicionado para que excees lanadas por tentativa de
           * abrir projetos no formato de permisses antigo no impeam a
           * abertura do gerenciador de projetos.
           */
          try {
            pmd.setSharingType(ccp.getSharingType());
          }
          catch (IllegalStateException ise) {
            ccp.close(false);
            continue;
          }

          pmd.setUsers(ccp.getUsersRO(), ccp.getUsersRW());
          pmd.setOwner(true);

          ccp.close(false);

          projectList.add(pmd);
        }
      }
      catch (ServiceFailureException sfe) {
        /*
         * Esta exceo  lanada quando tentamos abrir um projeto que espera
         * alocao ou desalocao de rea. A execuo no deve ser interrompida
         * por isto.
         */
        continue;
      }
      catch (BugException be) {
        /*
         * Esta exceo  lanada quando o arquivo .csbase_project_info do
         * projeto que estamos tentando abrir no existe ou est corrompido. A
         * execuo no deve parar por causa disto, apenas uma mensagem de erro
         * deve ser exibida.
         */
        failedProjectNames.add(prjName);
        continue;
      }
    }
  }

  /**
   * Pega os projetos onde o usurio no  o criador do projeto,  apenas um
   * usurio compartilhado.
   * 
   * @param projectList Lista de projetos a ser atualizada.
   * @throws RemoteException
   * @throws Exception
   */
  private void getProjectsWhereUserIsNotOwner(
    final List<ProjectsManagerData> projectList) throws RemoteException,
    Exception {

    final ProjectServiceInterface psi = ClientRemoteLocator.projectService;
    final DiskUsageServiceInterface dus = ClientRemoteLocator.diskUsageService;

    final ProjectsManager projectsManager = getProjectsManager();
    final User currentUser = projectsManager.getUser();

    /*
     * Agora pegamos os projetos que no tem nenhuma alocao e cujo dono no 
     * o usurio corrente da aplicao.
     */

    /*
     * TODO - Possivel bug (contornado). Projetos vem repetidos.
     */
    final List<UserProjectInfo> usrPrjInf =
      psi.getAllUserProjectsFromOthers(currentUser.getId());

    final Iterator<UserProjectInfo> iter = usrPrjInf.iterator();
    while (iter.hasNext()) {
      final UserProjectInfo upi = iter.next();

      final CommonClientProject ccp;
      try {
        ccp = psi.openProject(upi.getUserId(), upi.getProjectName(), false);

        if (ccp.isPublic()) {
          if (checkProjectAlreadyInserted(projectList, upi.getUserId(), upi
            .getProjectName())) {
            continue;
          }
        }

        final long allocatedSpace = ccp.getLockingAreaSize();

        /*
         * Se o projeto nao tem alocao, ento ele no foi lido pelo loop
         * anterior.
         */
        final ProjectSpaceAllocation psa =
          ProjectSpaceAllocation.getItemFromSpace(allocatedSpace);
        if (psa == ProjectSpaceAllocation.NO_ALLOCATION
          || psa == ProjectSpaceAllocation.ALLOCATED) {
          //&& !ccp.isPublic()) {

          final ProjectsManagerData pmd =
            new ProjectsManagerData(ccp.getName(), (String) ccp.getUserId());

          pmd.setOwnerName(User.getName(ccp.getUserId()));
          pmd.setProjectSpaceAllocation(ProjectSpaceAllocation
            .getItemFromSpace(allocatedSpace));
          pmd.setProjectDescription(ccp.getDescription());
          pmd.setCreationDate(ccp.getCreationDate());
          pmd.setLocationInServer(ccp.getLocationInServer());
          pmd.setModificationDate(ccp.getLastModificationDate());

          if (loadOccupiedSpace) {
            pmd.setOccupiedSpace(dus.getUsedSpaceForProjectMb(ccp.getUserId(),
              ccp.getName()));
          }
          else {
            pmd.setOccupiedSpace(-1);
          }

          pmd.setSharingType(ccp.getSharingType());
          pmd.setUsers(ccp.getUsersRO(), ccp.getUsersRW());
          pmd.setOwner(false);
          ccp.close(false);
          projectList.add(pmd);
        }
      }
      catch (ServiceFailureException sfe) {
        /*
         * Esta exceo  lanada quando tentamos abrir um projeto que espera
         * alocao ou desalocao de rea, ou no pertence ao usurio. A
         * execuo no deve ser interrompida por isto.
         */
        continue;
      }
      catch (BugException be) {
        /*
         * Esta exceo  lanada quando o arquivo .csbase_project_info do
         * projeto que estamos tentando abrir no existe ou est corrompido. A
         * execuo no deve parar por causa disto, apenas uma mensagem de erro
         * deve ser exibida.
         */
        failedProjectNames.add(upi.getProjectName());
        continue;
      }
    }
  }

  /**
   * Verifica se determinado projeto ja foi adicionado.
   * <p>
   * TODO - Este mtodo s  necessrio caso haja mesmo um bug na task
   * mencionada acima.
   * 
   * @param projectList vetor de projetos.
   * @param userId Id do usurio
   * @param projectName Nome do projeto
   * @return boolean indicando se o projeto j foi inserido.
   */
  private boolean checkProjectAlreadyInserted(
    final List<ProjectsManagerData> projectList, final Object userId,
    final String projectName) {

    for (ProjectsManagerData pmd : projectList) {
      if (pmd.getProjectName().equals(projectName)
        && pmd.getOwnerId().equals(userId)) {
        return true;
      }
    }

    return false;
  }

}