package csbase.client.applications.sgamonitor;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import csbase.client.applicationmanager.ApplicationManager;
import csbase.client.applications.Application;
import csbase.client.applications.ApplicationFrame;
import csbase.client.desktop.DesktopComponentFrame;
import csbase.client.desktop.DesktopErrorEvent;
import csbase.client.desktop.DesktopFrame;
import csbase.client.facilities.monitoringtable.MonitoringTableEvent;
import csbase.client.facilities.monitoringtable.MonitoringTableObserver;
import csbase.client.facilities.sgatable.SGATable;
import csbase.client.util.ClientUtilities;
import csbase.logic.SGASet;

/**
 * A classe <code>ServerMonitorFrame</code> representa um dilogo de
 * monitorao dos servidores de algoritmos (SGAs).
 *
 * @author Andr Luiz Clinio
 * @version $Revision: 82966 $
 */
public class ServerMonitorFrame implements MonitorFrame, 
  MonitoringTableObserver {
  /** Aplicao */
  private Application app;

  /** Elementos de Interface */
  private ApplicationFrame mainFrame;
  private JButton monitorButton;
  private SGATable sgaTable;
  private DesktopErrorEvent errorEvent = null;

  /**
   * Exibe a janela de monitorao.
   */
  private void display() {
    final DesktopFrame desktop = DesktopFrame.getInstance();
    final DesktopComponentFrame frame = desktop.getDesktopFrame();

    mainFrame.pack();
    mainFrame.center(frame);
    mainFrame.setVisible(true);
  }

  /**
   * Termina a monitorao.
   */
  public void close() {
    sgaTable.stopThread();
  }

  /**
   * Cria a janela de monitorao dos servidores.
   */
  private void createMainFrame() {
    mainFrame = app.getApplicationFrame();
    Container cp = mainFrame.getContentPane();
    cp.setLayout(new BorderLayout(5, 5));
    sgaTable = new SGATable();
    sgaTable.addObserver(this);
    cp.add(new JScrollPane(sgaTable), BorderLayout.CENTER);
    cp.add(createButtons(), BorderLayout.SOUTH);
  }

  /**
   * Cria o painel de botes.
   *
   * @return o painel de botes criado
   */
  private JPanel createButtons() {
    JPanel panel = new JPanel(new FlowLayout());
    JButton closeButton = new JButton(app.getString("MON_CLOSE"));
    closeButton.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent ev) {
          close();
          app.finishApplication();
        }
      });
    monitorButton = new JButton(app.getString("MON_DETAIL"));
    monitorButton.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          int viewRow = sgaTable.getSelectedRow();
          if (viewRow != -1) {
            int modelRow = sgaTable.convertRowIndexToModel(viewRow);
            SGASet sga = sgaTable.getSGASet(modelRow);
            new SGAInfoFrame(sga.getName(), app);
          }
        }
      });
    monitorButton.setEnabled(false);
    addSelectionListener();
    ClientUtilities.adjustEqualSizes(monitorButton, closeButton);
    panel.add(monitorButton);
    panel.add(closeButton);
    return panel;
  }

  /**
   * Adiciona ouvidor para selecao de linhas, e habilita o botao  de visualizar
   * monitoracao se estiver alguma linha em selecao, senao o botao e
   * desabilitado.
   */
  public void addSelectionListener() {
    ListSelectionModel selectModel = sgaTable.getSelectionModel();
    selectModel.addListSelectionListener(
      new ListSelectionListener() {
        public void valueChanged(ListSelectionEvent e) {
          if (sgaTable.getSelectedRowCount() == 0) {
            monitorButton.setEnabled(false);
          }
          else {
            monitorButton.setEnabled(true);
          }
        }
      });
  }

  /**
   * Recebe notificaes da SGATable toda vez que esta  atualizada, ou na
   * ocorrncia de algum erro.
   *
   * @param sgaTableEvent Um evento do tipo MonitoringTableEvent.
   */
  public void update(final MonitoringTableEvent sgaTableEvent) {
    int event = sgaTableEvent.event;
    if (event == MonitoringTableEvent.REFRESH) {
      if (!sgaTable.isAnySgaAlive()) {
        monitorButton.setEnabled(false);
      }
      if (errorEvent != null) {
        mainFrame.resetError(errorEvent);
        monitorButton.setEnabled(false); //Nenhum SGA est selecionado
        errorEvent = null;
      }
    }
    else if (event == MonitoringTableEvent.REMOTE_ERROR) {
      JComponent[] disabled =
        new JComponent[] { sgaTable, monitorButton };
      errorEvent =
        new DesktopErrorEvent(
          app.getString("MON_COMM_ERROR"), null, disabled);
      mainFrame.signalError(errorEvent);
    }
  }

  /**
   * Construtor padro. Constri um monitorador de servidores (SGA's) apenas
   * com as colunas padro.
   *
   * @param app .
   */
  public ServerMonitorFrame(Application app) {
    this.app = app;
    createMainFrame();
    display();
  }
}
