/*
 * $Id: DesktopComponentDialog.java 70226 2007-11-15 15:43:21Z taciana $
 */

package csbase.client.desktop;

import java.awt.*;
import java.text.MessageFormat;

import javax.swing.*;

import tecgraf.javautils.gui.GUIUtils;

/**
 * Dilogo de componente de desktop.
 *
 * @author Tecgraf/PUC-Rio
 */
public class DesktopComponentDialog extends JDialog implements
  DesktopWindowInterface {

  /**
   * Janela a partir da qual esta foi criada.
   */
  private final Window owner;

  /**
   * Cria um dilogo.
   *
   * @param window A janela pai deste dilogo. No pode ser nula e tem que ser
   *        uma instncia de {@link JFrame} ou {@link JDialog}.
   * @return O dilogo (Pode ser <code>null</code>).
   * @throws IllegalArgumentException se a janela (window) for de tipo errado.
   */
  public static DesktopComponentDialog createDialog(Window window) {
    if (window == null) {
      return new DesktopComponentDialog((JFrame) null);
    }
    if (window instanceof JDialog) {
      return new DesktopComponentDialog((JDialog) window);
    }
    if (window instanceof JFrame) {
      return new DesktopComponentDialog((JFrame) window);
    }
    final String message = MessageFormat.format(
      "A classe da janela (window) passada  {0},"
        + " porm ela deveria ser {1} ou {2}.", new Object[] {
          window.getClass().getName(), JDialog.class.getName(),
          JFrame.class.getName() });
    throw new IllegalArgumentException(message);
  }

  /**
   * Construo da janela
   */
  private void build() {
    setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
    DesktopFrame desktopFrame = DesktopFrame.getInstance();
    if (desktopFrame != null) {
      desktopFrame.addWindow(this);
    }
  }

  /**
   * Centralizao
   */
  public void center() {
    this.center(owner);
  }

  /**
   * Centralizao
   *
   * @param window a janela de referncia para centralizao
   */
  public void center(Window window) {
    if (window == null) {
      GUIUtils.centerOnScreen(this);
      return;
    }

    if (window instanceof JFrame) {
      /*
       * getSize(), getX() e getY() no levam em considerao o fato da janela
       * de referncia estar maximizada; neste caso, centralizamos na tela
       */
      JFrame jframe = (JFrame) window;
      final int windowState = jframe.getExtendedState();
      if ((windowState & Frame.MAXIMIZED_BOTH) != 0) {
        GUIUtils.centerOnScreen(this);
        return;
      }
    }
    Dimension currentSize = this.getSize();
    Dimension windowSize = window.getSize();
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    int minX = window.getX() + (windowSize.width - currentSize.width) / 2;
    if ((minX + currentSize.width) > screenSize.width) {
      minX = screenSize.width - currentSize.width;
    }
    if (minX < 0) {
      minX = 0;
    }
    int minY = window.getY() + (windowSize.height - currentSize.height) / 2;
    if ((minY + currentSize.height) > screenSize.height) {
      minY = screenSize.height - currentSize.height;
    }
    if (minY < 0) {
      minY = 0;
    }
    this.setLocation(minX, minY);
  }

  /**
   * Mtodo que retorna a janela a partir da qual esta foi criada.
   * No utilizamos o <code>getOwner</code> da <code>Window</code> porque,
   * quando indicamos no construtor que o pai do <code>JDialog</code>
   *  null, o Swing internamente atribui uma janela como owner.
   * @return a janela a partir da qual esta foi criada (pode ser null).
   */
  public Window getMyOwner() {
    return owner;
  }

  /**
   * {@inheritDoc}
   */
  public void close() {
    DesktopFrame desktopFrame = DesktopFrame.getInstance();
    if (desktopFrame != null) {
      desktopFrame.removeWindow(this);
    }
    dispose();
  }

  /**
   * Construtor
   *
   * @param owner frame pai
   */
  public DesktopComponentDialog(JFrame owner) {
    super(owner, true);
    this.owner = owner;
    build();
  }

  /**
   * Construtor
   *
   * @param owner dilogo pai.
   */
  public DesktopComponentDialog(JDialog owner) {
    super(owner, true);
    this.owner = owner;
    build();
  }

  /**
   * Construtor
   *
   * @param owner frame pai
   * @param title ttulo
   */
  public DesktopComponentDialog(JFrame owner, String title) {
    super(owner, title, true);
    this.owner = owner;
    build();
  }

  /**
   * Construtor
   *
   * @param owner dilogo pai.
   * @param title ttulo
   */
  public DesktopComponentDialog(JDialog owner, String title) {
    super(owner, title, true);
    this.owner = owner;
    build();
  }
}
