/*
 * $Id: ProjectFileNameAndTypeFilter.java 94888 2009-08-11 15:20:25Z juliana $
 */
package csbase.client.desktop;

import java.rmi.RemoteException;
import java.util.regex.Pattern;

import csbase.logic.ProjectFile;
import csbase.logic.ProjectFileFilter;

/**
 * <p>
 * Filtro para arquivos de projeto, incluindo nome E tipo de arquivo. Este
 * filtro  <i>Case-Insensitive</i>.
 * </p>
 * <p>
 * Os filtros digitados pelo usurio, que so argumentos para o construtor, no
 * so expresses regulares mas uma simplificao convencionada para permitir
 * flexibilidade sem adicionar a complexidade das expresses regulares. Alm do
 * caracteres alfanumricos, os caracteres especiais reconhecidos pelo filtro
 * so:
 * <ol>
 * <li>'<b>?</b>': Aceita um caracter alfanumrico qualquer;</li>
 * <li>'<b>*</b>': Aceita qualquer nmero de caracteres alfanumricos;</li>
 * <li>'<b>$</b>': Marca o fim do arquivo; por <i>default</i>, a classe
 * considera que o filtro entrado  sempre o <u>incio</u> do filtro, isto ,
 * aceita qualquer nmero de caracteres alfanumricos aps aqueles digitados
 * pelo usurio. Se o caracter '$' for entrado, o filtro no aceitar nenhum
 * caracter alm daqueles digitados pelo usurio.</li>
 * </ol>
 * </p>
 * 
 * @author Tecgraf
 */
public class ProjectFileNameAndTypeFilter implements ProjectFileFilter {

  /** Expresso regular a ser comparada com o nome do arquivo. */
  private Pattern namePattern;
  /** Tipo de arquivos permitidos pelo filtro. */
  private String fileType;
  /**
   * No caso de um diretrio, verifica se algum dos arquivos filhos so aceitos
   * pelo filtro tambm.
   */
  private boolean checkDescendant = true;

  /**
   * Cria o filtro.
   * 
   * @param name filtro de nome definido pelo usurio. Nulos no so permitidos.
   *        Se um valor vazio for passado, o filtro permitir quaisquer nomes de
   *        arquivos.
   * @param typeCode tipo de arquivo aceito, tambm escolhido pelo usurio. Se
   *        um valor nulo for passado, o filtro permitir qualquer tipo de
   *        arquivo.
   */
  public ProjectFileNameAndTypeFilter(String name, String typeCode) {
    if (name == null) {
      throw new IllegalArgumentException("name == null");
    }
    name = name.replaceAll("\\.", "\\\\.");
    name = name.replaceAll("\\?", "\\.");
    name = name.replaceAll("\\*", "\\.\\*");
    if ((name.length() == 0 || name.charAt(name.length() - 1) != '*')
      && (name.indexOf('$') == -1)) {
      name += ".*";
    }
    namePattern = Pattern.compile(name, Pattern.CASE_INSENSITIVE);
    fileType = typeCode;
  }

  /**
   * Cria o filtro.
   * 
   * @param name filtro de nome definido pelo usurio. Nulos no so permitidos.
   *        Se um valor vazio for passado, o filtro permitir quaisquer nomes de
   *        arquivos.
   * @param typeCode tipo de arquivo aceito, tambm escolhido pelo usurio. Se
   *        um valor nulo for passado, o filtro permitir qualquer tipo de
   *        arquivo.
   * @param checkDescendant se esse valor for true e o arquivo  um diretrio,
   *        verifica-se se algum dos arquivos filhos  aceito pelo filtro
   *        tambm. Se for false, apenas o arquivo corrente  verificado. O
   *        valor default  true.
   */
  public ProjectFileNameAndTypeFilter(String name, String typeCode,
    boolean checkDescendant) {
    if (name == null) {
      throw new IllegalArgumentException("name == null");
    }
    name = name.replaceAll("\\.", "\\\\.");
    name = name.replaceAll("\\?", "\\.");
    name = name.replaceAll("\\*", "\\.\\*");
    if ((name.length() == 0 || name.charAt(name.length() - 1) != '*')
      && (name.indexOf('$') == -1)) {
      name += ".*";
    }
    namePattern = Pattern.compile(name, Pattern.CASE_INSENSITIVE);
    fileType = typeCode;
    this.checkDescendant = checkDescendant;
  }

  /**
   * {@inheritDoc}
   */
  public boolean accept(final ProjectFile file) {
    final RemoteTask<Boolean> task = new RemoteTask<Boolean>() {
      @Override
      public void performTask() throws Exception {
        final boolean result = typeMatches(file) && nameMatches(file);
        setResult(result);
      }
    };
    task.execute(null, null, "");

    /* Caso haja falha, os arquivos so aceitos (tratamento de exceo). */
    if (!task.getStatus()) {
      return true;
    }
    final boolean result = task.getResult();
    return result;
  }

  /**
   * Verifica se o nome do arquivo  aceito pelo filtro.
   * 
   * @param file arquivo sendo verificado.
   * 
   * @return true se o nome do arquivo for aceito pelo filtro.
   * @throws RemoteException se {@link #descendantMatchesName(ProjectFile)} o
   *         fizer
   */
  private boolean nameMatches(ProjectFile file) throws RemoteException {
    if (file.isDirectory() && checkDescendant) {
      return descendantMatchesName(file);
    }
    return namePattern.matcher(file.getName()).matches();
  }

  /**
   * Verifica se o tipo do arquivo  aceito pelo filtro.
   * 
   * @param file arquivo sendo verificado.
   * 
   * @return true se o tipo do arquivo for aceito pelo filtro. *
   * @throws RemoteException se {@link #descendantMatchesType(ProjectFile)} o
   *         fizer
   */
  private boolean typeMatches(ProjectFile file) throws RemoteException {
    if (fileType == null) {
      return true;
    }
    if (file.isDirectory() && checkDescendant) {
      return descendantMatchesType(file);
    }
    return file.getType().equals(fileType);
  }

  /**
   * Verifica se o nome de algum arquivo na subrvore do diretrio especificado
   *  aceito pelo filtro.
   * 
   * @param dir diretrio sendo filtrado.
   * 
   * @return true se o nome de algum arquivo na subrvore do diretrio for
   *         aceito pelo filtro.
   * @throws RemoteException se a busca por filhos ({@link ProjectFile#getChildren()})
   *         o fizer
   */
  private boolean descendantMatchesName(ProjectFile dir) throws RemoteException {
    if (namePattern.matcher(dir.getName()).matches()) {
      return true;
    }
    ProjectFile[] children = dir.getChildren();
    for (int i = 0; i < children.length; i++) {
      ProjectFile child = children[i];
      if (child.isDirectory() && descendantMatchesName(child)) {
        return true;
      }
      if (namePattern.matcher(child.getName()).matches()) {
        return true;
      }
    }
    return false;
  }

  /**
   * Verifica se o tipo de algum arquivo na subrvore do diretrio especificado
   *  aceito pelo filtro.
   * 
   * @param dir diretrio sendo filtrado.
   * 
   * @return true se o tipo de algum arquivo na subrvore do diretrio for
   *         aceito pelo filtro.
   * @throws RemoteException se a busca por filhos ({@link ProjectFile#getChildren()})
   *         o fizer
   */
  private boolean descendantMatchesType(ProjectFile dir) throws RemoteException {
    ProjectFile[] children = dir.getChildren();
    for (int i = 0; i < children.length; i++) {
      ProjectFile child = children[i];
      if (child.isDirectory() && descendantMatchesType(child)) {
        return true;
      }
      if (child.getType().equals(fileType)) {
        return true;
      }
    }
    return false;
  }
}