/*
 * $Id: Task.java 99211 2009-11-26 14:40:35Z pizzol $
 */

package csbase.client.desktop;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DecimalFormat;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.Timer;
import javax.swing.border.BevelBorder;
import javax.swing.text.JTextComponent;

import tecgraf.javautils.LNG;
import tecgraf.javautils.configurationmanager.Configuration;
import tecgraf.javautils.configurationmanager.ConfigurationManager;
import tecgraf.javautils.configurationmanager.ConfigurationManagerException;
import tecgraf.javautils.gui.GUIUtils;
import csbase.client.util.StandardErrorDialogs;

/**
 * A classe <code>Task</code> modela uma tarefa possivelmente demorada executada
 * no cliente. Uma thread  criada para a execuo da tarefa, para que eventos
 * de interface (como redesenho) possam ser atendidos. As interaes de teclado
 * e mouse, contudo, so bloqueadas. Caso a durao da tarefa ultrapasse um
 * valor limite (1 seg), um dilogo (modal) contendo uma barra de progresso ser
 * exibido. Este dilogo ser fechado automaticamente ao final da tarefa. Aps
 * um intervalo de valor configurvel, um boto para fechamento do dilogo
 * (cancelamento da tarefa)  oferecido ao usurio. O intervalo default  de
 * 30s. O texto do boto (FECHAR, CANCELAR)  tambm configurvel, pois depende
 * do tipo de tarefa, e do efeito de seu cancelamento.
 * 
 * @param <R> A classe do resultado da tarefa.
 * @author Tecgraf/PUC-Rio
 */
public abstract class Task<R> implements Runnable {

  /**
   * Nome da propriedade que indica opcionalmente o tempo esperado para o
   * aparecimento do boto de cancelar aps o lanamento da Task. A propriedade
   *  definida com base no {@link ConfigurationManager} do sistema.
   * 
   * @see #DEFAULT_CANCEL_DELAY
   */
  final private String CANCEL_DELAY_PROP_NAME = "cancel.delay.seconds";

  /**
   * Delay
   */
  protected int progressDialogDelay = 2;

  /**
   * Check delay
   */
  protected static final int PROGRESS_CHECK_DELAY = 1;

  /**
   * Cancel delay default. Este valor pode ser alterado pelo
   * {@link ConfigurationManager} do sistema pela propriedade definida por
   * {@link #CANCEL_DELAY_PROP_NAME}.
   */
  public static final int DEFAULT_CANCEL_DELAY = 30;

  /**
   * Tipo de texto de cancelamento
   */
  public static final int CANCEL_BUTTON = 0;

  /**
   * Tipo de texto de fechamento
   */
  public static final int CLOSE_BUTTON = 1;

  /**
   * Indica se progresso pode ser determinado
   */
  private boolean showsProgress = false;

  /**
   * Indica se passos so exibidos
   */
  private boolean showsSteps = false;

  /**
   * Thread que executa a tarefa
   */
  private Thread worker;

  /**
   * Indica se a execuo da tarefa foi cancelada
   */
  private boolean cancelled;

  /**
   * Resultado gerado pela tarefa
   */
  private R result = null;

  /**
   * Status de execuo da tarefa
   */
  private boolean status = false;

  /**
   * Erro detetado durante a execuo da tarefa
   */
  private Exception error;

  /**
   * Campo de texto para tempo decorrido
   */
  private JTextField elapsedText = new JTextField();

  /**
   * Campo de texto para tempo total
   */
  private JTextField totalText = new JTextField(" ");

  /**
   * Campo de texto para passo corrente
   */
  private JTextComponent stepText = new JTextField(20);

  /**
   * Espao destinado a colocao de imagem no canto esquerdo do dilogo.
   */
  private JLabel imageLabel = new JLabel();

  /**
   * Formatador de inteiro para (hh:mm:ss) default
   */
  private static DecimalFormat integerTimeFormatter = new DecimalFormat("00");

  /**
   * Janela a partir da qual a tarefa  comandada
   */
  protected Window parentWindow;

  /**
   * Dilogo de espera, indicando atividade da tarefa
   */
  protected JDialog progressDialog;

  /**
   * barra de progresso
   */
  protected final JProgressBar progressBar = new JProgressBar();

  /**
   * Ttulo da tarefa
   */
  protected String taskTitle;

  /**
   * Mensagem para o dilogo da tarefa
   */
  private String taskMessage;

  /**
   * Tempo decorrido, em segundos
   */
  protected long elapsedSeconds = 0L;

  /**
   * Decide se o dilogo de progresso ser exibido
   */
  private boolean progressDialogEnabled = true;

  /**
   * Tarefa a ser executada. Esse mtodo somente  implementado somente por
   * especializaes "concretas" desta classe, ou seja, por tarefas especficas.
   * 
   * @throws Exception quando a ocorrncia de um erro (representado pela
   *         exceo) impediu a realizao da tarefa
   */
  protected abstract void performTask() throws Exception;

  /**
   * Execuo da tarefa. Especializaes dessa classe (tarefas remotas, tarefas
   * locais) realizam o tratamento adequado para erros que possam ocorrer
   * durante a execuo da tarefa.
   */
  public final void run() {
    try {
      performTask();
      status = true;
      error = null;
    }
    catch (Exception exception) {
      status = false;
      error = exception;
    }
  }

  /**
   * Armazena o resultado da tarefa executada, para obteno posterior.
   * 
   * @param result o resultado da tarefa
   */
  protected final void setResult(R result) {
    this.result = result;
  }

  /**
   * Obtm o resultado da tarefa executada.
   * 
   * @return o valor de resultado da tarefa
   */
  public final R getResult() {
    return result;
  }

  /**
   * Obtm o status da tarefa executada.
   * 
   * @return true se a tarefa foi completada false se um erro (exceo) impediu
   *         a realizao da tarefa
   */
  public final boolean getStatus() {
    return status;
  }

  /**
   * Obtm o erro detetado durante a execuo da tarefa.
   * 
   * @return exceo que representa o erro
   */
  public final Exception getError() {
    return error;
  }

  /**
   * Verifica se a tarefa foi cancelada pelo usurio. Esse mtodo pode ser
   * periodicamente chamado por uma implementao de <code>performTask</code>
   * que execute uma tarefa composta de uma sequncia de passos, ou subtarefas.
   * Este mtodo  sincronizado, de forma a garantir que a alterao dessa
   * condio (realizada pelo acionamento do boto, na thread de eventos) 
   * percebida pela thread que executa a tarefa.
   * 
   * @return true se o boto de cancelamento foi acionado (cancelando a tarefa)
   */
  public synchronized boolean wasCancelled() {
    return cancelled;
  }

  /**
   * Cancela a tarefa, em consequncia do acionamento do boto de cancelamento.
   * Este mtodo  sincronizado, de forma a garantir que a alterao dessa
   * condio (realizada pelo acionamento do boto, na thread de eventos) 
   * percebida pela thread que executa a tarefa.
   */
  protected synchronized void cancelTask() {
    cancelled = true;
    status = false;
  }

  /**
   * Tratamento de erros especfico para cada tipo de tarefa. Esse tratamento 
   * chamado na thread do Swing. Essa implementao representa um tratamento
   * mnimo, apenas para bugs no cliente. Pode ser redefinida por uma
   * especializao dessa classe.
   * 
   * @param error - exceo que representa o erro durante execuo da tarefa.
   */
  protected void handleError(Exception error) {
    StandardErrorDialogs.showErrorDialog(parentWindow, taskTitle, LNG
      .get("desktop.msg.clientbug"), error, getAdditionalInfo());
  }

  /**
   * Sobrecarga em mtodo
   * 
   * @param window
   * @param title
   * @param taskMessage
   * @param cancelDelayInSecs
   * @param cancelButtonType
   * @param showProgress
   * @param showStep
   * @return um flag
   * @see #execute(Window, String, String)
   */
  public final boolean execute(Window window, String title, String taskMessage,
    int cancelDelayInSecs, int cancelButtonType, boolean showProgress,
    boolean showStep) {
    return execute(window, title, taskMessage, cancelDelayInSecs,
      cancelButtonType, showProgress, showStep, null);
  }

  /**
   * Inicia a execuo da tarefa. Cria uma thread para executar a tarefa, e
   * aguarda o seu trmino durante um segundo. Se aps esse tempo a tarefa ainda
   * no terminou, cria e exibe o dilogo modal de progresso. Se a tarefa for
   * executada at o final (sem falhas sinalizadas por excees), o status
   * <code>true</code>  retornado. O resultado da tarefa pode ser obtido
   * atravs da chamada ao mtodo <code>getResult</code> Se ocorrer um erro
   * (exceo) durante a execuo da tarefa (incluindo seu "cancelamento"), o
   * status <code>false</code>  retornado, e nenhum resultado estar
   * disponvel. Antes deste retorno, o procedimento para tratamento especfico
   * de erros (definido nas subclasses)  invocado. O dilogo de progresso pode
   * ser configurado de forma a exibir e atualizar os sucessivos passos de uma
   * tarefa. Tambm pode ser configurado para permitir a atualizao do
   * percentual da tarefa j executado (quando o progresso pode ser
   * determinado).
   * 
   * @param window janela a partir da qual a tarefa foi comandada
   * @param title ttulo do dilogo de progresso
   * @param taskMessage mensagem no dilogo de progresso
   * @param cancelDelayInSecs intervalo em segundos para oferecimento do boto
   *        de cancelamento (se 0,  imediato)
   * @param cancelButtonType determina o tipo do boto de cancelamento (CANCEL
   *        ou CLOSE)
   * @param showProgress indica se progresso pode ser determinado
   * @param showSteps indica se exibe passo corrente
   * @param image imagem associada.
   * @return true se a tarefa foi executada at o final (h um resultado) false
   *         se a ocorrncia de um erro impediu a execuo da tarefa (no h
   *         resultado associado)
   */
  public final boolean execute(Window window, String title, String taskMessage,
    int cancelDelayInSecs, int cancelButtonType, boolean showProgress,
    boolean showSteps, ImageIcon image) {

    this.taskTitle = title;
    this.taskMessage = taskMessage;

    if (Thread.currentThread() instanceof MultiTaskThread) {
      // j estamos dentro de uma 'task principal', ou seja, j estamos
      // executando em uma thread em background ...
      // Ento devemos executar o procedimento nessa thread mesmo:
      ((MultiTaskThread) Thread.currentThread()).setCurrentTask(this);
      run();
      ((MultiTaskThread) Thread.currentThread()).resetCurrentProcedure();
      return handleTaskTermination(true);
    }

    parentWindow = window;
    if (parentWindow == null) {
      if (DesktopFrame.getInstance() != null) {
        parentWindow = DesktopFrame.getInstance().getDesktopFrame();
      }
    }
    this.showsProgress = showProgress;
    this.showsSteps = showSteps;
    if (parentWindow != null) {
      parentWindow.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
    }
    cancelled = false;
    worker = createWorkerThread();
    worker.start();
    try {
      if (progressDialogEnabled) {
        if (progressDialogDelay > 0)
          worker.join(progressDialogDelay * 1000);
      }
      else
        worker.join();
    }
    catch (InterruptedException ie) {
      return false;
    }
    if (!worker.isAlive()) {
      return handleTaskTermination(false);
    }
    createProgressDialog(parentWindow, title, taskMessage, image);
    Timer elapsedTimer = createElapsedTimer();
    elapsedTimer.start();
    final Timer showCancelTimer = createCancelTimer(cancelDelayInSecs,
      cancelButtonType);
    showCancelTimer.start();
    final Timer closeDialogTimer = new Timer(PROGRESS_CHECK_DELAY * 1000, null);
    closeDialogTimer.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (!worker.isAlive()) {
          progressDialog.setVisible(false);
          closeDialogTimer.stop();
          if (showCancelTimer.isRunning()) {
            showCancelTimer.stop();
          }
        }
      }
    });
    closeDialogTimer.start();
    progressDialog.setVisible(true);
    /* Janela de progresso foi fechada */
    elapsedTimer.stop();
    if (closeDialogTimer.isRunning()) {
      closeDialogTimer.stop();
    }
    progressDialog.dispose();
    return handleTaskTermination(false);
  }

  /**
   * @return a thread utilizada para realizar a tarefa em 'background'. Para
   *         criar uma tarefa 'global', que tenha outras tarefas aninhadas, use
   *         uma {@link MultiTaskThread}
   */
  protected Thread createWorkerThread() {
    return new Thread(this);
  }

  /**
   * Procedimento de finalizao da execuo de uma tarefa. Caso algum erro
   * tenha ocorrido durante a execuo da tarefa, o tratamento especfico 
   * invocado.
   * 
   * @param innerTask - TRUE indica que  uma tarefa aninhada, e deve relanar
   *        excees para a tarefa-me, ao invs de trat-las.
   * @return true se a tarefa foi executada sem erros (excees) false caso
   *         contrrio
   */
  private boolean handleTaskTermination(boolean innerTask) {
    if (parentWindow != null)
      parentWindow.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));

    if (getStatus())
      return true; // sucesso

    if (!cancelled) {
      // houve um erro ...
      // tarefas aninhadas devem apenas relanar as excees para a tarefa-me
      if (innerTask)
        throw new InnerTaskException(this, error);

      // se essa for tarefa me, devemos capturar exceo de tarefa aninhada e
      // executar o handleError() 'interno', j que pode ter sido sobrescrito:
      if (error instanceof InnerTaskException) {
        InnerTaskException innerTaskException = (InnerTaskException) error;
        Exception realException = (Exception) innerTaskException.getCause();
        innerTaskException.task.handleError(realException);
      }
      else {
        //  uma tarefa comum, comportamento padro ...
        handleError(error);
      }
    }

    setResult(null);
    return false;
  }

  /**
   * Cria um temporizador para exibir boto de fechar/cancelar. A ao do boto
   * solicita a interrupo da tarefa e fecha o o dilogo de progresso.
   * 
   * @param cancelDelayInSecs timeout em segundos para oferecimento do boto de
   *        cancelamento (se 0,  imediato)
   * @param cancelButtonType determina o tipo do boto de cancelamento (CANCEL
   *        ou CLOSE)
   * @return o temporizador criado
   */
  private Timer createCancelTimer(int cancelDelayInSecs,
    final int cancelButtonType) {
    Timer showCancelTimer = new Timer(cancelDelayInSecs * 1000, null);
    showCancelTimer.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (!worker.isAlive()) {
          return;
        }
        JButton cancelButton = new JButton();
        String msg = cancelButtonType == CANCEL_BUTTON ? LNG
          .get("desktop.cancel") : LNG.get("desktop.close");
        cancelButton.setText(msg);
        cancelButton.addActionListener(new ActionListener() {
          public void actionPerformed(ActionEvent ev) {
            try {
              worker.interrupt();
            }
            catch (Exception ex) {
            } // Applet no pode interromper thread !!!
            cancelTask();
            progressDialog.setVisible(false);
          }
        });

        JPanel buttonPanel = new JPanel();
        buttonPanel.add(cancelButton);
        Container cp = progressDialog.getContentPane();
        cp.add(buttonPanel, BorderLayout.SOUTH);
        cp.invalidate();
        progressDialog.pack();
      }
    });
    showCancelTimer.setRepeats(false);
    return showCancelTimer;
  }

  /**
   * Cria um dilogo modal contendo uma mensagem, o tempo decorrido e uma barra
   * de progresso de durao "indeterminada".
   * 
   * @param wnd janela "base" do dilogo criado
   * @param title ttulo do dilogo
   * @param msg mensagem a ser exibida
   * @param image a imagem a ser colocada (ou null)
   */
  private void createProgressDialog(Window wnd, String title, String msg,
    ImageIcon image) {
    progressDialog = (wnd != null && wnd instanceof Dialog) ? new JDialog(
      (Dialog) wnd, title, true) : new JDialog((Frame) wnd, title, true);
    Container cp = progressDialog.getContentPane();
    cp.setLayout(new BorderLayout());
    JPanel infoPanel = new JPanel(new GridBagLayout());
    GridBagConstraints c = new GridBagConstraints();
    setTaskImage(image);
    c.fill = GridBagConstraints.VERTICAL;
    c.gridy = 0;
    c.gridx = 0;
    c.insets = new Insets(0, 10, 0, 5);
    c.anchor = GridBagConstraints.CENTER;
    c.gridheight = 2;
    infoPanel.add(imageLabel, c);
    c.gridheight = 1;
    c.fill = GridBagConstraints.HORIZONTAL;
    c.gridy = 0;
    c.gridx = 1;
    c.anchor = GridBagConstraints.CENTER;
    c.insets = new Insets(20, 10, 10, 15);
    infoPanel.add(new JLabel(msg, JLabel.CENTER), c);
    c.gridy = 1;
    c.gridx = 1;
    c.anchor = GridBagConstraints.WEST;
    c.insets = new Insets(5, 10, 5, 5);
    c.weightx = 1.0;
    infoPanel.add(createTimeComponents(), c);
    if (!showsProgress) {
      progressBar.setIndeterminate(true);
      progressBar.setStringPainted(false);
    }
    else {
      progressBar.setIndeterminate(false);
      progressBar.setMinimum(0);
      progressBar.setMaximum(100);
      progressBar.setValue(0);
      progressBar.setStringPainted(true);
    }
    c.gridy = 2;
    c.gridx = 0;
    c.insets = new Insets(10, 10, 5, 5);
    c.gridwidth = 2;
    c.anchor = GridBagConstraints.CENTER;
    infoPanel.add(progressBar, c);
    cp.add(infoPanel, BorderLayout.CENTER);
    imageLabel.setHorizontalAlignment(SwingConstants.CENTER);
    imageLabel.setVerticalAlignment(SwingConstants.CENTER);
    progressDialog.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
    progressDialog.setResizable(false);
    progressDialog.pack();

    /* Define a localizao do dilogo de progresso */
    Dimension wSize = progressDialog.getSize();
    Dimension sSize = Toolkit.getDefaultToolkit().getScreenSize();
    int rx = (int) (Math.random() * 200.0);
    int ry = (int) (Math.random() * 100.0);
    int newX = ((sSize.width - wSize.width) / 3) + rx;
    int newY = ((sSize.height - wSize.height) / 2) + ry;
    progressDialog.setLocation(newX, newY);
  }

  /**
   * Cria um painel com elementos de temporizao e acompanhamento de progresso.
   * 
   * @return painel com os elementos
   */
  private JPanel createTimeComponents() {
    ArrayList<JComponent[]> compList = new ArrayList<JComponent[]>();
    elapsedText.setText(secToString(0));
    elapsedText.setFont(new Font("Monospaced", Font.PLAIN, 12));
    elapsedText.setEditable(false);
    elapsedText.setHorizontalAlignment(JTextField.RIGHT);
    compList.add(new JComponent[] { new JLabel(LNG.get("UTIL_ELAPSED_TIME")),
        elapsedText, null });
    if (showsProgress) {
      totalText.setFont(new Font("Monospaced", Font.PLAIN, 12));
      totalText.setEditable(false);
      totalText.setHorizontalAlignment(JTextField.RIGHT);
      compList.add(new JComponent[] { new JLabel(LNG.get("UTIL_TOTAL_TIME")),
          totalText, null });
    }
    if (showsSteps) {
      stepText = createStepTextArea();
      compList.add(new JComponent[] { new JLabel(LNG.get("UTIL_STEP_TEXT")),
          stepText, null });
    }
    JComponent[][] components = new JComponent[compList.size()][];
    components = compList.toArray(components);
    JPanel timePanel = GUIUtils.createBasicGridPanel(components);
    timePanel.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
    return timePanel;
  }

  /**
   * @return rea de texto para mostrar o passo atualmente sendo executado.
   */
  protected JTextComponent createStepTextArea() {
    JTextField stepText = new JTextField(20);
    stepText.setFont(new Font("Monospaced", Font.PLAIN, 12));
    stepText.setEditable(false);
    stepText.setHorizontalAlignment(JTextField.RIGHT);
    return stepText;
  }

  /**
   * Cria um temporizador para contabilizar o tempo decorrido, atualizando o
   * dilogo de progresso.
   * 
   * @return .
   */
  private Timer createElapsedTimer() {
    elapsedSeconds = 1L;
    Timer elapsedTimer = new Timer(1000, null);
    elapsedTimer.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (worker.isAlive()) {
          elapsedText.setText(secToString(elapsedSeconds));
          if (showsProgress) {
            final double perc = progressBar.getValue();
            double p;
            p = perc < 0.0 ? 0.0 : perc;
            p = perc > 100.0 ? 100.0 : perc;
            if (p != 0.0) {
              long tsec = (long) (elapsedSeconds * 100.0 / p);
              totalText.setText(secToString(tsec));
            }
          }
          elapsedSeconds++;
        }
      }
    });
    return elapsedTimer;
  }

  /**
   * Atualiza o passo corrente.
   * 
   * @param step identificao do passo corrente
   */
  public void setStepText(final String step) {
    if (showsSteps) {
      SwingUtilities.invokeLater(new Runnable() {
        public void run() {
          stepText.setText(step);
        }
      });
    }
  }

  /**
   * Atualiza a imagem do dilogo de espera.
   * 
   * @param image a imagem a ser colocada.
   */
  private void setTaskImage(ImageIcon image) {
    if (image == null) {
      imageLabel.setVisible(false);
      return;
    }
    int wid = image.getIconWidth();
    int hei = image.getIconHeight();
    Dimension siz = new Dimension(wid, hei);
    imageLabel.setSize(siz);
    imageLabel.setMinimumSize(siz);
    imageLabel.setPreferredSize(siz);
    imageLabel.setIcon(image);
  }

  /**
   * Atualiza a barra de progresso.
   * 
   * @param percentage percentual da tarefa j completada
   */
  public void setProgressStatus(final int percentage) {
    if (showsProgress) {
      SwingUtilities.invokeLater(new Runnable() {
        public void run() {
          int p;
          p = percentage < 0 ? 0 : percentage;
          p = percentage > 100 ? 100 : percentage;
          progressBar.setIndeterminate(false);
          progressBar.setStringPainted(true);
          progressBar.setValue(p);
          int c = 200 - (int) (200.0 * p / 100.0);
          int b = 255 - (int) (150.0 * p / 100.0);
          progressBar.setForeground(new Color(c, c, b));
        }
      });
    }
  }

  /**
   * Atualiza a barra de progresso para desconhecido.
   */
  public void setUnknownStatus() {
    if (showsProgress) {
      progressBar.setIndeterminate(true);
      progressBar.setStringPainted(false);
    }
  }

  /**
   * Converso de inteiro para string com dois dgitos.
   * 
   * @param v a quantidade.
   * @return uma string formatada.
   */
  private String timeFormat(long v) {
    return integerTimeFormatter.format(v);
  }

  /**
   * Conversao de segundos para uma string de tempo.
   * 
   * @param seconds a quantidade de segundos.
   * @return uma string formatada.
   */
  private String secToString(long seconds) {
    if (seconds < 0) {
      return "-";
    }
    long sec = seconds;
    long min = 0;
    long hou = 0;
    if (sec >= 60) {
      min = (int) sec / 60;
      sec = sec - (min * 60);
    }
    if (min >= 60) {
      hou = (int) min / 60;
      min = min - (hou * 60);
    }
    return " " + timeFormat(hou) + "h " + timeFormat(min) + "m "
      + timeFormat(sec) + "s";
  }

  /**
   * Consulta do tempo (em segundos) para aparecimento do boto de cancelamento
   * aps a task entrar.
   * 
   * @return o tempo
   */
  private int getCancelDelaySecs() {
    try {
      final ConfigurationManager cnfManager = ConfigurationManager
        .getInstance();
      final Configuration configuration = cnfManager.getConfiguration(this
        .getClass());
      final int delay = configuration.getOptionalIntegerProperty(
        CANCEL_DELAY_PROP_NAME, DEFAULT_CANCEL_DELAY);
      return delay;
    }
    catch (ConfigurationManagerException e) {
      return DEFAULT_CANCEL_DELAY;
    }
  }

  /**
   * Atalho para funo sem imagem (ltimo parmetro).
   * 
   * @param window janela a partir da qual a tarefa foi comandada
   * @param title ttulo do dilogo de progresso
   * @param taskMessage mensagem no dilogo de progresso
   * @return true se a tarefa foi executada at o final (h um resultado) false
   *         se a ocorrncia de um erro impediu a execuo da tarefa (no h
   *         resultado associado)
   */
  public final boolean execute(Window window, String title, String taskMessage) {
    final int cancelDelay = getCancelDelaySecs();
    return execute(window, title, taskMessage, cancelDelay, CLOSE_BUTTON,
      false, false, null);
  }

  /**
   * Inicia a execuo de uma tarefa. O boto de cancelamento  exibido aps o
   * tempo default, com texto "CANCELAR". O dilogo de progresso no exibe
   * percentual de progresso, nem passo corrente.
   * 
   * @param window janela a partir da qual a tarefa foi comandada
   * @param title ttulo do dilogo de progresso
   * @param taskMessage mensagem no dilogo de progresso
   * @param icon imagem associada  tarefa (null equivale a no ter imagem).
   * @return true se a tarefa foi executada at o final (h um resultado) false
   *         se a ocorrncia de um erro impediu a execuo da tarefa (no h
   *         resultado associado)
   */
  public final boolean execute(Window window, String title, String taskMessage,
    ImageIcon icon) {
    final int cancelDelay = getCancelDelaySecs();
    return execute(window, title, taskMessage, cancelDelay, CLOSE_BUTTON,
      false, false, icon);
  }

  /**
   * Verso do dilogo de execuo que permite a exibio do progresso e dos
   * passos sendo executados pela tarefa.
   * 
   * @param window janela a partir da qual a tarefa foi comandada
   * @param title ttulo do dilogo de progresso
   * @param taskMessage mensagem no dilogo de progresso
   * @param showProgress exibe o progresso da tarefa
   * @param showSteps exibe os passos sendo executados pela tarefa
   * @return true se a tarefa foi executada at o final (h um resultado) false
   *         se a ocorrncia de um erro impediu a execuo da tarefa (no h
   *         resultado associado)
   */
  public final boolean execute(Window window, String title, String taskMessage,
    boolean showProgress, boolean showSteps) {
    final int cancelDelay = getCancelDelaySecs();
    return execute(window, title, taskMessage, cancelDelay, CLOSE_BUTTON,
      showProgress, showSteps);
  }

  /**
   * Inicia a execuo de uma tarefa. O tempo para oferecimento do boto de
   * cancelamento e seu tipo so configurveis. O dilogo de progresso no exibe
   * percentual de progresso, nem passo corrente.
   * 
   * @param window janela a partir da qual a tarefa foi comandada
   * @param title ttulo do dilogo de progresso
   * @param taskMessage mensagem no dilogo de progresso
   * @param cancelDelayInSecs intervalo em segundos para oferecimento do boto
   *        de cancelamento (se 0,  imediato)
   * @param cancelButtonType determina o tipo do boto de cancelamento (CANCEL
   *        ou CLOSE)
   * @return true se a tarefa foi executada at o final (h um resultado) false
   *         se a ocorrncia de um erro impediu a execuo da tarefa (no h
   *         resultado associado)
   */
  public final boolean execute(Window window, String title, String taskMessage,
    int cancelDelayInSecs, int cancelButtonType) {
    return execute(window, title, taskMessage, cancelDelayInSecs,
      cancelButtonType, false, false);
  }

  /**
   * Tempo de espera para exibio do dilogo
   * 
   * @return Tempo em segundos
   */
  public final int getProgressDialogDelay() {
    return progressDialogDelay;
  }

  /**
   * Configura tempo de espera para exibio do dilogo
   * 
   * @param progressDialogDelay tempo em segundos
   */
  public final void setProgressDialogDelay(int progressDialogDelay) {
    this.progressDialogDelay = progressDialogDelay;
  }

  /**
   * Desabilita exibio do dilogo de progresso durante execuo da tarefa.
   * 
   * @param b indicativo de progresso habilitado.
   */
  public final void setProgressDialogEnabled(boolean b) {
    this.progressDialogEnabled = b;
  }

  /**
   * @return a mensagem que aparece no dilogo de progresso dessa tarefa.
   */
  public String getTaskMessage() {
    return taskMessage;
  }

  /**
   * @return lista de informaes adicionais que vo aparecer nos dilogos de
   *         erro para essa tarefa, e sero enviadas ao email do administrador
   *         se necessrio. Ex: 'Nome do projeto', 'Caminho do arquivo'. Cada
   *         informao ser mostrada em uma linha. Retornar NULL para nenhuma
   *         informao adicional.
   */
  protected String[] getAdditionalInfo() {
    return null;
  }

  // --------------------------------------------------------------------------
  // --------------------------------------------------------------------------
  // --------------------------------------------------------------------------

  /**
   * Classe que serve como 'marcador' de uma exceo ocorrida em uma task
   * aninhada.
   */
  private static class InnerTaskException extends RuntimeException {

    /** A tarefa aninhada onde ocorreu uma exceo durante a execuo */
    final Task<?> task;

    /**
     * Construtor.
     * 
     * @param task - a tarefa aninhada onde ocorreu um erro durante a execuo.
     * @param error - o erro original.
     */
    InnerTaskException(Task<?> task, Exception error) {
      super(error);
      this.task = task;
    }
  }

  // --------------------------------------------------------------------------
  // --------------------------------------------------------------------------
  // --------------------------------------------------------------------------

  /**
   * Thread para executar vrias tarefas aninhadas, mostrando-as como passos de
   * uma tarefa principal. Deve-se usar essa thread na 'task' principal, sabendo
   * que dentro de seu 'performTask' sero executadas outras tasks. Essa thread
   * ento executa todas as tasks, cujas mensagens de texto aparecem como passos
   * no dilogo de progresso da task principal.
   * <p>
   * Uso: sobrescrever o mtodo abaixo ao criar a task:
   * 
   * <pre>
   * protected Thread createWorkerThread() {
   *   return new MultiTaskThread(this);
   * }
   * </pre>
   */
  public static class MultiTaskThread extends Thread {

    /**
     * Tarefa que mostrar o dilogo de progresso.
     */
    private Task<?> mainTask;

    /**
     * Construtor.
     * 
     * @param mainTask - a tarefa principal.
     */
    public MultiTaskThread(Task<?> mainTask) {
      super(mainTask, "Task-WorkerThread");
      this.mainTask = mainTask;
    }

    /**
     * Retorna  mensagem original [da tarefa principal].
     */
    void resetCurrentProcedure() {
      mainTask.setStepText(mainTask.getTaskMessage());
    }

    /**
     * @param task - tarefa executando no momento.
     */
    void setCurrentTask(Task<?> task) {
      mainTask.setStepText(task.getTaskMessage());
    }
  }

}
