/*
 * $Id$
 */
package csbase.client.facilities.commandtable;

import java.awt.Window;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Vector;
import java.util.concurrent.atomic.AtomicBoolean;

import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.table.ObjectTableModel;
import tecgraf.javautils.gui.table.ObjectTableProvider;
import tecgraf.javautils.gui.table.SortableTable;
import csbase.client.algorithms.commands.cache.CommandsCache;
import csbase.client.algorithms.commands.cache.CommandsFilter;
import csbase.client.algorithms.commands.cache.events.AbstractCacheUpdatedEventListener;
import csbase.client.desktop.RemoteTask;
import csbase.client.facilities.monitoringtable.MonitoringTableEvent;
import csbase.client.facilities.monitoringtable.MonitoringTableObserver;
import csbase.logic.CommandInfo;
import csbase.logic.MonitoringSet;

/**
 * A classe <code>AbstractCommandMonitoringTable</code> implementa
 * funcionalidades bsicas de uma tabela de monitorao de comandos.
 * Caractersticas:<br>
 * <ul>
 * <li> observvel. Notifica os observadores quando  atualizada.</li>
 * <li>Ela se atualiza atravs de um observador que instala na
 * {@link CommandsCache}.</li>
 * <li>Ela mantm a ordenao e as selees originais mesmo aps a atualizao.</li>
 * <li>Cria a tabela com descritoes de coluna recebidos do mtodo abstrato
 * getTableColumnAttributes()</li>
 * </ul>
 * -  observvel. Notifica os observadores quando  atualizada. <br>
 * -
 */
public abstract class AbstractCommandMonitoringTable extends SortableTable {

  /** Flag indicando se o sistema de monitoraco por listener est ativo. */
  private final AtomicBoolean running;
  /** Listener de comandos. */
  private final AbstractCacheUpdatedEventListener listener;
  /** Modelo para a tabela. */
  protected ObjectTableModel<CommandMonitoringSet> model;
  /** Coleo de observadores da tabela */
  private final Collection<MonitoringTableObserver> observers;

  /** Indica se devemos mostrar a coluna 'Progresso' */
  protected boolean showProgressColumn;

  /**
   * Construtor que constri por default uma tabela de seleo nica.
   * 
   * @param filter filtro que indica que comandos devem aparecer nesta tabela.
   */
  public AbstractCommandMonitoringTable(CommandsFilter filter) {
    this(ListSelectionModel.SINGLE_SELECTION, filter, false);
  }

  /**
   * Construtor que permite escolher o modo de seleo da tabela.
   * 
   * @param selectionMode Indica o modo de seleo da tabela.
   * @param filter filtro que indica que comandos devem aparecer nesta tabela.
   * @see javax.swing.JTable#setSelectionMode(int)
   */
  public AbstractCommandMonitoringTable(int selectionMode, CommandsFilter filter) {
    this(selectionMode, filter, false);
  }

  /**
   * Construtor que permite escolher o modo de seleo da tabela.
   * 
   * @param selectionMode Indica o modo de seleo da tabela.
   * @param filter filtro que indica que comandos devem aparecer nesta tabela.
   * @param showProgress - TRUE indica para mostrarmos a coluna 'progresso' na
   *        tabela de algoritmos em execuo, FALSE para omitir tal coluna.
   *        Valor obtido da propriedade de mesmo nome.
   * @see javax.swing.JTable#setSelectionMode(int)
   */
  public AbstractCommandMonitoringTable(int selectionMode,
    CommandsFilter filter, boolean showProgress) {
    this.showProgressColumn = showProgress;
    this.running = new AtomicBoolean(false);
    this.listener = new AbstractCacheUpdatedEventListener(filter) {
      @Override
      public void eventFired(final Collection<CommandInfo> commands) {
        reloadTable(commands);
      }

      @Override
      protected void eventInterrupted(Exception exception, String description) {
        notifyObservers(new MonitoringTableEvent(
          MonitoringTableEvent.REMOTE_ERROR));
      }
    };
    this.observers = new HashSet<MonitoringTableObserver>();
    this.setSelectionMode(selectionMode);
    this.model =
      new ObjectTableModel<CommandMonitoringSet>(new Vector<CommandMonitoringSet>(),
        getObjectTableProvider());
    this.setModel(model);
    this.setRowSelectionAllowed(true);
    
    TableColumnModel columnModel = this.getColumnModel();
    for (int inx = 0; inx < columnModel.getColumnCount(); inx++) {      
      TableColumn column = columnModel.getColumn(inx);
      TableCellRenderer renderer = createTableCellRenderer(inx);
      if (null != renderer) {
        column.setCellRenderer(renderer);
      }
      TableCellEditor editor = createTableCellEditor(inx);
      if (null != editor) {
        column.setCellEditor(editor);
      }      
    }
  }

  /**
   * Altera o filtro que indica que tipo de comandos devem aparecer na tabela.
   * 
   * @param filter o filtro.
   */
  public void setFilter(CommandsFilter filter) {
    listener.setFilter(filter);
  }

  /**
   * Inicia a atualizao.
   */
  public void start() {
    if (running.compareAndSet(false, true)) {
      initializeData(this.listener.getFilter());
      CommandsCache.getInstance().addEventListener(listener);
    }
  }

  /**
   * Interrompe a atualizao.
   */
  public void stop() {
    if (running.compareAndSet(true, false)) {
      CommandsCache.getInstance().removeEventListener(this.listener);
    }
  }

  /**
   * Retorna um objeto representando a linha especificada.
   * 
   * @param modelRow ndice da linha no modelo da tabela.
   * @return objeto representando a linha especificada.
   */
  public CommandMonitoringSet getMonitoringSet(int modelRow) {
    return (CommandMonitoringSet)model.getRow(modelRow);
  }

  /**
   * Adiciona um observador  lista de observadores da tabela.
   * 
   * @param o observador.
   */
  public void addObserver(MonitoringTableObserver o) {
    observers.add(o);
  }

  /**
   * Remove um observador da lista de observadores da tabela.
   * 
   * @param o observador.
   */
  public void removeObserver(MonitoringTableObserver o) {
    observers.remove(o);
  }
  
  /**
   * Retorna um objeto contendo informaes sobre o comando correspondente 
   * linha especificada.
   * 
   * @param viewRow ndice de linha na viso tabela.
   * 
   * @return objeto contendo informaes sobre um comando.
   */
  public CommandInfo getCommandInfo(int viewRow) {
    int modelRow = convertRowIndexToModel(viewRow);
    return getMonitoringSet(modelRow).getCommandInfo();
  }
  
  /**
   * Obtm os comandos selecionados.
   * 
   * @return Um array contendo os comandos selecionados.
   */
  public CommandInfo[] getSelectedCommands() {
    int[] rows = getSelectedRows();
    CommandInfo[] commands = new CommandInfo[rows.length];
    for (int inx = 0; inx < rows.length; inx++) {      
      commands[inx] = getCommandInfo(rows[inx]);
    }
    
    return commands;
  }

  protected TableCellRenderer createTableCellRenderer(int columnIndex) {
    return null;
  }
  
  protected TableCellEditor createTableCellEditor(int columnIndex) {
    return null;
  }
  
  /**
   * Mtodo abstrato para obter o provedor de dados/metadados da tabela.
   * 
   * @return provedor de dados/metadados da tabela.
   */
  protected abstract ObjectTableProvider getObjectTableProvider();

  private List<CommandMonitoringSet> createMonitoringSets(
    Collection<CommandInfo> commands) {
    List<CommandMonitoringSet> monitoringSets = new ArrayList<CommandMonitoringSet>();
    for (CommandInfo command : commands) {
      monitoringSets.add(new CommandMonitoringSet(command));
    }

    return monitoringSets;
  }
    
  private void initializeData(final CommandsFilter filter) {
    RemoteTask<Collection<CommandInfo>> task =
      new RemoteTask<Collection<CommandInfo>>() {
        @Override
        protected void performTask() throws Exception {
          Collection<CommandInfo> data =
            CommandsCache.getInstance().getCommands(filter);
          setResult(data);
        }
      };
    Window window = SwingUtilities.windowForComponent(this);
    String taskTitle = LNG.get("AbstractCommandMonitoringTable.title");
    String taskMessage = LNG.get("AbstractCommandMonitoringTable.message");
    if (task.execute(window, taskTitle, taskMessage)) {
      reloadTable(task.getResult());
    }
  }

  private void reloadTable(Collection<CommandInfo> commands) {
    final List<CommandMonitoringSet> monitoringSets = createMonitoringSets(commands);
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        setTableData(monitoringSets);
        notifyObservers(new MonitoringTableEvent(MonitoringTableEvent.REFRESH));
      }
    });
  }

  /**
   * Atualiza os dados da tabela de monitorao. Este mtodo deve ser chamado na
   * thread no Swing.
   * 
   * @param data os dados a serem apresentados na tabela.
   */
  private void setTableData(List<CommandMonitoringSet> data) {
    Vector<String> selectedKeys = null;

    // Guarda as chaves das linhas atualmente selecionadas na tabela.
    if (data.size() > 0 && model.getRowCount() > 0) {
      selectedKeys = new Vector<String>();
      int[] selectedRows = getSelectedRows();
      if ((selectedRows != null) && (selectedRows.length > 0)) {
        for (int i = 0; i < selectedRows.length; i++) {
          int tableIndex = selectedRows[i];
          int modelIndex = convertRowIndexToModel(tableIndex);
          if (modelIndex >= 0) {
            MonitoringSet selectedSet = getMonitoringSet(modelIndex);
            selectedKeys.add(selectedSet.getKey());
          }
        }
      }
    }

    /*
     * Atualiza as linhas da tabela. O mtodo setRows j chama o mtodo
     * fireTableChanged(new TableModelEvent(this)) que repassa a notificao
     * para todos os listeners da tabela informando que todo contedo da tabela
     * foi modificada.
     */
    model.setRows(data);

    if (selectedKeys == null) {
      return;
    }

    // Procura os ndices correspondentes  antiga seleo nas linhas
    // da tabela aps a atualizao.
    for (String selectedKey : selectedKeys) {
      for (int inx = 0; inx < data.size(); inx++) {
        MonitoringSet newSet = data.get(inx);
        if (newSet.getKey().equals(selectedKey)) {
          int viewIndex = convertRowIndexToView(inx);
          // Seleciona as linhas de ndices encontrados para
          // restaurar a seleo antes da atualizao.
          addRowSelectionInterval(viewIndex, viewIndex);
          break;
        }
      }
    }
  }

  /**
   * Notifica os observadores da tabela da ocorrncia de um evento.
   * 
   * @param event evento a ser notificado.
   */
  private void notifyObservers(MonitoringTableEvent event) {
    for (MonitoringTableObserver obs : observers) {
      obs.update(event);
    }
  }
}
