/*
 * PermissionInfoDialog.java
 * 
 * $Id: PermissionInfoDialog.java 98336 2009-10-30 21:56:25Z costa $
 */
package csbase.client.ias;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.lang.reflect.Constructor;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import javax.swing.AbstractListModel;
import javax.swing.BorderFactory;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.BevelBorder;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.StandardDialogs;
import tecgraf.javautils.gui.selector.ContainerSelection;
import tecgraf.javautils.gui.selector.ContainerSelectionListener;
import tecgraf.javautils.gui.table.ObjectTableProvider;
import csbase.client.desktop.DesktopComponentDialog;
import csbase.client.desktop.RemoteTask;
import csbase.client.remote.srvproxies.PermissionProxy;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.AttributesPermission;
import csbase.logic.ChoicePermission;
import csbase.logic.Permission;
import csbase.logic.UserPasswordPermission;

/**
 * A classe <code>PermissionInfoDialog</code>  responsvel por incluir ou
 * modificar as informaes de uma permisso, atravs de uma interface
 * especfica.
 * 
 * @author Marcus Leal
 * @version $Revision: 98336 $
 */
public class PermissionInfoDialog {
  /** Classe genrica para permisses simples */
  private static final String SIMPLE_PERMISSION_CLASS =
    "csbase.logic.SimplePermission";

  /** Classe genrica para permisses com atributos */
  private static final String ATTRIBUTES_PERMISSION_CLASS =
    "csbase.logic.AttributesPermission";

  /** Classe genrica para permisses com usurio/senha */
  private static final String USER_PASSWORD_PERMISSION_CLASS =
    "csbase.logic.UserPasswordPermission";

  /** Classe genrica para permisses com atributos dinmicos. */
  private static final String CHOICE_PERMISSION_CLASS =
    "csbase.logic.ChoicePermission";

  /**
   * Sufixo para localizao da descrio da classe de permisso no arquivo de
   * propriedades.
   */
  private static final String DESCRIPTION_SUFFIX = ".desc";

  /**
   * Indica que todos os campos esto validados e prontos para serem
   * persistidos.
   */
  private static final int FIELDS_OK = 0;

  /** Indica que existem campos no preenchidos */
  private static final int EMPTY_FIELDS = -1;

  /** Indica que a senha no confere com a confirmao da mesma */
  private static final int PASSWORDS_DONT_MATCH = -2;

  /** Janela pai */
  private Window owner;

  /** Ttulo da janela */
  private String windowLabel;

  /**
   * Objeto a ser usado para persistir os dados de permisso entrados pelo
   * usurio.
   */
  private Permission permission;

  /** A janela de insero/edio de permisses */
  private DesktopComponentDialog window;

  /** Campo nome da permisso */
  private JTextField name;

  /** Campo descrio da permisso */
  private JTextField description;

  /** Label login (opcional) */
  private JLabel loginL;

  /** Campo login (opcional) */
  private JTextField login;

  /** Label senha (opcional) */
  private JLabel passwordL;

  /** Campo senha (opcional) */
  private JPasswordField password;

  /** Label confirmao de senha (opcional) */
  private JLabel confirmPasswordL;

  /** Campo confirmao de senha (opcional) */
  private JPasswordField confirmPassword;

  /** Label para entrada de novos atributos (opcional) */
  private JLabel newAttributeL;

  /** Campo para entrada de novos atributos (opcional) */
  private JTextField newAttribute;

  /** Modelo para a lista de atributos (opcional) */
  private SortedListModel attribModel;

  /** Map com atributos para criao/alterao de uma ChoicePermission */
  private Map map;

  /** Objeto para percorrer os atributos de uma ChoicePermission */
  AttributesNavigator navigator;

  /** Combo para seleo do nome do atributo de uma ChoicePermission */
  private JLabel attributeName;

  /** Chave que mapeia o texto que ser exibido em {@link #attributeName} */
  private String attributeNameKey;

  /**
   * Combo para seleo do valor para um atributo de uma ChoicePermission, caso
   * este seja exclusivo
   */
  private JComboBox attributeValue;

  /**  */
  private Map attributeRange;

  /** Container para seleo de mltiplos valores de uma ChoicePermission. */
  private ContainerSelection containerSelection;

  /** Boto para navegar pelos atributos de uma ChoicePermission */
  private JButton nextButton;

  /** Boto para navegar pelos atributos de uma ChoicePermission */
  private JButton previousButton;

  /** Componente para scroll da lista de atributos (opcional) */
  private JScrollPane scrollList;

  /** Combo de seleo da classe de permisso */
  private JComboBox permissionClass;

  /** Campo de descrio da classe de permisso */
  private JTextArea permissionClassDesc;

  /** Boto de incluso de nova permisso */
  private JButton includeButton;

  /** Boto de incluso/alterao da permisso */
  private JButton includePermissionButton;

  /** Boto de remoo de permisso */
  private JButton removeButton;

  /** String que indica qual a classe genrica atualmente selecionada */
  private String selectedPermissionClass;

  /** Indica se uma nova permisso est sendo criada */
  private boolean isNew;

  /**
   * Cria uma janela para incluso de dados de uma nova permisso.
   * 
   * @param owner janela que originou esta PermissionInfoDialog.
   */
  public PermissionInfoDialog(Window owner) {
    this(owner, null);
  }

  /**
   * Cria uma janela para modificao de dados de uma permisso j cadastrada no
   * sistema, ou de incluso de nova permisso, caso o parmetro permission seja
   * nulo.
   * 
   * @param owner janela que originou esta PermissionInfoDialog.
   * @param permission a permisso que vai ser modificada
   */
  public PermissionInfoDialog(Window owner, Permission permission) {
    this.owner = owner;
    if (permission == null) {
      isNew = true;
      windowLabel = LNG.get("IAS_PERMISSION_INCLUSION_TITLE");
    }
    else {
      windowLabel = LNG.get("IAS_PERMISSION_UPDATE_TITLE");
      isNew = false;
      this.permission = permission;
    }
    display();
  }

  /**
   * Cria e mostra a janela de incluso/modificao da permisso.
   */
  protected void display() {
    window = new DesktopComponentDialog((JFrame) owner, windowLabel);

    // Cria o painel de fundo
    JPanel backPane = new JPanel(new BorderLayout());
    backPane.add(makeButtonPanel(), BorderLayout.SOUTH);
    // Cria o painel com os campos e o boto limpar.
    JPanel mainPane = new JPanel(new BorderLayout());
    mainPane.setBorder(BorderFactory.createTitledBorder(LNG
      .get("IAS_PERMISSION_REGISTRATION")));
    mainPane.add(makeMainPanel(), BorderLayout.CENTER);
    mainPane.add(makePermissionsButtonPanel(), BorderLayout.SOUTH);
    backPane.add(mainPane);
    // Cria a janela
    window.getContentPane().add(backPane);
    window.setDefaultCloseOperation(DesktopComponentDialog.DISPOSE_ON_CLOSE);
    window.pack();
    window.center(owner);
    window.setVisible(true);
  }

  /**
   * Constri o painel para preenchimento ou alterao dos dados da permisso de
   * usurio. De acordo com a classe de permisso que for selecionada, dois
   * painis opcionais podero ou no ser exibidos: o painel de usurio/senha e
   * o painel de atributos (OBS: neste caso, o termo "painel" no representa um
   * JPanel, mas um simples agrupamento lgico de campos).
   * 
   * @return o painel principal da janela
   */
  protected Container makeMainPanel() {
    GridBagConstraints gbc = new GridBagConstraints();
    GridBagLayout gbl = new GridBagLayout();
    Container container = new JPanel();
    container.setLayout(gbl);
    // Painel principal
    int lastLineIndex = makeCommonPanel(gbc, gbl, container);
    // Painel opcional de usurio/senha
    makeUserPasswordPanel(gbc, gbl, container, lastLineIndex);
    // Painel opcional de atributos
    makeAttributesPanel(gbc, gbl, container, lastLineIndex);
    // Painel opcional de atributos dinmicos
    makeChoicePermissionPanel(gbc, gbl, container, lastLineIndex);
    // Inicializa os campos
    initFields();
    return container;
  }

  /**
   * Constri o grupo de componentes requeridos para todas as permisses.
   * 
   * @param gbc GrigBagConstraints para a janela inteira
   * @param gbl GridBagLayout sendo usado na janela inteira
   * @param container Container de todos os componentes da janela
   * 
   * @return ndice da ltima linha adicionada  janela.
   */
  private int makeCommonPanel(GridBagConstraints gbc, GridBagLayout gbl,
    Container container) {
    int lineIndex = 0;

    // Cria os componentes
    JLabel nameL = new JLabel(LNG.get("IAS_PERMISSION"));
    name = new JTextField(20);
    JLabel descriptionL = new JLabel(LNG.get("IAS_DESCRIPTION"));
    description = new JTextField(20);
    JLabel permissionClassL = new JLabel(LNG.get("IAS_PERMISSION_CLASS"));
    permissionClass = makeClassComboBox();
    permissionClass.setSelectedIndex(-1);
    permissionClassDesc = makePermissionClassDescription(permissionClass);
    // Label permisso
    gbc.insets = new Insets(10, 10, 5, 5); // tlbr
    gbc.gridx = 0;
    gbc.gridy = lineIndex;
    gbc.weightx = 0;
    gbc.weighty = 0;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.anchor = GridBagConstraints.NORTHWEST;
    gbc.fill = GridBagConstraints.NONE;
    gbl.setConstraints(nameL, gbc);
    container.add(nameL);
    // Campo permisso
    gbc.insets = new Insets(5, 0, 0, 10);
    gbc.gridx = 1;
    gbc.gridy = lineIndex;
    gbc.weightx = 1;
    gbc.weighty = 0;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.anchor = GridBagConstraints.NORTHWEST;
    gbc.fill = GridBagConstraints.BOTH;
    gbl.setConstraints(name, gbc);
    container.add(name);
    // Label descrio
    gbc.insets = new Insets(5, 10, 5, 5);
    gbc.gridx = 0;
    gbc.gridy = ++lineIndex;
    gbc.weightx = 0;
    gbc.weighty = 0;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.anchor = GridBagConstraints.NORTHWEST;
    gbc.fill = GridBagConstraints.NONE;
    gbl.setConstraints(descriptionL, gbc);
    container.add(descriptionL);
    // Campo descrio
    gbc.insets = new Insets(0, 0, 0, 10);
    gbc.gridx = 1;
    gbc.gridy = lineIndex;
    gbc.weightx = 1;
    gbc.weighty = 0;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.anchor = GridBagConstraints.NORTHWEST;
    gbc.fill = GridBagConstraints.BOTH;
    gbl.setConstraints(description, gbc);
    container.add(description);
    // Label classe de permisso
    gbc.insets = new Insets(5, 10, 5, 5);
    gbc.gridx = 0;
    gbc.gridy = ++lineIndex;
    gbc.weightx = 0;
    gbc.weighty = 0;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.anchor = GridBagConstraints.NORTHWEST;
    gbc.fill = GridBagConstraints.NONE;
    gbl.setConstraints(permissionClassL, gbc);
    container.add(permissionClassL);
    // Campo classe de permisso
    gbc.insets = new Insets(0, 0, 0, 10);
    gbc.gridx = 1;
    gbc.gridy = lineIndex;
    gbc.weightx = 1;
    gbc.weighty = 0;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.anchor = GridBagConstraints.NORTHWEST;
    gbc.fill = GridBagConstraints.BOTH;
    gbl.setConstraints(permissionClass, gbc);
    container.add(permissionClass);
    // Campo descrio da classe de permisso
    gbc.insets = new Insets(0, 10, 5, 10);
    gbc.gridx = 0;
    gbc.gridy = ++lineIndex;
    gbc.weightx = 1;
    gbc.weighty = 2;
    gbc.gridwidth = 2;
    gbc.gridheight = 2;
    lineIndex++;
    gbc.anchor = GridBagConstraints.NORTHWEST;
    gbc.fill = GridBagConstraints.BOTH;
    JScrollPane scrollDesc = new JScrollPane(permissionClassDesc);
    gbl.setConstraints(scrollDesc, gbc);
    container.add(scrollDesc);
    return lineIndex;
  }

  /**
   * Constri o componente com a lista de classes de permisso disponveis.
   * 
   * @return o componente que mostra todas as classes de permisso
   */
  private JComboBox makeClassComboBox() {
    final JComboBox combo = new JComboBox();
    Vector classes =
      PermissionProxy.getPermissionClasses(window, windowLabel, LNG
        .get("IAS_WAITING_ALL_PERMISSION_CLASSES"));
    Vector classesNames = null;
    try {
      classesNames = makePermissionClassNames(classes);
      Collections.sort(classesNames);
      int classesSize = classesNames.size();
      for (int i = 0; i < classesSize; i++) {
        combo.addItem(classesNames.get(i));
      }
      if (classesSize > 0) {
        combo.setSelectedIndex(0);
      }
      combo.addItemListener(new ItemListener() {
        public void itemStateChanged(ItemEvent evt) {
          if (combo.getSelectedIndex() > -1) {
            Object item = evt.getItem();
            int stateChange = evt.getStateChange();
            classStateChanged(item, stateChange);
          }
        }
      });
      combo.setEnabled(isNew);
    }
    catch (Exception e) {
      combo.setEditable(false);
      StandardErrorDialogs.showErrorDialog(window, LNG.get("ERRO") + " - "
        + windowLabel, e);
    }
    return combo;
  }

  /**
   * Dado um Vector de classes de permisso, retorna um Vector com Wrappers de
   * classes de permisso, com os dados relevantes para exibio.
   * 
   * @param listClasses Vector contendo as classes de permisso
   * 
   * @return Vector contendo os Wrappers de classes de permisso
   * 
   * @throws ClassNotFoundException caso ocorra problema na instanciao do
   *         Wrapper de permisso.
   */
  private Vector makePermissionClassNames(Vector listClasses)
    throws ClassNotFoundException {
    if (listClasses == null) {
      return null;
    }
    Vector listClassNames = new Vector();
    for (int i = 0; i < listClasses.size(); i++) {
      listClassNames.add(new PermissionClassName((String) listClasses.get(i)));
    }
    return listClassNames;
  }

  /**
   * Atualiza a exibio dos campos da tabela de acordo com o evento ocorrido
   * com o combo de classes de permisso. Se uma nova classe tiver sido
   * selecionada, exibe os campos correspondentes; se uma classe j selecionada
   * tiver sido desmarcada, esconde os campos correspondentes (OBS: sempre que o
   * usurio seleciona uma classe de permisso, dois eventos so disparados -
   * evento classe anterior desmarcada e evento nova classe selecionada).
   * 
   * @param item Classe de permisso selecionada
   * @param stateChange Indica qual tipo de evento ocorreu (classe selecionada e
   *        classe desmarcada so eventos de interesse)
   */
  private void classStateChanged(Object item, int stateChange) {
    PermissionClassName permissionClassName = (PermissionClassName) item;
    if (permissionClassName == null) {
      return;
    }
    // Atualiza a descrio da classe de permisso
    permissionClassDesc
      .setText(getPermissionClassDescription(permissionClassName));
    selectedPermissionClass = permissionClassName.getSuperClassName();
    // De acordo com o item selecionado, exibe/oculta o "painel" associado
    if (stateChange == ItemEvent.SELECTED) {
      if (selectedPermissionClass.equals(ATTRIBUTES_PERMISSION_CLASS)) {
        setAttributesPanelVisible(true);
      }
      else if (selectedPermissionClass.equals(USER_PASSWORD_PERMISSION_CLASS)) {
        setUserPasswordPanelVisible(true);
      }
      else if (selectedPermissionClass.equals(CHOICE_PERMISSION_CLASS)) {
        setChoicePermissionPanelVisible(true);
      }
    }
    else if (stateChange == ItemEvent.DESELECTED) {
      if (selectedPermissionClass.equals(ATTRIBUTES_PERMISSION_CLASS)) {
        setAttributesPanelVisible(false);
      }
      else if (selectedPermissionClass.equals(USER_PASSWORD_PERMISSION_CLASS)) {
        setUserPasswordPanelVisible(false);
      }
      else if (selectedPermissionClass.equals(CHOICE_PERMISSION_CLASS)) {
        setChoicePermissionPanelVisible(false);
      }
    }
    if (window != null) {
      window.pack();
    }
  }

  /**
   * Obtm a descrio da superclasse de permisso (ou classe genrica) a partir
   * do arquivo de propriedades.
   * 
   * @param item Wrapper para exibio de uma permisso
   * 
   * @return descrio textual da classe genrica
   */
  private String getPermissionClassDescription(PermissionClassName item) {
    String desc = "";
    if (item != null) {
      desc = LNG.get(item.getClassName() + DESCRIPTION_SUFFIX);
    }
    return desc;
  }

  /**
   * Cria um JTextArea contendo a descrio para a classe de permisso
   * atualmente selecionada no combo passado como parmetro.
   * 
   * @param combo JComboBox no qual a classe de permisso a ser descrita est
   *        selecionada.
   * 
   * @return JTextArea com a descrio da classe de permisso atualmente
   *         selecionada.
   */
  private JTextArea makePermissionClassDescription(JComboBox combo) {
    JTextArea textArea = new JTextArea(5, 30);
    textArea.setLineWrap(true);
    textArea.setWrapStyleWord(true);
    textArea.setEditable(false);
    textArea.setBorder(new BevelBorder(BevelBorder.LOWERED));
    textArea.setFont(textArea.getFont().deriveFont(Font.BOLD));
    return textArea;
  }

  /**
   * Constri o grupo de componentes de usurio/senha, somente visveis quando
   * uma classe filha da classe genrica de usurio/senha estiver selecionada.
   * 
   * @param gbc GrigBagConstraints para a janela inteira
   * @param gbl GridBagLayout sendo usado na janela inteira
   * @param container Container de todos os componentes da janela
   * @param lastLineIndex ltimo ndice de linha em que foram dispostos
   *        componentes na janela.
   */
  private void makeUserPasswordPanel(GridBagConstraints gbc, GridBagLayout gbl,
    Container container, int lastLineIndex) {
    // Cria os componentes
    loginL = new JLabel(LNG.get("IAS_USER"));
    login = new JTextField(20);
    passwordL = new JLabel(LNG.get("IAS_PERMISSION_PASSWORD"));
    password = new JPasswordField(20);
    confirmPasswordL = new JLabel(LNG.get("IAS_CONFIRM_PERMISSION_PASSWORD"));
    confirmPassword = new JPasswordField(20);
    // Label usurio
    gbc.insets = new Insets(5, 10, 5, 5);
    gbc.gridx = 0;
    gbc.gridy = ++lastLineIndex;
    gbc.weightx = 0;
    gbc.weighty = 0;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.anchor = GridBagConstraints.NORTHWEST;
    gbc.fill = GridBagConstraints.NONE;
    gbl.setConstraints(loginL, gbc);
    container.add(loginL);
    // Campo usurio
    gbc.insets = new Insets(0, 0, 0, 10);
    gbc.gridx = 1;
    gbc.gridy = lastLineIndex;
    gbc.weightx = 1;
    gbc.weighty = 0;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.anchor = GridBagConstraints.NORTHWEST;
    gbc.fill = GridBagConstraints.BOTH;
    gbl.setConstraints(login, gbc);
    container.add(login);
    // Label senha
    gbc.insets = new Insets(5, 10, 5, 5);
    gbc.gridx = 0;
    gbc.gridy = ++lastLineIndex;
    gbc.weightx = 0;
    gbc.weighty = 0;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.anchor = GridBagConstraints.NORTHWEST;
    gbc.fill = GridBagConstraints.NONE;
    gbl.setConstraints(passwordL, gbc);
    container.add(passwordL);
    // Campo senha
    gbc.insets = new Insets(0, 0, 0, 10);
    gbc.gridx = 1;
    gbc.gridy = lastLineIndex;
    gbc.weightx = 1;
    gbc.weighty = 0;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.anchor = GridBagConstraints.NORTHWEST;
    gbc.fill = GridBagConstraints.BOTH;
    gbl.setConstraints(password, gbc);
    container.add(password);
    // Label confirmao de senha
    gbc.insets = new Insets(5, 10, 10, 5);
    gbc.gridx = 0;
    gbc.gridy = ++lastLineIndex;
    gbc.weightx = 0;
    gbc.weighty = 0;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.anchor = GridBagConstraints.NORTHWEST;
    gbc.fill = GridBagConstraints.NONE;
    gbl.setConstraints(confirmPasswordL, gbc);
    container.add(confirmPasswordL);
    // Campo confirmao de senha
    gbc.insets = new Insets(0, 0, 5, 10);
    gbc.gridx = 1;
    gbc.gridy = lastLineIndex;
    gbc.weightx = 1;
    gbc.weighty = 0;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.anchor = GridBagConstraints.NORTHWEST;
    gbc.fill = GridBagConstraints.BOTH;
    gbl.setConstraints(confirmPassword, gbc);
    container.add(confirmPassword);
    // Esconde os componentes (na inicializao)
    setUserPasswordPanelVisible(false);
  }

  /**
   * Exibe/oculta os componentes especficos de permisses que requerem
   * usurio/senha.
   * 
   * @param visible indica se os componentes devem estar ou no visveis
   */
  private void setUserPasswordPanelVisible(boolean visible) {
    loginL.setVisible(visible);
    login.setVisible(visible);
    passwordL.setVisible(visible);
    password.setVisible(visible);
    confirmPasswordL.setVisible(visible);
    confirmPassword.setVisible(visible);
  }

  /**
   * Exibe/oculta os componentes especficos de permisses que requerem
   * atributos.
   * 
   * @param visible indica se os componentes devem estar ou no visveis
   */
  private void setAttributesPanelVisible(boolean visible) {
    newAttributeL.setVisible(visible);
    newAttribute.setVisible(visible);
    scrollList.setVisible(visible);
    includeButton.setVisible(visible);
    removeButton.setVisible(visible);
  }

  /**
   * Constri o grupo de componentes de atributos, somente visveis quando uma
   * classe filha da classe genrica de atributos estiver selecionada.
   * 
   * @param gbc GrigBagConstraints para a janela inteira
   * @param gbl GridBagLayout sendo usado na janela inteira
   * @param container Container de todos os componentes da janela
   * @param lastLineIndex ltimo ndice de linha em que foram dispostos
   *        componentes na janela.
   */
  private void makeAttributesPanel(GridBagConstraints gbc, GridBagLayout gbl,
    Container container, int lastLineIndex) {
    // Criao de componentes
    newAttributeL = new JLabel(LNG.get("IAS_PERMISSION_NEW_ATTRIBUTE"));
    newAttribute = new JTextField(20);
    attribModel = new SortedListModel();
    final JList attributesList = new JList(attribModel);
    attributesList.setBorder(new BevelBorder(BevelBorder.LOWERED));
    scrollList = new JScrollPane(attributesList);
    includeButton = new JButton(LNG.get("IAS_ADD"));
    includeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        String text = newAttribute.getText().trim();
        if (!text.equals("")) {
          attribModel.add(text);
        }
        newAttribute.setText("");
      }
    });
    removeButton = new JButton(LNG.get("IAS_REMOVE"));
    removeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        int[] indexes = attributesList.getSelectedIndices();
        if ((indexes != null) && (indexes.length != 0)) {
          for (int i = indexes.length - 1; i >= 0; i--) {
            attribModel.remove(indexes[i]);
          }
        }
      }
    });
    // Label atributos
    gbc.insets = new Insets(5, 10, 5, 5);
    gbc.gridx = 0;
    gbc.gridy = ++lastLineIndex;
    gbc.weightx = 0;
    gbc.weighty = 0;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.anchor = GridBagConstraints.NORTHWEST;
    gbc.fill = GridBagConstraints.NONE;
    gbl.setConstraints(newAttributeL, gbc);
    container.add(newAttributeL);
    // Campo atributo
    gbc.insets = new Insets(0, 0, 5, 10);
    gbc.gridx = 1;
    gbc.gridy = lastLineIndex;
    gbc.weightx = 1;
    gbc.weighty = 0;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.anchor = GridBagConstraints.NORTHWEST;
    gbc.fill = GridBagConstraints.BOTH;
    gbl.setConstraints(newAttribute, gbc);
    container.add(newAttribute);
    // Boto incluir
    gbc.insets = new Insets(5, 10, 5, 5);
    gbc.gridx = 0;
    gbc.gridy = ++lastLineIndex;
    gbc.weightx = 0;
    gbc.weighty = 0;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.anchor = GridBagConstraints.NORTHWEST;
    gbc.fill = GridBagConstraints.BOTH;
    gbl.setConstraints(includeButton, gbc);
    container.add(includeButton);
    // JTextArea lista de atributos
    gbc.insets = new Insets(0, 0, 5, 10);
    gbc.gridx = 1;
    gbc.gridy = lastLineIndex;
    gbc.weightx = 1;
    gbc.weighty = 0;
    gbc.gridwidth = 1;
    gbc.gridheight = 3;
    gbc.anchor = GridBagConstraints.NORTHWEST;
    gbc.fill = GridBagConstraints.BOTH;
    gbl.setConstraints(scrollList, gbc);
    container.add(scrollList);
    // Boto remover
    gbc.insets = new Insets(5, 10, 5, 5);
    gbc.gridx = 0;
    gbc.gridy = ++lastLineIndex;
    gbc.weightx = 0;
    gbc.weighty = 0;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.anchor = GridBagConstraints.NORTHWEST;
    gbc.fill = GridBagConstraints.BOTH;
    gbl.setConstraints(removeButton, gbc);
    container.add(removeButton);
    // Esconde os componentes (na inicializao)
    setAttributesPanelVisible(false);
  }

  private void makeChoicePermissionPanel(GridBagConstraints gbc,
    GridBagLayout gbl, Container container, int lastLineIndex) {
    if (isNew) {
      map = new HashMap();
    }
    else {
      if (permission instanceof ChoicePermission) {
        map = new HashMap(((ChoicePermission) permission).getAttributes());
      }
    }
    nextButton = new JButton(LNG.get("IAS_NEXT"));
    previousButton = new JButton(LNG.get("IAS_PREVIOUS"));
    navigator = new AttributesNavigator();
    nextButton.setVisible(false);
    nextButton.setEnabled(true);
    previousButton.setVisible(false);
    previousButton.setEnabled(true);
    nextButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        navigator.next();
      }
    });
    previousButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        navigator.previous();
      }
    });
    attributeName = new JLabel();
    attributeValue = new JComboBox();
    attributeValue.setSelectedIndex(-1);
    attributeValue.setVisible(false);
    attributeValue.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent a) {
        if (attributeValue.getSelectedIndex() != -1) {
          ArrayList value = new ArrayList();
          value.add(attributeRange.get(attributeValue.getSelectedItem()));
          map.put(attributeNameKey, value);
          nextButton.setEnabled(true);
          includePermissionButton.setEnabled(!nextButton.isVisible());
        }
        else {
          nextButton.setEnabled(false);
          includePermissionButton.setEnabled(false);
        }
        if (!isNew) {
          checkDataChange();
        }
      }
    });
    this.containerSelection =
      new ContainerSelection(new TableItemsFormatter(),
        new SelectedItemsFormatter(), true, false);
    this.containerSelection.adjustTableColumns();
    this.containerSelection
      .addContainerSelectionListener(new ContainerSelectionListener() {
        public void containerChanged() {
          List<Object> selectedItems = containerSelection.getSelectedItems();
          if (selectedItems.size() > 0) {
            ArrayList values = new ArrayList();
            for (int i = 0; i < selectedItems.size(); i++) {
              values.add(attributeRange.get(selectedItems.get(i)));
            }
            map.put(attributeNameKey, values);
            nextButton.setEnabled(true);
            includePermissionButton.setEnabled(!nextButton.isVisible());
          }
          else {
            nextButton.setEnabled(false);
            includePermissionButton.setEnabled(false);
          }
          if (!isNew) {
            checkDataChange();
          }
        }
      });
    this.containerSelection.getPanel();
    // Label com o nome do atributo
    gbc.gridx = 0;
    gbc.gridy = ++lastLineIndex;
    gbc.weightx = 1;
    gbc.weighty = 1;
    gbc.gridwidth = 2;
    gbc.gridheight = 1;
    gbc.fill = GridBagConstraints.BOTH;
    container.add(this.createChoicePermissionPanel(), gbc);
    // Esconde os componentes (na inicializao)
    setChoicePermissionPanelVisible(false);
  }

  private JPanel createChoicePermissionPanel() {
    JPanel choicePermissionPanel = new JPanel(new GridBagLayout());
    GridBagConstraints gbc = new GridBagConstraints();
    // Label com o nome do atributo
    gbc.insets = new Insets(5, 5, 5, 5);
    gbc.gridx = 0;
    gbc.gridy = 0;
    gbc.weightx = 0;
    gbc.weighty = 0;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    choicePermissionPanel.add(attributeName, gbc);
    // Combo com os valores do atributo
    gbc.insets = new Insets(5, 5, 5, 5);
    gbc.gridx = 1;
    gbc.gridy = 0;
    gbc.weightx = 1;
    gbc.weighty = 0;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.fill = GridBagConstraints.BOTH;
    choicePermissionPanel.add(attributeValue, gbc);
    // ContainerSelection com os valores do atributo
    gbc.insets = new Insets(5, 5, 5, 5);
    gbc.gridx = 0;
    gbc.gridy = 0;
    gbc.weightx = 1;
    gbc.weighty = 1;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.fill = GridBagConstraints.BOTH;
    choicePermissionPanel.add(this.containerSelection.getPanel(), gbc);
    return choicePermissionPanel;
  }

  private void setChoicePermissionPanelVisible(boolean visible) {
    attributeName.setVisible(visible);
    this.containerSelection.getPanel().setVisible(visible);
    includePermissionButton.setEnabled(!visible);
    if (visible) {
      navigator.init();
    }
    else {
      nextButton.setVisible(false);
      previousButton.setVisible(false);
      attributeValue.setVisible(false);
    }
  }

  /**
   * Inicializa os campos da janela com os dados da permisso a ser modificada
   * (a no ser que uma nova permisso esteja sendo criada). Atualiza a exibio
   * dos "painis" (grupamento de campos) opcionais.
   */
  protected void initFields() {
    // Listener para verificar alteraes do usurio nos campos da permisso
    KeyListener keyChangeListener = new KeyAdapter() {
      @Override
      public void keyReleased(KeyEvent e) {
        checkDataChange();
      }
    };
    if (!isNew) {
      // Preenche os campos principais
      name.setText(permission.getName());
      description.setText(permission.getDescription());
      PermissionClassName className = null;
      try {
        className = new PermissionClassName(permission.getClass().getName());
      }
      catch (ClassNotFoundException e) {
        StandardErrorDialogs.showErrorDialog(window, LNG.get("ERRO") + " - "
          + windowLabel, e);
      }
      if (className != null) {
        permissionClass.setSelectedItem(className);
      }
      name.addKeyListener(keyChangeListener);
      description.addKeyListener(keyChangeListener);
    }
    // Decide se os painis opcionais devem ou no ser mostrados
    classStateChanged(permissionClass.getSelectedItem(), ItemEvent.SELECTED);
    if (!isNew) {
      if (selectedPermissionClass.equals(USER_PASSWORD_PERMISSION_CLASS)) {
        // Preenche os campos de usurio / senha
        UserPasswordPermission userPwdPermission =
          (UserPasswordPermission) permission;
        login.setText(userPwdPermission.getUser());
        login.addKeyListener(keyChangeListener);
        password.addKeyListener(keyChangeListener);
      }
      else if (selectedPermissionClass.equals(ATTRIBUTES_PERMISSION_CLASS)) {
        // Preenche a lista de atributos
        AttributesPermission attribPermission =
          (AttributesPermission) permission;
        String[] attributes = attribPermission.getAttributes();
        for (int i = 0; i < attributes.length; i++) {
          attribModel.add(attributes[i]);
        }
        attribModel.addListDataListener(new ListDataListener() {
          public void intervalAdded(ListDataEvent e) {
            checkDataChange();
          }

          public void intervalRemoved(ListDataEvent e) {
            checkDataChange();
          }

          public void contentsChanged(ListDataEvent e) {
          }
        });
      }
    }
  }

  /**
   * Cria um painel contendo o boto limpar.
   * 
   * @return o painel criado com o boto de limpar campos
   */
  private JPanel makePermissionsButtonPanel() {
    JPanel panel = new JPanel();
    panel.add(makeClearButton());
    panel.add(previousButton);
    panel.add(nextButton);
    return panel;
  }

  /**
   * Constri o painel com os botes da janela de incluso/ modificao de
   * grupos de usurio. As aes previstas so para confirmar a operao
   * solicitada ou desistir da operao.
   * 
   * @return o painel de botes
   */
  protected JPanel makeButtonPanel() {
    JPanel buttonPanel = new JPanel();
    if (isNew) {
      buttonPanel.add(makeAddPermissionButton());
      buttonPanel.add(makeCancelButton());
    }
    else {
      buttonPanel.add(makeChangeButton());
      buttonPanel.add(makeCloseButton());
    }
    return buttonPanel;
  }

  /**
   * Cria o boto de limpar campos.
   * 
   * @return o boto de limpar campos.
   */
  private JButton makeClearButton() {
    JButton clearButton = new JButton(LNG.get("IAS_CLEAR"));
    clearButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ev) {
        clearFields();
      }
    });
    return clearButton;
  }

  /**
   * Cria o boto de incluso de permisso.
   * 
   * @return o boto de incluso de permisso.
   */
  protected JButton makeAddPermissionButton() {
    includePermissionButton = new JButton(LNG.get("IAS_INCLUDE"));
    includePermissionButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ev) {
        switch (validateFields()) {
          case FIELDS_OK:
            createPermission();
            break;
          case EMPTY_FIELDS:
            StandardErrorDialogs.showErrorDialog(window, windowLabel, LNG
              .get("IAS_FILL_ALL_FIELDS"));
            break;
          case PASSWORDS_DONT_MATCH:
            StandardErrorDialogs.showErrorDialog(window, windowLabel, LNG
              .get("IAS_CONFIRM_PERMISSION_PASSWORD_ERROR"));
            break;
        }
      }
    });
    return includePermissionButton;
  }

  /**
   * Cria o boto de modificao de permisso. Esse boto somente  habilitado
   * quando alguma modificao  feita nos campos.
   * 
   * @return o boto de modificao de dados da permisso.
   */
  protected JButton makeChangeButton() {
    includePermissionButton = new JButton(LNG.get("IAS_PERMISSION_UPDATE"));
    includePermissionButton.setEnabled(false);
    includePermissionButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ev) {
        switch (validateFields()) {
          case FIELDS_OK:
            modifyPermission();
            window.close();
            break;
          case EMPTY_FIELDS:
            StandardErrorDialogs.showErrorDialog(window, windowLabel, LNG
              .get("IAS_FILL_ALL_FIELDS"));
            break;
          case PASSWORDS_DONT_MATCH:
            StandardErrorDialogs.showErrorDialog(window, windowLabel, LNG
              .get("IAS_CONFIRM_PERMISSION_PASSWORD_ERROR"));
            break;
        }
      }
    });
    return includePermissionButton;
  }

  /**
   * Cria o boto de cancelamento de operao.
   * 
   * @return o boto de cancelamento.
   */
  protected JButton makeCancelButton() {
    JButton cancelButton = new JButton(LNG.get("IAS_CLOSE"));
    cancelButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ev) {
        window.close();
      }
    });
    return cancelButton;
  }

  /**
   * Cria o boto de fechamento da janela.
   * 
   * @return o boto de fechamento.
   */
  protected JButton makeCloseButton() {
    JButton closeButton = new JButton(LNG.get("IAS_CLOSE"));
    closeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ev) {
        // Se os campos nao forem alterados ou estiverem em branco, fecha
        // a janela.
        if (!hasChanged() || isEmpty()) {
          window.close();
          return;
        }
        Object[] options = { LNG.get("IAS_UPDATE"), LNG.get("IAS_CANCEL") };
        int selected =
          JOptionPane.showOptionDialog(window, LNG
            .get("IAS_PERMISSION_UPDATE_CONFIRMATION"), windowLabel,
            JOptionPane.DEFAULT_OPTION, JOptionPane.INFORMATION_MESSAGE, null,
            options, options[0]);
        if (options[selected] == LNG.get("IAS_CANCEL")) {
          window.close();
          return;
        }
        switch (validateFields()) {
          case FIELDS_OK:
            modifyPermission();
            window.close();
            break;
          case EMPTY_FIELDS:
            StandardErrorDialogs.showErrorDialog(window, windowLabel, LNG
              .get("IAS_FILL_ALL_FIELDS"));
            break;
          case PASSWORDS_DONT_MATCH:
            StandardErrorDialogs.showErrorDialog(window, windowLabel, LNG
              .get("IAS_CONFIRM_PERMISSION_PASSWORD_ERROR"));
            break;
        }

      }
    });
    return closeButton;
  }

  /**
   * Verifica se o campo de nome est preenchido (o campo de descrio no 
   * obrigatrio). Caso a permisso seja do tipo autenticada, os campos de
   * usurio e senha tambm so requeridos. A ausncia de parmetros para
   * permisses que aceitam atributos  vlida e portanto no verificada.
   * 
   * @return true se os campos estiverem preenchidos corretamente ou false se os
   *         campos estiverem incompletos ou com dados invlidos.
   * 
   * @see csbase.logic.AttributesPermission
   */
  protected int validateFields() {
    // Campos principais
    String nameText = name.getText().trim();
    if (nameText.equals("") || selectedPermissionClass == null) {
      return EMPTY_FIELDS;
    }

    // Campos de usurio/senha
    if (selectedPermissionClass == USER_PASSWORD_PERMISSION_CLASS) {
      String loginText = login.getText().trim();
      if (loginText.equals("")) {
        return EMPTY_FIELDS;
      }
      char[] passwordChars = password.getPassword();
      String passwordText = new String(passwordChars).trim();
      char[] confirmPasswordChars = confirmPassword.getPassword();
      String confirmPasswordText = new String(confirmPasswordChars).trim();
      if (!isNew && (passwordText.equals("") != confirmPasswordText.equals(""))) {
        return EMPTY_FIELDS;
      }
      if (isNew && (passwordText.equals("") || confirmPasswordText.equals(""))) {
        return EMPTY_FIELDS;
      }
      if (!passwordText.equals(confirmPasswordText)) {
        return PASSWORDS_DONT_MATCH;
      }
    }
    if (selectedPermissionClass == CHOICE_PERMISSION_CLASS) {
    }
    return FIELDS_OK;
  }

  /**
   * Limpa todos os campos.
   */
  private void clearFields() {
    if (isNew) {
      permission = null;
    }
    name.setText("");
    description.setText("");
    login.setText("");
    password.setText("");
    confirmPassword.setText("");
    newAttribute.setText("");
    attribModel.clear();
    map.clear();
    navigator.rewind();
    name.requestFocus();
  }

  /**
   * Verifica se os dados foram modificados. Em caso positivo, habilita o boto
   * de alterao da permisso.
   */
  private void checkDataChange() {
    if (hasChanged()) {
      includePermissionButton.setEnabled(true);
    }
    else {
      includePermissionButton.setEnabled(false);
    }
  }

  /**
   * Verifica se algum dos campos de preenchimento de dados da permisso foi
   * alterado.
   * 
   * @return true se algum campo foi editado ou false caso contrario
   */
  private boolean hasChanged() {
    if (!name.getText().trim().equals(permission.getName())) {
      return true;
    }
    if (!description.getText().trim().equals(permission.getDescription())) {
      return true;
    }
    if (selectedPermissionClass.equals(USER_PASSWORD_PERMISSION_CLASS)) {
      UserPasswordPermission userPwdPermission =
        (UserPasswordPermission) permission;
      if (!login.getText().trim().equals(userPwdPermission.getUser())) {
        return true;
      }
      String passwordText = new String(password.getPassword()).trim();
      if (!passwordText.equals("")) {
        return true;
      }
    }
    else if (selectedPermissionClass.equals(ATTRIBUTES_PERMISSION_CLASS)) {
      AttributesPermission attribPermission = (AttributesPermission) permission;
      Object[] attributesObj = attribModel.getAll();
      String[] screenAttributes = new String[attributesObj.length];
      System.arraycopy(attributesObj, 0, screenAttributes, 0,
        attributesObj.length);
      String[] currentAttributes = attribPermission.getAttributes();
      Arrays.sort(currentAttributes);
      if (!Arrays.equals(screenAttributes, currentAttributes)) {
        return true;
      }
    }
    else if (selectedPermissionClass.equals(CHOICE_PERMISSION_CLASS)) {
      return !((ChoicePermission) permission).getAttributes().equals(map);
    }
    return false;
  }

  /**
   * Verifica se os campos de preenchimento de dados da permisso esto sem
   * preenchimento.
   * 
   * @return true se os campos estiverem vazios ou false caso contrario
   */
  private boolean isEmpty() {
    return ((name.getText().trim().equals("")) && description.getText().trim()
      .equals(""));
  }

  /**
   * Obtm um objeto permisso com os dados digitados pelo usurio.
   * 
   * @return permisso com os dados digitados pelo usurio.
   */
  private Permission getScreenPermission() {
    Permission screenPermission = null;

    // Obtm os campos principais
    String nameText = name.getText().trim();
    String descriptionText = description.getText().trim();
    Class[] args;
    Object[] params;

    if (selectedPermissionClass.equals(USER_PASSWORD_PERMISSION_CLASS)) {
      // Prepara a instanciao de uma subclasse de UserPasswordPermission
      String loginText = login.getText().trim();
      String passwordText = new String(password.getPassword()).trim();
      args =
        new Class[] { String.class, String.class, String.class, String.class };
      params =
        new Object[] { nameText, descriptionText, loginText, passwordText };
    }
    else if (selectedPermissionClass.equals(ATTRIBUTES_PERMISSION_CLASS)) {
      // Prepara a instanciao de uma subclasse de AttributesPermission
      Object[] attributesObj = attribModel.getAll();
      String[] attributes = new String[attributesObj.length];
      System.arraycopy(attributesObj, 0, attributes, 0, attributesObj.length);
      args = new Class[] { String.class, String.class, attributes.getClass() };
      params = new Object[] { nameText, descriptionText, attributes };
    }
    else if (selectedPermissionClass.equals(CHOICE_PERMISSION_CLASS)) {
      args = new Class[] { String.class, String.class, Map.class };
      params = new Object[] { nameText, descriptionText, map };
    }
    else {
      // Prepara a instanciao de uma subclasse de SimplePermission
      args = new Class[] { String.class, String.class };
      params = new Object[] { nameText, descriptionText };
    }

    // Instancia a classe de permisso
    PermissionClassName item =
      (PermissionClassName) permissionClass.getSelectedItem();
    try {
      Class selectedClass = Class.forName(item.getClassName());
      Constructor constructor = selectedClass.getConstructor(args);
      screenPermission = (Permission) constructor.newInstance(params);
    }
    catch (Exception e) {
      StandardErrorDialogs.showErrorDialog(window, LNG.get("ERRO") + " - "
        + windowLabel, e);
    }
    return screenPermission;
  }

  /**
   * Executa a ao de criar a permisso, usando os dados preenchidos nos campos
   * da janela.
   */
  private void createPermission() {
    permission = getScreenPermission();
    if (permission == null) {
      return;
    }
    // Persiste o objeto de permisso
    permission =
      PermissionProxy.createPermission(window, windowLabel, LNG
        .get("IAS_WAITING_PERMISSION_CREATION"), permission);
    if (permission != null) {
      String msg = LNG.get("IAS_PERMISSION_INCLUDED_WITH_SUCCESS");
      msg = MessageFormat.format(msg, new Object[] { permission.getName() });
      StandardDialogs.showInfoDialog(window, windowLabel, msg);
      permission = null;
      clearFields();
    }
  }

  /**
   * Executa a ao de modificar a permisso, usando os dados preenchidos nos
   * campos da janela.
   */
  private void modifyPermission() {
    Permission modifiedPermission = getScreenPermission();
    if (modifiedPermission == null) {
      return;
    }
    modifiedPermission =
      PermissionProxy.modifyPermission(window, windowLabel, LNG
        .get("IAS_WAITING_PERMISSION_MODIFICATION"), permission.getId(),
        modifiedPermission);
    if (modifiedPermission != null) {
      this.permission = modifiedPermission;
      StandardDialogs.showInfoDialog(window, windowLabel, MessageFormat.format(
        LNG.get("IAS_PERMISSION_MODIFIED_WITH_SUCCESS"),
        new Object[] { permission.getName() }));
      password.setText("");
      confirmPassword.setText("");
      includePermissionButton.setEnabled(false);
    }
  }

  /**
   * Classe que representa um modelo em ordem alfabtica para os atributos de
   * uma permisso.
   */
  private class SortedListModel extends AbstractListModel {
    /** Itens do modelo */
    List items;

    /**
     * Instancia a lista de itens.
     */
    public SortedListModel() {
      items = new ArrayList();
    }

    /**
     * Obtm o tamanho da lista de itens.
     * 
     * @return tamanho da lista
     */
    public int getSize() {
      return items.size();
    }

    /**
     * Obtm o item especificado da lista.
     * 
     * @param index ndice do item desejado
     * 
     * @return item indicado pelo ndice
     */
    public Object getElementAt(int index) {
      return items.get(index);
    }

    /**
     * Obtm array contendo todos os itens da lista.
     * 
     * @return array contendo todos os itens da lista.
     */
    public Object[] getAll() {
      return items.toArray();
    }

    /**
     * Adiciona um item  lista, procurando o ponto exato de insero para
     * manter a ordem alfabtica. Permite a insero de duplicatas.
     * 
     * @param item item a ser inserido
     */
    public void add(Object item) {
      if (item == null) {
        return;
      }
      int index = Collections.binarySearch(items, item);
      if (index < 0) {
        index = -index - 1; // Ajuste para ponto de insero
        // ver javadoc de Collections.binarySearch
      }
      items.add(index, item);
      fireIntervalAdded(this, index, index);
    }

    /**
     * Remove o item especificado da lista.
     * 
     * @param index do item a ser removido
     * 
     * @return item recm-removido
     */
    public Object remove(int index) {
      if (index == -1) {
        return null;
      }
      if (items.size() == 0) {
        return null;
      }
      if ((index < 0) || (index > (items.size() - 1))) {
        return null;
      }
      Object item = items.remove(index);
      fireIntervalRemoved(this, index, index);
      return item;
    }

    /**
     * Limpa a lista.
     */
    public void clear() {
      items.clear();
      fireIntervalRemoved(this, 0, 0);
    }
  }

  /**
   * Classe que funciona como um Wrapper de classe de permisso, guardando
   * somente os dados relevantes para exibio.
   */
  private class PermissionClassName implements Comparable {
    private String className;
    private String presentationName;
    private String superClassName;

    /**
     * Cria o wrapper de permisso.
     * 
     * @param className .
     * 
     * @throws ClassNotFoundException caso no seja possvel instanciar a
     *         superclasse.
     */
    public PermissionClassName(String className) throws ClassNotFoundException {
      this.className = className;
      this.presentationName = LNG.get(className);
      this.superClassName = getSuperClassName(className);
    }

    /**
     * Obtm a classe base da permisso da qual herda.
     * 
     * @param className
     * 
     * @return .
     * 
     * @throws ClassNotFoundException .
     */
    private String getSuperClassName(String className)
      throws ClassNotFoundException {
      Class permissionClass = Class.forName(className);
      Class attributes = Class.forName(ATTRIBUTES_PERMISSION_CLASS);
      Class userPass = Class.forName(USER_PASSWORD_PERMISSION_CLASS);
      Class choicePermission = Class.forName(CHOICE_PERMISSION_CLASS);
      if (attributes.isAssignableFrom(permissionClass)) {
        return ATTRIBUTES_PERMISSION_CLASS;
      }
      else if (userPass.isAssignableFrom(permissionClass)) {
        return USER_PASSWORD_PERMISSION_CLASS;
      }
      else if (choicePermission.isAssignableFrom(permissionClass)) {
        return CHOICE_PERMISSION_CLASS;
      }
      else {
        return SIMPLE_PERMISSION_CLASS;
      }
    }

    /**
     * Obtm o nome da superclasse da permisso
     * 
     * @return superclasse (ou classe genrica) da permisso
     */
    public String getSuperClassName() {
      return superClassName;
    }

    /**
     * Obtm a classe da permisso
     * 
     * @return classe da permisso
     */
    public String getClassName() {
      return className;
    }

    /**
     * Obtm o nome a ser exibido para apresentao da permisso.
     * 
     * @return nome de apresentao (em contraste ao nome lgico) da permisso
     */
    public String getPresentationName() {
      return presentationName;
    }

    /**
     * Compara uma permisso a outra, para decidir seu posicionamento em uma
     * lista ordenada.
     * 
     * @param o .
     * 
     * @return int -1 se for "menor", 0 se for "igual", 1 se for "maior"
     */
    public int compareTo(Object o) {
      return this.presentationName
        .compareTo(((PermissionClassName) o).presentationName);
    }

    /**
     * Verifica se o objeto passado como parmetro  igual ao objeto no qual o
     * mtodo foi invocado.
     * 
     * @param o o objeto a ser comparado
     * 
     * @return true se ambos forem iguais
     */
    @Override
    public boolean equals(Object o) {
      return (o instanceof PermissionClassName && ((PermissionClassName) o).presentationName
        .equals(this.presentationName));
    }

    /**
     * Calcula o cdigo hash do objeto.
     * 
     * @return Inteiro que representa o cdigo hash do objeto.
     */
    @Override
    public int hashCode() {
      // XXX presentationName pode ser nulo?
      if (presentationName == null) {
        return "presentationName is null".hashCode();
      }
      return presentationName.hashCode();
    }

    /**
     * Retorna uma representao textual do objeto
     * 
     * @return representao textual do objeto
     */
    @Override
    public String toString() {
      return presentationName;
    }
  }

  private class AttributesNavigator {
    Object[] domainMultiSelectionMapping;
    Object[] domain = new Object[0];
    int position;
    ChoicePermission choicePermission;
    Map attributes = new HashMap();

    public void init() {
      PermissionClassName item =
        (PermissionClassName) permissionClass.getSelectedItem();
      try {
        Class selectedClass = Class.forName(item.getClassName());
        Constructor constructor = selectedClass.getConstructor(new Class[] {});
        choicePermission =
          (ChoicePermission) constructor.newInstance(new Object[] {});
      }
      catch (Exception e) {
        StandardErrorDialogs.showErrorDialog(window, LNG.get("ERRO") + " - "
          + windowLabel, e);
      }
      RemoteTask<Void> task = new RemoteTask<Void>() {
        @Override
        public void performTask() throws Exception {
          Map domainMap = choicePermission.getDomain();
          domain = domainMap.keySet().toArray();
          domainMultiSelectionMapping = domainMap.values().toArray();
          for (int i = 0; i < domain.length; i++) {
            attributes.put(domain[i], choicePermission
              .getRange((String) domain[i]));
          }
        }
      };
      if (!task.execute(PermissionInfoDialog.this.window, LNG
        .get("desktop.msg.wait"), LNG.get("desktop.msg.wait"))) {
        domain = new Object[0];
        attributeName.setVisible(false);
        attributeValue.setVisible(false);
        containerSelection.getPanel().setVisible(false);
      }
      position = -1;
      next();
    }

    private void previous() {
      if (position > 0) {
        position--;
        String key = (String) domain[position];
        attributeRange = (Map) attributes.get(key);
        go(key);
      }
    }

    private void next() {
      if (domain.length - 1 > position) {
        position++;
        String key = (String) domain[position];
        attributeRange = (Map) attributes.get(key);
        go(key);
      }
    }

    private void rewind() {
      attributeValue.setSelectedIndex(-1);
      attributeValue.invalidate();
      position = -1;
      next();
    }

    private void go(String key) {
      previousButton.setVisible((position > 0) ? true : false);
      nextButton.setVisible((domain.length - 1 > position) ? true : false);
      attributeNameKey = key;
      attributeName.setText(LNG.get(attributeNameKey));
      Boolean isMultiSelect = (Boolean) domainMultiSelectionMapping[position];
      Object mapValue = map.get(key);

      // Exibe o ContainerSelection para seleo mltipla.
      if (isMultiSelect.booleanValue()) {
        attributeName.setVisible(false);
        attributeValue.setVisible(false);
        containerSelection.getPanel().setVisible(true);
        if (mapValue != null) {
          List<Object> idList = (List) mapValue;
          Iterator<Object> keyIterator = attributeRange.keySet().iterator();
          List<Object> selectedItems = new ArrayList<Object>();
          while (keyIterator.hasNext()) {
            Object attributeKey = keyIterator.next();
            for (int i = 0; i < idList.size(); i++) {
              if (idList.get(i).equals(attributeRange.get(attributeKey))) {
                selectedItems.add(attributeKey);
                break;
              }
            }
          }
          containerSelection.loadItems(attributeRange.keySet(), selectedItems);
        }
        else {
          containerSelection
            .loadItems(attributeRange.keySet(), new ArrayList());
        }
      }

      // Exibe a JComboBlox para seleo nica.
      else {
        containerSelection.getPanel().setVisible(false);
        attributeName.setVisible(true);
        attributeValue.setVisible(true);
        attributeValue.setModel(new DefaultComboBoxModel(attributeRange
          .keySet().toArray()));
        if (mapValue != null) {
          attributeValue.setSelectedItem(((List) mapValue).get(0));
        }
        else {
          attributeValue.setSelectedIndex(-1);
        }
      }
    }
  }

  /**
   * Formatador dos itens da tabela ordenvel, que ser exibida no
   * ContainerSelection de edio de uma ChoicePermission.
   * 
   * @author Jorge Marques
   */
  class TableItemsFormatter implements ObjectTableProvider {
    public String[] getColumnNames() {
      return new String[] { LNG.get("IAS_AVAILABLE_ITEMS") };
    }

    public Class[] getColumnClasses() {
      return new Class[] { String.class };
    }

    public Object[] getCellValues(Object arg0) {
      return new Object[] { arg0.toString() };
    }
  }

  /**
   * Formatador dos itens da tabela ordenvel, que ser exibida no
   * ContainerSelection de edio de uma ChoicePermission.
   * 
   * @author Jorge Marques
   */
  class SelectedItemsFormatter implements ObjectTableProvider {
    public String[] getColumnNames() {
      return new String[] { LNG.get("IAS_SELECTED_ITEMS") };
    }

    public Class[] getColumnClasses() {
      return new Class[] { String.class };
    }

    public Object[] getCellValues(Object arg0) {
      return new Object[] { arg0.toString() };
    }
  }
}
