/*
 * UserManagerPanel.java
 *
 * $Author: schirley $
 * $Revision: 88482 $  - $Date: 2009-02-16 18:20:40 -0300 (Mon, 16 Feb 2009) $
 */
package csbase.client.ias;

import csbase.client.desktop.DesktopFrame;
import csbase.client.remote.srvproxies.UserGroupProxy;
import csbase.client.remote.srvproxies.UserProxy;
import csbase.client.util.ClientUtilities;
import csbase.client.util.StandardErrorDialogs;
import csbase.client.util.SwingObserverWrapper;
import csbase.logic.AdministrationEvent;
import csbase.logic.User;
import csbase.logic.UserGroup;
import csbase.logic.UserOutline;

import tecgraf.javautils.LNG;
import tecgraf.javautils.configurationmanager.Configuration;
import tecgraf.javautils.configurationmanager.ConfigurationManager;
import tecgraf.javautils.configurationmanager.ConfigurationManagerException;
import tecgraf.javautils.gui.StandardDialogs;
import tecgraf.javautils.gui.table.ObjectTableModel;
import tecgraf.javautils.gui.table.ObjectTableProvider;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.text.MessageFormat;

import java.util.Comparator;
import java.util.Observable;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.table.TableModel;

/**
 * A classe <code>UserManagerPanel</code> representa  o painel de
 * administrao dos usurios do sistema. Mostra uma tabela com a
 * identificao e o UserGroup de cada usurio. O administrador pode inclui e
 * remover um usurio,  alm de modificar os dados de um usurio qualquer.
 *
 * @author Maria Julia Lima
 * @version $Revision: 88482 $
 */
public class UserManagerPanel extends ManagerPanel {
  /** A janela onde esse painel est sendo usado. */
  private JFrame parent;
  private JButton importFromCSVFile;
  private JButton exportFromCSVFile;

  // Constante utilizada para testes de condies de erro.
  // 0: funcionamento normal
  // 1: dispara chamada ilegal ao mtodo setItems - antes da tabela ter sido
  //    criada
  // 2: envia um argumento nulo para o mtodo setItems
  // 3: dispara o tratamento para usurio no encontrado no mtodo delete
  private static final int debug = 0;

  /**
   * Remove os observadores de usurios e UserGroups.
   */
  public void beforeClose() {
    User.deleteObserver(new SwingObserverWrapper(this));
    UserGroup.deleteObserver(new SwingObserverWrapper(this));
  }

  /**
   * Exibe a janela de incluso de usurio.
   */
  public void add() {
    Vector userGroups =
      UserGroupProxy.getAllUserGroups(
        parent, parent.getTitle(), LNG.get("IAS_WAITING_ALL_USERGROUPS"));
    if ((userGroups == null) || (userGroups.size() == 0)) {
      StandardDialogs.showInfoDialog(
        parent, parent.getTitle(), LNG.get("IAS_NO_USERGROUPS"));
      return;
    }
    UserInfoDialog dialog = new IncludeUserInfoDialog(parent);
    dialog.setVisible(true);
  }

  /**
   * Exibe a janela de modificao de usurio.
   *
   * @param id identificador do usurio a ser modificado.
   */
  public void modify(Object id) {
    UserInfoDialog dialog;
    if (User.getLoggedUser().isAdmin()) {
      dialog = new AdminModifyUserInfoDialog(parent, id);
    }
    else {
      dialog = new ModifyUserInfoDialog(parent, id);
    }
    dialog.setVisible(true);
  }

  /**
   * Exibe a janela de remoo de usurio.
   *
   * @param id identificador do usurio a ser removido
   */
  public void delete(Object id) {
    User user =
      UserProxy.getUser(
        parent, parent.getTitle(), LNG.get("IAS_WAITING_USER_INFO"), id);
    if (debug == 3) {
      user = null; //@debug
    }
    if (user == null) {
      displayError(LNG.get("IAS_USERMGR_USER_NOT_FOUND"));
      return;
    }
    if (user.isAdmin()) {
      displayError(LNG.get("IAS_USERMGR_ADMIN_CANNOT_BE_REMOVED_ERROR"));
      return;
    }
    Object[] options =
      {
        LNG.get("IAS_USERMGR_BUTTON_REMOVE"),
        LNG.get("IAS_USERMGR_BUTTON_CANCEL")
      };
    int res =
      StandardDialogs.showOptionDialog(
        parent, parent.getTitle(),
        MessageFormat.format(
          LNG.get("IAS_USERMGR_USER_REMOVAL_CONFIRMATION"),
          new Object[] { user.getLogin() }), options);
    if (res == JOptionPane.YES_OPTION) {
      UserProxy.deleteUser(
        parent, parent.getTitle(),
        MessageFormat.format(
          LNG.get("IAS_USERMGR_WAITING_REMOVE_USER"),
          new Object[] { user.getLogin() }), id);
    }
  }

  /**
   * Recarrega a tabela de usurios, para refletir quaisquer mudanas que
   * possam ter ocorrido. Este mtodo  chamado pelo mtodo update sempre que
   * ocorre modificao nos dados de algum UserGroup, o que pode
   * potencialmente invalidar a exibio do nome do UserGroup de vrios
   * usurios; para garantir que isso no ocorra, este mtodo recarrega toda a
   * tabela de usurios, sincronizando-a com as informaes persistidas em
   * arquivo.
   */
  public void reload() {
    Vector userOutlines =
      UserProxy.getAllOutlines(
        parent, parent.getTitle(), LNG.get("IAS_WAITING_ADMIN_DATA"));
    if (userOutlines == null) {
      userOutlines = new Vector();
    }
    if (debug == 2) {
      userOutlines = null; //@debug
    }
    super.setItems(userOutlines);
  }

  /**
   * Estende a funcionalidade do mtodo update da classe pai, modificando
   * o comportamento no caso especfico de receber uma notificao de
   * alterao em UserGroup; nessa situao, toda a tabela  recarregada (ver
   * mtodo reload).
   *
   * @param observable o objeto que est sendo observado
   * @param arg o objeto que indica a mudana ocorrida
   */
  public void doUpdate(Observable observable, Object arg) {
    AdministrationEvent action = (AdministrationEvent)arg;
    Object item = action.item;
    if ((item instanceof UserGroup)) {
      if (action.type == AdministrationEvent.MODIFY) {
        reload();
      }
    }
    else {
      super.doUpdate(observable, arg);
    }
  }

  /**
   * Mostra uma janela com uma mensagem.
   *
   * @param title ttulo da janela de mensagem
   * @param message mensagem a ser exibida
   */
  protected void displayMessage(String title, String message) {
    StandardDialogs.showInfoDialog(parent, title, message);
  }

  /**
   * Mostra uma janela com uma mensagem de erro.
   *
   * @param message mensagem de erro a ser exibida
   */
  protected void displayError(String message) {
    StandardErrorDialogs.showErrorDialog(parent, parent.getTitle(), message);
  }

/**
   * Cria um painel de gerncia de usurios cadastrados no sistema. Registra-se
   * como observador das classes User e UserGroup, pois mudanas em quaisquer
   * das duas podem implicar em alteraes na exibio dos dados.
   *
   * @param parent Janela "pai"
   */
  public UserManagerPanel(JFrame parent) {
    this.parent = parent;
    Vector userOutlines =
      UserProxy.getAllOutlines(
        parent, parent.getTitle(), LNG.get("IAS_WAITING_ADMIN_DATA"));
    if (userOutlines == null) {
      userOutlines = new Vector();
    }
    if (debug == 1) {
      setItems(userOutlines); //@debug
    }
    ObjectTableProvider provider =
      new ObjectTableProvider() {
        public Object[] getCellValues(Object item) {
          UserOutline outline = (UserOutline)item;
          if (outline == null) {
            return null;
          }
          return new Object[] {
            outline.getId(), outline.getLogin(), outline.getName(),
            outline.getUserGroup()
          };
        }

        public String[] getColumnNames() {
          return new String[] {
            null, LNG.get("IAS_USER"), LNG.get("IAS_USER_NAME"),
            LNG.get("IAS_USERGROUP")
          };
        }

        public Class[] getColumnClasses() {
          return new Class[] {
            Object.class, String.class, String.class, String.class};
        }
      };
    TableModel model =
      new ObjectTableModel(userOutlines, provider);
    Comparator[] comparators =
      {
        null, ClientUtilities.getStringComparatorIgnoreCase(),
        ClientUtilities.getStringComparatorIgnoreCase(),
        ClientUtilities.getStringComparatorIgnoreCase()
      };
    make(model, comparators, true);
    User.addObserver(new SwingObserverWrapper(this));
    UserGroup.addObserver(new SwingObserverWrapper(this));
  }

  protected JPanel makeButtons() {
    JPanel panel = super.makeButtons();
    this.createImportButton(panel);
    this.createExportButton(panel);
    return panel;
  }

  private void createImportButton(JPanel panel) {
    this.importFromCSVFile = new JButton(LNG.get("IAS_IMPORT_USERS_BUTTON"));
    this.importFromCSVFile.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          new ImportUsersDialog(parent).showDialog();
        }
      });
    this.importFromCSVFile.setEnabled(enableButton());
    panel.add(this.importFromCSVFile);
  }

  private void createExportButton(JPanel panel) {
    this.exportFromCSVFile = new JButton(LNG.get("IAS_EXPORT_USERS_BUTTON"));
    this.exportFromCSVFile.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          new ExportUsersDialog(parent).showDialog();
        }
      });
    this.exportFromCSVFile.setEnabled(enableButton());
    panel.add(this.exportFromCSVFile);
  }

  private boolean hasProject() {
    if (DesktopFrame.getInstance() == null
      || DesktopFrame.getInstance().getProject() == null) {
      return false;
    }
    return true;
  }

  private boolean enableButton() {
    if (!useProject() || hasProject()) {
      return true;
    }
    return false;
  }

  /**
   * Indica se a aplicao utiliza o conceito de projeto.
   *
   * @return true, se aplicao utiliza projeto, false caso contrrio.
   */
  private boolean useProject() {
    try {
      Configuration configuration =
        ConfigurationManager.getInstance().getConfiguration(this.getClass());
      return configuration
        .getOptionalBooleanProperty("application.use.project", true);
    }
    catch (ConfigurationManagerException cme) {
      return true;
    }
  }
}
