package csbase.client.project;

import csbase.client.applications.ApplicationImages;
import csbase.logic.ClientProjectFile;
import csbase.logic.User;

import java.text.MessageFormat;

import javax.swing.ImageIcon;

import tecgraf.javautils.LNG;

/**
 * <p>
 * Essa classe deve ser utilizada para se obter propriedades de um
 * ClientProjectFile, mas que tenham semntica apenas no lado cliente.
 * </p>
 * 
 * <p>
 * Essas propriedades so cones e textos que so usados na exibio de um
 * ClientProjectFile na rvore de projetos ({@link
 * csbase.client.project.ProjectTree}), no painel de contedo de diretrios
 * {@link csbase.client.desktop.dircontents.DirectoryContentsPanel}, entre
 * outros.
 * </p>
 *
 * @author Rodrigo Carneiro Henrique (rodrigoh)
 */
public final class ClientProjectFileProperties {
  /**
   * Obtm o cone associado a um arquivo de projeto, quando este arquivo no
   * est recortado.
   *
   * @param file O arquivo de projeto.
   *
   * @return O cone, ou null, quanto o arquivo est num estado desconhecido.
   */
  public static ImageIcon getImageIcon(ClientProjectFile file) {
    if (file.isMoving()) {
      return getCutImageIcon(file);
    }    
    if (file.isDirectory()) {
      if (file.isUpdated()) {
        return ApplicationImages.ICON_FOLDER_16;        
      }
      return ApplicationImages.ICON_FOLDEROUTOFDATE_16;
    }
    ClientProjectFileType fileType = ClientProjectFileType.getFileType(file.getType());
    if (file.isUnderConstruction()) {
      return fileType.getUnderConstructionIcon();
    }
    if (file.isRegularFile()) {
      return fileType.getBaseIcon();
    }
    if (file.isPublishedByMe()) {
      return fileType.getPublishedByMeIcon();
    }
    if (file.isPublishedByOthers()) {
      return fileType.getPublishedByOtherIcon();
    }
    if (file.isGlobalFile()) {
      return fileType.getDataBaseIcon();
    }
    return null;
  }

  /**
   * Obtm o cone associado a um arquivo de projeto quando ele est no modo
   * "recortado" para o clipboard.
   *
   * @param file O arquivo de projeto.
   *
   * @return O cone do arquivo.
   */
  public static ImageIcon getCutImageIcon(ClientProjectFile file) {
    if (file.isDirectory()) {
      return ApplicationImages.ICON_FOLDERCUT_16;
    }
    ClientProjectFileType fileType = ClientProjectFileType.getFileType(file.getType());
    return fileType.getCutIcon();
  }

  /**
   * Obtm a descrio do modo de acesso do arquivo.
   *
   * @param file O arquivo de projeto.
   *
   * @return A descrio do modo de acesso do arquivo, ou um espao em branco,
   *         caso o arquivo esteja num estado desconhecido.
   */
  public static String getAccessMode(ClientProjectFile file) {
    if (file.isRegularFile()) {
      return LNG.get("PRJ_FILE_READ_AND_WRITE");
    }
    if (file.isPublishedFile()) {
      return LNG.get("PRJ_FILE_READ_ONLY")+" ("+LNG.get("PRJ_PUBLISHED_FILE")+
      ")";
    }
    if (file.isGlobalFile()) {
      return LNG.get("PRJ_FILE_READ_ONLY")+" ("+LNG.get("PRJ_GLOBAL_FILE")+")";
    }
    if (file.isUnderConstruction()) {
      return LNG.get("PRJ_FILE_READ_ONLY")+" ("+
      LNG.get("PRJ_UNDER_CONSTRUCTION_FILE")+")";
    }
    return " ";
  }

  /**
   * Obtm a descrio da origem do arquivo de projeto.
   *
   * @param file O arquivo de projeto.
   *
   * @return A descrio da origem do arquivo de projeto, ou um espao em
   *         branco, caso o arquivo esteja em um estado desconhecido.
   */
  public static String getSourceDescription(ClientProjectFile file) {
    if (file.isRegularFile()) {
      return LNG.get("PRJ_PROJECT");
    }
    if (file.isPublishedByMe()) {
      return LNG.get("PRJ_PROJECT");
    }
    if (file.isPublishedByOthers()) {
      Object userPublishedId = file.whoPublished();
      String userName = null;
      try {
        userName = User.getName(userPublishedId);
      }
      catch (Exception e) {}
      if (userName == null) {
        userName = (String)userPublishedId;
      }
      return MessageFormat.format(
        LNG.get("PRJ_PUBLISHED_FILE_SOURCE"), new Object[] { userName });
    }
    if (file.isGlobalFile()) {
      return LNG.get("PRJ_GLOBAL_FILE");
    }
    if (file.isUnderConstruction()) {
      return LNG.get("PRJ_PROJECT");
    }
    return " ";
  }
}
