/*
 * CommonProjectInfoDialog.java
 *
 * $Author: taciana $
 * $Date: 2008-06-15 15:45:34 -0300 (Sun, 15 Jun 2008) $
 * $Revision: 78501 $
 */
package csbase.client.project;

import java.awt.BorderLayout;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.GUIUtils;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.desktop.DesktopComponentDialog;
import csbase.client.desktop.RemoteTask;
import csbase.client.util.ClientUtilities;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.CommonClientProject;
import csbase.logic.CommonProjectInfo;
import csbase.logic.User;
import csbase.logic.UserProjectInfo;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.ProjectServiceInterface;

/**
 * A classe <code>CommonProjectInfoDialog</code> mostra um dilogo para
 * criao ou modificao das informaes de um projeto. Na criao de um
 * projeto, o usurio pode acionar o boto de criao, que confirma a operao,
 * ou pode acionar o boto de cancelamento, que desiste da operao. O boto de
 * criao somente  habilitado quando o usurio tiver informado um nome do
 * projeto. Na modificao de um projeto, o usurio pode acionar o boto de
 * alterar, que confirma a operao, ou pode acionar o boto de cancelamento,
 * que desiste da operao. O boto de alterao somente  habilitado quando o
 * usurio tiver alterado algum dado de projeto. No caso, o usurio pode
 * modificar apenas a descrio de um projeto. Ao confirmar as operao, seja de
 * criao ou de alterao, o servio  executado no servidor e o usurio 
 * notificado de qualquer problema que tenha ocorrido. Esta classe possui uma
 * seguestao de implementacao grafica. Ele deve ser estendida para incorporar
 * informacoes especificas dos projetos.
 *
 * @author Maria Julia de Lima
 */
public abstract class CommonProjectInfoDialog {
  /** A janela de visualizao de informaes do projeto. */
  protected DesktopComponentDialog dialog;
  /** A janela a partir da qual esse dilogo foi solicitado. */
  protected Window owner;
  /** O campo de nome do projeto. */
  protected JTextField nameField;
  /** O campo de descrio do projeto. */
  protected JTextField descrField;
  /** O botao default. */
  protected JButton defaultButton;
  /** O projeto criado ou modificado. */
  protected CommonClientProject project = null;

  /**
   * Mostra um dilogo com as informaes de um projeto existente. O usurio
   * pode alterar os dados do projeto. Caso a operao seja confirmada, os dados
   * so alterados no servidor.
   *
   * @param owner A janela a partir da qual esse dilogo foi solicitado.
   * @param project O projeto a ser alterado.
   *
   * @return O projeto modificado.
   */
  public static CommonClientProject updateProject(Window owner,
    CommonClientProject project) {
    return new CommonProjectModificationDialog(owner, project).project;
  }

  /**
   * Mostra um dilogo para que o usurio crie um projeto. O usurio informa os
   * dados do projeto. Caso a operao seja confirmada, o servidor  requisitado
   * para executar o servio de criao do projeto.
   *
   * @param owner A janela a partir da qual esse dilogo foi solicitado.
   *
   * @return O projeto criado.
   */
  public static CommonClientProject createProject(Window owner) {
    return new CommonProjectCreationDialog(owner).project;
  }

  /**
   * Cria o dilogo modal, a partir de uma janela principal. O dilogo serve
   * tanto para criao como para atualizao dos dados de projeto.
   *
   * @return O dilogo recm-criado.
   */
  protected DesktopComponentDialog makeDialog() {
    // Cria o painel principal da janela
    JPanel mainPanel = new JPanel(new BorderLayout(0, 5));
    mainPanel.add(makeNameDescrPanel(), BorderLayout.NORTH);
    mainPanel.add(makeAditionalPanel());
    mainPanel.add(makeButtonPanel(), BorderLayout.SOUTH);
    // Cria um dilogo modal
    DesktopComponentDialog dialog;
    if (owner instanceof JFrame) {
      dialog = new DesktopComponentDialog((JFrame) owner, getTitle());
    }
    else if (owner instanceof JDialog) {
      dialog = new DesktopComponentDialog((JDialog) owner, getTitle());
    }
    else {
      return null;
    }
    dialog.getContentPane().add(mainPanel);
    dialog.pack();
    dialog.center(owner);
    dialog.getRootPane().setDefaultButton(defaultButton);
    return dialog;
  }

  /**
   * Cria um painel com os campos de nome e descrio de projeto. Os campos de
   * nome e descrio so ambos campos de texto. de apenas uma nica linha.
   *
   * @return O painel recm-criado.
   */
  private JPanel makeNameDescrPanel() {
    nameField = new JTextField(20);
    descrField = new JTextField(20);
    JComponent[][] components = {
        { new JLabel(LNG.get("PRJ_NAME")), nameField },
        { new JLabel(LNG.get("PRJ_DESCRIPTION")), descrField } };
    return GUIUtils.createBasicGridPanel(components);
  }

  /**
   * Cria o projeto no servidor e o carrega no cliente.
   * Caso algum problema ocorra, o usurio  informado com uma mensagem em uma
   * outra janela de dilogo. Este metodo deve ser sobreescrito em cada sistema
   * se os projetos tiverem mais dados do que o nome e a a descricao.
   */
  protected void createProject() {
    final CommonProjectInfo info = new CommonProjectInfo();
    info.userId = User.getLoggedUser().getId();
    info.name = nameField.getText();
    info.description = descrField.getText();
    if (!ClientUtilities.isValidFileName(info.name)) {
      StandardErrorDialogs.showErrorDialog(owner, getTitle(), LNG
        .get("PRJ_PROJECT_FILE_NAME_CHARACTERES_ERROR"));
      return;
    }
    RemoteTask<CommonClientProject> task =
      new RemoteTask<CommonClientProject>() {
      @Override
      public void performTask() throws Exception {
        setResult(CommonClientProject.createProject(info));
        ClientRemoteLocator.server.setServerSessionProperty(
          ProjectServiceInterface.CURRENT_PROJECT_SESSION_PROPERTY,
          new UserProjectInfo(info.userId, info.name));
      }
    };
    if (task.execute(
      dialog, getTitle(), LNG.get("PRJ_WAITING_CREATE_PROJECT"))) {
      project = task.getResult();
      dialog.close();
    }
  }

  /**
   * Cria interface adicional de entrada especifica de cada projeto.
   *
   * @return .
   */
  protected abstract JPanel makeAditionalPanel();

  /**
   * Cria um painel com os botes que vo ser usados no dilogo requisitado.
   * Esse mtodo  abstrato. Depende da ao que originou o dilogo.
   *
   * @return O painel recm-criado.
   */
  protected abstract JPanel makeButtonPanel();

  /**
   * Informa o ttulo que vai ser utilizado na janela de dilogo. Esse mtodo 
   * abstrato. Depende da ao que originou o dilogo.
   *
   * @return O ttulo da janela.
   */
  protected abstract String getTitle();

  /**
   * Constri uma viso dos campos de dados de um projeto. A viso serve tanto
   * para criao de um projeto novo bem como para atualizao dos dados de um
   * projeto existente.
   *
   * @param owner A janela a partir da qual esse dilogo foi solicitado.
   */
  protected CommonProjectInfoDialog(Window owner) {
    this.owner = owner;
    dialog = makeDialog();
  }
}

/**
 * A classe <code>CommonProjectCreationDialog</code> usa o dilogo com
 * informaes de projeto, para o propsito de criao de um novo projeto no
 * servidor.
 */
class CommonProjectCreationDialog extends CommonProjectInfoDialog {
  /**
   * Limpa os campos do dilogo e atribui os listeners a esses campos.
   */
  private void clearDialog() {
    // Inicializa os campos
    nameField.setText("");
    descrField.setText("");
    // Adiciona um listener ao campo de nome para que,
    // quando um nome for digitado, o boto principal
    // seja habilitado.
    nameField.addKeyListener(new KeyAdapter() {
      @Override
      public void keyReleased(KeyEvent e) {
        String name = nameField.getText().trim();
        if (name.equals("")) {
          defaultButton.setEnabled(false);
        }
        else {
          defaultButton.setEnabled(true);
        }
      }
    });
  }

  /**
   * Cria interface adicional de entrada especifica de cada projeto.
   *
   * @return .
   */
  @Override
  protected JPanel makeAditionalPanel() {
    return new JPanel();
  }

  /**
   * Cria um painel com os botes que vo ser usados no dilogo de criao de
   * projeto. Na criao de um projeto, o usurio pode acionar o boto de
   * criao, que confirma a operao, ou pode acionar o boto de cancelamento,
   * que desiste da operao. O boto de criao somente  habilitado quando o
   * usurio tiver informado um nome do projeto.
   *
   * @return O painel recm-criado.
   */
  @Override
  protected JPanel makeButtonPanel() {
    JPanel panel = new JPanel();

    // Boto de criao de projeto
    defaultButton = (JButton) panel.add(new JButton(LNG.get("PRJ_CREATION")));
    JButton cancelButton = (JButton) panel.add(new JButton(LNG
      .get("PRJ_CANCEL")));
    defaultButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        handleAction();
      }
    });
    defaultButton.setEnabled(false);
    // Boto de cancelamento da operao de criao.
    cancelButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        dialog.close();
      }
    });
    ClientUtilities.adjustEqualSizes(defaultButton, cancelButton);
    return panel;
  }

  /**
   * Informa o ttulo que vai ser utilizado na janela de criao de projeto.
   *
   * @return O ttulo da janela.
   */
  @Override
  protected String getTitle() {
    return ClientUtilities.addSystemNameToTitle(LNG
      .get("project.creation_title"));
  }

  /**
   * Executa a ao de criao de um projeto.
   */
  private void handleAction() {
    createProject();
  }

  /**
   * Instancia a viso de informaes de projeto usada para criao de projeto.
   * O dilogo  criado e apresentado.
   *
   * @param owner A janela a partir da qual esse dilogo foi solicitado.
   */
  CommonProjectCreationDialog(Window owner) {
    super(owner);
    clearDialog();
    dialog.pack();
    dialog.setVisible(true);
  }
}

/**
 * A classe <code>CommonProjectModificationDialog</code> usa o dilogo com
 * informaes de projeto, para o propsito de atualizao dos dados de um
 * projeto. Atualmente, a nica informao que o usurio pode alterar  a
 * descrio de um projeto.
 */
class CommonProjectModificationDialog extends CommonProjectInfoDialog {
  /**
   * Cria interface adicional de entrada especifica de cada projeto.
   *
   * @return .
   */
  @Override
  protected JPanel makeAditionalPanel() {
    return new JPanel();
  }

  /**
   * Inicializa os campos do dilogo com os dados do projeto que vai ser
   * atualizado. Atribui os listeners aos campos que podem ser modificados para
   * que o boto principal seja habilitado ou no. Este metodo deve ser
   * sobrescrito para habilitar ou desabilitar campos especificos de cada
   * sistema integrador.
   *
   * @param project O projeto usado para o preenchimento dos campos.
   */
  private void loadDialog(CommonClientProject project) {
    // Preenche o nome e a descrio do projeto
    nameField.setText(project.getName());
    descrField.setText(project.getDescription());
    // Desabilita os campos de nome, projeo, datum e mc.
    // Apenas a descrio do projeto pode ser alterada.
    nameField.setEnabled(false);
    // Adiciona ao campo de descrio, um listener que habilita e
    // desabilita o boto de alterao, dependendo se a descrio
    // tiver sido ou no alterada.
    ChangeListener changeListener = new ChangeListener();
    descrField.addKeyListener(changeListener);
  }

  /**
   * Cria um painel com os botes que vo ser usados no dilogo de modificao
   * de dados de um projeto. Na modificao de um projeto, o usurio pode
   * acionar o boto de alterar, que confirma a operao, ou pode acionar o
   * boto de cancelamento, que desiste da operao. O boto de alterao
   * somente  habilitado quando o usurio tiver alterado algum dado de projeto.
   * No caso, o usurio pode modificar apenas a descrio de um projeto.
   *
   * @return O painel recm-criado.
   */
  @Override
  protected JPanel makeButtonPanel() {
    JPanel panel = new JPanel();
    defaultButton = (JButton) panel.add(new JButton(LNG.get("PRJ_UPDATE")));
    JButton cancelButton = (JButton) panel.add(new JButton(LNG
      .get("PRJ_CANCEL")));
    defaultButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        handleAction();
      }
    });
    defaultButton.setEnabled(false);
    cancelButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        dialog.close();
      }
    });
    return panel;
  }

  /**
   * Informa o ttulo que vai ser utilizado na janela de atualizao de projeto.
   *
   * @return O ttulo da janela.
   */
  @Override
  protected String getTitle() {
    return ClientUtilities
      .addSystemNameToTitle(LNG.get("project.update_title"));
  }

  /**
   * Informa se os dados do projeto foi alterado. O nico campo que permite
   * alterao  a descrio.
   *
   * @return true of false, caso o projeto tenha sido alterado ou no.
   */
  protected boolean projectHasChanged() {
    return !project.getDescription().equals(descrField.getText());
  }

  /**
   * Executa a ao de modificao de um projeto. O servio  chamado no
   * servidor. Caso algum problema ocorra, o usurio  informado com uma
   * mensagem em uma outra janela de dilogo.
   */
  private void handleAction() {
    project.setDescription(descrField.getText());
    RemoteTask<Void> task = new RemoteTask<Void>() {
      @Override
      public void performTask() throws Exception {
        project.modify();
      }
    };
    if (
      task.execute(dialog, getTitle(), LNG.get("PRJ_WAITING_UPDATE_PROJECT"))) {
      /* Informa que a modificao ocorreu com sucesso */
      StandardDialogs.showInfoDialog(dialog, getTitle(), LNG
        .get("PRJ_PROJECT_UPDATED_WITH_SUCCESS"));
      dialog.close();
    }
  }

  /**
   * A classe <code>ChangeListener</code>  responsvel por verificar se um
   * campo sofreu uma alterao, mantendo o boto principal do dilogo
   * habilitado ou desabilitado, de acordo.
   */
  class ChangeListener extends KeyAdapter {
    /**)
     * {@inheritDoc}
     * @see java.awt.event.KeyAdapter#keyReleased(java.awt.event.KeyEvent)
     */
    @Override
    public void keyReleased(KeyEvent e) {
      checkChange();
    }

    /**
     * Verifica se o projeto foi modificado.
     */
    private void checkChange() {
      if (projectHasChanged()) {
        defaultButton.setEnabled(true);
      }
      else {
        defaultButton.setEnabled(false);
      }
    }
  }

  /**
   * Instancia a viso de informaes de projeto usada para modificao de um
   * projeto existente. O dilogo  criado e apresentado.
   *
   * @param owner A janela a partir da qual esse dilogo foi solicitado.
   * @param project O projeto que vai sofrer a alterao.
   */
  CommonProjectModificationDialog(Window owner, CommonClientProject project) {
    super(owner);
    this.project = project;
    loadDialog(project);
    dialog.pack();
    dialog.setVisible(true);
  }
}
