/*
 * $Id: CommonProjectSelectDialog.java 95877 2009-09-04 21:57:56Z jnlopes $
 */
package csbase.client.project;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagLayout;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.ScrollPaneConstants;

import tecgraf.javautils.LNG;
import tecgraf.javautils.configurationmanager.Configuration;
import tecgraf.javautils.configurationmanager.ConfigurationManager;
import tecgraf.javautils.gui.GBC;
import tecgraf.javautils.gui.GUIUtils;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.desktop.DesktopComponentDialog;
import csbase.client.desktop.DesktopFrame;
import csbase.client.desktop.RemoteTask;
import csbase.client.project.action.ProjectCloseAction;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.CommonClientProject;
import csbase.logic.ProjectAdminInfo;
import csbase.logic.User;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.ProjectServiceInterface;
import csbase.util.StringUtils;

/**
 * A classe <code>CommonProjectSelectDialog</code> mostra um dilogo modal para
 * seleo ou remoo de um projeto de um usurio. A implementacao padrao e' da
 * acao da selecao para abertura do projeto. A
 * <code>CommonProjectDeleteDialog</code> estende esta classe e sobrescreve os
 * metodos necessarios. Na seleo de um projeto, o usurio pode acionar o boto
 * de seleo, que confirma a operao, ou pode acionar o boto de cancelamento,
 * que desiste da operao. O boto de seleo somente  habilitado quando o
 * usurio tiver informado um nome do projeto. Aps a seleo, o projeto 
 * aberto e um observador  instalado para ser notificado de qualquer alterao
 * remota que ocorrer sobre as informaes desse projeto. Na remoo de um
 * projeto, o usurio pode acionar o boto de remover, que confirma a operao,
 * ou pode acionar o boto de cancelamento, que desiste da operao. O boto de
 * remoo somente  habilitado quando o usurio tiver escolhido algum projeto.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class CommonProjectSelectDialog {

  /**
   * A janela que deu origem a esse dialogo
   */
  protected Window owner;

  /**
   * A lista com os nomes de projetos do usurio
   */
  protected JList projectList;

  /**
   * A janela de seleo de projeto.
   */
  protected DesktopComponentDialog dialog;

  /**
   * O boto default da janela
   */
  protected JButton defaultButton;

  /**
   * O array com os nomes dos projetos do usurio
   */
  protected String[] projectNames;

  /**
   * O projeto selecionado
   */
  protected CommonClientProject project;

  /** Indica se o dilogo e de abertura ou remoo de projetos */
  protected boolean openDialog;

  /**
   * Mostra um dilogo com a lista de nomes dos projetos de usurio e aguarda
   * que um deles seja selecionado. O dilogo  modal. No caso da confimao da
   * seleo, o projeto  aberto.
   * 
   * @param owner A janela a partir da qual esse dilogo foi chamado.
   * 
   * @return O projeto selecionado.
   */
  public static CommonClientProject openProject(Window owner) {
    try {
      return new CommonProjectSelectDialog(owner, true).project;
    } catch (Exception e) {
      StandardErrorDialogs.showErrorDialog(owner, LNG
          .get("CommonProjectSelectDialog.title.open"), LNG
          .get("CommonProjectSelectDialog.error.no.project"));
    }
    return null;
  }

  /**
   * Mostra um dilogo com a lista de nomes dos projetos de usurio e aguarda
   * que um deles seja selecionado. O dilogo  modal. No caso da confimao da
   * remoo, o projeto  removido.
   * 
   * @param owner A janela a partir da qual esse dilogo foi chamado.
   * 
   * @return O projeto selecionado.
   */
  public static boolean deleteProject(Window owner) {
    try {
      return new CommonProjectDeleteDialog(owner).isDeleted;
    } catch (Exception e) {
      StandardErrorDialogs.showErrorDialog(owner, LNG
          .get("CommonProjectSelectDialog.title.delete"), LNG
          .get("CommonProjectSelectDialog.error.no.project"));
    }
    return false;
  }

  /**
   * Cria um dilogo que aguarda a seleo de um projeto. Solicita ao servidor
   * que o projeto selecionado seja aberto.
   * 
   * @return .
   */
  private DesktopComponentDialog makeDialog() {
    // Cria o painel principal da janela
    JPanel mainPanel = new JPanel(new BorderLayout());
    mainPanel.add(makeProjectListPanel(), BorderLayout.CENTER);
    mainPanel.add(makeButtonPanel(), BorderLayout.SOUTH);

    // Cria um dilogo modal
    final DesktopComponentDialog d;
    if (owner instanceof JFrame) {
      d = new DesktopComponentDialog((JFrame) owner, getTitle());
    } else if (owner instanceof JDialog) {
      d = new DesktopComponentDialog((JDialog) owner, getTitle());
    } else {
      return null;
    }
    d.addWindowListener(new WindowAdapter() {
      @Override
      public void windowClosing(WindowEvent e) {
        d.close();
      }
    });
    d.getContentPane().add(mainPanel);
    d.pack();
    d.center(owner);
    d.getRootPane().setDefaultButton(defaultButton);
    return d;
  }

  /**
   * Obtem a lista de projetos de um usurio.
   * 
   * @return .
   */
  private String[] getProjectNames() {
    RemoteTask<String[]> task = new RemoteTask<String[]>() {
      @Override
      public void performTask() throws Exception {
        setResult(getAllProjectNames(User.getLoggedUser().getId()));
      }
    };

    final String msg = LNG.get("CommonProjectSelectDialog.info.get.projects");
    if (task.execute(owner, getTitle(), msg)) {
      final String[] projNames = task.getResult();
      return projNames;
    }
    return new String[0];
  }

  /**
   * Cria um painel com a lista dos projetos apresentados no dilogo. O contedo
   * da lista  requisitado ao servidor.
   * 
   * @return .
   */
  private JPanel makeProjectListPanel() {
    final String projectString = LNG.get("CommonProjectSelectDialog.projects");
    final JLabel projectLabel = new JLabel(projectString);

    projectList = new JList(projectNames);
    final ListSelectionModel selectionModel = projectList.getSelectionModel();
    if (openDialog) {
      final int selectionMode = ListSelectionModel.SINGLE_SELECTION;
      selectionModel.setSelectionMode(selectionMode);
    } else {
      final int selectionMode = ListSelectionModel.MULTIPLE_INTERVAL_SELECTION;
      selectionModel.setSelectionMode(selectionMode);
    }

    // Associa o duplo clique a ao principal do dilogo
    final MouseListener mouseListener = new MouseAdapter() {
      @Override
      public void mouseClicked(MouseEvent e) {
        if (e.getClickCount() == 2) {
          handleAction();
        }
      }
    };
    projectList.addMouseListener(mouseListener);
    projectList.setSelectedIndex(0);

    final JScrollPane usersScrollPane = new JScrollPane(projectList,
        ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
        ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);

    int width;
    int height;
    final int defaultWidth = 270;
    final int defaultHeight = 130;
    try {
      final ConfigurationManager mng = ConfigurationManager.getInstance();
      final Configuration cnf = mng.getConfiguration(this.getClass());
      width = cnf.getOptionalIntegerProperty("list.width", defaultWidth);
      height = cnf.getOptionalIntegerProperty("list.height", defaultHeight);
    } catch (Exception e) {
      width = defaultWidth;
      height = defaultHeight;
    }
    usersScrollPane.setPreferredSize(new Dimension(width, height));

    final JPanel panel = new JPanel(new GridBagLayout());
    panel.add(projectLabel, new GBC(0, 0).west().insets(10, 10, 5, 10));
    panel
        .add(usersScrollPane, new GBC(0, 1).both().west().insets(0, 10, 5, 10));

    return panel;
  }

  /**
   * Seleciona os projetos existentes. Este metodo deve ser sobrescrito para
   * abrir projeto especificos do cada sistema.
   * 
   * @param userId .
   * 
   * @return .
   * 
   * @throws Exception
   */
  protected String[] getAllProjectNames(Object userId) throws Exception {
    return CommonClientProject.getAllProjectNames(userId);
  }

  /**
   * Cria um painel com os botes que vo ser usados no dilogo requisitado.
   * Esse mtodo deve ser sobrescrito dependendo acao da selecao.
   * 
   * @return O painel recm-criado.
   */
  protected JPanel makeButtonPanel() {
    JPanel panel = new JPanel();
    defaultButton = (JButton) panel.add(new JButton(LNG
        .get("CommonProjectSelectDialog.open")));
    defaultButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        handleAction();
      }
    });

    JButton cancelButton = (JButton) panel.add(new JButton(LNG
        .get("CommonProjectSelectDialog.cancel")));
    cancelButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        dialog.close();
      }
    });

    GUIUtils
        .matchPreferredSizes(new JComponent[] { defaultButton, cancelButton });
    return panel;
  }

  /**
   * Executa a ao de abrir o projeto selecionado e instalar um observador para
   * ele.
   */
  protected void handleAction() {
    final String selectedProject = (String) projectList.getSelectedValue();
    if (selectedProject == null) {
      StandardDialogs.showErrorDialog(owner, getTitle(), LNG
          .get("CommonProjectSelectDialog.error.no.selection"));
      return;
    }

    if (!checkOpenableProject(selectedProject)) {
      return;
    }

    /* Se o resultado for null  porque a operao no pode ser efetuada. */
    project = DesktopFrame.getInstance().openProject(selectedProject);
    if (project != null) {
      dialog.close();
    }
  }

  /**
   * Verifica se projeto pode ser aberto (no est aguardando alocao de rea
   * solicitada).
   * 
   * @param selectedProject Nome do projeto a ser verificado.
   * @return Boolean indicando se projeto pode ser aberto (no est aguardando
   *         alocao de rea solicitada).
   * @throws Exception Erro na execuo remota da verificao.
   */
  private boolean checkOpenableProject(final String selectedProject) {

    final RemoteTask<Boolean> task = new RemoteTask<Boolean>() {

      @Override
      protected void performTask() throws Exception {
        final Object userId = User.getLoggedUser().getId();
        ProjectServiceInterface psi = ClientRemoteLocator.projectService;
        final ProjectAdminInfo[] projectAdminInfos = psi
            .getLockedProjectAdminInfo();
        for (ProjectAdminInfo projectAdminInfo : projectAdminInfos) {
          if (selectedProject.equals(projectAdminInfo.getProjectName())
              && userId.equals(projectAdminInfo.getUserId())) {
            setResult(!projectAdminInfo.isLocked());
            return;
          }
        }
        setResult(true);
      }
    };

    final DesktopFrame desktopFrame = DesktopFrame.getInstance();
    final JFrame frame = desktopFrame.getView();

    task.execute(frame, LNG
        .get("CommonProjectSelectDialog.checking.project.blocked.title"), LNG
        .get("CommonProjectSelectDialog.checking.project.blocked.msg"));

    if (task.wasCancelled()) {
      StandardDialogs.showErrorDialog(desktopFrame.getView(), LNG
          .get("CommonProjectSelectDialog.cancelled.task.title"), LNG
          .get("CommonProjectSelectDialog.cancelled.task.msg"));
      return false;
    }

    if (task.getStatus() != true) {
      StandardDialogs.showErrorDialog(desktopFrame.getView(), LNG
          .get("CommonProjectSelectDialog.locked.project.failure.title"), LNG
          .get("CommonProjectSelectDialog.locked.project.failure.msg"));
      return false;
    }

    final boolean openable = task.getResult();
    if (!openable) {
      StandardDialogs.showErrorDialog(desktopFrame.getView(), LNG
          .get("CommonProjectSelectDialog.locked.project.title"), LNG
          .get("CommonProjectSelectDialog.locked.project.msg"));
    }
    return openable;
  }

  /**
   * Informa o ttulo que vai ser utilizado na janela de dilogo. Esse mtodo
   * depende da ao que originou o dilogo.
   * 
   * @return O ttulo da janela.
   */
  protected String getTitle() {
    return LNG.get("CommonProjectSelectDialog.title.open");
  }

  /**
   * Constri uma viso de dilogo para seleo de projetos de um um usurio.
   * 
   * @param owner A janela a partir da qual esse dilogo foi chamado.
   * @param openDialog Indica se o dilogo  de abertura ou remoo de projetos.
   */
  protected CommonProjectSelectDialog(Window owner, boolean openDialog) {
    this.owner = owner;
    this.projectNames = getProjectNames();
    this.openDialog = openDialog;
    StringUtils.sort(projectNames);
    dialog = makeDialog();
    dialog.setVisible(true);
  }
}

/**
 * A classe <code>CommonProjectDeleteDialog</code> usa o dilogo com todos os
 * projetos de um usuario para remocao de um.
 */
class CommonProjectDeleteDialog extends CommonProjectSelectDialog {
  /** Resultado da remocao */
  protected boolean isDeleted = false;

  /**
   * Cria o painel com os botes do dilogo.
   * 
   * @return .
   */
  @Override
  protected JPanel makeButtonPanel() {
    JPanel panel = new JPanel();
    defaultButton = (JButton) panel.add(new JButton(LNG
        .get("CommonProjectSelectDialog.delete")));
    defaultButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        handleAction();
      }
    });

    JButton cancelButton = (JButton) panel.add(new JButton(LNG
        .get("CommonProjectSelectDialog.cancel")));
    cancelButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        dialog.close();
      }
    });
    return panel;
  }

  /**
   * Executa a ao de remover o projeto. Chama a ao de fechar o projeto caso
   * ele esteja aberto.
   */
  @Override
  protected void handleAction() {
    if (!confirmDeletion()) {
      return;
    }
    CommonClientProject openProject = DesktopFrame.getInstance().getProject();
    ProjectTree tree = DesktopFrame.getInstance().getTree();
    final Object[] selectedProjects = projectList.getSelectedValues();

    // Verifica se algum dos projetos selecionados est aberto.
    // Se tiver, fecha
    for (int i = 0; i < selectedProjects.length; i++) {
      String projName = (String) selectedProjects[i];
      if (openProject != null && projName.equals(openProject.getName())) {
        ProjectCloseAction pca = new ProjectCloseAction(tree);
        pca.close();
      }
    }

    // Remove os projetos
    RemoteTask<Void> task = new RemoteTask<Void>() {
      @Override
      protected void performTask() throws Exception {
        for (int i = 0; i < selectedProjects.length; i++) {
          CommonClientProject.remove(User.getLoggedUser().getId(),
              (String) selectedProjects[i]);
        }
      }
    };
    if (task.execute(owner, getTitle(), LNG
        .get("CommonProjectSelectDialog.info.remove.project"))) {
      isDeleted = true;
      dialog.close();
    }
  }

  /**
   * Exibe a janela de confirmao.
   * 
   * @return .
   */
  private boolean confirmDeletion() {
    String msg;
    Object[] options = { LNG.get("CommonProjectSelectDialog.delete"),
        LNG.get("CommonProjectSelectDialog.cancel") };
    if ((projectList.getSelectedValues()).length > 1) {
      msg = LNG.get("CommonProjectSelectDialog.info.confirm.removal.n");
    } else {
      msg = String.format(LNG
          .get("CommonProjectSelectDialog.info.confirm.removal"),
          (String) projectList.getSelectedValue());
    }
    int opt = StandardDialogs.showOptionDialog(owner, getTitle(), msg, options);
    if (opt != 0) {
      return false;
    }
    return true;
  }

  /**
   * {@inheritDoc}
   * 
   * @see csbase.client.project.CommonProjectSelectDialog#getTitle()
   */
  @Override
  protected String getTitle() {
    return LNG.get("CommonProjectSelectDialog.title.delete");
  }

  /**
   * Constri uma viso de dilogo para remoo de projetos de um um usurio.
   * 
   * @param owner A janela a partir da qual esse dilogo foi chamado.
   */
  protected CommonProjectDeleteDialog(Window owner) {
    super(owner, false);
  }
}
