/*
 * ProjectDirectoryCreationDialog.java
 *
 * $Author: juliana $
 * $Revision: 99257 $  - $Date: 2009-11-26 17:41:21 -0200 (Thu, 26 Nov 2009) $
 */
package csbase.client.project;

import java.awt.BorderLayout;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.GUIUtils;
import csbase.client.desktop.*;
import csbase.client.util.ClientUtilities;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.ProjectFile;

/**
 * A classe <code>ProjectDirectoryCreationDialog</code> mostra um dilogo para
 * criao de um sub-diretrio de projeto. O usurio pode acionar o boto de
 * criao, que confirma a operao, ou pode acionar o  boto de cancelamento,
 * que desiste da operao. O boto de criao somente  habilitado quando o
 * usurio tiver informado um nome do diretrio. Ao confirmar as operao, o
 * servio  executado no servidor e o usurio  notificado de qualquer
 * problema que tenha ocorrido.
 *
 * @author Maria Julia de Lima
 */
public class ProjectDirectoryCreationDialog {
  /** A janela de visualizao de informaes do projeto. */
  protected DesktopComponentDialog dialog;
  /** A janela a partir da qual esse dilogo foi solicitado. */
  protected Window owner;
  /** O campo de nome do diretrio */
  protected JTextField nameField;
  /** O botao default. */
  protected JButton defaultButton;
  /** O diretrio pai, onde o sub-diretrio est sendo criado. */
  protected ProjectFile directory;

  /**
   * Mostra um dilogo que pede o nome do diretrio a ser criado.
   *
   * @param owner A janela a partir da qual esse dilogo foi solicitado.
   * @param directory O diretrio que vai ter o novo sub-diretrio.
   */
  public static void show(Window owner, ProjectFile directory) {
    new ProjectDirectoryCreationDialog(owner, directory);
  }

  /**
   * Cria o dilogo modal, a partir de uma janela principal.
   *
   * @return O dilogo recm-criado.
   */
  protected DesktopComponentDialog makeDialog() {
    /* Cria o painel principal da janela */
    JPanel mainPanel = new JPanel(new BorderLayout(2, 5));
    mainPanel.add(makeNamePanel(), BorderLayout.NORTH);
    mainPanel.add(makeButtonPanel(), BorderLayout.SOUTH);
    /* Cria um dilogo modal */
    DesktopComponentDialog dialog;
    if (owner instanceof JDialog) {
      dialog = new DesktopComponentDialog((JDialog)owner, getTitle());
    }
    else {
      dialog = new DesktopComponentDialog((JFrame)owner, getTitle());
    }
    dialog.getContentPane().add(mainPanel);
    dialog.pack();
    dialog.center(owner);
    dialog.getRootPane().setDefaultButton(defaultButton);
    return dialog;
  }

  /**
   * Cria um painel com o campo de nome do diretrio. O campo de nome  um
   * campo de texto de apenas uma nica linha e possui um listener para que,
   * quando um nome for digitado, \ o boto principal seja habilitado.
   *
   * @return O painel recm-criado.
   */
  private JPanel makeNamePanel() {
    nameField = new JTextField(20);
    JComponent[][] components =
    {
      { new JLabel(LNG.get("PRJ_NAME")), nameField },
    };
    nameField.addKeyListener(
      new KeyAdapter() {
        @Override
        public void keyReleased(KeyEvent e) {
          String name = nameField.getText().trim();
          if (name.equals("")) {
            defaultButton.setEnabled(false);
          }
          else {
            defaultButton.setEnabled(true);
          }
        }
      });
    return GUIUtils.createBasicGridPanel(components);
  }

  /**
   * Cria um painel com os botes que vo ser usados no dilogo  de criao de
   * diretrio. Na criao de um diretrio, o usurio pode acionar o boto de
   * criao, que confirma a operao, ou pode acionar o  boto de
   * cancelamento, que desiste da operao. O boto de criao somente 
   * habilitado quando o usurio tiver informado um nome do diretrio.
   *
   * @return O painel recm-criado.
   */
  protected JPanel makeButtonPanel() {
    JPanel panel = new JPanel();
    /* Boto de criao de diretrio */
    defaultButton = (JButton)panel.add(new JButton(LNG.get("PRJ_CREATION")));
    JButton cancelButton =
      (JButton)panel.add(new JButton(LNG.get("PRJ_CANCEL")));
    defaultButton.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          handleAction();
        }
      });
    defaultButton.setEnabled(false);
    /* Boto de cancelamento da operao de criao. */
    cancelButton.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          dialog.close();
        }
      });
    return panel;
  }

  /**
   * Informa o ttulo que vai ser utilizado na janela de criao de projeto.
   *
   * @return O ttulo da janela.
   */
  protected String getTitle() {
    return ClientUtilities.addSystemNameToTitle(
             LNG.get("project.directory_creation_title"));
  }

  /**
   * Executa a ao de criao de um diretrio. O servio  chamado no
   * servidor. Caso algum problema ocorra, o usurio  informado com  uma
   * mensagem em uma outra janela de dilogo.
   */
  private void handleAction() {
    final String name = nameField.getText().trim();
    if (!ClientUtilities.isValidFileName(name)) {
      StandardErrorDialogs.showErrorDialog(
        owner, getTitle(), LNG.get("PRJ_PROJECT_FILE_NAME_CHARACTERES_ERROR"));
      return;
    }
    RemoteTask<Void> task =
      new RemoteTask<Void>() {
        @Override
        public void performTask() throws Exception {
          directory.createDirectory(name);
        }
      };
    if (
      task.execute(dialog, getTitle(), LNG.get("PRJ_WAITING_CREATE_DIRECTORY", 
        new String[]{name}))){
      dialog.close();
    }
  }

  /**
   * Constri o dilogo e mostra para o usurio.
   *
   * @param owner A janela a partir da qual esse dilogo foi solicitado.
   * @param directory O diretrio que vai ter o novo sub-diretrio.
   */
  private ProjectDirectoryCreationDialog(Window owner, ProjectFile directory) {
    this.owner = owner;
    this.directory = directory;
    dialog = makeDialog();
    dialog.setVisible(true);
  }
}
