/*
 * ProjectFileChooser.java $Author: ernani $ $Revision: 84801 $ - $Date:
 * 2006-08-16 18:31:12 -0300 (Wed, 16 Aug 2006) $
 */
package csbase.client.project;

import java.awt.BorderLayout;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.GUIUtils;
import csbase.client.desktop.DesktopComponentDialog;
import csbase.client.kernel.ClientException;
import csbase.client.util.ClientUtilities;
import csbase.logic.ClientProjectFile;
import csbase.logic.CommonClientProject;
import csbase.logic.ProjectFile;
import csbase.logic.ProjectFileFilter;
import csbase.logic.ProjectFileType;

/**
 * A classe <code>ProjectFileChooser</code> mostra um dilogo modal para seleo
 * de arquivos de projeto.
 * 
 * @author Maria Julia
 */
public abstract class ProjectFileChooser {
  protected static final String ALL_FILES = LNG.get("UTIL_ALL_FILES");

  /** Representa o modo de seleo de arquivo. */
  public static final int FILE_ONLY = 0;

  /** Representa o modo de seleo de diretrio. */
  public static final int DIRECTORY_ONLY = 1;

  /** Representa o modo de seleo de arquivo e diretrio. */
  public static final int FILE_AND_DIRECTORY = 2;

  /** A rvore de arquivos de um projeto */
  protected ProjectTree projectTree;

  /** Seleo do tipo de arquivo */
  protected JComboBox fileTypeList;

  /**
   * Campo de texto correspondente ao nome de um arquivo selecionado ou que est
   * sendo criado.
   */
  protected JTextField fileNameText;

  /** Array de arquivos selecionados na arvore */
  protected ProjectTreePath[] selectedPaths;

  /** O dilogo de seleo que ser mostrado para se efetuar a seleo. */
  protected DesktopComponentDialog dialog;

  /** Boto que confirma a seleo. */
  private JButton confirmButton;

  /** Representa o ouvinte de eventos de seleo na rvore de projeto. */
  private ProjectTreeSelectionListener treeSelectionListener;

  /**
   * Representa o modo de seleo escolhido.
   * 
   * @see #FILE_ONLY
   * @see #DIRECTORY_ONLY
   * @see #FILE_AND_DIRECTORY
   */
  protected int mode;

  private JLabel fileNameLabel;

  /**
   * Constri uma viso de dilogo para seleo de um arquivo de um projeto.
   * 
   * @param owner janela pai onde o dilogo ser exibido
   * @param title ttulo do dilogo
   * @param project projeto do qual o usurio vai selecionar um arquivo
   * @param isMultiSelectionEnabled define se o usurio pode selecionar mais de
   *        um arquivo
   * @param mode modo de seleo do dilogo
   * @see #FILE_ONLY
   * @see #DIRECTORY_ONLY
   * @see #FILE_AND_DIRECTORY
   * @param defaultFileType tipo de arquivo default do dilogo
   * @param visualFilter filtro de visualizao da rvore
   * @param selectionFilter filtro de seleo da rvore
   * @param canEditText define se o atributo {@link #fileNameText} ser editvel
   * @param defaultName valor sugerido inicialmente para o nome do arquivo
   *        (atributo {@link #fileNameText})
   * @throws ClientException exceo lanada caso ocorra algum erro na criao
   *         da rvore de projeto
   */
  protected ProjectFileChooser(Window owner, String title,
    CommonClientProject project, boolean isMultiSelectionEnabled, int mode,
    String defaultFileType, ProjectFileFilter visualFilter,
    ProjectFileFilter selectionFilter, boolean canEditText, String defaultName)
    throws ClientException {
    this.mode = mode;
    if (owner instanceof JFrame) {
      this.dialog = new DesktopComponentDialog((JFrame) owner, title);
    }
    else {
      this.dialog = new DesktopComponentDialog((JDialog) owner, title);
    }
    this.buildProjectTree(title, project, isMultiSelectionEnabled,
      visualFilter, selectionFilter);
    JPanel contentPanel = new JPanel(new BorderLayout(0, 5));
    contentPanel.add(new JScrollPane(this.projectTree.getTree()),
      BorderLayout.CENTER);
    contentPanel.add(makeTextFieldPanel(defaultFileType, canEditText),
      BorderLayout.SOUTH);
    JPanel mainPanel = new JPanel(new BorderLayout(0, 5));
    mainPanel.add(contentPanel, BorderLayout.CENTER);
    mainPanel.add(makeButtonPanel(), BorderLayout.SOUTH);
    this.dialog.getContentPane().add(mainPanel);
    this.dialog.pack();
    this.dialog.center(owner);
    this.dialog.getRootPane().setDefaultButton(this.confirmButton);
    this.dialog.addWindowListener(new WindowAdapter() {
      @Override
      public void windowClosing(WindowEvent e) {
        ProjectFileChooser.this.selectedPaths = null;
        ProjectFileChooser.this.close();
      }
    });
    this.projectTree.setSelectionPath(project.getPath());
    this.fileNameText.requestFocus();
    this.dialog.center();

    if (mode == FILE_ONLY) {
      if (defaultName != null) {
        this.fileNameText.setText(defaultName);
        this.confirmButton.setEnabled(true);
      }
    }
    else {
      // diretrios e/ou arquivos
      // habilitar o boto de confirmao, pois o prprio projeto estar
      // selecionado por default caso no tenha um texto padro
      this.confirmButton.setEnabled(true);
      if (mode == DIRECTORY_ONLY) {
        // no mostramos campo de texto para seleo de diretrios
        this.fileNameText.setVisible(false);
        this.fileNameLabel.setVisible(false);
      }
      else {
        // arquivos e diretrios
        if (defaultName != null)
          this.fileNameText.setText(defaultName);
      }
    }
  }

  /**
   * Cria o painel com os botes do dilogo.
   * <ul>
   * <li>o boto de confirmao da seleo.</li>
   * <li>o boto de cancelamento da seleo.</li>
   * </ul>
   * 
   * @return O painel com os botes.
   */
  private JPanel makeButtonPanel() {
    JPanel butPanel = new JPanel();
    this.confirmButton = new JButton(LNG.get("PRJ_FILE_CHOOSER_CONFIRM"));
    this.confirmButton.setEnabled(false);
    this.confirmButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (ProjectFileChooser.this.handleAction()) {
          ProjectFileChooser.this.close();
        }
      }
    });

    JButton cancelButton = new JButton(LNG.get("PRJ_FILE_CHOOSER_CANCEL"));
    cancelButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        ProjectFileChooser.this.selectedPaths = null;
        ProjectFileChooser.this.close();
      }
    });

    ClientUtilities.adjustEqualSizes(this.confirmButton, cancelButton);
    butPanel.add(this.confirmButton);
    butPanel.add(cancelButton);
    return butPanel;
  }

  /**
   * Cria o painel com os campos de nome de arquivo e de filtro.
   * 
   * @param defaultFileType O tipo padro que ser selecionado na criao do
   *        dilogo.
   * @param canEditText Define se o campo texto poder ser alterado pelo
   *        usurio.
   * @return O painel com os campos de arquivo e filtro.
   */
  private JPanel makeTextFieldPanel(String defaultFileType, boolean canEditText) {
    this.fileNameText = new JTextField(30);
    this.fileNameText.setEditable(canEditText);
    this.fileNameLabel = new JLabel(LNG.get("PRJ_FILE_CHOOSER_FILE_NAME"));
    this.fileNameText.getDocument().addDocumentListener(new DocumentListener() {
      public void changedUpdate(DocumentEvent e) {
        this.textChanged();
      }

      public void insertUpdate(DocumentEvent e) {
        this.textChanged();
      }

      public void removeUpdate(DocumentEvent e) {
        this.textChanged();
      }

      public void textChanged() {
        ProjectFileChooser.this.updateConfirmButton();
      }
    });

    JComponent[][] components = null;
    if (this.mode != DIRECTORY_ONLY) {
      JLabel fileTypeLabel =
        new JLabel(LNG.get("PRJ_FILE_CHOOSER_FILTER_NAME"));

      /* Tipos de arquivos */
      ProjectFileType[] fTypes = ProjectFileType.getAllFileTypes();
      this.fileTypeList = new JComboBox(new DefaultComboBoxModel(fTypes));
      this.fileTypeList.addItem(ALL_FILES);
      if (defaultFileType != null) {
        this.fileTypeList.setSelectedItem(ProjectFileType
          .getFileType(defaultFileType));
      }
      else {
        this.fileTypeList.setSelectedItem(ALL_FILES);
      }
      this.fileTypeList.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          String oldFileName = fileNameText.getText();
          Object selectedFileType = fileTypeList.getSelectedItem();
          ProjectFileFilter filter;
          if (selectedFileType.equals(ALL_FILES)) {
            filter = ProjectFileChooser.this.getVisualFilter();
          }
          else {
            filter =
              ProjectFileChooser.this
                .getVisualFilter(((ProjectFileType) fileTypeList
                  .getSelectedItem()).getCode());
          }
          projectTree.setVisualFilter(filter);
          if (selectedFileType.equals(ALL_FILES)) {
            filter = ProjectFileChooser.this.getSelectionFilter();
          }
          else {
            filter =
              ProjectFileChooser.this
                .getSelectionFilter(((ProjectFileType) fileTypeList
                  .getSelectedItem()).getCode());
          }
          projectTree.setSelectionFilter(filter);
          ProjectFileChooser.this.updateConfirmButton();
          fileNameText.setText(oldFileName);
        }
      });
      components =
        new JComponent[][] { { fileNameLabel, this.fileNameText },
            { fileTypeLabel, this.fileTypeList }, };
    }
    else {
      components = new JComponent[][] { { fileNameLabel, this.fileNameText } };
    }
    JPanel txtPanel = GUIUtils.createBasicGridPanel(components);
    return txtPanel;
  }

  /**
   * Habilita ou desabilita o {@link #confirmButton} de acordo com o estado
   * atual do dilogo.
   */
  private void updateConfirmButton() {
    if (mode == FILE_ONLY) {
      boolean hasFilename = fileNameText.getText().trim().length() > 0;
      this.confirmButton.setEnabled(hasFilename && getSelectedPath() != null);
    }
    else
      this.confirmButton.setEnabled(getSelectedPath() != null);
  }

  /**
   * Executa a ao de seleo.  executada quando o usurio clica no boto
   * {@link #confirmButton} ou quando ele executa um duplo clique num arquivo
   * selecionado.
   * 
   * @return true, se a ao foi executada com sucesso, ou false, caso
   *         contrrio.
   */
  protected abstract boolean handleAction();

  private void buildProjectTree(String title, CommonClientProject project,
    boolean isMultiSelectionEnabled, ProjectFileFilter visualFilter,
    ProjectFileFilter selectionFilter) throws ClientException {
    this.projectTree =
      ProjectTreeFactory.getInstance().makeProjectTree(this.dialog, title,
        project, visualFilter, selectionFilter);
    this.projectTree.setSingleSelectionMode(!isMultiSelectionEnabled);
    this.projectTree.getTree().addMouseListener(
      new ProjectTreeMouseAdapter(this.projectTree) {
        @Override
        public void doubleClickAction() {
          if (ProjectFileChooser.this.handleAction()) {
            ProjectFileChooser.this.close();
          }
        }
      });
    this.treeSelectionListener = new ProjectTreeSelectionListener() {
      public void update(ProjectTreeSelectionEvent event) {
        ProjectFileChooser.this.makeSelection(event.getSelectedFiles());
        ProjectFileChooser.this.updateConfirmButton();
      }
    };
    this.projectTree.addTreeSelectionListener(this.treeSelectionListener);
  }

  protected abstract void makeSelection(ClientProjectFile[] projectFileArray);

  protected abstract ProjectFileFilter getVisualFilter();

  protected abstract ProjectFileFilter getVisualFilter(String fileType);

  protected abstract ProjectFileFilter getSelectionFilter();

  protected abstract ProjectFileFilter getSelectionFilter(String fileType);

  /**
   * Obtm o caminho do ltimo arquivo selecionado na rvore.
   * 
   * @return o caminho do arquivo selecionado
   */
  public ProjectTreePath getSelectedPath() {
    if (this.selectedPaths == null || this.selectedPaths.length == 0) {
      return null;
    }
    return this.selectedPaths[this.selectedPaths.length - 1];
  }

  /**
   * Fecha o dilogo e libera os ouvintes que esse dilogo inseriu na rvore de
   * projetos.
   */
  private void close() {
    this.projectTree.removeTreeSelectionListener(this.treeSelectionListener);
    this.projectTree.release();
    this.dialog.close();
  }

  /**
   * Obtm o arquivo que est atualmente selecionado.
   * 
   * @return O arquivo selecionado ou null, caso no haja seleo.
   */
  protected ProjectFile getSelectedFile() {
    return this.projectTree.getSelectedFile();
  }

  /**
   * Obtm os arquivos que esto atualmente selecionados.
   * 
   * @return Os arquivos atualmente selecionados ou null, caso no haja seleo.
   */
  protected ProjectFile[] getSelectedFiles() {
    return this.projectTree.getSelectedFiles();
  }

  /**
   * Obtm os paths atualmente selecionados.
   * 
   * @return Os paths atualmente selecionados ou null, caso no haja seleo.
   */
  public ProjectTreePath[] getSelectedPaths() {
    return this.selectedPaths;
  }

  /**
   * @return um ttulo especial para o dilogo. Null para usar o padro.
   */
  protected String getTitle() {
    return null;
  }
}
