/*
 * $Id: ProjectTreeModel.java 88986 2009-03-06 19:33:11Z ururahy $
 */
package csbase.client.project;

import java.util.Comparator;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import java.util.Set;

import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

import csbase.logic.ClientProjectFile;
import csbase.logic.CommonClientProject;
import csbase.logic.DirRefreshedEvent;
import csbase.logic.NewProjectFileEvent;
import csbase.logic.NewProjectFilesEvent;
import csbase.logic.ProjectDeletedEvent;
import csbase.logic.ProjectEvent;
import csbase.logic.ProjectFile;
import csbase.logic.ProjectFileDeletedEvent;
import csbase.logic.ProjectFileFilter;
import csbase.logic.ProjectFileRenamedEvent;
import csbase.logic.ProjectFileStateChangedEvent;
import csbase.logic.ProjectFilesDeletedEvent;
import csbase.logic.ProjectInfoModifiedEvent;
import csbase.logic.TreeRefreshedEvent;
import csbase.logic.filters.ProjectFileNullFilter;

/**
 * Define um modelo de dados para uma rvore de projetos.
 * 
 * @author Rodrigo Carneiro Henrique (rodrigoh)
 */
public final class ProjectTreeModel implements TreeModel {
  /** O n raz da rvore de projetos. */
  private ProjectTreeNode rootNode;

  /** Coleo de ouvintes de eventos de alteraes do modelo. */
  private Set<TreeModelListener> listenerSet;
  
  /** Coleo de ouvintes de eventos de alteraes do modelo. */
  private Set<ProjectTreeStructureListener> listenerChangesSet;

  /** O projeto que  representado por este modelo. */
  private CommonClientProject clientProject;

  /**
   * Observador do projeto. Usado pelo modelo para se atualizar em relao a
   * mudanas no projeto.
   */
  private ProjectObserver projectObserver;

  /**
   * O filtro responsvel por definir que arquivos sero representados por esse
   * modelo.
   */
  private ProjectFileFilter filter;

  /**
   * Utilizado para ordenar a rvore. {@link ProjectTreeNodeExtensionComparator}{@link ProjectTreeNodeNameComparator}
   */
  private Comparator comparator;

  /**
   * Cria um modelo para a rvore de projetos que ter um filtro nulo, ou seja,
   * todos os arquivos do projeto sero representados na rvore. Alm disso,
   * ser utilizado um comparador que ordenar a rvore pelos nomes dos
   * arquivos.
   * 
   * @param clientProject O projeto que ser representado pelo modelo.
   */
  public ProjectTreeModel(CommonClientProject clientProject) {
    this(clientProject, ProjectFileNullFilter.getInstance(),
      ProjectTreeNodeNameComparator.getInstance());
  }

  /**
   * Cria um modelo para a rvore de projetos que ter um filtro nulo, ou seja,
   * todos os arquivos do projeto sero representados na rvore.
   * 
   * @param clientProject O projeto que ser representado pelo modelo.
   * @param comparator O comparador que ser utilizado para ordenar a rvore de
   *        projeto.
   */
  public ProjectTreeModel(CommonClientProject clientProject,
    Comparator comparator) {
    this(clientProject, ProjectFileNullFilter.getInstance(), comparator);
  }

  /**
   * Cria um modelo para a rvore de projetos que ser ordenada pelos nomes dos
   * arquivos.
   * 
   * @param clientProject O projeto que ser representado pelo modelo.
   * @param filter O filtro que definir que arquivos de projeto sero
   *        representados pelo modelo.
   */
  public ProjectTreeModel(CommonClientProject clientProject,
    ProjectFileFilter filter) {
    this(clientProject, filter, ProjectTreeNodeNameComparator.getInstance());
  }

  /**
   * Cria um modelo para a rvore de projetos.
   * 
   * @param clientProject O projeto que ser representado pelo modelo.
   * @param filter O filtro que definir que arquivos de projeto sero
   *        representados pelo modelo.
   * @param comparator O comparador que ser utilizado para ordenar a rvore de
   *        projeto.
   */
  public ProjectTreeModel(CommonClientProject clientProject,
    ProjectFileFilter filter, Comparator comparator) {
    this.listenerSet = new HashSet<TreeModelListener>();
    this.listenerChangesSet = new HashSet<ProjectTreeStructureListener>();
    this.projectObserver = new ProjectObserver();
    this.filter = filter;
    this.comparator = comparator;
    this.setProject(clientProject);
  }

  /**
   * Altera o modelo para que passe a representar um novo projeto.
   * 
   * @param clientProject O novo projeto que ser representado pelo modelo.
   */
  void setProject(CommonClientProject clientProject) {
    if (clientProject != null) {
      this.rootNode = new ProjectTreeNode(clientProject, this.filter,
        this.comparator);
      clientProject.addObserver(this.projectObserver);
    }
    else {
      this.rootNode = null;
    }
    if (this.clientProject != null) {
      this.clientProject.deleteObserver(this.projectObserver);
    }
    this.clientProject = clientProject;
    if (this.rootNode != null) {
      this.fireTreeStructureChanged(this.rootNode.getTreePath());
    }
    else {
      this.fireTreeStructureChanged(null);
    }
  }

  /**
   * Consulta o valor de clientProject
   * @return o valor
   */
  public final CommonClientProject getProject() {
    return clientProject;
  }

  /**
   * Altera o modelo para que passe a ser ordenado pelo novo comparador.
   * 
   * @param comparator O novo comparador do modelo.
   */
  void setComparator(Comparator comparator) {
    this.comparator = comparator;
    this.rootNode = new ProjectTreeNode(this.clientProject, this.filter,
      this.comparator);
    this.fireTreeStructureChanged(this.rootNode.getTreePath());
  }

  /**
   * Envia o evento de alterao na estrutura de uma sub-rvore para todos os
   * ouvintes cadastrados.
   * 
   * @param subTreeRoot O n-raz da sub-rvore que teve sua estrutura alterada.
   */
  void fireTreeStructureChanged(TreePath subTreeRoot) {
    TreeModelEvent treeModelEvent = new TreeModelEvent(this, subTreeRoot);
    this.fireTreeWillChange(subTreeRoot);
    Iterator<TreeModelListener> listenerIterator = this.listenerSet.iterator();
    while (listenerIterator.hasNext()) {
      TreeModelListener treeModelListener = listenerIterator.next();
      treeModelListener.treeStructureChanged(treeModelEvent);
    }
    this.fireTreeWasChanged(subTreeRoot);
  }

  public void addTreeWillChangeStructureListener(ProjectTreeStructureListener l) {
    this.listenerChangesSet.add(l);
  }

  public void removeTreeWillChangeStructureListener(
    ProjectTreeStructureListener l) {
    this.listenerChangesSet.remove(l);
  }

  private void fireTreeWillChange(TreePath subTreeRoot) {
    TreeModelEvent event = new TreeModelEvent(this, subTreeRoot);
    Iterator<ProjectTreeStructureListener> listenerChangesIterator = this.listenerChangesSet
      .iterator();
    while (listenerChangesIterator.hasNext()) {
      ProjectTreeStructureListener listener = listenerChangesIterator.next();
      listener.treeWillChange(event);
    }
  }

  private void fireTreeWasChanged(TreePath subTreeRoot) {
    TreeModelEvent event = new TreeModelEvent(this, subTreeRoot);
    Iterator<ProjectTreeStructureListener> listenerChangesIterator = this.listenerChangesSet
      .iterator();
    while (listenerChangesIterator.hasNext()) {
      ProjectTreeStructureListener listener = listenerChangesIterator.next();
      listener.treeWasChanged(event);
    }
  }

  /**
   * DOCUMENT ME!
   * 
   * @param removedIndexes
   * @param removedNodes
   */
  private void sortIndexes(int[] removedIndexes, ProjectTreeNode[] removedNodes) {
    for (int i = 0; i < removedNodes.length - 1; i++) {
      for (int j = i + 1; j < removedNodes.length; j++) {
        if (removedIndexes[j] < removedIndexes[i]) {
          int index = removedIndexes[i];
          removedIndexes[i] = removedIndexes[j];
          removedIndexes[j] = index;
          ProjectTreeNode treeNode = removedNodes[i];
          removedNodes[i] = removedNodes[j];
          removedNodes[j] = treeNode;
        }
      }
    }
  }

  /**
   * Envia o evento de insero de ns para todos os ouvintes cadastrados.
   * 
   * @param parentTreePath O n-raz (pai) onde os ns foram inseridos.
   * @param insertedIndexes Os ndices dos ns na lista de filhos do n pai.
   * @param insertedNodes Os ns que foram inseridos.
   */
  private void fireTreeNodesInserted(TreePath parentTreePath,
    int[] insertedIndexes, ProjectTreeNode[] insertedNodes) {
    this.sortIndexes(insertedIndexes, insertedNodes);
    TreeModelEvent treeModelEvent = new TreeModelEvent(this, parentTreePath,
      insertedIndexes, insertedNodes);
    Iterator<TreeModelListener> listenerIterator = this.listenerSet.iterator();
    while (listenerIterator.hasNext()) {
      TreeModelListener treeModelListener = listenerIterator.next();
      try {
        treeModelListener.treeNodesInserted(treeModelEvent);
      }
      catch (Throwable t) {
        t.printStackTrace();
      }
    }
  }

  /**
   * Envia o evento de remoo de ns para todos os ouvintes cadastrados.
   * 
   * @param parentTreePath O n-raz (pai) de onde os ns foram removidos.
   * @param removedIndexes Os ndices dos ns na lista de filhos do n pai.
   * @param removedNodes Os ns que foram removidos.
   */
  private void fireTreeNodesRemoved(TreePath parentTreePath,
    int[] removedIndexes, ProjectTreeNode[] removedNodes) {
    this.sortIndexes(removedIndexes, removedNodes);
    TreeModelEvent treeModelEvent = new TreeModelEvent(this, parentTreePath,
      removedIndexes, removedNodes);
    Iterator<TreeModelListener> listenerIterator = this.listenerSet.iterator();
    while (listenerIterator.hasNext()) {
      TreeModelListener treeModelListener = listenerIterator.next();
      try {
        treeModelListener.treeNodesRemoved(treeModelEvent);
      }
      catch (Throwable t) {
        t.printStackTrace();
      }
    }
  }

  /**
   * Envia o evento de alterao de ns para todos os ouvintes cadastrados.
   * 
   * @param parentTreePath O n-raz (pai) de onde os ns foram alterados.
   * @param changedIndexes Os ndices dos ns na lista de filhos do n pai.
   * @param changedNodes Os ns que foram alterados.
   */
  private void fireTreeNodesChanged(TreePath parentTreePath,
    int[] changedIndexes, ProjectTreeNode[] changedNodes) {
    this.sortIndexes(changedIndexes, changedNodes);
    TreeModelEvent treeModelEvent = new TreeModelEvent(this, parentTreePath,
      changedIndexes, changedNodes);
    Iterator<TreeModelListener> listenerIterator = this.listenerSet.iterator();
    while (listenerIterator.hasNext()) {
      TreeModelListener treeModelListener = listenerIterator.next();
      try {
        treeModelListener.treeNodesChanged(treeModelEvent);
      }
      catch (Throwable t) {
        t.printStackTrace();
      }
    }
  }

  public Object getRoot() {
    return this.rootNode;
  }

  public int getChildCount(Object parent) {
    ProjectTreeNode projectTreeNode = (ProjectTreeNode) parent;
    return projectTreeNode.getChildCount();
  }

  public boolean isLeaf(Object node) {
    ProjectTreeNode projectTreeNode = (ProjectTreeNode) node;
    return projectTreeNode.isLeaf();
  }

  public void addTreeModelListener(TreeModelListener l) {
    this.listenerSet.add(l);
  }

  public void removeTreeModelListener(TreeModelListener l) {
    this.listenerSet.remove(l);
  }

  public Object getChild(Object parent, int index) {
    ProjectTreeNode projectTreeNode = (ProjectTreeNode) parent;
    return projectTreeNode.getChildAt(index);
  }

  public int getIndexOfChild(Object parent, Object child) {
    ProjectTreeNode projectTreeNode = (ProjectTreeNode) parent;
    return projectTreeNode.getIndex((ProjectTreeNode) child);
  }

  public void valueForPathChanged(TreePath path, Object newValue) {
  }

  /**
   * Retorna o caminho na rvore onde um determinado arquivo se encontra.
   * 
   * @param file O arquivo do qual se deseja o caminho na rvore.
   * 
   * @return O caminho para o arquivo na rvore, ou null, caso o arquivos no
   *         exista na rvore.
   * 
   * @throws IllegalArgumentException Caso o arquivo recebido seja null.
   */
  public TreePath getTreePath(ProjectFile file) {
    if (file == null) {
      throw new IllegalArgumentException(
        "No  possvel obter o TreePath de um ProjectFile null.");
    }
    return this.getTreePath(file.getPath());
  }

  private TreePath getTreePath(String[] path) {
    if (path == null) {
      throw new IllegalArgumentException(
        "No  possvel obter o TreePath de um caminho null.");
    }
    ProjectTreeNode[] pathToRootArray = new ProjectTreeNode[path.length];
    ProjectTreeNode node = this.rootNode;
    pathToRootArray[0] = node;
    for (int i = 1; i < path.length; i++) {
      node = node.getChild(path[i]);
      if (node == null) {
        return null;
      }
      pathToRootArray[i] = node;
    }
    return new TreePath(pathToRootArray);
  }

  /**
   * Verifica se o path existe no modelo.
   * 
   * @param path O path a ser verificado.
   * 
   * @return true, se o path existe no modelo, ou false, caso contrrio.
   */
  public boolean exists(TreePath path) {
    if (this.rootNode == null) {
      return false;
    }
    return exists(this.rootNode, path);
  }

  private boolean exists(ProjectTreeNode node, TreePath path) {
    if (node.getTreePath().equals(path)) {
      return true;
    }
    for (int i = 0; i < node.getChildCount(); i++) {
      if (exists((ProjectTreeNode) node.getChildAt(i), path)) {
        return true;
      }
    }
    return false;
  }

  /**
   * Responsvel por observar mudanas no projeto e atualizar o modelo em
   * relao  essas mudanas.
   * 
   * @author Rodrigo Carneiro Henrique (rodrigoh)
   */
  private class ProjectObserver implements Observer {
    public void update(Observable o, Object arg) {
      final ProjectEvent event = (ProjectEvent) arg;
      try {
        switch (event.event) {
          case ProjectEvent.INFO_MODIFIED:
            handleEvent((ProjectInfoModifiedEvent) event);
            break;
          case ProjectEvent.PROJECT_DELETED:
            handleEvent((ProjectDeletedEvent) event);
            break;
          case ProjectEvent.NEW_FILE:
            handleEvent((NewProjectFileEvent) event);
            break;
          case ProjectEvent.FILE_DELETED:
            handleEvent((ProjectFileDeletedEvent) event);
            break;
          case ProjectEvent.FILES_DELETED:
            handleEvent((ProjectFilesDeletedEvent) event);
            break;
          case ProjectEvent.NEW_FILE_NAME:
            handleEvent((ProjectFileRenamedEvent) event);
            break;
          case ProjectEvent.FILE_STATE_CHANGED:
            handleEvent((ProjectFileStateChangedEvent) event);
            break;
          case ProjectEvent.TREE_REFRESHED:
            handleEvent((TreeRefreshedEvent) event);
            break;
          case ProjectEvent.DIR_REFRESHED:
            handleEvent((DirRefreshedEvent) event);
            break;
          case ProjectEvent.NEW_FILES:
            handleEvent((NewProjectFilesEvent) event);
            break;
          default:
            throw new IllegalStateException("Evento desconhecido. Nmero: "
              + event.event + " - Classe: " + event.getClass());
        }
      }
      catch (Throwable t) {
        t.printStackTrace();
      }
    }

  }

  /**
   * Trata o evento de modificao nas informaes do projeto.
   * 
   * @param event informaes sobre o evento de modificao das informaes do
   *        projeto
   */
  private void handleEvent(ProjectInfoModifiedEvent event) {
    final ClientProjectFile file = (ClientProjectFile) clientProject.getTree();
    file.resetChildren();
    final TreePath rootPath = getTreePath(file);
    ProjectTreeNode rootTreeNode = (ProjectTreeNode) rootPath
      .getLastPathComponent();
    rootTreeNode.loadGrandChildren();
  }

  /**
   * Trata o evento de remoo do projeto corrente.
   * 
   * @param event informaes sobre o evento de remoo de um projeto
   */
  private void handleEvent(ProjectDeletedEvent event) {
    ProjectTreeModel.this.setProject(null);
  }

  /**
   * Trata o evento de insero de um novo arquivo no modelo.
   * 
   * @param event informaes sobre o evento de criao de arquivo
   */
  private void handleEvent(NewProjectFileEvent event) {
    String[] path = event.getPath();
    ProjectFile insertedFile = event.getFile();
    addNewNode(path, insertedFile);
  }

  /**
   * Trata o evento de arquivos criados no servidor, atualizando a rvore no
   * cliente com os novos arquivos.
   * 
   * @param event informaes sobre o evento de criao de arquivos.
   */
  private void handleEvent(NewProjectFilesEvent event) {
    
    String[][] paths = event.getPaths();
    if (paths == null) {
      throw new IllegalArgumentException("paths == null");
    }
    ProjectFile[] files = event.getFiles();
    if (files == null) {
      throw new IllegalArgumentException("files == null");
    }
    if (paths.length != files.length) {
      throw new IllegalArgumentException("paths.length != files.length");
    }
    
    for (int inx = 0; inx < files.length; inx++) {
      addNewNode(paths[inx], files[inx]);
    }
  }  
  
  /**
   * Adiciona um novo n na rvore de projeto.
   * 
   * @param path o caminho do n pai, a partir da raiz da rvore, aonde o novo
   *        n.
   * @param file arquivo representando o novo n a ser inserido.
   */
  private void addNewNode(String[] path, ProjectFile file) {
    
    TreePath parentTreePath = ProjectTreeModel.this.getTreePath(path);
    if (parentTreePath == null) {
      return;
    }
    ProjectTreeNode parentTreeNode = (ProjectTreeNode) parentTreePath
      .getLastPathComponent();
    if (!parentTreeNode.addChild((ClientProjectFile) file)) {
      return;
    }
    ProjectTreeNode treeNode =
      parentTreeNode.getChild((ClientProjectFile) file);
    /**
     * Se o novo arquivo for um diretrio, a chamada abaixo ir resgatar seus 
     * filhos e netos do servidor para assim poder indicar na rvore que o 
     * diretrio tem filhos.
     */
    treeNode.loadGrandChildren();
    
    ProjectTreeNode[] insertedNodes = new ProjectTreeNode[] { treeNode };
    int[] insertedIndexes = parentTreeNode.getIndexes(insertedNodes);
    this.sort(insertedIndexes, insertedNodes);
    ProjectTreeModel.this.fireTreeNodesInserted(parentTreePath,
      insertedIndexes, insertedNodes);
  }
  
  /**
   * Trata o evento de remoo de um arquivo.
   * 
   * @param event informaes sobre o evento de remoo de arquivo
   */
  private void handleEvent(ProjectFileDeletedEvent event) {
    ProjectFile removedFile = event.getRemovedFile();
    TreePath parentTreePath = ProjectTreeModel.this.getTreePath(removedFile
      .getParent());
    if (parentTreePath == null) {
      return;
    }
    ProjectTreeNode parentTreeNode = (ProjectTreeNode) parentTreePath
      .getLastPathComponent();
    TreePath treePath = ProjectTreeModel.this.getTreePath(removedFile);
    if (treePath == null) {
      parentTreeNode.loadChildren();
      return;
    }
    ProjectTreeNode treeNode = (ProjectTreeNode) treePath
      .getLastPathComponent();
    ProjectTreeNode[] removedNodes = new ProjectTreeNode[] { treeNode };
    int[] removedIndexes = parentTreeNode.getIndexes(removedNodes);
    this.sort(removedIndexes, removedNodes);
    final boolean removed = parentTreeNode
      .removeChild((ClientProjectFile) removedFile);
    if (!removed) {
      return;
    }

    ProjectTreeModel.this.fireTreeNodesRemoved(parentTreePath,
    removedIndexes,
    removedNodes);
  }

  /**
   * Trata o evento de remoo de vrios arquivos.
   * 
   * @param event informaes sobre o evento de remoo de vrios arquivos
   */
  private void handleEvent(ProjectFilesDeletedEvent event) {
    ProjectFile[] removedFiles = event.getRemovedFiles();
    TreePath parentTreePath = ProjectTreeModel.this.getTreePath(removedFiles[0]
      .getParent());
    if (parentTreePath == null) {
      return;
    }
    ProjectTreeNode parentTreeNode = (ProjectTreeNode) parentTreePath
      .getLastPathComponent();
    List<ProjectTreeNode> removedNodes = new LinkedList<ProjectTreeNode>();
    for (int i = 0; i < removedFiles.length; i++) {
      TreePath treePath = ProjectTreeModel.this.getTreePath(removedFiles[i]);
      if (treePath != null) {
        removedNodes.add((ProjectTreeNode) treePath.getLastPathComponent());
      }
    }
    if (removedNodes.size() == 0) {
      parentTreeNode.loadChildren();
      return;
    }
    ProjectTreeNode[] removedNodesArray = removedNodes
      .toArray(new ProjectTreeNode[removedNodes.size()]);

    int[] removedIndexes = parentTreeNode.getIndexes(removedNodesArray);
    this.sort(removedIndexes, removedNodesArray);

    for (int i = 0; i < removedFiles.length; i++) {
      parentTreeNode.removeChild((ClientProjectFile) removedFiles[i]);
    }
    ProjectTreeModel.this.fireTreeNodesRemoved(parentTreePath, removedIndexes,
      removedNodesArray);
  }

  /**
   * Trata o evento de novo nome para um arquivo.
   * 
   * @param event informaes sobre o evento de novo nome para um arquivo
   */
  private void handleEvent(ProjectFileRenamedEvent event) {
    ProjectFile changedFile = event.getFile();
    TreePath parentTreePath = ProjectTreeModel.this.getTreePath(changedFile
      .getParent());
    if (parentTreePath == null) {
      return;
    }
    ProjectTreeNode parentTreeNode = (ProjectTreeNode) parentTreePath
      .getLastPathComponent();
    /*
     * Carga recursiva pois o n pai e seus descendendes podem estar todos
     * abertos
     */
    parentTreeNode.loadChildrenWithRecursion(true);
    TreePath treePath = ProjectTreeModel.this.getTreePath(changedFile);
    if (treePath == null) {
      return;
    }
    ProjectTreeModel.this.fireTreeStructureChanged(parentTreePath);
  }

  /**
   * Trata o evento de alterao do estado do arquivo.
   * 
   * @param event informaes sobre o evento de alterao do estado de um
   *        arquivo
   * @see ProjectFileStateChangedEvent
   */
  private void handleEvent(ProjectFileStateChangedEvent event) {
    ProjectFile changedFile = event.getFile();
    TreePath parentTreePath = ProjectTreeModel.this.getTreePath(changedFile
      .getParent());
    if (parentTreePath == null) {
      return;
    }

    ProjectTreeNode parentTreeNode = (ProjectTreeNode) parentTreePath
      .getLastPathComponent();

    TreePath treePath = ProjectTreeModel.this.getTreePath(changedFile);
    if (treePath == null) {
      return;
    }
    ProjectTreeNode treeNode = (ProjectTreeNode) treePath
      .getLastPathComponent();
    
    // Substituindo o arquivo da rvore pelo arquivo recebido no evento. O 
    // arquivo recebido no evento foi modificado pelo CommonClientProject, 
    // sendo assim, a consistncia do arquivo est garantida, pois no  um
    // arquivo serializado pelo servidor.
    treeNode.setClientProjectFile((ClientProjectFile)changedFile);
    
    ProjectTreeNode[] changedNodes = new ProjectTreeNode[] { treeNode };
    int[] changedIndexes = parentTreeNode.getIndexes(changedNodes);
    this.sort(changedIndexes, changedNodes);
    ProjectTreeModel.this.fireTreeNodesChanged(parentTreePath, changedIndexes,
      changedNodes);
  }

  /**
   * Trata o evento de refresh na rvore.
   * 
   * @param event informaes sobre o evento de refresh na rvore
   */
  private void handleEvent(TreeRefreshedEvent event) {
    ProjectFile rootFile = event.getNewTree();
    ProjectTreeModel.this.rootNode
      .setClientProjectFile((ClientProjectFile) rootFile);
    ProjectTreeModel.this.fireTreeStructureChanged(ProjectTreeModel.this
      .getTreePath(rootFile));
  }

  /**
   * Trata o evento de atualizao de diretrio. Atualiza o n da rvore
   * representando o diretrio selecionado com as informaes oriundas do
   * servidor.
   * 
   * @param event informaes sobre o evento (caminho e diretrio atualizado).
   */
  private void handleEvent(DirRefreshedEvent event) {
    ClientProjectFile dir = (ClientProjectFile) event.getDir();
    TreePath treePath = getTreePath(dir);
    if (treePath == null) {
      return;
    }
    ProjectTreeNode treeNode = (ProjectTreeNode) treePath
      .getLastPathComponent();
    treeNode.setClientProjectFile(dir);
    fireTreeStructureChanged(treePath);
  }

  /**
   * Ordena um array de ndices e seu respectivo array de ns.
   * <p>
   * Os eventos de alterao no modelo que recebem ndice, precisam receber
   * esses ndices ordenados. Os ns devem manter sua posio de array de acordo
   * com a posio de seu ndice.
   * </p>
   * 
   * @param indexes O array com os ndices.
   * @param nodes O array com os ns.
   * 
   * @throws IllegalArgumentException Caso os arrays possuam tamanhos
   *         diferentes.
   */
  private void sort(int[] indexes, ProjectTreeNode[] nodes) {
    if (indexes.length != nodes.length) {
      throw new IllegalArgumentException(
        "Os arrays no podem ter tamanhos diferentes.");
    }
    for (int i = 0; i < indexes.length - 1; i++) {
      for (int j = i + 1; j < indexes.length; j++) {
        if (indexes[i] > indexes[j]) {
          int auxInt = indexes[i];
          indexes[i] = indexes[j];
          indexes[j] = auxInt;
          ProjectTreeNode auxNode = nodes[i];
          nodes[i] = nodes[j];
          nodes[j] = auxNode;
        }
      }
    }
  }

  /**
   * Obtm o filtro utilizado para definir que arquivos podero fazer parte do
   * modelo.
   * 
   * @return O filtro do modelo.
   */
  ProjectFileFilter getFilter() {
    return this.filter;
  }

  /**
   * Altera o filtro do modelo.
   * 
   * @param filter O novo filtro do modelo.
   */
  void setFilter(ProjectFileFilter filter) {
    this.filter = filter;
    if (this.clientProject != null) {
      this.rootNode = new ProjectTreeNode(this.clientProject, this.filter,
        this.comparator);
      rootNode.loadChildrenWithRecursion(false);
      this.fireTreeStructureChanged(this.getTreePath(this.rootNode
        .getClientProjectFile()));

    }
  }
}
