/*
 * $Id: ProjectTreeNode.java 88986 2009-03-06 19:33:11Z ururahy $
 */

package csbase.client.project;

import java.util.Collections;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Stack;

import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import csbase.client.desktop.DesktopFrame;
import csbase.client.desktop.DesktopPreferences;
import csbase.client.project.tasks.GetChildrenTask;
import csbase.logic.ClientProjectFile;
import csbase.logic.CommonClientProject;
import csbase.logic.NoHiddenFileFilter;
import csbase.logic.ProjectFileFilter;

/**
 * Define um n de uma rvore de projetos.
 * 
 * @author Rodrigo Carneiro Henrique (rodrigoh)
 */
public final class ProjectTreeNode implements TreeNode {
  private ProjectTreeNode parent;
  /**
   * Arquivo associado ao n.
   */
  private ClientProjectFile file;
  private List<ProjectTreeNode> childList;
  private ProjectFileFilter filter;
  private Comparator comparator;

  ProjectTreeNode(CommonClientProject project, ProjectFileFilter filter,
    Comparator comparator) {
    this(null, (ClientProjectFile) project.getTree(), filter, comparator);
    loadGrandChildren();

  }

  ProjectTreeNode(ProjectTreeNode parent, ClientProjectFile file,
    ProjectFileFilter filter, Comparator comparator) {
    this.parent = parent;
    this.file = file;
    this.filter = filter;
    this.comparator = comparator;
    this.childList = new LinkedList<ProjectTreeNode>();
  }

  void loadGrandChildren() {
    if (!file.isDirectory()) return;
    loadChildren();
    for (ProjectTreeNode child : childList) {
      child.loadChildren();
    }
  }

  
  void loadChildrenWithRecursion(boolean bringFromServer) {
    if (!file.isDirectory()) return;
    loadChildren(bringFromServer);
    for (ProjectTreeNode child : childList) {
      child.loadChildrenWithRecursion(bringFromServer);
    }
  }
  
  void loadChildren() {
    loadChildren(true);
  }
  
  void loadChildren(boolean bringFromServer) {
    if (!file.isDirectory()) return;
    this.childList.clear();

    final ClientProjectFile[] children;
    if (bringFromServer) {
      children = GetChildrenTask.runTask(file);
    }
    else {
      // No queremos buscar no servidor os ns da rvore que ainda no foram
      // carregados, mas apenas filtrar os que j esto no cliente.
      children = file.getLocalChildren();
    }
    NoHiddenFileFilter noDotFileFilter = NoHiddenFileFilter.getInstance();
    boolean showHiddenFiles =
      DesktopFrame.getInstance().shouldShowHiddenFiles();
    for (int i = 0; i < children.length; i++) {
      /*
       * se o nome do arquivo comea com ".", ele  ignorado
       */
      if (!showHiddenFiles && !noDotFileFilter.accept(children[i])) {
        continue;
      }
      if (this.filter.accept(children[i])) {
        ProjectTreeNode treeNode =
          new ProjectTreeNode(this, children[i], this.filter, this.comparator);
        this.childList.add(treeNode);
      }
    }
    Collections.sort(this.childList, this.comparator);
  }

  public ClientProjectFile getClientProjectFile() {
    return this.file;
  }

  public Enumeration children() {
    return new ChildEnumeration();
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == null) {
      return false;
    }
    if (!getClass().equals(obj.getClass())) {
      return false;
    }
    ProjectTreeNode node = (ProjectTreeNode) obj;
    return this.file.getAbsolutePath().equals(node.file.getAbsolutePath());
  }

  public boolean getAllowsChildren() {
    return this.file.isDirectory();
  }

  ProjectTreeNode getChild(ClientProjectFile file) {
    Iterator childListIterator = this.childList.iterator();
    while (childListIterator.hasNext()) {
      ProjectTreeNode treeNode = (ProjectTreeNode) childListIterator.next();
      if (treeNode.file.equals(file)) {
        return treeNode;
      }
    }
    return null;
  }

  ProjectTreeNode getChild(String fileName) {
    Iterator childListIterator = this.childList.iterator();
    while (childListIterator.hasNext()) {
      ProjectTreeNode treeNode = (ProjectTreeNode) childListIterator.next();
      if (treeNode.file.getName().equals(fileName)) {
        return treeNode;
      }
    }
    return null;
  }

  public TreeNode getChildAt(int childIndex) {
    return (TreeNode) this.childList.get(childIndex);
  }

  public int getChildCount() {
    return this.childList.size();
  }

  public TreeNode getParent() {
    return this.parent;
  }

  public int getIndex(TreeNode node) {
    return this.childList.indexOf(node);
  }

  @Override
  public int hashCode() {
    return this.file.hashCode();
  }

  public boolean isLeaf() {
    return (this.getChildCount() == 0);
  }

  @Override
  public String toString() {
    return this.file.toString();
  }

  private final class ChildEnumeration implements Enumeration {
    private Iterator iterator;

    ChildEnumeration() {
      this.iterator = ProjectTreeNode.this.childList.iterator();
    }

    public boolean hasMoreElements() {
      return this.iterator.hasNext();
    }

    public Object nextElement() {
      return ((ProjectTreeNode) this.iterator.next());
    }
  }

  ProjectFileFilter getFilter() {
    return this.filter;
  }

  int[] getIndexes(ProjectTreeNode[] treeNodeArray) {
    int[] indexes = new int[treeNodeArray.length];
    for (int i = 0; i < indexes.length; i++) {
      indexes[i] = this.getIndex(treeNodeArray[i]);
    }
    return indexes;
  }

  TreePath getTreePath() {
    ProjectTreeNode node = this;
    Stack<ProjectTreeNode> pathToRoot = new Stack<ProjectTreeNode>();
    do {
      pathToRoot.push(node);
      node = (ProjectTreeNode) node.getParent();
    } while (node != null);
    ProjectTreeNode[] pathToRootArray = new ProjectTreeNode[pathToRoot.size()];
    node = (ProjectTreeNode) pathToRoot.pop();
    int i = 0;
    pathToRootArray[i] = node;
    while (!pathToRoot.isEmpty()) {
      i++;
      node = (ProjectTreeNode) pathToRoot.pop();
      if (node == null) {
        throw new IllegalStateException(
          "csbase.client.project.ProjectTreeNode.getTreePath\nFalha ao obter o n de "
            + this.file);
      }
      pathToRootArray[i] = node;
    }
    return new TreePath(pathToRootArray);
  }

  /**
   * Redefine o arquivo associado ao n, e atualiza a lista de filhos.
   * 
   * @param file - novo arquivo a ser associado ao n
   */
  public void setClientProjectFile(ClientProjectFile file) {
    this.file = file;
    this.loadChildrenWithRecursion(true);
  }

  /**
   * Adiciona um novo n  rvore, se esta operao for compatvel com o filtro
   * de visualizao atual. Arquivos com nome comeando com "." so exibidos
   * apenas se o usurio optou por visualizar tais arquivos.
   * 
   * @param file - arquivo associado ao novo n
   * @return true se um novo n foi criado na rvore (i.e. se o arquivo era
   *         compatvel com os filtros)
   * @see DesktopPreferences#shouldShowHiddenFiles()
   */
  public boolean addChild(ClientProjectFile file) {
    boolean showHiddenFiles =
      DesktopFrame.getInstance().shouldShowHiddenFiles();
    NoHiddenFileFilter noDotFileFilter = NoHiddenFileFilter.getInstance();
    /*
     * se o nome do arquivo comea com ponto, ele  ignorado
     */
    if (!showHiddenFiles && !noDotFileFilter.accept(file)) {
      return false;
    }
    if (this.filter.accept(file)) {
      this.childList.add(new ProjectTreeNode(this, file, this.filter,
        this.comparator));
      Collections.sort(this.childList, this.comparator);
      return true;
    }
    return false;
  }

  public boolean removeChild(ClientProjectFile file) {
    ProjectTreeNode treeNode = this.getChild(file);
    if (treeNode != null) {
      if (this.childList.remove(treeNode)) {
        Collections.sort(this.childList, this.comparator);
        return true;
      }
    }
    return false;
  }
}
