package csbase.client.project;

import csbase.logic.ClientProjectFile;
import csbase.logic.ProjectFileFilter;
import csbase.logic.filters.ProjectFileNullFilter;
import csbase.util.collections.NoDuplicatesCollection;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.RowMapper;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

/**
 * Define um modelo de seleo de uma rvore de projetos.
 *
 * @author Rodrigo Carneiro Henrique (rodrigoh)
 */
public final class ProjectTreeSelectionModel implements TreeSelectionModel {
  /**
   * Os mtodos addSelectionPath(s), setSelectionPath(s) e
   * removeSelectionPath(s) tem comportamentos diferentes, de acordo com o
   * modo de seleo escolhido. A classe ProjectTreeSelectionModel delega a
   * execuo desse mtodos para o SelectionModeState correspondente ao seu
   * modo de seleo.
   *
   * @author Rodrigo Carneiro Henrique (rodrigoh)
   * @version $Revision: 35526 $
   */
  private interface SelectionModeState {
    /**
     * Adiciona os elementos recebidos  lista de paths selecionados.
     *
     * @param treePathCollection Uma coleo com elementos do tipo {@link
     *        TreePath} que sero adicionados  seleo.
     */
    void addSelectionPaths(NoDuplicatesCollection treePathCollection);

    /**
     * Atualiza a lista de paths selecionados para conter apenas os elementos
     * recebidos.
     *
     * @param treePathCollection Uma coleo com elementos do tipo {@link
     *        TreePath} que comporo  seleo.
     */
    void setSelectionPaths(NoDuplicatesCollection treePathCollection);

    /**
     * Remove os elementos recebidos da lista de paths selecionados.
     *
     * @param treePathCollection Uma coleo com elementos do tipo {@link
     *        TreePath} que sero removidos da seleo.
     */
    void removeSelectionPaths(NoDuplicatesCollection treePathCollection);
  }

  /**
   * Responsvel por gerenciar a lista de paths selecionados quando o modo de
   * seleo  SINGLE_TREE_SELECTION. Nesse modo apenas um item pode estar
   * selecionado.
   *
   * @author Rodrigo Carneiro Henrique (rodrigoh)
   */
  private final class SelectionModeStateSingle implements SelectionModeState {
    public void addSelectionPaths(NoDuplicatesCollection treePathCollection) {
      this.setSelectionPaths(treePathCollection);
    }

    public void setSelectionPaths(NoDuplicatesCollection treePathCollection) {
      if (treePathCollection.size() == 0) {
        return;
      }
      TreePath oldLeadSelectionPath =
        ProjectTreeSelectionModel.this.getLeadSelectionPath();
      ProjectTreeSelectionModel.this.selectedPaths.clear();
      ProjectTreeSelectionModel.this.selectedPaths.addAll(treePathCollection);
      ProjectTreeSelectionModel.this.resetRowSelection();
      TreePath newLeadSelectionPath =
        ProjectTreeSelectionModel.this.getLeadSelectionPath();
      TreePath[] paths;
      boolean[] areNew;
      if (oldLeadSelectionPath != null) {
        if (oldLeadSelectionPath.equals(newLeadSelectionPath)) {
          return;
        }
        else {
          paths = new TreePath[] { oldLeadSelectionPath, newLeadSelectionPath };
          areNew = new boolean[] { false, true };
        }
      }
      else {
        paths = new TreePath[] { newLeadSelectionPath };
        areNew = new boolean[] { true };
      }
      TreeSelectionEvent treeSelectionEvent =
        new TreeSelectionEvent(
          this, paths, areNew, oldLeadSelectionPath, newLeadSelectionPath);
      ProjectTreeSelectionModel.this.fireValueChanged(treeSelectionEvent);
    }

    public void removeSelectionPaths(NoDuplicatesCollection treePathCollection) {
      if (treePathCollection.size() == 0) {
        return;
      }
      TreePath oldSelectionPath =
        ProjectTreeSelectionModel.this.getLeadSelectionPath();
      TreePath[] oldSelectedPaths =
        ProjectTreeSelectionModel.getTreePathArrayFromList(
          ProjectTreeSelectionModel.this.selectedPaths);
      ProjectTreeSelectionModel.this.selectedPaths.removeAll(
        treePathCollection);
      ProjectTreeSelectionModel.this.resetRowSelection();
      boolean[] areNew = new boolean[oldSelectedPaths.length];
      for (int i = 0; i < areNew.length; i++) {
        areNew[i] = !treePathCollection.contains(oldSelectedPaths[i]);
      }
      TreeSelectionEvent treeSelectionEvent =
        new TreeSelectionEvent(
          this, oldSelectedPaths, areNew, oldSelectionPath,
          ProjectTreeSelectionModel.this.getLeadSelectionPath());
      ProjectTreeSelectionModel.this.fireValueChanged(treeSelectionEvent);
    }
  }

  /**
   * Responsvel por gerenciar a lista de paths selecionados quando o modo de
   * seleo  DISCONTIGUOUS_TREE_SELECTION. Nesse modo mltiplos itens podem
   * ser selecionados, independente de sua posio. Essa seleo normalmente 
   * feita utilizando-se a tecla CTRL.
   *
   * @author Rodrigo Carneiro Henrique (rodrigoh)
   */
  private final class SelectionModeStateDiscontiguous
  implements SelectionModeState {
    public void addSelectionPaths(NoDuplicatesCollection treePathCollection) {
      if (treePathCollection.size() == 0) {
        return;
      }
      TreePath oldLeadSelectionPath =
        ProjectTreeSelectionModel.this.getLeadSelectionPath();
      ProjectTreeSelectionModel.this.selectedPaths.addAll(treePathCollection);
      ProjectTreeSelectionModel.this.resetRowSelection();
      TreePath[] paths =
        ProjectTreeSelectionModel.getTreePathArrayFromList(
          ProjectTreeSelectionModel.this.selectedPaths);
      boolean[] areNew = new boolean[paths.length];
      Arrays.fill(areNew, true);
      TreeSelectionEvent treeSelectionEvent =
        new TreeSelectionEvent(
          this, paths, areNew, oldLeadSelectionPath,
          ProjectTreeSelectionModel.this.getLeadSelectionPath());
      ProjectTreeSelectionModel.this.fireValueChanged(treeSelectionEvent);
    }

    public void setSelectionPaths(NoDuplicatesCollection treePathCollection) {
      if (treePathCollection.size() == 0) {
        return;
      }
      TreePath oldLeadSelectionPath =
        ProjectTreeSelectionModel.this.getLeadSelectionPath();
      NoDuplicatesCollection allPaths =
        new NoDuplicatesCollection(
          ProjectTreeSelectionModel.this.selectedPaths);
      allPaths.addAll(treePathCollection);
      ProjectTreeSelectionModel.this.selectedPaths.clear();
      ProjectTreeSelectionModel.this.selectedPaths.addAll(treePathCollection);
      ProjectTreeSelectionModel.this.resetRowSelection();
      TreePath newLeadSelectionPath =
        ProjectTreeSelectionModel.this.getLeadSelectionPath();
      TreePath[] paths =
        ProjectTreeSelectionModel.this.getTreePathArrayFromList(allPaths);
      boolean[] areNew = new boolean[paths.length];
      for (int i = 0; i < areNew.length; i++) {
        areNew[i] = treePathCollection.contains(paths[i]);
      }
      TreeSelectionEvent treeSelectionEvent =
        new TreeSelectionEvent(
          this, paths, areNew, oldLeadSelectionPath,
          ProjectTreeSelectionModel.this.getLeadSelectionPath());
      ProjectTreeSelectionModel.this.fireValueChanged(treeSelectionEvent);
    }

    public void removeSelectionPaths(NoDuplicatesCollection treePathCollection) {
      if (treePathCollection.size() == 0) {
        return;
      }
      TreePath oldSelectionPath =
        ProjectTreeSelectionModel.this.getLeadSelectionPath();
      TreePath[] oldSelectedPaths =
        ProjectTreeSelectionModel.getTreePathArrayFromList(
          ProjectTreeSelectionModel.this.selectedPaths);
      ProjectTreeSelectionModel.this.selectedPaths.removeAll(
        treePathCollection);
      ProjectTreeSelectionModel.this.resetRowSelection();
      boolean[] areNew = new boolean[oldSelectedPaths.length];
      for (int i = 0; i < areNew.length; i++) {
        areNew[i] = !treePathCollection.contains(oldSelectedPaths[i]);
      }
      TreeSelectionEvent treeSelectionEvent =
        new TreeSelectionEvent(
          this, oldSelectedPaths, areNew, oldSelectionPath,
          ProjectTreeSelectionModel.this.getLeadSelectionPath());
      ProjectTreeSelectionModel.this.fireValueChanged(treeSelectionEvent);
    }
  }

  /**
   * Filtro usado para avaliar se determinado arquivo de projeto pode ser
   * selecionado.
   */
  private ProjectFileFilter filter;

  /** Coleo de ouvintes de eventos de seleo do modelo. */
  private Set treeSelectionListenerSet;

  /** Coleo de paths que esto atualmente selecionados. */
  private NoDuplicatesCollection selectedPaths;

  /**
   * Corresponde  linha de cada path que est atualmente selecionado. {@link
   * RowMapper#getRowsForPaths(javax.swing.tree.TreePath[])}
   */
  private int[] selectedRows;
  private RowMapper rowMapper;

  /**  o responsvel por gerenciar a lista de paths selecionados. */
  private SelectionModeState selectionModeState;
  private final SelectionModeStateSingle SINGLE_TREE_SELECTION_STATE =
    new SelectionModeStateSingle();
  private final SelectionModeStateDiscontiguous DISCONTIGUOUS_TREE_SELECTION_STATE =
    new SelectionModeStateDiscontiguous();

  /** O modo de seleo do modelo. */
  private int selectionMode;

  /**
   * Coleo de ouvintes de eventos de mudana em propriedades do modelo. Esses
   * ouvintes esto interessados na propriedade {@link #selectionMode}.
   */
  private Set propertyChangeListenerSet;
  private static final String SELECTION_MODE_PROPERTY = "selectionMode";

  /**
   * Cria um ProjectTreeSelectionModel que aceitar a seleo de qualquer tipo
   * de arquivo de projeto.
   */
  public ProjectTreeSelectionModel() {
    this(ProjectFileNullFilter.getInstance());
  }

  /**
   * Cria um ProjectTreeSelectionModel que permitir a seleo apenas de
   * arquivos aceitos pelo filtro recebido.
   *
   * @param filter O filtro de seleo.
   */
  public ProjectTreeSelectionModel(ProjectFileFilter filter) {
    this.treeSelectionListenerSet = new HashSet();
    this.propertyChangeListenerSet = new HashSet();
    this.selectedPaths = new NoDuplicatesCollection();
    this.setSelectionMode(DISCONTIGUOUS_TREE_SELECTION);
    this.setFilter(filter);
  }

  ProjectFileFilter getFilter() {
    return this.filter;
  }

  /**
   * Atualiza o filtro de seleo de arquivos de projeto.
   *
   * @param filter O novo filtro de seleo.
   */
  void setFilter(ProjectFileFilter filter) {
    this.filter = filter;
    this.updateSelection();
  }

  private void updateSelection() {
    NoDuplicatesCollection newSelection =
      this.wipePaths(getTreePathArrayFromList(this.selectedPaths));
    if (newSelection.size() == 0) {
      this.selectionModeState.removeSelectionPaths(
        (NoDuplicatesCollection)this.selectedPaths.clone());
    }
    else {
      this.selectionModeState.setSelectionPaths(newSelection);
    }
    this.resetRowSelection();
  }

  /**
   * Obtm o ndice do ltimo path adicionado na lista.
   *
   * @return O ndice do ltimo path adicionado na lista.
   */
  private int getLeadSelectionIndex() {
    return (this.selectedPaths.size()-1);
  }

  public int getLeadSelectionRow() {
    if (this.rowMapper == null) {
      return -1;
    }
    return this.selectedRows[this.getLeadSelectionIndex()];
  }

  public int getMaxSelectionRow() {
    if (this.rowMapper == null) {
      return -1;
    }
    if (this.selectedRows.length == 0) {
      return -1;
    }
    int max = this.selectedRows[0];
    for (int i = 1; i < this.selectedRows.length; i++) {
      if (this.selectedRows[i] > max) {
        max = this.selectedRows[i];
      }
    }
    return max;
  }

  public int getMinSelectionRow() {
    if (this.rowMapper == null) {
      return -1;
    }
    if (this.selectedRows.length == 0) {
      return -1;
    }
    int min = this.selectedRows[0];
    for (int i = 1; i < this.selectedRows.length; i++) {
      if (this.selectedRows[i] < min) {
        min = this.selectedRows[i];
      }
    }
    return min;
  }

  public int getSelectionCount() {
    return this.selectedPaths.size();
  }

  public int getSelectionMode() {
    return this.selectionMode;
  }

  public void clearSelection() {
    TreePath[] treePathsSelected =
      this.getTreePathArrayFromList(this.selectedPaths);
    if (treePathsSelected.length == 0) {
      return;
    }
    boolean[] areNew = new boolean[treePathsSelected.length];
    for (int i = 0; i < areNew.length; i++) {
      areNew[i] = false;
    }
    TreeSelectionEvent treeSelectionEvent =
      new TreeSelectionEvent(
        this, treePathsSelected, areNew, this.getLeadSelectionPath(), null);
    this.fireValueChanged(treeSelectionEvent);
    this.selectedPaths.clear();
  }

  /**
   * Envia o evento de mudana na seleo para todos os ouvintes cadastrados.
   *
   * @param treeSelectionEvent O evento que ser enviado aos ouvintes.
   */
  private void fireValueChanged(TreeSelectionEvent treeSelectionEvent) {
    Iterator treeListenersIterator = this.treeSelectionListenerSet.iterator();
    while (treeListenersIterator.hasNext()) {
      TreeSelectionListener treeSelectionListener =
        (TreeSelectionListener)treeListenersIterator.next();
      treeSelectionListener.valueChanged(treeSelectionEvent);
    }
  }

  public void resetRowSelection() {
    if (this.rowMapper != null) {
      this.selectedRows =
        this.rowMapper.getRowsForPaths(
          this.getTreePathArrayFromList(this.selectedPaths));
    }
    else {
      this.selectedRows = null;
    }
  }

  public boolean isSelectionEmpty() {
    return this.selectedPaths.isEmpty();
  }

  public int[] getSelectionRows() {
    return this.selectedRows;
  }

  public void setSelectionMode(int mode) {
    if (mode == this.selectionMode) {
      return;
    }
    switch (mode) {
      case TreeSelectionModel.SINGLE_TREE_SELECTION:
        this.selectionModeState = SINGLE_TREE_SELECTION_STATE;
        break;
      case TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION:
        this.selectionModeState = DISCONTIGUOUS_TREE_SELECTION_STATE;
        break;
      default:
        throw new IllegalArgumentException(
          "Modo de seleo invlido. Use: "+
          TreeSelectionModel.SINGLE_TREE_SELECTION+" ou "+
          TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION+".");
    }
    int oldMode = this.selectionMode;
    this.selectionMode = mode;
    PropertyChangeEvent event =
      new PropertyChangeEvent(
        this, SELECTION_MODE_PROPERTY, new Integer(oldMode), new Integer(mode));
    this.firePropertyChange(event);
    this.updateSelection();
  }

  /**
   * Envia o evento de mudana no valor da propriedade de nome {@link
   * #SELECTION_MODE_PROPERTY} para todos os ouvintes cadastrados.
   *
   * @param event O evento que ser enviado aos ouvintes.
   */
  private void firePropertyChange(PropertyChangeEvent event) {
    Iterator iterator = this.propertyChangeListenerSet.iterator();
    while (iterator.hasNext()) {
      PropertyChangeListener listener = (PropertyChangeListener)iterator.next();
      listener.propertyChange(event);
    }
  }

  public boolean isRowSelected(int row) {
    if (this.selectedRows == null) {
      return false;
    }
    for (int i = 0; i < this.selectedRows.length; i++) {
      if (this.selectedRows[i] == row) {
        return true;
      }
    }
    return false;
  }

  public void addPropertyChangeListener(PropertyChangeListener listener) {
    this.propertyChangeListenerSet.add(listener);
  }

  public void removePropertyChangeListener(PropertyChangeListener listener) {
    this.propertyChangeListenerSet.remove(listener);
  }

  public void addTreeSelectionListener(TreeSelectionListener listener) {
    this.treeSelectionListenerSet.add(listener);
  }

  public void removeTreeSelectionListener(TreeSelectionListener listener) {
    this.treeSelectionListenerSet.remove(listener);
  }

  public RowMapper getRowMapper() {
    return this.rowMapper;
  }

  public void setRowMapper(RowMapper newMapper) {
    this.rowMapper = newMapper;
    this.resetRowSelection();
  }

  public TreePath getLeadSelectionPath() {
    if (this.selectedPaths.size() == 0) {
      return null;
    }
    return (TreePath)this.selectedPaths.getLast();
  }

  public TreePath getSelectionPath() {
    if (this.selectedPaths.size() == 0) {
      return null;
    }
    return (TreePath)this.selectedPaths.getFirst();
  }

  public TreePath[] getSelectionPaths() {
    return this.getTreePathArrayFromList(this.selectedPaths);
  }

  public void addSelectionPath(TreePath path) {
    if (path == null) {
      return;
    }
    TreePath[] paths = new TreePath[] { path };
    this.addSelectionPaths(paths);
  }

  public void removeSelectionPath(TreePath path) {
    if (path == null) {
      return;
    }
    TreePath[] paths = new TreePath[] { path };
    this.removeSelectionPaths(paths);
  }

  public void setSelectionPath(TreePath path) {
    if (path == null) {
      return;
    }
    TreePath[] paths = new TreePath[] { path };
    this.setSelectionPaths(paths);
  }

  public boolean isPathSelected(TreePath path) {
    return this.selectedPaths.contains(path);
  }

  public void addSelectionPaths(TreePath[] paths) {
    this.selectionModeState.addSelectionPaths(this.wipePaths(paths));
  }

  public void removeSelectionPaths(TreePath[] paths) {
    this.selectionModeState.removeSelectionPaths(this.wipePaths(paths));
  }

  public void setSelectionPaths(TreePath[] paths) {
    this.selectionModeState.setSelectionPaths(this.wipePaths(paths));
  }

  /**
   * Filtra os paths recebidos para que sejam selecionados apenas arquivos
   * aceitos pelo filtro do modelo. Alm disso, remove elementos duplicados.
   *
   * @param paths Os paths a serem enxugados. Se paths for null, retorna uma
   *        lista vazia.
   *
   * @return Uma lista com os elementos aceitos pelo filtro, sem elementos
   *         duplicados. Obs: Nunca retorna null, apenas lista vazia.
   */
  private NoDuplicatesCollection wipePaths(TreePath[] paths) {
    NoDuplicatesCollection filteredPathsList = new NoDuplicatesCollection();
    if (paths != null) {
      for (int i = 0; i < paths.length; i++) {
        if (paths[i] != null) {
          ProjectTreeNode treeNode =
            (ProjectTreeNode)paths[i].getLastPathComponent();
          ClientProjectFile projectFile = treeNode.getClientProjectFile();
          if (this.filter.accept(projectFile)) {
            filteredPathsList.add(paths[i]);
          }
        }
      }
    }
    return filteredPathsList;
  }

  /**
   * Criar um array de paths a partir de uma lista de elementos no-duplicados.
   *
   * @param list A lista com os paths que comporo o array gerado.
   *
   * @return O array de elementos no-duplicados.
   */
  private static TreePath[] getTreePathArrayFromList(
    NoDuplicatesCollection list) {
    Iterator iterator = list.iterator();
    TreePath[] treePath = new TreePath[list.size()];
    for (int i = 0; i < treePath.length; i++) {
      treePath[i] = (TreePath)iterator.next();
    }
    return treePath;
  }

  /**
   * Cria uma lista de elementos no-duplicados a partir de um array de paths.
   *
   * @param paths Os paths que comporo a lista gerada.
   *
   * @return A lista de elementos no-duplicados.
   */
  private static NoDuplicatesCollection getListFromTreePathArray(
    TreePath[] paths) {
    NoDuplicatesCollection list = new NoDuplicatesCollection();
    for (int i = 0; i < paths.length; i++) {
      list.add(paths[i]);
    }
    return list;
  }
}
