/*
 * PublicationDialog.java
 *
 * $Author: taciana $
 * $Revision: 72732 $ - $Date: 2008-02-15 10:15:26 -0200 (Fri, 15 Feb 2008) $
 */
package csbase.client.project;

import java.awt.BorderLayout;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;
import java.util.Vector;

import javax.swing.*;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.StandardDialogs;
import tecgraf.javautils.gui.selector.ContainerSelectionListener;
import csbase.client.desktop.DesktopComponentDialog;
import csbase.client.desktop.RemoteTask;
import csbase.client.util.UserContainerSelection;
import csbase.logic.*;

/**
 * A classe <code>PublicationDialog</code> modela um dilogo para  publicao
 * de arquivos de um projeto.  Os arquivos podem ser publicados para um
 * conjunto de usurios e/ou grupos de usurios.
 */
public class PublicationDialog {
  /** Ttulo do dilogo. */
  private final String frameTitle =
    LNG.get("PRJ_PROJECT_FILE_PUBLICATION_TITLE") + " - " + LNG.get("SERVER");
  /** Dilogo principal. */
  private DesktopComponentDialog dialog = null;
  /** Janela a partir do qual esse dilogo foi chamado. */
  private Window owner = null;
  /** Componente de seleo dos usurios. */
  private UserContainerSelection selectableItems = null;
  /** O boto que implementa a ao default: publicao ou despublicao */
  private JButton defaultButton;
  /** O arquivo a ser publicado. */
  private final ProjectFile[] files;

  /**
   * Mostra o dilogo para publicao de um arquivo. Esse dilogo j implementa
   * a ao de publicao, no caso do usurio confirmar a operao.
   *
   * @param owner A janela a partir do qual esse dilogo foi chamado.
   * @param file O arquivo a ser publicado.
   */
  public static void showToPublish(Window owner, ProjectFile file) {
    new PublicationDialog(owner, file).showDialogToPublish();
  }

  /**
   * Mostra o dilogo para despublicao de um arquivo. Esse dilogo j
   * implementa a ao de despublicao, no caso do usurio confirmar a
   * operao.
   *
   * @param owner A janela a partir do qual esse dilogo foi chamado.
   * @param file O arquivo a ser despublicado.
   */
  public static void showToUnPublish(Window owner, ProjectFile file) {
    new PublicationDialog(owner, file).showDialogToUnPublish();
  }

  public static void showToPublish(Window owner, ProjectFile[] files) {
    new PublicationDialog(owner, files).showDialogToPublish();
  }

  public static void showToUnPublish(Window owner, ProjectFile[] files) {
    new PublicationDialog(owner, files).showDialogToUnPublish();
  }

  /**
   * Cria o dilogo, composto de:
   *
   * <ul>
   * <li>
   * um componente que permite selecionar os usurios e grupos para os quais os
   * arquivos vo ser publicados;
   * </li>
   * <li>
   * um boto que confima a publicao, chamando o servidor para efetuar a
   * operao.
   * </li>
   * <li>
   * um boto que permite desistir da operao.
   * </li>
   * </ul>
   *
   *
   * @param enabled Indica se o container deve ser habilitado ou no.
   *
   * @return .
   */
  private DesktopComponentDialog makeDialog(boolean enabled) {
    JPanel panel = new JPanel(new BorderLayout());
    panel.add(makeSelectablePanel(enabled));
    panel.add(makeButtonPanel(), BorderLayout.SOUTH);
    /* Cria um dilogo modal */
    DesktopComponentDialog dialog;
    if (owner instanceof JDialog) {
      dialog = new DesktopComponentDialog((JDialog)owner, frameTitle);
    }
    else {
      dialog = new DesktopComponentDialog((JFrame)owner, frameTitle);
    }
    dialog.getContentPane().add(panel);
    dialog.center(owner);
    dialog.pack();
    dialog.getRootPane().setDefaultButton(defaultButton);
    return dialog;
  }

  /**
   * Cria o painel com o componente que permite selecionar os usurios e grupos
   * para os quais os arquivos vo ser publicados. Usa um componente que
   * mostra dois conjuntos, disponveis e  escolhidos, nos quais o usurio
   * pode movimentar elementos  entre eles.
   *
   * @param enabled Indica se o container deve ser habilitado ou no.
   *
   * @return .
   */
  private JPanel makeSelectablePanel(boolean enabled) {
    /* Monta a lista dos usurios disponveis e selecionados. */
    Vector<UserOutline> selectedUsers = retrieveAccessUsers();
    Vector<UserOutline> availableUsers = retrieveAllUsers();
    /**
     * Cria o componente que mostra os dois conjuntos de elementos: um com os
     * disponveis e o outro com os escolhidos.
     */
    selectableItems =
      new UserContainerSelection(enabled, availableUsers, selectedUsers);
    return selectableItems.getPanel();
  }

  /**
   * Cria o painel com os botes de confirmao e desistncia da operao de
   * publicao.
   *
   * @return .
   */
  private JPanel makeButtonPanel() {
    JPanel panel = new JPanel();
    panel.add(defaultButton);
    JButton cancelButton =
      (JButton)panel.add(new JButton(LNG.get("PRJ_CANCEL")));
    cancelButton.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          closeDialog();
        }
      });
    return panel;
  }

  /**
   * Cria o boto de publicao.
   *
   * @return .
   */
  private JButton makePublishButton() {
    JButton publishButton = new JButton(LNG.get("PRJ_PUBLISH"));
    publishButton.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          handlePublishAction();
        }
      });
    return publishButton;
  }

  /**
   * Cria o boto de despublicao.
   *
   * @return .
   */
  private JButton makeUnPublishButton() {
    JButton unpublishButton = new JButton(LNG.get("PRJ_UNPUBLISH"));
    unpublishButton.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          handleUnpublishAction();
        }
      });
    return unpublishButton;
  }

  /**
   * Exibe o dilogo de publicao.
   */
  private void showDialogToPublish() {
    /*
     * Cria os componentes do dilogo.
     */
    defaultButton = makePublishButton();
    dialog = makeDialog(true);
    defaultButton.setEnabled(false);
    /*
     * Habilita ou no o boto de publicao de acordo com o
     * nmero de elementos selecionados no container.
     */
    selectableItems.addContainerSelectionListener(
      new ContainerSelectionListener() {
        public void containerChanged() {
          if (selectableItems.getSelectedItems().length == 0) {
            defaultButton.setEnabled(false);
          }
          else {
            defaultButton.setEnabled(true);
          }
        }
      });
    dialog.pack();
    dialog.center(owner);
    dialog.setVisible(true);
  }

  /**
   * Exibe o dilogo de despublicao.
   */
  private void showDialogToUnPublish() {
    /*
     * Cria os componentes do dilogo.
     */
    defaultButton = makeUnPublishButton();
    dialog = makeDialog(true);
    /*
     * Habilita ou no o boto de atualizao se o container mudar.
     */
    defaultButton.setEnabled(false);
    selectableItems.addContainerSelectionListener(
      new ContainerSelectionListener() {
        public void containerChanged() {
          defaultButton.setEnabled(true);
        }
      });
    dialog.pack();
    dialog.center(owner);
    dialog.setVisible(true);
  }

  /**
   * Fecha o dilogo.
   */
  private void closeDialog() {
    dialog.close();
  }

  /**
   * Trata a ao de publicao de um arquivo. Esse mtodo  chamado pela
   * callback do boto de publicao.
   */
  private void handlePublishAction() {
    /* Verifica com o usurio se deve prosseguir com a tarefa de publicao. */
    if (!confirmPublication()) {
      return;
    }

    /* Recupera os identificadores dos usurios que teram acesso aos arquivos publicados. */
    final Object[] userIds = selectableItems.getSelectedItems();
    /* Recupera o identificador do usurio que est publicando os arquivos. */
    User publisher = User.getLoggedUser();
    final Object publisherId = (null == publisher) ? null : publisher.getId();

    /* Prepara a tarefa de publicao */
    RemoteTask<Boolean> task =
      new RemoteTask<Boolean>() {
        @Override
        public void performTask() throws Exception {
          boolean publishedAll = true;
          for (int i = 0; i < files.length; i++) {
            if (!files[i].publish(userIds, publisherId)) {
              publishedAll = false;
            }
          }
          setResult(publishedAll);
        }
      };
    /* Executa a tarefa de publicao no servidor */
    if (task.execute(
      owner, frameTitle, LNG.get("PRJ_WAITING_PUBLISH_FILE_MULTIPLE_FILES")) &&
      task.getResult()){
      closeDialog();
    }
  }

  /**
   * Gera um dilogo pedindo a confirmao ao usurio se deve publicar o arquivo
   * selecionado.
   * @return true se deve seguir a diante com a publicao, false caso o contrrio.
   */
  private boolean confirmPublication() {
    return StandardDialogs.showYesNoDialog(
        owner, frameTitle,
        LNG.get("PRJ_PROJECT_FILE_PUBLISH_CONFIRMATION_MULTIPLE_FILES")) == 0;
  }

  /**
   * Trata a ao de despublicao de um arquivo. Esse mtodo  chamado pela
   * callback do boto de despublicao.
   */
  private void handleUnpublishAction() {
    /* Recupera os ids dos usurios selecionados */
    final Object[] userIds = selectableItems.getSelectedItems();
    Object userId = null;
    User user = User.getLoggedUser();
    if (user != null) {
      userId = user.getId();
    }
    final Object _userId = userId;
    /* Prepara a tarefa de despublicao */
    RemoteTask<Boolean> task = new RemoteTask<Boolean>() {
        @Override
        public void performTask() throws Exception {
          boolean unPublishedAll = true;
          for (int i = 0; i < files.length; i++) {
            if (!files[i].updatePublishAccess(userIds, _userId)) {
              unPublishedAll = false;
            }
          }
          setResult(new Boolean(unPublishedAll));
        }
      };
    /* Executa a tarefa de publicao no servidor */
    boolean success = task.execute(
      owner, frameTitle, LNG.get("PRJ_WAITING_UNPUBLISH_FILE_MULTIPLE_FILES"))
      && task.getResult();
    if (success) {
      /* Avisa que a operao foi feita com sucesso */
      StandardDialogs.showInfoDialog(
        owner, frameTitle,
        LNG.get("PRJ_PROJECT_FILE_UNPUBLISH_CONFIRMATION_MULTIPLE_FILES"));
      /* Fecha o dilogo */
      closeDialog();
    }
  }

  /**
   * Recupera no servidor os usurios disponveis.
   *
   * @return O vetor com todos os usurios disponveis.
   */
  private Vector<UserOutline> retrieveAllUsers() {
    RemoteTask<List<UserOutline>> task =
      new RemoteTask<List<UserOutline>>() {
        @Override
        public void performTask() throws Exception {
          setResult(User.getAllOutlines());
        }
      };
    task.execute(owner, frameTitle, LNG.get("PRJ_WAITING_OBTAIN_USERS"));
    Vector<UserOutline> users = new Vector<UserOutline>();
    users.addAll(task.getResult());
    return users;
  }

  /**
   * Recupera no servidor os usurios para os quais o arquivo foi publicado. No
   * caso do arquivo no estar publicado, essa lista  vazia.
   *
   * @return O vetor com os usurios para os quais o arquivo est publicado.
   */
  private Vector<UserOutline> retrieveAccessUsers() {
    RemoteTask<Vector<UserOutline>> task =
      new RemoteTask<Vector<UserOutline>>() {
        @Override
        public void performTask() throws Exception {
          Vector<UserOutline> outlines = new Vector<UserOutline>();
          setResult(outlines);
          Object[] userIds = files[0].getPublishAccess();
          if (userIds == null) {
            return;
          }
          int size = userIds.length;
          for (int i = 0; i < size; i++) {
            outlines.add(User.getUser(userIds[i]).getOutline());
          }
        }
      };
    task.execute(
      owner, frameTitle,
      LNG.get("PRJ_WAITING_OBTAIN_ACCESS_USERS_MULTIPLE_FILES"));
    return task.getResult();
  }

  /**
   * Constri o dilogo de publicao.
   *
   * @param owner A janela pai a partir da qual esse dilogo  chamado.
   * @param file O arquivo a ser publicado.
   */
  private PublicationDialog(Window owner, ProjectFile file) {
    this.owner = owner;
    this.files = new ProjectFile[1];
    files[0] = file;
  }

  private PublicationDialog(Window owner, ProjectFile[] files) {
    this.owner = owner;
    this.files = files;
  }
}
