/*
 * AlgorithmManagementProxy.java
 * 
 * @author Ana $Revision: 98964 $ - $Date: 2009-07-10 14:25:16 -0300 (Fri, 10
 * Jul 2009) $
 */
package csbase.client.remote.srvproxies;

import java.awt.Window;
import java.util.Hashtable;
import java.util.List;
import java.util.SortedSet;

import javax.swing.JFrame;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.desktop.DesktopFrame;
import csbase.client.desktop.RemoteTask;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.FileInfo;
import csbase.logic.IPathFactory;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmOutline;
import csbase.logic.algorithms.AlgorithmProperty;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.logic.algorithms.Category;
import csbase.logic.algorithms.HistoryRecord;
import csbase.remote.AlgorithmServiceInterface;
import csbase.remote.ClientRemoteLocator;

/**
 * Classe que modela um proxy no cliente para o para acesso s operaes de
 * gerncia do repositrio de algoritmos (AlgoService).
 * 
 * @author Ana
 */
public class AlgorithmManagementProxy {
  /** Ttulo para janelas de tarefas. */
  private static String taskTitle = LNG.get("algomanager.title");

  /**
   * Copia um ou mais arquivos de uma pasta para outra com a possibilidade dar
   * permisso de execuo s cpias.
   * 
   * @param algoIdFrom identificador do algoritmo de origem.
   * @param versionIdFrom identificador da verso de origem.
   * @param fromFilePaths caminhos para os arquivos de origem.
   * @param algoIdTo identificador do algoritmo de destino.
   * @param versionIdTo identificador da verso de destino.
   * @param toFilePaths caminhos para os arquivos de destino.
   * @param owner janela pai para efeito de hierarquia de dilogos.
   * @param setExecutables se o valor for true, as cpias tero permisso de
   *        execuo.
   */
  public static void copyFile(final Object algoIdFrom,
    final Object versionIdFrom, final String[][] fromFilePaths,
    final Object algoIdTo, final Object versionIdTo,
    final String[][] toFilePaths, final boolean setExecutables, Window owner) {
    RemoteTask<Void> task = new RemoteTask<Void>() {
      @Override
      public void performTask() throws Exception {
        AlgorithmServiceInterface algoService = ClientRemoteLocator.algorithmService;
        algoService.copyFile(algoIdFrom, versionIdFrom, fromFilePaths,
          algoIdTo, versionIdTo, toFilePaths, setExecutables);
      }
    };
    task.execute(owner, taskTitle, LNG.get("algomanager.msg.wait"));
  }

  /**
   * Copia um ou mais arquivos de uma pasta para outra.
   * 
   * @param algoIdFrom identificador do algoritmo de origem.
   * @param versionIdFrom identificador da verso de origem.
   * @param fromFilePaths caminhos para os arquivos de origem.
   * @param algoIdTo identificador do algoritmo de destino.
   * @param versionIdTo identificador da verso de destino.
   * @param toFilePaths caminhos para os arquivos de destino.
   * @param owner janela pai para efeito de hierarquia de dilogos.
   */
  public static void copyFile(final Object algoIdFrom,
    final Object versionIdFrom, final String[][] fromFilePaths,
    final Object algoIdTo, final Object versionIdTo,
    final String[][] toFilePaths, Window owner) {
    RemoteTask<Void> task = new RemoteTask<Void>() {
      @Override
      public void performTask() throws Exception {
        AlgorithmServiceInterface algoService = ClientRemoteLocator.algorithmService;
        algoService.copyFile(algoIdFrom, versionIdFrom, fromFilePaths,
          algoIdTo, versionIdTo, toFilePaths);
      }
    };
    task.execute(owner, taskTitle, LNG.get("algomanager.msg.wait"));
  }

  /**
   * <p>
   * Copia arquivos de uma pasta para a outra com a possibilidade de dar
   * permisso de execuo s cpias.
   * </p>
   * <p>
   * Para definir o caminho do arquivo de destino, ser construido um FileInfo
   * usando o targetDir como path se este for diferente de <tt>null</tt> e
   * mantendo o nome do arquivo original. Este novo FileInfo ser passado como
   * parmetro para o {@link IPathFactory#getPath(FileInfo)
   * targetPathFactory.getPath(FileInfo)} que retornar o caminho fsico do novo
   * arquivo.
   * </p>
   * <p>
   * Ao final da execuo os algoritmos sero recarregados.
   * </p>
   * 
   * @param sourceVersion Informaes da verso de origem.
   * @param files Arquivos a serem transferidos.
   * @param sourcePathFactory Fbrica para criar o caminho dos arquivos a serem
   *        transferidos.
   * @param targetVersion Informaes da verso de destino.
   * @param targetDir diretrio de destino.
   * @param targetPathFactory Fbrica para criar o caminho dos arquivos de
   *        destino. Este caminho  relativo ao valor do parmetro targetDir.
   * @param setExecutables <tt>true</tt> se as cpias devero ter permisso de
   *        execuo ou no.
   * @param owner janela que requisitou esta operao.
   */
  public static void copyFiles(final AlgorithmVersionInfo sourceVersion,
    final List<FileInfo> files, final IPathFactory sourcePathFactory,
    final AlgorithmVersionInfo targetVersion, final FileInfo targetDir,
    final IPathFactory targetPathFactory, final boolean setExecutables,
    final Window owner) {

    RemoteTask<Void> task = new RemoteTask<Void>() {
      @Override
      public void performTask() throws Exception {
        AlgorithmServiceInterface algoService = ClientRemoteLocator.algorithmService;
        algoService.copyFiles(sourceVersion, files, sourcePathFactory,
          targetVersion, targetDir, targetPathFactory, setExecutables);
      }
    };
    task.execute(owner, taskTitle, LNG.get("algomanager.msg.wait"));
  }

  /**
   * Cria um novo algoritmo.
   * 
   * @param name nome do novo algoritmo
   * @param fieldValues propriedades do algoritmo
   * @param owner janela pai para efeito de hierarquia de dilogos.
   * 
   * @return informaes do novo algoritmo
   */
  public static AlgorithmInfo createAlgorithm(final String name,
    final Hashtable<String, String> fieldValues, Window owner) {
    RemoteTask<AlgorithmInfo> task = new RemoteTask<AlgorithmInfo>() {
      @Override
      public void performTask() throws Exception {
        setResult(ClientRemoteLocator.algorithmService.createAlgorithm(name,
          fieldValues));
      }
    };
    task.execute(owner, taskTitle, LNG.get("algomanager.msg.wait"));
    return task.getResult();
  }

  /**
   * Cria uma nova verso de algoritmo.
   * 
   * @param algoId identificador do algoritmo
   * @param major numero da verso
   * @param minor numero da reviso
   * @param patch numero da correo
   * @param properties propriedades da verso
   * @param owner janela pai para efeito de hierarquia de dilogos.
   * 
   * @return informaes atualizadas do algoritmo
   */
  public static AlgorithmInfo createVersion(final Object algoId,
    final int major, final int minor, final int patch,
    final Hashtable<String, String> properties, Window owner) {
    RemoteTask<AlgorithmInfo> task = new RemoteTask<AlgorithmInfo>() {
      @Override
      public void performTask() throws Exception {
        setResult(ClientRemoteLocator.algorithmService.createVersion(algoId,
          major, minor, patch, properties));
      }
    };
    task.execute(owner, taskTitle, LNG.get("algomanager.msg.wait"));
    return task.getResult();
  }

  /**
   * Cria uma categoria no servidor.
   * 
   * @param parentCategoryId identificador da categoria pai
   * @param id identificador da categoria a ser criada
   * @param name nome da categoria a ser criada
   * @param owner janela pai
   * 
   * @return a categoria criada
   * 
   */
  public static Category createCategory(final String parentCategoryId,
    final String id, final String name, Window owner) {

    RemoteTask<Category> task = new RemoteTask<Category>() {
      @Override
      protected void performTask() throws Exception {
        if (ClientRemoteLocator.algorithmService != null) {
          Category category = ClientRemoteLocator.algorithmService
            .createCategory(parentCategoryId, id, name);
          setResult(category);
        }
      }
    };
    task.execute(owner, taskTitle, LNG.get("algomanager.msg.wait"));
    return task.getResult();
  }

  /**
   * Cria uma categoria no servidor.
   * 
   * @param id identificador da categoria a ser removida
   * @param owner janela pai
   * 
   * @return retorna true se a categoria foi removida com sucesso, caso
   *         contrrio, retorna false
   * 
   */
  public static boolean removeCategory(final String id, Window owner) {

    RemoteTask<Void> task = new RemoteTask<Void>() {
      @Override
      protected void performTask() throws Exception {
        if (ClientRemoteLocator.algorithmService != null) {
          ClientRemoteLocator.algorithmService.removeCategory(id);
        }
      }
    };
    return task.execute(owner, taskTitle, LNG.get("algomanager.msg.wait"));
  }

  /**
   * Associa algoritmos s categorias.
   * 
   * @param algorithmIds identificadores dos algoritmos
   * @param categoryIds identificadores das categorias
   * @param owner janela pai
   * 
   * @return retorna true se os algoritmos foram associados s categorias com
   *         sucesso, caso contrrio, retorna false
   */
  public static boolean bindAlgorithmsAndCategories(
    final List<Object> algorithmIds, final List<String> categoryIds,
    Window owner) {

    RemoteTask<Void> task = new RemoteTask<Void>() {
      @Override
      protected void performTask() throws Exception {
        if (ClientRemoteLocator.algorithmService != null) {
          ClientRemoteLocator.algorithmService.bindAlgorithmsToCategories(
            algorithmIds, categoryIds);
        }
      }
    };
    return task.execute(owner, taskTitle, LNG.get("algomanager.msg.wait"));
  }

  /**
   * Duplica uma verso de um algoritmo.
   * 
   * @param algoId identificador do algoritmo cuja verso ser duplicada.
   * @param versionId identificador da verso a ser duplicada.
   * @param major nmero da verso.
   * @param minor nmero da reviso.
   * @param patch nmero da correo.
   * @param owner janela pai para efeito de hierarquia de dilogos.
   */
  public static void duplicateVersion(final Object algoId,
    final Object versionId, final int major, final int minor, final int patch,
    Window owner) {
    RemoteTask<Void> task = new RemoteTask<Void>() {
      @Override
      public void performTask() throws Exception {
        AlgorithmServiceInterface algoService = ClientRemoteLocator.algorithmService;
        algoService.duplicateVersion(algoId, versionId, major, minor, patch);
      }
    };
    task.execute(owner, taskTitle, LNG.get("algomanager.msg.wait"));
  }

  /**
   * Obtm todas as categorias instaladas no sistema.
   * 
   * @param owner janela pai para efeito de hierarquia de dilogos.
   * 
   * @return As categorias (caso no haja categorias, retornar um conjunto
   *         vazio) ou {@code null} em caso de erro.
   */
  public static SortedSet<Category> retrieveAllCategories(JFrame owner) {
    RemoteTask<SortedSet<Category>> task = new RemoteTask<SortedSet<Category>>() {
      @Override
      public void performTask() throws Exception {
        setResult(ClientRemoteLocator.algorithmService.getAllCategories());
      }
    };
    task.execute(owner, LNG.get("algomanager.title.consulting_data"), LNG
      .get("algomanager.msg.consulting_data"));
    return task.getResult();
  }

  /**
   * Obtem a lista de outlines de algoritmos cadastrados.
   * 
   * @param owner janela pai para efeito de hierarquia de dilogos.
   * 
   * @return o array de outlines de algoritmos
   */
  public static AlgorithmOutline[] getAllOutlines(Window owner) {
    RemoteTask<AlgorithmOutline[]> task = new RemoteTask<AlgorithmOutline[]>() {
      @Override
      public void performTask() throws Exception {
        AlgorithmServiceInterface algoService = ClientRemoteLocator.algorithmService;
        setResult(algoService.getAllOutlines());
      }
    };
    task.execute(owner, LNG.get("algorithm.title.getting"), LNG
      .get("algorithm.msg.getting"));
    return task.getResult();
  }

  // #TODO A remover quando todos sistemas CSBase forem assinados {
  /**
   * Obtem a URL para upload do configurador.
   * 
   * @param algoId identificador do algoritmo que ir receber o configurador.
   * @param versionId identificador da verso que ir receber o configurador.
   * @param owner janela pai para efeito de hierarquia de dilogos.
   * 
   * @return O endereo para upload.
   */
  public static String getConfiguratorURL(final Object algoId,
    final Object versionId, Window owner) {
    RemoteTask<String> task = new RemoteTask<String>() {
      @Override
      public void performTask() throws Exception {
        setResult(ClientRemoteLocator.algorithmService.retrieveConfigUploadURL(
          algoId, versionId));
      }
    };
    task.execute(owner, taskTitle, LNG.get("algomanager.msg.wait"));
    String result = task.getResult();
    if (result == null) {
      showError(LNG.get("HTTP_URL_ERROR"), null);
    }
    return result;
  }

  /**
   * Obtem a URL para upload de um executvel.
   * 
   * @param algoId identificador do algoritmo que ir receber o executvel.
   * @param versionId identificador da verso que ir receber o executvel.
   * @param platform plataforma que ir receber o executvel.
   * @param owner janela pai para efeito de hierarquia de dilogos.
   * 
   * @return O endereo para upload.
   */
  public static String getExecutableURL(final Object algoId,
    final Object versionId, final String platform, Window owner) {
    RemoteTask<String> task = new RemoteTask<String>() {
      @Override
      public void performTask() throws Exception {
        setResult(ClientRemoteLocator.algorithmService.retrieveExecUploadURL(
          algoId, versionId, platform));
      }
    };
    task.execute(owner, taskTitle, LNG.get("algomanager.msg.wait"));
    String result = task.getResult();
    if (result == null) {
      showError(LNG.get("HTTP_URL_ERROR"), null);
    }
    return result;
  }

  // #TODO } A remover quando todos sistemas CSBase forem assinados

  /**
   * Obtem as informaes de um algoritmo.
   * 
   * @param name nome do algoritmo.
   * @param owner janela pai para efeito de hierarquia de dilogos.
   * 
   * @return as informaes do algoritmo
   */
  public static AlgorithmInfo getInfoByName(final String name, Window owner) {
    RemoteTask<AlgorithmInfo> task = new RemoteTask<AlgorithmInfo>() {
      @Override
      public void performTask() throws Exception {
        setResult(AlgorithmInfo.getAlgorithmInfo(name));
      }
    };
    task.execute(owner, LNG.get("algorithm.title.getting"), LNG
      .get("algorithm.msg.getting"));
    return task.getResult();
  }

  /**
   * Obtem as informaes de um algoritmo.
   * 
   * @param algoId identificador do algoritmo
   * @param owner janela pai para efeito de hierarquia de dilogos.
   * 
   * @return as informaes do algoritmo
   */
  public static AlgorithmInfo getInfoById(final Object algoId, Window owner) {
    RemoteTask<AlgorithmInfo> task = new RemoteTask<AlgorithmInfo>() {
      @Override
      public void performTask() throws Exception {
        setResult(AlgorithmInfo.getAlgorithmInfo(algoId));
      }
    };
    task.execute(owner, LNG.get("algorithm.title.getting"), LNG
      .get("algorithm.msg.getting"));
    return task.getResult();
  }

  /**
   * Inclui uma plataforma para uma verso de algoritmo.
   * 
   * @param algoId identificador do algoritmo
   * @param versionId identificador da verso
   * @param platform plataforma a incluir
   * @param owner janela pai para efeito de hierarquia de dilogos.
   * 
   * @return informaes atualizadas do algoritmo
   */
  public static AlgorithmInfo includePlatform(final Object algoId,
    final Object versionId, final String platform, Window owner) {
    RemoteTask<AlgorithmInfo> task = new RemoteTask<AlgorithmInfo>() {
      @Override
      public void performTask() throws Exception {
        setResult(ClientRemoteLocator.algorithmService.includePlatform(algoId,
          versionId, platform));
      }
    };
    task.execute(owner, taskTitle, LNG.get("algomanager.msg.wait"));
    return task.getResult();
  }

  /**
   * Aplica a recarga doa algoritmos no servidor
   * 
   * @param owner janela pai para efeito de hierarquia de dilogos.
   */
  public static void reloadAlgorithms(Window owner) {
    RemoteTask<Boolean> task = new RemoteTask<Boolean>() {
      @Override
      public void performTask() throws Exception {
        setResult(ClientRemoteLocator.algorithmService.reloadAlgorithms());
      }
    };
    if ((task.execute(owner, LNG.get("algorithm.title.reloading"), LNG
      .get("algorithm.msg.reloading")))
      && (task.getResult())) {
      StandardDialogs.showInfoDialog(owner, LNG.get("algorithm.title.reload"),
        LNG.get("algorithm.msg.reload"));
    }
  }

  /**
   * Remove um algoritmo.
   * 
   * @param algoId identificador do algoritmo
   * @param owner janela pai para efeito de hierarquia de dilogos.
   * 
   * @return <code>true</code> se a remoo teve sucesso, <code>false</code>
   *         caso contrrio
   */
  public static boolean removeAlgorithm(final Object algoId, Window owner) {
    RemoteTask<Boolean> task = new RemoteTask<Boolean>() {
      @Override
      public void performTask() throws Exception {
        setResult(ClientRemoteLocator.algorithmService.removeAlgorithm(algoId));
      }
    };
    return (task.execute(owner, taskTitle, LNG.get("algomanager.msg.wait")) && task
      .getResult());
  }

  /**
   * Solicita ao servidor a remoo dos arquivos de configurao especificados.
   * 
   * @param algoId identificador do algoritmo do qual os arquivos sero
   *        removidos.
   * @param versionId identificador da verso da qual os arquivos sero
   *        removidos.
   * @param filePaths caminhos para os arquivos a serem removidos.
   * @param owner janela pai para efeito de hierarquia de dilogos.
   * 
   * @return true caso os arquivos tenham sido removidos com sucesso.
   */
  public static boolean removeFile(final Object algoId,
    final AlgorithmVersionId versionId, final String[][] filePaths, JFrame owner) {
    RemoteTask<Void> task = new RemoteTask<Void>() {
      @Override
      public void performTask() throws Exception {
        AlgorithmServiceInterface algoService = ClientRemoteLocator.algorithmService;
        algoService.removeFile(algoId, versionId, filePaths);
      }
    };
    return task.execute(owner, taskTitle, LNG.get("algomanager.msg.wait"));
  }

  /**
   * Remove arquivos executveis de uma verso de algoritmo.
   * 
   * @param version Verso que detm os arquivos.
   * @param platform Plataforma que detm os arquivos.
   * @param files Representao dos arquivos a serem removidos.
   * @param window Janela pai para efeito de hierarquia de dilogos.
   */
  public static boolean removeExecutableFiles(
    final AlgorithmVersionInfo version, final String platform,
    final FileInfo[] files, final Window window) {
    RemoteTask<Void> task = new RemoteTask<Void>() {
      @Override
      public void performTask() throws Exception {
        AlgorithmServiceInterface algoService = ClientRemoteLocator.algorithmService;
        algoService.removeExecutableFiles(version, platform, files);
      }
    };
    return task.execute(window, taskTitle, LNG.get("algomanager.msg.wait"));
  }

  /**
   * Remove arquivos de configurao de uma verso de algoritmo.
   * 
   * @param version Verso que detm os arquivos.
   * @param platform Plataforma que detm os arquivos.
   * @param files Representao dos arquivos a serem removidos.
   * @param window Janela pai para efeito de hierarquia de dilogos.
   */
  public static boolean removeConfigurationFiles(
    final AlgorithmVersionInfo version, final FileInfo[] files,
    final Window window) {
    RemoteTask<Void> task = new RemoteTask<Void>() {
      @Override
      public void performTask() throws Exception {
        AlgorithmServiceInterface algoService = ClientRemoteLocator.algorithmService;
        algoService.removeConfigurationFiles(version, files);
      }
    };
    return task.execute(window, taskTitle, LNG.get("algomanager.msg.wait"));
  }

  /**
   * Remove arquivos de documentao de uma verso de algoritmo.
   * 
   * @param version Verso que detm os arquivos.
   * @param platform Plataforma que detm os arquivos.
   * @param files Representao dos arquivos a serem removidos.
   * @param window Janela pai para efeito de hierarquia de dilogos.
   */
  public static boolean removeDocumentationFiles(
    final AlgorithmVersionInfo version, final FileInfo[] files,
    final Window window) {
    RemoteTask<Void> task = new RemoteTask<Void>() {
      @Override
      public void performTask() throws Exception {
        AlgorithmServiceInterface algoService = ClientRemoteLocator.algorithmService;
        algoService.removeDocumentationFiles(version, files);
      }
    };
    return task.execute(window, taskTitle, LNG.get("algomanager.msg.wait"));
  }

  /**
   * Remove uma plataforma de uma verso de algoritmo.
   * 
   * @param algoId identificador do algoritmo
   * @param versionId identificador da verso
   * @param platform plataforma a remover
   * @param owner janela pai para efeito de hierarquia de dilogos.
   * 
   * @return informaes atualizadas do algoritmo
   */
  public static AlgorithmInfo removePlatform(final Object algoId,
    final Object versionId, final String platform, Window owner) {
    RemoteTask<AlgorithmInfo> task = new RemoteTask<AlgorithmInfo>() {
      @Override
      public void performTask() throws Exception {
        setResult(ClientRemoteLocator.algorithmService.removePlatform(algoId,
          versionId, platform));
      }
    };
    task.execute(owner, taskTitle, LNG.get("algomanager.msg.wait"));
    return task.getResult();
  }

  /**
   * Remove uma verso de algoritmo.
   * 
   * @param algoId identificador do algoritmo
   * @param versionId identificador da verso
   * @param owner janela pai para efeito de hierarquia de dilogos.
   * 
   * @return informaes atualizadas do algoritmo
   */
  public static AlgorithmInfo removeVersion(final Object algoId,
    final Object versionId, Window owner) {
    RemoteTask<AlgorithmInfo> task = new RemoteTask<AlgorithmInfo>() {
      @Override
      public void performTask() throws Exception {
        setResult(ClientRemoteLocator.algorithmService.removeVersion(algoId,
          versionId));
      }
    };
    task.execute(owner, taskTitle, LNG.get("algomanager.msg.wait"));
    return task.getResult();
  }

  /**
   * Obtm todas as informaes sobre algoritmos existentes.
   * 
   * @param owner janela pai para efeito de hierarquia de dilogos.
   * 
   * @return lista de informaes de algoritmos.
   */
  public static AlgorithmInfo[] retrieveAllAlgorithmInfos(Window owner) {
    RemoteTask<AlgorithmInfo[]> task = new RemoteTask<AlgorithmInfo[]>() {
      @Override
      public void performTask() throws Exception {
        setResult(ClientRemoteLocator.algorithmService.getAllInfo());
      }
    };
    task.execute(owner, LNG.get("algomanager.title.consulting_data"), LNG
      .get("algomanager.msg.consulting_data"));
    return task.getResult();
  }

  /**
   * Obtm uma url para permitir o download de um arquivo qualquer a partir da
   * rvore de algoritmos para o disco rgido do usurio.
   * 
   * @param algoId identificador do algoritmo ao qual pertence o arquivo.
   * @param filePath caminho para o arquivo.
   * @param owner janela pai para efeito de hierarquia de dilogos.
   * 
   * @return url para download de um arquivo qualquer.
   */
  public static String retrieveDownloadURL(final Object algoId,
    final String[] filePath, Window owner) {
    RemoteTask<String> task = new RemoteTask<String>() {
      @Override
      public void performTask() throws Exception {
        AlgorithmServiceInterface algoService = ClientRemoteLocator.algorithmService;
        setResult(algoService.retrieveDownloadURL(algoId, filePath));
      }
    };
    task.execute(owner, taskTitle, LNG.get("algomanager.msg.wait"));
    return task.getResult();
  }

  // #TODO A remover quando todos sistemas CSBase forem assinados {
  /**
   * Obtm uma url para permitir o upload de um arquivo html a partir do disco
   * rgido do usurio para a rvore de algoritmos.
   * 
   * @param algoId identificador do algoritmo que receber o arquivo.
   * @param versionId identificador da verso que receber o arquivo.
   * @param owner janela pai para efeito de hierarquia de dilogos.
   * 
   * @return url para upload de um arquivo html.
   */
  public static String retrieveDocUploadURL(final Object algoId,
    final Object versionId, Window owner) {
    RemoteTask<String> task = new RemoteTask<String>() {
      @Override
      public void performTask() throws Exception {
        AlgorithmServiceInterface algoService = ClientRemoteLocator.algorithmService;
        setResult(algoService.retrieveDocUploadURL(algoId, versionId));
      }
    };
    task.execute(owner, taskTitle, LNG.get("algomanager.msg.wait"));
    return task.getResult();
  }

  // #TODO } A remover quando todos sistemas CSBase forem assinados

  /**
   * Obtm o histrico do caminho especificado.
   * 
   * @param path caminho cujo histrico deseja-se obter.
   * 
   * @return <code>List</code> contendo instncias de <code>HistoryRecord</code>
   *         , representando o histrico do caminho especificado.
   */
  public static List<HistoryRecord> retrieveHistory(final String[] path) {
    RemoteTask<List<HistoryRecord>> task = new RemoteTask<List<HistoryRecord>>() {
      @Override
      public void performTask() throws Exception {
        AlgorithmServiceInterface algoService = ClientRemoteLocator.algorithmService;
        setResult(algoService.retrieveHistory(path));
      }
    };
    JFrame owner = DesktopFrame.getInstance().getDesktopFrame();
    task.execute(owner, taskTitle, LNG.get("algomanager.msg.wait"));
    return task.getResult();
  }

  /**
   * Mtodo de aviso de erro
   * 
   * @param msg mensagem a ser exibida.
   * @param e exceo causadora do erro (ou null)
   */
  private static void showError(String msg, Exception e) {
    final DesktopFrame mainFrame = DesktopFrame.getInstance();
    if (mainFrame == null) {
      return;
    }
    JFrame jFrame = mainFrame.getDesktopFrame();
    if (jFrame == null) {
      return;
    }
    String title = LNG.get("algomanager.title.error") + " - "
      + LNG.get("algomanager.title");
    if (e != null) {
      StandardErrorDialogs.showErrorDialog(jFrame, title, msg, e);
    }
    else {
      StandardErrorDialogs.showErrorDialog(jFrame, title, msg);
    }
  }

  /**
   * Altera as propriedades estendidas de um algorimto.
   * 
   * @param name nome do algoritmo
   * @param newValues novos valores para os atributos
   * @param owner janela pai para efeito da hierarquia de dialogos
   * 
   * @return AlgorithmInfo alterado
   */
  public static AlgorithmInfo changeAlgorithmProperties(final String name,
    final Hashtable<String, String> newValues, Window owner) {
    RemoteTask<AlgorithmInfo> task = new RemoteTask<AlgorithmInfo>() {
      @Override
      public void performTask() throws Exception {
        setResult(ClientRemoteLocator.algorithmService
          .changeAlgorithmProperties(name, newValues));
      }
    };
    task.execute(owner, taskTitle, LNG.get("algomanager.msg.wait"));
    return task.getResult();
  }

  /**
   * Altera as propriedades estendidas de uma versao de algoritmo
   * 
   * @param name nome do algoritmo
   * @param versionId id da versao a ser alterada
   * @param newValues novos valores para os atributos
   * @param owner janela pai para efeito da hierarquia de dialogos
   * 
   * @return AlgorithmVersionInfo alterado
   */
  public static AlgorithmInfo changeVersionProperties(final String name,
    final AlgorithmVersionId versionId,
    final Hashtable<String, String> newValues, Window owner) {
    RemoteTask<AlgorithmInfo> task = new RemoteTask<AlgorithmInfo>() {
      @Override
      public void performTask() throws Exception {
        setResult(ClientRemoteLocator.algorithmService.changeVersionProperties(
          name, versionId, newValues));
      }
    };
    task.execute(owner, taskTitle, LNG.get("algomanager.msg.wait"));
    return task.getResult();
  }

  /**
   * Retorna a lista de propriedades da verso do algoritmo. Cada property
   * mapeia a chave da propriedade ao label (String de Apresentao).
   * 
   * @param owner .
   * 
   * @return lista de propriedades da verso do algoritmo (na ordem correta)
   */
  public static List<AlgorithmProperty> getAlgorithmVersionProperties(
    Window owner) {
    RemoteTask<List<AlgorithmProperty>> task = new RemoteTask<List<AlgorithmProperty>>() {
      @Override
      public void performTask() throws Exception {
        AlgorithmServiceInterface algoService = ClientRemoteLocator.algorithmService;
        setResult(algoService.getVersionProperties());
      }
    };
    task.execute(owner, taskTitle, LNG.get("algomanager.msg.wait"));
    return task.getResult();
  }

  /**
   * Retorna a lista de propriedades do algoritmo. Cada property mapeia a chave
   * da propriedade ao label (String de Apresentao).
   * 
   * @param owner .
   * 
   * @return lista de propriedades do algoritmo (na ordem correta)
   */
  public static List<AlgorithmProperty> getAlgorithmProperties(Window owner) {
    RemoteTask<List<AlgorithmProperty>> task = new RemoteTask<List<AlgorithmProperty>>() {
      @Override
      public void performTask() throws Exception {
        AlgorithmServiceInterface algoService = ClientRemoteLocator.algorithmService;
        setResult(algoService.getAlgorithmProperties());
      }
    };
    task.execute(owner, taskTitle, LNG.get("algomanager.msg.wait"));
    return task.getResult();
  }

  // #TODO A remover quando todos sistemas CSBase forem assinados {
  /**
   * Obtem a URL para upload de um pacote de verso.
   * 
   * @param algoId identificador do algoritmo que ir receber a verso.
   * @param owner janela pai para efeito de hierarquia de dilogos.
   * 
   * @return O endereo para upload ou {@code null} se houver um erro.
   */
  public static String getVersionURL(final Object algoId, Window owner) {
    RemoteTask<String> task = new RemoteTask<String>() {
      @Override
      public void performTask() throws Exception {
        AlgorithmServiceInterface algoService = ClientRemoteLocator.algorithmService;
        if (algoService != null) {
          setResult(algoService.retrieveVersionUploadURL(algoId));
        }
      }
    };
    if (!task.execute(owner, taskTitle, LNG.get("algomanager.msg.wait"))) {
      return null;
    }
    Object result = task.getResult();
    if (result == null) {
      showError(LNG.get("HTTP_URL_ERROR"), null);
    }
    return (String) result;
  }
  // #TODO } A remover quando todos sistemas CSBase forem assinados
}
