/*
 * HttpProxy.java
 * @author Andr Luiz Clinio
 * $Revision: 54777 $  - $Date: 2006-08-15 11:01:24 -0300 (Tue, 15 Aug 2006) $
 */
package csbase.client.remote.srvproxies;

import csbase.client.desktop.DesktopFrame;
import csbase.client.desktop.DesktopTask;
import csbase.client.remote.ClientRemoteMonitor;
import csbase.client.util.StandardErrorDialogs;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.HttpServiceInterface;

import javax.swing.JFrame;

import tecgraf.javautils.LNG;

/**
 * Classe que modela um proxy para o HttpService.
 *
 * @author Taciana Melcop
 */
public class HttpProxy {
  /** Texto de falha do PROXY */
  static String failureMessage = LNG.get("HTTP_PROXY_FAILURE");

  /** Texto de sucesso do PROXY */
  static String successMessage = LNG.get("HTTP_PROXY_SUCCESS");

  /** Ttulo da janela do PROXY */
  static String title = LNG.get("HTTP_PROXY_TITLE");

  /** Constante que representa o download */
  private static final String DOWNLOAD = "download";

  /** Constante que representa o upload */
  private static final String UPLOAD = "upload";

  /**
   * Mtodo de aviso de erro
   *
   * @param msg mensagem a ser exibida.
   */
  private static void showError(String msg) {
    final DesktopFrame dsk = DesktopFrame.getInstance();
    if (dsk == null) {
      return;
    }
    final JFrame mainFrame = dsk.getDesktopFrame();
    if (mainFrame == null) {
      return;
    }
    StandardErrorDialogs.showErrorDialog(mainFrame, title, msg);
  }

  /**
   * Obtem a url atraves da qual o arquivo deve ser baixado
   *
   * @param userId Identificador do usurio no servidor http remoto.
   * @param ownerId Identificador do usurio dono do projeto.
   * @param projectName identificador do projeto.
   * @param filePath caminho do arquivo local no projeto.
   *
   * @return String que representa a url do arquivo ou <code>null</code> caso
   *         ocorra algum problema durante a construo da url.
   */
  public static String getDownloadURL(
    Object userId, Object ownerId, String projectName, String[] filePath) {
    return getURL(userId, ownerId, projectName, filePath, null, DOWNLOAD);
  }

  /**
   * Obtem a url atraves da qual o arquivo deve ser inserido.
   *
   * @param userId Identificador do usurio no servidor http remoto.
   * @param ownerId Identificador do usurio dono do projeto.
   * @param projectName identificador do projeto.
   * @param filePath caminho do arquivo local no projeto.
   * @param fileType .
   *
   * @return String que representa a url do arquivo
   */
  public static String getUploadURL(
    Object userId, Object ownerId, String projectName, String[] filePath,
    String fileType) {
    return getURL(userId, ownerId, projectName, filePath, fileType, UPLOAD);
  }

  /**
   * Obtem a url atraves da qual o arquivo deve ser acessado.
   *
   * @param userId Identificador do usurio no servidor http remoto.
   * @param ownerId Identificador do usurio dono do projeto.
   * @param projectName identificador do projeto.
   * @param filePath caminho do arquivo local no projeto.
   * @param fileType Tipo do arquivo no caso do upload
   * @param type .
   *
   * @return String que representa a url do arquivo ou <code>null</code> caso
   *         ocorra algum problema durante a construo da url.
   */
  private static String getURL(
    final Object userId, final Object ownerId, final String projectName,
    final String[] filePath, final String fileType, final String type) {
    final HttpServiceInterface httpService = ClientRemoteLocator.httpService;
    final DesktopFrame dsk = DesktopFrame.getInstance();
    final JFrame mainFrame = dsk.getDesktopFrame();
    final String password = ClientRemoteMonitor.getInstance().getPassword();
    if ((mainFrame == null) || (httpService == null)) {
      showError(failureMessage);
      return null;
    }
    DesktopTask task =
      new DesktopTask() {
        public void performTask()
        throws Exception {
          if (type.equals(DOWNLOAD)) {
            setResult(
              httpService.getDownloadURL(
                userId, password, ownerId, projectName, filePath));
          }
          else {
            setResult(
              httpService.getUploadURL(
                userId, password, ownerId, projectName, filePath, fileType));
          }
        }
      };
    task.start(
      DesktopFrame.getInstance().getDesktopFrame(), title, successMessage);
    /* Informa se ocorreu uma exceo */
    Exception exception = task.getException();
    if (exception != null) {
      showError(exception.getMessage());
      return null;
    }
    String result = (String)task.getResult();
    if (result == null) {
      showError(LNG.get("HTTP_URL_ERROR"));
    }
    return result;
  }

  /**
   * Ajuste dos textos
   *
   * @param proxyTitle o ttulo.
   * @param proxyFailureText mensagem de falha
   * @param proxySuccessText de sucesso
   */
  public static void setTexts(
    String proxyTitle, String proxyFailureText, String proxySuccessText) {
    title = proxyTitle;
    failureMessage = proxyFailureText;
    successMessage = proxySuccessText;
  }
}
