/*
 * NotificationProxy.java
 * @author Andr Luiz Clinio
 * $Revision: 94309 $  - $Date: 2009-07-29 16:35:57 -0300 (Wed, 29 Jul 2009) $
 */
package csbase.client.remote.srvproxies;

import java.rmi.RemoteException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Observer;
import java.util.Vector;

import javax.swing.JFrame;

import tecgraf.javautils.LNG;
import csbase.client.desktop.DesktopFrame;
import csbase.client.desktop.DesktopTask;
import csbase.client.kernel.ClientException;
import csbase.client.remote.NotificationObserver;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.User;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.NotificationServiceInterface;

/**
 * Classe que modela um proxy client/NotificationService.
 *
 * @author Andr Luiz Clinio
 */
public class NotificationProxy {
  /** Hash de usurios */
  private static Hashtable<Object, NotificationObserver> userHash =
    new Hashtable<Object, NotificationObserver>();

  /**
   * Remove observer do usurio.
   *
   * @param userId .
   *
   * @throws RemoteException .
   */
  static void killUserObserver(Object userId)
  throws RemoteException {
    NotificationObserver observer = userHash.get(userId);
    if (observer != null) {
      observer.decRefs();
      if (observer.hasNoRefs()) {
        userHash.remove(userId);
        observer.uninstall();
      }
    }
  }

  /**
   * Busca de um observer para um usurio.
   *
   * @param userId .
   *
   * @return .
   *
   * @throws RemoteException .
   */
  static NotificationObserver getUserObserver(Object userId)
  throws RemoteException {
    NotificationObserver observer = userHash.get(userId);
    if (observer == null) {
      observer = new NotificationObserver();
      userHash.put(userId, observer);
    }
    observer.incRefs();
    return observer;
  }

  /**
   * Mtodo de busca do servio
   *
   * @return o servio de notificao.
   */
  private static NotificationServiceInterface getService() {
    return ClientRemoteLocator.notificationService;
  }

  /**
   * Mtodo de aviso de erro
   *
   * @param msg mensagem a ser exibida.
   */
  private static void showError(String msg) {
    final DesktopFrame mainFrame = DesktopFrame.getInstance();
    if (mainFrame == null) {
      return;
    }
    JFrame jFrame = mainFrame.getDesktopFrame();
    if (jFrame == null) {
      return;
    }
    StandardErrorDialogs.showErrorDialog(
      jFrame, LNG.get("notification.check.title"), msg);
  }

  /**
   * Mtodo para auto-notifificao do usurio.
   *
   * @param senderName string de identificao do remetente
   * @param content contedo da mensagem
   *
   * @return flag booleano de sucesso da operao
   */
  public static boolean autoNotify(String senderName, String content) {
    User loggedUser = User.getLoggedUser();
    if (loggedUser != null) {
      return notifyTo(senderName, loggedUser.getId(), content, true, false);
    }
    return false;
  }

  /**
   * Mtodo para notifificao a um usurio.
   *
   * @param senderName string de identificao do remetente
   * @param userId identificador do usurio destino
   * @param content contedo da mensagem
   * @param showMsg .
   * @param volatileFlag .
   *
   * @return flag booleano de sucesso da operao
   */
  public static boolean notifyTo(
    String senderName, Object userId, String content, boolean showMsg,
    final boolean volatileFlag) {
    return notifyTo(senderName, userId, content, showMsg, false, volatileFlag);
  }

  public static boolean notifyTo(
    String senderName, Object userId, String content, boolean showMsg,
    final boolean mustPopUp, final boolean volatileFlag) {
    final DesktopFrame mainFrame = DesktopFrame.getInstance();
    final NotificationServiceInterface service = getService();
    if ((mainFrame == null) || (service == null)) {
      showError(LNG.get("notification.no.server.error"));
      return false;
    }
    final String sndName = senderName;
    final Object uId = userId;
    final String cnt = content;
    DesktopTask task =
      new DesktopTask() {
        @Override
        public void performTask()
        throws Exception {
          if (!service.notifyTo(sndName, uId, cnt, mustPopUp, volatileFlag)) {
            throw new Exception(LNG.get("notification.failed.message"));
          }
        }
      };
    task.start(
      mainFrame.getDesktopFrame(), LNG.get("notification.send.title"),
      LNG.get("notification.send.message"));

    /* Informa se ocorreu uma exceo */
    Exception exception = task.getException();
    if (exception != null) {
      if (showMsg) {
        showError(exception.getMessage());
      }
      return false;
    }
    return true;
  }

  /**
   * Mtodo para notifificao de vrios usurios
   *
   * @param senderName string de identificao do remetente
   * @param userIds identificadores do usurio destino
   * @param content contedo da mensagem
   * @param volFlag flag de volatilidade.
   *
   * @return flag booleano de sucesso da operao
   */
  public static boolean notifyTo(
    final String senderName, final Vector userIds, final String content,
    final boolean volFlag) {
    return notifyTo(senderName, userIds, content, false, volFlag);
  }

  public static boolean notifyTo(
    final String senderName, final Vector userIds, final String content,
    final boolean mustPopUp, final boolean volFlag) {
    final DesktopFrame mainFrame = DesktopFrame.getInstance();
    final NotificationServiceInterface service = getService();
    if ((mainFrame == null) || (service == null)) {
      showError(LNG.get("notification.no.server.error"));
      return false;
    }
    DesktopTask task =
      new DesktopTask() {
        @Override
        public void performTask()
        throws Exception {
          for (int i = 0; i < userIds.size(); i++) {
            Object id = userIds.get(i);
            if (!service.notifyTo(senderName, id, content, mustPopUp, volFlag)) {
              throw new Exception(LNG.get("notification.failed.message"));
            }
          }
        }
      };
    task.start(
      mainFrame.getDesktopFrame(), LNG.get("notification.send.title"),
      LNG.get("notification.send.message"));

    /* Informa se ocorreu uma exceo */
    Exception exception = task.getException();
    if (exception != null) {
      showError(exception.getMessage());
      return false;
    }
    return true;
  }

  /**
   * Adio de observador remoto de notificaes.
   *
   * @param observer observador remoto.
   * @param userId identificador do usurio.
   */
  public static void addUserObserver(Observer observer, Object userId) {
    try {
      NotificationObserver not = getUserObserver(userId);
      not.addObserver(observer);
      not.addUserObserver(userId);
    }
    catch (Exception e) {
      showError(e.getMessage());
    }
  }

  /**
   * Remoo de observador remoto de notificaes.
   *
   * @param observer observador remoto.
   * @param userId identificador do usurio.
   */
  public static void deleteUserObserver(Observer observer, Object userId) {
    try {
      NotificationObserver not = getUserObserver(userId);
      not.deleteUserObserver(userId);
      not.deleteObserver(observer);
      killUserObserver(userId);
    }
    catch (Exception e) {
      showError(e.getMessage());
    }
  }

  /**
   * Remoo de todos os observadores.
   *
   * @throws Exception .
   * @throws ClientException .
   */
  public static void resetObservers()
  throws Exception {
    final NotificationServiceInterface service = getService();
    if (service == null) {
      throw new ClientException(LNG.get("notification.no.server.error"));
    }
    for (Enumeration e = userHash.keys(); e.hasMoreElements();) {
      Object uid = e.nextElement();
      NotificationObserver not = userHash.get(uid);
      not.deleteUserObserver(uid);
      not.deleteObservers();
      not.uninstall();
    }
    userHash = new Hashtable();
  }
}
