package csbase.client.util.gui.log.actions;

import java.awt.Window;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;

import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JToggleButton;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.applications.ApplicationImages;
import csbase.client.desktop.RemoteTask;
import csbase.client.util.gui.log.LogPanel;
import csbase.client.util.gui.log.actions.core.AbstractLogFileAction;
import csbase.logic.ClientProjectFile;
import csbase.logic.ClientProjectFileInputStream;

/**
 * Exporta o arquivo de log.
 * 
 * @author Tecgraf
 */
public class ExportLogAction extends AbstractLogFileAction {

  /** Boto que detm esta ao. */
  private JToggleButton parent;

  /**
   * File chooser nico para a exportao para manter o path do ltimo diretrio
   * escolhido.
   */
  private JFileChooser exportationFileChooser;

  /**
   * Construtor.
   * 
   * @param parent Boto que detm esta ao.
   * @param logPanel Painel de log que detm o boto dessa ao.
   */
  public ExportLogAction(JToggleButton parent, final LogPanel logPanel) {
    super(logPanel, ApplicationImages.ICON_EXPORT_16);
    this.parent = parent;
    this.exportationFileChooser = createCustomChooser();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void actionDone(JComponent component) throws Exception {

    // Obtm o arquivo local que conter o arquivo de log.
    final File localFile = getLocalFile();
    if (localFile == null) {
      parent.setSelected(false);
      return;
    }

    // Exporta o arquivo de log para o sistema de arquivos local.
    RemoteTask<Void> task = new RemoteTask<Void>() {
      @Override
      protected void performTask() throws Exception {
        parent.setSelected(false);
        exportAsStream(localFile);
      }
    };

    // Executa a task para exportar o arquivo de log.
    Window window = getLogPanel().getOwner();
    String title = LNG.get("ExportLogAction.export.title");
    String message = LNG.get("ExportLogAction.export.log");
    boolean result = task.execute(window, title, message);
    if (result) {
      message =
        LNG.get("ExportLogAction.export.log.success", new Object[] { localFile
          .getAbsolutePath() });
      StandardDialogs.showInfoDialog(window, title, message);
    }
  }

  /**
   * Obtm o arquivo local que conter o arquivo de log.
   * 
   * @return Arquivo local que conter o arquivo de log.
   */
  private File getLocalFile() {
    // Dilogo para a seleo do diretrio/arquivo de destino a ser exportado o
    // log.
    final int ret =
      exportationFileChooser.showDialog(getLogPanel(), LNG
        .get("ExportLogAction.chooser.approve.button"));
    if (ret == JFileChooser.CANCEL_OPTION) {
      return null;
    }
    File localFile = exportationFileChooser.getSelectedFile();
    if (localFile.isDirectory()) {
      // O usurio selecionou um diretrio. O log ser gravado nesse diretrio
      // com o mesmo nome do arquivo de log remoto.
      final ClientProjectFile remoteFile = getLogPanel().getFile();
      String nameLocalFile =
        localFile.getAbsolutePath() + "/" + remoteFile.getName();
      localFile = new File(nameLocalFile);
    }
    // Verifica se o arquivo de log j existe.
    if (localFile != null && localFile.exists()) {
      final String title = LNG.get("ExportLogAction.export.title");
      final String[] args =
        new String[] { localFile.getName(),
            localFile.getParentFile().getAbsolutePath() };
      final String msg = LNG.get("ExportLogAction.export.overwrite.msg", args);
      final int option =
        StandardDialogs.showYesNoDialog(getLogPanel().getOwner(), title, msg);
      if (option == 1) {
        return null;
      }
    }
    // Para manter o path do ltimo diretrio escolhido.
    exportationFileChooser.setCurrentDirectory(localFile);
    return localFile;
  }

  /**
   * Exporta o arquivo de log como stream.
   * 
   * @param localFile Arquivo local que conter o arquivo de log.
   * @throws Exception
   */
  private void exportAsStream(final File localFile) throws Exception {
    final int bufferSize = 48 * 1024;
    final ClientProjectFile remoteFile = getLogPanel().getFile();
    if (remoteFile == null || !remoteFile.exists())
      return;
    final ClientProjectFileInputStream prjStream =
      new ClientProjectFileInputStream(remoteFile, bufferSize);
    final BufferedInputStream in = new BufferedInputStream(prjStream);
    BufferedOutputStream out = null;

    final FileOutputStream fileStream = new FileOutputStream(localFile);
    out = new BufferedOutputStream(fileStream);
    byte[] buffer = new byte[bufferSize];
    int len = 0;
    while ((len = in.read(buffer)) > 0) {
      out.write(buffer, 0, len);
    }
    out.flush();
    out.close();
    in.close();
  }

  /**
   * Instncia um file chooser personalizado para escolher um arquivo para
   * exportao.
   * 
   * @return O JFileChooser.
   */
  private JFileChooser createCustomChooser() {
    exportationFileChooser = new JFileChooser();
    exportationFileChooser.setMultiSelectionEnabled(false);
    exportationFileChooser.setApproveButtonText(LNG
      .get("ExportLogAction.chooser.approve.button"));
    exportationFileChooser.setApproveButtonToolTipText(LNG
      .get("ExportLogAction.chooser.approve.button.tooltip"));
    exportationFileChooser.setDialogTitle(LNG
      .get("ExportLogAction.chooser.title"));
    exportationFileChooser
      .setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
    return exportationFileChooser;
  }

}