package csbase.client.util.sga;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

import tecgraf.javautils.LNG;
import tecgraf.javautils.configurationmanager.Configuration;
import tecgraf.javautils.configurationmanager.ConfigurationManager;
import tecgraf.javautils.configurationmanager.ConfigurationManagerException;
import csbase.client.algorithms.AlgorithmConfiguratorView;
import csbase.client.algorithms.parameters.validators.ValidationResult;
import csbase.client.applications.ApplicationImages;
import csbase.client.desktop.DesktopComponentDialog;
import csbase.client.remote.srvproxies.SchedulerProxy;
import csbase.client.util.ClientUtilities;
import csbase.logic.CommandInfo;
import csbase.logic.CommandSubmission;
import csbase.logic.CommonClientProject;
import csbase.logic.Priority;
import csbase.logic.SGASet;
import csbase.logic.algorithms.AlgorithmConfigurator;
import csbase.logic.algorithms.ExecutionType;
import csbase.logic.algorithms.flows.configurator.FlowAlgorithmConfigurator;

/**
 * O Dilogo de Execuo de Comando  um dilogo que permite ao usurio escolher
 * em qual servidor sga executar o seu algoritmo e executar o comando.
 * 
 * @author valeria
 */
public final class CommandExecutionDialog extends DesktopComponentDialog {
  /**
   * Nome da propriedade que habilita a escolha de prioridade.  usada apenas
   * nos sistemas que possuem fila de prioridades.
   */
  private static final String ENABLE_PRIORITY = "enable.priority";

  /**
   * Painel seletor de SGAs.
   */
  private SGASelectionPanel sgaPanel;

  /**
   * Boto Executar.
   */
  private JButton executeButton;

  /**
   * Boto Fechar.
   */
  private JButton closeButton;

  /**
   * Rtulo Descrio do Comando.
   */
  private JLabel commandDescriptionLabel;

  /**
   * Campo de Texto Descrio do Comando.
   */
  private JTextField commandDescriptionTextField;

  /**
   * CheckBox Notificar Trmino por E-mail.
   */
  private JCheckBox emailCheckBox;

  /**
   * Painel de botes.
   */
  private JPanel buttonPanel;

  /**
   * Rtulo Status dos parmetros do configurador.
   */
  private JLabel parameterStatusLabel;

  /**
   * Rtulo Indicativo de SGA corretamento escolhido/selecionado.
   */
  private JLabel sgaStatusLabel;

  /**
   * Rtulo Prioridade.
   */
  private JLabel priorityLabel;

  /**
   * ComboBox Prioridade.
   */
  private JComboBox priorityComboBox;

  /**
   * Projeto.
   */
  private final CommonClientProject project;

  /**
   * A viso do configurador do algoritmo selecionado.
   */
  private final AlgorithmConfiguratorView configuratorView;

  /**
   * O configurador de algoritmos selecionado.
   */
  private final AlgorithmConfigurator configurator;

  /**
   * Flag que indica se a opo de escolha de prioridade de um comando deve ser
   * exibida.
   */
  private boolean enablePriority;

  /**
   * Listeners interessados nos comandos solicitados com esse dilogo
   */
  private final List<CommandRequestedListener> listeners;

  /**
   * Cria um Dilogo de Execuo de Comandos.
   * 
   * @param parentFrame A janela que est requisitando este configurador (No
   *        pode ser nulo).
   * @param configuratorView O configurador do algoritmo a ser executado.
   * @param project O projeto.
   */
  public CommandExecutionDialog(JFrame parentFrame,
    AlgorithmConfiguratorView configuratorView, CommonClientProject project) {
    super(parentFrame);
    listeners = new LinkedList<CommandRequestedListener>();
    this.project = project;
    this.configuratorView = configuratorView;
    this.configurator = this.configuratorView.getConfigurator();

    Configuration configuration;
    try {
      configuration =
        ConfigurationManager.getInstance().getConfiguration(this.getClass());
      this.enablePriority =
        configuration.getOptionalBooleanProperty(ENABLE_PRIORITY, false);
    }
    catch (ConfigurationManagerException e) {
      this.enablePriority = false;
    }
    createGui();
    createSGASelectionPanel();
    populatePanel();
    setCommandDescription(configurator.getCommandDescription());
  }

  /**
   * Adiciona um listener ao dilogo.
   * 
   * @param listener
   */
  public void addCommandRequestedListener(CommandRequestedListener listener) {
    if (listener == null) {
      throw new IllegalArgumentException("O parmetro listener est nulo.");
    }
    listeners.add(listener);
  }

  /**
   * Remove um listener do dilogo.
   * 
   * @param listener
   * @return true se aquele listener estava cadastrado.
   */
  public boolean removeCommandRequestedListener(
    CommandRequestedListener listener) {
    if (listener == null) {
      throw new IllegalArgumentException("O parmetro listener est nulo.");
    }
    return listeners.remove(listener);
  }

  /**
   * Cria a interface grfica do configurador de seleo.
   */
  private void createGui() {
    this.executeButton =
      new JButton(LNG.get("CommandExecutionDialog.button.execute"));
    this.executeButton.setEnabled(false);
    this.executeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ev) {
        execute();
        close();
      }
    });
    this.closeButton =
      new JButton(LNG.get("CommandExecutionDialog.button.close"));
    this.closeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ev) {
        close();
      }
    });
    this.buttonPanel = new JPanel(new FlowLayout());
    this.buttonPanel.add(this.executeButton);
    this.buttonPanel.add(this.closeButton);
    ClientUtilities.adjustEqualSizes(executeButton, closeButton);
    this.commandDescriptionLabel =
      new JLabel(LNG.get("CommandExecutionDialog.label.command_description"));
    this.commandDescriptionTextField = new JTextField();
    if (this.enablePriority) {
      this.priorityLabel =
        new JLabel(LNG.get("CommandExecutionDialog.label.priority"));
      String[] priorityArray = new String[Priority.values().length];
      for (Priority p : Priority.values()) {
        priorityArray[p.ordinal()] =
          LNG.get("CommandExecutionDialog.combobox." + p.name());
      }
      this.priorityComboBox = new JComboBox(priorityArray);
      this.priorityComboBox.setSelectedIndex(Priority.getDefault().ordinal());
    }
    this.emailCheckBox =
      new JCheckBox(LNG.get("CommandExecutionDialog.checkbox.email"));
    ValidationResult validation = this.configuratorView.validate(false);
    if (!validation.isWellSucceded()) {
      this.parameterStatusLabel =
        new JLabel(validation.toString(),
        ApplicationImages.ICON_ERROR_16, SwingConstants.LEFT);
    }
    else if (this.configurator instanceof FlowAlgorithmConfigurator) {
      FlowAlgorithmConfigurator flowConfigurator =
        (FlowAlgorithmConfigurator) this.configurator;
      if (flowConfigurator.hasBypassedNodes()) {
        if (!flowConfigurator.canBeExecuted()) {
          this.parameterStatusLabel =
            new JLabel(LNG
            .get("CommandExecutionDialog.label.bypass_not_possible_status"),
            ApplicationImages.ICON_ERROR_16, SwingConstants.LEFT);
        }
        else {
          this.parameterStatusLabel =
            new JLabel(LNG
            .get("CommandExecutionDialog.label.bypassed_nodes_status"),
            ApplicationImages.ICON_INFORMATION_16, SwingConstants.LEFT);
        }
      }
    }
    else if (this.configurator.isSetDefaultValues()) {
      this.parameterStatusLabel =
        new JLabel(LNG
        .get("CommandExecutionDialog.label.default_parameter_status"),
        ApplicationImages.ICON_INFORMATION_16, SwingConstants.LEFT);
    }

    if (this.parameterStatusLabel == null) {
      this.parameterStatusLabel = new JLabel();
    }

    this.sgaStatusLabel = new JLabel();

    addWindowListener(new WindowAdapter() {
      @Override
      public void windowClosing(WindowEvent e) {
        close();
      }
    });
    setModal(false);
  }

  /**
   * Executa o comando.
   */
  private void execute() {
    CommandSubmission commandSubmission;
    commandSubmission =
      new CommandSubmission(this.configurator, project.getId());

    boolean mailAtEnd = this.emailCheckBox.isSelected();
    commandSubmission.setMailAtEnd(mailAtEnd);

    String commandDescription = this.commandDescriptionTextField.getText();
    commandSubmission.setDescription(commandDescription);

    String platformFilter = this.sgaPanel.getPlatformFilter();
    commandSubmission.setPlatform(platformFilter);

    Priority priority = getSelectedPriority();
    commandSubmission.setPriority(priority);

    ExecutionType type = this.configurator.getExecutionType();
    switch (type) {
      case SIMPLE:
        configureSimpleCommand(commandSubmission);
        break;
      case PARALLEL:
        configureParallelCommand(commandSubmission);
        break;
      case MULTIPLE:
        configureMultipleCommand(commandSubmission);
        break;
      case DISTRIBUTED:
        configureDistributedCommand(commandSubmission);
        break;
      case XAPPS:
        configureXAppsCommand(commandSubmission);
        break;
      default:
        throw new IllegalArgumentException(String.format(
          "Tipo de execuo %s desconhecido.", type));
    }
    submit(commandSubmission);
  }

  /**
   * Retorna a prioridade de execuo do algoritmo
   * 
   * @return a prioridade do algoritmo
   */
  private Priority getSelectedPriority() {
    /** Prioridade de execuo do algoritmo. */
    Priority priority = Priority.getDefault();
    if (this.enablePriority) {
      for (Priority p : Priority.values()) {
        if (p.ordinal() == this.priorityComboBox.getSelectedIndex()) {
          priority = p;
          break;
        }
      }
    }
    return priority;
  }

  /**
   * Atualiza o boto Executar.
   */
  private void updateCanExecute() {
    ValidationResult validation = this.configuratorView.validate(false);
    if (validation.isWellSucceded()
      && this.sgaPanel.isSelectionReady()) {
      this.executeButton.setEnabled(true);
    }
    else {
      this.executeButton.setEnabled(false);
    }

    if (this.sgaPanel.hasAvailableServers()) {
      this.sgaStatusLabel.setText(" ");
      this.sgaStatusLabel.setIcon(null);
    }
    else {
      final String text =
        LNG.get("CommandExecutionDialog.label.no_sga_available");
      this.sgaStatusLabel.setText(text);
      this.sgaStatusLabel.setIcon(ApplicationImages.ICON_ERROR_16);
    }

  }

  /**
   * Coloca todos os widgets na janela do configurador de seleo.
   */
  private void populatePanel() {
    Container contentPane = getContentPane();
    contentPane.removeAll();
    JPanel selectionPanel = new JPanel();
    selectionPanel.setLayout(new GridBagLayout());
    GridBagConstraints c = new GridBagConstraints();
    c.anchor = GridBagConstraints.WEST;
    c.gridx = 1;
    c.gridy = 1;
    c.gridwidth = 2;
    c.insets = new Insets(2, 2, 2, 2);
    c.weightx = 1.0;
    c.weighty = 0.0;
    c.fill = GridBagConstraints.HORIZONTAL;
    selectionPanel.add(this.sgaPanel, c);
    c.gridy++;
    c.gridwidth = 1;
    c.weightx = 0;
    c.fill = GridBagConstraints.NONE;
    selectionPanel.add(this.commandDescriptionLabel, c);
    c.gridx++;
    c.fill = GridBagConstraints.HORIZONTAL;
    c.weightx = 1;
    selectionPanel.add(this.commandDescriptionTextField, c);
    if (this.enablePriority) {
      c.gridx = 1;
      c.gridy++;
      c.gridwidth = 1;
      c.weightx = 0;
      c.fill = GridBagConstraints.NONE;
      selectionPanel.add(this.priorityLabel, c);
      c.gridx++;
      c.weightx = 1;
      selectionPanel.add(this.priorityComboBox, c);
    }
    c.gridy++;
    c.gridx = 1;
    c.gridwidth = 2;
    selectionPanel.add(this.emailCheckBox, c);
    c.gridy++;
    selectionPanel.add(this.parameterStatusLabel, c);
    c.gridy++;
    selectionPanel.add(this.sgaStatusLabel, c);

    contentPane.add(selectionPanel, BorderLayout.CENTER);
    contentPane.add(this.buttonPanel, BorderLayout.SOUTH);
    contentPane.validate();
    getRootPane().setDefaultButton(this.executeButton);
    validate();
    pack();
    Dimension currentSize = getSize();
    int currentWidth = (int) currentSize.getWidth();
    int currentHeight = (int) currentSize.getHeight();
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    int screenWidth = (int) screenSize.getWidth();
    int screenHeight = (int) screenSize.getHeight();
    int newWidth = Math.min(currentWidth, screenWidth);
    int newHeight = Math.min(currentHeight, screenHeight);
    setSize(newWidth, newHeight);
    validate();
    center(getOwner());
  }

  /**
   * Cria o painel de seleo de SGAs.
   * 
   * @throws IllegalStateException Tipo de execuo no esperado.
   */
  private void createSGASelectionPanel() {
    ExecutionType type = this.configurator.getExecutionType();
    switch (type) {
      case SIMPLE:
        sgaPanel = new SGASingleSelectionPanel(configuratorView);
        setTitle(LNG.get("CommandExecutionDialog.title.simple"));
        break;
      case XAPPS:
        sgaPanel = new SGASingleSelectionPanel(configuratorView);
        setTitle(LNG.get("CommandExecutionDialog.title.xapps"));
        break;
      case MULTIPLE:
        sgaPanel = new SGAMultipleSelectionPanel(configuratorView);
        setTitle(LNG.get("CommandExecutionDialog.title.multiple"));
        break;
      case DISTRIBUTED:
        sgaPanel = new SGAMultipleSelectionPanel(configuratorView);
        setTitle(LNG.get("CommandExecutionDialog.title.distributed"));
        break;
      case PARALLEL:
        sgaPanel = new SGAParallelSelectionPanel(configuratorView);
        setTitle(LNG.get("CommandExecutionDialog.title.parallel"));
        break;
      default:
        throw new IllegalStateException("Tipo de execuo desconhecido: "
          + configuratorView.getConfigurator().getExecutionType());
    }
    this.sgaPanel.populateGUIComponents();
    this.sgaPanel.addSGASelectionListener(new SGASelectionListener() {
      public void wasChangedPlatform() {
      }

      public void wasChangedServer() {
        updateCanExecute();
      }
    });
    this.updateCanExecute();
  }

  /**
   * Configura a submisso do comando (execuo simples).
   * 
   * @param commandSubmission informao sobre a submisso do comando.
   */
  private void configureSimpleCommand(CommandSubmission commandSubmission) {
    Collection<SGASet> serverCollection =
      this.sgaPanel.getSelectedServerCollection();
    if (serverCollection != null) {
      // Seleo manual
      SGASet sgaServer = serverCollection.iterator().next();
      commandSubmission.configureSimpleExecution(sgaServer.getName());
    }
    else {
      commandSubmission.configureSimpleExecution();
    }
  }

  /**
   * Configura a submisso do comando (execuo aplicao X).
   * 
   * @param commandSubmission informao sobre a submisso do comando.
   */
  private void configureXAppsCommand(CommandSubmission commandSubmission) {
    Collection<SGASet> serverCollection =
      this.sgaPanel.getSelectedServerCollection();

    if (serverCollection != null) {
      // Seleo manual
      SGASet sgaSet = serverCollection.iterator().next();
      commandSubmission.configureXAppExecution(sgaSet.getName());
    }
    else {
      commandSubmission.configureXAppExecution();
    }
  }

  /**
   * Configura a submisso do comando (execuo paralela).
   * 
   * @param commandSubmission informao sobre a submisso do comando.
   */
  private void configureParallelCommand(CommandSubmission commandSubmission) {
    Integer cpuCount =
      ((SGAParallelSelectionPanel) this.sgaPanel)
      .getCpuCountForParallelExecution();
    SGASet sgaServer = this.sgaPanel.getSelectedServer();
    if (sgaServer != null) {
      // Seleo manual
      commandSubmission.configureParallelExecution(sgaServer.getName(),
        cpuCount);
    }
    else {
      commandSubmission.configureParallelExecution(cpuCount);
    }
  }

  /**
   * Configura a submisso do comando (execuo mltipla).
   * 
   * @param commandSubmission informao sobre a submisso do comando.
   */
  private void configureMultipleCommand(CommandSubmission commandSubmission) {
    Collection<SGASet> serverCollection =
      this.sgaPanel.getSelectedServerCollection();
    //  Seleo automtica de servidores.
    if (serverCollection == null) {
      Integer executionCount =
        ((SGAMultipleSelectionPanel) this.sgaPanel)
        .getExecutionCountForMultipleExecution();
      commandSubmission.configureMultipleExecution(executionCount);
    }
    else {
      // Seleo manual de servidores.
      List<String> serverList = new LinkedList<String>();
      for (SGASet sgaSet : serverCollection) {
        serverList.add(sgaSet.getName());
      }
      Integer executionCountPerSGA =
        ((SGAMultipleSelectionPanel) this.sgaPanel)
        .getExecutionCountPerSGAForMultipleExecution();
      commandSubmission.configureMultipleExecution(serverList,
        executionCountPerSGA);
    }
  }

  /**
   * Configura a submisso do comando (execuo distribuda).
   * 
   * @param commandSubmission informao sobre a submisso do comando.
   */
  private void configureDistributedCommand(CommandSubmission commandSubmission) {
    Collection<SGASet> serverCollection =
      this.sgaPanel.getSelectedServerCollection();
    //  Seleo automtica de servidores.
    if (serverCollection == null) {
      Integer sgaCount =
        ((SGAMultipleSelectionPanel) this.sgaPanel)
        .getSgaCountForDistributedExecution();
      commandSubmission.configureDistributedExecution(sgaCount);
    }
    else {
      // Seleo manual de servidores.
      List<String> serverList = new LinkedList<String>();
      for (SGASet sgaSet : serverCollection) {
        serverList.add(sgaSet.getName());
      }
      Integer executionCountPerSGA =
        ((SGAMultipleSelectionPanel) this.sgaPanel)
        .getExecutionCountPerSGAForDistributedExecution();
      commandSubmission.configureDistributedExecution(serverList,
        executionCountPerSGA);
    }
  }

  /**
   * Submete um algoritmo para execuo no escalonador.
   * 
   * @param commandSubmission informaes sobre a submisso do comando.
   */
  private void submit(CommandSubmission commandSubmission) {
    Set<CommandInfo> submittedCommands =
      SchedulerProxy.submitCommand(this, commandSubmission);
    if (submittedCommands != null) {
      for (CommandRequestedListener listener : listeners) {
        listener.commandsWereRequested(submittedCommands);
      }
    }
  }

  /**
   * Estabelece um valor default para a descrio do comando que vai ser
   * executado.
   * 
   * @param description descrio do comando
   */
  public void setCommandDescription(String description) {
    commandDescriptionTextField.setText(description);
  }
}
