/*
 * SGASelection.java
 */
package csbase.client.util.sga;

import java.awt.event.ActionListener;
import java.util.*;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JComponent;

import tecgraf.javautils.gui.selector.ContainerSelectionListener;

/**
 * A classe <code>SGASelection</code> modela um componente que pode representar
 * uma lista de seleo simples de sgad ou que pode apresentar duas listas de
 * sgas para seleo mltipla. Este componente deve ser usado em qualquer
 * dilogo que se deseje selecionar sgas.
 */
public class SGASelection {
  /** Componente de seleo simples dos sgas. */
  private JComboBox singleItem = null;

  /** Componente de seleo multipla dos sgas. */
  private SGAContainerSelection multipleItems = null;

  /** Indica se o container faz seleo mltipla. */
  private boolean multiple;

  /**
   * Cria o painel com o componente que permite selecionar os sgas.
   *
   * @return Panel do container.
   */
  public JComponent getPanel() {
    if (multiple) {
      return multipleItems.getPanel();
    }
    else {
      return singleItem;
    }
  }

  /**
   * Carrega o conjunto de elementos disponveis com os sgas cadastrados.
   *
   * @param availableSGAs .
   */
  public void loadItems(Vector<String> availableSGAs) {
    if (multiple) {
      if (availableSGAs == null) {
        availableSGAs = new Vector<String>();
      }
      multipleItems.loadItems(availableSGAs, new Vector<String>());
      multipleItems.adjustTableColumns();
    }
    else {
      Collections.sort(availableSGAs);
      DefaultComboBoxModel sgaModel = new DefaultComboBoxModel();
      for (int i = 0; i < availableSGAs.size(); i++) {
        sgaModel.addElement(availableSGAs.get(i));
      }
      singleItem.setModel(sgaModel);
    }
  }

  /**
   * Indica se o container deve ser habilitado ou no.
   *
   * @param enabled .
   */
  public void setEnabled(boolean enabled) {
    if (multiple) {
      multipleItems.setEnabled(enabled);
    }
    else {
      singleItem.setEnabled(enabled);
    }
  }

  /**
   * Retorna se existe algum sga na lista.
   *
   * @return true se existir algum sga na lista, false caso contrrio.
   */
  public boolean hasAnyItem() {
    if (multiple) {
      return ((multipleItems.getSelectedItems().size() +
      multipleItems.getAvailableItems().size()) > 0);
    }
    else {
      return (singleItem.getItemCount() > 0);
    }
  }

  /**
   * Retorna os sgas selectionados.
   *
   * @return lista de sgas selecionados.
   */
  public List<String> getSelectedItems() {
    if (multiple) {
      return multipleItems.getSelectedItems();
    }
    else {
      String item = (String) singleItem.getSelectedItem();
      List<String> items = new ArrayList<String>();
      items.add(item);
      return items;
    }
  }

  /**
   * Retorna um sga selectionado.
   *
   * @return sga selecionado.
   */
  public String getSelectedItem() {
    if (multiple) {
      return multipleItems.getSelectedItems().get(0);
    }
    else {
      return (String) singleItem.getSelectedItem();
    }
  }

  /**
   * Atribui o sga selecionado apenas quando e' uma selecao simples.
   *
   * @param item sga que deve ser colocado como selecionado no comboBox.
   */
  public void setSelectedItem(Object item) {
    if (!multiple) {
      singleItem.setSelectedItem(item);
    }
  }

  public void clearSelectedItems() {
    if (multiple) {
      multipleItems.clearSelectedItems();
    }
    else {
      if (singleItem.getModel().getSize() != 0) {
        singleItem.setSelectedIndex(0);
      }
    }
  }

  /**
   * Adiciona um listener de modificao do container de seleo. Esse listener
   *  notificado sempre que algum elemento  colocado ou removido do
   * container com os elementos selecionados.
   *
   * @param l O listener de alterao do contedo do container.
   */
  public void addSGASelectionListener(Object l) {
    if (multiple) {
      multipleItems.addContainerSelectionListener(
        (ContainerSelectionListener)l);
    }
    else {
      singleItem.addActionListener((ActionListener)l);
    }
  }

  /**
   * Construtor
   *
   * @param multiple Indica se o container deve permitir seleo mltipla.
   * @param availableSGAs SGAs disponveis.
   */
  public SGASelection(boolean multiple, Vector<String> availableSGAs) {
    this.multiple = multiple;
    if (multiple) {
      if (availableSGAs == null) {
        availableSGAs = new Vector<String>();
      }

      /* Cria o componente que mostra os dois conjuntos de elementos:
       * um com os disponveis e o outro com os escolhidos.  */
      multipleItems = new SGAContainerSelection();
      multipleItems.loadItems(availableSGAs, new Vector<String>());
    }
    else {
      /* Cria o componente que mostra uma lista de sga disponveis. */
      singleItem = new JComboBox();
      this.loadItems(availableSGAs);
    }
  }
}
