/*
 * $Author: ururahy $
 * $Date: 2009-01-27 15:07:33 -0200 (Tue, 27 Jan 2009) $
 * $Release: $
 */
package csbase.client.util.sga;

import csbase.client.algorithms.AlgorithmConfiguratorView;
import csbase.client.kernel.ClientException;
import csbase.client.remote.srvproxies.SGAProxy;
import csbase.logic.SGASet;
import csbase.logic.algorithms.AlgorithmConfigurator;

import tecgraf.javautils.LNG;
import tecgraf.javautils.configurationmanager.*;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.border.TitledBorder;

/**
 * Define um painel para seleo de SGAs.
 * 
 * @author ururahy
 *
 */
public abstract class SGASelectionPanel extends JPanel {
  private List<SGASelectionListener> listenerList;
  protected Set<String> platformSet;
  protected Set<String> requirements;
  protected JPanel platformFilterPanel;
  protected JLabel platformFilterLabel;
  protected JComboBox platformFilterComboBox;
  protected String allPlatformsText;
  protected JRadioButton manualServerRadioButton;
  protected JRadioButton automaticServerRadioButton;
  private ButtonGroup serverSelectionButtonGroup;
  protected AlgorithmConfiguratorView configuratorView;
  private TitledBorder serverBorder;
  
  /**
   * Nome da propriedade que indica a escolha automtica de sgas como a opo
   * default.
   */
  private static final String AUTOMATIC_SELECTION = "automatic.selection";

  protected SGASelectionPanel(AlgorithmConfiguratorView configuratorView) {
    AlgorithmConfigurator configurator = configuratorView.getConfigurator();
    this.configuratorView = configuratorView;
    this.allPlatformsText = LNG.get("SGASelectionPanel.label.all_platforms");
    this.platformSet = new HashSet<String>();
    this.platformSet.addAll(configurator.getPlatforms());
    this.requirements = new HashSet<String>();
    this.requirements.addAll(configurator.getRequirements());
    this.listenerList = new LinkedList<SGASelectionListener>();
    createGUIComponents();
  }

  /**
   * Obtm a coleo de servidores escolhidos manualmente pelo usurio.
   *
   * @return A coleo de servidores selecionada pelo usurio.
   */
  abstract Collection<SGASet> getManualServerCollection();

  /**
   * Obtm o servidor selecionado (em modo seleo simples e paralela) ou
   * o  primeiro servidor selecionado (em modo seleo mltipla).
   *
   * @return O SGASet do servidor selecionado.
   */
  abstract SGASet getManualSelectedServer();

  /**
   * Indica se existe algum sga disponvel para execuo.
   * 
   * @return verdadeiro se existe algum sga disponvel para execuo ou falso
   *         caso contrrio.
   */
  abstract boolean hasAvailableServers();
  
  /**
   * Habilita/Desabilita componentes da interface grfica com o usurio
   * de  acordo com as interaes do usurio com essa interface. Por exemplo,
   * se  a seleo automtica de servidores for habilitada, a caixa de seleo
   * manual de SGAs deve ser desabilitada.
   */
  abstract void updateEnabledServers();

  /**
   * Recarrega o combo box dos SGAs, filtrados pela plataforma
   * selecionada.
   */
  abstract void reloadSGAs();

  /**
   * Sinaliza se a seleo de servidores est completa. Se a resposta for
   * afirmativa e a configurao de parmetros do algoritmo selecionado tambm
   * estiver sido concluda, a execuo do algoritmo est pronta para
   * ocorrer.
   *
   * @return Se a seleo de servidores est completa ou no.
   */
  public abstract boolean isSelectionReady();

  /**
   * Popula o painel de seleo de sgas. Cada tipo de algoritmo (simple,
   * multiple e parallel), possui um tipo de painel.
   */
  public abstract void populateGUIComponents();

  /**
   * Adiciona um listener a este painel.
   *
   * @param listener O listener.
   *
   * @throws IllegalArgumentException Se o parmetro listener estiver nulo.
   */
  public void addSGASelectionListener(SGASelectionListener listener) {
    if (listener == null) {
      throw new IllegalArgumentException("O parmetro listener est nulo.");
    }
    this.listenerList.add(listener);
  }

  /**
   * Obtm a plataforma selecionada.
   *
   * @return A plataforma selecionada.
   */
  public String getSelectedPlatform() {
    return (String) this.platformFilterComboBox.getSelectedItem();
  }

  /**
   * <p>Obtm uma coleo no-modificvel de servidores selecionados, se
   * o painel estiver em modo manual.</p>
   *  <p>Os objetos desta coleo so do tipo {@link SGASet}.</p>
   *
   * @return A coleo de servidores. Retornar uma coleo vazio se no
   *         houverem elementos selecionados.
   *
   * @see java.util.Collections#unmodifiableCollection(java.util.Collection)
   */
  public Collection<SGASet> getSelectedServerCollection() {
    if (isManualServerSelection()) {
      return this.getManualServerCollection();
    }
    return null;
  }

  /**
   * Obtm o servidor selecionado (em modo seleo simples e paralela) ou
   * o  primeiro servidor selecionado (em modo seleo mltipla), se o painel
   * estiver em modo manual.
   *
   * @return O servidor selecionado.
   */
  public SGASet getSelectedServer() {
    return this.getManualSelectedServer();
  }

  /**
   * Carrega o combo box dos SGAs, filtrados pela plataforma
   * selecionada.
   *
   * @return .
   *
   * @throws ClientException Em caso de erro.
   */
  protected Vector<SGASet> loadServers () throws ClientException {
    Map<String, SGASet> sgaSetMap = new HashMap<String, SGASet>();
    Collection<String> sgaNames = SGAProxy.getAllSGANames();
    if (sgaNames == null) {
      throw new ClientException(
        LNG.get("csbase.client.util.sga.exception.load_sgas"));
    }
    for (String sgaName : sgaNames) {
      SGASet sgaSet = SGAProxy.getSGASet(sgaName);
      sgaSetMap.put(sgaName, sgaSet);
    }
    Vector<String> selectedPlataforms = new Vector<String>();
    String selectedPlatform = getPlatformFilter();
    if (selectedPlatform == null) {
      selectedPlataforms.addAll(this.platformSet);
    }
    else {
      selectedPlataforms.add(selectedPlatform);
    }
    Vector<SGASet> validSGAs = new Vector<SGASet>();
    for (String sgaName : sgaSetMap.keySet()) {
      SGASet sgaSet = sgaSetMap.get(sgaName);
      if (
        sgaSet != null &&
        sgaSet.isPlatformSupported(selectedPlataforms) &&
        sgaSet.hasRequirements(this.requirements) &&
        sgaSet.mayExecuteCommand() && 
        // No caso de execuo paralela, restringe os sgas a clusters.
        (! this.configuratorView.getConfigurator().isParallelExecution() ||
         sgaSet.isCluster())) {
        validSGAs.add(sgaSet);
      }
    }
    Collections.sort(validSGAs);
    return validSGAs;
  }

  /**
   * Remove o listener da lista de observadores deste painel.
   *
   * @param listener O listener.
   *
   * @throws IllegalArgumentException Se o parmetro listener estiver nulo.
   */
  public void removeSGASelectionListener(SGASelectionListener listener) {
    if (listener == null) {
      throw new IllegalArgumentException("O parmetro listener est nulo.");
    }
    this.listenerList.remove(listener);
  }

  /**
   * Verifica se a seleo de servidores  manual.
   *
   * @return .
   */
  public boolean isManualServerSelection() {
    return this.manualServerRadioButton.isEnabled() &&
    this.manualServerRadioButton.isSelected();
  }

  /**
   * Este mtodo retorna o filtro de plataformas selecionado ou null
   * caso no haja nenhuma restrio selecionada..
   *
   * @return O filtro de plataformas selecionado.
   */
  public String getPlatformFilter() {
    String selectedPlatform = getSelectedPlatform();
    if (selectedPlatform.equals(allPlatformsText)) {
      return null;
    }
    return selectedPlatform;
  }

  /**
   * Cria os componentes da GUI.
   */
  private void createGUIComponents() {
    this.serverBorder = new TitledBorder(
        BorderFactory.createEtchedBorder(),
        LNG.get("SGASelectionPanel.label.server"));
    this.platformFilterLabel = 
      new JLabel(LNG.get("SGASelectionPanel.label.platform_filter"));
    this.platformFilterComboBox = makePlatformFilterComboBox();
    createPlatformFilterPanel();
    boolean automaticSelection = false;
    try {
      Configuration cnf =
        ConfigurationManager.getInstance().getConfiguration(this.getClass());
      automaticSelection = cnf.getOptionalBooleanProperty(AUTOMATIC_SELECTION, false);
    }
    catch (ConfigurationManagerException e) {
      automaticSelection = false;
    }
      
    this.manualServerRadioButton = new JRadioButton(
        LNG.get("SGASelectionPanel.radiobutton.manual_server"));
    this.manualServerRadioButton.setSelected(!automaticSelection);
    this.manualServerRadioButton.addItemListener(
      new ManualServerRadioButtonItemListener());
    this.manualServerRadioButton.setEnabled(false);
    this.automaticServerRadioButton = new JRadioButton(
        LNG.get("SGASelectionPanel.radiobutton.automatic_server"));
    this.automaticServerRadioButton.setSelected(automaticSelection);
    this.automaticServerRadioButton.addItemListener(
      new ManualServerRadioButtonItemListener());
    this.automaticServerRadioButton.setEnabled(false);
    this.serverSelectionButtonGroup = new ButtonGroup();
    this.serverSelectionButtonGroup.add(this.manualServerRadioButton);
    this.serverSelectionButtonGroup.add(this.automaticServerRadioButton);
    this.setBorder(this.serverBorder);
  }

  /**
   * Cria um painel para a insero do filtro de plataformas.
   */
  private void createPlatformFilterPanel() {
    this.platformFilterPanel = new JPanel();
    this.platformFilterPanel.setLayout(new GridBagLayout());
    GridBagConstraints c = new GridBagConstraints();
    c.anchor = GridBagConstraints.WEST;
    c.insets = new Insets(2, 2, 2, 2);
    c.gridx = 1;
    c.gridy = 1;
    c.gridwidth = 1;
    c.gridheight = 1;
    c.weightx = 0.0;
    c.weighty = 0.0;
    c.fill = GridBagConstraints.NONE;
    this.platformFilterPanel.add(this.platformFilterLabel, c);
    c.weightx = 1.0;
    c.gridx++;
    this.platformFilterPanel.add(this.platformFilterComboBox, c);
  }

  /**
   * Notifica os listeners que houve alterao na lista de servidores.
   */
  protected void fireChangedServer() {
    Iterator<SGASelectionListener> listenerIterator =
      this.listenerList.iterator();
    while (listenerIterator.hasNext()) {
      SGASelectionListener listener = listenerIterator.next();
      listener.wasChangedServer();
    }
  }

  /**
   * Cria um componente do tipo JComboBox para a seleo de plataformas.
   * 
   * @return O componente JComboBox criado e populado com as plataformas
   *         disponveis.
   */
  private JComboBox makePlatformFilterComboBox() {
    JComboBox combo = null;
    Vector<String> allPlatforms = new Vector<String>(platformSet);
    Collections.sort(allPlatforms);
    allPlatforms.add(0, allPlatformsText);
    combo = new JComboBox(allPlatforms);
    combo.setSelectedItem(allPlatformsText);
    combo.addItemListener(
      new ItemListener() {
        public void itemStateChanged(ItemEvent e) {
          reloadSGAs();
          fireChangedPlatform();
        }
      });
    return combo;
  }

  /**
   * Notifica os listeners que houve alterao na seleo de plataformas.
   */
  protected void fireChangedPlatform() {
    Iterator<SGASelectionListener> listenerIterator =
      this.listenerList.iterator();
    while (listenerIterator.hasNext()) {
      SGASelectionListener listener = listenerIterator.next();
      listener.wasChangedPlatform();
    }
  }

  private final class ManualServerRadioButtonItemListener
  implements ItemListener {
    /**
     * 
     * @see ItemListener#itemStateChanged(ItemEvent)
     */
    public void itemStateChanged(ItemEvent e) {
      updateEnabledServers();
      fireChangedServer();
    }
  }
}
