/*
 * $Id: ClientProjectFile.java 92186 2009-06-05 19:34:07Z ururahy $
 */

package csbase.logic;

import java.io.File;
import java.io.IOException;
import java.io.Serializable;
import java.nio.ByteBuffer;
import java.rmi.RemoteException;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Stack;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import csbase.exception.project.FileLockedException;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.ProjectFileChannel;
import csbase.remote.ProjectServiceInterface;
import csbase.server.services.projectservice.ProjectService;

/**
 * Modela a viso que um cliente tem de um arquivo ou diretrio de um projeto.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class ClientProjectFile implements ProjectFile, Serializable, Comparable {
  
  /**
   * Esta string substitui o acesso da classe da logica ao language
   */
  private static final String PRJ_FILE_INVALID_STATE =
    "Estado do arquivo incomptivel com a operao solicitada";

  /**
   * Rendering name de arquivo-pai.
   */
  private static final String PARENT_DIR_RENDERING_NAME = "../";

  /**
   * Path
   */
  private final String absolutePath;

  /**
   * Canal
   */
  private ProjectFileChannel channel;

  /**
   * Id do usurio que criou o arquivo.
   */
  private final Object createdBy;
  /**
   * Data de criao.
   */
  private final long creationDate;
  /**
   * Indicativo de diretrio
   */
  private boolean isDirectory;
  /**
   * Indicativo de arquivo global
   */
  private boolean isGlobalFile;

  /**
   * Indicativo de "publicado"
   */
  private boolean isPublishedFile;

  /**
   * Indicativo de "em construo"
   */
  private boolean isUnderConstruction;

  /**
   * Indica se o arquivo est bloqueado
   */
  private boolean isLocked;

  /**
   * Indica se arquivo est sendo movido
   */
  private boolean isMoving;

  /**
   * Data de modificao.
   */
  private long modificationDate;

  /**
   * Nome
   */
  private String name;

  /**
   * Nmero de canais abertos
   */
  private int numberOfChannelOpens;

  /**
   * Pai (diretrio)
   */
  private ClientProjectFile parent;

  /**
   * Identirficador do projeto.
   */
  private final Object projectId;

  /**
   * Lista de arquivos-filho
   */
  private List<ClientProjectFile> childList = null;
  /**
   * Lock utilizado para manter o sincronismo de threads no acesso ao childList.
   */
  private Lock childrenLock = new ReentrantLock();

  /** Indica se o arquivo est atualizado em relao ao servidor. */
  private boolean updated;

  /** Indica se o arquivo  atualizvel. */
  private final boolean updatable;

  /**
   * Indica o intervalo pelo qual o arquivo (somentes os atualizveis) est
   * sendo atualizado.
   * <p>
   * OBS: Para arquivos no atualizveis este valor  sempre 0 (zero).
   */
  private int updateInterval;

  /**
   * Nome para renderizao, caso seja til exibir um nome diferente para o
   * arquivo (como "../"). Utilizado pelo <code>ProjectFileCellRenderer</code>.
   */

  private String renderingName;
  /**
   * Tamanho
   */
  private long size;

  /**
   * Estado
   */
  private int state;

  /**
   * Tipo
   */
  private String type;

  /**
   * Identificador de que projeto publicou
   */
  private Object whichProjectPublishedId;

  /**
   * Identificador do usurio que publicou
   */
  private Object whoPublishedId;

  /**
   * Retorna o nome do arquivo.
   * 
   * @return Nome do arquivo.
   */
  public String getName() {
    return this.name;
  }

  /**
   * Muda o nome do arquivo.
   * 
   * @param name novo nome do arquivo.
   */
  public void setName(String name) {
    this.name = name;
    this.renderingName = name;
  }

  /**
   * Muda o tamanho do arquivo.
   * 
   * @param size novo tamanho do arquivo.
   */
  public void setSize(long size) {
    this.size = size;
  }

  /**
   * Muda a data de modificao do arquivo.
   * 
   * @param modificationDate nova data de modificao do arquivo.
   */
  public void setModificationDate(long modificationDate) {
    this.modificationDate = modificationDate;
  }

  /**
   * {@inheritDoc}
   */
  public Object getProjectId() {
    return this.projectId;
  }

  /**
   * Obtm uma string que representa o arquivo no projeto.
   * 
   * @return o path
   */
  public String getStringPath() {
    String[] pathArray = getSimplePath();
    if ((pathArray == null) || (pathArray.length <= 0)) {
      return "";
    }
    String path = "";
    for (int i = 0; i < (pathArray.length - 1); i++) {
      path = path + pathArray[i] + "/";
    }
    return path + pathArray[pathArray.length - 1];
  }

  /**
   * Retorna o caminho simplificado para o arquivo (sem o nome do projeto). O
   * array retornado  uma cpia do original.
   * 
   * @return Caminho at o arquivo, inclusive. No inclui o nome do projeto.
   * @see #getPath()
   */
  public String[] getSimplePath() {
    String[] path = getPath();
    String[] simplePath = new String[path.length - 1];
    for (int i = 1; i < path.length; i++) {
      simplePath[i - 1] = path[i];
    }
    return simplePath;
  }

  /**
   * Retorna o caminho completo para o arquivo (com o nome do projeto). O array
   * retornado  uma cpia do original.
   * 
   * @return Caminho at o arquivo, inclusive. Inclui o nome do projeto.
   * @see #getSimplePath()
   */
  public String[] getPath() {
    return ClientProjectFile.getPath(this);
  }

  /**
   * Retorna o caminho completo de um arquivo.
   * 
   * @param file o arquivo
   * @return o caminho at o arquivo, inclusive.
   */
  static public String[] getPath(final ClientProjectFile file) {
    Stack<String> pathStack = new Stack<String>();
    for (ClientProjectFile cpf = file; cpf != null; cpf = cpf.parent) {
      pathStack.push(cpf.name);
    }
    String[] pathArray = new String[pathStack.size()];
    for (int i = 0; i < pathArray.length; i++) {
      pathArray[i] = pathStack.pop();
    }
    return pathArray;
  }

  /**
   * Retorna o caminho para o arquivo. Esse caminho no  relativo, ou seja, o
   * caminho absoluto de onde o arquivo se encontra.
   * 
   * @return Caminho absoluto at o arquivo, inclusive.
   */
  public String getAbsolutePath() {
    return this.absolutePath;
  }

  /**
   * Informa o tipo do arquivo. Todos os tipos conhecidos pelo WebSintesi so
   * listados em {@link ProjectFileType}.
   * 
   * @return O tipo do arquivo, como uma constante de {@link ProjectFileType}.
   */
  public String getType() {
    return this.type;
  }

  /**
   * Informa o a data de criao arquivo.
   * 
   * @return a data de criao do arquivo em Milesegundos.
   */
  public long getCreationDate() {
    return this.creationDate;
  }

  /**
   * Retorna o usurio que publicou esse arquivo.
   * 
   * @return O nome do usurio que publicou esse arquivo.
   */
  public Object whoPublished() {
    return this.whoPublishedId;
  }

  /**
   * Retorna o identificador do projeto a partir do qual este arquivo foi
   * publicado.
   * 
   * @return O identificador do projeto pelo qual o arquivo foi publicado.
   */
  public Object whichProjectPublished() {
    return this.whichProjectPublishedId;
  }

  /**
   * Retorna o usurio que criou esse arquivo.
   * 
   * @return O nome do usurio que criou esse arquivo.
   */
  public Object whoCreated() {
    return this.createdBy;
  }

  /**
   * Muda o usurio que publicou o arquivo.
   * 
   * @param userId O nome do usurio que publicou o arquivo.
   */
  public void setWhoPublished(Object userId) {
    this.whoPublishedId = userId;
  }

  /**
   * Muda o projeto a partir do qual o arquivo foi publicado.
   * 
   * @param projectId Identificador do projeto que publicou o arquivo.
   */
  public void setWhichProjectPublished(Object projectId) {
    this.whichProjectPublishedId = projectId;
  }

  /**
   * Retorna o diretrio pai deste arquivo.<br>
   * Este mtodo no deve ser utilizado no servidor pois o valor da propriedade
   * a que ele referencia s ser atribuido no cliente.
   * 
   * @return O diretrio pai deste arquivo ou <code>null</code> caso este
   *         arquivo seja o diretrio raiz.
   */
  public ClientProjectFile getParent() {
    return this.parent;
  }

  /**
   * Reseta a lista de filhos.
   */
  public void resetChildren() {
    childList = null;
  }

  /**
   * Retorna a lista de arquivos deste diretrio, mas
   * no vai ao servidor buscar os filhos se estes ainda no tiverem sido
   * carregados.   
   * @return A lista de arquivos deste diretrio.
   */
  public ClientProjectFile[] getLocalChildren() {
    if (childList == null) {
      return new ClientProjectFile[0];
    }

    return createChildArray();
  }

  /**
   * {@inheritDoc}
   */
  public ClientProjectFile[] getChildren() throws RemoteException {
    buildChildListIfNull();

    return createChildArray();
  }

  /**
   * Monta a lista de filhos caso esta esteja zerada (null)
   * 
   * @throws RemoteException se houver falha de RMI
   */
  private void buildChildListIfNull() throws RemoteException {
    childrenLock.lock();
    try {
      if (null == childList) {
        childList = new ArrayList<ClientProjectFile>();
        // Se no for um diretrio no vai ao servidor buscar os filhos.
        if (!this.isDirectory()) {
          return;
        }
        final ProjectServiceInterface projectService =
          ClientRemoteLocator.projectService;
        final ClientProjectFile[] children = projectService.getChildren(this);
        if (children == null) {
          return;
        }
        for (ClientProjectFile child : children) {
          childList.add(child);
          /* Os filhos vm do servidor sem pai. */
          child.parent = this;
        }
      }
    }
    finally {
      childrenLock.unlock();
    }
  }

  /**
   * Cria um array com os filhos do arquivo de projeto atual.
   * 
   * @return O array com os filhos do arquivo de projeto atual. Caso no existam
   *         filhos, ser retornado um array de tamanho zero.
   */
  private ClientProjectFile[] createChildArray() {
    ClientProjectFile[] childArray =
      new ClientProjectFile[this.childList.size()];
    Iterator<ClientProjectFile> childIterator = this.childList.iterator();
    for (int i = 0; i < childArray.length; i++) {
      childArray[i] = childIterator.next();
    }
    return childArray;
  }

  /**
   * {@inheritDoc}
   */
  public ClientProjectFile[] getChildren(String fileType)
    throws RemoteException {
    ClientProjectFile[] children = getChildren();
    if (fileType == null) {
      return children;
    }
    List<ClientProjectFile> aux = new ArrayList<ClientProjectFile>();
    for (ClientProjectFile child : children) {
      if (child.isDirectory() || child.getType().equals(fileType)) {
        aux.add(child);
      }
    }
    ClientProjectFile[] result = new ClientProjectFile[aux.size()];
    for (int i = 0; i < aux.size(); i++) {
      result[i] = aux.get(i);
    }
    return result;
  }

  /**
   * Altera o conjunto de arquivos deste diretrio.
   * 
   * @param children O novo conujunto de arquivos.
   */
  public void setChildren(ClientProjectFile[] children) {
    this.childList = createList(children);
  }

  /**
   * Obtm um {@link ClientProjectFile arquivo} deste diretrio ou {@code null}
   * caso ele no seja encontrado ou este arquivo no represente um diretrio.
   * 
   * @param childName - path para o filho, a partir deste diretrio
   * 
   * @return o {@link ClientProjectFile filho} deste diretrio ou {@code null}
   *         caso ele no seja encontrado ou este arquivo no represente um
   *         diretrio.
   * 
   * @throws RemoteException se houver falha de RMI
   * 
   * @see csbase.logic.ProjectFile#getChild(java.lang.String)
   */
  public ClientProjectFile getChild(String childName) throws RemoteException {
    childrenLock.lock();
    try {
      buildChildListIfNull();
      /*
       * Utiliza o mtodo {@link #getChildHelper(String)} para buscar o arquivo.
       * Se este no for encontrado, considerando que haja uma inconsistncia
       * entre o cliente e o servidor, limpa a memria e tenta novamente.
       */
      ClientProjectFile child = getChildHelper(childName);
      if (null == child) {
        final ProjectServiceInterface projectService =
          ClientRemoteLocator.projectService;
        child = projectService.getChild(this, childName);
        if (child != null) {
          childList.add(child);
          /* O filho vm do servidor sem pai. */
          child.parent = this;
          Collections.sort(childList);
        }
      }

      return child;
    }
    finally {
      childrenLock.unlock();
    }
  }

  /**
   * <p>
   * Obtm um {@link ClientProjectFile arquivo} deste diretrio ou {@code null}
   * caso ele no seja encontrado ou este arquivo no represente um diretrio.
   * </p>
   * <p>
   * O que difere este mtodo do {@link #getChild(String)}  que apenas se este
   * diretrio nunca tiver carregado seus filhos, que ele ir procurar no
   * servidor. A partir desta carga este mtodo ir confiar no mtodo
   * {@link #addChild(ClientProjectFile)} para garantir a consistncia com o
   * servidor.
   * </p>
   * 
   * @param childName nome do filho
   * 
   * @return o {@link ClientProjectFile filho} deste diretrio ou {@code null}
   *         caso ele no seja encontrado ou este arquivo no represente um
   *         diretrio.
   * 
   * @throws RemoteException se houver falha de RMI
   */
  private ClientProjectFile getChildHelper(String childName)
    throws RemoteException {

    final ClientProjectFile[] children = getChildren();
    for (ClientProjectFile file : children) {
      if (file.name.equals(childName)) {
        return file;
      }
    }

    return null;
  }

  /**
   * Adiciona um novo arquivo neste diretrio se ele ainda no existir. Aps a
   * incluso, os arquivos deste diretrio so ordenados por nome.
   * 
   * @param newChild O arquivo (ou diretrio) a ser includo
   * @throws RemoteException se houver falha de RMI
   */
  public void addChild(ClientProjectFile newChild) throws RemoteException {
    buildChildListIfNull();

    childrenLock.lock();
    try {
      int index = childList.indexOf(newChild);
      if (index == -1) {
        newChild.parent = this;
        this.childList.add(newChild);
        this.sortChildren();
      }
    }
    finally {
      childrenLock.unlock();
    }
  }

  /**
   * Obtm o ndice de um arquivo dentro de um diretrio
   * 
   * @param child O arquivo (ou diretrio) do qual deseja-se o ndice
   * 
   * @return O ndice do arquivo ou -1 caso o arquivo no esteja presente na
   *         lista de filhos.
   * @throws RemoteException se houver falha de RMI
   */
  public int getChildIndex(ProjectFile child) throws RemoteException {
    final ClientProjectFile[] children = getChildren();
    final int sz = children.length;
    for (int i = 0; i < sz; i++) {
      ClientProjectFile file = children[i];
      if (child.getAbsolutePath().equals(file.getAbsolutePath())) {
        return i;
      }
    }
    return -1;
  }

  /**
   * Remove um arquivo deste diretrio. Aps a incluso, os arquivos deste
   * diretrio so ordenados por nome.
   * 
   * @param child O arquivo (ou diretrio) a ser removido
   * 
   * @return O ndice do arquivo removido ou -1 caso ocorra algum problema ao
   *         remover.
   * @throws RemoteException se houver falha de RMI
   */
  public int removeChild(ProjectFile child) throws RemoteException {
    buildChildListIfNull();

    childrenLock.lock();
    try {
      int index = childList.indexOf(child);
      if (index != -1) {
        childList.remove(index);
      }
      return index;
    }
    finally {
      childrenLock.unlock();
    }
  }

  /**
   * Ordena a lista de arquivos-filho
   * 
   * @throws RemoteException se houver falha de RMI
   */
  public void sortChildren() throws RemoteException {
    buildChildListIfNull();

    childrenLock.lock();
    try {
      Collections.sort(this.childList);
    }
    finally {
      childrenLock.unlock();
    }
  }

  /**
   * {@inheritDoc}
   */
  public void createDirectory(String name) throws RemoteException {
    ClientRemoteLocator.projectService.createDirectory(this.projectId,
      getPath(), name);
  }

  /**
   * Cria um arquivo dentro deste diretrio.
   * 
   * @param name Nome do arquivo a ser criado.
   * @param type Tipo do arquivo a ser criado.
   * 
   * @throws RemoteException falha de rmi
   */
  public void createFile(String name, String type) throws RemoteException {
    ClientRemoteLocator.projectService.createFile(this.projectId, getPath(),
      name, type);
  }

  /**
   * Cria vrios arquivos simultaneamente dentro deste diretrio.
   * 
   * @param fileInfoList lista de dados dos arquivos a serem criados.
   * 
   * @throws RemoteException falha de rmi
   */
  public void createFiles(List<ProjectFileInfo> fileInfoList)
    throws RemoteException {
    ClientRemoteLocator.projectService.createFiles(this.projectId, getPath(),
      fileInfoList);
  }

  /**
   * Troca o nome deste arquivo.
   * 
   * @param name Novo nome do arquivo.
   * 
   * @throws RemoteException falha de rmi
   */
  public void rename(String name) throws RemoteException {
    ClientRemoteLocator.projectService.renameFile(this.projectId, getPath(),
      name);
  }

  /**
   * Copia este arquivo para outro diretrio.
   * 
   * @param directory Diretrio destino.
   * 
   * @throws RemoteException Falha de rmi
   */
  public void copy(ProjectFile directory) throws RemoteException {
    Object targetProjectId = ((ClientProjectFile) directory).getProjectId();
    Object sourceProjectId = this.projectId;
    if (targetProjectId.equals(sourceProjectId)) {
      ClientRemoteLocator.projectService.copyFile(sourceProjectId, getPath(),
        directory.getPath());
    }
    else {
      ClientRemoteLocator.projectService.copyFile(sourceProjectId, getPath(),
        targetProjectId, directory.getPath());
    }
  }

  /**
   * Move este arquivo para outro diretrio.
   * 
   * @param directory Diretrio destino.
   * 
   * @throws RemoteException erro no acesso ao servidor
   */
  public void move(ProjectFile directory) throws RemoteException {
    Object targetProjectId = ((ClientProjectFile) directory).getProjectId();
    Object sourceProjectId = this.projectId;
    if (targetProjectId.equals(sourceProjectId)) {
      ClientRemoteLocator.projectService.moveFile(this.projectId, getPath(),
        directory.getPath());
    }
    else {
      ClientRemoteLocator.projectService.moveFile(sourceProjectId, getPath(),
        targetProjectId, directory.getPath());
    }
  }

  /**
   * Remove este arquivo. Aps chamado este mtodo, o objeto que representa o
   * arquivo no poder mais ser utilizado.
   * 
   * @throws RemoteException falha de comunicao com o servidor
   */
  public void remove() throws RemoteException {
    if (this.isPublishedFile()) {
      if (this.whichProjectPublished() == this.getProjectId()) {
        this.updatePublishAccess(new Object[0], User.getLoggedUser().getId());
      }
    }
    ClientRemoteLocator.projectService.removeFile(this.projectId, getPath());
  }

  /**
   * Atualiza as informaes desse arquivo buscando os dados reais no servidor.
   * 
   * @throws RemoteException Falha de rmi
   */
  public void updateInfo() throws RemoteException {
    ClientProjectFileInfo info =
      ClientRemoteLocator.projectService.getUpdatedFileInfo(this.projectId,
        getPath());
    this.type = info.getType();
    this.size = info.getSize();
    this.modificationDate = info.getModificationDate();
    this.isGlobalFile = info.isGlobalFile();
    this.isPublishedFile = info.isPublishedFile();
    this.isUnderConstruction = info.isUnderConstruction();
    this.whoPublishedId = info.getWhoPublishedId();
    this.whichProjectPublishedId = info.getWhichProjectPublishedId();
    this.isLocked = info.isLocked();
  }

  /**
   * Indica se este arquivo  um diretrio.
   * 
   * @return Verdadeiro caso seja um diretrio e falso caso contrrio.
   */
  public boolean isDirectory() {
    return this.isDirectory;
  }

  /**
   * Indica se este arquivo  um arquivo global.
   * 
   * @return Verdadeiro caso seja um arquivo global e falso caso contrrio.
   */
  public boolean isGlobalFile() {
    return this.isGlobalFile;
  }

  /**
   * Indica se este arquivo  um arquivo publicado.
   * 
   * @return Verdadeiro caso seja um arquivo publicado e falso caso contrrio.
   */
  public boolean isPublishedFile() {
    return this.isPublishedFile;
  }

  /**
   * Indica se este arquivo  um arquivo em construo.
   * 
   * @return Verdadeiro caso seja um arquivo em construo e falso caso
   *         contrrio.
   */
  public boolean isUnderConstruction() {
    return this.isUnderConstruction;
  }

  /**
   * Indica se este arquivo est presente no diretrio de projetos do Usurio,
   * ou se  apenas um link para um arquivo na rea global, obtido atravs do
   * gerenciamento de publicaes.
   * 
   * @return Verdadeiro caso esteja presente no diretrio e falso caso seja um
   *         link.
   */
  public boolean isRegularFile() {
    return !isDirectory && !isPublishedFile && !isGlobalFile;
  }

  /**
   * Muda o estado do arquivo: global, publicado, construo.
   * 
   * @param isGlobal true se o arquivo for global
   * @param isPublished true se o arquivo estiver publicado
   * @param isUnderConstruction true se o arquivo estiver em construo
   */
  public void setState(boolean isGlobal, boolean isPublished,
    boolean isUnderConstruction) {
    this.isGlobalFile = isGlobal;
    this.isPublishedFile = isPublished;
    this.isUnderConstruction = isUnderConstruction;
  }

  /**
   * Muda o estado do arquivo para bloqueado ou desbloqueado
   * 
   * @param isLocked true se o arquivo estiver bloqueado
   */
  public void setLocked(boolean isLocked) {
    this.isLocked = isLocked;
  }

  /**
   * Obtm a descrio atual deste arquivo.
   * 
   * @return O texto com a descrio do arquivo.
   * 
   * @throws RemoteException falha de rmi
   */
  public String getDescription() throws RemoteException {
    return ClientRemoteLocator.projectService.getFileDescription(
      this.projectId, getPath());
  }

  /**
   * Muda a descrio atual deste arquivo.
   * 
   * @param text novo texto da descrio do arquivo.
   * 
   * @throws RemoteException falha de rmi
   */
  public void setDescription(String text) throws RemoteException {
    ClientRemoteLocator.projectService.setFileDescription(this.projectId,
      getPath(), text);
  }

  /**
   * Anexa um texto  descrio atual deste arquivo. Permite a gerao de
   * histricos de modificao.
   * 
   * @param text texto a ser anexado.
   * 
   * @throws RemoteException falha de rmi
   */
  public void appendDescription(String text) throws RemoteException {
    ClientRemoteLocator.projectService.appendFileDescription(this.projectId,
      getPath(), text);
  }

  /**
   * Publica este arquivo.
   * 
   * @param userIds contendo os identificadores dos usurios que tero acesso ao
   *        arquivo publicado.
   * @param whoPublishedId Identificador do usurio que publicou o arquivo.
   * 
   * @return Verdadeiro em caso de sucesso e falso caso a operao falhe.
   * 
   * @throws RemoteException falha de rmi
   */
  public boolean publish(Object[] userIds, Object whoPublishedId)
    throws RemoteException {
    return ClientRemoteLocator.publicationService.publishFile(this.projectId,
      getPath(), userIds, whoPublishedId);
  }

  /**
   * Atualiza a lista de publicao do arquivo.
   * 
   * @param userIds .
   * @param userId Identificador do usurio que quer publicar o arquivo
   * 
   * @return Verdadeiro em caso de sucesso e falso caso a operao falhe.
   * 
   * @throws RemoteException Em caso de erro.
   */
  public boolean updatePublishAccess(Object[] userIds, Object userId)
    throws RemoteException {
    return ClientRemoteLocator.publicationService.updatePublishAccess(
      this.projectId, getPath(), userIds, userId);
  }

  /**
   * Informa a lista de usurios para os quais esse arquivo foi publicado. No
   * caso do arquivo no estar publicado, retorna null.
   * 
   * @return Um array com os identificadores dos usurios para os quais esse
   *         arquivo foi publicado.
   * 
   * @throws RemoteException falha de rmi
   */
  public Object[] getPublishAccess() throws RemoteException {
    return ClientRemoteLocator.publicationService.getPublishedFileAccess(
      this.projectId, getPath());
  }

  /**
   * Cria um link para um arquivo publicado. Dessa forma, o arquivo publicado
   * pode ser manipulado na rea de projeto.
   * 
   * @param info Informaes sobre a publicao a ser referenciada.
   * @throws RemoteException falha de rmi
   */
  public void linkToPublishedFile(PublicationInfo info) throws RemoteException {
    ClientRemoteLocator.publicationService.linkToPublishedFile(this.projectId,
      getPath(), info);
  }

  /**
   * Informa o tamanho corrente deste arquivo. Reflete o nmero corrente de
   * bytes do arquivo.
   * 
   * @return Tamanho corrente do arquivo.
   */
  public long size() {
    return this.size;
  }

  /**
   * Informa a data da ltima modificao deste arquivo (formato numrico).
   * 
   * @return data da ltima modificao deste arquivo (formato numrico)
   */
  public long getModificationDate() {
    return this.modificationDate;
  }

  /**
   * Indica se o arquivo est atualizado.
   * 
   * @return true se est atualizado e false caso contrrio.
   */
  public boolean isUpdated() {
    return updated;
  }

  /**
   * Informa se o arquivo pode estar desatualizado.
   * 
   * @param updated false se pode estar desatualizado.
   */
  public void setUpdated(boolean updated) {
    this.updated = updated;
  }

  /**
   * Obtm n filho indicado pelo caminho fornecido.
   * 
   * @param path array de Strings com o caminho para o arquivo, incluindo o nome
   *        do diretrio corrente.
   * 
   * @return O n filho pedido ou null caso este no exista.
   * @throws RemoteException se houver falha de RMI
   */
  public ClientProjectFile getChild(String[] path) throws RemoteException {
    if (path == null || path.length == 0) {
      return null;
    }
    if (path.length == 1) {
      return null;
    }
    return getChild(path, 1);
  }

  /**
   * Obtm o filho do diretrio corrente a partir do {@link ProjectFileInfo} que
   * o representa.
   * 
   * @param prjFileInfo - propriedades do arquivo a ser obtido
   * @return o arquivo solicitado ou null caso no exista ou se o arquivo
   *         corrente no  um diretrio
   * @throws RemoteException
   */
  public ClientProjectFile getChild(ProjectFileInfo prjFileInfo)
    throws RemoteException {
    return getChild(prjFileInfo.getPath(), 0);
  }

  /**
   * Obtm o filho do diretrio corrente associado a um path especfico,
   * descartando (opcionalmente) parte do path.
   * 
   * @param path - path para o arquivo
   * @param startIndex - ndice do path a partir do qual deve-se fazer a busca
   * @return o arquivo solicitado ou null caso no exista ou se o arquivo
   *         corrente no  um diretrio
   * @throws RemoteException
   */
  private ClientProjectFile getChild(String[] path, int startIndex)
    throws RemoteException {
    if (!isDirectory) {
      return null;
    }
    ClientProjectFile child = this;
    for (int i = startIndex; i < path.length; i++) {
      child = child.getChild(path[i]);
      if (child == null) {
        return null;
      }
    }
    return child;
  }

  /**
   * Verifica se o arquivo existe no servidor.
   * 
   * @return true se o arquivo existe no servidor.
   * @throws RemoteException se houver falha de RMI
   */
  public boolean exists() throws RemoteException {
    return ClientProjectFile.existsFile(this.projectId, getPath());
  }

  /**
   * Verifica se o arquivo existe no servidor.
   * 
   * @param projectId id do projeto
   * @param path o path do arquivo.
   * 
   * @return true se o arquivo existe no servidor.
   * @throws RemoteException se houver falha de RMI
   */
  static public boolean existsFile(final Object projectId, final String[] path)
    throws RemoteException {
    return ClientRemoteLocator.projectService.existsFile(projectId, path);
  }

  /**
   * Mtodo interno para decomposio de um caminho de arquivo (<i>path</i>) em
   * um array de strings; onde cada elemento  montado no separador
   * <code>"/"</code>.
   * 
   * @param path o caminho lgico do arquivo.
   * 
   * @return o array.
   */
  static public String[] splitPath(final String path) {
    String[] pathArray = path.split("/");
    int i = 0;
    while ((i < pathArray.length) && pathArray[i].equals("")) {
      i++;
    }
    if (i > 0) {
      final String[] newPath = new String[pathArray.length - i];
      System.arraycopy(pathArray, i, newPath, 0, newPath.length);
      pathArray = newPath;
    }
    return pathArray;
  }

  /**
   * Ajusta o tamanho corrente deste arquivo.
   * 
   * @param newSize o novo tamanho a ser ajustado.
   * 
   * @throws IOException Em caso de erro.
   */
  public void truncate(long newSize) throws IOException {
    this.channel.truncate(newSize);
  }

  /**
   * Abre este arquivo para leitura e escrita.
   * 
   * @param readOnly True se o arquivo for aberto apenas para leitura, ou false,
   *        para leitura e escrita.
   * 
   * @return Verdadeiro em caso de sucesso e falso caso a operao falhe.
   * 
   * @throws RemoteException erro na criao do channel no servidor
   */
  public boolean openFileChannel(boolean readOnly) throws RemoteException {
    updateInfo(); // atualiza info do arquivo antes de abrir
    if (this.state == CLOSED) {
      this.channel =
        ClientRemoteLocator.projectService.openFileChannel(this.projectId,
          getPath(), readOnly);
      if (this.channel != null) {
        if (this.channel.open(readOnly)) {
          this.state = OPENED_AS_CHANNEL;
          this.numberOfChannelOpens = 1;
          return true;
        }
        this.channel = null;
      }
      return false;
    }
    if (this.state == OPENED_AS_CHANNEL) {
      this.numberOfChannelOpens++;
      return true;
    }
    return false;
  }

  /**
   * Abre este arquivo para leitura exclusiva que no  feita com o uso de
   * <code>ByteBuffer</code>. A implementao desse mtodo determina o uso de
   * RMI para transmisso dos dados.
   * 
   * @return Verdadeiro em caso de sucesso e falso caso a operao falhe.
   * 
   * @throws RemoteException falha de rmi
   */
  public boolean openForReading() throws RemoteException {
    if (this.state != CLOSED) {
      return false;
    }
    ClientRemoteLocator.projectService.openFileForReading(this.projectId,
      getPath());
    this.state = OPENED_FOR_READING;
    return true;
  }

  /**
   * Abre este arquivo para escrita exclusiva que no  feita com o uso de
   * <code>ByteBuffer</code>. A implementao desse mtodo determina o uso de
   * RMI para transmisso dos dados.
   * 
   * @return Verdadeiro em caso de sucesso e falso caso a operao falhe.
   * 
   * @throws RemoteException falha de rmi
   */
  public boolean openForWriting() throws RemoteException {
    if (this.state != CLOSED) {
      return false;
    }
    ClientRemoteLocator.projectService.openFileForWriting(this.projectId,
      getPath());
    this.state = OPENED_FOR_WRITING;
    return true;
  }

  /**
   * Tenta obter lock exclusivo de escrita para o arquivo no servidor.
   * 
   * @see ProjectService#acquireExclusiveLock(Object, String[])
   * 
   * @return identificador do lock se o lock foi obtido, ou null caso contrrio
   * @throws RemoteException falha de rmi
   */
  public Object acquireExclusiveLock() throws RemoteException {
    return ClientRemoteLocator.projectService.acquireExclusiveLock(
      getProjectId(), getPath());
  }

  /**
   * Tenta obter um lock de escrita exclusivo para o arquivo. Locks exclusivos
   * so obtidos apenas uma vez.
   * 
   * Este mtodo cadastra uma solicitao de lock numa fila priorizada pela
   * ordem de chegada dos pedidos de lock e espera pela notificao de que o
   * lock foi obtido ou seu timeout foi atingido. Quando o lock  obtido, o
   * mtodo FileLockListenerInterface.fileLocked  chamado. Quando o tempo
   * mximo de espera do lock  atingido e o lock no foi obtido o mtodo
   * FileLockListenerInterface.fileLockExpired  chamado.
   * 
   * @param listener observador do lock cadastrado no servidor
   * @param timeout tempo mximo de espera pelo lock em milisegundos. A
   *        constante ProjectServiceInterface.INFINITE_TIMOUT foi criada para
   *        timeout infinito.
   * @return identificador do lock se o lock foi obtido, ou null caso contrrio
   * 
   * @throws RemoteException falha de rmi
   */
  public Object acquireExclusiveLock(ClientFileLockListener listener,
    long timeout) throws RemoteException {
    return ClientRemoteLocator.projectService.acquireExclusiveLock(
      getProjectId(), getPath(), listener, timeout);
  }

  /**
   * Tenta obter lock compartilhado de escrita para o arquivo no servidor.
   * 
   * @see ProjectService#acquireExclusiveLock(Object, String[])
   * 
   * @return identificador do lock se o lock foi obtido, ou null caso contrrio
   * @throws RemoteException falha de rmi
   */
  public Object acquireSharedLock() throws RemoteException {
    return ClientRemoteLocator.projectService.acquireSharedLock(getProjectId(),
      getPath());
  }

  /**
   * Tenta obter lock compartilhado para o arquivo. Locks compartilhados podem
   * ser obtidos mltiplas vezes.
   * 
   * Este mtodo cadastra uma solicitao de lock compartilhado em uma fila
   * priorizada pela ordem de chegada dos pedidos de lock e espera pela
   * notificao de que o lock foi obtido ou seu timeout foi atingido. Quando o
   * lock  obtido, o mtodo FileLockListenerInterface.fileLocked  chamado.
   * Quando o tempo mximo de espera do lock  atingido e o lock no foi obtido
   * o mtodo FileLockListenerInterface.fileLockExpired  chamado.
   * 
   * @param listener observador do lock cadastrado no servidor
   * @param timeout tempo mximo de espera pelo lock em milisegundos. A
   *        constante ProjectServiceInterface.INFINITE_TIMOUT foi criada para
   *        timout infinito.
   * @return o identificador do lock se o lock foi obtido com sucesso, ou null
   *         caso contrrio
   * 
   * @throws RemoteException falha de rmi
   */
  public Object acquireSharedLock(ClientFileLockListener listener, long timeout)
    throws RemoteException {
    return ClientRemoteLocator.projectService.acquireSharedLock(getProjectId(),
      getPath(), listener, timeout);
  }

  /**
   * Libera o lock para o arquivo no servidor.
   * 
   * @param lockId identificador do lock
   * 
   * @see ProjectService#releaseLock(Object, String[], Object)
   * 
   * @return contador de locks ainda ativos sobre o arquivo. Para locks
   *         exclusivos, este contador tem que ser zero se a operao foi bem
   *         sucedida. Para locks compartilhados, pode ainda haver outros locks
   *         ativos.
   * 
   * @throws RemoteException falha de rmi
   */
  public int releaseLock(Object lockId) throws RemoteException {
    return ClientRemoteLocator.projectService.releaseLock(getProjectId(),
      getPath(), lockId);
  }

  /**
   * {@inheritDoc}
   */
  public boolean isLocked() {
    return isLocked;
  }

  /**
   * L um objeto gravado no arquivo. A implementao desse mtodo utiliza RMI
   * para envio do objeto serializado.
   * 
   * @return O objeto lido do arquivo.
   * 
   * @throws Exception Em caso de erro.
   */
  public Object readObject() throws Exception {
    if (this.state != OPENED_FOR_READING) {
      throw new Exception(PRJ_FILE_INVALID_STATE);
    }
    return (ClientRemoteLocator.projectService.readObject(this.projectId,
      getPath()));
  }

  /**
   * Grava um objeto no arquivo. A implementao desse mtodo utiliza RMI para
   * envio do objeto serializado.
   * 
   * @param obj O objeto a ser gravado no arquivo.
   * 
   * @throws Exception Em caso de erro.
   */
  public void writeObject(Object obj) throws Exception {
    if (this.state != OPENED_FOR_WRITING) {
      throw new Exception(PRJ_FILE_INVALID_STATE);
    }
    ClientRemoteLocator.projectService.writeObject(this.projectId, getPath(),
      obj);
  }

  /**
   * Indica se este arquivo est aberto, ou seja, se qualquer um dos 3 mtodos
   * de abertura j foi chamado e ainda no foi chamado o mtodo
   * {@link #close()}.
   * 
   * @return Verdadeiro caso este arquivo esteja aberto, falso caso contrrio.
   * 
   * @throws IllegalStateException Estado diferente {@link #OPENED_AS_CHANNEL},
   *         {@link #OPENED_FOR_READING}, {@link #OPENED_FOR_WRITING} ou
   *         {@link #CLOSED}.
   * 
   * @see #openForReading()
   * @see #openForWriting()
   */
  public boolean isOpen() {
    switch (this.state) {
      case OPENED_AS_CHANNEL:
      case OPENED_FOR_READING:
      case OPENED_FOR_WRITING:
        return true;
      case CLOSED:
        return false;
      default:
        String message =
          MessageFormat.format("Estado inesperado.\nEstado esperado: {0}.",
            new Object[] { new Integer(this.state) });
        throw new IllegalStateException(message);
    }
  }

  /**
   * Informa a posio corrente deste arquivo. Reflete o nmero de bytes a
   * partir do incio do arquivo at a posio corrente.
   * 
   * @return Posio corrente do arquivo.
   * 
   * @throws IOException Em caso de erro.
   */
  public long position() throws IOException {
    return this.channel.position();
  }

  /**
   * Altera a posio corrente deste arquivo. Reflete o nmero de bytes a partir
   * do incio do arquivo at a posio corrente.
   * 
   * @param newPosition Nova posio corrente do arquivo.
   * 
   * @throws IOException Em caso de erro.
   */
  public void position(long newPosition) throws IOException {
    this.channel.position(newPosition);
  }

  /**
   * L uma seqncia de bytes deste arquivo a partir da posio fornecida. Veja
   * a interface ReadableByteChannel para uma descrio completa da semntica
   * deste mtodo.
   * 
   * @param dst O buffer no qual os bytes sero escritos.
   * @param position A posio do arquivo a partir do qual os bytes sero lidos.
   * 
   * @return O nmero de bytes lidos.
   * 
   * @throws Exception Em caso de erro.
   */
  public int read(ByteBuffer dst, long position) throws Exception {
    return this.channel.read(dst, position);
  }

  /**
   * Escreve uma seqncia de bytes neste arquivo a partir da posio fornecida.
   * 
   * @param src O buffer no qual esto os bytes a serem escritos.
   * @param position A posio do arquivo a partir da qual os bytes sero
   *        escritos
   * 
   * @return O nmero de bytes escritos.
   * 
   * @throws IOException Em caso de erro.
   * @throws FileLockedException caso o arquivo esteja bloqueado para escrita
   *         por outra sesso de usurio
   */
  public int write(ByteBuffer src, long position) throws IOException,
    FileLockedException {
    return this.channel.write(src, position);
  }

  /**
   * Faz upload de um arquivo.
   * 
   * @param file o arquivo
   * @param chunkSize o tamnaho do chunk de envio.
   * @param listener o listener
   * @throws IOException se houver falha de I/O
   * @throws FileLockedException caso haja falha de locking.
   */
  public void upload(final File file, final long chunkSize,
    final ProjectFileChannelLoadListener listener) throws IOException,
    FileLockedException {
    this.openFileChannel(false);
    this.channel.upload(file, chunkSize, listener);
    this.close();
  }

  /**
   * Faz download de um arquivo.
   * 
   * @param file o arquivo
   * @param chunkSize o tamnaho do chunk de envio.
   * @param listener o listener
   * @throws IOException se houver falha de I/O
   * @throws FileLockedException caso haja falha de locking.
   */
  public void download(final File file, final long chunkSize,
    final ProjectFileChannelLoadListener listener) throws IOException,
    FileLockedException {
    this.openFileChannel(false);
    this.channel.download(file, chunkSize, listener);
    this.close();
  }

  /**
   * Fecha este arquivo. Aps chamado este mtodo, o objeto que representa o
   * arquivo no poder mais ser utilizado para leitura e/ou escrita.
   * 
   * @throws IOException se houver falha de I/O
   * @throws RemoteException erro ao fechar o channel no servidor
   */
  public void close() throws RemoteException, IOException {
    switch (this.state) {
      case OPENED_AS_CHANNEL:
        if (--this.numberOfChannelOpens == 0) {
          this.channel.close();
          this.channel = null;
          this.state = CLOSED;
        }
        break;
      case OPENED_FOR_READING:
      case OPENED_FOR_WRITING:
        ClientRemoteLocator.projectService.closeFile(this.projectId, getPath());
        this.state = CLOSED;
        break;
    }
  }
  
  /**
   * Fecha este arquivo. Aps chamado este mtodo, o objeto que representa o
   * arquivo no poder mais ser utilizado para leitura e/ou escrita.
   * 
   * @throws IOException se houver falha de I/O
   * @throws RemoteException erro ao fechar o channel no servidor
   */
  public void closeOnError() throws RemoteException, IOException {
    switch (this.state) {
      case OPENED_AS_CHANNEL:
        if (--this.numberOfChannelOpens == 0) {
          this.channel.closeOnError();
          this.channel = null;
          this.state = CLOSED;
        }
        break;
      case OPENED_FOR_READING:
      case OPENED_FOR_WRITING:
        ClientRemoteLocator.projectService.closeFile(this.projectId, getPath());
        this.state = CLOSED;
        break;
    }
  } 

  /**
	 * Retorna um texto descritivo do arquivo.
	 * 
	 * @return .
	 */
  @Override
  public String toString() {
    if (this.renderingName != null) {
      return this.renderingName;
    }
    return this.name;
  }

  /**
   * Muda o tipo do arquivo localmente.
   * 
   * @param newType O novo tipo do arquivo.
   */
  public void setType(String newType) {
    this.type = newType;
  }

  /**
   * Solicita ao servidor para modificar o tipo do arquivo
   * 
   * @param newType O novo tipo do arquivo.
   * 
   * @throws RemoteException falha de rmi
   */
  public void changeType(String newType) throws RemoteException {
    ClientRemoteLocator.projectService.changeFileType(this.projectId,
      getPath(), newType);
  }

  /**
   * Verifica se o arquivo foi publicado pelo projeto dado
   * 
   * @param projId Identificador do projeto
   * 
   * @return .
   */
  public boolean wasPublishedBy(Object projId) {
    return (this.isPublishedFile && projId.equals(this.whichProjectPublishedId));
  }

  /**
   * Testa a igualdade de dois arquivos. Dois <code>ClientProjectFile</code> so
   * considerados iguais se seus paths absolutos forem os mesmos.
   * 
   * @param o Objeto a ser comparado com o atual
   * 
   * @return true se ambos os objetos forem considerados iguais
   */
  @Override
  public boolean equals(Object o) {
    if (o == null) {
      return false;
    }
    if (!this.getClass().equals(o.getClass())) {
      return false;
    }
    ClientProjectFile other = (ClientProjectFile) o;
    return this.absolutePath.equals(other.absolutePath);
  }

  /**
   * Redefine a chave pela qual o arquivo deve ser buscado/inserido em
   * hashtables.
   * 
   * @return chave para insero/busca em hashtables.
   */
  @Override
  public int hashCode() {
    return this.absolutePath.hashCode();
  }

  /**
   * Construtor
   * 
   * @param projectId identificador do projeto
   * @param name nome do arquivo
   * @param parent diretrio-pai. Se estiver sendo construido no servidor, este
   *        valor deve ser null.
   * @param type tipo do arquivo
   * @param children arquivos-filho
   * @param isDirectory indicativo de diretrio
   * @param isGlobalFile indicativo de arquivo global
   * @param isPublishedFile indicativo de publicao
   * @param isUnderConstruction indicaivo de construo
   * @param absolutePath camimho absoluto (path)
   * @param whoPublishedId usurio que publicou (se for o caso)
   * @param whichProjectPublishedId projeto que publicou (se for o caso)
   * @param createdBy usurio que criou o arquivo/diretrio
   * @param creationDate data de criao
   * @param size tamanho do arquivo
   * @param modificationDate data da ltima modificao.
   * @param updatable indica se o arquivo  atualizvel.
   * @param isLocked indica se o arquivo est bloqueado
   */
  public ClientProjectFile(Object projectId, String name,
    ClientProjectFile parent, String type, ClientProjectFile[] children,
    boolean isDirectory, boolean isGlobalFile, boolean isPublishedFile,
    boolean isUnderConstruction, String absolutePath, Object whoPublishedId,
    Object whichProjectPublishedId, Object createdBy, long creationDate,
    long size, long modificationDate, boolean updatable, boolean isLocked) {
    this.projectId = projectId;
    this.name = name;
    this.renderingName = name;
    this.parent = parent;
    this.type = type;
    // this.childList = createList(children);
    this.isDirectory = isDirectory;
    this.isGlobalFile = isGlobalFile;
    this.isPublishedFile = isPublishedFile;
    this.isLocked = isLocked;
    this.isUnderConstruction = isUnderConstruction;
    this.channel = null;
    this.absolutePath = absolutePath;
    this.whoPublishedId = whoPublishedId;
    this.whichProjectPublishedId = whichProjectPublishedId;
    this.state = CLOSED;
    this.numberOfChannelOpens = 0;
    this.createdBy = createdBy;
    this.creationDate = creationDate;
    this.size = size;
    this.modificationDate = modificationDate;
    this.updated = true;
    this.updatable = updatable;
  }

  /**
   * Cria a lsita de filhos.
   * 
   * @param children array de filhos
   * @return a lista
   */
  private static List<ClientProjectFile> createList(ClientProjectFile[] children) {
    List<ClientProjectFile> childList = new ArrayList<ClientProjectFile>();
    if (children != null) {
      for (int i = 0; i < children.length; i++) {
        childList.add(children[i]);
      }
    }
    return childList;
  }

  /**
   * {@inheritDoc}
   */
  public int compareTo(Object other) {
    ClientProjectFile that = (ClientProjectFile) other;
    return this.name.compareTo(that.name);
  }

  /**
   * @return nome especial para exibio
   */
  public String getRenderingName() {
    return this.renderingName;
  }

  /**
   * Indica se o nome especial (rendering)  pai.
   * 
   * @return indicativo
   * @see #PARENT_DIR_RENDERING_NAME
   */
  public boolean isParentDir() {
    return (this.renderingName.equals(PARENT_DIR_RENDERING_NAME));
  }

  /**
   * Define o nome especial para exibio. Este nome ser usado no lugar do nome
   * real do arquivo.
   * 
   * @param newValue - nome para exibio
   */
  public void setRenderingName(String newValue) {
    this.renderingName = newValue;
  }

  /**
   * Ajusta o arquivo como arquivo pai.
   * 
   * @see #PARENT_DIR_RENDERING_NAME
   */
  public void setAsParentDir() {
    this.renderingName = PARENT_DIR_RENDERING_NAME;
  }

  /**
   * @see #isPublishedByOthers()
   * 
   * @return true se o arquivo  um arquivo publicado e se o projeto em que se
   *         encontra o arquivo  o mesmo a partir do qual este foi publicado
   */
  public boolean isPublishedByMe() {
    if (isPublishedFile()) {
      return (getProjectId().equals(whichProjectPublished()));
    }
    return false;
  }

  /**
   * @see #isPublishedByMe()
   * 
   * @return true se o arquivo  um arquivo publicado e se o projeto em que se
   *         encontra o arquivo <b>no</b>  o mesmo a partir do qual este foi
   *         publicado.
   */
  public boolean isPublishedByOthers() {
    if (isPublishedFile()) {
      return (!getProjectId().equals(whichProjectPublished()));
    }
    return false;
  }

  /**
   * Indica se este arquivo/diretrio est sendo movido de lugar. Esta
   * informao ser utilizada para definir o cone do arquivo/diretrio.
   * 
   * @param isMoving indicativo
   */
  public void setMoving(boolean isMoving) {
    if (this.isMoving != isMoving) {
      this.isMoving = isMoving;
    }
  }

  /**
   * @return true se este arquivo/diretrio e/ou seu diretrio pai estiver sendo
   *         movido. Esta informao ser utilizada para definir o cone do
   *         arquivo/diretrio.
   */
  public boolean isMoving() {
    return this.isMoving || (null != getParent() && getParent().isMoving());
  }

  /**
   * Verifica se algum arquivo de projeto  diretrio.
   * 
   * @param projectFiles Os arquivos que se deseja verificar.
   * 
   * @return true, caso algum dos arquivos de projeto seja diretrio, ou false,
   *         caso contrrio.
   */
  public static boolean hasDirectories(ProjectFile[] projectFiles) {
    for (int i = 0; i < projectFiles.length; i++) {
      if (projectFiles[i].isDirectory()) {
        return true;
      }
    }
    return false;
  }

  /**
   * Verifica se algum arquivo de projeto est em construo.
   * 
   * @param projectFiles Os arquivos que se deseja verificar.
   * 
   * @return true, caso algum dos arquivos de projeto esteja em construo, ou
   *         false, caso contrrio.
   */
  public static boolean hasFileUnderConstruction(ProjectFile[] projectFiles) {
    for (int i = 0; i < projectFiles.length; i++) {
      if (projectFiles[i].isUnderConstruction()) {
        return true;
      }
    }
    return false;
  }

  /**
   * Verifica se  possvel publicar o arquivo.
   * 
   * @return true, caso seja possvel publicar o arquivo, ou false, caso
   *         contrrio.
   */
  public boolean canPublishFile() {
    if (!this.isRegularFile()) {
      return false;
    }
    if (this.isPublishedFile()) {
      return false;
    }
    return true;
  }

  /**
   * Verifica se  possvel publicar os arquivos de projeto.
   * 
   * @param projectFiles Os arquivos de projeto que se deseja verificar.
   * 
   * @return true, caso seja possvel publicar todos os arquivos de projeto, ou
   *         false, caso contrrio.
   */
  public static boolean canPublishFiles(ClientProjectFile[] projectFiles) {
    for (int i = 0; i < projectFiles.length; i++) {
      if (!projectFiles[i].canPublishFile()) {
        return false;
      }
    }
    return true;
  }

  /**
   * Verifica se  possvel despublicar o arquivo.
   * 
   * @return true, caso seja possvel despublicar o arquivo, ou false, caso
   *         contrrio.
   */
  public boolean canUnpublishFile() {
    return this.wasPublishedBy(this.getProjectId());
  }

  /**
   * Verifica se o arquivo  atualizvel.
   * 
   * @return {@code true} caso o arquivo seja atualizvel, ou {@code false},
   *         caso contrrio.
   */
  public boolean isUpdatable() {
    return this.updatable;
  }

  /**
   * Obtm o intervalo pelo qual o arquivo (somentes os atualizveis) est sendo
   * atualizado.
   * 
   * @return O intervalo pelo qual o arquivo est sendo atulizado. Caso o
   *         arquivo no esteja sendo atualizado ou no seja atualizvel,
   *         retorna 0 (zero).
   */
  public int getUpdateInterval() {
    return this.updateInterval;
  }

  /**
   * Define o intervalo pelo qual um arquivo est sendo atualizado.
   * 
   * @param updateInterval O intervalo pelo qual o arquivo est sendo atulizado.
   *        <p>
   *        Caso o arquivo no esteja sendo atulizado ou no seja atualizvel, o
   *        valor deve ser 0 (zero).
   */
  public void setUpdateInterval(int updateInterval) {
    this.updateInterval = updateInterval;
  }

  /**
   * <p>
   * Verifica se  possvel despublicar os arquivos de projeto.
   * </p>
   * <p>
   * Para que vrios arquivos de projeto possam ser despublicados ao mesmo tempo
   *  necessrio que, alm de cada um poder ser despublicado individualmente,
   * que eles tenham sido publicados para o mesmo conjunto de usurios.
   * </p>
   * 
   * @param projectFiles Os arquivos de projeto que se deseja verificar.
   * 
   * @return true, caso seja possvel despublicar todos os arquivos de projeto,
   *         ou false, caso contrrio.
   * 
   * @throws RemoteException Caso ocorra algum problema ao obter a lista de
   *         usurios dos arquivos de projeto.
   */
  public static boolean canUnpublishFiles(ClientProjectFile[] projectFiles)
    throws RemoteException {
    for (int i = 0; i < projectFiles.length; i++) {
      if (!projectFiles[i].canUnpublishFile()) {
        return false;
      }
    }
    Object[] previousPublishAccess = projectFiles[0].getPublishAccess();
    for (int i = 1; i < projectFiles.length; i++) {
      Object[] currentPublishAccess = projectFiles[i].getPublishAccess();
      if (!Arrays.equals(previousPublishAccess, currentPublishAccess)) {
        return false;
      }
      previousPublishAccess = currentPublishAccess;
    }
    return true;
  }
}
