/**
 * Tecgraf
 * websintesi
 * $Author: taciana $
 * $Revision: 69244 $ - $Date: 2007-10-20 11:41:00 -0200 (Sat, 20 Oct 2007) $ * 
 */
package csbase.logic;

import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;
import java.rmi.RemoteException;

/**
 * Implementa um leitor (stream) de um arquivo (client-side).
 */
public class ClientProjectFileInputStream extends InputStream {
  
  /** Arquivo a ser lido */
  private final ClientProjectFile file;
  
  /** Cache contendo bytes que podem ser lidos sem fazer chamadas remotas */
  private final ByteBuffer cache;
  
  /** Marca anteriormente registrada para o stream (relativa ao arquivo). */
  private long fileMark = 0;
  
  /** Posio corrente no arquivo */
  private long currentFilePosition = 0;
  
  /**
   * <p>
   * Mtodo para ler bytes (efetivamente) do arquivo remoto armazenando-os na
   * cache.
   * </p>
   * Se a cache for grande o suficiente, o mtodo ler a partir da posio 
   * especificada at o fim do arquivo. Caso contrrio, ler apenas o nmero
   * de bytes que couberem na cache.
   * 
   * @param startReadPosition posio a partir da qual ser feita a leitura no 
   *        arquivo. Se a posio for maior ou igual que o tamanho do arquivo, o 
   *        mtodo retorna sem lanar exceo.
   * 
   * @throws IOException
   */
  private void fillCache(long startReadPosition) throws IOException {
    try {
      final long size = file.size();
      if (startReadPosition >= size) {
        cache.position(cache.limit());
        return;
      }
      int expectedBytes = cache.capacity(); 
      long fileRemaining = size - startReadPosition; 
      if (fileRemaining < cache.capacity()) {
        expectedBytes = (int)fileRemaining;
      }
      cache.clear(); 
      int totalBytesRead = 0;
      do {
        long bytesRead = file.read(cache, startReadPosition);
        totalBytesRead += bytesRead;
      }
      while (totalBytesRead < expectedBytes);
      cache.flip();
    }
    catch (Throwable t) {
      throw new IOException(t.getMessage());
    }
  }

  /**
   * Quantidade de bytes disponveis sem necessidade de I/O.
   * 
   * @return a quantidade restante de bytes na cache.
   */
  @Override
  public int available() {
    return cache.remaining();
  }

  /**
   * Mtodo para marca de posio.
   * 
   * @param readlimit limite mximo de bytes lidos a ser suportado pela marca.
   */
  @Override
  public void mark(int readlimit) {
    fileMark = currentFilePosition;
  }

  /**
   * Indicao de suporte  marca.
   * 
   * @return true (sempre).
   */
  @Override
  public boolean markSupported() {
    return true;
  }

  /**
   * L o prximo byte disponvel desta <code>InputStream</code>. O valor  
   * retornado como um inteiro, variando de 0 a 255.
   * 
   * @return byte lido ou -1 se o fim-de-arquivo for encontrado.
   * 
   * @throws IOException
   */
  @Override
  public int read() throws IOException {
    if (cache.remaining() == 0) {
      fillCache(currentFilePosition);
      if (cache.remaining() == 0) {
        return -1;
      }
    }
    int data = cache.get() & 0xFF;
    currentFilePosition++;
    return data;
  }
  
  /**
   * L at <code>buffer.length</code> bytes desta <code>InputStream</code>
   * dentro do array especificado. O mtodo bloqueia at que todo o buffer seja
   * preenchido ou o fim do arquivo seja encontrado.
   * 
   * @param buffer
   *          o array de escrita.
   * 
   * @return a quantidade de bytes lidos ou -1 se o fim-de-arquivo for
   *         encontrado.
   * 
   * @throws IOException
   */
  @Override
  public int read(byte[] buffer) throws IOException {
    if (cache.remaining() == 0) {
      fillCache(currentFilePosition);
      if (cache.remaining() == 0) {
        return -1;
      }
    }
    int bufferRemaining = buffer.length;
    int totalBytesRead = 0;
    while (bufferRemaining > cache.remaining()) {
      int requestedBytes = cache.remaining();
      cache.get(buffer, totalBytesRead, requestedBytes);
      totalBytesRead += requestedBytes;
      currentFilePosition += requestedBytes;
      bufferRemaining -= requestedBytes;
      fillCache(currentFilePosition);
      if (cache.remaining() == 0) {
        return totalBytesRead;
      }
    }
    int requestedBytes = bufferRemaining;
    cache.get(buffer, totalBytesRead, requestedBytes);
    totalBytesRead += requestedBytes;
    currentFilePosition += requestedBytes;
    bufferRemaining -= requestedBytes;
    return totalBytesRead;
  }
  
  /**
   * Volta  marca anterior.
   * 
   * @throws IOException
   */
  @Override
  public void reset() throws IOException {
    if (fileMark >= 0) {
      fillCache(fileMark);
    }
    fileMark = -1;
  }

  /**
   * "Pula" e descarta n bytes de dados desta <code>InputStream</code>.
   * Se o fim-de-arquivo for atingido antes de completar o salto, somente o
   * nmero de bytes realmente "pulado"  descartado.
   * Se n for negativo, nada  "pulado".
   * 
   * @param n o nmero de bytes a serem "pulados".
   * 
   * @return nmero de bytes realmente "pulados".
   * 
   * @throws IOException
   */
  @Override
  public long skip(long n) throws IOException {
    if (n < 0) {
      return n;
    }
    currentFilePosition += n;
    if (currentFilePosition > file.size()) {
      n -= currentFilePosition - file.size();
      currentFilePosition = file.size();
    }
    if (n > cache.remaining()) {
      fillCache(currentFilePosition);
    }
    else {
      cache.position(cache.position()+(int)n);
    }
    return n;
  }  

  /**
   * Fecha o <code>InputStream</code>.
   * 
   * @throws IOException
   */
  @Override
  public void close() throws IOException {
    file.close();
  }

  /**
   * Obtm o <code>ClientProjectFile</code>.
   * @return o <code>ClientProjectFile</code>
   */
  public ClientProjectFile getClientProjectFile() {
    return file;
  }
  
  /**
   * Cria um <code>InputStream</code> a partir de um arquivo da rea de 
   * projetos, abrindo-o para leitura/escrita e lendo um bloco (cache) inicial.
   * 
   * @param file Arquivo da rea de projetos (<code>ClientProjectFile</code>).
   * 
   * @param bSize Tamanho desejado para a cache.
   *                 
   * @throws IOException
   * @throws RemoteException falha de rmi na criao do channel no servidor
   */
  public ClientProjectFileInputStream(final ClientProjectFile file, 
  final int bSize) throws IOException,
    RemoteException{
    int cacheSize = bSize <= 0 ? 16 * 1024 : bSize;
    this.file = file;
    cache = ByteBuffer.allocate(cacheSize);
    file.openFileChannel(true);
    fillCache(0);
  }
}
