/*
 * CommandAction.java
 * 
 * $Author: isabella $ $Revision: 93000 $ - $Date: 2008-06-06 16:24:00 -0300
 * (Fri, 06 Jun 2008) $
 */
package csbase.logic;

import java.io.Serializable;

/**
 * Representa eventos que podem ocorrer em um comando. Objetos dessa classe so
 * criados pelo servidor sempre que acontecer alguma modificao em um comando.
 * Os metodos desta classe nao devem lancar excecoes, uma vez que so chamados
 * por metodos que definem a idl sga-manager.
 */
public final class CommandEvent implements Serializable {
  /**
   * @author ururahy
   * 
   */
  public enum EventType {

    /** Tipo de evento que indica que um comando foi perdido. */
    LOST,
        /** Tipo de evento que indica que um comando foi abortado. */
    KILLED,
        /** Tipo de evento que indica que um comando nao foi iniciado. */
    INIT_FAILURE,
        /**
     * Tipo de evento que indica que um comando terminou (mas no se sabe como).
     */
    COMPLETED,
        /** Tipo de evento que indica que um comando terminou com erro. */
    ERROR,
        /** Tipo de evento que indica que um comando terminou com sucesso. */
    SUCCESS,
        /**
     * Tipo de evento que indica que um comando terminou, mas no foi possvel
     * ler o cdigo de retorno, para determinar se foi bem sucedido.
     */
    NO_CODE;
  }

  /**
   * Indica qual foi o evento ocorrido no servidor. Pode ser uma das constantes
   * definidas nessa classe.
   */
  private Object commandId;

  /** Indica o tipo do evento. */
  private EventType type;

  /**
   * Tempo de parede (em segundos).
   */
  private int elapsedTime;
  /**
   * Tempo em modo usurio (em segundos).
   */
  private int userTime;
  /**
   * Tempo em modo sistema (em segundos).
   */
  private int cpuTime;

  /**
   * Cria um evento de comando.
   * 
   * @param commandId O identificador do comando.
   * @param type O tipo do comando.
   */
  public CommandEvent(Object commandId, EventType type) {
    this(commandId, type, -1, -1, -1);
  }

  /**
   * Cria um evento.
   * 
   * @param commandId Identificador do comando
   * @param type O tipo do evento.
   * @param elapsedTime Tempo de parede (em segundos).
   * @param userTime Tempo em modo usurio (em segundos).
   * @param cpuTime Tempo em modo sistema (em segundos).
   */
  public CommandEvent(Object commandId, EventType type, int elapsedTime,
    int userTime, int cpuTime) {
    this.commandId = commandId;
    this.type = type;
    this.elapsedTime = elapsedTime;
    this.userTime = userTime;
    this.cpuTime = cpuTime;
  }

  /**
   * Obtem o identificador do comando.
   * 
   * @return .
   */
  public Object getCommandId() {
    return this.commandId;
  }

  /**
   * Obtm o tipo do evento.
   * 
   * @return O tipo do evento.
   */
  public EventType getType() {
    return this.type;
  }

  /**
   * Obtm o tempo de parede.
   * 
   * @return O tempo de parede.
   */
  public int getElapsedTime() {
    return this.elapsedTime;
  }

  /**
   * Obtm o tempo do usurio.
   * 
   * @return O tempo do usurio.
   */
  public int getUserTime() {
    return this.userTime;
  }

  /**
   * Obtm o tempo do sistema.
   * 
   * @return O tempo do sistema.
   */
  public int getCpuTime() {
    return this.cpuTime;
  }
}
