/*
 * CommandInfo.java $author$ $date$ $revision$
 */
package csbase.logic;

import java.io.Serializable;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import csbase.logic.algorithms.AlgorithmConfigurator;
import csbase.logic.algorithms.ExecutionType;
import csbase.remote.ClientRemoteLocator;
import csbase.server.services.schedulerservice.Scheduler;

/**
 * Representa as informas bsicas aos comandos.
 * 
 * @author Bruno Oliveira Silvestre
 */
public class CommandInfo implements Serializable {
  private static final String ABBREV_DEFAULT = "abbr";
  /** Tamanho do campo de identificacao do algoritmo. */
  private static final int ABBREV_LENGTH = 4;
  private static final String SYSNAME_DEFAULT = "C";
  /** Tamanho do campo de identificacao do sistema. */
  private static final int SYSNAME_LENGTH = 1;
  private static final String USERID_DEFAULT = "user";
  /** Tamanho do campo de identificacao do usuario que disparou o comando. */
  private static final int USERID_LENGTH = 4;

  /** Configurador do algoritmo. No  transiente para poder ir pro cliente. */
  private AlgorithmConfigurator configurator;

  /** Descrio para o comando. */
  private String description;
  /**
   * Posio global do comando na fila.  usado apenas para leitura, quando um
   * cliente pede todos os seus comandos. No  usado para controle.
   */
  private int globalPosition;
  /** Identificador de comando (nico) para efeito de log e auditagem. */
  private String id;
  /** Indica se um e-mail de aviso ser enviado ao trmino do comando. */
  private boolean mailAtEnd;
  /** Array com endereos que devem receber um e-mail de aviso ao trmino do comando. */
  private String[] emailList;

  /** Nmero de execues que sero disparadas na execuo mltipla. */
  private Integer executionCountForMultipleExecution;

  /**
   * O nmero de execues por sga que ser utilizado na execuo mltipla
   * manual.
   */
  private Integer executionCountPerSGAForMultipleExecution;

  /** Quantos sgas vo executar o comando na execuo distribuda. */
  private Integer sgaCountForDistributedExecution;

  /**
   * O nmero de execues por sga que ser utilizado na execuo distribuda
   * manual.
   */
  private Integer executionCountPerSGAForDistributedExecution;

  /** O nmero de cpus que sero utilizados na execuo paralela. */
  private Integer cpuCountForParallelExecution;

  /*
   * Comeo dos dados utilizados para a monitorao da execuo do comando.
   */
  /** O total de CPU utilizada por todos os processos do comando. */
  private Double cpuPerc;
  /** O total de tempo de CPU utilizado por todos os processos do comando. */
  private Double cpuTimeSec;
  /** O total de memria RAM utilizada por todos os processos do comando. */
  private Double ramMemoryMB;
  /** O percentual de memria RAM utilizada por todos os processos do comando. */
  private Double ramMemoryPerc;
  /** O total de memria Swap utilizada por todos os processos do comando. */
  private Double swapMemoryMB;
  /** O total de memria Swap utilizada por todos os processos do comando. */
  private Double swapMemoryPerc;
  /** As informaes especficas do comando. */
  private Map<String, String> specificData;
  /** O tempo total de execuo do processo principal. */
  private Integer wallTimeSec;
  /** Uma descrio dos ns que executam o comando. */
  private String executionNodeDescription;
  /** Tempo de execuo do processo no nvel de usurio */
  private Double userTimeSec;
  /** Tempo de execuo do processo no nvel do kernel */
  private Double systemTimeSec;
  /** Quantidade de memria virtual ocupada pelo processo */
  private Double virtualMemorySizeMB;
  /** Quantidade de bytes recebidos na interface de rede */
  private Double bytesInKB;
  /** Quantidade de bytes enviados pela interface de rede */
  private Double bytesOutKB;
  /** Quantidade de bytes lidos no disco */
  private Double diskBytesReadKB;
  /** Quantidade de bytes escritos no disco */
  private Double diskBytesWriteKB;

  /*
   * Fim dos dados utilizados para a monitorao da execuo do comando.
   */

  /** Parmetros escolhidos pelo usurio. */
  private Map<Object, Object> parameters;
  /** Prioridade do comando. */
  private Priority priority;
  /** Identificador do projeto. */
  private Object projectId;
  /** Nome do SGA que executa o comando. */
  private String sgaName;
  /** Servidores selecionados pelo usurio para a execuo do comando. */
  private List<String> selectedSGAsNames;
  /** Estado do comando. */
  private CommandStatus status;
  /**
   * O que levou esse comando ao estado de {@link CommandStatus#FINISHED} ou
   * {@link CommandFinalizationType#NOT_FINISHED} caso este comando ainda no
   * tenha terminado de ser executado.
   */
  private CommandFinalizationType finalizationType;
  /** Indica a data e hora que o comando foi submetido para a fila de execuo */
  private Date submittedDate;
  /** Descrio do algoritmo. */
  private String tip;
  /** Identificador do usurio que executou o comando */
  private Object userId;
  /** Filtro de plataformas para a execuo do comando. */
  private String platformFilter;
  /** Indica se a escolha de servidores  automtica. */
  private boolean isAutomatic;
  /** Define o tipo de execuo: simples, paralela ou distribuda. */
  private ExecutionType executionType;
  /** Indica se as informaes dinmicas do comando esto vlidas. */
  private boolean isValid;
  /**
   * Indica se o comando est esperando execuo na fila de um escalonador
   * externo ao CSBase (atualmente, esse atributo s  usado no caso do PBS).
   * Essa flag _no_ se refere  fila do {@link Scheduler}). Para saber se o
   * comando est enfileirado no {@link Scheduler}, verificar se o status do
   * comando  {@link CommandStatus#SCHEDULED} . Essa flag deve ser utilizada
   * somente por comandos com status {@link CommandStatus#EXECUTING} (que j
   * foram disparados pelo SGA), para que se diferencie os que realmente esto
   * rodando dos que esto parados na fila de um escalonador sobre o qual o
   * CSBase no tem controle.
   */
  private boolean isQueued;

  /**
   * Informaes de progresso cacheadas para uso na tabela de monitorao.
   * {@link #getProgressData()}
   */
  private transient Object[] cachedProgressData;

  /**
   * Cria um comando obtido a partir dos dados de persistncia.
   * 
   * @param id Identificador de comando (nico) para efeito de log e auditagem.
   * @param globalPosition Posio global do comando na fila.
   * @param mailAtEnd Indica se um e-mail de aviso ser enviado ao trmino do
   *        comando.
   * @param projectId Identificador do projeto no qual o comando foi submetido.
   * @param submittedDate Data de submisso do comando.
   * @param userId Usurio que submeteu o comando.
   * @param priority Prioridade atual do comando.
   * @param status Estado atual do comando.
   * @param finalizationType Indica o que fez ocm que o comando terminasse ou
   *        que ele ainda no terminou.
   * @param tip Dica sobre a descrio do algoritmo.
   */
  public CommandInfo(String id, int globalPosition, boolean mailAtEnd,
    Object projectId, Date submittedDate, Object userId, Priority priority, 
    CommandStatus status, CommandFinalizationType finalizationType, String tip) {
    if (null == finalizationType) {
      try {
        throw new IllegalArgumentException("finalizationType");
      }
      catch (Exception e) {
        e.printStackTrace();
      }
    }
    this.id = id;
    this.projectId = projectId;
    this.globalPosition = globalPosition;
    this.mailAtEnd = mailAtEnd;
    this.emailList = null;
    this.submittedDate = submittedDate;
    this.userId = userId;
    this.priority = priority;
    setStatus(status, finalizationType);
    this.selectedSGAsNames = new ArrayList<String>();
    this.tip = tip;
    this.isValid = true;
    this.isQueued = false;
    configureSimpleExecution();
  }

  /**
   * Construtor de um ComandInfo.
   * 
   * @param configurator Configurador do algoritmo. (No pode ser {@code null}.)
   * @param submission Dados da submisso do comando.
   * @param serverName Nome do servidor.
   * @param userId Identificador do usurio que submeteu o comando.
   */
  public CommandInfo(AlgorithmConfigurator configurator,
    CommandSubmission submission, String serverName, String userId) {
    this.configurator = configurator;
    this.projectId = submission.getProjectId();
    /* Obtm a abreviao do algoritmo. */
    String abbrev = configurator.getAbbreviation();
    if (abbrev == null) {
      /* Obtm o nome do projeto desse comando. */
      abbrev = projectId.toString();
    }
    this.id = createId(userId.toString(), abbrev, serverName);
    this.description = submission.getDescription();
    this.priority = submission.getPriority();
    this.mailAtEnd = submission.isMailAtEnd();
    this.emailList = submission.getEmailList();
    this.submittedDate = submission.getSubmittedDate();
    this.executionCountForMultipleExecution =
      submission.getExecutionCountForMultipleExecution();
    this.executionCountPerSGAForMultipleExecution =
      submission.getExecutionCountPerSGAForMultipleExecution();
    this.executionCountPerSGAForDistributedExecution =
      submission.getExecutionCountPerSGAForDistributedExecution();
    this.sgaCountForDistributedExecution =
      submission.getSgaCountForDistributedExecution();
    this.cpuCountForParallelExecution =
      submission.getCpuCountForParallelExecution();
    this.isAutomatic = submission.isAutomatic();
    this.platformFilter = submission.getPlatform();
    this.selectedSGAsNames = new ArrayList<String>(submission.getSGANames());
    this.executionType = submission.getExecutionType();
    this.tip = configurator.toString();
    this.userId = userId;
    setStatus(CommandStatus.SCHEDULED, CommandFinalizationType.NOT_FINISHED);
    this.isValid = true;
    this.isQueued = false;
  }

  /**
   * Invalida os dados dinmicos do comando. Deve ser usado quando a conexo com
   * o servidor cai e os dados no so mais vlidos.
   */
  public void invalidateDynamicFields() {
    cpuPerc = null;
    cpuTimeSec = null;
    ramMemoryPerc = null;
    swapMemoryPerc = null;
    wallTimeSec = null;
    executionNodeDescription = null;
    specificData = null;
    userTimeSec = null;
    systemTimeSec = null;
    virtualMemorySizeMB = null;
    bytesInKB = null;
    bytesOutKB = null;
    diskBytesReadKB = null;
    diskBytesWriteKB = null;
    isValid = false;
    isQueued = false;
  }

  /**
   * Indica se o comando est preso na fila de um escalonador externo ao CSBase.
   * 
   * @return verdadeiro se o comando est na fila do escalonador externo ou
   *         falso, caso contrrio.
   * @see #isQueued
   */
  public boolean isQueued() {
    return isQueued;
  }

  /**
   * Determina se o comando est preso na fila de um escalonador externo ao
   * CSBase.
   * 
   * @param isQueued verdadeiro se o comando est na fila do escalonador externo
   *        ou falso, caso contrrio.
   * @see #isQueued
   */
  public void setQueued(boolean isQueued) {
    this.isQueued = isQueued;
  }

  /**
   * Vlida os dados dinmicos do comando.
   */
  public void validateDynamicFields() {
    isValid = true;
  }

  /*
   * Comeo dos mtodos utilizados para a monitorao da execuo do comando.
   */
  /**
   * Obtm o total de CPU utilizada por todos os processos do comando.
   * 
   * @return o total de CPU utilizada por todos os processos do comando.
   */
  public Double getCpuPerc() {
    return cpuPerc;
  }

  /**
   * Define o total de CPU utilizada por todos os processos do comando.
   * 
   * @param cpuPerc o total de CPU utilizada por todos os processos do comando.
   */
  public void setCpuPerc(Double cpuPerc) {
    this.cpuPerc = cpuPerc;
  }

  /**
   * Otm o tempo total de CPU, em segundos, utilizado por todos os processos 
   * do comando. 
   * 
   * @return o tempo total de CPU, em segundos, utilizadao por todos os processos 
   * do comando.
   */   
  public Double getCpuTimeSec() {
    return cpuTimeSec;
  }
     
  /**
   * Define o tempo total de CPU, em segundos, utilizado por todos os processos 
   * do comando. 
   * 
   * @param cpuTimeSec o tempo total de CPU, em segundos, utilizado por todos 
   * os processos do comando.
   */  
  public void setCpuTimeSec(Double cpuTimeSec) {
    this.cpuTimeSec = cpuTimeSec;
  }  
  
  /**
   * Obtm o total de memria RAM utilizada por todos os processos do comando.
   * 
   * @return o total de memria RAM utilizada por todos os processos do comando.
   */
  public Double getRAMMemoryMB() {
    return ramMemoryMB;
  }

  /**
  * Define o total de memria RAM utilizada por todos os processos do 
  * comando.
  * 
  * @param ramMemoryMB o total de memria RAM utilizada por todos os 
  * processos do comando. 
  */    
  public void setRAMMemoryMB(Double ramMemoryMB) {
    this.ramMemoryMB = ramMemoryMB;
  }
   
  /**
  * Obtm o percentual de memria RAM utilizada por todos os processos do comando.
  * 
  * @return o percentual de memria RAM utilizada por todos os processos do comando.
  */
  public Double getRAMMemoryPerc() {
    return ramMemoryPerc;
  }

  /**
   * Define o percentual de memria RAM utilizada por todos os processos do comando.
   * 
   * @param ramMemoryPerc o total de memria RAM utilizada por todos os
   *        processos do comando.
   */
  public void setRAMMemoryPerc(Double ramMemoryPerc) {
    this.ramMemoryPerc = ramMemoryPerc;
  }

  
  /**
   * Obtm o total de memria Swap utilizada por todos os processos do comando.
   * 
   * @return o total de memria Swap utilizada por todos os processos do comando.
   */
  public Double getSwapMemoryMB() {
    return swapMemoryMB;
  }

  /**
  * Define o total de memria Swap utilizada por todos os processos do 
  * comando.
  * 
  * @param swapMemoryMB o total de memria Swap utilizada por todos os 
  * processos do comando. 
  */    
  public void setSwapMemoryMB(Double swapMemoryMB) {
    this.swapMemoryMB = swapMemoryMB;
  }
   
  /**
   * Obtm o total de memria Swap utilizada por todos os processos do comando.
   * 
   * @return o total de memria Swap utilizada por todos os processos do
   *         comando.
   */
  public Double getSwapMemoryPerc() {
    return swapMemoryPerc;
  }

  /**
   * Define o total de memria Swap utilizada por todos os processos do comando.
   * 
   * @param swapMemoryPerc o total de memria Swap utilizada por todos os
   *        processos do comando.
   */
  public void setSwapMemoryPerc(Double swapMemoryPerc) {
    this.swapMemoryPerc = swapMemoryPerc;
  }

  /**
   * Obtm as informaes especficas do comando.
   * 
   * @return as informaes especficas do comando.
   */
  public Map<String, String> getSpecificData() {
    return specificData;
  }

  /**
   * Define as informaes especficas do comando.
   * 
   * @param specificData as informaes especficas do comando.
   */
  public void setSpecificData(Map<String, String> specificData) {
    this.specificData = specificData;
  }

  /**
   * Obtm o tempo total de execuo do processo principal.
   * 
   * @return o tempo total de execuo do processo principal.
   */
  public Integer getWallTimeSec() {
    return wallTimeSec;
  }

  /**
   * Define o valor do tempo total de execuo do processo principal.
   * 
   * @param wallTimeSec o valor do tempo total de execuo do processo
   *        principal.
   */
  public void setWallTimeSec(Integer wallTimeSec) {
    this.wallTimeSec = wallTimeSec;
  }
  
  /**
   * Otm o tempo total de execuo no nvel do kernel para todos os processos 
   * do comando. 
   * 
   * @return o tempo de execuo no nvel do kernel para todos os processos do 
   *         comando.
   */    
  public Double getSystemTimeSec() {
    return systemTimeSec;
  }
   
  /**
   * Define o valor de tempo de execuo no nvel do kernel para todos os 
   * processos do comando.
   * 
   * @param systemTimeSec o valor do tempo de execuo no nvel do kernel para
   *        todos os processos do comando.
   */     
  public void setSystemTimeSec(Double systemTimeSec) {
    this.systemTimeSec = systemTimeSec;
  }
  
  /**
   * Otm a quantidade de memria virtual usada por todos os processos do comando. 
   * 
   * @return a quantidade de memria virtual usada por todos os processos do 
   *         comando.
   */ 
  public Double getVirtualMemorySizeMB() {
    return virtualMemorySizeMB;
  }
  
  /**
   * Define o valor da quantidade de memria virtual usada por todos os 
   * processos do comando.
   * 
   * @param virtualMemorySizeMB o valor da quantidade de memria virtual usada 
   * por todos os processos do comando.
   */ 
  public void setVirtualMemorySizeMB(Double virtualMemorySizeMB) {
    this.virtualMemorySizeMB = virtualMemorySizeMB;
  }
  
  /**
   * Otm a quantidade de bytes recebidos na interface de rede. 
   * 
   * @return a quantidade de bytes recebidos na interface de rede.
   */ 
  public Double getBytesInKB() {
    return bytesInKB;
  }
  
  /**
   * Define a quantidade de bytes recebidos na interface de rede.
   * 
   * @param bytesInKB a quantidade de bytes recebidos na interface de rede.
   */ 
  public void setBytesInKB(Double bytesInKB) {
    this.bytesInKB = bytesInKB;
  }
  
  /**
   * Otm a quantidade de bytes enviados pela interface de rede. 
   * 
   * @return a quantidade de bytes enviados pela interface de rede.
   */ 
  public Double getBytesOutKB() {
    return bytesOutKB;
  }
  
  /**
   * Define a quantidade de bytes enviados pela interface de rede.
   * 
   * @param bytesOutKB a quantidade de bytes enviados pela interface de rede.
   */ 
  public void setBytesOutKB(Double bytesOutKB) {
    this.bytesOutKB = bytesOutKB;
  }
  
  /**
   * Otm a quantidade de bytes lidos do disco.  
   * 
   * @return a quantidade de bytes lidos do disco. 
   */ 
  public Double getDiskBytesReadKB() {
    return diskBytesReadKB;
  }
  
  /**
   * Define a quantidade de bytes lidos do disco.
   * 
   * @param diskBytesReadKB a quantidade de bytes lidos do disco.
   */ 
  public void setDiskBytesReadKB(Double diskBytesReadKB) {
    this.diskBytesReadKB = diskBytesReadKB;
  }
    
  /**
   * Otm a quantidade de bytes escritos em disco.  
   * 
   * @return a quantidade de bytes escritos em disco.
   */ 
  public Double getDiskBytesWriteKB() {
    return diskBytesWriteKB;
  }
    
  /**
   * Define a quantidade de bytes escritos em disco.
   * 
   * @param diskBytesWriteKB a quantidade de bytes escritos em disco.
   */ 
  public void setDiskBytesWriteKB(Double diskBytesWriteKB) {
    this.diskBytesWriteKB = diskBytesWriteKB;
  }
  
  /**
   * Define o valor de tempo de execuo no nvel do usurio para o processo 
   * principal.
   * 
   * @param userTimeSec o valor de tempo de execuo no nvel do usurio para 
   * o processo principal.
   */ 
  public void setUserTimeSec(Double userTimeSec) {
    this.userTimeSec = userTimeSec;
  }
  
  /**
   * Obtm o valor de tempo de execuo no nvel do usurio para o processo 
   * principal.
   * 
   * @return o valor de tempo de execuo no nvel do usurio para o processo 
   * principal.
   */ 
  public Double getUserTimeSec() {
    return this.userTimeSec;
  }

  /**
   * Obtm uma descrio dos ns que executam o comando.
   * 
   * @return uma descrio dos ns que executam o comando.
   */
  public String getExecutionNodeDescription() {
    return executionNodeDescription;
  }

  /**
   * Define uma descrio dos ns que executam o comando.
   * 
   * @param executionNodeDescription uma descrio dos ns que executam o
   *        comando.
   */
  public void setExecutionNodeDescription(String executionNodeDescription) {
    this.executionNodeDescription = executionNodeDescription;
  }

  /*
   * Fim dos mtodos utilizados para a monitorao da execuo do comando.
   */

  public Integer getSgaCountForDistributedExecution() {
    return sgaCountForDistributedExecution;
  }

  public void setSgaCountForDistributedExecution(
    Integer sgaCountForDistributedExecution) {
    this.sgaCountForDistributedExecution = sgaCountForDistributedExecution;
  }

  public String getSgaName() {
    return sgaName;
  }

  public void setSgaName(String sgaName) {
    this.sgaName = sgaName;
  }

  public void setId(String id) {
    this.id = id;
  }

  public void setMailAtEnd(boolean mailAtEnd) {
    this.mailAtEnd = mailAtEnd;
  }

  public void setExecutionCountForMultipleExecution(
    Integer executionCountForMultipleExecution) {
    this.executionCountForMultipleExecution =
      executionCountForMultipleExecution;
  }

  public void setExecutionCountPerSGAForMultipleExecution(
    Integer executionCountPerSGAForMultipleExecution) {
    this.executionCountPerSGAForMultipleExecution =
      executionCountPerSGAForMultipleExecution;
  }

  public void setExecutionCountPerSGAForDistributedExecution(
    Integer executionCountPerSGAForDistributedExecution) {
    this.executionCountPerSGAForDistributedExecution =
      executionCountPerSGAForDistributedExecution;
  }

  public void setCpuCountForParallelExecution(
    Integer cpuCountForParallelExecution) {
    this.cpuCountForParallelExecution = cpuCountForParallelExecution;
  }

  public void setParameters(Map<Object, Object> parameters) {
    this.parameters = parameters;
  }

  public void setProjectId(Object projectId) {
    this.projectId = projectId;
  }

  public void setSubmittedDate(Date submittedDate) {
    this.submittedDate = submittedDate;
  }

  public void setUserId(Object userId) {
    this.userId = userId;
  }

  public void setAutomatic(boolean isAutomatic) {
    this.isAutomatic = isAutomatic;
  }

  public void setExecutionType(ExecutionType executionType) {
    this.executionType = executionType;
  }

  /**
   * Define as informaes sobre um comando distribudo.
   * 
   * @param sgaCount Nmero de servidores que participaro da execuo
   *        distribuda. Usado na seleo automtica.
   */
  public void configureDistributedExecution(int sgaCount) {
    executionType = ExecutionType.DISTRIBUTED;
    isAutomatic = true;
    executionCountPerSGAForDistributedExecution = 1;
    sgaCountForDistributedExecution = sgaCount;
  }

  /**
   * Configura os dados da execuo distribuda.
   * 
   * @param sgaNames Nomes dos sgas selecionados pelo usurio.
   * @param executionCountPerSGA Nmero de processos que sero disparados em
   *        cada um dos servidores selecionados. Usado na seleo manual.
   */
  public void configureDistributedExecution(List<String> sgaNames,
    int executionCountPerSGA) {
    executionType = ExecutionType.DISTRIBUTED;
    isAutomatic = false;
    setSelectedSGAsNames(sgaNames);
    sgaCountForDistributedExecution = selectedSGAsNames.size();
    executionCountPerSGAForDistributedExecution = executionCountPerSGA;
  }

  /**
   * Configura os dados da execuo mltipla.
   * 
   * @param executionCount Nmero de execues que sero disparadas.
   */
  public void configureMultipleExecution(int executionCount) {
    executionType = ExecutionType.MULTIPLE;
    isAutomatic = true;
    executionCountForMultipleExecution = executionCount;
  }

  /**
   * Configura os dados da execuo mltipla.
   * 
   * @param sgaNames Nomes dos sgas selecionados pelo usurio.
   * @param executionCountPerSGA Nmero de execues por servidor.
   */
  public void configureMultipleExecution(List<String> sgaNames,
    int executionCountPerSGA) {
    executionType = ExecutionType.MULTIPLE;
    isAutomatic = false;
    setSelectedSGAsNames(sgaNames);
    executionCountPerSGAForMultipleExecution = executionCountPerSGA;
    executionCountForMultipleExecution =
      executionCountPerSGA * selectedSGAsNames.size();
  }

  /**
   * Configura os dados da execuo paralela automtica.
   * 
   * @param cpuCount O nmero de cpus que sero utilizadas.
   */
  public void configureParallelExecution(int cpuCount) {
    executionType = ExecutionType.PARALLEL;
    isAutomatic = true;
    cpuCountForParallelExecution = cpuCount;
  }

  /**
   * Configura os dados da execuo paralela manual.
   * 
   * @param sgaName Nome do sgas selecionados pelo usurio.
   * @param cpuCount O nmero de cpus que sero utilizadas.
   */
  public void configureParallelExecution(String sgaName, int cpuCount) {
    executionType = ExecutionType.PARALLEL;
    isAutomatic = false;
    setSelectedSGAName(sgaName);
    cpuCountForParallelExecution = cpuCount;
  }

  /**
   * Configura os dados da execuo simples.
   */
  public void configureSimpleExecution() {
    executionType = ExecutionType.SIMPLE;
    isAutomatic = true;
  }

  /**
   * Configura os dados da execuo simples.
   * 
   * @param sgaName Nome do sgas selecionado pelo usurio.
   */
  public void configureSimpleExecution(String sgaName) {
    executionType = ExecutionType.SIMPLE;
    isAutomatic = false;
    setSelectedSGAName(sgaName);
  }

  /**
   * Configura os dados de aplicaes X.
   */
  public void configureXAppExecution() {
    executionType = ExecutionType.XAPPS;
    isAutomatic = true;
  }

  /**
   * Configura os dados de aplicaes X.
   * 
   * @param sgaName Nome do sgas selecionado pelo usurio.
   */
  public void configureXAppExecution(String sgaName) {
    executionType = ExecutionType.XAPPS;
    isAutomatic = false;
    setSelectedSGAName(sgaName);
  }

  /**
   * Mtodo para comparar objetos da mesma classe.
   * 
   * @param obj Objeto a ser comparado.
   * @return Verdadeiro se ambos forem iguais.
   */
  @Override
  public boolean equals(Object obj) {
    if (!(obj instanceof CommandInfo)) {
      return false;
    }

    if (id == null)
      return false;

    CommandInfo other = (CommandInfo) obj;
    return id.equals(other.id);
  }

  /**
   * Obtm o configurador do algoritmo.
   * 
   * @return .
   * @throws RemoteException falha de rmi
   */
  public AlgorithmConfigurator getConfigurator() throws RemoteException {
    if (configurator == null) {
      // Obtm o configurador no servio de persistncia.
      configurator =
        ClientRemoteLocator.commandPersistenceService
          .readAlgorithmConfigurator(getProjectId(), getId());
    }
    return configurator;
  }

  /**
   * Obtm a descrio do comando.
   * 
   * @return .
   */
  public String getDescription() {
    return description;
  }

  /**
   * Obtm o tipo da execuo do comando {@link ExecutionType}.
   * 
   * @return .
   */
  public ExecutionType getExecutionType() {
    return executionType;
  }

  /**
   * Obtm a posio do comando na fila do escalonador.
   * 
   * @return .
   */
  public int getGlobalPosition() {
    return globalPosition;
  }

  /**
   * Obtm o identificador do comando.
   * 
   * @return .
   */
  public String getId() {
    return id;
  }

  /**
   * Obtm os parmetros selecionados pelo usurio na submisso.
   * 
   * @return mapa um mapa com pares nome, parmetro.
   */
  public Map<Object, Object> getParameters() {
    return this.parameters;
  }

  /**
   * Obtm a prioridade do comando.
   * 
   * @return .
   */
  public Priority getPriority() {
    return priority;
  }

  /**
   * Obtm o identificador do projeto no qual o comando foi submetido.
   * 
   * @return .
   */
  public Object getProjectId() {
    return projectId;
  }

  /**
   * Retorna o nome do SGA que executa o comando. Pode ser {@code null} quando o
   * comando ainda est na fila do escalonador e  automtico.
   * 
   * @return .
   */
  public String getSGAName() {
    return sgaName;
  }

  /**
   * Retorna os nomes dos SGAs que foram selecionados pelo usurio. Usado no
   * caso da execuo paralela onde submetemos apenas um comando, mas passamos
   * para o algoritmo uma lista com os servidores de algoritmos selecionados.
   * 
   * @return sgas Os nomes dos SGAs que foram selecionados pelo usurio.
   */
  public List<String> getSelectedSGAsNames() {
    return Collections.unmodifiableList(selectedSGAsNames);
  }

  /**
   * Define quais os SGAs que sero usados neste comando.
   * 
   * @param selectedSGAsNames
   */
  public void setSelectedSGAsNames(List<String> selectedSGAsNames) {
    this.selectedSGAsNames.clear();
    this.selectedSGAsNames.addAll(selectedSGAsNames);
  }

  /**
   * Obtm o estado do comando.
   * 
   * @return .
   */
  public CommandStatus getStatus() {
    return status;
  }

  /**
   * Obtm o que fez com que esse comando fosse para o estado
   * {@link CommandStatus#FINISHED} ou
   * {@link CommandFinalizationType#NOT_FINISHED} caso este comando ainda no
   * tenha terminado.
   * 
   * @return o que fez com que esse comando fosse para o estado
   *         {@link CommandStatus#FINISHED} ou
   *         {@link CommandFinalizationType#NOT_FINISHED} caso este comando
   *         ainda no tenha terminado.
   */
  public CommandFinalizationType getFinalizationType() {
    if (null == finalizationType) {
      return CommandStatus.FINISHED == this.status ? CommandFinalizationType.UNKNOWN
        : CommandFinalizationType.NOT_FINISHED;
    }
    else {
      return finalizationType;
    }
  }

  /**
   * Obtm a data em que o comando foi submetido.
   * 
   * @return .
   */
  public Date getSubmittedDate() {
    return submittedDate;
  }

  /**
   * Obtm o nmero de execues que sero disparadas na execuo mltipla.
   * 
   * @return nmero de execues
   */
  public Integer getExecutionCountForMultipleExecution() {
    return executionCountForMultipleExecution;
  }

  /**
   * Obtm o nmero de execues por sga que ser utilizado na execuo mltipla
   * manual.
   * 
   * @return .
   */
  public Integer getExecutionCountPerSGAForMultipleExecution() {
    return executionCountPerSGAForMultipleExecution;
  }

  /**
   * Obtm o nmero de sgas que vo executar o comando na execuo distribuda.
   * 
   * @return nmero de sgas.
   */
  public Integer getSGACountForDistributedExecution() {
    return sgaCountForDistributedExecution;
  }

  /**
   * Obtm o nmero de execues por sga que ser utilizado na execuo
   * distribuda manual.
   * 
   * @return nmero de execues por sga.
   */
  public Integer getExecutionCountPerSGAForDistributedExecution() {
    return executionCountPerSGAForDistributedExecution;
  }

  /**
   * Obtm o nmero de cpus que sero utilizados na execuo paralela.
   * 
   * @return nmero de cpus.
   */
  public Integer getCpuCountForParallelExecution() {
    return cpuCountForParallelExecution;
  }

  /**
   * Obtm o identificador do usurio.
   * 
   * @return .
   */
  public Object getUserId() {
    return userId;
  }

  /**
   * Calcula o cdigo hash do objeto.
   * 
   * @return Inteiro que representa o cdigo hash do objeto.
   */
  @Override
  public int hashCode() {
    if (id == null)
      return "CommandInfoWhitoutId".hashCode();
    return id.hashCode();
  }

  /**
   * Indica se o sistema deve enviar um email para o usurio que submeteu o
   * comando quando este terminar.
   * 
   * @return true se deve enviar o email e false caso contrrio.
   */
  public boolean isMailAtEnd() {
    return mailAtEnd;
  }
  
  /**
   * Obtm a lista de endereos que devem receber email de aviso de trmino
   * de execuo do comando.
   * Os emails somente so enviados se estiver configurado para envio de emails.
   * @see #setMailAtEnd(boolean)
   * @return o array com endereos de email
   */
  public String[] getEmailList() {
    return emailList;
  }

  /**
   * Mtodo criado para auxiliar nos testes.
   * 
   * @return texto Texto com informaes do comando.
   */
  public String print() {
    StringBuffer buffer = new StringBuffer();
    buffer.append("id: " + id + "\n");
    buffer.append("tip: " + getTip() + "\n");
    buffer.append("description: " + description + "\n");
    buffer.append("sgaName: " + sgaName + "\n");
    buffer.append("userId: " + userId + "\n");
    return buffer.toString();
  }

  /**
   * Altera a posio do comando na fila do escalonador.
   * 
   * @param globalPosition a posio do comando na fila do escalonador.
   */
  public void setGlobalPosition(int globalPosition) {
    this.globalPosition = globalPosition;
  }

  /**
   * Altera a prioridade do comando.
   * 
   * @param priority a prioridade do comando.
   */
  public void setPriority(Priority priority) {
    this.priority = priority;
  }

  /**
   * Altera o nome do SGA para qual o comando ser submetido.
   * 
   * @param sgaName Nome do SGA.
   */
  public void setSGAName(String sgaName) {
    this.sgaName = sgaName;
  }

  /**
   * Altera o estado do comando.<br>
   * Se este mtodo for utilizado para atribuir o estado
   * {@link CommandStatus#FINISHED terminado} ao comando, ele ir alterar o seu
   * {@link CommandFinalizationType} para
   * {@link CommandFinalizationType#UNKNOWN}. Para esta finalidade,  prefervel
   * que se utilize o mtodo {@link #setFinished(CommandFinalizationType)}.
   * 
   * @param status O estado do comando {@link CommandStatus}.
   * @see #setFinished(CommandFinalizationType)
   */
  public void setStatus(CommandStatus status) {
    if (CommandStatus.FINISHED == status) {
      setStatus(status, CommandFinalizationType.UNKNOWN);
    }
    else {
      setStatus(status, CommandFinalizationType.NOT_FINISHED);
    }
  }

  /**
   * Altera o estado do comando para {@link CommandStatus#FINISHED terminado} e
   * atribui um valor ao {@link CommandFinalizationType modo em que este comando
   * foi finalizado}.
   * 
   * @param finalizationType O que levou o comando a terminar.
   */
  public void setFinished(CommandFinalizationType finalizationType) {
    setStatus(CommandStatus.FINISHED, finalizationType);
  }

  /**
   * Indica se as informaes dinmicas do comando esto vlidas.
   * 
   * @return verdadeiro se as informaes dinmicas do comando esto vlidas e
   *         falso caso contrrio.
   */
  public boolean isValid() {
    return isValid;
  }

  /**
   * Converte para uma string.
   * 
   * @return o nome da mquina e o nome do comando.
   */
  @Override
  public String toString() {
    return id + " ( " + getTip() + " )";
  }

  /**
   * Altera o estado e a indicao de se esse comando foi terminado e por que.<br>
   * O {@code finalizationType} dever ser
   * {@link CommandFinalizationType.NOT_FINISHED} exclusivamente no caso do
   * {@code status} ser {@link CommandStatus.FINISHED}.
   * 
   * @param status O estado do comando.
   * @param finalizationType O que levou o comando a terminar ou
   *        {@link CommandFinalizationType.NOT_FINISHED} caso ele ainda no
   *        tenha terminado.
   */
  private void setStatus(CommandStatus status,
    CommandFinalizationType finalizationType) {
    if (null == status) {
      throw new IllegalArgumentException(String.format(
        "Atribuindo estado nulo ao comando %s.", id));
    }
    if (null == finalizationType) {
      throw new IllegalArgumentException(String.format(
        "Atribuindo estado de trmino nulo ao comando %s.", id));
    }
    if ((CommandStatus.FINISHED == status && CommandFinalizationType.NOT_FINISHED == finalizationType)
      || (CommandStatus.FINISHED != status && CommandFinalizationType.NOT_FINISHED != finalizationType)) {
      throw new IllegalArgumentException(
        String
          .format(
            "Atribuindo estado inconsistente ao comando %s. CommandStatus: %s e CommandFinalisationType: %s",
            id, status, finalizationType));
    }
    this.status = status;
    this.finalizationType = finalizationType;
  }

  /**
   * Criao de um identificador de arquivo nico para auditagem
   * 
   * @param userId Identificador do usurio
   * @param abbrev Abreviao que identifica o algoritmo.
   * @param sysName Nome do sistema (WebSintesi, InfoGrid, por exemplo)
   * @return uma string que representa um nmero nico.
   */
  private String createId(String userId, String abbrev, String sysName) {
    return String.format("%s@%s.%s%s", formatField(userId, USERID_DEFAULT,
      USERID_LENGTH, "0"), formatField(abbrev, ABBREV_DEFAULT, ABBREV_LENGTH,
      "0"), formatField(sysName, SYSNAME_DEFAULT, SYSNAME_LENGTH, "0"),
      new TStamp());
  }

  /**
   * Formata o campo que ser usado no identificador
   * 
   * @param field o campo que ser formatado
   * @param defaultValue usado no caso de field ser null
   * @param defaultLength Tamanho que ser usado para 'truncar' o campo
   * @param defaultChar Caracter que ser usado para completar o campo se ele
   *        for menor que defaultLength.
   * @return field
   */

  private String formatField(String field, String defaultValue,
    int defaultLength, String defaultChar) {
    String result = field != null ? field : defaultValue;
    // Se o tamanho do campo for menor que defaultLength,
    // completa o campo com defaultChar.
    if (result.length() < defaultLength) {
      for (int i = defaultLength - result.length(); i > 0; i--) {
        result = result + defaultChar;
      }
    }
    // Se o tamanho do campo for maior que defaultLength, trunca
    // o campo.
    else {
      result = result.substring(0, defaultLength);
    }
    return result;
  }

  /**
   * Define qual o SGA que foi selecionado pelo usurio.
   * 
   * @param selectedSGAName
   */
  private void setSelectedSGAName(String selectedSGAName) {
    setSelectedSGAsNames(Arrays.asList(selectedSGAName));
  }

  /**
   * Obtm a plataforma na qual o comando dever ser executado.
   * 
   * @return plataforma .
   */
  public String getPlatformFilter() {
    return platformFilter;
  }

  /**
   * Indica se a seleo de servidores de algoritmos dever ser feita pelo
   * escalonador.
   * 
   * @return true se a seleo dever ser feita pelo escalonador e false caso
   *         contrrio.
   */
  public boolean isAutomatic() {
    return isAutomatic;
  }

  /**
   * Obtm uma dica para auxiliar a identificao do algoritmo do comando.
   * 
   * @return a dica do comando. (Pode ser {@code null}.)
   */
  public String getTip() {
    return tip;
  }

  /**
   * Define a descrio do comando.
   * 
   * @param description a descrio do comando.
   */
  public void setDescription(String description) {
    this.description = description;
  }

  /**
   * Define a plataforma na qual o comando dever ser executado.
   * 
   * @param platformFilter a plataforma na qual o comando deve ser executado.
   */
  public void setPlatformFilter(String platformFilter) {
    this.platformFilter = platformFilter;
  }

  /**
   * Obtm o progresso do comando em dois formatos: valor texto original e
   * tambm o valor numrico correspondente, entre 0 e 100%.
   * <ul>
   * <li>Exemplo de retorno: { "1/10", 10 }
   * </ul>
   * <p>
   * Se o texto no tiver um formato numrico conhecido, o valor fica
   * Double.NaN.
   * <ul>
   * <li>Exemplo de retorno: { "Passo 1: lendo arquivo", Double.NaN }
   * </ul>
   * <p>
   * Se o mapa de informaes dinmicas do comando no tiver a chave de
   * progresso total, mas tiver chaves parciais ['progresso.1', 'progresso.2'
   * ...] com valores numricos, calculamos a mdia simples entre os valores.
   * <p>
   * Para que seja extrado um valor numrico de progresso, espera-se que os
   * valores das chaves 'progresso' tenham um dos formatos abaixo:
   * <ul>
   * <li>1/12
   * <li>0.3 [entre 0 e 1]
   * <li>20% [entre 0 e 100]
   * <li><b>Opcionalmente um texto fixo pode ser indicado depois de um '#':</b>
   * <ul>
   * <li>1/12 #Um texto qualquer para o progresso atual
   * <li>10% #Um texto qualquer para o progresso atual
   * <li>0.3 #Um texto qualquer para o progresso atual.<br>
   * Nesses casos o texto retornado  o texto indicado, e no o texto numrico.
   * <ul>
   * <li>Exemplo de retorno: { "Texto aps #", 10 }
   * </ul>
   * </ul>
   * </ul>
   * 
   * @return O progresso do comando em dois formatos: valor texto original e
   *         tambm o valor numrico correspondente, entre 0 e 100%. <br>
   *         Ex: { "1/10", 10 } <br>
   *         <b>Se no houver informaes sobre progresso, esse mtodo retorna
   *         null.</b>
   */
  public Object[] getProgressData() {

    if (cachedProgressData != null)
      return cachedProgressData;

    if (specificData == null)
      return null;

    String progressKey = getProgressKey();
    String progressData = specificData.get(progressKey);
    if (progressData != null) {
      // h um registro para o progresso total ...  ele mesmo ! 
      return cachedProgressData = extractProgressData(progressData.trim());
    }

    // no h registro para o progresso total ... 
    // ver se temos chaves 'parciais' ... Ex: "progresso.1", "progresso.2"
    // adicionar todas na lista para fazer uma mdia depois:
    String partialProgressKeyRegex = progressKey + "\\..+";
    List<Double> values = new ArrayList<Double>();
    Set<Entry<String, String>> entrySet = specificData.entrySet();
    for (Entry<String, String> entry : entrySet) {
      if (entry.getKey().matches(partialProgressKeyRegex)) {
        Object[] data = extractProgressData(entry.getValue().trim());
        if (!data[1].equals(Double.NaN))
          values.add((Double) data[1]);
      }
    }

    if (values.isEmpty()) {
      // no temos informaes vlidas de 'progresso parcial'
      return null;
    }

    // temos N registros vlidos ... tirar a mdia:
    double total = 0;
    for (Double value : values)
      total += value;
    double percent = total / values.size();
    return cachedProgressData = new Object[] { progressData, percent };
  }

  /**
   * Analisa o texto indicado [proveniente de uma chave 'progresso...'] e extrai
   * o progresso do comando em dois formatos: valor texto de progresso e tambm
   * o valor numrico correspondente, entre 0 e 100%.
   * <ul>
   * <li>Exemplo: Texto: "1/10" --- Retorno: { "1/10", 10 }
   * </ul>
   * <p>
   * Se o texto no tiver um formato numrico conhecido, o valor fica
   * Double.NaN.
   * <ul>
   * <li>Exemplo: Texto: "Passo 1: lendo arquivo" --- Retorno: {
   * "Passo 1: lendo arquivo", Double.NaN }
   * </ul>
   * <p>
   * Para que seja extrado um valor numrico de progresso, espera-se que
   * 'progressData' tenha um dos formatos abaixo:
   * <ul>
   * <li>1/12
   * <li>0.3 [entre 0 e 1]
   * <li>20% [entre 0 e 100]
   * <li><b>Opcionalmente um texto fixo pode ser indicado depois de um '#':</b>
   * <ul>
   * <li>1/12 #Um texto qualquer para o progresso atual
   * <li>10% #Um texto qualquer para o progresso atual
   * <li>0.3 #Um texto qualquer para o progresso atual.<br>
   * Nesses casos o texto retornado  o texto indicado, e no o texto que gerou
   * o valor.
   * <ul>
   * <li>Exemplo: Texto: "1/10 #Passo 1" --- Retorno: { "Passo 1", 10 }
   * </ul>
   * </ul>
   * </ul>
   * 
   * @param progressData - string que corresponde ao valor do progresso desse
   *        algoritmo, como "1/10" ou "Passo 1".
   * @return O progresso do comando em dois formatos: valor texto original e
   *         tambm o valor numrico correspondente, entre 0 e 100%. <br>
   *         Ex: { "1/10", 10 }.
   */
  private Object[] extractProgressData(String progressData) {

    String valueText, markedStepText;
    int textMarkerIndex = progressData.indexOf('#');

    if (textMarkerIndex == 0) {
      // '#' no incio ...ento temos apenas texto ...
      return new Object[] { progressData.substring(1), Double.NaN };
    }
    else if (textMarkerIndex > 0) {
      // temos um valor e um texto fixo ... "1/10 #texto"
      valueText = progressData.substring(0, textMarkerIndex).trim();
      markedStepText = progressData.substring(textMarkerIndex + 1).trim();
      Double value = extractProgressValue(valueText);
      return new Object[] { markedStepText, value };
    }

    // no temos um texto fixo marcado por '#' ... o texto ser o valor em %

    Double value = extractProgressValue(progressData);
    if (value.equals(Double.NaN))
      return new Object[] { progressData, value };
    return new Object[] { String.format("%.0f%%", value), value };
  }

  /**
   * Extrai um valor numrico do texto indicado, que corresponde ao valor do
   * progresso desse algoritmo. O valor  convertido para porcentagem [entre 0 e
   * 100]. O texto pode ter um dos formatos abaixo:
   * <ul>
   * <li>"1/12"
   * <li>"0.3" [entre 0 e 1]
   * <li>"20%" [j entre 0 e 100]
   * </ul>
   * 
   * @param progressData - string que corresponde ao valor do progresso desse
   *        algoritmo.
   * @return o valor correspondente, ou Double.NaN para formatos desconhecidos.
   */
  private Double extractProgressValue(String progressData) {
    try {
      if (progressData.endsWith("%")) {
        String valueStr = progressData.substring(0, progressData.length() - 1);
        return Double.parseDouble(valueStr);
      }

      if (progressData.indexOf("/") > 0) {
        String[] stringValues = progressData.split("/");
        double v1 = Integer.parseInt(stringValues[0]);
        double v2 = Integer.parseInt(stringValues[1]);
        return (v1 / v2) * 100.0;
      }

      // no  '%' nem '/' ? Ento  um valor entre 0 e 1:
      return Double.parseDouble(progressData) * 100.0;
    }
    catch (Exception e) {
      // qualquer erro no formato ... nmero fica NaN ...
      return Double.NaN;
    }
  }

  /**
   * @return o valor para a chave que indica o progresso do algoritmo, para uso
   *         no mapa de informaes dinmicas.
   */
  private String getProgressKey() {
    return "progresso";
  }
}
