/*
 * $Id: ExternalCommand.java 74614 2008-04-07 13:42:03Z costa $
 */
package csbase.logic;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.StringReader;

/**
 * Fachada para um processo. Ao contrrio de {@link Process}, garante que
 * <code>waitFor()</code> e <code>exitValue()</code> consomem integralmente
 * a sada do comando (padro e erro) antes de aguardar o trmino da execuo,
 * para evitar deadlocks decorrente de problemas de esgotamento dos buffers do
 * SO. Os streams associados a <code>stdout</code> e <code>stderr</code> so
 * lidos em paralelo, pelo mesmo motivo.
 * <p>
 * Observar que, por default, todo o contedo de <code>stdout</code> e
 * <code>stderr</code>  armazenado internamente. Opcionalmente, pode-se
 * descartar o contedo de cada um dos streams (ou de ambos); ver mtodo
 * {@link #waitFor(boolean, boolean)}.
 * <p>
 * Como o contedo  armazenado, os mtodos <code>getXXXOutput(...)</code>
 * podem ser executados mltiplas vezes, se necessrio.
 * 
 * @see Process
 * @see Runtime#exec(String)
 * @see Runtime#exec(String[])
 * @see Runtime#exec(String, String[], File)
 */
public class ExternalCommand {

  private Process process;
  private String stdoutContent = null;
  private String stderrContent = null;
  private boolean outputProcessed = false;

  /**
   * Cria um processo para execuo de um comando e inicia a execuo.
   * 
   * @param cmdArray - array contendo o comando a ser executado e seus
   *        argumentos
   * @throws IOException se houve erro na criao do processo
   * @see Runtime#exec(String[])
   */
  public ExternalCommand(String[] cmdArray) throws IOException {
    process = Runtime.getRuntime().exec(cmdArray);
  }

  /**
   * Cria um processo para execuo de um comando e inicia a execuo.
   * 
   * @param command - comando
   * @throws IOException se houve erro na criao do processo
   * @see Runtime#exec(String)
   */
  public ExternalCommand(String command) throws IOException {
    process = Runtime.getRuntime().exec(command);
  }

  /**
   * Cria um processo para execuo de um comando e inicia a execuo.
   * 
   * @param command - comando
   * @param envp - array de strings no formato var=valor representando o
   *        ambiente que o comando deve usar, ou null caso o comando deva herdar
   *        o ambiente do processo corrente
   * @param workingDir - diretrio inicial para o comando, ou null caso o
   *        comando deva herdar o diretrio do processo corrente
   * @throws IOException se houve erro na criao do processo
   * @see Runtime#exec(String, String[], File)
   */
  public ExternalCommand(String command, String[] envp, File workingDir)
    throws IOException {
    process = Runtime.getRuntime().exec(command, envp, workingDir);
  }

  /**
   * Espera pelo trmino da execuo do comando, consumindo antes todo o
   * contedo dos streams <code>stdout</code> e <code>stderr</code>, para
   * evitar dedalocks. Os streams so lidos em paralelo.
   * 
   * @param keepStdout - flag indicando se a sada padro deve ser armazenada;
   *        se for false, ser consumida porm descartada
   * @param keepStderr - flag indicando se a sada de erro deve ser armazenada;
   *        se for false, ser consumida porm descartada
   * 
   * @return cdigo de retorno do comando
   * @throws InterruptedException se o comando foi cancelado
   */
  public int waitFor(final boolean keepStdout, final boolean keepStderr)
    throws InterruptedException {
    Thread task = null;
    if (stdoutContent == null) {
      /*
       * precisamos ler stdout e stderr em paralelo, para no corrermos o risco
       * de que o segundo stream a ser lido cause o deadlock
       */
      task = new Thread(new Runnable() {
        public void run() {
          readStdout(keepStdout);
        }
      });
      task.start();
    }
    if (stderrContent == null) {
      readStderr(keepStderr);
    }
    if (task != null) {
      /*
       * esperamos a leitura de stdout acabar antes de prosseguir
       */
      task.join();
    }
    int exitValue = process.waitFor();
    outputProcessed = true;
    return exitValue;
  }

  /**
   * Espera pelo trmino da execuo do comando, obtendo antes todo o contedo
   * dos streams <code>stdout</code> e <code>stderr</code>, para evitar
   * dedalocks. Os streams so lidos em paralelo.
   * 
   * @return cdigo de retorno do comando
   * @throws InterruptedException se o comando foi cancelado
   * @see #waitFor(boolean, boolean)
   */
  public int waitFor() throws InterruptedException {
    return waitFor(true, true);
  }

  /**
   * Retorna o contedo associado  sada padro do comando. Deve ser executado
   * apenas aps a execuo de um dos mtodos <code>waitFor(...)</code>.
   * 
   * @return sada padro do comando, ou null se ocorrer algum erro na sua
   *         leitura
   * @throws IllegalStateException se a sada do comando ainda no foi
   *         processada pelos mtodos <code>waitFor(...)</code>
   * @see #waitFor()
   * @see #waitFor(boolean, boolean)
   */
  public String getStandardOutput() {
    if (!outputProcessed) {
      throw new IllegalStateException(
        "a sada do comando ainda no foi processada");
    }
    return readStdout(true);
  }

  /**
   * L um conjunto consecutivo de linhas da sada padro. O resultado pode ser
   * parcial se o contedo no possuir uma quantidade de linhas compatvel.
   * 
   * @param firstLine - primeira linha a ser lida, comeando em 0
   * @param lastLine - ltima linha a ser lida
   * @return conjunto consecutivo de linhas da sada padro, uma string vazia se
   *         o contedo possua menos do que <code>firstLine</code> linhas, ou
   *         ainda null se houve algum erro no processamento
   * @throws IllegalArgumentException se <code>firstLine > lastLine</code>
   * @throws IllegalStateException se nenhum dos mtodos
   *         <code>waitFor(...)</code> foi executado ainda
   */
  public String getStandardOutput(int firstLine, int lastLine) {
    return readLinesFromString(getStandardOutput(), firstLine, lastLine);
  }

  /**
   * L um uma nica linha da sada padro.
   * 
   * @param line - linha a ser obtida (0 = primeira linha)
   * 
   * @return linha especificada da sada padro, ou string vazia se no havia
   *         linhas o suficiente, ou ainda null se houve algum erro no
   *         processamento
   * @throws IllegalStateException se nenhum dos mtodos
   *         <code>waitFor(...)</code> foi executado ainda
   */
  public String getStandardOutput(int line) {
    return readLinesFromString(getStandardOutput(), line, line);
  }

  /**
   * Retorna o contedo associado  sada de erro do comando. Deve ser executado
   * apenas aps a execuo de um dos mtodos <code>waitFor(...)</code>.
   * 
   * @return sada de erro do comando, ou null se ocorrer algum erro na sua
   *         leitura
   * @throws IllegalStateException se a sada do comando ainda no foi
   *         processada pelos mtodos <code>waitFor(...)</code>
   * @see #waitFor()
   * @see #waitFor(boolean, boolean)
   */
  public String getErrorOutput() {
    if (!outputProcessed) {
      throw new IllegalStateException(
        "a sada do comando ainda no foi processada");
    }
    return readStderr(true);
  }

  /**
   * L um conjunto consecutivo de linhas da sada de erro. O resultado pode ser
   * parcial se o contedo no possuir uma quantidade de linhas compatvel.
   * 
   * @param firstLine - primeira linha a ser lida, comeando em 0
   * @param lastLine - ltima linha a ser lida
   * @return conjunto consecutivo de linhas da sada de erro, uma string vazia
   *         se o contedo possua menos do que <code>firstLine</code> linhas,
   *         ou ainda null se houve algum erro no processamento
   * @throws IllegalArgumentException se <code>firstLine > lastLine</code>
   * @throws IllegalStateException se nenhum dos mtodos
   *         <code>waitFor(...)</code> foi executado ainda
   */
  public String getErrorOutput(int firstLine, int lastLine) {
    return readLinesFromString(getErrorOutput(), firstLine, lastLine);
  }

  /**
   * L um uma nica linha da sada de erro.
   * 
   * @param line - linha a ser obtida (0 = primeira linha)
   * 
   * @return linha especificada da sada de erro, ou string vazia se no havia
   *         linhas o suficiente, ou ainda null se houve algum erro no
   *         processamento
   * @throws IllegalStateException se nenhum dos mtodos
   *         <code>waitFor(...)</code> foi executado ainda
   */
  public String getErrorOutput(int line) {
    return readLinesFromString(getErrorOutput(), line, line);
  }

  /**
   * L um conjunto de linhas consecutivas de uma string. O resultado pode ser
   * parcial, dependendo do contedo disponvel e dos limites solicitados.
   * 
   * @param str - string
   * @param firstLine - primeira linha a ser lida, comeando em 0
   * @param lastLine - ltima linha a ser lida
   * @return conjunto de linhas especificado, ou parte deste caso no haja
   *         linhas suficientes na string, ou ainda null se houve algum erro no
   *         processamento
   * @throws IllegalArgumentException se firstLine > lastLine
   */
  private String readLinesFromString(String str, int firstLine, int lastLine) {
    if (firstLine > lastLine) {
      throw new IllegalArgumentException("firstLine > lastLine");
    }
    if (str == null) {
      return null;
    }
    StringBuilder content = new StringBuilder();
    String line = null;
    int i = 0;
    BufferedReader reader = new BufferedReader(new StringReader(str));
    try {
      while (i <= lastLine && (line = reader.readLine()) != null) {
        if (i >= firstLine) {
          if (i > firstLine) {
            content.append('\n');
          }
          content.append(line);
        }
        i++;
      }
      reader.close();
    }
    catch (IOException e) {
      return null;
    }
    return content.toString();
  }

  /**
   * Retorna o contedo associado  sada padro do comando.
   * 
   * @param keepContent - indica se o contedo deve ser armazenado
   * @return sada padro do comando, ou null se ocorrer algum erro na sua
   *         leitura
   */
  private String readStdout(boolean keepContent) {
    if (stdoutContent == null) {
      stdoutContent = readContent(process.getInputStream(), keepContent);
    }
    return stdoutContent;
  }

  /**
   * Retorna o contedo associado  sada de erro do comando.
   * 
   * @param keepContent - indica se o contedo deve ser armazenado
   * @return sada de erro do comando, ou null se ocorrer algum erro na sua
   *         leitura
   */
  private String readStderr(boolean keepContent) {
    if (stderrContent == null) {
      stderrContent = readContent(process.getErrorStream(), keepContent);
    }
    return stderrContent;
  }

  /**
   * L o contedo de um stream. IMPORTANTE: O stream  esgotado (lido
   * integralmente), mas no necessariamente armazenado (ver parmetro
   * <code>keepContent</code>).
   * 
   * @param inputStream - {@link InputStream} de onde ser lido o contedo
   * @param keepContent - indica se o contedo lido deve ser armazenado. Se for
   *        false, o stream  consumido mas o contedo  descartado (equivalente
   *        p.ex. a direcionar a sada para <code>/dev/null</code> no Linux)
   * @return contedo associado ao stream, ou a string vazia caso tenha-se
   *         optado por descartar o contedo, ou null caso haja algum erro na
   *         sua leitura
   */
  private String readContent(InputStream inputStream, boolean keepContent) {
    BufferedReader reader =
      new BufferedReader(new InputStreamReader(inputStream));
    StringBuilder content = new StringBuilder();
    String tmp = null;
    int i = 0;
    try {
      while ((tmp = reader.readLine()) != null) {
        if (keepContent) {
          if (i++ > 0) {
            content.append('\n');
          }
          content.append(tmp);
        }
      }
      inputStream.close();
    }
    catch (IOException e) {
      return null;
    }
    return content.toString();
  }

  /**
   * @return {@link Process} associado ao commando
   */
  public Process getProcess() {
    return process;
  }

  /**
   * Retorna o cdigo de retorno do comando. Por conveno, 0 = sucesso. Garante
   * que o comando terminou e que os contedos de <code>stdout</code> e
   * <code>stderr</code> foram lidos integralmente e esto disponveis para
   * consulta.
   * 
   * @return cdigo de retorno do comando
   * @throws InterruptedException se o comando foi interrompido
   * @see #waitFor()
   * @see #waitFor(boolean, boolean)
   */
  public int exitValue() throws InterruptedException {
    return waitFor();
  }

  /**
   * Mtodo <code>main</code> para testes.
   * 
   * @param args
   * @throws IOException
   * @throws InterruptedException
   */
  public static void main(String[] args) throws IOException,
    InterruptedException {

    // TODO transformar em testes usando o JUnit
    ExternalCommand command = new ExternalCommand("ls -R /root /bin");
    System.out.println("resultado = " + command.waitFor());
    System.out.println("\noutput = " + command.getStandardOutput());
    System.out.println("\nerro = " + command.getErrorOutput());

    command = new ExternalCommand("ls -R /root /bin");
    System.out.println("resultado = " + command.waitFor(false, true));
    System.out.println("\noutput = " + command.getStandardOutput());
    System.out.println("\nerro = " + command.getErrorOutput());

    command = new ExternalCommand("ls -R /root /bin");
    System.out.println("resultado = " + command.waitFor(true, false));
    System.out.println("\noutput = " + command.getStandardOutput());
    System.out.println("\nerro = " + command.getErrorOutput());

    command = new ExternalCommand("ls -R /root /bin");
    System.out.println("resultado = " + command.waitFor(true, false));
    System.out.println("\noutput[5, 10] = " + command.getStandardOutput(5, 10));
    System.out.println("\noutput[3] = " + command.getStandardOutput(3));
    System.out.println("\noutput[1000] = " + command.getStandardOutput(1000));
    System.out.println("\noutput[1000, 2000] = "
      + command.getStandardOutput(1000, 2000));

    command = new ExternalCommand("ls -R /root /bin");
    /*
     * o comando abaixo vai gerar uma IllegalStateException
     */
    System.out.println("\noutput = " + command.getStandardOutput());
  }

  /**
   * Encerra a execuo do processo.
   * 
   * @see Process#destroy()
   */
  public void destroy() {
    process.destroy();
  }
}
