/*
 * ProjectAdminInfo.java
 * 
 * $Id: ProjectAdminInfo.java 96246 2009-09-16 19:31:33Z jnlopes $
 */
package csbase.logic;

import java.io.Serializable;
import java.rmi.RemoteException;
import java.util.Comparator;
import java.util.Date;
import java.util.Observable;
import java.util.Observer;

import csbase.remote.ClientRemoteLocator;
import csbase.util.restart.RestartListener;
import csbase.util.restart.RestartManager;

/**
 * Modela informaes administrativas sobre um projeto.
 */
public class ProjectAdminInfo implements Serializable {
  /** Objeto que permite a observao da classe ProjectAdminInfo. */
  private static Observable observable = new Observable() {
    @Override
    public void notifyObservers(Object arg) {
      setChanged();
      super.notifyObservers(arg);
    }
  };

  /** Estado de um projeto bloqueado devido a requisio de rea. */
  public static final int WAITING_AREA_ALLOCATION = 0;

  /** Estado de um projeto desbloqueado que usa uma rea reservada. */
  public static final int UNLOCKED_WITH_AREA_ALLOCATED = 1;

  /** Estado de um projeto removido aguardando liberao de rea. */
  public static final int WAITING_AREA_FREE = 2;

  /** Identificador do usrio. */
  private Object userId;

  /** Nome do projeto. */
  private String projectName;

  /** Caminho fsico do diretrio do projeto. */
  private String absolutePath;

  /** Tamanho da rea solicitada. */
  private long areaRequestSize;

  /** Data em que a rea foi solicitada. */
  private Date areaRequestDate;

  /** Tamanho da rea alocada para o projeto. */
  private long areaLockedSize;

  /** Situao atual do projeto. */
  private int state;

  /**
   * Obtem todas as informaes de projetos.
   * 
   * @return .
   * 
   * @throws RemoteException
   */
  public static ProjectAdminInfo[] getAll() throws RemoteException {
    return ClientRemoteLocator.projectService.getLockedProjectAdminInfo();
  }

  /**
   * Informa que um ou mais projetos tiveram sua rea concedida. O projeto 
   * desbloqueado.
   * 
   * @param infos Um array com os objetos <code>ProjectAdminInfo</code> com as
   *        informaes de desbloqueio de projetos devido a alocao de rea
   *        pelo administrador.
   * 
   * @throws RemoteException
   */
  public static void allocateAreaForProjects(ProjectAdminInfo[] infos)
    throws RemoteException {
    ClientRemoteLocator.projectService.unlockProjectsWithAllocation(infos);
  }

  /**
   * Informa que um ou mais projetos tiveram sua rea liberada.
   * 
   * @param infos Um array com os objetos <code>ProjectAdminInfo</code> com as
   *        informaes de desbloqueio de projetos devido a alocao de rea
   *        pelo administrador.
   * 
   * @throws RemoteException
   */
  public static void freeAreaForProjects(ProjectAdminInfo[] infos)
    throws RemoteException {
    ClientRemoteLocator.projectService.freeAreaForProjects(infos);
  }

  /**
   * Obtm o identificador do usurio que criou o projeto.
   * 
   * @return O identificador do usurio.
   */
  public Object getUserId() {
    return userId;
  }

  /**
   * Obtm o nome do projeto.
   * 
   * @return O nome do projeto.
   */
  public String getProjectName() {
    return projectName;
  }

  /**
   * Obtm o caminho fsico do diretrio do projeto.
   * 
   * @return O caminho fsico do diretrio do projeto.
   */
  public String getAbsolutePath() {
    return absolutePath;
  }

  /**
   * Obtm o tamanho da rea solicitada para o projeto.
   * 
   * @return O tamanho da rea solicitada para o projeto.
   */
  public long getAreaRequestSize() {
    return areaRequestSize;
  }

  /**
   * Obtm o tamanho da rea alocada para o projeto.
   * 
   * @return O tamanho da rea alocada para o projeto.
   */
  public long getAreaLockedSize() {
    return areaLockedSize;
  }

  /**
   * Obtm o estado atual do projeto.
   * 
   * @return O estado do projeto.
   */
  public int getState() {
    return state;
  }

  /**
   * Verifica se o projeto teve sua solicitao de espao cancelada.
   * 
   * @return verdadeiro, se o projeto teve sua solicitao de espao cancelada
   *         ou falso, caso contrrio.
   */
  public boolean isLocked() {
    return (state == WAITING_AREA_ALLOCATION);
  }

  /**
   * Verifica se o projeto est desbloqueado utilizando rea reservada.
   * 
   * @return verdadeiro, se o projeto est desbloqueado utilizando rea
   *         reservada ou falso, caso contrrio.
   */
  public boolean isUnlockedWithAreaAllocated() {
    return (state == UNLOCKED_WITH_AREA_ALLOCATED);
  }

  /**
   * Verifica se o projeto est aguardando liberao de rea.
   * 
   * @return verdadeiro, se o projeto est aguardando liberao de rea
   *         reservada ou falso, caso contrrio.
   */
  public boolean isWaitingAreaFree() {
    return (state == WAITING_AREA_FREE);
  }

  /**
   * Obtm a data em que foi solicitada uma rea para o projeto.
   * 
   * @return A data de solicitao de reserva de rea.
   */
  public Date getAreaRequestDate() {
    return areaRequestDate;
  }

  /**
   * Muda o estado do projeto.
   * 
   * @param state O novo estado do projeto.
   */
  public void setState(int state) {
    this.state = state;
  }

  /**
   * Informa o tamanho da rea requisitada para o projeto.
   * 
   * @param size O tamanho da rea requisitada.
   */
  public void setAreaRequestSize(long size) {
    this.areaRequestSize = size;
  }

  /**
   * Informa o tamanho da rea alocada para o projeto.
   * 
   * @param size O tamanho da rea alocada pelo administrador.
   */
  public void setAreaLockedSize(long size) {
    this.areaLockedSize = size;
  }

  /**
   * Informa a data da requisio de rea para o projeto.
   * 
   * @param date A data em que a requisio de rea foi solicitada.
   */
  public void setAreaRequestDate(Date date) {
    this.areaRequestDate = date;
  }

  /**
   * Obtm um comparador de <code>ProjectAdminInfo</code> pelo critrio de ordem
   * alfabtica do nome do projeto.
   * 
   * @return um comparador por nome do projeto.
   */
  public static Comparator<ProjectAdminInfo> getNameComparator() {
    Comparator<ProjectAdminInfo> c = new Comparator<ProjectAdminInfo>() {
      public int compare(ProjectAdminInfo info1, ProjectAdminInfo info2) {
        return info1.projectName.compareTo(info2.projectName);
      }
    };
    return c;
  }

  /**
   * Obtm um comparador de <code>ProjectAdminInfo</code> pelo critrio de ordem
   * alfabtica do nome do usurio dono do projeto.
   * 
   * @return um comparador por nome do usurio dono do projeto.
   */
  public static Comparator<ProjectAdminInfo> getOwnerComparator() {
    final Comparator<ProjectAdminInfo> c = new Comparator<ProjectAdminInfo>() {
      public int compare(final ProjectAdminInfo info1,
        final ProjectAdminInfo info2) {
        try {
          final User user1 = User.getUser(info1.userId);
          final User user2 = User.getUser(info2.userId);
          return user1.getLogin().compareTo(user2.getLogin());
        }
        catch (Exception e) {
          return 0;
        }
      }
    };
    return c;
  }

  /**
   * Obtm um comparador de <code>ProjectAdminInfo</code> pelo critrio de ordem
   * alfabtica do caminho do projeto.
   * 
   * @return um comparador por caminho do projeto.
   */
  public static Comparator<ProjectAdminInfo> getPathComparator() {
    final Comparator<ProjectAdminInfo> c = new Comparator<ProjectAdminInfo>() {
      public int compare(final ProjectAdminInfo info1,
        final ProjectAdminInfo info2) {
        return info1.absolutePath.compareTo(info2.absolutePath);
      }
    };
    return c;
  }

  /**
   * Obtm um comparador de <code>ProjectAdminInfo</code> pelo critrio de ordem
   * alfabtica do estado do projeto.
   * 
   * @return um comparador por estado do projeto.
   */
  public static Comparator<ProjectAdminInfo> getStateComparator() {
    final Comparator<ProjectAdminInfo> c = new Comparator<ProjectAdminInfo>() {
      public int compare(final ProjectAdminInfo info1,
        final ProjectAdminInfo info2) {
        return (Integer.valueOf(info1.state)).compareTo(Integer
          .valueOf(info2.state));
      }
    };
    return c;
  }

  /**
   * Obtm um comparador de <code>ProjectAdminInfo</code> pelo critrio de ordem
   * do tamanho da rea requerida.
   * 
   * @return um comparador por tamanho da rea requerida.
   */
  public static Comparator<ProjectAdminInfo> getAreaReqSizeComparator() {
    final Comparator<ProjectAdminInfo> c = new Comparator<ProjectAdminInfo>() {
      public int compare(final ProjectAdminInfo info1,
        final ProjectAdminInfo info2) {
        return (Long.valueOf(info1.areaRequestSize)).compareTo(Long
          .valueOf(info2.areaRequestSize));
      }
    };
    return c;
  }

  /**
   * Obtm um comparador de <code>ProjectAdminInfo</code> pelo critrio de ordem
   * de data de requisio de rea.
   * 
   * @return um comparador por data.
   */
  public static Comparator<ProjectAdminInfo> getAreaReqDateComparator() {
    final Comparator<ProjectAdminInfo> c = new Comparator<ProjectAdminInfo>() {
      public int compare(final ProjectAdminInfo info1,
        final ProjectAdminInfo info2) {
        final Date d1 = info1.areaRequestDate;
        final Date d2 = info2.areaRequestDate;
        if (d1 == null) {
          return -1;
        }
        if (d2 == null) {
          return 1;
        }
        return (d1).compareTo(d2);
      }
    };
    return c;
  }

  /**
   * Obtm um comparador de <code>ProjectAdminInfo</code> pelo critrio de ordem
   * do tamanho da rea reservada.
   * 
   * @return um comparador por tamanho da rea reservada.
   */
  public static Comparator<ProjectAdminInfo> getAreaLockSizeComparator() {
    final Comparator<ProjectAdminInfo> c = new Comparator<ProjectAdminInfo>() {
      public int compare(final ProjectAdminInfo info1,
        final ProjectAdminInfo info2) {
        return (Long.valueOf(info1.areaLockedSize)).compareTo(Long
          .valueOf(info2.areaLockedSize));
      }
    };
    return c;
  }

  /**
   * Adiciona um observador local da classe. Sempre que a classe  avisada pelo
   * servidor que alguma mudana ocorreu, os observadores locais tambm so
   * notificados.
   * 
   * @param o um observador local
   */
  public static void addObserver(Observer o) {
    observable.addObserver(o);
  }

  /**
   * Remove um observador local da classe.
   * 
   * @param o o observador a ser removido
   */
  public static void deleteObserver(Observer o) {
    observable.deleteObserver(o);
  }

  /**
   * Esse mtodo  chamado quando o servidor sofreu alguma alterao nas
   * informaes administrativas de projetos.
   * 
   * @param action a ao que ocorreu no provedor de usurios
   */
  public static void update(AdministrationEvent action) {
    observable.notifyObservers(action);
  }

  /**
   * Constri um <code>ProjectAdminInfo</code> com as informaes sobre um
   * projeto.
   * 
   * @param userId Identificador do usrio.
   * @param projectName Nome do projeto.
   * @param absolutePath Caminho fsico do diretrio do projeto.
   * @param state O estado do projeto
   */
  public ProjectAdminInfo(final Object userId, final String projectName,
    final String absolutePath, final int state) {
    this.userId = userId;
    this.projectName = projectName;
    this.absolutePath = absolutePath;
    this.state = state;
    this.areaRequestSize = 0;
    this.areaRequestDate = null;
  }

  static {
    RestartManager.getInstance().addListener(new RestartListener() {
      public void restart() {
        ProjectAdminInfo.observable.deleteObservers();
      }
    });
  }
}
