package csbase.logic;

import java.rmi.RemoteException;

/**
 * Modela um arquivo ou diretrio de um projeto. A partir do ProjectFile raiz,
 *  possvel percorrer toda a rvore de diretrios e arquivos que compe um
 * projeto de um usurio.
 */
public interface ProjectFile {
  /** Definem os possveis estados de um arquivo. */
  public static final int CLOSED = 0;
  public static final int OPENED_AS_CHANNEL = 1;
  public static final int OPENED_FOR_READING = 2;
  public static final int OPENED_FOR_WRITING = 3;

  /**
   * Retorna o nome do arquivo.
   *
   * @return Nome do arquivo.
   */
  String getName();

  /**
   * Retorna o caminho completo para o arquivo.
   *
   * @return Caminho at o arquivo, inclusive.
   */
  String[] getPath();

  /**
   * Retorna o caminho para o arquivo. Esse caminho no  relativo, ou seja, o
   * caminho absoluto de onde o arquivo se encontra gravado no servidor.
   *
   * @return Caminho absoluto at o arquivo, inclusive.
   */
  public String getAbsolutePath();

  /**
   * Informa o tipo do arquivo. Todos os tipos conhecidos pelo WebSintesi so
   * listados em {@link FileType}.
   *
   * @return O tipo do arquivo, como uma constante de {@link FileType}.
   */
  String getType();

  /**
   * Informa o a data de criao arquivo.
   *
   * @return a data de criao do arquivo em Milesegundos.
   */
  long getCreationDate();

  /**
   * Retorna a data de modificao do arquivo.
   *
   * @return data de modificao do arquivo
   */
  long getModificationDate();

  /**
   * Muda o tipo do arquivo.
   *
   * @param newType O novo tipo do arquivo.
   *
   * @throws Exception
   */
  void changeType(String newType)
  throws Exception;

  /**
   * Retorna o usurio que publicou esse arquivo.
   *
   * @return O id do usurio que publicou esse arquivo.
   */
  public Object whoPublished();

  /**
   * Retorna o identificador do projeto a partir do qual este arquivo foi
   * publicado.
   *
   * @return O identificador do projeto pelo qual o arquivo foi publicado.
   */
  public Object whichProjectPublished();

  /**
   * Retorna o usurio que criou esse arquivo.
   *
   * @return O nome do usurio que criou esse arquivo.
   */
  public Object whoCreated();

  /**
   * Retorna o diretrio pai deste arquivo.
   *
   * @return O diretrio pai deste arquivo ou <code>null</code> caso este
   *         arquivo seja o diretrio raiz.
   */
  ProjectFile getParent();

  /**
   * Retorna a lista de arquivos deste diretrio.
   *
   * @return A lista de arquivos deste diretrio.
   * @throws RemoteException
   */
  ProjectFile[] getChildren() throws RemoteException;

  /**
   * Retorna a lista de arquivos deste diretrio do tipo especificado.
   *
   * @param fileType Tipo dos arquivos que devem ser retornados.
   *
   * @return A lista de arquivos deste diretrio.
   * @throws RemoteException
   */
  ProjectFile[] getChildren(String fileType) throws RemoteException;

  /**
   * Retorna um determinado arquivo contido neste diretrio.
   *
   * @param name Nome do arquivo a ser recuperado.
   *
   * @return O arquivo desejado.
   * @throws RemoteException
   */
  ProjectFile getChild(String name) throws RemoteException;

  /**
   * Retorna o identificador do projeto ao qual esse arquivo pertence.
   *
   * @return O identificador do projeto.
   */
  Object getProjectId();

  /**
   * Cria um diretrio dentro deste diretrio.
   *
   * @param name Nome do diretrio a ser criado.
   *
   * @throws Exception
   */
  void createDirectory(String name)
  throws Exception;

  /**
   * Troca o nome deste arquivo.
   *
   * @param name Novo nome do arquivo.
   *
   * @throws Exception
   */
  void rename(String name) throws Exception;

  /**
   * Copia este arquivo para outro diretrio.
   *
   * @param directory Diretrio destino.
   *
   * @throws Exception
   */
  void copy(ProjectFile directory) throws Exception;

  /**
   * Move este arquivo para outro diretrio.
   *
   * @param directory Diretrio destino.
   *
   * @throws Exception
   */
  void move(ProjectFile directory) throws Exception;

  /**
   * Remove este arquivo. Aps chamado este mtodo, o objeto que representa o
   * arquivo no poder mais ser utilizado.
   *
   * @throws Exception
   */
  void remove() throws Exception;

  /**
   * Indica se este arquivo  um diretrio.
   *
   * @return Verdadeiro caso seja um diretrio e falso caso contrrio.
   */
  boolean isDirectory();

  /**
   * Indica se este arquivo  um arquivo global.
   *
   * @return Verdadeiro caso seja um arquivo global e falso caso contrrio.
   */
  boolean isGlobalFile();

  /**
   * Indica se este arquivo  um arquivo publicado.
   *
   * @return Verdadeiro caso seja um arquivo publicado e falso caso contrrio.
   */
  boolean isPublishedFile();

  /**
   * Indica se este arquivo  um arquivo em construo.
   *
   * @return Verdadeiro caso seja um arquivo em construo e falso caso
   *         contrrio.
   */
  boolean isUnderConstruction();

  /**
   * Indica se este arquivo  um arquivo normal.
   *
   * @return Verdadeiro caso seja um arquivo normal e falso caso contrrio.
   */
  boolean isRegularFile();

  /**
   * Indica se o arquivo est bloqueado.
   * @return true se o arquivo estiver bloqueado
   */
  boolean isLocked();

  /**
   * Obtm a descrio atual deste arquivo.
   *
   * @return O texto com a descrio do arquivo.
   *
   * @throws Exception
   */
  String getDescription()
  throws Exception;

  /**
   * Muda a descrio atual deste arquivo.
   *
   * @param text novo texto da descrio do arquivo.
   *
   * @throws Exception
   */
  void setDescription(String text)
  throws Exception;

  /**
   * Anexa um texto  descrio atual deste arquivo. Permite a gerao de
   * histricos de modificao.
   *
   * @param text texto a ser anexado.
   *
   * @throws Exception
   */
  void appendDescription(String text)
  throws Exception;

  /**
   * Publica este arquivo.
   *
   * @param userIds contendo os identificadores dos usurios que tero acesso
   *        ao arquivo publicado.
   * @param whoPublishedId Identificador do usurio que publicou o arquivo.
   *
   * @return Verdadeiro em caso de sucesso e falso caso a operao falhe.
   *
   * @throws Exception
   */
  boolean publish(Object[] userIds, Object whoPublishedId)
  throws Exception;

  /**
   * Despublica este arquivo. Despublicar significa retirar todos os usurios
   * da lista de acesso ao arquivo publicado. Assim, quem tem link para ele
   * continua tendo, quem no tem, no consegue mais.
   *
   * @param userIds .
   * @param userId Identificador do usurio que quer despublicar um arquivo
   *
   * @return Verdadeiro em caso de sucesso e falso caso a operao falhe.
   *
   * @throws Exception .
   */

  /*
     boolean unPublish(Object userId) throws Exception;
   */

  /**
   * Atualiza a lista de publicao do arquivo.
   *
   * @param userIds Identificadores do usurios que devem ter acesso ao arquivo
   *        publicado.
   * @param userId Identificador do usurio que quer despublicar um arquivo
   *
   * @return Verdadeiro em caso de sucesso e falso caso a operao falhe.
   *
   * @throws Exception
   */
  public boolean updatePublishAccess(Object[] userIds, Object userId)
  throws Exception;

  /**
   * Informa a lista de usurios para os quais esse arquivo foi publicado. No
   * caso do arquivo no estar publicado, retorna null.
   *
   * @return Um array com os identificadores dos usurios para os quais  esse
   *         arquivo foi publicado.
   *
   * @throws Exception
   */
  public Object[] getPublishAccess()
  throws Exception;

  /**
   * Informa o tamanho corrente deste arquivo. Reflete o nmero corrente de
   * bytes do arquivo.
   *
   * @return Tamanho corrente do arquivo.
   */
  long size();

  /**
   * Verifica se o arquivo foi publicado pelo projeto dado.
   *
   * @param projId
   *
   * @return .
   */
  public boolean wasPublishedBy(Object projId);
}
