/* 
 * $Id$ 
 */
package csbase.logic;

import java.rmi.RemoteException;

import csbase.exception.CSBaseException;

/**
 * <code>SingleServerManager</code>  a base para a monitorao de um nico servidor.
 * 
 * 
 * @author Tecgraf/PUC-Rio
 *
 */
public abstract class SingleServerManager extends ServerManager {

  /** A instncia de monitorao */
  private SingleMonitoredServer singleMonitoredServer;
  
  /**
   * Implementa a monitorao do servidor.
   * 
   * @author Tecgraf/PUC-Rio
   *
   */
  final class SingleMonitoredServer extends MonitoredServer {

    /**
     * Constri a monitorao
     * 
     * @param serverURI A URI do servidor a ser monitorado
     * @param locator A classe do locator
     */
    protected SingleMonitoredServer(ServerURI serverURI, Class<?> locator) {
      super(serverURI, locator);
    }

    @Override
    public Session performLogin() throws CSBaseException, RemoteException {
      return SingleServerManager.this.login();
    }

    @Override
    public void postLogin() throws CSBaseException {
      /** nada a fazer */
    }
    
    @Override
    protected String lng(String key) {
      return lang(key);
    }

  }

  
  /**
   * Mtodo usado para traduo de termos
   * 
   * @param key A chave do termo 
   * @return O valor traduzido
   */
  protected abstract String lang(String key);
  
  /**
   * Executa o login do servidor
   * 
   * @return A sesso do cliente logado
   * @throws CSBaseException
   * @throws RemoteException
   */
  protected abstract Session login() throws CSBaseException, RemoteException;
  
  
  /**
   * Constri o gerente 
   * 
   * @param serverURI A URI do servidor a ser gerenciado
   * @param locator A classe do locator
   */
  public SingleServerManager(ServerURI serverURI, Class<?> locator) {
    if (serverURI == null)
      throw new IllegalArgumentException("URI no pode ser nula!");
    this.singleMonitoredServer = new SingleMonitoredServer(serverURI, locator);
    this.addServer(this.singleMonitoredServer);
  }
  
  /**
   * Tenta estebelecer comunicao com o servidor. Em caso de sucesso,
   * efetua o login do usurio, obtm os servios e instala todos os
   * observadores remotos no servidor.
   *
   * @return true em caso de sucesso, false caso contrrio
   *
   * @throws CSBaseException Caso o servidor no esteja disponvel.
   * @throws RemoteException Em caso de falha na comunicao
   */
  public final boolean lookup() throws CSBaseException, RemoteException {
    if (super.performLookup()) {
      if (super.validate())
        return true;
      else
        return false;
    }
    throw new CSBaseException("Servidor fora do ar.");
  }

  /**
   * Interrompe o monitor. Remove os observadores, executa o logout
   * 
   */
  public final synchronized void stop() {
    this.getMonitor().stopMonitoring();
    this.removeObservers();
    this.logout(true);
  }
  
  /**
   * Inicia a monitorao do servidor
   */
  public final void startMonitoring() {
    this.getMonitor().startMonitoring();
  }
  
  /**
   * Testa se o servidor est alcanvel.
   *
   * @return true, caso o servidor esteja alcanvel, ou false, caso contrrio.
   */
  public final boolean ping() {
    return this.getMonitor().ping();
  }

  /**
   * Fora a verificao do estado do servidor.
   */
  @Override
  public final void update() {
    this.getMonitor().update();
  }

  /**
   * Define o nome do sistema a ser usado
   * 
   * @param systemName O nome do sistema
   */
  public final void setSystemName(String systemName) {
    this.setSystemName(this.singleMonitoredServer.getURI(), systemName);
  }
  
  /**
   * Executa o logout no servidor e o retira da monitorao
   */
  public final void logout() {
    this.logout(true);
  }
}
