/*
 * $Id: Category.java 99009 2009-11-22 21:39:54Z pizzol $
 */

package csbase.logic.algorithms;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Observable;
import java.util.Observer;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import csbase.logic.AlgoEvent;

/**
 * Categoria de algoritmo.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class Category implements Comparable<Category>, Serializable {

  /**
   * Os algoritmos filhos desta categoria.
   */
  private final Set<AlgorithmInfo> algorithms;

  /**
   * As categorias que so filhas desta categoria.
   */
  private final SortedSet<Category> categories;

  /**
   * O identificador da categoria.
   */
  private String id;

  /**
   * O nome da categoria.
   */
  private String name;

  /** Objeto que permite a observao local dessa categoria */
  private static Observable observable = new Observable() {
    @Override
    public void notifyObservers(Object arg) {
      setChanged();
      super.notifyObservers(arg);
    }
  };

  /**
   * Construtor
   * 
   * @param id O identificador (No aceita {@code null}).
   * @param name O nome (No aceita {@code null}).
   */
  public Category(final String id, final String name) {
    algorithms = new HashSet<AlgorithmInfo>();
    categories = new TreeSet<Category>();
    setId(id);
    setName(name);
  }

  /**
   * Constri uma categoria a partir de uma categoria base.
   * 
   * @param category categoria base
   */
  public Category(Category category) {
    setId(category.getId());
    setName(category.getName());
    algorithms = category.getAlgorithms();
    categories = category.getCategories();
  }

  /**
   * Adiciona um algoritmo a esta categoria.
   * 
   * @param algorithm O de algoritmo (No aceita {@code null}).
   * @return {@code true} em caso de sucesso ou {@code false} em caso de erro.
   */
  public boolean addAlgorithm(final AlgorithmInfo algorithm) {
    if (algorithm == null) {
      throw new IllegalArgumentException("algorithm == null");
    }

    return algorithms.add(algorithm);
  }

  /**
   * Adiciona uma categoria filha.
   * 
   * @param category A categoria filha (No aceita {@code null}).
   * @return {@code true} em caso de sucesso ou {@code false} se a categoria j
   *         est na lista de categorias filhas ou se for ela mesma.
   */
  public boolean addCategory(final Category category) {
    if (category == null) {
      throw new IllegalArgumentException("category == null");
    }
    if (equals(category)) {
      return false;
    }
    return categories.add(category);
  }

  /**
   * {@inheritDoc}
   */
  public int compareTo(final Category other) {
    return getName().compareTo(other.getName());
  }

  /**
   * Verifica se o algoritmo est nesta categoria.
   * 
   * @param algorithm O algoritmo (no aceita {@code null}).
   * @return indicativo
   */
  public boolean contains(final AlgorithmInfo algorithm) {
    if (algorithm == null) {
      throw new IllegalArgumentException("algorithm == null");
    }
    if (algorithms.contains(algorithm)) {
      return true;
    }
    for (final Category category : categories) {
      if (category.contains(algorithm)) {
        return true;
      }
    }
    return false;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean equals(final Object obj) {
    if (obj == null) {
      return false;
    }
    if (getClass() != obj.getClass()) {
      return false;
    }
    final Category other = (Category) obj;
    if (!getId().equals(other.getId())) {
      return false;
    }
    return true;
  }

  /**
   * Obtm os algoritmos desta categoria.
   * 
   * @return Os algoritmos desta categoria. O conjunto retornado  imutvel.
   *         Caso no haja algoritmos, retornar um conjunto vazio.
   */
  public Set<AlgorithmInfo> getAlgorithms() {
    return algorithms;
    // return Collections.unmodifiableSet(algorithms);
  }

  /**
   * Obtm as categorias filhas desta categoria.
   * 
   * @return As categorias filhas. O conjunto retornado  imutvel. Caso no
   *         haja categorias filhas, retornar um conjunto vazio.
   */
  public SortedSet<Category> getCategories() {
    return categories;
    // return Collections.unmodifiableSortedSet(categories);
  }

  /**
   * Obtm o identificador.
   * 
   * @return o identificador.
   */
  public String getId() {
    return id;
  }

  /**
   * Consulta o nome
   * 
   * @return o nome
   */
  public String getName() {
    return name;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public int hashCode() {
    return id.hashCode();
  }

  /**
   * Remove um algoritmo desta categoria.
   * 
   * @param algorithm O de algoritmo (No aceita {@code null}).
   * @return {@code true} em caso de sucesso ou {@code false} em caso de erro.
   */
  public boolean removeAlgorithm(final AlgorithmInfo algorithm) {
    if (algorithm == null) {
      throw new IllegalArgumentException("algorithm == null");
    }
    return algorithms.remove(algorithm);
  }

  /**
   * Remove todos os algoritmos dessa categoria.
   * 
   */
  public void removeAllAlgorithms() {
    algorithms.clear();
  }

  /**
   * Remove uma sub-categoria dessa categoria.
   * 
   * @param category categoria a ser removida
   * 
   * @return retorna true se a sub-categoria foi removida com sucesso, caso
   *         contrrio, retorna false
   */
  public boolean removeCategory(Category category) {
    if (category == null) {
      throw new IllegalArgumentException("category == null");
    }
    return categories.remove(category);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public String toString() {
    return name;
  }

  /**
   * Atribui um identificador a esta categoria.
   * 
   * @param id O identificador (No aceita {@code null}).
   */
  private void setId(final String id) {
    if (id == null) {
      throw new IllegalArgumentException("id == null");
    }
    this.id = id;
  }

  /**
   * Atribui o nome a esta categoria.
   * 
   * @param name O nome (No aceita {@code null}).
   */
  private void setName(final String name) {
    if (name == null) {
      throw new IllegalArgumentException("name == null");
    }
    this.name = name;
  }

  /**
   * Esse mtodo  chamado quando o servio de algoritmos sofre alguma alterao
   * relativa a categoria de algoritmos.
   * 
   * Sempre que a classe  avisada pelo servio de algoritmos de que alguma
   * mudana ocorreu, os observadores locais tambm so notificados.
   * 
   * @param event o evento que ocorreu no servio de algoritmo
   * 
   * @throws Exception Em caso de erro.
   */
  public static void update(AlgoEvent event) throws Exception {
    observable.notifyObservers(event);
  }

  /**
   * Adiciona um observador local da classe.
   * 
   * @param o um observador local
   */
  public static void addObserver(Observer o) {
    observable.addObserver(o);
  }

  /**
   * Remove um observador local da classe.
   * 
   * @param o o observador a ser removido
   */
  public static void deleteObserver(Observer o) {
    observable.deleteObserver(o);
  }

}
