package csbase.logic.algorithms;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

/**
 * Contexto para a criao da linha de comando.
 * 
 * @author ururahy e lmoreira
 * 
 */
public final class CommandLineContext {

  /**
   * O identificador do comando.
   */
  private String commandId;

  /**
   * O nmero de cpus que sero utilizados na execuo paralela.
   */
  private int cpuCountForParallelExecution;

  /**
   * O nmero de execues por sga que sero utilizados na execuo distribuda.
   */
  private int executionCountPerSGAForDistributedExecution;

  /**
   * O separador de arquivo.
   */
  private String fileSeparator;
  /**
   * O identificador da plataforma.
   */
  private String platformId;
  /**
   * O caminho para o diretrio do projeto relativo ao repositrio de projetos.
   */
  private String projectDirectory;

  /**
   * O caminho para o diretrio do servidor.
   */
  private String serverDirectory;

  /**
   * Os nomes dos SGAs utilizados na execuo distribuda.
   */
  private List<String> sgaNamesForDistributedExecution;

  /**
   * Indica se o comando executa um fluxo.
   */
  private boolean isFlow;

  /**
   * @param commandId O identificador do comando (No aceita {@code null}).
   * @param platformId O identificador da plataforma (No aceita {@code null}).
   * @param fileSeparator O separador de arquivo (No aceita {@code null}).
   * @param serverDirectory O caminho para o diretrio do servidor (No aceita
   *        {@code null}).
   * @param projectDirectory O caminho para o diretrio do projeto relativo ao
   *        diretrio do servidor (No aceita {@code null}).
   */
  private CommandLineContext(String commandId, String platformId,
    String fileSeparator, String serverDirectory, String projectDirectory) {
    this.commandId = commandId;
    this.platformId = platformId;
    this.fileSeparator = fileSeparator;
    this.serverDirectory = serverDirectory;
    this.projectDirectory = projectDirectory;
    this.cpuCountForParallelExecution = 1;
    this.sgaNamesForDistributedExecution = new LinkedList<String>();
    this.executionCountPerSGAForDistributedExecution = 1;
    this.isFlow = false;
  }

  /**
   * @param commandId O identificador do comando (No aceita {@code null}).
   * @param platformId O identificador da plataforma (No aceita {@code null}).
   * @param fileSeparator O separador de arquivo (No aceita {@code null}).
   * @param serverDirectory O caminho para o diretrio do servidor (No aceita
   *        {@code null}).
   * @param projectDirectory O caminho para o diretrio do projeto relativo ao
   *        diretrio do servidor (No aceita {@code null}).
   * @param cpuCountForParallelExecution O nmero de CPU.
   */
  private CommandLineContext(String commandId, String platformId,
    String fileSeparator, String serverDirectory, String projectDirectory,
    int cpuCountForParallelExecution) {
    this(commandId, platformId, fileSeparator, serverDirectory,
      projectDirectory);
    this.cpuCountForParallelExecution = cpuCountForParallelExecution;
  }

  /**
   * @param commandId O identificador do comando (No aceita {@code null}).
   * @param platformId O identificador da plataforma (No aceita {@code null}).
   * @param fileSeparator O separador de arquivo (No aceita {@code null}).
   * @param serverDirectory O caminho para o diretrio do servidor (No aceita
   *        {@code null}).
   * @param projectDirectory O caminho para o diretrio do projeto relativo ao
   *        repositrio de projetos (No aceita {@code null}).
   * @param sgaNamesForDistributedExecution Os sgas que sero utilizados na
   *        distribuio do comando (No aceita {@code null}).
   * @param executionCountPerSGAForDistributedExecution O nmero de execues
   *        por SGA.
   */
  private CommandLineContext(String commandId, String platformId,
    String fileSeparator, String serverDirectory, String projectDirectory,
    List<String> sgaNamesForDistributedExecution,
    int executionCountPerSGAForDistributedExecution) {
    this(commandId, platformId, fileSeparator, serverDirectory,
      projectDirectory);
    if (sgaNamesForDistributedExecution.isEmpty()) {
      throw new IllegalStateException("Os nomes dos SGAs no foram informados.");
    }
    this.sgaNamesForDistributedExecution
      .addAll(sgaNamesForDistributedExecution);
    this.executionCountPerSGAForDistributedExecution =
      executionCountPerSGAForDistributedExecution;
  }

  /**
   * Cria o contexto para a execuo distribuda.
   * 
   * @param commandId O identificador do comando (No aceita {@code null}).
   * @param platformId O identificador da plataforma (No aceita {@code null}).
   * @param fileSeparator O separador de arquivo (No aceita {@code null}).
   * @param serverDirectory O caminho para o diretrio do servidor (No aceita
   *        {@code null}).
   * @param projectDirectory O caminho para o diretrio do projeto relativo ao
   *        repositrio de projetos (No aceita {@code null}).
   * @param sgaNames Os sgas que sero utilizados na distribuio do comando
   *        (No aceita {@code null}).
   * @param executionCountPerSGA O nmero de execues por SGA.
   * 
   * @return o contexto.
   */
  public static CommandLineContext createDistributedContext(String commandId,
    String platformId, String fileSeparator, String serverDirectory,
    String projectDirectory, List<String> sgaNames, int executionCountPerSGA) {
    return new CommandLineContext(commandId, platformId, fileSeparator,
      serverDirectory, projectDirectory, sgaNames, executionCountPerSGA);
  }

  /**
   * Cria o contexto para a execuo paralela.
   * 
   * @param commandId O identificador do comando (No aceita {@code null}).
   * @param platformId O identificador da plataforma (No aceita {@code null}).
   * @param fileSeparator O separador de arquivo (No aceita {@code null}).
   * @param serverDirectory O caminho para o diretrio do servidor (No aceita
   *        {@code null}).
   * @param projectDirectory O caminho para o diretrio do projeto relativo ao
   *        diretrio do servidor (No aceita {@code null}).
   * @param cpuCount O nmero de CPU.
   * 
   * @return o contexto.
   */
  public static CommandLineContext createParallelContext(String commandId,
    String platformId, String fileSeparator, String serverDirectory,
    String projectDirectory, int cpuCount) {
    return new CommandLineContext(commandId, platformId, fileSeparator,
      serverDirectory, projectDirectory, cpuCount);
  }

  /**
   * Cria o contexto para a execuo paralela.
   * 
   * @param commandId O identificador do comando (No aceita {@code null}).
   * @param platformId O identificador da plataforma (No aceita {@code null}).
   * @param fileSeparator O separador de arquivo (No aceita {@code null}).
   * @param serverDirectory O caminho para o diretrio do servidor (No aceita
   *        {@code null}).
   * @param projectDirectory O caminho para o diretrio do projeto relativo ao
   *        diretrio do servidor (No aceita {@code null}).
   * 
   * @return o contexto.
   */
  public static CommandLineContext createSimpleContext(String commandId,
    String platformId, String fileSeparator, String serverDirectory,
    String projectDirectory) {
    return new CommandLineContext(commandId, platformId, fileSeparator,
      serverDirectory, projectDirectory);
  }

  /**
   * Obtm o identificador do comando.
   * 
   * @return O identificador do comando.
   */
  public String getCommandId() {
    return commandId;
  }

  /**
   * Obtm o nmero de CPUs para execuo paralela.
   * 
   * @return O nmero de CPUs.
   */
  public int getCPUCountForParallelExecution() {
    return cpuCountForParallelExecution;
  }

  /**
   * Obtm o nmero de execues por SGA para execuo distribuda.
   * 
   * @return O nmero de execues.
   */
  public int getExecutionCountPerSGAForDistributedExecution() {
    return executionCountPerSGAForDistributedExecution;
  }

  /**
   * Obtm o separador de diretrios.
   * 
   * @return o separador de diretrios
   */
  public String getFileSeparator() {
    return fileSeparator;
  }

  /**
   * Obtm o separador de diretrios.
   *
   * @return o separador de diretrios
   */
  public char getFileSeparatorChar() {
    return fileSeparator.charAt(0);
  }

  /**
   * Obtm o identificador da plataforma.
   * 
   * @return O identificador da plataforma.
   */
  public String getPlatformId() {
    return platformId;
  }

  /**
   * Obtm o caminho para o diretrio do projeto relativo ao repositrio de
   * projetos.
   * 
   * @return .
   */
  public String getProjectDirectory() {
    return projectDirectory;
  }

  /**
   * Obtm o caminho para o diretrio do servidor.
   * 
   * @return .
   */
  public String getServerDirectory() {
    return serverDirectory;
  }

  /**
   * Indica se o comando executa um fluxo.
   * 
   * @return Verdadeiro se o comando executa um fluxo, ou falso, caso contrrio.
   */
  public boolean isFlow() {
    return isFlow;
  }

  /**
   * Determina se o comando executa um fluxo.
   * 
   * @param isFlow Verdadeiro se o comando executa um fluxo, ou falso, caso
   *        contrrio.
   */
  public void setIsFlow(boolean isFlow) {
    this.isFlow = isFlow;
  }

  /**
   * Obtm os nomes dos SGAs para a execuo distribuda.
   * 
   * @return Os nomes dos SGAs.
   */
  public List<String> getSGANamesForDistributedExecution() {
    return Collections.unmodifiableList(sgaNamesForDistributedExecution);
  }

}
