package csbase.logic.algorithms.commands;

import java.io.Serializable;
import java.util.Date;

/**
 * Propriedades do comando.
 */
public final class CommandProperties implements Serializable {

  /**
   * A descrio.
   */
  private String description;
  
  /**
   * O identificador.
   */
  private String id;
  
  /**
   * Dica para auxiliar a identificao do comando. 
   */
  private String tip;  
  
  /**
   * O identificador do projeto.
   */
  private Object projectId;
  
  /**
   * O nome do SGA.
   */
  private String sgaName;

  /**
   * A data inicial.
   */
  private Date startDate;

  /**
   * O estado do comando.
   */
  private Status status;

  /**
   * O identificador do usurio.
   */
  private Object userId;
 
  /**
   * Cria as propriedades do comando.
   * 
   * @param id O identificador (no aceita {@code null}).
   * @param tip dica para auxiliar a identificao do comando. 
   *        
   *        Se {@code tip==null}, ento {@code tip=""}. 
   * @param projectId O identificador do projeto (no aceita {@code null}).
   * @param sgaName O nome do SGA (aceita {@code null}).
   * @param userId O identificador do usurio (aceita {@code null}).
   * @param startDate A data inicial (aceita {@code null}).
   * @param description A descrio (aceita {@code null}).
   * @param status O estado do comando ({@code null} para {@link Status#UNKNOWN}).
   */
  public CommandProperties(String id, String tip, Object projectId, String sgaName,
    Object userId, Date startDate, String description, Status status) {
    setId(id);
    setTip(tip);
    setProjectId(projectId);
    setSgaName(sgaName);
    setUserId(userId);
    setStartDate(startDate);
    setDescription(description);
    setStatus(status);
  }

  @Override
  public boolean equals(Object obj) {
    if (this == obj) {
      return true;
    }
    if (obj == null) {
      return false;
    }
    if (getClass() != obj.getClass()) {
      return false;
    }
    CommandProperties other = (CommandProperties) obj;
    if (!id.equals(other.id)) {
      return false;
    }
    return true;
  }

  /**
   * Obtm a descrio.
   * 
   * @return A descrio ou {@code null} se ela no existir.
   */
  public String getDescription() {
    return description;
  }

  /**
   * Obtm o identificador.
   * 
   * @return O identificador.
   */
  public String getId() {
    return id;
  }
  
  /**
   * Obtm dica para auxiliar a identificao do algoritmo do comando. 
   * 
   * @return A dica.
   */
  public String getTip() {
    return tip;
  }  

  /**
   * Obtm o identificador do projeto.
   * 
   * @return O identificador do projeto.
   */
  public Object getProjectId() {
    return projectId;
  }

  /**
   * Obtm o nome do SGA.
   * 
   * @return O nome do SGA ou {@code null} se a informao no estiver
   *  disponvel.
   */
  public String getSgaName() {
    return sgaName;
  }

  /**
   * Obtm a data inicial.
   * 
   * @return A data inicial ou {@code null} se a informao no estiver
   *  disponvel.
   */
  public Date getStartDate() {
    return startDate;
  }

  /**
   * Obtm o estado do comando.
   * 
   * @return O estado do comando.
   */
  public Status getStatus() {
    return status;
  }

  /**
   * Obtm o identificador do usurio.
   * 
   * @return O identificador do usurio ou {@code null} se a informao no 
   * estiver disponvel.
   */
  public Object getUserId() {
    return userId;
  }

  @Override
  public int hashCode() {
    return id.hashCode();
  }

  /**
   * Atribui a descrio.
   * 
   * @param description A descrio (aceita {@code null}).
   */
  public void setDescription(String description) {
    if ((description != null) && (description.length() == 0)) {
      this.description = null;
    }
    else {
      this.description = description;
    }
  }

  /**
   * Atribui o estado a este comando.
   * 
   * @param status O estado do comando ({@code null} para {@link Status#UNKNOWN}).
   */
  public void setStatus(Status status) {
    if (status == null) {
      status = Status.UNKNOWN;
    }
    this.status = status;
  }

  @Override
  public String toString() {
    String text = String.format("Identificador do comando: %s.\n", id);
    if (sgaName != null) {
      text += String.format("Nome do SGA: %s\n", startDate);
    }
    if (userId != null) {
      text += String.format("Identificador do usurio: %s\n", userId);
    }
    if (startDate != null) {
      text += String.format(
        "Data de requisio: %1$tY/%1$tm/%1$td - %1$tH:%1$tM:%1$tS\n",
        startDate);
    }
    if (description != null) {
      text += String.format("Descrio: %s\n", description);
    }
    return text;
  }

  /**
   * Atribui o identificador.
   * 
   * @param id O identificador (no aceita {@code null}).
   */
  private void setId(String id) {
    if (id == null) {
      throw new IllegalArgumentException("O parmetro id est nulo.");
    }
    this.id = id;
  }
  
  /**
   * Atribui dica para auxiliar a identificao do comando. 
   * 
   * @param id Dica para auxiliar a identificao do comando. 
   *        Se {@code tip==null}, ento {@code tip=""}. 
   */
  private void setTip(String tip) {
    if (tip == null) {
      this.tip = "";
    }
    else {      
      this.tip = tip;
    }
  }

  /**
   * Atribui o nome do sga a esta propriedade.
   * 
   * @param sgaName O nome do sga (aceita {@code null}).
   */
  private void setSgaName(String sgaName) {
    this.sgaName = sgaName;
  }

  /**
   * Obtm a data inicial.
   * 
   * @param startDate A data inicial (aceita {@code null}).
   */
  private void setStartDate(Date startDate) {
    this.startDate = startDate;
  }

  /**
   * Atribui o identificador do projeto a esta propriedade.
   * 
   * @param projectId O identificador do projeto (no aceita {@code null}).
   */
  private void setProjectId(Object projectId) {
    if (projectId == null) {
      throw new IllegalArgumentException("O parmetro projectId est nulo.");
    }
    this.projectId = projectId;
  }
  
  /**
   * Atribui o identificador do usurio a esta propriedade.
   * 
   * @param userId O identificador do usurio (aceita {@code null}).
   */
  private void setUserId(Object userId) {
    this.userId = userId;
  }
  
  /**
   * O estado do comando.
   */
  public enum Status {
    /**
     * Terminado.
     */
    FINISHED,
    /**
     * Em execuo.
     */
    IS_RUNNING,
    /**
     * Desconhecido.
     */
    UNKNOWN,
  }
}
