package csbase.logic.algorithms.flows.configurator;

import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import csbase.exception.BugException;
import csbase.exception.ParseException;
import csbase.exception.algorithms.ParameterNotFoundException;
import csbase.logic.algorithms.AlgorithmConfigurator;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.logic.algorithms.CommandLineContext;
import csbase.logic.algorithms.ExecutionType;
import csbase.logic.algorithms.FileParameterValue;
import csbase.logic.algorithms.flows.Flow;
import csbase.logic.algorithms.flows.FlowLink;
import csbase.logic.algorithms.flows.FlowNode;
import csbase.logic.algorithms.flows.LinkParameter;
import csbase.logic.algorithms.flows.NodeParameter;
import csbase.logic.algorithms.parameters.InputFileParameter;
import csbase.logic.algorithms.parameters.OutputFileParameter;

/**
 * <p>
 * Configurador de Fluxo de Algoritmos:
 * </p>
 * 
 * <p>
 * Um fluxo de algoritmos  um multigrafo orientado cujos os vrtices so os ns
 * de algoritmo e as aretas so conexes entre parmetro do tipo arquivo destes
 * algoritmos.
 * </p>
 * 
 * @author lmoreira
 */
public final class FlowAlgorithmConfigurator extends AlgorithmConfigurator {
  /**
   * O ndice da captura da expresso {@link #PARAMETER_NAME_REGEX} que se
   * refere ao identificador do n.
   */
  private static final int NODE_ID_INDEX = 1;
  /**
   * A expresso regular que  utilizada para entender os nomes codificados para
   * parmetros nos mtodos: {@link #getParameterNames()},
   * {@link #getParameterLabel(String)}, {@link #getParameterType(String)},
   * {@link #getParameterValue(String)},
   * {@link #setParameterValue(String, String)}.
   */
  private static final String PARAMETER_NAME_REGEX = "([0-9]*)\\.(.*)";

  /**
   * O ndice da captura da expresso {@link #PARAMETER_NAME_REGEX} que se
   * refere ao nome do parmetro.
   */
  private static final int REAL_PARAMETER_NAME_INDEX = 2;

  /**
   * Uma estrutura de dados que descreve o fluxo.
   */
  private Flow flow;

  /**
   * Os ns indexados pelo identificador.
   */
  private Map<Integer, Node> nodesById;

  /**
   * Determina se  possvel executar com a configurao atual (levando em
   * consideraco possveis desvios feitos no fluxo)
   */
  private boolean canBeExecuted;

  /**
   * Cria um configurador.
   * 
   * @param algorithmVersion A verso do algoritmo (No aceita {@code null}).
   * @param flow A estrutura que descreve o fluxo (No aceita {@code null}).
   */
  public FlowAlgorithmConfigurator(AlgorithmVersionInfo algorithmVersion,
    Flow flow) {
    super(algorithmVersion, null, ExecutionType.SIMPLE, "FLOWABBR", false);
    setFlow(flow);
  }

  /**
   * Cria um configurador.
   * 
   * @param filePath O caminho para o arquivo que descreve o fluxo (No aceita
   *        {@code null}).
   * 
   * @param flow A estrutura que descreve o fluxo (No aceita {@code null}).
   */
  public FlowAlgorithmConfigurator(String filePath, Flow flow) {
    super(filePath, null, ExecutionType.SIMPLE, "FLOWABBR", false);
    setFlow(flow);
  }

  /**
   * Cria um configurador.
   * 
   * @param flow A estrutura que descreve o fluxo (No aceita {@code null}).
   */
  public FlowAlgorithmConfigurator(Flow flow) {
    super(null, ExecutionType.SIMPLE, "FLOWABBR", false);
    setFlow(flow);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Map<Object, Object> exportValues() {
    Map<Object, Object> values = new HashMap<Object, Object>();
    for (Node node : this.nodesById.values()) {
      values.put(node.getId(), node.getConfigurator().exportValues());
    }
    return values;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Set<String> getBinaryDirectories(String platformId,
    String fileSeparator) {
    Set<String> binaryDirectories = new HashSet<String>();
    for (Node node : this.nodesById.values()) {
      AlgorithmConfigurator configurator = node.getConfigurator();
      binaryDirectories.addAll(configurator.getBinaryDirectories(platformId,
        fileSeparator));
    }
    return binaryDirectories;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Set<FileParameterValue> getDirectories() {
    Set<FileParameterValue> directories = new HashSet<FileParameterValue>();
    for (Node node : this.nodesById.values()) {
      AlgorithmConfigurator configurator = node.getConfigurator();
      Set<FileParameterValue> currentDirectories =
        configurator.getDirectories();
      directories.addAll(currentDirectories);
    }
    return Collections.unmodifiableSet(directories);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public List<InputFileParameter> getInputFileParameters() {
    List<InputFileParameter> inputFileParameters =
      new LinkedList<InputFileParameter>();
    for (Node node : getNodes()) {
      AlgorithmConfigurator configurator = node.getConfigurator();
      inputFileParameters.addAll(configurator.getInputFileParameters());
    }
    return Collections.unmodifiableList(inputFileParameters);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Set<FileParameterValue> getInputFiles() {
    Set<FileParameterValue> inputFiles = new HashSet<FileParameterValue>();
    for (Node node : this.nodesById.values()) {
      AlgorithmConfigurator configurator = node.getConfigurator();
      Set<FileParameterValue> currentInputFiles = configurator.getInputFiles();
      inputFiles.addAll(currentInputFiles);
    }
    return Collections.unmodifiableSet(inputFiles);
  }

  /**
   * Obtm a estrutura que descreve o fluxo.
   * 
   * @return a estrutura que descreve o fluxo.
   */
  public Flow getFlow() {
    return this.flow;
  }

  /**
   * <p>
   * Obtm todos os arquivos de log.
   * </p>
   * 
   * <p>
   * O conjunto retornado  imutvel (Collections.unmodifiableSet(Set)).
   * </p>
   * 
   * @return o conjunto de arquivos de log (se no existir arquivo de log,
   *         retornar um conjunto vazio).
   */
  @Override
  public Set<FileParameterValue> getLogFiles() {
    Set<FileParameterValue> logFiles = new HashSet<FileParameterValue>();
    for (Node node : getNodes()) {
      AlgorithmConfigurator configurator = node.getConfigurator();
      if (configurator instanceof FlowAlgorithmConfigurator) {
        FlowAlgorithmConfigurator flowConfigurator =
          (FlowAlgorithmConfigurator) configurator;
        logFiles.addAll(flowConfigurator.getLogFiles());
      }
      else {
        FileParameterValue logFile = configurator.getLogFile();
        if (logFile != null) {
          logFiles.add(logFile);
        }
      }
    }
    return Collections.unmodifiableSet(logFiles);
  }

  /**
   * Obtm o nome do fluxo.
   * 
   * @return O nome do fluxo ou {@code null} se ele no tiver um nome.
   */
  public String getName() {
    return this.flow.getName();
  }

  /**
   * Obtm um n de algoritmo atravs do seu identificador.
   * 
   * @param id O identificador.
   * 
   * @return O n ou {@code null} se ele no existir.
   */
  public Node getNode(int id) {
    return this.nodesById.get(id);
  }

  /**
   * <p>
   * Obtm o conjunto de ns deste fluxo.
   * </p>
   * 
   * <p>
   * O conjunto retornado  imutvel (veja
   * {@link Collections#unmodifiableSet(Set)}).
   * </p>
   * 
   * @return O conjunto (Retorna um conjunto vazio se no houver ns).
   */
  public Set<Node> getNodes() {
    Set<Node> nodes = new HashSet<Node>();
    nodes.addAll(nodesById.values());
    return Collections.unmodifiableSet(nodes);
  }

  /**
   * Determina se o fluxo contm algum n que est sendo desviado.
   * 
   * @return verdadeiro se o fluxo contm pelo menos um n desviado ou falso
   *         caso contrrio.
   */
  public boolean hasBypassedNodes() {
    for (FlowNode node : flow.getNodes()) {
      if (node.isBypassed()) {
        return true;
      }
    }
    return false;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public List<OutputFileParameter> getOutputFileParameters() {
    List<OutputFileParameter> outputFileParameters =
      new LinkedList<OutputFileParameter>();
    for (Node node : getNodes()) {
      AlgorithmConfigurator configurator = node.getConfigurator();
      outputFileParameters.addAll(configurator.getOutputFileParameters());
    }
    return Collections.unmodifiableList(outputFileParameters);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Set<FileParameterValue> getOutputFiles() {
    Set<FileParameterValue> outputFiles = new HashSet<FileParameterValue>();
    for (Node node : this.nodesById.values()) {
      AlgorithmConfigurator configurator = node.getConfigurator();
      Set<FileParameterValue> currentOutputFiles =
        configurator.getOutputFiles();
      outputFiles.addAll(currentOutputFiles);
    }
    return Collections.unmodifiableSet(outputFiles);
  }

  /**
   * @param encodedParameterName Nome codificado do parmetro. Para uma
   *        explicao sobre o nome codificado veja {@link #getParameterNames()}
   *        .
   */
  @Override
  public String getParameterLabel(String encodedParameterName)
    throws ParameterNotFoundException {
    Integer nodeId = getNodeIdFromEncodedParameterName(encodedParameterName);
    if (nodeId == null) {
      throw new ParameterNotFoundException(encodedParameterName);
    }
    Node node = nodesById.get(nodeId);
    if (node == null) {
      throw new ParameterNotFoundException(encodedParameterName);
    }
    String realParameterName =
      getRealParameterNameFromEncodedParameterName(encodedParameterName);
    if (realParameterName == null) {
      return null;
    }
    return node.getConfigurator().getParameterLabel(realParameterName);
  }

  /**
   * <p>
   * Os nomes dos parmetros so codificados utilizando com a seguinte
   * gramtica:
   * 
   * <pre>
   * &lt;nome_codificado&gt; := &lt;id_do_no&gt;&lt;nome_codificado&gt;
   * ou
   * &lt;nome_codificado&gt; := &lt;nome_do_parametro&gt;
   * </pre>
   * 
   * <ul>
   * <li>nome_codificado  o nome do codificado do parmetro.</li>
   * <li>id_do_no  o identificador do n.</li>
   * <li>nome_do_parametro  o nome real do parmetro.</li>
   * </ul>
   * </p>
   */
  @Override
  public Set<String> getParameterNames() {
    Set<String> allParameterNames = new HashSet<String>();
    for (Node node : nodesById.values()) {
      AlgorithmConfigurator configurator = node.getConfigurator();
      Set<String> thisNodeParameterNames = configurator.getParameterNames();
      for (String parameterName : thisNodeParameterNames) {
        allParameterNames.add(node.getId() + "." + parameterName);
      }
    }
    return Collections.unmodifiableSet(allParameterNames);
  }

  /**
   * @param encodedParameterName Nome codificado do parmetro. Para uma
   *        explicao sobre o nome codificado veja {@link #getParameterNames()}
   *        .
   */
  @Override
  public String getParameterType(String encodedParameterName)
    throws ParameterNotFoundException {
    Integer nodeId = getNodeIdFromEncodedParameterName(encodedParameterName);
    if (nodeId == null) {
      throw new ParameterNotFoundException(encodedParameterName);
    }
    Node node = nodesById.get(nodeId);
    if (node == null) {
      throw new ParameterNotFoundException(encodedParameterName);
    }
    String realParameterName =
      getRealParameterNameFromEncodedParameterName(encodedParameterName);
    if (realParameterName == null) {
      return null;
    }
    return node.getConfigurator().getParameterType(realParameterName);
  }

  /**
   * @param encodedParameterName Nome codificado do parmetro. Para uma
   *        explicao sobre o nome codificado veja {@link #getParameterNames()}
   *        .
   */
  @Override
  public String getParameterValue(String encodedParameterName)
    throws ParameterNotFoundException {
    Integer nodeId = getNodeIdFromEncodedParameterName(encodedParameterName);
    if (nodeId == null) {
      throw new ParameterNotFoundException(encodedParameterName);
    }
    Node node = nodesById.get(nodeId);
    if (node == null) {
      throw new ParameterNotFoundException(encodedParameterName);
    }
    String realParameterName =
      getRealParameterNameFromEncodedParameterName(encodedParameterName);
    if (realParameterName == null) {
      return null;
    }
    return node.getConfigurator().getParameterValue(realParameterName);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Set<String> getPlatforms() {
    Set<String> platforms = new HashSet<String>();
    if (!this.nodesById.isEmpty()) {
      Iterator<Node> nodeIterator = this.nodesById.values().iterator();
      Node node = nodeIterator.next();
      AlgorithmConfigurator configurator = node.getConfigurator();
      platforms.addAll(configurator.getPlatforms());
      while (nodeIterator.hasNext()) {
        node = nodeIterator.next();
        configurator = node.getConfigurator();
        platforms.retainAll(configurator.getPlatforms());
      }
    }
    return Collections.unmodifiableSet(platforms);
  }

  /**
   * {@inheritDoc}
   */
  @SuppressWarnings("unchecked")
  @Override
  public void importValues(Map<Object, Object> values) throws ParseException {
    for (Node node : this.nodesById.values()) {
      Map<Object, Object> nodeValues =
        (Map<Object, Object>) values.get(node.getId());
      if (nodeValues != null) {
        node.getConfigurator().importValues(nodeValues);
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean isEnabled() {
    for (Node node : nodesById.values()) {
      AlgorithmConfigurator configurator = node.getConfigurator();
      if (configurator != null) {
        if (configurator.isEnabled()) {
          return true;
        }
      }
    }
    return false;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean isSetDefaultValues() {
    for (Node node : this.nodesById.values()) {
      if (!node.getConfigurator().isSetDefaultValues()) {
        return false;
      }
    }
    return true;
  }

  /**
   * O comando gerado cria named-pipes para representar as conexes e permitir
   * comunicar os dados entre os algoritmos. Ele tambm solicita a execuo
   * simultnea de todos os comandos gerados por cada um dos ns de algoritmos.
   */
  @Override
  public String makeCommandLine(CommandLineContext context) {
    if (this.nodesById.isEmpty()) {
      return null;
    }
    if (this.nodesById.size() == 1) {
      Iterator<Node> nodeIterator = this.nodesById.values().iterator();
      Node node = nodeIterator.next();
      AlgorithmConfigurator configurator = node.getConfigurator();
      return configurator.makeCommandLine(context);
    }
    /*
     * S pode setar como fluxo aqui, no caso de ter mais de um n, para que a
     * linha de comando seja montada da maneira correta
     */
    context.setIsFlow(true);
    Map<Integer, Map<String, String>> fromPipes = getFromPipes();
    Map<Integer, Map<String, String>> toPipes = getToPipes();
    Collection<String> tees = getTees(fromPipes, toPipes);
    List<String> commands = makeCommands(context, fromPipes, toPipes);
    String command = "./flowmonitor";
    command += " \"";
    for (Map<String, String> pipes : fromPipes.values()) {
      for (String pipe : pipes.values()) {
        command += pipe;
        command += " ";
      }
    }
    for (Map<String, String> pipes : toPipes.values()) {
      for (String pipe : pipes.values()) {
        command += pipe;
        command += " ";
      }
    }
    command += "\"";
    for (String tee : tees) {
      command += " \"";
      command += tee;
      command += "\"";
    }
    for (String cmd : commands) {
      command += " \"";
      command += cmd;
      command += "\"";
    }
    return command;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void resetValues() {
    for (Node node : this.nodesById.values()) {
      node.getConfigurator().resetValues();
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean setDefaultInputFile(FileParameterValue inputFile) {
    return false;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean setEnabled(boolean isEnabled) {
    if (isEnabled() == isEnabled) {
      return false;
    }
    for (Node node : nodesById.values()) {
      AlgorithmConfigurator configurator = node.getConfigurator();
      if (configurator != null) {
        configurator.setEnabled(isEnabled);
      }
    }
    fireWasSetEnabled();
    return true;
  }

  /**
   * @param encodedParameterName Nome codificado do parmetro. Para uma
   *        explicao sobre o nome codificado veja {@link #getParameterNames()}
   *        .
   */
  @Override
  public void setParameterValue(String encodedParameterName,
    String parameterValue) throws ParameterNotFoundException, ParseException {
    Integer nodeId = getNodeIdFromEncodedParameterName(encodedParameterName);
    if (nodeId == null) {
      throw new ParameterNotFoundException(encodedParameterName);
    }
    Node node = nodesById.get(nodeId);
    if (node == null) {
      throw new ParameterNotFoundException(encodedParameterName);
    }
    String realParameterName =
      getRealParameterNameFromEncodedParameterName(encodedParameterName);
    if (realParameterName == null) {
      throw new ParameterNotFoundException(encodedParameterName);
    }
    AlgorithmConfigurator configurator = node.getConfigurator();
    configurator.setParameterValue(realParameterName, parameterValue);

    // Altera o valor no parmetro armazenad o no Flow.
    Flow flow = getFlow();
    if (null != flow) {
      FlowNode flowNode = flow.getNode(node.getId());
      if (null != flowNode) {
        NodeParameter flowNodeParameter =
          flowNode.getParameter(realParameterName);
        if (null != flowNodeParameter) {
          flowNodeParameter.setValue(parameterValue);
        }
      }
    }
  }

  /**
   * <p>
   * Atribui o valor do arquivo que deve receber a sada padro da execuo de
   * um algoritmo representado por este configurador.
   * </p>
   * <p>
   * Este valor ser repassado para todos os ns deste fluxo.
   * </p>
   * 
   * @param standardOutputFile
   */
  @Override
  public void setStandardOutputFile(FileParameterValue standardOutputFile) {
    flow.setStandardOutputFile(standardOutputFile);
  }

  /**
   * Obtm um conjunto imutvel contendo os arquivos de sada padro de cada n
   * que sero utilizadas durante a execuo de um fluxo.
   * 
   * @return um conjunto imutvel contendo os arquivos de sada padro de cada
   *         n que sero utilizadas durante a execuo de um fluxo.
   */
  @Override
  public Set<FileParameterValue> getStandardOutputFiles() {
    return flow.getStandardOutputFiles();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void setExitCodeLogFile(FileParameterValue exitCodeLogFile) {
    flow.setExitCodeLogFile(exitCodeLogFile);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Set<FileParameterValue> getExitCodeLogFiles() {
    return flow.getExitCodeLogFiles();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean hasExitCode() {
    return flow.hasExitCode();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected String getCurrentParameterFileVersion() {
    return "1.0";
  }

  /**
   * Obtm os caminhos para os named-pipes se originam dos ns indexados pelo
   * identificador do n.
   * 
   * @return O mapa associativo (ele estar vazio se no houver ns que originam
   *         conexes).
   */
  private Map<Integer, Map<String, String>> getFromPipes() {
    Random random;
    try {
      random = SecureRandom.getInstance("SHA1PRNG");
    }
    catch (NoSuchAlgorithmException e) {
      e.printStackTrace();
      random = new Random();
    }
    Map<Integer, Map<String, String>> fromPipes =
      new HashMap<Integer, Map<String, String>>();
    for (Node node : this.nodesById.values()) {
      Map<String, String> pipes = fromPipes.get(node.getId());
      if (pipes == null) {
        pipes = new HashMap<String, String>();
        fromPipes.put(node.getId(), pipes);
      }
      for (Output output : node.getOutputs()) {
        if (!output.getInputs().isEmpty()) {
          String fromNamedPipe = "/tmp/.from" + random.nextLong();
          pipes.put(output.getParameterName(), fromNamedPipe);
        }
      }
    }
    return fromPipes;
  }

  /**
   * Obtm o identificador do n a partir do nome do parmetro codificado.
   * 
   * @param encodedParameterName O nome de um parmetro codificado (No pode ser
   *        {@code null}).
   * 
   * @return O identificador do n ou {@code null} o texto informado no estiver
   *         no formato esperado.
   */
  private Integer getNodeIdFromEncodedParameterName(String encodedParameterName) {
    Pattern pattern = Pattern.compile(PARAMETER_NAME_REGEX);
    Matcher matcher = pattern.matcher(encodedParameterName);
    if (!matcher.matches()) {
      return null;
    }
    String nodeIdAsText = matcher.group(NODE_ID_INDEX);
    return Integer.parseInt(nodeIdAsText);
  }

  /**
   * Obtm o nome do parmetro a partir do nome do parmetro codificado.
   * 
   * @param encodedParameterName O nome de um parmetro codificado (No pode ser
   *        {@code null}).
   * 
   * @return O nome do parmetro ou {@code null} o texto informado no estiver
   *         no formato esperado.
   */
  private String getRealParameterNameFromEncodedParameterName(
    String encodedParameterName) {
    Pattern pattern = Pattern.compile(PARAMETER_NAME_REGEX);
    Matcher matcher = pattern.matcher(encodedParameterName);
    if (!matcher.matches()) {
      return null;
    }
    return matcher.group(REAL_PARAMETER_NAME_INDEX);
  }

  /**
   * Obtm os comandos <i>tee</i> que devem ser utilizados na linha de comando
   * gerada por este configurador de fluxo de algoritmos.
   * 
   * @param fromPipes Os pipes de entrada (No pode ser {@code null}).
   * @param toPipes Os pipes de sada (No pode ser {@code null}).
   * 
   * @return Uma coleo no-modificvel com os comandos (a coleo estar
   *         vazia, caso no haja necessidade de criar tees).
   */
  private Collection<String> getTees(
    Map<Integer, Map<String, String>> fromPipes,
    Map<Integer, Map<String, String>> toPipes) {
    Collection<String> tees = new LinkedList<String>();
    Collection<Node> nodes = this.nodesById.values();
    for (Node node : nodes) {
      for (Output output : node.getOutputs()) {
        Collection<Input> inputs = output.getInputs();
        if (!inputs.isEmpty()) {
          String tee = "tee ";
          for (Input input : inputs) {
            int nodeId = input.getNodeId();
            String parameterName = input.getParameterName();
            String toPipe = toPipes.get(nodeId).get(parameterName);
            tee += toPipe + " ";
          }
          int nodeId = output.getNodeId();
          String parameterName = output.getParameterName();
          String fromPipe = fromPipes.get(nodeId).get(parameterName);
          tee += "< ";
          tee += fromPipe;
          tee += " > /dev/null ";
          tees.add(tee);
        }
      }
    }
    return tees;
  }

  /**
   * Obtm os caminhos para os named-pipes se destinam a ns indexados pelo
   * identificador do n.
   * 
   * @return O mapa associativo (ele estar vazio se no houver ns que so
   *         destino de conexes).
   */
  private Map<Integer, Map<String, String>> getToPipes() {
    Random random;
    try {
      random = SecureRandom.getInstance("SHA1PRNG");
    }
    catch (NoSuchAlgorithmException e) {
      e.printStackTrace();
      random = new Random();
    }
    Map<Integer, Map<String, String>> toPipes =
      new HashMap<Integer, Map<String, String>>();
    for (Node node : this.nodesById.values()) {
      for (Output output : node.getOutputs()) {
        for (Input input : output.getInputs()) {
          Map<String, String> pipes = toPipes.get(input.getNodeId());
          if (pipes == null) {
            pipes = new HashMap<String, String>();
            toPipes.put(input.getNodeId(), pipes);
          }
          String toNamedPipe = "/tmp/.to" + random.nextLong();
          pipes.put(input.getParameterName(), toNamedPipe);
        }
      }
    }
    return toPipes;
  }

  /**
   * Obtm a lista com todos os comandos gerados por cada um dos ns do fluxo.
   * 
   * @param context O contexto para a criao da linha de comando.
   * @param fromPipes Um mapa com os pipe de sada de um n (No pode ser
   *        {@code null}).
   * @param toPipes Um mapa com os pipe de entrada para um n (No pode ser
   *        {@code null}).
   * 
   * @return A lista de comandos (se no houver ns, a lista estar vazia).
   */
  private List<String> makeCommands(CommandLineContext context,
    Map<Integer, Map<String, String>> fromPipes,
    Map<Integer, Map<String, String>> toPipes) {
    List<String> commands = new LinkedList<String>();
    for (Node node : this.nodesById.values()) {
      AlgorithmConfigurator configurator = node.getConfigurator();
      Map<String, String> parameterValuesByName = new HashMap<String, String>();
      for (String parameterName : configurator.getParameterNames()) {
        try {
          String parameterValue = configurator.getParameterValue(parameterName);
          parameterValuesByName.put(parameterName, parameterValue);
        }
        catch (ParameterNotFoundException e) {
          throw new BugException(e);
        }
      }
      Map<String, String> toPipesByParameterName = toPipes.get(node.getId());
      if (toPipesByParameterName != null) {
        parameterValuesByName.putAll(toPipesByParameterName);
      }
      Map<String, String> fromPipesByParameterName =
        fromPipes.get(node.getId());
      if (fromPipesByParameterName != null) {
        parameterValuesByName.putAll(fromPipesByParameterName);
      }
      try {
        configurator.setParameterValuesByName(parameterValuesByName);
      }
      catch (ParseException e) {
        throw new BugException(e);
      }
      String command = node.getConfigurator().makeCommandLine(context);
      if (command != null) {
        commands.add(command);
      }
    }
    return commands;
  }

  /**
   * Atribui a estrutura que descreve o fluxo de algoritmos a este configurador.
   * 
   * @param flow A estrutura (No pode ser {@code null}).
   */
  private void setFlow(Flow flow) {
    if (flow == null) {
      throw new IllegalArgumentException("O parmetro flow est nulo.");
    }
    this.flow = flow;
    this.canBeExecuted = false;
    Map<Integer, Node> nodesMap = createNodesMap(flow);
    if (nodesMap != null) {
      this.nodesById = nodesMap;
      this.canBeExecuted = true;
    }
    else {
      this.nodesById = Collections.emptyMap();
    }
  }

  /**
   * Retorna se  possvel executar o fluxo com a configurao atual (levando em
   * consideraco possveis desvios feitos no fluxo)
   * 
   * @return verdadeiro se for possvel executar o fluxo com a configurao
   *         atual ou falso caso contrrio
   */
  public boolean canBeExecuted() {
    return canBeExecuted;
  }

  /**
   * Extrai os ns {@link FlowNode} e ligaes {@link FlowLink} de um fluxo
   * {@link Flow} e o coloca num mapa de ns {@link Node} indexados pelos seus
   * identificadores.
   * 
   * @param flow Fluxo a ser utilizado para a criao do mapa.
   * 
   * @return mapa com os ns (no desviados) do fluxo indexados por seus
   *         identificadores.
   */
  protected Map<Integer, Node> createNodesMap(Flow flow) {
    Map<Integer, Node> nodesMap = new HashMap<Integer, Node>();
    if (isConfigured(flow)) {
      addNodes(nodesMap, flow);
      if (addLinks(nodesMap, flow)) {
        return nodesMap;
      }
    }
    return null;
  }

  /**
   * Determina se um fluxo est configurado (todos os algoritmos do fluxo tm
   * configurador)
   * 
   * @param flow O fluxo que deve ser testado.
   * @return verdadeiro se o fluxo estiver pronto para execuo ou falso caso
   *         contrrio.
   */
  protected boolean isConfigured(Flow flow) {
    for (FlowNode flowNode : flow.getNodes()) {
      AlgorithmConfigurator algorithmConfigurator =
        flowNode.getAlgorithmConfigurator();
      if (algorithmConfigurator == null) {
        return false;
      }
    }
    return true;
  }

  /**
   * Adiciona os ns do fluxo no mapa indexado por seus identificadores. Ns que
   * esto sendo marcados como desviados no so adicionados porque no devero
   * ser executados.
   * 
   * @param nodesMap mapa com os ns (no desviados) do fluxo indexados por seus
   *        identificadores.
   * @param flow O fluxo a ser utilizado.
   */
  protected void addNodes(Map<Integer, Node> nodesMap, Flow flow) {
    for (FlowNode flowNode : flow.getNodes()) {
      if (!flowNode.isBypassed()) {
        AlgorithmConfigurator algorithmConfigurator =
          flowNode.getAlgorithmConfigurator();
        Node node = new Node(flowNode.getId(), algorithmConfigurator);
        nodesMap.put(node.getId(), node);
      }
    }
  }

  /**
   * Adiciona as ligaes do fluxo nos ns que esto no mapa. Caso hajam ns
   * desviados no fluxo, pode no ser possvel determinar automaticamente como
   * as ligaes devem ser feitas nos ns restantes. Nesse caso, o mtodo
   * retorna falso. Se for possvel determinar, sem ambiguidades, como as
   * ligaes devem ser feitas, o mtodo retorna verdadeiro.
   * 
   * @param nodesMap mapa com os ns (no desviados) do fluxo indexados por seus
   *        identificadores.
   * @param flow O fluxo a ser utilizado.
   * @return verdadeiro se foi possvel adicionar todos os links sem
   *         ambiguidade.
   */
  protected boolean addLinks(Map<Integer, Node> nodesMap, Flow flow) {
    /*
     * Lista de ligaes cujos outputs esto conectadas a ns desviados. Cada
     * uma dessas ligaes vai ser analisada para se tentar determinar a qual n
     * ela seu output dever ser ligado.
     */
    List<FlowLink> pendingLinks = new ArrayList<FlowLink>();
    /*
     * Lista de ligaes cujos inputs esto conectados a ns desviados. Essas
     * ligaes vo ser ignoradas na execuo do fluxo.
     */
    List<FlowLink> bypassedLinks = new ArrayList<FlowLink>();

    for (FlowLink flowLink : flow.getLinks()) {
      LinkParameter inputLinkParameter = flowLink.getInput();
      int inputNodeId = inputLinkParameter.getNodeId();
      Node inputNode = nodesMap.get(inputNodeId);
      /*
       * Se inputNode estiver nulo,  porque o n no foi adicionado ao mapa, o
       * que quer dizer que foi feito desvio desse n e, portanto, essa aresta
       * vai ser ignorada durante a execuo.
       */
      if (inputNode == null) {
        bypassedLinks.add(flowLink);
      }
      else {
        LinkParameter outputLinkParameter = flowLink.getOutput();
        int outputNodeId = outputLinkParameter.getNodeId();
        Node outputNode = nodesMap.get(outputNodeId);
        /*
         * Se outputNode estiver nulo,  porque o n no foi adicionado ao mapa,
         * o que quer dizer que foi feito desvio desse n e, portanto, essa
         * aresta vai ter que ser analisada para ver se pode ser ligada a outro
         * n.
         */
        if (outputNode == null) {
          pendingLinks.add(flowLink);
        }
        else {
          /*
           * Se a ligao no estiver conectada a nenhum n desviado, pode ser
           * adiciona sem alteraes
           */
          addLink(outputNode, outputLinkParameter, inputLinkParameter);
        }
      }
    }
    if (pendingLinks.size() > 0) {
      /*
       * Adicionar ligaes que ficaram pendentes
       */
      return addPendingLinks(nodesMap, pendingLinks, bypassedLinks);
    }
    return true;
  }

  /**
   * Adiciona ligao ao n de sada atravs dos parmetros de entrada e sada
   * especificados.
   * 
   * @param outputNode N ao qual ser adicionado a ligao.
   * @param outputLinkParameter Parmetro de sada da ligao.
   * @param inputLinkParameter Parmetro de entrada da ligao.
   */
  protected void addLink(Node outputNode, LinkParameter outputLinkParameter,
    LinkParameter inputLinkParameter) {
    Input input =
      new Input(inputLinkParameter.getNodeId(), inputLinkParameter.getName());
    Output output = outputNode.getOutput(outputLinkParameter.getName());
    output.addInput(input);
  }

  /**
   * Adiciona ligaes que ficaram pendentes, se for possvel determinar sem
   * ambiguidade como as ligaes devem ser reconectadas de acordo com os
   * desvios feitos no fluxo. Para cada ligao pendente  procurada uma nica
   * ligao desviada que ela vai substituir. Se no for possvel encontrar
   * nenhuma ligao equivalente ou existirem diversas ligaes que podem ser
   * equivalentes, o fluxo no poder ser executado com a configurao atual. O
   * usurio deve fazer o desvio manualmente ou refazer os desvios para que no
   * gerem ambiguidades.
   * 
   * @param nodesMap mapa com os ns (no desviados) do fluxo indexados por seus
   *        identificadores.
   * @param pendingLinks ligaes que devem ser refeitas.
   * @param bypassedLinks ligaes que devem ser ignoradas pelo fluxo.
   * @return verdadeiro se for possvel refazer as ligaes sem ambiguidade ou
   *         falso caso contrrio.
   */
  protected boolean addPendingLinks(Map<Integer, Node> nodesMap,
    List<FlowLink> pendingLinks, List<FlowLink> bypassedLinks) {
    /*
     * Ligaes agrupadas por output. Somente uma ligao de cada output precisa
     * ser analisada, j que todas devem ser conectadas ao mesmo n.
     */
    Map<LinkParameter, List<FlowLink>> pendingLinksByOutput =
      groupPendingLinksByOutput(pendingLinks);
    /*
     * Mapeia as ligaes que podem ser refeitas a suas ligaes desviadas
     * correspondentes.
     */
    Map<FlowLink, FlowLink> linkMatches = new HashMap<FlowLink, FlowLink>();
    for (List<FlowLink> pendingInputLinks : pendingLinksByOutput.values()) {
      FlowLink pendingInputLink = pendingInputLinks.get(0);
      FlowLink matchingLink = getMatchingLink(bypassedLinks, pendingInputLink);
      if (matchingLink != null) {
        for (FlowLink link : pendingInputLinks) {
          linkMatches.put(link, matchingLink);
        }
      }
    }
    if (areAllPendingLinksMatched(pendingLinks, linkMatches)) {
      if (areAllBypassedLinksMatched(nodesMap, bypassedLinks, linkMatches)) {
        for (FlowLink pendingLink : linkMatches.keySet()) {
          addPendingLink(nodesMap, pendingLink, linkMatches.get(pendingLink));
        }
        return true;
      }
    }
    return false;
  }

  /**
   * Verifica se foi possvel achar as ligaes correspondentes de todas as
   * ligaes pendentes.
   * 
   * @param pendingLinks Lista de ligaes pendentes.
   * @param linkMatches Mapa com as ligaes pendentes e as ligaes desviadas
   *        correspondentes.
   * @return verdadeiro se todas as ligaes pendentes tm suas correspondentes
   *         mapeadas ou falso caso contrrio.
   */
  private boolean areAllPendingLinksMatched(List<FlowLink> pendingLinks,
    Map<FlowLink, FlowLink> linkMatches) {
    return linkMatches.keySet().size() == pendingLinks.size();
  }

  /**
   * Verifica se todas as ligaes desviadas que deveriam ser consideradas foram
   * tratadas. @see {@link #countBypassedLinks(List, Map)}
   * 
   * @param nodesMap mapa com os ns (no desviados) do fluxo indexados por seus
   *        identificadores.
   * @param bypassedLinks Lista de ligaes que foram desviadas.
   * @param linkMatches Mapa com as ligaes pendentes e as ligaes desviadas
   *        correspondentes.
   * @return verdadeiro se todas as ligaes desviadas que deveriam ser
   *         consideradas foram tratadas ou falso caso contrrio.
   */
  protected boolean areAllBypassedLinksMatched(Map<Integer, Node> nodesMap,
    List<FlowLink> bypassedLinks, Map<FlowLink, FlowLink> linkMatches) {
    /*
     * Adiciona no set para verificar o nmero de ligaes correspondentes
     * nicas
     */
    Set<FlowLink> matchingLinksSet = new HashSet<FlowLink>();
    matchingLinksSet.addAll(linkMatches.values());
    return countBypassedLinks(bypassedLinks, nodesMap) == matchingLinksSet
      .size();
  }

  /**
   * Conta o nmero de ligaes desviadas que deveriam ser tratadas. As ligaes
   * que conectam dois ns desviados so descartas e ignoradas durante a
   * execuo. J as ligaes que s a entrada ou a sada esto conectadas a um
   * n desviado so contadas e devem ser tratadas. Cada ligao dessa deve
   * corresponder a pelo menos uma ligao pendente.
   * 
   * @param bypassedLinks Lista de ligaes que foram desviadas.
   * @param nodesMap mapa com os ns (no desviados) do fluxo indexados por seus
   *        identificadores.
   * @return nmero de ligaes desviadas que devem ser tratadas.
   */
  protected int countBypassedLinks(List<FlowLink> bypassedLinks,
    Map<Integer, Node> nodesMap) {
    List<FlowLink> links = new ArrayList<FlowLink>();
    for (FlowLink flowLink : bypassedLinks) {
      Node inputNode = nodesMap.get(flowLink.getInput().getNodeId());
      Node outputNode = nodesMap.get(flowLink.getOutput().getNodeId());
      if (inputNode != null || outputNode != null) {
        links.add(flowLink);
      }
    }
    return links.size();
  }

  /**
   * Adiciona a ligao pendente, substituindo a ligao desviada
   * correspondente.
   * 
   * @param nodesMap mapa com os ns (no desviados) do fluxo indexados por seus
   *        identificadores.
   * @param pendingLink Ligao pendente.
   * @param matchingLink Ligao desviada correspondente.
   */
  protected void addPendingLink(Map<Integer, Node> nodesMap,
    FlowLink pendingLink, FlowLink matchingLink) {
    LinkParameter inputParam = pendingLink.getInput();
    LinkParameter outputParam = matchingLink.getOutput();
    int newOutputNodeId = outputParam.getNodeId();
    Node outputNode = nodesMap.get(newOutputNodeId);
    addLink(outputNode, outputParam, inputParam);
  }

  /**
   * Agrupa as ligaes pendentes pelo parmetro de sada do n ao qual elas
   * esto conectadas.
   * 
   * @param pendingLinks Lista de ligaes pendentes.
   * @return mapa com as ligaes pendentes agrupadas pelo parmetro de sada do
   *         n ao qual esto conectadas.
   */
  protected Map<LinkParameter, List<FlowLink>> groupPendingLinksByOutput(
    List<FlowLink> pendingLinks) {
    Map<LinkParameter, List<FlowLink>> pendingLinksByOutput =
      new HashMap<LinkParameter, List<FlowLink>>();
    for (FlowLink link : pendingLinks) {
      List<FlowLink> linkList = pendingLinksByOutput.get(link.getOutput());
      if (linkList == null) {
        linkList = new ArrayList<FlowLink>();
        pendingLinksByOutput.put(link.getOutput(), linkList);
      }
      linkList.add(link);
    }
    return pendingLinksByOutput;
  }

  /**
   * Procura a ligao desviada que corresponde  ligao pendente. Retorna nulo
   * se no encontrar nenhuma ligao correspondente ou encontrar mais de uma
   * ligao correspondente (gerando ambiguidade).
   * 
   * @param bypassedLinks Lista de ligaes desviadas.
   * @param pendingInputLink Ligao pendente.
   * @return a ligao desviada que corresponde  ligao pendente ou nulo se
   *         no for possvel encontrar uma nica correspondente
   */
  protected FlowLink getMatchingLink(List<FlowLink> bypassedLinks,
    FlowLink pendingInputLink) {
    List<FlowLink> matchingLinks =
      getMatchingLinksByType(pendingInputLink, bypassedLinks);
    if (matchingLinks != null) {
      if (matchingLinks.size() == 1) {
        return matchingLinks.get(0);
      }
      else if (matchingLinks.size() > 1) {
        matchingLinks = getMatchingLinksByName(pendingInputLink, matchingLinks);
        if (matchingLinks != null && matchingLinks.size() == 1) {
          return matchingLinks.get(0);
        }
      }
    }
    return null;
  }

  /**
   * Retorna uma lista com todas as ligaes desviadas correspondentes que tm o
   * mesmo tipo da ligao pendente especificada (@see
   * {@link #getMatchingLinksByTypeRecursively}). Retorna nulo se no encontrar
   * nenhuma ligao correspondente com o tipo em questo.
   * 
   * @param pendingLink Ligao pendente
   * @param candidateLinks Lista de ligaes desviadas que tm o mesmo tipo da
   *        ligao pendente
   * @return lista com as ligaes correspondentes do tipo especificado ou nulo
   *         se no for possvel encontrar nenhuma ligao desviada
   *         correspondente do tipo especificado.
   */
  protected List<FlowLink> getMatchingLinksByType(FlowLink pendingLink,
    List<FlowLink> candidateLinks) {
    List<FlowLink> matchingLinks = new ArrayList<FlowLink>();
    for (FlowLink candidateLink : candidateLinks) {
      if (pendingLink.getOutput().getNodeId() == candidateLink.getInput()
        .getNodeId()) {
        matchingLinks.addAll(getMatchingLinksByTypeRecursively(pendingLink,
          candidateLink, candidateLinks));
      }
    }
    if (matchingLinks.size() > 0) {
      return matchingLinks;
    }
    else {
      return null;
    }
  }

  /**
   * Procura recursivamente as ligaes desviadas correspondentes que tm o
   * mesmo tipo da ligao pendente especificada a partir de uma ligao
   * candidata a ser a correspondente. Caso a ligao candidata esteja conectada
   * a sada de um n desviado, o mtodo  chamado recursivamente para as
   * ligaes de entrada deste n. Quando a ligao candidata est ligada a um
   * n que no foi desviado e tem o tipo correto, a ligao  considerada
   * correspondente.
   * 
   * @param pendingLink Ligao pendente
   * @param candidateLink Ligao candidata
   * @param candidateLinks Lista de ligaes desviadas candidatas
   * @return a lista de ligaes desviadas consideradas correspondentes
   */
  protected List<FlowLink> getMatchingLinksByTypeRecursively(
    FlowLink pendingLink, FlowLink candidateLink, List<FlowLink> candidateLinks) {
    List<FlowLink> matchingLinks = new ArrayList<FlowLink>();

    FlowNode parentNode = flow.getNode(candidateLink.getOutput().getNodeId());
    if (!parentNode.isBypassed()) {
      if (getInputParamType(pendingLink).equals(
        getInputParamType(candidateLink))) {
        matchingLinks.add(candidateLink);
      }
    }
    else {
      for (FlowLink link : candidateLinks) {
        if (link.getInput().getNodeId() == parentNode.getId()) {
          matchingLinks.addAll(getMatchingLinksByTypeRecursively(pendingLink,
            link, candidateLinks));
        }
      }
    }
    return matchingLinks;
  }

  /**
   * Retorna uma lista com todas as ligaes desviadas correspondentes que tm o
   * mesmo nome do parmetro de entrada da ligao pendente especificada.
   * Retorna nulo se no encontrar nenhuma ligao correspondente com o nome em
   * questo.
   * 
   * @param pendingLink Ligao pendente
   * @param candidateLinks Lista de ligaes desviadas candidatas
   * @return lista com as ligaes desviadas correspondentes que tm o mesmo
   *         nome do parmetro de entrada da ligao pendente especificada.
   */
  protected List<FlowLink> getMatchingLinksByName(FlowLink pendingLink,
    List<FlowLink> candidateLinks) {
    List<FlowLink> matchingLinks = new ArrayList<FlowLink>();
    for (FlowLink candidateLink : candidateLinks) {
      if (pendingLink.getOutput().getNodeId() == candidateLink.getInput()
        .getNodeId()) {
        String linkName = pendingLink.getInput().getName();
        String candidateLinkName = candidateLink.getInput().getName();
        if (linkName.equals(candidateLinkName)) {
          matchingLinks.add(candidateLink);
        }
      }
    }
    if (matchingLinks.size() > 0) {
      return matchingLinks;
    }
    else {
      return null;
    }
  }

  /**
   * Retorna o tipo do parmetro de entrada da ligao especificada.
   * 
   * @param link A ligao especificada.
   * @return o tipo do parmetro de entrada da ligao especificada.
   */
  protected String getInputParamType(FlowLink link) {
    LinkParameter input = link.getInput();
    FlowNode node = flow.getNode(input.getNodeId());
    String inputName = input.getName();
    NodeParameter inputParameter = node.getParameter(inputName);
    return inputParameter.getType();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public String toString() {
    String name = getName();
    if (name != null) {
      return name;
    }
    return super.toString();
  }
}
