/*
 * $Author:$ $Date:$ $Release:$
 */
package csbase.logic.algorithms.parameters;

import csbase.exception.ParseException;
import csbase.exception.algorithms.ExpressionFunctionExecutionException;
import csbase.exception.algorithms.ExpressionFunctionNotFoundException;
import csbase.exception.algorithms.FormulaCreationException;
import csbase.exception.algorithms.FormulaEvaluationException;
import csbase.exception.algorithms.ParameterIsOffException;
import csbase.exception.algorithms.ParameterNotFoundException;
import csbase.exception.algorithms.ParameterValueNotExistsException;

import tecgraf.javautils.formula.Formula;
import tecgraf.javautils.formula.FunctionCaller;
import tecgraf.javautils.formula.SymbolEvaluator;
import tecgraf.javautils.formula.Variant;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.Serializable;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;
import java.util.Vector;

public final class Expression implements Serializable {
  private SimpleAlgorithmConfigurator configurator;
  private boolean error;
  private String expressionText;
  private transient Formula formula;
  private Set<SimpleParameter> parameters;

  public Expression(String expressionText)
  throws FormulaCreationException, ParseException {
    this.parameters = new HashSet<SimpleParameter>();
    setExpressionText(expressionText);
    createFormula();
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == null) {
      return false;
    }
    if (!getClass().equals(obj.getClass())) {
      return false;
    }
    Expression expression = (Expression)obj;
    return toString().equals(obj.toString());
  }

  public boolean evaluate(SimpleAlgorithmConfigurator configurator)
  throws FormulaEvaluationException {
    if (configurator == null) {
      throw new IllegalArgumentException("O parmetro configurator est nulo.");
    }
    this.error = false;
    this.configurator = configurator;
    try {
      Variant variant = this.formula.evaluate();
      if (this.error) {
        return false;
      }
      return variant.getBooleanValue();
    }
    catch (ParameterIsOffException e) {
      return true;
    }
    catch (ParameterValueNotExistsException e) {
      return false;
    }
    catch (FormulaEvaluationException e) {
      throw e;
    }
    catch (Exception e) {
      throw new FormulaEvaluationException(
        e, "Ocorreu um erro ao avaliar a expresso {0}.\nDetalhes: {1}.",
        this, e.getLocalizedMessage());
    }
    finally {
      this.configurator = null;
    }
  }

  public SimpleAlgorithmConfigurator getConfigurator() {
    return this.configurator;
  }

  public Set<SimpleParameter> getParameters() {
    return Collections.unmodifiableSet(this.parameters);
  }

  @Override
  public int hashCode() {
    return toString().hashCode();
  }

  @Override
  public String toString() {
    return this.expressionText;
  }

  private boolean addParameter(SimpleParameter parameter) {
    if (parameter == null) {
      throw new IllegalArgumentException("O parmetro parameter est nulo.");
    }
    return this.parameters.add(parameter);
  }

  private void createFormula()
  throws FormulaCreationException, ParseException {
    try {
      this.formula = new Formula(this.expressionText);
    }
    catch (Exception e) {
      throw new FormulaCreationException(
        e, "Ocorreu erro ao criar a frmula {0}.\nDetalhes: {1}.\n", this,
        e.getLocalizedMessage());
    }
    this.formula.setFunctionCaller(new ExpressionFunctionCaller());
    this.formula.setSymbolEvaluator(new ExpressionSymbolEvaluator());
    try {
      this.formula.parse();
    }
    catch (Exception e) {
      throw new ParseException(
        e, "Ocorreu erro ao analisar a expresso {0}.\nDetalhes: {1}.\n", this,
        e.getLocalizedMessage());
    }
  }

  private void readObject(ObjectInputStream inputStream)
  throws IOException, ClassNotFoundException {
    inputStream.defaultReadObject();
    try {
      createFormula();
    }
    catch (FormulaCreationException e) {
      IllegalStateException ise = new IllegalStateException();
      ise.initCause(e);
      throw ise;
    }
    catch (ParseException e) {
      IllegalStateException ise = new IllegalStateException();
      ise.initCause(e);
      throw ise;
    }
  }

  private void setExpressionText(String expressionText) {
    if (expressionText == null) {
      throw new IllegalArgumentException(
        "O parmetro expressionText est nulo.\n");
    }
    this.expressionText = expressionText;
  }

  private final class ExpressionFunctionCaller implements FunctionCaller {
    public Variant getFunctionValue(String name, Vector args)
    throws FormulaEvaluationException {
      Class[] parameterClasses = new Class[args.size()];
      Object[] arguments = new Object[args.size()];
      for (int i = 0; i < args.size(); i++) {
        Variant argument = (Variant)args.get(i);
        if (argument.isBoolean()) {
          parameterClasses[i] = boolean.class;
          arguments[i] = new Boolean(argument.getBooleanValue());
        }
        else if (argument.isDouble()) {
          parameterClasses[i] = double.class;
          arguments[i] = new Double(argument.getDoubleValue());
        }
        else if (argument.isString()) {
          parameterClasses[i] = String.class;
          arguments[i] = argument.getStringValue();
        }
        else {
          throw new IllegalArgumentException(
            "O parmetro arguments possui pelo menos um elemento que no  "+
            "do tipo suportado (booleano, string ou double).\n");
        }
      }
      Object result;
      try {
        result = ExpressionFunctionExecutor.getInstance()
          .execute(name, getConfigurator(), arguments);
      }
      catch (ExpressionFunctionExecutionException e) {
        throw new FormulaEvaluationException(e);
      }
      catch (ExpressionFunctionNotFoundException e) {
        throw new FormulaEvaluationException(e);
      }
      return new Variant(result);
    }
  }

  private final class ExpressionSymbolEvaluator implements SymbolEvaluator {
    public void setValue(String name, Variant value) {
      throw new UnsupportedOperationException(
        "As expresses so somente-leitura, "+
        "logo esta operao no  suportada.\n");
    }

    public Variant getValue(String name)
    throws ParameterNotFoundException, ParameterValueNotExistsException, ParameterIsOffException {
      SimpleParameter parameter = getConfigurator().getSimpleParameter(name);
      if (parameter == null) {
        throw new ParameterNotFoundException(name);
      }
      if (!parameter.isEnabled() || !parameter.isVisible()) {
        throw new ParameterIsOffException(parameter.getName());
      }
      addParameter(parameter);
      Object value = parameter.getExpressionValue();
      if (value == null) {
        throw new ParameterValueNotExistsException(name);
      }
      return new Variant(value);
    }
  }
}