package csbase.logic.algorithms.parameters;

import csbase.exception.ConfigurationException;
import csbase.exception.algorithms.ExpressionFunctionExecutionException;
import csbase.exception.algorithms.ExpressionFunctionNotFoundException;

import tecgraf.javautils.configurationmanager.Configuration;
import tecgraf.javautils.configurationmanager.ConfigurationManager;
import tecgraf.javautils.configurationmanager.ConfigurationManagerException;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

public final class ExpressionFunctionExecutor {
  private static ExpressionFunctionExecutor instance;
  private Set<ExpressionFunction> expressionFunctions;

  private ExpressionFunctionExecutor() {
    this.expressionFunctions = new HashSet<ExpressionFunction>();
    this.createExpressionFunctions();
  }
  
  public static ExpressionFunctionExecutor getInstance() {
    if (instance == null) {
      instance = new ExpressionFunctionExecutor();
    }
    return instance;
  }

  /**
   * Cria e carrega instncias de ExpressionFunction a partir das configuraes
   * desta classe.
   * 
   * @see #createExpressionFunctions(Configuration)
   * @see ConfigurationManager#getConfiguration(Class)
   */
  public void createExpressionFunctions() {
    try {
      Configuration configuration =
        ConfigurationManager.getInstance().getConfiguration(getClass());
      createExpressionFunctions(configuration);
    }
    catch (ConfigurationManagerException e) {
      throw new ConfigurationException(e);
    }
  }   
  
  /**
   * Cria e carrega instncias de ExpressionFunction a partir de configuraes.
   * <br/>
   * As configuraes devem estar de acordo com o formato abaixo: 
   * <ul>
   * <li> Para criar instncias de ReflectionExpressionFunction, as propriedades 
   * devem estar de acordo com o formato:<br>
   * class.[0..n] = &lt;nome da classe cujos mtodos estticos, pblicos e no 
   * abstratos devem ser encapsulados por uma ReflectionExpressionFunction&gt;<br>
   * Ex.:<br>
   * class.0 = {@link Math}
   * </li>
   * <li> Para criar instncias de classes que estendem ExpressionFuncion, 
   * as propriedades devem estar no formato:<br>   
   * function.[0..n] = &lt;nome da classe que estende ExpressionFunction&gt;<br>
   * Ex.:<br>
   * function.0 = {@link CheckExpressionFunction}
   * </li>
   * </ul>
   * 
   * @param configuration configuraes descrevendo que ExpressionFunctions 
   * devem ser criadas.
   */
  public void createExpressionFunctions(Configuration configuration) {
    try {
      int ix = 0;
      Class<?> functionsClass =
        configuration.getOptionalClassProperty("class."+ix);
      while (functionsClass != null) {
        createReflexiveExpressionFunctions(functionsClass);
        ix++;
        functionsClass = configuration.getOptionalClassProperty("class."+ix);
      }
      ix = 0;
      Class<ExpressionFunction> functionClass =
        configuration.getOptionalClassProperty("function."+ix);
      while (functionClass != null) {
        Constructor<ExpressionFunction> constructor =
          functionClass.getConstructor(new Class[] {  });
        ExpressionFunction function =
          constructor.newInstance(new Object[] {  });
        addExpressionFunction(function);
        ix++;
        functionClass = configuration.getOptionalClassProperty("function."+ix);
      }
    }
    catch (ClassNotFoundException e) {
      throw new ConfigurationException(e);
    }
    catch (NoSuchMethodException e) {
      throw new ConfigurationException(e);
    }
    catch (InstantiationException e) {
      throw new ConfigurationException(e);
    }
    catch (IllegalAccessException e) {
      throw new ConfigurationException(e);
    }
    catch (InvocationTargetException e) {
      throw new ConfigurationException(e.getTargetException());
    }
  }    
  
  public void createReflexiveExpressionFunctions(Class clazz) {
    for (Method method : clazz.getMethods()) {
      if (
        Modifier.isPublic(method.getModifiers()) &&
        Modifier.isStatic(method.getModifiers()) &&
        !Modifier.isAbstract(method.getModifiers())) {
        addExpressionFunction(new ReflectionExpressionFunction(method));
      }
    }
  }

  public boolean addExpressionFunction(ExpressionFunction expressionFunction) {
    if (expressionFunction == null) {
      throw new IllegalArgumentException(
        "O parmetro expressionFunction est nulo.");
    }
    return this.expressionFunctions.add(expressionFunction);
  }
  
  public boolean addAllExpressionFunction(
    Set<ExpressionFunction> expressionFunctions) {

    if (expressionFunctions == null) {
      throw new IllegalArgumentException(
        "O parmetro expressionFunctions est nulo.");
    }
    
    return this.expressionFunctions.addAll(expressionFunctions);
  }  

  public Object execute(
    String functionName, SimpleAlgorithmConfigurator configurator,
    Object... parameters)
  throws ExpressionFunctionExecutionException,
    ExpressionFunctionNotFoundException {
    if (functionName == null) {
      throw new IllegalArgumentException(
        "O parmetro functionName no foi encontrado.");
    }
    if (configurator == null) {
      throw new IllegalArgumentException("O parmetro configurator est nulo.");
    }
    if (parameters == null) {
      throw new IllegalArgumentException(
        "O parmetro parameters no foi encontrado.");
    }
    for (ExpressionFunction expressionFunction : this.expressionFunctions) {
      if (
        expressionFunction.getName().equals(functionName) &&
        expressionFunction.acceptParameters(parameters)) {
        return expressionFunction.execute(configurator, parameters);
      }
    }
    Class[] parameterTypes = new Class[parameters.length];
    for (int i = 0; i < parameters.length; i++) {
      parameterTypes[i] = parameters[i].getClass();
    }
    throw new ExpressionFunctionNotFoundException(functionName, parameterTypes);
  }

  /**
   * Retorna uma viso imutvel da lista das possveis funes a serem executadas.
   * @return uma viso imutvel da lista das possveis funes a serem executadas.
   */
  public Set<ExpressionFunction> getExpressionFunctions() {
    return Collections.unmodifiableSet(this.expressionFunctions);
  }

  /**
   * Apaga todas as funes da lista. 
   */
  public void clear() {
    this.expressionFunctions.clear();
  }
}