package csbase.logic.algorithms.parameters;

import csbase.exception.ParseException;
import csbase.util.parser.exception.AutomatonException;

import java.text.MessageFormat;

import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * Parmetro do Tipo Lista.
 * 
 * @param <E> O tipo do elemento armazenado na lista.
 *
 * @author lmoreira
 */
public abstract class ListParameter<E> extends SimpleParameter<List<E>> {
  /**
   * Cria o parmetro.
   * 
   * @param name O nome (No aceita {@code null}).
   * @param label O rtulo (No aceita {@code null}).
   * @param description A descrio (No aceita {@code null}).
   * @param isOptional Indica se o parmetro  opcional.
   * @param commandLinePattern O padro para construo da linha de comando. O 
   *        padro ser utilizado para escrever o trecho da linha do comando 
   *        referente ao parmetro. Esta string ser passada para o mtodo
   *        MessageFormat.format(String,Object...). O primeiro formato ({0})
   *         referente ao nome e o segundo formato ({1})  referente
   *        ao valor. Se {@code null} o parmetro no produzir
   *        sada na linha de comando. 
   */
  protected ListParameter(String name, String label,
    String description, boolean isOptional, String commandLinePattern) {
    super(name, label, description, null, isOptional, commandLinePattern);
  }

  /**
   * <p>Adiciona um elemento ao valor deste parmetro.</p>
   * 
   * <p>Gera o evento 
   * {@link ParameterListener#valueWasChanged(Parameter)}.
   * </p>
   * 
   * @param element O elemento (No aceita {@code null}).
   */
  public final void addElement(E element) {
    if (element == null) {
      throw new IllegalArgumentException("O parmetro element est nulo.");
    }
    List<E> values = getValue();
    List<E> newValues;
    if (values == null) {
      newValues = new LinkedList<E>();
    }
    else {
      newValues = new LinkedList<E>(values);
    }
    newValues.add(element);
    setValue(Collections.unmodifiableList(newValues));
  }

  @Override
  public final Object getExpressionValue() {
    return getName();
  }
  
  /**
   * {@inheritDoc}
   */
  @Override
  public String getValueAsText() {
    List<E> values = getValue();
    if (values == null || values.isEmpty()) {
      return null;
    }
    StringBuffer buffer = new StringBuffer();
    buffer.append(ListParameterParser.START_LINE_CHAR);
    Iterator<E> valueIterator = values.iterator();
    String separator = "";
    while (valueIterator.hasNext()) {
      buffer.append(separator);
      buffer.append(getItemValueAsText(valueIterator.next()));
      separator = Character.toString(ListParameterParser.ELEMENT_SEPARATOR_CHAR);
    }
    buffer.append(ListParameterParser.END_LINE_CHAR);
    return buffer.toString();
  }

  /**
   * <p>Remove um elemento especfico.</p>
   * 
   * <p>Gera o evento 
   * {@link ParameterListener#valueWasChanged(Parameter)}.
   * </p>
   * 
   * @param index O ndice do elemento (A faixa aceita  de 0 a 
   *        quantidade de elementos - 1).
   */
  public final void removeElement(int index) {
    List<E> values = getValue();
    if (values == null) {
      throw new UnsupportedOperationException(
        "A lista de elementos est vazia.");
    }
    if (index < 0 || index >= values.size()) {
      String message =
        MessageFormat.format(
          "O ndice {0} est fora da faixa suportada.\nTamanho atual: {1}.",
          new Object[] { new Integer(index), new Integer(values.size()) });
      throw new IllegalArgumentException(message);
    }
    if (values.size() == 1) {
      setValue(null);
    }
    else {
      List<E> newValues = new LinkedList<E>(values);
      newValues.remove(index);
      setValue(newValues);
    }
  }

  /**
   * <p>Troca um elemento por outro.</p>
   * 
   * <p>Gera o evento 
   * {@link ParameterListener#valueWasChanged(Parameter)}.
   * </p>
   * 
   * @param index O ndice do elemento (A faixa aceita  de 0 a 
   *        quantidade de elementos - 1).
   * @param newElement O novo elemento (Aceita {@code null}).
   * 
   * 
   * @return {@code true} em caso de sucesso ou {@code false} se o elemento que
   *         estava armazenado na posio informada  igual ao elemento 
   *         fornecido.
   */
  public final boolean setElement(int index, E newElement) {
    List<E> values = getValue();
    if (values == null) {
      throw new UnsupportedOperationException(
        "A lista de elementos est vazia.");
    }
    if (index < 0 || index >= values.size()) {
      String message =
        MessageFormat.format(
          "O ndice {0} est fora da faixa suportada.\nTamanho atual: {1}.",
          new Object[] { new Integer(index), new Integer(values.size()) });
      throw new IllegalArgumentException(message);
    }
    Object oldElement = values.get(index);
    if (oldElement.equals(newElement)) {
      return false;
    }
    List<E> newValues = new LinkedList<E>(values);
    newValues.set(index, newElement);
    setValue(Collections.unmodifiableList(newValues));
    return true;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void setValueAsText(String parameterValue)
  throws ParseException {
    if (parameterValue != null) {
      ListParameterParser parser = new ListParameterParser();
      List<E> values = new LinkedList<E>();
      List<String> textValues;
      try {
        textValues = parser.parseText(parameterValue);
      }
      catch (AutomatonException e) {
        throw new ParseException(
          e,
          "No  possivel atribuir o valor\n({0}),"+
          "j que ele no representa uma codificao vlida para listas.\n"+
          "Parmetro envolvido: {1}.", new Object[] { parameterValue, this });
      }
      Iterator<String> textValueIterator = textValues.iterator();
      while (textValueIterator.hasNext()) {
        String textValue = textValueIterator.next();
        values.add(getItemValueFromText(textValue));
      }
      setValue(values);
    }
    else {
      setValue(null);
    }
  }

  //TODO Renomear este mtodo
  /**
   * Decodifica um texto criando um elemento.
   * 
   * @param elementAsText O valor do elemento codificado (Aceita {@code null}).
   * 
   * @return O valor do elemento.
   * 
   * @throws ParseException Se o valor codificado no representar um elemento
   *         que possa ser armazenado neste parmetro.
   */
  protected abstract E getItemValueFromText(String elementAsText)
  throws ParseException;

  //TODO Renomear este mtodo
  /**
   * Codifica um elemento criando um texto.
   * 
   * @param element O elemento a ser codificado (Aceita {@code null}).
   * 
   * @return O texto codificado 
   * ({@code null} se o valor do elemento for {@code null}).
   */
  protected String getItemValueAsText(E element) {
    if (element == null) {
      return null;
    }
    return element.toString();
  }

  @Override
  protected final String getCommandValue(String projectDirectory, String fileSeparator) {
    List<E> values = getValue();
    if (values == null) {
      return null;
    }
    StringBuffer buffer = new StringBuffer();
    String separator = "";
    Iterator<E> valueIterator = values.iterator();
    while (valueIterator.hasNext()) {
      E value = valueIterator.next();
      buffer.append(separator);
      buffer.append(getCommandItemValue(value));
      separator = ",";
    }
    return buffer.toString();
  }

  /**
   * Obtm o valor de um tem da lista pronto para ser utilizado na linha de 
   * comando.
   * 
   * @param itemValue O valor (No aceita {@code null}).
   * 
   * @return O valor convertido.
   */
  protected String getCommandItemValue(E itemValue) {
    return itemValue.toString();
  }
}
