/**
 * $Id$
 */
package csbase.logic.algorithms.parameters;

import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * Grupo de parmetros.
 * 
 * @author lmoreira
 */
public final class ParameterGroup extends Parameter<Object> {
  /**
   * Os grupos filhos deste grupo.
   */
  private List<ParameterGroup> groups;
  /**
   * Os parmetros do tipo lista de arquivos de entrada que so filhos deste
   * grupo.
   */
  private List<InputFileListParameter> inputFileListParameters;
  /**
   * Os parmetros do tipo arquivos de entrada que so filhos deste grupo.
   */
  private List<InputFileParameter> inputParameters;
  /**
   * Os parmetros do arquivos de sada que so filhos deste grupo.
   */
  private List<OutputFileParameter> outputParameters;
  /**
   * Os parmetros filhos deste grupo.
   */
  private List<Parameter<?>> parameters;
  /**
   * Carregador de parmetros.
   */
  private ParameterLoader parameterLoader;
  /**
   * Os parmetros simples filhos deste grupo indexados pelo nome.
   */
  private Map<String, SimpleParameter<?>> simpleParametersByName;

  /**
   * Cria um grupo.
   * 
   * @param name O nome do grupo (No aceita {@code null}).
   */
  public ParameterGroup(String name) {
    super(name);
    this.groups = new LinkedList<ParameterGroup>();
    this.inputParameters = new LinkedList<InputFileParameter>();
    this.inputFileListParameters = new LinkedList<InputFileListParameter>();
    this.outputParameters = new LinkedList<OutputFileParameter>();
    this.parameters = new LinkedList<Parameter<?>>();
    this.simpleParametersByName = new HashMap<String, SimpleParameter<?>>();
    this.parameterLoader = null;
  }

  /**
   * Adiciona um parmetro filho.
   * 
   * @param parameter O parmetro (No aceita {@code null}).
   * 
   * @return {@code true} em caso de sucesso ou {@code false} se j houver um
   *         filho com o mesmo nome do parmetro informado.
   */
  public boolean addParameter(Parameter<?> parameter) {
    if (parameter == null) {
      throw new IllegalArgumentException("O parmetro parameter est nulo.");
    }
    if (this.parameters.contains(parameter)) {
      return false;
    }
    if (parameter instanceof SimpleParameter<?>) {
      if (parameter instanceof InputFileParameter) {
        this.inputParameters.add((InputFileParameter) parameter);
      }
      else if (parameter instanceof OutputFileParameter) {
        this.outputParameters.add((OutputFileParameter) parameter);
      }
      else if (parameter instanceof InputFileListParameter) {
        this.inputFileListParameters.add((InputFileListParameter) parameter);
      }
      SimpleParameter<?> simpleParameter = (SimpleParameter<?>) parameter;
      this.simpleParametersByName.put(simpleParameter.getName(),
        simpleParameter);
    }
    else if (parameter instanceof ParameterGroup) {
      ParameterGroup group = (ParameterGroup) parameter;
      this.groups.add(group);
    }
    this.parameters.add(parameter);
    return true;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Map<String, Object> exportValue() {
    Map<String, Object> parameterValues = new HashMap<String, Object>();
    for (Parameter<?> parameter : this.parameters) {
      Map<String, Object> values = parameter.exportValue();
      for (String key : values.keySet()) {
        parameterValues.put(key, values.get(key));
      }
    }
    return Collections.unmodifiableMap(parameterValues);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public String getCommandLine(String projectPath, String fileSeparator) {
    String commandLine = "";
    String separator = "";
    for (Parameter<?> parameter : this.parameters) {
      String parameterCommandLine =
        parameter.getCommandLine(projectPath, fileSeparator);
      if (parameterCommandLine != null) {
        commandLine += separator;
        commandLine += parameter.getCommandLine(projectPath, fileSeparator);
        separator = " ";
      }
    }
    if (commandLine.length() == 0) {
      return null;
    }
    return commandLine;
  }

  /**
   * Obtm os grupos-filhos deste grupo.
   * <p>
   * O conjunto retornado  imutvel (veja
   * {@link Collections#unmodifiableSet(Set)}).
   * </p>
   * 
   * @return Os grupos-filhos (Se no houver grupos-filhos, o conjunto estar
   *         vazio).
   */
  public Set<ParameterGroup> getGroups() {
    Set<ParameterGroup> groups = new HashSet<ParameterGroup>();
    groups.addAll(this.groups);
    for (ParameterGroup group : groups) {
      groups.addAll(group.getGroups());
    }
    return groups;
  }

  /**
   * Obtm os parmetros do tipo lista de arquivo de entrada que so filhos
   * deste grupo.
   * <p>
   * A lista retornada  imutvel (veja
   * {@link Collections#unmodifiableList(List)}).
   * </p>
   * 
   * @return Os parmetros-filhos (Se no houver grupos-filhos, a lista estar
   *         vazia).
   */
  public List<InputFileListParameter> getInputFileListParameters() {
    List<InputFileListParameter> parameters =
      new LinkedList<InputFileListParameter>();
    parameters.addAll(this.inputFileListParameters);
    for (ParameterGroup group : getGroups()) {
      parameters.addAll(group.getInputFileListParameters());
    }
    return Collections.unmodifiableList(parameters);
  }

  /**
   * Obtm os parmetros do tipo arquivo de entrada que so filhos deste grupo.
   * <p>
   * A lista retornada  imutvel (veja
   * {@link Collections#unmodifiableList(List)}).
   * </p>
   * 
   * @return Os parmetros-filhos (Se no houver grupos-filhos, a lista estar
   *         vazia).
   */
  public List<InputFileParameter> getInputFileParameters() {
    List<InputFileParameter> inputFileParameters =
      new LinkedList<InputFileParameter>();
    inputFileParameters.addAll(this.inputParameters);
    for (ParameterGroup group : getGroups()) {
      inputFileParameters.addAll(group.getInputFileParameters());
    }
    return Collections.unmodifiableList(inputFileParameters);
  }

  /**
   * Obtm os parmetros do tipo arquivo de sada que so filhos deste grupo.
   * <p>
   * A lista retornada  imutvel (veja
   * {@link Collections#unmodifiableList(List)}).
   * </p>
   * 
   * @return Os parmetros-filhos (Se no houver grupos-filhos, a lista estar
   *         vazia).
   */
  public List<OutputFileParameter> getOutputFileParameters() {
    List<OutputFileParameter> outputFileParameters =
      new LinkedList<OutputFileParameter>();
    outputFileParameters.addAll(this.outputParameters);
    for (ParameterGroup group : getGroups()) {
      outputFileParameters.addAll(group.getOutputFileParameters());
    }
    return Collections.unmodifiableList(outputFileParameters);
  }

  /**
   * Obtm o carregador de parmtros.
   * 
   * @return o carregador de parmtros, ou {@code null} caso no exista.
   */
  public ParameterLoader getParameterLoader() {
    return parameterLoader;
  }

  /**
   * Obtm os parmetros que so filhos deste grupo.
   * <p>
   * A lista retornada  imutvel (veja
   * {@link Collections#unmodifiableList(List)}).
   * </p>
   * 
   * @return Os parmetros-filhos (Se no houver grupos-filhos, a lista estar
   *         vazia).
   */
  public List<Parameter<?>> getParameters() {
    return Collections.unmodifiableList(this.parameters);
  }

  /**
   * Obtm um parmetro simples que  filho deste grupo atravs de seu nome.
   * 
   * @param parameterName O nome do parmetro (No aceita {@code null}).
   * 
   * @return O parmetro-filho ou {@code null} se ele no existir.
   */
  public SimpleParameter<?> getSimpleParameter(String parameterName) {
    SimpleParameter<?> parameter =
      this.simpleParametersByName.get(parameterName);
    if (parameter == null) {
      for (ParameterGroup group : getGroups()) {
        parameter = group.getSimpleParameter(parameterName);
      }
    }
    return parameter;
  }

  /**
   * Obtm os parmetros simples que so filhos deste grupo.
   * <p>
   * O conjunto retornado  imutvel (veja
   * {@link Collections#unmodifiableSet(Set)}).
   * </p>
   * 
   * @return Os parmetros-filhos (Se no houver grupos-filhos, a lista estar
   *         vazia).
   */
  public Set<SimpleParameter<?>> getSimpleParameters() {
    Set<SimpleParameter<?>> simpleParameters =
      new HashSet<SimpleParameter<?>>();
    simpleParameters.addAll(this.simpleParametersByName.values());
    for (ParameterGroup group : getGroups()) {
      simpleParameters.addAll(group.getSimpleParameters());
    }
    return Collections.unmodifiableSet(simpleParameters);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void importValue(Map parameterValue) {
    if (parameterValue == null) {
      throw new IllegalArgumentException(
        "O parmetro parameterValues est nulo.");
    }
    for (Parameter<?> parameter : this.parameters) {
      parameter.importValue(parameterValue);
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean isSetDefaultValue() {
    for (Parameter<?> parameter : getParameters()) {
      if (!parameter.isSetDefaultValue()) {
        return false;
      }
    }
    return true;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean isVisible() {
    for (Parameter<?> parameter : this.parameters) {
      if (parameter.isVisible()) {
        return true;
      }
    }
    return false;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void resetValue() {
    for (Parameter<?> parameter : this.parameters) {
      parameter.resetValue();
    }
  }

  /**
   * Atualiza o carregador de parmtros.
   * 
   * @param parameterLoader carregador de parmtros. No aceita {@code null}.
   * @return {@code true} se no havia sido definido anteriormente nenhum
   *         carregador de parmetros.
   */
  public boolean setParameterLoader(ParameterLoader parameterLoader) {
    if (this.parameterLoader != null) {
      return false;
    }
    if (parameterLoader == null) {
      throw new IllegalArgumentException(
        "O parmetro parameterLoader est nulo.");
    }
    this.parameterLoader = parameterLoader;
    return true;
  }
}
