package csbase.logic.algorithms.parameters;

import csbase.util.parser.FiniteAutomaton;
import csbase.util.parser.Parser;
import csbase.util.parser.State;
import csbase.util.parser.Token;
import csbase.util.parser.actions.AppendAction;
import csbase.util.parser.actions.CompositeTransitionAction;
import csbase.util.parser.actions.DiscardAction;
import csbase.util.parser.actions.GenerateTokenAction;
import csbase.util.parser.exception.AutomatonException;
import csbase.util.parser.iterators.CharSymbolIterator;
import csbase.util.parser.symbols.CharSymbol;
import csbase.util.parser.symbols.Symbol;

import java.io.IOException;
import java.io.Reader;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
 
final class TableParameterParser extends Parser {
  static final char START_CHARACTER = '{';
  static final char END_CHARACTER = '}';
  static final char SEPARATOR_CHARACTER = ',';
  private RowParameterParser elementGeneratorParser;

  TableParameterParser() {
    super(new TableParameterFiniteAutomaton());
    this.elementGeneratorParser = new RowParameterParser();
  }

  List<List<String>> parseText(Reader reader)
  throws AutomatonException, IOException {
    StringBuilder buffer = new StringBuilder();
    int current;
    while((current = reader.read()) != -1) {
        buffer.append((char)current);
    }
    String text = buffer.toString();
    return parseText(text);
  }

  List<List<String>> parseText(String text)
  throws AutomatonException {
    List<List<String>> rows = new LinkedList<List<String>>();
    List<Token> tokens = parse(new CharSymbolIterator(text));
    for (Token token : tokens) {
      StringBuffer buffer = new StringBuffer();
      List<CharSymbol> symbols = token.getSymbolList();
      for (CharSymbol symbol : symbols) {
        buffer.append(symbol.getObject());
      }
      List<String> row =
        this.elementGeneratorParser.parseText(buffer.toString());
      rows.add(row);
    }
    return rows;
  }

  private static final class RowParameterParser extends Parser {
    RowParameterParser() {
      super(new RowFiniteAutomaton());
    }
    
    List<String> parseText(String text)
    throws AutomatonException {
      List<Token> tokens = parse(new CharSymbolIterator(text));
      List<String> row = new ArrayList<String>(tokens.size());
      for (Token token : tokens) {
        StringBuffer buffer = new StringBuffer();
        List<Symbol> symbols = token.getSymbolList();
        for (Symbol symbol : symbols) {
          Character object = (Character)symbol.getObject();
          buffer.append(object);
        }
        String cellValue;
        if (buffer.length() != 0) {
          cellValue = buffer.toString();
        }
        else {
          cellValue = null;
        }
        row.add(cellValue);
      }
      return row;
    }

    private static final class RowFiniteAutomaton extends FiniteAutomaton {
      private static final Symbol START_SYMBOL =
        new CharSymbol(START_CHARACTER);
      private static final Symbol SEPARATOR_SYMBOL =
        new CharSymbol(SEPARATOR_CHARACTER);
      private static final Symbol END_SYMBOL =
        new CharSymbol(END_CHARACTER);
      
      RowFiniteAutomaton() {
        super(new State(false));
        State startLineState = getInitialState();
        State elementState = new State(false);
        State endLineState = new State(true);
        startLineState.addTransition(
          START_SYMBOL, DiscardAction.getInstance(), elementState);
        elementState.addTransition(
          SEPARATOR_SYMBOL, GenerateTokenAction.getInstance(),
          elementState);
        elementState.addTransition(
          END_SYMBOL, new CompositeTransitionAction(
          GenerateTokenAction.getInstance(), DiscardAction.getInstance()),
          endLineState);
        elementState.setDefaultTransition(
          AppendAction.getInstance(), elementState);
      }
    }
  }
  
  private static final class TableParameterFiniteAutomaton extends FiniteAutomaton {
    private static final Symbol START_SYMBOL =
      new CharSymbol(START_CHARACTER);
    private static final Symbol SEPARATOR_SYMBOL =
      new CharSymbol(SEPARATOR_CHARACTER);
    private static final Symbol END_SYMBOL =
      new CharSymbol(END_CHARACTER);

    private TableParameterFiniteAutomaton() {
      super(new State(false));
      State startTable = getInitialState();
      State startLine = new State(false);
      State buildingElement = new State(false);
      State endLine = new State(false);
      State endTable = new State(true);
      startTable.addTransition(
        START_SYMBOL, DiscardAction.getInstance(), startLine);
      startLine.addTransition(
        START_SYMBOL, AppendAction.getInstance(), buildingElement);
      startLine.addTransition(
        END_SYMBOL, DiscardAction.getInstance(), endTable);
      buildingElement.setDefaultTransition(
        AppendAction.getInstance(), buildingElement);
      CompositeTransitionAction compositeAction =
        new CompositeTransitionAction();
      compositeAction.addAction(AppendAction.getInstance());
      compositeAction.addAction(GenerateTokenAction.getInstance());
      buildingElement.addTransition(END_SYMBOL, compositeAction, endLine);
      endLine.addTransition(
        SEPARATOR_SYMBOL, DiscardAction.getInstance(), startLine);
      endLine.addTransition(END_SYMBOL, DiscardAction.getInstance(), endTable);
    }
  }
}
