package csbase.logic.algorithms.parsers;

import csbase.exception.BugException;
import csbase.exception.ParseException;
import csbase.logic.algorithms.parameters.FileParameterMode;
import csbase.logic.algorithms.parameters.FileParameterPipeAcceptance;
import csbase.logic.algorithms.parameters.InputFileListParameter;
import csbase.logic.algorithms.parameters.InputFileParameter;
import csbase.logic.algorithms.parameters.OutputFileParameter;
import csbase.logic.algorithms.parameters.SimpleParameter;

/**
 * Classe abstrata que serve de base para analisadores de parmetros <T> com
 * valores que representam arquivos como {@link InputFileParameter} e
 * {@link OutputFileParameter}.
 * 
 * @param <T> Tipo do parmetro do qual essa classe faz parsing
 */
public abstract class AbstractFileParameterParser<T extends SimpleParameter<?>>
  extends SimpleParameterParser<T> {

  
  /**
   * <p>
   * O atributo {@value #FILE_PARAMETER_ELEMENT_MUST_SORT_ATTRIBUTE} do
   * elemento {@link InputFileListParameter}.
   * </p>
   * 
   * Indica se o {@link InputFileParameter arquivo de entrada} se a relao de
   * arquivos ser ordenada,  opcional, o seu valor-padro 
   * {@link #FILE_PARAMETER_ELEMENT_MUST_SORT_DEFAULT_VALUE} e  do tipo
   * booleano. </p>
   */
  protected static final String FILE_PARAMETER_ELEMENT_MUST_SORT_ATTRIBUTE =
    "ordenar";

  /**
   * <p>
   * O valor-padro para o atributo
   * {@link #FILE_PARAMETER_ELEMENT_MUST_SORT_ATTRIBUTE} do elemento
   * {@link InputFileListParameter}
   * </p>
   * <p>
   * O seu valor 
   * {@value #FILE_PARAMETER_ELEMENT_MUST_SORT_DEFAULT_VALUE}.
   * </p>
   */
  protected static final boolean FILE_PARAMETER_ELEMENT_MUST_SORT_DEFAULT_VALUE =
    true;


  /**
   * <p>
   * O atributo {@value #FILE_PARAMETER_ELEMENT_CAN_USE_PIPE_ATTRIBUTE} dos
   * elementos:
   * <ul>
   * <li>{@link InputFileParameter}</li>
   * <li>{@link OutputFileParameter}</li>
   * </ul>
   * </p>
   * <p>
   * Indica o se o parmetro do tipo arquivo aceita trabalhar com pipes, 
   * opcional, o seu valor-padro 
   * {@link #FILE_PARAMETER_ELEMENT_CAN_USE_PIPE_DEFAULT_VALUE} e  do tipo
   * string e os valores vlidos para ele so:
   * <ul>
   * <li>{@link FileParameterPipeAcceptance#TRUE}</li>
   * <li>{@link FileParameterPipeAcceptance#FALSE}</li>
   * <li>
   * {@link FileParameterPipeAcceptance#ALWAYS}</li>
   * </ul>
   * </p>
   */
  protected static final String FILE_PARAMETER_ELEMENT_CAN_USE_PIPE_ATTRIBUTE =
    "permitir_pipe";

  /**
   * <p>
   * O valor-padro para o atributo
   * {@link #FILE_PARAMETER_ELEMENT_CAN_USE_PIPE_ATTRIBUTE} dos elementos:
   * <ul>
   * <li>{@link InputFileParameter}</li>
   * <li>{@link OutputFileParameter}</li>
   * </ul>
   * </p>
   * <p>
   * O seu valor  {@value #FILE_PARAMETER_ELEMENT_CAN_USE_PIPE_DEFAULT_VALUE}.
   * </p>
   */
  protected static final FileParameterPipeAcceptance FILE_PARAMETER_ELEMENT_CAN_USE_PIPE_DEFAULT_VALUE =
    FileParameterPipeAcceptance.TRUE;

  /**
   * <p>
   * O atributo {@value #FILE_PARAMETER_ELEMENT_CATEGORY_ATTRIBUTE} dos
   * elementos:
   * <ul>
   * <li>{@link InputFileParameter}</li>
   * <li>{@link OutputFileParameter}</li>
   * </ul>
   * </p>
   * <p>
   * Indica o {@link FileParameterMode modo de funcionamento} do parmetros do
   * tipo arquivo},  opcional, o seu valor-padro 
   * {@link #FILE_PARAMETER_ELEMENT_CATEGORY_VALUE_REGULAR_FILE},  do tipo
   * string e o valores vlidos para ele so:
   * <ul>
   * <li>{@link #FILE_PARAMETER_ELEMENT_CATEGORY_VALUE_REGULAR_FILE}</li>
   * <li>{@link #FILE_PARAMETER_ELEMENT_CATEGORY_VALUE_DIRECTORY}</li>
   * <li>
   * {@link #FILE_PARAMETER_ELEMENT_CATEGORY_VALUE_DIRECTORY_AND_REGULAR_FILE}</li>
   * </ul>
   * .
   * </p>
   */
  protected static final String FILE_PARAMETER_ELEMENT_CATEGORY_ATTRIBUTE =
    "categoria";

  /**
   * <p>
   * O valor que indica o {@link FileParameterMode#DIRECTORY modo apenas
   * diretrio} para o atributo
   * {@link #FILE_PARAMETER_ELEMENT_CATEGORY_ATTRIBUTE} dos elementos:
   * <ul>
   * <li>{@link InputFileParameter}</li>
   * <li>{@link OutputFileParameter}</li>
   * </ul>
   * </p>
   * <p>
   * O seu valor  {@value #FILE_PARAMETER_ELEMENT_CATEGORY_VALUE_DIRECTORY}.
   * </p>
   */
  protected static final String FILE_PARAMETER_ELEMENT_CATEGORY_VALUE_DIRECTORY =
    "diretorio";

  /**
   * <p>
   * O valor que indica o {@link FileParameterMode#DIRECTORY_AND_REGULAR_FILE
   * modo diretrio e/ou arquivos} para o atributo
   * {@link #FILE_PARAMETER_ELEMENT_CATEGORY_ATTRIBUTE} dos elementos:
   * <ul>
   * <li>{@link InputFileParameter}</li>
   * <li>{@link OutputFileParameter}</li>
   * </ul>
   * </p>
   * <p>
   * O seu valor 
   * {@value #FILE_PARAMETER_ELEMENT_CATEGORY_VALUE_DIRECTORY_AND_REGULAR_FILE}.
   * </p>
   */
  protected static final String FILE_PARAMETER_ELEMENT_CATEGORY_VALUE_DIRECTORY_AND_REGULAR_FILE =
    "ambos";

  /**
   * <p>
   * O valor que indica o {@link FileParameterMode#REGULAR_FILE modo apenas
   * arquivos} para o atributo
   * {@link #FILE_PARAMETER_ELEMENT_CATEGORY_ATTRIBUTE} dos elementos:
   * <ul>
   * <li>{@link InputFileParameter}</li>
   * <li>{@link OutputFileParameter}</li>
   * </ul>
   * </p>
   * <p>
   * O seu valor  {@value #FILE_PARAMETER_ELEMENT_CATEGORY_VALUE_REGULAR_FILE}.
   * </p>
   */
  protected static final String FILE_PARAMETER_ELEMENT_CATEGORY_VALUE_REGULAR_FILE =
    "arquivo";

  /**
   * <p>
   * O atributo {@value #FILE_PARAMETER_ELEMENT_TYPE_ATTRIBUTE} dos elementos:
   * <ul>
   * <li>{@link InputFileParameter}</li>
   * <li>{@link OutputFileParameter}</li>
   * </ul>
   * </p>
   * 
   * Indica tipo de arquivo aceito pelo parmetro do tipo arquivo,  opcional e
   * o seu tipo  string. </p>
   */
  protected static final String FILE_PARAMETER_ELEMENT_TYPE_ATTRIBUTE = "tipo";

  /**
   * {@inheritDoc}
   */
  @Override
  public T createSimpleParameter(XmlParser parser, String name, String label,
    String description, boolean isOptional, String commandLinePattern)
    throws ParseException {
    String type =
      parser.extractAttributeValue(FILE_PARAMETER_ELEMENT_TYPE_ATTRIBUTE, null);
    return createFileParameter(parser, name, label, description, isOptional,
      commandLinePattern, type);
  }

  /**
   * Cria uma instncia do parmetro de tipo <T>, a partir dos atributos bsicos
   * do parmetro. As subclasses devem implementar esse mtodo, fazendo a
   * extrao dos demais atributos do parmetro.
   * 
   * @param parser Parser xml do configurador.
   * @param name Nome do parmetro.
   * @param label Rtulo do parmetro.
   * @param description Dica do parmetro.
   * @param isOptional Indica se o parmetro  opcional.
   * @param commandLinePattern Padro da linha de comando do parmetro.
   * @param type Tipo do arquivo do parmetro.
   * @return Uma instncia do parmetro com os atributos bsicos e especficos
   *         preenchidos.
   * @throws ParseException Caso no seja possvel criar a instncia do
   *         parmetro com os atributos especificados.
   */
  public abstract T createFileParameter(XmlParser parser, String name,
    String label, String description, boolean isOptional,
    String commandLinePattern, String type) throws ParseException;

  /**
   * Converte strings para valores da enumerao FileParameterPipeAcceptance.
   */
  protected class FileParameterPipeAcceptanceConverter implements
    StringToEnumConverter<FileParameterPipeAcceptance> {

    /**
     * Retorna as strings que correspondem a cada um dos possveis valores da
     * enumerao {@link FileParameterPipeAcceptance}.
     * 
     * @param value O valor da enumerao.
     * @return Um array com todas as strings que podem representar o valor.
     */
    public String[] getPossibleMatches(FileParameterPipeAcceptance value) {
      switch (value) {
        case TRUE:
          return XmlParser.TRUE_VALUES;
        case FALSE:
          return XmlParser.FALSE_VALUES;
        case ALWAYS:
          return new String[] { "sempre", "always" };
        default:
          throw new BugException();
      }
    }

    /**
     * {@inheritDoc}
     * 
     */
    @Override
    public FileParameterPipeAcceptance valueOf(String string) {
      String upperCaseString = string.toUpperCase();
      for (FileParameterPipeAcceptance mode : FileParameterPipeAcceptance
        .values()) {
        String[] possibleMatches = getPossibleMatches(mode);
        for (String possibleMatch : possibleMatches) {
          if (possibleMatch.toUpperCase().equals(upperCaseString)) {
            return mode;
          }
        }
      }
      return null;
    }

  }

}
