package csbase.logic.algorithms.serializer;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import csbase.exception.ConfigurationException;
import csbase.exception.OperationFailureException;
import csbase.exception.ParseException;
import csbase.exception.algorithms.AlgorithmNotFoundException;
import csbase.logic.algorithms.AlgorithmConfigurator;
import csbase.logic.algorithms.flows.Flow;
import csbase.logic.algorithms.flows.FlowAlgorithmParser;
import csbase.logic.algorithms.flows.configurator.FlowAlgorithmConfigurator;
import csbase.logic.algorithms.serializer.exception.AlgorithmConfigurationSerializerException;
import csbase.logic.algorithms.serializer.exception.AlgorithmConfigurationSerializerIOException;
import csbase.logic.algorithms.serializer.exception.AlgorithmConfigurationSerializerParseException;

/**
 * Objeto utilizado para (des)serializar as configuraces de um configurador de
 * fluxo de algortmos.
 * 
 * @author Tecgraf / PUC-Rio
 */
public final class FlowAlgorithmConfigurationSerializer implements
  IAlgorithmConfigurationSerializer {

  /**
   * @param input fluxo de dados com a configuraco de um
   *        FlowAlgorithmConfigurator serializada (No aceita {@code null}).
   * 
   * @return Um configurador de fluxo de algortmos com seus parmetros
   *         preenchidos.
   * 
   * @throws AlgorithmConfigurationSerializerParseException Quando os dados do
   *         fluxo no estiverem formatados corretamente.
   * @throws AlgorithmConfigurationSerializerIOException Quando ocorrer um erro
   *         de IO no fluxo.
   * @throws AlgorithmNotFoundException Quando a verso do algoritmo utilizado
   *         para criar este configurador no existe mais.
   * @throws AlgorithmConfigurationSerializerException Demais erros que possam
   *         ocorrer na implementaco deste mtodo.
   */
  public AlgorithmConfigurator read(InputStream input)
    throws AlgorithmConfigurationSerializerParseException,
    AlgorithmConfigurationSerializerIOException,
    AlgorithmConfigurationSerializerException,
    AlgorithmNotFoundException {

    try {
      Flow flow = new FlowAlgorithmParser().read(input);
      return new FlowAlgorithmConfigurator(flow);
    }
    catch (OperationFailureException e) {
      throw new AlgorithmConfigurationSerializerException(e);
    }
    catch (ConfigurationException e) {
      throw new AlgorithmConfigurationSerializerException(e);
    }
    catch (ParseException e) {
      throw new AlgorithmConfigurationSerializerParseException(e);
    }
    catch (IOException e) {
      throw new AlgorithmConfigurationSerializerIOException(e);
    }
  }

  /**
   * @param configurator Um configurador de um fluxo de algortmos (No aceita
   *        {@code null}).
   * @param output Fluxo de dados para o qual ser serializada a configuraco do
   *        configurador de algortmos (No aceita {@code null}).
   * 
   * @throws AlgorithmConfigurationSerializerIOException Quando ocorrer um erro
   *         de IO no fluxo.
   * @throws AlgorithmConfigurationSerializerException
   */
  public void write(AlgorithmConfigurator configurator, OutputStream output)
    throws AlgorithmConfigurationSerializerIOException,
    AlgorithmConfigurationSerializerException {

    if (!(configurator instanceof FlowAlgorithmConfigurator)) {
      throw new IllegalArgumentException(
        "O configurador deve ser do tipo FlowAlgorithmConfigurator");
    }

    FlowAlgorithmConfigurator flowConfigurator =
      (FlowAlgorithmConfigurator) configurator;
    try {
      new FlowAlgorithmParser().write(output, flowConfigurator.getFlow());
    }
    catch (Exception e) {
      throw new AlgorithmConfigurationSerializerException(e);
    }
  }
}
