package csbase.logic.algorithms.xml.category;

import java.io.IOException;
import java.io.Writer;
import java.util.List;
import java.util.Set;
import java.util.SortedSet;

import tecgraf.javautils.xml.XMLBasicElement;
import tecgraf.javautils.xml.XMLElementInterface;
import tecgraf.javautils.xml.XMLEmptyElement;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.Category;

/**
 * Esta classe implementa o elemento XML que representa uma categoria de
 * algoritmos.
 * 
 */
public class XMLCategoryElement extends XMLBasicElement {

  /**
   * Constri o elemento xml que descreve uma categoria.
   */
  public XMLCategoryElement() {
    super();
    setTag(XmlCategoriesUtils.CATEGORY_TAG);
  }

  /**
   * Construtor utilizado no processo de escrita do arquivo xml de categorias. A
   * categoria da aplicao especificada deve ser usada no contexto da
   * aplicao.
   * 
   * @param category categoria de algoritmos a ser escrita
   */
  public XMLCategoryElement(Category category) {
    this();
    setAppContextObject(category);
  }

  /**
   * Obtm o objeto categoria que representa o contexto da aplicao.
   * 
   * @return a categoria de algoritmos
   */
  public Category getCategory() {
    return (Category) getAppContextObject();
  }

  /**
   * Executado quando a tag de fim do elemento XML  encontrada. Deve-se criar o
   * objeto correspondente na camada da aplicao.
   * 
   * @see tecgraf.javautils.xml.XMLElementInterface#endTag(java.util.List)
   */
  public void endTag(List<XMLElementInterface> childrenList) {
  }

  /**
   * Escreve todos os elementos que descrevem os algoritmos pertencentes 
   * categoria.
   * 
   * @param writer stream de sada
   * @param ident identao corrente
   * @throws IOException errro de I/o
   */
  private void writeAllAlgorithmElement(final Writer writer, final String ident)
    throws IOException {
    String newIdent = XMLBasicElement.getNextIdentation(ident);

    Set<AlgorithmInfo> algorithms = getCategory().getAlgorithms();
    if (!algorithms.isEmpty()) {
      for (AlgorithmInfo algorithmInfo : algorithms) {
        writeAlgorithmElement(writer, newIdent, algorithmInfo);
      }
    }
  }

  /**
   * Escreve todos os elementos que descrevem as sub-categorias na categoria do
   * contexto da aplicao.
   * 
   * @param writer stream de sada
   * @param ident identao corrente
   * @throws IOException erro de I/O
   */
  private void writeAllSubCategoryElement(Writer writer, String ident)
    throws IOException {
    String newIdent = XMLBasicElement.getNextIdentation(ident);

    SortedSet<Category> subCategories = getCategory().getCategories();
    if (!subCategories.isEmpty()) {
      for (Category category : subCategories) {
        writeSubCategoryElement(writer, newIdent, category);
      }
    }
  }

  /**
   * Escreve o elemento que descreve uma sub-categoria na categoria do contexto
   * da aplicao.
   * 
   * @param writer stream de sada
   * @param ident identao corrente
   * @param subCategory sub-categoria a ser escrita na categoria
   * @throws IOException erro de I/O
   */
  private void writeSubCategoryElement(final Writer writer, final String ident,
    Category subCategory) throws IOException {
    XMLCategoryElement xmlcategory = new XMLCategoryElement(subCategory);
    xmlcategory.write(writer, ident);
  }

  /**
   * Escreve o elemento que descreve um algoritmo na categoria.
   * 
   * @param writer stream de sada
   * @param ident identao corrente
   * @param algorithm algoritmo que pertence  categoria
   * @throws IOException erro de I/O
   */
  private void writeAlgorithmElement(Writer writer, String ident,
    AlgorithmInfo algorithm) throws IOException {
    XMLEmptyElement xmlAlgoElement = new XMLEmptyElement();
    xmlAlgoElement.setTag(XmlCategoriesUtils.ALGORITHM_TAG);

    String algoId = algorithm.getId();
    xmlAlgoElement.newAttribute(XmlCategoriesUtils.ALGORITHM_ID_ATTR, algoId);
    xmlAlgoElement.write(writer, ident);
  }

  @Override
  public void write(Writer writer, String ident) throws IOException {
    String categoryId = getCategory().getId();
    String categoryName = getCategory().getName();
    newAttribute(XmlCategoriesUtils.CATEGORY_ID_ATTR, categoryId);
    newAttribute(XmlCategoriesUtils.CATEGORY_NAME_ATTR, categoryName);
    writeStartTagln(writer, ident);
    writeAllSubCategoryElement(writer, ident);
    writeAllAlgorithmElement(writer, ident);
    writeEndTag(writer, ident);
  }

}
