/*
 * $Id: ApplicationRegistry.java 95364 2009-08-25 13:30:11Z costa $
 */
package csbase.logic.applicationservice;

import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.Properties;
import java.util.ResourceBundle;

/**
 * Classe que representa um registro de aplicao.
 * 
 * @author Tecgraf/PUC-Rio
 */
public final class ApplicationRegistry implements Serializable {

  /**
   * cone configurado para a aplicao.
   */
  final private byte[] iconDefinition;

  /**
   * Imagem configurada para a aplicao.
   */
  final private byte[] imageDefinition;

  /**
   * Identificador da aplicao.
   */
  final private String id;

  /**
   * Resource bundle da aplicao para strings de texto
   */
  private AppPropertyResourceBundle propResBundle = null;

  /**
   * Tipos de arquivos que abrem esta aplicacao com o duplo clique
   */
  final private List<String> fileTypes = new ArrayList<String>();

  /**
   * Propriedades especficas da aplicao.
   */
  private Properties properties = null;

  /**
   * Identificador da chave de propriedade: nome do programa
   */
  private static final String APP_NAME_PROPERTY = "name";

  /**
   * Identificador da chave de propriedade: descrio do programa
   */
  private static final String DESCRIPTION_PROPERTY = "description";

  /**
   * Identificador da chave de propriedade: classe (Java) do programa
   */
  private static final String CLASS_NAME_PROPERTY = "class.name";

  /**
   * Identificador da chave de propriedade: Habilitao
   */
  private static final String IS_ENABLED_PROPERTY = "enabled";

  /**
   * Identificador da chave de propriedade: Flag de uso de prjetos.
   */
  private static final String REQUIRE_PROJECT_PROPERTY = "require.project";

  /**
   * Identificador da chave de propriedade: Visibilidade do dilogo
   */
  private static final String MAIN_FRAME_IS_VISIBLE_PROPERTY =
    "main.frame.visible";

  /**
   * Nome da propriedade que indica se a aplicao ser exibida com group frame.
   */
  private static final String SHOWN_AT_APP_PANEL_PROPERTY =
    "shown.at.application.panel";

  /**
   * Nome da propridade que indica se a aplicao ser exibida no menu de
   * aplicaes.
   */
  private static final String SHOWN_AT_MENU_PROPERTY =
    "shown.at.application.menu";

  /**
   * Identificador da chave de propriedade: Flag de necessidade de bundle.
   */
  private static final String NEED_BUNDLE_PROPERTY = "need.bundle";

  /**
   * Identificador da chave de propriedade: Comando
   */
  private static final String APP_COMMAND_PROPERTY = "application.command";

  /**
   * Identificador da chave de propriedade: Nome do autor
   */
  private static final String AUTHOR_NAME_PROPERTY = "author.name";

  /**
   * Identificador da chave de propriedade: Email de contato do autor
   */
  private static final String AUTHOR_MAIL_PROPERTY = "author.mail";

  /**
   * Identificador da chave de propriedade: Cdigo da verso
   */
  private static final String VERSION_PROPERTY = "version.number";

  /**
   * Identificador da chave de propriedade: tipos de arquivos tratados
   */
  private static final String FILE_TYPES_PROPERTY = "file.types";

  /**
   * Identificador da chave de propriedade: singleton
   */
  private static final String IS_SINGLETON_PROPERTY = "singleton";

  /**
   * Identificador dos arquivos adicionais de idiomas. Esta propriedade deve ser
   * definida no arquivo de propriedades da prpria aplicao, usando o mesmo
   * prefixo que os demais atributos, e seu valor deve ser o caminho at o nome
   * do arquivo de propriedades com '/' ao invs de '.', sem incluir o sufixo.
   * Por se tratar de uma propriedade do tipo "lista de strings", o nome da
   * propriedade deve ter um sufixo numrico (".1", ".2" etc.)
   * <p>
   * P.ex.: para definirmos um bundle adicional para a aplicao
   * <p>
   * <code>csbase.client.applications.notepad.Notepad</code>
   * <p>
   * a definio da propriedade ficaria no arquivo que define as propriedades da
   * aplicao e teria p.ex. o seguinte valor (o nome do arquivo  arbitrrio):
   * <p>
   * <code>/csbase/client/applications/notepad/resources/NotepadExtras</code>
   * <p>
   * Isto faria com que, para pt_BR o arquivo
   * <p>
   * <code>/csbase/client/applications/notepad/resources/NotepadExtras_pt_BR.properties</code>
   * <p>
   * fosse carregado.
   * <p>
   * <b>IMPORTANTE:</b> o caminho deve comear com '/' .
   */
  private static final String ADDITIONAL_LANGUAGE_FILE_PROPERTY =
    "additional.language.file";

  /**
   * Carrega os arquivos de idioma. O primeiro bundle carregado  o arquivo da
   * prpria aplicao (p.ex. <code>AppClassName_pt_BR.properties</code>).
   * Arquivos adicionais definidos via
   * {@link #ADDITIONAL_LANGUAGE_FILE_PROPERTY} so carregados em seguida, na
   * ordem em que foram definidos. A busca por tradues acontece do ltimo
   * arquivo definido para o primeiro; o primeiro que possuir a chave desejada
   * ser usado.
   * 
   * @param locale O locale do usurio logado.
   */
  final public void loadBundles(final Locale locale) {
    try {
      final List<String> allLanguageFilePathPrefixList =
        new LinkedList<String>();

      if (isBundleRequired()) {
        final String defaultLanguageFilePathPrefix =
          getDefaultLanguageFilePathPrefix();
        allLanguageFilePathPrefixList.add(defaultLanguageFilePathPrefix);
      }

      final List<String> additionalLanguageFilePathPrefixes =
        getStringListSpecificProperty(ADDITIONAL_LANGUAGE_FILE_PROPERTY);
      allLanguageFilePathPrefixList.addAll(additionalLanguageFilePathPrefixes);

      final String language = locale.getLanguage();
      final String country = locale.getCountry();

      AppPropertyResourceBundle resourceBundle = null;
      AppPropertyResourceBundle parent = null;
      for (final String filePathPrefix : allLanguageFilePathPrefixList) {
        final String idiomFileName =
          filePathPrefix + "_" + language + "_" + country + ".properties";
        final InputStream in = getClass().getResourceAsStream(idiomFileName);
        if (in == null) {
          throw new IllegalStateException(
            "Falha na carga do arquivo de idiomas: " + filePathPrefix);
        }
        resourceBundle = new AppPropertyResourceBundle(in);
        if (parent != null) {
          resourceBundle.setParent(parent);
        }
        parent = resourceBundle;
        in.close();
      }
      propResBundle = resourceBundle;
    }
    catch (final IOException e) {
      final String err = "Falha de carga de resource bundle da aplicao: ";
      final String className = getClassName();
      throw new IllegalStateException(err + className + " - " + e.getMessage());
    }
  }

  /**
   * Obtm o nome do arquivo de idioma padro da aplicao. O arquivo padro
   * fica no diretrio resources no mesmo pacote que a classe principal da
   * aplicaAo,
   * 
   * @return o nome do arquivo de idioma padro da aplicao
   */
  private String getDefaultLanguageFilePathPrefix() {
    final String className = getClassName();
    final int idx = className.lastIndexOf(".");
    final String path = className.substring(0, idx);
    final int length = className.length();
    final String classLastName = className.substring(idx + 1, length);
    final String sep = "/";
    final String replaced = path.replace(".", sep);
    final String resPath =
      sep + replaced + sep + "resources" + sep + classLastName;
    return resPath;
  }

  /**
   * Obtem todos os tipos de arquivos que abrem diretamente esta aplicacao.
   */
  private void loadFileTypes() {
    if (isPropertyNull(FILE_TYPES_PROPERTY)) {
      return;
    }
    final String prop = getStringSpecificProperty(FILE_TYPES_PROPERTY);
    final String[] aux = prop.split(",");
    for (final String element : aux) {
      fileTypes.add(element.trim().toUpperCase());
    }
  }

  /**
   * Consulta da existncia de uma string de idioma.
   * 
   * @param key a chave de busca.
   * @return o texto no idioma correto.
   * @see #propResBundle
   */
  final public boolean hasString(final String key) {
    try {
      if (this.propResBundle == null) {
        return false;
      }
      return this.propResBundle.containsKey(key);
    }
    catch (final Exception e) {
      return false;
    }
  }

  /**
   * Obtem uma string de idioma.
   * 
   * @param key a chave de busca.
   * @return o texto no idioma correto.
   * @see #propResBundle
   */
  final public String getString(final String key) {
    try {
      if (this.propResBundle == null) {
        return "<<<no bundle>>>";
      }
      return this.propResBundle.getString(key);
    }
    catch (final Exception e) {
      return (key == null) ? "<<<null>>>" : ("<<<" + key + ">>>");
    }
  }

  /**
   * Obtem uma string de idioma.
   * 
   * @param key a chave de busca.
   * @param objs os objetos de formatao
   * @return o texto no idioma correto.
   * @see #propResBundle
   */
  final public String getString(final String key, final Object[] objs) {
    try {
      final String pattern = getString(key);
      return MessageFormat.format(pattern, objs);
    }
    catch (final Exception e) {
      return (key == null) ? "<<<null>>>" : ("<<<" + key + ">>>");
    }
  }

  /**
   * Consulta se aplicao aparece em menu.
   * 
   * @return um flag indicativo.
   */
  final public boolean isShownAtApplicationMenu() {
    return getBooleanSpecificProperty(SHOWN_AT_MENU_PROPERTY);
  }

  /**
   * Consulta se aplicao aparece no app-panel
   * 
   * @return um flag indicativo.
   */
  final public boolean isShownAtApplicationPanel() {
    return getBooleanSpecificProperty(SHOWN_AT_APP_PANEL_PROPERTY);

  }

  /**
   * Consulta se aplicao necessita de projeto.
   * 
   * @return um flag indicativo.
   */
  final public boolean requireProject() {
    return getBooleanSpecificProperty(REQUIRE_PROJECT_PROPERTY);
  }

  /**
   * Consulta se a frame da aplicao deve ficar visvel.
   * 
   * @return um flag indicativo.
   */
  final public boolean mainFrameIsVisible() {
    return getBooleanSpecificProperty(MAIN_FRAME_IS_VISIBLE_PROPERTY);
  }

  /**
   * Obtm o id da aplicao.
   * 
   * @return o id com uma string.
   * @see #id
   */
  final public String getId() {
    return this.id;
  }

  /**
   * Obtm o resource bundle da aplicao
   * 
   * @return o bundle
   * @see #propResBundle
   */
  final public ResourceBundle getResourceBundle() {
    return this.propResBundle;
  }

  /**
   * Obtm a classe da aplicao.
   * 
   * @return o nome da classe.
   */
  final public String getClassName() {
    return getStringSpecificProperty(CLASS_NAME_PROPERTY);
  }

  /**
   * Obtm os tipos de arquivo que abrem diretamente esta aplicao.
   * 
   * @return os tipos de arquivo.
   */
  final public List<String> getFileTypes() {
    return this.fileTypes;
  }

  /**
   * Consulta ao comando da aplicao.
   * 
   * @return um texto com a string.
   */
  final public String getApplicationCommand() {
    if (isPropertyNull(APP_COMMAND_PROPERTY)) {
      return null;
    }
    return getStringSpecificProperty(APP_COMMAND_PROPERTY);
  }

  /**
   * Consulta ao diretrio onde est o comando da aplicao.
   * 
   * @return o diretrio onde est o comando da aplicao
   */
  final public String getApplicationCommandDir() {
    if (isPropertyNull(APP_COMMAND_PROPERTY)) {
      return null;
    }
    final String appCommand = getStringSpecificProperty(APP_COMMAND_PROPERTY);
    return ApplicationRegistryUtilities.getCommandPath(appCommand);
  }

  /**
   * Consulta a habilitao do aplicativo.
   * 
   * @return um flag booleano indicativo.
   * @see #isEnabled
   */
  final public boolean isEnabled() {
    return getBooleanSpecificProperty(IS_ENABLED_PROPERTY);
  }

  /**
   * Indica se a aplicao possui uma linha de comando e seu o comando foi
   * encotrado no <code>PATH</code> do cliente.
   * 
   * @return true se a aplicao pode ser executada
   */
  final public boolean isRunnable() {
    final String appCommand = getApplicationCommandDir();
    if (appCommand == null) {
      return true;
    }
    final String appCommandDir =
      ApplicationRegistryUtilities.getCommandPath(appCommand);
    return appCommandDir != null;
  }

  /**
   * Consulta se uma aplicao  "singleton"
   * 
   * @return um indicativo
   */
  final public boolean isSingleton() {
    return getBooleanSpecificProperty(IS_SINGLETON_PROPERTY);
  }

  /**
   * Indicativo de propriedade sem valor.
   * 
   * @param propName nome da propriedade.
   * @return indicativo
   */
  final public boolean isPropertyNull(final String propName) {
    final String tag = id + "." + propName;
    final String value = properties.getProperty(tag);
    if (value == null) {
      return true;
    }
    if (value.isEmpty()) {
      return true;
    }
    return false;
  }

  /**
   * Construtor padro.
   * 
   * @param id identificador da aplicao.
   * @param properties propriedades.
   * @param iconDefinition cone 16
   * @param imageDefinition imagem 32
   * @throws ApplicationRegistryCreationException em caso de falha.
   */
  public ApplicationRegistry(final String id, final Properties properties,
    final byte[] iconDefinition, final byte[] imageDefinition)
    throws ApplicationRegistryCreationException {
    this.id = id;
    this.iconDefinition = iconDefinition;
    this.imageDefinition = imageDefinition;
    this.properties = properties;

    checkProperties();

    loadFileTypes();
  }

  /**
   * Faz checagem de existncia de propriedades no registry criado no servidor
   * para no impedir que o servidor entre mal configurado.
   * 
   * @throws ApplicationRegistryCreationException em caso de erro.
   */
  private void checkProperties() throws ApplicationRegistryCreationException {
    final String[] mustProperties =
      new String[] { CLASS_NAME_PROPERTY, IS_ENABLED_PROPERTY,
          IS_SINGLETON_PROPERTY, MAIN_FRAME_IS_VISIBLE_PROPERTY,
          NEED_BUNDLE_PROPERTY, REQUIRE_PROJECT_PROPERTY,
          SHOWN_AT_APP_PANEL_PROPERTY, SHOWN_AT_MENU_PROPERTY };
    for (String prp : mustProperties) {
      if (isPropertyNull(prp)) {
        throw new ApplicationRegistryCreationException(id, prp);
      }
    }
  }

  /**
   * Consulta de bundle  necessrio.
   * 
   * @return indicativo.
   */
  public final boolean isBundleRequired() {
    return getBooleanSpecificProperty(NEED_BUNDLE_PROPERTY);
  }

  /**
   * Obtm uma lista de valores de propriedades que seguem o padro para
   * propriedades multi-valoradas usado pelo JDK da Sun. </p>
   * <ul>
   * <li>propriedade.1=ABCD</li>
   * <li>propriedade.2=EFGH</li>
   * </ul>
   * <ul>
   * <li>propriedade.nome.1=ABCD</li>
   * <li>propriedade.nome.2=EFGH</li>
   * </ul>
   * 
   * @param name O nome da propriedade (nos exemplos acima, seriam "propriedade"
   *        e "propriedade.nome" respectivamente).
   * 
   * @return Uma lista com todas as propriedades (caso no existam propriedades,
   *         a lista estar vazia).
   */
  final public List<String> getStringListSpecificProperty(final String name) {
    final List<String> list = new LinkedList<String>();
    for (int i = 1; true; i++) {
      final String pName = name + "." + i;
      if (isPropertyNull(pName)) {
        break;
      }
      final String value = getStringSpecificProperty(pName);
      list.add(value);
    }
    return Collections.unmodifiableList(list);
  }

  /**
   * Busca da imagem configurada para a aplicao.
   * 
   * @return a imagem
   */
  final public byte[] getImageDefinition() {
    return imageDefinition;
  }

  /**
   * Busca do cone configurada para a aplicao.
   * 
   * @return a imagem
   */
  final public byte[] getIconDefinition() {
    return iconDefinition;
  }

  /**
   * Consulta de propriedade formatada.
   * 
   * @param propertyName o nome da tag desejada.
   * @param locale locale
   * @return o texto com o valor da propriedade desejada.
   */
  private String getLocaledProperty(final String propertyName,
    final Locale locale) {
    final String language = locale.getLanguage();
    final String country = locale.getCountry();
    final Object[] args = new Object[] { language, country };
    final String key = propertyName + ".{0}.{1}";
    final String fmtKey = MessageFormat.format(key, args);
    if (isPropertyNull(fmtKey)) {
      return "<<" + key + ">>>";
    }
    final String value = getStringSpecificProperty(fmtKey);
    return value;
  }

  /**
   * Consulta ao email de contato.
   * 
   * @return o email como string
   */
  final public String getAuthorMail() {
    if (isPropertyNull(AUTHOR_MAIL_PROPERTY)) {
      return "-";
    }
    return getStringSpecificProperty(AUTHOR_MAIL_PROPERTY);
  }

  /**
   * Consulta ao nome do autor do programa.
   * 
   * @return o nome como string
   */
  final public String getAuthorName() {
    if (isPropertyNull(AUTHOR_NAME_PROPERTY)) {
      return "-";
    }
    return getStringSpecificProperty(AUTHOR_NAME_PROPERTY);
  }

  /**
   * Montagem de um nmero (convencionado) para uma verso de aplicativo.
   * 
   * @param vTxt o texto da verso.
   * @return um nmero de verso.
   */
  private int getVersionNumber(final String vTxt) {
    if (vTxt == null) {
      return 0;
    }
    final String txt = vTxt.trim();
    if (txt.equals("")) {
      return 0;
    }
    final String[] elems = txt.split("\\.");
    if (elems == null || elems.length != 3) {
      return 0;
    }

    int mj, mn, pt;
    try {
      mj = Integer.parseInt(elems[0]);
    }
    catch (final Throwable t) {
      mj = 0;
    }
    try {
      mn = Integer.parseInt(elems[1]);
    }
    catch (final Throwable t) {
      mn = 0;
    }
    try {
      pt = Integer.parseInt(elems[2]);
    }
    catch (final Throwable t) {
      pt = 0;
    }

    return (mj * 1000000) + (mn * 1000) + (pt);
  }

  /**
   * Consulta  verso do programa
   * 
   * @return a verso como nmero
   * @see #getVersionNumber(String)
   */
  final public int getVersionNumber() {
    return getVersionNumber(getVersion());
  }

  /**
   * Consulta  verso do programa
   * 
   * @return a verso como string
   */
  final public String getVersion() {
    if (isPropertyNull(VERSION_PROPERTY)) {
      return "0.0.0";
    }
    return getStringSpecificProperty(VERSION_PROPERTY);
  }

  /**
   * Obtm uma propriedade especfica do tipo inteiro.
   * 
   * @param propName nome da propriedade.
   * 
   * @return propriedade especfica do tipo inteiro.
   * 
   * @throws IllegalStateException se a propriedade no foi definida ou se o
   *         valor no pode ser convertido para um nmero inteiro
   */
  public final int getIntSpecificProperty(final String propName) {
    final String str = getStringSpecificProperty(propName);
    try {
      final int value = Integer.parseInt(str);
      return value;
    }
    catch (final Exception e) {
      final String err = "Propriedade " + propName + " inteira mal formatada";
      throw new IllegalStateException(err + " - applicao:" + getId());
    }
  }

  /**
   * Obtm uma propriedade especfica do tipo booleana.
   * 
   * @param propName nome da propriedade.
   * @return propriedade especfica do tipo booleana.
   */
  public final boolean getBooleanSpecificProperty(final String propName) {
    final String str = getStringSpecificProperty(propName);
    final String[] accepts = new String[] { "true", "false" };
    for (final String acc : accepts) {
      if (acc.equals(str.trim())) {
        final boolean value = Boolean.parseBoolean(str);
        return value;
      }
    }
    final String err = "Propriedade " + propName + " booleana mal formatada";
    throw new IllegalArgumentException(err + " - aplicao:" + getId());
  }

  /**
   * Obtm uma propriedade especfica.
   * 
   * @param propName nome da propriedade.
   * 
   * @return propriedade especfica.
   * 
   * @throws IllegalStateException se a propriedade no foi definida
   */
  final public String getStringSpecificProperty(final String propName) {
    if (isPropertyNull(propName)) {
      final String err = "Prop. indefinida [" + propName + "] para apl. " + id;
      throw new IllegalStateException(err);
    }

    final String tag = id + "." + propName;
    final String value = properties.getProperty(tag);
    if (value == null) {
      return null;
    }
    if (value.isEmpty()) {
      return null;
    }
    return value.trim();
  }

  /**
   * Consulta  propriedades especficas.
   * 
   * @return as propriedades.
   * @deprecated usar mtodos para os tipos especficos
   *             {@link #getIntSpecificProperty(String)},
   *             {@link #getBooleanSpecificProperty(String)}
   *             {@link #getStringSpecificProperty(String)} etc.
   */
  @Deprecated
  final public Properties getSpecificProperties() {
    return properties;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  final public String toString() {
    return getId();
  }

  /**
   * Consulta o nome de uma aplicao.
   * 
   * @param locale locale.
   * @return o nome
   */
  public final String getApplicationName(final Locale locale) {
    return getLocaledProperty(APP_NAME_PROPERTY, locale);
  }

  /**
   * Consulta o nome de uma aplicao.
   * 
   * @param locale locale.
   * @return o nome
   */
  public final String getApplicationDescription(final Locale locale) {
    return getLocaledProperty(DESCRIPTION_PROPERTY, locale);
  }

}
