package csbase.logic.trustedserver;

import java.io.Serializable;
import java.security.cert.Certificate;

import csbase.logic.ServerURI;

/**
 * <p>
 * Representa um servidor confivel.
 * </p>
 * 
 * <p>
 * Um servidor confivel  um objeto imutvel.
 * </p>
 *
 * @author Tecgraf/PUC-Rio
 */
public final class TrustedServer implements Serializable, Comparable {
  /** O identificador do servidor */
  private Integer id;

  /** O nome do servidor  */
  private String name;

  /** Indica se o servidor est suspenso. */
  private boolean suspended;

  /** Indica se o servidor  local */
  private boolean isLocal;

  /** A URI para conexo com o servidor */
  private ServerURI uri;

  /**
   * Indica se o servidor est completo (j possui certificado
   * cadastrado).
   */
  private boolean completed;

  /** O certificado do servidor  */
  private transient Certificate certificate;

  /**
   * <p>
   * Cria um servidor confivel.
   * </p>
   * 
   * <p>
   * Esse construtor deve ser utilizado apenas pelo servio de servidores
   * confiveis.
   * </p>
   *
   * @param id O identificador do servidor
   * @param name O nome do servidor
   * @param suspended Indica se o servidor est suspenso.
   * @param isLocal Indica se o servidor  local
   * @param uri A URI para conexo com o servidor
   *
   * @throws IllegalArgumentException Caso o algum dos parmetros recebido seja
   *         nulo.
   */
  public TrustedServer(Integer id, String name, boolean suspended,
    boolean isLocal, ServerURI uri) {
    if (id == null) {
      throw new IllegalArgumentException(
        "Identificador do servidor no pode ser nulo.");
    }
    if (name == null) {
      throw new IllegalArgumentException("Nome do servidor no pode ser nulo.");
    }
    if (uri == null)
      throw new IllegalArgumentException("A URI no pode ser nula");

    this.id = id;
    this.name = name;
    this.suspended = suspended;
    this.completed = false;
    this.isLocal = isLocal;
    this.uri = uri;
  }

  /**
   * <p>
   * Cria um servidor confivel.
   * </p>
   * 
   * <p>
   * Esse construtor deve ser utilizado apenas pelo servio de servidores
   * confiveis.
   * </p>
   *
   * @param id O identificador do servidor 
   * @param name O nome do servidor 
   * @param suspended Indica se o servidor est suspenso.
   * @param isLocal Indica se o servidor  local
   * @param uri A URI para conexo com o servidor
   * @param certificate O certificado do servidor 
   *
   * @throws IllegalArgumentException Caso o algum dos parmetros recebido seja
   *         nulo.
   */
  private TrustedServer(Integer id, String name, boolean suspended,
    boolean isLocal, ServerURI uri, Certificate certificate) {
    this(id, name, suspended, isLocal, uri);
    if (certificate == null) {
      throw new IllegalArgumentException(
        "O certificado do servidor no pode ser nulo.");
    }
    this.certificate = certificate;
    this.completed = true;
  }

  /**
   * <p>
   * Cria um servidor
   * </p>
   * 
   * <p>
   * OBS: O identificador do servidor confivel utilizado como base no deve ser
   * nulo.
   * </p>
   * 
   * <p>
   * Esse construtor deve ser utilizado apenas pelo servio de servidores
   * confivies
   * </p>
   *
   * @param trustedServer O servidor confivel que ser utilizado como base para a
   *        criao do novo servidor confivel.
   */
  public TrustedServer(TrustedServer trustedServer) {
    this(trustedServer.id, trustedServer.name, trustedServer.suspended,
      trustedServer.isLocal, trustedServer.uri);
  }

  /**
   * <p>
   * Cria um servidor 
   * </p>
   * 
   * <p>
   * Esse construtor deve ser utilizado apenas pelo servio de servidores
   * confiveis.
   * </p>
   *
   * @param id O identificador do servidor que est sendo criado.
   * @param serverInfo As informaes necessrias para a criao do novo
   *        servidor confivel.
   */
  public TrustedServer(Integer id, TrustedServerInfo serverInfo) {
    this(id, serverInfo.getName(), serverInfo.isSuspended(), serverInfo
      .isLocal(), serverInfo.getURI());
  }

  /**
   * <p>
   * Cria um servidor 
   * </p>
   * 
   * <p>
   * OBS: O identificador do servidor utilizado como base no deve ser
   * nulo.
   * </p>
   * 
   * <p>
   * Esse construtor deve ser utilizado apenas pelo servio de servidores
   * confiveis.
   * </p>
   *
   * @param trustedServer O servidor confivel que ser utilizado como base para a
   *        criao do novo servidor confivel.
   * @param certificate O certificado do servidor confivel que est sendo criado.
   */
  public TrustedServer(TrustedServer trustedServer, Certificate certificate) {
    this(trustedServer.id, trustedServer.name, trustedServer.suspended,
      trustedServer.isLocal, trustedServer.getURI(), certificate);
  }

  /**
   * <p>
   * Cria um servidor
   * </p>
   * 
   * <p>
   * Esse construtor deve ser utilizado apenas pelo servio de servidores
   * confiveis.
   * </p>
   *
   * @param id O identificador do servidor confivel que est sendo criado.
   * @param trustedServerInfo As informaes necessrias para a criao do novo
   *        servidor confivel.
   * @param certificate O certificado do servidor confivel que est sendo criado.
   */
  public TrustedServer(Integer id, TrustedServerInfo trustedServerInfo,
    Certificate certificate) {
    this(id, trustedServerInfo.getName(), trustedServerInfo.isSuspended(),
      trustedServerInfo.isLocal(), trustedServerInfo.getURI(), certificate);
  }

  /**
   * Obtm o nome do servidor
   *
   * @return O nome do servidor
   */
  public String getName() {
    return this.name;
  }

  /**
   * Obtm o identificador do servidor
   *
   * @return O identificador do servidor
   */
  public Object getId() {
    return this.id;
  }

  /**
   * Verifica se o servidor est suspenso.
   *
   * @return true, se estiver suspenso, ou false, caso contrrio.
   */
  public boolean isSuspended() {
    return this.suspended;
  }

  /**
   * Verifica se o servidor est completo.
   *
   * @return true, se estiver completo, ou false, caso contrrio.
   */
  public boolean isCompleted() {
    return this.completed;
  }

  /**
   * Obtm o certificado do servidor 
   *
   * @return O certificado do servidor, ou null, caso o servidor 
   *         ainda no possua certificado cadastrado.
   */
  public Certificate getCertificate() {
    return this.certificate;
  }

  /**
   * Verifica se o servidor  local
   * 
   * @return true se o servidor  local, false caso contrrio
   */
  public boolean isLocal() {
    return this.isLocal;
  }

  /**
   * @return a URI do servidor
   */
  public ServerURI getURI() {
    return this.uri;
  }

  public boolean equals(Object object) {
    if (object == null) {
      return false;
    }
    if (!this.getClass().equals(object.getClass())) {
      return false;
    }
    TrustedServer localServer = (TrustedServer) object;
    return this.name.equals(localServer.name);
  }

  public int hashCode() {
    return this.name.hashCode();
  }

  public String toString() {
    return this.uri.toString();
  }

  public int compareTo(Object object) {
    TrustedServer trustedServer = (TrustedServer) object;
    return this.name.compareTo(trustedServer.name);
  }

}
