/*
 * CommandPersistenceServiceInterface.java
 *
 * $Author:$ $Date:$ $Revision: 92186 $
 */

package csbase.remote;

import java.rmi.RemoteException;
import java.util.Collections;
import java.util.List;
import java.util.Set;

import csbase.exception.ServiceFailureException;
import csbase.logic.CommandInfo;
import csbase.logic.algorithms.AlgorithmConfigurator;
import csbase.logic.algorithms.commands.CommandPersistenceCommandNotificationData;
import csbase.util.callbak.ICallback;

/**
 * <p>
 * Interface do servio de persistncia de comandos.
 * </p>
 * Persiste os comandos no projeto corrente.
 *
 * @author TecGraf / PUC-Rio
 */
public interface CommandPersistenceServiceInterface extends ServiceInterface {

  /**
   * Nome do servio
   */
  String SERVICE_NAME = "CommandPersistenceService";

  /**
   * Obtm todas as informaes dos comandos salvas no projeto especificado.<br>
   * Diferente do {@link #getCommandInfos(Object)}, este mtodo retorna assim
   * que  chamado e os comandos so entregues, aos poucos, atravs do callback
   * a cada <i>desiredResponseTime</i>.
   *
   * @param projectId O identificador do projeto (No pode ser {@code null}).
   * @param desiredResponseTime O tempo, em milisegundos, que o cliente gostaria
   *        de esperar para cada pacote de comandos.
   * @param callback Objeto que ir receber os comandos lidos, agrupados em
   *        conjuntos no-modificveis ({@link Collections#unmodifiableSet(Set)}).
   *
   * @throws ServiceFailureException Em caso de erro ao tentar ler as
   *         informaes. Por ex.: erro de IO ao ler do dispositivo de
   *         persistncia.
   * @throws RemoteException Em caso de erro de RMI.
   */
  void getCommandInfos(Object projectId, final long desiredResponseTime,
    final ICallback<Set<CommandInfo>> callback) throws RemoteException;

  /**
   * Obtm todas as informaes dos comandos salvas no projeto especificado.
   *
   * @param projectId O identificador do projeto (No pode ser {@code null}).
   *
   * @return O conjunto de informaes no-modificvel
   *         ({@link Collections#unmodifiableSet(Set)}).
   *         Se no houver informaes, o conjunto retornado estar vazio.
   *
   * @throws ServiceFailureException Em caso de erro ao tentar ler as
   *         informaes. Por ex.: erro de IO ao ler do dispositivo de
   *         persistncia.
   * @throws RemoteException Em caso de erro de RMI.
   */
  Set<CommandInfo> getCommandInfos(Object projectId)
    throws RemoteException;

  /**
   * Obtm as informaes de comando especfico dado o identificador do
   * projeto e o identificador do comando.
   *
   * @param projectId O identificador do projeto (No pode ser {@code null}).
   * @param commandId O identificador do comando (No pode ser {@code null}).
   *
   * @return As informaes do comando ou {@code null} se ele no existir.
   *
   * @throws ServiceFailureException Em caso de erro ao tentar ler as
   *         propriedades. Por ex.: erro de IO ao ler do dispositivo de
   *         persistncia.
   * @throws RemoteException Em caso de erro de RMI.
   */
  CommandInfo getCommandInfo(Object projectId, String commandId)
    throws RemoteException;

  /**
   * <p>
   * Remove o comando especfico.
   * </p>
   * <p>
   * Caso a remoo tenha sido feita com sucesso, gera uma notificao
     * {@link CommandPersistenceCommandNotificationData} do tipo
     * {@link CommandPersistenceCommandNotificationData.Type#REMOVED}.
   * </p>
   *
   * @param projectId Identificador do projeto aonde o comando foi persistido
   *        (No pode ser {@code null}).
   * @param commandId Identificador do comando (No pode ser {@code null}).
   *
   * @return {@code false} e o comando no existir ou {@code true} em caso de
   *         sucesso.
   *
   * @throws ServiceFailureException Em caso de erro ao tentar ler as
   *         propriedades. Por ex.: erro de IO ao ler do dispositivo de
   *         persistncia.
   * @throws RemoteException Em caso de erro de RMI.
   */
   boolean removeCommandInfo(Object projectId, String commandId)
    throws RemoteException;

   /**
     * <p>
     * Remove comandos.
     * </p>
     * <p>
     * Para cada remoo feita com sucesso, gera uma notificao
     * {@link CommandPersistenceCommandNotificationData} do tipo
     * {@link CommandPersistenceCommandNotificationData.Type#REMOVED}.
     * </p>
     *
     * @param projectIds lista de identificador dos projetos respectivos a lista
     *        de identificadores de comandos (No pode ser {@code null}).
     * @param commandIds lista de identificador dos comandos a serem removidos
     *        (No pode ser {@code null}).
     *
     * @return uma lista de booleanos indicando o sucesso {@code true} da
     *         remoo de cada comando.
     *
     * @throws ServiceFailureException Em caso de erro ao tentar ler as
     *         propriedades. Por ex.: erro de IO ao ler do dispositivo de
     *         persistncia.
     * @throws RemoteException Em caso de erro de RMI.
     */
   boolean[] removeCommandInfos(List<Object> projectIds, List<String> commandIds)
    throws RemoteException;

  /**
   * <p>
   * Atualiza a decrio de um comando.
   * </p>
   *
   * <p>
   * Caso a atualizao tenha sido feita com sucesso, gera uma notificao
     * {@link CommandPersistenceCommandNotificationData} do tipo
     * {@link CommandPersistenceCommandNotificationData.Type#UPDATED}.
   * </p>
   *
   * @param projectId O identificador do projeto no qual o comando foi
   *        persistido (No aceita {@code null}).
   * @param commandId O identificador do comando (No aceita {@code null}).
   * @param description A nova descrio do comando (No aceita {@code null}).
   *
   * @throws ServiceFailureException Em caso de erro ao tentar escrever as
   *         informaes. Por ex.: erro de IO ao ler do dispositivo de
   *         persistncia.
   * @throws RemoteException Em caso de erro de RMI.
   */
  void updateCommandDescription(Object projectId, String commandId, String description)
    throws RemoteException;

  /**
   * Obtm o configurador de comando especfico dado o identificador do
   * projeto e o identificador do comando.
   *
   * @param projectId O identificador do projeto (No pode ser {@code null}).
   * @param commandId O identificador do comando (No pode ser {@code null}).
   *
   * @return O configurador do comando ou {@code null} se ele no existir.
   *
   * @throws ServiceFailureException Em caso de erro ao tentar ler o
   *         configurador. Por ex.: erro de IO ao ler do dispositivo de
   *         persistncia.
   * @throws RemoteException Em caso de erro de RMI.
   */
  AlgorithmConfigurator readAlgorithmConfigurator(Object projectId,
    String commandId) throws RemoteException;
}
