/*
 * HttpServiceInterface.java
 * $Author: rodrigoh $
 * $Revision: 61629 $  - $Date: 2007-03-19 15:39:56 -0300 (Mon, 19 Mar 2007) $
 */
package csbase.remote;

import java.rmi.RemoteException;

import java.util.Map;

import csbase.exception.HttpServiceException;

/**
 * Interface remota para acesso ao Servio de download de  arquivos.
 *
 * @author Taciana Melcop
 */
public interface HttpServiceInterface extends ServiceInterface {
  /** Nome do servio para o <code>ServiceManager</code> */
  public String SERVICE_NAME = "HttpService";

  //tipos de downloads

  /** tipo desconhecido */
  public int UNKNOWN_DOWNLOAD_TYPE = 0;

  /** tipo arquivo usando o caminho unix completo */
  public int FILE_DOWNLOAD_TYPE = 1;

  /** tipo URL */
  public int URL_DOWNLOAD_TYPE = 2;

  /** tipo texto ascii */
  public int TEXT_DOWNLOAD_TYPE = 3;

  /** tipo relatrio pdf */
  public int PDF_REPORT_DOWNLOAD_TYPE = 4;

  /** tipo texto csv */
  public int CSV_DOWNLOAD_TYPE = 5;
  
  /**
   * Obtem a url atraves da qual o arquivo deve ser acessado para download.
   *
   * @param userId Identificador do usurio no servidor http remoto.
   * @param password Senha do usurio para autenticar o usurio que est
   *        fazendo a chamada do mtodo.
   * @param ownerId Identificador do usurio dono do projeto.
   * @param projectName identificador do projeto.
   * @param filePath caminho do arquivo local no projeto.
   *
   * @return String que representa a url do arquivo
   *
   * @throws RemoteException .
   */
  public String getDownloadURL(
    Object userId, String password, Object ownerId, String projectName,
    String[] filePath)
  throws RemoteException;

  /**
   * Obtem a url atraves da qual o arquivo deve ser acessado para upload.
   *
   * @param userId Identificador do usurio no servidor http remoto.
   * @param password Senha do usurio para autenticar o usurio que est
   *        fazendo a chamada do mtodo.
   * @param ownerId Identificador do usurio dono do projeto.
   * @param projectName identificador do projeto.
   * @param filePath caminho do arquivo local no projeto.
   * @param fileType tipo do arquivo a ser obtido.
   *
   * @return String que representa a url do arquivo ou <code>null</code> caso
   *         ocorra algum problema durante a construo da url.
   *
   * @throws RemoteException .
   */
  public String getUploadURL(
    Object userId, String password, Object ownerId, String projectName,
    String[] filePath, String fileType)
  throws RemoteException;

  /**
   * Obtem o tipo do arquivo a ser exibido
   *
   * @param accessCode de acesso do arquivo
   * @param filePath caminho do arquivo
   *
   * @return .
   *
   * @throws RemoteException .
   */
  public String getFileType(String accessCode, String filePath)
  throws RemoteException;

  /**
   * Obtem o caminho do arquivo a ser exibido
   *
   * @param accessCode de acesso do arquivo
   *
   * @return caminho canonico do arquivo
   *
   * @throws RemoteException .
   */
  public String getFilePath(String accessCode)
  throws RemoteException;

  /**
   * Obtem o caminho do arquivo a ser exibido
   *
   * @param accessCode de acesso do arquivo
   * @param filePath caminho do arquivo
   *
   * @return caminho canonico do arquivo
   *
   * @throws RemoteException .
   */
  public String getFilePath(String accessCode, String filePath)
  throws RemoteException;

  /**
   * Obtem o caminho do arquivo de entrada do upload.
   *
   * @param accessCode de acesso do arquivo
   *
   * @return caminho canonico do arquivo
   *
   * @throws RemoteException .
   */
  public String getPresentationPath(String accessCode)
  throws RemoteException;

  /**
   * Obtem o caminho do arquivo de resultado do upload.
   *
   * @param accessCode de acesso do arquivo
   *
   * @return caminho canonico do arquivo
   *
   * @throws RemoteException .
   */
  public String getResultPath(String accessCode)
  throws RemoteException;

  /**
   * Remove o upload da lista de requisicoes e informa que o arquivo nao esta'
   * mais em construcao.
   *
   * @param accessCode
   * @param fileName O nome original do arquivo que sofreu upload.
   *
   * @return boolean se a operacao foi realizada com sucesso.
   *
   * @throws HttpServiceException .
   * @throws RemoteException .
   */
  public boolean finishUpload(String accessCode, String fileName)
  throws HttpServiceException, RemoteException;

  /**
   * Verifica se o arquivo e' diretorio.
   *
   * @param accessCode
   *
   * @return boolean que indica se o arquivo e' diretorio
   *
   * @throws RemoteException .
   */
  public boolean isDirectory(String accessCode)
  throws RemoteException;

  /**
   * Cria um arquivo
   *
   * @param accessCode
   * @param fileName Nome do arquivo
   *
   * @return caso o tipo tenha sido setado com sucesso
   *
   * @throws RemoteException .
   */
  public boolean createFile(String accessCode, String fileName)
  throws RemoteException;

  /**
   * Pega o texto que est armazenado na RequestInfo referente ao accessCode
   * passado. Se o accessCode for null ou invlido o mtodo retornar null.
   *
   * @param accessCode
   *
   * @return Texto a ser jogado no browser
   *
   * @throws RemoteException
   */
  public String getText(String accessCode)
  throws RemoteException;

  /**
   * Pega o tipo de mtodo utilizado no download referente ao accessCode
   * passado. Se o accessCode for null ou invlido o mtodo retornar
   * <code>UNKNOWN_DOWNLOAD_TYPE</code>.
   *
   * @param accessCode
   *
   * @return tipo de donwload
   *
   * @throws RemoteException
   */
  public int getDownloadType(String accessCode)
  throws RemoteException;

  /**
   * Pega a URL que ser chamada pelo browser quando o tipo de dowload for
   * TEXTO.
   *
   * @param userId
   * @param password
   * @param text
   *
   * @return
   *
   * @throws RemoteException
   */
  public String getDownloadTextURL(Object userId, String password, String text)
  throws RemoteException;

  /**
   * Pega a URL que ser chamada pelo browser quando o tipo de dowload for
   * TEXTO, permitindo a escolha do nome do arquivo.
   *
   * @param userId
   * @param password
   * @param text
   * @param fileName
   *
   * @return
   *
   * @throws RemoteException
   */
  public String getDownloadTextURL(Object userId, String password, String text,
      String fileName) throws RemoteException;
  
  /**
   * Pega a URL que ser chamada pelo browser quando o tipo de dowload for
   * CSV.
   *
   * @param userId identificao do usurio.
   * @param password senha do usurio.
   * @param text texto contendo o contedo do CSV.
   *
   * @return URL de download do arquivo CSV.
   *
   * @throws RemoteException
   */
  public String getDownloadCSVURL(Object userId, String password, String text)
  throws RemoteException;

  public String getDownloadCSVURL(Object userId, String password, String text, String fileName)
  throws RemoteException;

  /**
   * Obtm os parmetros da RequestInfo para usos gerais
   *
   * @param accessCode
   *
   * @return
   *
   * @throws RemoteException
   */
  public Map getParameters(String accessCode)
  throws RemoteException;

  /**
   * Obtm o nome da classe do driver JDBC para criao de uma conexo com um
   * banco de dados a partir do servlets(upload e download), caso seja
   * necessrio.
   *
   * @param accessCode
   *
   * @return
   *
   * @throws RemoteException
   */
  public String getJdbcDriverClassName(String accessCode)
  throws RemoteException;

  /**
   * Obtm URL JDBC para criao de uma conexo com um  banco de dados a partir
   * do servlets(upload e download), caso seja necessrio.
   *
   * @param accessCode
   *
   * @return
   *
   * @throws RemoteException
   */
  public String getJdbcURL(String accessCode)
  throws RemoteException;

  /**
   * Obtm o usurio JDBC para criao de uma conexo com um  banco de dados a
   * partir do servlets(upload e download), caso seja necessrio.
   *
   * @param accessCode
   *
   * @return
   *
   * @throws RemoteException
   */
  public String getJdbcUser(String accessCode)
  throws RemoteException;

  /**
   * Obtm a senha JDBC para criao de uma conexo com um  banco de dados a
   * partir do servlets(upload e download), caso seja necessrio.
   *
   * @param accessCode
   *
   * @return
   *
   * @throws RemoteException
   */
  public String getJdbcPassword(String accessCode)
  throws RemoteException;
}
