/*
 * $Id: ProjectServiceInterface.java 96246 2009-09-16 19:31:33Z jnlopes $
 */
package csbase.remote;

import java.io.File;
import java.rmi.RemoteException;
import java.util.Hashtable;
import java.util.List;
import java.util.Locale;
import java.util.Set;
import java.util.Vector;

import csbase.logic.ClientProjectFile;
import csbase.logic.ClientProjectFileInfo;
import csbase.logic.CommonClientProject;
import csbase.logic.CommonProjectInfo;
import csbase.logic.FileInfoSearchResult;
import csbase.logic.FileLockListenerInterface;
import csbase.logic.ProjectAdminInfo;
import csbase.logic.ProjectFileInfo;
import csbase.logic.ProjectFileTypeInfo;
import csbase.logic.UserProjectInfo;
import csbase.logic.ProjectPermissions.SharingType;

/**
 * Define a interface remota do servio de projetos do Server.
 */
public interface ProjectServiceInterface extends ServiceInterface,
  RemoteObservable {
  /** Nome do servio para o <code>ServiceManager</code> */
  public static final String SERVICE_NAME = "ProjectService";

  /**
   * Nome da propriedade da sesso do usurio no servidor que indica o projeto
   * corrente (na rvore) do usurio.
   */
  public static final String CURRENT_PROJECT_SESSION_PROPERTY =
    "current_project";

  /**
   * Constante que representa a espera, sem timeout, por um lock de arquivo.
   */
  public static final int INFINITE_TIMEOUT = -1;

  //
  // Mtodos relativos a projetos:
  //
  /**
   * Retorna os nomes de todos os projetos de um usurio (apenas aqueles dos
   * quais  o dono, no inclui projetos aos quais apenas tem acesso).
   * 
   * @param userId Identificador do usurio.
   * 
   * @return Array de strings contendo os nomes dos projetos do quais o usurio
   *          dono (no inclui os projetos aos quais apenas tem acesso).
   * 
   * @throws RemoteException
   */
  String[] getAllProjectNames(Object userId) throws RemoteException;

  /**
   * Obtm todos os projetos que um usurio participa, mas no  o administrador
   * do projeto.
   * 
   * @param userId Identificador do usurio.
   * 
   * @return Vector de UserProjectInfo contendo os nomes dos projetos e os
   *         identificadores dos usurios administradores de cada projeto.
   * 
   * @throws RemoteException
   */
  public Vector<UserProjectInfo> getAllUserProjectsFromOthers(Object userId)
    throws RemoteException;

  /**
   * Configura as permisses de acesso ao projeto.
   * 
   * @param projectID - identificador do projeto
   * @param sharingType - tipo do compartilhamento
   * @param usersRO - conjunto de usurios com acesso RO. O valor
   *        <code>null</code> indica que a lista no foi alterada
   * @param usersRW - conjunto de usurios com acesso RW. O valor
   *        <code>null</code> indica que a lista no foi alterada
   * @throws RemoteException
   */
  public void updateUsers(Object projectID, SharingType sharingType,
    Set<Object> usersRO, Set<Object> usersRW) throws RemoteException;

  /**
   * Remove um usurio das listas de acesso. Cada usurio por construo pode
   * pertencer a apenas uma das listas; este mtodo tenta a remoo em ambas as
   * listas antes de falhar.
   * 
   * @param projectID - identificador do projeto
   * @param userID - identificador do usurio
   * @return flag indicando se o usurio foi removido de alguma das listas
   * @throws RemoteException
   */
  public boolean removeUser(Object projectID, Object userID)
    throws RemoteException;

  /**
   * Remove um conjunto de usurios de ambas as listas de acesso (RO e RW).
   * 
   * @param projectID - identificador do projeto
   * @param users - conjunto de usurios a serem removidos
   * @throws RemoteException
   */
  public void removeUsers(Object projectID, Set<Object> users)
    throws RemoteException;

  /**
   * Abre um projeto de um usurio.
   * 
   * @param userId Identificador do usurio.
   * @param projectName Nome do projeto a ser aberto.
   * @param notify indicativo de notificao aos clientes de projeto aberto.
   * 
   * @return Objeto da classe {@link CommonClientProject} que representa o
   *         projeto.
   * 
   * @throws RemoteException falha de rmi
   */
  CommonClientProject openProject(Object userId, String projectName,
    boolean notify) throws RemoteException;

  /**
   * Cria, abre e retorna um novo projeto.
   * 
   * @param info Informaes do projeto.
   * 
   * @return o projeto criado
   * 
   * @throws RemoteException falha de rmi
   */
  CommonClientProject createProject(CommonProjectInfo info)
    throws RemoteException;

  /**
   * Cria um novo projeto solicitando alocao de rea em disco.
   * 
   * @param info Informaes do projeto.
   * @param size O tamanho da rea de alocao.
   * 
   * @throws RemoteException falha de rmi
   */
  public void createProjectWithAllocation(CommonProjectInfo info, long size)
    throws RemoteException;

  /**
   * Informa que um ou mais projetos tiveram sua rea concedida. Os projetos
   * passam a estar bloqueados. Esse mtodo somente pode ser chamado pelo
   * administrador do sistema.
   * 
   * @param infos Um array com os objetos <code>ProjectAdminInfo</code> com as
   *        informaes de desbloqueio de projetos devido a alocao de rea
   *        pelo administrador.
   * 
   * @throws RemoteException
   */
  public void unlockProjectsWithAllocation(ProjectAdminInfo[] infos)
    throws RemoteException;

  /**
   * Informa que um ou mais projetos tiveram sua rea liberada. Esse mtodo
   * somente pode ser chamado pelo administrador do sistema.
   * 
   * @param infos Um array com os objetos <code>ProjectAdminInfo</code> com as
   *        informaes de liberao da rea.
   * 
   * @throws RemoteException
   */
  public void freeAreaForProjects(ProjectAdminInfo[] infos)
    throws RemoteException;

  /**
   * Modifica as informaes de um projeto.
   * 
   * @param id Identificador do projeto.
   * @param info Novas informaes do projeto.
   * 
   * @return As novas informaes do projeto. Caso alguma informao no possa
   *         ser alterada, o valor anterior  retornado.
   * 
   * @throws RemoteException
   */
  CommonProjectInfo modifyProject(Object id, CommonProjectInfo info)
    throws RemoteException;

  /**
   * Remove um projeto.
   * 
   * @param userId Identificador do projeto.
   * @param projectName .
   * 
   * @throws RemoteException falha de rmi
   */
  void removeProject(Object userId, String projectName) throws RemoteException;

  /**
   * Fecha um projeto.
   * 
   * @param id Identificador do projeto a ser fechado.
   * @param notify indicativo de notificao ao cliente.
   * 
   * @throws RemoteException
   */
  void closeProject(Object id, boolean notify) throws RemoteException;

  /**
   * Reconstri a rvore do projeto a partir do sistema de arquivos do servidor.
   * 
   * @param id Identificador do projeto.
   * 
   * @throws RemoteException
   */
  public void rebuildTree(Object id) throws RemoteException;

  /**
   * Reconstri o diretrio especificado de um projeto a partir do sistema de
   * arquivos do servidor.
   * 
   * @param projectId identificador do projeto.
   * @param path caminho para o diretrio a ser atualizado.
   * 
   * @throws RemoteException
   */
  public void refreshDir(Object projectId, String[] path)
    throws RemoteException;

  //
  // Mtodos relativos a rvores de projetos:
  //
  /**
   * Cria um novo diretrio dentro de um projeto.
   * 
   * @param id Identificador do projeto.
   * @param path Caminho do diretrio pai.
   * @param name Nome do diretrio a ser criado.
   * 
   * @throws RemoteException erro de rmi
   */
  void createDirectory(Object id, String[] path, String name)
    throws RemoteException;

  /**
   * Cria um novo diretrio, ou uma nova rvore de diretrios, dentro de um
   * projeto.
   * 
   * @param id Identificador do projeto.
   * @param path Caminho do diretrio a ser criado.
   * 
   * @throws RemoteException erro de rmi
   */
  void createDirectory(Object id, String[] path) throws RemoteException;

  /**
   * Retorna a lista de arquivos-filho.
   * 
   * @param clientFile arquivo
   * @return lista de arquivos
   * 
   * @throws RemoteException erro de rmi
   */
  ClientProjectFile[] getChildren(ClientProjectFile clientFile)
    throws RemoteException;

  /**
   * Retorna o arquivo.
   * 
   * @param parent diretrio aonde o arquivo se encontra.
   * @param name nome do arquivo.
   * 
   * @return o arquivo.
   * 
   * @throws RemoteException erro de rmi
   */
  ClientProjectFile getChild(ClientProjectFile parent, String name)
    throws RemoteException;

  /**
   * Cria um novo arquivo dentro de um projeto.
   * 
   * @param id Identificador do projeto.
   * @param path Caminho do diretrio pai.
   * @param name Nome do arquivo a ser criado.
   * @param type Tipo do arquivo a ser criado.
   * 
   * @throws RemoteException erro de rmi
   */
  void createFile(Object id, String[] path, String name, String type)
    throws RemoteException;

  /**
   * Cria simultaneamente vrios arquivos dentro de um mesmo diretrio. Os
   * caminhos dos arquivos devem ser passados relativamente ao diretrio-base,
   * que atua como raiz da rvore de arquivos.
   * 
   * @param projectId identificador do projeto.
   * @param parentPath caminho do diretrio-base, ancestral de todos os arquivos
   *        a serem criados.
   * @param fileInfoList lista de dados dos arquivos a serem criados.
   * @throws RemoteException erro de rmi
   */
  public void createFiles(Object projectId, String[] parentPath,
    List<ProjectFileInfo> fileInfoList) throws RemoteException;

  /**
   * Troca o nome de um arquivo de um projeto.
   * 
   * @param id Identificador do projeto.
   * @param path Caminho do arquivo a ser renomeado.
   * @param name Novo nome do arquivo.
   * 
   * @throws RemoteException falha de rmi
   */
  void renameFile(Object id, String[] path, String name) throws RemoteException;

  /**
   * Muda o tipo de um arquivo de um projeto.
   * 
   * @param id Identificador do projeto.
   * @param path Caminho do arquivo cujo tipo vai ser alterado.
   * @param type Novo tipo do arquivo.
   * 
   * @throws RemoteException
   */
  public void changeFileType(Object id, String[] path, String type)
    throws RemoteException;

  /**
   * Copia um arquivo de um projeto para outro diretrio.
   * 
   * @param id Identificador do projeto.
   * @param filePath Caminho do arquivo a ser copiado.
   * @param dirPath Caminho do diretrio destino.
   * 
   * @throws RemoteException erro de rmi
   */
  void copyFile(Object id, String[] filePath, String[] dirPath)
    throws RemoteException;

  /**
   * Copia um arquivo de um projeto para um diretrio de outro projeto.
   * 
   * @param sourceProjectId Identificador do projeto Origem.
   * @param filePathSource Caminho do arquivo a ser copiado.
   * @param targetProjectId Identificador do projeto Destino.
   * @param dirPathTarget Caminho do diretrio destino.
   * 
   * @throws RemoteException erro de rmi
   */
  void copyFile(Object sourceProjectId, String[] filePathSource,
    Object targetProjectId, String[] dirPathTarget) throws RemoteException;

  /**
   * Move um arquivo de um projeto para outro diretrio.
   * 
   * @param id Identificador do projeto.
   * @param filePath Caminho do arquivo a ser movido.
   * @param dirPath Caminho do diretrio destino.
   * 
   * @throws RemoteException erro de rmi
   */
  void moveFile(Object id, String[] filePath, String[] dirPath)
    throws RemoteException;

  /**
   * Move um arquivo de um projeto para um diretrio de outro projeto.
   * 
   * @param sourceProjectId Identificador do projeto Origem.
   * @param filePathSource Caminho do arquivo a ser copiado.
   * @param targetProjectId Identificador do projeto Destino.
   * @param dirPathTarget Caminho do diretrio destino.
   * 
   * @throws RemoteException erro de rmi
   */
  void moveFile(Object sourceProjectId, String[] filePathSource,
    Object targetProjectId, String[] dirPathTarget) throws RemoteException;

  /**
   * Remove um arquivo de um projeto.
   * 
   * @param id Identificador do projeto.
   * @param path Caminho do arquivo a ser removido.
   * 
   * @throws RemoteException
   */
  void removeFile(Object id, String[] path) throws RemoteException;

  /**
   * Atualiza as informaes de um arquivo.
   * 
   * @param id Identificador do projeto.
   * @param path Caminho do arquivo a ser atualizado.
   * 
   * @return .
   * 
   * @throws RemoteException
   */
  ClientProjectFileInfo getUpdatedFileInfo(Object id, String[] path)
    throws RemoteException;

  /**
   * Remove vrios arquivos em uma s chamada ao servidor.
   * 
   * @param id Identificador do projeto.
   * @param paths lista de paths, que cada cada item  um caminho de arquivo a
   *        ser removido.
   * 
   * @throws RemoteException
   */
  void removeFiles(Object id, String[][] paths) throws RemoteException;

  //
  // Mtodos relativos a manipulao de arquivos:
  //
  /**
   * Obtm a descrio atual deste arquivo.
   * 
   * @param id Identificador do projeto.
   * @param path Caminho do arquivo.
   * 
   * @return O texto com a descrio do arquivo.
   * 
   * @throws RemoteException
   */
  String getFileDescription(Object id, String[] path) throws RemoteException;

  /**
   * Muda a descrio atual deste arquivo.
   * 
   * @param id Identificador do projeto.
   * @param path Caminho do arquivo.
   * @param text O novo texto da descrio do arquivo.
   * 
   * @throws RemoteException
   */
  void setFileDescription(Object id, String[] path, String text)
    throws RemoteException;

  /**
   * Anexa um texto  descrio atual deste arquivo. Permite a gerao de
   * histricos de modificao.
   * 
   * @param id Identificador do projeto.
   * @param path Caminho do arquivo.
   * @param text O texto a ser anexado.
   * 
   * @throws RemoteException
   */
  void appendFileDescription(Object id, String[] path, String text)
    throws RemoteException;

  /**
   * Verifica se o arquivo existe no servidor.
   * 
   * @param projectId Identificador do projeto
   * @param filePath Caminho para o arquivo cujo o estado ser alterado
   * 
   * @return true se o arquivo existe no servidor.
   * @throws RemoteException
   */
  boolean existsFile(Object projectId, String[] filePath)
    throws RemoteException;

  /**
   * Abre este arquivo para leitura e escrita.
   * 
   * @param id Identificador do projeto.
   * @param path Caminho do arquivo a ser aberto.
   * @param readOnly indica se o arquivo deve ser aberto apenas para leitura ou
   *        para leitura e escrita.
   * 
   * @return O channel que conecta ao arquivo no servidor ou <code>null</code>
   *         caso a operao falhe.
   * @throws RemoteException falha de rmi
   */
  public ProjectFileChannel openFileChannel(Object id, String[] path,
    boolean readOnly) throws RemoteException;

  /**
   * Abre um arquivo para leitura exclusiva que no  feita com o uso de
   * <code>ByteBuffer</code>. A implementao desse mtodo determina o uso de
   * RMI para transmisso dos dados.
   * 
   * @param id Identificador do projeto.
   * @param path Caminho do arquivo a ser aberto.
   * 
   * @throws RemoteException
   */
  public void openFileForReading(Object id, String[] path)
    throws RemoteException;

  /**
   * Abre um arquivo para escrita exclusiva que no  feita com o uso de
   * <code>ByteBuffer</code>. A implementao desse mtodo determina o uso de
   * RMI para transmisso dos dados.
   * 
   * @param id Identificador do projeto.
   * @param path Caminho do arquivo a ser aberto.
   * 
   * @throws RemoteException falha de rmi
   */
  public void openFileForWriting(Object id, String[] path)
    throws RemoteException;

  /**
   * Obtm lock de escrita exclusivo para um arquivo. Locks exclusivos so
   * obtidos apenas uma vez.
   * 
   * @param id - identificador do projeto
   * @param path - path para o arquivo, relativo ao projeto
   * @return identificador do lock se o lock foi obtido, ou null caso contrrio
   * 
   * @throws RemoteException falha de rmi
   */
  public Object acquireExclusiveLock(Object id, String[] path)
    throws RemoteException;

  /**
   * Informa aonde o diretrio do projeto est instalado no servidor.
   * 
   * @param id - identificador do projeto
   * @return o canonical path do diretrio do projeto no servidor.
   * @see File#getCanonicalPath()
   * 
   * @throws RemoteException falha de rmi
   */
  public String getProjectLocationInServer(Object id) throws RemoteException;

  /**
   * Tenta obter um lock de escrita exclusivo para um arquivo. Locks exclusivos
   * so obtidos apenas uma vez.
   * 
   * Este mtodo cadastra uma solicitao de lock numa fila priorizada pela
   * ordem de chegada dos pedidos de lock. Quando o lock  obtido, o mtodo
   * FileLockListenerInterface.fileLocked  chamado. Quando o tempo mximo de
   * espera do lock  atingido e o lock no foi obtido o mtodo
   * FileLockListenerInterface.fileLockExpired  chamado.
   * 
   * @param id identificador do projeto
   * @param path path para o arquivo, relativo ao projeto
   * @param listener observador do lock cadastrado
   * @param timeout tempo mximo de espera pelo lock em milisegundos. A
   *        constante INFINITE_TIMOUT foi criada para timout infinito.
   * @return identificador do pedido de lock
   * 
   * @throws RemoteException falha de rmi
   */
  public Object acquireExclusiveLock(Object id, String[] path,
    FileLockListenerInterface listener, long timeout) throws RemoteException;

  /**
   * Obtm lock de escrita compartilhado para um arquivo. Locks compartilhados
   * podem ser obtidos mltiplas vezes; cada novo lock para o arquivo incrementa
   * o contador de referncias para o lock. Locks compartilhados s podem ser
   * removidos se o contador de referncias for igual a zero.
   * 
   * @param id - identificador do projeto
   * @param path - path para o arquivo, relativo ao projeto
   * @return identificador do lock se o lock foi obtido, ou null caso contrrio
   * 
   * @throws RemoteException falha de rmi
   */
  public Object acquireSharedLock(Object id, String[] path)
    throws RemoteException;

  /**
   * Tenta obter lock de escrita compartilhado para um arquivo. Locks
   * compartilhados podem ser obtidos mltiplas vezes; cada novo lock para o
   * arquivo incrementa o contador de referncias para o lock. Locks
   * compartilhados s podem ser removidos se o contador de referncias for
   * igual a zero.
   * 
   * Este mtodo cadastra uma solicitao de lock compartilhado em uma fila
   * priorizada pela ordem de chegada dos pedidos de lock. Quando o lock 
   * obtido, o mtodo FileLockListenerInterface.fileLocked  chamado. Quando o
   * tempo mximo de espera do lock  atingido e o lock no foi obtido o mtodo
   * FileLockListenerInterface.fileLockExpired  chamado.
   * 
   * @param id identificador do projeto
   * @param path path para o arquivo, relativo ao projeto
   * @param listener observador do lock cadastrado
   * @param timeout tempo mximo de espera pelo lock em milisegundos. A
   *        constante INFINITE_TIMOUT foi criada para timout infinito.
   * @return identificador do pedido de lock
   * 
   * @throws RemoteException falha de rmi
   */
  public Object acquireSharedLock(Object id, String[] path,
    FileLockListenerInterface listener, long timeout) throws RemoteException;

  /**
   * Libera lock de um arquivo. O lock s  liberado se o identificador for o
   * mesmo do lock obtido. Se o lock  compartilhado, a contagem de referncias
   * para o lock  decrementada. Quando o nmero de referncias chega a zero, o
   * lock  efetivamente removido.
   * 
   * @param id identificador do projeto
   * @param path path para o arquivo, relativo ao projeto
   * @param lockId identificador do lock
   * @return contagem de referncias para o lock
   * 
   * @throws RemoteException falha de rmi
   */
  public int releaseLock(Object id, String[] path, Object lockId)
    throws RemoteException;

  /**
   * Fora a remoo do lock do arquivo. S pode ser usado pelo administrador ou
   * pelo usurio dono do projeto, se acontencer alguma situao de ter um lock
   * "preso". Situaes onde as sesso do usurio que foi finalizadas por falha
   * na comunicao, ou usurios que terminam o cliente abruptamente j esto
   * previstas e no devem precisar deste mecanismo de segurana.
   * 
   * @param id identificador do projeto
   * @param path path para o arquivo, relativo ao projeto
   * @return contagem de referncias para o lock
   * 
   * @throws RemoteException falha de rmi
   */
  public int forceReleaseLock(Object id, String[] path) throws RemoteException;

  /**
   * L um objeto gravado no arquivo. A implementao desse mtodo utiliza RMI
   * para envio do objeto serializado.
   * 
   * @param id Identificador do projeto.
   * @param path Caminho do arquivo a ser lido.
   * 
   * @return O objeto lido do arquivo.
   * 
   * @throws Exception
   * @throws RemoteException
   */
  public Object readObject(Object id, String[] path) throws Exception,
    RemoteException;

  /**
   * Grava um objeto no arquivo. A implementao desse mtodo utiliza RMI para
   * envio do objeto serializado.
   * 
   * @param id Identificador do projeto.
   * @param path Caminho do arquivo a ser gravado.
   * @param obj O objeto a ser gravado no arquivo.
   * 
   * @throws Exception
   * @throws RemoteException
   */
  public void writeObject(Object id, String[] path, Object obj)
    throws Exception, RemoteException;

  /**
   * Fecha um arquivo aberto para leitura ou escrita que no usa
   * <code>ByteBuffer</code>.
   * 
   * @param id Identificador do projeto.
   * @param path Caminho do arquivo a ser aberto.
   * 
   * @throws RemoteException
   */
  public void closeFile(Object id, String[] path) throws RemoteException;

  /**
   * Retorna o tamanho de um arquivo.
   * 
   * @param id - identificador do projeto
   * @param path - path para o arquivo (relativo ao projeto)
   * @return tamanho do arquivo
   * 
   * @throws RemoteException
   */
  public long fileSize(Object id, String[] path) throws RemoteException;

  /**
   * Retorna a data de modificao de um arquivo.
   * 
   * @param id - identificador do projeto
   * @param path - path para o arquivo (relativo ao projeto)
   * @return data de modificao do arquivo
   * 
   * @throws RemoteException
   */
  public long getModificationDate(Object id, String[] path)
    throws RemoteException;

  /**
   * Registra um observador interessado nas mudanas que ocorrerem em um
   * projeto.
   * 
   * @param observer O observador remoto que registra o interesse nas mudanas
   *        sobre um projeto.
   * @param arg Identificador do projeto no qual o observador est interessado.
   * 
   * @throws RemoteException
   */
  public void addObserver(RemoteObserver observer, Object arg)
    throws RemoteException;

  /**
   * Remove um observador registrado em um projeto.
   * 
   * @param observer O observador remoto a ser removido.
   * @param arg Identificador do projeto.
   * 
   * @return .
   * 
   * @throws RemoteException
   */
  public boolean deleteObserver(RemoteObserver observer, Object arg)
    throws RemoteException;

  /**
   * Obtem todas as propriedades dos arquivos.
   * 
   * @param loc .
   * 
   * @return Hashtable com todos os tipos de arquivos.
   * 
   * @throws RemoteException
   */
  public Hashtable<String, ProjectFileTypeInfo> getAllFileTypes(Locale loc)
    throws RemoteException;

  /**
   * Obtem todas as propriedades de um tipo de arquivo.
   * 
   * @param type o tipo do arquivo desejado
   * @return informaes de um tipo de arquivo
   * @throws RemoteException
   */
  public ProjectFileTypeInfo getFileType(String type) throws RemoteException;

  /**
   * Obtm as informaes de todos os projetos dos usurios.
   * 
   * @return um array de <code>ProjectAdminInfo</code>.
   * 
   * @throws RemoteException
   */
  public ProjectAdminInfo[] getLockedProjectAdminInfo() throws RemoteException;

  /**
   * Verifica se um projeto est desbloqueado utilizando rea reservada.
   * 
   * @param userId o identificador do usurio
   * @param projectName o nome do projeto
   * 
   * @return verdadeiro, se o projeto est desbloqueado utilizando rea
   *         reservada ou falso, caso contrrio.
   * 
   * @throws RemoteException
   */
  public boolean isUnlockedWithAreaAllocated(Object userId, String projectName)
    throws RemoteException;

  /**
   * Muda a data de alterao de um arquivo
   * 
   * @param id Identificador do projeto
   * @param path Caminho do arquivo
   * @param date Data em long
   * 
   * @throws RemoteException
   */
  public void setFileModificationDate(Object id, String[] path, long date)
    throws RemoteException;

  /**
   * Obtm um conjunto com informaes sobre todos os projetos de um usurio
   * (inclusive os projetos compartilhados por outros usurios).
   * 
   * @param userId O identificador do usurio.
   * 
   * @return Um conjunto com informaes sobre todos os projetos de um usurio.
   *         Caso o usurio no tenha acesso a nenhum projeto, o conjunto ter
   *         tamanho zero.
   * 
   * @throws RemoteException
   */
  public Set<UserProjectInfo> getAllProjectsInfo(Object userId)
    throws RemoteException;

  /**
   * Obtm um conjunto com o identificador de todos os projetos gerenciveis
   * pelo usurio corrente. <br>
   * No caso de um usurio comum isso representa todos os projetos daquele
   * usurio, diferente do administrador que pode gerenciar todos os projetos.
   * 
   * @return Um conjunto com o identificador de todos os projetos gerenciveis
   *         pelo usurio corrente. Caso o usurio no tenha acesso a nenhum
   *         projeto, o conjunto ter tamanho zero.
   * 
   * @throws RemoteException
   */
  Set<Object> getManageableProjectsId() throws RemoteException;

  /**
   * Altera o estado do arquivo para o valor dado no parmetro
   * <code>underConstruction</code>.
   * 
   * @param projectId Identificador do projeto
   * @param filePath Caminho para o arquivo cujo o estado ser alterado
   * @param underConstruction Indica se o arquivo deve ficar em construo
   * 
   * @return true se o estado do arquivo foi alterado, ou false caso ocorra
   *         algum problema.
   * 
   * @throws RemoteException
   */
  public boolean setUnderConstruction(Object projectId, String[] filePath,
    boolean underConstruction) throws RemoteException;

  /**
   * Inicia ou altera o intervalo de atualizao de um arquivo.
   * 
   * @param projectId Identificador do projeto.
   * @param filePath Caminho para o arquivo atualizvel.
   * @param interval O intervalo (em segundos) de atualizao do arquivo.
   * 
   * @return {@code true} caso a atualizao seja iniciada, ou {@code false},
   *         caso contrrio.
   * 
   * @throws RemoteException
   */
  boolean startUpdate(Object projectId, String[] filePath, long interval)
    throws RemoteException;

  /**
   * Pra a atualizao de um arquivo.
   * 
   * @param projectId Identificador do projeto.
   * @param filePath Caminho para o arquivo atualizvel.
   * 
   * @return {@code true} caso a atualizao seja terminada, ou {@code false},
   *         caso contrrio.
   * 
   * @throws RemoteException
   */
  boolean stopUpdate(Object projectId, String[] filePath)
    throws RemoteException;

  /**
   * Obtm a lista de resultados de uma busca de arquivos em um projeto, de
   * acordo com as condies solicitadas (nome do arquivo, se deve ignorar
   * mausculas/minsculas, se usa ou no uma expresso regular).
   * 
   * @param userId - O identificador do usurio dodo projeto.
   * @param projectName - Nome do projeto.
   * @param text - Texto sendo pesquisado.
   * @param isCaseInsensitive - Considerar maisculas/minsculas.
   * @param isRegex - Indica que o texto  uma expresso regular.
   * 
   * @return Lista com as informaes sobre todos arquivos encontrados. Caso
   *         nenhum arquivo tenha sido encontrado, retorna uma lista vazia.
   * 
   * @throws RemoteException
   */
  public List<FileInfoSearchResult> getAllFileInfoSearchResult(Object userId,
    final String projectName, String text, boolean isCaseInsensitive,
    boolean isRegex) throws RemoteException;

  /**
   * Converte um projeto com alocao de rea em um projeto sem alocao.
   * 
   * OBS. Este mtodo foi criado para atender uma necessidade especfica. Deve
   * ser usado com a mesma cautela exigida pelo mtodo freeAreaForProjects()
   * (ambos descartam informaes administrativas).
   * 
   * @param ownerId Id do usurio.
   * @param projectName Nome do projeto.
   * @throws RemoteException Exceo no acesso remoto aos dados.
   */
  public void setAllocatedProjectAsUnallocated(final Object ownerId,
    final String projectName) throws RemoteException;

  /**
   * Converte um projeto sem alocao de rea em um projeto com alocao.
   * 
   * OBS. Este mtodo foi criado para atender uma necessidade especfica e deve
   * ser usado com cautela.
   * 
   * @param ownerId Id do usurio.
   * @param projectName Nome do projeto.
   * @param areaSize Tamanho (em Gb) da rea a ser alocada ao projeto.
   * @throws RemoteException Exceo no acesso remoto aos dados.
   */
  public void setUnallocatedProjectAsAllocated(final Object ownerId,
    final String projectName, final int areaSize) throws RemoteException;

  /**
   * Obtm as informaes administrativas de um projeto de um usurio.
   * 
   * @param userId o identificador do usurio
   * @param projectName o nome do projeto
   * 
   * @return O <code>ProjectAdminInfo</code> do projeto de um usurio.
   * @throws RemoteException Exceo no acesso remoto aos dados.
   */
  public ProjectAdminInfo getProjectAdminInfo(Object userId, String projectName)
    throws RemoteException;

  /**
   * Indica se o servio de projetos utiliza reserva de rea.
   * 
   * @return Boolean que indica se o servio de projetos utiliza reserva de
   *         rea.
   * @throws RemoteException Exceo no acesso remoto aos dados.
   */
  public boolean isAreaReserved() throws RemoteException;

}
