/*
 * RemoteObjectObservable.java
 * 
 * $Id: RemoteObjectObservable.java 90852 2009-04-27 17:43:11Z costa $
 */
package csbase.remote;

import java.rmi.ConnectException;
import java.rmi.NoSuchObjectException;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import csbase.logic.ObserverData;

/**
 * A classe <code>RemoteObjectObservable</code> representa um objeto que pode
 * ser observado remotamente. Essa classe pode ser estendida para representar um
 * objeto remoto que a aplicao precisa observar.
 * 
 * <p>
 * Um objeto pode ser observado por diversos observadores remotos. Um observador
 * remoto deve implementar a interface {@link csbase.remote.RemoteObserver
 * RemoteObserver}. Quando um objeto observado sofre alguma mudana, informada
 * pelo mtodo <code>setChanged</code>, todos os observadores so notificados
 * atravs do seu mtodo <code>update</code>.
 * </p>
 * 
 * @author Maria Julia Lima
 * @version $Revision: 90852 $
 */
public abstract class RemoteObjectObservable implements RemoteObservable {
  /**
   * Estrutura para armazenar os observadores remotos associados a um argumento.
   * Vector de ObserverData.
   */
  protected Vector<ObserverData> obs;
  /**
   * Flag que indica se o objeto sofreu alguma mudana.
   */
  private boolean changed;

  /**
   * Inclui um observador remoto no conjunto de observadores. Um observador
   * remoto deve implementar a interface {@link csbase.remote.RemoteObserver
   * RemoteObserver}. Se esse observador j pertencer ao conjunto, a nova
   * referncia substitui a anterior.
   * 
   * @param o um observador remoto que vai ser includo
   * @param arg argumento do observador
   */
  public synchronized void addObserver(RemoteObserver o, Object arg) {
    ObserverData obsDat = new ObserverData(o, arg);
    obs.removeElement(obsDat);
    obs.addElement(obsDat);
  }

  /**
   * Remove um observador remoto do conjunto de observadores desse objeto.
   * 
   * @param o o obsevador remoto a ser removido
   * @param arg o argumento do observador
   * @return <code>true</code> se o observador foi removido
   */
  public synchronized boolean deleteObserver(RemoteObserver o, Object arg) {
    return obs.removeElement(new ObserverData(o, arg));
  }

  /**
   * Remove um observador remoto do conjunto de observadores desse objeto.
   * 
   * @param od dados do observador remoto (observador + argumento)
   * @return <code>true</code> se o observador foi removido
   */
  public synchronized boolean deleteObserver(ObserverData od) {
    return obs.removeElement(od);
  }

  /**
   * Envia uma notificao para todos os observadores registrados quando o
   * objeto tiver alguma mudana indicada pelo mtodo <code>hasChanged</code>.
   * Aps a notificao ser enviada, invoca o mtodo <code>clearChanged</code>
   * para indicar que a mudana j foi notificada.
   * 
   * <p>
   * Cada observador  notificado atravs da chamada ao mtodo
   * <code>update</code> que recebe como parmetro um argumento para indicar
   * qual a mudana ocorrida.
   * </p>
   * 
   * @param event um argumento qualquer para a notificao de mudana
   * 
   * @see csbase.remote.RemoteObserver#update(java.lang.Object)
   */
  public synchronized void notifyObservers(final Object event) {
    if (!changed) {
      return;
    }
    clearChanged();
    new Thread(new Runnable() {
      public void run() {
        /*
         * mapa com os observadores que no foram notificados e devem ser
         * removidos. As chaves so os observadores e os valores as mensagens de
         * erro
         */
        Map<ObserverData, String> observersToRemove =
          new HashMap<ObserverData, String>();

        for (ObserverData od : obs) {
          RemoteObserver observer = od.observer;
          Object arg = od.arg;
          try {
            if (has2Update(arg, event)) {
              observer.update(event);
            }
          }
          catch (ConnectException ce) {
            observersToRemove.put(od, ce.getMessage());
          }
          catch (NoSuchObjectException nsoe) {
            observersToRemove.put(od, nsoe.getMessage());
          }
          catch (Exception ex) {
            System.err.print("Erro na notificao de observador: "
              + ex.toString());
            String msg = ex.getMessage();
            if (msg != null) {
              System.err.print(" >> " + msg);
            }
            System.err.println();
            // FIXME no deveramos remover o observador neste caso tambm?
          }
        }
        /*
         * se havia observadores a serem removidos, devemos faz-lo agora
         */
        for (Map.Entry<ObserverData, String> mapEntry : observersToRemove
          .entrySet()) {
          ObserverData od = mapEntry.getKey();
          // FIXME isso no deveria ser logado?
          System.err.println("Erro na notificao de observador. Erro: "
            + mapEntry.getValue());
          try {
            System.err.println("Removendo o observador...");
            deleteObserver(od);
          }
          catch (Exception re) {
            System.err.println("Erro na remoo de observador: "
              + re.getMessage());
          }
        }
      }
    }).start();
  }

  /**
   * Mtodo para indicar se o observador deve ser notificado.
   * 
   * @param arg Argumento do Observador
   * @param event Evento gerado para notificar o observador
   * 
   * @return True quando o observador deve ser notificado. False caso contrrio
   */
  protected abstract boolean has2Update(Object arg, Object event);

  /**
   * Remove todos os observadores registrados nesse objeto.
   */
  public synchronized void deleteObservers() {
    obs.removeAllElements();
  }

  /**
   * Marca que esse objeto sofreu alguma mudana. Consequentemente, o mtodo
   * <code>hasChanged</code> vai retornar <tt>true</tt>.
   */
  protected synchronized void setChanged() {
    changed = true;
  }

  /**
   * Indica que esse objeto no tem mais mudanas, ou seja, j notificou os
   * observadores das mudanas recentementemente efetuadas. Consequentemente, o
   * mtodo <code>hasChanged</code> passa a retornar <tt>false</tt>. O mtodo
   * <code>clearChanged</code>  chamado automaticamente pelos mtodos
   * <code>notifyObservers</code>.
   */
  protected synchronized void clearChanged() {
    changed = false;
  }

  /**
   * Verifica se esse objeto sofreu alguma mudana.
   * 
   * @return <tt>true</tt> se o mtodo <code>setChanged</code> foi chamado mais
   *         recentemente que o mtodo <code>clearChanged</code>; ou
   *         <tt>false</tt> no caso contrrio.
   */
  public synchronized boolean hasChanged() {
    return changed;
  }

  /**
   * Retorna o nmero de observadores registrados nesse objeto.
   * 
   * @return o nmero de observadores
   */
  public synchronized int countObservers() {
    return obs.size();
  }

  /**
   * Constri um objeto observvel
   */
  public RemoteObjectObservable() {
    obs = new Vector<ObserverData>();
    changed = false;
  }
}
