package csbase.remote.niotransfer;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.channels.FileChannel;
import java.nio.channels.SocketChannel;
import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;
import java.nio.charset.CharsetEncoder;

import tecgraf.javautils.configurationmanager.Configuration;
import tecgraf.javautils.configurationmanager.ConfigurationManager;
import tecgraf.javautils.configurationmanager.ConfigurationManagerException;
import csbase.exception.ConfigurationException;

/**
 * Classe responsvel por encapsular operaes de I/O de baixo nvel, abstraindo
 * essas tarefas das classes responsveis por implementar a semntica do 
 * protocolo de transmisso. 
 *
 * @author Tecgraf
 */
public class IOHandler {
  /** Codificao para transmisso de caracteres */
  private static final String ENCODING = "ISO-8859-1";
  /** Tamanho de um tipo byte em bytes. */
  static final int BYTE_SIZE = 1;
  /** Tamanho de um inteiro em bytes. */
  static final int INT_SIZE = 4;
  /** Tamanho de um tipo longo em bytes. */
  static final int LONG_SIZE = 8;
  /** Capacidade do buffer de controle da transmisso em bytes. */
  private static final int DEFAULT_BUFFER_CAPACITY = 1024;

  /** Socket de comunicao com outra JVM (cliente ou servidor). */
  private SocketChannel socketChannel;
  /** Buffer para transmisso em canais (java.nio). */
  private ByteBuffer nioBuffer;
  /** Buffer default para transmisso em canais (java.nio). */
  private ByteBuffer defaultNioBuffer;  
  /** Stream de leitura em arquivo local. */
  private InputStream in;  
  /** Stream de escrita em arquivo local. */
  private FileOutputStream fileOut;
  /** Canal para um arquivo local. */
  private FileChannel fileChannel;
  /** Total de bytes recebidos do arquivo at o momento. */
  private long bytesReceivedFromFile;
  /** Total de bytes recebidos do <b>pacote</b> at o momento. */
  private long bytesReceivedFromPack;
  /** Bytes do pacote remanescentes (a serem transmitidos ainda). */
  private long remainingPack;
  /** Bytes no buffer a serem transmitidos. */
  private int bytesToSend;
  /** 
   * Buffer composto por um array de bytes, para leitura das classes do java.io,
   * como <code>InputStream</code>. 
   */
  private byte[] ioBuffer;
  
  /** 
   * Contador de tempo para falsas leituras consecutivas (leituras de 0 bytes).
   * Se as falsas leituras no ultrapassarem o tempo de timeout, elas so
   * consideradas apenas atraso na rede; caso contrrio, a transferncia 
   * interrompida. 
   */
  private long startTimeFalseReads;
  
  /** 
   * Prazo mximo (em milissegundos) para leituras falsas consecutivas 
   * (0 bytes).
   */
  private long falseReadsTimeout;

  /**
   * Instancia um objeto para gerencia de operaes de I/O de baixo nvel,
   * indicando o canal de comunicao com outra JVM (cliente/servidor).
   * 
   * @param channel canal de comunicao com outra JVM (cliente/servidor).
   * @param capacity capacidade do buffer de transmisso do contedo.
   */
  public IOHandler(SocketChannel channel, int capacity) {
    this.socketChannel = channel;
    this.defaultNioBuffer = ByteBuffer.allocate(DEFAULT_BUFFER_CAPACITY);
    this.nioBuffer = defaultNioBuffer;
    this.ioBuffer = new byte[capacity];
    this.startTimeFalseReads = 0;
    try {
      Configuration configuration = 
        ConfigurationManager.getInstance().getConfiguration(getClass());
      String value = 
        configuration.getOptionalProperty("false.reads.timeout", "15000");
      this.falseReadsTimeout = Long.parseLong(value);
    } 
    catch (ConfigurationManagerException e) {
      throw new ConfigurationException(e);
    }
  }

  /**
   * Tenta ler um byte do canal remoto.
   * 
   * @return o byte lido ou -1 se nada tiver sido recebido.
   * 
   * @throws IOException
   */
  public byte readByte() throws IOException {
    if (nioBuffer.position() == 0) {
      nioBuffer.limit(BYTE_SIZE);
    }
    int bytesRead = socketChannel.read(nioBuffer);
    if (bytesRead == -1 ) {
      throw new IOException("O Canal foi fechado!!!");
    }
    if (nioBuffer.remaining() == 0) {
      nioBuffer.flip();
      byte result = nioBuffer.get();
      nioBuffer.clear();
      return result;
    }
    return -1;
  }  

  /**
   * Tenta ler um inteiro do canal remoto.
   * 
   * @return o inteiro lido ou -1 se no tiverem sido lidos bytes suficientes 
   *         para completar um inteiro.
   * 
   * @throws IOException
   */
  public int readInt() throws IOException {
    if (nioBuffer.position() == 0) {
      nioBuffer.limit(INT_SIZE);
    }
    int bytesRead = socketChannel.read(nioBuffer);
    if (bytesRead == -1 ) {
      throw new IOException("O Canal foi fechado!!!");
    }
    if (nioBuffer.remaining() == 0) {
      nioBuffer.flip();
      int result = nioBuffer.getInt();
      nioBuffer.clear();
      return result;
    }
    return -1;
  }

  /**
   * Tenta ler uma <code>String</code> do canal remoto, do tamanho especificado.
   * O mtodo assume que a <code>String</code> transmitida foi codificada
   * usando o mesmo cdigo com que ele tenta decodific-la.
   * 
   * @param size comprimento da <code>String</code> a ser lida.
   * 
   * @return o inteiro lido ou null se no tiverem sido lidos bytes suficientes 
   *         para completar um inteiro.
   * 
   * @throws IOException
   * 
   * @see #ENCODING
   */  
  public String readString(int size) throws IOException {
    if (size > DEFAULT_BUFFER_CAPACITY) {
      throw new IllegalArgumentException("size > BUFFER_CAPACITY");
    }
    if (nioBuffer.position() == 0) {
      nioBuffer.limit(size);
    }
    int bytesRead = socketChannel.read(nioBuffer);
    if (bytesRead == -1 ) {
      throw new IOException("O Canal foi fechado!!!");
    }
    if (nioBuffer.remaining() == 0) {
      nioBuffer.flip();
      Charset charset = Charset.forName(ENCODING);
      CharsetDecoder decoder = charset.newDecoder();
      CharBuffer charBuffer = decoder.decode(nioBuffer);
      String result = charBuffer.toString();
      nioBuffer.clear();
      return result;
    }
    return null;
  }

  /**
   * Abre uma <i>stream</i> de leitura para um arquivo, o qual ser a origem 
   * para a transferncia.
   * 
   * @param file referncia lgica para o arquivo.
   * 
   * @throws FileNotFoundException
   */
  public void createSource(File file) throws FileNotFoundException {
    in = new FileInputStream(file);
  }

  /**
   * Define a <code>InputStream</code> que servir de origem para a 
   * transferncia.
   * 
   * @param inputStream <code>InputStream</code> de onde sero lidos os dados a
   *        serem transferidos.
   */
  public void setSource(InputStream inputStream) {
    in = inputStream;
  }  
  
  /**
   * Cria um arquivo para servir de destino para a transferncia, abrindo um 
   * canal de escrita para ele. OBS: Se o arquivo j existir, ser sobrescrito.
   * 
   * @param file referncia lgica para o arquivo a ser criado.
   * 
   * @throws FileNotFoundException
   */
  public void createTarget(File file) throws FileNotFoundException {
    fileOut = new FileOutputStream(file);
    fileChannel = fileOut.getChannel();      
  }

  /**
   * Define a <code>OutputStream</code> que servir de destino para a 
   * transferncia, abrindo um canal de escrita para ele. Se o arquivo existir,
   * ser sobrescrito.
   * 
   * @param outputStream <code>OutputStream</code> onde sero escritos os dados 
   *        a serem transferidos.
   */
  public void setTarget(Object outputStream) {
    fileOut = (FileOutputStream)outputStream;
    fileChannel = fileOut.getChannel();
  }  
  
  /**
   * Tenta ler um <code>Long</code> do canal de leitura. Se no conseguir, deixa
   * o resultado intermedirio armazenado no buffer e retorna -1. Na prxima
   * chamada, concatena os bytes j lidos  nova leitura, repetindo este ciclo
   * at conseguir retornar um <code>Long</code>.
   * 
   * @return um nmero inteiro positivo ou -1 caso no tenha conseguido ler um
   *         nmero completo.
   * 
   * @throws IOException
   */
  public long readLong() throws IOException {
    if (nioBuffer.position() == 0) {
      nioBuffer.limit(LONG_SIZE);
    }
    int bytesRead = socketChannel.read(nioBuffer);
    if (bytesRead == -1 ) {
      throw new IOException("O Canal foi fechado!!!");
    }
    if (nioBuffer.remaining() == 0) {
      nioBuffer.flip();
      long result = nioBuffer.getLong();
      nioBuffer.clear();
      return result;
    }
    return -1;
  }

  /**
   * Tenta ler um <b>pacote</b> do canal remoto. Concatena esse <b>pacote</b>
   * ao contedo do arquivo sendo escrito no servidor.
   * 
   * @param fileSize tamanho do arquivo.
   * @param packSize tamanho do <b>pacote</b> sendo transmitido.
   * 
   * @return nmero de bytes lidos de um pacote de dados desde que iniciou a
   *         leitura (o objeto mantm o estado entre sucessivas invocaes deste
   *         mtodo) ou -1 caso o pacote no tenha sido inteiramente lido ainda.
   * 
   * @throws IOException
   */
  public long readPack(long fileSize, int packSize) throws IOException {
    // OBS: Enquanto houver o bug #6253145 da Sun, este mtodo s vai retornar 
    // int
    long bytesRead = fileChannel.transferFrom(
      socketChannel, bytesReceivedFromFile, packSize - bytesReceivedFromPack);
    checkFalseReads(bytesRead);
    bytesReceivedFromFile += bytesRead;
    bytesReceivedFromPack += bytesRead;
    if (bytesReceivedFromPack < packSize) {
      return -1;
    }
    long result = bytesReceivedFromPack;
    if (bytesReceivedFromFile == fileSize) {
      bytesReceivedFromFile = 0;
    }
    if (bytesReceivedFromPack == packSize) {
      bytesReceivedFromPack = 0;
    }
    return result;
  }

  /**
   * Verifica o nmero de bytes lidos e responde de acordo:
   * <ul>
   * <li>Se for negativo, significa que o canal foi fechado. Dispara uma 
   * exceo;</li>
   * <li>Se for zero, foi uma falsa leitura; verifica ento se o prazo mximo
   * para falsas leituras consecutivas foi atingido. Se for, dispara uma 
   * exceo;</li>
   * <li>Se for positivo, significa que est tudo ok; mesmo que tenham ocorrido
   * falsas leituras anteriormente, elas sero ignoradas, assumindo que houve
   * apenas um atraso na rede.</li>
   * </ul> 
   * 
   * @param bytesRead nmero de bytes lidos na ltima operao.
   * 
   * @throws IOException se o nmero de bytes for negativo ou se o prazo mximo
   *         para falsas leituras tiver se esgotado.
   */
  private void checkFalseReads(long bytesRead) throws IOException {
    if (bytesRead < 0 ) {
      throw new IOException("O Canal foi fechado!!!");
    }
    if (bytesRead > 0) {
      startTimeFalseReads = 0;
      return;
    }
    if (startTimeFalseReads == 0) {
      startTimeFalseReads = System.currentTimeMillis();
      return;
    }
    if ((System.currentTimeMillis() - startTimeFalseReads) > falseReadsTimeout) {
      throw new IOException("Timeout atingido para falsas leituras (0 bytes).");
    }
  }

  
  /**
   * Libera os recursos fsicos alocados por este objeto.
   *
   * @throws IOException
   */
  public void closeResources() throws IOException {
    if (in != null) in.close();
    if (fileOut != null) fileOut.close();
    if (fileChannel != null) fileChannel.close();
  }

  /**
   * Escreve um byte no canal remoto. Se houver algo no buffer, ignora o 
   * parmetro e tenta transmitir o contedo do buffer (para o caso da primeira
   * chamada ao mtodo no ter transmitido o byte).
   * 
   * @param data byte a ser escrito.
   * 
   * @return <code>true</code> se o byte (mesmo que o mtodo precise ter sido 
   *         chamado vrias vezes) tiver sido enviado nessa chamada.
   * 
   * @throws IOException
   */
  public boolean writeByte(byte data) throws IOException {
    if (nioBuffer.position() == 0) {
      // No existe nada no buffer a ser transmitido
      nioBuffer.clear();
      nioBuffer.put(data);
      nioBuffer.flip();
    }  
    return writeBuffer();
  }

  /**
   * Escreve um inteiro no canal remoto. Se houver algo no buffer, ignora o 
   * parmetro e tenta transmitir o contedo do buffer (para o caso da primeira
   * chamada ao mtodo no ter transmitido o inteiro).
   * 
   * @param data inteiro a ser escrito.
   * 
   * @return <code>true</code> se o inteiro (mesmo que o mtodo
   *         precise ter sido chamado vrias vezes) tiver sido integralmente 
   *         transmitido.
   * 
   * @throws IOException
   */
  public boolean writeInt(int data) throws IOException {
    if (nioBuffer.position() == 0) {
      // No existe nada no buffer a ser transmitido
      nioBuffer.clear();
      nioBuffer.putInt(data);
      nioBuffer.flip();
    }  
    return writeBuffer();
  }

  /**
   * Escreve uma <code>String</code> no canal remoto, codificada de acordo com o
   * padro definido nesta classe ({@link #ENCODING}). Se houver algo no buffer, 
   * ignora o parmetro e tenta transmitir o contedo do buffer (para o caso da 
   * primeira chamada ao mtodo no ter transmitido a <code>String</code>).
   * 
   * @param data <code>String</code> a ser transmitida.
   * 
   * @return <code>true</code> se a <code>String</code> (mesmo que o mtodo
   *         precise ter sido chamado vrias vezes) tiver sido integralmente 
   *         transmitida.
   * 
   * @throws IOException
   */
  public boolean writeString(String data) throws IOException {
    if (nioBuffer.position() == 0) {
      // No existe nada no buffer a ser transmitido
      CharBuffer charBuffer = CharBuffer.wrap(data.toCharArray());
      Charset charset = Charset.forName(ENCODING);
      CharsetEncoder encoder = charset.newEncoder();
      nioBuffer = encoder.encode(charBuffer);
    }  
    return writeBuffer();
  }

  /**
   * Escreve um tipo longo no canal remoto. Se houver algo no buffer, ignora o 
   * parmetro e tenta transmitir o contedo do buffer (para o caso da primeira
   * chamada ao mtodo no ter transmitido o tipo longo).
   * 
   * @param data tipo longo a ser escrito.
   * 
   * @return <code>true</code> se o tipo longo (mesmo que o mtodo
   *         precise ter sido chamado vrias vezes) tiver sido integralmente 
   *         transmitido.
   * @throws IOException
   */
  public boolean writeLong(long data) throws IOException {
    if (nioBuffer.position() == 0) {
      // No existe nada no buffer a ser transmitido
      nioBuffer.clear();
      nioBuffer.putLong(data);
      nioBuffer.flip();
    }  
    return writeBuffer();
  }
  
  /**
   * Transmite o contedo do buffer no canal remoto.
   * 
   * @return <code>true</code> se o contedo do buffer tiver sido integralmente
   *         transmitido.
   *         
   * @throws IOException caso ocorra algum problema ao escrever no canal remoto.
   */
  private boolean writeBuffer() throws IOException {
    int bytesSent = socketChannel.write(nioBuffer);
    if (bytesSent == -1 ) {
      throw new IOException("O Canal foi fechado!!!");
    }
    boolean allSent = !nioBuffer.hasRemaining();
    if (allSent) {
      nioBuffer = defaultNioBuffer;
      nioBuffer.clear();
    }
    return allSent;
  }

  /**
   * <p>
   * L um pacote de um <code>InputStream</code> e o escreve no canal remoto.
   * </p>
   * <p>
   * Se houver algo no buffer, tenta transmitir o contedo do buffer 
   * (para o caso da primeira chamada ao mtodo no ter transmitido todo o 
   * pacote). Quando o buffer for esvaziado, verifica se ainda existe parte do
   * pacote a ser transmitida e assim prossegue at que todo o pacote tenha
   * sido enviado.
   * </p>
   * NOTA DE IMPLEMENTAO: O mtodo {@link #createSource(File)} precisa
   * ter sido chamado previamente ao uso deste mtodo para informar o
   * arquivo que ser transmitido.
   * 
   * @param packSize nmero de bytes a serem transmitidos (tamanho do pacote).
   *        
   * @return <code>true</code> se o pacote (mesmo que o mtodo
   *         precise ter sido chamado vrias vezes) tiver sido integralmente 
   *         transmitido.
   *         
   * @throws IOException
   */
  public boolean writePack(long packSize) 
  throws IOException {
    if (packSize < 0) {
      throw new IllegalStateException("packSize < 0");
    }    
    if (packSize == 0) {
      return true;
    }
    if (in == null) {
      throw new IllegalStateException("fileIn == null");
    }
    if (remainingPack == 0) {
      remainingPack = packSize;
    }
    while (remainingPack > 0) {
      if (bytesToSend == 0) {
        bytesToSend = in.read(ioBuffer);
        if (bytesToSend == 0) {
          throw new IOException("Erro: o arquivo de origem est vazio.");
        }
        if (bytesToSend < 0) {
          throw new IllegalStateException("in: EOF");
        }
        nioBuffer = ByteBuffer.wrap(ioBuffer);
        nioBuffer.limit(bytesToSend);
      }
      int bytesSent = socketChannel.write(nioBuffer);
      if (bytesSent == -1) {
        throw new IOException("O Canal foi fechado!!!");
      }
      remainingPack -= bytesSent;
      bytesToSend -= bytesSent;
      if (nioBuffer.hasRemaining()) {
        return false;
      }
      nioBuffer.clear();
    }
    if (remainingPack == 0) {
      nioBuffer = defaultNioBuffer;
      return true;
    }
    throw new IllegalStateException(
      "Transmitidos "+(-remainingPack)+" bytes alm do esperado.");
  }

  /**
   * Retorna o nmero de bytes ainda disponveis para leitura no arquivo local.
   * 
   * @return nmero de bytes ainda disponveis para leitura no arquivo local.
   * 
   * @throws IOException
   */
  public int availableForReading() throws IOException {
    return in.available();
  }

  /**
   * Retorna a capacidade do buffer de transmisso do contedo do arquivo.
   * 
   * @return capacidade do buffer de transmisso do contedo do arquivo.
   */
  public int getContentBufferSize() {
    return ioBuffer.length;
  }

  /**
   * Mtodo utilizado para testes. Obtm referncia para o buffer de NIO.
   * 
   * @return buffer de NIO.
   */
  ByteBuffer getNioBuffer() {
    return nioBuffer;
  }

  /**
   * Mtodo utilizado para testes. Redefine o tamanho do buffer de contedo.
   * 
   * @param size novo tamanho para o buffer.
   */
  void setContentBufferSize(int size) {
    ioBuffer = new byte[size];
  }

  /**
   * Mtodo utilizado para testes. Obtm o nmero de bytes do pacote 
   * remanescentes a serem transferidos.
   * 
   * @return nmero de bytes remanescentes do pacote.
   */
  long getRemainingPack() {
    return remainingPack;
  }

  /**
   * Mtodo utilizado para testes. Obtm o nmero de bytes lidos na ltima
   * leitura pendentes para serem transmitidos.
   * 
   * @return nmero de bytes a serem transmitidos da ltima leitura.
   */
  int getBytesToSend() {
    return bytesToSend;
  }
  
  /**
   * Mtodo utilizado para testes. Obtm a capacidade default do buffer de
   * transmisso do contedo.
   * 
   * @return capacidade default do buffer.
   */
  static int getDefaultBufferCapacity() {
    return DEFAULT_BUFFER_CAPACITY;
  }

}
