/*
 * FileSystem.java
 * 
 * $Id: FileSystem.java 91491 2009-05-13 22:07:44Z clinio $
 */
package csbase.server;

import java.io.File;
import java.io.IOException;
import java.util.StringTokenizer;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import csbase.logic.ExternalCommand;

/**
 * Representa o sistema de arquivos do SSI.
 */
public class FileSystem {
  /**
   * Comando para cpia de arquivos.
   */
  private static final String CP_COMMAND = "cp";

  /**
   * Comando para criao de links simblicos.
   */
  private static final String SYMLINK_COMMAND = "ln -s";

  /**
   * Diretrio raiz de projetos.
   */
  public static final String PROJECT_DIR = "project";

  /**
   * Diretrio raiz de publicao.
   */
  public static final String PUBLISH_DIR = "published";

  /**
   * Diretrio raiz de dados globais.
   */
  public static final String GLOBAL_DIR = "global";

  /**
   * Sufixo de link utilizado pelo CYGWIN.
   */
  public static final String LINK_SUFFIX = ".lnk";

  /**
   * Diretrio raiz dos arquivos de propriedades.
   */
  public static final String PROPERTIES_DIR = "properties";

  /**
   * Verifica o espao de um determinado filesystem.
   * 
   * @param path O caminho do filesystem.
   * @param group nmero do grupo a ser capturado no resultado do comando que
   *        obtm o espao em disco (total, livre, em uso)
   * @return O nmero de bytes disponveis. -1 em caso de erro.
   */
  public static long getSpace(String path, int group) {
    final Server server = Server.getInstance();
    try {
      final String command = server.getStringProperty("DiskSpace.command");
      final String[] cmdArray = createCommandArray(command, path);
      final ExternalCommand process = new ExternalCommand(cmdArray);
      /*
       * Verifica se o comando foi executado com sucesso. Caso no tenha sido,
       * grava o erro no log.
       */
      if (process.waitFor() != 0) {
        final String errorOutput = process.getErrorOutput();
        server.logSevereMessage("Erro obtendo espao:\n"
          + (errorOutput != null ? errorOutput : "---"));
        return -1;
      }
      final String standardOutput = process.getStandardOutput();
      final String regex = server.getStringProperty("DiskSpace.regex");
      final Pattern pattern = Pattern.compile(regex);
      final Matcher matcher = pattern.matcher(standardOutput);
      if (matcher.find()) {
        final String strKbytes = matcher.group(group);
        long bytes = Long.parseLong(strKbytes) * 1024;
        return bytes;
      }
      return -1;
    }
    catch (Exception e) {
      server.logSevereMessage("Erro obtendo espao", e);
      return -1;
    }
  }

  /**
   * Verifica o espao disponvel para um determinado filesystem. O mtodo
   * File.getUsableSpace no funciona para filesystems maiores de 5Tb (bug
   * cadastrado na Sun). Para sistemas nessa situao, a propriedade
   * FreeDiskSpace.group deve ser definida no arquivo Server.properties de forma
   * que o espao em disco seja obtido atravs de uma chamada direta ao sistema.
   * 
   * @param path O caminho do filesystem
   * @return O nmero de bytes disponveis. -1 em caso de erro.
   */
  public static long getFreeSpace(String path) {
    final Server server = Server.getInstance();
    try {
      final String propertyName = "FreeDiskSpace.group";
      final boolean isNull = server.isPropertyNull(propertyName);
      if (isNull) {
        final File f = new File(path);
        if (f.exists()) {
          return f.getUsableSpace();
        }
        return -1;
      }
      final int group = server.getIntProperty(propertyName);
      return getSpace(path, group);
    }
    catch (Exception e) {
      server.logSevereMessage("Erro obtendo espao", e);
      return -1;
    }
  }

  /**
   * Verifica o espao total para um determinado filesystem. O mtodo
   * File.getTotalSpace no funciona para filesystems maiores de 5Tb (bug
   * cadastrado na Sun). Para sistemas nessa situao, a propriedade
   * TotalDiskSpace.group deve ser definida no arquivo Server.properties de
   * forma que o espao em disco seja obtido atravs de uma chamada direta ao
   * sistema.
   * 
   * @param path O caminho do filesystem.
   * @return O nmero de bytes disponveis. -1 em caso de erro.
   */
  public static long getTotalSpace(String path) {
    final Server server = Server.getInstance();
    try {
      final String propertyName = "TotalDiskSpace.group";
      final boolean isNull = server.isPropertyNull(propertyName);
      if (isNull) {
        final File f = new File(path);
        if (f.exists()) {
          return f.getTotalSpace();
        }
        return -1;
      }

      final int group = server.getIntProperty(propertyName);
      return getSpace(path, group);
    }
    catch (Exception e) {
      server.logSevereMessage("Erro obtendo espao", e);
      return -1;
    }
  }

  /**
   * Cria um link para um arquivo existente. Usa o comando Unix "ln -s" para
   * criao de link simblico.
   * 
   * @param real O objeto <code>File</code> que representa o arquivo real.
   * @param link O objeto<code>File</code> que representa o arquivo de link a
   *        ser criado.
   * 
   * @return Verdadeiro, se o link foi criado com sucesso. Falso caso o arquivo
   *         de link j exista, caso o arquivo real no exista ou caso tenha
   *         ocorrido erro na criao do link.
   */
  public static boolean createLink(File real, File link) {
    Server.getInstance().logInfoMessage(
      "FileSystem.createLink: de: " + real.getAbsolutePath() + "  para: "
        + link.getAbsolutePath());

    /* Testa se o arquivo real existe */
    if (!real.exists()) {
      Server.getInstance().logSevereMessage(
        "FileSystem.createLink: arquivo real no existe: "
          + real.getAbsolutePath());
      return false;
    }

    /* Testa se o arquivo de link no existe */
    if (link.exists()) {
      Server.getInstance().logSevereMessage(
        "FileSystem.createLink: link j existe: " + link.getAbsolutePath());
      return false;
    }
    try {
      String[] cmdArray =
        createCommandArray(SYMLINK_COMMAND, real.getAbsolutePath(), link
          .getAbsolutePath());
      Runtime.getRuntime().exec(cmdArray);
    }
    catch (IOException e) {
      Server.getInstance().logSevereMessage(
        "FileSystem.createLink: Erro na criao de link", e);
      return false;
    }
    return true;
  }

  /**
   * Move um arquivo para o outro. A implementao deste mtodo copia e depois
   * remove o original.
   * 
   * @param from O arquivo de origem.
   * @param to O arquivo destino.
   * @return Verdadeiro se foi movido com sucesso ou falso, caso contrrio.
   */
  public static boolean moveRegularFile(File from, File to) {
    Server.getInstance().logInfoMessage(
      "FileSystem.moveRegularFile:  de: " + from.getAbsolutePath() + "  para: "
        + to.getAbsolutePath());

    if (!from.exists()) {
      Server.getInstance().logSevereMessage(
        "FileSystem.moveRegularFile: arquivo origem no existe: "
          + from.getAbsolutePath());
      return false;
    }
    if (!copyFile(from, to)) {
      return false;
    }
    if (from.delete())
      return true;
    Server.getInstance().logSevereMessage(
      "FileSystem.moveRegularFile: Erro removendo arquivo origem: "
        + from.getAbsolutePath());
    return false;
  }

  /**
   * Move arquivos ou diretrios.
   * 
   * @param from arquivo ou diretrio de origem
   * @param to arquivo ou diretrio de destino
   * @return Verdadeiro se a movimentao foi realizada com sucesso, Falso caso
   *         contrrio
   */
  public static boolean move(File from, File to) {
    if (!from.isDirectory()) {
      return moveRegularFile(from, to);
    }
    if (!to.mkdirs()) {
      Server.getInstance().logSevereMessage(
        "FileSystem.move: Erro na criando diretrio(s) : "
          + to.getAbsolutePath());
      return false;
    }
    File[] children = from.listFiles();
    boolean success = true;
    for (int i = 0; i < children.length; i++) {
      File newChildFile = new File(to, children[i].getName());
      success = move(children[i], newChildFile);
      if (!success) {
        break;
      }
    }
    if (success) {
      if (from.delete())
        return true;
      Server.getInstance().logSevereMessage(
        "FileSystem.move: Erro removendo diretrio origem: "
          + from.getAbsolutePath());
    }
    return success;
  }

  /**
   * Copia o contedo de um arquivo para outro.
   * 
   * @param from O arquivo de origem.
   * @param to O arquivo destino.
   * 
   * @return Verdadeiro se a cpia foi feita com sucesso ou falso, caso
   *         contrrio.
   */
  public static boolean copyFile(File from, File to) {
    Server.getInstance().logInfoMessage(
      String.format("FileSystem.copyFile: de '%s' para '%s'", from
        .getAbsolutePath(), to.getAbsolutePath()));
    String[] cmdArray =
      createCommandArray(CP_COMMAND, from.getAbsolutePath(), to
        .getAbsolutePath());

    String cmd =
      String.format(
        "FileSystem:copyFile: Erro copiando arquivo (%s '%s' '%s')",
        cmdArray[0], cmdArray[1], cmdArray[2]);
    try {
      ExternalCommand command = new ExternalCommand(cmdArray);
      if (command.waitFor() != 0) {
        String errorOutput = command.getErrorOutput();
        Server.getInstance().logSevereMessage(
          cmd + " :\n" + (errorOutput != null ? errorOutput : "---"));
        return false;
      }
    }
    catch (InterruptedException e) {
      // Espera no deve ser interrompida...
      Server.getInstance().logSevereMessage(cmd, e);
      return false;
    }
    catch (IOException e) {
      Server.getInstance().logSevereMessage(cmd, e);
      return false;
    }
    return true;
  }

  /**
   * <p>
   * Habilita permisso de execuo para o arquivo especificado.
   * </p>
   * Este mtodo  especfico para cada sistema operacional, e portanto usa a
   * constante <code>ENABLE_EXECUTION_PERMISSION_COMMAND</code> para descobrir
   * qual o comando a ser executado, a qual por sua vez faz referncia a uma
   * propriedade em arquivo.<br>
   * Caso esse mtodo no faa sentido para o sistema operacional operacional
   * corrente, deixe essa propriedade em branco.<br>
   * OBS: O mtodo  bloqueante, isto , a thread de execuo ir aguardar at
   * que o processo termine para retornar.
   * 
   * @param filePath caminho para o arquivo no NFS.
   * 
   * @return <tt>true</tt> se a operao foi bem sucedida.
   * 
   * @throws ServerException caso no seja possvel executar o comando.
   */
  public static final boolean enableExecutionPermission(String filePath)
    throws ServerException {

    if (filePath == null) {
      throw new IllegalArgumentException("filePath == null");
    }
    if (filePath.equals("")) {
      throw new IllegalArgumentException("filePath est vazia");
    }

    try {
      return new File(filePath).setExecutable(true, false);
    }
    catch (Exception e) {
      // Espera no deve ser interrompida...
      throw new ServerException("Erro ao mudar execuo do arquivo '"
        + filePath + "'", e);
    }
  }

  /**
   * Cria um array de strings para ser usado pelo comando
   * {@link Runtime#exec(String[])}.
   * <p>
   * Esta deve ser a forma prefervel de se criar linhas de comando ao invs da
   * criao da linha de comando como uma string, pois se fose feito desta forma
   * (uma nica string) o mtodo {@link Runtime#exec(String)} usaria
   * {@link StringTokenizer} para identificar os tokens, e espaos nos
   * parmetros gerariam tokens indesejados. Envolver os parmetros com aspas
   * no d o resultado esperado (no UNIX p.ex. as aspas so repassadas para o
   * comando, o que gera erro).
   * 
   * @param cmd - comando. Pode conter espaos; neste caso, o comando ser
   *        quebrado em tokens usando {@link StringTokenizer}. O primeiro token
   *        ser usado como comando, e os demais como parmetros, que sero
   *        inseridos antes dos especificados pelo parmetro <code>args</code>.
   * @param args - parmetros
   * @return array de strings ({cmd, args[0], args[1], ...})
   */
  private static String[] createCommandArray(String cmd, String... args) {
    final StringTokenizer cmdTokenizer = new StringTokenizer(cmd);
    final int numTokens = cmdTokenizer.countTokens();
    final String[] cmdArray = new String[args.length + numTokens];
    int i = 0;
    while (i < numTokens) {
      cmdArray[i++] = cmdTokenizer.nextToken();
    }
    System.arraycopy(args, 0, cmdArray, i, args.length);
    return cmdArray;
  }
}
