/*
 * $Id: Server.java 97827 2009-10-20 18:52:53Z costa $
 */
package csbase.server;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;
import java.text.MessageFormat;
import java.util.List;
import java.util.Locale;
import java.util.Properties;
import java.util.logging.ConsoleHandler;
import java.util.logging.FileHandler;
import java.util.logging.Level;
import java.util.logging.Logger;

import tecgraf.javautils.FormatUtils;
import tecgraf.javautils.configurationmanager.ConfigurationManager;
import csbase.exception.InitFailureException;
import csbase.logic.Version;
import csbase.remote.ServerEntryPoint;
import csbase.server.keystore.CSKeyStore;

/**
 * Classe que representa um servidor genrico.
 * 
 * @author Tecgraf/PUC-Rio
 */
public abstract class Server {
  /**
   * Chave usada para criptografar os arquivos de permisses.
   */
  private static final String ADMIN_PASSWORD = "aquaman";

  /**
   * Instncia do servidor
   */
  private static Server instance = null;

  /**
   * Interface remota do servidor.
   */
  protected Registry registry = null;

  /**
   * Log do servidor
   */
  final private Logger serverLogger;

  /**
   * Propriedades do servidor
   */
  private final ServerSideProperties serverProperties;

  /**
   * Propriedades do override do servidor (vinda da command-line).
   */
  private final ServerSideProperties systemProperties;

  /**
   * Nome do servidor. Por exemplo: InfoGrid, WebSintesi, MARLIM.
   */
  final private String serverName;

  /**
   * A senha da chave privada do servidor.
   */
  private final String privateKeyPassword;

  /**
   * Porta na qual o RMI registry aceita requisies.
   */
  private final int rmiPort;

  /**
   * Configura o endereo utilizado para exportar objetos RMI do servidor. *
   */
  private final String serverHostName;

  /**
   * Representa o ponto de entrada do servidor.
   */
  private ServerEntryPoint entryPoint;

  /**
   * Locale default do servidor
   */
  private final Locale defaultLocale;

  /**
   * Arquivo de sistema (configurao override) do servidor; que pode ser
   * <code>null</code> no caso de serem usadas as configuraes default do
   * sistema.
   */
  private String systemFileName = null;

  /**
   * Mtodo para checagem e criao de diretrios (se necessrio).
   * 
   * @param dirPath o caminho relativo para verificao.
   * 
   * @throws ServerException 1) se o caminho existir e no representar um
   *         diretrio; ou 2) se o caminho no existir e no for possvel criar
   *         um novo diretrio no mesmo.
   */
  public void checkDirectory(final String dirPath) throws ServerException {
    if (dirPath == null) {
      throw new IllegalArgumentException("dirPath == null");
    }
    final File directory = new File(dirPath);
    if (!directory.exists()) {
      if (directory.mkdirs()) {
        final String msg = "Criado o diretrio: " + dirPath + ". ";
        if (serverLogger == null) {
          System.out.println(msg);
        }
        else {
          logInfoMessage(msg);
        }
      }
      else {
        final String err = "Impossvel criar diretrio: " + dirPath + ".";
        throw new ServerException(err);
      }
    }
    else {
      if (!directory.isDirectory()) {
        final String err = "Diretrio invlido: [" + dirPath + "].";
        throw new ServerException(err);
      }
    }
  }

  /**
   * Verifica se comunicao com o registry.
   * 
   * @return true em caso haja comunicao, false caso contrrio.
   */
  private boolean checkRegistry() {
    try {
      registry.list();
      return true;
    }
    catch (final Throwable t) {
    }
    return false;
  }

  /**
   * Cria o repositrio de chaves do servidor.
   */
  private void createKeyStore() {
    /* Propriedade do caminho do repositrio de chaves do servidor */
    final String KS_PATH_PROPERTY = "keyStorePath";

    /* Propriedade da senha de chave do servidor */
    final String KS_PASSWORD_PROPERTY = "keyStorePassword";

    final boolean nullKeyStorePath = isPropertyNull(KS_PATH_PROPERTY);
    if (nullKeyStorePath) {
      final String fmt = "No foi encontrada a propriedade [%s].";
      final String message = String.format(fmt, KS_PATH_PROPERTY);
      logInfoMessage(message);
    }
    else {
      final String keyStorePath = getStringProperty(KS_PATH_PROPERTY);
      final boolean nullKeyStorePassword = isPropertyNull(KS_PASSWORD_PROPERTY);
      if (nullKeyStorePassword) {
        final String fmt = "No foi encontrada a propriedade [%s].";
        final String message = String.format(fmt, KS_PASSWORD_PROPERTY);
        this.logWarningMessage(message);
        CSKeyStore.createInstance(keyStorePath);
      }
      else {
        final String keyStorePassword = getStringProperty(KS_PASSWORD_PROPERTY);
        CSKeyStore.createInstance(keyStorePath, keyStorePassword);
      }
    }
  }

  /**
   * Cria a porta de entrada para acesso as funcionalidade do servidor, o
   * <code>ServerEntryPointImpl</code>.
   * 
   * @return o <code>ServerEntryPointImpl</code> que  a porta de entrada para
   *         acesso as funcionalidade do servido
   */
  protected ServerEntryPoint createServerEntryPoint() {
    return new ServerEntryPointImpl();
  }

  /**
   * Mtodo responsvel para criao dos servios. A partir deste mtodo, deve
   * ser chamado o mtodo <code>createService</code> de cada servio.
   * 
   * @throws ServerException caso ocorra falha na criao do servio.
   */
  public abstract void createServices() throws ServerException;

  /**
   * Retorna a senha do administrador.
   * 
   * @return string a senha.
   */
  public String getAdminPassword() {
    return ADMIN_PASSWORD;
  }

  /**
   * Obtm o nome do servidor central.
   * 
   * @return o nome do servidor central
   */
  public String getCentralServerName() {
    final String propertyName = "centralServer";
    final boolean isNull = isPropertyNull(propertyName);
    if (isNull) {
      return null;
    }
    final String centralServerName = getStringProperty(propertyName);
    return centralServerName;
  }

  /**
   * Retorna o {@code Locale} default configurado na instalao do servidor
   * CSBase.
   * 
   * @return O {@code Locale} ou {@code null} caso no haja um configurado.
   */
  public Locale getDefaultLocale() {
    return defaultLocale;
  }

  /**
   * Obtm o ponto de entrada do servidor.
   * 
   * @return o ponto de entrada.
   */
  public final ServerEntryPoint getEntryPoint() {
    return this.entryPoint;
  }

  /**
   * Retorna a mensagem de "boas-vindas" do sistema (exibida quando a
   * inicializao foi bem-sucedida). Pode ser redefinido por subclasses que
   * precisem ou queiram exibir outras informaes.
   * 
   * @return mensagem de boas-vindas do sistema
   */
  protected String getGreetingMessage() {
    return "Servidor do " + getServerName() + " iniciado em: "
      + FormatUtils.format(System.currentTimeMillis());
  }

  /**
   * Retorna o endereo utilizado para exportar objetos RMI do servidor.
   * 
   * @return O endereo utilizado para exportar objetos RMI do servidor.
   */
  public String getHostName() {
    return serverHostName;
  }

  /**
   * Retorna a porta na qual o RMI registry aceita requisies.
   * 
   * @return Porta na qual o RMI registry aceita requisies.
   */
  public int getPort() {
    return this.rmiPort;
  }

  /**
   * Obtm a senha da chave privada do servidor. S existir quando o servidor
   * for servidor local.
   * 
   * @return A senha da chave privada ou null, caso o servidor no seja servidor
   *         local ou se a senha no foi informada.
   */
  public String getPrivateKeyPassword() {
    return this.privateKeyPassword;
  }

  /**
   * Retorna o nome verdadeiro da chave com prefixo.
   * 
   * @param key a chave
   * @return a chave com prefixo (se houver)
   */
  final private String getRealPropertyKey(final String key) {
    final String realKey = "Server." + key;
    return realKey;
  }

  /**
   * Indica se o servidor faz override da propriedade de um servio
   * 
   * @param prefixedKey a chave (j prefixada pelo servio)
   * @return indicativo.
   */
  final protected boolean overridesServiceProperty(String prefixedKey) {
    final boolean inServer = serverProperties.hasProperty(prefixedKey);
    final boolean inSystem;
    if (systemProperties != null) {
      inSystem = systemProperties.hasProperty(prefixedKey);
    }
    else {
      inSystem = false;
    }
    return inServer || inSystem;
  }

  /**
   * Faz checagem de propriedades corretamente carregadas.
   */
  private void checkPropertiesLoaded() {
    if (serverProperties == null) {
      final String msg =
        "Falha interna de implementao do servidor!\n\n"
          + "Propriedades do servidor ainda no devidamente carregadas";
      throw new IllegalStateException(msg);
    }
  }

  /**
   * Recupera uma propriedade da aplicao (string)
   * 
   * @param key a identificao da propriedade
   * @return o valor da propriedade.
   * @throws IllegalStateException se a propriedade no estiver setada.
   * @see ServerSideProperties#getStringProperty(String)
   */
  final protected String getStringProperty(final String key) {
    return _propertyString(key, true);
  }

  /**
   * Recupera uma propriedade da aplicao (string)
   * 
   * @param key a identificao da propriedade
   * @return o valor da propriedade.
   * @throws IllegalStateException se a propriedade no estiver setada.
   * @see ServerSideProperties#getStringProperty(String)
   */
  final String getStringServiceProperty(final String key) {
    return _propertyString(key, false);
  }

  /**
   * Recupera uma propriedade da aplicao (double)
   * 
   * @param key a chave
   * @return o valor
   * @throws IllegalStateException se a propriedade no estiver setada.
   */
  protected final double getDoubleProperty(final String key) {
    return _propertyDouble(key, true);
  }

  /**
   * Recupera uma propriedade da aplicao (double)
   * 
   * @param key a chave
   * @return o valor
   * @throws IllegalStateException se a propriedade no estiver setada.
   */
  final double getDoubleServiceProperty(final String key) {
    return _propertyDouble(key, false);
  }

  /**
   * Recupera uma propriedade da aplicao (double)
   * 
   * @param key a chave
   * @param prefix indica que o tag da propriedade deve concatenar
   *        <code>"Server."</code>.
   * @return o valor
   * @throws IllegalStateException se a propriedade no estiver setada.
   */
  final private double _propertyDouble(final String key, final boolean prefix) {
    checkPropertiesLoaded();
    final String realKey = prefix ? getRealPropertyKey(key) : key;
    if (systemProperties != null && systemProperties.hasProperty(realKey)) {
      return systemProperties.getDoubleProperty(realKey);
    }
    return serverProperties.getDoubleProperty(realKey);
  }

  /**
   * Recupera uma propriedade da aplicao (string)
   * 
   * @param key a chave
   * @param prefix indica que o tag da propriedade deve concatenar
   *        <code>"Server."</code>.
   * @return o valor
   * @throws IllegalStateException se a propriedade no estiver setada.
   */
  final private String _propertyString(final String key, final boolean prefix) {
    checkPropertiesLoaded();
    final String realKey = prefix ? getRealPropertyKey(key) : key;
    if (systemProperties != null && systemProperties.hasProperty(realKey)) {
      return systemProperties.getStringProperty(realKey);
    }
    return serverProperties.getStringProperty(realKey);
  }

  /**
   * Recupera uma propriedade da aplicao (string)
   * 
   * @param key a chave
   * @param prefix indica que o tag da propriedade deve concatenar
   *        <code>"Server."</code>.
   * @return o valor
   * @throws IllegalStateException se a propriedade no estiver setada.
   */
  final private List<String> _propertyStringList(final String key,
    final boolean prefix) {
    checkPropertiesLoaded();
    final String realKey = prefix ? getRealPropertyKey(key) : key;
    if (systemProperties != null
      && systemProperties.hasProperty(realKey.concat(".1"))) {
      return systemProperties.getStringListProperty(realKey);
    }
    return serverProperties.getStringListProperty(realKey);
  }

  /**
   * Recupera uma propriedade da aplicao (int)
   * 
   * @param key a chave
   * @param prefix indica que o tag da propriedade deve concatenar
   *        <code>"Server."</code>.
   * @return o valor
   * @throws IllegalStateException se a propriedade no estiver setada.
   */
  final private int _propertyInt(final String key, final boolean prefix) {
    checkPropertiesLoaded();
    final String realKey = prefix ? getRealPropertyKey(key) : key;
    if (systemProperties != null && systemProperties.hasProperty(realKey)) {
      return systemProperties.getIntProperty(realKey);
    }
    return serverProperties.getIntProperty(realKey);
  }

  /**
   * Recupera uma propriedade da aplicao (int)
   * 
   * @param key a chave
   * @param prefix indica que o tag da propriedade deve concatenar
   *        <code>"Server."</code>.
   * @return o valor
   * @throws IllegalStateException se a propriedade no estiver setada.
   */
  final private boolean _propertyBool(final String key, final boolean prefix) {
    checkPropertiesLoaded();
    final String realKey = prefix ? getRealPropertyKey(key) : key;
    if (systemProperties != null && systemProperties.hasProperty(realKey)) {
      return systemProperties.getBooleanProperty(realKey);
    }
    return serverProperties.getBooleanProperty(realKey);
  }

  /**
   * Recupera uma propriedade da aplicao (int)
   * 
   * @param key a chave
   * @return o valor
   * @throws IllegalStateException se a propriedade no estiver setada.
   */
  final public int getIntProperty(final String key) {
    return _propertyInt(key, true);
  }

  /**
   * Recupera uma propriedade da aplicao (int)
   * 
   * @param key a chave
   * @return o valor
   * @throws IllegalStateException se a propriedade no estiver setada.
   */
  final int getIntServiceProperty(final String key) {
    return _propertyInt(key, false);
  }

  /**
   * Recupera uma propriedade da aplicao (boolean)
   * 
   * @param key a chave
   * 
   * @return o valor
   * @throws IllegalStateException se a propriedade no estiver setada.
   */
  final protected boolean getBooleanProperty(final String key) {
    return _propertyBool(key, true);
  }

  /**
   * Recupera uma propriedade da aplicao (int)
   * 
   * @param key a chave
   * @return o valor
   * @throws IllegalStateException se a propriedade no estiver setada.
   */
  final boolean getBooleanServiceProperty(final String key) {
    return _propertyBool(key, false);
  }

  /**
   * Recupera uma propriedade da aplicao (lista)
   * 
   * @param key a chave (prefixo)
   * @return o valor
   * @throws IllegalStateException se a propriedade no estiver setada.
   */
  final protected List<String> getStringListProperty(final String key) {
    return _propertyStringList(key, true);
  }

  /**
   * Recupera uma propriedade da aplicao (lista)
   * 
   * @param key a chave (prefixo)
   * @return o valor
   * @throws IllegalStateException se a propriedade no estiver setada.
   */
  final List<String> getStringListServiceProperty(final String key) {
    return _propertyStringList(key, false);
  }

  /**
   * Verifica a nulidade de uma propriedade
   * 
   * @param key a identificao da propriedade
   * 
   * @return indicativo
   */
  final protected boolean isPropertyNull(final String key) {
    checkPropertiesLoaded();
    final String realKey = getRealPropertyKey(key);

    // Verificao se a propriedade foi redefinida pelo arquivo system.
    if (systemProperties != null && systemProperties.hasProperty(realKey)) {
      // Se no existir no arquivo sistema, ser levantada exceo!
      return systemProperties.isPropertyNull(realKey);
    }

    // Se no existir no arquivo servidor, ser levantada exceo!
    return serverProperties.isPropertyNull(realKey);
  }

  /**
   * Verifica se h um RMI registry na porta indicada. Caso no haja, tenta
   * rodar um. Em qualquer caso, guarda no campo privado registry a referncia
   * para o registry. Caso uma referncia no possa ser obtida, retorna falso.
   * 
   * @param registryPort a porta do registry.
   * 
   * @return true em caso de sucesso, false em caso de falha.
   */
  private boolean getRegistry(final int registryPort) {
    try {
      registry = LocateRegistry.getRegistry(registryPort);
    }
    catch (final Exception e) {
    }
    if (!checkRegistry()) {
      try {
        registry = LocateRegistry.createRegistry(registryPort);
      }
      catch (final Exception e) {
      }
    }
    return checkRegistry();
  }

  /**
   * Nome do servidor. Por exemplo: InfoGrid, WebSintesi, MARLIM.
   * 
   * @return O nome do servidor.
   */
  public String getServerName() {
    return this.serverName;
  }

  /**
   * Retorna a verso do servidor como uma string no formato "V: X.Y.Z
   * (dd/mm/aa)" onde X = <i>major version</i>, Y = <i>minor version</i>, Z =
   * <i>patch number</i> e dd/mm/aa  a data em que o servidor foi gerado.
   * 
   * @return string com identificador da verso do servidor
   */
  protected String getServerVersion() {
    final Version v = Version.getInstance();
    return "V: " + v.getVersion() + " (" + v.getReleaseDate() + ")";
  }

  /**
   * Pega a URL para execuo do cliente applet do sistema.
   * 
   * @return A URL.
   */
  public String getSystemURL() {
    final String propertyName = "systemURL";
    final boolean isNull = isPropertyNull(propertyName);
    if (isNull) {
      return null;
    }
    final String systemURL = getStringProperty(propertyName);
    return systemURL;
  }

  /**
   * Cria o gerente de configuraes (veja {@link ConfigurationManager}).
   */
  private void initConfigurationManager() {
    final String propertyName = "configurationManager.extensionPrefix";
    final boolean isNull = isPropertyNull(propertyName);

    if (isNull) {
      logInfoMessage("Criando o gerente de configurao sem "
        + "prefixo de extenso.");
      ConfigurationManager.createInstance();
    }
    else {
      final String extensionPrefix = getStringProperty(propertyName);
      logInfoMessage("Criando o gerente de configurao com o "
        + "prefixo de extenso: " + extensionPrefix + ".");
      ConfigurationManager.createInstance(extensionPrefix);
    }
  }

  /**
   * Mtodo para inicializao do log do gerente de servios.
   * 
   * @return o logger.
   * @throws ServerException em caso de falha.
   */
  private Logger initMainLog() throws ServerException {
    System.out.println("Inicializando sistema de logs...");

    // Classe do log manager
    final String logManagerClassName = ServiceLogManager.class.getName();
    System.out.println(" - LogManager: " + logManagerClassName);
    final String loggingMgrKey = "java.util.logging.manager";
    System.setProperty(loggingMgrKey, logManagerClassName);

    // Ajustando path para "logging config file"
    final String logCnfKey = "logging.config.file";
    if (isPropertyNull(logCnfKey)) {
      final String err =
        "Propriedade " + logCnfKey + " no ser anulada no servidor!";
      throw new ServerException(err);
    }

    final String logCnfFile = getStringProperty(logCnfKey);
    System.setProperty("java.util.logging.config.file", logCnfFile);
    System.out.println(" - Usando " + logCnfKey + ": " + logCnfFile);
    checkLoggingProperties(logCnfFile);

    // Criao do diretrio-base de logs
    final String logRootDirPath = getLogRootDirectoryName();
    checkDirectory(logRootDirPath);

    final String logDevDirPath = getLogDevDirectoryName();
    checkDirectory(logDevDirPath);

    try {
      /*
       * O carregamento da classe a seguir  necessrio para que o nvel
       * personalizado "DEBUG", definido como um campo esttico da classe
       * ServerLevel, seja carregado na coleo de nveis conhecidos do
       * LogManager. Desta forma, se este nvel for especificado como valor para
       * a propriedade ".level" do arquivo "properties/Logging.properties", o
       * mtodo esttico Level.parse, invocado durante a inicializao do
       * mainLog, j ser capaz de identific-la e us-la como thresold para os
       * logs a serem visualizados.
       */
      final String sep = File.separator;
      final String serverLoggerPath = logDevDirPath + sep + "Server.log";
      Class.forName(ServerLevel.class.getName());
      final Logger logger = Logger.getLogger(serverLoggerPath);
      logger.setUseParentHandlers(false);
      final FileHandler mainLogFileHandler = new FileHandler(serverLoggerPath);
      logger.addHandler(mainLogFileHandler);
      /*
       * adiciona um handler para o console. O nvel de log default para ele 
       * definido no Logging.properties, e por default  SEVERE. Para
       * desabilitar o log para o console, basta mudar o nvel default para OFF.
       */
      logger.addHandler(new ConsoleHandler());
      System.out.println(" - Mensagens armazenadas em: " + serverLoggerPath);
      return logger;
    }
    catch (final Exception e) {
      e.printStackTrace();
      throw new ServerException("Erro na criao do log do Servidor", e);
    }
  }

  /**
   * Checagem de definio de formatadores de handlers de log. Por enquanto, s
   * checamos os handlers do Java do pacote java.util.logging
   * 
   * @param logCnfFile path para o arquivo de definio de log
   * @throws ServerException em caso de m configurao.
   */
  private void checkLoggingProperties(final String logCnfFile)
    throws ServerException {
    try {
      // Carga
      final Properties logProps = new Properties();
      logProps.load(new FileInputStream(new File(logCnfFile)));

      //Handlers
      final String hndTag = "handlers";
      final String hndProp = logProps.getProperty(hndTag);
      if (hndProp == null || hndProp.trim().isEmpty()) {
        final String err = "No foram definidos handlers de log!";
        throw new ServerException(err);
      }

      // Level e formatters de handlers
      final String[] classNames = { "FileHandler", "ConsoleHandler" };
      for (String className : classNames) {
        final String fmtTag = "java.util.logging." + className + ".formatter";
        final String fmtProp = logProps.getProperty(fmtTag);
        if (fmtProp == null || fmtProp.trim().isEmpty()) {
          final String err = "No foi definido um formatador log: " + fmtTag;
          throw new ServerException(err);
        }

        final String levTag = "java.util.logging." + className + ".level";
        final String levProp = logProps.getProperty(levTag);
        if (levProp == null || fmtProp.trim().isEmpty()) {
          final String err = "No foi definido um nvel (level) log: " + levTag;
          throw new ServerException(err);
        }
      }
    }
    catch (Exception e) {
      final String err = "Falha na carga de configurao de log: " + logCnfFile;
      throw new ServerException(err, e);
    }
  }

  /**
   * Verifica se o servidor  servidor central.
   * 
   * @return true, caso seja um servidor central, ou false, caso contrrio.
   */
  public boolean isCentralServer() {
    final String centralServerName = this.getCentralServerName();
    return (centralServerName == null);
  }

  /**
   * Carrega o bundle default do servidor.
   * 
   * @return o locale default.
   */
  private Locale loadAndSetDefaultLocale() {
    final String propertyName = "clientDefaultLocale";
    final boolean isNull = isPropertyNull(propertyName);
    final Locale locale;
    if (!isNull) {
      final String localeStr = getStringProperty(propertyName);
      locale = FormatUtils.parseLocale(localeStr);
    }
    else {
      locale = Locale.getDefault();
    }
    logInfoMessage("Locale default do servidor: " + locale);
    return locale;
  }

  /**
   * Carrega o arquivo com as propriedades do sistema.
   * 
   * @return as propriedades do servidor.
   * 
   * @throws ServerException se no existir diretrio de propriedades.
   */
  private ServerSideProperties loadAndSetServerProperties()
    throws ServerException {
    final String propDirPath = getPropertiesRootDirectoryName();
    checkDirectory(propDirPath);

    final String fileName = "Server.properties";
    final String filePath = propDirPath + File.separator + fileName;
    System.out.println("Carregando propriedades (servidor) de: " + filePath);
    final ServerSideProperties props = new ServerSideProperties(filePath);
    try {
      final File file = new File(filePath);
      final FileInputStream stream = new FileInputStream(file);
      props.load(stream);
    }
    catch (FileNotFoundException e) {
      final String msg = "No existe arquivo de propriedades:" + filePath;
      throw new ServerException(msg);
    }
    System.out.println(" - Carga propriedades terminada");
    return props;
  }

  /**
   * Carrega o arquivo com as propriedades do sistema.
   * 
   * @return as propriedades do servidor.
   * 
   * @throws ServerException se no existir diretrio de propriedades.
   */
  private ServerSideProperties loadAndSetSystemProperties(Properties overprops)
    throws ServerException {

    if ((systemFileName == null) && overprops.isEmpty()) {
      System.out.println("Propriedades (sistema) no definidas! "
        + "Usando configurao default...");
      return null;
    }
    final ServerSideProperties props;
    if (systemFileName == null) {
      props = new ServerSideProperties();
    }
    else {
      props = new ServerSideProperties(systemFileName);
      System.out.println("Carregando propriedades (sistema) de: "
        + systemFileName);
      try {
        final File file = new File(systemFileName);
        final FileInputStream stream = new FileInputStream(file);
        props.load(stream);
      }
      catch (FileNotFoundException e) {
        final String msg =
          "No foi localizado o arquivo de propriedades:" + systemFileName;
        throw new ServerException(msg);
      }
      System.out.println(" - Carga propriedades (sistema) terminada");
    }
    /*
     * Sobrescreve as propriedades do sistema com as passadas na linha de
     * comando.
     */
    for (Object k : overprops.keySet()) {
      final String key = (String) k;
      final String value = overprops.getProperty(key);
      props.overrideProperty(key, value);
      System.out.println("Carregando propriedades (linha de comando): " + key
        + " = " + value);
    }
    System.out.println(" - Carga propriedades (linha de comando) terminada");
    return props;
  }

  /**
   * Carrega a propriedade que define o nome do servidor.
   * 
   * @return o nome do servidor.
   */
  private String loadAndSetServerName() {
    final String propertyName = "name";
    final boolean isNull = isPropertyNull(propertyName);
    if (isNull) {
      final String fmt = "No foi encontrada a propriedade %s.";
      final String message = String.format(fmt, propertyName);
      logSevereMessage(message);
      return null;
    }

    final String srvName = this.getStringProperty(propertyName);
    return srvName;

  }

  /**
   * Carrega a propriedade que define a porta do servidor.
   * 
   * @return a porta
   */
  private int loadAndSetServerPort() {
    final int defaultRMIPort = 1099;
    int port = defaultRMIPort;

    final String propertyName = "rmiPort";
    final boolean isNull = isPropertyNull(propertyName);
    if (!isNull) {
      port = this.getIntProperty(propertyName);
    }
    else {
      logInfoMessage("Propriedade de conexo RMI indefinida. Usando default...");
    }
    final String fmt = "Servidor com conexo RMI na %d";
    logInfoMessage(String.format(fmt, port));

    return port;
  }

  /**
   * Faz checagem de inicializao do log quando usa-se uma string para seu uso.
   * 
   * @param msg a string a ser loggada.
   * @throws RuntimeException se o log no estiver ainda inicializado.
   */
  private void checkLogInited(final String msg) {
    if (serverLogger == null) {
      final String err1 = "Erro interno no servidor por";
      final String err2 = "uso de log antes de sua incializao!";
      final String txt = "Mensagem original: [" + msg + "]";
      final String message = err1 + " " + err2 + " - " + txt;
      throw new RuntimeException(message);
    }
  }

  /**
   * Gerao de log de mensagem com severidade.
   * 
   * @param msg a mensagem a ser gerada.
   */
  public final void logDebugMessage(final String msg) {
    checkLogInited(msg);
    final String message = ServerLogFormatter.getQualifiedMessage(msg);
    serverLogger.log(ServerLevel.DEBUG, message);
  }

  /**
   * Adiciona a mensagem de informao ao arquivo de logging.
   * 
   * @param msg a mensagem de informao
   */
  public void logInfoMessage(final String msg) {
    checkLogInited(msg);
    final String message = ServerLogFormatter.getQualifiedMessage(msg);
    serverLogger.info(message);
  }

  /**
   * Adiciona a mensagem de erro ao arquivo de logging.
   * 
   * @param msg mensagem a ser anexada ao arquivo
   */
  public void logSevereMessage(final String msg) {
    checkLogInited(msg);
    final String message = ServerLogFormatter.getQualifiedMessage(msg);
    serverLogger.severe(message);
  }

  /**
   * Adiciona a mensagem de erro e a causa ao arquivo de logging.
   * 
   * @param msg messagem de erro
   * @param t causa do erro
   */
  public void logSevereMessage(final String msg, final Throwable t) {
    checkLogInited(msg);
    final String message = ServerLogFormatter.getQualifiedMessage(msg);
    serverLogger.log(Level.SEVERE, message, t);
  }

  /**
   * Adiciona a mensagem de alerta ao arquivo de logging.
   * 
   * @param msg a mensagem de alerta
   */
  public void logWarningMessage(final String msg) {
    checkLogInited(msg);
    final String message = ServerLogFormatter.getQualifiedMessage(msg);
    serverLogger.warning(message);
  }

  /**
   * Cria a thread que  acionada quando o processo do servidor  interrompido.
   */
  private void setShutdownHook() {
    final Thread t = new Thread(new Runnable() {
      public synchronized void run() {
        try {
          stopServerHook();
        }
        catch (final Exception e) {
          e.printStackTrace();
        }
      }
    });
    Runtime.getRuntime().addShutdownHook(t);
  }

  /**
   * Finaliza o ambiente de execuo no servidor.
   */
  public void shutdown() {
    logSevereMessage("Shutdown do servidor acionado!");
    System.exit(0);
  }

  /**
   * Inicia o servidor, inicializando todos os servios locais. Caso esteja
   * definido um servidor central,  feito o o login nele como admin. Note que a
   * conexo ao servidor central s pode ser feita aps a inicializao dos
   * servios locais pois  necessria a senha de admin para efetuar o login.
   * Por outro lado, o monitor tem que exitir antes que os servios sejam
   * inicializados para que os observadores possam ser colocados.
   * 
   * @return indicativo.
   */
  public boolean start() {
    try {
      if (!this.isCentralServer()) {
        startCentralServerMonitor();
      }
      startServices();
      startRMI();

      logDebugMessage("Servios locais ativos.");
      if (!this.isCentralServer()) {
        startCentralServerConnection();
      }
      logDebugMessage("Preparando ps-inicializao do servidor.");

      postInitialization();

      logInfoMessage("Servidor iniciado.");
      System.out.println(getGreetingMessage());
      System.out.println(getServerVersion());
    }
    catch (Exception e) {
      logSevereMessage("Falha no start-up do servidor", e);
      System.out.println("\n");
      System.out.println("[ERRO] - Falha no start-up do servidor!");
      System.out.println("       > " + e.getMessage());
      System.out.println("\n");
      return false;
    }
    return true;
  }

  /**
   * Inicia a comunicao com o servidor central.
   */
  private void startCentralServerConnection() {
    final String centralServerName = getCentralServerName();
    if (centralServerName == null) {
      return;
    }
    LocalServerRemoteMonitor.getInstance().setSystemName(
      MessageFormat.format("Servidor local {0}", this.getServerName()));
    logInfoMessage("Iniciando conexo ao servidor central: "
      + centralServerName);
    LocalServerRemoteMonitor.getInstance().start();
    logInfoMessage("Conexo ao servidor central concluda.");
  }

  /**
   * Inicia a classe que ser usada pelo servidor local para monitorar o estado
   * da conexo com um servidor central.
   */
  public abstract void startCentralServerMonitor();

  /**
   * Cria a porta de entrada para acesso as funcionalidade do servidor e a
   * exporta via rmi
   * 
   * @throws ServerException caso ocorra alguma falha na exportao via rmi
   */
  private void startRMI() throws ServerException {
    try {
      this.entryPoint = createServerEntryPoint();
      UnicastRemoteObject.exportObject(this.entryPoint, 0);
      registry.rebind(ServerEntryPoint.LOOKUP, this.entryPoint);
    }
    catch (final Exception e) {
      throw new ServerException("Falha no export/bind.", e);
    }
  }

  /**
   * Cria os servios e os inicia em seguida.
   * 
   * @throws ServerException caso ocorra alguma falha na criao ou na
   *         inicializao dos servios
   */
  private void startServices() throws ServerException {
    ServiceManager.init();
    createServices();
    ServiceManager.getInstance().initAllServices();
  }

  /**
   * Mtodo chamado quando pela thread acionada quando o processo do servidor 
   * interrompido.
   */
  private void stopServerHook() {
    System.out.println("Finalizando o servidor.");
    System.out.println("Acionado stop server hook!");
  }

  /**
   * Retorna a instncia do servidor.
   * 
   * @return a instncia
   */
  public static Server getInstance() {
    if (instance == null) {
      System.out.println("SERVIDOR NO INSTANCIADO!");
    }
    return instance;
  }

  /**
   * Mtodo de consulta ao nome do diretrio-raiz de log.
   * 
   * @return uma string com o nome do diretrio-raiz.
   */
  final public String getLogRootDirectoryName() {
    final String dirName = getStringProperty("logging.directory");
    return dirName;
  }

  /**
   * Mtodo de consulta ao nome do diretrio-raiz de log.
   * 
   * @return uma string com o nome do diretrio-raiz.
   */
  final public String getLogDevDirectoryName() {
    final String dirName = getLogRootDirectoryName();
    return dirName + File.separator + "develop";
  }

  /**
   * Mtodo de consulta ao nome do diretrio-raiz de dados persistidos.
   * 
   * @return uma string com o nome do diretrio-raiz.
   */
  public String getPersistencyRootDirectoryName() {
    final String dirName = getStringProperty("persistency.directory");
    return dirName;
  }

  /**
   * Mtodo de consulta ao nome do diretrio-raiz de propriedades.
   * 
   * @return uma string com o nome do diretrio-raiz.
   */
  public static String getPropertiesRootDirectoryName() {
    return FileSystem.PROPERTIES_DIR;
  }

  /**
   * Ponto de entrada da aplicao que inicia o servidor do sistema. Instala um
   * SecurityManager e configura o provedor de objetos para atender ao servidor.
   * 
   * @param args argumentos da linha de comando do servidor.
   * @throws ServerException em caso de falha.
   */
  protected Server(final String[] args) throws ServerException {
    Properties props = parseCommandLineArgs(args);
    try {
      if (instance != null) {
        final String msg = "Servidor instanciado pela segunda vez!";
        throw new ServerException(msg);
      }

      setShutdownHook();

      // Carga das propriedades do servidor.
      this.serverProperties = loadAndSetServerProperties();
      this.systemProperties = loadAndSetSystemProperties(props);

      // Ajuste da classe que faz gerncia de logs
      serverLogger = initMainLog();

      // Criao do diretrio de peristncia (se necessrio)
      final String persistDirPath = getPersistencyRootDirectoryName();
      checkDirectory(persistDirPath);

      // Configuration Manager precisa das propriedades carregadas.
      initConfigurationManager();

      this.defaultLocale = loadAndSetDefaultLocale();

      // Acesso ao RMI
      this.rmiPort = loadAndSetServerPort();
      checkServerPort();

      // Carga de configurao de nomes do servidor.
      this.serverHostName = loadAndSetServerHostName();
      this.serverName = loadAndSetServerName();

      // Torna os identificadores dos objetos remotos aleatrios. Essa
      // caracterstica  a base do mecanismo de autenticao e autorizao,
      // garantindo que um usurio s consegue fazer chamadas aos proxies
      // criados especificamente para ele.
      System.setProperty("java.rmi.server.randomIDs", "true");

      // Preparando chaves
      this.createKeyStore();
      this.privateKeyPassword = loadAndSetPrivateKeyPassword();

      instance = this;
    }
    catch (RuntimeException rte) {
      throw new ServerException(rte);
    }
  }

  /**
   * @param args os argumentos
   */
  private Properties parseCommandLineArgs(String[] args) {
    final Properties props = new Properties();
    if (args == null) {
      return props;
    }
    final int nargs = args.length;
    int i = 0;
    while (i < nargs) {
      final String arg = args[i];
      if (arg.equals("-p")) {
        i++;
        systemFileName = (i < nargs) ? args[i] : null;
      }
      else if (arg.equals("-override")) {
        i++;
        final String key = (i < nargs) ? args[i++] : null;
        final String value = (i < nargs) ? args[i] : null;
        if (key != null && value != null) {
          props.setProperty(key, value);
        }
      }
      else {
        System.out.println("Ignorando argumento " + arg + "...");
      }
      i++;
    }
    return props;
  }

  /**
   * Faz a carga da chave primria do servidor.
   * 
   * @return a chave privada ou <code>null</code> se a mesma no estiver setada.
   */
  private String loadAndSetPrivateKeyPassword() {
    final String propertyName = "privateKeyPassword";
    final boolean isNull = isPropertyNull(propertyName);
    if (isNull) {
      final String fmt = "No foi encontrada a propriedade [%s].";
      final String message = String.format(fmt, propertyName);
      logWarningMessage(message);
      return null;
    }
    final String pwd = getStringProperty(propertyName);
    return pwd;
  }

  /**
   * Faz checagem RMI da porta selecionada para o servidor.
   * 
   * @throws ServerException se a checagem falhar
   */
  private void checkServerPort() throws ServerException {
    if (!getRegistry(this.rmiPort)) {
      final String err = "Falha de identificao da porta RMI!";
      logSevereMessage(err);
      throw new ServerException(err);
    }
  }

  /**
   * Faz a carga do nome do hostname (configura o endereo utilizado para
   * exportar objetos RMI do servidor).
   * 
   * @return o host name do servidor.
   * @throws ServerException se endereo IP no for encontrado.
   */
  private String loadAndSetServerHostName() throws ServerException {
    String hostname;
    final String propertyName = "hostname";
    final boolean isNull = isPropertyNull(propertyName);
    if (isNull) {
      final String info =
        "Propriedade <hostname> do servidor no informada! Inferindo...";
      logInfoMessage(info);

      try {
        final InetAddress localHost = InetAddress.getLocalHost();
        hostname = localHost.getCanonicalHostName();
      }
      catch (final UnknownHostException e) {
        final String msg = "Endereo IP do servidor no pode ser encontrado";
        logSevereMessage(msg);
        throw new ServerException(msg, e);
      }
    }
    else {
      hostname = getStringProperty(propertyName);
    }

    System.setProperty("java.rmi.server.hostname", hostname);
    logInfoMessage("Propriedade hostname do servidor: " + hostname);
    return hostname;
  }

  /**
   * Mtodo chamado aps a inicializao dos servios.
   * 
   * @throws InitFailureException caso ocorra alguma falha na ps inicializao
   */
  public abstract void postInitialization() throws InitFailureException;

}
