/*
 * $Id:
 */
package csbase.server;

import java.io.Serializable;
import java.net.MalformedURLException;
import java.net.URL;
import java.rmi.Remote;
import java.text.MessageFormat;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.TimeZone;

import csbase.logic.PreLoginData;
import csbase.logic.ServerURI;
import csbase.logic.Session;
import csbase.logic.UserOutline;
import csbase.logic.Version;
import csbase.remote.HttpServiceInterface;
import csbase.remote.ServerEntryPoint;
import csbase.remote.ServiceInterface;
import csbase.server.keystore.CSKeyStore;
import csbase.server.services.httpservice.HttpService;
import csbase.server.services.loginservice.LoginService;

/**
 * Implementa a porta de entrada para acesso as funcionalidade do servidor.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class ServerEntryPointImpl implements ServerEntryPoint {
  /**
   * Nome do parmetro que indica a porta RMI onde o servidor est rodando.
   */
  private static final String SERVER_PORT_PARAM = "server_port";

  /**
   * Nome do parmetro que indica a pgina principal do sistema que ser
   * inserida na URL do sistema
   */
  private static final String PAGE_PARAM = "page";

  /**
   * Sufixo que ser adicionado ao nome do sistema no parmetro
   * {@link #PAGE_PARAM}
   */
  private static final String PAGE_SUFFIX = ".jnlp";

  /**
   * {@inheritDoc}
   */
  public boolean canChangePasswords() {
    return LoginService.getInstance().canChangePasswords();
  }

  /**
   * {@inheritDoc}
   */
  public HttpServiceInterface fetchHttpService() {
    return HttpService.getInstance();
  }

  /**
   * {@inheritDoc}
   */
  public ServiceInterface fetchService(final Object sessionKey, final String serviceName) {
    return ServiceManager.getInstance().getRemoteService(sessionKey,
      serviceName);
  }

  /**
   * {@inheritDoc}
   */
  public Map<String, ServiceInterface> fetchServices(final Object sessionKey,
    final Set<String> servicesNames) {
    return ServiceManager.getInstance().getRemoteServices(sessionKey,
      servicesNames);
  }

  /**
   * Gera a URL de acesso ao sistema.
   * 
   * @param locale A Locale que o usurio escolheu no login.
   * @param token Identificao de um preLogin.
   * 
   * @return A URL de acesso ao sistema.
   */
  private String generateSystemURL(final String token, final Locale locale) {
    return this.generateSystemURL(token, locale, null);
  }

  /**
   * Gera a URL de acesso ao sistema.
   * 
   * @param locale A Locale que o usurio escolheu no login.
   * @param token Identificao de um preLogin.
   * @param systemName o nome do sistema que se deseja obter a URL. Este nome 
   *        usado para definir o parmetro <code>page</code> da URL, que ser
   *        acrescido da extenso do tipo de aplicativo web. (i.e.: ".jnlp")
   * 
   * @return A URL de acesso ao sistema.
   */
  private String generateSystemURL(final String token, final Locale locale,
    final String systemName) {
    final String LOCALE_PARAM = "locale";
    final String TOKEN_PARAM = "token";

    String url = Server.getInstance().getSystemURL();
    final String port = Integer.toString(Server.getInstance().getPort());
    url =
      MessageFormat.format("{0}?{1}={2}", new Object[] { url,
          SERVER_PORT_PARAM, port });
    url =
      MessageFormat.format("{0}&{1}={2}", new Object[] { url, TOKEN_PARAM,
          token });
    url =
      MessageFormat.format("{0}&{1}={2}", new Object[] { url, LOCALE_PARAM,
          locale });
    if (systemName != null) {
      url =
        MessageFormat.format("{0}&{1}={2}{3}", new Object[] { url, PAGE_PARAM,
            systemName, PAGE_SUFFIX });
    }
    return url;
  }

  /**
   * {@inheritDoc}
   */
  public UserOutline[] getLoggedUsers() {
    return LoginService.getInstance().getLoggedUsers();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public String getServerName() {
    final Server server = Server.getInstance();
    final String serverName = server.getServerName();
    return serverName;
  }

  /**
   * {@inheritDoc}
   */
  public String getSystemURLWithRMIPort(final String systemName) {
    if (systemName == null) {
      throw new IllegalArgumentException("systemName = null");
    }
    if (systemName.trim().equals("")) {
      throw new IllegalArgumentException("systemName vazio");
    }
    String url = Server.getInstance().getSystemURL();
    final String port = Integer.toString(Server.getInstance().getPort());
    url =
      MessageFormat.format("{0}?{1}={2}", new Object[] { url,
          SERVER_PORT_PARAM, port });
    url =
      MessageFormat.format("{0}&{1}={2}{3}", new Object[] { url, PAGE_PARAM,
          systemName, PAGE_SUFFIX });
    return url;
  }

  /**
   * {@inheritDoc}
   */
  public String getTag() {
    return Version.getInstance().getName();
  }

  /**
   * {@inheritDoc}
   */
  public boolean isLocalLogin() {
    return LoginService.getInstance().isLocalLogin();
  }

  /**
   * {@inheritDoc}
   */
  public Map<String, Serializable> isValidSession(final Object sessionKey,
    final String serverName, final byte[] signedServerName) {

    if (sessionKey == null || serverName == null || signedServerName == null) {
      throw new IllegalArgumentException(
        "sessionKey == null || serverName == null || signedServerName == null");
    }

    final Server server = Server.getInstance();
    try {
      final CSKeyStore keyStore = CSKeyStore.getInstance();
      if (keyStore == null) {
        final String msg = "O repositrio de chaves/certificados no existe. Sesso no validada.";
        server.logWarningMessage(msg);
        return null;
      }

      if (!keyStore.containsAlias(serverName)) {
        final String fmt = "O servidor %s no possui certificado cadastrado. Sesso no validada.";
        final String message = String.format(fmt, serverName);
        server.logWarningMessage(message);
        return null;
      }

      if (!keyStore.verify(serverName, serverName, signedServerName)) {
        final String fmt = "A assinatura do servidor %s est invlida. Sesso no validada.";
        final String message = String.format(fmt, serverName);
        server.logWarningMessage(message);
        return null;
      }
    }
    catch (final Exception e) {
      server.logSevereMessage("Erro validando sesso .");
      return null;
    }

    if (LoginService.getInstance().isValidSession(sessionKey)) {
      return LoginService.getInstance().getSessionAttributes(sessionKey);
    }
    return null;
  }

  /**
   * {@inheritDoc}
   */
  public Session login(final ServerURI referedServerURI,
    final Map<String, Serializable> attr, final boolean copyServerSessionAttrs,
    final Object sessionkey, final String login, final String delegatedLogin, final Locale locale,
    final TimeZone tz) {
    return LoginService.getInstance().login(referedServerURI, attr,
      copyServerSessionAttrs, sessionkey, login, delegatedLogin, locale, tz);
  }

  /**
   * {@inheritDoc}
   */
  public Session login(final String token) {
    return LoginService.getInstance().login(token);
  }

  /**
   * {@inheritDoc}
   */
  public Session login(final String localServerName, final byte[] signedLocalServerName,
    final Locale locale) {
    return LoginService.getInstance().login(localServerName,
      signedLocalServerName, locale);
  }

  /**
   * {@inheritDoc}
   */
  public Session login(final String login, final String password, final Locale locale) {
    return LoginService.getInstance().login(login, password, locale);
  }

  /**
   * {@inheritDoc}
   */
  public Session login(final String login, final String password, final Locale locale,
    final String delegatedLogin) {
    return LoginService.getInstance().login(login, password, locale,
      delegatedLogin);
  }

  /**
   * {@inheritDoc}
   */
  public Session login(final String login, final String password, final Locale locale,
    final TimeZone timeZone) {
    return LoginService.getInstance().login(login, password, locale, timeZone);
  }

  /**
   * {@inheritDoc}
   */
  public Session login(final String login, final String password, final Locale locale,
    final TimeZone tz, final String delegatedLogin) {
    return LoginService.getInstance().login(login, password, locale,
      delegatedLogin, tz, null);
  }

  /**
   * {@inheritDoc}
   */
  public Session login(final String login, final String password, final Locale locale,
    final TimeZone tz, final String delegatedLogin, final Map<String, Serializable> params) {
    return LoginService.getInstance().login(login, password, locale,
      delegatedLogin, tz, params);
  }

  /**
   * {@inheritDoc}
   */
  public Session login(final String token, final TimeZone timeZone) {
    return LoginService.getInstance().login(token, timeZone);
  }

  /**
   * {@inheritDoc}
   */
  public void logout(final Object sessionKey) {
    LoginService.getInstance().logout(sessionKey);
  }

  /**
   * {@inheritDoc}
   */
  public void ping() {
    /*
     * No h necessidade de implementao.  utilizado como mtodo remoto dummy
     * para que um cliente possa verificar se a conexo com o servidor ainda
     * est vlida.
     */
  }

  /**
   * {@inheritDoc}
   */
  public PreLoginData preLogin(final String login, final String password, final Locale locale) {
    final PreLoginData data =
      LoginService.getInstance().preLogin(login, password, locale);
    if (data == null) {
      return null;
    }
    final String token = data.getToken();
    try {
      data.setUrl2Login(new URL(this.generateSystemURL(token, locale)));
    }
    catch (final MalformedURLException e) {
      e.printStackTrace();
      return null;
    }
    return data;
  }

  /**
   * {@inheritDoc}
   */
  public PreLoginData preLogin(final String login, final String password, final Locale locale,
    final Map<String, Serializable> attributes) {
    final PreLoginData data =
      LoginService.getInstance().preLogin(login, password, locale, attributes);
    if (data == null) {
      return null;
    }
    final String token = data.getToken();
    try {
      data.setUrl2Login(new URL(this.generateSystemURL(token, locale)));
    }
    catch (final MalformedURLException e) {
      e.printStackTrace();
      return null;
    }
    return data;
  }

  /**
   * {@inheritDoc}
   */
  public PreLoginData preLogin(final String login, final String password, final Locale locale,
    final String delegatedLogin, final Remote control, final Serializable userData) {
    final PreLoginData data =
      LoginService.getInstance().preLogin(login, password, locale,
        delegatedLogin, control, userData);
    if (data == null) {
      return null;
    }
    final String token = data.getToken();
    try {
      data.setUrl2Login(new URL(this.generateSystemURL(token, locale)));
    }
    catch (final MalformedURLException e) {
      e.printStackTrace();
      return null;
    }
    return data;
  }

  /**
   * {@inheritDoc}
   */
  public String preLogin2Web(final String login, final String password, final Locale locale) {
    final PreLoginData data =
      LoginService.getInstance().preLogin(login, password, locale);
    if (data == null) {
      return null;
    }
    return generateSystemURL(data.getToken(), locale);
  }

  /**
   * {@inheritDoc}
   */
  public String preLogin2Web(final String login, final String password, final Locale locale,
    final Map<String, Serializable> attributes) {
    final PreLoginData data =
      LoginService.getInstance().preLogin(login, password, locale, attributes);
    if (data == null) {
      return null;
    }
    return generateSystemURL(data.getToken(), locale);
  }

  /**
   * {@inheritDoc}
   */
  public String preLogin2Web(final String login, final String password, final Locale locale,
    final String systemName) {
    final PreLoginData data =
      LoginService.getInstance().preLogin(login, password, locale);
    if (data == null) {
      return null;
    }
    return this.generateSystemURL(data.getToken(), locale, systemName);
  }

  /**
   * {@inheritDoc}
   */
  public String preLogin2Web(final String login, final String password, final Locale locale,
    final String delegatedLogin, final String systemName) {
    final PreLoginData data =
      LoginService.getInstance().preLogin(login, password, locale,
        delegatedLogin);
    if (data == null) {
      return null;
    }
    return this.generateSystemURL(data.getToken(), locale, systemName);
  }

  /**
   * {@inheritDoc}
   */
  public void setServerSessionProperty(final String propertyName,
    final Serializable propertyValue) {
    if (propertyValue == null) {
      LoginService.getInstance().removeSessionProperty(Service.getKey(),
        propertyName);
    }
    else {
      LoginService.getInstance().setSessionProperty(Service.getKey(),
        propertyName, propertyValue);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void setSystemName(final Object sessionKey, final String systemName) {
    LoginService.getInstance().setSystemName(sessionKey, systemName);
  }
}