package csbase.server.services.algorithmservice.niotransfer;

import java.nio.channels.SelectionKey;

/**
 * <p>
 * Objetos desta classe representam um pedido de mudana no estado de uma 
 * determinada chave, chave esta representando a associao de um canal a um 
 * seletor.
 * </p>
 * <p>
 * As mudanas podem ser de dois tipos: 
 * <ol>
 * <li>Modificao no conjunto de interesses da chave;</li>
 * <li>Cancelamento da chave (incluindo fechamento do canal associado).</li>
 * </ol>
 * </p>
 * 
 * @author Tecgraf
 */
public class ChangeRequest {
  /** Chave cujo conjunto de interesses deve ser modificado. */
  private SelectionKey key;
  /** Conjunto de interesses que dever substituir o conjunto atual. */
  private int ops;
  /** Tipo de mudana sendo solicitada. */
  private Type type;

  /** Tipos de mudana possveis */
  public enum Type {
    /** Modificao no conjunto de interesses da chave */
    CHANGE_OPS,
    /** Cancelamento da chave e fechamento do canal */
    CANCEL
  }
  
  /**
   * Cria uma solicitao de mudana no conjunto de interesses de uma chave.
   * 
   * @param key chave cujo conjunto de interesses deve ser modificado.
   * @param ops conjunto de interesses que dever substituir o conjunto atual.
   */
  ChangeRequest(SelectionKey key, int ops) {
    this.key = key;
    this.ops = ops;
    this.type = Type.CHANGE_OPS;
  }

  /**
   * Cria uma solicitao de cancelamento de uma chave.
   * 
   * @param key chave a ser cancelada.
   */
  public ChangeRequest(SelectionKey key) {
    this.key = key;
    this.type = Type.CANCEL;
  }

  /**
   * Retorna a chave que dever sofrer a mudana.
   * 
   * @return chave cujo conjunto de interesses deve ser modificado.
   */
  public SelectionKey getKey() {
    return key;
  }

  /**
   * Retorna o conjunto de interesses que deve substituir o conjunto atual de
   * interesses da chave.
   * 
   * @return novo conjunto de interesses.
   */
  public int getOps() {
    return ops;
  }

  /**
   * Retorna o tipo de mudana sendo solicitada. Pode ser: {@link Type#CANCEL}
   * ou {@link Type#CHANGE_OPS}.
   * 
   * @return tipo de mudana.
   */
  public Type getType() {
    return type;
  }

}
