package csbase.server.services.algorithmservice.niotransfer;

import java.io.File;
import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.nio.channels.ClosedChannelException;
import java.nio.channels.SelectableChannel;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;
import java.security.DigestException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import tecgraf.javautils.configurationmanager.Configuration;
import tecgraf.javautils.configurationmanager.ConfigurationManager;
import tecgraf.javautils.configurationmanager.ConfigurationManagerException;

import csbase.exception.ConfigurationException;
import csbase.logic.SecureKey;
import csbase.remote.niotransfer.RequestTransferInfo;
import csbase.remote.niotransfer.SimpleFileTransferProtocol.Operation;


/**
 * Servidor para importao de arquivos, semelhante a um servidor FTP, mas 
 * usando a API NIO (com sockets no-bloqueantes) do Java. Qualquer entidade
 * que desejar transferir arquivos entre cliente e servidor (atualmente, 
 * servios) devem instanciar um objeto desta classe em uma thread separada.
 * Este objeto ficar aguardando conexes de clientes solicitando importaes
 * ou exportaes de arquivos.
 * 
 * @author Leonardo Barros
 */
public class FileServer {
  /**
   * Multiplexador (<code>Selector</code>) no qual sero registrados
   * sockets. Sempre que esse multiplexador receber um evento do SO, vai
   * verificar se existe algum socket interessado e disparar a callback 
   * associada ao mesmo.
   */
  private Selector selector;

  /** Pool de threads para a execuo das callbacks de tratamento. */
  private ExecutorService executorsPool;
  
  /** 
   * Mapa associando uma chave s informaes necessrias para realizar a
   * transferncia, para garantir a autenticao do cliente.
   */
  private Map<String, DoTransferInfo> transferInfoMap;
  
  /**
   * Porta em que o servidor est aguardando pedidos de conexes para 
   * transferncia de arquivos.
   */
  private int listenPort;
  
  /** 
   * Lista de solicitaes de mudanas no conjunto de interesses das chaves
   * registradas no seletor.
   */
  private List<ChangeRequest> pendingChanges;

  /** Entidade que deseja transmitir arquivos entre cliente e servidor. */
  private FileTransferrer transferrer;
  
  /** Tamanho do buffer para transferncia. */
  private int bufferSize;
  
  /**
   * Cria um multiplexador (<code>Selector</code>) no qual sero registrados
   * sockets. Sempre que esse multiplexador receber um evento do SO, vai
   * verificar se existe algum socket interessado e disparar a callback 
   * associada ao mesmo.
   * 
   * @param transferrer entidade que deseja permitir a transferncia de
   *        arquivos entre cliente e servidor.
   */
  public FileServer(FileTransferrer transferrer) {
    this.transferrer = transferrer;
    transferInfoMap = new HashMap<String, DoTransferInfo>();
    pendingChanges = new LinkedList<ChangeRequest>();
    try {
      Configuration configuration = 
        ConfigurationManager.getInstance().getConfiguration(getClass());
      String value = 
        configuration.getOptionalProperty("buffer.size", "1048576");
      bufferSize = Integer.parseInt(value);
      value = configuration.getOptionalProperty("pool.size", "50");
      int poolSize = Integer.parseInt(value);
      executorsPool = Executors.newFixedThreadPool(poolSize);
    } 
    catch (ConfigurationManagerException e) {
      throw new ConfigurationException(e);
    }
    try {
      selector = Selector.open();
      ServerSocketChannel serverChannel = createServerSocketChannel();
      serverChannel.register(selector, SelectionKey.OP_ACCEPT);
    }
    catch (ClosedChannelException e) {
      transferrer.logSevereMessage(
        "No foi possvel registrar o socket do servidor.", e);
    }
    catch (IOException e) {
      transferrer.logSevereMessage(
        "Erro de I/O ao instanciar o servidor.", e);
    }
  }

  /**
   * Dispara o tratamento de eventos nos canais registrados junto ao servidor.
   * ATENO: Este mtodo bloqueia aguardando eventos, portanto geralmente
   * deve ser invocado em uma nova thread.
   */
  public void dispatch() {
    try {
      while (true) {
        synchronized (this) {
          for (ChangeRequest change : pendingChanges) {
            SelectionKey key = change.getKey();
            switch (change.getType()) {
              case CHANGE_OPS: 
                key.interestOps(change.getOps());
                break;
              case CANCEL:
                key.channel().close();
                key.cancel();
                break;
              default:
                throw new IllegalStateException(
                  "Tipo desconhecido: "+change.getType());
            }
          }
          pendingChanges.clear();
        }
        int selectedKeysCount = 0;
        selectedKeysCount = selector.select();
        if (selectedKeysCount == 0) continue;
        Iterator it = selector.selectedKeys().iterator();
        while (it.hasNext()) {
          SelectionKey key = (SelectionKey) it.next();
          if (!key.isValid()) {
            continue;
          }
          if (key.isAcceptable()) {
            acceptConnection(bufferSize, key);
          }
          else if (key.isReadable() || key.isWritable()) {
            // Necessrio para que esse evento no seja novamente selecionado
            // at que o tratador termine de processar.  responsabilidade do
            // tratador renovar o interesse (se for o caso) aps o 
            // processamento.
            stopSelecting(key);
            handleEvent(key);
          }
          it.remove();
        }
      }
    }
    catch (IOException e) {
      transferrer.logSevereMessage(
        "No foi possvel tratar um evento do servidor.", e);
    }
  }

  /**
   * Acrescenta uma lista de solicitao de mudana no estado de uma chave 
   * lista de requisies pendentes.
   * 
   * @param change nova solicitao de mudana.
   */
  synchronized void addChangeRequest(ChangeRequest change) {
    pendingChanges.add(change);
    selector.wakeup();
  }
  
  /**
   * Cria um novo <code>ServerSocketChannel</code>, no-bloqueante, na 
   * primeira porta disponvel. 
   * 
   * @return <code>ServerSocketChannel</code>
   * 
   * @throws IOException
   */
  private ServerSocketChannel createServerSocketChannel() throws IOException {
    ServerSocketChannel serverChannel = ServerSocketChannel.open();
    ServerSocket serverSocket = serverChannel.socket();
    serverSocket.bind(new InetSocketAddress(0));// Primeira porta disponvel
    listenPort = serverSocket.getLocalPort();
    serverChannel.configureBlocking(false);
    return serverChannel;
  }

  /**
   * Estabelece conexo com um socket cliente. Registra esse socket para eventos
   * de leitura (sempre que o socket cliente estiver disponvel para leitura, o
   * multiplexador ser avisado).
   *
   * @param capacity capacidade do buffer de transmisso de contedo.
   * @param serverKey chave contendo o socket do servidor.
   * 
   * @throws IOException
   */
  private void acceptConnection(int capacity, SelectionKey serverKey) 
  throws IOException {
    ServerSocketChannel serverChannel = 
      (ServerSocketChannel)serverKey.channel();
    SocketChannel clientChannel = serverChannel.accept();
    registerChannel(clientChannel, SelectionKey.OP_READ);
    SelectionKey clientKey = clientChannel.keyFor(selector);
    ServerSimpleFileTransferProtocol protocol = 
      new ServerSimpleFileTransferProtocol(
        this, capacity, clientKey, clientChannel, transferrer);
    clientKey.attach(protocol);
  }

  /**
   * Registra o canal especificado no multiplexador.
   * 
   * @param channel canal a ser registrado.
   * @param ops eventos (operaes) a observar no canal.
   * 
   * @throws IOException
   */
  private void registerChannel(SelectableChannel channel, int ops) 
  throws IOException {
    if (channel == null) {
      return;
    }
    channel.configureBlocking(false);
    channel.register(selector, ops);
  }

  /**
   * Invoca o protocolo associado  chave especificada para tratar um evento 
   * ocorrido no canal.
   * 
   * @param key chave representando o evento.
   */
  private void handleEvent(SelectionKey key) {
    ServerSimpleFileTransferProtocol protocol = 
      (ServerSimpleFileTransferProtocol)key.attachment();
    if (protocol == null) {
      throw new IllegalStateException("Handler no encontrado!!!");
    }
    executorsPool.execute(protocol);
  }
  
  /**
   * Inibe a seleo de quaisquer eventos associados  chave especificada a
   * partir deste momento.
   * 
   * @param key chave associada a um canal, cujos eventos devem ser ignorados.
   */
  private void stopSelecting(SelectionKey key) {
    key.interestOps(0);
  }

  /**
   * Mtodo utilizado por um <code>FileTransferrer</code> para solicitar a 
   * importao de arquivos.
   * 
   * @param targetPath caminho de destino para a importao. 
   * @param notifyArgs informaes a serem repassadas ao objeto solicitante ao 
   *        trmino da importao.
   * @return objeto contendo as informaes necessrias para o cliente solicitar
   *         a transferncia. 
   * 
   * @throws DigestException se ocorrer algum problema na gerao da chave.
   * @throws IOException se o arquivo de destino no puder ser criado no 
   *         servidor.
   */
  public synchronized RequestTransferInfo prepareUpload(
    String targetPath, Map notifyArgs) 
    throws DigestException, IOException {
    new File(targetPath).createNewFile();
    String key = new SecureKey().digest();
    DoTransferInfo transferInfo = 
      new DoTransferInfo(Operation.IMPORT, targetPath, notifyArgs);
    transferInfoMap.put(key, transferInfo);
    return new RequestTransferInfo(key, listenPort);
  }


  /**
   * Mtodo utilizado por um <code>FileTransferrer</code> para solicitar a 
   * exportao de arquivos.
   * 
   * @param sourcePath caminho de origem para a exportao. 
   *
   * @return objeto contendo as informaes necessrias para o cliente solicitar
   *         a transferncia. 
   * 
   * @throws DigestException se ocorrer algum problema na gerao da chave.
   * @throws IOException 
   */
  public synchronized RequestTransferInfo prepareDownload(String sourcePath)
    throws DigestException, IOException {
    if (!(new File(sourcePath)).exists()) {
      throw new IOException("Arquivo "+sourcePath+" no existe no servidor.");
    }
    String key = new SecureKey().digest();
    DoTransferInfo transferInfo = 
      new DoTransferInfo(Operation.EXPORT, sourcePath);
    transferInfoMap.put(key, transferInfo);
    return new RequestTransferInfo(key, listenPort);
  }
  
  /**
   * Retorna as informaes necessrias para executar a transferncia.
   * 
   * @param key chave para localizar as informaes sobre a transferncia.
   * 
   * @return informaes necessrias para executar a transferncia.
   */
  synchronized DoTransferInfo getTransferInfo(String key) {
    return transferInfoMap.get(key);
  }

  /**
   * Callback invocada quando um canal com o cliente est para ser fechado,
   * indicando que uma importao acabou de ser realizada (este mtodo no 
   * invocado na exportao).
   * 
   * @param key chave representando a importao.
   */
  void channelClosing(String key) {
    DoTransferInfo info = getTransferInfo(key);
    transferrer.channelClosing(info.getNotifyArgs());
  }
  
}
