/*
 * CommandPersistenceService.java
 * 
 * $Id: CommandPersistenceService.java 97319 2009-10-09 00:45:10Z costa $
 */
package csbase.server.services.commandpersistenceservice;

import java.io.Closeable;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.concurrent.Executors;
import java.util.concurrent.ThreadFactory;

import tecgraf.javautils.FileUtils;
import csbase.exception.OperationFailureException;
import csbase.exception.ParseException;
import csbase.exception.ServiceFailureException;
import csbase.exception.algorithms.AlgorithmNotFoundException;
import csbase.logic.CommandFinalizationType;
import csbase.logic.CommandInfo;
import csbase.logic.CommandStatus;
import csbase.logic.Priority;
import csbase.logic.ProjectFile;
import csbase.logic.ProjectFileInfo;
import csbase.logic.ProjectFileType;
import csbase.logic.User;
import csbase.logic.algorithms.AlgorithmConfigurator;
import csbase.logic.algorithms.ExecutionType;
import csbase.logic.algorithms.FileParameterValue;
import csbase.logic.algorithms.commands.CommandPersistenceCommandNotificationData;
import csbase.logic.algorithms.commands.CommandPersistenceCommandNotificationData.Type;
import csbase.logic.algorithms.flows.configurator.FlowAlgorithmConfigurator;
import csbase.logic.algorithms.serializer.DefaultAlgorithmConfigurationSerializer;
import csbase.logic.algorithms.serializer.FlowAlgorithmConfigurationSerializer;
import csbase.logic.algorithms.serializer.IAlgorithmConfigurationSerializer;
import csbase.logic.algorithms.serializer.exception.AlgorithmConfigurationSerializerException;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.CommandPersistenceServiceInterface;
import csbase.server.ServerException;
import csbase.server.Service;
import csbase.server.services.notificationservice.NotificationService;
import csbase.server.services.projectservice.ProjectService;
import csbase.server.services.projectservice.ServerFileLockListener;
import csbase.server.services.projectservice.ServerProject;
import csbase.server.services.projectservice.ServerProjectFile;
import csbase.server.services.projectservice.ServerProjectFileException;
import csbase.util.callbak.ICallback;

/**
 * @see CommandPersistenceServiceInterface
 * @author TecGraf/PUC-Rio
 */
public final class CommandPersistenceService extends Service implements
  CommandPersistenceServiceInterface {

  /**
   * Fbrica de threads utilizada pelo mtodo
   * {@link #getCommandInfos(Object, long, ICallback)}. Ao ser chamado, este
   * mtodo cria uma thread permitindo retornar imediatamente. Essa thread fica
   * responsvel por carregar os comandos e enviar em blocos ao usurio.
   */
  private static ThreadFactory threadFactory = Executors.defaultThreadFactory();

  /** Nome do arquivo de redirecionamento da sada da execuo do comando. */
  private static final String OUT_LOG = "out.log";

  /** Nome do arquivo de captura do cdigo de sada do comando. */
  private static final String EXIT_CODE_LOG = "exit_code.log";

  /** Tempo mximo de espera pelo lock exclusivo em milesegundos. */
  private static final int LOCK_EXCLUSIVE_TIMEOUT = 4000;

  /** Tempo mximo de espera pelo lock compartilhado em milesegundos. */
  private static final int LOCK_SHARED_TIMEOUT = 1000;

  /**
   * Nome da propriedade que indica o nome do diretrio que ser usado para
   * armazenar os dados dos comandos executados.
   */
  private static final String COMMANDS_DIRECTORY_NAME_PROPERTY =
    "commandsDirectoryName";

  /**
   * Propriedade que indica o nome do diretrio que ser usado para armazenar os
   * dados dos comandos executados.
   */
  private String commandsDirectoryName;

  /**
   * O nome do diretrio de logs.
   */
  private static final String LOGS_DIRECTORY_NAME = "logs";

  /**
   * O nome do arquivo de propriedades do comando.
   */
  private static final String COMMAND_PROPERTIES_FILE_NAME = "cmd.properties";

  /**
   * O nome do arquivo contendo a configuraco de um algoritmo instalado na
   * rvore de algoritmos.
   */
  private static final String DEFAULT_CONFIGURATION_FILE_NAME =
    "cmd.parameters";

  /**
   * O tipo do arquivo contendo a configuraco de um algoritmo instalado na
   * rvore de algoritmos.
   */
  private static final String DEFAULT_CONFIGURATION_FILE_TYPE = "PARAMETERS";

  /**
   * O nome do arquivo contendo a configuraco de um fluxo.
   */
  private static final String FLOW_CONFIGURATION_FILE_NAME = "cmd.flx";

  /**
   * O tipo do arquivo contendo a configuraco de um fluxo.
   */
  private static final String FLOW_CONFIGURATION_FILE_TYPE = "FLX";

  /**
   * Construtor da classe.
   * 
   * @throws ServerException se houver erro na inicializao.
   */
  private CommandPersistenceService() throws ServerException {
    super(SERVICE_NAME);
    // XXX - Temporrio. Aguardando a soluo do Bug #5847.
    ClientRemoteLocator.commandPersistenceService = this;
    commandsDirectoryName = getStringProperty(COMMANDS_DIRECTORY_NAME_PROPERTY);
    /*
     * Remove caractres invlidos do nome do projeto. - Remove os espaos antes
     * e depois do nome. - Caractres invlidos so substituidos por '_'. -
     * Somente ser mantido o '.' se este for a primeira letra do nome.
     */
    commandsDirectoryName = commandsDirectoryName.trim();
    if (commandsDirectoryName.startsWith(".")) {
      commandsDirectoryName =
        '.' + FileUtils.fixDirectoryName(commandsDirectoryName.substring(1));
    }
    else {
      commandsDirectoryName = FileUtils.fixDirectoryName(commandsDirectoryName);
    }
  }

  /**
   * Retorno da instncia do servio.
   * 
   * @return o servio.
   */
  public static CommandPersistenceService getInstance() {
    return (CommandPersistenceService) getInstance(SERVICE_NAME);
  }

  /**
   * Constri a instncia do servio de escalonamento.
   * 
   * @throws ServerException se houver erro na inicializao.
   */
  public static void createService() throws ServerException {
    new CommandPersistenceService();
  }

  /**
   * Inicializao do servio
   */
  @Override
  public void initService() {
  }

  /**
   * Mtodo invocado no trmino do servio.
   */
  @Override
  public void shutdownService() {
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected boolean has2Update(Object arg, Object event) {
    return true;
  }

  /**
   * {@inheritDoc}
   */
  public void getCommandInfos(Object projectId, final long desiredResponseTime,
    final ICallback<Set<CommandInfo>> callback) {
    if (projectId == null) {
      throw new IllegalArgumentException("O parmetro projectId est nulo.");
    }
    if (callback == null) {
      throw new IllegalArgumentException("O parmetro callback est nulo.");
    }
    if (1 > desiredResponseTime) {
      throw new IllegalArgumentException(
        "O parmetro averageResponseTime tem que ser maior que 0.");
    }
    ServerProject project;
    try {
      project = getProjectById(projectId);
    }
    catch (OperationFailureException e) {
      logSevereMessage(String.format(
        "Erro ao tentar ler os comandos do projeto %s.\n", projectId), e);
      throw new ServiceFailureException(e.getMessage());
    }

    final List<ServerProjectFile> cmdDirectories =
      getCommandsDirectories(project);
    threadFactory.newThread(new Runnable() {

      @Override
      public void run() {
        int index = 0;
        ServerProjectFile cmdDirectory = null;
        long now = System.currentTimeMillis();
        /*
         * Enquanto existir comandos a serem lidos, l e envia para o cliente.
         */
        while (index < cmdDirectories.size()) {
          long end = now + desiredResponseTime;
          /*
           * L as propriedades dos comandos at que os comandos terminem ou o
           * tempo de resposta seja alcanado. Caso haja erro ao ler algum
           * comando, este ser ignorado.
           */
          Set<CommandInfo> cmdsInfo = new HashSet<CommandInfo>();
          for (; index < cmdDirectories.size() && now < end; index++) {
            try {
              cmdDirectory = cmdDirectories.get(index);
              CommandInfo commandInfo = readCommandInfoProperties(cmdDirectory);
              cmdsInfo.add(commandInfo);
            }
            catch (OperationFailureException e) {
              String message =
                String
                  .format(
                    "Erro ao tentar ler as informaes de comando do diretrio %s.\n",
                    cmdDirectory.getAbsolutePath());
              logSevereMessage(message, e);
            }
            now = System.currentTimeMillis();
          }
          /*
           * Envia os comandos lidos at o momento para o cliente atravs do
           * callback.
           */
          if (0 < cmdsInfo.size()) {
            try {
              callback.call(Collections.unmodifiableSet(cmdsInfo));
            }
            catch (Exception e) {
              String message =
                "Erro ao enviar as informaes de comandos ao usurio.\n";
              logSevereMessage(message, e);
            }
          }
        }
      }
    }).start();
  }

  /**
   * {@inheritDoc}
   */
  public Set<CommandInfo> getCommandInfos(Object projectId) {
    if (projectId == null) {
      throw new IllegalArgumentException("O parmetro projectId est nulo.");
    }
    ServerProject project;
    try {
      project = getProjectById(projectId);
    }
    catch (OperationFailureException e) {
      logSevereMessage(String.format(
        "Erro ao tentar ler os comandos do projeto %s.\n", projectId), e);
      throw new ServiceFailureException(e.getMessage());
    }
    //    System.out.println("Infos1"); // NO COMMITAR!
    Set<CommandInfo> cmdsInfo = new HashSet<CommandInfo>();
    List<ServerProjectFile> cmdDirectories = getCommandsDirectories(project);
    for (ServerProjectFile cmdDirectory : cmdDirectories) {
      try {
        /** XXX - O que fazer quando der problema em um arquivo. Abortamos tudo? */
        CommandInfo commandInfo = readCommandInfoProperties(cmdDirectory);
        cmdsInfo.add(commandInfo);
      }
      catch (OperationFailureException e) {
        String message =
          String.format(
            "Erro ao tentar ler as informaes do comando do diretrio %s.\n",
            cmdDirectory.getAbsolutePath());
        logSevereMessage(message, e);
      }
    }
    //    System.out.println("Infos2"); // Quando o formato est errado no chega aqui!
    return Collections.unmodifiableSet(cmdsInfo);
  }

  /**
   * {@inheritDoc}
   */
  public CommandInfo getCommandInfo(Object projectId, final String commandId) {
    if (projectId == null)
      throw new IllegalArgumentException("O parmetro projectId est nulo.");
    if (commandId == null)
      throw new IllegalArgumentException("O parmetro commandId est nulo.");
    try {
      ServerProject project = getProjectById(projectId);
      ServerProjectFile commandDirectory =
        getCommandDirectory(project, commandId);
      return readCommandInfoProperties(commandDirectory);
    }
    catch (OperationFailureException e) {
      String message = getMessage("error_read_command_info", commandId);
      throw new ServiceFailureException(message, e);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void updateCommandDescription(Object projectId, String commandId,
    String description) {
    if (projectId == null)
      throw new IllegalArgumentException("O parmetro projectId est nulo.");

    if (commandId == null)
      throw new IllegalArgumentException("O parmetro commandId est nulo.");

    CommandInfo command = getCommandInfo(projectId, commandId);
    command.setDescription(description);
    updateCommandInfo(command);
  }

  /**
   * Atualiza as informaes do comando.
   * 
   * @param command comando a ser atualizado.
   */
  public void updateCommandInfo(CommandInfo command) {
    if (command == null)
      throw new IllegalArgumentException("O parmetro command est nulo.");

    String commandId = command.getId();
    Object projectId = command.getProjectId();
    try {
      ServerProject project = getProjectById(projectId);
      saveCommandInfoProperties(project, command);
      notifyUsers(project.getAllUsers(true),
        new CommandPersistenceCommandNotificationData(projectId, commandId,
          Type.UPDATED));
      //      setChanged();
      //      notifyObservers(new CommandPersistenceCommandWasUpdatedEvent(projectId,
      //        commandId));
    }
    catch (OperationFailureException e) {
      String message = getMessage("error_save_command_info", commandId);
      throw new ServiceFailureException(message, e);
    }
  }

  /**
   * {@inheritDoc}
   */
  public boolean removeCommandInfo(Object projectId, String commandId) {
    try {
      ServerProject project = getProjectById(projectId);
      ServerProjectFile cmdDirectory = getCommandDirectory(project, commandId);
      if (cmdDirectory == null) {
        return false;
      }
      cmdDirectory.remove();
      notifyUsers(project.getAllUsers(true),
        new CommandPersistenceCommandNotificationData(projectId, commandId,
          Type.REMOVED));
      //      setChanged();
      //      notifyObservers(new CommandPersistenceCommandWasRemovedEvent(projectId,
      //        commandId));
      return true;
    }
    catch (OperationFailureException e) {
      String message = getMessage("error_remove_command_info", commandId);
      throw new ServiceFailureException(message, e);
    }
    catch (ServerProjectFileException e) {
      String message = getMessage("error_remove_command_info", commandId);
      throw new ServiceFailureException(message, e);
    }
  }

  /**
   * {@inheritDoc}
   */
  public boolean[] removeCommandInfos(List<Object> projectIds,
    List<String> commandIds) {
    boolean[] result = new boolean[commandIds.size()];
    Arrays.fill(result, false);

    for (int inx = 0; inx < commandIds.size(); inx++) {
      result[inx] = removeCommandInfo(projectIds.get(inx), commandIds.get(inx));
    }

    return result;
  }

  /**
   * <p>
   * Salva um comando no projeto informado.
   * </p>
   * <p>
   * Caso o comando tenha sido salvo com sucesso, gera uma notificao
   * {@link CommandPersistenceCommandNotificationData} do tipo
   * {@link CommandPersistenceCommandNotificationData.Type#SAVED}.
   * </p>
   * 
   * @param command O comando a ser salvo (No aceita {@code null}).
   * 
   * @throws OperationFailureException Se houver uma falha ao salvar o comando.
   */
  public void saveCommandInfo(CommandInfo command)
    throws OperationFailureException {

    if (command == null) {
      throw new IllegalArgumentException("O parmetro command est nulo.");
    }
    AlgorithmConfigurator configurator;
    try {
      configurator = command.getConfigurator();
    }
    catch (RemoteException e) {
      throw new IllegalArgumentException(String.format(
        "No foi possvel determinar um nome para o arquivo de metadados do "
          + "configurador do comando %s.\n", command.getId()));
    }
    String configurationFileName;
    String configurationFileType;
    IAlgorithmConfigurationSerializer configurationSerializer;
    if ((configurator.getAlgorithmName() != null)
      && (configurator.getAlgorithmVersionId() != null)) {
      configurationFileName = DEFAULT_CONFIGURATION_FILE_NAME;
      configurationFileType = DEFAULT_CONFIGURATION_FILE_TYPE;
      configurationSerializer = new DefaultAlgorithmConfigurationSerializer();
    }
    else if (configurator instanceof FlowAlgorithmConfigurator) {
      configurationFileName = FLOW_CONFIGURATION_FILE_NAME;
      configurationFileType = FLOW_CONFIGURATION_FILE_TYPE;
      configurationSerializer = new FlowAlgorithmConfigurationSerializer();
    }
    else {
      throw new IllegalArgumentException(String.format(
        "No foi possvel determinar um nome para o arquivo de metadados do "
          + "configurador do comando %s.\n", command.getId()));
    }
    Object projectId = command.getProjectId();
    ServerProject project = getProjectById(projectId);
    createProjectFiles(project, command, configurationFileName,
      configurationFileType);

    try {
      saveCommandInfoProperties(project, command);
      saveAlgorithmConfiguration(project, command, configurationFileName,
        configurationSerializer);
    }
    catch (OperationFailureException ofe) {
      try {
        removeCommandInfo(command.getProjectId(), command.getId());
      }
      catch (ServiceFailureException sfe) {
        //logSevereMessage(String.format("Erro ao tentar remover os arquivos gerados para persistir o comando %s.\n", commandId), e);
      }
      throw ofe;
    }

    /**
     * Notifica a todos os interessados que o comando foi salvo com sucesso.
     */
    notifyUsers(project.getAllUsers(true),
      new CommandPersistenceCommandNotificationData(command.getProjectId(),
        command.getId(), Type.SAVED));
  }

  /**
   * Salva as propriedades de um comando e os seus parmetros
   * 
   * @param project projeto a partir do qual o comando foi executado
   * @param command informaes do comando
   * @param configurationFileName nome do arquivo de parmetros do comando
   * @param configurationSerializer responsvel por persistir as configuraes
   *        do comando
   * @return stream de sada
   * @throws OperationFailureException
   */
  private OutputStream saveAlgorithmConfiguration(ServerProject project,
    CommandInfo command, String configurationFileName,
    IAlgorithmConfigurationSerializer configurationSerializer)
    throws OperationFailureException {
    String commandId = command.getId();
    ServerProjectFile cmdDirectory = getCommandDirectory(project, commandId);
    if (cmdDirectory == null) {
      String message =
        getMessage("error_command_directory_not_found", commandId);
      throw new OperationFailureException(message);
    }
    ServerProjectFile configurationFile =
      (ServerProjectFile) cmdDirectory.getChild(configurationFileName);
    if (configurationFile == null) {
      String message =
        getMessage("error_file_not_found", cmdDirectory.getCanonicalPath(),
          configurationFileName);
      throw new OperationFailureException(message);
    }
    OutputStream output = null;
    Object lockId = null;
    try {
      ServerFileLockListener lockListener = new ServerFileLockListener();
      lockId =
        configurationFile.acquireExclusiveLock(lockListener,
          LOCK_EXCLUSIVE_TIMEOUT);
      if (lockListener.getLock()) {
        output = configurationFile.openOutputStream();
        AlgorithmConfigurator configurator = command.getConfigurator();
        configurationSerializer.write(configurator, output);
      }
      else {
        String message =
          String.format(
            getString("CommandPersistenceService.error.get.exclusive.lock"),
            configurationFile.getCanonicalPath());
        throw new OperationFailureException(message);
      }
    }
    catch (ServerProjectFileException e) {
      throw new OperationFailureException(String.format(
        getMessage("error.open_configuration_file"), commandId,
        configurationFileName));
    }
    catch (AlgorithmConfigurationSerializerException e) {
      throw new OperationFailureException(String.format(
        getMessage("error.parse_configuration_file"), commandId,
        configurationFileName));
    }
    catch (RemoteException e) {
      throw new OperationFailureException(String.format(
        getMessage("error.no_configurator"), commandId, configurationFileName));
    }
    finally {
      close(output);
      if (lockId != null) {
        configurationFile.releaseLock(lockId);
      }
    }
    return output;
  }

  /**
   * Cria os diretrios e arquivos onde so persistidas as informaes dos
   * comandos.
   * 
   * @param project projeto
   * @param command informaes do comando
   * @param configurationFileName nome do arquivo onde so gravadas as
   *        configuraes do comando
   * @param configurationFileType tipo do arquivo de configurao
   * @return o projeto onde foram persistidas as informaes do comando
   * @throws OperationFailureException
   */
  private ServerProject createProjectFiles(ServerProject project,
    CommandInfo command, String configurationFileName,
    String configurationFileType) throws OperationFailureException {

    AlgorithmConfigurator configurator;
    try {
      configurator = command.getConfigurator();
    }
    catch (RemoteException e) {
      throw new IllegalArgumentException(String.format(
        "No foi possvel determinar um nome para o arquivo de metadados do "
          + "configurador do comando %s.\n", command.getId()));
    }

    String commandId = command.getId();
    try {
      String commandDirectoryName = getCommandDirectoryName(commandId);
      List<ProjectFileInfo> projectFileInfos =
        new LinkedList<ProjectFileInfo>();
      if (getCommandRepositoryDirectory(project) == null) {
        // Cria o diretrio de comandos se ele no existir
        projectFileInfos.add(new ProjectFileInfo(
          new String[] { commandsDirectoryName },
          ProjectFileType.DIRECTORY_TYPE));
      }
      projectFileInfos.add(new ProjectFileInfo(new String[] {
          commandsDirectoryName, commandDirectoryName },
        ProjectFileType.DIRECTORY_TYPE));
      projectFileInfos.add(new ProjectFileInfo(new String[] {
          commandsDirectoryName, commandDirectoryName, configurationFileName },
        configurationFileType));
      projectFileInfos.add(new ProjectFileInfo(new String[] {
          commandsDirectoryName, commandDirectoryName,
          COMMAND_PROPERTIES_FILE_NAME }, "TEXT"));
      projectFileInfos.add(new ProjectFileInfo(new String[] {
          commandsDirectoryName, commandDirectoryName, LOGS_DIRECTORY_NAME },
        ProjectFileType.DIRECTORY_TYPE));

      // Atribui valor ao log de sada padro do algoritmo.
      String outputFilePath =
        FileUtils.joinPath(File.separatorChar, commandsDirectoryName,
          commandDirectoryName, LOGS_DIRECTORY_NAME, OUT_LOG);
      FileParameterValue outputFile =
        new FileParameterValue(outputFilePath, "LOG");
      configurator.setStandardOutputFile(outputFile);
      /*
       * Cria os arquivos de log. Pede-se os arquivos a serem criados ao
       * configurador pois, se ele for de fluxo, ele ter criado arquivos para
       * cada um de seus ns a partir do arquivo atribuido acima.
       */
      for (FileParameterValue stdout : configurator.getStandardOutputFiles()) {
        projectFileInfos.add(new ProjectFileInfo(stdout.getPathAsArray(),
          stdout.getType()));
      }

      if (configurator.hasExitCode()) {
        String exitCodeLogFilePath =
          FileUtils.joinPath(File.separatorChar, commandsDirectoryName,
            commandDirectoryName, LOGS_DIRECTORY_NAME, EXIT_CODE_LOG);
        FileParameterValue exitCodeLogFile =
          new FileParameterValue(exitCodeLogFilePath, "LOG");
        configurator.setExitCodeLogFile(exitCodeLogFile);

        for (FileParameterValue exitLog : configurator.getExitCodeLogFiles()) {
          projectFileInfos.add(new ProjectFileInfo(exitLog.getPathAsArray(),
            exitLog.getType()));
        }
      }

      ServerProjectFile rootFile = (ServerProjectFile) project.getTree();
      Object userId = command.getUserId();
      rootFile.createFiles(projectFileInfos, userId);
    }
    catch (ServerProjectFileException e) {
      throw new OperationFailureException(getMessage(
        "error_create_project_files", commandId), e);
    }
    return project;
  }

  /**
   * Obtm o repositrio de comandos de um projeto especfico.
   * 
   * @param project O projeto (No aceita {@code null}).
   * 
   * @return O repositrio ou {@code null} se no existir.
   */
  private ServerProjectFile getCommandRepositoryDirectory(
    final ServerProject project) {
    ServerProjectFile projectRootFile = (ServerProjectFile) project.getTree();
    return (ServerProjectFile) projectRootFile.getChild(commandsDirectoryName);
  }

  /**
   * <p>
   * Obtm os diretrios de comandos do repositrio de comandos do projeto
   * fornecido.
   * </p>
   * 
   * @param project O projeto (No aceita {@code null}).
   * 
   * @return A lista de diretrios (se no houver comandos, a lista estar
   *         vazia).
   */
  private List<ServerProjectFile> getCommandsDirectories(
    final ServerProject project) {
    ServerProjectFile commandRepositoryDirectory =
      getCommandRepositoryDirectory(project);
    if (commandRepositoryDirectory == null) {
      return Collections.emptyList();
    }
    ProjectFile[] children = commandRepositoryDirectory.getChildren();
    List<ServerProjectFile> files =
      new ArrayList<ServerProjectFile>(children.length);
    for (ProjectFile child : children) {
      files.add((ServerProjectFile) child);
    }
    return files;
  }

  /**
   * Obtm um projeto especfico.
   * 
   * @param projectId O identificador do projeto (No aceita {@code null}).
   * 
   * @return O projeto.
   * 
   * @throws OperationFailureException Se o projeto no existir.
   */
  private ServerProject getProjectById(Object projectId)
    throws OperationFailureException {
    ServerProject project = ProjectService.getInstance().getProject(projectId);
    if (project == null) {
      String message = getMessage("error_project_not_found", projectId);
      throw new OperationFailureException(message);
    }
    return project;
  }

  /**
   * L as informaes do comando de um diretrio do comando fornecido. Apenas
   * as propriedades do comando so carregadas.
   * 
   * @param commandDirectory O diretrio do comando (No aceita {@code null}).
   * 
   * @return As informaes.
   * 
   * @throws OperationFailureException Se houver um erro ao ler as informaes.
   */
  private CommandInfo readCommandInfoProperties(
    final ServerProjectFile commandDirectory) throws OperationFailureException {
    ServerProjectFile commandPropertyFile =
      (ServerProjectFile) commandDirectory
        .getChild(COMMAND_PROPERTIES_FILE_NAME);
    if (commandPropertyFile == null) {
      String message =
        getMessage("error_command_properties_not_found", commandDirectory
          .getAbsolutePath(), COMMAND_PROPERTIES_FILE_NAME);
      throw new OperationFailureException(message);
    }

    InputStream inputStream = null;
    Object lockId = null;
    try {
      ServerFileLockListener lockListener = new ServerFileLockListener();
      lockId =
        commandPropertyFile
          .acquireSharedLock(lockListener, LOCK_SHARED_TIMEOUT);
      if (lockListener.getLock()) {
        Object projectId = commandDirectory.getProjectId();
        inputStream = commandPropertyFile.openInputStream();
        CommandInfo command =
          readCommandInfoPropertiesFromStream(projectId, inputStream);
        return command;
      }
      else {
        String message =
          String.format(
            getString("CommandPersistenceService.error.get.shared.lock"),
            commandPropertyFile.getCanonicalPath());
        throw new OperationFailureException(message);
      }
    }
    catch (IOException e) {
      String message =
        getMessage("error_read_file", commandPropertyFile.getCanonicalPath());
      throw new OperationFailureException(message, e);
    }
    catch (ServerProjectFileException e) {
      String message =
        getMessage("error_open_file", commandPropertyFile.getCanonicalPath());
      throw new OperationFailureException(message, e);
    }
    catch (ParseException e) {
      String message =
        getMessage("error_file_format", commandPropertyFile.getCanonicalPath());
      throw new OperationFailureException(message, e);
    }
    finally {
      close(inputStream);
      if (lockId != null) {
        commandPropertyFile.releaseLock(lockId);
      }
    }
  }

  /**
   * Salva o arquivo {@value #COMMAND_PROPERTIES_FILE_NAME} de um comando
   * fornecido.
   * 
   * @param project O projeto (No aceita {@code null}).
   * @param command O comando (No aceita {@code null}).
   * 
   * @throws OperationFailureException Se houver algum erro ao escrever no
   *         arquivo.
   */
  private void saveCommandInfoProperties(final ServerProject project,
    final CommandInfo command) throws OperationFailureException {
    String commandId = command.getId();
    ServerProjectFile commandDirectory =
      getCommandDirectory(project, commandId);
    if (commandDirectory == null) {
      String message =
        getMessage("error_command_directory_not_found", commandId);
      throw new ServiceFailureException(message);
    }
    ServerProjectFile commandPropertiesFile =
      (ServerProjectFile) commandDirectory
        .getChild(COMMAND_PROPERTIES_FILE_NAME);
    if (commandPropertiesFile == null) {
      String message =
        getMessage("error_file_not_found", commandDirectory.getCanonicalPath(),
          COMMAND_PROPERTIES_FILE_NAME);
      throw new OperationFailureException(message);
    }
    OutputStream outputStream = null;
    Object lockId = null;
    try {
      ServerFileLockListener lockListener = new ServerFileLockListener();
      lockId =
        commandPropertiesFile.acquireExclusiveLock(lockListener,
          LOCK_EXCLUSIVE_TIMEOUT);
      if (lockListener.getLock()) {
        outputStream = commandPropertiesFile.openOutputStream();
        saveCommandInfoPropertiesToStream(outputStream, command);
      }
      else {
        String message =
          String.format(
            getString("CommandPersistenceService.error.get.exclusive.lock"),
            commandPropertiesFile.getCanonicalPath());
        throw new OperationFailureException(message);
      }
    }
    catch (ServerProjectFileException e) {
      String message =
        getMessage("error_write_command_properties", commandId,
          commandPropertiesFile.getCanonicalPath());
      throw new OperationFailureException(message, e);
    }
    catch (IOException e) {
      String message =
        getMessage("error_write_command_properties", commandId,
          commandPropertiesFile.getCanonicalPath());
      throw new OperationFailureException(message, e);
    }
    finally {
      close(outputStream);
      if (lockId != null) {
        commandPropertiesFile.releaseLock(lockId);
      }
    }
  }

  /**
   * Persiste as propriedades do comando.
   * 
   * @param outputStream stream de sada
   * @param command informaes do comando
   * @throws IOException caso ocorra falha na persistncia do comando
   */
  private void saveCommandInfoPropertiesToStream(OutputStream outputStream,
    CommandInfo command) throws IOException {
    CommandPropertyParser parser = new CommandPropertyParser();
    parser.setId(command.getId());
    parser.setUserId(command.getUserId());
    parser.setSubmittedDate(command.getSubmittedDate());
    parser.setWallTimeSec(command.getWallTimeSec());
    parser.setGlobalPosition(command.getGlobalPosition());
    parser.setMailAtEnd(command.isMailAtEnd());
    parser.setPriority(command.getPriority());
    parser.setStatus(command.getStatus());
    parser.setFinalizationType(command.getFinalizationType());
    parser.setDescription(command.getDescription());
    parser.setSGAName(command.getSGAName());

    parser.setPlatformFilter(command.getPlatformFilter());
    parser.setTip(command.getTip());
    parser.setExecutionType(command.getConfigurator().getExecutionType());

    parser.setIsAutomatic(command.isAutomatic());
    boolean isAutomatic = command.isAutomatic();
    ExecutionType executionType = command.getConfigurator().getExecutionType();
    switch (executionType) {
      case PARALLEL:
        parser.setCpuCountForParallelExecution(command
          .getCpuCountForParallelExecution());
        if (!isAutomatic) {
          parser.setSelectedSGAsNames(command.getSelectedSGAsNames());
        }
        break;
      case MULTIPLE:
        if (isAutomatic) {
          parser.setExecutionCountForMultipleExecution(command
            .getExecutionCountForMultipleExecution());
        }
        else {
          parser.setExecutionCountPerSGAForMultipleExecution(command
            .getExecutionCountPerSGAForMultipleExecution());
          parser.setSelectedSGAsNames(command.getSelectedSGAsNames());

        }
        break;
      case DISTRIBUTED:
        if (isAutomatic) {
          parser.setSgaCountForDistributedExecution(command
            .getSGACountForDistributedExecution());
        }
        else {
          parser.setExecutionCountPerSGAForDistributedExecution(command
            .getExecutionCountPerSGAForDistributedExecution());
          parser.setSelectedSGAsNames(command.getSelectedSGAsNames());
        }
        break;
      case SIMPLE:
        if (!isAutomatic) {
          parser.setSelectedSGAsNames(command.getSelectedSGAsNames());
        }
        break;
      case XAPPS:
        if (!isAutomatic) {
          parser.setSelectedSGAsNames(command.getSelectedSGAsNames());
        }
        break;
      default:
        throw new IllegalStateException(String.format(
          "Tipo de execuo invlido no arquivo de persistncia %s.",
          executionType));
    }
    parser.saveProperties(outputStream);
  }

  /**
   * Cria um comando a partir da leitura de seu arquivo de propriedades.
   * 
   * @param projectId O identificador do projeto (No aceita {@code null}).
   * @param inputStream O stream de entrada com o contedo do arquivo (No
   *        aceita {@code null}).
   * @return commandInfo O comando lido.
   * 
   * @throws ParseException Em caso de erro no formato do arquivo.
   * @throws IOException Se houver um erro de ES ao ler no stream.
   */
  private CommandInfo readCommandInfoPropertiesFromStream(Object projectId,
    InputStream inputStream) throws ParseException, IOException {
    CommandPropertyParser parser = new CommandPropertyParser();
    parser.loadProperties(inputStream);

    String id = parser.getId();
    String userId = parser.getUserId();
    Date submittedDate = parser.getSubmittedDate();
    int globalPosition = parser.getGlobalPosition();
    boolean mailAtEnd = parser.getMailAtEnd();
    Priority priority = parser.getPriority();
    CommandStatus status = parser.getStatus();
    CommandFinalizationType finalizationType =
      parser.getFinalizationType(status);
    String tip = parser.getTip();
    CommandInfo commandInfo =
      new CommandInfo(id, globalPosition, mailAtEnd, projectId, submittedDate,
        userId, priority, status, finalizationType, tip);

    Integer wallTimeSec = parser.getWallTimeSec();
    if (wallTimeSec != null) {
      commandInfo.setWallTimeSec(wallTimeSec);
    }
    String description = parser.getDescription();
    if (description != null) {
      commandInfo.setDescription(description);
    }
    String sgaName = parser.getSGAName();
    if (sgaName != null) {
      commandInfo.setSGAName(sgaName);
    }
    String platformFilter = parser.getPlatformFilter();
    if (platformFilter != null) {
      commandInfo.setPlatformFilter(platformFilter);
    }

    boolean isAutomatic = parser.getIsAutomatic();
    ExecutionType executionType = parser.getExecutionType();
    switch (executionType) {
      case DISTRIBUTED:
        if (isAutomatic) {
          int sgaCount = parser.getSgaCountForDistributedExecution();
          commandInfo.configureDistributedExecution(sgaCount);
        }
        else {
          int executionCountPerSGA =
            parser.getExecutionCountPerSGAForDistributedExecution();
          List<String> selectedSGAsNames = parser.getSelectedSGAsNames();
          commandInfo.configureDistributedExecution(selectedSGAsNames,
            executionCountPerSGA);
        }
        break;
      case MULTIPLE:
        if (isAutomatic) {
          int executionCount = parser.getExecutionCountForMultipleExecution();
          commandInfo.configureMultipleExecution(executionCount);
        }
        else {
          int executionCountPerSGA =
            parser.getExecutionCountPerSGAForMultipleExecution();
          List<String> selectedSGAsNames = parser.getSelectedSGAsNames();
          commandInfo.configureMultipleExecution(selectedSGAsNames,
            executionCountPerSGA);
        }
        break;
      case PARALLEL:
        int cpuCount = parser.getCpuCountForParallelExecution();
        if (isAutomatic) {
          commandInfo.configureParallelExecution(cpuCount);
        }
        else {
          String selectedSgaName = parser.getSelectedSGAName();
          commandInfo.configureParallelExecution(selectedSgaName, cpuCount);
        }
        break;
      case SIMPLE:
        if (isAutomatic) {
          commandInfo.configureSimpleExecution();
        }
        else {
          String selectedSgaName = parser.getSelectedSGAName();
          commandInfo.configureSimpleExecution(selectedSgaName);
        }
        break;
      case XAPPS:
        if (isAutomatic) {
          commandInfo.configureXAppExecution();
        }
        else {
          String selectedSgaName = parser.getSelectedSGAName();
          commandInfo.configureXAppExecution(selectedSgaName);
        }
        break;
      default:
        throw new IllegalStateException(String.format(
          "Tipo de execuo invlido no arquivo de persistncia %s.",
          executionType));
    }
    return commandInfo;
  }

  /**
   * {@inheritDoc}
   */
  public AlgorithmConfigurator readAlgorithmConfigurator(Object projectId,
    final String commandId) {
    if (projectId == null)
      throw new IllegalArgumentException("O parmetro projectId est nulo.");
    if (commandId == null)
      throw new IllegalArgumentException("O parmetro commandId est nulo.");
    ServerProject project;
    try {
      project = getProjectById(projectId);
    }
    catch (OperationFailureException e) {
      String message = getMessage("error_read_command_info", commandId);
      throw new ServiceFailureException(message, e);
    }
    ServerProjectFile commandDirectory =
      getCommandDirectory(project, commandId);
    IAlgorithmConfigurationSerializer configuratorSerializer =
      new DefaultAlgorithmConfigurationSerializer();
    ServerProjectFile configurationFile =
      (ServerProjectFile) commandDirectory
        .getChild(DEFAULT_CONFIGURATION_FILE_NAME);
    if (configurationFile == null) {
      configuratorSerializer = new FlowAlgorithmConfigurationSerializer();
      configurationFile =
        (ServerProjectFile) commandDirectory
          .getChild(FLOW_CONFIGURATION_FILE_NAME);
    }
    if (configurationFile == null) {
      throw new ServiceFailureException(getMessage(
        "error_configuration_file_not_found", commandDirectory
          .getCanonicalPath()));
    }
    InputStream inputStream = null;
    Object lockId = null;
    try {
      ServerFileLockListener lockListener = new ServerFileLockListener();
      lockId =
        configurationFile.acquireSharedLock(lockListener, LOCK_SHARED_TIMEOUT);
      if (lockListener.getLock()) {
        inputStream = configurationFile.openInputStream();
        AlgorithmConfigurator configurator =
          configuratorSerializer.read(inputStream);
        if (configurator.getAdditionalLogsDirectory() == null) {
          //Seta o diretrio padro para arquivos de log adicionais.
          String additionalLogsDirPath =
            FileUtils.joinPath(File.separatorChar, commandsDirectoryName,
              getCommandDirectoryName(commandId), LOGS_DIRECTORY_NAME);
          FileParameterValue additionalLogsDirectory =
            new FileParameterValue(additionalLogsDirPath);
          configurator.setAdditionalLogsDirectory(additionalLogsDirectory);
        }
        return configurator;
      }
      else {
        String message =
          String
            .format(
              getString("CommandPersistenceService.error.get.shared.lock.config"),
              commandId);
        throw new ServiceFailureException(message);
      }
    }
    catch (ServerProjectFileException e) {
      String message =
        getMessage("error_read_file", configurationFile.getCanonicalPath());
      throw new ServiceFailureException(message, e);
    }
    catch (AlgorithmConfigurationSerializerException e) {
      String message =
        getMessage("error_create_configurator", configurationFile
          .getCanonicalPath());
      throw new ServiceFailureException(message, e);
    }
    catch (AlgorithmNotFoundException e) {
      throw new ServiceFailureException(e.getMessage(), e);
    }
    finally {
      close(inputStream);
      if (lockId != null) {
        configurationFile.releaseLock(lockId);
      }
    }
  }

  /**
   * Obtm o caminho do diretrio onde devem ser gerados os arquivos de log.
   * Este caminho  dado a partir do diretrio do projeto (no includo).
   * 
   * @param commandId O identificador do comando (No aceita {@code null}).
   * 
   * @return O caminho do diretrio.
   */
  public String[] getLogDirectory(String commandId) {
    String commandDirectoryName = getCommandDirectoryName(commandId);
    return new String[] { commandsDirectoryName, commandDirectoryName,
        LOGS_DIRECTORY_NAME };
  }

  /**
   * Obtm o diretrio aonde foram/sero salvos os dados do comando.
   * 
   * @param project O projeto (No aceita {@code null}).
   * @param commandId O identificador do comando (No aceita {@code null}).
   * 
   * @return O diretrio do comando ou {@code null} se o comando no existir.
   */
  private ServerProjectFile getCommandDirectory(ServerProject project,
    String commandId) {

    ServerProjectFile commandRepositoryDirectory =
      getCommandRepositoryDirectory(project);
    if (commandRepositoryDirectory == null) {
      return null;
    }
    String commandDirectoryName = getCommandDirectoryName(commandId);
    return (ServerProjectFile) commandRepositoryDirectory
      .getChild(commandDirectoryName);
  }

  /**
   * Obtm o nome do diretrio do comando a partir do identificador do comando.
   * 
   * @param commandId O identificador do comando (No aceita {@code null}).
   * 
   * @return O nome do diretrio.
   */
  private String getCommandDirectoryName(final String commandId) {
    String commandDirectoryName = FileUtils.fixDirectoryName(commandId);
    return commandDirectoryName;
  }

  /**
   * Notifica usurios sobre eventos ocorridos com comandos.
   * 
   * @param allUsers Usurios a serem notificados alm do admin.
   * @param notification Notificao a ser enviada indicando se um comando foi
   *        salvo, removido ou atualizado na persistncia.
   */
  public void notifyUsers(Collection<Object> allUsers,
    CommandPersistenceCommandNotificationData notification) {
    NotificationService ntSrv = NotificationService.getInstance();

    for (Object projectUserId : allUsers) {
      ntSrv.notifyUserFromService(this, projectUserId, notification, false);
    }
    /*
     * Como o administrador tem acesso a todos os comandos, ele deve receber
     * suas notificaes de trmino. Porm, essa notificao no deve ser
     * exposta ao usurio administrador, ela deve apenas ser tratada pelo seu
     * cliente.
     */
    Object adminId = User.getAdminId();
    if (!allUsers.contains(adminId)) {
      ntSrv.notifyUserFromService(this, adminId, notification, false);
    }
  }

  /**
   * Fecha um fluxo de dados sem gerar exceo.
   * 
   * @param stream O fluxo de dados. Se for {@code null}, o mtodo no faz nada.
   */
  private void close(final Closeable stream) {
    if (stream != null) {
      try {
        stream.close();
      }
      catch (IOException e) {
        printStackTrace(e);
      }
    }
  }

  /**
   * Imprime na sada de erro as pilhas de execuo da execeo de todas as suas
   * causas.
   * 
   * @param throwable A exceo (No aceita {@code null}).
   */
  private void printStackTrace(final Throwable throwable) {
    Throwable currentThrowable = throwable;
    while (currentThrowable != null
      && currentThrowable != currentThrowable.getCause()) {
      currentThrowable.printStackTrace();
      currentThrowable = currentThrowable.getCause();
    }
  }

  /**
   * Obtm uma mensagem no mecanismo de internacionalizao.
   * 
   * @param keySuffix O sufixo da chave (o nome da classe ser pr-fixado ao
   *        sufixo) da mensagem a ser procurada no servico de lingugem.
   * @param args Possveis argumentos utilizados na formataco da mensagem
   *        utilizando para isso, o mtodo
   *        {@link String#format(String, Object...)}
   * 
   * @return A mensagem formatada.
   */
  private String getMessage(final String keySuffix, final Object... args) {
    String key =
      CommandPersistenceService.class.getSimpleName() + "." + keySuffix;
    String format = getString(key);
    return String.format(format, args);
  }
}
