package csbase.server.services.dbmanagerservice;

import java.io.File;
import java.io.FileInputStream;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.Iterator;
import java.util.Properties;
import java.util.logging.FileHandler;
import java.util.logging.Level;
import java.util.logging.Logger;

import csbase.server.Server;
import csbase.server.ServerException;
import csbase.server.ServerLogFormatter;

/**
 * Classe que implementa a definio de um pool para conexo com um banco de
 * dados qualquer.
 * 
 */
public abstract class Pool {

  /** tamanho mximo em bytes do arquivo de log */
  private final int LOGGER_LIMIT = 1048576;

  /** qtde de arquivos a ser usado pelo log */
  private final int LOGGER_COUNT = 6;

  /** adicionar no final do arquivo de log ? */
  private final boolean LOGGER_APPEND = true;

  /** nome identificador do pool */
  protected String name;

  /** url jdbc para conexo com o banco */
  protected String url;

  /** Usurio do banco para conexo */
  protected String user;

  /** Senha do banco para conexo */
  protected String password;

  /** Driver do banco */
  protected String driver;

  /** Propriedades do pool carregadas pelo arquivo de propriedades. */
  private Properties properties = null;

  /** Logger a ser usado pelo pool. */
  public Logger logger;

  /**
   * Inicializa logger e carrega properties
   * 
   * @param name Nome dado a este pool
   * 
   * @throws ServerException
   */
  public Pool(String name) throws ServerException {
    this.name = name;
    final Server server = Server.getInstance();
    String serviceLogFileName = server.getLogRootDirectoryName()
      + File.separator + this.getClass().getSimpleName() + ".log";
    initServiceLog(serviceLogFileName);
    String propertyFileName = Server.getPropertiesRootDirectoryName()
      + File.separator + this.getClass().getSimpleName() + ".properties";
    loadProperties(propertyFileName);
    logInfoMessage(properties.toString());
  }

  /**
   * @return Retorna uma conexo do pool
   */
  public abstract Connection getConnection();

  /**
   * Verifica se a senha utilizada com o banco mudou.
   * 
   * @param password A senha para testar.
   */
  public abstract void checkPassword(String password);

  /**
   * Devolve a conexo para o pool
   * 
   * @param conn
   * @param stmt
   * @param rs
   */
  public abstract void releaseConnection(Connection conn, Statement stmt,
    ResultSet rs);
  
  /**
   * Devolve a conexo para o pool
   * 
   * @param conn
   * @param stmt
   * @param rs
   * @param dispose Se true a conexo ser descartada.
   */
  public abstract void releaseConnection(Connection conn, Statement stmt,
    ResultSet rs, boolean dispose);

  /** Finaliza o pool */
  public abstract void destroy();

  /**
   * Inicializa o pool
   * 
   * @return true se inicializao feita com sucesso
   */
  public abstract boolean init();

  /**
   * A senha para conexo no banco
   * 
   * @param password
   */
  public final void setPassword(String password) {
    this.password = password;
  }

  /**
   * Usurio do banco
   * 
   * @param user
   */
  public final void setUser(String user) {
    this.user = user;
  }

  /**
   * Url jdbc
   * 
   * @param url
   */
  public final void setUrl(String url) {
    this.url = url;
  }

  /**
   * Driver usado pelo pool
   * 
   * @param driver
   */
  public void setDriver(String driver) {
    this.driver = driver;
  }

  /**
   * @return Retorna senha utilizada por este pool
   */
  public final String getPassword() {
    return password;
  }

  /**
   * @return Retorna url de conexo com o banco
   */
  public final String getUrl() {
    return url;
  }

  /**
   * @return Usurio de conexo com o banco
   */
  public final String getUser() {
    return user;
  }

  /**
   * @return Driver de conexo com o banco
   */
  public final String getDriver() {
    return driver;
  }

  /**
   * @param key a chave da propriedade
   * @return Retorna uma propriedade do tipo int do pool
   * @throws ServerException
   */
  public final int getIntProperty(String key) throws ServerException {
    try {
      return Integer.parseInt(getProperty(key));
    }
    catch (Exception e) {
      logger.log(Level.SEVERE, "Erro lendo propriedade: " + key);
      throw new ServerException(e);
    }
  }

  /**
   * @param key chave da propriedade
   * @return Retorna uam propriedade do tipo long do pool
   * @throws ServerException
   */
  public final long getLongProperty(String key) throws ServerException {
    try {
      return Long.parseLong(getProperty(key));
    }
    catch (Exception e) {
      logger.log(Level.SEVERE, "Erro lendo propriedade: " + key, e);
      throw new ServerException(e);
    }
  }

  /**
   * @param key chave da propriedade
   * @return Retorna uma propriedade do pool
   * @throws ServerException
   */
  public final String getProperty(String key) throws ServerException {
    String v = properties.getProperty(key);
    if (v == null) {
      logger.log(Level.SEVERE, "Erro lendo propriedade: " + key);
      throw new ServerException("Propriedade no encontrada: " + key);
    }
    return v;
  }

  /**
   * Nome identificador deste pool
   * 
   * @return
   */
  public final String getName() {
    return name;
  }

  /**
   * @return Propriedades deste pool
   */
  public final Properties getProperties() {
    return properties;
  }

  /**
   * @return Retorna o logger do pool
   */
  public final Logger getLogger() {
    return logger;
  }

  /**
   * Gerao de log de informao.
   * 
   * @param msg a mensagem a ser gerada.
   */
  public final void logInfoMessage(String msg) {
    if (logger == null)
      System.out.println(msg);
    else
      logger.info(msg);
  }

  /**
   * Gerao de log severo
   * 
   * @param msg A mensagem a ser logada
   * @param t Exceo ocorrida a ser logada
   */
  public final void logSevereMessage(String msg, Throwable t) {
    if (this.logger == null) {
      System.err.println(msg);
      t.printStackTrace();
    }
    else
      logger.log(Level.SEVERE, msg, t);
  }

  /**
   * Gerao de log severo
   * 
   * @param msg A mensagem a ser logada
   */
  public final void logSevereMessage(String msg) {
    if (this.logger == null)
      System.err.println(msg);
    else
      logger.log(Level.SEVERE, msg);
  }

  /**
   * Carrega as propriedades do pool contidas no arquivo que possui o mesmo nome
   * da classe que implementa Pool.
   * 
   * @param propertyFileName
   * @throws ServerException
   */
  private void loadProperties(String propertyFileName) throws ServerException {
    properties = new Properties();
    try {
      final FileInputStream in = new FileInputStream(propertyFileName);
      properties.load(in);
      in.close();
      Properties trimmedProps = new Properties();
      Iterator it = properties.keySet().iterator();
      while (it.hasNext()) {
        String k = (String) it.next();
        String v = properties.getProperty(k);
        trimmedProps.setProperty(k, v.trim());
      }
      properties = trimmedProps;
    }
    catch (Exception e) {
      logSevereMessage("Erro lendo " + propertyFileName, e);
      throw new ServerException(e);
    }
  }

  /**
   * Inicializao do log de execuo do pool.
   * 
   * @param serviceLogFileName nome dado ao logger
   * 
   * @throws ServerException se houver falha de inicializao.
   */
  public void initServiceLog(String serviceLogFileName) throws ServerException {
    try {
      logger = Logger.getLogger(serviceLogFileName);
      logger.setUseParentHandlers(false);
      FileHandler serviceLogFileHandler = new FileHandler(serviceLogFileName,
        LOGGER_LIMIT, LOGGER_COUNT, LOGGER_APPEND);
      logger.addHandler(serviceLogFileHandler);
    }
    catch (Exception e) {
      throw new ServerException(e);
    }
  }
}
