/*
 * $Id: DiskUsageService.java 94183 2009-07-27 17:01:05Z clinio $
 */
package csbase.server.services.diskusageservice;

import java.io.File;
import java.util.Collection;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;

import csbase.exception.InvalidRequestException;
import csbase.logic.FileSystemSpaceNotificationData;
import csbase.logic.User;
import csbase.logic.diskusageservice.DiskOccupation;
import csbase.remote.DiskUsageServiceInterface;
import csbase.server.FileSystem;
import csbase.server.ServerException;
import csbase.server.Service;
import csbase.server.services.eventlogservice.EventLogService;
import csbase.server.services.mailservice.MailService;
import csbase.server.services.notificationservice.NotificationService;

/**
 * A classe <code>DiskUsageService</code> implementa o servio de uso de espao
 * de disco.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class DiskUsageService extends Service implements
  DiskUsageServiceInterface {

  /**
   * Identificador de rea de projetos.
   */
  private static final String DIR_PROJECT_ID = "project";

  /**
   * Identificador de rea do servidor.
   */
  private static final String DIR_SERVER_ID = "server";

  /**
   * Flag indicativo de sada da thread de auditagem de usurios
   */
  private boolean exitAuditThread = false;

  /**
   * Thread que faz a auditagem do espao em disco ocupado.
   */
  private Thread auditThread = null;

  /**
   * Lista de diretrios a serem monitorados.
   */
  final private HashMap<String, MonitoredDirectory> monitoredList =
    new HashMap<String, MonitoredDirectory>();

  /**
   * Valor default de alerta ao administrador para ocupao de espao em disco.
   */
  final private int DEFAULT_WARNING_PERC = 75;

  /**
   * Valor default de alerta ao administrador para ocupao de espao em disco.
   */
  final private int DEFAULT_ALERT_PERC = 90;

  /**
   * Percentual (de espao em disco) para aviso ao administrador. Esse valor 
   * lido das properties do servio e tem seu valor default defindo por
   * {@link #DEFAULT_WARNING_PERC}.
   */
  private int warningPerc = DEFAULT_WARNING_PERC;

  /**
   * Percentual (de espao em disco) para alerta ao administrador. Esse valor 
   * lido das properties do servio e tem seu valor default defindo por
   * {@link #DEFAULT_ALERT_PERC}
   */
  private int alertPerc = DEFAULT_ALERT_PERC;

  /**
   * Indicativo de envio de email para admin no caso de alerta.
   */
  private boolean mailOnAlert;

  /**
   * Busca da referncia ao servio.
   * 
   * @return o objeto do servidor <code>DiskUsageService</code>.
   */
  public static DiskUsageService getInstance() {
    return (DiskUsageService) getInstance(SERVICE_NAME);
  }

  /**
   * Criao da thread que faz a gravao de auditagem.
   * 
   * @see #initService()
   */
  private void createAuditThread() {
    /*
     * Busca das propriedades que identificam os tempos para gravao
     * (auditagem) dos dados em disco
     */
    final int ONE_HOUR = 1;
    long auditIntervalHours = getIntProperty("auditIntervalHours");
    if (auditIntervalHours < ONE_HOUR) {
      auditIntervalHours = ONE_HOUR;
    }
    logInfoMessage("Intervalo de auditagens: " + auditIntervalHours + " horas.");
    final long sleepTime = auditIntervalHours * 60 * 60 * 1000;
    // final long sleepTime = 5 * 1000;
    auditThread = new Thread(new Runnable() {
      public void run() {
        while (!exitAuditThread) {
          try {
            Thread.sleep(sleepTime);
            logDebugMessage("Thread iniciada: createAuditThread.");
            auditData();
            logDebugMessage("Thread terminada: createAuditThread.");
          }
          catch (InterruptedException ie) {
            logWarningMessage("Gravao de auditagem interrompida!");
          }
        }
        logWarningMessage("Finalizando thread de gravao de auditagem...");
      }
    });

    warningPerc = getIntProperty("warningPercentage");
    alertPerc = getIntProperty("alertPercentage");

    warningPerc = Math.abs(warningPerc);
    alertPerc = Math.abs(alertPerc);

    final int minWarnPerc = 10;
    final int minAlertPerc = 20;
    if (warningPerc < minWarnPerc) {
      warningPerc = minWarnPerc;
    }
    if (alertPerc < minAlertPerc) {
      alertPerc = minAlertPerc;
    }

    final int maxPerc = 100;
    if (warningPerc > maxPerc) {
      warningPerc = maxPerc;
    }
    if (alertPerc > maxPerc) {
      alertPerc = maxPerc;
    }

    if (alertPerc <= warningPerc) {
      alertPerc = warningPerc + 1;
    }

    logInfoMessage("Percentual para aviso: " + warningPerc + "%");
    logInfoMessage("Percentual para alerta: " + alertPerc + "%");

    exitAuditThread = false;
    
    final String threadPrefixName = getClass().getSimpleName();
    auditThread.setName(threadPrefixName+"::auditThread");
    auditThread.start();
  }

  /**
   * Consulta ao usurio.
   * 
   * @param userId o identificador do usurio
   * @return o usurio
   */
  private User getUser(final Object userId) {
    try {
      final User user = User.getUser(userId);
      if (user == null) {
        logSevereMessage("Detectado usurio nulo para: " + userId);
      }
      return user;
    }
    catch (Exception ue) {
      logSevereMessage("Falha na deteco de usurio!");
    }
    return null;
  }

  /**
   * Consuta ao diretrio do usurio para avaliao.
   * 
   * @param userId identificador do usurio.
   * 
   * @return String com o diretrio.
   */
  private String getUserDirectoryName(final Object userId) {
    final String sep = File.separator;
    final String pt = ".";
    final User user = getUser(userId);
    if (user == null) {
      return null;
    }
    return pt + sep + FileSystem.PROJECT_DIR + sep + user.getId();
  }

  /**
   * Consuta ao diretrio do usurio para avaliao.
   * 
   * @param userId identificador do usurio.
   * @param ownerId identificador do owner.
   * @param projName identificador do projeto
   * 
   * @return String com o diretrio.
   */
  private String getUserProjectDirectoryName(final Object userId,
    final Object ownerId, final String projName) {
    final String uPath = getUserDirectoryName(ownerId);
    final String sep = File.separator;
    return uPath + sep + projName;
  }

  /**
   * Montagem de uma hash de ocupao por usurio.
   * 
   * @return uma hash table
   */
  private Hashtable<Object, Double> mountUserHash() {
    final Hashtable<Object, Double> hash = new Hashtable<Object, Double>();
    try {
      final List<User> all = User.getAllUsers();
      for (int i = 0; i < all.size(); i++) {
        final User user = all.get(i);
        double mega = -9;
        try {
          mega = getUsedSpaceForUserMb(user.getId());
        }
        catch (Exception e) {
          mega = 0;
        }
        hash.put(user.getId(), new Double(mega));
      }
      return hash;
    }
    catch (Exception e) {
      return null;
    }
  }

  /**
   * Formatador de nmero para arquivo de auditagem.
   * @param value valor
   * @return string
   */
  private String formatToAudit(final double value) {
    final String str = String.format("%.3f", value);
    return str;
  }
  /**
   * Faz a auditagem do espao em disco da rea do servidor e de projetos.
   */
  private void auditData() {
    final String eventLogPrefix = "disk";
    final Collection<MonitoredDirectory> monitors = monitoredList.values();
    for (MonitoredDirectory monitor : monitors) {
      final String id = monitor.getId();
      final DiskOccupation occupation = getDiskOccupation(id);
      final EventLogService eventService = EventLogService.getInstance();
      final String usedFS = "[" + id + "]";
      final String[] usedQueue =
        new String[] { eventLogPrefix, id + "-file-system" };
      final double usedSrvPerc = occupation.getUsedSpacePerc();
      final double usedSrvMb = occupation.getUsedSpaceMb();
      final double totalSrvMb = occupation.getTotalSpaceMb();
      final String[] usedInfo =
        new String[] { usedFS, formatToAudit(usedSrvPerc),
        formatToAudit(usedSrvMb), formatToAudit(totalSrvMb) };
      eventService.addServerInformation(usedQueue, usedInfo);
      notifyAdminIfNeeded(id, occupation);
    }
  }

  /**
   * Notifica o administrador o estado de um file system (se necessrio)
   * 
   * @param fsName nome do file system.
   * @param occupation a ocupao do file system.
   */
  private void notifyAdminIfNeeded(final String fsName,
    final DiskOccupation occupation) {
    final NotificationService ntSrv = NotificationService.getInstance();
    final double percentage = occupation.getUsedSpacePerc();

    if (percentage >= alertPerc) {
      final FileSystemSpaceNotificationData data =
        new FileSystemSpaceNotificationData(fsName, percentage, alertPerc,
          false);
      final Object notifId = ntSrv.notifyUserFromService(this, User.ADMIN_LOGIN, data);
      if (notifId == null) {
        final String percMsg = percentage + "%";
        final String err = "Falha de notificao de alerta do administrador: " + fsName + " - " + percMsg;
        logSevereMessage(err);
      }

      if (mailOnAlert) {
        final MailService mailSrv = MailService.getInstance();
        mailSrv.mailUserFromService(this, User.ADMIN_LOGIN, data.toString());
      }
    }
    else {
      if (percentage >= warningPerc) {
        final FileSystemSpaceNotificationData data =
          new FileSystemSpaceNotificationData(fsName, percentage, warningPerc,
            true);
        final Object notifId = ntSrv.notifyUserFromService(this, User.ADMIN_LOGIN, data);
        if (notifId == null) {
          final String percMsg = percentage + "%";
          final String err = "Falha de notificao de warning do administrador: " + fsName + " - " + percMsg;
          logSevereMessage(err);
        }
      }
    }
  }

  /**
   * Retorno do tamanho ocupado.
   * 
   * @param dirName o nome remetente.
   * @return espao em megabytes.
   */
  private double getUsedMegaBytes(final String dirName) {
    final File dir = new File(dirName);
    return getDiskUsageMb(dir);
  }

  /**
   * Avaliao do espao (Mb) ocupado por uma estrutura do tipo File do Java que
   * retorna o tamanho do arquivo ou busca recursivamente outros 'Files' nos
   * diretrios.
   * 
   * @param dir o diretrio de pesquisa.
   * @return espao em megabytes.
   */
  private double getDiskUsageMb(final File dir) {
    double nb = 0;
    final File[] files = dir.listFiles();
    if (files == null) {
      return 0;
    }
    for (int i = 0; i < files.length; i++) {
      final File file = files[i];
      if (file.isDirectory()) {
        final double du = getDiskUsageMb(file);
        nb = nb + du;
      }
      else {
        double du = (file.length()) / 1024.0 / 1024.0;
        nb = nb + du;
      }
    }
    return nb;
  }

  /**
   * {@inheritDoc}
   */
  final public Hashtable<Object, Double> getUsersList() {
    return mountUserHash();
  }

  /**
   * {@inheritDoc}
   */
  final public double getUsedSpaceForUserMb(final Object userId) {
    if (userId == null) {
      throw new InvalidRequestException("Usurio no identificado: id nulo.");
    }
    final String dirName = getUserDirectoryName(userId);
    if (dirName == null) {
      logSevereMessage("Usurio " + userId + "no tem diretrio!");
      return -1;
    }
    return getUsedMegaBytes(dirName);
  }

  /**
   * {@inheritDoc}
   */
  final public double getUsedSpaceForProjectMb(final Object ownerId,
    final String projName) {
    if (ownerId == null) {
      throw new InvalidRequestException("Dono do projeto no identificado");
    }
    if (projName == null) {
      throw new InvalidRequestException("Nome do projeto no identificado");
    }
    Object userId = Service.getUser().getId();
    final String dirName =
      getUserProjectDirectoryName(userId, ownerId, projName);

    if (dirName == null) {
      final String id = ownerId + File.separator + projName;
      logSevereMessage("Usurio/Projeto [" + id + "] no tem diretrio!");
      return -1;
    }
    return getUsedMegaBytes(dirName);
  }

  /**
   * {@inheritDoc}
   */
  final public DiskOccupation getServerOccupation() {
    final DiskOccupation oc = getDiskOccupation(DIR_SERVER_ID);
    return oc;
  }

  /**
   * {@inheritDoc}
   */
  final public DiskOccupation getProjectOccupation() {
    final DiskOccupation oc = getDiskOccupation(DIR_PROJECT_ID);
    return oc;
  }

  /**
   * Atualiza os dados de um FileSystem.
   * 
   * @param id o path do fileSystem a atualizar
   * @return a ocupao
   */
  private DiskOccupation getDiskOccupation(final String id) {
    if (!monitoredList.containsKey(id)) {
      logSevereMessage("No localizar id :" + id + ".");
      return new DiskOccupation();
    }

    final MonitoredDirectory md = monitoredList.get(id);
    final File directory = md.getDirectory();
    if (!directory.exists()) {
      final String path = directory.getAbsolutePath();
      logWarningMessage("No foi possvel localizar diretrio: " + path
        + ". Diretrio ou arquivo no existe.");
      return new DiskOccupation();
    }

    final String path = directory.getAbsolutePath();
    final long total = FileSystem.getTotalSpace(path);
    final long free = FileSystem.getFreeSpace(path);

    //    final long total = FileSystem.getTotalSpace(path);
    //    final long used = (long)(getDiskUsageMb(file) * 1024. * 1024.);
    //    final long free = total - used;

    return new DiskOccupation(free, total);
  }

  /**
   * Trmino do servio.
   * 
   * @throws ServerException se houver exceo no servidor.
   */
  @Override
  public void shutdownService() throws ServerException {
    logInfoMessage("Terminando servio de utilizao de disco...");
    exitAuditThread = true;
    if (auditThread != null) {
      auditThread.interrupt();
    }
    auditThread = null;
  }

  /**
   * Inicializao do servio de gerncia de SGAs.
   * 
   * @throws ServerException se ocorrer um erro na inicializao
   */
  @Override
  public void initService() throws ServerException {
    logInfoMessage("Iniciando servio de utilizao de disco...");

    // Monitorao default: rea do servidor.
    final String serverPath = ".";
    addToMonitor(DIR_SERVER_ID, serverPath);

    // Monitorao default: rea de projetos.
    final String projectPath = serverPath + File.separator + "project";
    addToMonitor(DIR_PROJECT_ID, projectPath);

    final String idPropName = "monitor.id";
    final List<String> ids = getStringListProperty(idPropName);
    final String pathPropName = "monitor.path";
    final List<String> paths = getStringListProperty(pathPropName);
    if (ids.size() != paths.size()) {
      final String prefix = "Tamanhos distintos nas propriedades: ";
      final String err = prefix + "["+ idPropName+ "]  e [" + pathPropName + "]";
      throw new ServerException(err);
    }
    
    final int size = ids.size();
    for (int i = 0; i < size; i++) {
      final String id = ids.get(i);
      final String path = paths.get(i);
      addToMonitor(id, path);
    }

    mailOnAlert = getBooleanProperty("mailOnAlert");

    boolean auditData = getBooleanProperty("enableAudit");
    if (auditData) {
      createAuditThread();
      logInfoMessage("Auditagem de ocupao de disco habilitada.");
    }
    else {
      auditThread = null;
      logInfoMessage("Auditagem de ocupao de disco desabilitada.");
    }
    logInfoMessage("Iniciado o servio de utilizao de disco.");
  }

  /**
   * Constri a instncia do servio.
   * 
   * @throws ServerException se houver exceo no servidor.
   */
  public static void createService() throws ServerException {
    new DiskUsageService();
  }

  /**
   * Adiciona um item a monitorao.
   * 
   * @param id identificador.
   * @param path path do diretrio
   * @throws ServerException em caso de m configurao.
   */
  private void addToMonitor(final String id, final String path)
    throws ServerException {
    if (monitoredList.containsKey(id)) {
      final String err = "Id de diretrio duplicado: " + id;
      throw new ServerException(err);
    }
    logInfoMessage("rea monitorada (" + id + "): " + path);
    monitoredList.put(id, new MonitoredDirectory(id, path));
  }

  /**
   * Construtor protegido.
   * 
   * @throws ServerException se ocorrer um erro na inicializao.
   */
  protected DiskUsageService() throws ServerException {
    super(SERVICE_NAME);
  }

  /**
   * {@inheritDoc}
   * 
   * Retorna sempre verdadeiro.
   */
  @Override
  protected boolean has2Update(Object arg, Object event) {
    return true;
  }

}
