/*
 * $Id: EventLogService.java 91364 2009-05-11 19:50:37Z clinio $
 */

package csbase.server.services.eventlogservice;

import java.io.File;
import java.io.FileWriter;
import java.rmi.RemoteException;
import java.text.MessageFormat;
import java.util.Date;

import csbase.exception.ServiceFailureException;
import csbase.logic.User;
import csbase.remote.EventLogServiceInterface;
import csbase.server.*;

/**
 * A classe <code>EventLogService</code> implementa o servio de log de eventos,
 * que precisem ser auditados no servidor. O servio apresenta o conceito de
 * fila ou grupo de log que  definido hierarquicamente com um array de strings,
 * podendo atingir qualquer nvel de detalhe.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class EventLogService extends Service implements
  EventLogServiceInterface {

  /**
   * Nome da fila padro, caso seja enviado algum log-cliente sem a correta
   * definio de uma fila
   */
  final private static String NO_NAME = "no-name";

  /** Campo que armazena o sufixo dos arquivos de log-cliente. */
  private final String FILE_SUFIX = "log";

  /**
   * Criao de um identificador de arquivo nico para auditagem
   * 
   * @return uma string que representa um nmero nico.
   */
  private String createId() {
    final String format = "{0,date,yyyy-MM-dd}";
    final MessageFormat formatter = new MessageFormat(format);
    Object[] args = new Object[1];
    args[0] = new Date();
    StringBuffer text = new StringBuffer();
    formatter.format(args, text, null);
    return text.toString();
  }

  /**
   * Mtodo de montagem do nome do arquivo de log com base na fila.
   * 
   * @param queue fila de armazenamento
   * @return um nome de arquivo que define o log do grupo ou fila concatenado
   *         com <code>.xxx</code> (sufixo de arquivo definido).
   * @throws ServerException caso ocorra falha na verificao do diretrio
   */
  final private String getQueueFileName(final String[] queue)
    throws ServerException {
    final Server server = Server.getInstance();
    final String sep = File.separator;
    final String logDirName = server.getLogRootDirectoryName();
    final String eventDirName = logDirName + sep + "events";
    final File eventDir = new File(eventDirName);
    if (!eventDir.exists()) {
      final boolean lgMkdirsOk = eventDir.mkdirs();
      if (!lgMkdirsOk) {
        final String err = "Falha na criao de dir. de log: " + eventDirName;
        logSevereMessage(err);
        return null;
      }
    }

    final String pt = ".";
    final String id = createId();

    if (queue == null || queue.length < 1) {
      return eventDirName + sep + NO_NAME + pt + id + pt + FILE_SUFIX;
    }

    final int n = queue.length;
    if (n == 1) {
      final String fName = (queue[0] == null ? NO_NAME : queue[0]);
      return fName + pt + id + pt + FILE_SUFIX;
    }

    int min = 4;
    if (n < 5) {
      min = n - 1;
    }
    StringBuffer baseDirName = new StringBuffer(eventDirName);
    int i;
    for (i = 0; i < min; i++) {
      String dName = (queue[i] == null ? NO_NAME : queue[i]);
      baseDirName.append(sep + dName);
      server.checkDirectory(baseDirName.toString());
    }
    if (n < 5) {
      String fName = (queue[i] == null ? NO_NAME : queue[i]);
      return baseDirName + sep + fName + pt + id + pt + FILE_SUFIX;
    }

    String fName = baseDirName + sep;
    final String xName = (queue[4] == null ? NO_NAME : queue[4]);
    fName = fName + xName;
    for (i = 5; i < n; i++) {
      final String qName = (queue[i] == null ? NO_NAME : queue[i]);
      fName = fName + pt + qName;
    }
    return fName + pt + id + pt + FILE_SUFIX;
  }

  /**
   * Mtodo de montagem do nome de uma fila ou grupo de log-cliente, com base no
   * array de strings que o define. Caso o array seja invlido ou inconsistente,
   * ser retornado o nome padro.
   * 
   * @param q array de strings da fila.
   * @return um nome da fila
   */
  final private String getQueueName(final String[] q) {
    final String bar = "|";
    if (q == null || q.length <= 0)
      return NO_NAME;
    String qName = q[0];
    final int n = q.length;
    for (int i = 1; i < n; i++) {
      qName = qName + bar + q[i];
    }
    return "[" + qName + "]";
  }

  /**
   * Definio padro do mtodo abstrato de <code>Service</code>.
   */
  @Override
  final public boolean has2Update(Object o1, Object o2) {
    return true;
  }

  /**
   * Retorna a instncia do servio no lado servidor.
   * 
   * @return o objeto <code>EventLogService</code>
   */
  public static EventLogService getInstance() {
    return (EventLogService) getInstance(SERVICE_NAME);
  }

  /**
   * Adio de informao de eventlog relativa ao servidor.
   * 
   * @param queue a fila de dados a ser gravada (que ter o prefixo
   *        <code>"server"</code>.
   * @param info o array de informaes.
   * @return um indicativo do correto registro.
   */
  public boolean addServerInformation(final String[] queue, final String[] info) {
    String[] reQueue = null;
    if (queue == null || queue.length <= 0) {
      reQueue = new String[] { "server" };
    }
    else {
      final int N = queue.length;
      reQueue = new String[N + 1];
      reQueue[0] = "server";
      for (int i = 0; i < N; i++) {
        reQueue[i + 1] = queue[i];
      }
    }
    return addInformation(reQueue, info);
  }

  /**
   * {@inheritDoc}
   */
  public boolean addInformation(final String[] queue, final String[] info) {
    final String queueName = getQueueName(queue);
    try {
      final User user = Service.getUser();
      final String userName = (user == null ? " - " : user.getId().toString());
      final String fileName = getQueueFileName(queue);
      final File file = new File(fileName);
      if (!file.exists()) {
        logInfoMessage("Pedido de criao de: " + fileName);
        file.createNewFile();
        logInfoMessage("Arquivo feito: " + fileName);
        final String dt = (new Date()).toString();
        final FileWriter writer = new FileWriter(file);
        writer.write("# Arquivo de auditagem de eventos CSBASE.\n");
        writer.write("# Arquivo: [" + queueName + "]\n");
        writer.write("# Abertura em: " + dt + "\n");
        writer.write("# \n");
        writer.close();
        logInfoMessage("Header gravado em: " + fileName);
      }
      if (file.isDirectory()) {
        logSevereMessage("<<" + queueName + ">>: Local de escrita  diretrio!");
        throw new RemoteException("Directrio existe: " + fileName);
      }
      if (!file.canWrite()) {
        logSevereMessage("<<" + queueName + ">>: Sem permisso de escrita");
        throw new RemoteException("Sem permisso de escrita: " + fileName);
      }

      final FileWriter writer = new FileWriter(file, true);
      final Date dt = new Date();
      final long tm = dt.getTime();
      writer.write(tm + " ; " + dt + " ; " + userName);
      final String[] data =
        ((info == null || info.length == 0) ? new String[] { "*" } : info);

      for (int i = 0; i < data.length; i++) {
        writer.write(" ; " + data[i]);
      }
      writer.write("\n");
      writer.flush();
      writer.close();
      return true;
    }
    catch (Exception e) {
      throw new ServiceFailureException(e.getMessage(), e);
    }
  }

  /**
   * Inicializao do servio.
   * 
   * @throws ServerException se ocorrer um erro na inicializao
   */
  @Override
  final public void initService() throws ServerException {
    logInfoMessage("Servio de log-eventos iniciado.");
  }

  /**
   * Trmino do servio.
   * 
   * @throws ServerException se houver falha, o que no acontece.
   */
  @Override
  final public void shutdownService() throws ServerException {
    logInfoMessage("Servio de log de eventos terminado.");
  }

  /**
   * Constri a instncia do servio, atravs de seu construtor
   * <code>protected</code>.
   * 
   * @throws ServerException se houver exceo na instanciao.
   */
  public static void createService() throws ServerException {
    new EventLogService();
  }

  /**
   * Construtor padro que somente define o nome do servio com base em
   * <code>EventLogServiceInterface.SERVICE_NAME</code>.
   * 
   * @throws ServerException se houver falha na construo do servio.
   */
  protected EventLogService() throws ServerException {
    super(SERVICE_NAME);
  }
}
