/*
 * $Id: LanguageService.java 88276 2009-02-11 19:19:08Z clinio $
 */

package csbase.server.services.languageservice;

import java.io.InputStream;
import java.util.Hashtable;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.PropertyResourceBundle;
import java.util.ResourceBundle;

import csbase.remote.LanguageServiceInterface;
import csbase.server.ServerException;
import csbase.server.Service;
import csbase.server.services.loginservice.LoginService;

/**
 * Implementa a interface {@link LanguageServiceInterface} para prover o servio
 * de i18n para o Servidor. Para que um sistema carregue um bundle especfico 
 * necessrio especializar esse servio.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class LanguageService extends Service implements
  LanguageServiceInterface {

  /**
   * Constri a instncia do servio.
   * 
   * @throws ServerException se ocorrer erro.
   */
  public static void createService() throws ServerException {
    new LanguageService();
  }

  /**
   * Obtm uma instncia do servio.
   * 
   * @return LanguageService
   */
  public static LanguageService getInstance() {
    return (LanguageService) getInstance(SERVICE_NAME);
  }

  /**
   * Bundles carregados pelo servio.
   */
  protected Hashtable<Locale, ResourceBundle> bundles =
    new Hashtable<Locale, ResourceBundle>();

  /**
   * Traduz uma determinada chave na sua string correspondente de acordo com o
   * bundle carregado.
   * 
   * @param key chave do bundle.
   * 
   * @return String com o texto respectivo a chave.
   */
  public String get(final String key) {
    Locale userLocale = null;
    try {
      userLocale = getLocale();
    }
    catch (final Exception e) {
      logWarningMessage("No foi possvel determinar o locale do usurio. Usando default...");
      userLocale = getDefaultBackupLocale();
    }

    try {
      if (key == null) {
        return "(null key)";
      }
      if (!bundles.containsKey(userLocale)) {
        loadBundle(userLocale);
      }
      final ResourceBundle bundle = bundles.get(userLocale);
      if (bundle == null) {
        return "NULL BUNDLE [" + key + "]!!";
      }
      return bundle.getString(key);
    }
    catch (final MissingResourceException mre) {
      logSevereMessage("Texto no definido: " + key);
      return "<<<" + key + ">>>";
    }
    catch (final Exception e) {
      logSevereMessage("No foi possvel carregar bundle para locale: "
        + userLocale);
      return "<<<" + key + ">>>";
    }
  }

  /**
   * Retorna Locale de backup (em caso de falha na busca do default).
   * 
   * @return locale
   */
  private Locale getDefaultBackupLocale() {
    final String langKey = "default.language";
    final String countryKey = "default.country";
    if (!isPropertyNull(langKey) && !isPropertyNull(countryKey)) {
      final String language = getStringProperty(langKey);
      final String country = getStringProperty(countryKey);
      return new Locale(language, country);
    }

    System.out.println("Locale da sesso inexistente. Usando fallback...");
    return Locale.getDefault();
  }

  /**
   * Obtm o <i>locale</i> do usurio da <i>thread</i> corrente.
   * 
   * @return O <i>locale</i> do usurio.
   */
  protected Locale getLocale() {
    final LoginService loginService = LoginService.getInstance();
    final Locale locale = loginService.getUserSessionLocale(getKey());
    if (locale != null) {
      return locale;
    }
    System.out.println("Locale da sesso no encontrado. Usando default...");
    return getDefaultBackupLocale();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected boolean has2Update(final Object arg, final Object event) {
    return false;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void initService() throws ServerException {
  }

  /**
   * Faz a carga de um resource bundle com base em um locale.
   * 
   * @param locale locale
   * @throws Exception no caso de falha na carga do resource bundle.
   */
  protected void loadBundle(final Locale locale) throws Exception {
    final InputStream in =
      LanguageService.class
        .getResourceAsStream("/csbase/client/resources/properties/language/idiom_"
          + locale + ".properties");
    final ResourceBundle bundle = new PropertyResourceBundle(in);
    bundles.put(locale, bundle);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void shutdownService() throws ServerException {
  }

  /**
   * Construtor
   * 
   * @throws ServerException em caso de falha
   */
  protected LanguageService() throws ServerException {
    super(SERVICE_NAME);
  }
}
