/*
 * $Id$
 */
package csbase.server.services.openbusupdaterservice;

import java.io.Serializable;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;

import openbusidl.acs.Credential;

import org.omg.CORBA.UserException;

import tecgraf.openbus.Openbus;
import tecgraf.openbus.data_service.DataKey;
import tecgraf.openbus.data_service.InvalidDataKey;
import csbase.logic.User;
import csbase.server.ServerException;
import csbase.server.Service;
import csbase.server.services.loginservice.LoginService;
import csbase.server.services.loginservice.LoginServiceListener;
import csbase.server.services.openbusservice.OpenBusService;
import csbase.server.services.projectservice.ProjectService;
import csbase.server.services.projectservice.UpdaterService;
import csbase.server.services.projectservice.openbus.ProjectDataService;

/**
 * Servico de atualizaco de arquivos vindos do OpenBus.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class OpenBusUpdaterService extends Service implements UpdaterService {
  /**
   * O nome do servico.
   */
  private static final String SERVICE_NAME = "OpenBusUpdaterService";
  /**
   * Responsvel por agendar as tarefas de atualizaco de arquivos.
   */
  private Timer scheduler;
  /**
   * As tarefas de atualizaco de arquivos.
   */
  private Map<Path, TimerTask> tasks;
  /**
   * Trata notificaes de <i>logout</i> dos usurios para poder finalizar as
   * suas atualizaes.
   */
  private LoginServiceListenerImpl loginServiceListener;

  /**
   * Cria o servico de atualizaco.
   * 
   * @throws ServerException Caso ocorra algum erro na criaco do servico.
   */
  private OpenBusUpdaterService() throws ServerException {
    super(SERVICE_NAME);
    this.loginServiceListener = new LoginServiceListenerImpl();
    this.tasks = new HashMap<Path, TimerTask>();
  }

  /**
   * Cria a instncia nica do servio.
   * 
   * @throws ServerException Caso ocorra algum erro ao criar a instncia.
   */
  public static void createService() throws ServerException {
    new OpenBusUpdaterService();
  }

  /**
   * Obtm a instncia nica do servio.
   * 
   * @return A instncia nica do servio.
   */
  public static OpenBusUpdaterService getInstance() {
    return (OpenBusUpdaterService) getInstance(SERVICE_NAME);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void initService() {
    ProjectService.getInstance().addUpdaterService(this.getClass(),
      OpenBusUpdaterService.getInstance());
    this.scheduler = new Timer(true);
    LoginService.getInstance().addListener(this.loginServiceListener);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void shutdownService() {
    LoginService.getInstance().removeListener(this.loginServiceListener);
    this.scheduler.cancel();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected boolean has2Update(Object arg, Object event) {
    return false;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean startUpdate(String[] pathArray, final Serializable userData,
    long interval) {
    if (!OpenBusService.getInstance().isEnabled()) {
      return false;
    }
    Openbus bus = Openbus.getInstance();
    Credential serverCredential = bus.getCredential();
    final Credential userCredential =
      new Credential(serverCredential.identifier, serverCredential.owner,
        Service.getUser().getLogin());
    final Path path = new Path(pathArray);
    TimerTask task;
    synchronized (this.tasks) {
      if (this.tasks.containsKey(path)) {
        return true;
      }
      task = new TimerTask() {
        @Override
        public void run() {
          copyFrom(userCredential, path, (byte[]) userData);
        }
      };
      this.tasks.put(path, task);
    }

    this.scheduler.schedule(task, interval * 1000, interval * 1000);
    this.loginServiceListener.addFilePath(path);
    return true;
  }

  /**
   * Solicita a atualizao de um arquivo.<br>
   * OBS: este procedimento  assncrono.
   * 
   * @param userLogin O login do usurio que est executando a operao.
   * @param pathArray O caminho para o arquivo.
   * @param userData A chave (unvoca) do dado no OpenBus.
   * 
   * @return {@code true}, caso a atualizao seja solicitada, ou {@code false},
   *         caso o Servio do OpenBus esteja desabilitado.
   */
  public boolean singleUpdate(String userLogin, final String[] pathArray,
    final Serializable userData) {
    if (!OpenBusService.getInstance().isEnabled()) {
      return false;
    }
    Openbus bus = Openbus.getInstance();
    Credential serverCredential = bus.getCredential();
    final Credential userCredential =
      new Credential(serverCredential.identifier, serverCredential.owner,
        userLogin);
    final Path path = new Path(pathArray);
    TimerTask task;
    synchronized (this.tasks) {
      if (this.tasks.containsKey(path)) {
        return true;
      }
      task = new TimerTask() {
        @Override
        public void run() {
          copyFrom(userCredential, path, (byte[]) userData);
          stopUpdate(path);
        }
      };
      this.tasks.put(path, task);
    }

    this.scheduler.schedule(task, 0);
    return true;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean stopUpdate(String[] path, Serializable userData) {
    synchronized (this.tasks) {
      return this.stopUpdate(new Path(path));
    }
  }

  /**
   * Pra a atualizao de um arquivo.
   * 
   * @param path O caminho para o arquivo cuja atualizao foi solicitada.
   * 
   * @return {@code true} caso a atualizao seja terminada, ou {@code false},
   *         caso contrrio.
   */
  boolean stopUpdate(Path path) {
    synchronized (this.tasks) {
      TimerTask task = this.tasks.remove(path);
      if (task == null) {
        return true;
      }
      task.cancel();
    }
    this.loginServiceListener.removeFilePath(path);
    return true;
  }

  /**
   * /** Pra a atualizao de um conjunto de arquivos.
   * 
   * @param filePathSet O conjunto de caminhos dos arquivos cuja atualizao foi
   *        solicitada.
   */
  void stopUpdate(Set<Path> filePathSet) {
    synchronized (this.tasks) {
      for (Path filePath : filePathSet) {
        this.stopUpdate(filePath);
      }
    }
  }

  /**
   * Atualiza um arquivo.
   * 
   * @param credential A credencial do usurio solicitante (delegado).
   * @param path O caminho do arquivo de destino, relativo ao projeto.
   * @param sourceDataKey A chave do dado de origem do arquivo.
   */
  void copyFrom(Credential credential, Path path, byte[] sourceDataKey) {
    Openbus.getInstance().setThreadCredential(credential);
    String userId = credential.owner;
    if (!credential.delegate.equals("")) {
      userId = credential.delegate;
    }
    Service.setUserId(userId);

    ProjectDataService projectDataService =
      ProjectService.getInstance().getDataService();
    String targetDataId = ProjectDataService.generateDataId(path.path);
    DataKey targetDataKey;
    try {
      targetDataKey = projectDataService.generateDataKey(targetDataId);
    }
    catch (InvalidDataKey e) {
      this.logSevereMessage("Erro ao criar chave unvoca do arquivo.", e);
      return;
    }
    try {
      projectDataService.updateDataFrom(targetDataKey.getKey(), sourceDataKey);
    }
    catch (UserException e) {
      this.logSevereMessage("Erro ao atualizar arquivo.", e);
      return;
    }
  }

  /**
   * Representa um caminho de arquivo.
   * 
   * @author Tecgraf/PUC-Rio
   */
  private class Path {
    /**
     * As componentes do caminho do arquivo.
     */
    String[] path;

    /**
     * Constri um caminho de arquivo.
     * 
     * @param path As componentes do caminho do arquivo.
     */
    Path(String[] path) {
      this.path = path;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean equals(Object obj) {
      if (obj == null) {
        return false;
      }
      if (!obj.getClass().equals(String[].class)) {
        return false;
      }
      return Arrays.equals(this.path, (String[]) obj);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int hashCode() {
      return Arrays.hashCode(this.path);
    }
  }

  /**
   * Tratador de notificaes do Servio de Login.
   * 
   * @author Tecgraf/PUC-Rio
   */
  private class LoginServiceListenerImpl implements LoginServiceListener {
    /**
     * Informa os arquivos utilizados por um determinado usurio.
     */
    private Map<String, Set<Path>> usersToFilePathSet;

    /**
     * Cria um tratador de notificaes.
     */
    LoginServiceListenerImpl() {
      this.usersToFilePathSet = new HashMap<String, Set<Path>>();
    }

    /**
     * Adiciona um arquivo  lista do usurio corrente.
     * 
     * @param filePath O caminho do arquivo.
     */
    void addFilePath(Path filePath) {
      User user = Service.getUser();
      if (user == null) {
        return;
      }
      Set<Path> filePathSet = this.usersToFilePathSet.get(user.getLogin());
      if (filePathSet == null) {
        filePathSet = new HashSet<Path>();
      }
      filePathSet.add(filePath);
      this.usersToFilePathSet.put(user.getLogin(), filePathSet);
    }

    /**
     * Remove um arquivo da lista do usurio corrente.
     * 
     * @param filePath O caminho do arquivo.
     */
    void removeFilePath(Path filePath) {
      User user = Service.getUser();
      if (user == null) {
        return;
      }
      Set<Path> filePathSet = this.usersToFilePathSet.get(user.getLogin());
      filePathSet.remove(filePath);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void userLoggingOut(String login, Object sessionKey,
      String systemName, long time) {
      Set<Path> filePathSet = this.usersToFilePathSet.get(login);
      if (filePathSet != null) {
        stopUpdate(new HashSet<Path>(filePathSet));
      }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void systemNameSet(String login, Object sessionKey,
      String systemName, long time) {
      // Nada a ser feito.
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void sessionCreated(String login, Object sessionKey, long time) {
      // Nada a ser feito.
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void connectionLost(String login, Object sessionKey,
      String systemName, long time) {
      // Nada a ser feito.
    }
  }
}
