/*
 * $Id: ProjectService.java 98981 2009-11-19 17:59:39Z juliana $
 */
package csbase.server.services.projectservice;

import static java.lang.Byte.parseByte;

import java.io.BufferedInputStream;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.RandomAccessFile;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.InvocationTargetException;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.channels.ServerSocketChannel;
import java.rmi.RemoteException;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import openbusidl.rs.Property;
import openbusidl.rs.ServiceOffer;

import org.omg.CORBA.UserException;

import scs.core.ComponentId;
import scs.core.IComponent;
import scs.core.IComponentHelper;
import scs.core.servant.ComponentBuilder;
import scs.core.servant.ComponentContext;
import scs.core.servant.ExtendedFacetDescription;
import tecgraf.ftc.common.exception.FailureException;
import tecgraf.ftc.server.FileServer;
import tecgraf.ftc.server.FileServerConfig;
import tecgraf.ftc.server.FileServerConfigImpl;
import tecgraf.ftc.server.FileServerOwner;
import tecgraf.ftc.utils.Utils;
import tecgraf.javautils.FileUtils;
import tecgraf.javautils.core.IOUtils;
import tecgraf.openbus.Openbus;
import tecgraf.openbus.data_service.IHierarchicalDataServiceHelper;
import csbase.exception.InvalidRequestException;
import csbase.exception.OperationFailureException;
import csbase.exception.PermissionException;
import csbase.exception.ServiceFailureException;
import csbase.logic.AdminPermission;
import csbase.logic.AdministrationEvent;
import csbase.logic.AllProjectsArgObserver;
import csbase.logic.ClientProjectFile;
import csbase.logic.ClientProjectFileInfo;
import csbase.logic.CommonClientProject;
import csbase.logic.CommonProjectInfo;
import csbase.logic.FileInfoSearchResult;
import csbase.logic.FileLockListenerInterface;
import csbase.logic.NotificationData;
import csbase.logic.Permission;
import csbase.logic.ProjectAdminInfo;
import csbase.logic.ProjectAdminPermission;
import csbase.logic.ProjectArgObserver;
import csbase.logic.ProjectEvent;
import csbase.logic.ProjectFile;
import csbase.logic.ProjectFileInfo;
import csbase.logic.ProjectFileTypeInfo;
import csbase.logic.ProjectNotificationData;
import csbase.logic.ProjectPermissions;
import csbase.logic.ProjectUserEvent;
import csbase.logic.SecureKey;
import csbase.logic.SharedProjectEvent;
import csbase.logic.User;
import csbase.logic.UserProjectInfo;
import csbase.logic.UserProjectsArgObserver;
import csbase.logic.UsersNotificationData;
import csbase.logic.ProjectPermissions.SharingType;
import csbase.logic.filters.LocalFileFinder;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.ProjectFileChannel;
import csbase.remote.ProjectFileChannelProtocol;
import csbase.remote.ProjectServiceInterface;
import csbase.remote.RemoteObserver;
import csbase.server.FileSystem;
import csbase.server.Server;
import csbase.server.ServerException;
import csbase.server.Service;
import csbase.server.services.administrationservice.AdministrationService;
import csbase.server.services.mailservice.MailService;
import csbase.server.services.notificationservice.NotificationService;
import csbase.server.services.openbusservice.OpenBusService;
import csbase.server.services.projectservice.FileLockInterface.LockStatus;
import csbase.server.services.projectservice.openbus.ProjectDataService;

/**
 * Implementa a interface {@link ProjectServiceInterface} para prover o servio
 * de projetos do {@link Server}. Essa classe cria os objetos do tipo
 * {@link CommonClientProject} que sero enviados para o usurio. Tambm  ela
 * que gera identificadores para cada CommonClientProject. Dado um identificador
 * de um CommonClientProject, esta classe deve saber achar o ServerProject
 * correspondente. Implementa o servio de acesso a arquivos, baseado em
 * conexes socket.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class ProjectService extends Service implements ProjectServiceInterface,
  Runnable, FileServerOwner {

  /**
   * Prefixo utilizado nos nomes das chaves de traduo.
   */
  private static final String LNG_PREFIX = "ProjectService.";
  
  
  /** 
   * Nome default do componente para publicao no <code>OpenBus</code> 
   */
  public static final String COMPONENT_NAME = "ProjectService";
  
  /** 
   * Verso default do componente para publicao no <code>OpenBus</code> 
   */
  public static final String COMPONENT_VERSION = "1.0.0";

  /**
   * Mensagem para diretrio no encontrado no sistema de arquivos.
   */
  private static final String DIR_NOT_FOUND_MSG =
    LNG_PREFIX + "error.dir.not.found";

  /**
   * Mensagem para parmetros invlidos/inadequados passsados pelo cliente.
   */
  private static final String INVALID_REQUEST_MSG =
    LNG_PREFIX + "error.invalid.request";

  /**
   * Contador para as threads que tratam as conexes do cliente
   */
  private static long handlerCount = 0;

  /**
   * Configuraes do servidor FTC.
   */
  private FileServerConfig config = new FileServerConfigImpl();

  /**
   * Socket Channel de arquivo.
   * 
   * @author Tecgraf/PUC-Rio
   */
  class SocketFileChannel extends ProjectFileChannelProtocol implements
    Runnable {

    /**
     * Arquivo
     */
    private RandomAccessFile file;

    /**
     * Channel do arquivo
     */
    private FileChannel fileChannel;

    /**
     * Path
     */
    private String path;

    /**
     * Arquivo servidor associado.
     */
    private ServerProjectFile spf = null;

    /**
     * Buffer para transmisso do contedo dos arquivos. Criado somente para uso
     * conjunto com o mtodo
     * {@link IOUtils#transferTo(FileChannel, long, long, java.nio.channels.WritableByteChannel, ByteBuffer)}
     * Se o referido mtodo deixar de ser usado, este buffer pode ser removido.
     */
    private final ByteBuffer transferBuffer = ByteBuffer.allocate(1024 * 1024);

    /**
     * Cdigo de identificao desta thread. Usado para o log, por ser mais
     * compacto do que o "id", que apresenta todo o caminho para o arquivo.
     */
    private final long code;
    /**
     * Chave associada  sesso que abriu este canal.
     */
    private String sessionKey;

    /**
     * Realiza o tratamento das solicitaes do cliente, at que este solicite o
     * fechamento (opcode = CLOSE) da conexo.
     */
    public void run() {
      try {
        logDebugMessage("Thread " + code + " iniciada: SocketFileChannel");
        // Se abriu para leitura e escrita, coloca underConstruction=true*/
        if ((opcode == OPEN_READ_WRITE) && (spf != null)) {
          // FIXME deveramos verificar antes se no est bloqueado?
          spf.setUnderConstruction(true);
        }
        if (socket == null) {
          return;
        }
        boolean close = false;
        while (!close) {
          // Bloqueia aguardando a prxima operao e seus parmetros
          readOperation();
          switch (opcode) {
            case READ:
              clientRead(params[0], params[1]);
              break;
            case WRITE:
              clientWrite(params[0], params[1]);
              break;
            case UPLOAD:
              clientUpload(params[0]);
              break;
            case DOWNLOAD:
              clientDownload();
              break;
            case GET_POSITION:
              clientGetPosition();
              break;
            case SET_POSITION:
              clientSetPosition(params[0]);
              break;
            case GET_SIZE:
              clientGetSize();
              break;
            case TRUNCATE:
              clientTruncate(params[0]);
              break;
            case CLOSE:
              clientSetPosition(this.fileChannel.size());
              close = true;
              break;
            default:
              log("SocketFileChannel:run: erro no protocolo: "
                + "opcode desconhecido: " + opcode);
              clientSetPosition(this.fileChannel.size());
              close = true;
          }
        }
        logDebugMessage("Thread " + code + " terminada: SocketFileChannel");
        socketChannel.close();
        socket.close();
        fileChannel.close();
        file.close();
      }
      catch (Throwable t) {
        log("SocketFileChannel:run", t);
      }
      finally {
        try {
          if (spf != null) {
            spf.setUnderConstruction(false);
          }
        }
        catch (ServerProjectFileException spfe) {
          log("SocketFileChannel:run", spfe);
        }
      }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void log(String text) {
      logSevereMessage(text);
    }

    /**
     * Registra no log uma exceo e uma mensagem associada a ela.
     * 
     * @param text mensagem
     * @param t exceo
     */
    protected void log(String text, Throwable t) {
      logSevereMessage(text, t);
    }

    /**
     * Faz operao de consulta de posio.
     */
    private void clientGetPosition() {
      try {
        buffer.clear();
        buffer.putLong(fileChannel.position());
        writeBuffer();
      }
      catch (Exception e) {
        log("SocketFileChannel:clientGetPosition", e);
      }
    }

    /**
     * Transmite o tamanho do arquivo ao cliente.
     */
    private void clientGetSize() {
      try {
        buffer.clear();
        buffer.putLong(this.fileChannel.size());
        writeBuffer();
      }
      catch (Exception e) {
        log("SocketFileChannel:clientGetSize", e);
      }
    }

    /**
     * Faz operao de leitura
     * 
     * @param position posio do arquivo
     * @param count num de bytes.
     */
    private void clientRead(long position, long count) {
      try {
        // Este mtodo foi substitudo em virtude do bug
        // http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6253145.
        // Quando este bug for resolvido, o cdigo comentado pode retornar.
        // sent = fileChannel.transferTo(position, count, socketChannel);
        IOUtils.transferTo(fileChannel, position, count, socketChannel,
          transferBuffer);

      }
      catch (IOException e) {
        log("SocketFileChannel:clientRead: erro de I/O", e);
        log("clientRead: position=" + position + " count=" + count + " sent=0");
      }
    }

    /**
     * Faz operao de ajuste de posicionamento de arquivo.
     * 
     * @param position posio
     */
    private void clientSetPosition(long position) {
      try {
        fileChannel.position(position);
      }
      catch (IOException e) {
        log("SocketFileChannel:clientSetPosition: erro de I/O", e);
      }
    }

    /**
     * Faz a operao de truncagem de arquivo.
     * 
     * @param newSize novo tamanho
     */
    private void clientTruncate(long newSize) {
      try {
        this.fileChannel.truncate(newSize);
      }
      catch (Exception e) {
        log("SocketFileChannel:clientTruncate", e);
      }
    }

    /**
     * Faz a operao de upload.
     * 
     * @param count nmero de bytes
     */
    private void clientUpload(final long count) {
      LockStatus lockStatus =
        FileLockManager.getInstance().isLocked(spf, sessionKey);
      // DEBUG
      // System.out.println("CHNL UPLOAD SRV: lockStatus = " + lockStatus);
      try {
        buffer.clear();
        if (lockStatus == LockStatus.LOCKED_BY_OTHERS) {
          /* arquivo bloqueado por outra sesso */
          buffer.putLong(-1L);
        }
        else {
          /*
           * usurio possua o lock OU no havia lock OU lock expirou
           * 
           * ATENO: o lock  por _sesso_, o que quer dizer que um usurio
           * pode concorrer com ele mesmo se os acessos partirem de uma mesma
           * sesso. Este controle  responsabilidade do cliente.
           */
          buffer.putLong(0L);
        }
        // enviamos o resultado da operao
        writeBuffer();
        if (lockStatus == LockStatus.LOCKED_BY_OTHERS) {
          // FIXME talvez no devessemos nem ter colocado em construo...
          spf.setUnderConstruction(false);
          return;
        }
      }
      catch (Exception e) {
        log("SocketFileChannel:clientUpload", e);
      }

      try {
        fileChannel.position(0);
        long received = 0;
        long remaining = count;
        while (received < count) {
          long transfered =
            fileChannel.transferFrom(socketChannel, received, remaining);
          received += transfered;
          remaining -= transfered;
        }
      }
      catch (IOException e) {
        log("SocketFileChannel:clientUpload: erro de I/O", e);
      }
    }

    /**
     * Faz a operao de download.
     */
    private void clientDownload() {
      long size;
      try {
        size = fileChannel.size();
        buffer.clear();
        buffer.putLong(size);
        writeBuffer();
      }
      catch (Exception e) {
        size = 0;
        log("SocketFileChannel:clientDownload", e);
      }

      try {
        fileChannel.position(0);
        long sent = 0;
        long remaining = size;
        // final long chunkSize = 1024*1024*1024*1; // gb
        while (sent < size) {
          final long blockSize = remaining < size ? remaining : size;
          long transfered =
            fileChannel.transferTo(sent, blockSize, socketChannel);
          sent += transfered;
          remaining -= transfered;
        }
      }
      catch (IOException e) {
        log("SocketFileChannel:clientDownload: erro de I/O", e);
      }
    }

    /**
     * Faz a operao de escrita.
     * 
     * @param position posio
     * @param count nmero de bytes.
     */
    private void clientWrite(long position, long count) {
      LockStatus lockStatus =
        FileLockManager.getInstance().isLocked(spf, sessionKey);
      // DEBUG
      // System.out.println("CHNL WRITE SRV: lockStatus = " + lockStatus);
      try {
        buffer.clear();
        if (lockStatus == LockStatus.LOCKED_BY_OTHERS) {
          /*
           * arquivo bloqueado por outra sesso
           */
          buffer.putLong(-1L);
        }
        else {
          /*
           * usurio possua o lock OU no havia lock OU lock expirou ATENO: o
           * lock  por _sesso_, o que quer dizer que um usurio pode concorrer
           * com ele mesmo se os acessos partirem de uma mesma sesso. Este
           * controle  responsabilidade do cliente.
           */
          buffer.putLong(0L);
        }
        // enviamos o resultado da operao
        writeBuffer();
        if (lockStatus == LockStatus.LOCKED_BY_OTHERS) {
          // FIXME talvez no devessemos nem ter colocado em construo...
          spf.setUnderConstruction(false);
          return;
        }
      }
      catch (Exception e) {
        log("SocketFileChannel:clientWrite", e);
      }
      try {
        int received = 0;
        long missing;
        int remaining = buffer.remaining();
        /*
         * Testa se o buffer possui um resto que pode ser parte dos dados do
         * arquivo a ser gravado.
         */
        if (remaining != 0) {
          /*
           * O resto do buffer tem dados suficientes para a gravacao, preciso
           * garantir que estamos consumindo apenas o que foi solicitado para
           * gravar no arquivo
           */
          if (remaining >= count) {
            int limit = buffer.limit();
            int newLimit = (int) (buffer.position() + count);
            buffer.limit(newLimit);
            received = fileChannel.write(buffer, position);
            buffer.limit(limit);
            if (received != count) {
              log("SocketFileChannel:clientWrite: erro na transferencia: "
                + "solicitado(s) " + count + " recebido(s) " + received
                + " byte(s)");
            }
            return;
          }
          /*
           * Nesse caso, o buffer possui menos do que foi solicitado. Consome
           * tudo que resta no buffer e depois continuo a transferencia direto
           * para o fileChannel.
           */
          received = fileChannel.write(buffer, position);
          if (received != remaining) {
            log("SocketFileChannel:clientWrite: erro na transferencia "
              + " do restante: " + "solicitado(s) " + count + " recebido(s) "
              + received + " byte(s)");
            return;
          }
        }
        /*
         * Faco a transferencia apenas do que nao estava no resto do buffer
         */
        final long newPosition = position + received;
        missing = count - remaining;
        received +=
          fileChannel.transferFrom(socketChannel, newPosition, missing);
        if (received != count) {
          log("SocketFileChannel:clientWrite: erro na transferencia: "
            + "solicitado(s) " + count + " recebido(s) " + received
            + " byte(s)");
        }
      }
      catch (IOException e) {
        log("SocketFileChannel:clientWrite: erro de I/O", e);
      }
    }

    /**
     * Inicia o tratamento do socket cliente especificado.
     * 
     * @param socket socket cliente que solicitou conexo.
     */
    SocketFileChannel(Socket socket) {
      super(socket);
      this.code = handlerCount++;
      try {
        // Procura obter o opcode inicial (OPEN_READ_WRITE ou OPEN_READ_ONLY)
        // e o tamanho do caminho para o arquivo.
        readOperation();
        if ((opcode != OPEN_READ_WRITE) && (opcode != OPEN_READ_ONLY)) {
          log("SocketFileChannel:SocketFileChannel: erro no "
            + "protocolo: opcode desconhecido: " + opcode);
          throw new Exception("Primeiro opcode no  de OPEN");
        }
        int pathSize = (int) params[0];
        // Procura obter o caminho para o arquivo.
        path = readString(pathSize);

        // Caso o arquivo no seja regular e tenha sido aberto para leitura e
        // escrita, levanta exceo
        if (opcode == OPEN_READ_WRITE) {
          /*
           * o tamanho da chave da sesso veio como 2o parmetro, e a chave
           * propriamente dita veio aps o path para o arquivo
           */
          int hashSize = (int) params[1];
          sessionKey = readString(hashSize);

          File fileAux = new File(path);
          String projId = ServerProjectFile.findFileProjectId(fileAux);
          if (projId != null) {
            String[] fullPath = ServerProjectFile.findFileRelativePath(fileAux);
            spf = ServerProjectFile.findFile(projId, fullPath);
            if (spf == null) {
              String pathStr = Arrays.toString(fullPath);
              throw new InvalidRequestException(getString(INVALID_REQUEST_MSG),
                "Caminho " + pathStr + " no encontrado na rvore do servidor");
            }
            if (!spf.isRegularFile()) {
              throw new Exception("Arquivos globais, publicados e diretrios "
                + "no podem ser abertos para escrita");
            }
          }
        }
        String mode = ((opcode == OPEN_READ_ONLY) ? "r" : "rw");
        this.file = new RandomAccessFile(path, mode);
        this.fileChannel = file.getChannel();
      }
      catch (Throwable t) {
        log("SocketFileChannel:SocketFileChannel", t);
        log("SocketFileChannel:SocketFileChannel: Fechando o socket...");
        try {
          socketChannel.close();
          socket.close();
        }
        catch (Throwable t1) {
          log("SocketFileChannel:SocketFileChannel", t1);
        }
        socketChannel = null;
      }
      id = path;
    }

    /**
     * L uma string do buffer. Se nem todos os bytes forem lidos, apenas
     * registra no log e prossegue.
     * <p>
     * FIXME este era o comportamento anterior, depois precisamos rever isto
     * 
     * @param strLen - tamanho do string
     * @return string lido, ou null caso nem todos os dados tenham sido lidos
     * @throws Exception em caso de falha.
     */
    private String readString(int strLen) throws Exception {
      int received = readBuffer(strLen);
      if (received < strLen) {
        log("SocketFileChannel:SocketFileChannel: erro no protocolo: "
          + "mensagem de tamanho " + received);
        throw new Exception(
          "falha na comunicao com o servidor (leitura parcial de string)");
      }
      byte[] data = new byte[strLen];
      buffer.get(data);
      return new String(data, "UTF8");
    }
  }

  /**
   * Porta do protocolo
   */
  private static int fileChannelPort = 7780;

  /**
   * Servidor do protocolo
   */
  private static String hostName = null;

  /**
   * Instncia do administrador de projetos.
   */
  private ProjectAdministrator prjAdmin = null;

  /**
   * Informaes dos projetos que cada usurio participa. Cada registro  um
   * par: usurio - Vector de UserProjectInfo.
   */
  // TODO avaliar se podemos usar Set ao invs de List
  private Hashtable<Object, Vector<UserProjectInfo>> userProjects;

  /**
   * Conjunto de projetos pblicos, ou seja, projetos em que todos os usurios
   * tm acesso.
   */
  private Set<UserProjectInfo> publicProjects;

  /**
   * Servios de atualizao de arquivos de projeto;
   */
  private Map<Class<? extends UpdaterService>, UpdaterService> updaterServices;

  /**
   * Representa um servidor de arquivos que podem ser transferidos via NIO.
   */
  private FileServer fileServer;
  /**
   * Representa os arquivos abertos para transferncia.
   */
  private Map<byte[], RandomAccessFile> openedFiles;
  /**
   * Indica se o servio deve ser ou no exportado para o OpenBus.
   */
  private boolean exportToOpenBus;
  /**
   * A faceta que exporta o Servio de Projetos no OpenBus.
   */
  private ProjectDataService projectDataService;
  /**
   * O tamanho mnimo do identificador nico dos arquivos.
   */
  public static final int MINIMUM_FILE_ID_SIZE = 2;
  /**
   * O separador do identificador nico dos arquivos.
   */
  public static final String FILE_ID_SEPARATOR = "/";

  /**
   * Indica se o servio de projetos utiliza reserva de rea.
   */
  private boolean areaReserved;

  /**
   * Consulta  porta utilizada no protocolo.
   * 
   * @return o nmero da porta
   */
  public static int getFileChannelPort() {
    return fileChannelPort;
  }

  /**
   * Obtm uma instncia do servio.
   * 
   * @return ProjectService
   */
  public static ProjectService getInstance() {
    return (ProjectService) getInstance(SERVICE_NAME);
  }

  public ProjectDataService getDataService() {
    return this.projectDataService;
  }

  /**
   * Consulta ao nome do servidor do servio de projetos. Esse nome  obtido a
   * partir da propriedade <code>hostname</code> do Server.properties ou por
   * consulta ao <code>InetAddress.getLocalHost()</code> pelo mtodo
   * <code>.getCanonicalHostName</code>.
   * 
   * @return o nome do servidor.
   * @see #run()
   */
  public static String getHostName() {
    return hostName;
  }

  /**
   * Obtem todas as propriedades dos arquivos.
   * 
   * @param loc .
   * @return Map com todos os tipos de arquivos.
   */
  public Hashtable<String, ProjectFileTypeInfo> getAllFileTypes(Locale loc) {
    try {
      return ServerProjectFileType.getInstance().getAllTypes(loc);
    }
    catch (ServerProjectException t) {
      throw new ServiceFailureException(t.getMessage(), t);
    }
  }

  /**
   * Obtem todas as propriedades de um tipo de arquivo.
   * 
   * @param type o tipo do arquivo desejado
   * @return informaes de um tipo de arquivo
   */
  public ProjectFileTypeInfo getFileType(String type) {
    try {
      return ServerProjectFileType.getInstance().getFileType(type);
    }
    catch (ServerProjectException t) {
      throw new ServiceFailureException(t.getMessage(), t);
    }
  }

  /**
   * {@inheritDoc}
   */
  public String[] getAllProjectNames(Object userId) {
    try {
      return ServerProject.getAllProjectNames(userId);
    }
    catch (ServerProjectException t) {
      throw new ServiceFailureException(t.getMessage(), t);
    }
  }

  /**
   * Obtm todos os projetos que um usurio participa, mas no  o administrador
   * do projeto.
   * 
   * @param userId Identificador do usurio.
   * @return Vector de UserProjectInfo contendo os nomes dos projetos e os
   *         identificadores dos usurios administradores de cada projeto.
   */
  public Vector<UserProjectInfo> getAllUserProjectsFromOthers(Object userId) {
    Vector<UserProjectInfo> projects = new Vector<UserProjectInfo>();
    if (this.publicProjects == null) {
      return null;
    }
    synchronized (publicProjects) {
      for (UserProjectInfo info : publicProjects) {
        if (!info.getUserId().equals(userId)) {
          projects.add(info);
        }
      }
    }
    if (userProjects == null) {
      return null;
    }
    Vector<UserProjectInfo> sharedProjects = userProjects.get(userId);
    if (sharedProjects != null) {
      projects.addAll(sharedProjects);
    }
    return projects;
  }

  /**
   * Verifica se o usuario e' membro do projeto.
   * 
   * @param userId Identificador do usuario a ser verificado.
   * @param ownerId Identificador do proprietario do projeto.
   * @param prjName Nome do projeto
   * @return true caso o usuario seja membro do projeto. False, caso contrario.
   */
  protected boolean isUserMember(Object userId, Object ownerId, String prjName) {
    try {
      if (ownerId.equals(userId)) {
        return true;
      }
      UserProjectInfo info = new UserProjectInfo(ownerId, prjName);
      if (getAllUserProjectsFromOthers(userId).contains(info)) {
        return true;
      }
      return false;
    }
    catch (Exception e) {
      return false;
    }
  }

  /**
   * Retorna os arquivos filhos de um diretrio do projeto.
   * 
   * @param userId usurio (login).
   * @param proj nome do projeto.
   * @param path o path do diretrio.
   * @return os arquivos filhos de um diretrio do projeto
   */
  public File[] getChildrenFiles(String userId, String proj, String path) {
    String id = ServerProject.getId(userId, proj);
    String[] newPath = buildFilePath(proj, path);
    ServerProjectFile spf = ServerProjectFile.findFile(id, newPath);
    ServerProjectFile[] children = (ServerProjectFile[]) spf.getChildren();
    File[] childrenFiles = new File[children.length];
    for (int i = 0, l = childrenFiles.length; i < l; i++) {
      childrenFiles[i] = children[i].getFile();
    }
    return childrenFiles;
  }

  /**
   * Obtm o arquivo de um projeto.
   * 
   * @param userId usurio (login).
   * @param proj nome do projeto.
   * @param path um path local ao projeto.
   * @return o arquivo de um projeto
   */
  public ServerProjectFile getServerFile(Object userId, String proj, String path) {
    String id = ServerProject.getId(userId, proj);
    String[] newPath = buildFilePath(proj, path);
    return ServerProjectFile.findFile(id, newPath);
  }

  /**
   * Obtm o arquivo de um projeto.
   * 
   * @param userId usurio (login).
   * @param proj nome do projeto.
   * @param path um path local ao projeto.
   * @return o arquivo de um projeto
   */
  public File getFile(Object userId, String proj, String path) {
    ServerProjectFile spf = getServerFile(userId, proj, path);
    return spf.getFile();
  }

  /**
   * Muda a descrio atual deste arquivo.
   * 
   * @param id Identificador do projeto.
   * @param path Caminho do arquivo.
   * @param text O novo texto da descrio do arquivo.
   */
  public void setFileDescription(Object id, String[] path, String text) {
    this.checkWritePermission(id);
    try {
      ServerProjectFile spf = ServerProjectFile.findFile(id, path);
      spf.setDescription(text);
    }
    catch (ServerProjectFileException t) {
      throw new ServiceFailureException(t.getMessage(), t);
    }
  }

  /**
   * Obtm a descrio atual deste arquivo.
   * 
   * @param id Identificador do projeto.
   * @param path Caminho do arquivo.
   * @return O texto com a descrio do arquivo.
   */
  public String getFileDescription(Object id, String[] path) {
    try {
      ServerProjectFile spf = ServerProjectFile.findFile(id, path);
      return spf.getDescription();
    }
    catch (ServerProjectFileException t) {
      throw new ServiceFailureException(t.getMessage(), t);
    }
  }

  /**
   * Retorna o tipo do arquivo.
   * 
   * @param userId usurio (login).
   * @param proj nome do projeto.
   * @param path o path do arquivo.
   * @return o tipo do arquivo
   */
  public String getFileType(Object userId, String proj, String path) {
    String id = ServerProject.getId(userId, proj);
    String[] newPath = buildFilePath(proj, path);
    ServerProjectFile spf = ServerProjectFile.findFile(id, newPath);
    return spf.getType();
  }

  /**
   * Busca de informaes sobre um arquivo.
   * 
   * @param userId usurio (login).
   * @param projectId nome do projeto.
   * @param path o caminho lgico.
   * @param recursive flag de busca recursiva.
   * @return um array de paths do projeto.
   */
  public csbase.server.services.projectservice.ProjectFileInfo[] getFilesInfo(
    Object userId, Object projectId, String path, boolean recursive) {
    return null;
  }

  /**
   * Obtm as informaes de todos os projetos dos usurios.
   * 
   * FIXME - Renomear este mtodo para "getAllProjectAdminInfo()".
   * 
   * @return um array com todos os <code>ProjectAdminInfo</code>.
   */
  public ProjectAdminInfo[] getLockedProjectAdminInfo() {
    return prjAdmin.getAllProjectAdminInfo();
  }

  /**
   * Obtm as informaes administrativas de um projeto de um usurio.
   * 
   * @param userId o identificador do usurio
   * @param projectName o nome do projeto
   * 
   * @return O <code>ProjectAdminInfo</code> do projeto de um usurio.
   */
  //  public ProjectAdminInfo getProjectAdminInfo(Object userId, String projectName) {
  //    return prjAdmin.getProjectAdminInfo(userId, projectName);
  //  }
  /**
   * Verifica se um projeto est desbloqueado utilizando rea reservada.
   * 
   * @param userId o identificador do usurio
   * @param projectName o nome do projeto
   * @return verdadeiro, se o projeto est desbloqueado utilizando rea
   *         reservada ou falso, caso contrrio.
   */
  public boolean isUnlockedWithAreaAllocated(Object userId, String projectName) {
    try {
      return prjAdmin.isUnlockedWithAreaAllocated(userId, projectName);
    }
    catch (ServerProjectException t) {
      throw new ServiceFailureException(t.getMessage());
    }
  }

  /**
   * Retorno dos nomes dos projetos.
   * 
   * @param userId usurio (login).
   * @return Um array de strings com os nomes dos projetos.
   */
  public String[] getProjectNames(Object userId) {
    return getAllProjectNames(userId);
  }

  /**
   * Retorna o diretrio raiz de um projeto.
   * 
   * @param userId usurio (login).
   * @param proj nome do projeto.
   * @return .
   */
  public File getRoot(Object userId, String proj) {
    String id = ServerProject.getId(userId, proj);
    return new File(id);
  }

  /**
   * Informa o tamanho corrente deste arquivo. Reflete o nmero corrente de
   * bytes do arquivo.
   * 
   * @param id Identificador do projeto.
   * @param path Caminho do arquivo.
   * @return Tamanho corrente do arquivo.
   */
  public long fileSize(Object id, String[] path) {
    ServerProjectFile spf = ServerProjectFile.findFile(id, path);
    return spf.size();
  }

  /**
   * Informa a data da ltima modificao deste arquivo (formato numrico).
   * 
   * @param id Identificador do projeto.
   * @param path Caminho do arquivo.
   * @return data da ltima modificao deste arquivo (formato numrico)
   */
  public long getModificationDate(Object id, String[] path) {
    ServerProjectFile spf = ServerProjectFile.findFile(id, path);
    return spf.getModificationDate();
  }

  /**
   * Registra um observador interessado nas mudanas que ocorrerem em um
   * projeto. O observador pode est interessado nas mudanas de todos os
   * projetos, dos seus projetos ou de um projeto especfico.
   * 
   * @param observer O observador remoto que registra o interesse nas mudanas
   *        sobre um projeto.
   * @param arg Argumento que indica em que projetos o observador est
   *        interessado.
   */
  @Override
  public void addObserver(RemoteObserver observer, Object arg) {
    try {
      if (arg instanceof ProjectArgObserver) {
        setProjectObserver(observer, (ProjectArgObserver) arg);
      }
      else if (arg instanceof AllProjectsArgObserver) {
        super.addObserver(observer, arg);
      }
      else if (arg instanceof UserProjectsArgObserver) {
        super.addObserver(observer, arg);
      }
      else {
        logSevereMessage("No foi possvel registrar o observador."
          + " Argumento no definido");
      }
    }
    catch (OperationFailureException ope) {
      throw new ServiceFailureException(ope.getMessage(), ope);
    }
  }

  /**
   * Registra o observador no ServerProject.
   * 
   * @param observer observador remoto
   * @param arg argumento do observador remoto
   * @throws OperationFailureException caso ocorra uma falha ao abrir o projeto
   */
  private void setProjectObserver(RemoteObserver observer,
    ProjectArgObserver arg) throws OperationFailureException {
    try {
      Object userId = arg.getUserId();
      String projectName = arg.getProjectName();
      Object id = ServerProject.getId(userId, projectName);
      ServerProject sp = ServerProject.getProject(id);
      if (sp == null) {
        sp = ServerProject.openProject(userId, userId, projectName, false);
        if (sp == null) {
          throw new OperationFailureException("Projeto " + id
            + " no encontrado");
        }
      }
      sp.addObserver(observer, id);
    }
    catch (ServerProjectException t) {
      throw new OperationFailureException(t.getMessage());
    }
  }

  /**
   * Remove o observador. Se o argumento dado for instncia de
   * ProjectArgObserver, remove o observador do ServerProject.
   * 
   * @param observer O observador remoto a ser removido.
   * @param arg O argumento indica se o observador de projeto deve ser removido
   *        ou o observador do servio.
   * @return .
   */
  @Override
  public boolean deleteObserver(RemoteObserver observer, Object arg) {
    try {
      if (arg instanceof ProjectArgObserver) {
        return deleteProjectObserver(observer, (ProjectArgObserver) arg);
      }
      else if (arg instanceof AllProjectsArgObserver) {
        return super.deleteObserver(observer, arg);
      }
      else if (arg instanceof UserProjectsArgObserver) {
        return super.deleteObserver(observer, arg);
      }
      else {
        logSevereMessage("No foi possvel remover o observador."
          + " Argumento no definido");
        return false;
      }
    }
    catch (OperationFailureException ope) {
      throw new ServiceFailureException(ope.getMessage(), ope);
    }
  }

  /**
   * Remove o observador do ServerProject.
   * 
   * @param observer observador remoto
   * @param arg argumento do observador remoto
   * @return true se o observador foi removido
   * @throws OperationFailureException falha na obtena
   */
  private boolean deleteProjectObserver(RemoteObserver observer,
    ProjectArgObserver arg) throws OperationFailureException {
    Object id = ServerProject.getId(arg.getUserId(), arg.getProjectName());
    ServerProject sp = ServerProject.getProject(id);
    if (sp == null) {
      throw new OperationFailureException("Projeto " + id + " no encontrado.");
    }
    return sp.deleteObserver(observer, id);
  }

  /**
   * O observador s  notificado o identificador do usurio no seu argumento
   * for igual ao identificador do usurio no evento gerado.
   * 
   * @param arg .
   * @param event .
   * @return .
   */
  @Override
  protected boolean has2Update(Object arg, Object event) {
    if ((arg == null) || (event == null)) {
      return false;
    }
    if (arg instanceof AllProjectsArgObserver) {
      return true;
    }

    // Obtem o identificador do usurio no argumento.
    Object userId = null;
    Object item = null;
    if (arg instanceof UserProjectsArgObserver) {
      // Obtem o identificador do usurio no argumento.
      userId = ((UserProjectsArgObserver) arg).getUserId();
      if (event instanceof AdministrationEvent) {
        item = ((AdministrationEvent) event).item;
        if (item instanceof CommonProjectInfo) {
          CommonProjectInfo cpi = (CommonProjectInfo) item;
          if (userId.equals(cpi.userId)) {
            return true;
          }
          return false;
        }
        else if (item instanceof ProjectAdminInfo) {
          ProjectAdminInfo projAdminInfo = (ProjectAdminInfo) item;
          if (userId.equals(projAdminInfo.getUserId())) {
            return true;
          }
          return false;
        }
        else if (item instanceof ProjectAdminInfo[]) {
          ProjectAdminInfo[] projAdminInfos = (ProjectAdminInfo[]) item;
          for (ProjectAdminInfo projAdminInfo : projAdminInfos) {
            if (userId.equals(projAdminInfo.getUserId())) {
              return true;
            }
          }
          return false;
        }
      }

      // Projeto criado ou removido
      else if (event instanceof ProjectUserEvent) {
        item = ((ProjectUserEvent) event).item;
        CommonProjectInfo cpi = (CommonProjectInfo) item;

        // Testando se userId  o dono do projeto
        if (userId.equals(cpi.userId)) {
          return true;
        }

        // S para projeto removido
        if (((ProjectUserEvent) event).type == ProjectUserEvent.DELETE) {
          // Pegando usurios que participam do projeto
          Set<Object> allUsers = ProjectPermissions.getAllUsers(cpi);
          // Testa se userId participa do projeto
          if (allUsers.contains(userId)) {
            return true;
          }
        }
        return false;
      }
      else if (event instanceof SharedProjectEvent) {
        SharedProjectEvent sharedProjectEvent = (SharedProjectEvent) event;
        Collection<Object> eventUsersId = sharedProjectEvent.getEventUsersId();
        if (eventUsersId == null) {
          return true;
        }
        return eventUsersId.contains(userId);
      }
    }
    return false;
  }

  /**
   * Anexa um texto  descrio atual deste arquivo. Permite a gerao de
   * histricos de modificao.
   * 
   * @param id Identificador do projeto.
   * @param path Caminho do arquivo.
   * @param text O texto a ser anexado.
   */
  public void appendFileDescription(Object id, String[] path, String text) {
    this.checkWritePermission(id);
    try {
      ServerProjectFile spf = ServerProjectFile.findFile(id, path);
      spf.appendDescription(text);
    }
    catch (ServerProjectFileException t) {
      throw new ServiceFailureException(t.getMessage(), t);
    }
  }

  /**
   * Muda o tipo de um arquivo de um projeto.
   * 
   * @param id Identificador do projeto.
   * @param path Caminho do arquivo cujo tipo vai ser alterado.
   * @param type Novo tipo do arquivo.
   */
  public void changeFileType(Object id, String[] path, String type) {
    this.checkWritePermission(id);
    try {
      ServerProjectFile spf = ServerProjectFile.findFile(id, path);
      spf.changeType(type);
    }
    catch (ServerProjectFileException t) {
      throw new ServiceFailureException(t.getMessage(), t);
    }
  }

  /**
   * Fecha um arquivo aberto para leitura ou escrita que no usa
   * <code>ByteBuffer</code>.
   * 
   * @param id Identificador do projeto.
   * @param path Caminho do arquivo a ser aberto.
   */
  public void closeFile(Object id, String[] path) {
    try {
      ServerProjectFile spf = ServerProjectFile.findFile(id, path);
      spf.close();
    }
    catch (ServerProjectFileException t) {
      throw new ServiceFailureException(t.getMessage(), t);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void closeProject(Object id, boolean notify) {
    ServerProject sp = ServerProject.getProject(id);
    if (sp == null) {
      throw new ServiceFailureException("Projeto " + id + " no encontrado.");
    }
    sp.close(notify);
  }

  /**
   * {@inheritDoc}
   */
  public void copyFile(Object id, String[] filePath, String[] dirPath) {
    this.checkWritePermission(id);
    try {
      ServerProjectFile file = ServerProjectFile.findFile(id, filePath);
      ServerProjectFile dir = ServerProjectFile.findFile(id, dirPath);
      file.copy(dir);
    }
    catch (ServerProjectFileException t) {
      throw new ServiceFailureException(t.getMessage(), t);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void copyFile(Object sourceProjectId, String[] filePathSource,
    Object targetProjectId, String[] dirPathTarget) {
    this.checkWritePermission(targetProjectId);
    try {
      ServerProjectFile file =
        ServerProjectFile.findFile(sourceProjectId, filePathSource);
      ServerProjectFile dir =
        ServerProjectFile.findFile(targetProjectId, dirPathTarget);
      file.copy(dir);
    }
    catch (ServerProjectFileException t) {
      throw new ServiceFailureException(t.getMessage(), t);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void createDirectory(Object id, String[] path, String name) {
    this.checkWritePermission(id);
    try {
      ServerProjectFile spf = ServerProjectFile.findFile(id, path);
      spf.createDirectory(name);
    }
    catch (ServerProjectFileException t) {
      throw new ServiceFailureException(t.getMessage(), t);
    }
  }

  /**
   * {@inheritDoc}
   */
  public ClientProjectFile[] getChildren(final ClientProjectFile clientParent) {
    try {
      if (!clientParent.isDirectory()) {
        return null;
      }
      final Object id = clientParent.getProjectId();
      final String[] path = clientParent.getPath();
      final ServerProjectFile dir = ServerProjectFile.findFile(id, path);
      if (!dir.isDirectory())
        return null;
      final ProjectFile[] children = dir.getChildren();
      if (children == null) {
        return null;
      }
      final int sz = children.length;
      final ClientProjectFile[] clientFiles = new ClientProjectFile[sz];
      for (int i = 0; i < sz; i++) {
        ServerProjectFile srvChild = (ServerProjectFile) children[i];
        clientFiles[i] =
          buildSingleClientProjectFile(id, clientParent, srvChild);
      }

      return clientFiles;
    }
    catch (Exception t) {
      throw new ServiceFailureException(t.getMessage(), t);
    }
  }

  /**
   * {@inheritDoc}
   */
  public ClientProjectFile getChild(ClientProjectFile clientParent, String name) {
    try {
      if (!clientParent.isDirectory()) {
        return null;
      }
      final Object id = clientParent.getProjectId();
      final String[] path = clientParent.getPath();
      final ServerProjectFile dir = ServerProjectFile.findFile(id, path);
      if (!dir.isDirectory())
        return null;

      ServerProjectFile srvChild = (ServerProjectFile) dir.getChild(name);
      if (null == srvChild) {
        return null;

      }
      return buildSingleClientProjectFile(id, clientParent, srvChild);
    }
    catch (Exception t) {
      throw new ServiceFailureException(t.getMessage(), t);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void createDirectory(Object id, String[] path) {
    this.checkWritePermission(id);
    try {
      String[] root = new String[] { path[0] };
      ServerProjectFile parentDir = ServerProjectFile.findFile(id, root);
      for (int i = 1; i < path.length; i++) {
        String dirName = path[i];
        ServerProjectFile currentDir =
          (ServerProjectFile) parentDir.getChild(dirName);
        if (currentDir != null) {
          //  preciso verificar se  mesmo um diretrio
          if (!currentDir.isDirectory()) {
            throw new ServiceFailureException(currentDir.getName() + " em "
              + parentDir.getName()
              + "  um arquivo. No  possvel criar diretrio nele. ");
          }
        }
        else {
          parentDir.createDirectory(dirName);
          currentDir = (ServerProjectFile) parentDir.getChild(dirName);
        }
        parentDir = currentDir;
      }
    }
    catch (ServerProjectFileException t) {
      throw new ServiceFailureException(t.getMessage(), t);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void createFile(Object id, String[] dirPath, String name, String type) {
    this.checkWritePermission(id);
    try {
      ServerProjectFile dir = ServerProjectFile.findFile(id, dirPath);
      Object userId = Service.getUser().getId();
      dir.createFile(name, type, userId);
    }
    catch (ServerProjectFileException t) {
      throw new ServiceFailureException(t.getMessage(), t);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void createFiles(Object projectId, String[] parentPath,
    List<ProjectFileInfo> fileInfoList) {
    this.checkWritePermission(projectId);
    try {
      if (projectId == null) {
        throw new InvalidRequestException(
          "ProjectService: createFile: projectId == null");
      }
      if (parentPath == null) {
        throw new InvalidRequestException(
          "ProjectService: createFile: parentPath == null");
      }
      ServerProjectFile dir = ServerProjectFile.findFile(projectId, parentPath);
      Object userId = Service.getUser().getId();
      dir.createFiles(fileInfoList, userId);
    }
    catch (ServerProjectFileException t) {
      throw new ServiceFailureException(t.getMessage(), t);
    }
  }

  /**
   * {@inheritDoc}
   */
  public CommonClientProject createProject(CommonProjectInfo info) {
    ServerProject sp;
    try {
      sp = ServerProject.createProject(info);
      CommonClientProject cp = buildClientProject(sp.getId(), sp);

      logDebugMessage("Projeto " + info.name + " do usurio " + info.userId
        + " criado com sucesso");
      AdministrationEvent action =
        new AdministrationEvent(AdministrationEvent.CREATE, info);

      setChanged();
      notifyObservers(action);
      /* Notificando ProjectAdminObserver da criao do projeto */
      ProjectUserEvent event =
        new ProjectUserEvent(ProjectUserEvent.CREATE, info);
      setChanged();
      notifyObservers(event);
      return cp;
    }
    catch (ServerProjectException t) {
      throw new ServiceFailureException(t.getMessage(), t);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void createProjectWithAllocation(CommonProjectInfo info, long size) {
    try {
      ServerProject.createProject(info);
      ProjectAdminInfo adminInfo =
        prjAdmin.lockProject(info.userId, info.name, size);
      logInfoMessage("Projeto " + info.name + " do usurio " + info.userId
        + " criado com sucesso e aguardando liberao do administrador");
      AdministrationEvent action =
        new AdministrationEvent(AdministrationEvent.CREATE, info);
      setChanged();
      notifyObservers(action);
      String message =
        String.format(getString(LNG_PREFIX
          + "msg.project.created.with.allocation"), info.name, info.userId);
      notifyAdmin(adminInfo, message);
    }
    catch (ServerProjectException t) {
      throw new ServiceFailureException(t.getMessage(), t);
    }
  }

  /**
   * Retorna o caminho para um arquivo do projeto.
   * 
   * @param userId usurio (login).
   * @param proj nome do projeto.
   * @param file o arquivo procurado.
   * @return caminho.
   */
  public String fileToPath(Object userId, String proj, File file) {
    String path = file.getAbsolutePath();
    int index = path.indexOf(proj);
    path = path.substring(index + proj.length() + 1);
    return path;
  }

  /**
   * Verifica se o usurio informado  o administrador ou, caso contrrio, se
   * ele tem permisso de administrao sobre o servio de projeto do servidor
   * corrente.
   * 
   * @param user usurio.
   * @return indicativo.
   */
  protected boolean checkPermission(User user) {
    if (user.isAdmin()) {
      return true;
    }
    Server server = Server.getInstance();
    String serverName = AdminPermission.LOCAL + server.getServerName();
    Permission p =
      user.getMatchAttributesPermission(ProjectAdminPermission.class,
        serverName);
    if (p != null) {
      return true;
    }
    return false;
  }

  /**
   * Verifica se o usurio que efetuou a chamada  o administrador ou, caso
   * contrrio, se ele tem permisso de administrao sobre o servio de projeto
   * do servidor corrente. Em caso negatico, lana PermissionException.
   * 
   * @throws PermissionException em caso de falha de permisso.
   */
  private void checkPermission() throws PermissionException {
    if (!checkPermission(Service.getUser())) {
      throw new PermissionException();
    }
  }

  /**
   * Informa que um ou mais projetos tinham rea alocada e foram removidos,
   * tiveram a rea que estava alocada a eles liberada. Esse mtodo somente pode
   * ser chamado pelo administrador do sistema.
   * 
   * @param infos Um array com os objetos <code>ProjectAdminInfo</code> com as
   *        informaes de liberao da rea.
   */
  public void freeAreaForProjects(ProjectAdminInfo[] infos) {
    checkPermission();
    try {
      for (int i = 0; i < infos.length; i++) {
        infos[i] =
          prjAdmin.freeProjectArea(infos[i].getUserId(), infos[i]
            .getProjectName());
      }
      AdministrationEvent action =
        new AdministrationEvent(AdministrationEvent.DELETE, infos);
      setChanged();
      notifyObservers(action);
    }
    catch (ServerProjectException t) {
      throw new ServiceFailureException(t.getMessage(), t);
    }
  }

  /**
   * Cria o ServerSocketChannel e faz o accept. Essa comunicao, que serve
   * apenas para leitura e escrita de arquivos, deve ser no-bloqueante. Deve
   * haver, tambm, um pool de threads que recebam/enviem os dados disponveis
   * de/para os clientes. Nesta primeira implementao, criamos um novo thread
   * para cada conexo recebida.
   */
  @Override
  public void initService() throws csbase.server.ServerException {
    (new Thread(this)).start();
    loadProjects();
    FileLockManager.getInstance().startThread();
    logInfoMessage("Servio de projetos iniciado.");
    OpenBusService openbus = OpenBusService.getInstance();
    if (openbus != null) {
      if (openbus.isEnabled() && this.exportToOpenBus) {
        Thread thread = new Thread(new Runnable() {
          @Override
          public void run() {
            fileServer.dispatch();
          }
        });
        thread.start();
      }
    }
  }

  /**
   * Modifica as informaes de um projeto.
   * 
   * @param id Identificador do projeto.
   * @param info Novas informaes do projeto.
   * @return As novas informaes do projeto. Caso alguma informao no possa
   *         ser alterada, o valor anterior  retornado.
   */
  public CommonProjectInfo modifyProject(Object id, CommonProjectInfo info) {
    this.checkWritePermission(id);
    try {
      ServerProject sp = ServerProject.getProject(id);
      if (sp == null) {
        throw new InvalidRequestException(getString(INVALID_REQUEST_MSG),
          "Projeto " + id + " no encontrado no servidor");
      }
      sp.setName(info.name);
      sp.setDescription(info.description);
      sp.setAttributes(info.getAttributes());
      sp.modify();
      info.name = sp.getName();
      info.description = sp.getDescription();
      info.setAttributes(sp.getAttributes());
      return info;
    }
    catch (ServerProjectException t) {
      throw new ServiceFailureException(t.getMessage());
    }
  }

  /**
   * {@inheritDoc}
   */
  public void moveFile(Object id, String[] filePath, String[] dirPath) {
    moveFile(id, filePath, id, dirPath);
  }

  /**
   * {@inheritDoc}
   */
  public void moveFile(Object sourceProjectId, String[] filePathSource,
    Object targetProjectId, String[] dirPathTarget) {
    this.checkWritePermission(sourceProjectId);
    this.checkWritePermission(targetProjectId);
    try {
      ServerProjectFile file =
        ServerProjectFile.findFile(sourceProjectId, filePathSource);
      ServerProjectFile dir =
        ServerProjectFile.findFile(targetProjectId, dirPathTarget);
      file.move(dir);
    }
    catch (ServerProjectFileException t) {
      throw new ServiceFailureException(t.getMessage(), t);
    }
  }

  /**
   * {@inheritDoc}
   */
  public boolean existsFile(Object projectId, String[] filePath) {
    try {
      return null != ServerProjectFile.findFile(projectId, filePath);
    }
    catch (ServiceFailureException e) {
      return false;
    }
  }

  /**
   * {@inheritDoc}
   */
  public ProjectFileChannel openFileChannel(Object id, String[] path,
    boolean readOnly) {
    ServerProjectFile spf = null;
    try {
      spf = ServerProjectFile.findFile(id, path);
      if (!readOnly) {
        this.checkWritePermission(spf.getProjectId());
        /*
         * no podemos simplesmente usar isLocked() aqui porque criaramos uma
         * condio de corrida (um lock poderia ser obtido entre a verificao
         * retornar OK e a chamada a spf.openForWriting())
         */
        FileLockManager lockManager = FileLockManager.getInstance();
        synchronized (lockManager) {
          FileLockInterface lock = lockManager.getLock(spf.getAbsolutePath());
          if (lock != null) {
            if (lock.checkLockStatus((SecureKey) getKey()) == LockStatus.LOCKED_BY_OTHERS) {
              throw new ServiceFailureException(String.format(
                getString("ProjectService.error.file.locked.open"), spf
                  .getName()));
            }
            /*
             * como estamos lidando com locks, precisamos fazer a operao ainda
             * no trecho sincronizado
             */
            return spf.openChannel(readOnly, getKey());
          }
        }
      }
      /*
       * execuo no caso em que no estamos usando locks
       */
      return spf.openChannel(readOnly, getKey());
    }
    catch (ServerProjectFileException t) {
      throw new ServiceFailureException(t.getMessage(), t);
    }
  }

  /**
   * {@inheritDoc}
   */
  public Object acquireExclusiveLock(Object id, String[] path) {
    ServerProjectFile spf = ServerProjectFile.findFile(id, path);
    return spf.acquireExclusiveLock();
  }

  /**
   * {@inheritDoc}
   */
  public String getProjectLocationInServer(Object id) throws RemoteException {
    final ServerProject sp = ServerProject.getProject(id);
    final File file = new File(sp.getAbsolutePath());
    try {
      final String canonicalPath = file.getCanonicalPath();
      return canonicalPath;
    }
    catch (IOException e) {
      e.printStackTrace();
      return null;
    }
  }

  /**
   * {@inheritDoc}
   */
  public Object acquireExclusiveLock(Object id, String[] path,
    FileLockListenerInterface listener, long timeout) {
    ServerProjectFile spf = ServerProjectFile.findFile(id, path);
    return spf.acquireExclusiveLock(listener, timeout);
  }

  /**
   * {@inheritDoc}
   */
  public Object acquireSharedLock(Object id, String[] path) {
    ServerProjectFile spf = ServerProjectFile.findFile(id, path);
    return spf.acquireSharedLock();
  }

  /**
   * {@inheritDoc}
   */
  public Object acquireSharedLock(Object id, String[] path,
    FileLockListenerInterface listener, long timeout) {
    ServerProjectFile spf = ServerProjectFile.findFile(id, path);
    return spf.acquireSharedLock(listener, timeout);
  }

  /**
   * {@inheritDoc}
   */
  public int releaseLock(Object id, String[] path, Object lockId) {
    ServerProjectFile spf = ServerProjectFile.findFile(id, path);
    return spf.releaseLock(lockId);
  }

  /**
   * {@inheritDoc}
   */
  public int forceReleaseLock(Object id, String[] path) {
    checkOwnerPermission(id);
    ServerProjectFile spf = ServerProjectFile.findFile(id, path);
    return spf.forceReleaseLock();
  }

  /**
   * Verifica se o usurio  o dono do projeto ou o administrador.
   * 
   * @param projectId identificador do projeto
   */
  private void checkOwnerPermission(Object projectId) {
    ServerProject project = ServerProject.getProject(projectId);
    if (project == null) {
      throw new PermissionException();
    }
    User user = Service.getUser();
    if (project.getUserId().equals(user.getId()) || user.isAdmin()) {
      return;
    }
    throw new PermissionException();
  }

  /**
   * Verifica se o usurio corrente tem permisso de escrita no projeto. O
   * administrador possui permisso de escrita em qualquer projeto.
   * 
   * @param id O identificador do projeto.
   * @throws PermissionException se o usurio no possui permisso
   */
  private void checkWritePermission(Object id) throws PermissionException {
    // FIXME no poderamos retornar true ou false ao invs de lanar uma
    // exceo?
    ServerProject project = ServerProject.getProject(id);
    User user = Service.getUser();
    if (project == null || !project.userHasAccessRW(user.getId())) {
      throw new PermissionException();
    }
  }

  /**
   * Abre um arquivo para leitura exclusiva que no  feita com o uso de
   * <code>ByteBuffer</code>. A implementao desse mtodo determina o uso de
   * RMI para transmisso dos dados.
   * 
   * @param id Identificador do projeto.
   * @param path Caminho do arquivo a ser aberto.
   */
  public void openFileForReading(Object id, String[] path) {
    try {
      ServerProjectFile spf = ServerProjectFile.findFile(id, path);
      spf.openForReading();
    }
    catch (ServerProjectFileException t) {
      throw new ServiceFailureException(t.getMessage(), t);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void openFileForWriting(Object id, String[] path) {
    this.checkWritePermission(id);
    try {
      ServerProjectFile spf = ServerProjectFile.findFile(id, path);
      /*
       * no podemos simplesmente usar isLocked() aqui porque criaramos uma
       * condio de corrida (um lock poderia ser obtido entre a verificao
       * retornar OK e a chamada a spf.openForWriting())
       */
      FileLockManager lockManager = FileLockManager.getInstance();
      synchronized (lockManager) {
        FileLockInterface lock = lockManager.getLock(spf.getAbsolutePath());
        if (lock != null) {
          /*
           * j existe um lock para o path especificado, precisamos verificar se
           * a sesso associada corresponde  sesso corrente e se o lock ainda
           * est vlido
           */
          if (lock.checkLockStatus((SecureKey) getKey()) == LockStatus.LOCKED_BY_OTHERS) {

            throw new ServiceFailureException(String
              .format(getString("ProjectService.error.file.locked.open"), spf
                .getName()));
          }
          /*
           * como estamos lidando com locks, precisamos fazer a operao ainda
           * no trecho sincronizado
           */
          spf.openForWriting();
          return;
        }
      }
      /*
       * execuo no caso em que no estamos usando locks
       */
      spf.openForWriting();
    }
    catch (ServerProjectFileException t) {
      throw new ServiceFailureException(t.getMessage(), t);
    }
  }

  /**
   * {@inheritDoc}
   */
  public CommonClientProject openProject(Object ownerId, String projectName,
    boolean notify) {
    try {
      if (prjAdmin.isLocked(ownerId, projectName)) {
        String message =
          String.format(getString(LNG_PREFIX + "error.project.locked"),
            projectName, ownerId);
        throw new ServiceFailureException(message);
      }
      Object userId = Service.getUser().getId();
      ServerProject sp =
        ServerProject.openProject(userId, ownerId, projectName, notify);
      return buildClientProject(sp.getId(), sp);
    }
    catch (ServerProjectException t) {
      throw new ServiceFailureException(t.getMessage(), t);
    }
  }

  /**
   * L um objeto gravado no arquivo. A implementao desse mtodo utiliza RMI
   * para envio do objeto serializado.
   * 
   * @param id Identificador do projeto.
   * @param path Caminho do arquivo a ser gravado.
   * @return O objeto lido do arquivo.
   */
  public Object readObject(Object id, String[] path) {
    try {
      ServerProjectFile spf = ServerProjectFile.findFile(id, path);
      return spf.readObject();
    }
    catch (Throwable t) {
      throw new ServiceFailureException(t.getMessage(), t);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void rebuildTree(Object id) {
    try {
      ServerProject sp = ServerProject.getProject(id);
      if (sp == null) {
        throw new ServiceFailureException("Projeto " + id + " no encontrado.");
      }
      sp.refreshTree();
    }
    catch (ServerProjectException t) {
      throw new ServiceFailureException(t.getMessage(), t);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void refreshDir(Object projectId, String[] path) {
    if (projectId == null) {
      throw new InvalidRequestException(getString(INVALID_REQUEST_MSG),
        "projectId == null");
    }
    if (path == null) {
      throw new InvalidRequestException(getString(INVALID_REQUEST_MSG),
        "path == null");
    }
    try {
      ServerProjectFile spf = ServerProjectFile.findFile(projectId, path);
      spf.refreshDir();
    }
    catch (OperationFailureException ex) {
      throw new ServiceFailureException(getString(DIR_NOT_FOUND_MSG), ex);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void removeFile(Object id, String[] path) {
    this.checkWritePermission(id);
    try {
      ServerProjectFile spf = ServerProjectFile.findFile(id, path);
      spf.remove();
    }
    catch (ServerProjectFileException t) {
      throw new ServiceFailureException(t.getMessage(), t);
    }
  }

  /**
   * Atualiza as informaes de um arquivo de projeto.
   * 
   * @param id Identificador do projeto.
   * @param path Caminho do arquivo a ser atualizado.
   * @return as informaes atualizadas de um arquivo de projeto
   */
  public ClientProjectFileInfo getUpdatedFileInfo(Object id, String[] path) {
    ServerProjectFile spf = ServerProjectFile.findFile(id, path);
    return spf.getUpdatedInfo();
  }

  /**
   * Filtra dentre os paths passados, aqueles que devem realmente ser removidos.
   * Esse filtro existe para o caso de tentar-se remover um diretrio e um ou
   * mais arquivos que este contm; no  necessrio remover os arquivos, basta
   * remover o diretrio. O filtro tambm remove eventuais caminhos nulos.
   * 
   * @param inputPaths lista de caminhos de arquivos candidatos  remoo.
   * @return lista de caminhos de arquivos que realmente precisam ser removidos.
   */
  private String[][] filterPathsToRemove(String[][] inputPaths) {
    if (inputPaths == null) {
      return null;
    }
    LinkedHashSet<String[]> candidates = new LinkedHashSet<String[]>();
    Iterator<String[]> iter = null;
    inputPathsLoop: for (int inputIndex = 0; inputIndex < inputPaths.length; inputIndex++) {
      if (inputPaths[inputIndex] == null) {
        continue inputPathsLoop;
      }
      if (candidates.size() == 0) {
        candidates.add(inputPaths[inputIndex]);
        continue inputPathsLoop;
      }
      LinkedHashSet<String[]> candidatesClone =
        (LinkedHashSet<String[]>) candidates.clone();
      iter = candidatesClone.iterator();
      while (iter.hasNext()) {
        String[] candidatePath = iter.next();
        if (ServerProjectFile.isAncestor(candidatePath, inputPaths[inputIndex])) {
          continue inputPathsLoop;
        }
        if (ServerProjectFile.isAncestor(inputPaths[inputIndex], candidatePath)) {
          candidates.remove(candidatePath);
        }
      }
      candidates.add(inputPaths[inputIndex]);
    }
    String[][] outputPaths = new String[candidates.size()][];
    iter = candidates.iterator();
    int i = 0;
    while (iter.hasNext()) {
      String[] candidatePath = iter.next();
      outputPaths[i++] = candidatePath;
    }
    return outputPaths;
  }

  /**
   * {@inheritDoc}
   */
  public void removeFiles(Object id, String[][] inputPaths) {
    this.checkWritePermission(id);
    String[][] paths = filterPathsToRemove(inputPaths);
    HashMap<ProjectFile, List<String[]>> parentMap =
      new HashMap<ProjectFile, List<String[]>>();
    for (int i = 0; i < paths.length; i++) {
      ServerProjectFile spf = ServerProjectFile.findFile(id, paths[i]);
      if (!spf.removeFile()) {
        String errMsg = "Falha ao remover " + spf.getAbsolutePath();
        throw (new ServiceFailureException(errMsg));
      }
      ProjectFile parent = spf.getParent();
      List<String[]> removedPathsList = parentMap.get(parent);
      if (removedPathsList == null) {
        removedPathsList = new ArrayList<String[]>();
      }
      removedPathsList.add(spf.getPath());
      parentMap.put(parent, removedPathsList);
    }
    Iterator<List<String[]>> iter = parentMap.values().iterator();
    while (iter.hasNext()) {
      List<String[]> removedPathsList = iter.next();
      String[][] removedFilePaths = new String[removedPathsList.size()][];
      for (int i = 0; i < removedPathsList.size(); i++) {
        removedFilePaths[i] = removedPathsList.get(i);
      }
      fireFilesDeletedEvent(id, removedFilePaths);
    }
  }

  /**
   * Notifica aos observadores que um arquivo foi removido.
   * 
   * @param projectId .
   * @param paths .
   */
  private static void fireFilesDeletedEvent(Object projectId, String[][] paths) {
    Object[] arg = { paths };
    ServerProject project = ServerProject.getProject(projectId);
    project.notifyObservers(ProjectEvent.makeEvent(ProjectEvent.FILES_DELETED,
      arg));
  }

  /**
   * Obtm um projeto no ServerProject. Abre o projeto se ele no estiver
   * aberto.
   * 
   * @param id identificador do projeto
   * @return o projeto.
   */
  public ServerProject getProject(Object id) {
    ServerProject sp = ServerProject.getProject(id);
    if (sp != null) {
      return sp;
    }
    Object owner;
    String name;
    try {
      owner = ServerProject.getOwnerId((String) id);
    }
    catch (ServerProjectException se) {
      throw new ServiceFailureException(
        "No foi possvel obter o dono do projeto " + id + ".", se);
    }
    try {
      name = ServerProject.getProjectName((String) id);
    }
    catch (ServerProjectException se) {
      throw new ServiceFailureException(
        "No foi possvel obter o nome do projeto " + id + ".", se);

    }
    return getProject(id, owner, name);
  }

  /**
   * Obtm um projeto no ServerProject. Abre o projeto se ele no estiver
   * aberto.
   * 
   * @param owner dono do projeto
   * @param name nome do projeto
   * @return o projeto.
   */
  public ServerProject getProject(Object owner, String name) {
    Object id = ServerProject.getId(owner, name);
    return getProject(id, owner, name);
  }

  /**
   * Obtm um projeto no ServerProject. Abre o projeto se ele no estiver
   * aberto.
   * 
   * @param id identificador do projeto
   * @param owner dono do projeto
   * @param name nome do projeto
   * @return o projeto.
   */
  public ServerProject getProject(Object id, Object owner, String name) {
    ServerProject sp = ServerProject.getProject(id);
    if (sp == null) {
      Object userId = Service.getUser().getId();
      try {
        sp = ServerProject.openProject(userId, owner, name, false);
        if (sp == null) {
          throw new ServiceFailureException("Projeto " + id
            + " no pode ser aberto.");
        }
      }
      catch (ServerProjectException se) {
        throw new ServiceFailureException("Projeto " + id
          + " no pode ser aberto.", se);
      }
    }
    return sp;
  }

  /**
   * {@inheritDoc}
   */
  public void removeProject(Object ownerId, String projectName) {
    try {
      String id = ServerProject.getId(ownerId, projectName);
      ServerProject sp = ServerProject.getProject(id);
      if (sp == null) {
        Object userId = Service.getUser().getId();
        sp = ServerProject.openProject(userId, ownerId, projectName, false);
        if (sp == null) {
          throw new ServiceFailureException("Projeto " + id
            + " no encontrado.");
        }
      }
      this.checkWritePermission(id);
      /* Remove os usurios do projeto e notifica */
      Set<Object> removedIds = removeUsers(sp);
      if (removedIds != null) {
        notifyUsers(UsersNotificationData.REMOVE_PROJECT, removedIds, sp);
      }
      sp.remove();
      if (prjAdmin.isUnlockedWithAreaAllocated(ownerId, projectName)) {
        ProjectAdminInfo info =
          prjAdmin.setAllocatedProjectRemoved(ownerId, projectName);
        AdministrationEvent action =
          new AdministrationEvent(AdministrationEvent.MODIFY, info);
        setChanged();
        notifyObservers(action);

        final String message =
          String.format(getString(LNG_PREFIX
            + "msg.project.removed.with.allocation"), info.getProjectName(),
            info.getUserId());
        notifyAdmin(info, message);
      }
      else if (prjAdmin.isLocked(ownerId, projectName)) {
        final ProjectAdminInfo info =
          prjAdmin.getProjectAdminInfo(ownerId, projectName);
        prjAdmin.removeProjectAdminInfo(ownerId, projectName);

        final String message =
          String.format(getString(LNG_PREFIX
            + "msg.project.removed.without.allocation"), info.getProjectName(),
            info.getUserId());
        notifyAdmin(info, message);
      }
      /* Notificando ProjectAdminObserver da remoo do projeto */
      ProjectUserEvent event =
        new ProjectUserEvent(ProjectUserEvent.DELETE, sp.getInfo());
      setChanged();
      notifyObservers(event);
      logInfoMessage("ProjectService:removeProject: projeto" + id
        + " removido com sucesso");
    }
    catch (OperationFailureException ofe) {
      throw new ServiceFailureException(ofe.getMessage(), ofe);
    }
    catch (ServerProjectException spe) {
      throw new ServiceFailureException(spe.getMessage(), spe);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void renameFile(Object id, String[] path, String name) {
    this.checkWritePermission(id);
    try {
      ServerProjectFile spf = ServerProjectFile.findFile(id, path);
      spf.rename(name);
    }
    catch (ServerProjectFileException t) {
      throw new ServiceFailureException(t.getMessage(), t);
    }
  }

  /**
   * Criao de thread de <code>accept</code> em um canal socket.  feita a
   * amarrao dos campos <code>serverName</code> e <code>fileChannelPort</code>
   * pelo arquivo de propriedades ou por valors default.
   */
  public void run() {
    try {
      logDebugMessage("Thread iniciada: ProjectService");
      hostName = Server.getInstance().getHostName();
      logInfoMessage("Servidor = " + hostName);
      fileChannelPort = getIntProperty("ProjectFileChannelProtocol.port");
      logInfoMessage("ProjectFileChannelProtocol.port = " + fileChannelPort);
      InetSocketAddress addr = new InetSocketAddress(hostName, fileChannelPort);
      ServerSocketChannel server = ServerSocketChannel.open();
      ServerSocket socket = server.socket();
      socket.bind(addr);
      logDebugMessage("Thread terminada: ProjectService");
      while (true) {
        (new Thread(new SocketFileChannel(socket.accept()))).start();
      }
    }
    catch (IOException e) {
      logInfoMessage("Erro de inicializao (IO), ProjectService.run: " + e);
    }
  }

  /**
   * {@inheritDoc}
   * 
   * Encerra o servio pelo trmino de <code>prjAdmin</code>.
   */
  @Override
  public void shutdownService() throws csbase.server.ServerException {
    FileLockManager.getInstance().stopThread();
    OpenBusService openbus = OpenBusService.getInstance();
    if (openbus != null) {
      if (openbus.isEnabled() && this.exportToOpenBus) {
        this.fileServer.stop();
      }
    }
    ProjectAdministrator.finish();
    ServerProjectFile.clearAllCache();
  }

  /**
   * Desbloqueia um projeto de um usurio. Esse mtodo somente pode ser chamado
   * pelo administrador do sistema.
   * 
   * @param infos Um array com os objetos <code>ProjectAdminInfo</code> com as
   *        informaes de desbloqueio de projetos devido a alocao de rea
   *        pelo administrador.
   */
  public void unlockProjectsWithAllocation(ProjectAdminInfo[] infos) {
    checkPermission();
    Object userId = Service.getUser().getId();
    try {
      for (int i = 0; i < infos.length; i++) {
        ServerProject sp =
          ServerProject.openProject(userId, infos[i].getUserId(), infos[i]
            .getProjectName(), false);
        if (sp == null) {
          throw new ServiceFailureException("Projeto "
            + infos[i].getProjectName() + " do usurio " + infos[i].getUserId()
            + " no encontrado.");
        }
        Hashtable<String, Object> attributes = sp.getAttributes();
        attributes.put(CommonClientProject.LOCKING_AREA_SIZE, Long
          .valueOf(infos[i].getAreaLockedSize()));
        sp.setAttributes(attributes);
        sp.modify();
        sp.close(false);
        ProjectAdminInfo info =
          prjAdmin.unlockProject(infos[i].getUserId(), infos[i]
            .getProjectName(), infos[i].getAreaLockedSize());
        infos[i] = info;
        /*
         * envia a notificao
         */
        final NotificationData notification = new ProjectNotificationData(info);
        final NotificationService ntSrv = NotificationService.getInstance();
        ntSrv.notifyUserFromService(this, infos[i].getUserId(), notification);
      }
      AdministrationEvent action =
        new AdministrationEvent(AdministrationEvent.MODIFY, infos);
      setChanged();
      notifyObservers(action);
    }
    catch (ServerProjectException spe) {
      throw new ServiceFailureException(spe.getMessage(), spe);
    }
  }

  /**
   * Atualiza a lista de usurios com acesso RO a um projeto.
   * 
   * @param prj - projeto
   * @param usersRO - nova lista de usurios com acesso RO ao projeto
   * 
   * @see #updateUsers(Object, SharingType, Set, Set)
   */
  private void updateUsersRO(ServerProject prj, Set<Object> usersRO) {
    updateUsers(prj.getId(), prj.getSharingType(), usersRO, null);
  }

  /**
   * Atualiza a lista de usurios com acesso RW a um projeto.
   * 
   * @param prj - projeto
   * @param usersRW - nova lista de usurios com acesso RW ao projeto
   * 
   * @see #updateUsers(Object, SharingType, Set, Set)
   */
  private void updateUsersRW(ServerProject prj, Set<Object> usersRW) {
    updateUsers(prj.getId(), prj.getSharingType(), null, usersRW);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void updateUsers(Object projectID, SharingType sharingType,
    Set<Object> usersRO, Set<Object> usersRW) {
    checkWritePermission(projectID);
    try {
      ServerProject sp = ServerProject.getProject(projectID);
      if (sp == null) {
        throw new ServiceFailureException(String.format(getString(LNG_PREFIX
          + "error.project.closed"), projectID));
      }
      SharingType oldSharingType = sp.getSharingType();
      Set<Object> oldUsersRO = sp.getUsersRO();
      Set<Object> oldUsersRW = sp.getUsersRW();

      sp.setSharingType(sharingType);
      if (usersRO != null) {
        sp.setUsersRO(usersRO);
      }
      if (usersRW != null) {
        sp.setUsersRW(usersRW);
      }
      sp.modify();

      switch (sharingType) {
        case ALL_RO:
        case ALL_RW:
          prjAccessBecamePublic(sp, oldSharingType, oldUsersRO, oldUsersRW);
          break;

        case PARTIAL:
          /*
           * acesso passou a ser seletivo. Temos que comparar as listas de
           * acesso antigas com as atuais e notificar aqueles usurios que
           * mudaram de status.
           */
          prjAccessBecamePartial(sp, oldSharingType, oldUsersRO, oldUsersRW);
          break;

        case PRIVATE:
          /*
           * acesso passou a ser privado. Temos que avisar a todos que
           * eventualmente tinham acesso que agora no tm mais.
           */
          prjAccessBecamePrivate(sp, oldSharingType, oldUsersRO, oldUsersRW);
          break;
      }
    }
    catch (ServerProjectException spe) {
      throw new ServiceFailureException(spe.getMessage(), spe);
    }
  }

  /**
   * Compartilhamento do projeto passou a ser seletivo.
   * 
   * @param sp - projeto
   * @param oldSharingType - modo de compartilhamento anterior
   * @param oldUsersRO - usurios que possuam acesso RO
   * @param oldUsersRW - usurios que possuam acesso RW
   */
  private void prjAccessBecamePartial(ServerProject sp,
    SharingType oldSharingType, Set<Object> oldUsersRO, Set<Object> oldUsersRW) {
    Set<Object> newUsersRO = sp.getUsersRO();
    Set<Object> newUsersRW = sp.getUsersRW();
    Set<Object> usersToNotify;
    switch (oldSharingType) {
      case PRIVATE:
        /*
         * projeto era privado. Precisamos avisar a todos os usurios que
         * ganharam acesso.
         */
        usersGainedAccessToProject(sp, newUsersRO);
        usersGainedAccessToProject(sp, newUsersRW);
        break;

      case ALL_RO:
      case ALL_RW:
        /*
         * projeto era pblico. Precisamos remov-lo da lista de projetos
         * pblicos e avisar aos usurios para os quais o tipo de acesso foi
         * alterado
         */
        synchronized (publicProjects) {
          publicProjects.remove(new UserProjectInfo(sp.getUserId(), sp
            .getName()));
        }
        usersToNotify =
          new HashSet<Object>(AdministrationService.getInstance()
            .getAllUserIds());
        Set<Object> alreadyHadAccess;
        Set<Object> gainedAccess;
        if (oldSharingType == SharingType.ALL_RO) {
          alreadyHadAccess = newUsersRO;
          gainedAccess = newUsersRW;
        }
        else {
          alreadyHadAccess = newUsersRW;
          gainedAccess = newUsersRO;
        }
        usersToNotify.removeAll(alreadyHadAccess);
        usersLostAccessToProject(sp, usersToNotify);
        /*
         * agora notificamos todos os usurios que ganharam acesso
         */
        usersGainedAccessToProject(sp, gainedAccess);
        break;

      case PARTIAL:
        /*
         * primeiro notificamos todos os usurios RO que perderam o acesso
         */
        usersLostAccessToProject(sp, oldUsersRO, newUsersRO);
        /*
         * agora notificamos todos os usurios RO que ganharam acesso
         */
        usersGainedAccessToProject(sp, oldUsersRO, newUsersRO);
        /*
         * agora notificamos todos os usurios RW que perderam acesso
         */
        usersLostAccessToProject(sp, oldUsersRW, newUsersRW);
        /*
         * agora notificamos todos os usurios RW que ganharam acesso
         */
        usersGainedAccessToProject(sp, oldUsersRW, newUsersRW);
        break;
    }
    /*
     * precisamos enviar o evento de alterao no compartilhamento de projeto
     * para a unio de todos os conjuntos de usurios: os que tinham acesso (RO
     * + RW) e os que ganharam acesso (RO + RW). As listas que so enviadas no
     * evento so as dos usurios que ainda tm acesso.
     */
    usersToNotify = new HashSet<Object>(oldUsersRO);
    usersToNotify.addAll(newUsersRO);
    usersToNotify.addAll(oldUsersRW);
    usersToNotify.addAll(newUsersRW);
    SharedProjectEvent event =
      new SharedProjectEvent(usersToNotify, sp.getInfo(), newUsersRO,
        newUsersRW, sp.getSharingType());
    setChanged();
    notifyObservers(event);
  }

  /**
   * {@inheritDoc}
   */
  public boolean removeUser(Object projectID, Object userID) {
    checkWritePermission(projectID);
    ServerProject sp = ServerProject.getProject(projectID);
    if (sp == null) {
      throw new ServiceFailureException(String.format(getString(LNG_PREFIX
        + "error.project.closed"), projectID));
    }
    if (sp.userHasSelectiveAccessRO(userID)) {
      Set<Object> newUsers = removeUserFrom(userID, sp.getUsersRO());
      updateUsersRO(sp, newUsers);
      return true;
    }
    if (sp.userHasSelectiveAccessRW(userID)) {
      Set<Object> newUsers = removeUserFrom(userID, sp.getUsersRW());
      updateUsersRW(sp, newUsers);
      return true;
    }
    return false;
  }

  /**
   * {@inheritDoc}
   */
  public void removeUsers(Object projectID, Set<Object> users) {
    checkWritePermission(projectID);
    ServerProject sp = ServerProject.getProject(projectID);
    if (sp == null) {
      throw new ServiceFailureException(String.format(getString(LNG_PREFIX
        + "error.project.closed"), projectID));
    }
    Set<Object> usersRO = new HashSet<Object>(sp.getUsersRO());
    if (usersRO.removeAll(users)) {
      updateUsersRO(sp, usersRO);
    }
    Set<Object> usersRW = new HashSet<Object>(sp.getUsersRW());
    if (usersRW.removeAll(users)) {
      updateUsersRW(sp, usersRW);
    }
  }

  /**
   * Cria um <b>novo</b> conjunto de usurios igual ao original, e remove deste
   * o usurio especificado.
   * 
   * @param userID - identificador do usurio
   * @param users - conjunto de usurios de onde o usurio ser removido
   * @return <b>novo</b> conjunto de usurios igual ao original, de onde o
   *         usurio foi removido
   */
  private Set<Object> removeUserFrom(Object userID, Set<Object> users) {
    HashSet<Object> newUsers = new HashSet<Object>(users);
    newUsers.remove(userID);
    return newUsers;
  }

  /**
   * Acesso ao projeto tornou-se privado. Precisamos remover o acesso a quem j
   * tinha, notificando-os.
   * 
   * @param sp - projeto
   * @param oldSharingType - modo de compartilhamento anterior
   * @param oldUsersRO - usurios que possuam acesso RO anteriormente
   * @param oldUsersRW - usurios que possuam acesso RW anteriormente
   */
  private void prjAccessBecamePrivate(ServerProject sp,
    SharingType oldSharingType, Set<Object> oldUsersRO, Set<Object> oldUsersRW) {
    Set<Object> usersToNotify = null;
    switch (oldSharingType) {
      case PARTIAL:
        /*
         * o compartilhamento anterior era seletivo, precisamos notificar todos
         * os usurios que tinham acesso (RO ou RW)
         */
        usersLostAccessToProject(sp, oldUsersRO);
        usersLostAccessToProject(sp, oldUsersRW);
        break;

      case ALL_RO:
      case ALL_RW:
        /*
         * projeto era pblico (RO ou RW), precisamos remov-lo da lista de
         * projetos pblicos e avisar a todos os usurios que eles perderam a
         * permisso de acesso
         */
        synchronized (publicProjects) {
          publicProjects.remove(new UserProjectInfo(sp.getUserId(), sp
            .getName()));
        }
        usersToNotify = AdministrationService.getInstance().getAllUserIds();
        usersLostAccessToProject(sp, usersToNotify);
        break;

      case PRIVATE:
        // nada a fazer
        break;
    }
    SharedProjectEvent event =
      new SharedProjectEvent(sp.getInfo(), sp.getSharingType());
    setChanged();
    notifyObservers(event);
  }

  /**
   * Notifica os usurios quando um projeto passou a ser pblico (ALL_RO ou
   * ALL_RW). Caso o projeto j estivesse compartilhado seletivamente, os
   * usurios que j possuam o novo tipo de acesso no so notificados. P.ex.
   * se o projeto tornou-se ALL_RO, os usurios que j tinham acesso RO no so
   * notificados.
   * 
   * @param sp - projeto
   * @param oldSharingType - modo de compartilhamento anterior
   * @param oldUsersRO - usurios que possuam acesso RO
   * @param oldUsersRW - usurios que possuam acesso RW
   */
  private void prjAccessBecamePublic(ServerProject sp,
    SharingType oldSharingType, Set<Object> oldUsersRO, Set<Object> oldUsersRW) {
    /*
     * precisamos atualizar a estrutura que armazena os projetos que so
     * pblicos
     */
    synchronized (publicProjects) {
      publicProjects.add(new UserProjectInfo(sp.getUserId(), sp.getName()));
    }
    SharingType sharingType = sp.getSharingType();
    switch (oldSharingType) {
      case PARTIAL:
        /*
         * se o projeto tornou-se pblico RO, devemos avisar aos usurios que
         * tinham acesso RW que no o tem mais. Analogamente, se o projeto
         * tornou-se pblico RW, avisamos aos antigos usurios RO (a rigor, no
         * seria necessrio, mas fazemos por fins de simetria)
         */
        boolean isReadOnly = sharingType == SharingType.ALL_RO;
        Set<Object> usersToNotify = isReadOnly ? oldUsersRW : oldUsersRO;
        usersLostAccessToProject(sp, usersToNotify);
        break;

      case PRIVATE:
        Set<Object> users =
          new HashSet<Object>(AdministrationService.getInstance()
            .getAllUserIds());
        // o dono do projeto no precisa ser notificado
        users.remove(sp.getUserId());
        /*
         * no chamamos usersGainedAccessToProject() porque no queremos inserir
         * este projeto na lista de projetos aos quais o usurio tem acesso
         * (projetos pblicos tm tratamento especial para isto)
         */
        notifyUsers(UsersNotificationData.INSERT, users, sp);
        break;

      case ALL_RO:
      case ALL_RW:
        // nada a ser feito
        break;
    }
    SharedProjectEvent event =
      new SharedProjectEvent(sp.getInfo(), sharingType);
    setChanged();
    notifyObservers(event);
  }

  /**
   * Remove o usurio de um projeto do qual ele no seja o dono.
   * 
   * @param project
   * @param userId
   * @return true caso o usurio tenha sido removido da lista de acesso daquele
   *         projeto com sucesso ou false caso o usurio no tenha acesso ao
   *         projeto, seja dono do projeto ou o projeto seja pblico.
   */
  public boolean removeUserFromOthersProject(ServerProject project,
    Object userId) {

    if (project.isPublic() || project.userIsOwner(userId)
      || !project.userHasAcces(userId)) {
      return false;
    }
    /*
     * se conseguirmos remover o usurio do conjunto RO, atualizamos a lista no
     * projeto, enviamos as notificaes e podemos retornar
     */
    Set<Object> usersRO = project.getUsersRO();
    // TODO precisamos sincronizar por usersRO?
    if (usersRO.remove(userId)) {
      updateUsersRO(project, usersRO);
      return true;
    }
    /*
     * usurio no estava na lista RO, fazemos o mesmo procedimento para a RW
     */
    Set<Object> usersRW = project.getUsersRW();
    // TODO precisamos sincronizar por usersRW?
    if (usersRW.remove(userId)) {
      updateUsersRW(project, usersRW);
      return true;
    }
    /*
     * usurio no estava em nenhuma das listas
     */
    return false;
  }

  /**
   * Grava um objeto no arquivo. A implementao desse mtodo utiliza RMI para
   * envio do objeto serializado.
   * 
   * @param id Identificador do projeto.
   * @param path Caminho do arquivo a ser gravado.
   * @param obj O objeto a ser gravado no arquivo.
   */
  public void writeObject(Object id, String[] path, Object obj) {
    try {
      ServerProjectFile spf = ServerProjectFile.findFile(id, path);
      spf.writeObject(obj);
    }
    catch (ServerProjectFileException t) {
      throw new ServiceFailureException(t.getMessage(), t);
    }
  }

  /**
   * Recupera a propriedade do caminho dos arquivos de tipos de arquivos.
   * 
   * @return caminho dos arquivos de tipos de arquivos.
   */
  protected String getFileTypesProperty() {
    String prop = getStringProperty("FileTypesPath");
    return getOSPropertyPath(prop);
  }

  /**
   * Cria um <code>ClientProject</code> a partir de um
   * <code>ServerProject</code>. A rvore de <code>ClientProjectFile</code>
   * tambm  construda.
   * 
   * @param id ide
   * @param sp projeto servidor
   * @return projeto cliente
   */
  static CommonClientProject buildClientProject(Object id, ServerProject sp) {
    CommonProjectInfo info = new CommonProjectInfo();
    info.userId = sp.getUserId();
    info.name = sp.getName();
    info.description = sp.getDescription();
    info.setAttributes(sp.getAttributes());
    ServerProjectFile serverTree = (ServerProjectFile) sp.getTree();
    ClientProjectFile clientTree =
      buildSingleClientProjectFile(id, null, serverTree);
    return new CommonClientProject(id, info, clientTree, sp.getPath());
  }

  /**
   * Retorna o arquivo pai do arquivo fornecido.
   * 
   * @param projectId identificador do projeto.
   * @param file o arquivo.
   * @return o arquivo-pai (como ClientProjectFile)
   */
  static ClientProjectFile buildParentClientProjectFile(Object projectId,
    ServerProjectFile file) {
    final ServerProjectFile parent = (ServerProjectFile) file.getParent();
    if (parent == null) {
      return buildClientProjectFile(projectId, null, file);
    }
    final ClientProjectFile cParent =
      buildParentClientProjectFile(projectId, parent);
    return buildClientProjectFile(projectId, cParent, file);
  }

  /**
   * Cria um <code>ClientProjectFile</code> a partir de um
   * <code>ServerProjectFile</code>. O pai no  definido.
   * 
   * @param projectId identificador do projeto.
   * @param clientParentDir diretrio pai.
   * @param serverFile <code>ServerProjectFile</code>.
   * @return <code>ClientProjectFile</code>.
   */
  static ClientProjectFile buildClientProjectFile(Object projectId,
    ClientProjectFile clientParentDir, ServerProjectFile serverFile) {
    ClientProjectFile clientFile =
      buildSingleClientProjectFile(projectId, clientParentDir, serverFile);
    ServerProjectFile[] serverChildren =
      (ServerProjectFile[]) serverFile.getChildren();

    if (serverChildren != null) {
      ClientProjectFile[] clientChildren = null;
      clientChildren = new ClientProjectFile[serverChildren.length];
      for (int j = 0; j < serverChildren.length; j++) {
        clientChildren[j] =
          buildClientProjectFile(projectId, clientFile, serverChildren[j]);
      }
      clientFile.setChildren(clientChildren);
    }
    return clientFile;
  }

  /**
   * Criao de client project file
   * 
   * @param clientParentDir diretrio pai.
   * @param projectId id do projeto
   * @param serverFile arquivo servidor
   * @return o arquivo
   */
  static ClientProjectFile buildSingleClientProjectFile(Object projectId,
    ClientProjectFile clientParentDir, ServerProjectFile serverFile) {
    ClientProjectFile clientFile =
      new ClientProjectFile(projectId, serverFile.getName(), clientParentDir,
        serverFile.getType(), null, serverFile.isDirectory(), serverFile
          .isGlobalFile(), serverFile.isPublishedFile(), serverFile
          .isUnderConstruction(), serverFile.getAbsolutePath(), serverFile
          .whoPublished(), serverFile.whichProjectPublished(), serverFile
          .whoCreated(), serverFile.getCreationDate(), serverFile.size(),
        serverFile.getModificationDate(),
        serverFile.getUpdatableFileInfo() != null, serverFile.isLocked());
    return clientFile;
  }

  /**
   * Cria um array de <code>ClientProjectFiles</code> a partir de um array de
   * <code>ServerProjectFiles</code>. Os <code>ClientProjectFiles</code> no
   * contm seus pais definidos.
   * 
   * @param projectId identificador do projeto.
   * @param clientParentDir diretrio pai.
   * @param serverFiles array de <code>ServerProjectFiles</code>.
   * @return array de <code>ClientProjectFiles</code>.
   */
  static ClientProjectFile[] buildClientProjectFiles(Object projectId,
    ClientProjectFile clientParentDir, ServerProjectFile[] serverFiles) {
    int fileCount = serverFiles.length;
    ClientProjectFile[] clientFiles = new ClientProjectFile[fileCount];
    for (int i = 0; i < fileCount; i++) {
      clientFiles[i] =
        buildClientProjectFile(projectId, clientParentDir, serverFiles[i]);
    }
    return clientFiles;
  }

  /**
   * Obtm um array com o caminho para um arquivo dentro do projeto.
   * 
   * @param projectName nome do projeto
   * @param path path
   * @return array
   */
  private String[] buildFilePath(String projectName, String path) {
    String[] newPath = null;
    String[] pathToFile = FileUtils.splitPath(path);
    int length = pathToFile.length;
    if ((length == 1) && pathToFile[0].equals(".")) {
      newPath = new String[length];
      newPath[0] = projectName;
      return newPath;
    }
    newPath = new String[length + 1];
    newPath[0] = projectName;
    for (int i = 0; i < length; i++) {
      if (pathToFile[i].equals(".")) {
        continue;
      }
      if (pathToFile[i].equals("..") && (i > 0)) {
        i--;
      }
      newPath[i + 1] = pathToFile[i];
    }
    return newPath;
  }

  /**
   * Constri o UserProject e adiciona a Hashtable usersProject.
   * 
   * @param configFile Informaes do projeto
   * @throws Exception em caso da falha.
   */
  private void buildUserProject(File configFile) throws Exception {
    if (!configFile.isFile()) {
      return;
    }
    /* L as configuraes do projeto */
    CommonProjectInfo info = null;
    ObjectInputStream in = null;
    try {
      in =
        new ObjectInputStream(new DataInputStream(new BufferedInputStream(
          new FileInputStream(configFile))));
      info = (CommonProjectInfo) in.readObject();
    }
    catch (Exception ex) {
      // impede a criao do objeto;
      String errMsg =
        "ServerProject:ServerProject: error ao ler "
          + configFile.getAbsolutePath();
      throw (new ServerProjectException(errMsg, ex));
    }
    finally {
      if (in != null) {
        in.close();
      }
    }
    if (info == null) {
      return;
    }
    UserProjectInfo userProj = new UserProjectInfo(info.userId, info.name);
    boolean isPublic = ProjectPermissions.isPublic(info);
    if (isPublic) {
      synchronized (publicProjects) {
        publicProjects.add(userProj);
      }
    }
    if (ProjectPermissions.getSharingType(info) == SharingType.PARTIAL) {
      /*
       * projeto  compartilhado seletivamente, obtm todos os usurios que tm
       * acesso ao projeto
       */
      Set<Object> u = ProjectPermissions.getAllUsers(info);
      synchronized (userProjects) {
        for (Object user : u) {
          Vector<UserProjectInfo> userProjInfos = userProjects.get(user);
          if (userProjInfos != null) {
            userProjInfos.add(userProj);
          }
          else {
            /*
             * usurio ainda no est associado a nenhum projeto
             */
            userProjInfos = new Vector<UserProjectInfo>();
            userProjInfos.add(userProj);
            userProjects.put(user, userProjInfos);
          }
        }
      }
    }
  }

  /**
   * Carrega os projetos do sistema, preenchendo o conjunto de projetos
   * pblicos. Alm disso, carrega o mapa de usurios e projetos.
   */
  private void loadProjects() {
    logDebugMessage("Carregando usurios do projeto.");
    publicProjects =
      Collections.synchronizedSet(new HashSet<UserProjectInfo>());
    userProjects = new Hashtable<Object, Vector<UserProjectInfo>>();
    File rootDir = new File(FileSystem.PROJECT_DIR);
    if (!rootDir.exists()) {
      return;
    }
    try {
      /* Todos os diretrios da raiz so os identificadores dos usurios */
      File[] users = rootDir.listFiles(getDirectoryFilter());
      for (int i = 0; i < users.length; i++) {
        /* No diretorio dos usurios esto os projetos e suas configuraes */
        File[] projs = users[i].listFiles(getDirectoryFilter());
        if ((projs == null) || (projs.length == 0)) {
          continue;
        }
        for (int j = 0; j < projs.length; j++) {
          /* projeto */
          try {
            File projectDir = projs[j];
            File configFile = ServerProject.getConfigFile(projectDir);
            if (configFile == null) {
              continue;
            }
            buildUserProject(configFile);
          }
          catch (Exception ex) {
            logSevereMessage("Falha na obteno do projeto "
              + projs[j].getName() + " do usurio " + users[i].getName()
              + ".(ProjectService.loadProjects): " + ex.getMessage());
          }
        }
      }
    }
    catch (Exception ex2) {
      logSevereMessage("Falha na obteno dos projetos por usurio "
        + "(ProjectService.loadProjects): " + ex2.getMessage());
    }
  }

  /**
   * Constroi um filtro de diretrios
   * 
   * @return o filtro
   */
  private FileFilter getDirectoryFilter() {
    return new FileFilter() {
      public boolean accept(File file) {
        return file.isDirectory();
      }
    };
  }

  /**
   * Notifica o administrador sobre um evento do servio de projetos.
   * 
   * @param info A informao sobre o evento ocorrido.
   * @param mailMessage .
   */
  private void notifyAdmin(ProjectAdminInfo info, String mailMessage) {
    try {
      List<Object> adminUsers = new ArrayList<Object>();
      List<User> allUsers = User.getAllUsers();
      for (int i = 0; i < allUsers.size(); i++) {
        User user = allUsers.get(i);
        if (checkPermission(user)) {
          Object uid = user.getId();
          adminUsers.add(uid);
          if (mailMessage != null) {
            final MailService mSrv = MailService.getInstance();
            mSrv.mailUserFromService(this, uid, mailMessage);
          }
        }
      }
      final NotificationData notification = new ProjectNotificationData(info);
      final NotificationService ntSrv = NotificationService.getInstance();
      ntSrv.notifySomeUsersFromService(this, adminUsers, notification);
    }
    catch (Exception e) {
      logSevereMessage("ProjectService:notifyAdmin", e);
    }
  }

  /**
   * Envia notificaes para os usurios quando estes ganham ou perdem acesso a
   * um projeto. Isto acontece nas seguintes situaes:
   * <ul>
   * <li>projeto do qual o usurio participava foi removido
   * <li>projeto tornou-se pblico
   * <li>usurio ganhou acesso ao projeto
   * <li>usurio perdeu acesso ao projeto
   * </ul>
   * 
   * @param eventType - tipo do evento (ver constantes de
   *        {@link UsersNotificationData})
   * @param usersToNotify - usurios a serem notificados
   * @param project - projeto para o qual se ganhou ou perdeu acesso
   */
  private void notifyUsers(int eventType, Collection<Object> usersToNotify,
    ServerProject project) {
    if (usersToNotify.isEmpty()) {
      return;
    }

    /*
     * O administrador nunca perde/ganha acesso a projetos, portanto no h
     * porque notific-lo.
     */
    usersToNotify.remove(User.getAdminId());

    final NotificationData notification =
      new UsersNotificationData(eventType, usersToNotify, project.getUserId(),
        project.getName());
    final NotificationService ntSrv = NotificationService.getInstance();
    ntSrv.notifySomeUsersFromService(this, usersToNotify, notification);
  }

  /**
   * Atualiza a estrutura em memria dos projetos de que cada usurio participa,
   * removendo alguns usurios de um projeto especfico. Envia notificaes para
   * cada usurio que foi removido.
   * 
   * @param sp - projeto do qual os usurios foram removidos
   * @param usersRemoved - identificadores dos usurios removidos do projeto
   */
  private void usersLostAccessToProject(ServerProject sp,
    Set<Object> usersRemoved) {
    if (usersRemoved == null || usersRemoved.isEmpty()) {
      return;
    }
    UserProjectInfo prjInfo = new UserProjectInfo(sp.getUserId(), sp.getName());
    synchronized (userProjects) {
      /*
       * para cada usurio, verificamos se ele estava associado ao projeto em
       * questo; se estava, removemos a associao, e se esta era sua nica
       * associao, removemos o usurio da tabela de associaes
       */
      for (Object uid : usersRemoved) {
        Vector<UserProjectInfo> userProjInfos = userProjects.get(uid);
        if (userProjInfos != null) {
          if (userProjInfos.remove(prjInfo)) {
            if (userProjInfos.isEmpty()) {
              userProjects.remove(uid);
            }
          }
        }
      }
    }
    notifyUsers(UsersNotificationData.REMOVE, usersRemoved, sp);
  }

  /**
   * Atualiza a estrutura em memria dos projetos de que cada usurio participa,
   * removendo alguns usurios de um projeto especfico. Identifica aqueles que
   * devem receber a notificao a partir da lista dos que tinham acesso e da
   * lista dos que tm acesso. Cabe a quem executa o mtodo garantir que as
   * listas dizem respeito aos mesmos tipos de acesso (RO ou RW).
   * 
   * @param sp - projeto
   * @param hadAccess - conjunto dos usurios que tinham acesso de um
   *        determinado tipo ao projeto
   * @param haveAccess - conjunto dos usurios que tm acesso ao projeto
   */
  private void usersLostAccessToProject(ServerProject sp,
    Set<Object> hadAccess, Set<Object> haveAccess) {
    if (hadAccess.isEmpty() || hadAccess.equals(haveAccess)) {
      return;
    }
    Set<Object> usersToNotify = new HashSet<Object>(hadAccess);
    /*
     * no queremos notificar os que tm acesso...
     */
    usersToNotify.removeAll(haveAccess);
    usersLostAccessToProject(sp, usersToNotify);
  }

  /**
   * Atualiza a estrutura em memria dos projetos de que cada usurio participa,
   * adicionando alguns usurios a um projeto especfico. Envia notificaes
   * para cada usurio que foi adicionado.
   * 
   * @param sp - projeto do qual os usurios foram removidos
   * @param usersAdded - identificadores dos usurios adicionados ao projeto
   */
  private void usersGainedAccessToProject(ServerProject sp,
    Set<Object> usersAdded) {
    if (usersAdded == null || usersAdded.isEmpty()) {
      return;
    }
    UserProjectInfo prjInfo = new UserProjectInfo(sp.getUserId(), sp.getName());
    synchronized (userProjects) {
      /*
       * para cada usurio, adicionamos o projeto em questo  sua lista de
       * acessos a projetos
       */
      for (Object uid : usersAdded) {
        Vector<UserProjectInfo> userProjInfos = userProjects.get(uid);
        if (userProjInfos == null) {
          userProjInfos = new Vector<UserProjectInfo>();
          userProjects.put(uid, userProjInfos);
        }
        /*
         * a rigor, permitimos duplicidade por usarmos uma lista ao invs de um
         * conjunto... mas, por construo, no deve acontecer (caso contrrio,
         * deveramos verificar se j no existe antes de adicionar)
         */
        userProjInfos.add(prjInfo);
      }
    }
    notifyUsers(UsersNotificationData.INSERT, usersAdded, sp);
  }

  /**
   * Atualiza a estrutura em memria dos projetos de que cada usurio participa,
   * adicionando alguns usurios a um projeto especfico. Identifica aqueles que
   * devem receber a notificao a partir da lista dos que tinham acesso e da
   * lista dos que tm acesso. Cabe a quem executa o mtodo garantir que as
   * listas dizem respeito aos mesmos tipos de acesso (RO ou RW).
   * 
   * @param sp - projeto
   * @param hadAccess - conjunto dos usurios que tinham acesso de um
   *        determinado tipo ao projeto
   * @param haveAccess - conjunto dos usurios que tm acesso ao projeto
   */
  private void usersGainedAccessToProject(ServerProject sp,
    Set<Object> hadAccess, Set<Object> haveAccess) {
    if (haveAccess.isEmpty() || hadAccess.equals(haveAccess)) {
      return;
    }
    Set<Object> usersToNotify = new HashSet<Object>(haveAccess);
    /*
     * no queremos notificar os que j tinham acesso...
     */
    usersToNotify.removeAll(hadAccess);
    usersGainedAccessToProject(sp, usersToNotify);
  }

  /**
   * Remove os usurios de um projeto.
   * 
   * @param sp Identificador do dono do projeto.
   * @return Vetor de identificadores dos usurios que foram removidos do
   *         projeto.
   * @throws OperationFailureException Falha na remoo dos usrios do projeto
   */
  private Set<Object> removeUsers(ServerProject sp)
    throws OperationFailureException {
    if (sp.isPublic()) {
      Set<Object> allUsers =
        AdministrationService.getInstance().getAllUserIds();
      allUsers.remove(sp.getUserId());
      synchronized (publicProjects) {
        publicProjects
          .remove(new UserProjectInfo(sp.getUserId(), sp.getName()));
      }
      return allUsers;
    }
    Set<Object> users = new HashSet<Object>();
    synchronized (userProjects) {
      for (Enumeration<Object> e = userProjects.keys(); e.hasMoreElements();) {
        Object key = e.nextElement();
        Vector<UserProjectInfo> userProjInfos = userProjects.get(key);

        UserProjectInfo info =
          new UserProjectInfo(sp.getUserId(), sp.getName());
        boolean removed = userProjInfos.remove(info);
        if (userProjInfos.size() == 0) {
          userProjects.remove(key);
        }
        if (removed) {
          users.add(key);
        }
      }
    }
    if (users.size() <= 0) {
      return null;
    }
    return users;
  }

  /**
   * Constri a instncia do servio.
   * 
   * @throws ServerException se ocorrer erro.
   */
  public static void createService() throws ServerException {
    new ProjectService();
  }

  /**
   * Construtor
   * 
   * @throws ServerException se houver falha de servio
   */
  protected ProjectService() throws ServerException {
    super(SERVICE_NAME);
    this.exportToOpenBus = this.getBooleanProperty("export.to.openbus");
    // XXX - Temporrio. Aguardando a soluo do Bug #5847.
    ClientRemoteLocator.projectService = this;
    prjAdmin = new ProjectAdministrator(this);
    this.updaterServices =
      new HashMap<Class<? extends UpdaterService>, UpdaterService>();
    this.openedFiles = new HashMap<byte[], RandomAccessFile>();
    OpenBusService openbus = OpenBusService.getInstance();
    if (openbus != null) {
      if (openbus.isEnabled() && this.exportToOpenBus) {
        try {
          this.config.setHostName(Server.getInstance().getHostName());
          this.config.setPort(this.getIntProperty("file.server.port"));
          this.fileServer = new tecgraf.ftc.server.FileServer(this);
        }
        catch (IOException e) {
          throw new ServerException(e);
        }
        ComponentContext context;
        try {
          context = this.createComponent();
        }
        catch (Exception e) {
          throw new ServerException(e);
        }
        Openbus bus = Openbus.getInstance();
        org.omg.CORBA.Object obj = context.getIComponent();
        IComponent component = IComponentHelper.narrow(obj);
        String iComponentIOR = bus.getORB().object_to_string(component);
        this.projectDataService =
          (ProjectDataService) context.getFacets().get("ProjectDataService");
        projectDataService.setFacetInfo("IComponent", iComponentIOR);
        projectDataService.setFileServer(this.fileServer);
        openbus.addComponent(new ServiceOffer(new Property[0], component));
        logInfoMessage("Servio de Projetos publicado no OpenBus");
      }
    }
    areaReserved = getBooleanProperty("area.reserved");
    // Cria o path completo do diretrio de projetos.
    createProjectDir();
  }

  /**
   * Cria o path completo do diretrio de projetos.
   * 
   * @throws ServerException 
   */
  public void createProjectDir() throws ServerException {
    File prjDir = new File(FileSystem.PROJECT_DIR);
    if (prjDir.exists() && prjDir.isDirectory()) {
      return;
    }
    else {
      final String warn =
        "Diretrio de projeto no existe em " + FileSystem.PROJECT_DIR
          + ". Tentando criar diretrio.";
      logWarningMessage(warn);
    }
    boolean result = prjDir.mkdirs();
    if (!result) {
      final String err =
        "Erro ao criar o diretrio de projetos em " + FileSystem.PROJECT_DIR;
      throw new ServerException(err);
    }    

  }
  
  /**
   * Cria o componente do Servio de Projeto que ser exportado no OpenBus.
   * 
   * @return O contexto do componente SCS.
   * 
   * @throws UserException
   * @throws InstantiationException
   * @throws IllegalAccessException
   * @throws ClassNotFoundException
   * @throws InvocationTargetException
   * @throws NoSuchMethodException
   */
  private ComponentContext createComponent() throws UserException,
    InstantiationException, IllegalAccessException, ClassNotFoundException,
    InvocationTargetException, NoSuchMethodException {
    Openbus bus = Openbus.getInstance();
    ComponentBuilder builder =
      new ComponentBuilder(bus.getRootPOA(), bus.getORB());
    ExtendedFacetDescription[] descriptions = new ExtendedFacetDescription[1];
    descriptions[0] =
      new ExtendedFacetDescription("ProjectDataService",
        IHierarchicalDataServiceHelper.id(), ProjectDataService.class
          .getCanonicalName());
    String componentName = this.getStringProperty("component.name");
    if (componentName==null || componentName.trim().isEmpty()) {
      componentName = COMPONENT_NAME;
    }
    String componentVersion = this.getStringProperty("component.version");
    if (componentVersion==null || componentVersion.trim().isEmpty()) {
      componentVersion = COMPONENT_VERSION;
    }
    final String[] tmp = componentVersion.split("[\\.]");
    byte major = parseByte(tmp[0]);
    byte minor = tmp.length >= 2 ? parseByte(tmp[1]) : 0;
    byte patch = tmp.length >= 3 ? parseByte(tmp[2]) : 0;
    logInfoMessage("Componente ProjectService: nome="+componentName + 
      " verso="+componentVersion);
    return builder.newComponent(descriptions, null, new ComponentId(
      componentName, major, minor, patch, "Java"));
  }

  /**
   * {@inheritDoc}
   */
  public void setFileModificationDate(Object id, String[] path, long date) {
    this.checkWritePermission(id);
    ServerProjectFile spf = ServerProjectFile.findFile(id, path);
    spf.setModificationDate(date);
  }

  /**
   * {@inheritDoc}
   */
  public Set<UserProjectInfo> getAllProjectsInfo(Object userId) {
    Set<UserProjectInfo> projects = new HashSet<UserProjectInfo>();
    String[] projectNames = this.getAllProjectNames(userId);
    if (projectNames != null) {
      for (String projectName : projectNames) {
        projects.add(new UserProjectInfo(userId, projectName));
      }
    }
    Vector<UserProjectInfo> userProjectInfos =
      this.getAllUserProjectsFromOthers(userId);
    if (userProjectInfos != null) {
      projects.addAll(userProjectInfos);
    }
    return projects;
  }

  /**
   * {@inheritDoc}
   */
  public boolean setUnderConstruction(Object projectId, String[] filePath,
    boolean underConstruction) {
    this.checkWritePermission(projectId);
    try {
      ServerProjectFile spf = ServerProjectFile.findFile(projectId, filePath);
      spf.setUnderConstruction(underConstruction);
    }
    catch (ServerProjectFileException e) {
      logSevereMessage("ProjectService:setUnderConstruction", e);
      return false;
    }
    return true;
  }

  /**
   * {@inheritDoc}
   */
  public Set<Object> getManageableProjectsId() {

    Set<Object> manageables = new HashSet<Object>();

    User user = Service.getUser();
    if (user.isAdmin()) {

      Set<Object> allUsers =
        AdministrationService.getInstance().getAllUserIds();

      for (Object anUserId : allUsers) {
        loadAllProjectsId(anUserId, manageables);
      }
    }
    else {
      loadAllProjectsId(user.getId(), manageables);
    }

    return manageables;
  }

  /**
   * Carrega na lista {@code projectsId} o identificador dos projetos
   * pertencentes ao usurio {@code ownerId}
   * 
   * @param ownerId identificador nico do usurio.
   * @param projectsId lista que receber a carga de identificadores nicos dos
   *        projetos pertencentes ao usurio {@code ownerId}.
   */
  private void loadAllProjectsId(Object ownerId, Collection<Object> projectsId) {
    String[] projectNames = this.getAllProjectNames(ownerId);
    if (projectNames != null) {
      for (String projectName : projectNames) {
        Object projectId = ServerProject.getId(ownerId, projectName);
        if (null != projectId) {
          projectsId.add(projectId);
        }
      }
    }
  }

  /**
   * Define as informaes sobre a atualizao de um arquivo.
   * 
   * @param file O arquivo.
   * @param updatableFileInfo As informaes sobre a atualizao do arquivo.
   * 
   * @return {@code true}, caso o arquivo tenha sido atualizado, ou {@code
   *         false}, caso contrrio.
   */
  public boolean setUpdatableFileInfo(ClientProjectFile file,
    UpdatableFileInfo updatableFileInfo) {
    Object projectId = file.getProjectId();
    this.checkWritePermission(projectId);
    ServerProjectFile spf =
      ServerProjectFile.findFile(projectId, file.getPath());
    if (spf == null) {
      return false;
    }
    spf.setUpdatableFileInfo(updatableFileInfo);
    ServerProjectFile parent = (ServerProjectFile) spf.getParent();
    try {
      parent.refreshDir();
    }
    catch (OperationFailureException e) {
      this.logSevereMessage(e.getMessage(), e);
    }
    return true;
  }

  /**
   * {@inheritDoc}
   */
  public boolean startUpdate(Object projectId, String[] filePath, long interval) {
    ServerProjectFile spf = ServerProjectFile.findFile(projectId, filePath);
    UpdatableFileInfo updatableFileInfo = spf.getUpdatableFileInfo();
    if (updatableFileInfo == null) {
      throw new ServiceFailureException(MessageFormat.format(this
        .getString(LNG_PREFIX + "error.not.updatable"), new Object[] { spf
        .getName() }));
    }
    UpdaterService updaterService;
    try {
      updaterService =
        this.updaterServices.get(updatableFileInfo.getServiceClass());
    }
    catch (ClassNotFoundException e) {
      throw new ServiceFailureException(this.getString(LNG_PREFIX
        + "error.updater.service.not.found"));
    }
    return updaterService.startUpdate(spf.getPath(), updatableFileInfo
      .getUserData(), interval);
  }

  /**
   * {@inheritDoc}
   */
  public boolean stopUpdate(Object projectId, String[] filePath) {
    ServerProjectFile spf = ServerProjectFile.findFile(projectId, filePath);
    UpdatableFileInfo updatableFileInfo = spf.getUpdatableFileInfo();
    if (updatableFileInfo == null) {
      throw new ServiceFailureException(MessageFormat.format(this
        .getString(LNG_PREFIX + "error.not.updatable"), new Object[] { spf
        .getName() }));
    }
    UpdaterService updaterService;
    try {
      updaterService =
        this.updaterServices.get(updatableFileInfo.getServiceClass());
    }
    catch (ClassNotFoundException e) {
      throw new ServiceFailureException(this.getString(LNG_PREFIX
        + "error.updater.service.not.found"));
    }
    return updaterService.stopUpdate(spf.getPath(), updatableFileInfo
      .getUserData());
  }

  /**
   * Adiciona um servio responsvel pela atualizao de arquivos.
   * 
   * @param updaterServiceClass A clsse do servio.
   * @param updaterService O servio.
   */
  public void addUpdaterService(
    Class<? extends UpdaterService> updaterServiceClass,
    UpdaterService updaterService) {
    this.updaterServices.put(updaterServiceClass, updaterService);
  }

  /**
   * {@inheritDoc}
   */
  public FileChannel createFileChannel(Object userId, byte[] fileId,
    boolean readOnly) throws tecgraf.ftc.common.exception.PermissionException,
    FailureException {
    Service.setUserId(userId);

    ServerProjectFile spf;
    try {
      spf = this.getProjectFile(fileId);
    }
    catch (ServiceFailureException e) {
      this
        .logSevereMessage(
          "Erro ao obter arquivo de projeto a partir de seu identificador nico.",
          e);
      throw new FailureException(e);
    }
    catch (UnsupportedEncodingException e) {
      this
        .logSevereMessage(
          "Erro ao obter arquivo de projeto a partir de seu identificador nico.",
          e);
      throw new FailureException(e);
    }

    if (spf == null) {
      return null;
    }

    String mode;
    if (readOnly) {
      mode = "r";
    }
    else {
      try {
        this.checkWritePermission(spf.getProjectId());
      }
      catch (PermissionException e) {
        throw new tecgraf.ftc.common.exception.PermissionException(
          MessageFormat.format(this.getString(LNG_PREFIX + "no_permission"),
            spf.getAbsolutePath()));
      }
      mode = "rw";
    }

    try {
      RandomAccessFile file = new RandomAccessFile(spf.getRealFile(), mode);
      if (!readOnly) {
        try {
          spf.setUnderConstruction(true);
        }
        catch (ServerProjectFileException e) {
          this.logSevereMessage("Erro ao colocar o arquivo {@} em construo.",
            new Object[] { spf.getAbsolutePath() }, e);
        }
      }
      this.openedFiles.put(fileId, file);
      return file.getChannel();
    }
    catch (FileNotFoundException e) {
      this.logSevereMessage("O arquivo {0} no foi encontrado.",
        new Object[] { spf.getAbsolutePath() }, e);
      return null;
    }
  }

  /**
   * {@inheritDoc}
   */
  public void fileChannelClosed(Object userId, byte[] fileId) {
    Service.setUserId(userId);

    RandomAccessFile file = this.openedFiles.remove(fileId);
    if (file != null) {
      ServerProjectFile spf = null;
      try {
        spf = this.getProjectFile(fileId);
      }
      catch (UnsupportedEncodingException e) {
        this
          .logSevereMessage(
            "Erro ao obter arquivo de projeto a partir de seu identificador nico.",
            e);
      }

      try {
        file.close();
      }
      catch (IOException e) {
        if (spf == null) {
          this
            .logSevereMessage(
              "Erro ao tentar fechar arquivo utilizado pelo servidor de arquivos.",
              e);
        }
        else {
          this.logSevereMessage("Erro ao tentar fechar o arquivo {0}.",
            new Object[] { spf.getAbsolutePath() }, e);
        }
      }

      if (spf != null) {
        try {
          spf.setUnderConstruction(false);
        }
        catch (ServerProjectFileException e) {
          this.logSevereMessage(
            "Erro ao retirar o arquivo {0} do estado em construo.",
            new Object[] { spf.getAbsolutePath() }, e);
        }
        catch (ServiceFailureException e) {
          this.logSevereMessage(
            "Erro ao retirar o arquivo {0} do estado em construo.",
            new Object[] { spf.getAbsolutePath() }, e);
        }
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  public boolean isLocked(Object userId, byte[] fileId) {
    Service.setUserId(userId);

    ServerProjectFile file;
    try {
      file = this.getProjectFile(fileId);
    }
    catch (UnsupportedEncodingException e) {
      this
        .logSevereMessage(
          "Erro ao obter arquivo de projeto a partir de seu identificador nico.",
          e);
      return true;
    }
    return FileLockManager.getInstance().isLocked(file.getAbsolutePath());
  }

  /**
   * Obtm um arquivo de projeto a partir de um vetor de bytes, representando
   * seu caminho absoluto.
   * 
   * @param fileId O vetor de bytes, representando o caminho absoluto do
   *        arquivo.
   * 
   * @return O arquivo de projeto, ou {@code null}, caso o arquivo no seja
   *         encontrado.
   * 
   * @throws UnsupportedEncodingException Caso ocorra algum erro ao converter o
   *         identificador
   */
  private ServerProjectFile getProjectFile(byte[] fileId)
    throws UnsupportedEncodingException {
    String fileIDString = new String(fileId, Utils.CHARSET_ENCODING);
    String[] fullPath = fileIDString.split(FILE_ID_SEPARATOR);
    String[] path = new String[fullPath.length - 1];
    for (int i = 0; i < path.length; i++) {
      path[i] = fullPath[i + 1];
    }
    Object ownerId = fullPath[0];
    String projectName = path[0];
    ServerProject project = this.getProject(ownerId, projectName);
    return ServerProjectFile.findFile(project.getId(), path);
  }

  /**
   * Gera um identificador unco para um arquivo de projeto.
   * 
   * @param file O arquivo de projeto.
   * 
   * @return O identificador unco.
   */
  public static String generateFileId(ClientProjectFile file) {
    String fileId = (String) file.whoCreated();
    String[] path = file.getPath();
    for (int i = 0; i < path.length; i++) {
      fileId = fileId + FILE_ID_SEPARATOR + path[i];
    }
    return fileId;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void exceptionRaised(Exception e) {
    this.logSevereMessage("Erro no servidor de arquivos de projeto.", e);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void exceptionRaised(Exception e, byte[] fileId) {
    try {
      String fileIdString = new String(fileId, Utils.CHARSET_ENCODING);
      this.logSevereMessage(
        "Erro no servidor de arquivos de projeto ao acessar o arquivo {0}.",
        new Object[] { fileIdString }, e);
    }
    catch (UnsupportedEncodingException uee) {
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public FileServerConfig getConfig() {
    return config;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void setConfig(FileServerConfig config) {
    if (config != null)
      this.config = config;
  }

  /**
   * {@inheritDoc}
   */
  public List<FileInfoSearchResult> getAllFileInfoSearchResult(
    final Object userId, final String projectName, String text,
    boolean isCaseInsensitive, boolean isRegex) {

    String projectId = ServerProject.getId(userId, projectName);
    if (projectId == null)
      return null;

    LocalFileFinder finder =
      new LocalFileFinder(text, isCaseInsensitive, isRegex, true);
    File startDir = new File(projectId);
    List<File> fileList = finder.find(startDir);
    int startDirPrefixLength = startDir.getPath().length();

    final List<FileInfoSearchResult> result =
      new ArrayList<FileInfoSearchResult>();
    for (File file : fileList) {
      /*
       * no queremos exibir o path antes
       */
      String relativePath = file.getParent().substring(startDirPrefixLength);
      result.add(new FileInfoSearchResult(file.getName(), relativePath
        .isEmpty() ? "/" : relativePath, projectName));
    }

    return result;
  }

  /**
   * {@inheritDoc}
   */
  public void setUnallocatedProjectAsAllocated(final Object ownerId,
    final String projectName, final int areaSize) throws RemoteException {

    if (ownerId == null || projectName == null) {
      throw new IllegalArgumentException(getString(LNG_PREFIX
        + "illegal.argument.exception.msg"));
    }

    if (areaSize <= 0) {
      throw new IllegalArgumentException(getString(LNG_PREFIX
        + "illegal.argument.value.exception.msg"));
    }

    final CommonClientProject ccp1 = openProject(ownerId, projectName, false);
    if (ccp1 == null) {
      throw new ServiceFailureException(getString(LNG_PREFIX
        + "project.not.found.exception.msg"));
    }

    ccp1.close(false);
    final boolean isAllocated =
      isUnlockedWithAreaAllocated(ownerId, projectName);
    if (isAllocated) {
      throw new ServiceFailureException(getString(LNG_PREFIX
        + "project.allocated.exception.msg"));
    }

    prjAdmin.lockProject(ownerId, projectName, areaSize);
    try {
      prjAdmin.unlockProject(ownerId, projectName, areaSize);
    }
    catch (final ServerProjectException e) {
      throw new ServiceFailureException(e.getMessage());
    }

    final CommonClientProject ccp2 = openProject(ownerId, projectName, false);
    if (ccp2 == null) {
      throw new ServiceFailureException(getString(LNG_PREFIX
        + "modified.project.access.error.exception.msg"));
    }

    ccp2.setLockingAreaSize(areaSize);
    ccp2.close(false);
  }

  /**
   * {@inheritDoc}
   */
  public void setAllocatedProjectAsUnallocated(final Object ownerId,
    final String projectName) throws RemoteException {

    if (ownerId == null || projectName == null) {
      throw new IllegalArgumentException(getString(LNG_PREFIX
        + "illegal.argument.exception.msg"));
    }

    final CommonClientProject ccp1 = openProject(ownerId, projectName, false);
    if (ccp1 == null) {
      throw new ServiceFailureException(getString(LNG_PREFIX
        + "project.not.found.exception.msg"));
    }

    ccp1.close(false);
    boolean isAllocated = isUnlockedWithAreaAllocated(ownerId, projectName);
    if (!isAllocated) {
      throw new ServiceFailureException(getString(LNG_PREFIX
        + "project.unallocated.exception.msg"));
    }

    prjAdmin.removeProjectAdminInfo(ownerId, projectName);

    final CommonClientProject ccp2 = openProject(ownerId, projectName, false);
    if (ccp2 == null) {
      throw new ServiceFailureException(getString(LNG_PREFIX
        + "modified.project.access.error.exception.msg"));
    }

    ccp2.setLockingAreaSize(0);
    ccp2.close(false);
  }

  /**
   * {@inheritDoc}
   */
  public boolean isAreaReserved() {
    return areaReserved;
  }

  /**
   * {@inheritDoc}
   */
  public ProjectAdminInfo getProjectAdminInfo(Object userId, String projectName) {
    return prjAdmin.getProjectAdminInfo(userId, projectName);
  }
}
