/*
 * $Id$
 */
package csbase.server.services.projectservice.openbus;

import java.rmi.RemoteException;

import tecgraf.openbus.data_service.DataAccessDenied;
import tecgraf.openbus.data_service.DataDescription;
import tecgraf.openbus.data_service.DataNotFound;
import tecgraf.openbus.data_service.DataView;
import tecgraf.openbus.data_service.IHierarchicalDataServicePOA;
import tecgraf.openbus.data_service.InvalidDataKey;
import tecgraf.openbus.data_service.ServiceFailure;
import tecgraf.openbus.data_service.UnknownViewInterface;
import tecgraf.openbus.data_service.UnknownViews;
import csbase.exception.ServiceFailureException;
import csbase.logic.ClientProjectFile;
import csbase.logic.CommonClientProject;
import csbase.logic.UserProjectInfo;
import csbase.remote.ProjectServiceInterface;
import csbase.server.Service;
import csbase.server.services.openbusservice.OpenBusService;
import csbase.server.services.projectservice.ProjectService;

/**
 * Representa um Servio de Dados de Projeto restrito a um nico projeto.
 * 
 * @author Tecgraf/PUC-Rio
 */
public final class RestrictedProjectDataService extends
  IHierarchicalDataServicePOA {
  /**
   * O nome do projeto ao qual este servio est restrito.
   */
  private String projectName;
  /**
   * O Servio de Dados de Projetos original.
   */
  private ProjectDataService projectDataService;

  /**
   * Cria um Servio de Dados de Projeto restrito a um nico projeto.
   * 
   * @param projectName O nome do projeto.
   * @param projectDataService O Servio de Dados de Projeto original.
   */
  public RestrictedProjectDataService(String projectName,
    ProjectDataService projectDataService) {
    this.projectName = projectName;
    this.projectDataService = projectDataService;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public DataDescription[] getRoots() throws ServiceFailure {
    String userId = OpenBusService.getInstance().getRequesterUserId();
    Service.setUserId(userId);
    try {
      ProjectServiceInterface service = ProjectService.getInstance();
      CommonClientProject project =
        service.openProject(userId, this.projectName, false);
      System.out.println(project);
      ClientProjectFile rootFile = (ClientProjectFile) project.getTree();
      System.out.println(rootFile);
      if (rootFile == null) {
        throw new IllegalStateException("rootFile == null");
      }
      try {
        return new DataDescription[] { this.projectDataService
          .createDataDescription(rootFile) };
      }
      catch (InvalidDataKey e) {
        ProjectService.getInstance().logSevereMessage(
          "Erro ao criar descrio para o arquivo {0}.",
          new Object[] { rootFile.getAbsolutePath() }, e);
        throw new ServiceFailure();
      }
    }
    catch (ServiceFailureException e) {
      ProjectService.getInstance().logSevereMessage(
        "Ocorreu um erro ao abrir o projeto {0} do usurio {1}.",
        new Object[] { this.projectName, userId }, e);
      throw new ServiceFailure();
    }
    catch (RuntimeException e) {
      e.printStackTrace();
      throw e;
    }
    catch (RemoteException e) {
      e.printStackTrace();
      throw new ServiceFailure();
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public DataDescription[] getChildren(byte[] fKey) throws ServiceFailure,
    InvalidDataKey {
    try {
      this.checkKey(fKey);
      return this.projectDataService.getChildren(fKey);
    }
    catch (RuntimeException e) {
      e.printStackTrace();
      throw e;
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public DataDescription getParent(byte[] fKey) throws ServiceFailure,
    InvalidDataKey {
    this.checkKey(fKey);
    return this.projectDataService.getParent(fKey);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public DataDescription getDataDescription(byte[] fKey) throws ServiceFailure,
    InvalidDataKey {
    this.checkKey(fKey);
    return this.projectDataService.getDataDescription(fKey);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public DataView getDataView(byte[] fKey, String fViewInterface)
    throws ServiceFailure, UnknownViewInterface, InvalidDataKey {
    try {
      this.checkKey(fKey);
      return this.projectDataService.getDataView(fKey, fViewInterface);
    }
    catch (RuntimeException e) {
      e.printStackTrace();
      throw e;
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public DataView[] getDataViewSeq(byte[][] fKeys, String fViewInterface)
    throws ServiceFailure, UnknownViewInterface, InvalidDataKey {
    for (byte[] key : fKeys) {
      this.checkKey(key);
    }
    return this.projectDataService.getDataViewSeq(fKeys, fViewInterface);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public byte[] createData(byte[] fParentKey, DataDescription fPrototype)
    throws ServiceFailure, InvalidDataKey {
    this.checkKey(fParentKey);
    return this.projectDataService.createData(fParentKey, fPrototype);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public byte[] copyData(byte[] fParentKey, byte[] fSourceKey)
    throws ServiceFailure, UnknownViews, InvalidDataKey {
    this.checkKey(fParentKey);
    this.checkKey(fSourceKey);
    return this.projectDataService.copyData(fParentKey, fSourceKey);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void moveData(byte[] fKey, byte[] fNewParentKey)
    throws ServiceFailure, UnknownViews, InvalidDataKey {
    this.checkKey(fKey);
    this.checkKey(fNewParentKey);
    this.projectDataService.moveData(fKey, fNewParentKey);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void updateData(byte[] fKey, byte[] fSourceKey) throws ServiceFailure,
    UnknownViews, InvalidDataKey {
    this.checkKey(fKey);
    this.checkKey(fSourceKey);
    this.projectDataService.updateData(fKey, fSourceKey);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void deleteData(byte[] fKey) throws ServiceFailure, InvalidDataKey {
    this.checkKey(fKey);
    this.projectDataService.deleteData(fKey);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public byte[] copyDataFrom(byte[] fParentKey, byte[] fSourceKey)
    throws ServiceFailure, InvalidDataKey, UnknownViews, DataAccessDenied,
    DataNotFound {
    this.checkKey(fParentKey);
    return this.projectDataService.copyDataFrom(fParentKey, fSourceKey);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void updateDataFrom(byte[] fKey, byte[] fSourceKey)
    throws ServiceFailure, InvalidDataKey, UnknownViews, DataNotFound,
    DataAccessDenied {
    this.checkKey(fKey);
    this.projectDataService.updateDataFrom(fKey, fSourceKey);
  }

  /**
   * Verifica se uma chave pertence ao projeto.
   * 
   * @param key A chave.
   * 
   * @throws InvalidDataKey Caso a chave seja invlida.
   */
  private void checkKey(byte[] key) throws InvalidDataKey {
    UserProjectInfo projectInfo = ProjectDataService.getUserProjectInfo(key);
    if (!this.projectName.equals(projectInfo.getProjectName())) {
      throw new InvalidDataKey(key);
    }
  }
}