/*
 * $Id: ProjectSynchronizationService.java 95014 2009-08-13 19:52:28Z juliana $
 */
package csbase.server.services.projectsynchronizationservice;

import java.nio.ByteBuffer;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import csbase.exception.ServiceFailureException;
import csbase.logic.ClientProjectFile;
import csbase.logic.ClientProjectFileInfo;
import csbase.logic.CommonClientProject;
import csbase.logic.ProjectFile;
import csbase.logic.ProjectFileFilter;
import csbase.logic.ProjectSynchronizationData;
import csbase.logic.ProjectSynchronizationFileInfo;
import csbase.logic.ProjectSynchronizationListProjects;
import csbase.logic.ProjectSynchronizationResult;
import csbase.logic.ProjectSynchronizationUnit;
import csbase.logic.Session;
import csbase.logic.filters.ProjectFileNameFilter;
import csbase.logic.filters.ProjectFileNullFilter;
import csbase.remote.ProjectFileChannel;
import csbase.remote.ProjectServiceInterface;
import csbase.remote.ProjectSynchronizationMonitor;
import csbase.remote.ProjectSynchronizationServiceInterface;
import csbase.remote.ServerEntryPoint;
import csbase.server.ServerException;
import csbase.server.Service;
import csbase.server.services.loginservice.LoginService;
import csbase.server.services.projectservice.ProjectService;

/**
 * Implementao do Servio de sincronizao de projetos. Este Servio realiza a
 * sincronizao entre dois projetos quaisquer, copiando os arquivos indicados
 * de um para o outro.<br>
 * Atualmente,  necessrio fornecer o usurio e a senha a serem utilizados na
 * conexo com servidores remotos. No futuro, caso o usurio fornecido para a
 * conexo com o servidor remoto seja o mesmo que chama o Servio, o suporte de
 * chaves pblica/privada sero utilizado para que no seja preciso fornecer a
 * senha. No caso de no haver registro de chaves no servidor destino, ou do
 * usurio ser outro, a senha continuar sendo necessria.
 */
public class ProjectSynchronizationService extends Service implements
  ProjectSynchronizationServiceInterface {
  /** Tamanho do buffer usado na sincronizao de arquivos. */
  private static final int BUFFER_SIZE = 64 * 1024; // 64 KB
  /** Nmero mximo de tentativas de escrita */
  private static final int MAX_FAILED_WRITE_CALLS = 1024;
  /** Tempo mximo de espera entre sucessivas transferncias de dados. */
  private static final int MAX_MONITOR_TIME = 30 * 1000; // 30 seg.

  /**
   * Construtor do servio
   * 
   * @throws ServerException
   */
  protected ProjectSynchronizationService() throws ServerException {
    super(SERVICE_NAME);
  }

  /**
   * Cria o servio
   * 
   * @throws ServerException
   */
  public static void createService() throws ServerException {
    new ProjectSynchronizationService();
  }

  /**
   * Obtm a instncia do servio de sincronizao de projetos
   * 
   * @return a instncia do servio de sincronizao de projetos
   */
  public static ProjectSynchronizationService getInstance() {
    return (ProjectSynchronizationService) getInstance(SERVICE_NAME);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void initService() throws ServerException {
    logInfoMessage("Servio de sincronizao de projetos iniciado.");
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void shutdownService() throws ServerException {
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected boolean has2Update(Object arg, Object event) {
    return false;
  }

  /**
   * Adiciona um ProjectSynchronizationFileInfo em map representando o node, se
   * este for um arquivo. Se for um diretrio, faz recurso nos filhos.
   * 
   * @param service
   * @param node .
   * @param map .
   * @throws RemoteException a Exceo lanada.
   */
  private void addFileInfo(ProjectServiceInterface service,
    ClientProjectFile node, Map<String, ProjectSynchronizationFileInfo> map)
    throws RemoteException {
    // Verifica se o arquivo a ser adicionado est na lista de filtros de 
    // arquivos a serem ignorados na sincronizao.
    if (acceptFileFilters(node)) {
      return;
    }
    if (node.isDirectory()) {
      // Temos um problema por causa da carga sob demanda da rvore de
      // projetos. Quando precisamos de mais um n, no podemos pedir ao
      // servidor, pois o projeto em questo pode estar em um servidor
      // remoto no apontado por ele.
      ClientProjectFile[] children = service.getChildren(node);
      for (int i = 0; i < children.length; i++) {
        addFileInfo(service, children[i], map);
      }
    }
    else {
      ClientProjectFileInfo cpfi =
        new ClientProjectFileInfo(node.getType(), node.size(), node
          .getModificationDate(), node.isGlobalFile(), node.isPublishedFile(),
          node.isUnderConstruction(), node.whoPublished(), node
            .whichProjectPublished(), node.isLocked());
      ProjectSynchronizationFileInfo psfi =
        new ProjectSynchronizationFileInfo(cpfi, node.getPath(), false);
      map.put(psfi.generateAbsoluteFilename(), psfi);
    }
  }

  /**
   * Note que alguns mtodos dos objetos da lgica que representam o projeto e
   * suas partes chamam diretamente o servidor atravs do ClientRemoteLocator.
   * Nesse ponto, ns no podemos usar o ClientRemoteLocator pois o projeto em
   * questo pode estar em um servidor remoto no apontado por ele. Com isso,
   * algumas chamadas precisam ser feitas diretamente ao Servio remoto.
   * 
   * @param service .
   * @param projectName .
   * @param userId .
   * 
   * @return .
   * 
   * @throws ServiceFailureException
   */
  private Map<String, ProjectSynchronizationFileInfo> getAllFileInfos(
    ProjectServiceInterface service, String projectName, Object userId) {
    try {
      CommonClientProject project =
        service.openProject(userId, projectName, true);
      if (project == null) {
        throw new ServiceFailureException("Projeto inexistente: " + projectName);
      }
      ProjectFile tree = project.getTree();
      Map<String, ProjectSynchronizationFileInfo> map =
        new HashMap<String, ProjectSynchronizationFileInfo>();
      addFileInfo(service, (ClientProjectFile) tree, map);
      service.closeProject(project.getId(), true);
      return map;
    }
    catch (RemoteException e) {
      throw new ServiceFailureException(
        getString("ProjectSynchronizationService.error.rmi"), e);
    }
  }

  /**
   * Faz o login remoto.
   * 
   * @param serverName nome do servidor. Pode ser <b>null</b>; neste caso, ser
   *        usado o servidor corrente
   * @param login identificador do usurio
   * @param password senha do usurio no servidor remoto
   * @return informaes do login remoto
   */
  private LoginInfo remoteLogin(String serverName, String login, String password) {
    LoginInfo loginInfo = new LoginInfo();
    if (serverName == null) {
      loginInfo.projectService = ProjectService.getInstance();
      return loginInfo;
    }
    try {
      String url = "rmi://" + serverName + "/" + ServerEntryPoint.LOOKUP;
      try {
        loginInfo.server = (ServerEntryPoint) Naming.lookup(url);
        // Engloba as excees NotBoundException, RemoteException e 
        // MalformedURLException que podem ser lanadas por Naming.lookup(url).
      }
      catch (Exception e) {
        loginInfo.server = null;
      }
      if (loginInfo.server == null) {
        String errorMsg =
          this.getString("ProjectSynchronizationService.error.invalid.server");
        errorMsg = MessageFormat.format(errorMsg, serverName);
        throw new ServiceFailureException(errorMsg);
      }
      Object key = Service.getKey();
      Locale locale = LoginService.getInstance().getUserSessionLocale(key);
      loginInfo.serverName = serverName;
      loginInfo.session = loginInfo.server.login(login, password, locale);
      if (loginInfo.session == null) {
        final String errorMsg =
          this
            .getString("ProjectSynchronizationService.error.invalid.user.password");
        throw new ServiceFailureException(errorMsg);
      }
      loginInfo.projectService =
        (ProjectServiceInterface) loginInfo.server.fetchService(
          loginInfo.session.getKey(), ProjectServiceInterface.SERVICE_NAME);

      // System.out.println("-> Testar demora maior que
      // LoginService.cleanTime!");
      // try { Thread.sleep(5000); } catch (Exception e) { e.printStackTrace();
      // }
      return loginInfo;
    }
    catch (Exception e) {
      // Mensagem de erro mais geral.
      String errorMsg = this.getString("ProjectSynchronizationService.error");
      errorMsg = MessageFormat.format(errorMsg, serverName);
      // Concatenao com mensagem de erro mais especfica.
      if (e instanceof ServiceFailureException) {
        if (e.getMessage() != null && !e.getMessage().isEmpty()) {
          errorMsg += " " + e.getMessage();
        }
      }
      throw new ServiceFailureException(errorMsg, e);
    }
  }

  /**
   * Faz o logout no servidor remoto
   * 
   * @param loginInfo informaes do login remoto
   */
  private void remoteLogout(LoginInfo loginInfo) {
    try {
      if (loginInfo.server != null) {
        loginInfo.server.logout(loginInfo.session.getKey());
      }
    }
    catch (Exception e) {
      String errorMsg =
        this.getString("ProjectSynchronizationService.error.close.connection");
      errorMsg = MessageFormat.format(errorMsg, loginInfo.serverName);
      throw new ServiceFailureException(errorMsg, e);
    }
  }

  /**
   * Ajusta o parmetro <code>transfer</code> de cada arquivo de mA para que o
   * arquivo seja transferido se s existir em mA ou se existir em mA e mB mas
   * tiver sido alterado mais recentemente em mA. Caso o arquivo deva ser
   * transferido segundo o critrio acima, mas exista e seja apenas de leitura
   * em mB, ele no  ajustado para ser transferido.
   * 
   * @param mA .
   * @param mB .
   */
  private void adjustTransfer(Map<String, ProjectSynchronizationFileInfo> mA,
    Map<String, ProjectSynchronizationFileInfo> mB) {
    Iterator<String> it = mA.keySet().iterator();
    while (it.hasNext()) {
      String fileName = it.next();
      ProjectSynchronizationFileInfo i1 = mA.get(fileName);
      ProjectSynchronizationFileInfo i2 = mB.get(fileName);
      if ((i2 == null)
        || ((i1.getClientFileInfo().getModificationDate() > i2
          .getClientFileInfo().getModificationDate()) && (!i2
          .getClientFileInfo().isGlobalFile()
          && !i2.getClientFileInfo().isPublishedFile() && !i2
          .getClientFileInfo().isUnderConstruction()))) {
        i1.setTransfer(true);
        i1.setNewFile(true);
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  public ProjectSynchronizationData buildTransferMaps(
    final ProjectSynchronizationData data) {
    ProjectSynchronizationUnit unitA = data.getUnitA();
    ProjectSynchronizationUnit unitB = data.getUnitB();
    if (unitA.getProjectName() == null) {
      throw new ServiceFailureException("Projeto de origem nulo.");
    }
    if (unitB.getProjectName() == null) {
      throw new ServiceFailureException("Projeto de destino nulo.");
    }
    LoginInfo loginInfo =
      remoteLogin(unitA.getServer(), unitA.getLogin(), unitA.getPassword());
    Map<String, ProjectSynchronizationFileInfo> mapA =
      getAllFileInfos(loginInfo.projectService, unitA.getProjectName(), unitA
        .getProjectOwnerId());
    remoteLogout(loginInfo);
    loginInfo =
      remoteLogin(unitB.getServer(), unitB.getLogin(), unitB.getPassword());
    Map<String, ProjectSynchronizationFileInfo> mapB =
      getAllFileInfos(loginInfo.projectService, unitB.getProjectName(), unitB
        .getProjectOwnerId());
    remoteLogout(loginInfo);
    adjustTransfer(mapA, mapB);
    adjustTransfer(mapB, mapA);
    unitA.setFiles(mapA);
    unitB.setFiles(mapB);
    return data;
  }

  /**
   * {@inheritDoc}
   */
  public ProjectSynchronizationListProjects buildListProjects(
    ProjectSynchronizationData data) {
    ProjectSynchronizationUnit unitA = data.getUnitA();
    ProjectSynchronizationUnit unitB = data.getUnitB();
    try {
      LoginInfo loginInfo =
        remoteLogin(unitA.getServer(), unitA.getLogin(), unitA.getPassword());
      Set setA =
        loginInfo.projectService.getAllProjectsInfo(unitA.getProjectOwnerId());
      List listA = new ArrayList(setA);
      Collections.sort(listA);
      remoteLogout(loginInfo);
      loginInfo =
        remoteLogin(unitB.getServer(), unitB.getLogin(), unitB.getPassword());
      Set setB =
        loginInfo.projectService.getAllProjectsInfo(unitB.getProjectOwnerId());
      List listB = new ArrayList(setB);
      Collections.sort(listB);
      remoteLogout(loginInfo);
      ProjectSynchronizationListProjects pslp =
        new ProjectSynchronizationListProjects(listA, listB);
      return pslp;
    }
    catch (RemoteException e) {
      throw new ServiceFailureException("Erro ao obter os projetos.", e);
    }
  }

  /**
   * Transfere integralmente os dados do canal de origem para o canal de
   * destino, utilizando o buffer fornecido.
   * 
   * @param channelFrom canal de origem
   * @param channelTo canal de destino
   * @param buffer buffer a ser usado na transferncia
   * @param monitor monitor da operao de sincronizao
   * 
   * @throws MaxFailedWriteCallsException
   * @throws CancelledException
   * @throws Exception
   */
  private void transfer(ProjectFileChannel channelFrom,
    ProjectFileChannel channelTo, ByteBuffer buffer, Monitor monitor)
    throws MaxFailedWriteCallsException, CancelledException, Exception {
    long size = channelFrom.size();
    long transfered = 0;
    long t0 = System.currentTimeMillis();
    while (transfered < size) {
      buffer.clear();
      int bytesRead = channelFrom.read(buffer, transfered);
      if (bytesRead == 0) {
        transfered = channelFrom.size();
      }
      buffer.flip();
      int bytesWritten = 0;
      int failedWriteCalls = 0;
      while (bytesWritten < bytesRead) {
        int moreBytes = channelTo.write(buffer, transfered + bytesWritten);
        if (moreBytes == 0) {
          failedWriteCalls++;
          if (failedWriteCalls > MAX_FAILED_WRITE_CALLS) {
            throw new MaxFailedWriteCallsException();
          }
        }
        else {
          failedWriteCalls = 0;
        }
        bytesWritten += moreBytes;
        long now = System.currentTimeMillis();
        if ((now - t0) > MAX_MONITOR_TIME) {
          if (monitor.isCancelled()) {
            throw new CancelledException();
          }
          t0 = now;
        }
      }
      transfered += bytesWritten;
    }
    channelTo.truncate(size);
    while (channelTo.size() != size) {
      ;
    }
  }

  /**
   * Copia do projeto de origem para o projeto de destino os arquivos que
   * estiverem marcados para serem transferidos.
   * 
   * @param serviceFrom Servio de projetos de origem
   * @param projectFrom Projeto de origem
   * @param serviceTo Servio de projetos de destino
   * @param projectTo Projeto de destino
   * @param fileInfos Informaes sobre os arquivos
   * @param buffer buffer a ser usado na transferncia
   * @param results Lista onde sero includas mensagens de resultado para cada
   *        arquivo transferido
   * @param monitor Monitor da operao de sincronizao
   */
  private synchronized void copyFiles(ProjectServiceInterface serviceFrom,
    CommonClientProject projectFrom, ProjectServiceInterface serviceTo,
    CommonClientProject projectTo,
    Map<String, ProjectSynchronizationFileInfo> fileInfos, ByteBuffer buffer,
    List<String> results, Monitor monitor) {
    Iterator<String> it = fileInfos.keySet().iterator();
    while (it.hasNext() && (monitor.isCancelled() == false)) {
      String key = it.next();
      ProjectSynchronizationFileInfo fileInfo = fileInfos.get(key);
      if (!fileInfo.getTransfer()) {
        continue;
      }
      String fileName = fileInfo.getFilename();
      String backupName = null;
      boolean fileCreated = false;
      try {
        // Pegando a data
        long modificationDate =
          fileInfo.getClientFileInfo().getModificationDate();

        // Verifica se o arquivo existe no destino como diretrio. Se existir,
        // anota o erro e retorna.
        ProjectFile toFile =
          findFile(serviceTo, projectTo.getTree(), fileInfo
            .getAbsoluteFilename(projectTo.getName()));
        if ((toFile != null) && toFile.isDirectory()) {
          String msg =
            "O projeto " + projectTo.getName()
              + " possui um diretrio com o mesmo nome que " + fileName;
          monitor.partialResult(fileInfo.getAbsoluteFilename(), msg);
          results.add(msg);
          continue;
        }

        ProjectFile toDir =
          findFile(serviceTo, projectTo.getTree(), fileInfo
            .getAbsoluteDirPath(projectTo.getName()));

        // Se no existe o diretorio no destino, cria ele e seus filhos
        if (toDir == null) {
          serviceTo.createDirectory(projectTo.getId(), fileInfo
            .getAbsoluteDirPath(projectTo.getName()));
        }
        else {
          // Se existe, precisamos verificar se no destino, ele  um diretrio
          // Se for um arquivo, anotamos o erro e passamos para o prximos
          if (!toDir.isDirectory()) {
            String msg =
              "O projeto " + projectTo.getName()
                + " possui um arquivo com o mesmo nome que o diretrio "
                + toDir.getName();
            monitor.partialResult(fileInfo.getAbsoluteFilename(), msg);
            results.add(msg);
            continue;
          }
        }

        // verificando se existe arquivo no destino
        if (toFile != null) {
          // Fazendo backup do arquivo no destino
          backupName =
            generateBackupName(serviceTo, projectTo.getTree(), fileInfo
              .getAbsoluteFilename(projectTo.getName()));
          serviceTo.renameFile(projectTo.getId(), fileInfo
            .getAbsoluteFilename(projectTo.getName()), backupName);
        }
        monitor.initTransfer(fileInfo.getFilename());
        // Criando arquivo no destino
        serviceTo.createFile(projectTo.getId(), fileInfo
          .getAbsoluteDirPath(projectTo.getName()), fileInfo.getFilename(),
          fileInfo.getClientFileInfo().getType());
        fileCreated = true;
        serviceTo.closeFile(projectTo.getId(), fileInfo
          .getAbsoluteFilename(projectTo.getName()));
        String description =
          serviceFrom.getFileDescription(projectFrom.getId(), fileInfo
            .getAbsoluteFilename());
        // Setando description do arquivo no destino
        serviceTo.setFileDescription(projectTo.getId(), fileInfo
          .getAbsoluteFilename(projectTo.getName()), description);

        // abrindo canal para o arquivo de origem
        ProjectFileChannel channelFrom =
          serviceFrom.openFileChannel(projectFrom.getId(), fileInfo
            .getAbsoluteFilename(), true);
        channelFrom.open(true);

        // abrindo canal para o arquivo de destino
        ProjectFileChannel channelTo =
          serviceTo.openFileChannel(projectTo.getId(), fileInfo
            .getAbsoluteFilename(projectTo.getName()), false);
        channelTo.open(false);
        // copiando bytes entre os canais
        transfer(channelFrom, channelTo, buffer, monitor);
        // fechando canais
        channelFrom.close();
        channelTo.close();
        serviceTo.setFileModificationDate(projectTo.getId(), fileInfo
          .getAbsoluteFilename(projectTo.getName()), modificationDate);
        // arquivo copiando com sucesso
        monitor.partialResult(fileInfo.getAbsoluteFilename(), null);
      }
      catch (MaxFailedWriteCallsException e) {
        logSevereMessage(e.getMessage() + " - " + projectTo.getName() + " - "
          + fileName);
        String result = "Falha na transferncia de: " + fileName + " - " + e;
        monitor.partialResult(fileInfo.getAbsoluteFilename(), result);
        results.add(result);
        tryRecovery(backupName, fileCreated, serviceTo, projectTo, fileInfo,
          results, monitor);
        backupName = null;
      }
      catch (Exception e) {
        String result =
          "Falha na transferncia de: " + fileName + " - " + e.getMessage();
        logSevereMessage(result, e);
        monitor.partialResult(fileInfo.getAbsoluteFilename(), result);
        results.add(result);
        tryRecovery(backupName, fileCreated, serviceTo, projectTo, fileInfo,
          results, monitor);
        backupName = null;
      }
      finally {
        if (backupName != null) {
          String[] delete =
            fileInfo.getAbsoluteFilename(projectTo.getName()).clone();
          int index = delete.length - 1;
          delete[index] = backupName;
          try {
            serviceTo.removeFile(projectTo.getId(), delete);
          }
          catch (Exception e) {
            String result =
              "Falha na remoo de arquivo de backup: " + delete[index] + " - "
                + e;
            monitor.partialResult(delete, result);
            results.add(result);
          }
        }
      }
    }
  }

  /**
   * Tenta recuperar o backup do arquivo feito antes da cpia.
   * 
   * @param backupName o nome do arquivo de backup
   * @param fileCreated flag para indicar se existe um arquivo destino
   *        inconcistente
   * @param service Servio de projeto
   * @param project Projeto
   * @param fileInfo Informaes sobre o arquivo
   * @param results Lista onde sero includas mensagens de resultado para cada
   *        arquivo transferido
   * @param monitor Monitor da operao de sincronizao
   */
  private void tryRecovery(String backupName, boolean fileCreated,
    ProjectServiceInterface service, CommonClientProject project,
    ProjectSynchronizationFileInfo fileInfo, List<String> results,
    Monitor monitor) {
    try {
      if (fileCreated) {
        service.removeFile(project.getId(), fileInfo
          .getAbsoluteFilename(project.getName()));
      }
      if (backupName != null) {
        String[] actualFilename =
          fileInfo.getAbsoluteFilename(project.getName()).clone();
        actualFilename[actualFilename.length - 1] = backupName;
        service.renameFile(project.getId(), actualFilename, fileInfo
          .getFilename());
      }
    }
    catch (Exception e) {
      logSevereMessage(e.getMessage() + " - " + project.getName());
      String result =
        "Falha na recuperao de: " + fileInfo.getFilename() + " - " + e;
      monitor.partialResult(fileInfo.getAbsoluteFilename(), result);
      results.add(result);
    }
  }

  /**
   * Verifica se existe o arquivo representando pelo <code>absolutePath</code>.
   * 
   * @param service Servio de projetos onde deve procurar o caminho
   * @param pf ProjectFile
   * @param absolutePath path absoluto do arquivo
   * 
   * @return boolean true se o arquivo existe
   * @throws RemoteException
   */
  private boolean existsFile(ProjectServiceInterface service, ProjectFile pf,
    String[] absolutePath) throws RemoteException {
    return (findFile(service, pf, absolutePath) != null);
  }

  /**
   * Retorna o <code>ProjectFile</code> referente ao caminho dado.
   * 
   * @param service Servio de projetos onde deve procurar o caminho
   * @param root diretrio a partir de onde a busca  feita
   * @param path path do arquivo
   * 
   * @return o arquivo se ele existir, ou null caso contrrio
   * @throws RemoteException
   */
  private ProjectFile findFile(ProjectServiceInterface service,
    ProjectFile root, String[] path) throws RemoteException {
    ClientProjectFile file = (ClientProjectFile) root;
    for (int i = 1; i < path.length; i++) {
      file = service.getChild(file, path[i]);
      if (file == null) {
        return null;
      }
    }
    return file;
  }

  /**
   * Gera um nome de backup para um arquivo com base na hora local. A operao 
   * repetida se o nome j existir.
   * 
   * @param service Servio de projetos que contm o arquivo
   * @param pf
   * @param absolutePath
   * 
   * @return o nome
   * @throws RemoteException
   */
  private String generateBackupName(ProjectServiceInterface service,
    ProjectFile pf, String[] absolutePath) throws RemoteException {
    String filename = absolutePath[absolutePath.length - 1];
    String[] path = absolutePath.clone();
    do {
      long time = System.currentTimeMillis();
      path[path.length - 1] = filename + Long.toString(time);
    } while (existsFile(service, pf, path));
    return path[path.length - 1];
  }

  /**
   * Transfere os arquivos entre os projetos A e B, seguindo as indicaes dos
   * atributos <code>transfer</code> da lista A e da lista B.
   * 
   * @param serviceA Servio de projetos que hospeda o projeto A
   * @param serviceB Servio de projetos que hospeda o projeto B
   * @param unitA Dados do projeto A
   * @param unitB Dados do projeto B
   * @param resultsA Lista onde sero includos os resultados para cada
   *        transferncia de arquivo de A para B
   * @param resultsB Lista onde sero includos os resultados para cada
   *        transferncia de arquivo de B para A
   * @param monitor Monitor da operao de sincronizao
   * 
   * @throws ServiceFailureException
   */
  private void transferFiles(ProjectServiceInterface serviceA,
    ProjectServiceInterface serviceB, ProjectSynchronizationUnit unitA,
    ProjectSynchronizationUnit unitB, List<String> resultsA,
    List<String> resultsB, Monitor monitor) {
    try {
      CommonClientProject projectA =
        serviceA.openProject(unitA.getProjectOwnerId(), unitA.getProjectName(),
          true);
      if (projectA == null) {
        String server = (unitA.getServer() != null) ? unitA.getServer() : "";
        String msg =
          String.format(
            getString("ProjectSynchronizationService.error.project.not.found"),
            unitA.getProjectName(), server);
        throw new ServiceFailureException(msg);
      }
      CommonClientProject projectB =
        serviceB.openProject(unitB.getProjectOwnerId(), unitB.getProjectName(),
          true);
      if (projectB == null) {
        String server = (unitB.getServer() != null) ? unitB.getServer() : "";
        String msg =
          String.format(
            getString("ProjectSynchronizationService.error.project.not.found"),
            unitB.getProjectName(), server);
        throw new ServiceFailureException(msg);
      }
      ByteBuffer buffer = ByteBuffer.allocateDirect(BUFFER_SIZE);
      try {
        copyFiles(serviceA, projectA, serviceB, projectB, unitA.getFiles(),
          buffer, resultsA, monitor);
        copyFiles(serviceB, projectB, serviceA, projectA, unitB.getFiles(),
          buffer, resultsB, monitor);
      }
      catch (CancelledException e) {
        // Nada a fazer:  considerado um trmino normal.
      }
      serviceA.closeProject(projectA.getId(), true);
      serviceB.closeProject(projectB.getId(), true);
    }
    catch (RemoteException e) {
      throw new ServiceFailureException(
        getString("ProjectSynchronizationService.error.rmi"), e);
    }
  }

  /**
   * {@inheritDoc}
   */
  public ProjectSynchronizationResult synchronizeProjects(
    ProjectSynchronizationData data, ProjectSynchronizationMonitor monitor) {
    String errorMsg = null;
    ProjectSynchronizationResult result = null;
    List<String> resultsA = new Vector<String>();
    List<String> resultsB = new Vector<String>();
    Monitor localMonitor = new Monitor(monitor);
    try {
      ProjectSynchronizationUnit unitA = data.getUnitA();
      LoginInfo loginInfoA =
        remoteLogin(unitA.getServer(), unitA.getLogin(), unitA.getPassword());
      ProjectSynchronizationUnit unitB = data.getUnitB();
      LoginInfo loginInfoB =
        remoteLogin(unitB.getServer(), unitB.getLogin(), unitB.getPassword());
      transferFiles(loginInfoA.projectService, loginInfoB.projectService,
        unitA, unitB, resultsA, resultsB, localMonitor);
      remoteLogout(loginInfoA);
      remoteLogout(loginInfoB);
    }
    catch (ServiceFailureException e) {
      errorMsg = e.getMessage();
      result =
        new ProjectSynchronizationResult(errorMsg, e.getCause(), resultsA,
          resultsB);
    }
    localMonitor.finalResult(errorMsg);
    if (result == null) {
      result = new ProjectSynchronizationResult(resultsA, resultsB);
    }
    return result;
  }

  /**
   * Monitor da sincronizao. Esta classe  utilizada como uma casca sobre o
   * monitor recebido para tratar as eventuais excees remotas e tambm o caso
   * do monitor ser nulo. Dessa forma, a Implementao dos mtodos no precisam
   * se preocupar com esses casos.
   */
  private class Monitor implements ProjectSynchronizationMonitor {
    /** Referncia para o observador remoto. */
    private final ProjectSynchronizationMonitor monitor;

    /**
     * Construtor do monitor local.
     * 
     * @param monitor monitor remoto
     */
    Monitor(ProjectSynchronizationMonitor monitor) {
      this.monitor = monitor;
    }

    /**
     * {@inheritDoc}
     */
    public void partialResult(String[] file, String result) {
      if (monitor == null) {
        return;
      }
      try {
        monitor.partialResult(file, result);
      }
      catch (RemoteException e) {
        logWarningMessage("Ignorando falha em monitor.partialResult: " + e);
      }
    }

    /**
     * {@inheritDoc}
     */
    public void finalResult(String result) {
      if (monitor == null) {
        return;
      }
      try {
        monitor.finalResult(result);
      }
      catch (RemoteException e) {
        logWarningMessage("Ignorando falha em monitor.finalResult: " + e);
      }
    }

    /**
     * {@inheritDoc}
     */
    public boolean isCancelled() {
      if (monitor == null) {
        return false;
      }
      try {
        return monitor.isCancelled();
      }
      catch (RemoteException e) {
        logWarningMessage("Ignorando falha em monitor.isCancelled: " + e);
        return false;
      }
    }

    /**
     * {@inheritDoc}
     */
    public void initTransfer(String file) throws RemoteException {
      monitor.initTransfer(file);
    }
  }

  /**
   * Estrutura de dados que armazenas as Informaes do login remoto.
   */
  private static class LoginInfo {
    /** Nome do servidor */
    String serverName;
    /** Sesso do login */
    Session session;
    /** Ponto de entrada do servidor remoto */
    ServerEntryPoint server;
    /** Interface remota do servio de projetos */
    ProjectServiceInterface projectService;
  }

  /**
   * Exceo que acusa o excesso de falhas no envio de dados para um arquivo.
   */
  private static class MaxFailedWriteCallsException extends Exception {
    /**
     * Construtor
     */
    MaxFailedWriteCallsException() {
      super("Excesso de falhas consecutivas na escrita.");
    }
  }

  /**
   * Exceo que acusa o cancelamento pelo cliente da operao de sincronizao.
   */
  private static class CancelledException extends RuntimeException {
  }

  /**
   * Verifica se o arquivo passado  aceito pela lista de arquivos a serem
   * ignorados na sincronizao de projetos. Se o nome arquivo casa com pelo
   * menos um filtro, o arquivo  desconsiderado.
   * 
   * @param node Arquivo onde os filtros sero aplicados.
   * @return Se o arquivo casa com algums dos filtros de arquivos a serem
   *         ignorados.
   */
  private boolean acceptFileFilters(final ClientProjectFile node) {

    List<String> filtersList =
      this.getStringListProperty("filesFilterPatternsToHide");
    if (filtersList == null || filtersList.isEmpty()) {
      return false;
    }

    // Verifica se o nome de arquivo casa com algum dos filtros de arquivos.
    boolean accept = false;
    for (String filterName : filtersList) {
      ProjectFileFilter filter = null;
      if (filterName.length() == 0) {
        filter = ProjectFileNullFilter.getInstance();
      }
      else {
        filter = new ProjectFileNameFilter(filterName);
      }
      if (filter.accept(node)) {
        accept = true;
        break;
      }
    } // Fim do FOR dos filtros.
    return accept;

  }

}
