/*
 * PublicationService.java $Author: taciana $ $Revision: 72732 $ - $Date:
 * 2006-09-15 13:43:55 -0300 (Fri, 15 Sep 2006) $
 */
package csbase.server.services.publicationservice;

import java.io.File;
import java.util.*;

import tecgraf.javautils.FileUtils;
import csbase.exception.ServiceFailureException;
import csbase.logic.*;
import csbase.remote.PublicationServiceInterface;
import csbase.remote.ServiceInterface;
import csbase.server.*;
import csbase.server.services.administrationservice.AdministrationService;
import csbase.server.services.administrationservice.AdministrationServiceObserver;
import csbase.server.services.notificationservice.NotificationService;
import csbase.server.services.projectservice.*;

/**
 * A classe <code>PublicationService</code> implementa o servio de gerncia
 * dos dados publicados para os usurios, nos diferentes SSI's.
 *
 * @author Julia
 * @version $Revision: 72732 $
 */
public class PublicationService extends Service implements ServiceInterface,
  PublicationServiceInterface, AdministrationServiceObserver {
  /**
   * Cache com informaes sobre as publicaes efetuadas nos diversos SSIs. A
   * hashtable tem como chave o identificador de um usurios e seu contedo  um
   * vetor de <code>PublicationInfo</code>. Mapeia userid em Vector de
   * PublicationInfo.
   */
  private Hashtable<Object, Vector<PublicationInfo>> publicationData;

  /** Representa o diretrio onde so armazenados os arquivos publicados. */
  private File rootDir;

  /** Define uma ao relacionada a cancelar uma publicao. */
  private static final int UNPUBLISH = 1;

  /** Define uma ao relacionada a criar uma publicao. */
  private static final int PUBLISH = 0;

  /**
   * Obtm a instncia para o servio de publicao.
   *
   * @return A instncia para o servio de publicao.
   */
  public static PublicationService getInstance() {
    return (PublicationService) getInstance(SERVICE_NAME);
  }

  /**
   * Inicializao do servio de gerncia de dados publicados.
   *
   * @throws ServerException se ocorrer um erro na inicializao
   */
  @Override
  public void initService() throws ServerException {
    try {
      logDebugMessage("Iniciando servio de publicao...");
      publicationData = new Hashtable<Object, Vector<PublicationInfo>>();
      this.rootDir = new File(FileSystem.PUBLISH_DIR);
      loadUserPublicationData(this.rootDir);
      AdministrationService.getInstance().addObserver(this);
    }
    catch (Exception e) {
      throw new ServerException(e);
    }
    logInfoMessage("Servio de publicao iniciado.");
  }

  /**
   * Trmino do servio.
   *
   * @see Service#shutdownService()
   */
  @Override
  public final void shutdownService() {
    AdministrationService.getInstance().removeObserver(this);
  }

  /**
   * Obtm todos as informaes de publicao para um usurio.
   *
   * @param userId O identificador do usurio que deseja saber os dados de
   *        publicaes feitas para ele.
   *
   * @return Um vetor com <code>PublicationInfo</code>.
   */
  public Vector<PublicationInfo> getAllPublicationInfo(final Object userId) {
    Vector<PublicationInfo> allPubInfo;
    synchronized (publicationData) {
      allPubInfo = publicationData.get(userId);
      if (allPubInfo == null) {
        allPubInfo = new Vector<PublicationInfo>();
      }
    }
    return allPubInfo;
  }

  /**
   * Publica um arquivo.
   *
   * @param id Identificador do projeto.
   * @param path Caminho do arquivo a ser publicado.
   * @param userIds Lista contendo os identificadores dos usurios que tero
   *        acesso ao arquivo publicado.
   * @param whoPublishedId Identificador do usurio que publicou o arquivo.
   *
   * @return flag indicativo de sucesso.
   */
  public boolean publishFile(final Object id, final String[] path,
    final Object[] userIds, final Object whoPublishedId) {
    try {
      final ServerProjectFile spf = ServerProjectFile.findFile(id, path);
      final boolean success = spf.publish(userIds, whoPublishedId);
      if (success) {
        final File file = spf.getRealFile();
        PublicationInfo info = createPublicationInfo(whoPublishedId, id, file);
        for (int i = 0; i < userIds.length; i++) {
          addPublicationInfo(userIds[i], info);
        }
        notifyUsersPublication(PUBLISH, userIds, info);
      }
      return success;
    }
    catch (ServerProjectFileException e) {
      throw new ServiceFailureException(e.getMessage(), e);
    }
  }

  /**
   * Despublica um arquivo. Despublicar significa retirar todos os usurios da
   * lista de acesso ao arquivo publicado. Assim, quem tem link para ele
   * continua tendo, quem no tem, no consegue mais.
   *
   * @param id Identificador do projeto.
   * @param path Caminho do arquivo a ser despublicado.
   * @param userIds .
   * @param userId Identificador do usurio que quer publicar o arquivo
   *
   * @return Verdadeiro em caso de sucesso e falso caso a operao falhe.
   */

//     public boolean unPublishFile(Object id, String[] path, Object userId)
//     {
//        try
//        {
//           ServerProjectFile spf = ServerProjectFile.findFile(id, path);
//           if (spf == null)
//           {
//              throw new Exception(
//                 "PublicationService: unPublishFile: arquivo no encontrado em "
//                    + id);
//           }
//           File file = spf.getRealFile();
//           Object[] userIds = (Object[]) spf.getPublishAccess().clone();
//           Hashtable publishAccess = (Hashtable) spf.getAccess().clone();
//           boolean success = spf.unPublish(userId);
//           if (success)
//           {
//              for (int i = 0; i < userIds.length; i++)
//              {
//                 PublicationInfo info =
//                    createPublicationInfo(publishAccess.get(userIds[i]), file);
//                 removePublicationInfo(userIds[i], info);
//              }
//              PublicationInfo info = createPublicationInfo(userId, file);
//              notifyUsersPublication(UNPUBLISH, userIds, info);
//           }
//           return success;
//        }
//        catch (Exception e)
//        {
//           logSevereMessage(e.getMessage());
//           return false;
//        }
//     }
  /**
   * Atualiza a lista de publicao do arquivo.
   *
   * @param id Identificador do projeto.
   * @param path Caminho do arquivo publicado.
   * @param userIds Os identificadores dos usurios que, aps essa operao, vo
   *        ter acesso ao arquivo publicado.
   * @param userId Identificador do usurio que quer publicar o arquivo
   *
   * @return Verdadeiro em caso de sucesso e falso caso a operao falhe.
   */
  public boolean updatePublishAccess(final Object id, final String[] path,
    final Object[] userIds, final Object userId) {
    try {
      final ServerProjectFile spf = ServerProjectFile.findFile(id, path);
      Vector<Object> currentAccess = new Vector<Object>(Arrays.asList(spf
        .getPublishAccess()));
      Vector<Object> newAccess = new Vector<Object>(Arrays.asList(userIds));
      Vector<Object> usersToRemove = (Vector<Object>) currentAccess.clone();
      usersToRemove.removeAll(newAccess);
      newAccess.removeAll(currentAccess);
      /*
       * Mantm a informao das publicaes, para remover da cache depois que
       * so removidos do FileConfig
       */
      Hashtable<Object, Object> publishAccess = (Hashtable<Object, Object>) spf
        .getAccess().clone();
      boolean success = spf.updatePublishAccess(usersToRemove.toArray(),
        newAccess.toArray(), userId);
      File file = spf.getRealFile();
      if (success) {
        /* Remoo da publicao */
        for (Enumeration<Object> users = usersToRemove.elements(); users
          .hasMoreElements();) {
          Object user = users.nextElement();
          PublicationInfo unPubInfo = createPublicationInfo(publishAccess
            .get(user), id, file);
          notifyUserPublication(UNPUBLISH, user, unPubInfo);
          removePublicationInfo(user, unPubInfo);
        }
        PublicationInfo info = createPublicationInfo(userId, id, file);
        /* Incluso na publicao */
        for (Enumeration<Object> users = newAccess.elements(); users
          .hasMoreElements();) {
          addPublicationInfo(users.nextElement(), info);
        }
        notifyUsersPublication(PUBLISH, newAccess.toArray(), info);
      }
      return success;
    }
    catch (Exception e) {
      logSevereMessage(e.getMessage());
      return false;
    }
  }

  /**
   * Retorna a lista de usurios para os quais um arquivo foi publicado. Se o
   * arquivo no estiver publicado ou no for encontrado, retorna null.
   *
   * @param id Identificador do projeto.
   * @param path Caminho do arquivo a ser despublicado.
   *
   * @return Um array com os identificadores dos usurios para os quais o
   *         arquivo foi publicado.
   */
  public Object[] getPublishedFileAccess(Object id, String[] path) {
    try {
      ServerProjectFile spf = ServerProjectFile.findFile(id, path);
      return spf.getPublishAccess();
    }
    catch (Exception e) {
      logSevereMessage(e.getMessage());
      return null;
    }
  }

  /**
   * {@inheritDoc}
   */
  public void linkToPublishedFile(Object id, String[] path,
    PublicationInfo info) {
    try {
      ServerProjectFile spf = ServerProjectFile.findFile(id, path);
      if (!spf.isDirectory()) {
        String error =
          String.format(getString("PublicationService.error.destination.dir"),
            spf.getName());
        throw new ServiceFailureException(error);
      }
      File publishedFile = new File(info.getAbsolutePath());
      if (!publishedFile.exists()) {
        String msgError = String.format(
          getString("PublicationService.error.published.file.not.found"),
          publishedFile.getName());
        throw new ServiceFailureException(msgError);
      }
      FileConfig config = ServerProjectFile.getFileConfig(publishedFile);
      if (config == null) {
        String msgError = String.format(
          getString("PublicationService.error.config.published.file.not.found"),
          publishedFile.getName());
        throw new ServiceFailureException(msgError);
      }
      Object[] access = config.getPublishAccess();
      boolean authorized = false;
      Object userId = getUser().getId();
      for (int i = 0; i < access.length; i++) {
        if (userId.equals(access[i])) {
          authorized = true;
          break;
        }
      }
      if (!authorized) {
        String msgError = String.format(
          getString("PublicationService.error.user.not.authorized"),
          userId, publishedFile.getName());
        throw new ServiceFailureException(msgError);
      }
      String linkName = publishedFile.getName();
      File linkFile = new File(spf.getFile(), linkName);
      if (linkFile.exists()) {
        String msgError = String.format(
          getString("PublicationService.error.file.exists"),
          linkName, spf.getName());
        throw new ServiceFailureException(msgError);
      }
      spf.createLinkToFile(linkName, publishedFile, userId);
      /* Cria mensagem no histrico do projeto */
      ServerProject project = ServerProject.getProject(id);
      if (project != null) {
        String text = format(info.getUserWhoPublished(), userId, path,
          publishedFile.getAbsolutePath());
        project.getTree().appendDescription(text);
      }
    } catch (Exception e) {
      throw new ServiceFailureException(e.getMessage(), e);
    }
  }

  /**
   * Retorna uma string com o texto que ser anexado ao histrico do projeto,
   * referente ao resultado de recuperao de arquivos publicados.
   *
   * @param whoPublishedId O identificador do usurio que est publicando.
   * @param userId O identificador do usurio para quem o arquivo est sendo
   *        publicado.
   * @param dir .
   * @param absolutePath .
   *
   * @return .
   */
  private String format(Object whoPublishedId, Object userId, String[] dir,
    String absolutePath) {
    StringBuffer text = new StringBuffer();
    long date = Calendar.getInstance().getTimeInMillis();
    String dateStr = Utilities.getFormattedDate(date);
    text.append("\n*** Arquivo obtido por publicao em " + dateStr
      + " ***\n\n");
    String whoPubName = "";
    String userName = "";
    try {
      whoPubName = User.getName(whoPublishedId);
      userName = User.getName(userId);
    }
    catch (Exception e) {
      if ("".equals(whoPubName)) {
        whoPubName = (String) whoPublishedId;
      }
      if ("".equals(userName)) {
        userName = (String) userId;
      }
    }
    text.append(" Arquivo publicado por " + whoPubName);
    text.append(" e obtido com sucesso por " + userName + ".\n");
    String filePath = FileUtils.joinPath(dir);

    /* Pega o ultimo nome do caminho absoluto para concatenar com o dir */
    String[] completePath = FileUtils.splitPath(absolutePath);
    if (completePath != null) {
      filePath = filePath + File.separator
        + completePath[completePath.length - 1];
    }
    text.append(" - Arquivo: " + filePath + "\n");
    return text.toString();
  }

  /**
   * Atualiza o cache com uma nova informao de publicao.
   *
   * @param userId O usurio para o qual a publicao foi feita.
   * @param info A informao de publicao a ser adicionada.
   */
  private void addPublicationInfo(Object userId, PublicationInfo info) {
    synchronized (publicationData) {
      Vector<PublicationInfo> pubInfoVector = publicationData.get(userId);
      if (pubInfoVector == null) {
        pubInfoVector = new Vector<PublicationInfo>();
        publicationData.put(userId, pubInfoVector);
      }
      pubInfoVector.add(info);
    }
  }

  /**
   * Montagem de um path simplificado (IHM) para exibio.
   *
   * @param info a informao.
   *
   * @return um texto.
   */
  private String getSimplePathFromInfo(final PublicationInfo info) {
    final String[] filePath = info.getPath();
    String pth = "";
    for (int i = 0; i < filePath.length; i++) {
      pth = pth + "/" + filePath[i];
    }
    return pth;
  }

  /**
   * Notifica os usurios sobre o evento de publicao/despublicao.
   *
   * @param type O tipo do evento: PUBLISH ou UNPUBLISH.
   * @param userIds O array com os ids dos usurios a serem notificados.
   * @param info A informao sobre o evento ocorrido.
   */
  private void notifyUsersPublication(final int type, final Object[] userIds,
    final PublicationInfo info) {
    final String tag = (type == PUBLISH ? "publication.add.message"
      : "publication.del.message");
    final String whoPublished = info.getUserWhoPublished().toString();
    final String filePath = getSimplePathFromInfo(info);
    final String[] objs = new String[] { filePath, whoPublished };
    final String msg = getFormattedString(tag, objs);
    final NotificationData notification = new UserNotificationData(msg);
    final NotificationService ntSrv = NotificationService.getInstance();
    ntSrv.notifySomeUsersFromService(this, userIds, notification);
  }

  /**
   * Notifica o usurio sobre o evento de publicao/despublicao.
   *
   * @param type O tipo do evento: PUBLISH ou UNPUBLISH.
   * @param userId o identificador do usurio a ser notificado.
   * @param info A informao sobre o evento ocorrido.
   */
  private void notifyUserPublication(final int type, final Object userId,
    final PublicationInfo info) {
    final String tag = (type == PUBLISH ? "publication.add.message"
      : "publication.del.message");
    final String whoPublished = info.getUserWhoPublished().toString();
    final String filePath = getSimplePathFromInfo(info);
    final String[] objs = new String[] { filePath, whoPublished };
    final String msg = getFormattedString(tag, objs);
    final NotificationData notification = new UserNotificationData(msg);
    final NotificationService ntSrv = NotificationService.getInstance();
    ntSrv.notifyUserFromService(this, userId, notification);
  }

  /**
   * Remove do cache uma informao de publicao para um usurio.
   *
   * @param userId O usurio para o qual a publicao foi removida.
   * @param info A informao de publicao a ser removida.
   */
  private void removePublicationInfo(final Object userId,
    final PublicationInfo info) {
    synchronized (publicationData) {
      final Vector<PublicationInfo> allPubInfo = publicationData.get(userId);
      if (allPubInfo != null) {
        allPubInfo.removeElement(info);
      }
    }
  }

  /**
   * Cria uma informao de publicao para um usurio.
   *
   * @param whoPublishedId O identificador do usurio que publicou o arquivo.
   * @param projectId O identificador do projeto de origem.
   * @param publishedFile O arquivo publicado.
   *
   * @return a informao.
   */
  private PublicationInfo createPublicationInfo(final Object whoPublishedId,
    Object projectId, final File publishedFile) {
    String projectOwner;
    try {
      projectOwner = ServerProjectFile.findPubFileUserId(publishedFile);
      if (projectOwner == null) {
        projectOwner = (String) whoPublishedId;
      }
    }
    catch (ServerProjectFileException e) {
      projectOwner = (String) whoPublishedId;
    }
    String filePath = publishedFile.getPath();
    String sep = File.separator;
    String prefix = FileSystem.PUBLISH_DIR + sep + projectOwner + sep;
    String path = filePath
      .substring(filePath.indexOf(prefix) + prefix.length());
    String projectName = path.substring(0, path.indexOf(sep));
    path = path.substring(projectName.length() + sep.length());
    final PublicationInfo info = new PublicationInfo(null, null,
      whoPublishedId, projectId, projectName, FileUtils.splitPath(path),
      publishedFile.getPath(), publishedFile.length());
    return info;
  }

  /**
   * Carrega a hashtable com informaes sobre publicaes, obtida a partir do
   * sistema de arquivos. A hashtable mapeia userid em Vector de
   * PublicationInfo.
   *
   * @param file O arquivo (ou diretrio) a partir do qual so extradas as
   *        informaes sobre publicao.
   *
   * @throws Exception Caso ocorra alguma problema ao carregar o arquivo.
   */
  private void loadUserPublicationData(File file) throws Exception {
    if (file.isDirectory()) {
      File[] files = file.listFiles();
      for (int i = 0; i < files.length; i++) {
        loadUserPublicationData(files[i]);
      }
      return;
    }
    if (!FileConfig.isControlledFileName(file.getName())) {
      FileConfig config = ServerProjectFile.getFileConfig(file);
      if (config == null) {
        return;
      }
      Hashtable<Object, Object> userIds = config.getAccess();
      if (userIds == null) {
        return;
      }
      for (Enumeration<Object> e = userIds.keys(); e.hasMoreElements();) {
        Object publishedTo = e.nextElement();
        PublicationInfo info = createPublicationInfo(userIds.get(publishedTo),
          config.getSourceProjectId(), file);
        addPublicationInfo(publishedTo, info);
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected boolean has2Update(Object arg, Object event) {
    return true;
  }

  /**
   * Constri o servio que gerencia os dados publicados.
   *
   * @throws ServerException Caso ocorra algum erro na criao do servio.
   */
  public static void createService() throws ServerException {
    new PublicationService();
  }

  /**
   * Cria uma instncia do servio de publicao.
   *
   * @throws ServerException Caso ocorra algum erro na criao do servio.
   */
  protected PublicationService() throws ServerException {
    super(SERVICE_NAME);
  }

  /**
   * {@inheritDoc}
   */
  public void userWillBeDeleted(User user) {
    List<PublicationInfo> publicationList = new ArrayList<PublicationInfo>(this
      .getAllPublicationInfo(user.getId()));
    Iterator<PublicationInfo> publicationIterator = publicationList.iterator();
    while (publicationIterator.hasNext()) {
      PublicationInfo publicationInfo = publicationIterator.next();
      String[] publicationPath = publicationInfo.getPath();
      String[] sourcepublicationPath = new String[publicationPath.length + 1];
      sourcepublicationPath[0] = publicationInfo.getProjectName();
      for (int i = 1; i < sourcepublicationPath.length; i++) {
        sourcepublicationPath[i] = publicationPath[i - 1];
      }
      Object[] usersId = this.getPublishedFileAccess(publicationInfo
        .getProjectId(), sourcepublicationPath);
      if (usersId != null && usersId.length > 0) {
        Object[] newUsersId = new Object[usersId.length - 1];
        int j = 0;
        for (int i = 0; i < usersId.length; i++) {
          if (!usersId[i].equals(user.getId())) {
            newUsersId[j] = usersId[i];
            j++;
          }
        }
        this.updatePublishAccess(publicationInfo.getProjectId(),
          sourcepublicationPath, newUsersId, publicationInfo
            .getUserWhoPublished());
      }
    }
  }
}
