/*
 * $Id$
 */
package csbase.server.services.schedulerservice;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.Vector;

import csbase.logic.CommandEvent;
import csbase.logic.CommandFailedNotificationData;
import csbase.logic.CommandInfo;
import csbase.logic.Priority;
import csbase.logic.SGASet;
import csbase.logic.CommandEvent.EventType;
import csbase.logic.algorithms.AlgorithmConfigurator;
import csbase.logic.algorithms.ExecutionType;
import csbase.server.Service;
import csbase.server.services.notificationservice.NotificationService;
import csbase.server.services.sgaservice.SGAService;

/**
 * Classe responsvel por submeter os comandos ao <code>SGAService</code>.
 *
 * Para submeter um comando para um servidor, o <code>Scheduler</code>
 * considera os seguintes critrios: - peso do algoritmo (CPU e memria); -
 * disponibilidade da mquina; - permisso do usurio;
 *
 * O envio de comandos para o <code>SGAService</code>  avaliado de acordo com
 * o seu tipo (simples, distribudo ou paralelo).
 *
 * Na configurao da execuo de um comando, o usurio escolhe manualmente os
 * servidores para execuo ou deixa que o <code>Scheduler</code> tome a
 * deciso de onde o comando ser executado.
 */
public class Scheduler {
  /** Thread do escalonador */
  private Thread schedulerThread;
  /** Indica o trmino do processamento da fila de comandos. */
  private boolean exitSchedulerThread;
  /** Fila que controla a prioridade dos processos */
  private final PriorityQueue commandQueue;
  /** Referncia para o servio. */
  private final SchedulerService service;
  /** SGAService */
  private final SGAService sgaService;
  /** Mapeamento dos sgas que acabaram de receber comandos a uma data */
  private final HashMap<String, Long> recentUsedServers;
  /**
   * Tempo que o SGA fica de quarentena depois que recebe um comando. O valor
   * default  zero. Neste caso no h quarentena.
   */
  private final int sgaQuarantine;
  /**
   * Intervalo de processamento da fila (milisegundos). O valor default  duas
   * vezes o intervalo de atualizao das informaes do SGA.
   */
  private final long queueProcessingInterval;

  /** Objeto que executa a poltica de escalonamento ativa */
  private SchedulerPolicyInterface schedPolicyInterface;

  /** Identificador da poltica de escalonamento que deve ser empregada na
   * escolha dos melhores SGAs ativos */
  private final String schedPolicyName;

  /**
   * Constri a representao do Scheduler.
   * @param commandQueue A fila de comandos a ser processada.
   * @param queueProcessingInterval Intervalo de processamento da fila.
   * @param sgaQuarantine Tempo de quarentena dos SGAs.
   * @param schedPolicyName Poltica de escalonamento ativa.
   */
  protected Scheduler(PriorityQueue commandQueue,
                      long queueProcessingInterval,
                      int sgaQuarantine,
                      String schedPolicyName) {
    this.commandQueue = commandQueue;
    this.queueProcessingInterval = queueProcessingInterval;
    this.sgaQuarantine = sgaQuarantine;
    this.schedPolicyName = schedPolicyName;
    this.sgaService = SGAService.getInstance();
    this.service = SchedulerService.getInstance();
    this.recentUsedServers = new HashMap<String, Long>();
  }

  /**
   * Cria a thread de processamento da fila de comandos. A thread  acordada a
   * cada novo comando submetido ou no intervalo de
   * <code>queueProcessingInterval</code>. Para cada comando, escolhe o
   * servidor para execuo e submete o comando no servidor escolhido. Coloca o
   * servidor em quarentena, depois que ele  escolhido. Caso no encontre
   * servidor disponvel para executar, o comando permanece na fila.
   */
  protected void startScheduler() {
    loadSchedulerPolicy();

    schedulerThread = new Thread(new Runnable() {
      public void run() {
        while (!exitSchedulerThread) {
          service.logDebugMessage("Incio do processamento, se houver comandos");

          CommandInfo[] cmds = commandQueue.getAndWaitForCommands();
          List<String> sgasNames = null;
          sgasNames = sgaService.getAllSGANames();
          clearQuarantine();

          int numOfCommandsToSchedule = cmds.length;
          for (int i = 0; i < cmds.length && !exitSchedulerThread; i++) {
            CommandInfo cmd = cmds[i];
            List<String> serversChoosed = chooseServers(cmd, sgasNames);
            if (serversChoosed == null) {
              if (cmd.getPriority() == Priority.ROOT) {
                // Retira da lista
                commandQueue.remove(cmd.getId());
                numOfCommandsToSchedule--;
                service
                  .logWarningMessage("Falha ao disparar comando com prioridade ROOT: "
                    + cmd.getId());
                // Notifica que no disparou!
                notifyCommandFailed(cmd);
              }
              continue;
            }

            // O primeiro da lista  o mestre quando a execuo  distribuda.
            // Para simples ou paralela, s h um servidor escolhido.
            String serverChoosed = serversChoosed.get(0);
            service.logDebugMessage("Servidor aprovado: " + serverChoosed);

            // Submete o comando!
            boolean ok = false;
            try {
              cmd.setSGAName(serverChoosed);
              ok =
                sgaService.executeCommand(cmd, service
                  .getCommandObservers(cmd.getId()));
            } catch (Exception e) {
              service.logSevereMessage(String
                .format("Erro ao submeter o comando %s - %s.",
                  cmd.getId(),
                  cmd.getTip()), e);
            }

            if (ok) {
              commandQueue.remove(cmd.getId());
              service.removeCommandObservers(cmd.getId());
              // Coloca os servidores na tabela de recm utilizado
              for (String server : serversChoosed) {
                recentUsedServers.put(server, System.currentTimeMillis());
              }
              numOfCommandsToSchedule--;
            }
            else {
              // Notifica que no disparou!
              notifyCommandFailed(cmd);
              // Retira da lista
              commandQueue.remove(cmd.getId());
              numOfCommandsToSchedule--;
            }
          }

          // Espera um tempo para reprocessar os elementos que ainda esto
          // na fila. Mas, nesse meio tempo podem ter entrado outros processos.
          if ((commandQueue.size() > 0)
            && (commandQueue.size() == numOfCommandsToSchedule)) {
            service
            .logDebugMessage("Ainda h comandos para processar. Fila em sleep.");
            commandQueue.sleep(queueProcessingInterval);
          }
        }
      }
    });

    schedulerThread.start();
  }

  /**
   * Interrompe a execuo da <code>Thread</code>.
   */
  protected void stopScheduler() {
    this.exitSchedulerThread = true;
  }

  /**
   * Instancia um objeto contendo a poltica de escalonamento ativa.
   * @return Objeto com a poltica de escalonamento ativa.
   */
  private SchedulerPolicyInterface loadSchedulerPolicy() {    
    try {
      Class c = Class.forName(schedPolicyName);
      schedPolicyInterface = (SchedulerPolicyInterface) c.newInstance();
    }
    catch (Exception e) {
      schedPolicyInterface = new PercCPUPolicy();
      service.logSevereMessage(String
        .format("Erro ao ativar a poltica de escalonamento %s.",
          schedPolicyName), e);
    }
 
    service.logInfoMessage("Classe de escalonamento instanciada: " +
      schedPolicyInterface.getClass().getName() + ".java.");

    return schedPolicyInterface;
  }

  /**
   * Escolhe, dentre os servidores disponveis, um conjunto para processar o 
   * comando. Quando o comando  simples ou paralelo, retornamos uma lista com 
   * apenas um servidor. Quando o comando  distribudo, retornamos a lista com
   * os servidores.
   * @param command Comando para execuo
   * @param serversNames Lista com todos os servidores disponveis no servidor.
   * 
   * @return Uma lista com os servidores escolhidos para execuo do comando.
   */
  private List<String> chooseServers(CommandInfo command,
                                     List<String> serversNames) {
    // Indica qual usurio est executando o comando.
    Service.setUserId(command.getUserId());

    try {
      AlgorithmConfigurator configurator = command.getConfigurator();
      // Atualiza informaes sobre o algoritmo.
      configurator.updateAlgorithmVersion();
      Set<String> platformsSet = configurator.getPlatforms();
      String platformRestriction =
        command.getPlatformFilter();
      Vector<String> platforms = new Vector<String>();
      if (platformRestriction == null) {
        platforms.addAll(platformsSet);
      }
      else {
        if (platformsSet.contains(platformRestriction)) {
          platforms.add(platformRestriction);
        }
        else {
          return null;
        }
      }
      float cpuAmount = configurator.getCpuAmount();
      float memoryAmount = configurator.getMemoryAmount();
      Set<String> requirements = configurator.getRequirements();

      service.logDebugMessage("Processando o comando: " + command);
      service.logDebugMessage("Tipo do comando: "
        + configurator.getExecutionType());
      service.logDebugMessage("Unidade de processamento necessria: "
        + cpuAmount);
      service.logDebugMessage("Memria necessria: " + memoryAmount);
      for (String requirement : requirements) {
        service.logDebugMessage("Requisito necessrio: " + requirement);
      }
      if (command.isAutomatic()) {
        service.logDebugMessage("Seleo automtica de servidores");
      } else {
        service.logDebugMessage("Seleo manual. Servidor escolhido: " +
          command.getSGAName());
      }

      ExecutionType type = configurator.getExecutionType();

      // Escolhe o servidor de acordor com o seu tipo do configurador.
      List<String> serversChoosed = null;
      String serverChoosed;
      switch (type) {
        case PARALLEL:
          serverChoosed = chooseServerForParallelCommand(command,
            serversNames,
            platforms);
          if (serverChoosed != null) {
            serversChoosed = new ArrayList<String>();
            serversChoosed.add(serverChoosed);
          }
          break;
        case DISTRIBUTED:
          serversChoosed = chooseServersForDistributedCommand(command,
            serversNames,
            platforms,
            cpuAmount,
            memoryAmount,
            requirements);
          break;
        default:
          // Simples e mltipla
          serverChoosed = chooseServerForSimpleCommand(command,
            serversNames,
            platforms,
            cpuAmount,
            memoryAmount,
            requirements);
          if (serverChoosed != null) {
            serversChoosed = new ArrayList<String>();
            serversChoosed.add(serverChoosed);
          }
      }
      return serversChoosed;
    } catch (Exception e) {
      service.logSevereMessage(String
        .format("Erro ao processar o comando %s - %s.",
          command.getId(),
          command.getTip()), e);
      return null;
    }
  }

  /**
   * Obtm o servidor para execuo do comando paralelo. No comando paralelo, 
   * necessrio verificar apenas se o servidor  cluster, se o usurio possui
   * permisso e se o algoritmo possui binrios na plataforma do servidor.
   *
   * @param command Comando para execuo.
   * @param servers Lista ordenada de servidores.
   * @param platforms Lista das plataformas em que o algoritmo pode ser
   *        executado.
   * @return Servidor escolhido para submisso do comando.
   */
  private String chooseServerForParallelCommand(CommandInfo command,
                                                List<String> servers,
                                                Vector<String> platforms) {
    // Automtica
    if (command.isAutomatic()) {
      List<String> candidateServers = new ArrayList<String>();
      for (String serverName : servers) {
        service.logDebugMessage("Tentando sga: " + serverName);
        if (checkServerParallelRestrictions(serverName, platforms))
          candidateServers.add(serverName);
      }
      
      return schedPolicyInterface.chooseServer(command, candidateServers);
    }
    // Manual
    String serverName = command.getSGAName();
    service.logDebugMessage("Verificando sga: " + serverName);
    if (checkServerParallelRestrictions(serverName, platforms)) {
      return serverName;
    }
    return null;
  }

  /**
   * Obtm o sga para execuo do comando simples.
   *
   * @param command Comando para execuo.
   * @param servers Lista ordenada de servidores.
   * @param platforms Lista das plataformas em que o algoritmo pode ser
   *        executado.
   * @param cpuAmount Unidade de processamento do algoritmo, ou seja, quanto de
   *        CPU o algoritmo precisa.
   * @param memoryAmount Quantidade de memria (em Mb) necessria para executar
   *        o comando.
   * @param requirements Requisitos necessrios para executar o comando.
   * @return sga escolhido para submisso do comando.
   */
  private String chooseServerForSimpleCommand( CommandInfo command,
    List<String> servers, Vector<String> platforms, float cpuAmount,
    float memoryAmount, Set<String> requirements) {
    // Execuo Automtica
    if (command.isAutomatic()) {
      List<String> candidateServers = new ArrayList<String>();
      for (String serverName : servers) {
        service.logDebugMessage("Tentando sga: " + serverName);
        if (checkServer(serverName, platforms, cpuAmount, memoryAmount,
          requirements))
          candidateServers.add(serverName);
      }
      
      return schedPolicyInterface.chooseServer(command, candidateServers);
    }
    // Execuo Manual
    String serverName = command.getSGAName();
    service.logDebugMessage("Verificando sga: " + serverName);
    if (checkServer(serverName, platforms, cpuAmount, memoryAmount,
      requirements)) {
      return serverName;
    }
    return null;
  }

  /**
   * Escolhe a lista de servidores para processar o comando distribudo. Para
   * executar um comando distribudo  necessrio verificar a disponibilidade de
   * todos os servidores escolhidos quando a execuo  manual. Quando a
   * execuo  automtica,  necessrio alocar o nmero de servidores indicado.
   *
   * Retorna a lista de servidores aptos ou null quando no h disponibilidade.
   *
   * @param command Comando para execuo.
   * @param servers Lista ordenada de servidores.
   * @param platforms Lista das plataformas em que o algoritmo pode ser
   *        executado.
   * @param cpuAmount Unidade de processamento do algoritmo, ou seja, quanto de
   *        CPU o algoritmo precisa.
   * @param memoryAmount Quantidade de memria (em Mb) necessria para executar
   *        o comando.
   * @param requirements Requisitos necessrios para executar o comando.
   * @return Retorna a lista de servidores aptos ou null quando no h
   *         disponibilidade.
   */
  private List<String> chooseServersForDistributedCommand(CommandInfo command,
    List<String> servers, Vector<String> platforms, float cpuAmount,
    float memoryAmount, Set<String> requirements) {
    List<String> availableServers;

    // Temos que considerar o nmero de vezes que o comando ser disparado
    // por mquina.
    int numExec = command.getExecutionCountPerSGAForDistributedExecution();
    float cpuTotal = cpuAmount * numExec;
    float memoryTotal = memoryAmount * numExec;
    service.logDebugMessage("Nmero de execues por servidor: " + numExec);

    // Se a escolha for automtica, buscaremos numServers entre todos os
    // servidores disponveis. Se for manual, verificamos se os selecionados
    // esto disponveis.
    if (command.isAutomatic()) {
      List<String> selectedServers = new ArrayList<String>();
      List<String> candidateServers = new ArrayList<String>();
      availableServers = servers;
      int numServers = command.getSGACountForDistributedExecution();

      for (int i = 0; i < availableServers.size(); i++) {
        String serverName = availableServers.get(i);
        service.logDebugMessage("Verificando sga: " + serverName);
        if (checkServer(serverName, platforms, cpuTotal, memoryTotal,
          requirements)) {
          candidateServers.add(serverName);
        }
      }
      selectedServers =
        schedPolicyInterface.chooseServers(command, candidateServers, numServers);
      if(selectedServers == null)
        return null;
      command.setSelectedSGAsNames(selectedServers);
      service.logDebugMessage("Linha de comando trocada.");
      return selectedServers;
    }
    // Seleo Manual
    for (String serverName : command.getSelectedSGAsNames()) {
      service.logDebugMessage("Verificando sga: " + serverName);
      if (!checkServer(serverName, platforms, cpuTotal, memoryTotal,
        requirements)) {
        return null;
      }
    }
    service.logDebugMessage("Servidores verificados.");
    return command.getSelectedSGAsNames();
  }

  /**
   * Obtm o <code>SGASet</code> para o nome dado.
   *
   * @param serverName Nome do SGA.
   * @return O <code>SGASet</code>, ou null caso no haja SGASet para o nome
   *         dado.
   */
  private SGASet getSGASet(String serverName) {
    SGASet sgaSet = null;
    // O mtodo getSGASet j verifica se o usurio possui permisso para
    // executar comandos. Caso no possua permisso,
    // o SGASet fica desabilitado.
    sgaSet = sgaService.getSGASet(serverName);
    if (sgaSet == null) {
      service.logSevereMessage("Falha na obteno do sga " + serverName);
    }
    return sgaSet;
  }

  /**
   * Faz todas as verificaes necessrias para saber se o servidor pode
   * executar o comando.
   *
   * @param server Nome do servidor
   * @param platforms Lista das plataformas em que o comando pode ser executado
   * @param cpuAmount Unidade de processamento do algoritmo, ou seja, quanto de
   *        CPU o algoritmo precisa.
   * @param memoryAmount Quantidade de memria (em Mb) necessria para executar
   *        o comando.
   * @param requirements Requisitos necessrios para executar o comando.
   * @return true se o servidor estiver apto a executar o comando, ou false caso
   *         contrrio.
   */
  private boolean checkServer(String server, Vector<String> platforms,
    float cpuAmount, float memoryAmount, Set<String> requirements) {
    SGASet sgaSet = getSGASet(server);
    if (sgaSet == null) {
      return false;
    }
    // Poderamos retornar diretamente a chamada dos mtodos com o operador "&&"
    // Fizemos desta maneira para colocar logs.
    boolean inQuarantine = isInQuarantine(server);
    service.logDebugMessage("isInQuarantine: "
      + inQuarantine);
    if (inQuarantine) return false;

    boolean checkServerRestrictions = checkServerRestrictions(sgaSet, platforms);
    service.logDebugMessage("checkServerRestrictions: "
      + checkServerRestrictions);
    if (!checkServerRestrictions) return false;

    boolean checkServerResources = checkServerResources(sgaSet,
      cpuAmount, memoryAmount, requirements);
    service.logDebugMessage("checkServerResources: " + checkServerResources);
    if (!checkServerResources) return false;

    return true;
  }

  /**
   * Retira os SGAs da quarentena se seus tempos j expiraram.
   */
  private void clearQuarantine() {
    Iterator<Long> iterator = recentUsedServers.values().iterator();
    while (iterator.hasNext()) {
      Long quarantineExpirationTime = iterator.next();
      // Verifica se a quarentena j expirou. Nesse caso, retira
      // o SGA da lista e retorna true.
      long now = System.currentTimeMillis();
      if (now - quarantineExpirationTime > sgaQuarantine) {
        iterator.remove();
      }
    }
  }

  /**
   * Verifica se o servidor est na quarentena.
   *
   * @param sgaName Nome do servidor.
   * @return <code>true</code> se o servidor est na quarentena,
   *         <code>false</code> caso contrrio.
   */
  private boolean isInQuarantine(String sgaName) {
    if (!recentUsedServers.containsKey(sgaName)) {
      return false;
    }
    // Verifica se a quarentena j expirou. Nesse caso, retira
    // o SGA da lista e retorna true.
    long now = System.currentTimeMillis();
    if (now - recentUsedServers.get(sgaName) <= sgaQuarantine) {
      return true;
    }
    recentUsedServers.remove(sgaName);
    return false;
  }
  
  /**
   * Verifica se o servidor  cluster, se est disponvel, se o usurio possui
   * permisso e se o algoritmo possui binrios na plataforma do servidor. Usa o
   * mtodo
   * <code>checkSGARestrictions(SGASet sgaSet, Vector<String> platforms)</code>.
   *
   * @param serverName Nome do servidor.
   * @param platforms Lista das plataformas em que o algoritmo pode ser
   *        executado.
   * @return <code>true</code> se o servidor puder executar o algoritmo,
   *         <code>false</code> caso contrrio.
   */
  private boolean checkServerParallelRestrictions(String serverName,
                                                  Vector<String> platforms) {
    SGASet sgaSet = getSGASet(serverName);
    if ((sgaSet == null) || (!sgaSet.isCluster())) {
      return false;
    }

    return checkServerRestrictions(sgaSet, platforms);
  }

  /**
   * Verifica se o servidor est disponvel, se o usurio possui permisso e se
   * o algoritmo possui binrios na plataforma do servidor.
   *
   * @param sgaSet SGA
   * @param platforms Lista das plataformas em que o algoritmo pode ser
   *        executado.
   * @return <code>true</code> se o servidor puder executar o algoritmo,
   *         <code>false</code> caso contrrio.
   */
  private boolean checkServerRestrictions(SGASet sgaSet,
                                          Vector<String> platforms) {
    if (sgaSet.isPlatformSupported(platforms) && sgaSet.mayExecuteCommand()) {
      return true;
    }
    return false;
  }

  /**
   * Verifica se o server possui CPU e memria suficientes para a execuo.
   *
   * @param sgaSet SGA
   * @param cpuAmount Quantidade de CPU requerida
   * @param memoryAmount Quantidade de memria requerida.
   * @param requirements Requisitos necessrios para executar o comando.
   * @return <code>true</code> se houver recursos para executar o algoritmo,
   *         <code>false</code> caso contrrio.
   */
  private boolean checkServerResources(SGASet sgaSet, float cpuAmount,
    float memoryAmount, Set<String> requirements) {
    if (cpuAmount == 0 && memoryAmount == 0 && requirements.size() == 0)
      return true;

    double localNumProcessors = sgaSet.getNumProcessors()
      - (sgaSet.getCPULoad1() * sgaSet.getNumProcessors() / 100);
    service.logDebugMessage("CPU livre: " + localNumProcessors);
    service
      .logDebugMessage("Memria (Mb) livre: " + sgaSet.getRAMFreeMemoryMb());
    boolean hasAllRequirements = true;
    /* Mesmo quando um falhar, continua o lao para saber quais os
       requisitos que o n no possui. */
    for (String requirement : requirements) {
      boolean hasRequirement = sgaSet.hasRequirement(requirement);
      hasAllRequirements = hasAllRequirements && hasRequirement;
      service.logDebugMessage("Requisito " + requirement +  ": " +
        hasRequirement);
    }

    if (((cpuAmount <= 0) || (localNumProcessors >= cpuAmount)) &&
      (sgaSet.getRAMFreeMemoryMb() >= memoryAmount) && hasAllRequirements) {
      return true;
    }

    return false;
  }

  /**
   * Notificao de falha de comando para um  usurio e para os observadores
   * registrados no servio
   *
   * @param command comando.
   */
  private void notifyCommandFailed(CommandInfo command) {
    /* Envia notificao para os usurios */
    Object userId = null;
    try {
      userId = command.getUserId();
      final CommandFailedNotificationData notData =
        new CommandFailedNotificationData(command.getId(),
          command.getDescription(), command.getTip(),
          command.getSubmittedDate().getTime(), null, command.getProjectId());
      NotificationService.getInstance().notifyUserFromService(service,
        userId, notData);
    }
    catch (Exception e) {
      service.logSevereMessage(
        "Impossvel notificar o usurio ("+userId+") "+
        "sobre a falha do comando ("+command.getId()+")", e);
    }
    /* Envia evento para os observadores do servio */
    CommandEvent event = new CommandEvent(command.getId(), EventType.INIT_FAILURE);
    service.notifyObservers(event);
  }
}
