/*
 * SchedulerService.java
 * 
 * $Author: mjulia $
 * $Date: 2009-09-08 16:18:03 -0300 (Tue, 08 Sep 2009) $
 * $Revision: 95933 $
 */
package csbase.server.services.schedulerservice;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.List;
import java.util.Set;
import java.util.Vector;

import csbase.exception.InitFailureException;
import csbase.exception.OperationFailureException;
import csbase.exception.ServiceFailureException;
import csbase.logic.CommandEvent;
import csbase.logic.CommandFinalizationType;
import csbase.logic.CommandInfo;
import csbase.logic.CommandSubmission;
import csbase.logic.Priority;
import csbase.logic.SchedulerCommandRemovedEvent;
import csbase.logic.SchedulerStateChangedEvent;
import csbase.logic.algorithms.AlgorithmConfigurator;
import csbase.remote.RemoteObserver;
import csbase.remote.SchedulerServiceInterface;
import csbase.server.Server;
import csbase.server.ServerException;
import csbase.server.Service;
import csbase.server.services.commandpersistenceservice.CommandPersistenceService;
import csbase.server.services.sgaservice.SGAService;

/**
 * Servio escalonador de comandos.
 * 
 * @author Bruno Oliveira Silvestre
 *
 */
public class SchedulerService extends Service implements
  SchedulerServiceInterface {
  
  /** Fila que controla a prioridade dos processos */
  private PriorityQueue commandQueue;
  /** Thread do escalonador */
  private Scheduler scheduler;
  /** Hashtable para observadores de comandos - (por commandId ) */
  private Hashtable<Object, Vector<RemoteObserver>> commandObservers;

  /** Propriedade que define o tempo de quarentena do SGA */
  final static private String QUARANTINE_PROPERTY = "sga.quarantine.time";
  /** Propriedade que define o tempo de processamento da fila (milisegundos) */ 
  private static final String PROCESSING_INTERVAL_PROPERTY = "processing.interval";
  /** Propriedade que define a poltica de escalonamento a ser empregada */
  private static final String SCHED_POLICY_PROPERTY = "sga.schedule.policy";
  /** Nome do arquivo de backup */
  final static private String BACKUP_FILE_NAME = "queue-backup.dat"; 

  /**
   * Construtor da classe.
   * 
   * @throws ServerException se houver erro na inicializao.
   */
  protected SchedulerService() throws ServerException {
    super(SERVICE_NAME);
  }

  /**
   * {@inheritDoc}
   */
  public Set<CommandInfo> submitCommand(CommandSubmission submission) {
    return submitCommand(submission, (RemoteObserver[])null);
  }

  /**
   * Submete um comando para a execuo com a opo de passar uma lista de 
   * observadores de trmino do comando. Cada comando pode disparar um ou mais
   * jobs, dependendo do tipo do algoritmo.
   * @param submission a descrio do comando para execuao
   * @param observers zero ou mais observadores
   * @return o conjunto de identificadores dos jobs submetidos
   */
  public Set<CommandInfo> submitCommand(CommandSubmission submission, RemoteObserver... observers) {
    Set<CommandInfo> commands = new HashSet<CommandInfo>();
    switch (submission.getExecutionType()) {
      case SIMPLE:
      case XAPPS:
      case DISTRIBUTED:
      case PARALLEL:
        CommandInfo commandInfo = createCommand(submission, observers);
        if (submission.isManual()) {
          String sgaName = submission.getSGANames().get(0);
          commandInfo.setSGAName(sgaName);
        }
        commands.add(commandInfo);
        commandQueue.add(commandInfo);
        break;
      case MULTIPLE:
        if (submission.isManual()) {
          int executionCountPerSGA =
            submission.getExecutionCountPerSGAForMultipleExecution();
          for (String sgaName : submission.getSGANames()) {
            for (int i = 0; i < executionCountPerSGA; i++) {
              CommandInfo command = createCommand(submission, observers);
              command.setSGAName(sgaName);
              commands.add(command);
              commandQueue.add(command);
            }
          }
        }
        else {
          int executionCount = submission.getExecutionCountForMultipleExecution();
          for (int i = 0; i < executionCount; i++) {
            CommandInfo command = createCommand(submission, observers);
            commands.add(command);
            commandQueue.add(command);
          }
        }
        break;
      default:
        throw new IllegalArgumentException("Tipo de execuo invlida.");
    }
    return commands;
  }

  /**
   * Cria um comando.
   * <p>
   * Persiste o comando utilizando o
   * {@link CommandPersistenceService servio de persistncia de comandos}. O
   * servio de persistncia de comandos pode no estar disponvel em todos os
   * sistemas.
   * </p>
   * Registra os observadores do comando criado.
   * 
   * @param submission dados da submisso do comando.
   * @param observers um ou mais observadores do comando.
   * 
   * @return o comando.
   * @throws ServiceFailureException
   */
  private CommandInfo createCommand(CommandSubmission submission, RemoteObserver... observers) {
    Object userId = Service.getUser().getId();
    String submissionInfo = submission + "usurio: " + userId;
    try {
      CommandInfo command;
      logInfoMessage(submissionInfo);
      AlgorithmConfigurator configurator = submission.createAlgorithmConfigurator();
      String serverName = Server.getInstance().getServerName();
      command = new CommandInfo(configurator, submission, serverName, (String) userId);

      CommandPersistenceService service =
        CommandPersistenceService.getInstance();
      if (service != null) {
        service.saveCommandInfo(command);
      }
      if (observers != null) {
        for (RemoteObserver observer : observers) {
          addCommandObserver(observer, command.getId());
        }
      }
      return command;
    }
    catch (OperationFailureException e) {
      logSevereMessage(
        "Ocorreu um erro ao salvar o comando no servio de persistncia", e);
      throw new ServiceFailureException(
        "Falha na submisso de comando. Ocorreu um erro ao salvar o comando no servio de persistncia"
          + submissionInfo, e);
    }
    catch(Exception e) {
      e.printStackTrace();
      throw new ServiceFailureException(
        "Falha na submisso de comando. No foi possvel recuperar o configurador do comando "
          + submissionInfo, e);
    }
  }
  
  /**
   * {@inheritDoc}
   */
  public boolean setPosition(Object commandId, int position) {
    logDebugMessage("Alterao da posicao do comando " + commandId +
      " para " + position);
    if (position < 0 || position >= commandQueue.size()) {
      return false;
    }
    return commandQueue.setPosition(commandId, position);
  }

  /**
   * {@inheritDoc}
   */
  public boolean setPriority(Object commandId, Priority priority) {
    logDebugMessage("Alterao da prioridade do comando " + commandId +
      " para " + priority);
    return commandQueue.setPriority(commandId, priority);    
  }

  /**
   * {@inheritDoc}
   */
  public void updateCommandDescription(String commandId, String description) {
    for (CommandInfo scheduledCommand : commandQueue.getCommands()) {
      if (scheduledCommand.getId().equals(commandId)) {
        scheduledCommand.setDescription(description);
        return;
      }
    }
  }  
  
  /**
   * {@inheritDoc}
   */
  public boolean removeCommand(Object commandId) {
    logDebugMessage("Remoo do comando : " + commandId);

    try {
      // Remove o comando da fila
      CommandInfo cmd = commandQueue.remove(commandId);
      if (null == cmd) {
        return false;
      }
      // Atribui o status de morto
      cmd.setFinished(CommandFinalizationType.KILLED);
      CommandPersistenceService service =
        CommandPersistenceService.getInstance();
      if (service != null) {
        service.saveCommandInfo(cmd);
      }
      notifyObservers(new SchedulerCommandRemovedEvent(cmd));
      /**
       * Alm dos observadores, aqui deveria notificar os usurios do projeto, 
       * de que o comando foi terminado. <br>
       * A exemplo do SGAService#notifyCommandEnd(Command,CommandEvent)<br>
       * <code>
       * notifyCommandEndToProject(command, event);
       * mailCommandEndToProject(command, event); 
       * </code>
       */
      return true;
    }
    catch (OperationFailureException e) {
      logSevereMessage(
        "Ocorreu um erro ao salvar o comando com o status KILLED no servio de persistncia",
        e);
      throw new ServiceFailureException(
        "Falha na remoo de comando. Ocorreu um erro ao salvar o comando com o status KILLED no servio de persistncia",
        e);
    }  
  }

  /**
   * {@inheritDoc}
   */
  public void setBlocked(boolean blocked) {
    logDebugMessage("Bloqueio da fila chamado: " + blocked);
    commandQueue.setBlocked(blocked);
    setChanged();
    notifyObservers(new SchedulerStateChangedEvent(blocked));
  }
  
  /**
   * {@inheritDoc}
   */
  public boolean isBlocked() {
    return commandQueue.isBlocked();
  }
  
  /**
   * {@inheritDoc}
   */
  public CommandInfo[] getQueuedCommands()  {
    Object userId = Service.getUser().getId();
    boolean isAdmin = Service.getUser().isAdmin();
    CommandInfo cmds[] = commandQueue.getCommands();
    ArrayList<CommandInfo> commandsArray = 
      new ArrayList<CommandInfo>();
    for(int i = 0; i < cmds.length; i++) {
      if(isAdmin || cmds[i].getUserId().equals(userId)) {
        commandsArray.add(cmds[i]);
        cmds[i].setGlobalPosition(i+1);
      }
    }
    return commandsArray.toArray(new CommandInfo[0]);
  }

  /**
   * Constri a instncia do servio de escalonamento.
   *
   * @throws ServerException se houver erro na inicializao.
   */
  public static void createService() throws ServerException {
    new SchedulerService();
  }
  
  /**
   * Retorno da instncia do servio.
   * @return o servio.
   */
  final public static SchedulerService getInstance() {
    return (SchedulerService)getInstance(SERVICE_NAME);
  }

  /**
   * Inicializao do servio de escalonamento.
   * 
   * @throws ServerException caso ocorra um erro na inicializao
   */
  @Override
  final public void initService() throws ServerException {
    commandObservers = new Hashtable<Object, Vector<RemoteObserver>>();
    SGAService sgaService = SGAService.getInstance();
    long queueProcessingInterval;
    int sgaQuarantine;
    String schedPolicyName;
    

      /* Faz a leitura do intervalo de processamento da fila. Quando
       * no  definido no arquivo de propriedades, fixamos em 2 vezes
       * o intervalo de atualizao das informaes do SGA.
       */
      if (isPropertyNull(PROCESSING_INTERVAL_PROPERTY)) {
        queueProcessingInterval = sgaService.getUpdateInterval() * 1000 * 2;
      } else {
        queueProcessingInterval =  getIntProperty(PROCESSING_INTERVAL_PROPERTY);
      }
      logInfoMessage("Intervalo de processamento da fila de comandos: " +
        queueProcessingInterval + " milisegundo(s).");

      try {
      /*
       *  Obtm o tempo de quarentena que o SGA permanece depois que um comando
       *   enviado para ele.
       */
      sgaQuarantine = getIntProperty(QUARANTINE_PROPERTY);
      logInfoMessage("Intervalo de quarentena dos SGAs: " + 
        sgaQuarantine + " segundo(s).");
    } catch (Exception e) {
      throw new ServerException(new InitFailureException(
        "Falha na inicializao do tempo de quarentena do SGA.", e));
    }
    
    /*
     * Estabelece a poltica de escalonamento do SchedulerService.
     */
    try {
      schedPolicyName = getStringProperty(SCHED_POLICY_PROPERTY);
      logInfoMessage("Poltica de escalonamento a ser ativada: " + schedPolicyName + ".");
    }
    catch(Exception e) {
      throw new ServerException(new InitFailureException(
        "Falha na inicializao da poltica de escalonamento.", e));      
    }

    /*
     * Estabelece o caminho para o arquivo de backup.
     */
    String backupFilename;
    try {
      backupFilename = sgaService.getCommandsPersistencyDirectoryName() + 
        File.separator + BACKUP_FILE_NAME; 
    } 
    catch(OperationFailureException e) {
      throw new ServerException(new InitFailureException(
        "Falha na inicializao do arquivo para backup.", e));
    }
    logInfoMessage("Arquivo de backup: " + backupFilename);
    
    /*
     * Cria a fila e carrega o seu contedo do arquivo de backup.
     *  comum ocorrer erro na leitura antes que o primeiro backup
     * seja feito, por isso o log apenas como warning. 
     */
    commandQueue = new PriorityQueue(backupFilename);
    File backupFile = new File(backupFilename);
    if (backupFile.exists()) {
      commandQueue.loadFromBackup();
    } else {    
      logWarningMessage(
        "Arquivo de backup no encontrado." + 
        "Fila no recuperada na inicializao do servidor.");      
    }
    scheduler = new Scheduler(
      commandQueue, queueProcessingInterval,sgaQuarantine, schedPolicyName);
    scheduler.startScheduler();
  }

  /**
   * Mtodo invocado no trmino do servio.
   */
  @Override
  public void shutdownService() {
    scheduler.stopScheduler();
    logInfoMessage("Servio encerrado.");    
  }

  /**
   * Mtodo para indicar se o observador deve ser notificado.
   *
   * @param arg Argumento do observador.
   * @param event Evento gerado para notificar o observador.
   *
   * @return <code>true</code> quando o observador deve ser notificado.
   */
  @Override
  protected boolean has2Update(final Object arg, final Object event) {
    return true;
  }
  
  /**
   * Registra um observador interessado nas mudanas que ocorrerem em um comando
   * em execuo. O observador  removido quando:
   * <ul>
   * <li> ocorrer uma falha na inicializao do comando: envia o evento e remove
   * o observador.
   * <li> no momento que repassa o comando para o SGAService: registra o observador
   * no SGAService e remove.
   * </ul>
   * @param observer O observador remoto que registra este interesse.
   * @param arg identificador do comando no qual o observador est interessado.
   */
  final private void addCommandObserver(final RemoteObserver observer, final Object arg) {
    if ((observer == null) || (arg == null)) {
      logSevereMessage("Pedido de registro de observador de comando invalido");
      return;
    }
    String cmdId = (String) arg;
    synchronized (commandObservers) {
      Vector<RemoteObserver> observers = commandObservers.get(cmdId);
      if (observers == null) {
        observers = new Vector<RemoteObserver>();
        commandObservers.put(cmdId, observers);
      }
      observers.add(observer);
    }
  }
  
  /**
   * Notifica os observadores.
   * 
   * @param event evento a ser enviado para os observadores.
   */
  protected void notifyObservers(final CommandEvent event) {
    synchronized (commandObservers) {
      Object cmdId = event.getCommandId();
      final Vector<RemoteObserver> observers = commandObservers.get(cmdId);
      if (observers == null) {
        return;
      }
      final Object[] obs = observers.toArray();
      new Thread(new Runnable() {
        public void run() {
          logInfoMessage("Thread iniciada: notifyObservers");
          for (int i = 0; i < obs.length; i++) {
            try {
              final RemoteObserver observer = (RemoteObserver) obs[i];
              observer.update(event);
            }
            catch (Exception e) {
              logWarningMessage("Erro na notificao de observador: "
                + e.getMessage());
            }
          }
          logInfoMessage("Thread terminada: notifyObservers");
        }
      }).start();
      commandObservers.remove(cmdId);
    }
  }
  
  /**
   * Remove os observadores de um determinado comando.
   * 
   * @param cmdId o identificador do comando
   */
  void removeCommandObservers(Object cmdId) {
    commandObservers.remove(cmdId);
  }
  
  /**
   * Obtm os observadores de um determinado comando.
   * 
   * @param cmdId o identificador do comando
   * @return a coleo com os observadores 
   */
  Collection<RemoteObserver> getCommandObservers(Object cmdId) {
    List<RemoteObserver> observers = commandObservers.get(cmdId);
    if (observers==null) return null;
    return Collections.unmodifiableCollection(observers);
  }
}
