/*
 * $Id: WIOServerFileSystem.java 88773 2009-02-28 01:06:06Z clinio $
 */
package csbase.server.services.wioservice;

import java.io.File;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Vector;

import org.omg.PortableServer.POA;

import csbase.logic.User;
import csbase.logic.UserProjectInfo;
import csbase.server.services.loginservice.LoginService;
import csbase.server.services.projectservice.ProjectService;
import csbase.server.services.wioservice.idl.AlgorithmInfo;
import csbase.server.services.wioservice.idl.UserProject;
import csbase.server.services.wioservice.idl.WIOFileSystemPOA;
import csbase.server.services.wioservice.idl.WIOProject;
import csbase.server.services.wioservice.idl.WIOProjectHelper;
import csbase.server.services.wioservice.idl.WIOServiceException;

/**
 * A classe <code>WIOServerFileSystem</code> implementa o <i>servant</i>
 * associado  interface idl <code>WIOFileSystem</code>. As requisies
 * recebidas dos cliente so repassadas para  implementao do servio.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class WIOServerFileSystem extends WIOFileSystemPOA {
  /**
   * Separador de arquivos no servidor
   */
  protected static String FILE_SEPARATOR = "|";

  /**
   * Referncia para a implementao do servio SGAService
   */
  private WIOService service = null;

  /**
   * Nmero de projetos abertos neste file system.
   */
  private int numProjects = 0;

  /**
   * Nmero total de arquivos abertos neste file system.
   */
  private int numTotalFiles = 0;

  /**
   * Checagem de senha do usurio
   * 
   * @param userId identificador do usurio.
   * @param passwd senha.
   * 
   * @return .
   */
  public boolean checkPassword(final String userId, final String passwd) {
    return isCorrectPassword(userId, passwd);
  }

  /**
   * Check de instrumentao da existncia do servio.
   * 
   * @throws Exception em caso de erro de acesso.
   */
  private void checkService() throws Exception {
    if (service == null) {
      final String err = "Servico de WIO com referencia nula!";
      throw new Exception(err);
    }
  }

  /**
   * Incrementa o nmero de projetos abertos de WIOServerFile.
   * 
   * @throws Exception em caso de erro de acesso.
   */
  protected void decNumFiles() throws Exception {
    checkService();
    numTotalFiles--;
  }

  /**
   * Decrementa o nmero de projetos abertos
   * 
   * @throws Exception se houver underflow de limite permitido.
   */
  protected void decNumProjects() throws Exception {
    checkService();
    final int MAX_NUM_PROJS = service.getMaxOpenedProjects();
    if (numProjects <= 0) {
      final String err = "Underflow em numero de projetos.";
      throw new Exception(err);
    }
    numProjects--;
    service.logInfoMessage("Decremento do numero de projetos abertos: "
      + numProjects + "/" + MAX_NUM_PROJS);
  }

  /**
   * Consulta ao mximo de projetos abertos por servidor.
   * 
   * @return o nmero mximo de projetos.
   * 
   * @throws WIOServiceException em caso de erro de acesso.
   */
  public int getMaxOpenedProjects() throws WIOServiceException {
    try {
      checkService();
      return service.getMaxOpenedProjects();
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      service.logSevereMessage(err);
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * Consulta a tamanho mximo (Kb) para leitura.
   * 
   * @return o tamanho mximo.
   * 
   * @throws WIOServiceException em caso de erro de acesso.
   */
  public int getMaxReadKb() throws WIOServiceException {
    try {
      checkService();
      return service.getMaxReadKb();
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      service.logSevereMessage(err);
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * Consulta a tamanho mximo (Kb) para escrita.
   * 
   * @return o tamanho mximo.
   * 
   * @throws WIOServiceException em caso de erro de acesso.
   */
  public int getMaxWriteKb() throws WIOServiceException {
    try {
      checkService();
      return service.getMaxWriteKb();
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      service.logSevereMessage(err);
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * Consulta ao nmero de projetos abertos no servidor.
   * 
   * @return o nmero de projetos abertos.
   * 
   * @throws WIOServiceException em caso de erro de acesso.
   */
  public int getNumOpenedFiles() throws WIOServiceException {
    try {
      checkService();
      return numTotalFiles;
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      service.logSevereMessage(err);
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * Consulta ao nmero de projetos abertos no servidor.
   * 
   * @return o nmero de projetos abertos.
   * 
   * @throws WIOServiceException em caso de erro de acesso.
   */
  public int getNumOpenedProjects() throws WIOServiceException {
    try {
      checkService();
      return numProjects;
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      service.logSevereMessage(err);
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * Consulta aos projetos prprios do usurio.
   * 
   * @param userId identificador do usurio.
   * @param passwd senha.
   * 
   * @return um array com os nomes dos projetos do usurio.
   * 
   * @throws WIOServiceException em caso de erro de acesso.
   */
  public String[] getProjects(final String userId, final String passwd)
    throws WIOServiceException {
    if (!isCorrectPassword(userId, passwd)) {
      String msg = "Falha de autenticacao na busca por projetos. ";
      msg = msg + "Usuario: " + userId;
      service.logSevereMessage(msg);
      throw new WIOServiceException(msg);
    }

    final ProjectService prjService;
    final String[] names;
    try {
      prjService = getProjectService();
      names = prjService.getProjectNames(userId);
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      final String msg = "Falha na busca de projetos proprios";
      service.logSevereMessage(msg + " : " + err);
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }

    if (names == null || names.length <= 0) {
      final String msg = "Nao ha projetos disponiveis";
      throw new WIOServiceException(msg);
    }

    Arrays.sort(names, new Comparator<String>() {
      public int compare(final String n1, final String n2) {
        return n1.compareToIgnoreCase(n2);
      }
    });
    return names;
  }

  /**
   * Busca do servio de projetos.
   * 
   * @return o servio de projetos.
   * 
   * @throws Exception em caso de erro de acesso.
   */
  private ProjectService getProjectService() throws Exception {
    return ProjectService.getInstance();
  }

  /**
   * Consulta aos projetos prprios do usurio.
   * 
   * @param userId identificador do usurio.
   * @param passwd senha.
   * 
   * @return um array com os nomes dos projetos do usurio.
   * 
   * @throws WIOServiceException em caso de erro de acesso.
   */
  public UserProject[] getProjectsFromOthers(final String userId,
    final String passwd) throws WIOServiceException {
    if (!isCorrectPassword(userId, passwd)) {
      String msg = "Falha de autenticacao na busca por projetos compartilhados";
      msg = msg + "\nUsuario: " + userId;
      service.logSevereMessage(msg);
      throw new WIOServiceException(msg);
    }
    try {
      final ProjectService prjService = getProjectService();
      final Vector<UserProjectInfo> userProjects =
        prjService.getAllUserProjectsFromOthers(userId);
      if (userProjects == null || userProjects.size() <= 0) {
        final String err = "Nao ha projetos compartilhados disponiveis!";
        throw new Exception(err);
      }
      final UserProject[] ups = new UserProject[userProjects.size()];
      for (int i = 0; i < userProjects.size(); i++) {
        final UserProjectInfo uInfo = userProjects.get(i);
        ups[i] = new UserProject();
        ups[i].ownerId = (String) uInfo.getUserId();
        ups[i].ownerName = User.getName(uInfo.getUserId());
        ups[i].projectId = uInfo.getProjectName();
      }
      Arrays.sort(ups, new Comparator<UserProject>() {
        public int compare(final UserProject o1, final UserProject o2) {
          final String n1 = o1.projectId;
          final String n2 = o2.projectId;
          return n1.compareToIgnoreCase(n2);
        }
      });
      return ups;
    }
    catch (final Exception e) {
      String err = "Falha na busca de projetos compartilhados";
      err = err + "\n" + e.getMessage();
      service.logSevereMessage(err);
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * Consuta ao separador de arquivos.
   * 
   * @return a string (caracter) de separao.
   */
  public String getSeparatorChar() {
    return FILE_SEPARATOR;
  }

  /**
   * Consulta a identificao da verso utilizada pelo servidor.
   * 
   * @return a identificao da verso
   */
  public String getServerVersion() {
    return VERSION;
  }

  /**
   * Incrementa o nmero de projetos abertos de WIOServerFile.
   * 
   * @throws Exception em caso de erro de acesso.
   */
  protected void incNumFiles() throws Exception {
    checkService();
    numTotalFiles++;
  }

  /**
   * Incrementa o nmero de projetos abertos de WIOServerFile.
   * 
   * @throws Exception se houver estouro de limite permitido.
   */
  protected void incNumProjects() throws Exception {
    checkService();
    final int MAX_NUM_PROJS = service.getMaxOpenedProjects();
    if (numProjects >= MAX_NUM_PROJS) {
      final String err = "Overflow em numero de projetos.";
      throw new Exception(err);
    }
    numProjects++;
    service.logInfoMessage("Incremento do numero de projetos abertos: "
      + numProjects + "/" + MAX_NUM_PROJS);
  }

  /**
   * Checagem de senha do usurio
   * 
   * @param userId identificador do usurio.
   * @param passwd senha.
   * 
   * @return um flag indicativo da corretude da senha.
   */
  private boolean isCorrectPassword(final String userId, final String passwd) {
    try {
      final LoginService loginService = LoginService.getInstance();
      final User user = loginService.checkLogin(userId, passwd);
      if (user == null) {
        return false;
      }
      return true;
    }
    catch (final Exception e) {
      service.logSevereMessage("Falha na autenticao do WIOService!" + " --> "
        + e.getMessage());
      return false;
    }
  }

  /**
   * Abertura real do projeto.
   * 
   * @param userId identificador do usuario que quer
   * @param ownerId o identificador do dono do arquivo.
   * @param projectId o identificador do projeto.
   * 
   * @return a referncia ao projeto.
   * 
   * @throws Exception em caso de erro de acesso.
   */
  private WIOProject openPrj(final String userId, final String ownerId,
    final String projectId) throws Exception {
    final WIOServerProject prj;
    try {
      prj = new WIOServerProject(this, userId, ownerId, projectId);
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      service.logSevereMessage("Excecao detectada: " + err);
      e.printStackTrace();
      throw new Exception(err == null ? "<null message>" : err);
    }

    service.logInfoMessage("Instanciado projeto [" + projectId + "]...");
    final POA poa = prj.activateCorbaProject();
    if (poa == null) {
      final String err = "Falha detectada na ativ. de projeto";
      throw new Exception(err);
    }
    final WIOProject cprj =
      WIOProjectHelper.narrow(poa.servant_to_reference(prj));
    if (cprj == null) {
      final String err = "Falha de narrow ao criar CORBA do projeto";
      service.logSevereMessage(err + ":" + projectId);
      throw new Exception(err);
    }
    if (cprj._non_existent()) {
      final String err = "Falha de non-existent CORBA em projeto";
      service.logSevereMessage(err + ":" + projectId);
      throw new Exception(err);
    }
    return cprj;
  }

  /**
   * Abertura de projeto prprio.
   * 
   * @param userId identificador do usurio.
   * @param pwd senha.
   * @param projectId identificador do projeto.
   * 
   * @return .
   * 
   * @throws WIOServiceException em caso de erro de acesso.
   */
  public WIOProject openProject(final String userId, final String pwd,
    final String projectId) throws WIOServiceException {
    service.logInfoMessage("Pedido de abertura do projeto [" + projectId
      + "]...");
    if (!isCorrectPassword(userId, pwd)) {
      String msg = "Falha de autenticacao na abertura do projeto ";
      msg = msg + "\nProjeto: " + userId;
      msg = msg + "\nUsuario: " + userId;
      service.logSevereMessage(msg);
      throw new WIOServiceException(msg);
    }
    try {
      final WIOProject prj = openPrj(userId, userId, projectId);
      service.logInfoMessage("Sucesso na abertura de [" + projectId + "]");
      return prj;
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      service.logSevereMessage(err);
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * Abertura de projeto compartilhado.
   * 
   * @param userId identificador do usurio.
   * @param passwd senha.
   * @param ownerId .
   * @param projectId identificador do projeto.
   * 
   * @return .
   * 
   * @throws WIOServiceException em caso de erro de acesso.
   */
  public WIOProject openProjectFromOthers(final String userId,
    final String passwd, final String ownerId, final String projectId)
    throws WIOServiceException {
    service.logInfoMessage("Pedido de abertura " + "do projeto compartilhado ["
      + projectId + "]...");
    if (!isCorrectPassword(userId, passwd)) {
      String msg = "Falha de autenticacao na abertura do projeto ";
      msg = msg + "\nProjeto: " + userId;
      msg = msg + "\nUsuario: " + userId;
      msg = msg + "\nDono: " + ownerId;
      service.logSevereMessage(msg);
      throw new WIOServiceException(msg);
    }
    try {
      final WIOProject prj = openPrj(userId, ownerId, projectId);
      service.logInfoMessage("Sucesso na abertura de [" + projectId
        + "] (shared)");
      return prj;
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      service.logSevereMessage(err);
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * <p>
   * Executa o pr-login de um usurio no servidor CSBase e retorna uma URL para
   * acesso ao sistema com esse usurio.
   * </p>
   * <p>
   * Quando executar o sistema com essa URL de acesso, o projeto informado
   * (pertencente ao usurio) ser aberto e o arquivo informado ser executado.
   * </p>
   * 
   * @param login O login do usurio.
   * @param password A senha do usurio.
   * @param projectName .
   * @param algorithmInfo O nome de um projeto a ser aberto ao se usar a URL de
   *        acesso.
   * 
   * @return Uma URL para acesso ao sistema.
   */
  public String preLogin2Web(final String login, final String password,
    final String projectName, final AlgorithmInfo algorithmInfo) {
    return this.service.preLogin2Web(login, password, projectName,
      algorithmInfo);
  }

  /**
   * Constri um servant.
   * 
   * @param srv referncia para a implementao do servio.
   */
  protected WIOServerFileSystem(final WIOService srv) {
    FILE_SEPARATOR = File.separator;
    if (FILE_SEPARATOR.equals("\\")) {
      FILE_SEPARATOR = "\\\\";
    }
    service = srv;
  }
}
