/*
 * $Id: WIOServerProject.java 88773 2009-02-28 01:06:06Z clinio $
 */
package csbase.server.services.wioservice;

import org.omg.PortableServer.POA;

import tecgraf.javautils.FileUtils;
import csbase.server.services.projectservice.ProjectFileInfo;
import csbase.server.services.projectservice.ProjectService;
import csbase.server.services.projectservice.ServerProject;
import csbase.server.services.wioservice.idl.FileInfo;
import csbase.server.services.wioservice.idl.WIOFile;
import csbase.server.services.wioservice.idl.WIOFileHelper;
import csbase.server.services.wioservice.idl.WIOProjectPOA;
import csbase.server.services.wioservice.idl.WIOServiceException;

/**
 * A classe <code>WIOServerProject</code> implementa o <i>servant</i> associado
 *  interface idl <code>WIOProject</code>.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class WIOServerProject extends WIOProjectPOA {
  /**
   * Identificao
   */
  private String id = null;

  /**
   * Projeto no servidor
   */
  private ServerProject serverProject = null;

  /**
   * Num de arquivos abertos no projeto.
   */
  private int numFiles = 0;

  /**
   * File system associado.
   */
  private WIOServerFileSystem fileSystem = null;

  /**
   * O login do usurio autenticado.
   */
  private final String loggedUserId;

  /**
   * Mtodo de ativao do objeto CORBA WIOProject.
   * 
   * @return o POA associado ao objeto ativado.
   * 
   * @throws Exception em caso de erro.
   */
  protected POA activateCorbaProject() throws Exception {
    final WIOService wioService = getWIOService();
    return wioService.activateCorbaObject(null, this);
  }

  /**
   * Acrescenta uma descrio ao arquivo.
   * 
   * @param path o path para o arquivo.
   * @param description a string com a descrio.
   * 
   * @throws Exception em caso de erro.
   */
  protected void appendDescription(final String path, final String description)
    throws Exception {
    final String[] srvPath = mountServerPath(path);
    checkExists(srvPath);
    final String text = (description == null ? "<null>" : description);
    serverProject.appendFileDescription(srvPath, text);
  }

  /**
   * Verificao de no existncia de path no servidor.
   * 
   * @param srvPath o path
   * @throws Exception em caso de erro.
   */
  private void checkExists(final String[] srvPath) throws Exception {
    if (srvPath == null) {
      final String err = "Path array nulo em projeto " + this.toString();
      throw new Exception(err);
    }
    if (!serverProject.exists(srvPath)) {
      final String path = FileUtils.joinPath(srvPath);
      final String err =
        "[" + path + "] inexistente no projeto " + this.toString();
      throw new Exception(err);
    }
  }

  /**
   * Verificao de no existncia de path no servidor.
   * 
   * @param srvPath o path
   * 
   * @throws Exception em caso de erro.
   */
  private void checkNotExists(final String[] srvPath) throws Exception {
    if (srvPath == null) {
      final String err = "Path array nulo em projeto " + this.toString();
      throw new Exception(err);
    }
    if (serverProject.exists(srvPath)) {
      final String path = FileUtils.joinPath(srvPath);
      final String err =
        "[" + path + "] j existente no projeto " + this.toString();
      throw new Exception(err);
    }
  }

  /**
   * Checagem do <code>serverProject</code> associado.
   * 
   * @throws Exception em caso de erro.
   */
  private void checkServerProject() throws Exception {
    if (serverProject == null) {
      final String err = "Projeto nulo detectado.";
      throw new Exception(err);
    }
  }

  /**
   * Cpia de arquivo.
   * 
   * @param fromPath o caminho lgico origem do arquivo.
   * @param toPath o caminho lgico destino do arquivo.
   * 
   * @throws WIOServiceException em caso de erro.
   */
  public void copyFile(final String fromPath, final String toPath)
    throws WIOServiceException {
    try {
      final ProjectService prjService = getProjectService();
      final java.lang.Object myId = getId();
      final String[] fromSrvPath = mountServerPath(fromPath);
      final String[] toSrvPath = mountServerPath(toPath);
      checkExists(fromSrvPath);
      prjService.copyFile(myId, fromSrvPath, toSrvPath);
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * Criao de um diretrio no projeto.
   * 
   * @param path o caminho lgico no projeto.
   * 
   * @return um objeto <code>WIOFile</code> associado ao novo diretrio.
   * 
   * @throws WIOServiceException em caso de erro.
   */
  public WIOFile createDirectory(final String path) throws WIOServiceException {
    try {
      final String[] srvPath = mountServerPath(path);
      checkNotExists(srvPath);
      serverProject.createDirectory(srvPath);
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
    final WIOFile file = this.getFile(path);
    if (file == null) {
      final String err = "Diretorio [" + path + "] nao apareceu no projeto ";
      throw new WIOServiceException(err);
    }
    if (!file.isDirectory()) {
      final String err =
        "[" + path + "] nao apareceu no projeto como diretorio";
      throw new WIOServiceException(err);
    }
    return file;
  }

  /**
   * Criao de um arquivo no servidor.
   * 
   * @param path o caminho lgico do arquivo.
   * @param type o tipo CSBASE associado.
   * @param userId Identificador do usurio que cria o arquivo.
   * 
   * @return um objeto <code>WIOFile</code> associado.
   * 
   * @throws WIOServiceException em caso de erro.
   */
  public WIOFile createFile(final String path, final String type,
    final String userId) throws WIOServiceException {
    try {
      final String[] srvPath = mountServerPath(path);
      if (!serverProject.exists(srvPath)) {
        serverProject.createFile(srvPath, type, userId, false);
      }
      return this.getFile(path);
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * Mtodo de desativao do objeto CORBA WIOProject.
   * 
   * @throws Exception em caso de erro.
   */
  protected void deactivateCorbaProject() throws Exception {
    final WIOService wioService = getWIOService();
    wioService.deactivateCorbaObject(this);
  }

  /**
   * Decrementa o nmero de arquivos abertos no projeto via mtodo de
   * <code>destroy</code> de WIOServerFile.
   * 
   * @throws Exception se houver underflow de limite permitido.
   */
  protected void decNumFiles() throws Exception {
    if (numFiles <= 0) {
      final String err = "Underflow em numero de arquivos abertos no projeto.";
      logSevere(err);
      throw new Exception(err);
    }
    numFiles--;
    fileSystem.decNumFiles();
  }

  /**
   * Deleo de arquivo no file system do projeto.
   * 
   * @param path o caminho lgico do arquivo.
   * 
   * @throws WIOServiceException em caso de erro.
   */
  public void deleteFile(final String path) throws WIOServiceException {
    try {
      final String[] srvPath = mountServerPath(path);
      checkExists(srvPath);
      serverProject.removeFile(srvPath);
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * Destruio do objeto
   * 
   * @throws WIOServiceException em caso de erro
   */
  public void destroy() throws WIOServiceException {
    try {
      final String uid = getUser();
      final String pid = getId();
      logInfo("Finalizao do projeto " + pid + " (usurio: " + uid + ")");
      if (serverProject != null) {
        serverProject.close(true);
        serverProject = null;
      }
      deactivateCorbaProject();
      fileSystem.decNumProjects();
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }

  }

  /**
   * Consulta de existncia de um arquivo.
   * 
   * @param path o caminho lgico do arquivo.
   * 
   * @return um flag indicativo booleano de existncia.
   * 
   * @throws WIOServiceException em caso de erro.
   */
  public boolean fileExists(final String path) throws WIOServiceException {
    try {
      final String[] srvPath = mountServerPath(path);
      return serverProject.exists(srvPath);
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * Consulta a descrio do arquivo.
   * 
   * @param path o path para o arquivo.
   * 
   * @return a string com a descrio.
   * 
   * @throws Exception em caso de erro.
   */
  protected String getDescription(final String path) throws Exception {
    final String[] srvPath = mountServerPath(path);
    checkExists(srvPath);
    String desc = serverProject.getFileDescription(srvPath);
    desc = (desc == null || desc.trim().equals("") ? "" : desc);
    return desc;
  }

  /**
   * Busca de um arquivo no projeto.
   * 
   * @param path o caminho lgico do arquivo.
   * 
   * @return um objeto <code>WIOFILE</code> associado.
   * 
   * @throws WIOServiceException em caso de erro.
   */
  public WIOFile getFile(final String path) throws WIOServiceException {
    try {
      checkServerProject();
      final WIOServerFile file = new WIOServerFile(this, path);
      final POA poa = file.activateCorbaFile();
      final WIOFile corbaFile =
        WIOFileHelper.narrow(poa.servant_to_reference(file));
      if (corbaFile == null) {
        final String err = "Arquivo nulo detectado (path: " + path + ")";
        throw new Exception(err);
      }
      return corbaFile;
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * No implementado.
   * 
   * @param path o caminho lgico ao arquivo..
   * @param recur flag indicativo de busca recursiva.
   * 
   * @return um array de informaes de arquivos.
   * 
   * @throws WIOServiceException em caso de erro.
   */
  public FileInfo[] getFilesInfo(final String path, final boolean recur)
    throws WIOServiceException {
    try {
      final ProjectService prjService = getProjectService();
      checkServerProject();
      final String uid = getUser();
      final String pid = getId();
      final ProjectFileInfo[] paths =
        prjService.getFilesInfo(uid, pid, path, recur);
      if (paths == null) {
        final String err =
          "Falha nos paths em getFilesInfo() " + this.toString();
        throw new Exception(err);
      }
      final FileInfo[] infos = new FileInfo[paths.length];
      for (int i = 0; i < paths.length; i++) {
        final FileInfo info = infos[i];
        info.fileType = paths[i].fileType;
        info.sizeBytes = paths[i].sizeBytes;
        info.path = paths[i].path;
        info.isDirectory = paths[i].isDirectory;
        info.underConstruction = paths[i].underConstruction;
      }
      return infos;
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }

  }

  /**
   * Retorno do identificador do projeto (no escopo do usurio associado).
   * 
   * @return uma string de identificao.
   * 
   * @throws WIOServiceException em caso de erro.
   */
  public String getId() throws WIOServiceException {
    try {
      checkServerProject();
      return id;
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * Obtm o usurio autenticado.
   * 
   * @return O usurio autenticado.
   */
  String getLoggedUserId() {
    return this.loggedUserId;
  }

  /**
   * Retorna o nmero mximo de arquivos abertos no projeto.
   * 
   * @return o nmero mximo.
   * 
   * @throws WIOServiceException em caso de erro.
   */
  public int getMaxOpenedFiles() throws WIOServiceException {
    try {
      final WIOService wioService = getWIOService();
      final int MAX_NUM_FILES = wioService.getMaxOpenedFiles();
      return MAX_NUM_FILES;
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }

  }

  /**
   * Retorna o nmero de arquivos abertos no projeto.
   * 
   * @return o nmero.
   * 
   * @throws WIOServiceException em caso de erro.
   */
  public int getNumOpenedFiles() throws WIOServiceException {
    try {
      if (numFiles < 0) {
        final String err = "Valor negativo detectado.";
        throw new Exception(err);
      }
      return numFiles;
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }

  }

  /**
   * Mtodo de busca do servio de projetos
   * 
   * @return o servio de projetos no servidor.
   * 
   * @throws Exception em caso de erro.
   */
  protected ProjectService getProjectService() throws Exception {
    return ProjectService.getInstance();
  }

  /**
   * Busca do diretrio raiz do projeto.
   * 
   * @return um objeto <code>WIOFile</code> associado  raiz do projeto.
   * 
   * @throws WIOServiceException em caso de erro.
   */
  public WIOFile getRootFile() throws WIOServiceException {
    WIOFile corbaFile = null;
    try {
      checkServerProject();
      final WIOServerFile root = new WIOServerFile(this, ".");
      final POA poa = root.activateCorbaFile();
      corbaFile = WIOFileHelper.narrow(poa.servant_to_reference(root));
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }

    if (corbaFile == null) {
      final String err = "Falha de diretorio nulo raiz";
      throw new WIOServiceException(err);
    }
    return corbaFile;
  }

  /**
   * Consulta ao login (id) do usurio do projeto.
   * 
   * @return uma string de identificao.
   * 
   * @throws WIOServiceException em caso de erro.
   */
  public String getUser() throws WIOServiceException {
    try {
      checkServerProject();
      return (String) serverProject.getUserId();
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * Mtodo de busca do servio de projetos
   * 
   * @return o servio de projetos no servidor.
   * 
   * @throws Exception em caso de erro.
   */
  protected WIOService getWIOService() throws Exception {
    return WIOService.getInstance();
  }

  /**
   * Incrementa o nmero de arquivos abertos no projeto via construtor de
   * WIOServerFile.
   * 
   * @throws Exception se houver estouro de limite permitido.
   */
  protected void incNumFiles() throws Exception {
    final WIOService wioService = getWIOService();
    final int MAX_NUM_FILES = wioService.getMaxOpenedFiles();
    if (numFiles >= MAX_NUM_FILES) {
      final String err = "Overflow em numero de arquivos abertos no projeto.";
      logSevere(err);
      throw new Exception(err);
    }
    numFiles++;
    fileSystem.incNumFiles();
  }

  /**
   * Mtodo de logging de auditagem no servio de WIO.
   * 
   * @param msg a mensagem
   */
  private void logInfo(final String msg) {
    try {
      final WIOService wioService = getWIOService();
      if (wioService != null) {
        wioService.logInfoMessage(msg);
      }
      else {
        System.out.println("WIOPROJECT: " + msg);
      }
    }
    catch (final Exception e) {
      System.out.println("FALHA DE LOG WIOPROJECT: " + e.getMessage());
      System.out.println(msg);
      e.printStackTrace();
    }
  }

  /**
   * Mtodo de logging no servio de WIO.
   * 
   * @param msg a mensagem
   */
  private void logSevere(final String msg) {
    try {
      final WIOService wioService = getWIOService();
      if (wioService != null) {
        wioService.logSevereMessage(msg);
      }
      else {
        System.out.println("WIOPROJECT: " + msg);
      }
    }
    catch (final Exception e) {
      System.out.println("FALHA DE LOG WIOPROJECT: " + e.getMessage());
      System.out.println(msg);
      e.printStackTrace();
    }
  }

  /**
   * Montagem de um path para uso no servio de projetos.
   * 
   * @param path o caminho lgico no projeto.
   * @return um array de strings.
   * @throws Exception em caso de erro.
   */
  private String[] mountServerPath(final String path) throws Exception {
    if (path == null) {
      final String err =
        "Path nulo para mountServerPath() : " + this.toString();
      throw new Exception(err);
    }
    final String[] result = FileUtils.splitPath(path);
    if (result == null) {
      final String err =
        "Falha de split em mountServerPath() : " + this.toString();
      throw new Exception(err);
    }
    return result;
  }

  /**
   * Transferncia de arquivo no file system do projeto.
   * 
   * @param fromPath o caminho lgico origem do arquivo.
   * @param toPath o caminho lgico destino do arquivo.
   * 
   * @throws WIOServiceException em caso de erro.
   */
  public void moveFile(final String fromPath, final String toPath)
    throws WIOServiceException {
    try {
      final ProjectService prjService = getProjectService();
      final java.lang.Object myId = getId();
      final String[] fromSrvPath = mountServerPath(fromPath);
      final String[] toSrvPath = mountServerPath(toPath);
      checkExists(fromSrvPath);
      prjService.moveFile(myId, fromSrvPath, toSrvPath);
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * Renomeao de arquivo.
   * 
   * @param path o caminho lgico do arquivo.
   * @param newName o novo nome do arquivo.
   * 
   * @throws WIOServiceException em caso de erro.
   */
  public void renameFile(final String path, final String newName)
    throws WIOServiceException {
    try {
      final String[] srvPath = mountServerPath(path);
      checkExists(srvPath);
      serverProject.renameFile(srvPath, newName);
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * Ajusta a descrio do arquivo.
   * 
   * @param path o path para o arquivo.
   * @param description a string com a descrio.
   * 
   * @throws Exception em caso de erro.
   */
  protected void setDescription(final String path, final String description)
    throws Exception {
    final String[] srvPath = mountServerPath(path);
    checkExists(srvPath);
    final String text = (description == null ? "<null>" : description);
    serverProject.setFileDescription(srvPath, text);
  }

  /**
   * Ajuste do flag de cosntruo no arquivo servidor.
   * 
   * @param path caminho lgico do arquivo.
   * @param flag indicativo booleano de construo.
   * 
   * @throws Exception em caso de erro.
   */
  protected void setUnderConstruction(final String path, final boolean flag)
    throws Exception {
    final String[] srvPath = mountServerPath(path);
    checkExists(srvPath);
    serverProject.setUnderConstruction(srvPath, flag);
  }

  /**
   * Mtodo de dump como texto.
   * 
   * @return uma string na forma usurio/projeto.
   */
  @Override
  public String toString() {
    String uid = null;
    String pid = null;
    try {
      uid = getUser();
      pid = getId();
    }
    catch (final Exception e) {
      uid = "<usurio nulo>";
      pid = "<projeto nulo>";
    }
    if (this.serverProject == null) {
      return "[WIO project]: NULL";
    }
    return "[WIO project]:" + uid + "@" + pid;
  }

  /**
   * Construtor de projeto remoto.
   * 
   * @param fs .
   * @param userId identificador o usuario logado
   * @param ownerId identificador do usurio dono do projeto
   * @param projectId identificador do projeto.
   * 
   * @throws Exception se houver falha no servio de WIO.
   */
  protected WIOServerProject(final WIOServerFileSystem fs, final String userId,
    final String ownerId, final String projectId) throws Exception {
    if ((ownerId == null) || (projectId == null)) {
      final String msg = "Usuario/projeto nao identificado em WIOServerProject";
      throw new Exception(msg);
    }
    if (fs == null) {
      final String msg = "File-system nao identificado em WIOServerProject";
      throw new Exception(msg);
    }
    logInfo("Criao do projeto " + projectId + " (usurio: " + ownerId + ")");
    this.id = projectId;
    this.fileSystem = fs;
    this.serverProject =
      ServerProject.openProject(userId, ownerId, projectId, true);
    this.loggedUserId = userId;
    logInfo("Usurio solicitante: " + this.loggedUserId);
    fileSystem.incNumProjects();
  }
}
