/*
 * $Id: WIOService.java 89469 2009-03-18 20:05:47Z clinio $
 */
package csbase.server.services.wioservice;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.Properties;

import org.omg.PortableServer.POA;
import org.omg.PortableServer.Servant;

import csbase.remote.WIOServiceInterface;
import csbase.server.Server;
import csbase.server.ServerEntryPointImpl;
import csbase.server.ServerException;
import csbase.server.Service;
import csbase.server.services.wioservice.idl.AlgorithmInfo;
import csbase.server.services.wioservice.idl.AlgorithmParameter;
import csbase.server.services.wioservice.idl.WIOServiceException;

/**
 * A classe <code>WIOService</code> implementa o servio de acesso aos arquivos
 * de projeto do WebSintesi.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class WIOService extends Service implements WIOServiceInterface {
  /**
   * Constri a instncia do servio de gerncia de WIO
   * 
   * @throws ServerException se houver falha no servidor.
   */
  public static void createService() throws ServerException {
    new WIOService();
  }

  /**
   * Formatao de Mensagem de exceo para ser serializada por CORBA.
   * 
   * @param e a exceo Java
   * 
   * @return a string formatada.
   */
  public static String getExceptionString(final Exception e) {
    String err = e.getMessage();
    err = (err == null ? "<null>" : err);
    err = err.replaceAll("[^\\p{ASCII}]", "?");
    return err;
  }

  /**
   * Busca do servio de WIO.
   * 
   * @return o servio de WIO.
   */
  public static final WIOService getInstance() {
    return (WIOService) getInstance(SERVICE_NAME);
  }

  /**
   * Nmero mximo de arquivos projetos.
   */
  private int maxNumProjects = -9;

  /**
   * Nmero mximo de arquivos abertos por projeto.
   */
  private int maxNumFiles = -9;

  /**
   * Tamanho mximo para read de arquivo remoto
   */
  private int maxReadKb = -9;

  /**
   * Tamanho mximo para write de arquivo remoto
   */
  private int maxWriteKb = -9;

  /**
   * Servant para a interface idl WIOFileSystem
   */
  private WIOServerFileSystem wioFileSystem = null;

  /**
   * Servidor CORBA responsvel pela interao do servio.
   */
  private ORBDealer orbDealer = null;

  /**
   * Porta do wio
   */
  private int serverPort;

  /**
   * Objeto de auditagem.
   */
  WIOAudit audit = null;

  /**
   * Ativao de objeto CORBA.
   * 
   * @param id string de identificao.
   * @param o o servant CORBA.
   * 
   * @return .
   * 
   * @throws WIOServiceException se houver exceo remota a ser lanada.
   */
  protected final POA activateCorbaObject(final String id, final Servant o)
    throws WIOServiceException {
    if (orbDealer == null) {
      final String err =
        "Sem ORB dealer na ativacao CORBA [" + o.toString() + "]";
      final WIOServiceException exception = new WIOServiceException(err);
      logSevereMessage(err);
      throw exception;
    }
    return orbDealer.activateObject(id, o);
  }

  /**
   * Criao de thread de monitorao de projetos abertos.
   */
  private void createAuditThread() {
    /* Busca da propriedade que determina o intervalo de atualizao */
    final int FIVE_SECONDS = 5;
    final int ONE_HOUR = 60 * 60;
    int timeSec = getIntProperty("auditIntervalSeconds");
    if (timeSec < FIVE_SECONDS) {
      timeSec = FIVE_SECONDS;
    }
    if (timeSec > ONE_HOUR) {
      timeSec = ONE_HOUR;
    }
    logInfoMessage("Intervalo de auditagem: " + timeSec + " s.");
    audit = new WIOAudit(this, timeSec * 1000);
  }

  /**
   * Desativao de objeto CORBA.
   * 
   * @param o o servant CORBA.
   * 
   * @throws WIOServiceException se houver exceo remota a ser lanada.
   */
  protected final void deactivateCorbaObject(final Servant o)
    throws WIOServiceException {
    if (orbDealer == null) {
      final String err = "Sem ORB na desativacao: [" + o.toString() + "]";
      final WIOServiceException exception = new WIOServiceException(err);
      logSevereMessage(err);
      throw exception;
    }
    orbDealer.deactivateObject(o);
  }

  /**
   * Consulta ao nmero mximo permitido de arquivos abertos.
   * 
   * @return o nmero de arquivos.
   */
  public final int getMaxOpenedFiles() {
    return maxNumFiles;
  }

  /**
   * Consulta ao nmero mximo permitido de projetos abertos.
   * 
   * @return o nmero de projetos.
   */
  public final int getMaxOpenedProjects() {
    return maxNumProjects;
  }

  /**
   * Consulta ao tamanho mximo permitido ao servidor responder em uma leitura
   * de arquivo remoto.
   * 
   * @return o tamanho da leitura em bytes.
   */
  public final int getMaxReadKb() {
    return maxReadKb;
  }

  /**
   * Consulta ao tamanho mximo permitido ao servidor responder em uma escrita
   * de arquivo remoto.
   * 
   * @return o tamanho da leitura em bytes.
   */
  public final int getMaxWriteKb() {
    return maxWriteKb;
  }

  /**
   * Consulta ao nmero de arquivos abertos.
   * 
   * @return o nmero de arquivos.
   */
  public final int getNumOpenedFiles() {
    try {
      return (wioFileSystem != null ? wioFileSystem.getNumOpenedFiles() : 0);
    }
    catch (final WIOServiceException e) {
      return 0;
    }
  }

  /**
   * Consulta ao nmero de projetos abertos.
   * 
   * @return o nmero de projetos.
   */
  public final int getNumOpenedProjects() {
    try {
      return (wioFileSystem != null ? wioFileSystem.getNumOpenedProjects() : 0);
    }
    catch (final WIOServiceException e) {
      return 0;
    }
  }

  /**
   * Obtm o valor de uma propriedade como um nmero inteiro positivo.
   * 
   * @param propName nome da propriedade.
   * @return o valor ajustado.
   */
  private int getPositiveInt(final String propName) {
    final int prp = getIntProperty(propName);
    return Math.abs(prp);
  }

  /**
   * Retorna a porta do servidor wio
   * 
   * @return o nmero da porta.
   */
  public final String getServerPort() {
    return "" + serverPort;
  }

  /**
   * Sempre retorna verdadeiro.
   * 
   * @param arg argumanto.
   * @param event o evento.
   * 
   * @return .
   */
  @Override
  protected final boolean has2Update(final Object arg, final Object event) {
    return true;
  }

  /**
   * Inicializao do ORB do servio. Cria o servidor CORBA responsvel pela
   * interao com os clientes e ativa o objeto responsvel pela implementao
   * da interface
   * 
   * @throws ServerException se ocorrer um erro na criao do servidor CORBA ou
   *         na ativao do servant.
   */
  private void initORB() throws ServerException {
    try {
      logInfoMessage("Acessando ambiente ORB...");
      final Properties props = getExternalPropertyFile("ORB.file");
      orbDealer = new ORBDealer(props, this);
      logInfoMessage("Acesso ao ambiente ORB inicializado");
    }
    catch (final Exception e) {
      throw new ServerException("Impossvel criar servidor CORBA", e);
    }
    try {
      // Ativa o objeto CORBA que implementa a interface WIOFileSystem
      logDebugMessage("Instanciando FS remoto...");
      wioFileSystem = new WIOServerFileSystem(this);
      logDebugMessage("Ativando CORBA FS remoto...");
      orbDealer.activateObject("WIOFileSystem", wioFileSystem);
      logDebugMessage("FS remoto ativado!");
    }
    catch (final Exception e) {
      throw new ServerException("Impossvel criar handler para o FS remoto", e);
    }
  }

  /**
   * Inicializao do servio.
   * 
   * @throws ServerException se ocorrer um erro na inicializao
   */
  @Override
  public final void initService() throws ServerException {
    initORB();
    readConfiguration();
    createAuditThread();
    logInfoMessage("WIOService iniciado!");
  }

  /**
   * <p>
   * Executa o pr-login de um usurio no servidor CSBase e retorna uma URL para
   * acesso ao sistema com esse usurio.
   * </p>
   * <p>
   * Quando executar o sistema com essa URL de acesso, o projeto informado
   * (pertencente ao usurio) ser aberto e o algoritmo informado ser
   * executado.
   * </p>
   * 
   * @param login O login do usurio.
   * @param password A senha do usurio.
   * @param projectName O nome de um projeto a ser aberto ao se usar a URL de
   *        acesso.
   * @param algorithmInfo Dados sobre um algoritmo que ser executado ao se usar
   *        a URL de acesso.
   * 
   * @return Uma URL para acesso ao sistema.
   */
  public String preLogin2Web(final String login, final String password,
    final String projectName, final AlgorithmInfo algorithmInfo) {
    final ServerEntryPointImpl entryPoint =
      (ServerEntryPointImpl) Server.getInstance().getEntryPoint();
    if (entryPoint == null) {
      return null;
    }
    final Map<String, Serializable> attributes =
      new HashMap<String, Serializable>();
    attributes.put("project_name", projectName);
    attributes.put("algorithm_name", algorithmInfo.algorithmName);
    final Map<String, String> algorithmParameters =
      new HashMap<String, String>();
    for (final AlgorithmParameter parameter : algorithmInfo.parameters) {
      algorithmParameters.put(parameter.name, parameter.value);
    }
    attributes.put("algorithm_parameters", (Serializable) algorithmParameters);
    return entryPoint.preLogin2Web(login, password, new Locale("pt", "BR"),
      attributes);
  }

  /**
   * Leitura da configurao do servio.
   */
  private void readConfiguration() {
    maxWriteKb = getPositiveInt("maxWriteSizeKb");
    maxReadKb = getPositiveInt("maxReadSizeKb");
    maxNumFiles = getPositiveInt("maxOpenedFiles");
    maxNumProjects = getPositiveInt("maxOpenedProjects");
    logInfoMessage("Tam. mximo de leitura de arquivo: " + maxReadKb + " Kb");
    logInfoMessage("Tam. mximo de escrita de arquivo: " + maxWriteKb + " Kb");
    logInfoMessage("# mximo de arquivos por projeto: " + maxNumFiles + "");
    logInfoMessage("# mximo de projetos: " + maxNumProjects + "");
  }

  /**
   * Trmino do servio que destri o servant que implementa a interface de
   * comunicao com os WIO, e desativa o servidor CORBA correspondente.
   * 
   * @throws ServerException se houver exceo no servidor.
   */
  @Override
  public final void shutdownService() throws ServerException {
    try {
      if (orbDealer != null) {
        orbDealer.shutdown();
      }
      wioFileSystem = null;
      orbDealer = null;
      audit.finish();
    }
    catch (final Exception e) {
      throw new ServerException("Impossvel desativar servidor CORBA", e);
    }
  }

  /**
   * Construtor padro de repasse do nome do servio.
   * 
   * @throws ServerException se houver falha no servidor.
   */
  protected WIOService() throws ServerException {
    super(SERVICE_NAME);
  }
}
