package csbase.servlet;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;

import java.net.URL;

import java.util.Enumeration;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * <p>
 * Servlet responsvel por construir uma pgina JNLP para carregar uma
 * aplicao  via Java Web Start.
 * </p>
 * 
 * <p>
 * A pgina construda  uma cpia de um pgina modelo, especfica para cada
 * aplicao, na qual so modificados alguns parmetros e includos outros.
 * Esses parmetros so passados atravs da requisio
 * (<code>HttpServletRequest</code>).
 * </p>
 * 
 * <p>
 * A pgina modelo deve existir na mesma URL base deste servlet. O nome da
 * pgina pode ser fornecido atravs do parmetro <i>page</i>. Se o nome no
 * for fornecido,  assumido <i>index.jnlp</i>.
 * </p>
 * 
 * <p>
 * A pgina construda por este servlet inclui um parmetro para o applet
 * indicando o endereo do cliente que fez a requisio. Todos os demais
 * parmetros recebidos pelo servlet tambm so repassados  pgina,  exceo
 * do parmetro <i>page</i>.
 * </p>
 *
 * @author Leonardo Barros
 */
public class InitServlet extends HttpServlet {
  /** Tipo da resposta que ser gerada para o navegador */
  private static final String CONTENT_TYPE = "application/x-java-jnlp-file";

  /** Parmetro que permite a carga da pgina modelo especificada */
  private static final String PAGE = "page";

  /** Pgina modelo a ser carregada se nenhuma tiver sido especificada */
  private static final String DEFAULT_PAGE = "index.jnlp";

  /** Parmetro host do servidor */
  private static final String SERVER_HOST = "server_host";

  /** Parmetro porta do servidor */
  private static final String SERVER_PORT = "server_port";

  /** Porta do servidor default se nenhuma tiver sido especificada */
  private static final String DEFAULT_SERVER_PORT = "1099";

  /** Parmetro host do servidor de aplicao */
  private static final String APP_SERVER_HOST = "app_server_host";

  /** Parmetro porta do servidor de aplicao */
  private static final String APP_SERVER_PORT = "app_server_port";

  /** Parmetro IP do cliente */
  private static final String CLIENT_ADDRESS = "client_address";

  /**
   * Constri e retorna uma pgina JNLP, usando as informaes fornecidas.
   *
   * @param request informaes sobre a requisio.
   * @param response informaes sobre a resposta.
   *
   * @throws IOException
   * @throws ServletException
   */
  public void doGet(HttpServletRequest request, HttpServletResponse response)
  throws IOException, ServletException {
    response.setContentType(CONTENT_TYPE);
    PrintWriter out = response.getWriter();
    try {
      String url = getCommonFileUrl(request);
      String commonFile = readCommonFile(url);
      String customFile = customizeCommonFile(commonFile, request);
      out.println(customFile);
    }
    catch (Exception ex) {
      out.println(getErrorPage(ex));
      throw new ServletException(ex);
    }
    finally {
      if (out != null) {
        out.close();
      }
    }
  }

  /**
   * Obtm uma pgina de erro que exibe uma exceo.
   *
   * @param ex exceo ocorrida.
   *
   * @return .
   */
  private String getErrorPage(Exception ex) {
    StringBuffer output = new StringBuffer();
    output.append("<html>").append("\n");
    output.append("<head><title>Erro</title></head>").append("\n");
    output.append("<body>").append("\n");
    output.append("<h2>Erro!</h2>").append("\n");
    output.append("Por favor, entre em contato com o suporte do sistema<br/>")
    .append("\n");
    output.append("e fornea a seguinte mensagem: ").append("<hr/>\n");
    output.append("<p>").append(ex.getMessage()).append("</p>").append("\n");
    StackTraceElement[] stackLines = ex.getStackTrace();
    for (int i = 0; i < stackLines.length; i++) {
      output.append("&nbsp;&nbsp;&nbsp;").append(stackLines[i]).append(
        "<br/>\n");
    }
    output.append("<hr/></body>").append("\n");
    output.append("</html>");
    return output.toString();
  }

  /**
   * Obtm o endereo para o arquivo comum no servidor http, extrado do
   * <code>HttpServletRequest</code> recebido por este servlet. Constri a
   * mesma URL usada para acessar este servlet, trocando o nome do arquivo
   * (init.html) pelo nome da pgina original.
   *
   * @param request informaes sobre a requisio do servlet.
   *
   * @return endereo para o arquivo comum no servidor http.
   */
  String getCommonFileUrl(HttpServletRequest request) {
    String serverPort = Integer.toString(request.getServerPort());
    String fileURL =
      "http://"+request.getServerName()+":"+serverPort+request.getRequestURI();
    String fileName = getParamValue(PAGE, request, DEFAULT_PAGE);
    int lastSlashPos = fileURL.lastIndexOf("/");
    return fileURL.substring(0, lastSlashPos+1)+fileName;
  }

  /**
   * L o arquivo comum a todas as aplicaes.
   *
   * @param commonFileUrl endereo do arquivo comum no servidor http.
   *
   * @return <code>String</code> com todo o contedo do arquivo comum.
   *
   * @throws IOException
   */
  String readCommonFile(String commonFileUrl)
  throws IOException {
    URL url = new URL(commonFileUrl);
    BufferedReader br = null;
    StringBuffer buffer = new StringBuffer();
    try {
      br = new BufferedReader(new InputStreamReader(url.openStream()));
      String line = null;
      while ((line = br.readLine()) != null) {
        buffer.append(line).append("\n");
      }
    }
    finally {
      if (br != null) {
        br.close();
      }
    }
    return buffer.toString();
  }

  /**
   * Personaliza o arquivo comum para uma aplicao, modificando as
   * propriedades pertinentes  aplicao e adicionando novos parmetros.
   *
   * @param commonFile contedo do arquivo comum.
   * @param request informaes sobre a requisio do servlet.
   *
   * @return contedo do arquivo personalizado para a aplicao.
   */
  String customizeCommonFile(String commonFile, HttpServletRequest request) {
    // Parmetros embutidos na requisio
    String appServerHost = request.getServerName();
    String customFile =
      replaceParam(APP_SERVER_HOST, appServerHost, commonFile);
    String appServerPort = Integer.toString(request.getServerPort());
    customFile = replaceParam(APP_SERVER_PORT, appServerPort, customFile);
    String client_address = request.getRemoteAddr();
    customFile = replaceParam(CLIENT_ADDRESS, client_address, customFile);
    // Parmetros externos conhecidos
    String serverHost =
      getParamValue(SERVER_HOST, request, request.getServerName());
    customFile = replaceParam(SERVER_HOST, serverHost, customFile);
    String serverPort =
      getParamValue(SERVER_PORT, request, DEFAULT_SERVER_PORT);
    customFile = replaceParam(SERVER_PORT, serverPort, customFile);
    // Todos os outros parmetros externos
    for (Enumeration e = request.getParameterNames(); e.hasMoreElements();) {
      String key = (String)e.nextElement();
      if (
        key.equals(SERVER_HOST) || key.equals(SERVER_PORT) || key.equals(PAGE)) {
        continue;
      }
      String value = request.getParameter(key);
      customFile = appendParam(key, value, customFile);
    }
    return customFile;
  }

  /**
   * Extrai o parmetro obrigatrio especificado do mapa de parmetros.
   *
   * @param key chave do parmetro a ser extrado.
   * @param request informaes sobre a requisio do servlet.
   * @param defaultValue valor default, caso no tenha sido informado.
   *
   * @return valor associado ao parmetro no mapa.
   */
  private String getParamValue(
    String key, HttpServletRequest request, String defaultValue) {
    String value = request.getParameter(key);
    if (value == null || value.equals("")) {
      value = defaultValue;
    }
    return value;
  }

  /**
   * Substitui o parmetro especificado no texto especificado pelo valor
   * especificado.
   *
   * @param key chave do parmetro a ser substitudo.
   * @param value valor que deve substituir a chave.
   * @param text texto no qual ser feita a substituio.
   *
   * @return texto com a chave substituda pelo valor especificado.
   *
   * @throws IllegalStateException .
   */
  private String replaceParam(String key, String value, String text) {
    if (value == null || value.equals("")) {
      throw new IllegalStateException("parmetro "+key+" no encontrado");
    }
    String customFile = text.replaceAll("\\$"+key+"\\$", value);
    return customFile;
  }

  /**
   * Adiciona um parmetro ao texto especificado.
   *
   * @param key chave do parmetro.
   * @param value valor do parmetro.
   * @param text texto no qual ser feita a substituio.
   *
   * @return .
   *
   * @throws IllegalArgumentException .
   */
  String appendParam(String key, String value, String text) {
    int appendPos = text.toLowerCase().indexOf("</application-desc>");
    if (appendPos == -1) {
      throw new IllegalArgumentException("text no possui <application-desc/>");
    }
    String appendText = "  <argument>--"+key+" "+value+"</argument>\n  ";
    StringBuffer buffer = new StringBuffer(text.substring(0, appendPos));
    buffer.append(appendText);
    buffer.append(text.substring(appendPos, text.length()));
    return buffer.toString();
  }
}
