package csbase.client.algorithms.parameters.validators;

import java.util.HashSet;
import java.util.Locale;
import java.util.Set;

import tecgraf.javautils.LNG;
import tecgraf.javautils.configurationmanager.ConfigurationManager;

import csbase.client.algorithms.parameters.validators.ValidationResult;
import csbase.logic.algorithms.parameters.FormulaParameter;
import csbase.logic.algorithms.parameters.SimpleParameter;

import junit.framework.TestCase;

/**
 * Testes unitrios para a classe {@link FormulaParameterValidator}.
 *
 * @author lmoreira
 */
public final class FormulaParameterValidatorTest extends TestCase {
  static {
    LNG.load("csbase.client.resources.properties.language.idiom", new Locale(
      "pt", "BR"));
    try {
      ConfigurationManager.createInstance();
    }
    catch (IllegalStateException e) {}
  }

  /**
   * Testando criar um {@link FormulaParameterValidator} opcional.
   */
  public void testFormulaParameterValidator1() {
    FormulaParameterValidator validator = new FormulaParameterValidator(true);
    assertTrue(validator.isOptional());
  }

  /**
   * Testando criar um {@link FormulaParameterValidator} obrigatrio.
   */
  public void testFormulaParameterValidator2() {
    FormulaParameterValidator validator = new FormulaParameterValidator(false);
    assertFalse(validator.isOptional());
  }

  /**
   * Testando o mtodo 
   * {@link FormulaParameterValidator#validateValue(SimpleParameter, String)}
   * opcional sem valor.
   */
  public void testValidateValue1() {
    SimpleParameter<String> parameter = 
      new FormulaParameter("Nome do parmetro", "Rtulo do parmetro", 
        "Descrio do parmetro", true, "{0}={1}");
    FormulaParameterValidator validator = new FormulaParameterValidator(true);
    assertTrue(validator.isOptional());
    ValidationResult result = validator.validateValue(parameter, null);
    assertTrue(result.isWellSucceded());
  }

  /**
   * Testando o mtodo 
   * {@link FormulaParameterValidator#validateValue(SimpleParameter, String)}
   * obrigatrio sem valor.
   */
  public void testValidateValue2() {
    SimpleParameter<String> parameter = 
      new FormulaParameter("Nome do parmetro", "Rtulo do parmetro", 
        "Descrio do parmetro", true, "{0}={1}");
    FormulaParameterValidator validator = new FormulaParameterValidator(false);
    assertFalse(validator.isOptional());
    ValidationResult result = validator.validateValue(parameter, null);
    assertFalse(result.isWellSucceded());
  }
  
  /**
   * Testando o mtodo 
   * {@link FormulaParameterValidator#validateValue(SimpleParameter, String)}
   * com todos os caracteres possveis.
   */
  public void testValidateValue3() {
    Set<Character> acceptedChars = new HashSet<Character>();
    for (char ch = 'a'; ch <= 'z'; ch++) {
      acceptedChars.add(ch);
    }
    for (char ch = 'A'; ch <= 'Z'; ch++) {
      acceptedChars.add(ch);
    }
    for (char ch = '0'; ch <= '9'; ch++) {
      acceptedChars.add(ch);
    }
    acceptedChars.add('_');
    acceptedChars.add('(');
    acceptedChars.add(')');
    acceptedChars.add(';');
    acceptedChars.add('[');
    acceptedChars.add(']');
    acceptedChars.add('{');
    acceptedChars.add('}');
    acceptedChars.add('+');
    acceptedChars.add('-');
    acceptedChars.add('*');
    acceptedChars.add('/');
    acceptedChars.add('%');
    acceptedChars.add('<');
    acceptedChars.add('>');
    acceptedChars.add(' ');
    acceptedChars.add('.');
    acceptedChars.add('=');
    acceptedChars.add('^');
    acceptedChars.add('&');

    SimpleParameter<String> parameter = 
      new FormulaParameter("Nome do parmetro", "Rtulo do parmetro", 
        "Descrio do parmetro", true, "{0}={1}");
    FormulaParameterValidator validator = new FormulaParameterValidator(true);
    assertTrue(validator.isOptional());
    for (char ch = Character.MIN_VALUE; ch < Character.MAX_VALUE; ch++) {
      String value = Character.toString(ch);
      parameter.setValue(value);
      ValidationResult result = validator.validateValue(parameter, value);
      boolean isWellSucceded = acceptedChars.contains(ch);
      assertEquals(isWellSucceded, result.isWellSucceded());
    }
  }
}
