package csbase.client.ias.algorithm.niotransfer;

import static org.junit.Assert.*;

import java.io.File;
import java.io.IOException;
import java.security.DigestException;

import org.junit.BeforeClass;
import org.junit.Test;

import tecgraf.javautils.FileUtils;
import tecgraf.javautils.configurationmanager.ConfigurationManager;
import util.TestUtils;
import csbase.client.externalresources.StandaloneLocalFile;
import csbase.remote.niotransfer.RequestTransferInfo;
import csbase.server.services.algorithmservice.niotransfer.FileServer;
import csbase.server.services.algorithmservice.niotransfer.MockFileTransferrer;

/**
 * Testes funcionais para a classe {@link FileClient}.
 * 
 * Em um caso de uso real, o cliente acessaria um servio atravs de um
 * <i>stub</i> RMI; como nesta classe estamos concentrados apenas na lgica de
 * transferncia, a camada RMI est sendo abstrada e o cliente est acessando
 * diretamente o servio.
 * 
 * @author Tecgraf
 */
public class FileClientTest {
  /** Diretrios onde se encontram os dados necessrios para este teste. */
  private static String dataDir;
  /** Objeto que solicita as transferncias. */
  private static MockFileTransferrer service;
  /** Servidor de transferncia de arquivos. */
  private static FileServer server;

  /**
   * Setup do servidor mock.
   */
  @BeforeClass
  public static void runBeforeClass() {
    dataDir = TestUtils.getDataDir(FileClientTest.class);
    try {
      ConfigurationManager.createInstance();
    }
    catch (IllegalStateException e) {
    }
    service = new MockFileTransferrer();
    server = new FileServer(service);
    service.setServer(server);
    new Thread() {
      @Override
      public void run() {
        server.dispatch();
      }
    }.start();
  }

  /**
   * Testa a exportao de um arquivo pequeno.
   * 
   * @throws DigestException
   * @throws IOException
   */
  @Test
  public void testReceive1() throws DigestException, IOException {
    String sourcePath = dataDir + "origem1.doc";
    RequestTransferInfo info = service.prepareDownload(sourcePath);
    File targetFile = new File(dataDir + "destino2.doc");
    FileClient client = new FileClient("localhost", info);
    client.receive(new StandaloneLocalFile(targetFile));
    // Para simplificao, estou considerando razovel apenas a comparao dos
    // tamanhos.
    File sourceFile = new File(sourcePath);
    assertFalse(sourceFile.length() == 0);
    assertEquals(sourceFile.length(), targetFile.length());
    if (!targetFile.delete()) {
      throw new IOException("O arquivo " + targetFile.getName()
        + " no pde ser removido.");
    }
  }

  /**
   * Testa a importao de um arquivo pequeno.
   * 
   * @throws DigestException
   * @throws IOException
   */
  @Test
  public void testSend1() throws DigestException, IOException {
    String targetPath = dataDir + "destino2.txt";
    RequestTransferInfo info = service.prepareUpload(targetPath, null);
    FileClient client = new FileClient("localhost", info);
    File sourceFile = new File(dataDir + "origem2.txt");
    client.send(new StandaloneLocalFile(sourceFile));
    // Para simplificao, estou considerando razovel apenas a comparao dos
    // tamanhos.
    assertFalse(sourceFile.length() == 0);
    File targetFile = new File(targetPath);
    assertEquals(sourceFile.length(), targetFile.length());
    if (!targetFile.delete()) {
      throw new IOException("O arquivo " + targetFile.getName()
        + " no pde ser removido.");
    }
  }

  /**
   * Testa a importao de um arquivo inexistente.
   * 
   * @throws DigestException
   * @throws IOException
   */
  @Test
  public void testSend2() throws DigestException, IOException {
    String targetPath = dataDir + "destino.txt";
    RequestTransferInfo info = service.prepareUpload(targetPath, null);
    FileClient client = new FileClient("localhost", info);
    File sourceFile = new File(dataDir + "inexistente.txt");
    try {
      client.send(new StandaloneLocalFile(sourceFile));
      fail("Esperada IOException");
    }
    catch (IOException e) {
    }
    File targetFile = new File(targetPath);
    if (!targetFile.delete()) {
      throw new IOException("O arquivo " + targetFile.getName()
        + " no pde ser removido");
    }
  }

  /**
   * Testa a importao sobrescrevendo um arquivo j existente no servidor. OBS:
   * se for necessria uma confirmao de sobrescrita, deve estar a cargo do
   * servio que solicitou a transferncia, no momento da preparao ({@code
   * prepareUpload(..)} .
   * 
   * @throws DigestException
   * @throws IOException
   */
  @Test
  public void testSend3() throws DigestException, IOException {
    String targetPath = dataDir + "jaexiste.txt";
    File targetFile = new File(targetPath);
    assertFalse(targetFile.length() == 0);
    File backupFile = new File(dataDir + "jaexiste.bak");
    TestUtils.copy(targetFile, backupFile);
    File sourceFile = new File(dataDir + "origem2.txt");
    assertFalse(sourceFile.length() == targetFile.length());
    RequestTransferInfo info = service.prepareUpload(targetPath, null);
    FileClient client = new FileClient("localhost", info);
    client.send(new StandaloneLocalFile(sourceFile));
    assertEquals(sourceFile.length(), targetFile.length());
    if (!backupFile.renameTo(targetFile)) {
      throw new IOException("No foi possvel restaurar o backup!");
    }
  }

  /**
   * Testa a importao para um destino invlido (diretrio inexistente). O
   * teste supe que o diretrio tenha sido removido aps a validao na
   * preparao e antes da chamada do cliente solicitando a importao
   * propriamente dita.
   * 
   * @throws DigestException
   * @throws IOException
   */
  @Test
  public void testSend4() throws DigestException, IOException {
    String tempDirPath = dataDir + "temp";
    File tempDir = new File(tempDirPath);
    tempDir.mkdir();
    String targetPath =
      tempDir.getAbsolutePath() + File.separator + "destino.txt";
    RequestTransferInfo info = service.prepareUpload(targetPath, null);
    if (!FileUtils.delete(tempDir)) {
      throw new IOException("O diretrio " + tempDir.getAbsolutePath()
        + " no pde ser removido.");
    }
    FileClient client = new FileClient("localhost", info);
    File sourceFile = new File(dataDir + "origem2.txt");
    try {
      client.send(new StandaloneLocalFile(sourceFile));
      fail("Esperada IOException");
    }
    catch (IOException e) {
    }
  }

  /**
   * Testa a exportao de um arquivo inexistente. Este teste verifica a
   * condio de erro se o arquivo tiver sido removido aps a invocao do
   * mtodo de preparao.
   * 
   * @throws DigestException
   * @throws IOException
   */
  @Test
  public void testReceive2() throws DigestException, IOException {
    String sourcePath = dataDir + "inexistente.bin";
    File sourceFile = new File(sourcePath);
    sourceFile.createNewFile();
    RequestTransferInfo info = service.prepareDownload(sourcePath);
    if (!sourceFile.delete()) {
      throw new IOException("Arquivo " + sourcePath + " no pde ser removido.");
    }
    File targetFile = new File(dataDir + "destino.bin");
    FileClient client = new FileClient("localhost", info);
    try {
      client.receive(new StandaloneLocalFile(targetFile));
      fail("Esperada IOException");
    }
    catch (IOException e) {
    }
    if (!targetFile.delete()) {
      throw new IOException("O arquivo " + targetFile.getName()
        + " no pde ser removido.");
    }
  }

  /**
   * Testa a exportao sobrescrevendo um arquivo j existente no cliente. OBS:
   * se for necessria uma confirmao de sobrescrita, deve estar a cargo do
   * cliente que solicitou a transferncia, antes de solicitar a exportao.
   * 
   * @throws DigestException
   * @throws IOException
   */
  @Test
  public void testReceive3() throws DigestException, IOException {
    String targetPath = dataDir + "jaexiste.txt";
    File targetFile = new File(targetPath);
    assertFalse(targetFile.length() == 0);
    File backupFile = new File(dataDir + "jaexiste.bak");
    TestUtils.copy(targetFile, backupFile);
    String sourcePath = dataDir + "origem2.txt";
    File sourceFile = new File(sourcePath);
    assertFalse(sourceFile.length() == targetFile.length());
    RequestTransferInfo info = service.prepareDownload(sourcePath);
    FileClient client = new FileClient("localhost", info);
    client.receive(new StandaloneLocalFile(targetFile));
    assertEquals(sourceFile.length(), targetFile.length());
    if (!backupFile.renameTo(targetFile)) {
      throw new IOException("No foi possvel restaurar o backup!");
    }
  }

  /**
   * Testa a exportao para um destino invlido (diretrio inexistente).
   * 
   * @throws DigestException
   * @throws IOException
   */
  @Test
  public void testReceive4() throws DigestException, IOException {
    String targetPath = "/naoexiste/destino.txt";
    File targetFile = new File(targetPath);
    String sourcePath = dataDir + "origem2.txt";
    RequestTransferInfo info = service.prepareDownload(sourcePath);
    FileClient client = new FileClient("localhost", info);
    try {
      client.receive(new StandaloneLocalFile(targetFile));
      fail("Esperada IOException");
    }
    catch (IOException e) {
    }
  }
}
