package csbase.logic.algorithms;

import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.Hashtable;

import junit.framework.TestCase;

/**
 * Testes unitrios para a classe {@link Category}.
 */
public final class CategoryTest extends TestCase {

  /**
   * Testando o construtor {@link Category#Category(String, String)}:
   * Informando um identificador e nome de categorias vlidos.
   */
  public void testCategory1() {
    String id = "ID";
    String name = "Nome";
    Category category = new Category(id, name);
    assertEquals(id, category.getId());
    assertEquals(name, category.getName());
  }

  /**
   * Testando o construtor {@link Category#Category(String, String)}:
   * Informando um identificador nulo.
   */
  public void testCategory2() {
    String id = null;
    String name = "Nome";
    try {
      new Category(id, name);
      fail();
    }
    catch (IllegalArgumentException e) {
      assertEquals("id == null", e.getLocalizedMessage());
    }
  }

  /**
   * Testando o construtor {@link Category#Category(String, String)}:
   * Informando um nome nulo.
   */
  public void testCategory3() {
    String id = "ID";
    String name = null;
    try {
      new Category(id, name);
      fail();
    }
    catch (IllegalArgumentException e) {
      assertEquals("name == null", e.getLocalizedMessage());
    }
  }

  /**
   * Testando o mtodo {@link Category#addCategory(Category)}: Com sucesso.
   */
  public void testAddCategory1() {
    Category category = new Category("1", "Nome 1");
    assertNotNull(category.getCategories());
    assertTrue(category.getCategories().isEmpty());
    Category childCategory = new Category("1.1", "Nome 1.1");
    assertTrue(category.addCategory(childCategory));
    assertNotNull(category.getCategories());
    assertEquals(1, category.getCategories().size());
    assertEquals(childCategory, category.getCategories().iterator().next());
  }

  /**
   * Testando o mtodo {@link Category#addCategory(Category)}: Tentando
   * adicionar uma categoria a ela mesma.
   */
  public void testAddCategory2() {
    Category category = new Category("1", "Nome 1");
    assertNotNull(category.getCategories());
    assertTrue(category.getCategories().isEmpty());
    assertFalse(category.addCategory(category));
    assertNotNull(category.getCategories());
    assertTrue(category.getCategories().isEmpty());
  }

  /**
   * Testando o mtodo {@link Category#addCategory(Category)}: Tentando
   * adicionar uma categoria repetida.
   */
  public void testAddCategory3() {
    Category category = new Category("1", "Nome 1");
    assertNotNull(category.getCategories());
    assertTrue(category.getCategories().isEmpty());
    Category childCategory = new Category("1.1", "Nome 1.1");
    assertTrue(category.addCategory(childCategory));
    assertNotNull(category.getCategories());
    assertEquals(1, category.getCategories().size());
    assertEquals(childCategory, category.getCategories().iterator().next());
    assertFalse(category.addCategory(childCategory));
    assertNotNull(category.getCategories());
    assertEquals(1, category.getCategories().size());
    assertEquals(childCategory, category.getCategories().iterator().next());
  }

  /**
   * Testando o mtodo {@link Category#addCategory(Category)}: Tentando
   * adicionar uma categoria nula.
   */
  public void testAddCategory4() {
    Category category = new Category("1", "Nome 1");
    assertNotNull(category.getCategories());
    assertTrue(category.getCategories().isEmpty());
    try {
      category.addCategory(null);
    }
    catch (IllegalArgumentException e) {
      assertEquals("category == null", e.getLocalizedMessage());
    }
    assertNotNull(category.getCategories());
    assertTrue(category.getCategories().isEmpty());
  }

  /**
   * Testando o mtodo {@link Category#addAlgorithm(AlgorithmInfo)}: Com
   * sucesso.
   */
  public void testAddAlgorithm1() {
    Category category = new Category("Categoria 1", "Categoria 1");
    AlgorithmInfo algorithm = createAlgorithm("Algoritmo 1");
    assertNotNull(category.getAlgorithms());
    assertTrue(category.getAlgorithms().isEmpty());
    assertTrue(category.addAlgorithm(algorithm));
    assertNotNull(category.getAlgorithms());
    assertEquals(1, category.getAlgorithms().size());
    assertEquals(algorithm, category.getAlgorithms().iterator().next());
  }

  /**
   * Testando o mtodo {@link Category#addAlgorithm(AlgorithmInfo)}:
   * Adicionando um algoritmo repetido.
   */
  public void testAddAlgorithm2() {
    Category category = new Category("Categoria 1", "Categoria 1");
    AlgorithmInfo algorithm = createAlgorithm("Algoritmo 1");
    assertNotNull(category.getAlgorithms());
    assertTrue(category.getAlgorithms().isEmpty());
    assertTrue(category.addAlgorithm(algorithm));
    assertNotNull(category.getAlgorithms());
    assertEquals(1, category.getAlgorithms().size());
    assertEquals(algorithm, category.getAlgorithms().iterator().next());
    assertFalse(category.addAlgorithm(algorithm));
    assertNotNull(category.getAlgorithms());
    assertEquals(1, category.getAlgorithms().size());
    assertEquals(algorithm, category.getAlgorithms().iterator().next());
  }

  /**
   * Testando o mtodo {@link Category#addAlgorithm(AlgorithmInfo)}:
   * Adicionando um algoritmo nulo.
   */
  public void testAddAlgorithm3() {
    Category category = new Category("Categoria 1", "Categoria 1");
    assertNotNull(category.getAlgorithms());
    assertTrue(category.getAlgorithms().isEmpty());
    try {
      category.addAlgorithm(null);
    }
    catch (IllegalArgumentException e) {
      assertEquals("algorithm == null", e.getLocalizedMessage());
    }
    assertNotNull(category.getAlgorithms());
    assertTrue(category.getAlgorithms().isEmpty());
  }

  /**
   * Testando o mtodo {@link Category#removeAlgorithm(AlgorithmInfo)}:
   * Removendo um algoritmo que est cadastrado e no  o nico algoritmo.
   */
  public void testRemoveAlgorithm1() {
    Category category = new Category("Categoria 1", "Categoria 1");
    AlgorithmInfo algorithm1 = createAlgorithm("Algoritmo 1");
    AlgorithmInfo algorithm2 = createAlgorithm("Algoritmo 2");
    AlgorithmInfo algorithm3 = createAlgorithm("Algoritmo 3");
    assertTrue(category.addAlgorithm(algorithm1));
    assertTrue(category.addAlgorithm(algorithm2));
    assertTrue(category.addAlgorithm(algorithm3));
    assertEquals(new HashSet<AlgorithmInfo>(Arrays.asList(algorithm1,
      algorithm2, algorithm3)), category.getAlgorithms());
    assertTrue(category.removeAlgorithm(algorithm2));
    assertEquals(new HashSet<AlgorithmInfo>(Arrays.asList(algorithm1,
      algorithm3)), category.getAlgorithms());
  }

  /**
   * Testando o mtodo {@link Category#removeAlgorithm(AlgorithmInfo)}:
   * Removendo um algoritmo que est cadastrado e  o nico algoritmo.
   */
  public void testRemoveAlgorithm2() {
    Category category = new Category("Categoria 1", "Categoria 1");
    AlgorithmInfo algorithm1 = createAlgorithm("Algoritmo 1");
    assertTrue(category.addAlgorithm(algorithm1));
    assertEquals(new HashSet<AlgorithmInfo>(Arrays.asList(algorithm1)),
      category.getAlgorithms());
    assertTrue(category.removeAlgorithm(algorithm1));
    assertEquals(Collections.emptySet(), category.getAlgorithms());
  }

  /**
   * Testando o mtodo {@link Category#removeAlgorithm(AlgorithmInfo)}:
   * Removendo um algoritmo que no est cadastrado, mas existem algoritmos
   * cadastrados.
   */
  public void testRemoveAlgorithm3() {
    Category category = new Category("Categoria 1", "Categoria 1");
    AlgorithmInfo algorithm1 = createAlgorithm("Algoritmo 1");
    AlgorithmInfo algorithm2 = createAlgorithm("Algoritmo 2");
    assertTrue(category.addAlgorithm(algorithm1));
    assertTrue(category.addAlgorithm(algorithm2));
    assertEquals(new HashSet<AlgorithmInfo>(Arrays.asList(algorithm1,
      algorithm2)), category.getAlgorithms());
    AlgorithmInfo algorithm3 = createAlgorithm("Algoritmo 3");
    assertFalse(category.removeAlgorithm(algorithm3));
    assertEquals(new HashSet<AlgorithmInfo>(Arrays.asList(algorithm1,
      algorithm2)), category.getAlgorithms());
  }

  /**
   * Testando o mtodo {@link Category#removeAlgorithm(AlgorithmInfo)}:
   * Removendo sem algoritmos cadastrados.
   */
  public void testRemoveAlgorithm4() {
    Category category = new Category("Categoria 1", "Categoria 1");
    AlgorithmInfo algorithm1 = createAlgorithm("Algoritmo 1");
    assertEquals(Collections.emptySet(), category.getAlgorithms());
    assertFalse(category.removeAlgorithm(algorithm1));
    assertEquals(Collections.emptySet(), category.getAlgorithms());
  }

  /**
   * Testando o mtodo {@link Category#removeAlgorithm(AlgorithmInfo)}:
   * Removendo sem algoritmos cadastrados.
   */
  public void testRemoveAlgorithm5() {
    Category category = new Category("Categoria 1", "Categoria 1");
    assertEquals(Collections.emptySet(), category.getAlgorithms());
    try {
      category.removeAlgorithm(null);
      fail();
    }
    catch (IllegalArgumentException e) {
      assertEquals("algorithm == null", e.getMessage());
    }
    assertEquals(Collections.emptySet(), category.getAlgorithms());
  }

  /**
   * Testando o mtodo {@link Category#equals(Object)}: Comparando 2 categorias
   * com identificadores iguais.
   */
  public void testEquals1() {
    String id = "Identificador";

    String name1 = "Nome 1";
    Category category1 = new Category(id, name1);

    String name2 = "Nome 2";
    Category category2 = new Category(id, name2);

    assertTrue(category1.equals(category2));
  }

  /**
   * Testando o mtodo {@link Category#equals(Object)}: Comparando 2 categorias
   * com identificadores diferentes.
   */
  public void testEquals2() {
    String name = "Nome";

    String id1 = "Identificador 1";
    Category category1 = new Category(id1, name);

    String id2 = "Identificador 2";
    Category category2 = new Category(id2, name);

    assertFalse(category1.equals(category2));
  }

  /**
   * Testando o mtodo {@link Category#equals(Object)}: Comparando com
   * {@code null}.
   */
  public void testEquals3() {
    String id = "Identificador";
    String name = "Nome";
    Category category = new Category(id, name);
    assertFalse(category.equals(null));
  }

  /**
   * Testando o mtodo {@link Category#equals(Object)}: Comparando com um
   * objeto de outra classe.
   */
  public void testEquals4() {
    String id = "Identificador";
    String name = "Nome";
    Category category = new Category(id, name);
    assertFalse(category.equals("Objeto de outra classe"));
  }

  /**
   * Testando o mtodo {@link Category#hashCode()}: Comparando 2 categorias com
   * identificadores iguais.
   */
  public void testHashCode1() {
    String id = "Identificador";

    String name1 = "Nome 1";
    Category category1 = new Category(id, name1);

    String name2 = "Nome 2";
    Category category2 = new Category(id, name2);

    assertEquals(category1, category2);

    assertEquals(category1.hashCode(), category2.hashCode());
  }

  /**
   * Testando o mtodo {@link Category#toString()}.
   */
  public void testToString1() {
    String id = "Identificador da categoria";
    String name = "Nome da categoria";
    Category category = new Category(id, name);
    assertEquals(name, category.toString());
  }

  /**
   * Testando o mtodo {@link Category#compareTo(Category)}: Com categorias
   * iguais.
   */
  public void testCompareTo1() {
    String id = "ID";
    String name = "Nome";
    Category category1 = new Category(id, name);
    Category category2 = new Category(id, name);
    assertEquals(0, category1.compareTo(category2));
  }

  /**
   * Testando o mtodo {@link Category#compareTo(Category)}: A primeira
   * categoria  menor do que a segunda.
   */
  public void testCompareTo2() {
    String id1 = "z";
    String name1 = "AAA";
    String id2 = "a";
    String name2 = "BBB";
    Category category1 = new Category(id1, name1);
    Category category2 = new Category(id2, name2);
    assertTrue(category1.compareTo(category2) < 0);
  }

  /**
   * Testando o mtodo {@link Category#compareTo(Category)}: A primeira
   * categoria  maior do que a segunda.
   */
  public void testCompareTo3() {
    String id1 = "a";
    String name1 = "BBB";
    String id2 = "z";
    String name2 = "AAA";
    Category category1 = new Category(id1, name1);
    Category category2 = new Category(id2, name2);
    assertTrue(category1.compareTo(category2) > 0);
  }

  /**
   * Testando o mtodo {@link Category#compareTo(Category)}: A segunda
   * categoria  {@code null}.
   */
  public void testCompareTo4() {
    String id = "ID";
    String name = "Nome";
    Category category = new Category(id, name);
    try {
      category.compareTo(null);
      fail();
    }
    catch (NullPointerException e) {
    }
  }

  /**
   * Cria um algoritmo com o identificador fornecido.
   * 
   * @param algorithmId O identificador (No aceita {@code null}).
   * 
   * @return O algoritmo.
   */
  private static AlgorithmInfo createAlgorithm(String algorithmId) {
    return new AlgorithmInfo(algorithmId, "Nome de " + algorithmId,
      "diretrio", new Hashtable<String, String>());
  }
}
