package csbase.logic.algorithms.flows;

import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import junit.framework.TestCase;

import csbase.exception.ConfigurationException;
import csbase.exception.OperationFailureException;
import csbase.exception.ParseException;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.MockAlgorithmService;

/**
 * Testes unitrio para a classe {@link FlowAlgorithmParser}.
 * 
 * @author lmoreira
 */
public final class FlowAlgorithmParserTest extends TestCase {

  static {
    ClientRemoteLocator.algorithmService = new MockAlgorithmService();   
  }
  
  /**
   * Testando a leitura de um fluxo vazio.
   * 
   * @throws ParseException
   * @throws OperationFailureException 
   * 
   */
  public void testRead1() throws ParseException, OperationFailureException {
    
    Flow expectedFlow = new Flow(null, new HashSet<FlowNode>(),
      new HashSet<FlowLink>());
    String text = "";
    text += "<?xml version='1.0' encoding='iso8859-1'?>\n";
    text += "<fluxo>\n";
    text += "</fluxo>\n";
    checkDecodingFlow(text, expectedFlow);
  }

  /**
   * Testando ler um fluxo com 1 n (com 1 algoritmo).
   * 
   * @throws ParseException
   * @throws OperationFailureException 
   * 
   */
  public void testRead2() throws ParseException, OperationFailureException {
    
    int nodeId = 1;
    String algorithmName = "Nome do algoritmo";
    AlgorithmVersionId algorithmVersionId = new AlgorithmVersionId(11, 22, 33);
    Set<NodeParameter> parameters = new HashSet<NodeParameter>();
    int x = 10;
    int y = 20;
    int width = 30;
    int height = 40;
    FlowNode node = new FlowNode(nodeId, algorithmName, algorithmVersionId,
      parameters, x, y, width, height);
    Set<FlowNode> nodes = new HashSet<FlowNode>();
    nodes.add(node);
    Flow expectedFlow = new Flow(algorithmName, nodes, new HashSet<FlowLink>());
    String text = "";
    text += "<?xml version='1.0' encoding='iso8859-1'?>\n";
    text += "<fluxo>\n";
    text += "<no id = '" + nodeId + "'>\n";
    text += "<algoritmo nome = '" + algorithmName + "' versao = '"
      + algorithmVersionId + "'/>\n";
    text += "<ponto x = '" + x + "' y = '" + y + "'/>\n";
    text += "<tamanho altura = '" + height + "' largura = '" + width + "'/>\n";
    text += "</no>\n";
    text += "</fluxo>\n";
    checkDecodingFlow(text, expectedFlow);
  }

  /**
   * Testando ler um fluxo com 1 n (com 1 algoritmo) e com 1 parmetro.
   * 
   * @throws ParseException
   * @throws OperationFailureException 
   * 
   */
  public void testRead3() throws ParseException, OperationFailureException {
    
    int nodeId = 1;
    String algorithmName = "Nome do algoritmo";
    AlgorithmVersionId algorithmVersionId = new AlgorithmVersionId(11, 22, 33);
    Set<NodeParameter> parameters = new HashSet<NodeParameter>();
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterType = "Tipo do parmetro";
    NodeParameter parameter = new NodeParameter(parameterName, parameterLabel,
      parameterType, null);
    parameters.add(parameter);
    int x = 10;
    int y = 20;
    int width = 30;
    int height = 40;
    FlowNode node = new FlowNode(nodeId, algorithmName, algorithmVersionId,
      parameters, x, y, width, height);
    Set<FlowNode> nodes = new HashSet<FlowNode>();
    nodes.add(node);
    Flow expectedFlow = new Flow(algorithmName, nodes, new HashSet<FlowLink>());
    String text = "";
    text += "<?xml version='1.0' encoding='iso8859-1'?>\n";
    text += "<fluxo>\n";
    text += "<no id = '" + nodeId + "'>\n";
    text += "<algoritmo nome = '" + algorithmName + "' versao = '"
      + algorithmVersionId + "'>\n";
    text += "<parametro nome = '" + parameterName + "' rotulo = '"
      + parameterLabel + "' tipo = '" + parameterType + "'/>";
    text += "</algoritmo>\n";
    text += "<ponto x = '" + x + "' y = '" + y + "'/>\n";
    text += "<tamanho altura = '" + height + "' largura = '" + width + "'/>\n";
    text += "</no>\n";
    text += "</fluxo>\n";
    checkDecodingFlow(text, expectedFlow);
  }

  /**
   * Testando ler um fluxo com 1 n (com 1 algoritmo) e com 1 parmetro com
   * valor (escrito como atributo do elemento parmetro).
   * 
   * @throws ParseException
   * @throws OperationFailureException 
   * 
   */
  public void testRead4() throws ParseException, OperationFailureException {
    
    int nodeId = 1;
    String algorithmName = "Nome do algoritmo";
    AlgorithmVersionId algorithmVersionId = new AlgorithmVersionId(11, 22, 33);
    Set<NodeParameter> parameters = new HashSet<NodeParameter>();
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterType = "Tipo do parmetro";
    String parameterValue = "Valor do parmetro";
    NodeParameter parameter = new NodeParameter(parameterName, parameterLabel,
      parameterType, parameterValue);
    parameters.add(parameter);
    int x = 10;
    int y = 20;
    int width = 30;
    int height = 40;
    FlowNode node = new FlowNode(nodeId, algorithmName, algorithmVersionId,
      parameters, x, y, width, height);
    Set<FlowNode> nodes = new HashSet<FlowNode>();
    nodes.add(node);
    Flow expectedFlow = new Flow(algorithmName, nodes, new HashSet<FlowLink>());
    String text = "";
    text += "<?xml version='1.0' encoding='iso8859-1'?>\n";
    text += "<fluxo>\n";
    text += "<no id = '" + nodeId + "'>\n";
    text += "<algoritmo nome = '" + algorithmName + "' versao = '"
      + algorithmVersionId + "'>\n";
    text += "<parametro nome = '" + parameterName + "' rotulo = '"
      + parameterLabel + "' tipo = '" + parameterType + "' valor='"
      + parameterValue + "'/>";
    text += "</algoritmo>\n";
    text += "<ponto x = '" + x + "' y = '" + y + "'/>\n";
    text += "<tamanho altura = '" + height + "' largura = '" + width + "'/>\n";
    text += "</no>\n";
    text += "</fluxo>\n";
    checkDecodingFlow(text, expectedFlow);
  }

  /**
   * Testando ler um fluxo com 1 n (com 1 algoritmo) e com 1 parmetro com
   * valor (escrito como CDATA do elemento parmetro).
   * 
   * @throws ParseException
   * @throws OperationFailureException 
   * 
   */
  public void testRead5() throws ParseException, OperationFailureException {
    
    int nodeId = 1;
    String algorithmName = "Nome do algoritmo";
    AlgorithmVersionId algorithmVersionId = new AlgorithmVersionId(11, 22, 33);
    Set<NodeParameter> parameters = new HashSet<NodeParameter>();
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterType = "Tipo do parmetro";
    String parameterValue = "Valor do parmetro";
    NodeParameter parameter = new NodeParameter(parameterName, parameterLabel,
      parameterType, parameterValue);
    parameters.add(parameter);
    int x = 10;
    int y = 20;
    int width = 30;
    int height = 40;
    FlowNode node = new FlowNode(nodeId, algorithmName, algorithmVersionId,
      parameters, x, y, width, height);
    Set<FlowNode> nodes = new HashSet<FlowNode>();
    nodes.add(node);
    Flow expectedFlow = new Flow(algorithmName, nodes, new HashSet<FlowLink>());
    String text = "";
    text += "<?xml version='1.0' encoding='iso8859-1'?>\n";
    text += "<fluxo>\n";
    text += "<no id = '" + nodeId + "'>\n";
    text += "<algoritmo nome = '" + algorithmName + "' versao = '"
      + algorithmVersionId + "'>\n";
    text += "<parametro nome = '" + parameterName + "' rotulo = '"
      + parameterLabel + "' tipo = '" + parameterType + "'>\n";
    text += "<valor>\n";
    text += "<![CDATA["+parameterValue+"]]>";
    text += "</valor>\n";
    text += "</parametro>\n";
    text += "</algoritmo>\n";
    text += "<ponto x = '" + x + "' y = '" + y + "'/>\n";
    text += "<tamanho altura = '" + height + "' largura = '" + width + "'/>\n";
    text += "</no>\n";
    text += "</fluxo>\n";
    checkDecodingFlow(text, expectedFlow);
  }
  
  /**
   * Testando ler um fluxo com 1 uma conexo.
   * 
   * @throws ParseException
   * @throws OperationFailureException 
   * 
   */
  public void testRead6() throws ParseException, OperationFailureException {
    
    Set<FlowNode> nodes = new HashSet<FlowNode>();
    Set<FlowLink> links = new HashSet<FlowLink>();
    int id = 1;
    FlowLink link = new FlowLink(id, null, null, new LinkedList<Point>());
    links.add(link);
    Flow expectedFlow = new Flow(null, nodes, links);
    String text = "";
    text += "<?xml version='1.0' encoding='iso8859-1'?>\n";
    text += "<fluxo>\n";
    text += "<conexao id = '" + id + "'/>\n";
    text += "</fluxo>\n";
    checkDecodingFlow(text, expectedFlow);
  }

  /**
   * Testando ler um fluxo com 1 uma conexo e com a sada conectada a um
   * parmetro.
   * 
   * @throws ParseException
   * @throws OperationFailureException 
   * 
   */
  public void testRead7() throws ParseException, OperationFailureException {
    
    Set<FlowNode> nodes = new HashSet<FlowNode>();
    Set<FlowLink> links = new HashSet<FlowLink>();
    int id = 1;
    int nodeId = 2;
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    LinkParameter linkParameter = new LinkParameter(nodeId, parameterName,
      parameterLabel);
    FlowLink link = new FlowLink(id, linkParameter, null,
      new LinkedList<Point>());
    links.add(link);
    Flow expectedFlow = new Flow(null, nodes, links);
    String text = "";
    text += "<?xml version='1.0' encoding='iso8859-1'?>\n";
    text += "<fluxo>\n";
    text += "<conexao id = '" + id + "'>\n";
    text += "<saida no='" + nodeId + "' parametro='" + parameterName
      + "' rotulo='" + parameterLabel + "'/>\n";
    text += "</conexao>\n";
    text += "</fluxo>\n";
    checkDecodingFlow(text, expectedFlow);
  }

  /**
   * /** Testando ler um fluxo com 1 uma conexo e com a entrada conectada a um
   * parmetro.
   * 
   * @throws ParseException
   * @throws OperationFailureException 
   * 
   */
  public void testRead8() throws ParseException, OperationFailureException {
    
    Set<FlowNode> nodes = new HashSet<FlowNode>();
    Set<FlowLink> links = new HashSet<FlowLink>();
    int id = 1;
    int nodeId = 2;
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    LinkParameter linkParameter = new LinkParameter(nodeId, parameterName,
      parameterLabel);
    FlowLink link = new FlowLink(id, null, linkParameter,
      new LinkedList<Point>());
    links.add(link);
    Flow expectedFlow = new Flow(null, nodes, links);
    String text = "";
    text += "<?xml version='1.0' encoding='iso8859-1'?>\n";
    text += "<fluxo>\n";
    text += "<conexao id = '" + id + "'>\n";
    text += "<entrada no='" + nodeId + "' parametro='" + parameterName
      + "' rotulo='" + parameterLabel + "'/>\n";
    text += "</conexao>\n";
    text += "</fluxo>\n";
    checkDecodingFlow(text, expectedFlow);
  }

  /**
   * Testando ler um fluxo com 1 uma conexo com pontos.
   * 
   * @throws ParseException
   * @throws OperationFailureException 
   * 
   */
  public void testRead9() throws ParseException, OperationFailureException {
    
    int id = 1;
    Set<FlowNode> nodes = new HashSet<FlowNode>();
    Set<FlowLink> links = new HashSet<FlowLink>();
    List<Point> points = new LinkedList<Point>();
    int x1 = 11;
    int y1 = 111;
    points.add(new Point(x1, y1));
    int x2 = 22;
    int y2 = 222;
    points.add(new Point(x2, y2));
    FlowLink link = new FlowLink(id, null, null, points);
    links.add(link);
    Flow expectedFlow = new Flow(null, nodes, links);
    String text = "";
    text += "<?xml version='1.0' encoding='iso8859-1'?>\n";
    text += "<fluxo>\n";
    text += "<conexao id = '" + id + "'>\n";
    text += "<ponto x='" + x1 + "' y='" + y1 + "'/>\n";
    text += "<ponto x='" + x2 + "' y='" + y2 + "'/>\n";
    text += "</conexao>\n";
    text += "</fluxo>\n";
    checkDecodingFlow(text, expectedFlow);
  }

  /**
   * Testando escrever um fluxo vazio.
   * 
   * @throws ParseException
   * @throws OperationFailureException 
   * 
   */
  public void testWrite1() throws ParseException,  OperationFailureException {
    
    Flow expectedFlow = new Flow(null, new HashSet<FlowNode>(),
      new HashSet<FlowLink>());
    checkEncodingFlow(expectedFlow);
  }

  /**
   * Testando escrever um fluxo com 1 n.
   * 
   * @throws ParseException
   * @throws OperationFailureException 
   * 
   */
  public void testWrite2() throws ParseException, OperationFailureException {
    
    int nodeId = 1;
    String algorithmName = "Nome do algoritmo";
    AlgorithmVersionId algorithmVersionId = new AlgorithmVersionId(11, 22, 33);
    Set<NodeParameter> parameters = new HashSet<NodeParameter>();
    int x = 10;
    int y = 20;
    int width = 30;
    int height = 40;
    FlowNode node = new FlowNode(nodeId, algorithmName, algorithmVersionId,
      parameters, x, y, width, height);
    Set<FlowNode> nodes = new HashSet<FlowNode>();
    nodes.add(node);
    Flow expectedFlow = new Flow(algorithmName, nodes, new HashSet<FlowLink>());
    checkEncodingFlow(expectedFlow);
  }

  /**
   * Testando escrever um fluxo com 1 n com parmetro sem valor.
   * 
   * @throws ParseException
   * @throws OperationFailureException 
   * 
   */
  public void testWrite3() throws ParseException, OperationFailureException {
    
    int nodeId = 1;
    String algorithmName = "Nome do algoritmo";
    AlgorithmVersionId algorithmVersionId = new AlgorithmVersionId(11, 22, 33);
    Set<NodeParameter> parameters = new HashSet<NodeParameter>();
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterType = "Tipo do parmetro";
    NodeParameter parameter = new NodeParameter(parameterName, parameterLabel,
      parameterType, null);
    parameters.add(parameter);
    int x = 10;
    int y = 20;
    int width = 30;
    int height = 40;
    FlowNode node = new FlowNode(nodeId, algorithmName, algorithmVersionId,
      parameters, x, y, width, height);
    Set<FlowNode> nodes = new HashSet<FlowNode>();
    nodes.add(node);
    Flow expectedFlow = new Flow(algorithmName, nodes, new HashSet<FlowLink>());
    checkEncodingFlow(expectedFlow);
  }

  /**
   * Testando escrever um fluxo com 1 n com parmetro com valor.
   * 
   * @throws ParseException
   * @throws OperationFailureException 
   * 
   */
  public void testWrite4() throws ParseException, OperationFailureException {
    
    int nodeId = 1; 
    String algorithmName = "Nome do algoritmo";
    AlgorithmVersionId algorithmVersionId = new AlgorithmVersionId(11, 22, 33);
    Set<NodeParameter> parameters = new HashSet<NodeParameter>();
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterType = "Tipo do parmetro";
    String parameterValue = "Valor do parmetro";
    NodeParameter parameter = new NodeParameter(parameterName, parameterLabel,
      parameterType, parameterValue);
    parameters.add(parameter);
    int x = 10;
    int y = 20;
    int width = 30;
    int height = 40;
    FlowNode node = new FlowNode(nodeId, algorithmName, algorithmVersionId,
      parameters, x, y, width, height);
    Set<FlowNode> nodes = new HashSet<FlowNode>();
    nodes.add(node);
    Flow expectedFlow = new Flow(algorithmName, nodes, new HashSet<FlowLink>());
    checkEncodingFlow(expectedFlow);
  }

  /**
   * Testando escrever um fluxo com 1 conexo.
   * 
   * @throws ParseException
   * @throws OperationFailureException 
   * 
   */
  public void testWrite5() throws ParseException, OperationFailureException {
    
    Set<FlowNode> nodes = new HashSet<FlowNode>();
    Set<FlowLink> links = new HashSet<FlowLink>();
    int id = 1;
    FlowLink link = new FlowLink(id, null, null, new LinkedList<Point>());
    links.add(link);
    Flow expectedFlow = new Flow(null, nodes, links);
    checkEncodingFlow(expectedFlow);
  }

  /**
   * Testando escrever um fluxo com 1 conexo com parmetro de sada.
   * 
   * @throws ParseException
   * @throws OperationFailureException 
   * 
   */
  public void testWrite6() throws ParseException, OperationFailureException {
    
    Set<FlowNode> nodes = new HashSet<FlowNode>();
    Set<FlowLink> links = new HashSet<FlowLink>();
    int id = 1;
    int nodeId = 2;
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    LinkParameter linkParameter = new LinkParameter(nodeId, parameterName,
      parameterLabel);
    FlowLink link = new FlowLink(id, linkParameter, null,
      new LinkedList<Point>());
    links.add(link);
    Flow expectedFlow = new Flow(null, nodes, links);
    checkEncodingFlow(expectedFlow);
  }

  /**
   * Testando escrever um fluxo com 1 conexo de entrada.
   * 
   * @throws ParseException
   * @throws OperationFailureException 
   * 
   */
  public void testWrite7() throws ParseException, OperationFailureException {
    
    Set<FlowNode> nodes = new HashSet<FlowNode>();
    Set<FlowLink> links = new HashSet<FlowLink>();
    int id = 1;
    int nodeId = 2;
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    LinkParameter linkParameter = new LinkParameter(nodeId, parameterName,
      parameterLabel);
    FlowLink link = new FlowLink(id, null, linkParameter,
      new LinkedList<Point>());
    links.add(link);
    Flow expectedFlow = new Flow(null, nodes, links);
    checkEncodingFlow(expectedFlow);
  }

  /**
   * Testando escrever um fluxo com pontos.
   * 
   * @throws ParseException
   * @throws OperationFailureException 
   * 
   */
  public void testWrite8() throws ParseException, OperationFailureException {
    
    int id = 1;
    Set<FlowNode> nodes = new HashSet<FlowNode>();
    Set<FlowLink> links = new HashSet<FlowLink>();
    List<Point> points = new LinkedList<Point>();
    int x1 = 11;
    int y1 = 111;
    points.add(new Point(x1, y1));
    int x2 = 22;
    int y2 = 222;
    points.add(new Point(x2, y2));
    FlowLink link = new FlowLink(id, null, null, points);
    links.add(link);
    Flow expectedFlow = new Flow(null, nodes, links);
    checkEncodingFlow(expectedFlow);
  }

  /**
   * Verifica se 2 fluxos so idnticos.
   * 
   * @param expectedFlow O fluxo esperado (No aceita {@code null}).
   * @param foundFlow O fluxo encontrado (No aceita {@code null}).
   */
  private void assertFlows(Flow expectedFlow, Flow foundFlow) {
    assertEquals(expectedFlow.getLinks(), foundFlow.getLinks());
    assertLinks(expectedFlow, foundFlow);
    assertNodes(expectedFlow, foundFlow);
  }

  /**
   * Verifica se 2 conexes so idnticas.
   * 
   * @param expectedLink A conexo esperada (No aceita {@code null}).
   * @param foundLink A conexo encontrada (No aceita {@code null}).
   */
  private void assertLink(FlowLink expectedLink, FlowLink foundLink) {
    assertEquals(expectedLink.getId(), foundLink.getId());
    assertLinkParameters(expectedLink.getInput(), foundLink.getInput());
    assertLinkParameters(expectedLink.getOutput(), foundLink.getOutput());
    assertEquals(expectedLink.getPoints(), foundLink.getPoints());
  }

  /**
   * Verifica se 2 parmetros de conexes so idnticos.
   * 
   * @param expectedParameter O parmetro esperado (No aceita {@code null}).
   * @param foundParameter O parmetro encontrado (No aceita {@code null}).
   */
  private void assertLinkParameters(LinkParameter expectedParameter,
    LinkParameter foundParameter) {
    assertEquals(expectedParameter, foundParameter);
    if (expectedParameter != null && foundParameter != null) {
      assertEquals(expectedParameter.getNodeId(), foundParameter.getNodeId());
      assertEquals(expectedParameter.getName(), foundParameter.getName());
      assertEquals(expectedParameter.getLabel(), foundParameter.getLabel());
    }
  }

  /**
   * Verifica se as conexes de 2 fluxos so idnticas.
   * 
   * @param expectedFlow O fluxo esperado (No aceita {@code null}).
   * @param foundFlow O fluxo encontrado (No aceita {@code null}).
   */
  private void assertLinks(Flow expectedFlow, Flow foundFlow) {
    for (FlowLink expectedLink : expectedFlow.getLinks()) {
      for (FlowLink foundLink : foundFlow.getLinks()) {
        if (expectedLink.equals(foundLink)) {
          assertLink(expectedLink, foundLink);
        }
      }
    }
  }

  /**
   * Verifica se os parmetros de 2 ns so idnticas.
   * 
   * @param expectedNode O n esperado (No aceita {@code null}).
   * @param foundNode O n encontrado (No aceita {@code null}).
   */
  private void assertNodeParameters(FlowNode expectedNode, FlowNode foundNode) {
    assertEquals(expectedNode.getParameters(), foundNode.getParameters());
    for (NodeParameter expectedNodeParameter : expectedNode.getParameters()) {
      for (NodeParameter foundNodeParameter : foundNode.getParameters()) {
        assertEquals(expectedNodeParameter.getName(), foundNodeParameter
          .getName());
        assertEquals(expectedNodeParameter.getLabel(), foundNodeParameter
          .getLabel());
        assertEquals(expectedNodeParameter.getType(), foundNodeParameter
          .getType());
        assertEquals(expectedNodeParameter.getValue(), foundNodeParameter
          .getValue());
      }
    }
  }

  /**
   * Verifica se os ns de 2 fluxos so idnticas.
   * 
   * @param expectedFlow O fluxo esperado (No aceita {@code null}).
   * @param foundFlow O fluxo encontrado (No aceita {@code null}).
   */
  private void assertNodes(Flow expectedFlow, Flow foundFlow) {
    assertEquals(expectedFlow.getNodes(), foundFlow.getNodes());
    for (FlowNode expectedNode : expectedFlow.getNodes()) {
      for (FlowNode foundNode : foundFlow.getNodes()) {
        if (expectedNode.equals(foundNode)) {
          assertNodes(expectedNode, foundNode);
        }
      }
    }
  }

  /**
   * Verifica se 2 ns so idnticas.
   * 
   * @param expectedNode O n esperado (No aceita {@code null}).
   * @param foundNode O n encontrado (No aceita {@code null}).
   */
  private void assertNodes(FlowNode expectedNode, FlowNode foundNode) {
    assertEquals(expectedNode.getId(), foundNode.getId());
    assertEquals(expectedNode.getAlgorithmName(), foundNode.getAlgorithmName());
    assertEquals(expectedNode.getAlgorithmVersionId(), foundNode
      .getAlgorithmVersionId());
    assertNodeParameters(expectedNode, foundNode);
    assertEquals(expectedNode.getX(), foundNode.getX());
    assertEquals(expectedNode.getY(), foundNode.getY());
    assertEquals(expectedNode.getWidth(), foundNode.getWidth());
    assertEquals(expectedNode.getHeight(), foundNode.getHeight());
  }

  /**
   * Verifica se texto passado  uma representao de um fluxo idntico ao fluxo
   * fornecido.
   * 
   * @param text O texto codificado (No aceita {@code null}).
   * @param flow O fluxo esperado (No aceita {@code null}).
   * 
   * @throws ParseException
   * @throws OperationFailureException 
   * 
   */
  private void checkDecodingFlow(String text, Flow flow) throws ParseException,
    OperationFailureException {
    Flow foundFlow = decodeFlow(text);
    assertFlows(flow, foundFlow);
  }

  /**
   * Verifica se  possvel codificar o fluxo fornecido.
   * 
   * @param flow O fluxo (No aceita {@code null}).
   * 
   * @throws ParseException
   * @throws OperationFailureException 
   * 
   */
  private void checkEncodingFlow(Flow flow) throws ParseException,
    OperationFailureException {
    String text = encodeFlow(flow);
    Flow foundFlow = decodeFlow(text);
    assertFlows(flow, foundFlow);
  }

  /**
   * Cria um fluxo a partir de um texto codificado.
   * 
   * @param text O texto (No aceita {@code null}).
   * 
   * @return O fluxo.
   * 
   * @throws ConfigurationException
   * @throws ParseException
   * @throws OperationFailureException 
   * 
   */
  private Flow decodeFlow(String text) throws ConfigurationException,
    ParseException, OperationFailureException {
    FlowAlgorithmParser parser = new FlowAlgorithmParser();
    Flow foundFlow = parser.decode(text);
    return foundFlow;
  }

  /**
   * Codifica um fluxo.
   * 
   * @param flow O fluxo (No aceita {@code null}).
   * 
   * @return O texto com o fluxo codificado.
   * 
   * @throws ConfigurationException
   */
  private String encodeFlow(Flow flow) throws ConfigurationException {
    FlowAlgorithmParser parser = new FlowAlgorithmParser();
    return parser.encode(flow);
  }
}
