/*
 * $Author:$ $Date:$ $Release:$
 */
package csbase.logic.algorithms.parameters.conditions;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Hashtable;
import java.util.List;

import junit.framework.TestCase;
import csbase.logic.FileInfo;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.logic.algorithms.ExecutionType;
import csbase.logic.algorithms.parameters.MockSimpleParameter;
import csbase.logic.algorithms.parameters.Parameter;
import csbase.logic.algorithms.parameters.ParameterGroup;
import csbase.logic.algorithms.parameters.SimpleAlgorithmConfigurator;
import csbase.logic.algorithms.parameters.SimpleParameter;

public final class SimpleConditionTest extends TestCase {
  public void testSimpleCondition() {
    String parameterName = "Nome do parmetro";
    Object conditionValue = "Valor condio";
    SimpleCondition condition =
      new SimpleCondition(parameterName, conditionValue);
    assertEquals(parameterName, condition.getParameterName());
    assertEquals(conditionValue, condition.getValue());
  }

  public void testSimpleConditionWithNullParameter() {
    Object conditionValue = "Valor condio";
    try {
      new SimpleCondition(null, conditionValue);
      fail(
        "No deveria ser possvel criar uma condio-simples sem parmetro.");
    }
    catch (IllegalArgumentException e) {
    }
  }

  public void testEquals() {
    String parameterName = "Nome do parmetro";
    Object conditionValue = "Valor condio";
    SimpleCondition condition1 =
      new SimpleCondition(parameterName, conditionValue);
    SimpleCondition condition2 =
      new SimpleCondition(parameterName, conditionValue);
    assertTrue(condition1.equals(condition2));
  }

  public void testEqualsWithNullValues() {
    String parameterName = "Nome do parmetro";
    Object conditionValue = null;
    SimpleCondition condition1 =
      new SimpleCondition(parameterName, conditionValue);
    SimpleCondition condition2 =
      new SimpleCondition(parameterName, conditionValue);
    assertTrue(condition1.equals(condition2));
  }

  public void testEqualsWithDifferentParameter() {
    String parameterName1 = "Nome do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    Object conditionValue = "Valor condio";
    SimpleCondition condition1 =
      new SimpleCondition(parameterName1, conditionValue);
    SimpleCondition condition2 =
      new SimpleCondition(parameterName2, conditionValue);
    assertFalse(condition1.equals(condition2));
  }

  public void testEqualsWithDifferentValue() {
    String parameterName = "Nome do parmetro";
    Object conditionValue1 = "Valor condio 1";
    Object conditionValue2 = "Valor condio 2";
    SimpleCondition condition1 =
      new SimpleCondition(parameterName, conditionValue1);
    SimpleCondition condition2 =
      new SimpleCondition(parameterName, conditionValue2);
    assertFalse(condition1.equals(condition2));
  }

  public void testEqualsWithNullValueAndNotNullValue() {
    String parameterName = "Nome do parmetro";
    Object conditionValue1 = null;
    Object conditionValue2 = "Valor condio 2";
    SimpleCondition condition1 =
      new SimpleCondition(parameterName, conditionValue1);
    SimpleCondition condition2 =
      new SimpleCondition(parameterName, conditionValue2);
    assertFalse(condition1.equals(condition2));
  }

  public void testEqualsWithNullNotValueAndNullValue() {
    Object defaultValue = "Valor-padro";
    Parameter parameter = new MockSimpleParameter("Nome do parmetro",
      "Rtulo do parmtro", "Descrio do parmetro", defaultValue, false,
      "{0}{1}");
    Object conditionValue1 = "Valor condio 1";
    Object conditionValue2 = null;
    SimpleCondition condition1 =
      new SimpleCondition(parameter.getName(), conditionValue1);
    SimpleCondition condition2 =
      new SimpleCondition(parameter.getName(), conditionValue2);
    assertFalse(condition1.equals(condition2));
  }

  public void testEqualsWithNull() {
    SimpleCondition condition =
      new SimpleCondition("Nome do parmetro", "Valor condio");
    assertFalse(condition.equals(null));
  }

  public void testEqualsWithObjectOfAnotherClass() {
    SimpleCondition condition =
      new SimpleCondition("Nome do parmetro", "Valor condio");
    assertFalse(condition.equals("Objeto de outra classe"));
  }

  public void testHashCode() {
    String parameterName = "Nome do parmetro";
    Object conditionValue = "Valor condio";
    SimpleCondition condition1 =
      new SimpleCondition(parameterName, conditionValue);
    SimpleCondition condition2 =
      new SimpleCondition(parameterName, conditionValue);
    assertEquals(condition1.hashCode(), condition2.hashCode());
  }

  public void testEvaluate() {
    SimpleAlgorithmConfigurator configurator = createAlgorithmVersion();
    ParameterGroup group = new ParameterGroup("Nome do grupo");
    configurator.addGroup(group);
    Object defaultValue = "Valor-padro";
    SimpleParameter parameter = new MockSimpleParameter("Nome do parmetro",
      "Rtulo do parmtro", "Descrio do parmetro", defaultValue, false,
      "{0}{1}");
    group.addParameter(parameter);
    SimpleCondition condition =
      new SimpleCondition(parameter.getName(), defaultValue);
    assertTrue(condition.evaluate(configurator));
  }

  public void testEvaluateWithConfiguratorValueNotEqualsConditionValue() {
    SimpleAlgorithmConfigurator configurator = createAlgorithmVersion();
    ParameterGroup group = new ParameterGroup("Nome do grupo");
    configurator.addGroup(group);
    Object defaultValue = "Valor-padro";
    SimpleParameter parameter = new MockSimpleParameter("Nome do parmetro",
      "Rtulo do parmtro", "Descrio do parmetro", defaultValue, false,
      "{0}{1}");
    group.addParameter(parameter);
    Object conditionValue = "Outro valor";
    SimpleCondition condition =
      new SimpleCondition(parameter.getName(), conditionValue);
    assertFalse(condition.evaluate(configurator));
  }

  public void testEvaluateWithNullConfiguratorValueAndNullConditionValue() {
    SimpleAlgorithmConfigurator version = createAlgorithmVersion();
    ParameterGroup group = new ParameterGroup("Nome do grupo");
    version.addGroup(group);
    Object defaultValue = null;
    SimpleParameter parameter = new MockSimpleParameter("Nome do parmetro",
      "Rtulo do parmetro", "Descrio do parmetro", defaultValue, false,
      "{0}{1}");
    group.addParameter(parameter);
    SimpleCondition condition =
      new SimpleCondition(parameter.getName(), defaultValue);
    assertTrue(condition.evaluate(version));
  }

  public void testEvaluateWithNullParameterValueAndAConditionValue() {
    SimpleAlgorithmConfigurator configurator = createAlgorithmVersion();
    ParameterGroup group = new ParameterGroup("Nome do grupo");
    configurator.addGroup(group);
    Object defaultValue = null;
    SimpleParameter parameter = new MockSimpleParameter("Nome do parmetro",
      "Rtulo do parmetro", "Descrio do parmetro", defaultValue, false,
      "{0}{1}");
    group.addParameter(parameter);
    Object anotherValue = "Outro valor";
    SimpleCondition condition =
      new SimpleCondition(parameter.getName(), anotherValue);
    assertFalse(condition.evaluate(configurator));
  }

  public void testEvaluateWithAParameterValueAndNullConditionValue() {
    SimpleAlgorithmConfigurator configurator = createAlgorithmVersion();
    ParameterGroup group = new ParameterGroup("Nome do grupo");
    configurator.addGroup(group);
    Object defaultValue = "Valor-padro";
    SimpleParameter parameter = new MockSimpleParameter("Nome do parmetro",
      "Rtulo do parmetro", "Descrio do parmetro", defaultValue, false,
      "{0}{1}");
    group.addParameter(parameter);
    Object anotherValue = null;
    SimpleCondition condition =
      new SimpleCondition(parameter.getName(), anotherValue);
    assertFalse(condition.evaluate(configurator));
  }

  public void testEvaluateWithNull() {
    SimpleCondition condition =
      new SimpleCondition("Nome do parmetro", "Valor");
    try {
      condition.evaluate(null);
      fail("Esperada IllegalArgumentException");
    }
    catch (IllegalArgumentException e) {
    }
  }

  public void testSerialize()
  throws IOException, ClassNotFoundException {
    SimpleCondition outputCondition =
      new SimpleCondition("Nome do parmetro", "Valor");
    ByteArrayOutputStream outputSource = new ByteArrayOutputStream();
    ObjectOutputStream objectOutput = new ObjectOutputStream(outputSource);
    objectOutput.writeObject(outputCondition);
    ByteArrayInputStream inputSource =
      new ByteArrayInputStream(outputSource.toByteArray());
    ObjectInputStream objectInput = new ObjectInputStream(inputSource);
    SimpleCondition inputCondition = (SimpleCondition)objectInput.readObject();
    assertEquals(outputCondition, inputCondition);
    assertEquals(
      outputCondition.getParameterName(), inputCondition.getParameterName());
    assertEquals(outputCondition.getValue(), inputCondition.getValue());
  }

  private SimpleAlgorithmConfigurator createAlgorithmVersion() {
    AlgorithmInfo algorithmInfo =
      new AlgorithmInfo(
        "ID Algoritmo", "Algoritmo", "DIR Algoritmo", new Hashtable());
    AlgorithmVersionId algorithmVersionId = new AlgorithmVersionId(1, 2, 3);
    AlgorithmVersionInfo algorithmVersionInfo =
      new AlgorithmVersionInfo(
        algorithmInfo, algorithmVersionId, 
        new Hashtable<String, List<FileInfo>>(), 
        new Hashtable<String, String>());
    return new SimpleAlgorithmConfigurator(
      algorithmVersionInfo, null, ExecutionType.SIMPLE, null, false, "Comando",
      null, null, true);
  }
}
