/*
 * $Author:$
 * $Date:$
 * $Release:$
 */
package csbase.logic.algorithms.parameters.triggers;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Hashtable;
import java.util.List;

import junit.framework.TestCase;
import csbase.logic.FileInfo;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.logic.algorithms.ExecutionType;
import csbase.logic.algorithms.parameters.MockSimpleParameter;
import csbase.logic.algorithms.parameters.ParameterGroup;
import csbase.logic.algorithms.parameters.SimpleAlgorithmConfigurator;
import csbase.logic.algorithms.parameters.SimpleParameter;
import csbase.logic.algorithms.parameters.conditions.Condition;
import csbase.logic.algorithms.parameters.conditions.SimpleCondition;

/**
 * Testes unitrios para a classe {@link Trigger}.
 *
 * @author lmoreira
 */
public final class TriggerTest extends TestCase {
  /**
   * Testando criar um gatilho.
   */
  public void testTrigger1() {
    SimpleParameter<?> parameter = createMockParameter();
    Condition condition = new SimpleCondition(parameter.getName(), null);
    Trigger trigger = new MockTrigger(parameter, condition);
    assertEquals(parameter, trigger.getParameter());
    assertEquals(condition, trigger.getCondition());
  }

  /**
   * Testando criar um gatilho com parmetro {@code null}.
   */
  public void testTrigger2() {
    String parameterName = "Nome do parmetro";
    Condition condition = new SimpleCondition(parameterName, null);
    try {
      new MockTrigger(null, condition);
      fail(
        "No deveria ser possvel criar uma restrio com o parmetro nulo.");
    }
    catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando criar um gatilho com condio {@code null}.
   */
  public void testTriggerWithNullCondition() {
    SimpleParameter<?> parameter = createMockParameter();
    try {
      new MockTrigger(parameter, null);
      fail("No deveria ser possvel criar uma restrio com a condio nula.");
    }
    catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando o mtodo {@link Trigger#equals(Object)} com gatilhos com
   * parmetros iguais.
   */
  public void testEquals1() {
    String parameterName = "Nome do parmetro";
    Condition condition1 = new SimpleCondition(parameterName, "Valor 1");
    Condition condition2 = new SimpleCondition(parameterName, "Valor 2");
    SimpleParameter<?> parameter = createMockParameter();
    Trigger trigger1 = new MockTrigger(parameter, condition1);
    Trigger trigger2 = new MockTrigger(parameter, condition2);
    assertTrue(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link Trigger#equals(Object)} com parmetros diferentes.
   */
  public void testEquals2() {
    String parameterName1 = "Nome do parmetro 1";
    SimpleParameter<?> parameter1 = createMockParameter(parameterName1);
    String parameterName2 = "Nome do parmetro 2";
    SimpleParameter<?> parameter2 = createMockParameter(parameterName2);
    String parameterName3 = "Nome do parmetro 3";
    Condition condition = new SimpleCondition(parameterName3, null);
    Trigger trigger1 = new MockTrigger(parameter1, condition);
    Trigger trigger2 = new MockTrigger(parameter2, condition);
    assertFalse(trigger1.equals(trigger2));
  }


  /**
   * Testando o mtodo {@link Trigger#equals(Object)} com argumento 
   * {@code null}.
   */
  public void testEquals3() {
    SimpleParameter<?> parameter = createMockParameter();
    Condition condition1 = new SimpleCondition(parameter.getName(), "Valor 1");
    Trigger trigger1 = new MockTrigger(parameter, condition1);
    assertFalse(trigger1.equals(null));
  }

  /**
   * Testando o mtodo {@link Trigger#equals(Object)} com argumento 
   * de outra classe.
   */
  public void testEquals4() {
    SimpleParameter<?> parameter = createMockParameter();
    Condition condition1 = new SimpleCondition(parameter.getName(), "Valor 1");
    Trigger trigger1 = new MockTrigger(parameter, condition1);
    assertFalse(trigger1.equals("Um objeto de outra classe"));
  }

  /**
   * Testando o mtodo {@link Trigger#hashCode()}.
   */
  public void testHashCode() {
    String parameterName = "Nome do parmetro";
    SimpleParameter<?> parameter = createMockParameter(parameterName);
    String parameterName1 = "Nome do parmetro 1";
    Condition condition1 = new SimpleCondition(parameterName1, null);
    String parameterName2 = "Nome do parmetro 2";
    Condition condition2 = new SimpleCondition(parameterName2, null);
    Trigger trigger1 = new MockTrigger(parameter, condition1);
    Trigger trigger2 = new MockTrigger(parameter, condition2);
    assertEquals(trigger1.hashCode(), trigger2.hashCode());
  }

  /**
   * Testando o mtodo {@link Trigger#update(SimpleAlgorithmConfigurator)} com 
   * condio verdadeira.
   */
  public void testUpdate1() {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Nome do grupo");
    configurator.addGroup(group);
    Object defaultValue = "Valor-padro do parmetro";
    SimpleParameter<?> parameter = new MockSimpleParameter("Nome do parmetro",
      "Rtulo do parmtro", "Descrio do parmetro", defaultValue, false, null);
    group.addParameter(parameter);
    Condition condition =
      new SimpleCondition(parameter.getName(), defaultValue);
    CheckTrigger trigger = new CheckTrigger(parameter, condition);
    trigger.allowDo();
    assertTrue(trigger.update(configurator));
    assertTrue(trigger.wasDone());
  }

  /**
   * Testando o mtodo {@link Trigger#update(SimpleAlgorithmConfigurator)} com 
   * condio falsa.
   */
  public void testUpdate2() {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Nome do grupo");
    configurator.addGroup(group);
    Object defaultValue = "Valor-padro do parmetro";
    SimpleParameter<?> parameter = new MockSimpleParameter("Nome do parmetro",
      "Rtulo do parmtro", "Descrio do parmetro", defaultValue, false, null);
    group.addParameter(parameter);
    Object value = "Outro valor";
    Condition condition = new SimpleCondition(parameter.getName(), value);
    CheckTrigger trigger = new CheckTrigger(parameter, condition);
    trigger.allowUndo();
    assertFalse(trigger.update(configurator));
    assertTrue(trigger.wasUndone());
  }

  /**
   * Testa serializao.
   * 
   * @throws IOException Bug.
   * @throws ClassNotFoundException Bug.
   */
  public void testSerialize()
  throws IOException, ClassNotFoundException {
    String parameterName = "Nome do parmetro";
    SimpleParameter<?> parameter = createMockParameter(parameterName);
    Condition condition = new SimpleCondition(parameterName, null);
    Trigger outputTrigger = new MockTrigger(parameter, condition);
    ByteArrayOutputStream outputSource = new ByteArrayOutputStream();
    ObjectOutputStream objectOutput = new ObjectOutputStream(outputSource);
    objectOutput.writeObject(outputTrigger);
    ByteArrayInputStream inputSource =
      new ByteArrayInputStream(outputSource.toByteArray());
    ObjectInputStream objectInput = new ObjectInputStream(inputSource);
    Trigger inputTrigger = (Trigger)objectInput.readObject();
    assertEquals(outputTrigger, inputTrigger);
    assertEquals(outputTrigger.getCondition(), inputTrigger.getCondition());
    assertEquals(
      outputTrigger.getParameter(), inputTrigger.getParameter());
  }

  /**
   * Cria um configurador de algoritmos qualquer.
   * 
   * @return .
   */
  private SimpleAlgorithmConfigurator createAlgorithmConfigurator() {
    AlgorithmInfo algorithmInfo =
      new AlgorithmInfo(
        "ID Algoritmo", "Algoritmo", "DIR Algoritmo", new Hashtable<String, String>());
    AlgorithmVersionId algorithmVersionId = new AlgorithmVersionId(1, 2, 3);
    AlgorithmVersionInfo algorithmVersionInfo =
      new AlgorithmVersionInfo(
      algorithmInfo, algorithmVersionId,
      new Hashtable<String, List<FileInfo>>(),
      new Hashtable<String, String>());
    return new SimpleAlgorithmConfigurator(
      algorithmVersionInfo, null, ExecutionType.SIMPLE, null, false, "Comando",
      null, null, true);
  }
  
  /**
   * Cria um parmetro qualquer.
   * 
   * @return .
   */
  private SimpleParameter<?> createMockParameter() {
    String parameterName = "Nome do parmetro";
    return createMockParameter(parameterName);
  }

  /**
   * Cria um parmetro com o nome especificado.
   * 
   * @param parameterName O nome (No aceita {@code null}).
   * 
   * @return .
   */
  private SimpleParameter<?> createMockParameter(String parameterName) {
    return new MockSimpleParameter(parameterName, "Rtulo do parmetro",
      "Descrio do parmetro", "Valor-padro do parmetro", false, null);
  }
}
