package csbase.tools;

import java.lang.reflect.Method;

/**
 * Gerador de proxies de servio do csbase.
 *
 * @author Jorge Alexandre & Ana Moura
 */
public class NewProxyGenerator extends Generator {
  private String servicePermissionClassName;

  /**
   * Mtodo principal que ser executado quando o gerador for chamado da linha
   * de comando.
   * 
   * @param args args[0]: nome da interface remota do servio (com pacote);
   *        args[1]: nome da classe que implementa o servio (com pacote);
   *        args[2](opcional): nome da classe que representa a permisso do
   *        servio que ser verificada em cada mtodo do proxy gerado.
   * 
   * @throws Exception
   */
  public static void main(String[] args)
  throws Exception {
    if (
      args.length < 2 || args.length > 3 || getPackageName(args[0]) == null ||
      getPackageName(args[1]) == null) {
      usage();
      System.exit(1);
    }
    NewProxyGenerator proxygen = null;
    if (args.length == 2) {
      proxygen = new NewProxyGenerator(args[0], args[1]);
    }
    if (args.length == 3) {
      if (getPackageName(args[2]) == null) {
        usage();
        System.exit(1);
      }
      proxygen = new NewProxyGenerator(args[0], args[1], args[2]);
    }
    proxygen.generate();
    proxygen.flushToFile(proxygen.getBuffer());
    System.out.println("Proxy "+proxygen.genClassName+" gerado");
  }

  /**
   * Exibe mensagem explicando como usar o gerador.
   */
  protected static void usage() {
    Generator.usage();
    System.out.println(
      "\nargs[2](opcional): nome da classe que representa a permisso "+
      "do servio");
    System.out.println(
      " ex: websintesi.logic.permission.exe.VGEExecutionPermission");
  }

  /**
   * Contri uma instncia do gerador de proxies.
   *
   * @param interfaceName nome completo da interface remota do servio
   * @param serviceClassName nome completo da classe que implementa o servio
   */
  protected NewProxyGenerator(String interfaceName, String serviceClassName) {
    this(interfaceName, serviceClassName, null);
  }

  /**
   * Contri uma instncia do gerador de proxies.
   *
   * @param interfaceName nome completo da interface remota do servio
   * @param serviceClassName nome completo da classe que implementa o servio
   * @param servicePermissionClassName nome completo da classe que representa a
   *        permisso do servio.
   */
  protected NewProxyGenerator(
    String interfaceName, String serviceClassName,
    String servicePermissionClassName) {
    super(interfaceName, serviceClassName, "Proxy");
    this.servicePermissionClassName = servicePermissionClassName;
  }

  /**
   * Indica se o construtor da classe gerada invoca o construtor da super
   * classe.
   *
   * @return true se o construtor da super classe  invocado false caso
   *         contrrio
   */
  @Override
  protected boolean invokesSuperClassConstructor() {
    return true;
  }

  /**
   * Acrescenta as linhas de cdigo para o construtor do proxy.
   *
   * @param code buffer que armazena o cdigo
   * @param paremeterNames nomes dos parmetros passados pelo construtor.
   */
  @Override
  protected void getConstructorLines(
    StringBuffer code, String[] paremeterNames) {
  }

  /**
   * Indica se lana as excees relacionadas na interface.
   *
   * @return true se as excees so lanadas false caso contrrio
   */
  @Override
  protected boolean throwsInterfaceExceptions() {
    return true;
  }

  /**
   * Acrescenta ao cdigo a implementao de um mtodo.
   *
   * @param code buffer que armazena o cdigo
   * @param method descrio do mtodo a implementar
   * @param parameterNames nomes dos parmetros do mtodo
   */
  @Override
  protected void getMethodLines(
    StringBuffer code, Method method, String[] parameterNames) {
    String returnLine =
      getNoPackagesClassName(serviceClassName)+".getInstance()."+
      method.getName()+"(";
    for (int i = 0; i < parameterNames.length; i++) {
      if (i > 0) {
        returnLine += ", ";
      }
      returnLine += parameterNames[i];
    }
    returnLine += ");\n";
    if (!"void".equals(method.getReturnType().getName())) {
      returnLine = "return "+returnLine;
    }
    code.append("  try {\n");
    code.append("    Service.setKey(this.key);\n");
    if (servicePermissionClassName != null) {
      code.append("    User user = Service.getUser();\n");
      code.append("    Permission p =\n");
      code.append("      user.getPermission(");
      code.append(servicePermissionClassName+".class);\n");
      code.append("    if (p == null) {\n");
      code.append(
        "      LanguageService lngService = LanguageService.getInstance();\n");
      code.append("      throw new PermissionException(");
      code.append("lngService.get(\"permission.denied\"));\n");
      code.append("    }\n");
    }
    code.append("    "+returnLine);
    code.append("  } catch (ServiceFailureException fe) {\n");
    code.append("    logException(SERVICE_NAME,fe);\n");
    code.append(
      "    throw new ServiceFailureException(fe.getClientMessage());\n");
    code.append("  } catch (CSBaseRuntimeException re) {\n");
    code.append("    throw re;\n");
    code.append("  } catch (Throwable t) {\n");
    code.append("    logException(SERVICE_NAME,t);\n");
    code.append("    throw new BugException();\n");
    code.append("  }\n");
  }

  /**
   * Obtm a super classe do proxy.
   *
   * @return a classe que o proxy ir extender.
   */
  @Override
  protected Class getSuperClass() {
    return csbase.server.ServiceProxy.class;
  }

  /**
   * Obtm o nome da super classe da classe gerada.
   *
   * @return o nome da super classe que o classe gerada ir estender.
   */
  @Override
  protected String getSuperClassName() {
    return getSuperClass().getName();
  }

  /**
   * Obtm os pacotes a serem importados pelo proxy.
   *
   * @return um array com strings que representam os pacotes a serem
   *         importados.
   */
  @Override
  protected String[] getImports() {
    if (servicePermissionClassName == null) {
      String[] imports =
        {
          "csbase.server.Service", "csbase.exception.*"
        };
      return imports;
    }
    String[] imports =
      {
        "csbase.server.Service",
        "csbase.server.services.languageservice.LanguageService",
        "csbase.exception.*", "csbase.logic.User", "csbase.logic.Permission"
      };
    return imports;
  }
}
