/**
 * $Id: StringUtils.java 83049 2008-09-18 14:58:20Z costa $
 */
package csbase.util;

import java.text.CollationKey;
import java.text.Collator;
import java.util.Arrays;
import java.util.Comparator;

// TODO migrar para o JavaUtils?

/**
 * Mtodos utilitrios para manipulao e comparao de strings.
 * 
 * @author Tecgraf
 */
public class StringUtils {
  /**
   * Comparador de strings que no diferencia minsculas de maisculas.
   * 
   * @see #getPlainSortCollator()
   */
  private static Comparator<String> plainSortComparator;
  /**
   * {@link Collator} que no diferencia minsculas de maisculas. Este objeto 
   * um clone do <code>collator</code> default.
   */
  private static Collator plainSortCollator;

  /**
   * Retorna um {@link Collator} para comparao de strings que no diferencia
   * minsculas de maisculas, i.e. {ZZZ, aaa}  ordenado como {aaa, ZZZ}.
   * 
   * OBS.: o objeto retornado  um singleton
   * 
   * IMPORTANTE: se o collator for usado em mltiplas comparaes das mesmas
   * strings (p.ex. ordenao de longas listas), considerar o uso de
   * {@link CollationKey}. Referncia:
   * <code>http://java.sun.com/docs/books/tutorial/i18n/text/perform.html</code>
   * 
   * @return <code>Collator</code> para comparao de strings que no diferencia
   *         minsculas de maisculas
   */
  public static Collator getPlainSortCollator() {
    if (plainSortCollator == null) {
      /*
       * clonamos para garantir que no h efeitos colaterais
       */
      plainSortCollator = (Collator) Collator.getInstance().clone();
      plainSortCollator.setStrength(Collator.TERTIARY);
    }
    return plainSortCollator;
  }

  /**
   * Retorna um <code>Comparator&lt;String&gt;</code> que no diferencia
   * minsculas de maisculas.
   * 
   * OBS.: o objeto retornado  um <i>singleton</i>
   * 
   * @return <code>Comparator&lt;String&gt;</code> que no diferencia minsculas
   *         de maisculas
   */
  public static Comparator<String> getPlainSortComparator() {
    if (plainSortComparator == null) {
      plainSortComparator = new Comparator<String>() {
        @Override
        public int compare(String o1, String o2) {
          return StringUtils.compare(o1, o2);
        }
      };
    }
    return plainSortComparator;
  }

  /**
   * Compara alfabeticamente duas strings, sem diferenciar minsculas de
   * maisculas.
   * 
   * @see Collator#compare(String, String)
   * 
   * @param s1 - primeira string
   * @param s2 - segunda string
   * @return -1 se s1 < s2, 0 se s1 == s2 e 1 se s1 > s2
   */
  public static int compare(String s1, String s2) {
    return getPlainSortCollator().compare(s1, s2);
  }

  /**
   * Ordena alfabeticamente um array de strings, sem diferenciar minsculas de
   * maisculas. A ordenao acontece <i>in place</i>, i.e. o prprio array 
   * modificado e retornado como resultado.
   * 
   * @param array - array a ser ordenado
   * @return o array fornecido como entrada, ordenado
   */
  public static String[] sort(String[] array) {
    Arrays.sort(array, getPlainSortComparator());
    return array;
  }
}
