package csbase.util.collections;

import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.NoSuchElementException;
import java.util.Set;

/**
 * <p>
 * Representa uma coleo que no contm elementos duplicados e no contm
 * elementos nulos (null).
 * </p>
 * 
 * <p>
 * A diferena desta coleo para um {@link Set}  que ao inserirmos elementos
 * que j faam parte da coleo, eles so colocados no final da coleo e
 * ser informado que houve mudana na coleo. No caso do Set, a coleo
 * permaneceria sem modificaes.
 * </p>
 * 
 * <p>
 * Todos os elementos devem ser clonveis, ou o mtodo {@link #clone()}
 * levantar uma {@link CloneNotSupportedException}.
 * </p>
 *
 * @author Rodrigo Carneiro Henrique (rodrigoh)
 */
public final class NoDuplicatesCollection implements Collection, Cloneable {
  /** A lista que armazenar os elementos da coleo. */
  private LinkedList linkedList;

  /**
   * Cria uma coleo vazia.
   */
  public NoDuplicatesCollection() {
    this.linkedList = new LinkedList();
  }

  /**
   * Cria uma coleo contendo os elementos da coleo recebida. Todos os
   * elementos duplicados sero removidos.
   *
   * @param c A coleo com os elementos a serem inseridos.
   *
   * @throws IllegalArgumentException Caso a coleo passada como parmetro
   *         seja nula (null).
   */
  public NoDuplicatesCollection(Collection c) {
    if (c == null) {
      throw new IllegalArgumentException(
        "A coleo recebida no pode ser nula.");
    }
    this.linkedList = new LinkedList();
    this.addAll(c);
  }

  public int size() {
    return this.linkedList.size();
  }

  public void clear() {
    this.linkedList.clear();
  }

  public boolean isEmpty() {
    return this.linkedList.isEmpty();
  }

  public Object[] toArray() {
    return this.linkedList.toArray();
  }

  /**
   * Retorna o primeiro elemento da coleo.
   *
   * @return O primeiro elemento da coleo.
   *
   * @throws NoSuchElementException Caso a coleo esteja vazia.
   */
  public Object getFirst()
  throws NoSuchElementException {
    return this.linkedList.getFirst();
  }

  /**
   * Retorna o ltimo elemento da coleo.
   *
   * @return O ltimo elemento da coleo.
   *
   * @throws NoSuchElementException Caso a coleo esteja vazia.
   */
  public Object getLast()
  throws NoSuchElementException {
    return this.linkedList.getLast();
  }

  /**
   * Insere o elemento na ltima posio da coleo.
   * 
   * <ul>
   * <li>
   * Caso o elemento no esteja na coleo, ele  inserido na ltima posio.
   * </li>
   * <li>
   * Caso o elemento j esteja na coleo e esteja na ltima posio, nada 
   * feito.
   * </li>
   * <li>
   * Caso o elemento j esteja na coleo e no esteja na ltima posio, ser
   * movido para a ltima posio.
   * </li>
   * </ul>
   * 
   *
   * @param o O objeto a ser inserido na coleo.
   *
   * @return true, se a coleo foi alterada, ou false, caso contrrio.
   *
   * @throws IllegalArgumentException Caso o elemento recebido seja nulo.
   *
   * @see java.util.Collection#add(java.lang.Object)
   */
  public boolean add(Object o) {
    if (o == null) {
      throw new IllegalArgumentException(
        "Essa coleo no aceita elementos nulos.");
    }
    if (this.linkedList.contains(o)) {
      if (this.linkedList.indexOf(o) == (this.linkedList.size()-1)) {
        return false;
      }
      this.linkedList.remove(o);
    }
    return this.linkedList.add(o);
  }

  public boolean contains(Object o) {
    return this.linkedList.contains(o);
  }

  public boolean remove(Object o) {
    return this.linkedList.remove(o);
  }

  /**
   * Insere todos os elementos da coleo recebida. Possui o mesmo
   * funcionamento do mtodo {@link #add(Object)}. Conforme os elementos vo
   * aparecendo na coleo de origem, eles vo sendo inseridos ao final da
   * coleo destino (a instncia atual). Elementos repetidos so movidos para
   * o final.
   *
   * @param c A coleo de origem.
   *
   * @return true, se a coleo foi alterada, ou false, caso contrrio.
   *
   * @throws IllegalArgumentException Caso a coleo passada como parmetro
   *         seja nula (null).
   *
   * @see java.util.Collection#addAll(java.util.Collection)
   */
  public boolean addAll(Collection c) {
    if (c == null) {
      throw new IllegalArgumentException(
        "A coleo recebida no pode ser nula.");
    }
    boolean wasChanged = false;
    Iterator collectionIterator = c.iterator();
    while (collectionIterator.hasNext()) {
      if (this.add(collectionIterator.next()) == true) {
        wasChanged = true;
      }
    }
    return wasChanged;
  }

  public boolean containsAll(Collection c) {
    return this.linkedList.containsAll(c);
  }

  public boolean removeAll(Collection c) {
    return this.linkedList.removeAll(c);
  }

  /**
   * Mtodo no implementado.
   *
   * @see java.util.Collection#retainAll(java.util.Collection)
   */
  public boolean retainAll(Collection c) {
    throw new UnsupportedOperationException();
  }

  public Iterator iterator() {
    return this.linkedList.iterator();
  }

  public Object[] toArray(Object[] a) {
    return this.linkedList.toArray(a);
  }

  public Object clone() {
    try {
      NoDuplicatesCollection clone = (NoDuplicatesCollection)super.clone();
      clone.linkedList = (LinkedList)this.linkedList.clone();
      return clone;
    }
    catch (CloneNotSupportedException e) {
      e.printStackTrace();
      IllegalStateException illegalStateException =
        new IllegalStateException(
          "Algum elemento da coleo est errado. Todos os elementos deveriam "+
          "suportar clone.");
      illegalStateException.initCause(e);
      throw illegalStateException;
    }
  }

  public boolean equals(Object obj) {
    if (obj == null) {
      return false;
    }
    if (!this.getClass().equals(obj.getClass())) {
      return false;
    }
    NoDuplicatesCollection other = (NoDuplicatesCollection)obj;
    if (this.size() != other.size()) {
      return false;
    }
    Iterator iterator = this.iterator();
    Iterator otherIterator = other.iterator();
    while (iterator.hasNext()) {
      if (!iterator.next().equals(otherIterator.next())) {
        return false;
      }
    }
    return true;
  }

  public int hashCode() {
    return this.linkedList.hashCode();
  }

  public String toString() {
    return this.linkedList.toString();
  }
}
