package csbase.util.parser.xml;


import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import java.io.IOException;
import java.io.Reader;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.Stack;

import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

public abstract class XmlParser extends DefaultHandler {
  private static final String[] TRUE_VALUES =
    new String[] { "sim", "verdadeiro", "v", "s", Boolean.toString(true) };
  private static final String[] FALSE_VALUES =
    new String[] { "no", "nao", "falso", "n", "f", Boolean.toString(false) };
  private Properties specificProperties;
  private Properties genericProperties;
  private Stack stack;
  private Object result;

  protected XmlParser(Properties properties) {
    this.stack = new Stack();
    this.genericProperties = (Properties)properties.clone();
  }

  public static final Boolean getBoolean(
    String elementName, Map attributeMap, String attributeName,
    boolean isOptional)
  throws XmlParserException {
    String text =
      getString(elementName, attributeMap, attributeName, isOptional);
    if (text == null) {
      return null;
    }
    if (exists(TRUE_VALUES, text)) {
      return Boolean.TRUE;
    }
    if (exists(FALSE_VALUES, text)) {
      return Boolean.FALSE;
    }
    throw new XmlParserException(
      "xml.exception.boolean_format_error",
      new Object[] { elementName, attributeName, text });
  }

  public static final Double getDouble(
    String elementName, Map attributeMap, String attributeName,
    boolean isOptional)
  throws XmlParserException {
    String text =
      getString(elementName, attributeMap, attributeName, isOptional);
    try {
      if (text != null) {
        return new Double(text);
      }
      return null;
    }
    catch (NumberFormatException exception) {
      throw new XmlParserException(
        "xml.exception.double_format_error",
        new Object[] { elementName, attributeName, text }, exception);
    }
  }

  public static final Integer getInteger(
    String elementName, Map attributeMap, String attributeName,
    boolean isOptional)
  throws XmlParserException {
    String text =
      getString(elementName, attributeMap, attributeName, isOptional);
    try {
      if (text != null) {
        return new Integer(text);
      }
      return null;
    }
    catch (NumberFormatException exception) {
      throw new XmlParserException(
        "xml.exception.integer_format_error",
        new Object[] { elementName, attributeName, text }, exception);
    }
  }

  public static final String getString(
    String elementName, Map attributeMap, String attributeName,
    boolean isOptional)
  throws XmlParserException {
    Object value = attributeMap.remove(attributeName);
    if (value == null) {
      if (!isOptional) {
        throw new XmlParserException(
          "xml.exception.value_not_found",
          new Object[] { elementName, attributeName });
      }
    }
    if (value != null) {
      return value.toString().trim();
    }
    return null;
  }

  public static final String[] getStringArray(
    String elementName, Map attributeMap, String attributeName,
    boolean isOptional)
  throws XmlParserException {
    String arrayText =
      getString(elementName, attributeMap, attributeName, isOptional);
    if (arrayText == null) {
      return null;
    }
    String[] texts = arrayText.split(",");
    for (int i = 0; i < texts.length; i++) {
      texts[i] = texts[i].trim();
      if (texts[i].length() == 0) {
        throw new XmlParserException(
          "xml.exception.string_array_format_error",
          new Object[] { elementName, attributeName, arrayText });
      }
    }
    return texts;
  }

  public final void characters(char[] ch, int start, int length)
  throws SAXException {
    if (this.stack.isEmpty()) {
      return;
    }
    Object peek = this.stack.peek();
    Class peekClass = peek.getClass();
    Method methodSetData;
    try {
      methodSetData =
        peekClass.getMethod("setData", new Class[] { String.class });
    }
    catch (SecurityException e) {
      throw new SAXException(e);
    }
    catch (NoSuchMethodException e) {
      return;
    }
    String data = "";
    int end = start+length-1;
    for (int i = start; i <= end; i++) {
      data += ch[i];
    }
    data = data.trim();
    if (data.length() == 0) {
      return;
    }
    try {
      methodSetData.invoke(peek, new Object[] { data });
    }
    catch (InvocationTargetException e) {
      throw new SAXException(new Exception(e.getTargetException()));
    }
    catch (Exception exception) {
      throw new SAXException(exception);
    }
  }

  public final void endElement(String uri, String localName, String qName) {
    this.result = this.stack.pop();
  }

  public final void startElement(
    String uri, String localName, String qName, Attributes attributes)
  throws SAXException {
    try {
      Map attributeMap = new HashMap();
      for (int i = 0; i < attributes.getLength(); i++) {
        attributeMap.put(attributes.getQName(i), attributes.getValue(i));
      }
      String className = (String)this.specificProperties.get(qName);
      if (className == null) {
        throw new XmlParserException(
          "xml.exception.element_not_found", new Object[] { qName });
      }
      className = className.trim();
      Class configuratorClass;
      try {
        configuratorClass = Class.forName(className);
      }
      catch (ClassNotFoundException e) {
        throw new XmlParserException(
          "xml.exception.class_not_found", new Object[] { qName, className }, e);
      }
      createElement(qName, attributeMap, configuratorClass);
      if (!attributeMap.isEmpty()) {
        throw new XmlParserException(
          "xml.exception.attributes_not_processed",
          new Object[] { qName, attributeMap });
      }
    }
    catch (InvocationTargetException e) {
      throw new SAXException(new Exception(e.getTargetException()));
    }
    catch (SecurityException e) {
      throw new SAXException(e);
    }
    catch (IllegalArgumentException e) {
      throw new SAXException(e);
    }
    catch (XmlParserException e) {
      throw new SAXException(e);
    }
    catch (InstantiationException e) {
      throw new SAXException(e);
    }
    catch (IllegalAccessException e) {
      throw new SAXException(e);
    }
  }

  protected final Object createElement(
    Reader reader, Properties properties, Object data)
  throws XmlParserException {
    try {
      this.stack.clear();
      this.stack.push(data);
      this.specificProperties = new Properties();
      this.specificProperties.putAll(this.genericProperties);
      this.specificProperties.putAll(properties);
      createParser().parse(new InputSource(reader), this);
      if (this.stack.size() != 1) {
        throw new XmlParserException("xml.exception.element_not_closed");
      }
      this.stack.pop();
      if (this.result == null) {
        throw new XmlParserException("xml.exception.no_elements_found");
      }
      return this.result;
    }
    catch (SAXException e) {
      if (e.getException() == null) {
        throw new XmlParserException("xml.exception.sytanx_error", e);
      }
      throw new XmlParserException("xml.exception.sytanx_error", e.getException());
    }
    catch (IOException e) {
      throw new XmlParserException("xml.exception.io_error", e);
    }
    catch (FactoryConfigurationError e) {
      throw new XmlParserException("xml.exception.factory_config_error", e);
    }
    catch (ParserConfigurationException e) {
      throw new XmlParserException("xml.exception.config_error", e);
    }
    finally {
      this.result = null;
      this.stack.clear();
    }
  }

  private static boolean exists(String[] texts, String text) {
    for (int i = 0; i < texts.length; i++) {
      if (texts[i].equalsIgnoreCase(text)) {
        return true;
      }
    }
    return false;
  }

  private boolean areAssignableFrom(Class[] lValues, Class[] rValues) {
    if (rValues.length != lValues.length) {
      return false;
    }
    for (int j = 0; j < lValues.length; j++) {
      if (!lValues[j].isAssignableFrom(rValues[j])) {
        return false;
      }
    }
    return true;
  }

  private void createElement(
    String elementName, Map attributeMap, Class elementClass)
  throws XmlParserException, SecurityException, InstantiationException, 
    IllegalAccessException, IllegalArgumentException, InvocationTargetException {
    Object parentElement = this.stack.peek();
    Class[] paramTypes;
    if (attributeMap.isEmpty()) {
      paramTypes = new Class[] { parentElement.getClass(), String.class };
    }
    else {
      paramTypes =
        new Class[] { parentElement.getClass(), String.class, Map.class };
    }
    Constructor constructor;
    Object[] params;
    Object newElement;
    constructor = getConstructor(elementClass, paramTypes);
    if (constructor != null) {
      if (attributeMap.isEmpty()) {
        params = new Object[] { parentElement, elementName };
      }
      else {
        params = new Object[] { parentElement, elementName, attributeMap };
      }
    }
    else {
      constructor =
        getConstructor(elementClass, new Class[] { parentElement.getClass() });
      if (constructor == null) {
        throw new XmlParserException(
          "xml.exception.no_constructor_found",
          new Object[] { elementName, elementClass.getName() });
      }
      params = new Object[] { parentElement };
    }
    newElement = constructor.newInstance(params);
    this.stack.push(newElement);
  }

  private SAXParser createParser()
  throws FactoryConfigurationError, ParserConfigurationException, SAXException {
    SAXParserFactory factory = SAXParserFactory.newInstance();
    SAXParser parser = factory.newSAXParser();
    return parser;
  }

  private Constructor getConstructor(Class type, Class[] paramsType) {
    Constructor[] constructors = type.getConstructors();
    for (int i = 0; i < constructors.length; i++) {
      Constructor constructor = constructors[i];
      if (areAssignableFrom(constructor.getParameterTypes(), paramsType)) {
        return constructors[i];
      }
    }
    return null;
  }
}
