package csgrid.client.applications.dynamicimageviewer;

import csbase.client.applications.ApplicationFrame;
import csbase.client.applications.ApplicationProject;
import csbase.logic.ClientProjectFile;

import csbase.client.applications.ApplicationExitAction;
import csbase.client.applications.ApplicationAboutAction;
import csbase.client.applications.ApplicationImages;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.LayoutManager;
import java.awt.MediaTracker;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.AffineTransform;
import java.awt.image.ImageObserver;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.net.Socket;
//import java.nio.ByteBuffer; //pensar em usar NIO
import java.util.Properties;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JToolBar;

/**
 * A classe <code>DynamicImageViewer</code> implementa a aplicao de exibio de
 * imagens dinamicas em um projeto. Atualmente, o objetivo dessa aplicacao eh a 
 * visualizacao de um "Arquivo 3D" (VIS) gerado pelo algoritmo que conhece, previamente,
 * essa aplicacao.
 * 
 * No momento que o usuario especifica um arquivo, uma conexao via socket eh estabelecida
 * com o servidor e porta especificados no arquivo de configuracao. Em seguida, o caminho
 * absoluto do arquivo a ser visualizado eh enviado e o sistema aguarda a chegada das
 * imagens. Cada imagem eh recebida por uma unidade atomica constituida de uma linha com
 * o tamanho da imagem, seguida pelos bytes da imagem. O formato utilizado deve ser
 * compativel com o metodo JAVA createImage() na classe ToolKit. 
 * 
 * @author Marcelo Nery (inspirado no ImageViewer).
 */
public class DynamicImageViewer extends ApplicationProject {

  /** Painel onde ser exibida a imagem. */
  private ImgPanel imagePanel = null;
  private JScrollPane imageScrolledPanel = null;
  private Image img = null;

  /** A opcao de abrir arquivos eh desabilitada depois que o primeiro arquivo eh aberto */
  private JButton openButton; // usado na ToolBar
  private JMenuItem openItem; // usado no Menu

  /** Dilogo principal do visualizador. */
  private ApplicationFrame mainFrame = null;

  /** O numero de quadros por segundo. */
  final private JTextField fpsValue = new JTextField();

  /** A mensagem relativa ao FPS. */
  final private JLabel fpsMessage = new JLabel();
  
  /** Largura da figura atual */
  private int width = 0;

  /** Altura da figura atual */
  private int height = 0;

  /** Observador utilizado na formao da imagem */
  private ImgObserver observer = new ImgObserver();

  /** Thread para recuperacao de dados da rede */
  private ImageSocketThread socketThread;

  /**
   * Constri e exibe o dilogo de execuo de algoritmos
   * @param id .
   */
  public DynamicImageViewer(final String id) {
    super(id);
    createMainFrame();
  }
  
  /**
   * Cria a frame da aplicao
   *
   * @return referncia para a janela criada
   */
  private void createMainFrame() {
    final ApplicationFrame mainFrame = getApplicationFrame();
    mainFrame.setJMenuBar(createMenuBar());
    mainFrame.setSize(new Dimension(800, 600));
    Container cp = mainFrame.getContentPane();
    cp.add(createToolBar(), BorderLayout.NORTH);
    cp.add(createViewer(), BorderLayout.CENTER);
    cp.add(createFPSPanel(), BorderLayout.SOUTH);
  }
  
  /**
   * Mtodo para escolha de um arquivo.
   * @param mode .
   * @return .
   */
  private ClientProjectFile chooseFileOpen() {
    try {
      ClientProjectFile newFile = browseFileOpen("VIS");
      return newFile;
    }
    catch (Exception e) {
      showError(null, e.getMessage());
      return null;
    }
  }

  /**
   * Acao chamada pelo usuario para abrir um arquivo.
   */
  private synchronized void actionOpen() {
    ClientProjectFile newFile = chooseFileOpen();
    if (newFile!=null) {
    	openFile(newFile);
    }
  }
  
  /**
   * Desabilita os botoes de abertura de arquivo e inicializa a conexao com
   * o servidor de imagens.
   * 
   * @param file o arquivo a ser aberto, cujo caminho absoluto eh enviado pelo socket.
   */
  private void openFile(ClientProjectFile file) {
    if (file.isDirectory()) {
        showError(getApplicationFrame(), getString("error.open_directory"));
    }
    else {
	  	openButton.setEnabled(false);
	    openItem.setEnabled(false);
	    initConnection(file.getAbsolutePath());
    }
  }
  
  /**
   * Cria o painel de visualizaao de imagens.
   * @return painel de visualizaao de imagens
   */
  private JComponent createViewer() {
    imagePanel = new ImgPanel();
    imageScrolledPanel = new JScrollPane(imagePanel);
    imageScrolledPanel.setVerticalScrollBarPolicy(
      JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
    imageScrolledPanel.setHorizontalScrollBarPolicy(
      JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    return imageScrolledPanel;
  }

  /**
   * Cria o painel de exibicao de FPS (Quadros por Segundo da exibicao)
   * @return o painel de FPS
   */
  private JPanel createFPSPanel() {
    JPanel panel = new JPanel();
    LayoutManager layout = panel.getLayout();
    if (layout!=null && layout instanceof FlowLayout) {
    	((FlowLayout)layout).setAlignment(FlowLayout.LEFT);	
    }
    JLabel fpsText = new JLabel(getString("label.fps")+":");
	panel.add(fpsText);
	Dimension d = fpsText.getPreferredSize();
	// o numero de casas decimais, assim como a largura desse textField estao hardcoded.
	fpsValue.setPreferredSize(new Dimension(80,(int)(1.2D*d.height)));
	fpsValue.setEditable(false);
	panel.add(fpsValue);
	fpsMessage.setText(getString("loadingModel.message"));
	fpsMessage.setVisible(false);
	panel.add(fpsMessage);
	return panel;
  }

  /**
   * Montagem da toolbar.
   *
   * @return o painel.
   */
  private JToolBar createToolBar() {
    final JToolBar toolBar = new JToolBar(getName());
//    final JButton openButton = new JButton(ApplicationImages.ICON_OPEN_16);
    openButton = new JButton(ApplicationImages.ICON_OPEN_16);
    openButton.setToolTipText(getString("tooltip.file_open"));
    openButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          actionOpen();
        }
    });
    toolBar.add(openButton);
    return toolBar;
  }

  /**
   * Montagem do menu de arquivos.
   * @return o menu.
   */
  private JMenu createAboutMenu() {
    final JMenu aboutMenu = new JMenu(getString("menu.help"));
    aboutMenu.add(new ApplicationAboutAction(this));
    return aboutMenu;
  }

  /**
   * Montagem do menu de arquivos.
   * @return o menu.
   */
  private JMenu createFileMenu() {
    final JMenu fileMenu = new JMenu(getString("menu.file"));
//    final JMenuItem openItem = fileMenu.add(getString("menu.file.open"));
    openItem = fileMenu.add(getString("menu.file.open"));
    openItem.addActionListener( new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          actionOpen();
        }
    });
    openItem.setIcon(ApplicationImages.ICON_OPEN_16);
    fileMenu.addSeparator();
    fileMenu.add(new ApplicationExitAction(this));
    return fileMenu;
  }

  /**
   * Montagem do menu.
   * @return .
   */
  private JMenuBar createMenuBar() {
    final JMenuBar menuBar = new JMenuBar();
    menuBar.add(createFileMenu());
    menuBar.add(new JPanel());
    menuBar.add(createAboutMenu());
    return menuBar;
  }

  /**
   * Trmino ASAP da aplicao. Finaliza a conexao o que gera o termino da thread de
   * obtencao de imagens.
   */
  public void killApplication() {
  	finishConnection();
  }

  /**
   * Mtodo que permite sempre ao usurio matar a aplicao.
   * @return o flag indicativo.
   */
  public boolean userCanKillApplication() {
    return true;
  }

  /**
   * Recebe o arquivo para ser aberto diretamente (duplo clique na arvore de projetos)
   * @param name .
   * @param value .
   */
  public void sendMessage(String name, Object value) {
  	if (value == null) return;
    if (name.equals(PROJECT_FILE_MESSAGE)) {
       ClientProjectFile file = (ClientProjectFile)value;
       openFile(file);
    }
  }

  /**
   * Classe que implementa o observador utilizado na formao da imagem.
   */
  class ImgObserver implements ImageObserver {
    public boolean imageUpdate(Image i, int f, int x, int y, int w, int h) {
        if ((f & ALLBITS) == ALLBITS) {
        	return false;
        }
    	return true; 
    }
  }

  /**
   * Painel de Exibio de imagens
   */
  class ImgPanel extends JPanel {
    public void paintComponent(Graphics g) {
      super.paintComponent(g);
      if (img!=null) {
	      Graphics2D g2d = (Graphics2D)g;
	      //Centralizando a imagem
	      int dx = (imagePanel.getWidth()-width)/2;
	      int dy = (imagePanel.getHeight()-height)/2;
	      if (dx < 0) {
	        dx = 0;
	      }
	      if (dy < 0) {
	        dy = 0;
	      }
	      g2d.drawImage(img, AffineTransform.getTranslateInstance(dx, dy), observer);
      }
    }
  }

  /**
   * Inicializa a comunicacao com o servidor de imagens.
   * A principio nao eh possivel existir uma thread antiga, pois esse metodo eh chamado
   * apenas uma vez no uso atual dessa aplicacao. Porem, a implementacao eh capaz de 
   * tratar uma nova conexao, liberando a conexao antiga, caso exista, e reinicializando
   * as variaveis.
   * 
   * @param path o caminho absoluto do arquivo a ser enviado no inicio da comunicacao
   */
	public void initConnection(String path) {
		//para testes locais:
		//path = "/home/prj/csbase/workspace/csgrid/project/fabraham/visstable/picktest.vis";
		String host = getStringSpecificProperty("image_server_host");
		int port = getIntSpecificProperty("image_server_port", 5559);
 		if (socketThread!=null) {
 			finishConnection();
 		}
 		try {
    		socketThread = new ImageSocketThread();
     		socketThread.init(host,port,path);
    		socketThread.start();
     	}
     	catch (IOException exc) {
     		showError(getApplicationFrame(),getString("error.socket.host_unavailable")+" "+host+":"+port);
     	}
	}
  
  /**
   * Finaliza a comunicacao com o servidor, caso ela exista. 
   */
  public void finishConnection() {
	if (socketThread!=null && !socketThread.isDone()) {
	  	try {
	 	  	socketThread.finish();
	 	}
	 	catch (IOException exc) {
	 		showError(getApplicationFrame(),getString("error.socket.unable_to_finish")+" : "+exc.getMessage());
	 	}
	}
  }

  /**
   * 
   * Thread responsavel por estabelecer a comunicacao com o servidor, e atualizar a 
   * interface sempre que uma nova imagem estah disponivel. Essa classe abre o socket e
   * utiliza o protocolo de comunicacao definido para realizar a solicitacao e receber
   * os dados do servidor de imagens.
   *  
   * @author mnery
   *
   */
  class ImageSocketThread extends Thread {

  	/** o socket de comunicacao com o servidor de imagens */
  	private Socket imageSourceSocket;
  	
  	/** o input stream de leitura do socket */
  	private BufferedInputStream input;

  	/** o media tracker responsavel por bloquear a thread ateh que uma imagem tenha
  	 * sido completamente criada. */
  	private MediaTracker mediaTracker;
  	
  	/** indica se a thread nao estah ativa */ 
  	// esse conceito pode ser mudado para "running", desde que as devidas modificacoes sejam realizadas na classe
  	private boolean done;
  	
  	/** constante para indicar o tamanho da janela no calculo da media de FPS */
  	private static final int FPS_WINDOW_SIZE = 30;
  	
  	/** o tempo dos ultimos FPS_WINDOW_SIZE quadros recebidos */
  	private int[] time = new int[FPS_WINDOW_SIZE];
  	
  	/** indica se o FPS deve ser exibido, especialmente se a janela jah possui 30 valores setados */
  	private boolean showFPS = false;

  	/** Inicializacao basica da Thread, define apenas o media tracker sobre o painel
  	 * onde a imagem serah exibida */
  	public ImageSocketThread() {
  		mediaTracker = new MediaTracker(imagePanel);
  	}
  	
  	/**
  	 * Realiza a conexao ao servidor de imagens e inicia o protocolo enviando o caminho
  	 * absoluto do arquivo.
  	 * 
  	 * @param host O servidor de imagens (onde o algoritmo estah executando e ouvindo uma porta que conhece essa aplicacao)
  	 * @param port A porta do servidor de imagens
  	 * @param path O caminho absoluto a ser enviado
  	 * @throws IOException 
  	 */
  	public void init(String host, int port, String path) throws IOException {
  		imageSourceSocket = new Socket (host,port);
  		path = path + "\n";
  		BufferedOutputStream out = new BufferedOutputStream(imageSourceSocket.getOutputStream()); 
  		out.write(path.getBytes());
  		out.flush();
  		fpsMessage.setVisible(true);
  		input = new BufferedInputStream(imageSourceSocket.getInputStream());
  	}
  	
  	/**
  	 * Indica que a thread deve concluir no futuro proximo e fecha o socket.
  	 * Um protocolo de saida mais limpo pode ser implementado, porem o servidor hoje
  	 * nao espera receber nada quando a conexao eh encerrada. Potencialmente, uma excecao
  	 * serah lancada no metodo run. Caso o "done" nao esteja setado para true, a excecao
  	 * serah considerada um erro e uma mensagem serah apresentada.   
  	 */
  	public void finish() throws IOException {
  		done = true;
  		if (imageSourceSocket!=null) {
  			imageSourceSocket.close();
  		}
  	}

  	public void run() {
		try {
			int size;
			int fpsIndex = 0;
			long start = System.currentTimeMillis();
			long end;
	  		done = false;
			while (!done && (size=readSize())>0) {
				byte[] buf = new byte[size];
				int read = 0;
				// le todo o conteudo da imagem (definido pelo readSize() conforme protocolo)
				while (read<size) {
					read += input.read(buf,read,(size-read)); 
				}
				// cria uma imagem local
				final Image localImg = Toolkit.getDefaultToolkit().createImage(buf);
				mediaTracker.addImage(localImg,0);
				// faz com que a thread fique bloqueada ateh a conclusao da imagem
				mediaTracker.waitForID(0);
				mediaTracker.removeImage(localImg);
				// dado que a imagem estah pronta, o valor de largura e altura estao corretos
				width = localImg.getWidth(observer);
	            height = localImg.getHeight(observer);
	            // solicita o redesenho da imagem, o revalidate soh precisa ser chamado caso 
	            // a largura e altura tenha mudado, implementar isso caso o refresh estaja lento. 
	            javax.swing.SwingUtilities.invokeAndWait(new Runnable() {
					public void run() {
						img = localImg;
						if (!showFPS) {
							fpsMessage.setText(getString("calculatingFPS.message"));
							fpsMessage.repaint();
						}
						else {
							fpsMessage.setVisible(false);
						}
						imagePanel.setPreferredSize(new Dimension(width, height));
			            imagePanel.revalidate();
						imagePanel.repaint();
					}
				});
				end = System.currentTimeMillis();
				// atualiza o tempo de total do loop para esse quadro
				time[fpsIndex] = (int)(end-start);
				start = end;
				fpsIndex++;
				if (fpsIndex==FPS_WINDOW_SIZE) {
					fpsIndex = 0;
					showFPS = true;
				}
				if (showFPS) {
					showFPS();
				}
			}
		}
		catch (IOException exc) {
			if (!done) {
				showError(getApplicationFrame(),getString("error.socket.premature_EOF"));
			}
			done = true;
		}
		catch (InterruptedException exc) {
			showError(getApplicationFrame(),exc.getMessage());			
		}
		catch (InvocationTargetException exc) {
			showError(getApplicationFrame(),exc.getMessage());			
		}
  	}

  	/**
  	 * Obtem o tamanho da proxima imagem com base no protocolo, 
  	 * ou seja, uma string numerica terminada por um '\n'. Caso EOF seja
  	 * encontrado, retorna zero.
  	 * 
  	 * @return o tamanho da proxima imagem
  	 * @throws IOException
  	 */
  	public int readSize() throws IOException {
  		int response = 0;
  		int ch;
  		while ((ch=input.read())!=-1 && ch!='\n') {
  			response = response * 10 + (((char)ch)-'0');
  		}
  		if (ch==-1) {
  			response = 0;
  		}
  		return response;
  	}

  	/**
  	 * Calcula o FPS dos ultimos 30 quadros e apresenta na interface.
  	 */
  	public void showFPS() {
  		long totalTime = 0;
  		for (int i=0;i<FPS_WINDOW_SIZE;i++) {
  			totalTime += time[i];
  		}
  		// calcula o "frames per milisecond"
  		double fpms = ((double)FPS_WINDOW_SIZE/totalTime);
  		// define o fps
  		double fps = fpms*1000;
  		// arredonda o valor para duas casas decimais, truncando o 
  		// fps multiplicado por 100 e dividindo o double por 100
  		double formattedFPS = ((double)((int)(fps*100)))/100;
  		// eh interessante alterar o preferredSize na inicializacao do textField.
  		fpsValue.setText(""+formattedFPS);
  	}
  	
  	/**
  	 * Indica se essa thread jah concluiu sua tarefa.
  	 * @return true, caso ela jah tenha se conectado alguma vez e encerrado. 
  	 */
  	public boolean isDone() {
  		return done;
  	}
  }
}
