-- $Id: cscfg.lua 65295 2007-07-10 15:53:30Z rodrigoh $

require "getopt"
require "sandbox"
require "csglobal"

-- dir raiz default
local defaultRootDir = posix.getenv("CSMAKE_CO_DIR") or ".."

-- o arquivo de configurao deve ter o mesmo nome que este script,
-- apenas substituindo-se a extenso ".lua" por ".cfg". Ambos os
-- arquivos devem estar no mesmo diretrio.
local CFG_FILE = "./csmake.cfg"

-- nome do arquivo local (opcional) com redefinies para os valores
-- default
local LOCAL_CFG = "./local.cfg"

-- sandbox com o ambiente minimamente necessrio cada funo passvel de
-- estar no arquivo de configurao deve estar referenciada nesta tabela
-- Os mdulos so responsveis pelo registro das funes
local sandbox = Sandbox:new()

-- --------------------------------------------------
-- funes locais

-- reflete definies nos arquivos de configurao para o ambiente
-- global
local function _setGlobalEnv ()
    -- mdulos a serem ignorados (BUILD_IGNORE)
    local buildIgnore = sandbox:get("BUILD_IGNORE")
    if buildIgnore then
        abort(type(buildIgnore) == "table", "BUILD_IGNORE deve ser uma tabela")
        for _, m in ipairs(buildIgnore) do
            cs_buildIgnore[m] = true
        end
    end

    local versionFiles = sandbox:get("VERSION_FILES")
    if versionFiles then
        abort(type(versionFiles) == "table", "VERSION_FILES deve ser uma tabela")
        for _, f in ipairs(versionFiles) do
            cs_versionFiles[f] = true
        end
    end
end

-- carrega redefinies do ambiente do shell (definidas na tabela 'ENV'
-- no csmake.cfg ou no local.cfg)
local function _setShellEnv ()
    -- configura ambiente global (shell)
    local env = sandbox:get("ENV")
    local cache = {}
    if env then
        abort(type(env) == "table", "ENV deve ser uma tabela")
        for var, val in pairs(env) do
            val = string.gsub(val, "%$([%a_]+)", function (varName)
                if var == varName then
                    return posix.getenv(varName)
                else
                    return env[varName] or posix.getenv(varName)
                end
            end)
            posix.putenv(var.."="..val)
        end
    end
end

-- --------------------------------------------------
-- classe AlgorithmNode (private)
-- n que contm algoritmos

local AlgorithmNode = OO:new {
    blackList = { [".svn"] = true, ["."] = true, [".."] = true, },
}

-- armazena os mdulos marcados como algoritmos no arquivo de
-- configurao
local _algorithms

-- "mtodo" privado para obteno de um algoritmo a partir do seu nome.
-- Aborta se o nome for invlido
local function _getAlgByName (self, alg)
    if not self.algs then
        self:getDirInfo()
    end
    local i = self.algsByName[alg]
    abort(self.algsByName[alg], "algoritmo "..alg.." invalido")
    return self.algs[i]
end

-- define o diretrio principal dos algoritmos. Retorna este mesmo valor
function AlgorithmNode:setMasterDir ()
    self.masterDir = self.masterDir or (self.module.localDir .. self.module.module)
    return self.masterDir
end

-- gera o cache de dados de algoritmos: coleta nomes dos algoritmos e as
-- respectivas verses
function AlgorithmNode:getDirInfo ()
    self.algs = {}
    self:setMasterDir()
    for alg in dirList(self.masterDir, self.blackList) do
        local dir = self.masterDir.."/"..alg
        if isDir(dir) then
            if isDir(dir.."/versions") then
                local algData = { name = alg, versions = {} }
                -- coleta as verses disponveis para o algoritmo em
                -- questo
                for version in dirList(dir.."/versions", self.blackList) do
                    table.insert(algData.versions, version)
                    -- indexa tambm pelo nome, para referncia
                    -- rpida
                    algData.versions[version] = true
                end
                table.insert(self.algs, algData)
            else
                cs_log:error("'"..alg.."' nao possui diretorio com as versoes")
            end
        end
    end
    -- preenche tabela inversa (mapeando nome de algoritmo para
    -- o seu ndice)
    self.algsByName = {}
    for i, a in ipairs(self.algs) do
        self.algsByName[a.name] = i
    end
end

-- construtor
function AlgorithmNode:new (node)
    return OO.new(self, { module = node })
end

-- iterador sobre os nomes dos algoritmos associados ao diretrio
-- (mdulo) em questo
-- Retorna: nome do algoritmo, path para o mesmo, tabela com as verses
-- do algoritmo
function AlgorithmNode:listAlgs (rebuild)
    if self.module.localDir then
        if not self.algs or rebuild then
            self:getDirInfo()
        end
        return coroutine.wrap(function ()
            for _, alg in ipairs(self.algs) do
                coroutine.yield(alg.name, self.masterDir.."/"..alg.name,
                    alg.versions)
            end
        end)
    end
end

-- retorna true se uma determinada verso  vlida para um determinado
-- algoritmo
function AlgorithmNode:isVersion (algName, version)
    local alg = _getAlgByName(self, algName)
    return (alg.versions[version] ~= nil)
end

-- retorna as verses associadas a um determinado algoritmo
function AlgorithmNode:getVersions (algName)
    local alg = _getAlgByName(self, algName)
    return alg.versions
end

-- --------------------------------------------------
-- classe InstallTree (private)
-- rvore de diretrios do projeto

local InstallTree = OO:new()

-- essa tabela armazenar, para cada nvel da rvore de instalao, os
-- ns correspondentes
local _modulesByLevel = {}

-- construtor
function InstallTree:new (tree, moduleList, level)
    level = level or 0
    tree._level = level
    _modulesByLevel[level] = _modulesByLevel[level] or {}
    table.insert(_modulesByLevel[level], tree)
    local newTree = OO.new(InstallTree, tree)
    if not moduleList then
        moduleList = {}
        newTree.modules = moduleList
    end
    if tree then
        for k, node in pairs(tree) do
            -- atributos da rvore so associados a ndices
            -- alfanumricos, ns so associados a ndices numricos
            local nk = tonumber(k)
            if nk then
                abort(type(node) == "table", "no da arvore deve ser representado como tabela")
                newTree[nk] = InstallTree:new(node, moduleList, level+1)
            else
                newTree[k] = node
                -- se o n possui o campo 'isAlgorithm' = true,
                -- adiciona-o  lista de algoritmos
                if k == "isAlgorithm" then
                    abort(node == true, "atributo 'isAlgorithm' deve ser true ou ento no deve ser definido")
                    abort(_algorithms == nil, "apenas um no pode conter algoritmos")
                    _algorithms = AlgorithmNode:new(newTree)
                end

                -- se o n possui o campo 'main' = true,
                -- significa que  o mdulo principal
                if k == "main" then
                    abort(node == true, "atributo 'main' deve ser true ou ento no deve ser definido")
                    abort(ConfigFile.mainModule == nil, "apenas um mdulo pode ser o principal")
                    ConfigFile.mainModule = newTree
                end
            end
        end
    end
    -- referncia cruzada: moduleList[modname] contm o n
    -- associado a modname
    if newTree.module then
        moduleList[newTree.module] = newTree
    end
    return newTree
end

-- iterador para percorrimento "depth-first + preorder" dos ns da
-- rvore de instalao, i.e. para cada n, se este tem filhos estes so
-- visitados _aps_ o n em questo ser visitado (mas _antes_ dos irmos
-- do n serem visitados)
-- Retorna:
--      ndice correspondente  posio do n no nvel
--      n
--      diretrio aonde foi extrado o n
--      nome do mdulo correspondente
function InstallTree:nodes (rootDir)
    local traverse
    rootDir = rootDir or self.rootDir
    traverse = function (rootNode, dir)
        rootNode = rootNode or self
        dir = (dir or rootDir) .. "/"
        for i, n in ipairs(rootNode) do
            coroutine.yield(i, n, dir, n.module)
            if type(n) == "table" then
                traverse(n, dir .. n.module)
            end
        end
    end
    return coroutine.wrap(traverse)
end

-- --------------------------------------------------
-- classe ConfigFile

ConfigFile = Module:new {
    actionInfo = "lendo arquivo de configuracao",
    params = "c:",
    provides = NONCMD_MODULE.CFGFILE,
}

-- cadastramento do mdulo

cs_registerModule(ConfigFile)

function ConfigFile:check ()
    return self.loaded ~= nil
end

function ConfigFile:help ()
    print([[

Arquivo de configuracao:
----------
-c arq      : carrega arq de configuracao alternativo
]])
end

function ConfigFile:execute (args, params)
    self.getopt:parse(args, string.concat(params, self.params))
    if self.getopt:opt("h") then
        self:help()
        return
    end
    -- carregamos apenas uma vez
    if self.loaded then return end

    -- adiciona as funes necessrias  sandbox
    sandbox:add("repositories", function (repositoriesTable)
        repositories = repositoriesTable
    end)

    sandbox:add("installTree", function (tree)
        projTree = InstallTree:new(tree)
        projTree.rootDir = defaultRootDir
    end)

    -- carrega arquivo de configurao. As tentativas seguem a ordem:
    -- arq especificado via '-c arq' -> ./csmake.cfg
    self.name = self.getopt:opt("c") or CFG_FILE
    local stat, err = posix.access(self.name)
    abort(stat,
        string.format("arq de configuracao '%s' nao pode ser encontrado",
        self.name))

    -- ok, arq de configurao existe, carrega usando a sandbox
    local mainCfg = sandbox:loadFile(self.name)
    abort(mainCfg, "erro carregando arquivo de configuracao "..self.name)
    -- FIXME esta deveria ser uma chamada protegida contra erros (pcall())
    mainCfg()

    local localCfg = sandbox:loadFile(LOCAL_CFG)
    if localCfg then
        cs_log:info("carregando definicoes locais ("..LOCAL_CFG..")")
        localCfg()
    end

    -- carrega os repositrios e os indexa por seus nomes.
    self.repositoriesMap = {}
    for i, v in pairs (sandbox:get("repositories")) do
        self.repositoriesMap[v.name] = v
    end

    self.projTree = sandbox:get("projTree")
    self.projName = basename(posix.getcwd())
    self.loaded = mainCfg and true
    _setGlobalEnv()
    _setShellEnv()
end

-- retorna iterador para percorrimento dos ns da rvore definida no
-- arquivo de configurao
function ConfigFile:nodes (rootDir)
    return self.projTree:nodes(rootDir)
end

-- retorna todos os ns associados a um determinado nvel. Retorna
-- tambm ns abaixo do nvel especificado se 'belowToo' == true
-- Retorna:
--      ndice na lista DE NVEIS (e NO na lista global)
--      n
--      diretrio aonde foi extrado o n
--      nome do mdulo correspondente
--      nvel
function ConfigFile:nodesAtLevel (level, belowToo)
    level = level or 1
    return coroutine.wrap(function ()
        for k, v in ipairs(_modulesByLevel) do
            if (belowToo and (k >= level)) or (k == level) then
                for i, n in ipairs(v) do
                    coroutine.yield(i, n, n.localDir, n.module, k)
                end
            end
        end
    end)
end

-- retorna o diretrio aonde foi extrado um determinado mdulo
function ConfigFile:getModuleDir (module)
    local node = self.projTree.modules[module]
    abort(node, string.format("modulo %s invalido", module))
    if module then
        local localDir = node and node.localDir
        -- FIXME resolver esta questo do localDir em outro lugar
        abort(localDir, string.format("problemas com o checkout do modulo %s", module))
        dir = localDir .. module
    else
        dir = cs_exedir
    end
    return dir
end

-- retorna true se o mdulo especificado  realmente um mdulo do
-- projeto
function ConfigFile:isModule (module)
    return self.projTree.modules[module] ~= nil
end

-- obtm todos os repositrios definidos
function ConfigFile:getRepositories ()
    local repositories = {}
    for _, rep in pairs (sandbox:get("repositories")) do
        table.insert (repositories, rep)
    end
    return repositories
end

function ConfigFile:getRepository(repositoryName)
  return self.repositoriesMap[repositoryName]
end

-- wrappers

function ConfigFile:hasAlgs ()
    return _algorithms ~= nil
end

-- retorna lista de algoritmos
function ConfigFile:listAlgs ()
    abort(_algorithms, "nenhum no contem algoritmos")
    return _algorithms:listAlgs()
end

-- retorna o diretrio "pai" de todos os algoritmos
function ConfigFile:getAlgsMasterDir ()
    return _algorithms and _algorithms:setMasterDir()
end

-- retorna o nome do mdulo associado aos algoritmos
function ConfigFile:getAlgName ()
    return _algorithms and _algorithms.module.module
end

-- identifica se um mdulo  o repositrio de algoritmos (baseado no
-- nome)
function ConfigFile:isAlg (module)
    return _algorithms and module == _algorithms.module.module
end

-- retorna a tabela com as verses para um determinado algoritmo
function ConfigFile:getAlgVersion (alg)
    return _algorithms and _algorithms:getVersions(alg)
end

-- retorna true se uma determinada verso  vlida para um determinado
-- algoritmo
function ConfigFile:isVersion (alg, version)
    return _algorithms and _algorithms:isVersion(alg, version)
end

-- retorna o valor de uma varivel definida no arquivo de configurao
function ConfigFile:getVar (varname)
    return sandbox:get(varname)
end

-- retorna o mdulo principal definido no arquivo de configurao (o
-- mdulo que possui o atributo "main = true")
function ConfigFile:getMainModule()
    abort(self.mainModule, [[Nenhum modulo foi definido como principal.
        Para faze-lo, acrescente o campo 'main = true' a definicao do
        respectivo modulo no csmake.cfg.
        P.ex. no MARLIM esta definido { ... module = 'marlim', main = true }]])
    return self.mainModule
end
