-- $Id: csversions.lua 69807 2007-11-05 12:39:15Z costa $

require "global"
require "oo"
require "log"

local cacheFile = SCRIPT_PATH .. "/server.cfg"

-- --------------------------------------------------
-- classe VersionNumber (private)
-- tratamento de verses de aplicativos

VERSION = {
    OK = 0,
    NOTFOUND = 1,
    LOWVERSION = 2,
}

local VersionNumber = OO:new()

-- processa uma descrio de verso, acumulando a parte textual e
-- separando as partes numricas
function VersionNumber:parse (strver)
    self.strPart = ""
    self.numPart = {}
    string.gsub(strver, "%a+", function (s)
        self.strPart = self.strPart .. string.lower(s)
    end)
    string.gsub(strver, "%d+", function (n)
        table.insert(self.numPart, n)
    end)
end

-- retorna true se a parte textual de uma verso corresponde  de outra
-- (case-insensitive)
function VersionNumber:textMatch (other)
    return not (self.strPart ~= "" and other.strPart ~= "" and
        self.strPart ~= other.strPart)
end

-- retorna true se o nmero de verso do objeto corrente  maior (mais
-- recente) do que o de outro
function VersionNumber:isHigherThan (other)
    for i, n in ipairs(self.numPart) do
        n = tonumber(n)
        if other.numPart[i] then
            local n2 = tonumber(other.numPart[i])
            if n > n2 then
                return true
            elseif n2 > n then
                return false
            end
            -- se chegou at aqui, os nmeros de verso so iguais
        else
            return true
        end
    end
    -- at aqui os componentes so iguais; ser maior a verso que tiver
    -- mais componentes
    return table.getn(self.numPart) > table.getn(other.numPart)
end

-- --------------------------------------------------
-- funes locais

-- retorna todas as verses de um executvel que porventura existam no
-- path
local function _getAllVersions (exe)
    local versions = io.popen("bash -c 'type -pa "..exe.."'"):read("*a")
    return string.gsub(versions, "\n$", "")
end

-- prefixa um comando com um determinado path, expandindo variveis de
-- ambiente
local function _prependPathToCommand (path, cmd)
    if path == nil then return cmd end
    local newPath = expandEnvVars(path)
    if newPath ~= nil then
        if not isDir(newPath) then
            return nil, path.." nao e um diretorio"
        end
        newPath = newPath.."/"..cmd
        -- o gsub abaixo  necessrio para garantir que comparemos
        -- apenas o nome do comando, descartando eventuais parmetros
        local exeName = string.gsub(newPath, "^(%S+).*", "%1")
        if not isFile(exeName) then
            return nil, exeName.." nao e um arquivo"
        end
    end
    return newPath, "erro expandindo "..path
end

-- retorna o primeiro executvel disponvel no path, caso exista mais de
-- um
local function _getFirstVersion (path, exe)
    local newExe, errMsg = _prependPathToCommand(path, exe)
    if newExe == nil then
        cs_log:warn(exe..": "..errMsg..". Ignorando...")
    else
        exe = newExe
    end
    cs_log:debug("testando "..exe)
    local version = io.popen("bash -c 'type -p "..exe.."'"):read("*a")
    return string.gsub(version, "\n$", "")
end

-- --------------------------------------------------
-- classe Versions

Versions = OO:new {
    hosts = {},
    cache = {},
    apps = {
        svn = {
            cmd = "svn --version",
            regexp = "^svn,%s+%S+%s+(%d+%.%d+%.%d+).*",
        },
        ssh = {
            cmd = "ssh -V",
            regexp = "^([^,]+),.*$",
        },
        java = {
            cmd = "java -version",
            path = "$JAVA_HOME/bin",
            regexp = "^java version \"(.-)\"",
        },
        javac = {
            cmd = _getFirstVersion,
            path = "$JAVA_HOME/bin",
            params = { "javac" },
        },
        jar = {
            cmd = _getFirstVersion,
            path = "$JAVA_HOME/bin",
            params = { "jar" },
        },
        gzip = {
            cmd = "gzip --version",
            regexp = "gzip ([%d.]+).*",
            capture = "%1",
        },
        gunzip = {
            cmd = "gunzip --version",
            regexp = "gu?n?zip ([%d.]+).*",
            capture = "%1",
        },
        gtar = {
            cmd = "gtar --version",
            regexp = "tar %(GNU tar%) ([%d.]+).*",
            capture = "%1",
        },
        make = {
            cmd = "gmake -v",
            regexp = "^(GNU)%D*([%d.]+).*$",
            capture = "%1",
        },
        os = {
            cmd = function () return posix.uname("%s") end
        },
        osver = {
            cmd = function () return posix.uname("%r") end
        },
    },
}

-- coleta a verso de uma determinada aplicao para a mquina atual
function Versions:computeVersion (app)
    if not self.apps[app] then
        return false
    end
    local cmd = self.apps[app].cmd
    local regexp = self.apps[app].regexp
    local capture = self.apps[app].capture
    local params = self.apps[app].params
    local path = self.apps[app].path
    local output
    local ver, nc
    if type(cmd) == "string" then
        if path then
            local newCmd, errMsg = _prependPathToCommand(path, cmd)
            if newCmd == nil then
                cs_log:warn(errMsg..". Ignorando...")
            else
                cmd = newCmd
            end
        end
        -- o redirecionamento de stderr para stdout no funciona para
        -- bourne-shell "puro" (/bin/sh)
        cs_log:debug("testando "..cmd)
        output = io.popen(cmd.." 2>&1")
        for l in output:lines() do
            ver, nc = string.gsub(l, regexp, capture or "%1")
            -- estamos interessados apenas na 1a substituio
            if nc == 1 then break end
        end
        if nc == 0 then
            cs_log:warn("comando "..app.." nao disponivel")
        end
        return nc == 1 and ver or "N/A"
    elseif type(cmd) == "function" then
        ver = cmd(path, params and unpack(params))
        if not regexp then
            return ver
        else
            ver, nc = string.gsub(ver, regexp, capture or "%1")
            return nc > 0 and ver or "N/A"
        end
    end
end

-- atualiza o cache -- essencialmente, faz um dump dos dados em memria
-- para o arquivo
function Versions:updateCache ()
    self:computeCurrentHost()
    local cache, err = io.open(cacheFile, "w")
    abort(cache, err)
    for host, data in pairs(self.hosts) do
        cache:write(host .. " = {\n")
        for k, v in pairs(data) do
            cache:write(string.format("    %s = '%s',\n", k, v))
        end
        cache:write("}\n\n")
    end
    cache:close()
end

-- regrava o cache, armazenando apenas as informaes da mquina
-- corrente
function Versions:resetCache ()
    self:computeCurrentHost()
    abort(self.hosts[HOSTNAME], "nao ha dados para o no em questao")
    local cache, err = io.open(cacheFile, "w")
    abort(cache, err)
    cache:write(HOSTNAME .. " = {\n")
    for k, v in pairs(self.hosts[HOSTNAME]) do
        cache:write(string.format("    %s = '%s',\n", k, v))
    end
    cache:write("}\n\n")
    cache:close()
end

-- coleta verses para a mquina corrente
function Versions:computeCurrentHost ()
    local hostData = {}
    for appName, app in pairs(self.apps) do
        cs_log:debug("verificando versao de "..appName)
        hostData[appName] = self:computeVersion(appName)
    end
    -- TODO vale  pena armazenar uma verso "compilada" (objeto
    -- VersionNumber) em uma cache?
    self.hosts[HOSTNAME] = hostData
end

-- l os dados da cache
function Versions:loadCache ()
    local chunk, err = loadfile(cacheFile)
    if chunk then
        setfenv(chunk, self.hosts)
        chunk()
    else
        cs_log:warn("nao foi possivel ler cache " .. cacheFile)
        cs_log:debug("erro = " .. err)
    end
    if not self.hosts[HOSTNAME] then
        -- mquina corrente no est no cache; calcula os valores e
        -- atualiza o cache
        cs_log:warn("atualizando versoes dos aplicativos do servidor '" ..
            HOSTNAME .. "'")
        self:computeCurrentHost()
        self:updateCache()
    end
end

-- retorna a verso associada a uma determinada aplicao. O default 
-- retornar para o servidor corrente, mas um servidor especfico pode
-- ser requisitado
-- Caso no existam dados para o servidor solicitado, retorna false +
-- msg de erro
function Versions:get (app, host)
    host = host or HOSTNAME
    if not self.hosts[host] then
        return false, "nao existem dados para o servidor "..host
    end
    return self.hosts[host][app]
end

-- retorna lista das mquinas que possuem verso de app >= ver, na forma
-- { host = ver }
function Versions:findProviders (app, ver)
    local result = {}
    local req = VersionNumber:new()
    local found = VersionNumber:new()
    req:parse(ver)
    for host, data in pairs(self.hosts) do
        if data[app] then
            found:parse(data[app])
            if req:textMatch(found) and not req:isHigherThan(found) then
                result[host] = data[app]
            end
        end
    end
    return result
end

-- verifica se os aplicativos da mquina corrente atendem s verses
-- exigidas
function Versions:check (reqVersions)
    if not reqVersions then return true end
    local req = VersionNumber:new()
    local curr = VersionNumber:new()
    local versions = self.hosts[HOSTNAME]
    for exe, reqver in pairs(reqVersions) do
        self.cache[exe] = self.cache[exe] or {}
        if not self.cache[exe][reqver] then
            cs_log:debug(string.format("verificando versao: %s (%s)", exe, reqver))
            if not versions[exe] or versions[exe] == "N/A" then
                self.cache[exe][reqver] = VERSION.NOTFOUND
            else
                if reqver == "*" then
                    self.cache[exe][reqver] = VERSION.OK
                else
                    curr:parse(versions[exe])
                    req:parse(reqver)
                    if not req:textMatch(curr) or req:isHigherThan(curr) then
                        self.cache[exe][reqver] = VERSION.LOWVERSION
                    else
                        self.cache[exe][reqver] = VERSION.OK
                    end
                end
            end
        end
        if self.cache[exe][reqver] ~= VERSION.OK then
            return false, exe, self.cache[exe][reqver]
        end
    end
    return true
end

-- exibe as verses para um servidor especfico
function Versions:printVersions (host)
    host = host or HOSTNAME
    if not self.hosts[host] then
        cs_log:warn("nao existem dados para o servidor "..host)
    end
    for app, ver in pairs(self.hosts[host]) do
        print(app, ver)
    end
end

-- exibe todas as verses armazenadas (i.e. para todos os servidores)
function Versions:printAll ()
    for host in pairs(self.hosts) do
        print(string.format("\n%s\n-----", host))
        self:printVersions(host)
    end
end

-- --------------------------------------------------
-- inicializao do mdulo

Versions:loadCache()
