-- $Id: global.lua 70299 2007-11-16 20:17:22Z costa $

HOSTNAME = string.gsub(posix.uname("%n"), "^(.-)%..*", "%1")

local _savedDir

-- --------------------------------------------------
-- Funes genricas

function abort (cond, msg, code, showTrace)
    if not cond then
        io.stderr:write("\n[ERRO ] "..msg.."\n\n")
        if showTrace then
            io.stderr:write(debug.traceback().."\n")
        end
        os.exit(code or 1)
    end
end

-- exibe um uma mensagem e aguarda input; retorna resultado ou o valor
-- de 'default' caso o usurio digite apenas ENTER
-- se default = nil, retorna sempre result
function ask (msg, default)
    io.write(msg.." ")
    if default then
        io.write("["..default.."] ")
    end
    local result = io.read()
    return (result == "") and default or result
end

-- exibe uma pergunta relativa a uma confirmacao (resposta = 's' ou 'n')
-- e retorna true se o usuario respondeu 's' ou 'S' (isto vale tambem
-- para os casos em que 's' e o default)
function confirm(msg, default)
    local result = ask(msg, default)
    return string.lower(result) == "s"
end

-- --------------------------------------------------
-- Funes para manipulao de arquivos

-- troca a extenso de um nome de arquivo no formato nome.ext para
-- nome.newExt . O nome do arquivo pode conter pontos (apenas ltima
-- "extenso" ser trocada)
function renameExt (fileName, newExt)
    return string.gsub(fileName, "(.-)%.%a+$", "%1."..newExt)
end

-- retorna apenas o ltimo componente de um path
function basename (path)
    return string.gsub(path, "^.*/(.*)$", "%1")
end

-- verifica se um determinado arquivo  do tipo especificado
local function _checkFile(file, ftype, descr, errMsg)
    local fileInfo = posix.stat(file)
    if not fileInfo then
        return false, descr.." "..file.." nao encontrado"
    end
    if fileInfo.type ~= ftype then
        return false, file.." "..errMsg
    end
    return true
end

-- verifica se um nome corresponde a um diretrio
-- ATENO: aceita links, assumindo que estes apontam para diretrios.
-- Por tambm aceitar links no podemos usar a verso _checkFile() acima.
function isDir (dir)
    local dirInfo = posix.stat(dir)
    if not dirInfo then
        return false, "diretorio "..dir.." nao encontrado"
    end
    if dirInfo.type ~= "directory" and dirInfo.type ~= "link" then
        return false, dir.." nao e um diretorio"
    end
    return true
end

function isLink (link)
    return _checkFile(link, "link", "link", "nao e um link simbolico")
end

function isFile (file)
    return _checkFile(file, "regular", "arquivo", "nao e um arquivo")
end

-- retorna o tipo do arquivo ("file", "directory" etc.) apontado por
-- path caso este exista, ou nil caso contrrio
-- ATENO: apesar do nome, funciona tambm para diretrios
function fileExists (path)
    local info = posix.stat(path)
    return info and info.type
end

-- iterador sobre a lista de arquivos e diretrios de um diretrio,
-- opcionalmente evitando arquivos/diretrios especificados por uma
-- tabela no formato { "arqname" = true }
function fileList (dir, blackList)
    abort(isDir(dir))
    return coroutine.wrap(function ()
        for f in posix.files(dir) do
            if not blackList or (not blackList[f]) then
                coroutine.yield(f)
            end
        end
    end)
end

-- idem a fileList, mas retorna apenas diretrios
function dirList (dir, blackList)
    return coroutine.wrap(function ()
        for d in fileList(dir, blackList) do
            if isDir(dir.."/"..d) then
                coroutine.yield(d)
            end
        end
    end)
end

-- muda o diretrio corrente, salvando o atual para posterior
 -- restaurao via popd()
function pushd (dir)
    _savedDir = posix.getcwd()
    return posix.chdir(dir)
end

-- restaura o diretrio salvo por pushd()
function popd ()
    if _savedDir then
        return posix.chdir(_savedDir)
    else
        return nil
    end
end

-- retorna o path absoluto associado a um path (relativo)
function getAbsolutePath (path)
    local saveDir = posix.getcwd()
    local status, err = posix.chdir(path)
    if status ~= 0 then
        cs_log:fatal(err)
        os.exit(1)
    end
    local absPath = posix.getcwd()
    posix.chdir(saveDir)
    return absPath
end

-- --------------------------------------------------
-- Funes para manipulao de tabelas

-- acrescenta funcionalidade de 'merge' ao pacote 'table': copia todos
-- os campos de 'from' para 'to', preservando os que j existam.
-- Retorna 'to' (ou 'from' caso 'to' seja nil).
-- OBS.: se 'to' == {}, equivale a uma "shallow copy" de 'from'
function table.merge (from, to)
    if not to then return from end
    for k, v in pairs(from) do
        if not rawget(to, k) then to[k] = v end
    end
    return to
end

-- acrescenta funcionalidade de ordenacao de tabelas (arrays) compostas por
-- strings, ignorando a diferenca entre maiusculas e minusculas.
function table.sortIgnoreCase (t)
    table.sort (t, function (b1, b2)
        return string.lower(b1) < string.lower(b2)
    end)
end

-- --------------------------------------------------
-- Funes para manipulaes de strings

-- extenso do pacote 'string': concatena duas strings, mesmo que sejam
-- nil
function string.concat (s1, s2)
    return (s1 or "") .. (s2 or "")
end

-- remove espaos no incio e no final de uma string
function string.trim (str)
    str = string.gsub(str, "^%s*(.-%S)%s*$", "%1")
    return str
end

-- --------------------------------------------------
-- Funes que interagem com o ambiente

-- expande variveis de ambiente em uma string.
-- RETORNA:
-- . caso todas as variveis existam no ambiente: string com todas as
--   variveis expandidas
-- . caso alguma varivel no exista no ambiente: nil + a string
--   (parcialmente) expandida
function expandEnvVars (str)
    local numVars = 0
    local str, nc = string.gsub(str, "%$([%a_]+)", function (envVar)
        local envValue = posix.getenv(envVar)
        if envValue == nil then
            return "$"..envVar
        else
            numVars = numVars+1
            return envValue
        end
      end)
    if nc > numVars then
        return nil, str
    end
    return str
end
