/**
 * $Id: Validator.java 96766 2009-09-28 17:49:23Z ururahy $
 */
package validations;

import java.io.BufferedInputStream;
import java.io.Console;
import java.io.File;
import java.io.FileInputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Properties;
import java.util.logging.ConsoleHandler;
import java.util.logging.FileHandler;
import java.util.logging.Level;

import org.kohsuke.args4j.CmdLineException;
import org.kohsuke.args4j.CmdLineParser;
import org.kohsuke.args4j.Option;

import validations.AbstractValidation.Status;
import validations.util.ValidationsLogger;
import validations.util.ValidatorUtils;

/**
 * Classe que executa validaes sobre uma instalao do sistema.
 * <p>
 * Todas as subclasses de {@link AbstractValidation} implementadas no pacote
 * <code>validations</code> do CSBase e no pacote <code>validations_sys</code>
 * do projeto sero automaticamente executadas.
 * <p>
 * Os parmetros da linha de comando so processados e definem a configurao de
 * alguns parmetros da execuo que podem ser consultados pelas validaes. O
 * nico parmetro obrigatrio  o path para o arquivo que contm as
 * propriedades globais definidas para a instalao em questo do sistema. Deste
 * arquivo so extradas propriedades obrigatrias para o processo de validao:
 * <p>
 * <ul>
 * <li><code>ProjectService.baseProjectDir</code> path para o diretrio
 * <code>project</code> do CSBase. Pode ser absoluto ou relativo ao diretrio
 * onde se encontra o arquivo de propriedades.
 * <li><code>Validator.tempDir</code> path para o diretrio temporrio onde
 * sero criados os backups e onde ser armazenado o log da validao (
 * <code>validacoes.log</code>).
 * </ul>
 * Alm das propriedades obrigatrias, existem algumas propriedades opcionais
 * que podem ser usadas para ajustes finos:
 * <p>
 * <ul>
 * <li><code>Validator.logPath</code> path para o log da validao.
 * </ul>
 * <p>
 * As validaes recebem o validador como parmetro e podem us-lo p.ex. para
 * consultar as propriedades lidas do arquivo de propriedades.
 * <p>
 * Um logger especfico  criado por default, para registro do processo tanto no
 * console quanto no arquivo <code>validacoes.log</code>. O nvel de logging
 * para o arquivo  definido como {@link Level#FINE} enquanto que o nvel
 * default para o console  {@link Level#INFO}.
 * <p>
 * As validaes so instanciadas por reflexividade e ordenadas de acordo com
 * seus nomes. Normalmente a ordem entre as validaes no deve ser um fator
 * relevante mas, caso seja, pode-se adotar uma poltica semelhante  usada para
 * os scripts de inicializao UNIX (p.ex. <code>Val05XXX</code>,
 * <code>Val10YYY</code> etc.).
 * <p>
 * O validador pode ser executado em <i>modo validao</i> (parmetro
 * <code>-V</code>) ou em <i>modo patch</i> (default). No modo validao os
 * backups no so criados, e nenhuma alterao  feita. No modo patch, caso a
 * aplicao de um dos patches falhe, este e todos os demais aplicadas antes
 * dele so desfeitos, na ordem inversa em que foram executados.
 * <p>
 * Os diretrios de backup so removidos apenas em duas situaes:
 * <ol>
 * <li>quando todos os patches foram aplicados com sucesso
 * <li>quando algum patch falhou mas todos os <i>rollbacks</i> foram
 * bem-sucedidos
 * </ol>
 * Em todos os demais casos, os diretrios de backup so preservados para
 * restaurao "manual" dos dados alterados (que deve ser feita sob superviso
 * da equipe de desenvolvimento). As validaes no so executadas se seus
 * respectivos diretrios de backup no existem, i.e. uma tentativa de
 * instalao que tenha sido mal-sucedida e tenha deixado para trs diretrios
 * de backup impedir instalaes subsequentes at que o problema tenha sido
 * corrigido e os diretrios de backup tenham sido removidos.
 * <p>
 * Instalaes bem sucedidas podem ser repetidas sem efeito colateral, i.e.
 * patches aplicados com sucesso no sero aplicados novamente em instalaes
 * subsequentes.
 * 
 * @see #processArgs(String[])
 * @see #isValidatingOnly()
 * @see #isVerbose()
 * @see #getSystemProperty(String)
 * @see #getLogger()
 * @see #rollback(AbstractValidation)
 * @see #run()
 * @see ValidationsLogger
 * @see AbstractValidation
 * @see ValidatorUtils
 * 
 * @author Tecgraf
 */
public class Validator {
  /**
   * Nome do pacote que contm as validaes do CSBase.
   */
  private static final String FRAMEWORK_VALIDATIONS_PACKAGE = "validations";

  /**
   * Nome do pacote que contm as validaes do sistema.
   */
  private static final String SYSTEM_VALIDATIONS_PACKAGE = "validations_sys";

  /**
   * Chave para a propriedade que contm o path para o diretrio temporrio da
   * instalao.
   */
  public static final String VALIDATIONS_TEMP_DIR = "Validator.tempDir";

  /**
   * Nome do arquivo default de propriedades.
   */
  private static final String DEFAULT_PROPERTIES_FILE = "System.properties";

  /**
   * Nome da propriedade (opcional) que contm o path para o arquivo de log da
   * instalao.
   */
  private static final String LOGPATH_PROPERTY_NAME = "Validator.logPath";

  /**
   * Nome default para o arquivo de log que ser criado no diretrio temporrio
   * caso o usurio no especifique um arquivo de logs.
   */
  private static final String DEFAULT_LOGFILE_NAME = "validacoes.log";

  /**
   * Nome do arquivo de exemplo com valores default para as configuraes. Caso
   * o usurio no fornea o path para o arquivo de propriedades, ele ter a
   * chance de criar um arquivo de propriedades a partir deste.
   */
  public static final String SAMPLE_PROPERTIES_FILE =
    SYSTEM_VALIDATIONS_PACKAGE + File.separatorChar + DEFAULT_PROPERTIES_FILE
      + ".default";

  /**
   * Lista de validaes.
   * 
   * @see #registerValidations()
   */
  private List<AbstractValidation> validations;

  /**
   * Flag que indica se o usurio deseja apenas validar a instalao, sem
   * aplicar as correes.
   */
  private boolean validateOnly;

  /**
   * Propriedades do sistema.
   */
  private Properties systemProperties;

  /**
   * Logger para registro de mensagens. Possui sempre um {@link ConsoleHandler},
   * mas pode possuir tambm um {@link FileHandler} para cada validao.
   */
  private ValidationsLogger logger;

  /**
   * Path absoluto do diretrio corrente.
   */
  private String currDir;

  /**
   * Flag que indica se o validador foi instanciado com sucesso e est pronto
   * para ser executado.
   */
  private boolean readyToRun;

  /**
   * Path para o diretrio temporrio da instalao.
   */
  private String instTempDirPath;

  /**
   * Flag que indica se estamos em modo verbose.
   */
  private boolean verboseMode;

  /**
   * Classe auxiliar usada para processamento dos parmetros da linha de comando
   * pelo args4j. S  necessria porque nem todos os parmetros correspondem a
   * campos de Validator (seno poderamos usar a prpria).
   */
  private class CLOptions {
    /**
     * Apenas valida a instalao, no aplica nenhuma correo.
     */
    @Option(name = "-V", usage = "apenas valida a instalao (no faz mudanas)")
    boolean validateOnly = false;
    /**
     * Liga o modo verbose (opcional).
     */
    @Option(name = "-v", usage = "modo verbose")
    boolean verboseMode = false;
    /**
     * Path para o arquivo de log da validao (opcional).
     */
    @Option(name = "-l", usage = "path para o arquivo de log da validao")
    File logFile = null;
    /**
     * Path para o arquivo de propriedades (obrigatrio -- se no for fornecido,
     * ser obtido interativamente).
     */
    @Option(name = "-p", usage = "path para o arquivo global de propriedades")
    File propertiesFile = null;
  }

  /**
   * Construtor. Cria o logger e processa os parmetros da linha de comando.
   * 
   * @param args - parmetros da linha de comando
   */
  private Validator(String[] args) {
    currDir = System.getProperty("user.dir");
    logger = new ValidationsLogger();
    readyToRun = processArgs(args) && registerValidations();
  }

  /**
   * Obtm o path para o diretrio temporrio do arquivo de propriedades, e
   * garante que o diretrio correspondente existe.
   * 
   * @return <code>true</code> se o diretrio temporrio existe ou, se no
   *         existia, se foi criado com sucesso
   */
  private boolean prepareTempDir() {
    instTempDirPath = getSystemProperty(VALIDATIONS_TEMP_DIR);
    if (instTempDirPath == null) {
      logger.severe("Propriedade obrigatria " + VALIDATIONS_TEMP_DIR
        + " no foi definida");
      return false;
    }
    File tempDirFile = new File(instTempDirPath);
    if (!tempDirFile.exists()) {
      if (!tempDirFile.mkdir()) {
        logger.severe("No foi possvel criar o diretrio " + instTempDirPath
          + " para backups");
        return false;
      }
      logger.fine("Diretrio " + instTempDirPath + " criado para backups");
    }
    else if (!tempDirFile.isDirectory()) {
      logger.severe(instTempDirPath + " no  um diretrio");
      return false;
    }
    // TODO reclamar se o diretrio no estiver vazio (i.e. se uma instalao
    // prvia no foi concluda)
    logger.fine("Diretrio temporrio: " + tempDirFile.getAbsolutePath());
    return true;
  }

  /**
   * Cadastra as validaes que sero executadas. Instancia programaticamente as
   * classes dos pacotes <code>validations</code> e <code>validations_sys</code>
   * que derivam de {@link AbstractValidation} e no so abstratas. A lista de
   * validaes  ordenada alfabeticamente pelo nome das respectivas classes,
   * para possibilitar a priorizao de validaes em funo dos nomes das
   * mesmas, se desejado.
   * 
   * @return <code>true</code> se todas as validaes foram registradas com
   *         sucesso
   */
  private boolean registerValidations() {
    validations = new ArrayList<AbstractValidation>();
    if (!instantiateValidations(FRAMEWORK_VALIDATIONS_PACKAGE)
      || !instantiateValidations(SYSTEM_VALIDATIONS_PACKAGE)) {
      /*
       * o erro j foi informado
       */
      return false;
    }
    /*
     * ordenamos a lista de validaes em funo dos nomes das respectivas
     * classes
     */
    Collections.sort(validations, new Comparator<AbstractValidation>() {

      @Override
      public int compare(AbstractValidation v1, AbstractValidation v2) {
        return v1.getClass().getSimpleName().compareTo(
          v2.getClass().getSimpleName());
      }
    });
    /*
     * registramos no arquivo de log quais validaes sero aplicadas, e em qual
     * ordem
     */
    logger.fine("Validaes:");
    for (AbstractValidation val : validations) {
      logger.fine("    " + val.getClass().getSimpleName());
    }
    return true;
  }

  /**
   * Instancia as validaes de um determinado pacote. Apenas as classes que
   * derivam de {@link AbstractValidation} e no so abstratas so instanciadas.
   * 
   * @param pkg pacote que contm as validaes
   * @return <code>true</code> se todas as validaes foram instanciadas com
   *         sucessos
   */
  private boolean instantiateValidations(String pkg) {
    for (String className : getValidationsNames(pkg)) {
      try {
        Class<?> valClass = Class.forName(className);
        if (Modifier.isAbstract(valClass.getModifiers())
          || !AbstractValidation.class.isAssignableFrom(valClass)) {
          /*
           * a classe em questo era abstrata ou no era subclasse de
           * AbstractValidation, vamos ignor-la
           */
          continue;
        }
        validations.add((AbstractValidation) valClass.newInstance());
      }
      catch (Exception e) {
        logger.severe("no foi possvel instanciar validao " + className);
        logger.severe("erro: " + e.getClass().getSimpleName());
        return false;
      }
    }
    return true;
  }

  /**
   * Obtm uma lista com os nomes dos arquivos <code>.class</code> do pacote
   * especificado.
   * 
   * @param pkg nome do pacote que contm as validaes
   * 
   * @return lista com os nomes dos arquivos <code>.class</code> do pacote
   *         especificado
   */
  private List<String> getValidationsNames(final String pkg) {
    final List<String> namesList = new ArrayList<String>();
    File dir = new File(currDir + File.separatorChar + pkg);
    if (!dir.exists()) {
      return namesList;
    }
    dir.listFiles(new FilenameFilter() {

      @Override
      public boolean accept(File dir, String name) {
        /*
         * queremos apenas os arquivos .class que no so inner-classes (i.e.
         * no possuem '$' no nome)
         */
        if (name.endsWith(".class") && name.indexOf('$') == -1) {
          String n = name.substring(0, name.indexOf('.'));
          namesList.add(pkg + '.' + n);
        }
        return false;
      }

    });
    return namesList;
  }

  /**
   * Processa os argumentos fornecidos na linha de comando. Aceita os seguintes
   * parmetros:
   * <ul>
   * <li>-p path : path para o arquivo com as propriedades do sistema
   * (obrigatrio)
   * <li>-v : verbose (opcional). Exibe informaes durante o processamento.
   * <li>-V : validateOnly (opcional). Indica que apenas a validao deve ser
   * executada; as correes no sero aplicadas, mesmo que o validador seja
   * capaz de faz-lo.
   * <li>-l path (opcional): path para o log
   * </ul>
   * 
   * Qualquer argumento comeando com '-' diferente destes abortar o processo,
   * o mesmo valendo caso no haja nenhum argumento sem o '-'. Caso haja
   * mltiplos argumentos sem o '-', o ltimo ser usado como o path para o
   * arquivo de propriedades.
   * 
   * @param args - argumentos fornecidos na linha de comando
   * 
   * @return <code>true</code> se os parmetros foram processados com sucesso
   */
  private boolean processArgs(String[] args) {
    CLOptions options = new CLOptions();
    CmdLineParser parser = new CmdLineParser(options);
    try {
      parser.parseArgument(args);
    }
    catch (CmdLineException e) {
      System.err.println(e.getMessage());
      parser.printUsage(System.err);
      return false;
    }
    verboseMode = options.verboseMode;
    validateOnly = options.validateOnly;
    File logFile = options.logFile;
    File propertiesFile = options.propertiesFile;

    if (propertiesFile == null) {
      propertiesFile = getPropertiesFile();
      // propertiesFile ainda pode ser null, se houve algum erro
    }
    if (propertiesFile != null && !ValidatorUtils.fileExists(propertiesFile)) {
      logger.severe("Arquivo de propriedades " + propertiesFile.getPath()
        + " no existe");
      return false;
    }
    /*
     * l o arquivo de propriedades especificado. Neste ponto o arquivo pode ser
     * null, mas isto  tratado pelo mtodo que l as propriedades
     */
    if (!readSystemDefaultProperties(propertiesFile)) {
      return false;
    }
    /*
     * garante que existe o diretrio temporrio para backups
     */
    if (!prepareTempDir()) {
      return false;
    }
    /*
     * configura o nvel de log
     */
    if (!configureLogger(verboseMode, logFile)) {
      return false;
    }
    logger.blank(Level.FINE);
    logger.fine(Calendar.getInstance().getTime().toString());
    return true;
  }

  /**
   * Cria um handler para log em arquivo, e configura os nveis de logging. O
   * nvel de log para o console  controlado pelo usurio (defaul == INFO), mas
   * o log para arquivo  sempre FINE.
   * 
   * @param verboseMode indica se o usurio escolheu o modo verbose
   * @param logFile arquivo de log. Pode ser null; neste caso tentamos obter o
   *        path do arquivo de propriedades, seno criamos um log default no
   *        diretrio temporrio
   * @return <code>true</code> se o handler foi criado com sucesso
   * 
   * @see #LOGPATH_PROPERTY_NAME
   */
  private boolean configureLogger(boolean verboseMode, File logFile) {
    logger.setConsoleLevel(verboseMode ? Level.FINE : Level.INFO);

    String logFilePath;
    if (logFile != null) {
      if (!ValidatorUtils.fileExists(logFile)) {
        logger.severe("Arquivo de log " + logFile.getPath() + " no existe");
        return false;
      }
      logFilePath = logFile.getAbsolutePath();
    }
    else {
      /*
       * usurio no forneceu o path na linha de comando, verificamos se foi
       * definido como propriedade
       */
      logFilePath = getSystemProperty(LOGPATH_PROPERTY_NAME);
      if (logFilePath == null) {
        /*
         * tambm no definiu como propriedade, criamos um com um nome default
         * no diretrio temporrio
         */
        logFilePath =
          getTempDirPath() + File.separatorChar + DEFAULT_LOGFILE_NAME;
      }
    }
    return logger.setFileHandler(logFilePath);
  }

  /**
   * Obtm o path para o arquivo de propriedades default do sistema. Busca
   * primeiro por arquivos j existentes, na seguinte ordem:
   * <ul>
   * <li><code>../System.properties</code>
   * <li><code>validations_sys/System.properties.default</code> (se este arquivo
   * existe,  copiado para <code>../System.properties</code>, mediante
   * confirmao do usurio)
   * </ul>
   * Se nenhuma das opes acima for bem-sucedida, obtm um path
   * interativamente.
   * 
   * @return o arquivo de propriedades, ou null caso ocorra algum erro
   */
  private File getPropertiesFile() {
    logger.warning("No foi fornecido path para o arquivo de propriedades.");
    Console console = System.console();
    if (console == null) {
      /*
       * no conseguimos uma referncia para o console, o que provavelmente
       * significa que no  um shell interativo
       */
      logger.severe("Este terminal no  interativo");
      return null;
    }
    /*
     * agora verificamos se existe "validations_sys/System.properties.default".
     * Se existe, perguntamos se o usurio quer us-lo como referncia
     */
    boolean copySample = false;
    if (ValidatorUtils.fileExists(SAMPLE_PROPERTIES_FILE)) {
      logger
        .warning("Deseja criar um novo arquivo de propriedades a partir de ");
      logger.warning(SAMPLE_PROPERTIES_FILE + " ?");
      copySample =
        ValidatorUtils.readConfirmation(console, ValidatorUtils.Option.YES);
      System.out.println();
    }
    /*
     * obtemos o path para o arquivo de propriedades interativamente
     */
    logger.warning("Fornea o path para o arquivo de propriedades.");
    logger.warning("O path deve ser absoluto, ou relativo ao diretrio");
    logger.warning(currDir);
    logger.warning("e deve incluir o nome do arquivo.");
    String path = ValidatorUtils.readLine(console, "  > ");
    System.out.println();
    if (copySample) {
      if (ValidatorUtils.fileExists(path)) {
        logger.warning("O arquivo j existe. Deseja sobrescrev-lo?");
        boolean overwrite =
          ValidatorUtils.readConfirmation(console, ValidatorUtils.Option.YES);
        if (!overwrite) {
          logger.severe("Operao abortada pelo usurio");
          return null;
        }
        logger.info("Sobrescrevendo o arquivo de propriedades " + path);
      }
      else {
        logger.info("Criando o arquivo de propriedades " + path);
      }
      if (!ValidatorUtils.copyFile(SAMPLE_PROPERTIES_FILE, path, logger)) {
        return null;
      }
    }
    return new File(path);
  }

  /**
   * Carrega as propriedades default de um arquivo especfico.
   * 
   * @param propertiesFile - o arquivo com as propriedades
   * 
   * @return <code>true</code> se a carga foi bem-sucedida
   */
  private boolean readSystemDefaultProperties(File propertiesFile) {
    if (propertiesFile == null) {
      return false;
    }
    if (!propertiesFile.exists()) {
      logger
        .severe("Arquivo de propriedades " + propertiesFile + " no existe");
      return false;
    }

    BufferedInputStream in = null;
    try {
      in = new BufferedInputStream(new FileInputStream(propertiesFile));
      systemProperties = new Properties();
      systemProperties.load(in);
    }
    catch (Exception e) {
      logger.severe("Erro lendo arquivo de propriedades do sistema");
      return false;
    }
    finally {
      if (in != null) {
        try {
          in.close();
        }
        catch (IOException e) {
          return false;
        }
      }
    }
    logger.info("Arquivo de propriedades: " + propertiesFile.getPath());
    return true;
  }

  /**
   * Executa todas as validaes cadastradas. Para cada validao, cria um log
   * handler para o arquivo especificado pela validao (se este no for
   * <code>null</code>) e o registra no logger global. Aps o processamento, o
   * handler da validao  descadastrado.
   * 
   * @return <code>true</code> se todas as validaes foram aplicadas com
   *         sucesso
   */
  private boolean run() {
    if (!readyToRun) {
      logger.separator(Level.SEVERE);
      logger.severe("*** ERRO NA CONFIGURAO DAS VALIDAES");
      return false;
    }

    boolean allOK = true;
    AbstractValidation lastValidation = null;
    try {
      for (AbstractValidation validation : validations) {
        boolean succeeded = validation.run(this);
        validation.reportStatus();
        lastValidation = validation;
        if (!succeeded) {
          allOK = false;
          break;
        }
      }
    }
    catch (Exception e) {
      logger.exception(e);
      allOK = false;
    }

    if (allOK) {
      logger.info("Validao da instalao: OK");
      /*
       * todos os patches foram bem-sucedidos, podemos remover todos os backups
       */
      removeBackups();
    }
    else {
      logger.severe("Validao da instalao: *** ERRO ***");
      Status status = lastValidation.getStatus();
      if (status == Status.BACKUP_FAILED) {
        /*
         * como o erro foi na gerao dos backups, podemos (e devemos)
         * remov-los
         */
        removeBackups();
      }
      else if (!validateOnly && status == Status.PATCH_FAILED) {
        /*
         * algum patch falhou; se no estvamos apenas validando, precisamos
         * fazer os rollbacks na ordem inversa.
         */
        if (rollback(lastValidation)) {
          logger.separator(Level.SEVERE);
          logger
            .severe("*** TODAS AS ATUALIZAES FORAM DESFEITAS COM SUCESSO");
          logger.severe("*** O SISTEMA NO PODE SER EXECUTADO NA VERSO ATUAL");
          logger
            .severe("*** CORRIJA O PROBLEMA OU REINSTALE A VERSO ANTERIOR");
        }
        else {
          logger.severe("*** ALGUMAS ATUALIZAES NO PUDERAM SER DESFEITAS");
          logger.severe("*** O SISTEMA NO PODE SER EXECUTADO NA VERSO ATUAL");
          logger.severe("*** NO  POSSVEL REINSTALAR A VERSO ANTERIOR");
          logger.severe("*** CONTACTE A EQUIPE DE DESENVOLVIMENTO");
        }
      }
      System.err.println();
      logger.toConsole("O registro das converses foi anexado ao arquivo ");
      logger.toConsole(logger.getLogFilePath());
    }
    logger.close();
    return allOK;
  }

  /**
   * Removemos todos os backups, de todas as validaes.
   */
  private void removeBackups() {
    try {
      for (AbstractValidation validation : validations) {
        validation.removeBackupDir();
      }
    }
    catch (Exception e) {
      logger.exception("Erro removendo backups", e);
    }
  }

  /**
   * Percorre a lista de validaes a partir de uma validao que falhou at o
   * incio, desfazendo as alteraes na ordem inversa em que foram aplicadas.
   * Caso algum rollback falhe o processo ser interrompido, pois o rollback N-1
   * pode no ser possvel se o rollback N no for bem-sucedido.
   * 
   * @param lastValidation ltima validao aplicada (neste caso, a que falhou)
   * @return <code>true</code> se todas as modificaes foram desfeitas com
   *         sucesso
   */
  private boolean rollback(AbstractValidation lastValidation) {
    try {
      logger.sectionSeparator(Level.INFO);
      int i = validations.indexOf(lastValidation);
      for (; i >= 0; i--) {
        AbstractValidation val = validations.get(i);
        String validationName = val.getClass().getSimpleName();
        logger.info(validationName + " : desfazendo alteraes");
        if (!val.rollback()) {
          logger.severe(validationName + " : ERRO");
          return false;
        }
        logger.info(validationName + " : OK");
        /*
         * rollback bem-sucedido, removemos o respectivo backup
         */
        val.removeBackupDir();
      }

    }
    catch (Exception e) {
      logger.exception("Erro restaurando backups de "
        + lastValidation.getClass().getSimpleName(), e);
      return false;
    }
    return true;
  }

  /**
   * Mtodo de entrada. Para cada execuo, cria uma nova instncia do validador
   * e executa todas as validaes.
   * <p>
   * Se todas as validaes forem aplicadas com sucesso, encerra a JVM com
   * cdigo 0, seno encerra com cdigo 1.
   * 
   * @param args - parmetros da linha de comando
   */
  public static void main(String[] args) {
    System.out.println();
    Validator validator = new Validator(args);
    int status = validator.run() ? 0 : 1;
    System.exit(status);
  }

  /**
   * Obtm o path para o diretrio temporrio da instalao.
   * 
   * @return path para o diretrio temporrio da instalao
   */
  public String getTempDirPath() {
    return instTempDirPath;
  }

  /**
   * Obtm o logger.
   * 
   * @return logger
   */
  public ValidationsLogger getLogger() {
    return logger;
  }

  /**
   * Retorna o valor associado a uma propriedade do sistema, ou
   * <code>null</code> caso esta no exista.
   * 
   * @param propName nome da propriedade
   * 
   * @return valor associado  propriedade do sistema, ou <code>null</code> caso
   *         esta no exista
   */
  public String getSystemProperty(String propName) {
    return systemProperties.getProperty(propName);
  }

  /**
   * Verifica se estamos apenas validando (i.e. no ser feito nenhum ajuste).
   * As validaes devem usar este mtodo para configurar seu modo de operao.
   * 
   * @return <code>true</code> se estamos apenas validando
   */
  public boolean isValidatingOnly() {
    return validateOnly;
  }

  /**
   * Indica se estamos em modo verbose. As validaes devem usar este mtodo
   * para configurar seu modo de operao.
   * 
   * @return <code>true</code> se estamos em modo verbose
   */
  public boolean isVerbose() {
    return verboseMode;
  }
}
