/*
 * $Author:$ $Date:$ $Release:$
 */
package csbase.logic.algorithms.parsers;

import static csbase.logic.algorithms.parameters.OutputFileParameterFactory.*;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java.io.StringReader;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.Map.Entry;

import junit.framework.TestCase;
import tecgraf.javautils.configurationmanager.ConfigurationManager;
import csbase.exception.ParseException;
import csbase.exception.algorithms.FormulaCreationException;
import csbase.logic.FileInfo;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.logic.algorithms.ExecutionType;
import csbase.logic.algorithms.parameters.BooleanColumn;
import csbase.logic.algorithms.parameters.BooleanParameter;
import csbase.logic.algorithms.parameters.Column;
import csbase.logic.algorithms.parameters.DoubleColumn;
import csbase.logic.algorithms.parameters.DoubleListParameter;
import csbase.logic.algorithms.parameters.DoubleParameter;
import csbase.logic.algorithms.parameters.EnumerationItem;
import csbase.logic.algorithms.parameters.EnumerationListParameter;
import csbase.logic.algorithms.parameters.EnumerationParameter;
import csbase.logic.algorithms.parameters.FileParameter;
import csbase.logic.algorithms.parameters.FileParameterMode;
import csbase.logic.algorithms.parameters.FileParameterPipeAcceptance;
import csbase.logic.algorithms.parameters.FormulaParameter;
import csbase.logic.algorithms.parameters.HiddenParameter;
import csbase.logic.algorithms.parameters.InputFileListParameter;
import csbase.logic.algorithms.parameters.InputFileParameter;
import csbase.logic.algorithms.parameters.IntegerColumn;
import csbase.logic.algorithms.parameters.IntegerListParameter;
import csbase.logic.algorithms.parameters.IntegerParameter;
import csbase.logic.algorithms.parameters.OutputFileParameter;
import csbase.logic.algorithms.parameters.OutputFileParameterFactory;
import csbase.logic.algorithms.parameters.Parameter;
import csbase.logic.algorithms.parameters.ParameterGroup;
import csbase.logic.algorithms.parameters.SimpleAlgorithmConfigurator;
import csbase.logic.algorithms.parameters.SimpleParameter;
import csbase.logic.algorithms.parameters.TableParameter;
import csbase.logic.algorithms.parameters.TextColumn;
import csbase.logic.algorithms.parameters.TextListParameter;
import csbase.logic.algorithms.parameters.TextParameter;
import csbase.logic.algorithms.parameters.ValidationExpression;
import csbase.logic.algorithms.parameters.conditions.AndOperatorCondition;
import csbase.logic.algorithms.parameters.conditions.Condition;
import csbase.logic.algorithms.parameters.conditions.GenericCondition;
import csbase.logic.algorithms.parameters.conditions.NotOperatorCondition;
import csbase.logic.algorithms.parameters.conditions.OrOperatorCondition;
import csbase.logic.algorithms.parameters.conditions.SimpleCondition;
import csbase.logic.algorithms.parameters.triggers.ChangeDefaultValueTrigger;
import csbase.logic.algorithms.parameters.triggers.ChangeFileTypeTrigger;
import csbase.logic.algorithms.parameters.triggers.ChangeLabelTrigger;
import csbase.logic.algorithms.parameters.triggers.ChangeMaximumForDoublesTrigger;
import csbase.logic.algorithms.parameters.triggers.ChangeMaximumForIntegersTrigger;
import csbase.logic.algorithms.parameters.triggers.ChangeMinimumForDoublesTrigger;
import csbase.logic.algorithms.parameters.triggers.ChangeMinimumForIntegersTrigger;
import csbase.logic.algorithms.parameters.triggers.DisableParameterTrigger;
import csbase.logic.algorithms.parameters.triggers.EnableParameterTrigger;
import csbase.logic.algorithms.parameters.triggers.HideParameterTrigger;
import csbase.logic.algorithms.parameters.triggers.SetVisibleEnumerationItemTrigger;
import csbase.logic.algorithms.parameters.triggers.ShowParameterTrigger;
import csbase.logic.algorithms.parameters.triggers.Trigger;
import csbase.server.services.algorithmservice.AlgorithmService;
import csbase.test.util.Pair;

/**
 * Testes unitrios para a clsse {@link SimpleAlgorithmParser}.
 */
public final class SimpleAlgorithmParserTest extends TestCase {

  /**
   * Formato padro da linha de comando.
   */
  private static final String DEFAULT_COMMAND_LINE_PATTERN =
    "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO";

  static {
    try {
      ConfigurationManager.createInstance();
    }
    catch (IllegalStateException e) {
      // O ConfigurationManager j foi criado.
    }
  }

  /**
   * Testando criar algoritmo.
   * 
   * @throws ParseException
   */
  public void testAlgorithmCreation() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar algoritmo sem comando.
   */
  public void testAlgorithmCreationWithNoCommand() {
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo/>";
    assertXml(xml);
  }

  /**
   * Testando criar algoritmo com descrio.
   * 
   * @throws ParseException
   */
  public void testAlgorithmCreationWithDescription() throws ParseException {
    String command = "Comando_do_algoritmo";
    String description = "";
    description += "Esta  a descrio do algoritmo de teste.\n";
    description += "Este elemento cria a descrio do algoritmo.";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, description, ExecutionType.SIMPLE,
        null, false, null, null);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<descricao>\n";
    xml += description;
    xml += "\n</descricao>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar algoritmo com abreviatura.
   * 
   * @throws ParseException
   */
  public void testAlgorithmCreationWithAbbreviation() throws ParseException {
    String command = "Comando_do_algoritmo";
    String abbreviation = "Abreviatura da verso";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE,
        abbreviation, false, null, null);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml +=
      "<algoritmo comando='" + command + "' abreviatura='" + abbreviation
        + "'/>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar algoritmo com fornecer identificador falso.
   * 
   * @throws ParseException
   */
  public void testAlgorithmCreationWithNoCommandIdentifier()
    throws ParseException {
    String command = "Comando_do_algoritmo";
    boolean provideId = false;
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        provideId, null, null);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml +=
      "<algoritmo comando='" + command + "' fornecer_identificador='"
        + provideId + "'>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar algoritmo com fornecer identificador verdadeiro.
   * 
   * @throws ParseException
   */
  public void testAlgorithmCreationWithCommandIdentifier()
    throws ParseException {
    String command = "Comando_do_algoritmo";
    boolean provideId = true;
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        provideId, null, null);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml +=
      "<algoritmo comando='" + command + "' fornecer_identificador='"
        + provideId + "'>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar algoritmo com elemento descrio vazio.
   * 
   * @throws ParseException
   */
  public void testAlgorithmCreationWithEmptyDescription() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<descricao>\n";
    xml += "</descricao>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar algoritmo com shell.
   * 
   * @throws ParseException
   */
  public void testAlgorithmCreationWithShell() throws ParseException {
    String command = "Comando_do_algoritmo";
    String shell = "Shell_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, shell, null);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "' shell='" + shell + "'>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar algoritmo com arquivo de entrada padro.
   * 
   * @throws ParseException
   */
  public void testAlgorithmCreationWithDefaultInputFile() throws ParseException {
    String command = "Comando_do_algoritmo";
    String parameterName = "Nome do Arquivo de Entrada";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, parameterName);
    String groupName = "Nome do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    expectedVersion.addGroup(group);
    String parameterLabel = "Rtulo do Arquivo de Entrada";
    String parameterDescription = "Descrio do Arquivo de Entrada";
    InputFileParameter inputFileParameter =
      new InputFileParameter(parameterName, parameterLabel,
        parameterDescription, false, DEFAULT_COMMAND_LINE_PATTERN, null,
        FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE);
    group.addParameter(inputFileParameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml +=
      "<algoritmo comando='" + command + "' arquivo_de_entrada_padrao='"
        + parameterName + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_entrada nome='" + parameterName + "' rotulo='"
        + parameterLabel + "' dica='" + parameterDescription + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar algoritmo com arquivo de entrada padro que no existe.
   */
  public void testAlgorithmCreationWithNonExistantDefaultInputFile() {
    String command = "Comando_do_algoritmo";
    String otherParameterName = "Outro parmetro de entrada";
    String groupName = "Nome do grupo";
    String parameterName = "Nome do Arquivo de Entrada";
    String parameterLabel = "Rtulo do Arquivo de Entrada";
    String parameterDescription = "Descrio do Arquivo de Entrada";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml +=
      "<algoritmo comando='" + command + "' arquivo_de_entrada_padrao='"
        + otherParameterName + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_entrada nome='" + parameterName + "' rotulo='"
        + parameterLabel + "' dica='" + parameterDescription + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar algoritmo com tipo de execuo simples.
   * 
   * @throws ParseException
   */
  public void testAlgorithmCreationWithSimpleExecution() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "' tipo_de_execucao='simples'>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testAlgorithmCreationWithExitCode() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    expectedVersion.setHasExitCode(true);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml +=
      "<algoritmo comando='" + command + "' capturar_codigo_de_saida='sim'>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testAlgorithmCreationWithNoExitCode() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    expectedVersion.setHasExitCode(false);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml +=
      "<algoritmo comando='" + command + "' capturar_codigo_de_saida='nao'>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar algoritmo com tipo de execuo paralela.
   * 
   * @throws ParseException
   */
  public void testAlgorithmCreationWithParallelExecution()
    throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.PARALLEL, null,
        false, null, null);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "' tipo_de_execucao='paralela'>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar algoritmo com tipo de execuo mltipla.
   * 
   * @throws ParseException
   */
  public void testAlgorithmCreationWithMultipleExecution()
    throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.MULTIPLE, null,
        false, null, null);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "' tipo_de_execucao='multipla'>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar algoritmo com tipo de execuo distribuda.
   * 
   * @throws ParseException
   */
  public void testAlgorithmCreationWithDistributedExecution()
    throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.DISTRIBUTED,
        null, false, null, null);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml +=
      "<algoritmo comando='" + command + "' tipo_de_execucao='distribuida'>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar um algoritmo com atributos que no so conhecidos.
   */
  public void testAlgorithmCreationWithUnknownAttributes() {
    String command = "Comando_do_algoritmo";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "' desconhecido='valor'>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar um algoritmo com descrio e atributos na descrio.
   */
  public void testAlgorithmCreationWithUnknownDescriptionAttributes() {
    String command = "Comando_do_algoritmo";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<descricao desconhecido='valor'>\n";
    xml += "conteudo";
    xml += "</descricao>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar um algoritmo com descrio e elementos-filhos da descrio.
   */
  public void testAlgorithmCreationWithDescriptionChildElements() {
    String command = "Comando_do_algoritmo";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<descricao>\n";
    xml += "conteudo";
    xml += "<filho/>";
    xml += "</descricao>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar algoritmo definindo um parmetro e com todos os tipos de
   * parmetros.
   * 
   * @throws ParseException
   */
  public void testAlgorithmCreationWithAllParameterTypes()
    throws ParseException {
    String commandLinePattern =
      "Nome: $NOME_DO_PARAMETRO - Valor: $VALOR_DO_PARAMETRO";
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command);
    String groupName = "Parmetros";
    ParameterGroup group = new ParameterGroup(groupName);
    expectedVersion.addGroup(group);
    String booleanParameterName = "Booleano";
    String booleanParameterLabel = "Booleano";
    String booleanParameterDescription = "Booleano";
    BooleanParameter booleanParameter =
      new BooleanParameter(booleanParameterName, booleanParameterLabel,
        booleanParameterDescription, false, commandLinePattern, null, null);
    group.addParameter(booleanParameter);
    String textParameterName = "Texto";
    String textParameterLabel = "Texto";
    String textParameterDescription = "Texto";
    TextParameter textParameter =
      new TextParameter(textParameterName, textParameterLabel,
        textParameterDescription, null, false, commandLinePattern, null);
    group.addParameter(textParameter);
    String textListParameterName = "Lista_de_Texto";
    String textListParameterLabel = "Lista de Texto";
    String textListParameterDescription = "Lista de Texto";
    TextListParameter textListParameter =
      new TextListParameter(textListParameterName, textListParameterLabel,
        textListParameterDescription, false, commandLinePattern, null);
    group.addParameter(textListParameter);
    String integerParameterName = "Inteiro";
    String integerParameterLabel = "Inteiro";
    String integerParameterDescription = "Inteiro";
    IntegerParameter integerParameter =
      new IntegerParameter(integerParameterName, integerParameterLabel,
        integerParameterDescription, null, false, commandLinePattern, null,
        null);
    group.addParameter(integerParameter);
    String integerListParameterName = "Lista_de_Inteiros";
    String integerListParameterLabel = "Lista de Inteiros";
    String integerListParameterDescription = "Lista de Inteiros";
    IntegerListParameter integerListParameter =
      new IntegerListParameter(integerListParameterName,
        integerListParameterLabel, integerListParameterDescription, false,
        commandLinePattern, null, null);
    group.addParameter(integerListParameter);
    String doubleParameterName = "Real";
    String doubleParameterLabel = "Real";
    String doubleParameterDescription = "Real";
    DoubleParameter doubleParameter =
      new DoubleParameter(doubleParameterName, doubleParameterLabel,
        doubleParameterDescription, null, false, commandLinePattern, null,
        false, null, false);
    group.addParameter(doubleParameter);
    String doubleListParameterName = "Lista_de_Reais";
    String doubleListParameterLabel = "Lista de Reais";
    String doubleListParameterDescription = "Lista de Reais";
    DoubleListParameter doubleListParameter =
      new DoubleListParameter(doubleListParameterName,
        doubleListParameterLabel, doubleListParameterDescription, false,
        commandLinePattern, null, false, null, false);
    group.addParameter(doubleListParameter);
    String tableParameterName = "Tabela";
    String tableParameterLabel = "Tabela";
    String tableParameterDescription = "Tabela";
    String columnId = "Coluna";
    String columnName = "Coluna";
    TextColumn column =
      new TextColumn(columnId, columnName, null, false, true, null);
    List<Column<?>> columns = Arrays.<Column<?>> asList(column);
    TableParameter tableParameter =
      new TableParameter(tableParameterName, tableParameterLabel,
        tableParameterDescription, false, commandLinePattern, columns, false,
        false, null, null);
    group.addParameter(tableParameter);
    String inputFileParameterName = "Arquivo_de_entrada";
    String inputFileParameterLabel = "Arquivo de entrada";
    String inputFileParameterDescription = "Arquivo de entrada";
    InputFileParameter inputFileParameter =
      new InputFileParameter(inputFileParameterName, inputFileParameterLabel,
        inputFileParameterDescription, false, commandLinePattern, null,
        FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE);
    group.addParameter(inputFileParameter);
    String outputFileParameterName = "Arquivo_de_saida";
    String outputFileParameterLabel = "Arquivo de sada";
    String outputFileParameterDescription = "Arquivo de sada";
    OutputFileParameter outputFileParameter =
      new OutputFileParameter(outputFileParameterName,
        outputFileParameterLabel, outputFileParameterDescription, false,
        commandLinePattern, null, FileParameterMode.REGULAR_FILE,
        FileParameterPipeAcceptance.TRUE, false);
    group.addParameter(outputFileParameter);
    OutputFileParameter logFileParameter =
      OutputFileParameter.createLogFile(commandLinePattern);
    group.addParameter(logFileParameter);
    String enumerationItemId = "Item";
    String enumerationItemLabel = "Item";
    String enumerationItemValue = "Item";
    EnumerationItem enumerationItem =
      new EnumerationItem(enumerationItemId, enumerationItemLabel,
        enumerationItemValue, null);
    String enumerationParameterName = "Enumeracao";
    String enumerationParameterLabel = "Enumerao";
    String enumerationParameterDescription = "Enumerao";
    boolean enumerationMustSortItems = true;
    EnumerationParameter enumerationParameter =
      new EnumerationParameter(enumerationParameterName,
        enumerationParameterLabel, enumerationParameterDescription,
        enumerationItem, commandLinePattern, Arrays.asList(enumerationItem),
        enumerationMustSortItems);
    group.addParameter(enumerationParameter);
    String enumerationListParameterName = "Lista_de_Enumeracao";
    String enumerationListParameterLabel = "Lista de Enumerao";
    String enumerationListParameterDescription = "Lista de Enumerao";
    EnumerationListParameter enumerationListParameter =
      new EnumerationListParameter(enumerationListParameterName,
        enumerationListParameterLabel, enumerationListParameterDescription,
        false, commandLinePattern);
    enumerationListParameter.setItems(Arrays.asList(enumerationItem));
    group.addParameter(enumerationListParameter);
    String hiddenParameterName = "Oculto";
    String hiddenParameterValue = "Valor_do_Oculto";
    HiddenParameter hiddenParameter =
      new HiddenParameter(hiddenParameterName, hiddenParameterValue,
        commandLinePattern);
    expectedVersion.addHiddenParameter(hiddenParameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<booleano nome='" + booleanParameterName + "' rotulo='"
        + booleanParameterLabel + "' dica='" + booleanParameterDescription
        + "'/>";
    xml +=
      "<texto nome='" + textParameterName + "' rotulo='" + textParameterLabel
        + "' dica='" + textParameterDescription + "'/>";
    xml +=
      "<lista_de_texto nome='" + textListParameterName + "' rotulo='"
        + textListParameterLabel + "' dica='" + textListParameterDescription
        + "'/>";
    xml +=
      "<inteiro nome='" + integerParameterName + "' rotulo='"
        + integerParameterLabel + "' dica='" + integerParameterDescription
        + "'/>";
    xml +=
      "<lista_de_inteiros nome='" + integerListParameterName + "' rotulo='"
        + integerListParameterLabel + "' dica='"
        + integerListParameterDescription + "'/>";
    xml +=
      "<real nome='" + doubleParameterName + "' rotulo='"
        + doubleParameterLabel + "' dica='" + doubleParameterDescription
        + "'/>";
    xml +=
      "<lista_de_reais nome='" + doubleListParameterName + "' rotulo='"
        + doubleListParameterLabel + "' dica='"
        + doubleListParameterDescription + "'/>";
    xml +=
      "<tabela nome='" + tableParameterName + "' rotulo='"
        + tableParameterLabel + "' dica='" + tableParameterDescription + "'>\n";
    xml +=
      "<coluna_de_texto id='" + columnId + "' rotulo='" + columnName + "'/>\n";
    xml += "</tabela>\n";
    xml +=
      "<arquivo_de_entrada nome='" + inputFileParameterName + "' rotulo='"
        + inputFileParameterLabel + "' dica='" + inputFileParameterDescription
        + "'/>";
    xml +=
      "<arquivo_de_saida nome='" + outputFileParameterName + "' rotulo='"
        + outputFileParameterLabel + "' dica='"
        + outputFileParameterDescription + "'/>";
    xml += "<arquivo_de_log/>";
    xml +=
      "<enumeracao nome='" + enumerationParameterName + "' rotulo='"
        + enumerationParameterLabel + "' dica='"
        + enumerationParameterDescription + "' padrao='" + enumerationItemId
        + "' selecao_multipla='nao'>\n";
    xml +=
      "<item_de_enumeracao rotulo='" + enumerationItemLabel + "' id='"
        + enumerationItemId + "' valor='" + enumerationItemValue + "'/>";
    xml += "</enumeracao>\n";
    xml +=
      "<enumeracao nome='" + enumerationListParameterName + "' rotulo='"
        + enumerationListParameterLabel + "' dica='"
        + enumerationListParameterDescription + "' selecao_multipla='sim'>\n";
    xml +=
      "<item_de_enumeracao rotulo='" + enumerationItemLabel + "' id='"
        + enumerationItemId + "' valor='" + enumerationItemValue + "'/>";
    xml += "</enumeracao>\n";
    xml += "</grupo>\n";
    xml +=
      "<oculto nome='" + hiddenParameterName + "' valor='"
        + hiddenParameterValue + "'/>";
    xml += "<formato_no_comando>\n";
    xml += commandLinePattern;
    xml += "</formato_no_comando>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar grupos de parmetros.
   * 
   * @throws ParseException
   */
  public void testSimpleParameterGroup() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName1 = "Rtulo do grupo 1";
    String groupName2 = "Rtulo do grupo 2";
    ParameterGroup group1 = new ParameterGroup(groupName1);
    ParameterGroup group2 = new ParameterGroup(groupName2);
    expectedVersion.addGroup(group1);
    expectedVersion.addGroup(group2);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName1 + "'/>";
    xml += "<grupo rotulo='" + groupName2 + "'/>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar grupos de parmetros sem rtulo.
   */
  public void testParameterGroupWithNoLabel() {
    String command = "Comando_do_algoritmo";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo/>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar grupos de parmetros com rtulos repetidos.
   */
  public void testParameterGroupsWithSameLabel() {
    String command = "Comando_do_algoritmo";
    String groupName = "Grupo 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'/>";
    xml += "<grupo rotulo='" + groupName + "'/>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar grupos dentro de grupos.
   * 
   * @throws ParseException
   */
  public void testParameterGroupWithinAnotherGroup() throws ParseException {
    String command = "Comando";
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName1 = "Group1";
    ParameterGroup group1 = new ParameterGroup(groupName1);
    version.addGroup(group1);
    String groupName11 = "Group1.1";
    ParameterGroup group11 = new ParameterGroup(groupName11);
    group1.addParameter(group11);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + group1 + "'>";
    xml += "<grupo rotulo='" + group11 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  /**
   * Testando criar grupos com atributo no conhecido.
   */
  public void testParameterGroupWithUnknownAttribute() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "' desconhecido='valor'/>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar grupos com elementos-filhos desconhecidos.
   */
  public void testParameterGroupWithUnknownChildElement() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml += "<filho/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros booleanos.
   * 
   * @throws ParseException
   */
  public void testSimpleBooleanParameters() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean parameterDefaultValue1 = false;
    String trueCommand1 = "1";
    String falseCommand1 = "0";
    BooleanParameter parameter1 =
      new BooleanParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1,
        DEFAULT_COMMAND_LINE_PATTERN, falseCommand1, trueCommand1);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String trueCommand2 = "1";
    String falseCommand2 = "0";
    boolean parameterDefaultValue2 = false;
    BooleanParameter parameter2 =
      new BooleanParameter(parameterName2, parameterLabel2,
        parameterDescription2, parameterDefaultValue2,
        DEFAULT_COMMAND_LINE_PATTERN, falseCommand2, trueCommand2);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<booleano nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<booleano nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros booleanos repetidos.
   */
  public void testDuplicateBooleanParameters() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<booleano nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<booleano nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros booleanos com valor-padro.
   * 
   * @throws ParseException
   */
  public void testBooleanParameterWithDefaultValue() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean parameterDefaultValue1 = true;
    String trueCommand1 = "1";
    String falseCommand1 = "0";
    BooleanParameter parameter1 =
      new BooleanParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1,
        DEFAULT_COMMAND_LINE_PATTERN, falseCommand1, trueCommand1);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String trueCommand2 = "1";
    String falseCommand2 = "0";
    boolean parameterDefaultValue2 = false;
    BooleanParameter parameter2 =
      new BooleanParameter(parameterName2, parameterLabel2,
        parameterDescription2, parameterDefaultValue2,
        DEFAULT_COMMAND_LINE_PATTERN, falseCommand2, trueCommand2);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<booleano nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='"
        + parameterDefaultValue1 + "'/>";
    xml +=
      "<booleano nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "' padrao='"
        + parameterDefaultValue2 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros booleanos com valor-padro de tipo no-booleano.
   */
  public void testBooleanParameterWithNonBooleanDefaultValue() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<booleano nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1
        + "' padrao='Valor no booleano'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros booleanos sem nome.
   */
  public void testBooleanParameterWithNoName() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean parameterDefaultValue1 = true;
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<booleano rotulo='" + parameterLabel1 + "' dica='"
        + parameterDescription1 + "' padrao='" + parameterDefaultValue1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros booleanos sem rtulo.
   */
  public void testBooleanParameterWithNoLabel() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean parameterDefaultValue1 = true;
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<booleano nome='" + parameterName1 + "' dica='" + parameterDescription1
        + "' padrao='" + parameterDefaultValue1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros booleanos sem descrio.
   */
  public void testBooleanParameterWithNoDescription() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    boolean parameterDefaultValue1 = true;
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<booleano rotulo='" + parameterLabel1 + "' nome='" + parameterName1
        + "' padrao='" + parameterDefaultValue1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros booleanos com comando verdadeiro e falso.
   * 
   * @throws ParseException
   */
  public void testBooleanParameterWithCustomTrueAndFalseCommands()
    throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean parameterDefaultValue1 = false;
    String trueCommand1 = "SIM";
    String falseCommand1 = "NAO";
    BooleanParameter parameter1 =
      new BooleanParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1,
        DEFAULT_COMMAND_LINE_PATTERN, falseCommand1, trueCommand1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<booleano nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='"
        + parameterDefaultValue1 + "' falso='" + falseCommand1
        + "' verdadeiro='" + trueCommand1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros booleanos ocultos sem valor.
   */
  public void testHiddenBooleanParameterWithNoValue() {
    String command = "Comando";
    String parameterGroupName = "Grupo";
    String parameterName = "Nome do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + parameterGroupName + "'>\n";
    xml += "<booleano nome='" + parameterName + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros booleanos ocultos com valor de tipo no-booleano.
   */
  public void testHiddenBooleanParameterWithNonBooleanValue() {
    String command = "Comando";
    String parameterGroupName = "Grupo";
    String parameterName = "Nome do parmetro";
    String parameterValue = "Valor no-booleano";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + parameterGroupName + "'>\n";
    xml +=
      "<booleano nome='" + parameterName + "' valor='" + parameterValue
        + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar um parmetro booleano com atributos desconhecidos.
   */
  public void testBooleanParameterWithUnknownAttributes() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro 1";
    String parameterLabel = "Rtulo do parmetro 1";
    String parameterDescription = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<booleano nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "' desconhecido='valor'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros booleanos com padro do formato no comando
   * exibindo o nome e o valor.
   * 
   * @throws ParseException
   */
  public void testBooleanParameterWithCustomCommandPatternShowingNameAndValue()
    throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean parameterDefaultValue1 = false;
    String trueCommand1 = "1";
    String falseCommand1 = "0";
    BooleanParameter parameter1 =
      new BooleanParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1,
        "Nome:$NOME_DO_PARAMETRO Valor:$VALOR_DO_PARAMETRO", falseCommand1,
        trueCommand1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<booleano nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'>\n";
    xml +=
      "<formato_no_comando>Nome:$NOME_DO_PARAMETRO Valor:$VALOR_DO_PARAMETRO</formato_no_comando>\n";
    xml += "</booleano>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros booleanos com padro do formato no comando sem
   * nome e sem valor.
   * 
   * @throws ParseException
   */
  public void testBooleanParameterWithCustomCommandPatternWithNoNameOrValue()
    throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean parameterDefaultValue1 = false;
    String trueCommand1 = "1";
    String falseCommand1 = "0";
    BooleanParameter parameter1 =
      new BooleanParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, null, falseCommand1,
        trueCommand1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<booleano nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'>\n";
    xml += "<formato_no_comando/>\n";
    xml += "</booleano>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros reais.
   * 
   * @throws ParseException
   */
  public void testSimpleDoubleParameters() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Double parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    Double maximum1 = null;
    boolean includeMaximum1 = true;
    Double minimum1 = null;
    boolean includeMinimum1 = true;
    DoubleParameter parameter1 =
      new DoubleParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        DEFAULT_COMMAND_LINE_PATTERN, maximum1, includeMaximum1, minimum1,
        includeMinimum1);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    Double parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    Double maximum2 = null;
    boolean includeMaximum2 = true;
    Double minimum2 = null;
    boolean includeMinimum2 = true;
    DoubleParameter parameter2 =
      new DoubleParameter(parameterName2, parameterLabel2,
        parameterDescription2, parameterDefaultValue2, isOptional2,
        DEFAULT_COMMAND_LINE_PATTERN, maximum2, includeMaximum2, minimum2,
        includeMinimum2);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>\n";
    xml +=
      "<real nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros reais repetidos.
   */
  public void testDuplicateDoubleParameters() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>\n";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Criar parmetros reais sem nome.
   */
  public void testDoubleParameterWithNoName() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<real rotulo='" + parameterLabel1 + "' dica='" + parameterDescription1
        + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Criar parmetros reais sem rtulo.
   */
  public void testDoubleParameterWithNoLabel() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<real nome='" + parameterName1 + "' dica='" + parameterDescription1
        + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Criar parmetros reais sem descrio.
   */
  public void testDoubleParameterWithNoDescription() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Criar parmetros reais sem valor-padro.
   * 
   * @throws ParseException
   */
  public void testDoubleParameterWithNoDefaultValue() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Double parameterDefaultValue1 = new Double(100);
    boolean isOptional1 = false;
    Double maximum1 = null;
    boolean includeMaximum1 = true;
    Double minimum1 = null;
    boolean includeMinimum1 = true;
    DoubleParameter parameter1 =
      new DoubleParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        DEFAULT_COMMAND_LINE_PATTERN, maximum1, includeMaximum1, minimum1,
        includeMinimum1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='"
        + parameterDefaultValue1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Criar parmetros reais com valor-padro de tipo no-real.
   */
  public void testDoubleParameterWithNonDoubleDefaultValue() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterDefaultValue1 = "Valor no real";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='"
        + parameterDefaultValue1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Criar parmetros reais com valor-padro de tipo no-real.
   * 
   * @throws ParseException
   */
  public void testDoubleParameterWithNullDefaultValue() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Double parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    Double maximum1 = null;
    boolean includeMaximum1 = true;
    Double minimum1 = new Double(10);
    boolean includeMinimum1 = true;
    DoubleParameter parameter1 =
      new DoubleParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        DEFAULT_COMMAND_LINE_PATTERN, maximum1, includeMaximum1, minimum1,
        includeMinimum1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' minimo='" + minimum1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros reais com mnimo de tipo no-real.
   */
  public void testDoubleParameterWithNonDoubleMinimumValue() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String minimum1 = "Valor no real";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' minimo='" + minimum1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros reais com mximo.
   * 
   * @throws ParseException
   */
  public void testDoubleParameterWithMaximumValue() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Double parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    Double maximum1 = new Double(1000);
    boolean includeMaximum1 = true;
    Double minimum1 = null;
    boolean includeMinimum1 = true;
    DoubleParameter parameter1 =
      new DoubleParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        DEFAULT_COMMAND_LINE_PATTERN, maximum1, includeMaximum1, minimum1,
        includeMinimum1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' maximo='" + maximum1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros reais com mximo com valor do tipo no-real.
   */
  public void testDoubleParameterWithNonDoubleMaximumValue() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String maximum1 = "Valor no real";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' maximo='" + maximum1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Criar parmetros reais com mnimo maior do que o mximo.
   */
  public void testDoubleParameterWithMinimumValueGreaterThanMaximumValue() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Double minimum1 = new Double(1);
    Double maximum1 = new Double(-1);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' minimo='" + minimum1
        + "' maximo='" + maximum1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Criar parmetros reais com valor-padro maior do que o mximo.
   */
  public void testDoubleParameterWithDefaultValueGreaterThanMaximumValue() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Double defaultValue1 = new Double(20);
    Double minimum1 = new Double(1);
    Double maximum1 = new Double(10);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' minimo='" + minimum1
        + "' maximo='" + maximum1 + "' padrao='" + defaultValue1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Criar parmetros reais com valor-padro menor do que o mnimo.
   */
  public void testDoubleParameterWithDefaultValueSmallerThanMinimumValue() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Double defaultValue1 = new Double(20);
    Double minimum1 = new Double(1);
    Double maximum1 = new Double(10);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' minimo='" + minimum1
        + "' maximo='" + maximum1 + "' padrao='" + defaultValue1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Criar parmetros reais com "incluir mximo" falso.
   * 
   * @throws ParseException
   */
  public void testDoubleParameterWithNonInclusiveMaximumValue()
    throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Double parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    Double maximum1 = new Double(1000);
    boolean includeMaximum1 = false;
    Double minimum1 = null;
    boolean includeMinimum1 = true;
    DoubleParameter parameter1 =
      new DoubleParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        DEFAULT_COMMAND_LINE_PATTERN, maximum1, includeMaximum1, minimum1,
        includeMinimum1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' maximo='" + maximum1
        + "' incluir_maximo='" + includeMaximum1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Criar parmetros reais com "incluir mximo" verdadeiro.
   * 
   * @throws ParseException
   */
  public void testDoubleParameterWithInclusiveMaximumValue()
    throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Double parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    Double maximum1 = new Double(1000);
    boolean includeMaximum1 = true;
    Double minimum1 = null;
    boolean includeMinimum1 = true;
    DoubleParameter parameter1 =
      new DoubleParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        DEFAULT_COMMAND_LINE_PATTERN, maximum1, includeMaximum1, minimum1,
        includeMinimum1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' maximo='" + maximum1
        + "' incluir_maximo='" + includeMaximum1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Criar parmetros reais com "incluir mximo" de tipo no-booleano.
   */
  public void testDoubleParameterWithNonBooleanIncludeMaximumAttributeValue() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String includeMaximum1 = "Valor no booleano";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' incluir_maximo='"
        + includeMaximum1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Criar parmetros reais com "incluir mnimo" falso.
   * 
   * @throws ParseException
   */
  public void testDoubleParameterWithNonInclusiveMinimumValue()
    throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Double parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    Double maximum1 = null;
    boolean includeMaximum1 = true;
    Double minimum1 = new Double(1000);
    boolean includeMinimum1 = false;
    DoubleParameter parameter1 =
      new DoubleParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        DEFAULT_COMMAND_LINE_PATTERN, maximum1, includeMaximum1, minimum1,
        includeMinimum1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' minimo='" + minimum1
        + "' incluir_minimo='" + includeMinimum1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Criar parmetros reais com incluir mnimo verdadeiro.
   * 
   * @throws ParseException
   */
  public void testDoubleParameterWithInclusiveMinimumValue()
    throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Double parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    Double maximum1 = null;
    boolean includeMaximum1 = true;
    Double minimum1 = new Double(1000);
    boolean includeMinimum1 = true;
    DoubleParameter parameter1 =
      new DoubleParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        DEFAULT_COMMAND_LINE_PATTERN, maximum1, includeMaximum1, minimum1,
        includeMinimum1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' minimo='" + minimum1
        + "' incluir_minimo='" + includeMinimum1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Criar parmetros do tipo real com incluir mnimo do tipo no-booleano.
   */
  public void testDoubleParameterWithNonBooleanIncludeMinimumAttributeValue() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String includeMinimum1 = "Valor no booleano";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' incluir_maximo='"
        + includeMinimum1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Criar parmetro real com atributo opcional verdadeiro.
   * 
   * @throws ParseException
   */
  public void testOptionalDoubleParameter() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Double parameterDefaultValue1 = null;
    boolean isOptional1 = true;
    Double maximum1 = null;
    boolean includeMaximum1 = true;
    Double minimum1 = null;
    boolean includeMinimum1 = true;
    DoubleParameter parameter1 =
      new DoubleParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        DEFAULT_COMMAND_LINE_PATTERN, maximum1, includeMaximum1, minimum1,
        includeMinimum1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' opcional='" + isOptional1
        + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Criar parmetro real com atributo opcional falso.
   * 
   * @throws ParseException
   */
  public void testNonOptionalDoubleParameter() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Double parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    Double maximum1 = null;
    boolean includeMaximum1 = true;
    Double minimum1 = null;
    boolean includeMinimum1 = true;
    DoubleParameter parameter1 =
      new DoubleParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        DEFAULT_COMMAND_LINE_PATTERN, maximum1, includeMaximum1, minimum1,
        includeMinimum1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' opcional='" + isOptional1
        + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Criar parmetro real com atributo opcional do tipo no-booleano.
   */
  public void testDoubleParameterWithNonBooleanOptionalAttributeValue() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String isOptional1 = "Valor no booleano";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' opcional='" + isOptional1
        + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Criar parmetro real com valor mximo igual ao valor mnimo.
   * 
   * @throws ParseException
   */
  public void testDoubleParameterWithSameMaximumAndMinimumValues()
    throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Double maximum1 = new Double(10);
    Double minimum1 = maximum1;
    DoubleParameter parameter =
      new DoubleParameter(parameterName1, parameterLabel1,
        parameterDescription1, null, false, DEFAULT_COMMAND_LINE_PATTERN,
        maximum1, true, minimum1, true);
    group.addParameter(parameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' maximo='" + maximum1
        + "' minimo='" + minimum1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  /**
   * Testando criar parmetros reais com atributos desconhecidos.
   */
  public void testDoubleParameterWithUnknownAttributes() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' desconhecido='valor'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros reais com padro do formato no comando exibindo o
   * nome e o valor.
   * 
   * @throws ParseException
   */
  public void testDoubleParameterWithCommandPatternShowingNameAndValue()
    throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Double parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    Double maximum1 = null;
    boolean includeMaximum1 = true;
    Double minimum1 = null;
    boolean includeMinimum1 = true;
    DoubleParameter parameter1 =
      new DoubleParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        "Nome:$NOME_DO_PARAMETRO Valor:$VALOR_DO_PARAMETRO", maximum1,
        includeMaximum1, minimum1, includeMinimum1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'>\n";
    xml +=
      "<formato_no_comando>Nome:$NOME_DO_PARAMETRO Valor:$VALOR_DO_PARAMETRO</formato_no_comando>\n";
    xml += "</real>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros reais com padro do formato no comando sem nome e
   * sem valor.
   * 
   * @throws ParseException
   */
  public void testDoubleParameterWithCustomCommandPatternWithNoNameOrValue()
    throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Double parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    Double maximum1 = null;
    boolean includeMaximum1 = true;
    Double minimum1 = null;
    boolean includeMinimum1 = true;
    DoubleParameter parameter1 =
      new DoubleParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1, null,
        maximum1, includeMaximum1, minimum1, includeMinimum1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'>\n";
    xml += "<formato_no_comando/>\n";
    xml += "</real>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetro inteiros.
   * 
   * @throws ParseException
   */
  public void testSimpleIntegerParameter() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Integer parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    Integer maximum1 = null;
    Integer minimum1 = null;
    IntegerParameter parameter1 =
      new IntegerParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        DEFAULT_COMMAND_LINE_PATTERN, maximum1, minimum1);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    Integer parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    Integer maximum2 = null;
    Integer minimum2 = null;
    IntegerParameter parameter2 =
      new IntegerParameter(parameterName2, parameterLabel2,
        parameterDescription2, parameterDefaultValue2, isOptional2,
        DEFAULT_COMMAND_LINE_PATTERN, maximum2, minimum2);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<inteiro nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<inteiro nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Criar parmetro inteiro repetido.
   */
  public void testDuplicateIntegerParameter() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<inteiro nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<inteiro nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Criar parmetro inteiro sem atributo nome.
   */
  public void testIntegerParameterWithNoName() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<inteiro rotulo='" + parameterLabel1 + "' dica='"
        + parameterDescription1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Criar parmetro inteiro sem atributo rtulo.
   */
  public void testIntegerParameterWithNoLabel() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<inteiro nome='" + parameterName1 + "' dica='" + parameterDescription1
        + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Criar parmetro inteiro sem atributo descrio.
   */
  public void testIntegerParameterWithNoDescription() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<inteiro nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Criar parmetro inteiro com valor-padro.
   * 
   * @exception ParseException Em caso de erro.
   */
  public void testIntegerParameterWithDefaultValue() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Integer parameterDefaultValue1 = new Integer(100);
    boolean isOptional1 = false;
    Integer maximum1 = null;
    Integer minimum1 = null;
    IntegerParameter parameter1 =
      new IntegerParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        DEFAULT_COMMAND_LINE_PATTERN, maximum1, minimum1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<inteiro nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='"
        + parameterDefaultValue1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Criar parmetro inteiro com valor-padro no-inteiro.
   */
  public void testIntegerParameterWithNonIntegerDefaultValue() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterDefaultValue1 = "Valor no inteiro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<inteiro nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='"
        + parameterDefaultValue1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Criar parmetro inteiro com atributo mnimo.
   * 
   * @throws ParseException
   */
  public void testIntegerParameterWithMinimumValue() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Integer parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    Integer maximum1 = null;
    Integer minimum1 = new Integer(10);
    IntegerParameter parameter1 =
      new IntegerParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        DEFAULT_COMMAND_LINE_PATTERN, maximum1, minimum1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<inteiro nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' minimo='" + minimum1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros inteiros com mnimo de tipo no-inteiro.
   */
  public void testIntegerParameterWithNonIntegerMinimumValue() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String minimum1 = "Valor no inteiro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<inteiro nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' minimo='" + minimum1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros inteiros com mximo.
   * 
   * @exception ParseException Em caso de erro.
   */
  public void testIntegerParameterWithMaximuValue() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Integer parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    Integer maximum1 = new Integer(1000);
    Integer minimum1 = null;
    IntegerParameter parameter1 =
      new IntegerParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        DEFAULT_COMMAND_LINE_PATTERN, maximum1, minimum1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<inteiro nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' maximo='" + maximum1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros inteiros com mximo com valor no-inteiro.
   */
  public void testIntegerParameterWithNonIntegerMaximumValue() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String maximum1 = "Valor no inteiro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<inteiro nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' maximo='" + maximum1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros inteiros com mnimo igual ao mximo.
   * 
   * @throws ParseException
   */
  public void testIntegerParameterWithSameMinimumAndMaximumValues()
    throws ParseException {
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator("Comando", null, ExecutionType.SIMPLE, null,
        false, null, null);
    ParameterGroup group = new ParameterGroup("Grupo");
    Integer maximum;
    Integer minimum;
    maximum = minimum = new Integer(100);
    IntegerParameter parameter =
      new IntegerParameter("Nome do parmetro", "Rtulo do parmetro",
        "Descrio do parmetro", null, false, DEFAULT_COMMAND_LINE_PATTERN,
        maximum, minimum);
    group.addParameter(parameter);
    version.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + version.getCommandBinaryName() + "'>\n";
    xml += "<grupo rotulo='" + group.getName() + "'>";
    xml +=
      "<inteiro nome='" + parameter.getName() + "' rotulo='"
        + parameter.getLabel() + "' dica='" + parameter.getDescription()
        + "' maximo='" + maximum + "' minimo='" + minimum + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  /**
   * Testando criar parmetros inteiros com mnimo maior do que o mximo.
   */
  public void testIntegerParameterWithMininumValueGreaterThanMaximumValue() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Integer minimum1 = new Integer(1);
    Integer maximum1 = new Integer(-1);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<inteiro nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' minimo='" + minimum1
        + "' maximo='" + maximum1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros inteiros com valor-padro maior do que o mximo.
   */
  public void testIntegerParameterWithDefaultValueGreaterThanMaximumValue() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Integer defaultValue1 = new Integer(20);
    Integer minimum1 = new Integer(1);
    Integer maximum1 = new Integer(10);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<inteiro nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' minimo='" + minimum1
        + "' maximo='" + maximum1 + "' padrao='" + defaultValue1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros inteiros com valor-padro menor do que o valor
   * mnimo.
   */
  public void testIntegerParameterWithDefaultValueSmallerThanMinimumValue() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Integer defaultValue1 = new Integer(-2);
    Integer minimum1 = new Integer(1);
    Integer maximum1 = new Integer(10);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<inteiro nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' minimo='" + minimum1
        + "' maximo='" + maximum1 + "' padrao='" + defaultValue1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros inteiros opcional falso.
   * 
   * @throws ParseException
   */
  public void testNonOptionalIntegerParameter() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Integer parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    Integer maximum1 = null;
    Integer minimum1 = null;
    IntegerParameter parameter1 =
      new IntegerParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        DEFAULT_COMMAND_LINE_PATTERN, maximum1, minimum1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<inteiro nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' opcional='" + isOptional1
        + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Tenstanoid criar parmetros inteiros opcional verdadeiro.
   * 
   * @throws ParseException
   */
  public void testOptionalIntegerParameter17() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Integer parameterDefaultValue1 = null;
    boolean isOptional1 = true;
    Integer maximum1 = null;
    Integer minimum1 = null;
    IntegerParameter parameter1 =
      new IntegerParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        DEFAULT_COMMAND_LINE_PATTERN, maximum1, minimum1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<inteiro nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' opcional='" + isOptional1
        + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros inteiros com opcional com valor no-booleano.
   * 
   * @exception ParseException Em caso de erro.
   */
  public void testIntegerParameterNonBooleanOptionAttributeValue()
    throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Integer parameterDefaultValue1 = null;
    boolean isOptional1 = true;
    Integer maximum1 = null;
    Integer minimum1 = null;
    IntegerParameter parameter1 =
      new IntegerParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        DEFAULT_COMMAND_LINE_PATTERN, maximum1, minimum1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<inteiro nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' opcional='" + isOptional1
        + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros inteiros ocultos sem valor.
   */
  public void testHiddenIntegerParameterWithNoValue() {
    String command = "Comando";
    String parameterGroupName = "Grupo";
    String parameterName = "Nome do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + parameterGroupName + "'>\n";
    xml += "<inteiro nome='" + parameterName + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros inteiros ocultos com valor no-inteiro.
   */
  public void testHiddenIntegerParameterWithNonIntegerValue() {
    String command = "Comando";
    String parameterGroupName = "Grupo";
    String parameterName = "Nome do parmetro";
    String parameterValue = "Valor no-inteiro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + parameterGroupName + "'>\n";
    xml +=
      "<inteiro nome='" + parameterName + "' valor='" + parameterValue
        + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetro inteiros com atributos desconhecidos.
   */
  public void testIntegerParameterWithUnkownAttributes() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<inteiro nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' desconhecido='valor'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetro inteiros com formato no comando exibindo nome e
   * valor.
   * 
   * @throws ParseException
   */
  public void testIntegerParameterWithCommandPatternShowingNameAndValue()
    throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Integer parameterDefaultValue1 = null;
    String commandLinePattern1 =
      "Nome:$NOME_DO_PARAMETRO Valor:$VALOR_DO_PARAMETRO";
    boolean isOptional1 = false;
    Integer maximum1 = null;
    Integer minimum1 = null;
    IntegerParameter parameter1 =
      new IntegerParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        commandLinePattern1, maximum1, minimum1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<inteiro nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'>\n";
    xml += "<formato_no_comando>";
    xml += commandLinePattern1;
    xml += "</formato_no_comando>";
    xml += "</inteiro>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetro inteiros com formato no comando sem exibir nome e
   * valor.
   * 
   * @throws ParseException
   */
  public void testIntegerParameterWithCommandPatternWithNoNameOrValue()
    throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Integer parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    Integer maximum1 = null;
    Integer minimum1 = null;
    IntegerParameter parameter1 =
      new IntegerParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1, null,
        maximum1, minimum1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<inteiro nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'>\n";
    xml += "<formato_no_comando>";
    xml += "</formato_no_comando>";
    xml += "</inteiro>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo de texto.
   * 
   * @throws ParseException
   */
  public void testSimpleTextParameters() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    TextParameter parameter1 =
      new TextParameter(parameterName1, parameterLabel1, parameterDescription1,
        parameterDefaultValue1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo texto repetidos.
   */
  public void testDuplicateTextParameters() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo texto sem nome.
   */
  public void testTextParameterWithNoName() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto rotulo='" + parameterLabel1 + "' dica='" + parameterDescription1
        + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo texto sem rtulo.
   */
  public void testTextParameterWithNoLabel() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' dica='" + parameterDescription1
        + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo texto sem descrio.
   */
  public void testTextParameterWithNoDescription() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetro do tipo texto com valor-padro.
   * 
   * @throws ParseException
   */
  public void testTextParameterWithDefaultValue() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterDefaultValue1 = "Valor-padro do parmetro 1";
    boolean isOptional1 = false;
    TextParameter parameter1 =
      new TextParameter(parameterName1, parameterLabel1, parameterDescription1,
        parameterDefaultValue1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='"
        + parameterDefaultValue1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetro do tipo texto com opcional.
   * 
   * @throws ParseException
   */
  public void testOptionalTextParameter() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    TextParameter parameter1 =
      new TextParameter(parameterName1, parameterLabel1, parameterDescription1,
        parameterDefaultValue1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = true;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' opcional='" + isOptional1
        + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "' opcional='" + isOptional2
        + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo de texto com opcional com valor-no
   * booleano.
   */
  public void testTextParameterWithNonBooleanOptionalValue() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String isOptional1 = "Valor no booleano";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' opcional='" + isOptional1
        + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo texto com valor mximo de caracteres.
   * 
   * @throws ParseException
   */
  public void testTextParameterWithMaximumCharacters() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterDefaultValue1 = "Valor-padro 1";
    Integer maxCharacters1 = new Integer(10);
    boolean isOptional1 = false;
    TextParameter parameter1 =
      new TextParameter(parameterName1, parameterLabel1, parameterDescription1,
        parameterDefaultValue1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN,
        maxCharacters1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='"
        + parameterDefaultValue1 + "' opcional='" + isOptional1
        + "' maximo_de_caracteres='" + maxCharacters1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo texto com valor mximo no-inteiro.
   */
  public void testTextParameterWithNonIntegerMaximumCharacters() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterDefaultValue1 = "Valor-padro 1";
    boolean isOptional1 = false;
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='"
        + parameterDefaultValue1 + "' opcional='" + isOptional1
        + "' maximo_de_caracteres='Valor no-inteiro'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo texto com valor mximo no-positivo.
   */
  public void testTextParameterWithNegativeMaximumCharacters() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterDefaultValue1 = "Valor-padro 1";
    Integer maxCharacters1 = new Integer(0);
    boolean isOptional1 = false;
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='"
        + parameterDefaultValue1 + "' opcional='" + isOptional1
        + "' maximo_de_caracteres='" + maxCharacters1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo texto sem valor-padro.
   */
  public void testTextParameterWithNoDefaultValue() {
    String command = "Comando";
    String parameterGroupName = "Grupo";
    String parameterName = "Nome do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + parameterGroupName + "'>\n";
    xml += "<texto nome='" + parameterName + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo de texto com atributos desconhecidos
   */
  public void testTextParameterWithUnkownAttributes() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro 1";
    String parameterLabel = "Rtulo do parmetro 1";
    String parameterDescription = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "' desconhecido='valor'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo de texto com formato no comando exibindo
   * nome e valor.
   * 
   * @throws ParseException
   */
  public void testTextParameterWithCommandPatternShowingNameAndValue()
    throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    String commandLinePattern1 =
      "Nome:$NOME_DO_PARAMETRO Valor:$VALOR_DO_PARAMETRO";
    TextParameter parameter1 =
      new TextParameter(parameterName1, parameterLabel1, parameterDescription1,
        parameterDefaultValue1, isOptional1, commandLinePattern1, null);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'>\n";
    xml +=
      "<formato_no_comando>" + commandLinePattern1 + "</formato_no_comando>\n";
    xml += "</texto>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo de texto com formato no comando sem
   * exibir nome e valor.
   * 
   * @throws ParseException
   */
  public void testTextParameterWithCommandPatternWithNoNameOrValue()
    throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    TextParameter parameter1 =
      new TextParameter(parameterName1, parameterLabel1, parameterDescription1,
        parameterDefaultValue1, isOptional1, null, null);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'>\n";
    xml += "<formato_no_comando></formato_no_comando>\n";
    xml += "</texto>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo de frmula.
   * 
   * @throws ParseException
   */
  public void testSimpleFormulaParameters() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean isOptional1 = false;
    FormulaParameter parameter1 =
      new FormulaParameter(parameterName1, parameterLabel1,
        parameterDescription1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    boolean isOptional2 = false;
    FormulaParameter parameter2 =
      new FormulaParameter(parameterName2, parameterLabel2,
        parameterDescription2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<formula nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<formula nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo frmula repetidos.
   */
  public void testDuplicateFormulaParameters() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<formula nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<formula nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo frmula sem nome.
   */
  public void testFormulaParameterWithNoName() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<formula rotulo='" + parameterLabel1 + "' dica='"
        + parameterDescription1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo frmula sem rtulo.
   */
  public void testFormulaParameterWithNoLabel() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<formula nome='" + parameterName1 + "' dica='" + parameterDescription1
        + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo formula sem descrio.
   */
  public void testFormulaParameter5() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<formula nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetro do tipo frmula com opcional/obrigatrio.
   * 
   * @throws ParseException
   */
  public void testOptionalAndNonOptionalFormulaParameter()
    throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean isOptional1 = false;
    FormulaParameter parameter1 =
      new FormulaParameter(parameterName1, parameterLabel1,
        parameterDescription1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    boolean isOptional2 = true;
    FormulaParameter parameter2 =
      new FormulaParameter(parameterName2, parameterLabel2,
        parameterDescription2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<formula nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' opcional='" + isOptional1
        + "'/>";
    xml +=
      "<formula nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "' opcional='" + isOptional2
        + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo de frmula com opcional com valor-no
   * booleano.
   */
  public void testFormulaParameterWithNonBooleanOptionalAttributeValue() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String isOptional1 = "Valor no booleano";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<formula nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' opcional='" + isOptional1
        + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo de frmula com atributos desconhecidos
   */
  public void testFormulaParameterWithUnknownAttribute() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro 1";
    String parameterLabel = "Rtulo do parmetro 1";
    String parameterDescription = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<formula nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "' desconhecido='valor'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo de frmula com formato no comando
   * exibindo nome e valor.
   * 
   * @throws ParseException
   */
  public void testFormulaParameterWithCommandPatternShowingNameAndValue()
    throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean isOptional1 = false;
    String commandLinePattern1 =
      "Nome:$NOME_DO_PARAMETRO Valor:$VALOR_DO_PARAMETRO";
    FormulaParameter parameter1 =
      new FormulaParameter(parameterName1, parameterLabel1,
        parameterDescription1, isOptional1, commandLinePattern1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<formula nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'>\n";
    xml +=
      "<formato_no_comando>" + commandLinePattern1 + "</formato_no_comando>\n";
    xml += "</formula>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo de frmula com formato no comando sem
   * exibir nome e valor.
   * 
   * @throws ParseException
   */
  public void testFormulaParameterWithCommandPatternWithNoNameOrValue()
    throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean isOptional1 = false;
    FormulaParameter parameter1 =
      new FormulaParameter(parameterName1, parameterLabel1,
        parameterDescription1, isOptional1, null);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<formula nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'>\n";
    xml += "<formato_no_comando></formato_no_comando>\n";
    xml += "</formula>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo enumerao.
   * 
   * @throws ParseException
   */
  public void testSimpleEnumerationParameters() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterDefaultItemId1 = "ID 11";
    String parameterDefaultItemName1 = "Nome do item padro 1.1";
    String parameterDefaultItemValue1 = "V1.1";
    EnumerationItem parameterDefaultValue1 =
      new EnumerationItem(parameterDefaultItemId1, parameterDefaultItemName1,
        parameterDefaultItemValue1, null);
    String parameterItemId1 = "ID 12";
    String parameterItemName1 = "Nome do item padro 1.2";
    String parameterItemValue1 = "V1.2";
    EnumerationItem parameterValue1 =
      new EnumerationItem(parameterItemId1, parameterItemName1,
        parameterItemValue1, null);
    List<EnumerationItem> items1 = new LinkedList<EnumerationItem>();
    items1.add(parameterDefaultValue1);
    items1.add(parameterValue1);
    boolean mustSortItems = true;
    EnumerationParameter parameter1 =
      new EnumerationParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1,
        DEFAULT_COMMAND_LINE_PATTERN, items1, mustSortItems);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<enumeracao nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='"
        + parameterDefaultItemId1 + "'>";
    xml +=
      "<item_de_enumeracao id='" + parameterDefaultItemId1 + "' rotulo='"
        + parameterDefaultItemName1 + "' valor='" + parameterDefaultItemValue1
        + "'/>";
    xml +=
      "<item_de_enumeracao id='" + parameterItemId1 + "' rotulo='"
        + parameterItemName1 + "' valor='" + parameterItemValue1 + "'/>";
    xml += "</enumeracao>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo enumerao repetidos.
   */
  public void testDuplicateEnumerationParameters() {
    String parameterName1 = "Nome do parmetro 1";
    int parameterDefaultItemId1 = 1;
    int parameterDefaultItemId2 = 2;
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='comando'>\n";
    xml += "<grupo rotulo='grupo'>\n";
    xml += "<enumeracao nome='" + parameterName1 + "'\n";
    xml += "            rotulo='Rtulo do parmetro 1'\n";
    xml += "            dica='Dica do parmetro 1'\n";
    xml += "            padrao='" + parameterDefaultItemId1 + "'\n";
    xml += ">\n";
    xml += "<item_de_enumeracao id='" + parameterDefaultItemId1 + "'\n";
    xml += "                    rotulo='Nome 1'\n";
    xml += "                    valor='Valor 1'/>\n";
    xml += "</enumeracao>\n";
    xml += "<enumeracao nome='" + parameterName1 + "'\n";
    xml += "            rotulo='Rtulo do parmetro 2'\n";
    xml += "            dica='Dica do parmetro 2'\n";
    xml += "            padrao='" + parameterDefaultItemId2 + "'\n";
    xml += ">\n";
    xml += "<item_de_enumeracao id='" + parameterDefaultItemId2 + "'\n";
    xml += "                    rotulo='Nome 2'\n";
    xml += "                    valor='Valor 2'\n";
    xml += "/>\n";
    xml += "</enumeracao>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>\n";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo enumerao sem nome.
   */
  public void testEnumerationParameterWithNoName() {
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='comando'>\n";
    xml += "<grupo rotulo='grupo'>";
    xml += "<enumeracao rotulo='rotulo' dica='dica' padrao='1'>";
    xml += "<item_de_enumeracao id='1' rotulo='rotulo' valor='valor'/>";
    xml += "</enumeracao>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo enumerao sem rtulo.
   */
  public void testEnumerationParameterWithNoLabel() {
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='comando'>\n";
    xml += "<grupo rotulo='rotulo'>";
    xml += "<enumeracao nome='nome' dica='dica' padrao='1'>";
    xml += "<item_de_enumeracao id='1' rotulo='rotulo' valor='valor'/>";
    xml += "</enumeracao>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo enumerao sem descrio.
   */
  public void testEnumerationParameterWithNoDescription() {
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='comando'>\n";
    xml += "<grupo rotulo='rotulo'>";
    xml += "<enumeracao nome='nome' rotulo='rotulo' padrao='1'>";
    xml += "<item_de_enumeracao id='1' rotulo='rotulo' valor='valor'/>";
    xml += "</enumeracao>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo enumerao sem valor-padro.
   * 
   * @throws ParseException
   */
  public void testEnumerationParameterWithNoDefaultValue()
    throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterDefaultItemId1 = "ID 11";
    String parameterDefaultItemName1 = "Nome do item padro 1.1";
    String parameterDefaultItemValue1 = "V1.1";
    EnumerationItem parameterDefaultValue1 =
      new EnumerationItem(parameterDefaultItemId1, parameterDefaultItemName1,
        parameterDefaultItemValue1, null);
    String parameterItemId1 = "ID 12";
    String parameterItemName1 = "Nome do item padro 1.2";
    String parameterItemValue1 = "V1.2";
    EnumerationItem parameterValue1 =
      new EnumerationItem(parameterItemId1, parameterItemName1,
        parameterItemValue1, null);
    List<EnumerationItem> items1 = new LinkedList<EnumerationItem>();
    items1.add(parameterDefaultValue1);
    items1.add(parameterValue1);
    boolean mustSortItems1 = true;
    EnumerationParameter parameter1 =
      new EnumerationParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1,
        DEFAULT_COMMAND_LINE_PATTERN, items1, mustSortItems1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<enumeracao nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'>";
    xml +=
      "<item_de_enumeracao id='" + parameterDefaultItemId1 + "' rotulo='"
        + parameterDefaultItemName1 + "' valor='" + parameterDefaultItemValue1
        + "'/>";
    xml +=
      "<item_de_enumeracao id='" + parameterItemId1 + "' rotulo='"
        + parameterItemName1 + "' valor='" + parameterItemValue1 + "'/>";
    xml += "</enumeracao>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo enumerao com valor-padro que no est
   * cadastrado na enumerao.
   */
  public void testEnumerationParameter7() {
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='comando'>\n";
    xml += "<grupo rotulo='rotulo'>";
    xml += "<enumeracao nome='nome' rotulo='rotulo' dica='dica' padrao='2'>";
    xml += "<item_de_enumeracao id='1' rotulo='rotulo' valor='valor'/>";
    xml += "</enumeracao>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo enumerao com itens com ids duplicados.
   */
  public void testEnumerationParameter9() {
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='comando'>\n";
    xml += "<grupo rotulo='rotulo'>";
    xml += "<enumeracao nome='nome' rotulo='rotulo' dica='dica' padrao='1'>";
    xml += "<item_de_enumeracao id='1' rotulo='rotulo 1' valor='valor 1'/>";
    xml += "<item_de_enumeracao id='1' rotulo='rotulo 2' valor='valor 2'/>";
    xml += "</enumeracao>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo enumerao com itens com rotulos
   * duplicados.
   */
  public void testEnumerationParameter10() {
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='comando'>\n";
    xml += "<grupo rotulo='rotulo'>";
    xml += "<enumeracao nome='nome' rotulo='rotulo' dica='dica' padrao='1'>";
    xml += "<item_de_enumeracao id='1' rotulo='rotulo' valor='valor 1'/>";
    xml += "<item_de_enumeracao id='2' rotulo='rotulo' valor='valor 2'/>";
    xml += "</enumeracao>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo enumerao com item sem id.
   */
  public void testEnumerationParameter11() {
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='comando'>\n";
    xml += "<grupo rotulo='rotulo'>";
    xml += "<enumeracao nome='nome' rotulo='rotulo' dica='dica' padrao='1'>";
    xml += "<item_de_enumeracao rotulo='rotulo 1' valor='valor 1'/>";
    xml += "</enumeracao>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo enumerao com item sem rtulo.
   */
  public void testEnumerationParameter13() {
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='comando'>\n";
    xml += "<grupo rotulo='rotulo'>";
    xml += "<enumeracao nome='nome' rotulo='rotulo' dica='dica' padrao='1'>";
    xml += "<item_de_enumeracao id='1' valor='valor 1'/>";
    xml += "</enumeracao>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo enumerao com item sem valor.
   */
  public void testEnumerationParameter14() {
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='comando'>\n";
    xml += "<grupo rotulo='rotulo'>";
    xml += "<enumeracao nome='nome' rotulo='rotulo' dica='dica' padrao='1'>";
    xml += "<item_de_enumeracao id='1' rotulo='rotulo 1'/>";
    xml += "</enumeracao>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Tentando criar um parmetro do tipo enumerao com atributos desconhecidos.
   */
  public void testEnumerationParameter15() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterDefaultItemId1 = "ID 11";
    String parameterDefaultItemName1 = "Nome do item padro 1.1";
    String parameterDefaultItemValue1 = "V1.1";
    String parameterItemId1 = "ID 12";
    String parameterItemName1 = "Nome do item padro 1.2";
    String parameterItemValue1 = "V1.2";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<enumeracao nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='"
        + parameterDefaultItemId1 + "' desconhecido='valor'>";
    xml +=
      "<item_de_enumeracao id='" + parameterDefaultItemId1 + "' rotulo='"
        + parameterDefaultItemName1 + "' valor='" + parameterDefaultItemValue1
        + "'/>";
    xml +=
      "<item_de_enumeracao id='" + parameterItemId1 + "' rotulo='"
        + parameterItemName1 + "' valor='" + parameterItemValue1 + "'/>";
    xml += "</enumeracao>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Tentando criar itens de enumerao com atributos desconhecidos.
   */
  public void testEnumerationParameter16() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel = "Rtulo do parmetro 1";
    String parameterDescription = "Descrio do parmetro 1";
    String parameterDefaultItemId = "ID 11";
    String parameterDefaultItemName = "Nome do item padro 1.1";
    String parameterDefaultItemValue = "V1.1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<enumeracao nome='" + parameterName1 + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "' padrao='"
        + parameterDefaultItemId + "'>";
    xml +=
      "<item_de_enumeracao id='" + parameterDefaultItemId + "' rotulo='"
        + parameterDefaultItemName + "' valor='" + parameterDefaultItemValue
        + "' desconhecido='valor'/>";
    xml += "</enumeracao>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Tentando criar itens de enumerao com elementos-filhos.
   */
  public void testEnumerationParameter18() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel = "Rtulo do parmetro 1";
    String parameterDescription = "Descrio do parmetro 1";
    String parameterDefaultItemId = "ID 11";
    String parameterDefaultItemName = "Nome do item padro 1.1";
    String parameterDefaultItemValue = "V1.1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<enumeracao nome='" + parameterName1 + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "' padrao='"
        + parameterDefaultItemId + "'>";
    xml +=
      "<item_de_enumeracao id='" + parameterDefaultItemId + "' rotulo='"
        + parameterDefaultItemName + "' valor='" + parameterDefaultItemValue
        + "'>\n";
    xml += "<filho/>\n";
    xml += "</item_de_enumeracao>\n";
    xml += "</enumeracao>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>\n";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo enumerao com seleo simples e com
   * formato no comando exibindo nome e valor.
   * 
   * @throws ParseException
   */
  public void testEnumerationParameter19() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterDefaultItemId1 = "ID 11";
    String parameterDefaultItemName1 = "Nome do item padro 1.1";
    String parameterDefaultItemValue1 = "V1.1";
    EnumerationItem parameterDefaultValue1 =
      new EnumerationItem(parameterDefaultItemId1, parameterDefaultItemName1,
        parameterDefaultItemValue1, null);
    String parameterItemId1 = "ID 12";
    String parameterItemName1 = "Nome do item padro 1.2";
    String parameterItemValue1 = "V1.2";
    EnumerationItem parameterValue1 =
      new EnumerationItem(parameterItemId1, parameterItemName1,
        parameterItemValue1, null);
    String commandLinePattern1 =
      "Nome: $NOME_DO_PARAMETRO Valor: $VALOR_DO_PARAMETRO";
    List<EnumerationItem> items1 = new LinkedList<EnumerationItem>();
    items1.add(parameterDefaultValue1);
    items1.add(parameterValue1);
    boolean mustSortItems1 = true;
    EnumerationParameter parameter1 =
      new EnumerationParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, commandLinePattern1,
        items1, mustSortItems1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<enumeracao nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='"
        + parameterDefaultItemId1 + "' selecao_multipla='nao'>\n";
    xml +=
      "<item_de_enumeracao id='" + parameterDefaultItemId1 + "' rotulo='"
        + parameterDefaultItemName1 + "' valor='" + parameterDefaultItemValue1
        + "'/>\n";
    xml +=
      "<item_de_enumeracao id='" + parameterItemId1 + "' rotulo='"
        + parameterItemName1 + "' valor='" + parameterItemValue1 + "'/>\n";
    xml += "<formato_no_comando>\n";
    xml += commandLinePattern1;
    xml += "</formato_no_comando>\n";
    xml += "</enumeracao>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo enumerao com seleo simples e com
   * formato no comando sem exibir nome e valor.
   * 
   * @throws ParseException
   */
  public void testEnumerationParameter20() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterDefaultItemId1 = "ID 11";
    String parameterDefaultItemName1 = "Nome do item padro 1.1";
    String parameterDefaultItemValue1 = "V1.1";
    EnumerationItem parameterDefaultValue1 =
      new EnumerationItem(parameterDefaultItemId1, parameterDefaultItemName1,
        parameterDefaultItemValue1, null);
    String parameterItemId1 = "ID 12";
    String parameterItemName1 = "Nome do item padro 1.2";
    String parameterItemValue1 = "V1.2";
    EnumerationItem parameterValue1 =
      new EnumerationItem(parameterItemId1, parameterItemName1,
        parameterItemValue1, null);
    List<EnumerationItem> items1 = new LinkedList<EnumerationItem>();
    items1.add(parameterDefaultValue1);
    items1.add(parameterValue1);
    boolean mustSortItems1 = true;
    EnumerationParameter parameter1 =
      new EnumerationParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, null, items1,
        mustSortItems1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<enumeracao nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='"
        + parameterDefaultItemId1 + "' selecao_multipla='nao'>\n";
    xml +=
      "<item_de_enumeracao id='" + parameterDefaultItemId1 + "' rotulo='"
        + parameterDefaultItemName1 + "' valor='" + parameterDefaultItemValue1
        + "'/>\n";
    xml +=
      "<item_de_enumeracao id='" + parameterItemId1 + "' rotulo='"
        + parameterItemName1 + "' valor='" + parameterItemValue1 + "'/>\n";
    xml += "<formato_no_comando>\n";
    xml += "</formato_no_comando>\n";
    xml += "</enumeracao>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo enumerao com seleo mltipla e com
   * formato no comando exibindo nome e valor.
   * 
   * @throws ParseException
   */
  public void testEnumerationParameter21() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterItemId1 = "ID 11";
    String parameterItemName1 = "Nome do item padro 1.1";
    String parameterItemValue1 = "V1.1";
    EnumerationItem parameterValue1 =
      new EnumerationItem(parameterItemId1, parameterItemName1,
        parameterItemValue1, null);
    String parameterItemId2 = "ID 12";
    String parameterItemName2 = "Nome do item padro 1.2";
    String parameterItemValue2 = "V1.2";
    EnumerationItem parameterValue2 =
      new EnumerationItem(parameterItemId2, parameterItemName2,
        parameterItemValue2, null);
    String commandLinePattern1 =
      "Nome: $NOME_DO_PARAMETRO Valor: $VALOR_DO_PARAMETRO";
    List<EnumerationItem> items1 = new LinkedList<EnumerationItem>();
    items1.add(parameterValue1);
    items1.add(parameterValue2);
    EnumerationListParameter parameter1 =
      new EnumerationListParameter(parameterName1, parameterLabel1,
        parameterDescription1, false, commandLinePattern1);
    parameter1.setItems(items1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<enumeracao nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' selecao_multipla='sim'>\n";
    xml +=
      "<item_de_enumeracao id='" + parameterItemId1 + "' rotulo='"
        + parameterItemName1 + "' valor='" + parameterItemValue1 + "'/>\n";
    xml +=
      "<item_de_enumeracao id='" + parameterItemId2 + "' rotulo='"
        + parameterItemName2 + "' valor='" + parameterItemValue2 + "'/>\n";
    xml += "<formato_no_comando>\n";
    xml += commandLinePattern1;
    xml += "</formato_no_comando>\n";
    xml += "</enumeracao>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo enumerao com seleo mltipla e com
   * formato no comando sem exibir nome e valor.
   * 
   * @throws ParseException
   */
  public void testEnumerationParameter22() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterItemId1 = "ID 11";
    String parameterItemName1 = "Nome do item padro 1.1";
    String parameterItemValue1 = "V1.1";
    EnumerationItem parameterValue1 =
      new EnumerationItem(parameterItemId1, parameterItemName1,
        parameterItemValue1, null);
    String parameterItemId2 = "ID 12";
    String parameterItemName2 = "Nome do item padro 1.2";
    String parameterItemValue2 = "V1.2";
    EnumerationItem parameterValue2 =
      new EnumerationItem(parameterItemId2, parameterItemName2,
        parameterItemValue2, null);
    List<EnumerationItem> items1 = new LinkedList<EnumerationItem>();
    items1.add(parameterValue1);
    items1.add(parameterValue2);
    EnumerationListParameter parameter1 =
      new EnumerationListParameter(parameterName1, parameterLabel1,
        parameterDescription1, false, null);
    parameter1.setItems(items1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<enumeracao nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' selecao_multipla='sim'>\n";
    xml +=
      "<item_de_enumeracao id='" + parameterItemId1 + "' rotulo='"
        + parameterItemName1 + "' valor='" + parameterItemValue1 + "'/>\n";
    xml +=
      "<item_de_enumeracao id='" + parameterItemId2 + "' rotulo='"
        + parameterItemName2 + "' valor='" + parameterItemValue2 + "'/>\n";
    xml += "<formato_no_comando>\n";
    xml += "</formato_no_comando>\n";
    xml += "</enumeracao>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * @throws ParseException
   */
  public void testTextListParameter1() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean isOptional1 = false;
    TextListParameter parameter1 =
      new TextListParameter(parameterName1, parameterLabel1,
        parameterDescription1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    boolean isOptional2 = false;
    TextListParameter parameter2 =
      new TextListParameter(parameterName2, parameterLabel2,
        parameterDescription2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_texto nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<lista_de_texto nome='" + parameterName2 + "' rotulo='"
        + parameterLabel2 + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testTextListParameter2() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_texto nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<lista_de_texto nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testTextListParameter3() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_texto rotulo='" + parameterLabel1 + "' dica='"
        + parameterDescription1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testTextListParameter4() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_texto nome='" + parameterName1 + "' dica='"
        + parameterDescription1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testTextListParameter5() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_texto nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testTextListParameter6() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean isOptional1 = true;
    TextListParameter parameter1 =
      new TextListParameter(parameterName1, parameterLabel1,
        parameterDescription1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    boolean isOptional2 = false;
    TextListParameter parameter2 =
      new TextListParameter(parameterName2, parameterLabel2,
        parameterDescription2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_texto nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "' opcional='"
        + isOptional1 + "'/>";
    xml +=
      "<lista_de_texto nome='" + parameterName2 + "' rotulo='"
        + parameterLabel2 + "' dica='" + parameterDescription2 + "' opcional='"
        + isOptional2 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testTextListParameter7() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_texto nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1
        + "' opcional='Valor no booleano'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testTextListParameter8() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Integer maxCharacters1 = new Integer(10);
    boolean isOptional1 = false;
    TextListParameter parameter1 =
      new TextListParameter(parameterName1, parameterLabel1,
        parameterDescription1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN,
        maxCharacters1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_texto nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "' opcional='"
        + isOptional1 + "' maximo_de_caracteres='" + maxCharacters1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testTextListParameter9() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Integer maxCharacters1 = new Integer(0);
    boolean isOptional1 = false;
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_texto nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "' opcional='"
        + isOptional1 + "' maximo_de_caracteres='" + maxCharacters1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Tentando criar parmetros do tipo lista de texto com atributos
   * desconhecidos.
   */
  public void testTextListParameter10() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_texto nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "' "
        + "desconhecido='valor'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testTextListParameter12() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String commandLinePattern1 =
      "Nome: $NOME_DO_PARAMETRO Valor: $VALOR_DO_PARAMETRO";
    boolean isOptional1 = false;
    TextListParameter parameter1 =
      new TextListParameter(parameterName1, parameterLabel1,
        parameterDescription1, isOptional1, commandLinePattern1, null);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_texto nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "'>\n";
    xml += "<formato_no_comando>\n";
    xml += commandLinePattern1;
    xml += "</formato_no_comando>\n";
    xml += "</lista_de_texto>\n";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testTextListParameter13() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean isOptional1 = false;
    TextListParameter parameter1 =
      new TextListParameter(parameterName1, parameterLabel1,
        parameterDescription1, isOptional1, null, null);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_texto nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "'>\n";
    xml += "<formato_no_comando>\n";
    xml += "</formato_no_comando>\n";
    xml += "</lista_de_texto>\n";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo lista de inteiros.
   */
  public void testIntegerListParameter1() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean isOptional1 = false;
    IntegerListParameter parameter1 =
      new IntegerListParameter(parameterName1, parameterLabel1,
        parameterDescription1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN, null,
        null);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_inteiros nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo lista de inteiros repetidos.
   */
  public void testIntegerListParameter2() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_inteiros nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<lista_de_inteiros nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo lista de inteiros sem nome.
   */
  public void testIntegerListParameter3() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_inteiros rotulo='" + parameterLabel1 + "' dica='"
        + parameterDescription1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo lista de inteiros sem rtulo.
   */
  public void testIntegerListParameter4() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_inteiros nome='" + parameterName1 + "' dica='"
        + parameterDescription1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo lista de inteiros sem descrio.
   */
  public void testIntegerListParameter5() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_inteiros nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo lista de inteiros com mximo.
   */
  public void testIntegerListParameter6() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean isOptional1 = false;
    Integer maximum1 = new Integer(1000);
    IntegerListParameter parameter1 =
      new IntegerListParameter(parameterName1, parameterLabel1,
        parameterDescription1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN,
        maximum1, null);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_inteiros nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "' maximo='"
        + maximum1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo lista de inteiros com mximo com valor
   * no-inteiro.
   */
  public void testIntegerListParameter7() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_inteiros nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1
        + "' maximo='Valor no inteiro'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo lista de inteiros com mnimo.
   */
  public void testIntegerListParameter8() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean isOptional1 = false;
    Integer minimum1 = new Integer(1000);
    IntegerListParameter parameter1 =
      new IntegerListParameter(parameterName1, parameterLabel1,
        parameterDescription1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN, null,
        minimum1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_inteiros nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "' minimo='"
        + minimum1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo lista de inteiros com mnimo com valor
   * no-inteiro.
   */
  public void testIntegerListParameter9() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_inteiros nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1
        + "' minimo='Valor no inteiro'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo lista de inteiros com mnimo maior do que
   * o valor mximo.
   */
  public void testIntegerListParameter10() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Integer minimum1 = new Integer(1);
    Integer maximum1 = new Integer(-1);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_inteiros nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "' minimo='"
        + minimum1 + "' maximo='" + maximum1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo lista de inteiros com mnimo igual ao
   * mximo.
   */
  public void testIntegerListParameter11() throws ParseException {
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator("comando", null, ExecutionType.SIMPLE, null,
        false, null, null);
    Integer maximum;
    Integer minimum;
    maximum = minimum = new Integer(10);
    IntegerListParameter parameter =
      new IntegerListParameter("Nome do inteiro", "Rtulo do inteiro",
        "Descrio do inteiro", false, DEFAULT_COMMAND_LINE_PATTERN, maximum,
        minimum);
    ParameterGroup group = new ParameterGroup("Grupo");
    group.addParameter(parameter);
    version.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + version.getCommandBinaryName() + "'>\n";
    xml += "<grupo rotulo='" + group.getName() + "'>";
    xml +=
      "<lista_de_inteiros nome='" + parameter.getName() + "' rotulo='"
        + parameter.getLabel() + "' dica='" + parameter.getDescription()
        + "' minimo='" + minimum + "' maximo='" + maximum + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  /**
   * Testando criar parmetros do tipo lista de inteiros com opcional
   * verdadeiro.
   */
  public void testIntegerListParameter12() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean isOptional1 = true;
    IntegerListParameter parameter1 =
      new IntegerListParameter(parameterName1, parameterLabel1,
        parameterDescription1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN, null,
        null);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_inteiros nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "' opcional='"
        + isOptional1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo lista de inteiros com opcional falso.
   */
  public void testIntegerListParameter13() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean isOptional1 = false;
    IntegerListParameter parameter1 =
      new IntegerListParameter(parameterName1, parameterLabel1,
        parameterDescription1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN, null,
        null);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_inteiros nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "' opcional='"
        + isOptional1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo lista de inteiros com opcional
   * no-booleano.
   */
  public void testIntegerListParameter14() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_inteiros nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1
        + "' opcional='Valor no booleano'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Tentando criar uma lista de inteiros com atributos desconhecidos.
   */
  public void testIntegerListParameter15() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_inteiros nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "' "
        + "desconhecido='valor'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetro lista de inteiros com formato no comando exibindo
   * nome e valor.
   */
  public void testIntegerListParameter16() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String commandLinePattern1 =
      "Nome:$NOME_DO_PARAMETRO Valor:$VALOR_DO_PARAMETRO";
    boolean isOptional1 = false;
    Integer maximum1 = null;
    Integer minimum1 = null;
    IntegerListParameter parameter1 =
      new IntegerListParameter(parameterName1, parameterLabel1,
        parameterDescription1, isOptional1, commandLinePattern1, maximum1,
        minimum1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<lista_de_inteiros nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "'>\n";
    xml += "<formato_no_comando>";
    xml += commandLinePattern1;
    xml += "</formato_no_comando>";
    xml += "</lista_de_inteiros>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetro lista de inteiros com formato no comando sem
   * exibir nome e valor.
   */
  public void testIntegerListParameter17() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean isOptional1 = false;
    Integer maximum1 = null;
    Integer minimum1 = null;
    IntegerListParameter parameter1 =
      new IntegerListParameter(parameterName1, parameterLabel1,
        parameterDescription1, isOptional1, null, maximum1, minimum1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<lista_de_inteiros nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "'>\n";
    xml += "<formato_no_comando>";
    xml += "</formato_no_comando>";
    xml += "</lista_de_inteiros>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testDoubleListParameter1() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean isOptional1 = false;
    Double maximum1 = null;
    boolean includeMaximum1 = true;
    Double minimum1 = null;
    boolean includeMinimum1 = true;
    DoubleListParameter parameter1 =
      new DoubleListParameter(parameterName1, parameterLabel1,
        parameterDescription1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN,
        maximum1, includeMaximum1, minimum1, includeMinimum1);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    boolean isOptional2 = false;
    Double maximum2 = null;
    boolean includeMaximum2 = true;
    Double minimum2 = null;
    boolean includeMinimum2 = true;
    DoubleListParameter parameter2 =
      new DoubleListParameter(parameterName2, parameterLabel2,
        parameterDescription2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN,
        maximum2, includeMaximum2, minimum2, includeMinimum2);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<lista_de_reais nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "'/>\n";
    xml +=
      "<lista_de_reais nome='" + parameterName2 + "' rotulo='"
        + parameterLabel2 + "' dica='" + parameterDescription2 + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testDoubleListParameter2() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<lista_de_reais nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "'/>\n";
    xml +=
      "<lista_de_reais nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testDoubleListParameter3() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<lista_de_reais rotulo='" + parameterLabel1 + "' dica='"
        + parameterDescription1 + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testDoubleListParameter4() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<lista_de_reais nome='" + parameterName1 + "' dica='"
        + parameterDescription1 + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testDoubleListParameter5() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<lista_de_reais nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testDoubleListParameter6() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean isOptional1 = false;
    Double maximum1 = null;
    boolean includeMaximum1 = true;
    Double minimum1 = new Double(10);
    boolean includeMinimum1 = true;
    DoubleListParameter parameter1 =
      new DoubleListParameter(parameterName1, parameterLabel1,
        parameterDescription1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN,
        maximum1, includeMaximum1, minimum1, includeMinimum1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_reais nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "' minimo='"
        + minimum1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testDoubleListParameter7() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_reais nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1
        + "' minimo='Valor no real'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testDoubleListParameter8() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean isOptional1 = false;
    Double maximum1 = new Double(1000);
    boolean includeMaximum1 = true;
    Double minimum1 = null;
    boolean includeMinimum1 = true;
    DoubleListParameter parameter1 =
      new DoubleListParameter(parameterName1, parameterLabel1,
        parameterDescription1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN,
        maximum1, includeMaximum1, minimum1, includeMinimum1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_reais nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "' maximo='"
        + maximum1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testDoubleListParameter9() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_reais nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1
        + "' maximo='Valor no real'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testDoubleListParameter10() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Double minimum1 = new Double(1);
    Double maximum1 = new Double(-1);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_reais nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "' minimo='"
        + minimum1 + "' maximo='" + maximum1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testDoubleListParameter11() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean isOptional1 = false;
    Double maximum1 = new Double(1000);
    boolean includeMaximum1 = false;
    Double minimum1 = null;
    boolean includeMinimum1 = true;
    DoubleListParameter parameter1 =
      new DoubleListParameter(parameterName1, parameterLabel1,
        parameterDescription1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN,
        maximum1, includeMaximum1, minimum1, includeMinimum1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_reais nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "' maximo='"
        + maximum1 + "' incluir_maximo='" + includeMaximum1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testDoubleListParameter12() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean isOptional1 = false;
    Double maximum1 = new Double(1000);
    boolean includeMaximum1 = true;
    Double minimum1 = null;
    boolean includeMinimum1 = true;
    DoubleListParameter parameter1 =
      new DoubleListParameter(parameterName1, parameterLabel1,
        parameterDescription1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN,
        maximum1, includeMaximum1, minimum1, includeMinimum1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_reais nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "' maximo='"
        + maximum1 + "' incluir_maximo='" + includeMaximum1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testDoubleListParameter13() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean isOptional1 = false;
    Double maximum1 = null;
    boolean includeMaximum1 = true;
    Double minimum1 = new Double(1000);
    boolean includeMinimum1 = false;
    DoubleListParameter parameter1 =
      new DoubleListParameter(parameterName1, parameterLabel1,
        parameterDescription1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN,
        maximum1, includeMaximum1, minimum1, includeMinimum1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_reais nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "' minimo='"
        + minimum1 + "' incluir_minimo='" + includeMinimum1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testDoubleListParameter14() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Double maximum1 = new Double(1000);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_reais nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "' maximo='"
        + maximum1 + "' incluir_maximo='Valor no booleano'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testDoubleListParameter15() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean isOptional1 = false;
    Double minimum1 = new Double(1000);
    boolean includeMaximum1 = true;
    Double maximum1 = null;
    boolean includeMinimum1 = true;
    DoubleListParameter parameter1 =
      new DoubleListParameter(parameterName1, parameterLabel1,
        parameterDescription1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN,
        maximum1, includeMaximum1, minimum1, includeMinimum1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_reais nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "' minimo='"
        + minimum1 + "' incluir_minimo='" + includeMinimum1 + "'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testDoubleListParameter16() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Double minimum1 = new Double(1000);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_reais nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "' minimo='"
        + minimum1 + "' incluir_minimo='Valor no booleano'/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testDoubleListParameter17() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean isOptional1 = true;
    Double maximum1 = null;
    boolean includeMaximum1 = true;
    Double minimum1 = null;
    boolean includeMinimum1 = true;
    DoubleListParameter parameter1 =
      new DoubleListParameter(parameterName1, parameterLabel1,
        parameterDescription1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN,
        maximum1, includeMaximum1, minimum1, includeMinimum1);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    boolean isOptional2 = false;
    Double maximum2 = null;
    boolean includeMaximum2 = true;
    Double minimum2 = null;
    boolean includeMinimum2 = true;
    DoubleListParameter parameter2 =
      new DoubleListParameter(parameterName2, parameterLabel2,
        parameterDescription2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN,
        maximum2, includeMaximum2, minimum2, includeMinimum2);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<lista_de_reais nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "' opcional='"
        + isOptional1 + "'/>\n";
    xml +=
      "<lista_de_reais nome='" + parameterName2 + "' rotulo='"
        + parameterLabel2 + "' dica='" + parameterDescription2 + "' opcional='"
        + isOptional2 + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros lista de reais com padro do formato no comando
   * exibindo o nome e o valor.
   */
  public void testDoubleListParameter19() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean isOptional1 = false;
    Double maximum1 = null;
    boolean includeMaximum1 = true;
    Double minimum1 = null;
    boolean includeMinimum1 = true;
    DoubleListParameter parameter1 =
      new DoubleListParameter(parameterName1, parameterLabel1,
        parameterDescription1, isOptional1,
        "Nome:$NOME_DO_PARAMETRO Valor:$VALOR_DO_PARAMETRO", maximum1,
        includeMaximum1, minimum1, includeMinimum1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<lista_de_reais nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "'>\n";
    xml +=
      "<formato_no_comando>Nome:$NOME_DO_PARAMETRO Valor:$VALOR_DO_PARAMETRO</formato_no_comando>\n";
    xml += "</lista_de_reais>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando c +=
   * "<formato_no_comando>Valor:$VALOR_DO_PARAMETRO</formato_no_comando>\n"; xml
   * += "</lista_de_reais>\n"; xml += "</grupo>\n"; xml += "</algoritmo>";
   * assertXml(expectedVersion, xml); }
   * 
   * / Testando criar parmetros lista de reais com padro do formato no comando
   * sem nome e sem valor.
   */
  public void testDoubleListParameter22() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean isOptional1 = false;
    Double maximum1 = null;
    boolean includeMaximum1 = true;
    Double minimum1 = null;
    boolean includeMinimum1 = true;
    DoubleListParameter parameter1 =
      new DoubleListParameter(parameterName1, parameterLabel1,
        parameterDescription1, isOptional1, null, maximum1, includeMaximum1,
        minimum1, includeMinimum1);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<lista_de_reais nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "'>\n";
    xml += "<formato_no_comando/>\n";
    xml += "</lista_de_reais>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo tabela com colunas do tipo texto.
   */
  public void testTextColumn1() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    Column<String> column1 =
      new TextColumn(columnName1, columnName1, null, false, true, null);
    String columnName2 = "Nome da coluna 2";
    Column<String> column2 =
      new TextColumn(columnName2, columnName2, null, false, true, null);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1, column2),
        false, false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml += "<coluna_de_texto rotulo='" + columnName1 + "'/>\n";
    xml += "<coluna_de_texto rotulo='" + columnName2 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo tabela com colunas do tipo texto com
   * valor-padro.
   */
  public void testTextColumn2() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    String defaultValue1 = "Valor-padro 1";
    Column<String> column1 =
      new TextColumn(columnName1, columnName1, defaultValue1, false, true, null);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1), false,
        false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_texto rotulo='" + columnName1 + "' padrao='" + defaultValue1
        + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo tabela com colunas texto repetidas.
   */
  public void testTextColumn3() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml += "<coluna_de_texto rotulo='" + columnName1 + "'/>\n";
    xml += "<coluna_de_texto rotulo='" + columnName1 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo tabela com colunas de texto sem nome.
   */
  public void testTextColumn4() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml += "<coluna_de_texto/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo tabela com colunas de texto com opcional
   * verdadeiro.
   */
  public void testTextColumn5() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    boolean isOptional1 = true;
    Column<String> column1 =
      new TextColumn(columnName1, columnName1, null, isOptional1, true, null);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1), false,
        false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_texto rotulo='" + columnName1 + "' opcional='" + isOptional1
        + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo tabela com colunas de texto com opcional
   * verdadeiro.
   */
  public void testTextColumn6() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    boolean isOptional1 = false;
    Column<String> column1 =
      new TextColumn(columnName1, columnName1, null, isOptional1, true, null);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1), false,
        false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_texto rotulo='" + columnName1 + "' opcional='" + isOptional1
        + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo tabela com colunas de texto com opcional
   * no-booleano.
   */
  public void testTextColumn7() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String columnName1 = "Nome da coluna 1";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_texto rotulo='" + columnName1
        + "' opcional='Valor no-booleano'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo tabela com colunas de texto com valor
   * mximo de caracteres.
   */
  public void testTextColumn8() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    Integer maxCharacters1 = new Integer(5000);
    Column<String> column1 =
      new TextColumn(columnName1, columnName1, null, false, true,
        maxCharacters1);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1), false,
        false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_texto rotulo='" + columnName1 + "' maximo_de_caracteres='"
        + maxCharacters1 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo tabela com colunas de texto com valor
   * mximo de caracteres com valor no-inteiro.
   */
  public void testTextColumn9() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName = "Nome da coluna";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_texto rotulo='" + columnName
        + "' maximo_de_caracteres='Valor no-inteiro'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo tabela com colunas de texto com valor
   * mximo de caracteres com valor no-positivo.
   */
  public void testTextColumn10() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName = "Nome da coluna";
    int maxCharacters = 0;
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_texto rotulo='" + columnName + "' maximo_de_caracteres='"
        + maxCharacters + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo tabela com colunas do tipo texto com
   * atributo desconhecido.
   */
  public void testTextColumn11() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_texto rotulo='" + columnName1 + "' desconhecido='valor'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo tabela com colunas do tipo texto com
   * elementos-filhos.
   */
  public void testTextColumn12() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml += "<coluna_de_texto rotulo='" + columnName1 + "'>\n";
    xml += "<filho/>\n";
    xml += "</coluna_de_texto>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo tabela com colunas do tipo texto com
   * atributo editvel verdadeiro.
   */
  public void testTextColumn13() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    Column<String> column1 =
      new TextColumn(columnName1, columnName1, null, false, true, null);
    String columnName2 = "Nome da coluna 2";
    Column<String> column2 =
      new TextColumn(columnName2, columnName2, null, false, true, null);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1, column2),
        false, false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml += "<coluna_de_texto rotulo='" + columnName1 + "' editavel='sim'/>\n";
    xml += "<coluna_de_texto rotulo='" + columnName2 + "' editavel='sim'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo tabela com colunas do tipo texto com
   * atributo editvel falso.
   */
  public void testTextColumn14() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    Column<String> column1 =
      new TextColumn(columnName1, columnName1, null, false, false, null);
    String columnName2 = "Nome da coluna 2";
    Column<String> column2 =
      new TextColumn(columnName2, columnName2, null, false, false, null);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1, column2),
        false, false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml += "<coluna_de_texto rotulo='" + columnName1 + "' editavel='nao'/>\n";
    xml += "<coluna_de_texto rotulo='" + columnName2 + "' editavel='nao'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo tabela com colunas do tipo texto com
   * identificador.
   */
  public void testTextColumn15() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnId1 = "Id da coluna 1";
    String columnName1 = "Nome da coluna 1";
    Column<String> column1 =
      new TextColumn(columnId1, columnName1, null, false, true, null);
    String columnId2 = "Id da coluna 2";
    String columnName2 = "Nome da coluna 2";
    Column<String> column2 =
      new TextColumn(columnId2, columnName2, null, false, true, null);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1, column2),
        false, false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_texto id='" + columnId1 + "' rotulo='" + columnName1
        + "'/>\n";
    xml +=
      "<coluna_de_texto id='" + columnId2 + "' rotulo='" + columnName2
        + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar tabela com colunas de texto e com clulas.
   */
  public void testTextColumn16() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    boolean isOptional = true;
    String columnId = "Nome da coluna 1";
    String columnLabel = "Rtulo da coluna1";
    Column<String> column =
      new TextColumn(columnId, columnLabel, null, false, true, null);
    Integer rowIndex = 1;
    String cellValue = "Valor 1";
    column.addDefaultValue(rowIndex, cellValue);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        isOptional, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column), false,
        false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "' opcional='" + isOptional
        + "'>\n";
    xml +=
      "<coluna_de_texto rotulo='" + columnLabel + "' id='" + columnId + "'/>\n";
    xml +=
      "<celula indice_da_linha='" + rowIndex + "' id_da_coluna='" + columnId
        + "' valor='" + cellValue + "'/>";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar tabela com colunas de texto e com clulas sem ndice da
   * linha.
   */
  public void testTextColumn17() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    boolean isOptional = true;
    String columnId = "Nome da coluna 1";
    String columnLabel = "Rtulo da coluna1";
    String cellValue = "Valor 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "' opcional='" + isOptional
        + "'>\n";
    xml +=
      "<coluna_de_texto rotulo='" + columnLabel + "' id='" + columnId + "'/>\n";
    xml += "<celula id_da_coluna='" + columnId + "'>";
    xml += cellValue;
    xml += "</celula>";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar tabela com colunas de texto e com clulas com ndice da
   * linha negativo.
   */
  public void testTextColumn18() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    boolean isOptional = true;
    String columnId = "Nome da coluna 1";
    String columnLabel = "Rtulo da coluna1";
    String cellValue = "Valor 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "' opcional='" + isOptional
        + "'>\n";
    xml +=
      "<coluna_de_texto rotulo='" + columnLabel + "' id='" + columnId + "'/>\n";
    xml += "<celula indice_da_linha='-1' id_da_coluna='" + columnId + "'>";
    xml += cellValue;
    xml += "</celula>";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar tabela com colunas de texto e com clulas sem id da coluna.
   */
  public void testTextColumn19() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    boolean isOptional = true;
    String columnId = "Nome da coluna 1";
    String columnLabel = "Rtulo da coluna1";
    String cellValue = "Valor 1";
    Integer rowIndex = 0;
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "' opcional='" + isOptional
        + "'>\n";
    xml +=
      "<coluna_de_texto rotulo='" + columnLabel + "' id='" + columnId + "'/>\n";
    xml += "<celula indice_da_linha='" + rowIndex + "'>";
    xml += cellValue;
    xml += "</celula>";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar tabela com colunas de texto e com clulas com id da coluna
   * inexistente.
   */
  public void testTextColumn20() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    boolean isOptional = true;
    String columnId = "Nome da coluna 1";
    String columnLabel = "Rtulo da coluna1";
    String cellValue = "Valor 1";
    Integer rowIndex = 0;
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "' opcional='" + isOptional
        + "'>\n";
    xml +=
      "<coluna_de_texto rotulo='" + columnLabel + "' id='" + columnId + "'/>\n";
    xml +=
      "<celula indice_da_linha='" + rowIndex + "' id_da_coluna='outro_id'>";
    xml += cellValue;
    xml += "</celula>";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar tabela com colunas de texto e com clulas sem valor.
   */
  public void testTextColumn21() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    boolean isOptional = true;
    String columnId = "Nome da coluna 1";
    String columnLabel = "Rtulo da coluna1";
    Column<String> column =
      new TextColumn(columnId, columnLabel, null, false, true, null);
    Integer rowIndex = 1;
    String cellValue = null;
    column.addDefaultValue(rowIndex, cellValue);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        isOptional, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column), false,
        false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "' opcional='" + isOptional
        + "'>\n";
    xml +=
      "<coluna_de_texto rotulo='" + columnLabel + "' id='" + columnId + "'/>\n";
    xml +=
      "<celula indice_da_linha='" + rowIndex + "' id_da_coluna='" + columnId
        + "'/>";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar colunas para booleanos com valor do atributo valor-padro
   * com o padro e valor do atributo editvel com o padro.
   */
  public void testBooleanColumn1() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    Column<Boolean> column1 =
      new BooleanColumn(columnName1, columnName1, false, true, null, null);
    String columnName2 = "Nome da coluna 2";
    Column<Boolean> column2 =
      new BooleanColumn(columnName2, columnName2, false, true, null, null);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1, column2),
        false, false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml += "<coluna_de_booleanos rotulo='" + columnName1 + "'/>\n";
    xml += "<coluna_de_booleanos rotulo='" + columnName2 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar colunas para booleanos com valor do atributo valor-padro
   * com o padro e valor do atributo editvel veradeiro.
   */
  public void testBooleanColumn2() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    boolean isEditable = true;
    String columnName1 = "Nome da coluna 1";
    Column<Boolean> column1 =
      new BooleanColumn(columnName1, columnName1, false, isEditable, null, null);
    String columnName2 = "Nome da coluna 2";
    Column<Boolean> column2 =
      new BooleanColumn(columnName2, columnName2, false, isEditable, null, null);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1, column2),
        false, false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_booleanos rotulo='" + columnName1 + "' editavel='"
        + isEditable + "'/>\n";
    xml +=
      "<coluna_de_booleanos rotulo='" + columnName2 + "' editavel='"
        + isEditable + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar colunas para booleanos com valor do atributo valor-padro
   * com o padro e valor do atributo editvel falso.
   */
  public void testBooleanColumn3() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    boolean isEditable = false;
    Column<Boolean> column1 =
      new BooleanColumn(columnName1, columnName1, false, isEditable, null, null);
    String columnName2 = "Nome da coluna 2";
    Column<Boolean> column2 =
      new BooleanColumn(columnName2, columnName2, false, isEditable, null, null);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1, column2),
        false, false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_booleanos rotulo='" + columnName1 + "' editavel='"
        + isEditable + "'/>\n";
    xml +=
      "<coluna_de_booleanos rotulo='" + columnName2 + "' editavel='"
        + isEditable + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar colunas para booleanos com valor do atributo valor-padro
   * com o padro, valor do atributo editvel com o padro, como valor do
   * atributo "valor para o comando verdadeiro" e "valor para o comando falso"
   * definidos.
   */
  public void testBooleanColumn4() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    String falseCommand1 = "Comando Falso 1";
    String trueCommand1 = "Comando Verdadeiro 1";
    Column<Boolean> column1 =
      new BooleanColumn(columnName1, columnName1, false, true, falseCommand1,
        trueCommand1);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1), false,
        false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_booleanos rotulo='" + columnName1 + "' verdadeiro='"
        + trueCommand1 + "' falso='" + falseCommand1 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar colunas para booleanos com valor do atributo valor-padro
   * falso.
   */
  public void testBooleanColumn5() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    boolean defaultValue1 = false;
    Column<Boolean> column1 =
      new BooleanColumn(columnName1, columnName1, defaultValue1, true, null,
        null);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1), false,
        false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_booleanos rotulo='" + columnName1 + "' padrao='"
        + defaultValue1 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar colunas para booleanos com valor do atributo valor-padro
   * verdadeiro.
   */
  public void testBooleanColumn6() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    boolean defaultValue1 = true;
    Column<Boolean> column1 =
      new BooleanColumn(columnName1, columnName1, defaultValue1, true, null,
        null);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1), false,
        false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_booleanos rotulo='" + columnName1 + "' padrao='"
        + defaultValue1 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Tentando criar 2 colunas para booleanos com o mesmo nome.
   */
  public void testBooleanColumn7() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml += "<coluna_de_booleanos rotulo='" + columnName1 + "'/>\n";
    xml += "<coluna_de_booleanos rotulo='" + columnName1 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Tentando criar colunas para booleanos sem rtulo.
   */
  public void testBooleanColumn8() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml += "<coluna_de_booleanos/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Tentando criar um parmetro do tipo tabela com colunas booleanas com
   * atributos desconhecidos.
   */
  public void testBooleanColumn9() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_booleanos rotulo='" + columnName1 + "' "
        + "desconhecido='valor'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Tentando criar um parmetro do tipo tabela com colunas booleanas com
   * elementos-filhos.
   */
  public void testBooleanColumn10() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml += "<coluna_de_booleanos rotulo='" + columnName1 + "'>\n";
    xml += "<filho/>";
    xml += "</coluna_de_booleanos>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar colunas para booleanos com identificador.
   */
  public void testBooleanColumn11() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnId1 = "Id da coluna 1";
    String columnName1 = "Nome da coluna 1";
    Column<Boolean> column1 =
      new BooleanColumn(columnId1, columnName1, false, true, null, null);
    String columnId2 = "Id da coluna 2";
    String columnName2 = "Nome da coluna 2";
    Column<Boolean> column2 =
      new BooleanColumn(columnId2, columnName2, false, true, null, null);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1, column2),
        false, false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_booleanos id='" + columnId1 + "' rotulo='" + columnName1
        + "'/>\n";
    xml +=
      "<coluna_de_booleanos id='" + columnId2 + "' rotulo='" + columnName2
        + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo tabela com coluna de inteiros.
   */
  public void testIntegerColumn1() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    Integer defaultValue1 = null;
    boolean isEditable1 = true;
    boolean isOptional1 = false;
    Integer maximum1 = null;
    Integer minimum1 = null;
    Column<Integer> column1 =
      new IntegerColumn(columnName1, columnName1, defaultValue1, isOptional1,
        isEditable1, maximum1, minimum1);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1), false,
        false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml += "<coluna_de_inteiros rotulo='" + columnName1 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo tabela com coluna de inteiros com
   * valor-padro.
   */
  public void testIntegerColumn2() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    Integer defaultValue1 = new Integer(1234);
    boolean isOptional1 = false;
    boolean isEditable1 = true;
    Integer maximum1 = null;
    Integer minimum1 = null;
    Column<Integer> column1 =
      new IntegerColumn(columnName1, columnName1, defaultValue1, isOptional1,
        isEditable1, maximum1, minimum1);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1), false,
        false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_inteiros rotulo='" + columnName1 + "' padrao='"
        + defaultValue1 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo tabela com coluna de inteiros com
   * atributo editvel com valor o verdadeiro.
   */
  public void testIntegerColumn3() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    Integer defaultValue1 = null;
    boolean isEditable1 = true;
    boolean isOptional1 = false;
    Integer maximum1 = null;
    Integer minimum1 = null;
    Column<Integer> column1 =
      new IntegerColumn(columnName1, columnName1, defaultValue1, isOptional1,
        isEditable1, maximum1, minimum1);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1), false,
        false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_inteiros rotulo='" + columnName1 + "' editavel='"
        + isEditable1 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo tabela com coluna de inteiros com
   * atributo editvel com valor o falso.
   */
  public void testIntegerColumn4() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    Integer defaultValue1 = null;
    boolean isEditable1 = false;
    boolean isOptional1 = false;
    Integer maximum1 = null;
    Integer minimum1 = null;
    Column<Integer> column1 =
      new IntegerColumn(columnName1, columnName1, defaultValue1, isOptional1,
        isEditable1, maximum1, minimum1);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1), false,
        false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_inteiros rotulo='" + columnName1 + "' editavel='"
        + isEditable1 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo tabela com colunas de inteiros repetidas.
   */
  public void testIntegerColumn5() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml += "<coluna_de_inteiros rotulo='" + columnName1 + "'/>\n";
    xml += "<coluna_de_inteiros rotulo='" + columnName1 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo tabela com colunas de inteiros sem nome.
   */
  public void testIntegerColumn6() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml += "<coluna_de_inteiros/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo tabela com colunas de inteiros com
   * opcional verdadeiro.
   */
  public void testIntegerColumn7() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    Integer defaultValue1 = null;
    boolean isOptional1 = true;
    boolean isEditable1 = true;
    Integer maximum1 = null;
    Integer minimum1 = null;
    Column<Integer> column1 =
      new IntegerColumn(columnName1, columnName1, defaultValue1, isOptional1,
        isEditable1, maximum1, minimum1);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1), false,
        false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_inteiros rotulo='" + columnName1 + "' opcional='"
        + isOptional1 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo tabela com colunas de inteiros com
   * opcional falso.
   */
  public void testIntegerColumn8() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    Integer defaultValue1 = null;
    boolean isOptional1 = false;
    boolean isEditable1 = true;
    Integer maximum1 = null;
    Integer minimum1 = null;
    Column<Integer> column1 =
      new IntegerColumn(columnName1, columnName1, defaultValue1, isOptional1,
        isEditable1, maximum1, minimum1);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1), false,
        false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_inteiros rotulo='" + columnName1 + "' opcional='"
        + isOptional1 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo tabela com colunas de inteiros com
   * opcional no-booleano.
   */
  public void testIntegerColumn9() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String columnName1 = "Nome da coluna 1";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_inteiros rotulo='" + columnName1
        + "' opcional='Valor no-booleano'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo tabela com colunas de inteiros com
   * mximo.
   */
  public void testIntegerColumn10() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    Integer defaultValue1 = null;
    boolean isOptional1 = false;
    boolean isEditable1 = true;
    Integer maximum1 = new Integer(-10);
    Integer minimum1 = null;
    Column<Integer> column1 =
      new IntegerColumn(columnName1, columnName1, defaultValue1, isOptional1,
        isEditable1, maximum1, minimum1);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1), false,
        false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_inteiros rotulo='" + columnName1 + "' maximo='" + maximum1
        + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo tabela com colunas de inteiros com mximo
   * com valor no-inteiro.
   */
  public void testIntegerColumn11() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String columnName1 = "Nome da coluna 1";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_inteiros rotulo='" + columnName1
        + "' maximo='Valor no-inteiro'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo tabela com colunas de inteiros com
   * mnimo.
   */
  public void testIntegerColumn12() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    Integer defaultValue1 = null;
    boolean isOptional1 = false;
    boolean isEditable1 = true;
    Integer maximum1 = null;
    Integer minimum1 = new Integer(-10);
    Column<Integer> column1 =
      new IntegerColumn(columnName1, columnName1, defaultValue1, isOptional1,
        isEditable1, maximum1, minimum1);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1), false,
        false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_inteiros rotulo='" + columnName1 + "' minimo='" + minimum1
        + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar parmetros do tipo tabela com colunas de inteiros com mnimo
   * com valor-no inteiro.
   */
  public void testIntegerColumn13() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String columnName1 = "Nome da coluna 1";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_inteiros rotulo='" + columnName1
        + "' minimo='Valor no-inteiro'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo tabela com coluna de inteiros com mnimo
   * maior do que o mximo.
   */
  public void testIntegerColumn14() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String columnName1 = "Nome da coluna 1";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    Integer maximum = new Integer(-1);
    Integer minimum = new Integer(1);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_inteiros rotulo='" + columnName1 + "' minimo='" + minimum
        + "' maximo='" + maximum + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testnado criar parmetro do tipo tabela com coluna de inteiros com mnimo
   * igual ao mximo.
   */
  public void testIntegerColumn15() throws ParseException {
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator("comando", null, ExecutionType.SIMPLE, null,
        false, null, null);
    Integer maximum;
    Integer minimum;
    maximum = minimum = new Integer(10);
    Column<?> column =
      new IntegerColumn("Nome da coluna", "Nome da coluna", null, false, true,
        maximum, minimum);
    TableParameter parameter =
      new TableParameter("Nome da tabela", "Rtulo da tabela",
        "Descrio da tabela", false, DEFAULT_COMMAND_LINE_PATTERN, Arrays
          .asList(column), false, false, null, null);
    ParameterGroup group = new ParameterGroup("grupo");
    group.addParameter(parameter);
    version.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + version.getCommandBinaryName() + "'>";
    xml += "<grupo rotulo='" + group.getName() + "'>";
    xml +=
      "<tabela nome='" + parameter.getName() + "' rotulo='"
        + parameter.getLabel() + "' dica='" + parameter.getDescription() + "'>";
    xml +=
      "<coluna_de_inteiros rotulo='" + column.getLabel() + "' minimo='"
        + minimum + "' maximo='" + maximum + "'/>";
    xml += "</tabela>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  /**
   * Testando criar parmetro do tipo tabela com colunas de inteiros com
   * valor-padro menor do que o mnimo.
   */
  public void testIntegerColumn16() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String columnName = "Nome da coluna 1";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    Integer defaultValue = new Integer(-2);
    Integer maximum = new Integer(1);
    Integer minimum = new Integer(-1);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_inteiros rotulo='" + columnName + "' minimo='" + minimum
        + "' maximo='" + maximum + "' padrao='" + defaultValue + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetro do tipo tabela com colunas de inteiros com
   * valor-padro maior do que o mximo.
   */
  public void testIntegerColumn17() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String columnName = "Nome da coluna 1";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    Integer defaultValue = new Integer(2);
    Integer maximum = new Integer(1);
    Integer minimum = new Integer(-1);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_inteiros rotulo='" + columnName + "' minimo='" + minimum
        + "' maximo='" + maximum + "' padrao='" + defaultValue + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo tabela com coluna de inteiros com
   * atributos desconhecidos.
   */
  public void testIntegerColumn18() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_inteiros rotulo='" + columnName1 + "'"
        + " desconhecido='valor'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo tabela com coluna de inteiros com
   * elementos-filhos.
   */
  public void testIntegerColumn19() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml += "<coluna_de_inteiros rotulo='" + columnName1 + "'>\n";
    xml += "<filho/>\n";
    xml += "</coluna_de_inteiros>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo tabela com coluna de inteiros com
   * identificador.
   */
  public void testIntegerColumn20() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnId1 = "Identificador da coluna 1";
    String columnName1 = "Nome da coluna 1";
    Integer defaultValue1 = null;
    boolean isEditable1 = true;
    boolean isOptional1 = false;
    Integer maximum1 = null;
    Integer minimum1 = null;
    Column<?> column1 =
      new IntegerColumn(columnId1, columnName1, defaultValue1, isOptional1,
        isEditable1, maximum1, minimum1);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1), false,
        false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_inteiros id='" + columnId1 + "' rotulo='" + columnName1
        + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar colunas do tipo real.
   */
  public void testDoubleColumn1() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    Column<?> column1 =
      new DoubleColumn(columnName1, columnName1, null, false, true, null, true,
        null, true);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1), false,
        false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml += "<coluna_de_reais rotulo='" + columnName1 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar colunas do tipo real com valor-padro.
   */
  public void testDoubleColumn2() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    Double defaultValue1 = new Double(1234.56789);
    Column<?> column1 =
      new DoubleColumn(columnName1, columnName1, defaultValue1, false, true,
        null, true, null, true);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1), false,
        false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_reais rotulo='" + columnName1 + "' padrao='" + defaultValue1
        + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar colunas do tipo real com rtulo repetido.
   */
  public void testDoubleColumn3() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml += "<coluna_de_reais rotulo='" + columnName1 + "'/>\n";
    xml += "<coluna_de_reais rotulo='" + columnName1 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar colunas de reais sem rtulo.
   */
  public void testDoubleColumn4() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml += "<coluna_de_reais/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar colunas de reais com o atributo opcional com o valor
   * verdadeiro.
   */
  public void testDoubleColumn5() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    boolean isOptional1 = true;
    Column<?> column1 =
      new DoubleColumn(columnName1, columnName1, null, isOptional1, true, null,
        true, null, true);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1), false,
        false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_reais rotulo='" + columnName1 + "' opcional='" + isOptional1
        + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar colunas de reais com o atributo opcional com o valor falso.
   */
  public void testDoubleColumn6() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    boolean isOptional1 = false;
    Column<?> column1 =
      new DoubleColumn(columnName1, columnName1, null, isOptional1, true, null,
        true, null, true);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1), false,
        false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_reais rotulo='" + columnName1 + "' opcional='" + isOptional1
        + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar colunas de reais com atributo opcional com um valor que no
   * represente um valor booleano.
   */
  public void testDoubleColumn7() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String columnName1 = "Nome da coluna 1";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_reais rotulo='" + columnName1
        + "' opcional='Valor no-booleano'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar colunas reais com mximo.
   */
  public void testDoubleColumn8() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    boolean isOptional1 = false;
    boolean isEditable1 = true;
    Double maximum1 = new Double(-10.5);
    Double minimum1 = null;
    boolean includeMaximum1 = true;
    boolean includeMinimum1 = true;
    Column<?> column1 =
      new DoubleColumn(columnName1, columnName1, null, isOptional1,
        isEditable1, maximum1, includeMaximum1, minimum1, includeMinimum1);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1), false,
        false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_reais rotulo='" + columnName1 + "' maximo='" + maximum1
        + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar colunas reais com mximo com um valor no-real.
   */
  public void testDoubleColumn9() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String columnName1 = "Nome da coluna 1";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_reais rotulo='" + columnName1
        + "' maximo='Valor no-real'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar colunas reais com mnimo.
   */
  public void testDoubleColumn10() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    boolean isOptional1 = false;
    boolean isEditable1 = true;
    Double maximum1 = null;
    Double minimum1 = new Double(-10.5);
    boolean includeMaximum1 = true;
    boolean includeMinimum1 = true;
    Column<?> column1 =
      new DoubleColumn(columnName1, columnName1, null, isOptional1,
        isEditable1, maximum1, includeMaximum1, minimum1, includeMinimum1);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1), false,
        false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_reais rotulo='" + columnName1 + "' minimo='" + minimum1
        + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar colunas reais com mnimo no-real.
   */
  public void testDoubleColumn11() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String columnName1 = "Nome da coluna 1";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_reais rotulo='" + columnName1
        + "' minimo='Valor no-real'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar colunas reais com mximo menor do que o mnimo.
   */
  public void testDoubleColumn12() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String columnName1 = "Nome da coluna 1";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    Double maximum = new Double(-1.1);
    Double minimum = new Double(1.1);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_reais rotulo='" + columnName1 + "' minimo='" + minimum
        + "' maximo='" + maximum + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar colunas reais com mximo e incluir mximo falso.
   */
  public void testDoubleColumn13() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    boolean isOptional1 = false;
    boolean isEditable1 = true;
    Double maximum1 = new Double(123.456);
    boolean includeMaximum1 = false;
    Column<?> column1 =
      new DoubleColumn(columnName1, columnName1, null, isOptional1,
        isEditable1, maximum1, includeMaximum1, null, true);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1), false,
        false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_reais rotulo='" + columnName1 + "' maximo='" + maximum1
        + "' incluir_maximo='" + includeMaximum1 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar colunas reais com mximo e incluir mximo verdadeiro.
   */
  public void testDoubleColumn14() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    boolean isOptional1 = false;
    boolean isEditable1 = true;
    Double maximum1 = new Double(123.456);
    boolean includeMaximum1 = true;
    Column<?> column1 =
      new DoubleColumn(columnName1, columnName1, null, isOptional1,
        isEditable1, maximum1, includeMaximum1, null, true);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1), false,
        false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_reais rotulo='" + columnName1 + "' maximo='" + maximum1
        + "' incluir_maximo='" + includeMaximum1 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar colunas reais com mnimo e incluir mnimo verdadeiro.
   */
  public void testDoubleColumn15() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    boolean isOptional1 = false;
    boolean isEditable1 = true;
    Double minimum1 = new Double(123.456);
    boolean includeMinimum1 = true;
    Column<?> column1 =
      new DoubleColumn(columnName1, columnName1, null, isOptional1,
        isEditable1, null, false, minimum1, includeMinimum1);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1), false,
        false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_reais rotulo='" + columnName1 + "' minimo='" + minimum1
        + "' incluir_minimo='" + includeMinimum1 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar colunas reais com mnimo e incluir mnimo falso.
   */
  public void testDoubleColumn16() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    boolean isOptional1 = false;
    boolean isEditable1 = true;
    Double minimum1 = new Double(123.456);
    boolean includeMinimum1 = false;
    Column<?> column1 =
      new DoubleColumn(columnName1, columnName1, null, isOptional1,
        isEditable1, null, false, minimum1, includeMinimum1);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1), false,
        false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_reais rotulo='" + columnName1 + "' minimo='" + minimum1
        + "' incluir_minimo='" + includeMinimum1 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Tentando criar colunas do tipo real com atributos desconhecidos.
   */
  public void testDoubleColumn17() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_reais rotulo='" + columnName1 + "' "
        + "desconhecido='valor'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Tentando criar colunas do tipo real com elementos-filhos.
   */
  public void testDoubleColumn18() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml += "<coluna_de_reais rotulo='" + columnName1 + "'>\n";
    xml += "<filho/>\n";
    xml += "</coluna_de_reais>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar colunas do tipo real com atributo editvel verdadeiro.
   */
  public void testDoubleColumn19() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    boolean isEditable1 = true;
    String columnName1 = "Nome da coluna 1";
    Column<?> column1 =
      new DoubleColumn(columnName1, columnName1, null, false, isEditable1,
        null, true, null, true);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1), false,
        false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_reais rotulo='" + columnName1 + "' editavel='" + isEditable1
        + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar colunas do tipo real com atributo editvel falso.
   */
  public void testDoubleColumn20() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    boolean isEditable1 = false;
    String columnName1 = "Nome da coluna 1";
    Column<?> column1 =
      new DoubleColumn(columnName1, columnName1, null, false, isEditable1,
        null, true, null, true);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1), false,
        false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_reais rotulo='" + columnName1 + "' editavel='" + isEditable1
        + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar colunas do tipo real com identificador.
   */
  public void testDoubleColumn21() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnId1 = "Identificador da Coluna 1";
    String columnName1 = "Nome da coluna 1";
    Column<?> column1 =
      new DoubleColumn(columnId1, columnName1, null, false, true, null, true,
        null, true);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1), false,
        false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<coluna_de_reais id='" + columnId1 + "' rotulo='" + columnName1
        + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar tabela com atributo opcional verdadeiro.
   */
  public void testTableParameter1() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    boolean isOptional = true;
    String columnName1 = "Nome da coluna 1";
    Column<?> column1 =
      new TextColumn(columnName1, columnName1, null, false, true, null);
    String columnName2 = "Nome da coluna 2";
    Column<?> column2 =
      new TextColumn(columnName2, columnName2, null, false, true, null);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        isOptional, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1,
          column2), false, false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "' opcional='" + isOptional
        + "'>\n";
    xml += "<coluna_de_texto rotulo='" + columnName1 + "'/>\n";
    xml += "<coluna_de_texto rotulo='" + columnName2 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar tabela com atributo opcional falso.
   */
  public void testTableParameter2() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    boolean isOptional = false;
    String columnName1 = "Nome da coluna 1";
    Column<?> column1 =
      new TextColumn(columnName1, columnName1, null, false, true, null);
    String columnName2 = "Nome da coluna 2";
    Column<?> column2 =
      new TextColumn(columnName2, columnName2, null, false, true, null);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        isOptional, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1,
          column2), false, false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "' opcional='" + isOptional
        + "'>\n";
    xml += "<coluna_de_texto rotulo='" + columnName1 + "'/>\n";
    xml += "<coluna_de_texto rotulo='" + columnName2 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar colunas de texto com atributo opcional no-booleano.
   */
  public void testTableParameter3() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String columnName1 = "Nome da coluna 1";
    String columnName2 = "Nome da coluna 2";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription
        + "' opcional='Valor no booleano'>\n";
    xml += "<coluna_de_texto rotulo='" + columnName1 + "'/>\n";
    xml += "<coluna_de_texto rotulo='" + columnName2 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar tabela com atributo delimitar tabela falso.
   */
  public void testTableParameter4() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    boolean delimitTable = false;
    String columnName1 = "Nome da coluna 1";
    Column<?> column1 =
      new TextColumn(columnName1, columnName1, null, false, true, null);
    String columnName2 = "Nome da coluna 2";
    Column<?> column2 =
      new TextColumn(columnName2, columnName2, null, false, true, null);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1, column2),
        delimitTable, false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "' delimitar_tabela='"
        + delimitTable + "'>\n";
    xml += "<coluna_de_texto rotulo='" + columnName1 + "'/>\n";
    xml += "<coluna_de_texto rotulo='" + columnName2 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar tabela com atributo delimitar tabela verdadeiro.
   */
  public void testTableParameter5() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    boolean delimitTable = true;
    String columnName1 = "Nome da coluna 1";
    Column<?> column1 =
      new TextColumn(columnName1, columnName1, null, false, true, null);
    String columnName2 = "Nome da coluna 2";
    Column<?> column2 =
      new TextColumn(columnName2, columnName2, null, false, true, null);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1, column2),
        delimitTable, false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "' delimitar_tabela='"
        + delimitTable + "'>\n";
    xml += "<coluna_de_texto rotulo='" + columnName1 + "'/>\n";
    xml += "<coluna_de_texto rotulo='" + columnName2 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar tabela com atributo delimitar_tabela falso.
   */
  public void testTableParameter6() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String columnName1 = "Nome da coluna 1";
    String columnName2 = "Nome da coluna 2";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription
        + "' delimitar_tabela='Valor no booleano'>\n";
    xml += "<coluna_de_texto rotulo='" + columnName1 + "'/>\n";
    xml += "<coluna_de_texto rotulo='" + columnName2 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar tabela com atributo delimitar linhas falso.
   */
  public void testTableParameter7() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    boolean delimitRows = false;
    String columnName1 = "Nome da coluna 1";
    Column<?> column1 =
      new TextColumn(columnName1, columnName1, null, false, true, null);
    String columnName2 = "Nome da coluna 2";
    Column<?> column2 =
      new TextColumn(columnName2, columnName2, null, false, true, null);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1, column2),
        false, delimitRows, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "' delimitar_linhas='"
        + delimitRows + "'>\n";
    xml += "<coluna_de_texto rotulo='" + columnName1 + "'/>\n";
    xml += "<coluna_de_texto rotulo='" + columnName2 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar tabela com atributo delimitar linhas verdadeiro.
   */
  public void testTableParameter8() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    boolean delimitRows = true;
    String columnName1 = "Nome da coluna 1";
    Column<?> column1 =
      new TextColumn(columnName1, columnName1, null, false, true, null);
    String columnName2 = "Nome da coluna 2";
    Column<?> column2 =
      new TextColumn(columnName2, columnName2, null, false, true, null);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1, column2),
        false, delimitRows, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "' delimitar_linhas='"
        + delimitRows + "'>\n";
    xml += "<coluna_de_texto rotulo='" + columnName1 + "'/>\n";
    xml += "<coluna_de_texto rotulo='" + columnName2 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar tabela com atributo delimitar linhas no-booleano.
   */
  public void testTableParameter9() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String columnName1 = "Nome da coluna 1";
    String columnName2 = "Nome da coluna 2";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription
        + "' delimitar_linhas='Valor no booleano'>\n";
    xml += "<coluna_de_texto rotulo='" + columnName1 + "'/>\n";
    xml += "<coluna_de_texto rotulo='" + columnName2 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar tabela sem nome.
   */
  public void testTableParameter10() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String columnName1 = "Nome da coluna 1";
    String columnName2 = "Nome da coluna 2";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela rotulo='" + parameterLabel + "' dica='" + parameterDescription
        + "'>\n";
    xml += "<coluna_de_texto rotulo='" + columnName1 + "'/>\n";
    xml += "<coluna_de_texto rotulo='" + columnName2 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar tabela sem rtulo.
   */
  public void testTableParameter11() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String columnName1 = "Nome da coluna 1";
    String columnName2 = "Nome da coluna 2";
    String parameterName = "Nome do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' dica='" + parameterDescription
        + "'>\n";
    xml += "<coluna_de_texto rotulo='" + columnName1 + "'/>\n";
    xml += "<coluna_de_texto rotulo='" + columnName2 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar tabela sem dica.
   */
  public void testTableParameter12() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String columnName1 = "Nome da coluna 1";
    String columnName2 = "Nome da coluna 2";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel + "'>\n";
    xml += "<coluna_de_texto rotulo='" + columnName1 + "'/>\n";
    xml += "<coluna_de_texto rotulo='" + columnName2 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar tabela sem colunas.
   */
  public void testTableParameter13() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar parmetros do tipo tabela com atributos desconhecidos.
   */
  public void testTableParameter14() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String columnName1 = "Nome da coluna 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "' desconhecido='valor'>\n";
    xml += "<coluna_de_texto rotulo='" + columnName1 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando criar tabela informando a quantidade de linhas.
   */
  public void testTableParameter15() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    boolean isOptional = true;
    String columnName1 = "Nome da coluna 1";
    Column<?> column1 =
      new TextColumn(columnName1, columnName1, null, false, true, null);
    String columnName2 = "Nome da coluna 2";
    Column<?> column2 =
      new TextColumn(columnName2, columnName2, null, false, true, null);
    int rowCount = 10;
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        isOptional, DEFAULT_COMMAND_LINE_PATTERN, Arrays.asList(column1,
          column2), false, false, rowCount, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "' opcional='" + isOptional
        + "' quantidade_de_linhas='" + rowCount + "'>\n";
    xml += "<coluna_de_texto rotulo='" + columnName1 + "'/>\n";
    xml += "<coluna_de_texto rotulo='" + columnName2 + "'/>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar tabela com formato no comando exibindo nome e valor.
   */
  public void testTableParameter16() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    boolean isOptional = false;
    String columnName1 = "Nome da coluna 1";
    Column<?> column1 =
      new TextColumn(columnName1, columnName1, null, false, true, null);
    String commandLinePattern =
      "Nome: $NOME_DO_PARAMETRO Valor: $VALOR_DO_PARAMETRO";
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        isOptional, commandLinePattern, Arrays.asList(column1), false, false,
        null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml += "<coluna_de_texto rotulo='" + columnName1 + "'/>\n";
    xml += "<formato_no_comando>\n";
    xml += commandLinePattern;
    xml += "</formato_no_comando>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Testando criar tabela com formato no comando sem exibir nome e valor.
   */
  public void testTableParameter17() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    boolean isOptional = false;
    String columnName1 = "Nome da coluna 1";
    Column<?> column1 =
      new TextColumn(columnName1, columnName1, null, false, true, null);
    TableParameter parameter =
      new TableParameter(parameterName, parameterLabel, parameterDescription,
        isOptional, null, Arrays.asList(column1), false, false, null, null);
    group.addParameter(parameter);
    expectedVersion.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<tabela nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>\n";
    xml += "<coluna_de_texto rotulo='" + columnName1 + "'/>\n";
    xml += "<formato_no_comando>\n";
    xml += "</formato_no_comando>\n";
    xml += "</tabela>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testInputFileParameter1() throws ParseException {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    InputFileParameter inputFileParameter =
      new InputFileParameter(parameterName, parameterLabel,
        parameterDescription, false, DEFAULT_COMMAND_LINE_PATTERN, null,
        FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE);
    group.addParameter(inputFileParameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_entrada nome='" + parameterName + "' rotulo='"
        + parameterLabel + "' dica='" + parameterDescription + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  public void testInputFileParameter2() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_entrada nome='" + parameterName + "' rotulo='"
        + parameterLabel + "' dica='" + parameterDescription + "'/>\n";
    xml +=
      "<arquivo_de_entrada nome='" + parameterName + "' rotulo='"
        + parameterLabel + "' dica='" + parameterDescription + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testInputFileParameter3() throws ParseException {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    boolean parameterOptional = false;
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    InputFileParameter inputFileParameter =
      new InputFileParameter(parameterName, parameterLabel,
        parameterDescription, parameterOptional, DEFAULT_COMMAND_LINE_PATTERN,
        null, FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE);
    group.addParameter(inputFileParameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_entrada nome='" + parameterName + "' rotulo='"
        + parameterLabel + "' dica='" + parameterDescription + "' opcional='"
        + parameterOptional + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  public void testInputFileParameter4() throws ParseException {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    boolean parameterOptional = true;
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    InputFileParameter inputFileParameter =
      new InputFileParameter(parameterName, parameterLabel,
        parameterDescription, parameterOptional, DEFAULT_COMMAND_LINE_PATTERN,
        null, FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE);
    group.addParameter(inputFileParameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_entrada nome='" + parameterName + "' rotulo='"
        + parameterLabel + "' dica='" + parameterDescription + "' opcional='"
        + parameterOptional + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  public void testInputFileParameter7() throws ParseException {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    InputFileParameter inputFileParameter =
      new InputFileParameter(parameterName, parameterLabel,
        parameterDescription, false, DEFAULT_COMMAND_LINE_PATTERN, null,
        FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE);
    group.addParameter(inputFileParameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_entrada nome='" + parameterName + "' rotulo='"
        + parameterLabel + "' dica='" + parameterDescription
        + "' categoria='arquivo'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  public void testInputFileParameter8() throws ParseException {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    InputFileParameter inputFileParameter =
      new InputFileParameter(parameterName, parameterLabel,
        parameterDescription, false, DEFAULT_COMMAND_LINE_PATTERN, null,
        FileParameterMode.DIRECTORY, FileParameterPipeAcceptance.TRUE);
    group.addParameter(inputFileParameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_entrada nome='" + parameterName + "' rotulo='"
        + parameterLabel + "' dica='" + parameterDescription
        + "' categoria='diretorio'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  public void testInputFileParameter9() throws ParseException {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    InputFileParameter inputFileParameter =
      new InputFileParameter(parameterName, parameterLabel,
        parameterDescription, false, DEFAULT_COMMAND_LINE_PATTERN, null,
        FileParameterMode.DIRECTORY_AND_REGULAR_FILE,
        FileParameterPipeAcceptance.TRUE);
    group.addParameter(inputFileParameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_entrada nome='" + parameterName + "' rotulo='"
        + parameterLabel + "' dica='" + parameterDescription
        + "' categoria='ambos'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  public void testInputFileParameter10() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_entrada rotulo='" + parameterLabel + "' dica='"
        + parameterDescription + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testInputFileParameter11() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_entrada nome='" + parameterName + "' dica='"
        + parameterDescription + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testInputFileParameter12() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_entrada nome='" + parameterName + "' rotulo='"
        + parameterLabel + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testInputFileParameter13() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_entrada nome='" + parameterName + "' rotulo='"
        + parameterLabel + "' dica='" + parameterDescription
        + "' categoria='erro'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testInputFileParameter14() throws ParseException {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    InputFileParameter inputFileParameter =
      new InputFileParameter(parameterName, parameterLabel,
        parameterDescription, false, DEFAULT_COMMAND_LINE_PATTERN, null,
        FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE);
    group.addParameter(inputFileParameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_entrada nome='" + parameterName + "' rotulo='"
        + parameterLabel + "' dica='" + parameterDescription
        + "' permitir_pipe='true'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  public void testInputFileParameter15() throws ParseException {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    InputFileParameter inputFileParameter =
      new InputFileParameter(parameterName, parameterLabel,
        parameterDescription, false, DEFAULT_COMMAND_LINE_PATTERN, null,
        FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.FALSE);
    group.addParameter(inputFileParameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_entrada nome='" + parameterName + "' rotulo='"
        + parameterLabel + "' dica='" + parameterDescription
        + "' permitir_pipe='false'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  /**
   * Tentando criar arquivos de entrada com atributos desconhecidos.
   */
  public void testInputFileParameter16() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_entrada nome='" + parameterName + "' rotulo='"
        + parameterLabel + "' dica='" + parameterDescription + "' "
        + "desconhecido='valor'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testInputFileParameter18() throws ParseException {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    String commandLinePattern =
      "Nome: $NOME_DO_PARAMETRO Valor: $VALOR_DO_PARAMETRO";
    InputFileParameter inputFileParameter =
      new InputFileParameter(parameterName, parameterLabel,
        parameterDescription, false, commandLinePattern, null,
        FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE);
    group.addParameter(inputFileParameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_entrada nome='" + parameterName + "' rotulo='"
        + parameterLabel + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<formato_no_comando>" + commandLinePattern + "</formato_no_comando>";
    xml += "</arquivo_de_entrada>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  public void testInputFileParameter19() throws ParseException {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    InputFileParameter inputFileParameter =
      new InputFileParameter(parameterName, parameterLabel,
        parameterDescription, false, null, null,
        FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE);
    group.addParameter(inputFileParameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_entrada nome='" + parameterName + "' rotulo='"
        + parameterLabel + "' dica='" + parameterDescription + "'>\n";
    xml += "<formato_no_comando></formato_no_comando>";
    xml += "</arquivo_de_entrada>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  /**
   * Testando criar um parmetro do tipo lista de arquivo de entrada.
   */
  public void testInputFileListParameter1() throws ParseException {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    InputFileListParameter inputFileListParameter =
      new InputFileListParameter(parameterName, parameterLabel,
        parameterDescription, false, DEFAULT_COMMAND_LINE_PATTERN, null,
        FileParameterMode.REGULAR_FILE, true);
    group.addParameter(inputFileListParameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_entrada selecao_multipla='sim' nome='" + parameterName
        + "' rotulo='" + parameterLabel + "' dica='" + parameterDescription
        + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  /**
   * Testando criar um parmetro do tipo lista de arquivo de entrada obrigatrio
   * (explcito).
   */
  public void testInputFileListParameter3() throws ParseException {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    boolean parameterOptional = false;
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    InputFileListParameter inputFileListParameter =
      new InputFileListParameter(parameterName, parameterLabel,
        parameterDescription, parameterOptional, DEFAULT_COMMAND_LINE_PATTERN,
        null, FileParameterMode.REGULAR_FILE, true);
    group.addParameter(inputFileListParameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_entrada selecao_multipla='sim' nome='" + parameterName
        + "' rotulo='" + parameterLabel + "' dica='" + parameterDescription
        + "' opcional='" + parameterOptional + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  /**
   * Testando criar um parmetro do tipo lista de arquivo de entrada opcional.
   */
  public void testInputFileListParameter4() throws ParseException {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    boolean parameterOptional = true;
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    InputFileListParameter inputFileListParameter =
      new InputFileListParameter(parameterName, parameterLabel,
        parameterDescription, parameterOptional, DEFAULT_COMMAND_LINE_PATTERN,
        null, FileParameterMode.REGULAR_FILE, true);
    group.addParameter(inputFileListParameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_entrada selecao_multipla='sim' nome='" + parameterName
        + "' rotulo='" + parameterLabel + "' dica='" + parameterDescription
        + "' opcional='" + parameterOptional + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  /**
   * Testando criar um parmetro do tipo lista de arquivo de entrada categoria
   * arquivo regular (explcito).
   */
  public void testInputFileListParameter7() throws ParseException {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    InputFileListParameter inputFileListParameter =
      new InputFileListParameter(parameterName, parameterLabel,
        parameterDescription, false, DEFAULT_COMMAND_LINE_PATTERN, null,
        FileParameterMode.REGULAR_FILE, true);
    group.addParameter(inputFileListParameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_entrada selecao_multipla='sim' nome='" + parameterName
        + "' rotulo='" + parameterLabel + "' dica='" + parameterDescription
        + "' categoria='arquivo'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  /**
   * Testando criar um parmetro do tipo lista de arquivo de entrada categoria
   * diretrio.
   */
  public void testInputFileListParameter8() throws ParseException {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    InputFileListParameter inputFileListParameter =
      new InputFileListParameter(parameterName, parameterLabel,
        parameterDescription, false, DEFAULT_COMMAND_LINE_PATTERN, null,
        FileParameterMode.DIRECTORY, true);
    group.addParameter(inputFileListParameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_entrada selecao_multipla='sim' nome='" + parameterName
        + "' rotulo='" + parameterLabel + "' dica='" + parameterDescription
        + "' categoria='diretorio'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  /**
   * Testando criar um parmetro do tipo lista de arquivo de entrada categoria
   * ambos.
   */
  public void testInputFileListParameter9() throws ParseException {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    InputFileListParameter inputFileListParameter =
      new InputFileListParameter(parameterName, parameterLabel,
        parameterDescription, false, DEFAULT_COMMAND_LINE_PATTERN, null,
        FileParameterMode.DIRECTORY_AND_REGULAR_FILE, true);
    group.addParameter(inputFileListParameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_entrada selecao_multipla='sim' nome='" + parameterName
        + "' rotulo='" + parameterLabel + "' dica='" + parameterDescription
        + "' categoria='ambos'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  /**
   * Testando criar um parmetro do tipo lista de arquivo de entrada ordenar sim
   * (explcito).
   */
  public void testInputFileListParameter14() throws ParseException {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    boolean parameterMustSort = true;
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    InputFileListParameter inputFileListParameter =
      new InputFileListParameter(parameterName, parameterLabel,
        parameterDescription, false, DEFAULT_COMMAND_LINE_PATTERN, null,
        FileParameterMode.REGULAR_FILE, parameterMustSort);
    group.addParameter(inputFileListParameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_entrada selecao_multipla='sim' nome='" + parameterName
        + "' rotulo='" + parameterLabel + "' dica='" + parameterDescription
        + "' ordenar='" + parameterMustSort + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  /**
   * Testando criar um parmetro do tipo lista de arquivo de entrada ordenar
   * no.
   */
  public void testInputFileListParameter15() throws ParseException {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    boolean parameterMustSort = false;
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    InputFileListParameter inputFileListParameter =
      new InputFileListParameter(parameterName, parameterLabel,
        parameterDescription, false, DEFAULT_COMMAND_LINE_PATTERN, null,
        FileParameterMode.REGULAR_FILE, parameterMustSort);
    group.addParameter(inputFileListParameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_entrada selecao_multipla='sim' nome='" + parameterName
        + "' rotulo='" + parameterLabel + "' dica='" + parameterDescription
        + "' ordenar='" + parameterMustSort + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  /**
   * Testando a criao de {@link OutputFileParameter parmetros do tipo arquivo
   * de sada}: utilizando o mximo de valores-padro possveis.
   * 
   * @throws ParseException Em caso de falha no teste.
   */
  public void testOutputFileParameter1() throws ParseException {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    OutputFileParameter outputFileParameter =
      OutputFileParameterFactory.create(new Pair(NAME_PARAMETER_NAME,
        parameterName), new Pair(LABEL_PARAMETER_NAME, parameterLabel),
        new Pair(DESCRIPTION_PARAMETER_NAME, parameterDescription));
    group.addParameter(outputFileParameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_saida nome='" + parameterName + "' rotulo='"
        + parameterLabel + "' dica='" + parameterDescription + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  /**
   * Testando a criao de {@link OutputFileParameter parmetros do tipo arquivo
   * de sada}: criando 2 parmetros iguais.
   */
  public void testOutputFileParameter2() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_saida nome='" + parameterName + "' rotulo='"
        + parameterLabel + "' dica='" + parameterDescription + "'/>\n";
    xml +=
      "<arquivo_de_saida nome='" + parameterName + "' rotulo='"
        + parameterLabel + "' dica='" + parameterDescription + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando a criao de {@link OutputFileParameter parmetros do tipo arquivo
   * de sada}: com atributo opcional igual a no.
   * 
   * @throws ParseException Em caso de falha no teste.
   */
  public void testOutputFileParameter3() throws ParseException {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    boolean parameterOptional = false;
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    OutputFileParameter outputFileParameter =
      OutputFileParameterFactory.create(new Pair(NAME_PARAMETER_NAME,
        parameterName), new Pair(LABEL_PARAMETER_NAME, parameterLabel),
        new Pair(DESCRIPTION_PARAMETER_NAME, parameterDescription), new Pair(
          IS_OPTIONAL_PARAMETER_NAME, parameterOptional));
    group.addParameter(outputFileParameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_saida nome='" + parameterName + "' rotulo='"
        + parameterLabel + "' dica='" + parameterDescription + "' opcional='"
        + parameterOptional + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  /**
   * Testando a criao de {@link OutputFileParameter parmetros do tipo arquivo
   * de sada}: com atributo opcional igual a sim.
   * 
   * @throws ParseException Em caso de falha no teste.
   */
  public void testOutputFileParameter4() throws ParseException {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    boolean parameterOptional = true;
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    OutputFileParameter outputFileParameter =
      OutputFileParameterFactory.create(new Pair(NAME_PARAMETER_NAME,
        parameterName), new Pair(LABEL_PARAMETER_NAME, parameterLabel),
        new Pair(DESCRIPTION_PARAMETER_NAME, parameterDescription), new Pair(
          IS_OPTIONAL_PARAMETER_NAME, parameterOptional));
    group.addParameter(outputFileParameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_saida nome='" + parameterName + "' rotulo='"
        + parameterLabel + "' dica='" + parameterDescription + "' opcional='"
        + parameterOptional + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  /**
   * Testando a criao de {@link OutputFileParameter parmetros do tipo arquivo
   * de sada}: com categoria do tipo arquivo.
   * 
   * @throws ParseException Em caso de falha no teste.
   */
  public void testOutputFileParameter5() throws ParseException {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    FileParameterMode parameterMode = FileParameterMode.REGULAR_FILE;
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    OutputFileParameter outputFileParameter =
      OutputFileParameterFactory.create(new Pair(NAME_PARAMETER_NAME,
        parameterName), new Pair(LABEL_PARAMETER_NAME, parameterLabel),
        new Pair(DESCRIPTION_PARAMETER_NAME, parameterDescription), new Pair(
          MODE_PARAMETER_NAME, parameterMode));
    group.addParameter(outputFileParameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_saida nome='" + parameterName + "' rotulo='"
        + parameterLabel + "' dica='" + parameterDescription
        + "' categoria='arquivo'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  /**
   * Testando a criao de {@link OutputFileParameter parmetros do tipo arquivo
   * de sada}: com categoria do tipo diretrio.
   * 
   * @throws ParseException Em caso de falha no teste.
   */
  public void testOutputFileParameter6() throws ParseException {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    FileParameterMode parameterMode = FileParameterMode.DIRECTORY;
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    OutputFileParameter outputFileParameter =
      OutputFileParameterFactory.create(new Pair(NAME_PARAMETER_NAME,
        parameterName), new Pair(LABEL_PARAMETER_NAME, parameterLabel),
        new Pair(DESCRIPTION_PARAMETER_NAME, parameterDescription), new Pair(
          MODE_PARAMETER_NAME, parameterMode));
    group.addParameter(outputFileParameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_saida nome='" + parameterName + "' rotulo='"
        + parameterLabel + "' dica='" + parameterDescription
        + "' categoria='diretorio'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  /**
   * Testando a criao de {@link OutputFileParameter parmetros do tipo arquivo
   * de sada}: sem nome.
   * 
   * @throws ParseException Em caso de falha no teste.
   */
  public void testOutputFileParameter7() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_saida rotulo='" + parameterLabel + "' dica='"
        + parameterDescription + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando a criao de {@link OutputFileParameter parmetros do tipo arquivo
   * de sada}: sem rtulo.
   * 
   * @throws ParseException Em caso de falha no teste.
   */
  public void testOutputFileParameter8() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_saida nome='" + parameterName + "' dica='"
        + parameterDescription + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando a criao de {@link OutputFileParameter parmetros do tipo arquivo
   * de sada}: com categoria ambos.
   * 
   * @throws ParseException Em caso de falha no teste.
   */
  public void testOutputFileParameter9() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String category = "ambos";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_saida nome='" + parameterName + "' rotulo='"
        + parameterLabel + "' dica='" + parameterDescription + "' categoria='"
        + category + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando a criao de {@link OutputFileParameter parmetros do tipo arquivo
   * de sada}: sem dica.
   * 
   * @throws ParseException Em caso de falha no teste.
   */
  public void testOutputFileParameter10() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_saida nome='" + parameterName + "' rotulo='"
        + parameterLabel + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando a criao de {@link OutputFileParameter parmetros do tipo arquivo
   * de sada}: com categoria desconhecida.
   * 
   * @throws ParseException Em caso de falha no teste.
   */
  public void testOutputFileParameter11() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_saida nome='" + parameterName + "' rotulo='"
        + parameterLabel + "' dica='" + parameterDescription
        + "' categoria='erro'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando a criao de {@link OutputFileParameter parmetros do tipo arquivo
   * de sada}: com permitir pipe igual a sim.
   * 
   * @throws ParseException Em caso de falha no teste.
   */
  public void testOutputFileParameter12() throws ParseException {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    FileParameterPipeAcceptance parameterCanUsePipe =
      FileParameterPipeAcceptance.TRUE;
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    OutputFileParameter outputFileParameter =
      OutputFileParameterFactory.create(new Pair(NAME_PARAMETER_NAME,
        parameterName), new Pair(LABEL_PARAMETER_NAME, parameterLabel),
        new Pair(DESCRIPTION_PARAMETER_NAME, parameterDescription), new Pair(
          CAN_USE_PIPE_PARAMETER_NAME, parameterCanUsePipe));
    group.addParameter(outputFileParameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_saida nome='" + parameterName + "' rotulo='"
        + parameterLabel + "' dica='" + parameterDescription
        + "' permitir_pipe='" + parameterCanUsePipe + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  /**
   * Testando a criao de {@link OutputFileParameter parmetros do tipo arquivo
   * de sada}: com permitir pipe igual a no.
   * 
   * @throws ParseException Em caso de falha no teste.
   */
  public void testOutputFileParameter13() throws ParseException {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    FileParameterPipeAcceptance parameterCanUsePipe =
      FileParameterPipeAcceptance.FALSE;
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    OutputFileParameter outputFileParameter =
      OutputFileParameterFactory.create(new Pair(NAME_PARAMETER_NAME,
        parameterName), new Pair(LABEL_PARAMETER_NAME, parameterLabel),
        new Pair(DESCRIPTION_PARAMETER_NAME, parameterDescription), new Pair(
          CAN_USE_PIPE_PARAMETER_NAME, parameterCanUsePipe));
    group.addParameter(outputFileParameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_saida nome='" + parameterName + "' rotulo='"
        + parameterLabel + "' dica='" + parameterDescription
        + "' permitir_pipe='" + parameterCanUsePipe + "'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  /**
   * Testando a criao de {@link OutputFileParameter parmetros do tipo arquivo
   * de sada}: com um atributo desconhecido.
   * 
   * @throws ParseException Em caso de falha no teste.
   */
  public void testOutputFileParameter14() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_saida nome='" + parameterName + "' rotulo='"
        + parameterLabel + "' dica='" + parameterDescription + "' "
        + "desconhecido='valor'/>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Testando a criao de {@link OutputFileParameter parmetros do tipo arquivo
   * de sada}: com formato do comndo personalizado e diferente de vazio.
   * 
   * @throws ParseException Em caso de falha no teste.
   */
  public void testOutputFileParameter18() throws ParseException {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String parameterCommandLinePattern =
      "Nome: $NOME_DO_PARAMETRO Valor: $VALOR_DO_PARAMETRO";
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    OutputFileParameter outputFileParameter =
      OutputFileParameterFactory.create(new Pair(NAME_PARAMETER_NAME,
        parameterName), new Pair(LABEL_PARAMETER_NAME, parameterLabel),
        new Pair(DESCRIPTION_PARAMETER_NAME, parameterDescription), new Pair(
          COMMAND_LINE_PATTERN_PARAMETER_NAME, parameterCommandLinePattern));
    group.addParameter(outputFileParameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_saida nome='" + parameterName + "' rotulo='"
        + parameterLabel + "' dica='" + parameterDescription + "'>\n";
    xml +=
      "<formato_no_comando>" + parameterCommandLinePattern
        + "</formato_no_comando>";
    xml += "</arquivo_de_saida>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  /**
   * Testando a criao de {@link OutputFileParameter parmetros do tipo arquivo
   * de sada}: com formato do comndo vazio.
   * 
   * @throws ParseException Em caso de falha no teste.
   */
  public void testOutputFileParameter19() throws ParseException {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    OutputFileParameter outputFileParameter =
      OutputFileParameterFactory.create(new Pair(NAME_PARAMETER_NAME,
        parameterName), new Pair(LABEL_PARAMETER_NAME, parameterLabel),
        new Pair(DESCRIPTION_PARAMETER_NAME, parameterDescription), new Pair(
          COMMAND_LINE_PATTERN_PARAMETER_NAME, null));
    group.addParameter(outputFileParameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_saida nome='" + parameterName + "' rotulo='"
        + parameterLabel + "' dica='" + parameterDescription + "'>\n";
    xml += "<formato_no_comando></formato_no_comando>";
    xml += "</arquivo_de_saida>\n";
    xml += "</grupo>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  public void testExpression1() throws ParseException,
    FormulaCreationException, ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "P1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean parameterDefaultValue1 = false;
    String trueCommand1 = "1";
    String falseCommand1 = "0";
    BooleanParameter parameter1 =
      new BooleanParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1,
        DEFAULT_COMMAND_LINE_PATTERN, falseCommand1, trueCommand1);
    group.addParameter(parameter1);
    String parameterName2 = "P2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String trueCommand2 = "1";
    String falseCommand2 = "0";
    boolean parameterDefaultValue2 = false;
    BooleanParameter parameter2 =
      new BooleanParameter(parameterName2, parameterLabel2,
        parameterDescription2, parameterDefaultValue2,
        DEFAULT_COMMAND_LINE_PATTERN, falseCommand2, trueCommand2);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String errorMessage1 = "Mensagem de erro 1";
    String expressionText1 = "P1 && P2";
    String errorMessage2 = "Mensagem de erro 2";
    String expressionText2 = "P1 || P2";
    ValidationExpression expression1 =
      new ValidationExpression(expectedVersion, expressionText1, errorMessage1);
    ValidationExpression expression2 =
      new ValidationExpression(expectedVersion, expressionText2, errorMessage2);
    expectedVersion.addExpression(expression1);
    expectedVersion.addExpression(expression2);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<booleano nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<booleano nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<expressao erro='" + errorMessage1 + "'>";
    xml += "<![CDATA[" + expressionText1 + "]]>";
    xml += "</expressao>";
    xml += "<expressao erro='" + errorMessage2 + "'>";
    xml += "<![CDATA[" + expressionText2 + "]]>";
    xml += "</expressao>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testExpression2() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "P1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "P2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String errorMessage1 = "Mensagem de erro 1";
    String expressionText1 = "P1 && P2";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<booleano nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<booleano nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<expressao erro='" + errorMessage1 + "'>";
    xml += "<![CDATA[" + expressionText1 + "]]>";
    xml += "</expressao>";
    xml += "<expressao erro='" + errorMessage1 + "'>";
    xml += "<![CDATA[" + expressionText1 + "]]>";
    xml += "</expressao>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testExpression3() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "P1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "P2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String expressionText1 = "P1 && P2";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<booleano nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<booleano nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<expressao>";
    xml += "<![CDATA[" + expressionText1 + "]]>";
    xml += "</expressao>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testExpression4() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "P1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "P2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String errorMessage1 = "Mensagem de erro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<booleano nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<booleano nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<expressao erro='" + errorMessage1 + "'>";
    xml += "</expressao>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Tentando criar expresses com atributos desconhecidos.
   */
  public void testExpression5() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "P1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "P2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String errorMessage1 = "Mensagem de erro 1";
    String expressionText1 = "P1 && P2";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<booleano nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<booleano nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<expressao erro='" + errorMessage1 + "' desconhecido='valor'>";
    xml += "<![CDATA[" + expressionText1 + "]]>";
    xml += "</expressao>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Tentando criar expresses com elementos-filhos.
   */
  public void testExpression6() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "P1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "P2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String errorMessage1 = "Mensagem de erro 1";
    String expressionText1 = "P1 && P2";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<booleano nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<booleano nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<expressao erro='" + errorMessage1 + "'>";
    xml += "<![CDATA[" + expressionText1 + "]]>";
    xml += "<filho/>";
    xml += "</expressao>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testEnableParameterTrigger1() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    TextParameter parameter1 =
      new TextParameter(parameterName1, parameterLabel1, parameterDescription1,
        parameterDefaultValue1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    EnableParameterTrigger trigger =
      new EnableParameterTrigger(parameter1, condition);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<habilitar parametro='" + parameterName1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testEnableParameterTrigger2() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    TextParameter parameter1 =
      new TextParameter(parameterName1, parameterLabel1, parameterDescription1,
        parameterDefaultValue1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    Condition condition = new SimpleCondition(parameter2.getName(), null);
    EnableParameterTrigger trigger =
      new EnableParameterTrigger(parameter1, condition);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<habilitar parametro='" + parameterName1 + "'>";
    xml += "<condicao parametro='" + parameterName2 + "'/>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testEnableParameterTrigger3() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean parameterDefaultValue1 = true;
    String trueCommand1 = "1";
    String falseCommand1 = "0";
    BooleanParameter parameter1 =
      new BooleanParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1,
        DEFAULT_COMMAND_LINE_PATTERN, falseCommand1, trueCommand1);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    boolean parameterDefaultValue2 = true;
    String trueCommand2 = "1";
    String falseCommand2 = "0";
    BooleanParameter parameter2 =
      new BooleanParameter(parameterName2, parameterLabel2,
        parameterDescription2, parameterDefaultValue2,
        DEFAULT_COMMAND_LINE_PATTERN, falseCommand2, trueCommand2);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    Boolean value = Boolean.TRUE;
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    EnableParameterTrigger trigger =
      new EnableParameterTrigger(parameter1, condition);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<booleano nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='"
        + parameterDefaultValue1 + "'/>";
    xml +=
      "<booleano nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "' padrao='"
        + parameterDefaultValue2 + "'/>";
    xml += "</grupo>";
    xml += "<habilitar parametro='" + parameterName1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testEnableParameterTrigger4() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean parameterDefaultValue1 = true;
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    boolean parameterDefaultValue2 = true;
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<booleano nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='"
        + parameterDefaultValue1 + "'/>";
    xml +=
      "<booleano nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "' padrao='"
        + parameterDefaultValue2 + "'/>";
    xml += "</grupo>";
    xml += "<habilitar parametro='" + parameterName1 + "'>";
    xml += "<condicao parametro='" + parameterName2 + "'/>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testEnableParameterTrigger5() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean parameterDefaultValue1 = true;
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    boolean parameterDefaultValue2 = true;
    String xml = "";
    String conditionValue = "Valor de tipo errado.";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<booleano nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='"
        + parameterDefaultValue1 + "'/>";
    xml +=
      "<booleano nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "' padrao='"
        + parameterDefaultValue2 + "'/>";
    xml += "</grupo>";
    xml += "<habilitar parametro='" + parameterName1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + conditionValue
        + "'/>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testEnableParameterTrigger6() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Integer parameterDefaultValue1 = new Integer(1000);
    boolean isOptional1 = false;
    Integer maximum1 = null;
    Integer minimum1 = null;
    IntegerParameter parameter1 =
      new IntegerParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        DEFAULT_COMMAND_LINE_PATTERN, maximum1, minimum1);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    Integer parameterDefaultValue2 = new Integer(1000);
    boolean isOptional2 = false;
    Integer maximum2 = null;
    Integer minimum2 = null;
    IntegerParameter parameter2 =
      new IntegerParameter(parameterName2, parameterLabel2,
        parameterDescription2, parameterDefaultValue2, isOptional2,
        DEFAULT_COMMAND_LINE_PATTERN, maximum2, minimum2);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    Integer value = new Integer(10);
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    EnableParameterTrigger trigger =
      new EnableParameterTrigger(parameter1, condition);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<inteiro nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='"
        + parameterDefaultValue1 + "'/>";
    xml +=
      "<inteiro nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "' padrao='"
        + parameterDefaultValue2 + "'/>";
    xml += "</grupo>";
    xml += "<habilitar parametro='" + parameterName1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testEnableParameterTrigger7() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Integer parameterDefaultValue1 = new Integer(1000);
    boolean isOptional1 = false;
    Integer maximum1 = null;
    Integer minimum1 = null;
    IntegerParameter parameter1 =
      new IntegerParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        DEFAULT_COMMAND_LINE_PATTERN, maximum1, minimum1);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    Integer parameterDefaultValue2 = new Integer(1000);
    boolean isOptional2 = false;
    Integer maximum2 = null;
    Integer minimum2 = null;
    IntegerParameter parameter2 =
      new IntegerParameter(parameterName2, parameterLabel2,
        parameterDescription2, parameterDefaultValue2, isOptional2,
        DEFAULT_COMMAND_LINE_PATTERN, maximum2, minimum2);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    Condition condition = new SimpleCondition(parameter2.getName(), null);
    EnableParameterTrigger trigger =
      new EnableParameterTrigger(parameter1, condition);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<inteiro nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='"
        + parameterDefaultValue1 + "'/>";
    xml +=
      "<inteiro nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "' padrao='"
        + parameterDefaultValue2 + "'/>";
    xml += "</grupo>";
    xml += "<habilitar parametro='" + parameterName1 + "'>";
    xml += "<condicao parametro='" + parameterName2 + "'/>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testEnableParameterTrigger8() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    int parameterDefaultValue1 = 11;
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    int parameterDefaultValue2 = 222;
    String xml = "";
    String conditionValue = "Valor de tipo errado.";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<inteiro nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='"
        + parameterDefaultValue1 + "'/>";
    xml +=
      "<inteiro nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "' padrao='"
        + parameterDefaultValue2 + "'/>";
    xml += "</grupo>";
    xml += "<habilitar parametro='" + parameterName1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + conditionValue
        + "'/>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testEnableParameterTrigger9() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<habilitar parametro='" + parameterName1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testEnableParameterTrigger10() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Double parameterDefaultValue1 = new Double(1.5);
    boolean isOptional1 = false;
    Double maximum1 = null;
    boolean includeMaximum1 = true;
    Double minimum1 = null;
    boolean includeMinimum1 = true;
    DoubleParameter parameter1 =
      new DoubleParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        DEFAULT_COMMAND_LINE_PATTERN, maximum1, includeMaximum1, minimum1,
        includeMinimum1);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    Double parameterDefaultValue2 = new Double(0.5);
    boolean isOptional2 = false;
    Double maximum2 = null;
    boolean includeMaximum2 = true;
    Double minimum2 = null;
    boolean includeMinimum2 = true;
    DoubleParameter parameter2 =
      new DoubleParameter(parameterName2, parameterLabel2,
        parameterDescription2, parameterDefaultValue2, isOptional2,
        DEFAULT_COMMAND_LINE_PATTERN, maximum2, includeMaximum2, minimum2,
        includeMinimum2);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    Double value = new Double(1.25);
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    EnableParameterTrigger trigger =
      new EnableParameterTrigger(parameter1, condition);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='"
        + parameterDefaultValue1 + "'/>";
    xml +=
      "<real nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "' padrao='"
        + parameterDefaultValue2 + "'/>";
    xml += "</grupo>";
    xml += "<habilitar parametro='" + parameterName1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testEnableParameterTrigger11() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Double parameterDefaultValue1 = new Double(1000);
    boolean isOptional1 = false;
    Double maximum1 = null;
    boolean includeMaximum1 = true;
    Double minimum1 = null;
    boolean includeMinimum1 = true;
    DoubleParameter parameter1 =
      new DoubleParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        DEFAULT_COMMAND_LINE_PATTERN, maximum1, includeMaximum1, minimum1,
        includeMinimum1);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    Double parameterDefaultValue2 = new Double(1000);
    boolean isOptional2 = false;
    Double maximum2 = null;
    boolean includeMaximum2 = true;
    Double minimum2 = null;
    boolean includeMinimum2 = true;
    DoubleParameter parameter2 =
      new DoubleParameter(parameterName2, parameterLabel2,
        parameterDescription2, parameterDefaultValue2, isOptional2,
        DEFAULT_COMMAND_LINE_PATTERN, maximum2, includeMaximum2, minimum2,
        includeMinimum2);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    Condition condition = new SimpleCondition(parameter2.getName(), null);
    EnableParameterTrigger trigger =
      new EnableParameterTrigger(parameter1, condition);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='"
        + parameterDefaultValue1 + "'/>";
    xml +=
      "<real nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "' padrao='"
        + parameterDefaultValue2 + "'/>";
    xml += "</grupo>";
    xml += "<habilitar parametro='" + parameterName1 + "'>";
    xml += "<condicao parametro='" + parameterName2 + "'/>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testEnableParameterTrigger12() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    double parameterDefaultValue1 = 1.1;
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    double parameterDefaultValue2 = 22.22;
    String xml = "";
    String conditionValue = "Valor de tipo errado.";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='"
        + parameterDefaultValue1 + "'/>";
    xml +=
      "<real nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "' padrao='"
        + parameterDefaultValue2 + "'/>";
    xml += "</grupo>";
    xml += "<habilitar parametro='" + parameterName1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + conditionValue
        + "'/>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testEnableParameterTrigger13() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterDefaultItemId1 = "ID 11";
    String parameterDefaultItemName1 = "Nome do item padro 1.1";
    String parameterDefaultItemValue1 = "V1.1";
    EnumerationItem parameterDefaultValue1 =
      new EnumerationItem(parameterDefaultItemId1, parameterDefaultItemName1,
        parameterDefaultItemValue1, null);
    String parameterItemId1 = "ID 12";
    String parameterItemName1 = "Nome do item padro 1.2";
    String parameterItemValue1 = "V1.2";
    EnumerationItem parameterValue1 =
      new EnumerationItem(parameterItemId1, parameterItemName1,
        parameterItemValue1, null);
    List<EnumerationItem> items1 = new LinkedList<EnumerationItem>();
    items1.add(parameterDefaultValue1);
    items1.add(parameterValue1);
    boolean mustSortItems = true;
    EnumerationParameter parameter1 =
      new EnumerationParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1,
        DEFAULT_COMMAND_LINE_PATTERN, items1, mustSortItems);
    group.addParameter(parameter1);
    expectedVersion.addGroup(group);
    Condition condition =
      new SimpleCondition(parameter1.getName(), parameterDefaultValue1);
    EnableParameterTrigger trigger =
      new EnableParameterTrigger(parameter1, condition);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<enumeracao nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='"
        + parameterDefaultItemId1 + "'>";
    xml +=
      "<item_de_enumeracao id='" + parameterDefaultItemId1 + "' rotulo='"
        + parameterDefaultItemName1 + "' valor='" + parameterDefaultItemValue1
        + "'/>";
    xml +=
      "<item_de_enumeracao id='" + parameterItemId1 + "' rotulo='"
        + parameterItemName1 + "' valor='" + parameterItemValue1 + "'/>";
    xml += "</enumeracao>";
    xml += "</grupo>";
    xml += "<habilitar parametro='" + parameterName1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName1 + "' valor='"
        + parameterDefaultItemId1 + "'/>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testEnableParameterTrigger14() {
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='comando'>\n";
    xml += "<grupo rotulo='grupo'>\n";
    xml += "<enumeracao nome='nome_da_enumeracao'\n";
    xml += "            rotulo='rotulo da enumeracao'\n";
    xml += "            dica='dica da enumeracao'\n";
    xml += "            padrao='1'>\n";
    xml += "<item_de_enumeracao id='1' rotulo='item 1' valor='valor 1'/>\n";
    xml += "</enumeracao>\n";
    xml += "</grupo>\n";
    xml += "<habilitar parametro='nome_da_enumeracao'>\n";
    xml += "<condicao parametro='nome_da_enumeracao'/>\n";
    xml += "</habilitar>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testEnableParameterTrigger15() {
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='comando'>\n";
    xml += "<grupo rotulo='grupo'>\n";
    xml += "<enumeracao nome='nome_da_enumeracao'\n";
    xml += "            rotulo='rotulo da enumeracao'\n";
    xml += "            dica='dica da enumeracao'\n";
    xml += "            padrao='1'>\n";
    xml += "<item_de_enumeracao id='1' rotulo='item 1' valor='valor 1'/>\n";
    xml += "</enumeracao>\n";
    xml += "</grupo>\n";
    xml += "<habilitar parametro='nome_da_enumeracao'>\n";
    xml += "<condicao parametro='nome_da_enumeracao' valor='2'/>\n";
    xml += "</habilitar>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testEnableParameterTrigger16() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<habilitar>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testEnableParameterTrigger17() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    String notExistantParameterName = "Nome do parmetro que no existe";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<habilitar parametro='" + notExistantParameterName + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testEnableParameterTrigger18() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    String notExistantParameterName = "Nome do parmetro que no existe";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<habilitar parametro='" + parameterName2 + "'>";
    xml +=
      "<condicao parametro='" + notExistantParameterName + "' valor='" + value
        + "'/>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testEnableParameterTrigger19() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<habilitar parametro='" + parameterName2 + "'>";
    xml += "<condicao valor='" + value + "'/>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testEnableParameterTrigger20() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<habilitar parametro='" + parameterName2 + "'>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testEnableParameterTrigger21() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml += "</grupo>";
    xml += "<habilitar>";
    xml += "<condicao parametro='" + parameterName1 + "'/>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Tentando criar gatilhos habilitar parmetros com atributos desconhecidos.
   */
  public void testEnableParameterTrigger22() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml +=
      "<habilitar parametro='" + parameterName1 + "' "
        + "desconhecido='valor'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Tentando criar 2 gatilhos habilitar parmetros para o mesmo parmetro.
   */
  public void testEnableParameterTrigger23() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<habilitar parametro='" + parameterName1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</habilitar>";
    xml += "<habilitar parametro='" + parameterName1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName1 + "' valor='" + value + "'/>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testEnableParameterTrigger24() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    TextParameter parameter1 =
      new TextParameter(parameterName1, parameterLabel1, parameterDescription1,
        parameterDefaultValue1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    EnableParameterTrigger trigger1 =
      new EnableParameterTrigger(parameter1, condition);
    expectedVersion.addTrigger(trigger1);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<habilitar>";
    xml += "<parametro nome='" + parameterName1 + "'/>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testEnableParameterTrigger25() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    TextParameter parameter1 =
      new TextParameter(parameterName1, parameterLabel1, parameterDescription1,
        parameterDefaultValue1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    String parameterName3 = "Nome do parmetro 3";
    String parameterLabel3 = "Rtulo do parmetro 3";
    String parameterDescription3 = "Descrio do parmetro 3";
    String parameterDefaultValue3 = null;
    boolean isOptional3 = false;
    TextParameter parameter3 =
      new TextParameter(parameterName3, parameterLabel3, parameterDescription3,
        parameterDefaultValue3, isOptional3, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter3);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    EnableParameterTrigger trigger1 =
      new EnableParameterTrigger(parameter1, condition);
    expectedVersion.addTrigger(trigger1);
    EnableParameterTrigger trigger2 =
      new EnableParameterTrigger(parameter3, condition);
    expectedVersion.addTrigger(trigger2);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml +=
      "<texto nome='" + parameterName3 + "' rotulo='" + parameterLabel3
        + "' dica='" + parameterDescription3 + "'/>";
    xml += "</grupo>";
    xml += "<habilitar>";
    xml += "<parametro nome='" + parameterName1 + "'/>";
    xml += "<parametro nome='" + parameterName3 + "'/>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testEnableParameterTrigger26() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<habilitar>";
    xml += "<parametro nome='" + parameterName1 + "'/>";
    xml += "<condicao parametro='Nome do parmetro 3' valor='" + value + "'/>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testOrCondition1() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    TextParameter parameter1 =
      new TextParameter(parameterName1, parameterLabel1, parameterDescription1,
        parameterDefaultValue1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition1 = new SimpleCondition(parameter1.getName(), null);
    Condition condition2 = new SimpleCondition(parameter2.getName(), value);
    Condition condition3 = new SimpleCondition(parameter2.getName(), null);
    OrOperatorCondition condition =
      new OrOperatorCondition(condition1, condition2, condition3);
    EnableParameterTrigger trigger =
      new EnableParameterTrigger(parameter1, condition);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<habilitar parametro='" + parameterName1 + "'>";
    xml += "<ou>";
    xml += "<condicao parametro='" + parameterName1 + "'/>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "<condicao parametro='" + parameterName2 + "'/>";
    xml += "</ou>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testOrCondition2() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml += "</grupo>";
    xml += "<habilitar parametro='" + parameterName1 + "'>";
    xml += "<ou>";
    xml += "<condicao parametro='" + parameterName1 + "'/>";
    xml += "</ou>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testOrCondition3() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml += "</grupo>";
    xml += "<habilitar parametro='" + parameterName1 + "'>";
    xml += "<ou/>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Tentando criar operadores "ou" com atributos desconhecidos.
   */
  public void testOrCondition4() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<habilitar parametro='" + parameterName1 + "'>";
    xml += "<ou desconhecido='valor'>";
    xml += "<condicao parametro='" + parameterName1 + "'/>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</ou>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testAndCondition1() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    TextParameter parameter1 =
      new TextParameter(parameterName1, parameterLabel1, parameterDescription1,
        parameterDefaultValue1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition1 = new SimpleCondition(parameter1.getName(), null);
    Condition condition2 = new SimpleCondition(parameter2.getName(), value);
    Condition condition3 = new SimpleCondition(parameter2.getName(), null);
    AndOperatorCondition condition =
      new AndOperatorCondition(condition1, condition2, condition3);
    EnableParameterTrigger trigger =
      new EnableParameterTrigger(parameter1, condition);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<habilitar parametro='" + parameterName1 + "'>";
    xml += "<e>";
    xml += "<condicao parametro='" + parameterName1 + "'/>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "<condicao parametro='" + parameterName2 + "'/>";
    xml += "</e>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testAndCondition2() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml += "</grupo>";
    xml += "<habilitar parametro='" + parameterName1 + "'>";
    xml += "<e/>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testAndCondition3() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml += "</grupo>";
    xml += "<habilitar parametro='" + parameterName1 + "'>";
    xml += "<e>";
    xml += "<condicao parametro='" + parameterName1 + "'/>";
    xml += "</e>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Tentando criar operadores "e" com atributos desconhecidos.
   */
  public void testAndCondition4() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<habilitar parametro='" + parameterName1 + "'>";
    xml += "<e desconhecido='valor'>";
    xml += "<condicao parametro='" + parameterName1 + "'/>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</e>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testNotCondition1() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    TextParameter parameter1 =
      new TextParameter(parameterName1, parameterLabel1, parameterDescription1,
        parameterDefaultValue1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    NotOperatorCondition notCondition = new NotOperatorCondition(condition);
    EnableParameterTrigger trigger =
      new EnableParameterTrigger(parameter1, notCondition);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<habilitar parametro='" + parameterName1 + "'>";
    xml += "<nao>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</nao>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testNotCondition2() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml += "</grupo>";
    xml += "<habilitar parametro='" + parameterName1 + "'>";
    xml += "<nao/>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Tentando criar operadores "no" com atributos desconhecidos.
   */
  public void testNotCondition3() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<habilitar parametro='" + parameterName1 + "'>";
    xml += "<nao desconhecido='valor'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</nao>";
    xml += "</habilitar>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Tentando criar uma expresso genrica.
   */
  public void testGenericCondition1() throws ParseException,
    FormulaCreationException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Integer parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    IntegerParameter parameter1 =
      new IntegerParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        DEFAULT_COMMAND_LINE_PATTERN, null, null);
    group.addParameter(parameter1);
    String parameterName2 = "Nome_do_parametro_2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    Integer parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    IntegerParameter parameter2 =
      new IntegerParameter(parameterName2, parameterLabel2,
        parameterDescription2, parameterDefaultValue2, isOptional2,
        DEFAULT_COMMAND_LINE_PATTERN, null, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    int value = 3;
    String expressionText2 = parameterName2 + " == " + value;
    Condition condition = new GenericCondition(expressionText2);
    EnableParameterTrigger trigger =
      new EnableParameterTrigger(parameter1, condition);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<inteiro nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>\n";
    xml +=
      "<inteiro nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>\n";
    xml += "</grupo>\n";
    xml += "<habilitar parametro='" + parameterName1 + "'>\n";
    xml += "<condicao>\n";
    xml += "<![CDATA[" + expressionText2 + "]]>\n";
    xml += "</condicao>\n";
    xml += "</habilitar>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testShowParameterTrigger1() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    TextParameter parameter1 =
      new TextParameter(parameterName1, parameterLabel1, parameterDescription1,
        parameterDefaultValue1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    ShowParameterTrigger trigger =
      new ShowParameterTrigger(parameter1, condition);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<exibir parametro='" + parameterName1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</exibir>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testShowParameterTrigger2() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<exibir parametro='" + parameterName1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</exibir>";
    xml += "<exibir parametro='" + parameterName1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</exibir>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testShowParameterTrigger3() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml += "</grupo>";
    xml += "<exibir>";
    xml += "<condicao parametro='" + parameterName1 + "'/>";
    xml += "</exibir>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testShowParameterTrigger4() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml += "</grupo>";
    xml += "<exibir parametro='" + parameterName1 + "'/>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testShowParameterTrigger5() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<exibir parametro='" + parameterName1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</exibir>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Tentando criar gatilhos exibir parmetros com atributos desconhecidos.
   */
  public void testShowParameterTrigger6() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<exibir parametro='" + parameterName1 + "' desconhecido='valor'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</exibir>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testShowParameterTrigger7() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    TextParameter parameter1 =
      new TextParameter(parameterName1, parameterLabel1, parameterDescription1,
        parameterDefaultValue1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    ShowParameterTrigger trigger1 =
      new ShowParameterTrigger(parameter1, condition);
    expectedVersion.addTrigger(trigger1);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<exibir>";
    xml += "<parametro nome='" + parameterName1 + "'/>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</exibir>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testShowParameterTrigger8() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    TextParameter parameter1 =
      new TextParameter(parameterName1, parameterLabel1, parameterDescription1,
        parameterDefaultValue1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    String parameterName3 = "Nome do parmetro 3";
    String parameterLabel3 = "Rtulo do parmetro 3";
    String parameterDescription3 = "Descrio do parmetro 3";
    String parameterDefaultValue3 = null;
    boolean isOptional3 = false;
    TextParameter parameter3 =
      new TextParameter(parameterName3, parameterLabel3, parameterDescription3,
        parameterDefaultValue3, isOptional3, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter3);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    ShowParameterTrigger trigger1 =
      new ShowParameterTrigger(parameter1, condition);
    expectedVersion.addTrigger(trigger1);
    ShowParameterTrigger trigger2 =
      new ShowParameterTrigger(parameter3, condition);
    expectedVersion.addTrigger(trigger2);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml +=
      "<texto nome='" + parameterName3 + "' rotulo='" + parameterLabel3
        + "' dica='" + parameterDescription3 + "'/>";
    xml += "</grupo>";
    xml += "<exibir>";
    xml += "<parametro nome='" + parameterName1 + "'/>";
    xml += "<parametro nome='" + parameterName3 + "'/>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</exibir>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testShowParameterTrigger9() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<exibir>";
    xml += "<parametro nome='" + parameterName1 + "'/>";
    xml += "<condicao parametro='Nome do parmetro 3' valor='" + value + "'/>";
    xml += "</exibir>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testDisableParameterTrigger1() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    TextParameter parameter1 =
      new TextParameter(parameterName1, parameterLabel1, parameterDescription1,
        parameterDefaultValue1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    DisableParameterTrigger trigger =
      new DisableParameterTrigger(parameter1, condition);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<desabilitar parametro='" + parameterName1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</desabilitar>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testDisableParameterTrigger2() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<desabilitar parametro='" + parameterName1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</desabilitar>";
    xml += "<desabilitar parametro='" + parameterName1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</desabilitar>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testDisableParameterTrigger3() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml += "</grupo>";
    xml += "<desabilitar>";
    xml += "<condicao parametro='" + parameterName1 + "'/>";
    xml += "</desabilitar>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testDisableParameterTrigger4() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml += "</grupo>";
    xml += "<desabilitar parametro='" + parameterName1 + "'/>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testDisableParameterTrigger5() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<desabilitar parametro='" + parameterName1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</desabilitar>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Tentando criar gatilhos desabilitar parmetros com atributos desconhecidos.
   */
  public void testDisableParameterTrigger6() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml +=
      "<desabilitar parametro='" + parameterName1 + "' "
        + "desconhecido='valor'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</desabilitar>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testDisableParameterTrigger7() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    TextParameter parameter1 =
      new TextParameter(parameterName1, parameterLabel1, parameterDescription1,
        parameterDefaultValue1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    DisableParameterTrigger trigger1 =
      new DisableParameterTrigger(parameter1, condition);
    expectedVersion.addTrigger(trigger1);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<desabilitar>";
    xml += "<parametro nome='" + parameterName1 + "'/>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</desabilitar>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testDisableParameterTrigger8() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    TextParameter parameter1 =
      new TextParameter(parameterName1, parameterLabel1, parameterDescription1,
        parameterDefaultValue1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    String parameterName3 = "Nome do parmetro 3";
    String parameterLabel3 = "Rtulo do parmetro 3";
    String parameterDescription3 = "Descrio do parmetro 3";
    String parameterDefaultValue3 = null;
    boolean isOptional3 = false;
    TextParameter parameter3 =
      new TextParameter(parameterName3, parameterLabel3, parameterDescription3,
        parameterDefaultValue3, isOptional3, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter3);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    DisableParameterTrigger trigger1 =
      new DisableParameterTrigger(parameter1, condition);
    expectedVersion.addTrigger(trigger1);
    DisableParameterTrigger trigger2 =
      new DisableParameterTrigger(parameter3, condition);
    expectedVersion.addTrigger(trigger2);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml +=
      "<texto nome='" + parameterName3 + "' rotulo='" + parameterLabel3
        + "' dica='" + parameterDescription3 + "'/>";
    xml += "</grupo>";
    xml += "<desabilitar>";
    xml += "<parametro nome='" + parameterName1 + "'/>";
    xml += "<parametro nome='" + parameterName3 + "'/>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</desabilitar>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testDisableParameterTrigger9() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<desabilitar>";
    xml += "<parametro nome='" + parameterName1 + "'/>";
    xml += "<condicao parametro='Nome do parmetro 3' valor='" + value + "'/>";
    xml += "</desabilitar>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testHideParameterTrigger1() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    TextParameter parameter1 =
      new TextParameter(parameterName1, parameterLabel1, parameterDescription1,
        parameterDefaultValue1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    HideParameterTrigger trigger =
      new HideParameterTrigger(parameter1, condition);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<ocultar parametro='" + parameterName1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</ocultar>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testHideParameterTrigger2() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<ocultar parametro='" + parameterName1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</ocultar>";
    xml += "<ocultar parametro='" + parameterName1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</ocultar>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testHideParameterTrigger3() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml += "</grupo>";
    xml += "<ocultar>";
    xml += "<condicao parametro='" + parameterName1 + "'/>";
    xml += "</ocultar>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testHideParameterTrigger4() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml += "</grupo>";
    xml += "<ocultar parametro='" + parameterName1 + "'/>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testHideParameterTrigger5() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<ocultar parametro='" + parameterName1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</ocultar>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Tentando criar gatilhos ocultar parmetros com atributos desconhecidos.
   */
  public void testHideParameterTrigger6() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<ocultar parametro='" + parameterName1 + "' desconhecido='valor'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</ocultar>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testHideParameterTrigger7() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    TextParameter parameter1 =
      new TextParameter(parameterName1, parameterLabel1, parameterDescription1,
        parameterDefaultValue1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    HideParameterTrigger trigger1 =
      new HideParameterTrigger(parameter1, condition);
    expectedVersion.addTrigger(trigger1);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<ocultar>";
    xml += "<parametro nome='" + parameterName1 + "'/>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</ocultar>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testHideParameterTrigger8() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    TextParameter parameter1 =
      new TextParameter(parameterName1, parameterLabel1, parameterDescription1,
        parameterDefaultValue1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    String parameterName3 = "Nome do parmetro 3";
    String parameterLabel3 = "Rtulo do parmetro 3";
    String parameterDescription3 = "Descrio do parmetro 3";
    String parameterDefaultValue3 = null;
    boolean isOptional3 = false;
    TextParameter parameter3 =
      new TextParameter(parameterName3, parameterLabel3, parameterDescription3,
        parameterDefaultValue3, isOptional3, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter3);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    HideParameterTrigger trigger1 =
      new HideParameterTrigger(parameter1, condition);
    expectedVersion.addTrigger(trigger1);
    HideParameterTrigger trigger2 =
      new HideParameterTrigger(parameter3, condition);
    expectedVersion.addTrigger(trigger2);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml +=
      "<texto nome='" + parameterName3 + "' rotulo='" + parameterLabel3
        + "' dica='" + parameterDescription3 + "'/>";
    xml += "</grupo>";
    xml += "<ocultar>";
    xml += "<parametro nome='" + parameterName1 + "'/>";
    xml += "<parametro nome='" + parameterName3 + "'/>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</ocultar>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testHideParameterTrigger9() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<ocultar>";
    xml += "<parametro nome='" + parameterName1 + "'/>";
    xml += "<condicao parametro='Nome do parmetro 3' valor='" + value + "'/>";
    xml += "</ocultar>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testChangeDefaultValueTrigger1() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String itemId11 = "ID 11";
    String itemName11 = "Nome do item 1.1";
    String itemValue11 = "Valor do item 1.1";
    EnumerationItem item11 =
      new EnumerationItem(itemId11, itemName11, itemValue11, null);
    String itemId12 = "ID 12";
    String itemName12 = "Nome do item 1.2";
    String itemValue12 = "Valor do item 1.2";
    EnumerationItem item12 =
      new EnumerationItem(itemId12, itemName12, itemValue12, null);
    List<EnumerationItem> items1 = new LinkedList<EnumerationItem>();
    items1.add(item11);
    items1.add(item12);
    boolean mustSortItems = true;
    EnumerationParameter parameter1 =
      new EnumerationParameter(parameterName1, parameterLabel1,
        parameterDescription1, item11, DEFAULT_COMMAND_LINE_PATTERN, items1,
        mustSortItems);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    ChangeDefaultValueTrigger trigger =
      new ChangeDefaultValueTrigger(parameter1, condition, item12);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<enumeracao nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='" + itemId11 + "'>";
    xml +=
      "<item_de_enumeracao id='" + itemId11 + "' rotulo='" + itemName11
        + "' valor='" + itemValue11 + "'/>";
    xml +=
      "<item_de_enumeracao id='" + itemId12 + "' rotulo='" + itemName12
        + "' valor='" + itemValue12 + "'/>";
    xml += "</enumeracao>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml +=
      "<trocar_valor_padrao parametro='" + parameterName1
        + "' novo_valor_padrao='" + itemId12 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_valor_padrao>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testChangeDefaultValueTrigger2() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String value1 = "Valor 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml +=
      "<trocar_valor_padrao parametro='" + parameterName1
        + "' novo_valor_padrao='" + value1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_valor_padrao>";
    xml +=
      "<trocar_valor_padrao parametro='" + parameterName1
        + "' novo_valor_padrao='" + value1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_valor_padrao>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testChangeDefaultValueTrigger3() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String itemName11 = "Nome do item 1.1";
    String itemValue11 = "Valor do item 1.1";
    String itemName12 = "Nome do item 1.2";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<enumeracao nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='" + itemName11 + "'>";
    xml +=
      "<item_de_enumeracao nome='" + itemName11 + "' valor='" + itemValue11
        + "'/>";
    xml += "</enumeracao>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml +=
      "<trocar_valor_padrao parametro='" + parameterName1
        + "' novo_valor_padrao='" + itemName12 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_valor_padrao>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testChangeDefaultValueTrigger4() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String itemName11 = "Nome do item 1.1";
    String itemValue11 = "Valor do item 1.1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<enumeracao nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='" + itemName11 + "'>";
    xml +=
      "<item_de_enumeracao nome='" + itemName11 + "' valor='" + itemValue11
        + "'/>";
    xml += "</enumeracao>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<trocar_valor_padrao parametro='" + parameterName1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_valor_padrao>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testChangeDefaultValueTrigger5() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean defaultValue1 = false;
    BooleanParameter parameter1 =
      new BooleanParameter(parameterName1, parameterLabel1,
        parameterDescription1, defaultValue1, DEFAULT_COMMAND_LINE_PATTERN,
        null, null);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    Boolean newDefaultValue = Boolean.TRUE;
    ChangeDefaultValueTrigger trigger =
      new ChangeDefaultValueTrigger(parameter1, condition, newDefaultValue);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<booleano nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='" + defaultValue1
        + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml +=
      "<trocar_valor_padrao parametro='" + parameterName1
        + "' novo_valor_padrao='" + newDefaultValue + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_valor_padrao>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testChangeDefaultValueTrigger6() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean defaultValue1 = false;
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<booleano nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='" + defaultValue1
        + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<trocar_valor_padrao parametro='" + parameterName1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_valor_padrao>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testChangeDefaultValueTrigger7() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean defaultValue1 = false;
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    String newDefaultValue = "Valor no booleano";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<booleano nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='" + defaultValue1
        + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml +=
      "<trocar_valor_padrao parametro='" + parameterName1
        + "' novo_valor_padrao='" + newDefaultValue + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_valor_padrao>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testChangeDefaultValueTrigger8() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean isOptional1 = false;
    TextParameter parameter1 =
      new TextParameter(parameterName1, parameterLabel1, parameterDescription1,
        null, isOptional1, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    String newDefaultValue = "Novo valor-padro";
    ChangeDefaultValueTrigger trigger =
      new ChangeDefaultValueTrigger(parameter1, condition, newDefaultValue);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml +=
      "<trocar_valor_padrao parametro='" + parameterName1
        + "' novo_valor_padrao='" + newDefaultValue + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_valor_padrao>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testChangeDefaultValueTrigger9() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean isOptional1 = false;
    TextParameter parameter1 =
      new TextParameter(parameterName1, parameterLabel1, parameterDescription1,
        null, isOptional1, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    ChangeDefaultValueTrigger trigger =
      new ChangeDefaultValueTrigger(parameter1, condition, null);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<trocar_valor_padrao parametro='" + parameterName1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_valor_padrao>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testChangeDefaultValueTrigger10() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Integer parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    IntegerParameter parameter1 =
      new IntegerParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        DEFAULT_COMMAND_LINE_PATTERN, null, null);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    Integer newDefaultValue = new Integer(1000);
    ChangeDefaultValueTrigger trigger =
      new ChangeDefaultValueTrigger(parameter1, condition, newDefaultValue);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<inteiro nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml +=
      "<trocar_valor_padrao parametro='" + parameterName1
        + "' novo_valor_padrao='" + newDefaultValue + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_valor_padrao>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testChangeDefaultValueTrigger11() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Integer parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    IntegerParameter parameter1 =
      new IntegerParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        DEFAULT_COMMAND_LINE_PATTERN, null, null);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    ChangeDefaultValueTrigger trigger =
      new ChangeDefaultValueTrigger(parameter1, condition, null);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<inteiro nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<trocar_valor_padrao parametro='" + parameterName1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_valor_padrao>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testChangeDefaultValueTrigger12() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Double parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    DoubleParameter parameter1 =
      new DoubleParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        DEFAULT_COMMAND_LINE_PATTERN, null, false, null, false);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    Double newDefaultValue = new Double(1000);
    ChangeDefaultValueTrigger trigger =
      new ChangeDefaultValueTrigger(parameter1, condition, newDefaultValue);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml +=
      "<trocar_valor_padrao parametro='" + parameterName1
        + "' novo_valor_padrao='" + newDefaultValue + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_valor_padrao>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testChangeDefaultValueTrigger13() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Double parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    DoubleParameter parameter1 =
      new DoubleParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        DEFAULT_COMMAND_LINE_PATTERN, null, false, null, false);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    ChangeDefaultValueTrigger trigger =
      new ChangeDefaultValueTrigger(parameter1, condition, null);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<trocar_valor_padrao parametro='" + parameterName1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_valor_padrao>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testChangeDefaultValueTrigger14() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    String newDefaultValue = "Valor no-real";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml +=
      "<trocar_valor_padrao parametro='" + parameterName1
        + "' novo_valor_padrao='" + newDefaultValue + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_valor_padrao>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testChangeMaximumTrigger1() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Double parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    DoubleParameter parameter1 =
      new DoubleParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        DEFAULT_COMMAND_LINE_PATTERN, null, false, null, false);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    Double newMaximum = new Double(1000);
    boolean isNewMaximumIncluded = true;
    ChangeMaximumForDoublesTrigger trigger =
      new ChangeMaximumForDoublesTrigger(parameter1, condition, newMaximum,
        isNewMaximumIncluded);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml +=
      "<trocar_maximo parametro='" + parameterName1 + "' novo_maximo='"
        + newMaximum + "' incluir_novo_maximo='" + isNewMaximumIncluded + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_maximo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testChangeMaximumTrigger2() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    Double newMaximum = new Double(1000);
    boolean isNewMaximumIncluded = true;
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml +=
      "<trocar_maximo parametro='" + parameterName1 + "' novo_maximo='"
        + newMaximum + "' incluir_novo_maximo='" + isNewMaximumIncluded + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_maximo>";
    xml +=
      "<trocar_maximo parametro='" + parameterName1 + "' novo_maximo='"
        + newMaximum + "' incluir_novo_maximo='" + isNewMaximumIncluded + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_maximo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testChangeMaximumTrigger3() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Double parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    DoubleParameter parameter1 =
      new DoubleParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        DEFAULT_COMMAND_LINE_PATTERN, null, false, null, false);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    ChangeMaximumForDoublesTrigger trigger =
      new ChangeMaximumForDoublesTrigger(parameter1, condition, null, true);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<trocar_maximo parametro='" + parameterName1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_maximo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testChangeMaximumTrigger4() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    String newMaximum = "Valor no-real";
    boolean isNewMaximumIncluded = true;
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml +=
      "<trocar_maximo parametro='" + parameterName1 + "' novo_maximo='"
        + newMaximum + "' incluir_novo_maximo='" + isNewMaximumIncluded + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_maximo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testChangeMaximumTrigger5() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    Double newMaximum = new Double(1.2);
    String isNewMaximumIncluded = "valor no-booleano";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml +=
      "<trocar_maximo parametro='" + parameterName1 + "' novo_maximo='"
        + newMaximum + "' incluir_novo_maximo='" + isNewMaximumIncluded + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_maximo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Tentando criar gatilhos trocar mximo com atributos desconhecidos.
   */
  public void testChangeMaximumTrigger6() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    Double newMaximum = new Double(1000);
    boolean isNewMaximumIncluded = true;
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml +=
      "<trocar_maximo parametro='" + parameterName1 + "' novo_maximo='"
        + newMaximum + "' incluir_novo_maximo='" + isNewMaximumIncluded + "' "
        + "desconhecido='valor'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_maximo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testChangeMaximumTrigger7() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Integer parameterDefaultValue1 = null;
    Integer maximum1 = null;
    Integer minimum1 = null;
    boolean isOptional1 = false;
    IntegerParameter parameter1 =
      new IntegerParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        DEFAULT_COMMAND_LINE_PATTERN, maximum1, minimum1);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    Integer newMaximum = 1000;
    ChangeMaximumForIntegersTrigger trigger =
      new ChangeMaximumForIntegersTrigger(parameter1, condition, newMaximum);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<inteiro nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml +=
      "<trocar_maximo parametro='" + parameterName1 + "' novo_maximo='"
        + newMaximum + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_maximo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testChangeMaximumTrigger8() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Integer parameterDefaultValue1 = null;
    Integer maximum1 = null;
    Integer minimum1 = null;
    boolean isOptional1 = false;
    IntegerParameter parameter1 =
      new IntegerParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        DEFAULT_COMMAND_LINE_PATTERN, maximum1, minimum1);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    ChangeMaximumForIntegersTrigger trigger =
      new ChangeMaximumForIntegersTrigger(parameter1, condition, null);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<inteiro nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<trocar_maximo parametro='" + parameterName1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_maximo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testChangeMaximumTrigger9() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Integer maximum1 = null;
    Integer minimum1 = null;
    boolean isOptional1 = false;
    IntegerListParameter parameter1 =
      new IntegerListParameter(parameterName1, parameterLabel1,
        parameterDescription1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN,
        maximum1, minimum1);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    Integer newMaximum = 1000;
    ChangeMaximumForIntegersTrigger trigger =
      new ChangeMaximumForIntegersTrigger(parameter1, condition, newMaximum);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_inteiros nome='" + parameterName1 + "' " + "rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml +=
      "<trocar_maximo parametro='" + parameterName1 + "' novo_maximo='"
        + newMaximum + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_maximo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testChangeMaximumTrigger10() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Integer maximum1 = null;
    Integer minimum1 = null;
    boolean isOptional1 = false;
    IntegerListParameter parameter1 =
      new IntegerListParameter(parameterName1, parameterLabel1,
        parameterDescription1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN,
        maximum1, minimum1);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    ChangeMaximumForIntegersTrigger trigger =
      new ChangeMaximumForIntegersTrigger(parameter1, condition, null);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_inteiros nome='" + parameterName1 + "' " + "rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<trocar_maximo parametro='" + parameterName1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_maximo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testChangeMinimumTrigger1() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Double parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    DoubleParameter parameter1 =
      new DoubleParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        DEFAULT_COMMAND_LINE_PATTERN, null, false, null, false);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    Double newMinimum = new Double(1000);
    boolean isNewMinimumIncluded = true;
    ChangeMinimumForDoublesTrigger trigger =
      new ChangeMinimumForDoublesTrigger(parameter1, condition, newMinimum,
        isNewMinimumIncluded);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml +=
      "<trocar_minimo parametro='" + parameterName1 + "' novo_minimo='"
        + newMinimum + "' incluir_novo_minimo='" + isNewMinimumIncluded + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_minimo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testChangeMinimumTrigger2() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    Double newMinimum = new Double(1000);
    boolean isNewMinimumIncluded = true;
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml +=
      "<trocar_minimo parametro='" + parameterName1 + "' novo_minimo='"
        + newMinimum + "' incluir_novo_minimo='" + isNewMinimumIncluded + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_minimo>";
    xml +=
      "<trocar_minimo parametro='" + parameterName1 + "' novo_minimo='"
        + newMinimum + "' incluir_novo_minimo='" + isNewMinimumIncluded + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_minimo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testChangeMinimumTrigger3() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Double parameterDefaultValue1 = null;
    boolean isOptional1 = false;
    DoubleParameter parameter1 =
      new DoubleParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        DEFAULT_COMMAND_LINE_PATTERN, null, false, null, false);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    ChangeMinimumForDoublesTrigger trigger =
      new ChangeMinimumForDoublesTrigger(parameter1, condition, null, true);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<trocar_minimo parametro='" + parameterName1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_minimo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testChangeMinimumTrigger4() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    String newMinimum = "Valor no-real";
    boolean isNewMinimumIncluded = true;
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml +=
      "<trocar_minimo parametro='" + parameterName1 + "' novo_minimo='"
        + newMinimum + "' incluir_novo_minimo='" + isNewMinimumIncluded + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_minimo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testChangeMinimumTrigger5() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    Double newMinimum = new Double(1000);
    String isNewMinimumIncluded = "Valor no-booleano";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml +=
      "<trocar_minimo parametro='" + parameterName1 + "' novo_minimo='"
        + newMinimum + "' incluir_novo_minimo='" + isNewMinimumIncluded + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_minimo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Tentando criar gatilhos trocar mnimo com atributos desconhecidos.
   */
  public void testChangeMinimumTrigger6() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    Double newMinimum = new Double(1000);
    boolean isNewMinimumIncluded = true;
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<real nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml +=
      "<trocar_minimo parametro='" + parameterName1 + "' novo_minimo='"
        + newMinimum + "' incluir_novo_minimo='" + isNewMinimumIncluded
        + "' desconhecido='valor'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_minimo>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testChangeMinimumTrigger7() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Integer parameterDefaultValue1 = null;
    Integer maximum1 = null;
    Integer minimum1 = null;
    boolean isOptional1 = false;
    IntegerParameter parameter1 =
      new IntegerParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        DEFAULT_COMMAND_LINE_PATTERN, maximum1, minimum1);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    Integer newMinimum = 1000;
    ChangeMinimumForIntegersTrigger trigger =
      new ChangeMinimumForIntegersTrigger(parameter1, condition, newMinimum);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<inteiro nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml +=
      "<trocar_minimo parametro='" + parameterName1 + "' novo_minimo='"
        + newMinimum + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_minimo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testChangeMinimumTrigger8() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Integer parameterDefaultValue1 = null;
    Integer maximum1 = null;
    Integer minimum1 = null;
    boolean isOptional1 = false;
    IntegerParameter parameter1 =
      new IntegerParameter(parameterName1, parameterLabel1,
        parameterDescription1, parameterDefaultValue1, isOptional1,
        DEFAULT_COMMAND_LINE_PATTERN, maximum1, minimum1);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    ChangeMinimumForIntegersTrigger trigger =
      new ChangeMinimumForIntegersTrigger(parameter1, condition, null);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<inteiro nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<trocar_minimo parametro='" + parameterName1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_minimo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testChangeMinimumTrigger9() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Integer maximum1 = null;
    Integer minimum1 = null;
    boolean isOptional1 = false;
    IntegerListParameter parameter1 =
      new IntegerListParameter(parameterName1, parameterLabel1,
        parameterDescription1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN,
        maximum1, minimum1);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    Integer newMinimum = 1000;
    ChangeMinimumForIntegersTrigger trigger =
      new ChangeMinimumForIntegersTrigger(parameter1, condition, newMinimum);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_inteiros nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml +=
      "<trocar_minimo parametro='" + parameterName1 + "' novo_minimo='"
        + newMinimum + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_minimo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testChangeMinimumTrigger10() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    Integer maximum1 = null;
    Integer minimum1 = null;
    boolean isOptional1 = false;
    IntegerListParameter parameter1 =
      new IntegerListParameter(parameterName1, parameterLabel1,
        parameterDescription1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN,
        maximum1, minimum1);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    ChangeMinimumForIntegersTrigger trigger =
      new ChangeMinimumForIntegersTrigger(parameter1, condition, null);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>";
    xml +=
      "<lista_de_inteiros nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "'/>";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>";
    xml += "</grupo>";
    xml += "<trocar_minimo parametro='" + parameterName1 + "'>";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>";
    xml += "</trocar_minimo>";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testChangeFileTypeTrigger1() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean isOptional1 = false;
    OutputFileParameter parameter1 =
      new OutputFileParameter(parameterName1, parameterLabel1,
        parameterDescription1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN, null,
        FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE, false);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    String fileType = "Tipo";
    ChangeFileTypeTrigger trigger =
      new ChangeFileTypeTrigger(parameter1, condition, fileType);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_saida nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "'/>\n";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>\n";
    xml += "</grupo>\n";
    xml +=
      "<trocar_tipo_de_arquivo parametro='" + parameterName1 + "' novo_tipo='"
        + fileType + "'>\n";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>\n";
    xml += "</trocar_tipo_de_arquivo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testChangeFileTypeTrigger2() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    String fileType = "Tipo";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_saida nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "'/>\n";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>\n";
    xml += "</grupo>\n";
    xml +=
      "<trocar_tipo_de_arquivo parametro='" + parameterName1 + "' novo_tipo='"
        + fileType + "'>\n";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>\n";
    xml += "</trocar_tipo_de_arquivo>\n";
    xml +=
      "<trocar_tipo_de_arquivo parametro='" + parameterName1 + "' novo_tipo='"
        + fileType + "'>\n";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>\n";
    xml += "</trocar_tipo_de_arquivo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testChangeFileTypeTrigger3() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean isOptional1 = false;
    OutputFileParameter parameter1 =
      new OutputFileParameter(parameterName1, parameterLabel1,
        parameterDescription1, isOptional1, DEFAULT_COMMAND_LINE_PATTERN, null,
        FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE, false);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    ChangeFileTypeTrigger trigger =
      new ChangeFileTypeTrigger(parameter1, condition, null);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_saida nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "'/>\n";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>\n";
    xml += "</grupo>\n";
    xml += "<trocar_tipo_de_arquivo parametro='" + parameterName1 + "'>\n";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>\n";
    xml += "</trocar_tipo_de_arquivo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  /**
   * Tentando criar gatilhos trocar tipo de arquivo com atributos desconhecidos.
   */
  public void testChangeFileTypeTrigger4() {
    String command = "Comando_do_algoritmo";
    String groupName = "Rtulo do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String value = "Valor";
    String fileType = "Tipo";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<arquivo_de_saida nome='" + parameterName1 + "' rotulo='"
        + parameterLabel1 + "' dica='" + parameterDescription1 + "'/>\n";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>\n";
    xml += "</grupo>\n";
    xml +=
      "<trocar_tipo_de_arquivo parametro='" + parameterName1 + "' novo_tipo='"
        + fileType + "' desconhecido='valor'>\n";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>\n";
    xml += "</trocar_tipo_de_arquivo>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testShowEnumerationItemTrigger1() throws ParseException {
    String command = "Comando";
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Nome do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String defaultItemId = "ID 1";
    String defaultItemName = "Nome do item padro";
    String defaultItemValue = "Valor do item padro";
    EnumerationItem defaultItem =
      new EnumerationItem(defaultItemId, defaultItemName, defaultItemValue,
        null);
    boolean mustSortItems = true;
    List<EnumerationItem> items1 = new LinkedList<EnumerationItem>();
    items1.add(defaultItem);
    EnumerationParameter parameter1 =
      new EnumerationParameter(parameterName1, parameterLabel1,
        parameterDescription1, defaultItem, DEFAULT_COMMAND_LINE_PATTERN,
        items1, mustSortItems);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    boolean parameterValue2 = false;
    BooleanParameter parameter2 =
      new BooleanParameter(parameterName2, parameterLabel2,
        parameterDescription2, parameterValue2, DEFAULT_COMMAND_LINE_PATTERN,
        null, null);
    group.addParameter(parameter2);
    SimpleCondition condition =
      new SimpleCondition(parameterName2, new Boolean(parameterValue2));
    SetVisibleEnumerationItemTrigger trigger =
      new SetVisibleEnumerationItemTrigger(parameter1, condition,
        defaultItemId, true);
    version.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<enumeracao nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='" + defaultItemId
        + "'>\n";
    xml +=
      "<item_de_enumeracao id='" + defaultItemId + "' rotulo='" + defaultItemId
        + "' valor='" + defaultItemValue + "'/>";
    xml += "</enumeracao>";
    xml +=
      "<booleano nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "' padrao='" + parameterValue2
        + "'/>";
    xml += "</grupo>\n";
    xml +=
      "<exibir_item parametro='" + parameterName1 + "' item='" + defaultItemId
        + "'>\n";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + parameterValue2
        + "'/>\n";
    xml += "</exibir_item>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  /**
   * Tentando criar gatilhos exibir item de enumerao com atributos
   * desconhecidos.
   */
  public void testShowEnumerationItemTrigger2() {
    String command = "Comando";
    String groupName = "Nome do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String defaultItemId = "ID 1";
    String defaultItemValue = "Valor do item padro";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    boolean parameterValue2 = false;
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<enumeracao nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='" + defaultItemId
        + "'>\n";
    xml +=
      "<item_de_enumeracao id='" + defaultItemId + "' rotulo='" + defaultItemId
        + "' valor='" + defaultItemValue + "'/>";
    xml += "</enumeracao>";
    xml +=
      "<booleano nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "' padrao='" + parameterValue2
        + "'/>";
    xml += "</grupo>\n";
    xml +=
      "<exibir_item parametro='" + parameterName1 + "' item='" + defaultItemId
        + "' desconhecido='valor'>\n";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + parameterValue2
        + "'/>\n";
    xml += "</exibir_item>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testShowEnumerationItemTrigger3() throws ParseException {
    String command = "Comando";
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Nome do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String defaultItemId = "ID 1";
    String defaultItemName = "Nome do item padro";
    String defaultItemValue = "Valor do item padro";
    EnumerationItem defaultItem =
      new EnumerationItem(defaultItemId, defaultItemName, defaultItemValue,
        null);
    List<EnumerationItem> items1 = new LinkedList<EnumerationItem>();
    items1.add(defaultItem);
    boolean mustSortItems = true;
    EnumerationParameter parameter1 =
      new EnumerationParameter(parameterName1, parameterLabel1,
        parameterDescription1, defaultItem, DEFAULT_COMMAND_LINE_PATTERN,
        items1, mustSortItems);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    boolean parameterValue2 = false;
    BooleanParameter parameter2 =
      new BooleanParameter(parameterName2, parameterLabel2,
        parameterDescription2, parameterValue2, DEFAULT_COMMAND_LINE_PATTERN,
        null, null);
    group.addParameter(parameter2);
    Condition condition =
      new SimpleCondition(parameterName2, new Boolean(parameterValue2));
    SetVisibleEnumerationItemTrigger trigger =
      new SetVisibleEnumerationItemTrigger(parameter1, condition,
        defaultItemId, false);
    version.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<enumeracao nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='" + defaultItemId
        + "'>\n";
    xml +=
      "<item_de_enumeracao id='" + defaultItemId + "' rotulo='"
        + defaultItemName + "' valor='" + defaultItemValue + "'/>";
    xml += "</enumeracao>";
    xml +=
      "<booleano nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "' padrao='" + parameterValue2
        + "'/>";
    xml += "</grupo>\n";
    xml +=
      "<ocultar_item parametro='" + parameterName1 + "' item='" + defaultItemId
        + "'>\n";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + parameterValue2
        + "'/>\n";
    xml += "</ocultar_item>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  public void testHideEnumerationItemTrigger1() throws ParseException {
    String command = "Comando";
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Nome do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String defaultItemId = "ID 1";
    String defaultItemName = "Nome do item padro";
    String defaultItemValue = "Valor do item padro";
    EnumerationItem defaultItem =
      new EnumerationItem(defaultItemId, defaultItemName, defaultItemValue,
        null);
    boolean mustSortItems = true;
    List<EnumerationItem> items1 = new LinkedList<EnumerationItem>();
    items1.add(defaultItem);
    EnumerationParameter parameter1 =
      new EnumerationParameter(parameterName1, parameterLabel1,
        parameterDescription1, defaultItem, DEFAULT_COMMAND_LINE_PATTERN,
        items1, mustSortItems);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    boolean parameterValue2 = false;
    BooleanParameter parameter2 =
      new BooleanParameter(parameterName2, parameterLabel2,
        parameterDescription2, parameterValue2, DEFAULT_COMMAND_LINE_PATTERN,
        null, null);
    group.addParameter(parameter2);
    SimpleCondition condition =
      new SimpleCondition(parameterName2, new Boolean(parameterValue2));
    SetVisibleEnumerationItemTrigger trigger =
      new SetVisibleEnumerationItemTrigger(parameter1, condition,
        defaultItemId, false);
    version.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<enumeracao nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='" + defaultItemId
        + "'>\n";
    xml +=
      "<item_de_enumeracao id='" + defaultItemId + "' rotulo='" + defaultItemId
        + "' valor='" + defaultItemValue + "'/>";
    xml += "</enumeracao>";
    xml +=
      "<booleano nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "' padrao='" + parameterValue2
        + "'/>";
    xml += "</grupo>\n";
    xml +=
      "<ocultar_item parametro='" + parameterName1 + "' item='" + defaultItemId
        + "'>\n";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + parameterValue2
        + "'/>\n";
    xml += "</ocultar_item>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  /**
   * Tentando criar gatilhos ocultar item de enumerao com atributos
   * desconhecidos.
   */
  public void testHideEnumerationItemTrigger2() {
    String command = "Comando";
    String groupName = "Nome do grupo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    String defaultItemId = "ID 1";
    String defaultItemName = "Nome do item padro";
    String defaultItemValue = "Valor do item padro";
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    boolean parameterValue2 = false;
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<enumeracao nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "' padrao='" + defaultItemId
        + "'>\n";
    xml +=
      "<item_de_enumeracao id='" + defaultItemId + "' rotulo='"
        + defaultItemName + "' valor='" + defaultItemValue + "'/>";
    xml += "</enumeracao>";
    xml +=
      "<booleano nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "' padrao='" + parameterValue2
        + "'/>";
    xml += "</grupo>\n";
    xml +=
      "<ocultar_item parametro='" + parameterName1 + "' item='" + defaultItemId
        + "' desconhecido='valor'>\n";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + parameterValue2
        + "'/>\n";
    xml += "</ocultar_item>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testChangeLabelTrigger1() throws ParseException {
    String command = "Comando_do_algoritmo";
    SimpleAlgorithmConfigurator expectedVersion =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String groupName = "Rtulo do grupo";
    ParameterGroup group = new ParameterGroup(groupName);
    String parameterName1 = "Nome do parmetro 1";
    String parameterLabel1 = "Rtulo do parmetro 1";
    String parameterDescription1 = "Descrio do parmetro 1";
    boolean isOptional1 = false;
    TextParameter parameter1 =
      new TextParameter(parameterName1, parameterLabel1, parameterDescription1,
        null, isOptional1, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter1);
    String parameterName2 = "Nome do parmetro 2";
    String parameterLabel2 = "Rtulo do parmetro 2";
    String parameterDescription2 = "Descrio do parmetro 2";
    String parameterDefaultValue2 = null;
    boolean isOptional2 = false;
    TextParameter parameter2 =
      new TextParameter(parameterName2, parameterLabel2, parameterDescription2,
        parameterDefaultValue2, isOptional2, DEFAULT_COMMAND_LINE_PATTERN, null);
    group.addParameter(parameter2);
    expectedVersion.addGroup(group);
    String value = "Valor";
    Condition condition = new SimpleCondition(parameter2.getName(), value);
    String newLabel = "Novo Rtulo";
    ChangeLabelTrigger trigger =
      new ChangeLabelTrigger(parameter1, condition, newLabel);
    expectedVersion.addTrigger(trigger);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo rotulo='" + groupName + "'>\n";
    xml +=
      "<texto nome='" + parameterName1 + "' rotulo='" + parameterLabel1
        + "' dica='" + parameterDescription1 + "'/>\n";
    xml +=
      "<texto nome='" + parameterName2 + "' rotulo='" + parameterLabel2
        + "' dica='" + parameterDescription2 + "'/>\n";
    xml += "</grupo>\n";
    xml +=
      "<trocar_rotulo parametro='" + parameterName1 + "' novo_rotulo='"
        + newLabel + "'>\n";
    xml +=
      "<condicao parametro='" + parameterName2 + "' valor='" + value + "'/>\n";
    xml += "</trocar_rotulo>\n";
    xml += "</algoritmo>";
    assertXml(expectedVersion, xml);
  }

  public void testInputGroup1() throws ParseException {
    String command = "Comando";
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    ParameterGroup inputGroup = new ParameterGroup("Parmetros de entrada");
    version.addGroup(inputGroup);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo_de_entrada/>";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  public void testOutputGroup2() throws ParseException {
    String command = "Comando";
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    ParameterGroup outputGroup = new ParameterGroup("Parmetros de sada");
    version.addGroup(outputGroup);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo_de_saida/>";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  public void testHiddenParameter1() throws ParseException {
    String command = "Comando";
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String parameterName = "Nome_do_parametro";
    String parameterValue = "Valor do parmetro";
    HiddenParameter parameter =
      new HiddenParameter(parameterName, parameterValue,
        DEFAULT_COMMAND_LINE_PATTERN);
    version.addHiddenParameter(parameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml +=
      "<oculto nome='" + parameterName + "' valor='" + parameterValue + "'/>";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  public void testHiddenParameter2() {
    String command = "Comando";
    String parameterName = "Nome do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<oculto nome='" + parameterName + "'/>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testHiddenParameter3() {
    String command = "Comando";
    String parameterValue = "Valor do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<oculto valor='" + parameterValue + "'/>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  /**
   * Tentando criar parmetros ocultos com atributos desconhecidos.
   */
  public void testHiddenParameter4() {
    String command = "Comando";
    String parameterName = "Nome_do_parametro";
    String parameterValue = "Valor do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml +=
      "<oculto nome='" + parameterName + "' valor='" + parameterValue
        + "' desconhecido='valor'/>";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testHiddenParameter5() throws ParseException {
    String command = "Comando";
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    String parameterName = "Nome_do_parametro";
    String parameterValue = "Valor do parmetro";
    String commandLinePattern =
      "Nome: $NOME_DO_PARAMETRO Valor: $VALOR_DO_PARAMETRO";
    HiddenParameter parameter =
      new HiddenParameter(parameterName, parameterValue, commandLinePattern);
    version.addHiddenParameter(parameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml +=
      "<oculto nome='" + parameterName + "' valor='" + parameterValue + "'>\n";
    xml +=
      "<formato_no_comando>" + commandLinePattern + "</formato_no_comando>\n";
    xml += "</oculto>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  public void testHiddenParameter6() {
    String command = "Comando";
    String parameterName = "Nome_do_parametro";
    String parameterValue = "Valor do parmetro";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml +=
      "<oculto nome='" + parameterName + "' valor='" + parameterValue + "'>\n";
    xml += "<formato_no_comando></formato_no_comando>\n";
    xml += "</oculto>\n";
    xml += "</algoritmo>";
    assertXml(xml);
  }

  public void testLogFileParameter1() throws ParseException {
    String command = "Comando_do_algoritmo";
    String groupName = "Parmetros de log";
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    OutputFileParameter outputFileParameter =
      OutputFileParameter.createLogFile(DEFAULT_COMMAND_LINE_PATTERN);
    group.addParameter(outputFileParameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<log/>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  public void testLogFileParameter2() throws ParseException {
    String command = "Comando_do_algoritmo";
    String groupName = "Parmetros de log";
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    OutputFileParameter outputFileParameter =
      OutputFileParameter.createLogFile(DEFAULT_COMMAND_LINE_PATTERN);
    group.addParameter(outputFileParameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo_de_log>\n";
    xml += "<arquivo_de_log/>\n";
    xml += "</grupo_de_log>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  public void testLogFileParameter3() throws ParseException {
    String command = "Comando_do_algoritmo";
    String groupName = "Parmetros de log";
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    String commandLinePattern =
      "Nome: $NOME_DO_PARAMETRO Valor:$VALOR_DO_PARAMETRO";
    OutputFileParameter outputFileParameter =
      OutputFileParameter.createLogFile(commandLinePattern);
    group.addParameter(outputFileParameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo_de_log>\n";
    xml += "<arquivo_de_log>\n";
    xml +=
      "<formato_no_comando>" + commandLinePattern + "</formato_no_comando>\n";
    xml += "</arquivo_de_log>\n";
    xml += "</grupo_de_log>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  public void testLogFileParameter4() throws ParseException {
    String command = "Comando_do_algoritmo";
    String groupName = "Parmetros de log";
    SimpleAlgorithmConfigurator version =
      createAlgorithmConfigurator(command, null, ExecutionType.SIMPLE, null,
        false, null, null);
    ParameterGroup group = new ParameterGroup(groupName);
    version.addGroup(group);
    OutputFileParameter outputFileParameter =
      OutputFileParameter.createLogFile(null);
    group.addParameter(outputFileParameter);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>\n";
    xml += "<grupo_de_log>\n";
    xml += "<arquivo_de_log>\n";
    xml += "<formato_no_comando></formato_no_comando>\n";
    xml += "</arquivo_de_log>\n";
    xml += "</grupo_de_log>\n";
    xml += "</algoritmo>";
    assertXml(version, xml);
  }

  public void testSimpleParametersProperties() throws ParseException {
    String parameterName = "real";
    String parameterLabel = "Real";
    String parameterDescription = "Descrio do parmetro real";
    String command = "comando";
    SimpleAlgorithmConfigurator configurator =
      createAlgorithmConfigurator(command);
    String groupLabel = "Grupo";
    ParameterGroup group = new ParameterGroup(groupLabel);
    DoubleParameter parameter =
      new DoubleParameter(parameterName, parameterLabel, parameterDescription,
        null, false, DEFAULT_COMMAND_LINE_PATTERN, null, true, null, true);
    group.addParameter(parameter);
    configurator.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>";
    xml += "<grupo rotulo='" + groupLabel + "'>";
    xml +=
      "<real nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'  />";
    xml += "</grupo>";
    xml += "</algoritmo>";
    assertXml(configurator, xml);
  }

  public void testParametersPropertiesWithCustomCommandLinePattern()
    throws ParseException, IOException {
    String parameterName = "real";
    String parameterLabel = "Real";
    String parameterDescription = "Descrio do parmetro real";
    String command = "comando";
    String customCommandLinePattern =
      "$NOME_DO_PARAMETRO : $VALOR_DO_PARAMETRO";
    Map<String, String> configMap = createReplacementMap();
    String prop = "";
    prop +=
      "<real nome='" + parameterName + "' rotulo='" + parameterLabel
        + "' dica='" + parameterDescription + "'>";
    prop += "<formato_no_comando>";
    prop += customCommandLinePattern;
    prop += "</formato_no_comando>";
    prop += "</real>";
    configMap.put("real_especial", prop);
    SimpleAlgorithmConfigurator configurator =
      createAlgorithmConfigurator(command);
    String groupLabel = "Grupo";
    ParameterGroup group = new ParameterGroup(groupLabel);
    DoubleParameter parameter =
      new DoubleParameter(parameterName, parameterLabel, parameterDescription,
        null, false, customCommandLinePattern, null, true, null, true);
    group.addParameter(parameter);
    configurator.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>";
    xml += "<grupo rotulo='" + groupLabel + "'>";
    xml += "<real_especial/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    //assertXml(configurator, xml, configMap, createParamMap());
  }

  /**
   * <p>
   * Simulando o uso do arquivo config.properties para definir novos elementos
   * para um config.xml.
   * </p>
   * 
   * <p>
   * Este teste falha se o bug 6482 do CSBase se manifestar novamente.
   * </p>
   * 
   * <p>
   * Descrio do Bug:
   * </p>
   * 
   * <p>
   * O arquivo config.properties define novos elementos vlidos para uma verso
   * especfica de configurador de algoritmos. Este arquivo est sendo ignorado
   * e isto gera um erro no algoritmo Cria XSY do Websintesi.
   * </p>
   * 
   * @throws ParseException Falha
   * @throws IOException Falha ao ler arquivo de configurao
   */
  public void testConfigProperties1() throws ParseException, IOException {
    String parameterName = "ESPECIAL";
    String parameterLabel = "Especial";
    String parameterDescription = "Descrio do parmetro especial";
    Map<String, String> configProperties = createReplacementMap();
    Map<String, String> paramProperties = createParamMap();

    configProperties.put("especial", "<booleano nome='" + parameterName
      + "' rotulo='" + parameterLabel + "' dica='" + parameterDescription
      + "'/>");
    String command = "comando";
    SimpleAlgorithmConfigurator configurator =
      createAlgorithmConfigurator(command);
    String groupLabel = "Grupo";
    ParameterGroup group = new ParameterGroup(groupLabel);
    BooleanParameter parameter =
      new BooleanParameter(parameterName, parameterLabel, parameterDescription,
        false, DEFAULT_COMMAND_LINE_PATTERN, null, null);
    group.addParameter(parameter);
    configurator.addGroup(group);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<algoritmo comando='" + command + "'>";
    xml += "<grupo rotulo='" + groupLabel + "'>";
    xml += "<especial/>";
    xml += "</grupo>";
    xml += "</algoritmo>";
    StringReader configXmlReader = new StringReader(xml);
    AlgorithmVersionInfo algorithmVersion = configurator.getAlgorithmVersion();
    SimpleAlgorithmParser parser = new SimpleAlgorithmParser();
    SimpleAlgorithmConfigurator loadedConfigurator =
      parser.load(configXmlReader, configProperties, paramProperties,
        algorithmVersion);
    assertConfigurator(configurator, loadedConfigurator);
  }

  static final void assertXml(SimpleAlgorithmConfigurator expectedConfigurator,
    String xml, Map<String, String> replacementMap,
    Map<String, String> paramProperties) throws ParseException {
    SimpleAlgorithmConfigurator loadedVersion =
      loadXml(expectedConfigurator.getAlgorithmVersion(), xml, replacementMap,
        paramProperties);
    assertConfigurator(expectedConfigurator, loadedVersion);
  }

  static final void assertXml(SimpleAlgorithmConfigurator expectedConfigurator,
    String xml) throws ParseException {
    try {
      SimpleAlgorithmConfigurator loadedVersion =
        loadXml(expectedConfigurator.getAlgorithmVersion(), xml);
      assertConfigurator(expectedConfigurator, loadedVersion);
    }
    catch (IOException e) {
      fail("Erro ao carregar arquivo de configurao: " + e.getMessage());
    }

  }

  static final void assertXml(String xml) {
    try {
      loadXml(createAlgorithmConfigurator("comando").getAlgorithmVersion(), xml);
      fail("Esperada ParseException");
    }
    catch (ParseException e) {
    }
    catch (IOException e) {
      fail("Erro ao carregar arquivo de configurao: " + e.getMessage());
    }

  }

  static SimpleAlgorithmConfigurator createAlgorithmConfigurator(String command) {
    String description = null;
    ExecutionType executionType = ExecutionType.SIMPLE;
    String abbreviation = null;
    boolean provideId = false;
    String shell = null;
    String defaultInputFileParameterName = null;
    return createAlgorithmConfigurator(command, description, executionType,
      abbreviation, provideId, shell, defaultInputFileParameterName);
  }

  static SimpleAlgorithmConfigurator createAlgorithmConfigurator(
    String command, String description, ExecutionType executionType,
    String abbreviation, boolean provideId, String shell,
    String defaultInputFileParameterName) {
    AlgorithmVersionInfo algorithmVersion = createAlgorithmVersion();
    return new SimpleAlgorithmConfigurator(algorithmVersion, description,
      executionType, abbreviation, provideId, command, shell,
      defaultInputFileParameterName, true);
  }

  private static AlgorithmVersionInfo createAlgorithmVersion() {
    String algorithmName = "Algoritmo";
    int major = 1;
    int minor = 2;
    int patch = 3;
    AlgorithmInfo algorithm =
      new AlgorithmInfo("ID " + algorithmName, algorithmName, "DIR "
        + algorithmName, new Hashtable<String, String>());
    AlgorithmVersionInfo algorithmVersion =
      new AlgorithmVersionInfo(algorithm, new AlgorithmVersionId(major, minor,
        patch), new Hashtable<String, List<FileInfo>>(), new Hashtable());
    return algorithmVersion;
  }

  private static void assertConfigurator(
    SimpleAlgorithmConfigurator expectedVersion,
    SimpleAlgorithmConfigurator loadedVersion) {
    assertNotNull(loadedVersion);
    assertEquals(expectedVersion, loadedVersion);
    assertEquals(expectedVersion.getAlgorithmVersion(), loadedVersion
      .getAlgorithmVersion());
    assertEquals(expectedVersion.getCommandBinaryName(), loadedVersion
      .getCommandBinaryName());
    assertEquals(expectedVersion.getDescription(), loadedVersion
      .getDescription());
    assertEquals(expectedVersion.getShell(), loadedVersion.getShell());
    assertGroups(expectedVersion.getGroups(), loadedVersion.getGroups());
    assertExpressions(expectedVersion.getExpressions(), loadedVersion
      .getExpressions());
    assertTriggers(expectedVersion.getTriggers(), loadedVersion.getTriggers());
    assertEquals(expectedVersion.getExecutionType(), loadedVersion
      .getExecutionType());
    assertEquals(expectedVersion.getDefaultInputFileParameterName(),
      loadedVersion.getDefaultInputFileParameterName());
    assertEquals(expectedVersion.getAbbreviation(), loadedVersion
      .getAbbreviation());
    assertEquals(expectedVersion.provideId(), loadedVersion.provideId());
    assertEquals(expectedVersion.hasExitCode(), loadedVersion.hasExitCode());

    assertHiddenParameters(expectedVersion.getHiddenParameters(), loadedVersion
      .getHiddenParameters());
  }

  private static void assertColumns(List expectedColumns, List loadedColumns) {
    assertEquals(expectedColumns.size(), loadedColumns.size());
    Iterator expectedColumnIterator = expectedColumns.iterator();
    Iterator loadedColumnIterator = loadedColumns.iterator();
    while (expectedColumnIterator.hasNext()) {
      Column expectedColumn = (Column) expectedColumnIterator.next();
      Column loadedColumn = (Column) loadedColumnIterator.next();
      assertEquals(expectedColumn.getId(), loadedColumn.getId());
      assertEquals(expectedColumn.getLabel(), loadedColumn.getLabel());
      assertEquals(expectedColumn.getClass(), loadedColumn.getClass());
      assertEquals(expectedColumn.isOptional(), loadedColumn.isOptional());
      assertEquals(expectedColumn.isEditable(), loadedColumn.isEditable());
      assertEquals(expectedColumn.getDefaultValue(), loadedColumn
        .getDefaultValue());
      Set<Integer> rowIndexes = expectedColumn.getRowIndexes();
      for (Integer rowIndex : rowIndexes) {
        assertEquals(expectedColumn.getDefaultValue(rowIndex), loadedColumn
          .getDefaultValue(rowIndex));
      }
      if (expectedColumn instanceof TextColumn) {
        TextColumn expectedTextColumn = (TextColumn) expectedColumn;
        TextColumn loadedTextColumn = (TextColumn) loadedColumn;
        assertEquals(expectedTextColumn.getMaxCharacters(), loadedTextColumn
          .getMaxCharacters());
      }
      else if (expectedColumn instanceof IntegerColumn) {
        IntegerColumn expectedIntegerColumn = (IntegerColumn) expectedColumn;
        IntegerColumn loadedIntegerColumn = (IntegerColumn) loadedColumn;
        assertEquals(expectedIntegerColumn.getLabel(), loadedIntegerColumn
          .getLabel());
        assertEquals(expectedIntegerColumn.getMaximum(), loadedIntegerColumn
          .getMaximum());
        assertEquals(expectedIntegerColumn.getMinimum(), loadedIntegerColumn
          .getMinimum());
      }
      else if (expectedColumn instanceof DoubleColumn) {
        DoubleColumn expectedDoubleColumn = (DoubleColumn) expectedColumn;
        DoubleColumn loadedDoubleColumn = (DoubleColumn) loadedColumn;
        assertEquals(expectedDoubleColumn.getLabel(), loadedDoubleColumn
          .getLabel());
        assertEquals(expectedDoubleColumn.getMaximum(), loadedDoubleColumn
          .getMaximum());
        assertEquals(expectedDoubleColumn.getMinimum(), loadedDoubleColumn
          .getMinimum());
        assertEquals(expectedDoubleColumn.isMaximumIncluded(),
          loadedDoubleColumn.isMaximumIncluded());
        assertEquals(expectedDoubleColumn.isMinimumIncluded(),
          loadedDoubleColumn.isMinimumIncluded());
      }
      else if (expectedColumn instanceof BooleanColumn) {
        BooleanColumn expectedBooleanColumn = (BooleanColumn) expectedColumn;
        BooleanColumn loadedBooleanColumn = (BooleanColumn) loadedColumn;
        assertEquals(expectedBooleanColumn.getTrueCommand(),
          loadedBooleanColumn.getTrueCommand());
        assertEquals(expectedBooleanColumn.getFalseCommand(),
          loadedBooleanColumn.getFalseCommand());
      }
    }
  }

  private static void assertTriggers(Set expectedTriggers, Set loadedTriggers) {
    assertNotNull(loadedTriggers);
    assertEquals(expectedTriggers.size(), loadedTriggers.size());
    Iterator expectedTriggerIterator = expectedTriggers.iterator();
    Iterator loadedTriggerIterator = loadedTriggers.iterator();
    while (expectedTriggerIterator.hasNext()) {
      Trigger expectedTrigger = (Trigger) expectedTriggerIterator.next();
      Trigger loadedTrigger = (Trigger) loadedTriggerIterator.next();
      assertTrigger(expectedTrigger, loadedTrigger);
    }
  }

  private static void assertTrigger(Trigger expectedTrigger,
    Trigger loadedTrigger) {
    assertEquals(expectedTrigger, loadedTrigger);
    assertEquals(expectedTrigger.getCondition(), loadedTrigger.getCondition());
    assertEquals(expectedTrigger.getParameter(), loadedTrigger.getParameter());
    if (expectedTrigger instanceof ChangeMaximumForDoublesTrigger) {
      ChangeMaximumForDoublesTrigger expectedDoubleChangeMaximumTrigger =
        (ChangeMaximumForDoublesTrigger) expectedTrigger;
      ChangeMaximumForDoublesTrigger loadedDoubleChangeMaximumTrigger =
        (ChangeMaximumForDoublesTrigger) loadedTrigger;
      assertEquals(expectedDoubleChangeMaximumTrigger.getMaximum(),
        loadedDoubleChangeMaximumTrigger.getMaximum());
      assertEquals(expectedDoubleChangeMaximumTrigger.isMaximumIncluded(),
        loadedDoubleChangeMaximumTrigger.isMaximumIncluded());
    }
    else if (expectedTrigger instanceof ChangeMinimumForDoublesTrigger) {
      ChangeMinimumForDoublesTrigger expectedDoubleChangeMinimumTrigger =
        (ChangeMinimumForDoublesTrigger) expectedTrigger;
      ChangeMinimumForDoublesTrigger loadedDoubleChangeMinimumTrigger =
        (ChangeMinimumForDoublesTrigger) loadedTrigger;
      assertEquals(expectedDoubleChangeMinimumTrigger.getMinimum(),
        loadedDoubleChangeMinimumTrigger.getMinimum());
      assertEquals(expectedDoubleChangeMinimumTrigger.isMinimumIncluded(),
        loadedDoubleChangeMinimumTrigger.isMinimumIncluded());
    }
    else if (expectedTrigger instanceof ChangeFileTypeTrigger) {
      ChangeFileTypeTrigger expectedChangeFileTypeTrigger =
        (ChangeFileTypeTrigger) expectedTrigger;
      ChangeFileTypeTrigger loadedChangeFileTypeTrigger =
        (ChangeFileTypeTrigger) loadedTrigger;
      assertEquals(expectedChangeFileTypeTrigger.getFileType(),
        loadedChangeFileTypeTrigger.getFileType());
    }
    else if (expectedTrigger instanceof SetVisibleEnumerationItemTrigger) {
      SetVisibleEnumerationItemTrigger expectedSetVisibleEnumerationItemTrigger =
        (SetVisibleEnumerationItemTrigger) expectedTrigger;
      SetVisibleEnumerationItemTrigger loadedSetVisibleEnumerationItemTrigger =
        (SetVisibleEnumerationItemTrigger) loadedTrigger;
      assertEquals(expectedSetVisibleEnumerationItemTrigger.getItemId(),
        loadedSetVisibleEnumerationItemTrigger.getItemId());
      assertEquals(expectedSetVisibleEnumerationItemTrigger.isVisible(),
        loadedSetVisibleEnumerationItemTrigger.isVisible());
    }
    else if (expectedTrigger instanceof ChangeDefaultValueTrigger) {
      ChangeDefaultValueTrigger expectedChangeDefaultValueTrigger =
        (ChangeDefaultValueTrigger) expectedTrigger;
      ChangeDefaultValueTrigger loadedChangeDefaultValueTrigger =
        (ChangeDefaultValueTrigger) loadedTrigger;
      assertEquals(expectedChangeDefaultValueTrigger.getDefaultValue(),
        loadedChangeDefaultValueTrigger.getDefaultValue());
    }
  }

  private static void assertExpressions(Set expectedExpressions,
    Set loadedExpressions) {
    assertNotNull(loadedExpressions);
    assertEquals(expectedExpressions.size(), loadedExpressions.size());
    Iterator expectedExpressionIterator = expectedExpressions.iterator();
    Iterator loadedExpressionIterator = loadedExpressions.iterator();
    while (expectedExpressionIterator.hasNext()) {
      ValidationExpression expectedExpression =
        (ValidationExpression) expectedExpressionIterator.next();
      ValidationExpression loadedExpression =
        (ValidationExpression) loadedExpressionIterator.next();
      assertEquals(expectedExpression, loadedExpression);
      assertEquals(expectedExpression.toString(), loadedExpression.toString());
      assertEquals(expectedExpression.getErrorMessage(), loadedExpression
        .getErrorMessage());
    }
  }

  private static void assertGroups(List expectedGroups, List loadedGroups) {
    assertNotNull(loadedGroups);
    assertEquals(expectedGroups, loadedGroups);
    Iterator expectedGroupIterator = expectedGroups.iterator();
    Iterator loadedGroupIterator = loadedGroups.iterator();
    while (expectedGroupIterator.hasNext()) {
      ParameterGroup expectedGroup =
        (ParameterGroup) expectedGroupIterator.next();
      ParameterGroup loadedGroup = (ParameterGroup) loadedGroupIterator.next();
      assertGroups(expectedGroup, loadedGroup);
    }
  }

  private static void assertGroups(ParameterGroup expectedGroup,
    ParameterGroup loadedGroup) {
    assertEquals(expectedGroup, loadedGroup);
    assertEquals(expectedGroup.getName(), loadedGroup.getName());
    assertParameters(expectedGroup.getParameters(), loadedGroup.getParameters());
  }

  private static void assertHiddenParameters(
    Set<HiddenParameter> expectedHiddenParameters,
    Set<HiddenParameter> loadedHiddenParameters) {
    assertEquals(expectedHiddenParameters, loadedHiddenParameters);
    for (HiddenParameter expectedHiddenParameter : expectedHiddenParameters) {
      for (HiddenParameter loadedHiddenParameter : loadedHiddenParameters) {
        if (expectedHiddenParameter.equals(loadedHiddenParameter)) {
          assertEquals(expectedHiddenParameter.getValue(),
            loadedHiddenParameter.getValue());
          assertEquals(expectedHiddenParameter.getCommandLinePattern(),
            loadedHiddenParameter.getCommandLinePattern());
        }
      }
    }
  }

  private static void assertParameters(List expectedParameters,
    List loadedParameters) {
    assertNotNull(loadedParameters);
    assertEquals(expectedParameters.size(), loadedParameters.size());
    Iterator expectedParameterIterator = expectedParameters.iterator();
    Iterator loadedParameterIterator = loadedParameters.iterator();
    while (expectedParameterIterator.hasNext()) {
      Parameter expectedParameter =
        (Parameter) expectedParameterIterator.next();
      Parameter loadedParameter = (Parameter) loadedParameterIterator.next();
      assertEquals(expectedParameter.getClass(), loadedParameter.getClass());
      if (expectedParameter instanceof SimpleParameter) {
        SimpleParameter expectedSimpleParameter =
          (SimpleParameter) expectedParameter;
        SimpleParameter loadedSimpleParameter =
          (SimpleParameter) loadedParameter;
        assertSimpleParameters(expectedSimpleParameter, loadedSimpleParameter);
      }
      else if (expectedParameter instanceof ParameterGroup) {
        ParameterGroup expectedGroup = (ParameterGroup) expectedParameter;
        ParameterGroup loadedGroup = (ParameterGroup) loadedParameter;
        assertGroups(expectedGroup, loadedGroup);
      }
      else {
        fail("Tipo de parmetro desconhecido.\nTipo: "
          + expectedParameter.getClass().getName());
      }
    }
  }

  private static void assertSimpleParameters(SimpleParameter expectedParameter,
    SimpleParameter loadedParameter) {
    assertEquals(expectedParameter.getClass(), loadedParameter.getClass());
    assertEquals(expectedParameter, loadedParameter);
    assertEquals(expectedParameter.getDefaultValue(), loadedParameter
      .getDefaultValue());
    assertEquals(expectedParameter.getDescription(), loadedParameter
      .getDescription());
    assertEquals(expectedParameter.getLabel(), loadedParameter.getLabel());
    assertEquals(expectedParameter.getName(), loadedParameter.getName());
    assertEquals(expectedParameter.isOptional(), loadedParameter.isOptional());
    assertEquals(expectedParameter.getCommandLinePattern(), loadedParameter
      .getCommandLinePattern());
    if (expectedParameter instanceof IntegerParameter) {
      IntegerParameter expectedIntegerParameter =
        (IntegerParameter) expectedParameter;
      IntegerParameter loadedIntegerParameter =
        (IntegerParameter) loadedParameter;
      assertEquals(expectedIntegerParameter.getMaximum(),
        loadedIntegerParameter.getMaximum());
      assertEquals(expectedIntegerParameter.getMinimum(),
        loadedIntegerParameter.getMinimum());
    }
    else if (expectedParameter instanceof DoubleParameter) {
      DoubleParameter expectedDoubleParameter =
        (DoubleParameter) expectedParameter;
      DoubleParameter loadedDoubleParameter = (DoubleParameter) loadedParameter;
      assertEquals(expectedDoubleParameter.getMaximum(), loadedDoubleParameter
        .getMaximum());
      assertEquals(expectedDoubleParameter.getMinimum(), loadedDoubleParameter
        .getMinimum());
      assertEquals(expectedDoubleParameter.isMaximumIncluded(),
        loadedDoubleParameter.isMaximumIncluded());
      assertEquals(expectedDoubleParameter.isMinimumIncluded(),
        loadedDoubleParameter.isMinimumIncluded());
    }
    else if (expectedParameter instanceof IntegerListParameter) {
      IntegerListParameter expectedIntegerParameter =
        (IntegerListParameter) expectedParameter;
      IntegerListParameter loadedIntegerParameter =
        (IntegerListParameter) loadedParameter;
      assertEquals(expectedIntegerParameter.getMaximum(),
        loadedIntegerParameter.getMaximum());
      assertEquals(expectedIntegerParameter.getMinimum(),
        loadedIntegerParameter.getMinimum());
    }
    else if (expectedParameter instanceof DoubleListParameter) {
      DoubleListParameter expectedDoubleParameter =
        (DoubleListParameter) expectedParameter;
      DoubleListParameter loadedDoubleParameter =
        (DoubleListParameter) loadedParameter;
      assertEquals(expectedDoubleParameter.getMaximum(), loadedDoubleParameter
        .getMaximum());
      assertEquals(expectedDoubleParameter.getMinimum(), loadedDoubleParameter
        .getMinimum());
      assertEquals(expectedDoubleParameter.isMaximumIncluded(),
        loadedDoubleParameter.isMaximumIncluded());
      assertEquals(expectedDoubleParameter.isMinimumIncluded(),
        loadedDoubleParameter.isMinimumIncluded());
    }
    else if (expectedParameter instanceof EnumerationParameter) {
      EnumerationParameter expectedEnumerationParameter =
        (EnumerationParameter) expectedParameter;
      EnumerationParameter loadedEnumerationParameter =
        (EnumerationParameter) loadedParameter;
      assertNotNull(expectedEnumerationParameter.getItems());
      assertNotNull(loadedEnumerationParameter.getItems());
      assertEquals(expectedEnumerationParameter.getItems().size(),
        loadedEnumerationParameter.getItems().size());
    }
    else if (expectedParameter instanceof EnumerationListParameter) {
      EnumerationListParameter expectedEnumerationParameter =
        (EnumerationListParameter) expectedParameter;
      EnumerationListParameter loadedEnumerationParameter =
        (EnumerationListParameter) loadedParameter;
      assertNotNull(expectedEnumerationParameter.getItems());
      assertNotNull(loadedEnumerationParameter.getItems());
      assertEquals(expectedEnumerationParameter.getItems().size(),
        loadedEnumerationParameter.getItems().size());
    }
    else if (expectedParameter instanceof TextParameter) {
      TextParameter expectedTextParameter = (TextParameter) expectedParameter;
      TextParameter loadedTextParameter = (TextParameter) loadedParameter;
      assertEquals(expectedTextParameter.getMaxCharacters(),
        loadedTextParameter.getMaxCharacters());
    }
    else if (expectedParameter instanceof TextListParameter) {
      TextListParameter expectedTextListParameter =
        (TextListParameter) expectedParameter;
      TextListParameter loadedTextListParameter =
        (TextListParameter) loadedParameter;
      assertEquals(expectedTextListParameter.getMaxCharacters(),
        loadedTextListParameter.getMaxCharacters());
    }
    else if (expectedParameter instanceof TableParameter) {
      TableParameter expectedTableParameter =
        (TableParameter) expectedParameter;
      TableParameter loadedTableParameter = (TableParameter) loadedParameter;
      assertEquals(expectedTableParameter.delimitRows(), loadedTableParameter
        .delimitRows());
      assertEquals(expectedTableParameter.delimitTable(), loadedTableParameter
        .delimitTable());
      assertColumns(expectedTableParameter.getColumns(), loadedTableParameter
        .getColumns());
      assertEquals(expectedTableParameter.getRowCount(), loadedTableParameter
        .getRowCount());
      int rowCount = expectedTableParameter.getRowCount();
      int columnCount = expectedTableParameter.getColumnCount();
      for (int rowIndex = 0; rowIndex < rowCount; rowIndex++) {
        for (int columnIndex = 0; columnIndex < columnCount; columnIndex++) {
          assertEquals(expectedTableParameter.getItemValue(rowIndex,
            columnIndex), loadedTableParameter.getItemValue(rowIndex,
            columnIndex));
        }
      }
    }
    else if (expectedParameter instanceof BooleanParameter) {
      BooleanParameter expectedBooleanParameter =
        (BooleanParameter) expectedParameter;
      BooleanParameter loadedBooleanParameter =
        (BooleanParameter) loadedParameter;
      assertEquals(expectedBooleanParameter.getFalseCommand(),
        loadedBooleanParameter.getFalseCommand());
      assertEquals(expectedBooleanParameter.getTrueCommand(),
        loadedBooleanParameter.getTrueCommand());
    }
    else if (expectedParameter instanceof FileParameter) {
      FileParameter expectedFileParameter = (FileParameter) expectedParameter;
      FileParameter loadedFileParameter = (FileParameter) loadedParameter;
      assertEquals(expectedFileParameter.getMode(), loadedFileParameter
        .getMode());
      assertEquals(expectedFileParameter.getFileType(), loadedFileParameter
        .getFileType());
      assertEquals(expectedFileParameter.usesPipe(), loadedFileParameter
        .usesPipe());
      if (expectedParameter instanceof OutputFileParameter) {
        OutputFileParameter expectedOutputFileParameter =
          (OutputFileParameter) expectedParameter;
        OutputFileParameter loadedOutputFileParameter =
          (OutputFileParameter) loadedParameter;
        assertEquals(expectedOutputFileParameter.mustForceExtension(),
          loadedOutputFileParameter.mustForceExtension());
      }
      else if (!(expectedParameter instanceof InputFileParameter)) {
        fail("Tipo inesperado: " + expectedParameter.getClass().getName());
      }
    }
    else if (expectedParameter instanceof InputFileListParameter) {
      InputFileListParameter expectedInputFileListParameter =
        (InputFileListParameter) expectedParameter;
      InputFileListParameter loadedInputFileListParameter =
        (InputFileListParameter) loadedParameter;
      assertEquals(expectedInputFileListParameter.getFileType(),
        loadedInputFileListParameter.getFileType());
      assertEquals(expectedInputFileListParameter.getMode(),
        loadedInputFileListParameter.getMode());
      assertEquals(expectedInputFileListParameter.mustSort(),
        loadedInputFileListParameter.mustSort());
    }
    else if (expectedParameter instanceof FormulaParameter) {
      // Nada a checar
    }
    else {
      fail("Tipo inesperado: " + expectedParameter.getClass().getName());
    }
  }

  private static SimpleAlgorithmConfigurator loadXml(
    AlgorithmVersionInfo algorithmVersion, String xml) throws ParseException,
    IOException {
    return loadXml(algorithmVersion, xml, createReplacementMap(),
      createParamMap());
  }

  private static SimpleAlgorithmConfigurator loadXml(
    AlgorithmVersionInfo algorithmVersion, String xml,
    Map<String, String> replacementMap, Map<String, String> paramProperties)
    throws ParseException {
    Reader reader = new StringReader(xml);
    return new SimpleAlgorithmParser().load(reader, replacementMap,
      paramProperties, algorithmVersion);
  }

  private static HashMap<String, String> createParamMap() throws IOException {
    HashMap<String, String> paramMap = new HashMap<String, String>();
    String propertiesFile = "AlgorithmParameters.properties";
    return loadPropsToMap(paramMap, propertiesFile);
  }

  private static HashMap<String, String> loadPropsToMap(
    HashMap<String, String> paramMap, String propertiesFile)
    throws IOException, FileNotFoundException {
    InputStream inStream =
      AlgorithmService.class.getResourceAsStream(propertiesFile);
    if (inStream != null) {
      Properties props = new Properties();
      props.load(inStream);
      addPropertiesToMap(props, paramMap);
    }
    else {
      throw new FileNotFoundException("No foi possvel encontrar o arquivo "
        + propertiesFile + ".");
    }
    return paramMap;
  }

  private static HashMap<String, String> createReplacementMap()
    throws IOException {
    HashMap<String, String> replacementMap = new HashMap<String, String>();
    String propertiesFile = "AlgorithmParametersConfiguration.properties";
    return loadPropsToMap(replacementMap, propertiesFile);
  }

  private static <K, V> void addPropertiesToMap(final Properties properties,
    final Map<K, V> map) {
    for (Entry<Object, Object> entry : properties.entrySet()) {
      map.put((K) entry.getKey(), (V) entry.getValue());
    }
  }
}
